```@meta
CurrentModule = Oscar
DocTestSetup = quote
  using Oscar
end
```

```@setup oscar
using Oscar
```

```@contents
Pages = ["basics.md"]
```

# Basics

## [Elements of groups](@id elements_of_groups)

Given a group `G`, it is always possible to have access to some particular elements.

```@docs
GAPGroup
BasicGAPGroupElem{T<:GAPGroup}
elem_type(::Type{T}) where T <: GAPGroup
one(x::GAPGroup)
one(x::GAPGroupElem)
is_finiteorder(x::GAPGroupElem)
gens(::GAPGroup)
has_gens(::GAPGroup)
ngens(G::GAPGroup)
gen(::GAPGroup, i::Int)
small_generating_set(G::GAPGroup)
Base.rand(G::GAPGroup)
rand_pseudo(G::GAPGroup)
```

It is also possible to obtain the generators of `G` by typing
```julia
f1,f2,f3 = gens(G)
```
This is equivalent to
```julia
f1=G[1]; f2=G[2]; f3=G[3];
```

For a group `G` that has been created as a subgroup of another group,
generated by a list `L` of elements, `gens(G)` is equal to `L`.

## Operations on group elements

OSCAR supports the following operations and functions on group elements.

* `*`, multiplication between two elements in a group.
* `inv(x)` and `x^-1`, the inverse of `x`.
* `x/y`, the element `x y^-1`.
* `x^n`, the `n`-th power of `x`;
  if `n == 0`, the identity of the group is returned;
  if `n < 0`, the `-n`-th power of the inverse of `x` is returned.
* `isone(x)` returns whether `x` is the identity of the group.
* `conj(x,y)` and `x^y`, the conjugate of `x` by `y`,
  i.e., the element `y^-1 x y`.
* `comm(x,y)`, the commutator of `x` and `y`,
  i.e., the element `x^-1 y^-1 x y`.

!!! note
    In OSCAR, the expression `x^y^z` is equivalent to `x^(y^z)`.
    In other words, conjugations are evaluated from the right to the left.

```@docs
comm(x::GAPGroupElem, y::GAPGroupElem)
```

## Properties of groups

```@docs
is_finite(G::GAPGroup)
is_trivial(G::GAPGroup)
is_abelian(G::GAPGroup)
is_elementary_abelian
is_pgroup
is_pgroup_with_prime
is_nilpotent
is_supersolvable
is_solvable
is_perfect
is_simple(G::GAPGroup)
is_almostsimple
is_quasisimple
is_sporadic_simple
is_finitelygenerated
```


## Attributes of groups

```@docs
order(::Type{T}, x::Union{GAPGroupElem, GAPGroup}) where T <: IntegerUnion
exponent(G::GAPGroup)
describe(G::GAPGroup)
nilpotency_class(G::GAPGroup)
prime_of_pgroup
```
