```@meta
CurrentModule = Oscar
DocTestSetup = quote
  using Oscar
end
```

```@setup oscar
using Oscar
```

```@contents
Pages = ["intro.md"]
```

# Introduction

We start our discussion of PBW-algebras by recalling their definition.
Let $K$ be a field. Given a set of variables $x=\{x_1, \ldots, x_n\},$ we write
${\left\langle {x}\right\rangle}:=\langle x_{1},\ldots, x_{n} \rangle$ for the *free monoid* on $x$.
That is, the elements of $\langle x \rangle$ are the words in the finite alphabet $x$, multiplication
means concatenation of words, and the identity element is the empty word.
The *free associative $K$-algebra* generated by $x_{1},\dots, x_{n}$ is the corresponding monoid algebra

$K \langle {x}\rangle:= K \langle x_{1},\dots, x_{n} \rangle.$

We consider quotients of type $A = K\langle x_1, \dots, x_n \rangle/J$, for some $n$ and some
two-sided ideal $J$ of $K\langle x_1, \dots, x_n \rangle$. In case $J$ is given by a finite set
of two-sided generators $g_1, \dots, g_r$, we say that *$A$ is generated by $x_1, \dots, x_n$,
subject to the relations $g_1 = 0, \dots, g_r = 0$*, and write

$A = K\langle x_1, \dots , x_n \mid g_k=0, \ 1\leq k \leq r \rangle.$

The relations considered in this chapter are "commutation relations", written as
```math
x_jx_i = c_{ij}x_ix_j+d_{ij}.
```

Working with Gröbner bases requires that we take monomial orderings into account (see the section
on Gröbner bases in the commutative algebra chapter for monomial orderings). In our context here, we use the following notation.
A *standard monomial* in $K \langle x \rangle$ is a word of type $x^\alpha=x_{1}^{\alpha_{1}}\cdots x_{n}^{\alpha_{n}},$
where $\alpha=(\alpha_1,\dots,\alpha_n)\in\mathbb N^n$. A *standard polynomial* in $K \langle x \rangle$
is a $K$-linear combination of standard monomials. Each global monomial ordering $>$ on $K[x]$ gives rise to
a (total) well-ordering on the set of standard monomials. Abusing our notation, we denote the induced ordering
again by $>$, and refer to it as a *global monomial ordering on $A$*. Given $>$, it makes sense to speak of the
*leading monomial* $\text{LM}_>(f)$ of a standard polynomial $0\neq f \in K \langle x \rangle.$ The notion
of a global elimination ordering with respect to a subset of $\{ x_{1},\ldots, x_{n} \}$ carries over
from $K[x]$ to $A$.

**Definition.**  ``\;`` Let $A$ be a $K$-algebra of type
```math
A = K\langle x_1, \dots , x_n \mid x_jx_i = c_{ij}x_ix_j+d_{ij},  \ 1\leq i<j \leq n \rangle,
```
where the $c_{ij}\in K$ are nonzero scalars, and the $d_{ij}\in K\langle x_1, \dots , x_n\rangle$ are 
standard polynomials. Then $A$ is called a *PBW-algebra* if the following two conditions hold:

(1) ``\;`` There exists a global monomial ordering $>$ on $A$ such that

$d_{ij}=0\ \text{ or }\ x_ix_j> \text{LM}_>(d_{ij})\ \text{ for all }\ 1\leq i<j \leq n.$

(2) ``\;`` The standard monomials in $K \langle x \rangle$ represent a $K$-basis for $A$.  We then refer to this basis as a *PBW-basis* for $A$. 

Every ordering as in (1) is called *admissible* for the given relations or simply *admissible* for $A$.

Given a PBW-algebra $A$ as above, we sometimes abuse our notation by denoting the class of a standard monomial $x^{\alpha}$ in $A$ also by $x^{\alpha}$,
and refer to this class as a *standard monomial* in $A$. As these monomials form a $K$-basis for $A$, every element $0\neq f\in A$ has a unique representation
```math
f=\sum c_{\alpha}x^{\alpha}, \; \text{ with nonzero coefficients } \; c_{\alpha}\in K.
```
We refer to this representation as the *standard representation* of $f$, with *coefficients* $c_{\alpha}$, and *exponents* $\alpha$.

!!! note
    PBW-algebras are also known as *G-algebras* or *algebras of solvable type*. See Remark 1 in [LS03](@cite) for a brief historical account.
 
**Proposition.**  ``\;`` Let $A$ be a PBW-algebra. Then:
- ``A`` is an integral domain,
- ``A`` is (left and right) Noetherian.

Given any associative $K$-algebra $A = (A, +, \cdot)$, its *opposite algebra* is defined by setting
``A^{\text{op}} = (A, +, \ast)``, where $f\ast g:=g\cdot f$ for all $f, g\in A.$
If
```math
A = K\langle x_1, \dots , x_n \mid x_jx_i = c_{ij} x_ix_j+d_{ij},  \ 1\leq i<j \leq n \rangle
```
is a PBW-algebra, then $A^{\text{op}}$ is again a PBW-algebra in a natural way. Indeed, consider the automorphism ${\text{op}}$
of $K\langle x_1, \dots , x_n\rangle$ which sends a word $x_{i_1}\cdots x_{i_r}$ to the "opposite word" $x^{\text{op}}:=x_{i_r}\cdots x_{i_1}$.
Apply this automorphism to the relations of $A$ to obtain the "opposite relations"
```math
x_ix_j = c_{ij}x_jx_i+d_{ij}^{\text{\;\!op}}.
```
Also, if $\alpha=(\alpha_1, \ldots, \alpha_n)\in \N^n$, then set $\alpha^{\text{op}} =(\alpha_n, \ldots, \alpha_1)\in \N^n$,
and if $>$ is an admissible monomial ordering for $A$, then define the "opposite ordering" $ >^{\text{op}}$ by setting
```math
\alpha >^{\text{op}}  \beta \;\Leftrightarrow\;  \alpha^{\text{op}} > \beta^{\text{op}}.
```
Finally, reverse the order of the variables: set $x ^{\text{op}}=\{x_n, \ldots, x_1\}$, and consider the free associative $K$-algebra
$K \langle x^\text{op}\rangle = K \langle x_{n},\dots, x_{1} \rangle.$ Altogether, we obtain a PBW-algebra which can be naturally
identified with $A^{\text{op}}$:
```math
A ^{\text{op}}  = K\langle x_n, \dots , x_1 \mid x_ix_j = c_{ij}x_jx_i+d_{ij}^{\text{\;\!op}},  \ 1\leq i<j \leq n \rangle,
```
with admissible ordering $>^{\text{op}}$.

When implementing functionality for PBW-algebras, taking the opposite algebra into account often allows
one to focus on left ideals, left modules, and left Gröbner bases: Given a PBW-algebra $A$, right Gröbner
bases in $A$ are found by computing left Gröbner bases in $A^{\text{op}}$. Here, Gröbner bases are
considered with respect to an admissible ordering $>$ for $A$ and the opposite ordering $>^{\text{op}}$
for $A^{\text{op}}$, respectively. Each left Gröbner basis of  a two-sided ideal $I$ is also a right Gröbner
basis of $I$. Moreover, there is an algorithm which, starting from a left Gröbner basis
of $I$, computes a two-sided Gröbner basis of $I$ (see, for example,  [DL06](@cite)). 

