export abelian_d10_pi6
export abelian_d15_pi21_quintic_1
export abelian_d15_pi21_quintic_3
export bielliptic_d10_pi6
export bielliptic_d15_pi21
export bordiga
export castelnuovo
export cubic_scroll
export elliptic_d7_pi6
export elliptic_d8_pi7
export elliptic_d9_pi7
export elliptic_d10_pi9
export elliptic_d10_pi10
export elliptic_d11_pi12
export elliptic_d12_pi13
export elliptic_d12_pi14_ss_0
export elliptic_d12_pi14_ss_inf
export enriques_d9_pi6
export enriques_d10_pi8
export enriques_d11_pi10
export enriques_d13_pi16
export enriques_d13_pi16_two
export k3_d7_pi5
export k3_d8_pi6
export k3_d9_pi8
export k3_d10_pi9_quart_1
export k3_d10_pi9_quart_2
export k3_d11_pi11_ss_0
export k3_d11_pi11_ss_1
export k3_d11_pi11_ss_2
export k3_d11_pi11_ss_3
export k3_d11_pi12
export k3_d12_pi14
export k3_d13_pi16
export k3_d14_pi19
export quintic_elliptic_scroll
export rational_d7_pi4
export rational_d8_pi5
export rational_d8_pi6
export rational_d9_pi6
export rational_d9_pi7
export rational_d10_pi8
export rational_d10_pi9_quart_1
export rational_d10_pi9_quart_2
export rational_d11_pi11_ss_0
export rational_d11_pi11_ss_1
export rational_d11_pi11_ss_inf
export veronese

###############################

@doc raw"""
    abelian_d10_pi6()

Return a smooth abelian surface in $\mathbb P^4$ with degree `10` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function abelian_d10_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal(R, [-x^3*y^2*z-7318*x^4*z^2-x^2*z^3*u-x*y^3*u^2-14636*x^2*y*z*u^2-y*z^2*u^3-7318*y^2*u^4+y^5*v-8856*x^2*y*z^2*v+z^5*v-8856*y^2*z*u^2*v-7318*x^3*y*v^2-F(99)/52*x*z^3*v^2-F(99)/52*y^3*u*v^2-4*x*y*z*u*v^2-7318*z*u^3*v^2-8856*x*y^2*v^3-8856*z^2*u*v^3-F(99)/52*y*z*v^4+7318*x*u*v^4+v^6,-x*y^2*z*u^2-7318*x^2*z^2*u^2-z^3*u^3-7318*y*z*u^4+y^4*z*v+7318*x*y^2*z^2*v-7318*z^4*u*v-8856*y*z^2*u^2*v+x^2*y^2*v^2-F(99)/52*y^2*z*u*v^2-x*z^2*u*v^2+7318*x*y*u^2*v^2+7318*y^3*v^3+y*u*v^4,-x*y^2*u^3-7318*x^2*z*u^3-z^2*u^4-7318*y*u^5+y^4*u*v+7318*x*y^2*z*u*v-7318*z^3*u^2*v-8856*y*z*u^3*v-7318*x^3*u*v^2-F(99)/52*y^2*u^2*v^2-2*x*z*u^2*v^2-y^2*z*v^3-7318*x*z^2*v^3-8856*x*y*u*v^3-x^2*v^4-7318*y*v^5,y^5*z+7318*x*y^3*z^2-7318*y*z^4*u+x^3*y*u^2-8856*y^2*z^2*u^2-F(99)/52*x*y*z*u^3+7318*x^2*u^4+z*u^5+x^2*y^3*v-F(99)/52*y^3*z*u*v-x*y*z^2*u*v+14636*x*y^2*u^2*v+7318*z^2*u^3*v+7318*y^4*v^2+x*u^3*v^2+y^2*u*v^3,y^4*z^2+7318*x*y^2*z^3-7318*z^5*u+x^3*z*u^2-8856*y*z^3*u^2-F(99)/52*x*z^2*u^3-x*y*u^4-7318*u^6+2*x^2*y^2*z*v+7318*x^3*z^2*v-F(99)/52*y^2*z^2*u*v+y^3*u^2*v-2719*x*y*z*u^2*v-8856*z*u^4*v+x^4*v^2+7318*y^3*z*v^2-F(99)/52*x^2*z*u*v^2-F(99)/52*y*u^3*v^2+7318*x^2*y*v^3-8856*x*u^2*v^3-7318*u*v^5,-x*y^2*z^3-7318*x^2*z^4-z^5*u-7318*y*z^3*u^2-x^3*z^2*v+F(99)/52*x*z^3*u*v+x*y*z*u^2*v+7318*z*u^4*v-y^3*z*v^2-14636*x*y*z^2*v^2+8856*z^2*u^2*v^2-x^2*y*v^3+F(99)/52*y*z*u*v^3-7318*x*u^2*v^3-7318*y^2*v^4-u*v^5,y^3*z^3+7318*x*y*z^4-x^4*y*u+7318*y^4*z*u+8856*x*y^2*z^2*u+F(99)/52*x^2*y*z*u^2-7318*x^3*u^3-x*z*u^4+x^2*y*z^2*v-7318*x^2*y^2*u*v+y^2*z*u^2*v-7318*x*z^2*u^2*v+7318*y^2*z^2*v^2-x^2*u^2*v^2,-y^2*z^4-7318*x*z^5+x^4*z*u-7318*y^3*z^2*u-8856*x*y*z^3*u-F(99)/52*x^2*z^2*u^2-x^2*y*u^3-7318*x*u^5-x^2*z^3*v+7318*x^2*y*z*u*v-2*y*z^2*u^2*v-7318*y^2*u^3*v-8856*x*z*u^3*v-7318*y*z^3*v^2-u^4*v^2-7318*z*u^2*v^3,y^6+7318*x*y^4*z-7318*y^2*z^3*u-7318*x^4*u^2-8856*y^3*z*u^2-F(99)/52*x*y^2*u^3-x^2*z*u^3+y*u^5-7318*x^3*y^2*v-F(99)/52*y^4*u*v-4*x*y^2*z*u*v-14636*x^2*z^2*u*v-8856*x^2*y*u^2*v-z^3*u^2*v-8856*x*y^3*v^2-7318*z^4*v^2-x^3*u*v^2-8856*y*z^2*u*v^2-F(99)/52*y^2*z*v^3-x*z^2*v^3+y*v^5,x^4*y^2-7318*y^5*z-8856*x*y^3*z^2-7318*z^6-F(99)/52*x^2*y^2*z*u+x^3*z^2*u-8856*y*z^4*u+7318*x^3*y*u^2-F(99)/52*x*z^3*u^2-7318*z*u^5+7318*x^2*y^3*v-F(99)/52*y^2*z^3*v-x*z^4*v-2719*x*y*z^2*u*v-8856*z^2*u^3*v+2*x^2*y*u*v^2-F(99)/52*y*z*u^2*v^2+7318*x*u^3*v^2+y*z^2*v^3+7318*y^2*u*v^3+u^2*v^4,-x^3*y^3-7318*x^4*y*z-x^2*y*z^2*u-7318*x^2*y^2*u^2-7318*x*y^4*v-8856*x^2*y^2*z*v+y*z^4*v+7318*y^2*z^2*u*v-F(99)/52*x*y*z^2*v^2-x*y^2*u*v^2+7318*x^2*z*u*v^2+z^2*u^2*v^2+7318*z^3*v^3+x*z*v^4,-x^2*y^4-7318*x^3*y^2*z-2*x*y^2*z^2*u-7318*x^2*z^3*u-7318*x*y^3*u^2-z^4*u^2-7318*y*z^2*u^3-7318*y^5*v-8856*x*y^3*z*v-7318*z^5*v-8856*y*z^3*u*v-F(99)/52*y^2*z^2*v^2-x*z^3*v^2-y^3*u*v^2+7318*x*y*z*u*v^2+y*z*v^4,-x*y^5-7318*x^2*y^3*z-y^3*z^2*u-7318*y^4*u^2+7318*x^4*y*v+F(99)/52*x*y^3*u*v+x^2*y*z*u*v-y^2*u^3*v+8856*x^2*y^2*v^2-y*z^3*v^2-14636*y^2*z*u*v^2+F(99)/52*x*y*z*v^3-7318*x^2*u*v^3-z*u^2*v^3-7318*z^2*v^4-x*v^5,-x*y^4*u-7318*x^2*y^2*z*u-y^2*z^2*u^2-7318*y^3*u^3+7318*x^4*u*v+F(99)/52*x*y^2*u^2*v+x^2*z*u^2*v-y*u^4*v+x*y^2*z*v^2+7318*x^2*z^2*v^2+8856*x^2*y*u*v^2-7318*y*z*u^2*v^2+x^3*v^3+7318*x*y*v^4,-x^2*y^2*z^2-7318*x^3*z^3-x*z^4*u-7318*x*y*z^2*u^2-x^4*z*v+F(99)/52*x^2*z^2*u*v+x^2*y*u^2*v+7318*x*u^4*v-7318*x^2*y*z*v^2+y*z^2*u*v^2+7318*y^2*u^2*v^2+8856*x*z*u^2*v^2+u^3*v^3+7318*z*u*v^4,-x*y^3*z-7318*x^2*y*z^2-y*z^3*u-7318*y^2*z*u^2-x^3*y*v+F(99)/52*x*y*z*u*v-7318*x^2*u^2*v-z*u^3*v-7318*x*y^2*v^2-7318*z^2*u*v^2-x*u*v^3,7318*x^5+7318*y^5+8856*x*y^3*z+7318*z^5+F(99)/52*x^2*y^2*u+8856*y*z^3*u+F(99)/52*x*z^2*u^2+7318*u^5+8856*x^3*y*v+F(99)/52*y^2*z^2*v-4599*x*y*z*u*v+8856*z*u^3*v+F(99)/52*x^2*z*v^2+F(99)/52*y*u^2*v^2+8856*x*u*v^3+7318*v^5,-x^2*y^2*u-7318*x^3*z*u-x*z^2*u^2-7318*x*y*u^3-y^2*z^2*v-7318*x*z^3*v-7318*y^3*u*v-8856*x*y*z*u*v-x^2*z*v^2-y*u^2*v^2-7318*y*z*v^3,-x^5-y^5+8856*x^2*y*z^2-z^5+F(99)/52*x^3*z*u+8856*y^2*z*u^2+F(99)/52*x*y*u^3-u^5+F(99)/52*x*z^3*v+F(99)/52*y^3*u*v+5*x*y*z*u*v+8856*x^2*u^2*v+8856*x*y^2*v^2+8856*z^2*u*v^2+F(99)/52*y*z*v^3-v^5]))
end

@doc raw"""
    abelian_d15_pi21_quintic_3()

Return a smooth abelian surface in $\mathbb P^4$ with degree `15` and sectional genus `21` which is contained in a net of quintics.

The returned surface is defined over a prime field of characteristic 31991.
"""
function abelian_d15_pi21_quintic_3()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^4*y^2*z+9877*y^6*u-9533*x*y^4*z*u+F(5)/8*x^2*y^2*z^2*u+9877*y*z^5*u-9533*y^2*z^3*u^2+F(5)/8*y^3*z*u^3-x*y*z^2*u^3+9877*x^2*z*u^4+9877*y*u^6-x*y^5*v-9877*x^3*y*z^2*v-y^3*z^2*u*v+F(1)/122*x*y^2*z*u^2*v-9533*y*z*u^4*v-x*u^5*v+9877*y^4*z*v^2+2*x^2*y*z*u*v^2+F(5)/8*y*z^2*u^2*v^2-9877*x*z^2*u*v^3+z*u^2*v^4,x^3*y^2*z*u-9877*x^4*y*u^2-y*z^2*u^4+9877*x*z*u^5+x*y^2*z^3*v-y^5*u*v+12237*x^2*y*z^2*u*v+9533*x^2*y^2*u^2*v+9877*y^2*z*u^3*v-F(5)/8*x*y*u^4*v-u^6*v-9877*y*z^4*v^2+9533*y^2*z^2*u*v^2-F(5)/8*y^3*u^2*v^2+3*x*y*z*u^2*v^2+9877*x*y^2*u*v^3+9533*y*u^3*v^3-x^2*y*v^4-F(5)/8*y*z*u*v^4+9877*x*z*v^5-u*v^6,x*y^2*z^4-y^5*z*u-9877*x^2*y*z^3*u+9877*x*y^4*u^2-9877*y*z^5*v-x^2*y^3*u*v+9533*y^2*z^3*u*v-F(5)/8*y^3*z*u^2*v+2*x*y*z^2*u^2*v-9877*x^2*z*u^3*v-9877*y*u^5*v-12237*x*y^2*z*u*v^2+9533*y*z*u^3*v^2+x*u^4*v^2-x^2*y*z*v^3-F(5)/8*y*z^2*u*v^3+9877*x*z^2*v^4-z*u*v^5,x*y*z^5-y^4*z^2*u-9877*x^2*z^4*u+9877*x*y^3*z*u^2-9877*z^6*v-2*x^2*y^2*z*u*v+9533*y*z^4*u*v+9877*x^3*y*u^2*v-F(5)/8*y^2*z^2*u^2*v+x*z^3*u^2*v-9877*z*u^5*v-x^4*u*v^2-F(1)/122*x*y*z^2*u*v^2-F(5)/8*x^2*z*u^2*v^2+9533*z^2*u^3*v^2-F(5)/8*z^3*u*v^3+y*z*u^2*v^3-9877*y^2*u*v^4+9533*x*z*u*v^4+x*y*v^5-9877*z*v^6,-y^6*z-y*z^6+9877*x*y^5*u+9877*x*z^5*u-x^2*y^4*v-9877*x^4*z^2*v+9533*y^3*z^3*v-F(5)/8*x*y*z^4*v-F(5)/8*y^4*z*u*v+3*x*y^2*z^2*u*v+12237*x^2*y*z*u^2*v-z^4*u^2*v-9877*y^2*u^4*v+9877*x*y^3*z*v^2+9533*x^2*y*z^2*v^2+x^3*z*u*v^2+9877*y*z^3*u*v^2+9533*y^2*z*u^2*v^2+x*y*u^3*v^2-F(5)/8*y^2*z^2*v^3-y*z*v^5,9877*x^5+9877*y^5-9533*x*y^3*z+F(5)/8*x^2*y*z^2+9877*z^5-9533*y*z^3*u+F(5)/8*y^2*z*u^2+9877*u^5-9533*x^3*y*v-F(86)/103*x*y*z*u*v+F(5)/8*x^2*u^2*v-9533*z*u^3*v+F(5)/8*x*y^2*v^2+F(5)/8*z^2*u*v^2-9533*x*u*v^3+9877*v^5,x*y^3*z-9877*x^2*y^2*u+y*z^3*u-9877*x*z^2*u^2+x^3*y*v-9877*y^2*z^2*v+F(5)/8*x*y*z*u*v+z*u^3*v-9877*x^2*z*v^2-9877*y*u^2*v^2+x*u*v^3,-x^2*y*z^2+9877*x^3*z*u-y^2*z*u^2+9877*x*y*u^3+9877*x*z^3*v+9877*y^3*u*v-9533*x*y*z*u*v-x^2*u^2*v-x*y^2*v^2-z^2*u*v^2+9877*y*z*v^3,-x^5-y^5-z^5+9533*x^2*y^2*u-F(5)/8*x^3*z*u+9533*x*z^2*u^2-F(5)/8*x*y*u^3-u^5+9533*y^2*z^2*v-F(5)/8*x*z^3*v-F(5)/8*y^3*u*v+5*x*y*z*u*v+9533*x^2*z*v^2+9533*y*u^2*v^2-F(5)/8*y*z*v^3-v^5]))
end


@doc raw"""
    abelian_d15_pi21_quintic_1()

Return a smooth abelian surface in $\mathbb P^4$ with degree `15` and sectional genus `21` which is contained in precisely one quintic.

The returned surface is defined over a prime field of characteristic 31991.
"""
function abelian_d15_pi21_quintic_1()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(5)/8*x^3*y^3+2992*x^4*y*z-F(121)/98*x*y^2*z^3-F(96)/29*x^2*z^4+F(121)/89*x*y^3*z*u-4353*x^2*y*z^2*u+F(1)/2*z^5*u-F(8)/99*x^2*y^2*u^2-1983*x^3*z*u^2-117*y*z^3*u^2+F(6)/35*y^2*z*u^3-F(2)/5*x*z^2*u^3-1983*x*y*u^4+2992*x*y^4*v+F(96)/79*x^2*y^2*z*v+F(1)/2*x^3*z^2*v-F(5)/8*y*z^4*v+F(121)/89*x^3*y*u*v+F(19)/62*y^2*z^2*u*v-6369*x*z^3*u*v-1983*y^3*u^2*v-F(17)/42*x*y*z*u^2*v+F(6)/35*x^2*u^3*v-F(23)/11*z*u^4*v+F(1)/2*y^3*z*v^2+F(79)/60*x*y*z^2*v^2-4353*x*y^2*u*v^2+F(19)/62*x^2*z*u*v^2+14197*z^2*u^2*v^2-F(2)/5*y*u^3*v^2-F(121)/98*x^2*y*v^3+F(120)/29*z^3*v^3-6369*y*z*u*v^3-117*x*u^2*v^3-F(96)/29*y^2*v^4-F(5)/8*x*z*v^4+F(1)/2*u*v^5,F(1)/2*y^5*z-117*x*y^3*z^2+F(6)/35*x^2*y*z^3-F(5)/8*x*y^4*u+F(19)/62*x^2*y^2*z*u-1983*x^3*z^2*u-F(23)/11*y*z^4*u+F(1)/2*x^3*y*u^2+14197*y^2*z^2*u^2-F(2)/5*x*z^3*u^2+F(120)/29*y^3*u^3-6369*x*y*z*u^3-F(96)/29*x^2*u^4+F(1)/2*z*u^5-F(121)/98*x^2*y^3*v+F(121)/89*x^3*y*z*v-F(2)/5*y^2*z^3*v-1983*x*z^4*v+2992*x^4*u*v-6369*y^3*z*u*v-F(17)/42*x*y*z^2*u*v+F(79)/60*x*y^2*u^2*v-4353*x^2*z*u^2*v-117*z^2*u^3*v-F(5)/8*y*u^4*v-F(96)/29*y^4*v^2-4353*x*y^2*z*v^2-F(8)/99*x^2*z^2*v^2+F(96)/79*x^2*y*u*v^2+F(6)/35*z^3*u*v^2+F(19)/62*y*z*u^2*v^2-F(121)/98*x*u^3*v^2+F(5)/8*x^3*v^3-1983*y*z^2*v^3+F(1)/2*y^2*u*v^3+F(121)/89*x*z*u*v^3+2992*x*y*v^4,-x^2*y^3*z+1611*x^3*y*z^2+F(65)/99*y^2*z^4-F(21)/121*x*z^5+F(5)/4*x^3*y^2*u-F(19)/31*x^4*z*u-F(21)/121*y^3*z^2*u-10148*x*y*z^3*u-F(46)/11*x*y^2*z*u^2+3471*x^2*z^2*u^2-F(16)/99*x^2*y*u^3+F(97)/93*z^3*u^3+F(12)/35*y*z*u^4-F(21)/121*x*u^5-x^4*y*v-F(52)/99*y^4*z*v+F(122)/99*x*y^2*z^2*v-F(16)/99*x^2*z^3*v+F(13)/59*x*y^3*u*v+F(124)/101*x^2*y*z*u*v+F(12)/35*z^4*u*v+1611*x^3*u^2*v+9919*y*z^2*u^2*v+F(3)/7*y^2*u^3*v-10148*x*z*u^3*v+234*x^2*y^2*v^2+F(5)/4*x^3*z*v^2+F(3)/7*y*z^3*v^2+F(11)/4*y^2*z*u*v^2-F(46)/11*x*z^2*u*v^2+F(122)/99*x*y*u^2*v^2+F(65)/99*u^4*v^2-F(12)/35*y^3*v^3+F(13)/59*x*y*z*v^3-x^2*u*v^3-F(21)/121*z*u^2*v^3-F(52)/99*y*u*v^4,y^4*z^2-234*x*y^2*z^3+F(12)/35*x^2*z^4-F(90)/121*x*y^3*z*u+57*x^2*y*z^2*u-F(46)/11*z^5*u-F(99)/52*x^2*y^2*u^2-650*x^3*z*u^2-F(109)/4*y*z^3*u^2-2683*y^2*z*u^3-4373*x*z^2*u^3+F(16)/65*x*y*u^4+1611*u^6-F(72)/49*x^2*y^2*z*v+10613*x^3*z^2*v+6526*y*z^4*v-F(90)/121*x^3*y*u*v-3237*y^2*z^2*u*v-7261*x*z^3*u*v-650*y^3*u^2*v-F(4)/103*x*y*z*u^2*v-2683*x^2*u^3*v+F(11)/126*z*u^4*v+x^4*v^2+10613*y^3*z*v^2+3249*x*y*z^2*v^2+57*x*y^2*u*v^2-3237*x^2*z*u*v^2-F(70)/57*z^2*u^2*v^2-4373*y*u^3*v^2-234*x^2*y*v^3+9744*z^3*v^3-7261*y*z*u*v^3-F(109)/4*x*u^2*v^3+F(12)/35*y^2*v^4+6526*x*z*v^4-F(46)/11*u*v^5,-F(4)/5*y^6-F(113)/79*x*y^4*z-11797*x^2*y^2*z^2+F(96)/29*x^3*z^3+13879*x^2*y^3*u-15044*x^3*y*z*u-12643*y^2*z^3*u-F(1)/2*x*z^4*u+F(21)/121*x^4*u^2-F(121)/23*y^3*z*u^2+F(17)/38*x*y*z^2*u^2+3639*x*y^2*u^3+5300*x^2*z*u^3-F(5)/8*z^2*u^4+F(93)/109*y*u^5-12643*x^3*y^2*v-F(1)/2*x^4*z*v+13879*y^3*z^2*v-15044*x*y*z^3*v-F(69)/31*y^4*u*v-9826*x*y^2*z*u*v-F(22)/29*x^2*z^2*u*v-F(100)/23*x^2*y*u^2*v+9933*z^3*u^2*v-F(7)/80*y*z*u^3*v+3908*x*u^4*v-F(121)/23*x*y^3*v^2+F(17)/38*x^2*y*z*v^2+F(21)/121*z^4*v^2+9933*x^3*u*v^2-F(100)/23*y*z^2*u*v^2+F(49)/53*y^2*u^2*v^2+2111*x*z*u^2*v^2+3639*y^2*z*v^3+5300*x*z^2*v^3-F(7)/80*x*y*u*v^3-12076*u^3*v^3-F(5)/8*x^2*v^4+3908*z*u*v^4+F(93)/109*y*v^5,-F(5)/8*x^2*y^4+9933*x^3*y^2*z+F(21)/121*x^4*z^2-12076*y^3*z^3+3908*x*y*z^4-F(1)/2*x^4*y*u+3908*y^4*z*u+2111*x*y^2*z^2*u+5300*x^2*z^3*u+5300*x*y^3*u^2-F(22)/29*x^2*y*z*u^2-F(5)/8*z^4*u^2+F(96)/29*x^3*u^3+9933*y*z^2*u^3+F(21)/121*y^2*u^4-F(1)/2*x*z*u^4+F(93)/109*y^5*v-F(7)/80*x*y^3*z*v-F(100)/23*x^2*y*z^2*v+F(93)/109*z^5*v+F(17)/38*x^2*y^2*u*v-15044*x^3*z*u*v-F(7)/80*y*z^3*u*v-F(100)/23*y^2*z*u^2*v+F(17)/38*x*z^2*u^2*v-15044*x*y*u^3*v-12643*x^3*y*v^2+F(49)/53*y^2*z^2*v^2+3639*x*z^3*v^2+3639*y^3*u*v^2-9826*x*y*z*u*v^2-11797*x^2*u^2*v^2-12643*z*u^3*v^2-F(121)/23*x*y^2*v^3+13879*x^2*z*v^3-F(121)/23*z^2*u*v^3+13879*y*u^2*v^3-F(69)/31*y*z*v^4-F(113)/79*x*u*v^4-F(4)/5*v^6,F(33)/122*x^3*y^2*z+4022*x^4*z^2+F(52)/99*y^3*z^3-F(122)/117*x*y*z^4-F(52)/99*x^4*y*u-F(122)/117*y^4*z*u+F(83)/26*x*y^2*z^2*u+F(89)/58*x^2*z^3*u+F(89)/58*x*y^3*u^2+F(99)/70*x^2*y*z*u^2-F(12)/35*x^3*u^3+F(33)/122*y*z^2*u^3+4022*y^2*u^4-F(52)/99*x*z*u^4-F(89)/58*y^5*v-F(100)/71*x*y^3*z*v+F(78)/101*x^2*y*z^2*v-F(89)/58*z^5*v+F(63)/94*x^2*y^2*u*v-F(17)/31*x^3*z*u*v-F(100)/71*y*z^3*u*v+F(78)/101*y^2*z*u^2*v+F(63)/94*x*z^2*u^2*v-F(17)/31*x*y*u^3*v-2602*x^3*y*v^2+11793*y^2*z^2*v^2+8051*x*z^3*v^2+8051*y^3*u*v^2+12945*x*y*z*u*v^2+F(108)/109*x^2*u^2*v^2-2602*z*u^3*v^2-2520*x*y^2*v^3-F(95)/52*x^2*z*v^3-2520*z^2*u*v^3-F(95)/52*y*u^2*v^3+8051*y*z*v^4-F(65)/83*x*u*v^4-F(89)/58*v^6,-F(89)/58*y^6-F(65)/83*x*y^4*z+F(108)/109*x^2*y^2*z^2-F(12)/35*x^3*z^3-F(95)/52*x^2*y^3*u-F(17)/31*x^3*y*z*u-2602*y^2*z^3*u-F(52)/99*x*z^4*u+4022*x^4*u^2-2520*y^3*z*u^2+F(63)/94*x*y*z^2*u^2+8051*x*y^2*u^3+F(89)/58*x^2*z*u^3-F(89)/58*y*u^5-2602*x^3*y^2*v-F(52)/99*x^4*z*v-F(95)/52*y^3*z^2*v-F(17)/31*x*y*z^3*v+8051*y^4*u*v+12945*x*y^2*z*u*v+F(99)/70*x^2*z^2*u*v+F(78)/101*x^2*y*u^2*v+F(33)/122*z^3*u^2*v-F(100)/71*y*z*u^3*v-F(122)/117*x*u^4*v-2520*x*y^3*v^2+F(63)/94*x^2*y*z*v^2+4022*z^4*v^2+F(33)/122*x^3*u*v^2+F(78)/101*y*z^2*u*v^2+11793*y^2*u^2*v^2+F(83)/26*x*z*u^2*v^2+8051*y^2*z*v^3+F(89)/58*x*z^2*v^3-F(100)/71*x*y*u*v^3+F(52)/99*u^3*v^3-F(122)/117*z*u*v^4-F(89)/58*y*v^5,x^4*y^2+1611*y^5*z-F(77)/60*x*y^3*z^2+F(76)/69*x^2*y*z^3+1611*z^6-F(4)/5*x*y^4*u-15475*x^2*y^2*z*u+F(86)/19*x^3*z^2*u-F(77)/60*y*z^4*u-11038*x^3*y*u^2+F(58)/25*y^2*z^2*u^2+F(112)/43*x*z^3*u^2-F(21)/121*y^3*u^3-11234*x*y*z*u^3+1611*z*u^5-1611*x^2*y^3*v+F(55)/73*x^3*y*z*v+F(112)/43*y^2*z^3*v-11235*x*z^4*v+F(21)/121*x^4*u*v-11234*y^3*z*u*v-F(43)/2*x*y*z^2*u*v-12177*x*y^2*u^2*v+F(18)/107*x^2*z*u^2*v-F(77)/60*z^2*u^3*v-F(4)/5*y*u^4*v+F(18)/107*x*y^2*z*v^2+10973*x^2*z^2*v^2+2311*x^2*y*u*v^2+F(76)/69*z^3*u*v^2-15475*y*z*u^2*v^2-1611*x*u^3*v^2-F(65)/99*x^3*v^3+F(86)/19*y*z^2*v^3-11038*y^2*u*v^3+F(55)/73*x*z*u*v^3+F(21)/121*x*y*v^4+u^2*v^4,-x*y^5+234*x^2*y^3*z-F(12)/35*x^3*y*z^2+F(4)/5*x^3*y^2*u+F(21)/121*x^4*z*u+F(121)/49*y^3*z^2*u+F(25)/123*x*y*z^3*u+11038*y^4*u^2+8706*x*y^2*z*u^2+F(16)/99*x^2*z^2*u^2+F(21)/121*x^2*y*u^3-F(5)/4*z^3*u^3-5984*y*z*u^4+F(46)/11*x^4*y*v+F(5)/4*y^4*z*v-F(19)/31*x*y^2*z^2*v+F(21)/121*x^2*z^3*v+F(91)/108*x*y^3*u*v+F(17)/21*x^2*y*z*u*v-5984*z^4*u*v-F(12)/35*x^3*u^2*v+12551*y*z^2*u^2*v-y^2*u^3*v+F(25)/123*x*z*u^3*v+3597*x^2*y^2*v^2+F(4)/5*x^3*z*v^2-y*z^3*v^2-F(79)/30*y^2*z*u*v^2+8706*x*z^2*u*v^2-F(19)/31*x*y*u^2*v^2+2198*y^3*v^3+F(91)/108*x*y*z*v^3+234*x^2*u*v^3+F(121)/49*z*u^2*v^3+11038*z^2*v^4+F(5)/4*y*u*v^4-x*v^5,-x^5-y^5+7884*x*y^3*z-F(99)/23*x^2*y*z^2-z^5-3570*x^2*y^2*u+F(3)/100*x^3*z*u+7884*y*z^3*u-F(99)/23*y^2*z*u^2-3570*x*z^2*u^2+F(3)/100*x*y*u^3-u^5+7884*x^3*y*v-3570*y^2*z^2*v+F(3)/100*x*z^3*v+F(3)/100*y^3*u*v+F(125)/81*x*y*z*u*v-F(99)/23*x^2*u^2*v+7884*z*u^3*v-F(99)/23*x*y^2*v^2-3570*x^2*z*v^2-F(99)/23*z^2*u*v^2-3570*y*u^2*v^2+F(3)/100*y*z*v^3+7884*x*u*v^3-v^5]))
end


@doc raw"""
    bielliptic_d10_pi6()

Return a smooth bielliptic surface in $\mathbb P^4$ with degree `10` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 911.
"""
function bielliptic_d10_pi6()
   F = GF(911)  # SPECIAL
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(3)/20*x^2*y^4+72*x^3*y^2*z-F(21)/8*x^4*z^2-221*y^3*z^3+F(6)/19*x*y*z^4-12*x^4*y*u+189*y^4*z*u+F(9)/17*x*y^2*z^2*u-F(19)/16*x^2*z^3*u+F(13)/5*x*y^3*u^2+412*x^2*y*z*u^2-F(8)/11*z^4*u^2-291*x^3*u^3-F(19)/13*y*z^2*u^3+278*y^2*u^4+10*x*z*u^4+F(2)/3*y^5*v-F(3)/20*x*y^3*z*v+F(7)/3*x^2*y*z^2*v-187*z^5*v-F(6)/17*x^2*y^2*u*v+F(4)/21*x^3*z*u*v+F(7)/15*y*z^3*u*v+F(11)/4*y^2*z*u^2*v-F(1)/6*x*z^2*u^2*v-371*x*y*u^3*v+F(10)/19*u^5*v+373*x^3*y*v^2+345*y^2*z^2*v^2-F(10)/19*x*z^3*v^2+222*y^3*u*v^2-F(18)/7*x*y*z*u*v^2+F(21)/16*x^2*u^2*v^2-30*z*u^3*v^2-F(15)/16*x*y^2*v^3+423*x^2*z*v^3-31*z^2*u*v^3-F(7)/8*y*u^2*v^3+F(2)/9*y*z*v^4-F(18)/5*x*u*v^4+142*v^6,68*x*y^5+F(9)/2*x^2*y^3*z+68*x^3*y*z^2+F(17)/15*y^2*z^4-F(15)/16*x*z^5-F(11)/4*x^3*y^2*u-F(7)/13*x^4*z*u-394*y^3*z^2*u+F(11)/7*x*y*z^3*u-F(3)/19*y^4*u^2-F(14)/17*x*y^2*z*u^2+F(3)/13*x^2*z^2*u^2+315*x^2*y*u^3+F(7)/3*z^3*u^3+F(6)/5*y*z*u^4-25*x*u^5-F(9)/8*x^4*y*v+F(15)/8*y^4*z*v-F(3)/4*x*y^2*z^2*v+F(5)/6*x^2*z^3*v-35*x*y^3*u*v+18*x^2*y*z*u*v+F(13)/17*z^4*u*v+327*x^3*u^2*v+371*y*z^2*u^2*v-F(11)/2*y^2*u^3*v+145*x*z*u^3*v+377*x^2*y^2*v^2-67*x^3*z*v^2-30*y*z^3*v^2-F(1)/12*y^2*z*u*v^2+F(13)/6*x*z^2*u*v^2+123*x*y*u^2*v^2-313*u^4*v^2+F(19)/3*y^3*v^3+F(21)/11*x*y*z*v^3-271*x^2*u*v^3+F(13)/3*z*u^2*v^3+282*z^2*v^4-291*y*u*v^4+F(11)/4*x*v^5,68*x*y^4*z+F(9)/2*x^2*y^2*z^2+F(5)/13*x^3*z^3+168*y*z^5-F(11)/4*x^3*y*z*u+430*y^2*z^3*u+F(17)/7*x*z^4*u-395*x^4*u^2-27*y^3*z*u^2+F(19)/16*x*y*z^2*u^2-F(17)/16*x*y^2*u^3+F(16)/21*x^2*z*u^3-442*z^2*u^4+F(3)/4*y*u^5+F(12)/17*x^3*y^2*v+F(10)/17*x^4*z*v-F(9)/8*y^3*z^2*v-F(2)/5*x*y*z^3*v+F(14)/9*y^4*u*v+F(19)/15*x*y^2*z*u*v+125*x^2*z^2*u*v-105*x^2*y*u^2*v+110*z^3*u^2*v+67*y*z*u^3*v+312*x*u^4*v+F(16)/13*x*y^3*v^2+289*x^2*y*z*v^2+163*z^4*v^2-F(1)/2*x^3*u*v^2+F(2)/11*y*z^2*u*v^2-245*y^2*u^2*v^2-F(12)/13*x*z*u^2*v^2-F(5)/4*y^2*z*v^3+400*x*z^2*v^3-F(5)/3*x*y*u*v^3-370*u^3*v^3-F(8)/9*x^2*v^4+34*z*u*v^4-111*y*v^5,68*x*y^4*u+F(9)/2*x^2*y^2*z*u+F(5)/13*x^3*z^2*u+168*y*z^4*u-31*x^3*y*u^2+F(2)/9*y^2*z^2*u^2-30*x*z^3*u^2-F(5)/4*y^3*u^3+F(2)/7*x*y*z*u^3-F(11)/6*x^2*u^4-168*z*u^5+176*x^2*y^3*v-F(5)/8*x^3*y*z*v-55*y^2*z^3*v-357*x*z^4*v+F(11)/7*x^4*u*v-F(6)/13*y^3*z*u*v+289*x*y*z^2*u*v-218*x*y^2*u^2*v+F(3)/13*x^2*z*u^2*v-F(20)/13*z^2*u^3*v-F(10)/19*y*u^4*v-F(17)/8*y^4*v^2-266*x*y^2*z*v^2+358*x^2*z^2*v^2-74*x^2*y*u*v^2+371*z^3*u*v^2-F(15)/16*y*z*u^2*v^2+40*x*u^3*v^2+F(19)/11*x^3*v^3+F(8)/15*y*z^2*v^3+F(1)/15*y^2*u*v^3+F(16)/11*x*z*u*v^3-F(19)/12*x*y*v^4-41*u^2*v^4-F(4)/19*z*v^5,74*x^3*y^3+F(14)/13*x^4*y*z-444*y^4*z^2+22*x*y^2*z^3-F(10)/19*y^5*u-F(13)/20*x*y^3*z*u+F(19)/16*x^2*y*z^2*u+436*z^5*u-24*x^2*y^2*u^2+15*x^3*z*u^2+F(5)/6*y*z^3*u^2-F(11)/7*y^2*z*u^3+F(10)/13*x*z^2*u^3-F(19)/9*x*y*u^4-F(10)/21*u^6-241*x*y^4*v+26*x^2*y^2*z*v-F(4)/11*x^3*z^2*v+377*y*z^4*v-251*x^3*y*u*v-439*y^2*z^2*u*v+F(4)/17*x*z^3*u*v+F(19)/14*y^3*u^2*v+F(5)/21*x*y*z*u^2*v+F(8)/11*x^2*u^3*v-F(3)/20*z*u^4*v-F(10)/21*x^4*v^2+282*y^3*z*v^2-F(11)/21*x*y*z^2*v^2-F(10)/9*x*y^2*u*v^2+F(8)/9*x^2*z*u*v^2-395*z^2*u^2*v^2-35*y*u^3*v^2+125*x^2*y*v^3+265*z^3*v^3+F(8)/19*y*z*u*v^3-F(14)/9*x*u^2*v^3+F(3)/11*y^2*v^4+117*x*z*v^4+F(2)/5*u*v^5,74*x^3*y^2*z+F(14)/13*x^4*z^2-444*y^3*z^3+22*x*y*z^4+282*x^4*y*u+F(13)/4*y^4*z*u+85*x*y^2*z^2*u-F(18)/11*x^2*z^3*u+F(10)/11*x*y^3*u^2+F(16)/17*x^2*y*z*u^2-401*z^4*u^2+F(20)/11*x^3*u^3+F(10)/3*y*z^2*u^3-F(8)/19*y^2*u^4-F(17)/10*x*z*u^4-276*y^5*v+F(1)/7*x*y^3*z*v+177*x^2*y*z^2*v-F(19)/11*z^5*v-F(1)/14*x^2*y^2*u*v-359*x^3*z*u*v+F(1)/18*y*z^3*u*v-F(21)/20*y^2*z*u^2*v-245*x*z^2*u^2*v+135*x*y*u^3*v-F(19)/18*u^5*v-F(3)/13*x^3*y*v^2+187*y^2*z^2*v^2-F(14)/15*x*z^3*v^2-F(10)/7*y^3*u*v^2-F(1)/16*x*y*z*u*v^2+F(1)/10*x^2*u^2*v^2+z*u^3*v^2-168*x*y^2*v^3-F(1)/2*x^2*z*v^3-F(1)/13*z^2*u*v^3+F(14)/11*y*u^2*v^3-F(8)/17*y*z*v^4+F(11)/21*x*u*v^4-395*v^6,F(3)/20*x^2*y^3*z+72*x^3*y*z^2-443*y^2*z^4-423*x*z^5-12*x^4*z*u+F(5)/2*y^3*z^2*u-117*x*y*z^3*u-F(19)/18*y^4*u^2-F(7)/5*x*y^2*z*u^2+F(1)/10*x^2*z^2*u^2+F(14)/13*x^2*y*u^3-F(9)/17*z^3*u^3+255*y*z*u^4-F(16)/5*x*u^5+F(19)/13*x^4*y*v-F(11)/16*y^4*z*v+278*x*y^2*z^2*v-430*x^2*z^3*v+F(1)/21*x*y^3*u*v+F(19)/8*x^2*y*z*u*v-98*z^4*u*v+F(15)/4*x^3*u^2*v-276*y*z^2*u^2*v-F(17)/10*y^2*u^3*v+F(8)/13*x*z*u^3*v-F(19)/13*x^2*y^2*v^2-438*x^3*z*v^2-F(11)/14*y*z^3*v^2-F(11)/8*y^2*z*u*v^2+283*x*z^2*u*v^2+188*x*y*u^2*v^2-3*u^4*v^2+157*y^3*v^3+F(5)/8*x*y*z*v^3-79*x^2*u*v^3+F(12)/19*z*u^2*v^3-394*z^2*v^4-24*y*u*v^4+88*x*v^5,F(3)/20*x^2*y^3*u+72*x^3*y*z*u-443*y^2*z^3*u-423*x*z^4*u-19*x^4*u^2-F(15)/13*y^3*z*u^2+F(8)/21*x*y*z^2*u^2-F(14)/9*x*y^2*u^3-F(8)/19*x^2*z*u^3+29*z^2*u^4-F(11)/19*y*u^5-311*x^3*y^2*v-F(7)/2*x^4*z*v+F(1)/18*y^3*z^2*v+52*x*y*z^3*v+F(3)/10*y^4*u*v-F(6)/17*x*y^2*z*u*v-26*x^2*z^2*u*v+199*x^2*y*u^2*v+F(1)/17*z^3*u^2*v-F(17)/16*y*z*u^3*v+F(7)/5*x*u^4*v-142*x*y^3*v^2+F(10)/7*x^2*y*z*v^2+F(11)/8*z^4*v^2-F(21)/20*x^3*u*v^2+276*y*z^2*u*v^2-F(17)/7*y^2*u^2*v^2-F(2)/11*x*z*u^2*v^2-F(1)/18*y^2*z*v^3+F(5)/21*x*z^2*v^3-F(16)/7*x*y*u*v^3+373*u^3*v^3-25*x^2*v^4-F(13)/17*z*u*v^4+F(17)/16*y*v^5,74*x^3*y*z^2-423*y^2*z^4-317*x*z^5+282*x^4*z*u+F(3)/19*y^3*z^2*u+F(17)/13*x*y*z^3*u-F(3)/8*y^4*u^2-345*x*y^2*z*u^2+F(15)/17*x^2*z^2*u^2-F(19)/13*x^2*y*u^3+283*z^3*u^3-80*y*z*u^4-F(13)/3*x*u^5+147*x^4*y*v-F(3)/7*y^4*z*v-F(1)/21*x*y^2*z^2*v-123*x^2*z^3*v+293*x*y^3*u*v-F(21)/20*x^2*y*z*u*v+F(1)/19*z^4*u*v+F(1)/4*x^3*u^2*v+283*y*z^2*u^2*v+276*y^2*u^3*v+282*x*z*u^3*v+F(9)/7*x^2*y^2*v^2+F(2)/19*x^3*z*v^2+F(7)/4*y*z^3*v^2+417*y^2*z*u*v^2+F(8)/9*x*z^2*u*v^2-F(9)/8*x*y*u^2*v^2-F(9)/14*u^4*v^2-3*y^3*v^3+251*x*y*z*v^3-119*x^2*u*v^3-F(20)/7*z*u^2*v^3-F(9)/5*z^2*v^4+264*y*u*v^4-F(7)/5*x*v^5,F(15)/11*y^6-159*x*y^4*z+F(8)/21*x^2*y^2*z^2+F(17)/2*x^2*y^3*u+235*x^3*y*z*u+135*y^2*z^3*u+F(15)/7*x*z^4*u+F(15)/2*x^4*u^2-F(7)/15*y^3*z*u^2-F(8)/3*x*y*z^2*u^2-9*x*y^2*u^3-F(5)/4*x^2*z*u^3+402*z^2*u^4-382*y*u^5-39*x^3*y^2*v-F(3)/13*x^4*z*v+F(13)/3*y^3*z^2*v+169*x*y*z^3*v+276*y^4*u*v+212*x*y^2*z*u*v-F(16)/9*x^2*z^2*u*v-F(11)/8*x^2*y*u^2*v+F(13)/16*z^3*u^2*v-F(5)/21*y*z*u^3*v-F(19)/4*x*u^4*v+F(10)/19*x*y^3*v^2-34*x^2*y*z*v^2-F(13)/16*z^4*v^2-197*x^3*u*v^2+F(10)/17*y*z^2*u*v^2+270*y^2*u^2*v^2+41*x*z*u^2*v^2-F(1)/9*y^2*z*v^3-34*x*z^2*v^3-F(4)/11*x*y*u*v^3-F(13)/3*u^3*v^3-148*x^2*v^4-F(16)/13*z*u*v^4-F(12)/19*y*v^5,F(15)/11*y^5*z-F(17)/18*x*y^3*z^2-264*x^2*y*z^3-371*z^6+F(17)/2*x^2*y^2*z*u-F(3)/14*x^3*z^2*u+17*y*z^4*u-F(19)/7*x^3*y*u^2+F(11)/16*y^2*z^2*u^2-F(2)/13*x*z^3*u^2-320*y^3*u^3+F(5)/2*x*y*z*u^3+F(17)/19*x^2*u^4+438*z*u^5+13*x^2*y^3*v-401*x^3*y*z*v-F(10)/13*y^2*z^3*v+277*x*z^4*v-255*x^4*u*v-F(18)/19*y^3*z*u*v-F(13)/4*x*y*z^2*u*v+F(16)/3*x*y^2*u^2*v+34*x^2*z*u^2*v+F(14)/13*z^2*u^3*v+F(3)/11*y*u^4*v-F(4)/11*y^4*v^2+109*x*y^2*z*v^2-265*x^2*z^2*v^2+F(20)/7*x^2*y*u*v^2+F(5)/2*z^3*u*v^2-294*y*z*u^2*v^2+3*x*u^3*v^2-371*x^3*v^3-282*y*z^2*v^3+F(5)/19*y^2*u*v^3-F(21)/19*x*z*u*v^3+20*x*y*v^4+F(5)/2*u^2*v^4+F(1)/13*z*v^5,F(15)/11*y^5*u-F(17)/18*x*y^3*z*u-264*x^2*y*z^2*u-371*z^5*u+21*x^2*y^2*u^2-F(1)/20*x^3*z*u^2+F(14)/11*y*z^3*u^2+334*y^2*z*u^3+396*x*z^2*u^3+147*x*y*u^4+F(3)/5*u^6+199*x*y^4*v+F(5)/11*x^2*y^2*z*v+F(15)/4*x^3*z^2*v+F(1)/6*y*z^4*v-F(19)/16*x^3*y*u*v-F(13)/15*y^2*z^2*u*v+F(2)/13*x*z^3*u*v+22*y^3*u^2*v-117*x*y*z*u^2*v+F(6)/17*x^2*u^3*v-401*z*u^4*v-72*x^4*v^2-395*y^3*z*v^2+F(17)/20*x*y*z^2*v^2-292*x*y^2*u*v^2+F(5)/6*x^2*z*u*v^2-F(16)/7*z^2*u^2*v^2-52*y*u^3*v^2-F(6)/7*x^2*y*v^3-443*z^3*v^3-F(10)/11*y*z*u*v^3+F(13)/17*x*u^2*v^3-F(19)/5*y^2*v^4+187*x*z*v^4+F(10)/11*u*v^5,x^4*z^2+436*y^3*z^3-401*x*y*z^4+88*x*y^2*z^2*u-F(6)/17*x^2*z^3*u+F(18)/5*x*y^3*u^2+F(9)/16*x^2*y*z*u^2-F(3)/5*z^4*u^2+F(3)/7*x^3*u^3-F(19)/15*y*z^2*u^3-73*y^2*u^4+175*x*z*u^4-94*y^5*v+F(9)/5*x*y^3*z*v-124*x^2*y*z^2*v-F(2)/3*z^5*v-F(5)/3*x^2*y^2*u*v+F(14)/17*x^3*z*u*v+F(11)/7*y*z^3*u*v+282*y^2*z*u^2*v+F(13)/18*x*z^2*u^2*v+F(14)/13*x*y*u^3*v-F(21)/13*u^5*v-F(17)/18*x^3*y*v^2-F(19)/6*y^2*z^2*v^2+F(10)/9*x*z^3*v^2+219*y^3*u*v^2-32*x*y*z*u*v^2-6*x^2*u^2*v^2-F(5)/12*z*u^3*v^2-291*x*y^2*v^3-123*x^2*z*v^3-175*z^2*u*v^3-105*y*u^2*v^3+27*y*z*v^4+276*x*u*v^4-F(2)/19*v^6,74*x^3*y^2*u+F(14)/13*x^4*z*u-444*y^3*z^2*u+22*x*y*z^3*u-338*y^4*u^2+67*x*y^2*z*u^2-F(9)/7*x^2*z^2*u^2+417*x^2*y*u^3+19*z^3*u^3+F(7)/15*y*z*u^4+158*x*u^5-F(4)/9*x^4*y*v+109*y^4*z*v-F(14)/9*x*y^2*z^2*v-F(13)/20*x^2*z^3*v+F(3)/19*x*y^3*u*v-218*x^2*y*z*u*v-F(13)/19*z^4*u*v-438*x^3*u^2*v-F(17)/7*y*z^2*u^2*v-F(18)/7*y^2*u^3*v+F(20)/3*x*z*u^3*v+F(18)/17*x^2*y^2*v^2+F(2)/21*x^3*z*v^2+F(2)/15*y*z^3*v^2-F(7)/13*y^2*z*u*v^2+F(3)/11*x*z^2*u*v^2+265*x*y*u^2*v^2+402*u^4*v^2+F(11)/9*y^3*v^3-7*x*y*z*v^3-F(11)/13*x^2*u*v^3+F(10)/9*z*u^2*v^3+3*z^2*v^4-256*y*u*v^4-F(11)/8*x*v^5,F(3)/20*x^2*y^2*z^2-317*x^3*z^3+291*y*z^5-F(16)/19*y^2*z^3*u-F(17)/18*x*z^4*u+345*x^4*u^2-F(1)/7*y^3*z*u^2-437*x*y*z^2*u^2-7*x*y^2*u^3-F(9)/16*x^2*z*u^3+F(21)/20*z^2*u^4-F(19)/7*y*u^5+251*x^3*y^2*v-F(4)/9*x^4*z*v-F(13)/6*y^3*z^2*v+438*x*y*z^3*v+F(9)/5*y^4*u*v-251*x*y^2*z*u*v-F(10)/19*x^2*z^2*u*v-F(11)/7*x^2*y*u^2*v-312*z^3*u^2*v+F(8)/11*y*z*u^3*v+F(13)/19*x*u^4*v+266*x*y^3*v^2-F(12)/11*x^2*y*z*v^2+190*z^4*v^2+156*x^3*u*v^2-320*y*z^2*u*v^2+F(3)/8*y^2*u^2*v^2+F(1)/12*x*z*u^2*v^2-35*y^2*z*v^3-119*x*z^2*v^3+F(11)/2*x*y*u*v^3-F(20)/9*u^3*v^3+F(5)/3*x^2*v^4-F(11)/2*z*u*v^4+397*y*v^5,F(3)/20*x^2*y^2*z*u-317*x^3*z^2*u+291*y*z^4*u+189*x^3*y*u^2-408*y^2*z^2*u^2+F(5)/4*x*z^3*u^2-397*y^3*u^3-175*x*y*z*u^3+238*x^2*u^4-F(13)/19*z*u^5+F(10)/13*x^2*y^3*v+266*x^3*y*z*v+F(20)/21*y^2*z^3*v-F(13)/14*x*z^4*v+337*x^4*u*v+117*y^3*z*u*v-F(20)/21*x*y*z^2*u*v-F(16)/19*x*y^2*u^2*v+356*x^2*z*u^2*v+F(18)/13*z^2*u^3*v+F(19)/8*y*u^4*v-2*y^4*v^2-F(7)/4*x*y^2*z*v^2-F(5)/17*x^2*z^2*v^2-4*x^2*y*u*v^2-334*z^3*u*v^2-F(1)/11*y*z*u^2*v^2+218*x*u^3*v^2+188*x^3*v^3-16*y*z^2*v^3-F(4)/19*y^2*u*v^3+22*x*z*u*v^3-F(7)/8*x*y*v^4+441*u^2*v^4-F(7)/15*z*v^5,x^4*y*z+F(13)/9*y^4*z^2+F(11)/9*x*y^2*z^3-16*x^2*z^4+F(9)/10*x*y^3*z*u+313*x^2*y*z^2*u-F(9)/16*z^5*u-25*x^2*y^2*u^2-251*x^3*z*u^2-327*y*z^3*u^2-190*y^2*z*u^3+F(13)/11*x*z^2*u^3+124*x*y*u^4-F(6)/5*u^6+F(2)/19*x*y^4*v-x^2*y^2*z*v-F(3)/8*x^3*z^2*v+314*y*z^4*v-F(20)/13*x^3*y*u*v-345*y^2*z^2*u*v+F(17)/18*x*z^3*u*v+F(5)/4*y^3*u^2*v+F(9)/19*x*y*z*u^2*v+F(19)/3*x^2*u^3*v-213*z*u^4*v+F(6)/19*x^4*v^2-221*y^3*z*v^2-F(19)/3*x*y*z^2*v^2-F(1)/20*x*y^2*u*v^2-F(21)/10*x^2*z*u*v^2-F(5)/8*z^2*u^2*v^2-437*y*u^3*v^2+F(2)/5*x^2*y*v^3-206*z^3*v^3+F(20)/11*y*z*u*v^3-123*x*u^2*v^3-187*y^2*v^4-412*x*z*v^4-277*u*v^5,x^4*y*u+F(13)/9*y^4*z*u+F(11)/9*x*y^2*z^2*u-16*x^2*z^3*u-111*x*y^3*u^2-F(18)/13*x^2*y*z*u^2-F(8)/5*z^4*u^2+F(3)/16*x^3*u^3-290*y*z^2*u^3+F(2)/21*y^2*u^4+F(12)/17*x*z*u^4+148*y^5*v-F(9)/5*x*y^3*z*v-F(11)/14*x^2*y*z^2*v-F(9)/7*z^5*v+F(11)/15*x^2*y^2*u*v-264*x^3*z*u*v-F(3)/2*y*z^3*u*v+2*y^2*z*u^2*v+F(2)/17*x*z^2*u^2*v+F(1)/5*x*y*u^3*v+120*u^5*v-16*x^3*y*v^2+F(1)/20*y^2*z^2*v^2+377*x*z^3*v^2-F(17)/15*y^3*u*v^2-F(4)/3*x*y*z*u*v^2+F(1)/15*x^2*u^2*v^2-55*z*u^3*v^2-F(14)/5*x*y^2*v^3+72*x^2*z*v^3-F(16)/19*z^2*u*v^3+F(11)/18*y*u^2*v^3+F(5)/18*y*z*v^4+396*x*u*v^4-F(16)/21*v^6,x^4*z*u+436*y^3*z^2*u-401*x*y*z^3*u+36*y^4*u^2-F(20)/11*x*y^2*z*u^2+F(19)/16*x^2*z^2*u^2+F(2)/13*x^2*y*u^3-F(5)/3*z^3*u^3-F(14)/15*y*z*u^4+F(21)/11*x*u^5+F(1)/12*x^4*y*v-F(20)/11*y^4*z*v+270*x*y^2*z^2*v-F(3)/2*x^2*z^3*v+437*x*y^3*u*v+F(6)/11*x^2*y*z*u*v+F(1)/7*z^4*u*v+118*x^3*u^2*v-159*y*z^2*u^2*v+235*y^2*u^3*v-F(3)/7*x*z*u^3*v-F(13)/6*x^2*y^2*v^2+F(13)/15*x^3*z*v^2-11*y*z^3*v^2-402*y^2*z*u*v^2+29*x*z^2*u*v^2+148*x*y*u^2*v^2-290*u^4*v^2-3*y^3*v^3+320*x*y*z*v^3-346*x^2*u*v^3+F(5)/7*z*u^2*v^3+F(12)/19*z^2*v^4-15*y*u*v^4-F(3)/7*x*v^5,74*x^3*y*z*u-423*y^2*z^3*u-317*x*z^4*u+F(18)/11*x^4*u^2-F(18)/17*y^3*z*u^2+11*x*y*z^2*u^2-31*x*y^2*u^3+188*x^2*z*u^3-F(19)/13*z^2*u^4+397*y*u^5-439*x^3*y^2*v-F(9)/10*x^4*z*v-387*y^3*z^2*v-265*x*y*z^3*v+F(15)/2*y^4*u*v+F(9)/4*x*y^2*z*u*v+402*x^2*z^2*u*v+F(14)/19*x^2*y*u^2*v-85*z^3*u^2*v-358*y*z*u^3*v+F(19)/11*x*u^4*v-F(10)/9*x*y^3*v^2+294*x^2*y*z*v^2-39*z^4*v^2+F(10)/7*x^3*u*v^2+197*y*z^2*u*v^2-F(19)/13*y^2*u^2*v^2+F(13)/17*x*z*u^2*v^2-346*y^2*z*v^3+74*x*z^2*v^3+F(1)/21*x*y*u*v^3-371*u^3*v^3-444*x^2*v^4+F(11)/2*z*u*v^4+F(5)/6*y*v^5,x^4*y^2+F(13)/9*y^5*z+246*x*y^3*z^2+156*x^2*y*z^3-F(17)/19*z^6-329*x*y^4*u+F(4)/3*x^2*y^2*z*u-15*x^3*z^2*u-F(2)/11*y*z^4*u+F(7)/15*x^3*y*u^2+F(15)/13*y^2*z^2*u^2+5*x*z^3*u^2+245*y^3*u^3-F(7)/6*x*y*z*u^3+F(7)/9*x^2*u^4-F(1)/12*z*u^5+F(9)/16*x^2*y^3*v-F(9)/5*x^3*y*z*v-F(13)/9*y^2*z^3*v+219*x*z^4*v+F(2)/7*x^4*u*v-F(13)/6*y^3*z*u*v+15*x*y*z^2*u*v-F(15)/4*x*y^2*u^2*v+16*x^2*z*u^2*v-377*z^2*u^3*v-148*y*u^4*v+146*y^4*v^2-F(14)/3*x*y^2*z*v^2-F(13)/10*x^2*z^2*v^2+11*x^2*y*u*v^2-207*z^3*u*v^2-F(11)/18*y*z*u^2*v^2-357*x*u^3*v^2+F(4)/5*x^3*v^3+F(7)/12*y*z^2*v^3+F(19)/17*y^2*u*v^3+F(11)/19*x*z*u*v^3-359*x*y*v^4-386*u^2*v^4-17*z*v^5,F(15)/11*y^4*z^2-159*x*y^2*z^3+F(8)/21*x^2*z^4-438*x^2*y*z^2*u-26*z^5*u-296*x^2*y^2*u^2+317*x^3*z*u^2+F(11)/17*y*z^3*u^2+F(13)/15*y^2*z*u^3+219*x*z^2*u^3-F(18)/19*x*y*u^4-F(12)/7*u^6-F(1)/19*x^2*y^2*z*v-F(2)/17*x^3*z^2*v-F(21)/5*y*z^4*v-236*x^3*y*u*v-F(13)/7*y^2*z^2*u*v+221*x*z^3*u*v+5*y^3*u^2*v+371*x*y*z*u^2*v-F(12)/13*x^2*u^3*v+F(11)/10*z*u^4*v+F(4)/9*x^4*v^2+386*y^3*z*v^2-26*x*y*z^2*v^2-187*x*y^2*u*v^2+F(19)/6*x^2*z*u*v^2+396*z^2*u^2*v^2-F(3)/14*y*u^3*v^2-315*x^2*y*v^3-39*z^3*v^3+F(2)/7*y*z*u*v^3+437*x*u^2*v^3-68*y^2*v^4-6*x*z*v^4-360*u*v^5,F(15)/11*y^4*z*u-159*x*y^2*z^2*u+F(8)/21*x^2*z^3*u-290*x*y^3*u^2+134*x^2*y*z*u^2+F(9)/5*z^4*u^2-F(15)/13*x^3*u^3-F(9)/16*y*z^2*u^3-294*y^2*u^4-F(8)/15*x*z*u^4-F(1)/15*x*y^3*z*v-105*x^2*y*z^2*v+F(13)/9*z^5*v+F(21)/19*x^2*y^2*u*v+440*x^3*z*u*v-F(19)/9*y*z^3*u*v+F(12)/5*y^2*z*u^2*v-F(13)/19*x*z^2*u^2*v-F(8)/13*x*y*u^3*v-187*u^5*v-F(15)/8*x^3*y*v^2-358*y^2*z^2*v^2+F(9)/14*x*z^3*v^2+4*y^3*u*v^2-222*x*y*z*u*v^2+F(3)/10*x^2*u^2*v^2+F(12)/13*z*u^3*v^2+14*x*y^2*v^3+26*x^2*z*v^3-F(9)/13*z^2*u*v^3+F(16)/17*y*u^2*v^3+F(3)/8*y*z*v^4-F(3)/14*x*u*v^4+F(18)/5*v^6,68*x*y^3*z^2+F(9)/16*x^2*y*z^3-F(11)/20*z^6-430*x^3*z^2*u+168*y*z^4*u-F(5)/11*x^3*y*u^2-198*y^2*z^2*u^2-16*x*z^3*u^2-10*y^3*u^3+264*x*y*z*u^3-430*x^2*u^4-271*z*u^5+382*x^2*y^3*v+F(14)/15*y^2*z^3*v+F(18)/13*x*z^4*v-435*x^4*u*v-F(5)/2*y^3*z*u*v+442*x*y*z^2*u*v-F(21)/4*x*y^2*u^2*v-88*x^2*z*u^2*v+F(14)/3*z^2*u^3*v-F(5)/7*y*u^4*v-F(11)/9*y^4*v^2-72*x*y^2*z*v^2+396*x^2*z^2*v^2+222*x^2*y*u*v^2-F(8)/9*z^3*u*v^2+F(3)/13*y*z*u^2*v^2-F(17)/14*x*u^3*v^2-290*x^3*v^3-38*y*z^2*v^3-289*y^2*u*v^3-F(17)/3*x*z*u*v^3-F(9)/5*x*y*v^4+197*u^2*v^4-F(14)/3*z*v^5,68*x*y^3*z*u+F(9)/16*x^2*y*z^2*u-F(11)/20*z^5*u-327*x^2*y^2*u^2+F(9)/10*x^3*z*u^2-156*y*z^3*u^2-F(20)/17*y^2*z*u^3-8*x*z^2*u^3+55*x*y*u^4-74*u^6+266*x^2*y^2*z*v+F(16)/15*x^3*z^2*v-F(1)/11*y*z^4*v+F(17)/10*x^3*y*u*v+F(7)/11*y^2*z^2*u*v-78*x*z^3*u*v-F(15)/16*y^3*u^2*v-52*x*y*z*u^2*v+387*x^2*u^3*v-F(7)/17*z*u^4*v+F(15)/2*x^4*v^2-439*y^3*z*v^2+F(19)/11*x*y*z^2*v^2-289*x*y^2*u*v^2+F(9)/13*x^2*z*u*v^2-F(11)/4*z^2*u^2*v^2-F(11)/2*y*u^3*v^2+F(19)/13*x^2*y*v^3+F(17)/18*z^3*v^3+F(13)/6*y*z*u*v^3-316*x*u^2*v^3+219*y^2*v^4+327*x*z*v^4-220*u*v^5,-x^5-y^5+3*x*y^3*z-4*x^2*y*z^2-z^5+2*x^2*y^2*u+x^3*z*u+3*y*z^3*u-4*y^2*z*u^2+2*x*z^2*u^2+x*y*u^3-u^5+3*x^3*y*v+2*y^2*z^2*v+x*z^3*v+y^3*u*v-6*x*y*z*u*v-4*x^2*u^2*v+3*z*u^3*v-4*x*y^2*v^2+2*x^2*z*v^2-4*z^2*u*v^2+2*y*u^2*v^2+y*z*v^3+3*x*u*v^3-v^5]))
end


@doc raw"""
    bielliptic_d15_pi21()

Return a smooth bielliptic surface in $\mathbb P^4$ with degree `15` and sectional genus `21`.

The returned surface is defined over a prime field of characteristic 911.
"""
function bielliptic_d15_pi21()
   F = GF(911)  # SPECIAL
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(1)/14*x^3*y^3+F(11)/7*x^4*y*z+F(16)/19*y^4*z^2+F(4)/15*x*y^2*z^3+F(7)/8*x^2*z^4+176*y^5*u+F(1)/6*x^2*y*z^2*u+F(14)/19*z^5*u+F(15)/11*x^2*y^2*u^2-257*x^3*z*u^2+278*y*z^3*u^2-F(17)/21*y^2*z*u^3+F(12)/5*x*z^2*u^3+370*x*y*u^4+F(11)/9*u^6+F(2)/19*x*y^4*v-F(19)/18*x^2*y^2*z*v+F(3)/14*x^3*z^2*v-F(6)/17*y*z^4*v+F(17)/19*x^3*y*u*v+F(14)/11*y^2*z^2*u*v-35*x*z^3*u*v-F(6)/7*y^3*u^2*v+F(15)/16*x*y*z*u^2*v+F(16)/13*x^2*u^3*v+370*z*u^4*v-386*x^4*v^2-251*y^3*z*v^2+265*x*y*z^2*v^2+F(11)/18*x*y^2*u*v^2-25*x^2*z*u*v^2-F(3)/5*z^2*u^2*v^2-F(21)/5*y*u^3*v^2+177*x^2*y*v^3-F(14)/15*z^3*v^3-F(11)/16*y*z*u*v^3-F(5)/14*x*u^2*v^3+F(1)/15*y^2*v^4+189*x*z*v^4+F(3)/14*u*v^5,x^4*y^2-F(8)/17*y^5*z+26*x*y^3*z^2+F(17)/16*x^2*y*z^3+F(11)/9*z^6+F(18)/5*x*y^4*u-F(17)/15*x^2*y^2*z*u+168*x^3*z^2*u-F(7)/5*y*z^4*u-F(13)/18*x^3*y*u^2-F(7)/5*y^2*z^2*u^2+F(19)/8*x*z^3*u^2-282*y^3*u^3-F(7)/13*x*y*z*u^3-110*x^2*u^4-42*z*u^5-440*x^2*y^3*v-418*x^3*y*z*v+386*y^2*z^3*v-205*x*z^4*v+F(1)/8*x^4*u*v+282*y^3*z*u*v-F(21)/10*x*y*z^2*u*v+30*x*y^2*u^2*v-117*x^2*z*u^2*v+F(21)/10*z^2*u^3*v-F(6)/19*y*u^4*v-F(4)/3*y^4*v^2+9*x*y^2*z*v^2+F(7)/10*x^2*z^2*v^2-F(8)/19*x^2*y*u*v^2-135*z^3*u*v^2+F(9)/20*y*z*u^2*v^2+F(17)/12*x*u^3*v^2-F(5)/11*x^3*v^3+360*y*z^2*v^3+F(15)/11*y^2*u*v^3-360*x*z*u*v^3+10*x*y*v^4+111*u^2*v^4+F(16)/13*z*v^5,146*y^6+F(7)/20*x*y^4*z-F(13)/17*x^2*y^2*z^2+110*x^3*z^3+20*y*z^5+441*x^2*y^3*u+F(4)/9*x^3*y*z*u+F(3)/5*y^2*z^3*u-F(21)/20*x*z^4*u-F(2)/21*x^4*u^2-266*y^3*z*u^2-234*x*y*z^2*u^2-F(17)/14*x*y^2*u^3+F(13)/12*x^2*z*u^3+73*z^2*u^4-F(1)/16*y*u^5-F(21)/5*x^3*y^2*v-234*x^4*z*v+438*y^3*z^2*v-F(1)/16*x*y*z^3*v-F(9)/19*y^4*u*v-F(15)/4*x*y^2*z*u*v+F(15)/7*x^2*z^2*u*v+395*x^2*y*u^2*v-168*z^3*u^2*v-F(5)/19*y*z*u^3*v+F(5)/7*x*u^4*v-F(1)/16*x*y^3*v^2-207*x^2*y*z*v^2-138*z^4*v^2+359*x^3*u*v^2+386*y*z^2*u*v^2-F(17)/21*y^2*u^2*v^2-F(7)/5*x*z*u^2*v^2-80*y^2*z*v^3-F(11)/3*x*z^2*v^3-F(1)/8*x*y*u*v^3+278*u^3*v^3+F(1)/14*x^2*v^4-F(8)/15*z*u*v^4+23*y*v^5,386*x*y^5+F(1)/2*x^2*y^3*z+F(19)/9*x^3*y*z^2+F(1)/2*y^2*z^4+F(2)/21*x*z^5+2*x^3*y^2*u+26*x^4*z*u+F(10)/13*y^3*z^2*u+F(1)/10*x*y*z^3*u+F(7)/19*y^4*u^2+F(5)/21*x*y^2*z*u^2+F(20)/7*x^2*z^2*u^2+278*x^2*y*u^3-360*z^3*u^3-F(20)/9*y*z*u^4-F(1)/16*x*u^5+417*x^4*y*v-346*y^4*z*v+F(1)/14*x*y^2*z^2*v+205*x^2*z^3*v+222*x*y^3*u*v+F(3)/8*x^2*y*z*u*v+F(17)/13*z^4*u*v+F(11)/17*x^3*u^2*v+146*y*z^2*u^2*v-F(21)/8*y^2*u^3*v+F(4)/21*x*z*u^3*v+360*x^2*y^2*v^2-441*x^3*z*v^2+169*y*z^3*v^2+74*y^2*z*u*v^2+F(3)/11*x*z^2*u*v^2-408*x*y*u^2*v^2-119*u^4*v^2+F(17)/6*y^3*v^3+377*x*y*z*v^3+436*x^2*u*v^3-F(17)/21*z*u^2*v^3-145*z^2*v^4+F(15)/14*y*u*v^4-x*v^5,F(5)/11*x^2*y^4+109*x^3*y^2*z+F(1)/16*x^4*z^2+4*y^3*z^3-F(11)/10*x*y*z^4+F(20)/21*x^4*y*u-88*y^4*z*u+F(17)/2*x*y^2*z^2*u+F(4)/7*x^2*z^3*u-12*x*y^3*u^2-264*x^2*y*z*u^2+F(15)/19*z^4*u^2-F(7)/8*x^3*u^3+F(1)/4*y*z^2*u^3-F(20)/19*y^2*u^4-F(11)/13*x*z*u^4+12*y^5*v-F(8)/15*x*y^3*z*v+F(16)/9*x^2*y*z^2*v+177*z^5*v-F(1)/20*x^2*y^2*u*v-F(19)/2*x^3*z*u*v+24*y*z^3*u*v-277*y^2*z*u^2*v-F(13)/10*x*z^2*u^2*v-F(11)/10*x*y*u^3*v-F(18)/17*u^5*v+F(13)/21*x^3*y*v^2-F(9)/4*y^2*z^2*v^2-F(17)/4*y^3*u*v^2+F(10)/13*x*y*z*u*v^2+F(4)/21*x^2*u^2*v^2-F(1)/16*z*u^3*v^2-F(16)/9*x*y^2*v^3-F(1)/14*x^2*z*v^3+385*z^2*u*v^3-F(14)/15*y*u^2*v^3+F(14)/5*y*z*v^4+257*x*u*v^4+146*v^6,-124*x^4*y*z+F(12)/5*y^4*z^2+328*x*y^2*z^3+370*x^2*z^4+25*y^5*u-F(5)/21*x*y^3*z*u+F(1)/6*x^2*y*z^2*u+F(7)/9*z^5*u+F(2)/3*x^2*y^2*u^2+169*x^3*z*u^2+32*y*z^3*u^2-F(19)/21*y^2*z*u^3-F(1)/21*x*z^2*u^3-353*x*y*u^4+435*u^6+F(1)/18*x*y^4*v-F(12)/7*x^2*y^2*z*v+158*x^3*z^2*v-175*y*z^4*v-317*x^3*y*u*v-278*y^2*z^2*u*v+222*x*z^3*u*v+256*y^3*u^2*v+257*x*y*z*u^2*v-F(1)/9*x^2*u^3*v-212*z*u^4*v-205*x^4*v^2-334*y^3*z*v^2-F(11)/5*x*y*z^2*v^2-F(10)/3*x*y^2*u*v^2+F(6)/11*x^2*z*u*v^2+296*z^2*u^2*v^2-142*y*u^3*v^2+F(8)/9*x^2*y*v^3-386*z^3*v^3+294*y*z*u*v^3+189*x*u^2*v^3-F(11)/13*y^2*v^4-F(19)/16*x*z*v^4+F(16)/7*u*v^5,-417*x*y^4*z+F(10)/17*x^2*y^2*z^2+F(8)/17*x^3*z^3-F(6)/19*y*z^5-F(8)/3*x^2*y^3*u-F(11)/12*x^3*y*z*u+F(11)/19*y^2*z^3*u+F(2)/19*x*z^4*u-F(14)/5*x^4*u^2-22*y^3*z*u^2-F(5)/8*x*y*z^2*u^2-198*x*y^2*u^3-F(20)/17*x^2*z*u^3+17*z^2*u^4+124*y*u^5+98*x^3*y^2*v+124*x^4*z*v+F(1)/20*y^3*z^2*v+F(2)/5*x*y*z^3*v-246*y^4*u*v+222*x*y^2*z*u*v+190*x^2*z^2*u*v+188*x^2*y*u^2*v-F(4)/19*z^3*u^2*v-F(1)/18*y*z*u^3*v+F(1)/15*x*u^4*v-264*x*y^3*v^2-119*x^2*y*z*v^2+135*z^4*v^2+F(2)/5*x^3*u*v^2-F(4)/19*y*z^2*u*v^2-F(17)/19*y^2*u^2*v^2+F(19)/16*x*z*u^2*v^2-F(10)/19*y^2*z*v^3-193*x*z^2*v^3+328*x*y*u*v^3+F(14)/5*u^3*v^3+F(19)/15*x^2*v^4+F(20)/21*z*u*v^4-417*y*v^5,F(6)/17*x^3*y^2*z+F(21)/13*x^4*z^2-15*y^3*z^3+59*x*y*z^4-F(1)/13*x^4*y*u-F(13)/12*y^4*z*u-440*x*y^2*z^2*u+F(3)/2*x^2*z^3*u-127*x*y^3*u^2+F(5)/4*x^2*y*z*u^2+35*z^4*u^2+F(6)/17*x^3*u^3-278*y*z^2*u^3-F(11)/8*y^2*u^4+F(10)/3*x*z*u^4-F(13)/16*y^5*v+16*x*y^3*z*v-73*x^2*y*z^2*v-F(21)/13*z^5*v+138*x^2*y^2*u*v-10*x^3*z*u*v-188*y*z^3*u*v+F(18)/5*y^2*z*u^2*v-F(21)/5*x*z^2*u^2*v-F(5)/18*x*y*u^3*v-F(13)/16*u^5*v-33*x^3*y*v^2+F(16)/21*y^2*z^2*v^2-F(11)/8*x*z^3*v^2-y^3*u*v^2+F(13)/2*x*y*z*u*v^2+327*x^2*u^2*v^2-F(15)/13*z*u^3*v^2+F(21)/2*x*y^2*v^3+F(19)/4*x^2*z*v^3-F(4)/7*z^2*u*v^3+F(14)/9*y*u^2*v^3-F(11)/18*y*z*v^4+F(16)/5*x*u*v^4+30*v^6,-F(16)/9*y^5*z-F(9)/20*x*y^3*z^2+F(7)/20*x^2*y*z^3+177*z^6+437*x*y^4*u+F(4)/3*x^2*y^2*z*u+F(17)/18*x^3*z^2*u-109*y*z^4*u-F(1)/2*x^3*y*u^2+F(1)/10*y^2*z^2*u^2-441*x*z^3*u^2-237*y^3*u^3-F(2)/9*x*y*z*u^3-52*x^2*u^4-237*x^2*y^3*v+F(20)/7*x^3*y*z*v+257*y^2*z^3*v-F(11)/21*x*z^4*v-401*x^4*u*v+427*y^3*z*u*v-F(19)/7*x*y*z^2*u*v+21*x*y^2*u^2*v-212*x^2*z*u^2*v-F(11)/7*z^2*u^3*v-F(4)/15*y*u^4*v+F(2)/13*y^4*v^2+F(13)/20*x*y^2*z*v^2+193*x^2*z^2*v^2-F(10)/3*x^2*y*u*v^2-334*z^3*u*v^2+346*y*z*u^2*v^2+270*x*u^3*v^2-55*x^3*v^3-F(11)/13*y*z^2*v^3+F(17)/5*y^2*u*v^3-F(2)/17*x*z*u*v^3+F(1)/7*x*y*v^4+F(9)/2*u^2*v^4+37*z*v^5,234*x^2*y^3*z-F(15)/13*x^3*y*z^2-118*y^2*z^4+430*x*z^5+F(21)/2*x^3*y^2*u-40*x^4*z*u+159*y^3*z^2*u+F(9)/10*x*y*z^3*u-235*y^4*u^2+F(9)/2*x*y^2*z*u^2-F(13)/3*x^2*z^2*u^2+15*x^2*y*u^3+235*z^3*u^3-156*y*z*u^4-21*x*u^5+430*x^4*y*v-21*y^4*z*v-F(1)/11*x*y^2*z^2*v+F(19)/8*x^2*z^3*v+157*x*y^3*u*v-F(7)/17*x^2*y*z*u*v-F(6)/5*z^4*u*v-F(7)/10*x^3*u^2*v+F(19)/9*y*z^2*u^2*v-18*y^2*u^3*v+F(1)/15*x*z*u^3*v+159*x^2*y^2*v^2+63*x^3*z*v^2+F(13)/15*y*z^3*v^2-3*y^2*z*u*v^2-F(11)/18*x*z^2*u*v^2-F(3)/20*x*y*u^2*v^2-F(1)/2*u^4*v^2+396*y^3*v^3+187*x*y*z*v^3-F(16)/5*x^2*u*v^3-436*z*u^2*v^3+F(11)/19*z^2*v^4+F(9)/14*y*u*v^4-F(20)/19*x*v^5,-x^5-y^5-2*x*y^3*z-3*x^2*y*z^2-z^5-x^2*y^2*u+4*x^3*z*u-2*y*z^3*u-3*y^2*z*u^2-x*z^2*u^2+4*x*y*u^3-u^5-2*x^3*y*v-y^2*z^2*v+4*x*z^3*v+4*y^3*u*v+16*x*y*z*u*v-3*x^2*u^2*v-2*z*u^3*v-3*x*y^2*v^2-x^2*z*v^2-3*z^2*u*v^2-y*u^2*v^2+4*y*z*v^3-2*x*u*v^3-v^5]))
end


@doc raw"""
    bordiga()

Return a smooth rational surface in $\mathbb P^4$ with degree `6` and sectional genus `3`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function bordiga()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-x^2*y+F(26)/17*x*y^2+F(70)/17*y^3+F(96)/121*x^2*z+F(63)/82*x*y*z+F(115)/11*y^2*z-8114*x*z^2-F(40)/79*y*z^2+F(16)/125*z^3+3023*x^2*u-F(123)/70*x*y*u+3395*y^2*u-F(81)/119*x*z*u-F(23)/66*y*z*u+3626*z^2*u+F(18)/53*x*u^2+F(111)/58*y*u^2-F(34)/39*z*u^2+F(53)/40*u^3-F(94)/17*x^2*v-F(10)/19*x*y*v+F(81)/88*y^2*v-F(91)/33*x*z*v-9967*y*z*v-F(103)/4*z^2*v-F(26)/109*x*u*v+F(69)/97*y*u*v+F(92)/17*z*u*v-F(19)/96*u^2*v+F(10)/21*x*v^2+6147*y*v^2+F(32)/113*z*v^2-F(79)/82*u*v^2-F(77)/51*v^3,4347*x^2*y-9017*x*y^2+11327*y^3+F(18)/79*x^2*z-F(93)/43*x*y*z-F(35)/47*y^2*z+14704*x*z^2+10727*y*z^2-1764*z^3-612*x^2*u+F(20)/107*x*y*u-F(103)/89*y^2*u-F(39)/2*x*z*u+2345*y*z*u+10251*z^2*u-9984*x*u^2-10299*y*u^2+F(113)/118*z*u^2+F(37)/91*u^3+F(2)/31*x^2*v+9552*x*y*v-F(47)/100*y^2*v-3242*x*z*v+F(113)/27*y*z*v-11271*z^2*v-F(13)/79*x*u*v+15917*y*u*v+F(5)/114*z*u*v+F(103)/119*u^2*v-F(21)/55*x*v^2-F(59)/19*y*v^2+F(101)/68*z*v^2-7817*u*v^2-F(112)/29*v^3,7228*x^2*y-F(111)/113*x*y^2+5913*y^3+F(6)/43*x^2*z-11251*x*y*z+F(27)/121*y^2*z+F(97)/96*x*z^2-7398*y*z^2-F(97)/114*z^3+F(38)/15*x^2*u+5005*x*y*u-F(41)/126*y^2*u-F(61)/116*x*z*u+F(89)/9*y*z*u-4087*z^2*u+F(26)/15*x*u^2-F(92)/103*y*u^2+F(21)/68*z*u^2-4027*u^3+F(97)/91*x^2*v+5150*x*y*v-F(4)/47*y^2*v-2310*x*z*v+7307*y*z*v-F(77)/86*z^2*v+F(30)/83*x*u*v+413*y*u*v-50*z*u*v-F(103)/106*u^2*v+F(105)/73*x*v^2-F(109)/98*y*v^2+F(59)/63*z*v^2+715*u*v^2+963*v^3,x^3+3487*x^2*y-9744*x*y^2-13276*y^3-15213*x^2*z-F(118)/51*x*y*z+F(101)/104*y^2*z+2754*x*z^2+9111*y*z^2-F(17)/94*z^3+11136*x^2*u-F(43)/82*x*y*u-F(9)/41*y^2*u-7306*x*z*u-6839*y*z*u+5692*z^2*u-14682*x*u^2+F(37)/80*y*u^2-F(85)/97*z*u^2-6186*u^3+F(34)/15*x^2*v+F(84)/109*x*y*v+5086*y^2*v+F(27)/112*x*z*v-F(3)/40*y*z*v+F(19)/120*z^2*v+11222*x*u*v+F(38)/55*y*u*v-F(24)/83*z*u*v+15814*u^2*v-F(111)/61*x*v^2+F(49)/44*y*v^2+F(125)/81*z*v^2+1933*u*v^2-F(19)/71*v^3]))
end


@doc raw"""
    castelnuovo()

Return a smooth rational surface in $\mathbb P^4$ with degree `5` and sectional genus `2`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function castelnuovo()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([5153*x*y^2-F(98)/23*y^3-F(101)/51*x*y*z+F(33)/41*y^2*z+F(99)/79*x*z^2+7136*y*z^2-F(106)/111*z^3+F(119)/53*x*y*u+F(34)/57*y^2*u-F(77)/92*x*z*u+F(84)/73*y*z*u-F(109)/78*z^2*u-F(27)/56*x*u^2+10023*y*u^2+F(82)/103*z*u^2-F(34)/25*u^3+F(3)/2*x*y*v-F(68)/25*y^2*v+12721*x*z*v+F(4)/63*y*z*v-F(73)/21*z^2*v-7291*x*u*v-F(91)/53*y*u*v-F(4)/79*z*u*v-F(34)/91*u^2*v-F(122)/53*x*v^2+F(123)/70*y*v^2-F(64)/73*z*v^2+F(44)/65*u*v^2+F(14)/31*v^3,x*y^2-15202*y^3+10613*x*y*z+13640*y^2*z-F(107)/103*x*z^2+5292*y*z^2+F(19)/119*z^3-10042*x*y*u+2770*y^2*u+7957*x*z*u+14008*y*z*u+F(92)/121*z^2*u-F(92)/51*x*u^2+1178*y*u^2+F(1)/117*z*u^2-12726*u^3+F(82)/101*x*y*v-F(92)/17*y^2*v-F(107)/56*x*z*v+14233*y*z*v+F(79)/28*z^2*v+F(51)/50*x*u*v-F(31)/5*y*u*v+F(95)/91*z*u*v+F(19)/108*u^2*v+12151*x*v^2-F(69)/110*y*v^2+F(37)/89*z*v^2-F(63)/116*u*v^2-F(88)/23*v^3,-5153*x^2+F(37)/23*x*y+8706*y^2-13160*x*z+F(68)/115*y*z+5548*z^2-F(22)/61*x*u-F(113)/98*y*u+11818*z*u+2114*u^2-F(101)/97*x*v+F(89)/22*y*v-3355*z*v-F(113)/5*u*v-5521*v^2]))
end


@doc raw"""
    cubic_scroll()

Return a smooth rational surface in $\mathbb P^4$ with degree `3` and sectional genus `0`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function cubic_scroll()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-x*y+F(91)/6*y^2+F(82)/87*x*z-F(121)/13*y*z-F(98)/71*z^2+4788*x*u-3630*y*u-F(111)/77*z*u-565*u^2-F(99)/80*x*v+F(43)/94*y*v+F(97)/80*z*v+F(53)/91*u*v+F(115)/34*v^2,7471*x*y-11252*y^2-F(123)/109*x*z-F(97)/101*y*z+15074*z^2+F(43)/92*x*u+F(121)/84*y*u-97*z*u+F(99)/35*u^2-F(104)/53*x*v+F(119)/52*y*v+F(104)/119*z*v-1446*u*v+F(43)/42*v^2,x^2+3601*x*y+9949*y^2+7253*x*z+9638*y*z+F(97)/45*z^2+F(91)/111*x*u+F(48)/25*y*u-F(124)/55*z*u-F(2)/45*u^2+F(61)/39*x*v+F(93)/73*y*v+F(57)/68*z*v-F(74)/93*u*v-5405*v^2]))
end


@doc raw"""
    elliptic_d10_pi10()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `10` and sectional genus `10`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d10_pi10()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([11888*x^4*y+F(41)/79*x^3*y^2-F(89)/99*x^2*y^3-F(35)/41*x*y^4-F(106)/45*x^4*z+F(125)/61*x^3*y*z+10402*x^2*y^2*z+F(6)/19*x*y^3*z+7581*x^3*z^2-4818*x^2*y*z^2+F(7)/16*x*y^2*z^2+6262*x^2*z^3-42*x*y*z^3-10091*x*z^4-F(63)/10*x^4*u-3434*x^3*y*u+15882*x^2*y^2*u-2646*x*y^3*u+F(105)/71*x^3*z*u+14034*x^2*y*z*u-804*x*y^2*z*u+6060*x^2*z^2*u+7370*x*y*z^2*u+F(22)/5*x*z^3*u-10010*x^3*u^2+F(51)/56*x^2*y*u^2-12833*x*y^2*u^2-15492*x^2*z*u^2+F(52)/97*x*y*z*u^2-10580*x*z^2*u^2+F(65)/88*x^2*u^3+1607*x*y*u^3-F(15)/73*x*z*u^3+10520*x*u^4-F(67)/106*x^4*v+11021*x^3*y*v-F(99)/112*x^2*y^2*v+13502*x*y^3*v+F(104)/21*y^4*v-F(121)/20*x^3*z*v-F(76)/15*x^2*y*z*v-9360*x*y^2*z*v+F(112)/55*y^3*z*v+F(89)/90*x^2*z^2*v-F(112)/103*x*y*z^2*v+3572*y^2*z^2*v+632*x*z^3*v+F(51)/10*y*z^3*v+12418*z^4*v+13199*x^3*u*v+F(7)/46*x^2*y*u*v+11854*x*y^2*u*v-F(53)/92*y^3*u*v+F(103)/43*x^2*z*u*v-303*x*y*z*u*v-F(125)/17*y^2*z*u*v-6369*x*z^2*u*v+F(105)/124*y*z^2*u*v-7762*z^3*u*v-9248*x^2*u^2*v-F(44)/19*x*y*u^2*v+5455*y^2*u^2*v-F(110)/87*x*z*u^2*v-F(5)/98*y*z*u^2*v-F(73)/69*z^2*u^2*v+F(67)/111*x*u^3*v-12894*y*u^3*v-F(71)/125*z*u^3*v+F(111)/59*u^4*v+12462*x^3*v^2+F(91)/55*x^2*y*v^2+3883*x*y^2*v^2+F(73)/28*y^3*v^2-15134*x^2*z*v^2-2428*x*y*z*v^2+F(87)/41*y^2*z*v^2+F(23)/108*x*z^2*v^2-12218*y*z^2*v^2-F(17)/83*z^3*v^2+F(105)/76*x^2*u*v^2-F(121)/81*x*y*u*v^2+F(96)/97*y^2*u*v^2-F(69)/94*x*z*u*v^2-F(71)/35*y*z*u*v^2-F(6)/125*z^2*u*v^2+10305*x*u^2*v^2-3291*y*u^2*v^2+F(99)/104*z*u^2*v^2+F(73)/79*u^3*v^2+194*x^2*v^3+F(2)/57*x*y*v^3+1677*y^2*v^3-F(40)/49*x*z*v^3+3912*y*z*v^3-13762*z^2*v^3+F(11)/69*x*u*v^3-F(107)/5*y*u*v^3+14437*z*u*v^3-1492*u^2*v^3-F(13)/125*x*v^4-F(120)/77*y*v^4-F(65)/61*z*v^4+F(90)/107*u*v^4+F(71)/16*v^5,-11888*x^3*y^2-F(41)/79*x^2*y^3+F(89)/99*x*y^4+F(35)/41*y^5+F(106)/45*x^3*y*z-F(125)/61*x^2*y^2*z-10402*x*y^3*z-F(6)/19*y^4*z-7581*x^2*y*z^2+4818*x*y^2*z^2-F(7)/16*y^3*z^2-6262*x*y*z^3+42*y^2*z^3+10091*y*z^4+F(63)/10*x^3*y*u+3434*x^2*y^2*u-15882*x*y^3*u+2646*y^4*u-F(105)/71*x^2*y*z*u-14034*x*y^2*z*u+804*y^3*z*u-6060*x*y*z^2*u-7370*y^2*z^2*u-F(22)/5*y*z^3*u+10010*x^2*y*u^2-F(51)/56*x*y^2*u^2+12833*y^3*u^2+15492*x*y*z*u^2-F(52)/97*y^2*z*u^2+10580*y*z^2*u^2-F(65)/88*x*y*u^3-1607*y^2*u^3+F(15)/73*y*z*u^3-10520*y*u^4-F(39)/106*x^3*y*v+12288*x^2*y^2*v-F(35)/114*x*y^3*v-14601*y^4*v+F(41)/50*x^3*z*v-13888*x^2*y*z*v+15801*x*y^2*z*v-F(73)/12*y^3*z*v+151*x^2*z^2*v-F(49)/52*x*y*z^2*v+F(97)/18*y^2*z^2*v-F(7)/9*x*z^3*v+6197*y*z^3*v+F(1)/47*z^4*v+F(23)/24*x^3*u*v-12451*x^2*y*u*v-F(27)/29*x*y^2*u*v-F(76)/89*y^3*u*v-12943*x^2*z*u*v-13796*x*y*z*u*v+12151*y^2*z*u*v+F(84)/65*x*z^2*u*v-F(111)/115*y*z^2*u*v-7415*z^3*u*v-F(95)/99*x^2*u^2*v+12507*x*y*u^2*v+6427*y^2*u^2*v+F(15)/86*x*z*u^2*v+F(47)/25*y*z*u^2*v+8039*z^2*u^2*v+F(84)/25*x*u^3*v-5616*y*u^3*v+F(77)/37*z*u^3*v-F(105)/109*u^4*v-F(103)/84*x^3*v^2+F(59)/11*x^2*y*v^2+F(77)/6*x*y^2*v^2+6123*y^3*v^2+F(121)/15*x^2*z*v^2-10372*x*y*z*v^2-F(19)/87*y^2*z*v^2+F(118)/121*x*z^2*v^2+F(103)/54*y*z^2*v^2+F(37)/83*z^3*v^2+11189*x^2*u*v^2+F(31)/72*x*y*u*v^2+F(55)/114*y^2*u*v^2-F(90)/13*x*z*u*v^2+F(32)/77*y*z*u*v^2-6443*z^2*u*v^2-2788*x*u^2*v^2-11221*y*u^2*v^2+8180*z*u^2*v^2+11462*u^3*v^2+F(10)/53*x^2*v^3+F(36)/61*x*y*v^3-F(5)/68*y^2*v^3-F(32)/3*x*z*v^3+F(17)/62*y*z*v^3+F(78)/101*z^2*v^3-5368*x*u*v^3-F(51)/100*y*u*v^3+F(33)/19*z*u*v^3-F(101)/119*u^2*v^3-826*x*v^4+F(126)/5*y*v^4+F(20)/17*z*v^4-13028*u*v^4+F(76)/107*v^5,-11888*x^3*y*z-F(41)/79*x^2*y^2*z+F(89)/99*x*y^3*z+F(35)/41*y^4*z+F(106)/45*x^3*z^2-F(125)/61*x^2*y*z^2-10402*x*y^2*z^2-F(6)/19*y^3*z^2-7581*x^2*z^3+4818*x*y*z^3-F(7)/16*y^2*z^3-6262*x*z^4+42*y*z^4+10091*z^5+F(63)/10*x^3*z*u+3434*x^2*y*z*u-15882*x*y^2*z*u+2646*y^3*z*u-F(105)/71*x^2*z^2*u-14034*x*y*z^2*u+804*y^2*z^2*u-6060*x*z^3*u-7370*y*z^3*u-F(22)/5*z^4*u+10010*x^2*z*u^2-F(51)/56*x*y*z*u^2+12833*y^2*z*u^2+15492*x*z^2*u^2-F(52)/97*y*z^2*u^2+10580*z^3*u^2-F(65)/88*x*z*u^3-1607*y*z*u^3+F(15)/73*z^2*u^3-10520*z*u^4-F(8)/81*x^3*y*v-9303*x^2*y^2*v-12771*x*y^3*v-F(75)/82*y^4*v+F(32)/77*x^3*z*v-F(122)/111*x^2*y*z*v-F(113)/100*x*y^2*z*v+F(17)/114*y^3*z*v+14724*x^2*z^2*v-F(31)/99*x*y*z^2*v-15396*y^2*z^2*v-1636*x*z^3*v-4988*y*z^3*v+F(43)/80*z^4*v+15665*x^3*u*v+1694*x^2*y*u*v-F(51)/28*x*y^2*u*v+1991*y^3*u*v-14370*x^2*z*u*v-7027*x*y*z*u*v-F(103)/23*y^2*z*u*v-8793*x*z^2*u*v+F(107)/36*y*z^2*u*v-5361*z^3*u*v+F(1)/60*x^2*u^2*v-F(29)/68*x*y*u^2*v-13342*y^2*u^2*v-F(22)/51*x*z*u^2*v-F(85)/116*y*z*u^2*v+F(43)/62*z^2*u^2*v-F(39)/107*x*u^3*v-F(73)/106*y*u^3*v-358*z*u^3*v-F(1)/54*u^4*v+F(57)/116*x^3*v^2-F(50)/59*x^2*y*v^2-F(114)/17*x*y^2*v^2+6316*y^3*v^2-F(77)/96*x^2*z*v^2-F(95)/97*x*y*z*v^2-3922*y^2*z*v^2+F(18)/43*x*z^2*v^2-11778*y*z^2*v^2+F(40)/33*z^3*v^2+14203*x^2*u*v^2+5365*x*y*u*v^2-F(125)/88*y^2*u*v^2-9709*x*z*u*v^2-10726*y*z*u*v^2-1871*z^2*u*v^2-F(22)/57*x*u^2*v^2-F(29)/93*y*u^2*v^2-F(91)/19*z*u^2*v^2+F(105)/11*u^3*v^2+F(122)/63*x^2*v^3-F(55)/98*x*y*v^3+14082*y^2*v^3+7627*x*z*v^3+F(103)/57*y*z*v^3-15673*z^2*v^3+F(8)/9*x*u*v^3+F(121)/26*y*u*v^3-F(113)/20*z*u*v^3+F(1)/114*u^2*v^3+F(46)/15*x*v^4-F(94)/57*y*v^4+1151*z*v^4-F(115)/78*u*v^4-F(77)/65*v^5,-11888*x^3*y*u-F(41)/79*x^2*y^2*u+F(89)/99*x*y^3*u+F(35)/41*y^4*u+F(106)/45*x^3*z*u-F(125)/61*x^2*y*z*u-10402*x*y^2*z*u-F(6)/19*y^3*z*u-7581*x^2*z^2*u+4818*x*y*z^2*u-F(7)/16*y^2*z^2*u-6262*x*z^3*u+42*y*z^3*u+10091*z^4*u+F(63)/10*x^3*u^2+3434*x^2*y*u^2-15882*x*y^2*u^2+2646*y^3*u^2-F(105)/71*x^2*z*u^2-14034*x*y*z*u^2+804*y^2*z*u^2-6060*x*z^2*u^2-7370*y*z^2*u^2-F(22)/5*z^3*u^2+10010*x^2*u^3-F(51)/56*x*y*u^3+12833*y^2*u^3+15492*x*z*u^3-F(52)/97*y*z*u^3+10580*z^2*u^3-F(65)/88*x*u^4-1607*y*u^4+F(15)/73*z*u^4-10520*u^5+11490*x^3*y*v-F(46)/119*x^2*y^2*v-F(119)/54*x*y^3*v+F(104)/121*y^4*v+10405*x^3*z*v-F(48)/77*x^2*y*z*v-7581*x*y^2*z*v+F(35)/47*y^3*z*v-F(115)/67*x^2*z^2*v-2095*x*y*z^2*v+496*y^2*z^2*v+F(77)/47*x*z^3*v+42*y*z^3*v+8754*z^4*v-5247*x^3*u*v+F(35)/121*x^2*y*u*v+F(123)/37*x*y^2*u*v-F(15)/7*y^3*u*v-F(13)/28*x^2*z*u*v+F(74)/117*x*y*z*u*v-F(101)/85*y^2*z*u*v+F(48)/121*x*z^2*u*v+F(53)/7*y*z^2*u*v+2748*z^3*u*v+13184*x^2*u^2*v+F(99)/118*x*y*u^2*v-F(75)/41*y^2*u^2*v+F(41)/110*x*z*u^2*v-13121*y*z*u^2*v-F(11)/23*z^2*u^2*v-7325*x*u^3*v-F(97)/12*y*u^3*v+F(121)/31*z*u^3*v+14917*u^4*v+4165*x^3*v^2+F(117)/11*x^2*y*v^2+F(62)/119*x*y^2*v^2-F(110)/51*y^3*v^2-10728*x^2*z*v^2+F(25)/51*x*y*z*v^2+13638*y^2*z*v^2+F(65)/92*x*z^2*v^2+F(19)/3*y*z^2*v^2+14290*z^3*v^2+F(66)/53*x^2*u*v^2+8589*x*y*u*v^2-15185*y^2*u*v^2-7843*x*z*u*v^2+15805*y*z*u*v^2+F(41)/32*z^2*u*v^2+10541*x*u^2*v^2+F(17)/45*y*u^2*v^2-10167*z*u^2*v^2+F(47)/104*u^3*v^2-F(57)/125*x^2*v^3+F(29)/46*x*y*v^3-14268*y^2*v^3-15030*x*z*v^3+8321*y*z*v^3-F(121)/126*z^2*v^3-F(125)/124*x*u*v^3-F(61)/66*y*u*v^3-3792*z*u*v^3-1984*u^2*v^3-F(59)/69*x*v^4-F(29)/26*y*v^4+F(105)/11*z*v^4+F(89)/55*u*v^4+F(61)/43*v^5,11490*x^4*y-F(46)/119*x^3*y^2-F(119)/54*x^2*y^3+F(104)/121*x*y^4+10405*x^4*z-F(48)/77*x^3*y*z-7581*x^2*y^2*z+F(35)/47*x*y^3*z-F(115)/67*x^3*z^2-2095*x^2*y*z^2+496*x*y^2*z^2+F(77)/47*x^2*z^3+42*x*y*z^3+8754*x*z^4-117*x^4*u-14360*x^3*y*u+11312*x^2*y^2*u+F(109)/108*x*y^3*u+F(104)/21*y^4*u-15545*x^3*z*u+5136*x^2*y*z*u+F(55)/9*x*y^2*z*u+F(112)/55*y^3*z*u+8406*x^2*z^2*u-F(53)/115*x*y*z^2*u+3572*y^2*z^2*u+3380*x*z^3*u+F(51)/10*y*z^3*u+12418*z^4*u-5608*x^3*u^2+F(88)/65*x^2*y*u^2-F(10)/21*x*y^2*u^2-F(53)/92*y^3*u^2-F(27)/44*x^2*z*u^2+F(89)/112*x*y*z*u^2-F(125)/17*y^2*z*u^2+F(101)/81*x*z^2*u^2+F(105)/124*y*z^2*u^2-7762*z^3*u^2+F(54)/83*x^2*u^3-11095*x*y*u^3+5455*y^2*u^3+F(62)/19*x*z*u^3-F(5)/98*y*z*u^3-F(73)/69*z^2*u^3-1222*x*u^4-12894*y*u^4-F(71)/125*z*u^4+F(111)/59*u^5+4165*x^4*v+F(117)/11*x^3*y*v+F(62)/119*x^2*y^2*v-F(110)/51*x*y^3*v-10728*x^3*z*v+F(25)/51*x^2*y*z*v+13638*x*y^2*z*v+F(65)/92*x^2*z^2*v+F(19)/3*x*y*z^2*v+14290*x*z^3*v+5220*x^3*u*v+F(45)/4*x^2*y*u*v-11302*x*y^2*u*v+F(73)/28*y^3*u*v+9014*x^2*z*u*v-F(58)/55*x*y*z*u*v+F(87)/41*y^2*z*u*v+705*x*z^2*u*v-12218*y*z^2*u*v-F(17)/83*z^3*u*v-12609*x^2*u^2*v+F(31)/43*x*y*u^2*v+F(96)/97*y^2*u^2*v+F(33)/25*x*z*u^2*v-F(71)/35*y*z*u^2*v-F(6)/125*z^2*u^2*v+7537*x*u^3*v-3291*y*u^3*v+F(99)/104*z*u^3*v+F(73)/79*u^4*v-F(57)/125*x^3*v^2+F(29)/46*x^2*y*v^2-14268*x*y^2*v^2-15030*x^2*z*v^2+8321*x*y*z*v^2-F(121)/126*x*z^2*v^2-65*x^2*u*v^2+F(103)/85*x*y*u*v^2+1677*y^2*u*v^2+13182*x*z*u*v^2+3912*y*z*u*v^2-13762*z^2*u*v^2+5898*x*u^2*v^2-F(107)/5*y*u^2*v^2+14437*z*u^2*v^2-1492*u^3*v^2-F(59)/69*x^2*v^3-F(29)/26*x*y*v^3+F(105)/11*x*z*v^3+15078*x*u*v^3-F(120)/77*y*u*v^3-F(65)/61*z*u*v^3+F(90)/107*u^2*v^3+F(61)/43*x*v^4+F(71)/16*u*v^4,-11490*x^3*y^2+F(46)/119*x^2*y^3+F(119)/54*x*y^4-F(104)/121*y^5-10405*x^3*y*z+F(48)/77*x^2*y^2*z+7581*x*y^3*z-F(35)/47*y^4*z+F(115)/67*x^2*y*z^2+2095*x*y^2*z^2-496*y^3*z^2-F(77)/47*x*y*z^3-42*y^2*z^3-8754*y*z^4+116*x^3*y*u+5678*x^2*y^2*u-10326*x*y^3*u+12822*y^4*u+F(41)/50*x^3*z*u-F(76)/123*x^2*y*z*u-F(56)/15*x*y^2*z*u-F(95)/13*y^3*z*u+151*x^2*z^2*u-7572*x*y*z^2*u+F(16)/109*y^2*z^2*u-F(7)/9*x*z^3*u-F(103)/102*y*z^3*u+F(1)/47*z^4*u+F(23)/24*x^3*u^2+6356*x^2*y*u^2+F(53)/44*x*y^2*u^2+F(4)/85*y^3*u^2-12943*x^2*z*u^2+F(119)/104*x*y*z*u^2-F(89)/100*y^2*z*u^2+F(84)/65*x*z^2*u^2-F(56)/115*y*z^2*u^2-7415*z^3*u^2-F(95)/99*x^2*u^3-F(121)/50*x*y*u^3+F(13)/116*y^2*u^3+F(15)/86*x*z*u^3+F(17)/12*y*z*u^3+8039*z^2*u^3+F(84)/25*x*u^4-F(98)/67*y*u^4+F(77)/37*z*u^4-F(105)/109*u^5-4165*x^3*y*v-F(117)/11*x^2*y^2*v-F(62)/119*x*y^3*v+F(110)/51*y^4*v+10728*x^2*y*z*v-F(25)/51*x*y^2*z*v-13638*y^3*z*v-F(65)/92*x*y*z^2*v-F(19)/3*y^2*z^2*v-14290*y*z^3*v-F(103)/84*x^3*u*v+F(125)/57*x^2*y*u*v+F(26)/23*x*y^2*u*v-F(58)/3*y^3*u*v+F(121)/15*x^2*z*u*v-2529*x*y*z*u*v-F(85)/44*y^2*z*u*v+F(118)/121*x*z^2*u*v+10257*y*z^2*u*v+F(37)/83*z^3*u*v+11189*x^2*u^2*v-F(89)/122*x*y*u^2*v+F(23)/41*y^2*u^2*v-F(90)/13*x*z*u^2*v+F(81)/119*y*z*u^2*v-6443*z^2*u^2*v-2788*x*u^3*v-8453*y*u^3*v+8180*z*u^3*v+11462*u^4*v+F(57)/125*x^2*y*v^2-F(29)/46*x*y^2*v^2+14268*y^3*v^2+15030*x*y*z*v^2-8321*y^2*z*v^2+F(121)/126*y*z^2*v^2+F(10)/53*x^2*u*v^2+F(7)/97*x*y*u*v^2+8911*y^2*u*v^2-F(32)/3*x*z*u*v^2-F(109)/28*y*z*u*v^2+F(78)/101*z^2*u*v^2-5368*x*u^2*v^2+F(6)/125*y*u^2*v^2+F(33)/19*z*u^2*v^2-F(101)/119*u^3*v^2+F(59)/69*x*y*v^3+F(29)/26*y^2*v^3-F(105)/11*y*z*v^3-826*x*u*v^3+F(118)/79*y*u*v^3+F(20)/17*z*u*v^3-13028*u^2*v^3-F(61)/43*y*v^4+F(76)/107*u*v^4,-11490*x^3*y*z+F(46)/119*x^2*y^2*z+F(119)/54*x*y^3*z-F(104)/121*y^4*z-10405*x^3*z^2+F(48)/77*x^2*y*z^2+7581*x*y^2*z^2-F(35)/47*y^3*z^2+F(115)/67*x^2*z^3+2095*x*y*z^3-496*y^2*z^3-F(77)/47*x*z^4-42*y*z^4-8754*z^5-F(8)/81*x^3*y*u-9303*x^2*y^2*u-12771*x*y^3*u-F(75)/82*y^4*u+F(40)/43*x^3*z*u+F(53)/3*x^2*y*z*u-F(76)/85*x*y^2*z*u+F(7)/108*y^3*z*u-F(67)/125*x^2*z^2*u+12179*x*y*z^2*u-F(81)/29*y^2*z^2*u+F(73)/120*x*z^3*u+F(71)/118*y*z^3*u-7946*z^4*u+15665*x^3*u^2+1694*x^2*y*u^2-F(51)/28*x*y^2*u^2+1991*y^3*u^2+4437*x^2*z*u^2-F(73)/61*x*y*z*u^2-F(64)/101*y^2*z*u^2-2686*x*z^2*u^2+F(17)/95*y*z^2*u^2+F(31)/7*z^3*u^2+F(1)/60*x^2*u^3-F(29)/68*x*y*u^3-13342*y^2*u^3+F(72)/77*x*z*u^3-F(103)/11*y*z*u^3-6711*z^2*u^3-F(39)/107*x*u^4-F(73)/106*y*u^4-F(2)/111*z*u^4-F(1)/54*u^5-4165*x^3*z*v-F(117)/11*x^2*y*z*v-F(62)/119*x*y^2*z*v+F(110)/51*y^3*z*v+10728*x^2*z^2*v-F(25)/51*x*y*z^2*v-13638*y^2*z^2*v-F(65)/92*x*z^3*v-F(19)/3*y*z^3*v-14290*z^4*v+F(57)/116*x^3*u*v-F(50)/59*x^2*y*u*v-F(114)/17*x*y^2*u*v+6316*y^3*u*v+5575*x^2*z*u*v-F(47)/25*x*y*z*u*v-F(102)/71*y^2*z*u*v-F(47)/116*x*z^2*u*v+4408*y*z^2*u*v-F(37)/97*z^3*u*v+14203*x^2*u^2*v+5365*x*y*u^2*v-F(125)/88*y^2*u^2*v+11741*x*z*u^2*v+F(106)/89*y*z*u^2*v+F(55)/27*z^2*u^2*v-F(22)/57*x*u^3*v-F(29)/93*y*u^3*v-F(41)/14*z*u^3*v+F(105)/11*u^4*v+F(57)/125*x^2*z*v^2-F(29)/46*x*y*z*v^2+14268*y^2*z*v^2+15030*x*z^2*v^2-8321*y*z^2*v^2+F(121)/126*z^3*v^2+F(122)/63*x^2*u*v^2-F(55)/98*x*y*u*v^2+14082*y^2*u*v^2+7886*x*z*u*v^2-9666*y*z*u*v^2+F(48)/35*z^2*u*v^2+F(8)/9*x*u^2*v^2+F(121)/26*y*u^2*v^2-F(89)/118*z*u^2*v^2+F(1)/114*u^3*v^2+F(59)/69*x*z*v^3+F(29)/26*y*z*v^3-F(105)/11*z^2*v^3+F(46)/15*x*u*v^3-F(94)/57*y*u*v^3-F(59)/46*z*u*v^3-F(115)/78*u^2*v^3-F(61)/43*z*v^4-F(77)/65*u*v^4,-F(8)/81*x^4*y-9303*x^3*y^2-12771*x^2*y^3-F(75)/82*x*y^4-F(4)/109*x^4*z+6985*x^3*y*z-F(67)/90*x^2*y^2*z+F(29)/27*x*y^3*z+F(104)/21*y^4*z-F(113)/99*x^3*z^2-14676*x^2*y*z^2-F(89)/84*x*y^2*z^2+F(112)/55*y^3*z^2+2275*x^2*z^3-F(26)/33*x*y*z^3+3572*y^2*z^3+F(29)/7*x*z^4+F(51)/10*y*z^4+12418*z^5+15665*x^4*u+1694*x^3*y*u-F(51)/28*x^2*y^2*u+1991*x*y^3*u-F(49)/82*x^3*z*u+F(80)/47*x^2*y*z*u-11796*x*y^2*z*u-F(53)/92*y^3*z*u+3857*x^2*z^2*u-F(103)/27*x*y*z^2*u-F(125)/17*y^2*z^2*u+F(1)/30*x*z^3*u+F(105)/124*y*z^3*u-7762*z^4*u+F(1)/60*x^3*u^2-F(29)/68*x^2*y*u^2-13342*x*y^2*u^2+F(13)/125*x^2*z*u^2-F(6)/121*x*y*z*u^2+5455*y^2*z*u^2-F(75)/121*x*z^2*u^2-F(5)/98*y*z^2*u^2-F(73)/69*z^3*u^2-F(39)/107*x^2*u^3-F(73)/106*x*y*u^3-F(105)/64*x*z*u^3-12894*y*z*u^3-F(71)/125*z^2*u^3-F(1)/54*x*u^4+F(111)/59*z*u^4+F(57)/116*x^4*v-F(50)/59*x^3*y*v-F(114)/17*x^2*y^2*v+6316*x*y^3*v+10795*x^3*z*v+F(83)/43*x^2*y*z*v-39*x*y^2*z*v+F(73)/28*y^3*z*v-1742*x^2*z^2*v+F(110)/9*x*y*z^2*v+F(87)/41*y^2*z^2*v+F(63)/82*x*z^3*v-12218*y*z^3*v-F(17)/83*z^4*v+14203*x^3*u*v+5365*x^2*y*u*v-F(125)/88*x*y^2*u*v-F(125)/37*x^2*z*u*v+F(104)/85*x*y*z*u*v+F(96)/97*y^2*z*u*v-8338*x*z^2*u*v-F(71)/35*y*z^2*u*v-F(6)/125*z^3*u*v-F(22)/57*x^2*u^2*v-F(29)/93*x*y*u^2*v+5249*x*z*u^2*v-3291*y*z*u^2*v+F(99)/104*z^2*u^2*v+F(105)/11*x*u^3*v+F(73)/79*z*u^3*v+F(122)/63*x^3*v^2-F(55)/98*x^2*y*v^2+14082*x*y^2*v^2+F(44)/45*x^2*z*v^2+F(35)/19*x*y*z*v^2+1677*y^2*z*v^2+F(68)/123*x*z^2*v^2+3912*y*z^2*v^2-13762*z^3*v^2+F(8)/9*x^2*u*v^2+F(121)/26*x*y*u*v^2-F(36)/53*x*z*u*v^2-F(107)/5*y*z*u*v^2+14437*z^2*u*v^2+F(1)/114*x*u^2*v^2-1492*z*u^2*v^2+F(46)/15*x^2*v^3-F(94)/57*x*y*v^3-F(39)/50*x*z*v^3-F(120)/77*y*z*v^3-F(65)/61*z^2*v^3-F(115)/78*x*u*v^3+F(90)/107*z*u*v^3-F(77)/65*x*v^4+F(71)/16*z*v^4,F(8)/81*x^3*y^2+9303*x^2*y^3+12771*x*y^4+F(75)/82*y^5-F(105)/109*x^3*y*z+F(101)/49*x^2*y^2*z-F(91)/19*x*y^3*z+F(97)/85*y^4*z+F(41)/50*x^3*z^2+3379*x^2*y*z^2-10050*x*y^2*z^2+F(27)/88*y^3*z^2+151*x^2*z^3+F(4)/111*x*y*z^3-3893*y^2*z^3-F(7)/9*x*z^4+F(69)/73*y*z^4+F(1)/47*z^5-15665*x^3*y*u-1694*x^2*y^2*u+F(51)/28*x*y^3*u-1991*y^4*u+F(23)/24*x^3*z*u-F(23)/50*x^2*y*z*u+F(11)/84*x*y^2*z*u-10858*y^3*z*u-12943*x^2*z^2*u-5003*x*y*z^2*u+F(89)/27*y^2*z^2*u+F(84)/65*x*z^3*u-F(20)/93*y*z^3*u-7415*z^4*u-F(1)/60*x^2*y*u^2+F(29)/68*x*y^2*u^2+13342*y^3*u^2-F(95)/99*x^2*z*u^2+F(55)/48*x*y*z*u^2+F(76)/53*y^2*z*u^2+F(15)/86*x*z^2*u^2-F(46)/83*y*z^2*u^2+8039*z^3*u^2+F(39)/107*x*y*u^3+F(73)/106*y^2*u^3+F(84)/25*x*z*u^3+F(58)/73*y*z*u^3+F(77)/37*z^2*u^3+F(1)/54*y*u^4-F(105)/109*z*u^4-F(57)/116*x^3*y*v+F(50)/59*x^2*y^2*v+F(114)/17*x*y^3*v-6316*y^4*v-F(103)/84*x^3*z*v+F(82)/87*x^2*y*z*v-3669*x*y^2*z*v+F(113)/86*y^3*z*v+F(121)/15*x^2*z^2*v+F(26)/35*x*y*z^2*v-5137*y^2*z^2*v+F(118)/121*x*z^3*v-4254*y*z^3*v+F(37)/83*z^4*v-14203*x^2*y*u*v-5365*x*y^2*u*v+F(125)/88*y^3*u*v+11189*x^2*z*u*v-F(62)/89*x*y*z*u*v-7514*y^2*z*u*v-F(90)/13*x*z^2*u*v-F(32)/97*y*z^2*u*v-6443*z^3*u*v+F(22)/57*x*y*u^2*v+F(29)/93*y^2*u^2*v-2788*x*z*u^2*v-6165*y*z*u^2*v+8180*z^2*u^2*v-F(105)/11*y*u^3*v+11462*z*u^3*v-F(122)/63*x^2*y*v^2+F(55)/98*x*y^2*v^2-14082*y^3*v^2+F(10)/53*x^2*z*v^2-F(115)/41*x*y*z*v^2+F(49)/31*y^2*z*v^2-F(32)/3*x*z^2*v^2-F(101)/106*y*z^2*v^2+F(78)/101*z^3*v^2-F(8)/9*x*y*u*v^2-F(121)/26*y^2*u*v^2-5368*x*z*u*v^2-F(59)/113*y*z*u*v^2+F(33)/19*z^2*u*v^2-F(1)/114*y*u^2*v^2-F(101)/119*z*u^2*v^2-F(46)/15*x*y*v^3+F(94)/57*y^2*v^3-826*x*z*v^3+F(56)/17*y*z*v^3+F(20)/17*z^2*v^3+F(115)/78*y*u*v^3-13028*z*u*v^3+F(77)/65*y*v^4+F(76)/107*z*v^4,x^4*y-F(89)/70*x^3*y^2-F(108)/65*x^2*y^3-F(120)/29*x*y^4-F(104)/21*y^5-F(41)/50*x^4*z+F(53)/121*x^3*y*z+F(125)/22*x^2*y^2*z+12032*x*y^3*z-F(112)/55*y^4*z-151*x^3*z^2+F(63)/115*x^2*y*z^2-11617*x*y^2*z^2-3572*y^3*z^2+F(7)/9*x^2*z^3+F(48)/89*x*y*z^3-F(51)/10*y^2*z^3-F(1)/47*x*z^4-12418*y*z^4-F(23)/24*x^4*u-748*x^3*y*u+F(5)/68*x^2*y^2*u-9337*x*y^3*u+F(53)/92*y^4*u+12943*x^3*z*u+F(97)/28*x^2*y*z*u+F(14)/27*x*y^2*z*u+F(125)/17*y^3*z*u-F(84)/65*x^2*z^2*u-13381*x*y*z^2*u-F(105)/124*y^2*z^2*u+7415*x*z^3*u+7762*y*z^3*u+F(95)/99*x^3*u^2-F(71)/108*x^2*y*u^2-F(87)/16*x*y^2*u^2-5455*y^3*u^2-F(15)/86*x^2*z*u^2-F(89)/49*x*y*z*u^2+F(5)/98*y^2*z*u^2-8039*x*z^2*u^2+F(73)/69*y*z^2*u^2-F(84)/25*x^2*u^3+F(28)/25*x*y*u^3+12894*y^2*u^3-F(77)/37*x*z*u^3+F(71)/125*y*z*u^3+F(105)/109*x*u^4-F(111)/59*y*u^4+F(103)/84*x^4*v+F(55)/108*x^3*y*v+5899*x^2*y^2*v-10006*x*y^3*v-F(73)/28*y^4*v-F(121)/15*x^3*z*v-F(25)/74*x^2*y*z*v-F(17)/43*x*y^2*z*v-F(87)/41*y^3*z*v-F(118)/121*x^2*z^2*v-10962*x*y*z^2*v+12218*y^2*z^2*v-F(37)/83*x*z^3*v+F(17)/83*y*z^3*v-11189*x^3*u*v-1288*x^2*y*u*v-F(106)/111*x*y^2*u*v-F(96)/97*y^3*u*v+F(90)/13*x^2*z*u*v+F(8)/39*x*y*z*u*v+F(71)/35*y^2*z*u*v+6443*x*z^2*u*v+F(6)/125*y*z^2*u*v+2788*x^2*u^2*v+F(69)/35*x*y*u^2*v+3291*y^2*u^2*v-8180*x*z*u^2*v-F(99)/104*y*z*u^2*v-11462*x*u^3*v-F(73)/79*y*u^3*v-F(10)/53*x^3*v^2-10159*x^2*y*v^2-F(39)/59*x*y^2*v^2-1677*y^3*v^2+F(32)/3*x^2*z*v^2+F(117)/41*x*y*z*v^2-3912*y^2*z*v^2-F(78)/101*x*z^2*v^2+13762*y*z^2*v^2+5368*x^2*u*v^2+4595*x*y*u*v^2+F(107)/5*y^2*u*v^2-F(33)/19*x*z*u*v^2-14437*y*z*u*v^2+F(101)/119*x*u^2*v^2+1492*y*u^2*v^2+826*x^2*v^3+F(40)/29*x*y*v^3+F(120)/77*y^2*v^3-F(20)/17*x*z*v^3+F(65)/61*y*z*v^3+13028*x*u*v^3-F(90)/107*y*u*v^3-F(76)/107*x*v^4-F(71)/16*y*v^4,-F(79)/39*x^3*y+F(107)/91*x^2*y^2+F(83)/34*x*y^3-7626*y^4-5574*x^3*z+F(39)/35*x^2*y*z+F(126)/17*x*y^2*z+F(64)/89*y^3*z+2994*x^2*z^2-4550*x*y*z^2+5636*y^2*z^2+12390*x*z^3+F(73)/99*y*z^3-F(23)/126*z^4-120*x^3*u-F(67)/44*x^2*y*u+F(124)/123*x*y^2*u+F(59)/92*y^3*u-F(122)/67*x^2*z*u+6675*x*y*z*u-F(29)/20*y^2*z*u-F(99)/17*x*z^2*u+F(103)/43*y*z^2*u-F(101)/42*z^3*u+F(47)/22*x^2*u^2+3980*x*y*u^2+7317*y^2*u^2+10838*x*z*u^2+14863*y*z*u^2-F(123)/59*z^2*u^2-F(77)/3*x*u^3-1097*y*u^3+12169*z*u^3+F(71)/93*u^4+14917*x^3*v+7920*x^2*y*v-6771*x*y^2*v-7070*y^3*v+F(5)/72*x^2*z*v+F(53)/50*x*y*z*v+F(96)/53*y^2*z*v-F(117)/94*x*z^2*v+9162*y*z^2*v+F(97)/93*z^3*v+F(123)/115*x^2*u*v+11747*x*y*u*v-F(46)/67*y^2*u*v+F(47)/57*x*z*u*v-F(3)/82*y*z*u*v-F(7)/22*z^2*u*v+F(109)/3*x*u^2*v-F(121)/92*y*u^2*v+F(69)/8*z*u^2*v-9764*u^3*v+F(10)/31*x^2*v^2+F(67)/104*x*y*v^2-15698*y^2*v^2+6351*x*z*v^2-F(25)/97*y*z*v^2+5904*z^2*v^2+F(68)/11*x*u*v^2-2640*y*u*v^2-F(121)/35*z*u*v^2-8426*u^2*v^2+F(35)/61*x*v^3-4024*y*v^3-F(75)/52*z*v^3-86*u*v^3-F(29)/91*v^4,-3883*x^3*y-140*x^2*y^2-14453*x*y^3+F(103)/9*y^4-F(66)/85*x^3*z-10611*x^2*y*z+F(82)/25*x*y^2*z+2375*y^3*z-F(13)/55*x^2*z^2-7003*x*y*z^2+5837*y^2*z^2-F(57)/91*x*z^3+F(79)/26*y*z^3-13616*z^4+1988*x^3*u-F(73)/78*x^2*y*u+12594*x*y^2*u-F(29)/68*y^3*u-11864*x^2*z*u-F(19)/26*x*y*z*u-3793*y^2*z*u+F(114)/107*x*z^2*u-3088*y*z^2*u-3389*z^3*u+675*x^2*u^2+F(114)/77*x*y*u^2+F(26)/115*y^2*u^2+4789*x*z*u^2+2530*y*z*u^2+F(89)/124*z^2*u^2-F(43)/95*x*u^3-15877*y*u^3-F(103)/96*z*u^3+F(43)/70*u^4+F(13)/107*x^3*v-6559*x^2*y*v+F(82)/61*x*y^2*v+F(87)/71*y^3*v-12168*x^2*z*v-13096*x*y*z*v-F(17)/44*y^2*z*v+F(121)/15*x*z^2*v-F(1)/2*y*z^2*v-3849*z^3*v+F(96)/91*x^2*u*v+10545*x*y*u*v+F(62)/51*y^2*u*v-F(16)/9*x*z*u*v-F(16)/47*y*z*u*v+F(42)/41*z^2*u*v-4132*x*u^2*v+3304*y*u^2*v-F(22)/95*z*u^2*v-14118*u^3*v-F(76)/87*x^2*v^2+F(89)/102*x*y*v^2+F(28)/97*y^2*v^2+9948*x*z*v^2+F(56)/55*y*z*v^2+F(50)/123*z^2*v^2-F(9)/94*x*u*v^2-362*y*u*v^2-14510*z*u*v^2-15910*u^2*v^2+F(29)/4*x*v^3+F(113)/115*y*v^3+F(53)/96*z*v^3-15873*u*v^3+F(12)/43*v^4,F(19)/82*x^4+4634*x^3*y+F(18)/13*x^2*y^2+F(23)/93*x*y^3-F(59)/81*y^4+F(88)/53*x^3*z-F(54)/43*x^2*y*z+F(42)/61*x*y^2*z-F(31)/11*y^3*z-F(74)/7*x^2*z^2+1008*x*y*z^2-F(24)/103*y^2*z^2+F(86)/57*x*z^3+F(35)/109*y*z^3+F(1)/15*z^4+F(51)/7*x^3*u+8877*x^2*y*u+F(61)/121*x*y^2*u-F(65)/17*y^3*u-F(2)/39*x^2*z*u+8975*x*y*z*u-F(121)/26*y^2*z*u-F(37)/65*x*z^2*u-F(23)/14*y*z^2*u-F(56)/99*z^3*u+13684*x^2*u^2-F(37)/30*x*y*u^2-F(119)/23*y^2*u^2+F(82)/73*x*z*u^2-F(83)/67*y*z*u^2+13276*z^2*u^2-100*x*u^3+F(41)/61*y*u^3-F(98)/75*z*u^3-13040*u^4-F(43)/64*x^3*v+F(85)/31*x^2*y*v+10598*x*y^2*v-F(53)/35*y^3*v-6444*x^2*z*v-F(119)/62*x*y*z*v+F(71)/42*y^2*z*v-14388*x*z^2*v-F(125)/81*y*z^2*v+F(107)/22*z^3*v-F(107)/82*x^2*u*v-8548*x*y*u*v+F(65)/81*y^2*u*v-F(85)/6*x*z*u*v+11175*y*z*u*v-F(10)/73*z^2*u*v-F(76)/99*x*u^2*v-F(87)/107*y*u^2*v+F(118)/19*z*u^2*v+1694*u^3*v+4543*x^2*v^2-9860*x*y*v^2-3277*y^2*v^2-F(74)/59*x*z*v^2+F(59)/83*y*z*v^2-F(103)/17*z^2*v^2+15590*x*u*v^2+6630*y*u*v^2+2788*z*u*v^2+F(53)/119*u^2*v^2+F(46)/117*x*v^3-11721*y*v^3-644*z*v^3-F(52)/69*u*v^3-F(17)/124*v^4]))
end


@doc raw"""
    elliptic_d10_pi9()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `10` and sectional genus `9`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d10_pi9()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(44)/83*x^3*y^2-F(51)/22*x^2*y^3+F(104)/11*x*y^4+F(38)/119*y^5+F(22)/87*x^3*y*z-F(27)/61*x^2*y^2*z-F(49)/121*x*y^3*z-F(60)/43*y^4*z+F(124)/65*x^3*z^2+F(39)/107*x^2*y*z^2-10379*x*y^2*z^2-F(86)/125*y^3*z^2+F(33)/8*x^2*z^3+3501*x*y*z^3+F(53)/9*y^2*z^3-F(69)/106*x*z^4+441*y*z^4-F(19)/126*z^5+12584*x^3*y*u+F(21)/104*x^2*y^2*u+12937*x*y^3*u-F(113)/108*y^4*u+13758*x^3*z*u+4991*x^2*y*z*u+5360*x*y^2*z*u-11946*y^3*z*u-2442*x^2*z^2*u-9336*x*y*z^2*u+9027*y^2*z^2*u-F(112)/37*x*z^3*u+14035*y*z^3*u-8230*z^4*u+F(58)/83*x^3*u^2+9675*x^2*y*u^2+F(29)/114*x*y^2*u^2+10729*y^3*u^2+F(11)/23*x^2*z*u^2+3609*x*y*z*u^2+7065*y^2*z*u^2-2477*x*z^2*u^2+F(32)/37*y*z^2*u^2-F(109)/89*z^3*u^2-F(68)/109*x^2*u^3-F(107)/45*x*y*u^3-F(65)/53*y^2*u^3+12249*x*z*u^3-11480*y*z*u^3+F(4)/119*z^2*u^3+4418*x*u^4+11875*y*u^4-6961*z*u^4+2678*u^5-F(76)/71*x^3*y*v-F(87)/4*x^2*y^2*v+F(119)/27*x*y^3*v+F(20)/31*y^4*v+F(122)/65*x^3*z*v-F(116)/61*x^2*y*z*v-10976*x*y^2*z*v-5720*y^3*z*v-12559*x^2*z^2*v-F(109)/102*x*y*z^2*v-F(123)/121*y^2*z^2*v-F(18)/73*x*z^3*v-F(89)/124*y*z^3*v-F(50)/109*z^4*v-F(113)/50*x^3*u*v+F(31)/79*x^2*y*u*v-14121*x*y^2*u*v+F(11)/123*y^3*u*v-3754*x^2*z*u*v-1676*x*y*z*u*v-F(117)/113*y^2*z*u*v-11336*x*z^2*u*v-F(35)/44*y*z^2*u*v-F(116)/21*z^3*u*v+10245*x^2*u^2*v+F(17)/4*x*y*u^2*v-6432*y^2*u^2*v-12632*x*z*u^2*v+F(80)/47*y*z*u^2*v-F(9)/115*z^2*u^2*v-2607*x*u^3*v-F(79)/82*y*u^3*v-F(71)/63*z*u^3*v+F(71)/102*u^4*v-5212*x^3*v^2-2079*x^2*y*v^2+F(104)/107*x*y^2*v^2+F(50)/19*y^3*v^2+14719*x^2*z*v^2-F(7)/115*x*y*z*v^2-F(113)/6*y^2*z*v^2+F(123)/100*x*z^2*v^2+F(22)/9*y*z^2*v^2+15772*z^3*v^2-F(10)/111*x^2*u*v^2-F(35)/52*x*y*u*v^2-8041*y^2*u*v^2-F(17)/38*x*z*u*v^2-11677*y*z*u*v^2+F(54)/67*z^2*u*v^2+7941*x*u^2*v^2-8500*y*u^2*v^2-147*z*u^2*v^2-F(101)/68*u^3*v^2+7864*x^2*v^3-F(120)/61*x*y*v^3+13984*y^2*v^3+F(81)/101*x*z*v^3-F(79)/20*y*z*v^3-F(95)/7*z^2*v^3-F(19)/89*x*u*v^3-5837*y*u*v^3+F(112)/95*z*u*v^3+F(119)/87*u^2*v^3-F(51)/50*x*v^4-F(85)/16*y*v^4+F(3)/44*z*v^4+5109*u*v^4-9320*v^5,F(44)/83*x^4*y+7230*x^3*y^2-F(110)/69*x^2*y^3+5570*x*y^4+87*y^5-F(103)/18*x^4*z+F(93)/70*x^3*y*z+F(75)/37*x^2*y^2*z-61*x*y^3*z+7367*y^4*z+F(5)/31*x^3*z^2+9111*x^2*y*z^2-F(97)/44*x*y^2*z^2+F(28)/55*y^3*z^2+F(89)/12*x^2*z^3+1832*x*y*z^3-15162*y^2*z^3+F(81)/100*x*z^4+F(52)/9*y*z^4-F(3)/17*z^5+F(23)/118*x^4*u+F(101)/55*x^3*y*u-F(33)/47*x^2*y^2*u-149*x*y^3*u+7961*y^4*u-7891*x^3*z*u-F(81)/14*x^2*y*z*u+10246*x*y^2*z*u+F(51)/125*y^3*z*u+F(43)/64*x^2*z^2*u-F(68)/37*x*y*z^2*u+F(11)/5*y^2*z^2*u+5578*x*z^3*u-5440*y*z^3*u+F(61)/118*z^4*u-4318*x^3*u^2-69*x^2*y*u^2-15141*x*y^2*u^2+F(12)/13*y^3*u^2+12180*x^2*z*u^2-F(67)/6*x*y*z*u^2-F(18)/121*y^2*z*u^2+738*x*z^2*u^2-F(91)/115*y*z^2*u^2+F(105)/8*z^3*u^2-14085*x^2*u^3-5451*x*y*u^3+F(91)/79*y^2*u^3-F(43)/113*x*z*u^3-F(36)/11*y*z*u^3-F(89)/122*z^2*u^3+F(101)/59*x*u^4-F(125)/117*y*u^4-F(113)/25*z*u^4+F(63)/118*u^5+F(13)/106*x^4*v+11326*x^3*y*v+2583*x^2*y^2*v+14291*x*y^3*v+11795*y^4*v-9923*x^3*z*v+10823*x^2*y*z*v+F(30)/31*x*y^2*z*v-6558*y^3*z*v+F(89)/20*x^2*z^2*v-F(111)/109*x*y*z^2*v+10420*y^2*z^2*v-F(81)/44*x*z^3*v-F(76)/3*y*z^3*v-F(31)/55*z^4*v+3423*x^3*u*v+F(47)/80*x^2*y*u*v-7843*x*y^2*u*v-15365*y^3*u*v+978*x^2*z*u*v+F(73)/24*x*y*z*u*v-F(2)/39*y^2*z*u*v-F(28)/75*x*z^2*u*v+1849*y*z^2*u*v-10885*z^3*u*v+F(120)/91*x^2*u^2*v+9293*x*y*u^2*v-8837*y^2*u^2*v+F(109)/105*x*z*u^2*v+F(7)/93*y*z*u^2*v+4076*z^2*u^2*v-F(106)/5*x*u^3*v+8053*y*u^3*v-90*z*u^3*v+8598*u^4*v+F(81)/76*x^3*v^2-F(103)/42*x^2*y*v^2-6873*x*y^2*v^2+F(49)/73*y^3*v^2+8447*x^2*z*v^2+3846*x*y*z*v^2+F(107)/126*y^2*z*v^2+2145*x*z^2*v^2+F(53)/105*y*z^2*v^2-F(62)/75*z^3*v^2-F(11)/35*x^2*u*v^2+7066*x*y*u*v^2+F(69)/59*y^2*u*v^2-F(53)/64*x*z*u*v^2+8587*y*z*u*v^2+F(58)/125*z^2*u*v^2-15386*x*u^2*v^2+F(9)/121*y*u^2*v^2+14966*z*u^2*v^2-2518*u^3*v^2-F(37)/56*x^2*v^3-11840*x*y*v^3-F(23)/47*y^2*v^3+F(110)/81*x*z*v^3-81*y*z*v^3+2719*z^2*v^3-F(65)/118*x*u*v^3-1221*y*u*v^3-F(33)/82*z*u*v^3-F(106)/15*u^2*v^3+F(39)/112*x*v^4+13479*y*v^4-F(126)/109*z*v^4-2251*u*v^4+8907*v^5,-10723*x^3*y*z+F(16)/79*x^2*y^2*z+F(62)/49*x*y^3*z-5204*y^4*z-F(49)/3*x^3*z^2+F(91)/71*x^2*y*z^2+F(77)/3*x*y^2*z^2-F(88)/43*y^3*z^2-13949*x^2*z^3-9045*x*y*z^3+F(109)/16*y^2*z^3-F(89)/62*x*z^4-15674*y*z^4-10313*z^5-6773*x^3*y*u+F(124)/61*x^2*y^2*u-7483*x*y^3*u+F(18)/23*y^4*u-5136*x^3*z*u-F(3)/31*x^2*y*z*u-11796*x*y^2*z*u-F(56)/57*y^3*z*u-F(68)/95*x^2*z^2*u-7917*x*y*z^2*u+1349*y^2*z^2*u-F(17)/56*x*z^3*u+14713*y*z^3*u-7938*z^4*u+3104*x^3*u^2+14197*x^2*y*u^2-15411*x*y^2*u^2-14704*y^3*u^2+F(24)/77*x^2*z*u^2+F(78)/19*x*y*z*u^2-F(23)/51*y^2*z*u^2+F(98)/61*x*z^2*u^2+F(60)/71*y*z^2*u^2-F(103)/51*z^3*u^2-F(103)/42*x^2*u^3+F(15)/79*x*y*u^3+F(101)/62*y^2*u^3-15165*x*z*u^3+F(83)/5*y*z*u^3-F(61)/60*z^2*u^3+F(7)/78*x*u^4-3907*y*u^4+6635*z*u^4-12205*u^5-388*x^3*y*v+F(63)/59*x^2*y^2*v+F(88)/109*x*y^3*v-13978*y^4*v+F(80)/53*x^3*z*v+645*x^2*y*z*v+F(117)/25*x*y^2*z*v-F(81)/53*y^3*z*v+F(43)/72*x^2*z^2*v+F(27)/8*x*y*z^2*v-10620*y^2*z^2*v-13547*x*z^3*v-8282*y*z^3*v+13426*z^4*v-F(58)/119*x^3*u*v+480*x^2*y*u*v-15697*x*y^2*u*v-F(25)/58*y^3*u*v-115*x^2*z*u*v-F(46)/123*x*y*z*u*v-F(22)/13*y^2*z*u*v-F(88)/105*x*z^2*u*v-F(111)/62*y*z^2*u*v+3415*z^3*u*v-F(117)/77*x^2*u^2*v+F(18)/29*x*y*u^2*v-F(80)/91*y^2*u^2*v+7851*x*z*u^2*v-11079*y*z*u^2*v+F(39)/50*z^2*u^2*v+14917*x*u^3*v+1131*y*u^3*v+13259*z*u^3*v-F(122)/25*u^4*v-F(109)/37*x^3*v^2+F(7)/78*x^2*y*v^2-F(73)/13*x*y^2*v^2-5623*y^3*v^2+11389*x^2*z*v^2+F(63)/68*x*y*z*v^2+F(28)/27*y^2*z*v^2+F(69)/112*x*z^2*v^2-10851*y*z^2*v^2+15829*z^3*v^2+F(86)/51*x^2*u*v^2+2298*x*y*u*v^2+F(17)/103*y^2*u*v^2-F(97)/2*x*z*u*v^2+F(1)/5*y*z*u*v^2-F(104)/119*z^2*u*v^2+F(2)/95*x*u^2*v^2+11655*y*u^2*v^2-F(42)/65*z*u^2*v^2+F(37)/26*u^3*v^2-12355*x^2*v^3+35*x*y*v^3+F(20)/7*y^2*v^3+8032*x*z*v^3+9572*y*z*v^3+11508*z^2*v^3+10858*x*u*v^3+F(115)/81*y*u*v^3+10973*z*u*v^3+10832*u^2*v^3+F(33)/101*x*v^4-F(71)/88*y*v^4+F(67)/18*z*v^4-15473*u*v^4+F(11)/107*v^5,-F(28)/81*x^3*y^2-F(126)/61*x^2*y^3-F(70)/23*x*y^4+14521*y^5-F(3)/55*x^3*y*z-F(100)/107*x^2*y^2*z-10724*x*y^3*z+2013*y^4*z+1241*x^3*z^2+F(61)/104*x^2*y*z^2+3577*x*y^2*z^2-2748*y^3*z^2-15718*x^2*z^3+F(106)/5*x*y*z^3-F(41)/93*y^2*z^3-F(95)/61*x*z^4-F(1)/30*y*z^4-F(73)/4*z^5-F(59)/56*x^3*y*u-14867*x^2*y^2*u-11840*x*y^3*u+7444*y^4*u-F(38)/41*x^3*z*u-153*x^2*y*z*u-2654*x*y^2*z*u+F(73)/83*y^3*z*u+6940*x^2*z^2*u-1241*x*y*z^2*u+F(79)/17*y^2*z^2*u-F(39)/35*x*z^3*u-4590*y*z^3*u+F(8)/109*z^4*u+F(73)/34*x^3*u^2+15609*x^2*y*u^2+12721*x*y^2*u^2-F(95)/31*y^3*u^2+F(33)/14*x^2*z*u^2+F(77)/65*x*y*z*u^2-14476*y^2*z*u^2+F(73)/101*x*z^2*u^2-F(73)/101*y*z^2*u^2+F(92)/5*z^3*u^2-F(71)/94*x^2*u^3-F(97)/33*x*y*u^3+F(125)/82*y^2*u^3+10213*x*z*u^3-6039*y*z*u^3-F(87)/61*z^2*u^3-14084*x*u^4-F(43)/7*y*u^4+3169*z*u^4+4248*u^5+F(100)/101*x^3*y*v+624*x^2*y^2*v-4023*x*y^3*v-F(87)/65*y^4*v-2971*x^3*z*v-F(8)/21*x^2*y*z*v+8171*x*y^2*z*v-7776*y^3*z*v-F(55)/104*x^2*z^2*v+F(96)/113*x*y*z^2*v-7318*y^2*z^2*v+9436*x*z^3*v+F(42)/41*y*z^3*v-F(115)/14*z^4*v-2550*x^3*u*v+F(82)/29*x^2*y*u*v+3830*x*y^2*u*v+F(71)/97*y^3*u*v-F(98)/67*x^2*z*u*v-4319*x*y*z*u*v+F(33)/49*y^2*z*u*v-F(123)/49*x*z^2*u*v-F(35)/62*y*z^2*u*v-F(85)/6*z^3*u*v-10726*x^2*u^2*v+F(98)/85*x*y*u^2*v+F(24)/43*y^2*u^2*v-1382*x*z*u^2*v+F(39)/109*y*z*u^2*v-F(36)/71*z^2*u^2*v+15911*x*u^3*v-F(38)/35*y*u^3*v+2079*z*u^3*v+F(123)/29*u^4*v+4369*x^3*v^2-F(59)/126*x^2*y*v^2-11066*x*y^2*v^2+F(110)/117*y^3*v^2+14815*x^2*z*v^2-F(19)/49*x*y*z*v^2-F(29)/10*y^2*z*v^2+F(41)/38*x*z^2*v^2-F(96)/55*y*z^2*v^2+686*z^3*v^2+F(67)/30*x^2*u*v^2+F(123)/23*x*y*u*v^2-F(52)/25*y^2*u*v^2-55*x*z*u*v^2-F(23)/89*y*z*u*v^2-F(89)/77*z^2*u*v^2+9906*x*u^2*v^2-12766*y*u^2*v^2-F(104)/7*z*u^2*v^2+6533*u^3*v^2-F(65)/97*x^2*v^3+F(57)/2*x*y*v^3-11084*y^2*v^3+F(13)/81*x*z*v^3-F(23)/7*y*z*v^3-F(122)/111*z^2*v^3+224*x*u*v^3+5112*y*u*v^3+F(103)/30*z*u*v^3+F(78)/43*u^2*v^3+4811*x*v^4+6171*y*v^4-3422*z*v^4-1589*u*v^4-5541*v^5,F(28)/81*x^4*y-F(39)/88*x^3*y^2-F(101)/87*x^2*y^3+F(11)/10*x*y^4+F(29)/44*y^5-F(21)/94*x^4*z-F(100)/81*x^3*y*z+F(123)/73*x^2*y^2*z+5440*x*y^3*z-15190*y^4*z+14140*x^3*z^2+F(74)/27*x^2*y*z^2+F(92)/15*x*y^2*z^2+8048*y^3*z^2-F(95)/64*x^2*z^3-F(60)/89*x*y*z^3-F(11)/118*y^2*z^3+F(56)/15*x*z^4+F(102)/31*y*z^4-1571*z^5-2598*x^4*u-4426*x^3*y*u+2148*x^2*y^2*u-F(95)/11*x*y^3*u-6701*y^4*u-12741*x^3*z*u-1179*x^2*y*z*u+1671*x*y^2*z*u-F(93)/10*y^3*z*u-F(77)/78*x^2*z^2*u-F(35)/101*x*y*z^2*u-F(41)/89*y^2*z^2*u+1556*x*z^3*u+F(13)/121*y*z^3*u+1288*z^4*u-F(53)/89*x^3*u^2-F(3)/109*x^2*y*u^2-12018*x*y^2*u^2-5219*y^3*u^2+F(86)/49*x^2*z*u^2-F(1)/84*x*y*z*u^2+F(86)/113*y^2*z*u^2-F(85)/123*x*z^2*u^2+F(94)/85*y*z^2*u^2-10590*z^3*u^2-F(99)/38*x^2*u^3+2696*x*y*u^3+F(107)/13*y^2*u^3-144*x*z*u^3+F(8)/69*y*z*u^3-F(107)/33*z^2*u^3-F(117)/53*x*u^4+1924*y*u^4+F(75)/7*z*u^4+F(98)/113*u^5+13120*x^4*v-7308*x^3*y*v+F(49)/85*x^2*y^2*v+F(55)/64*x*y^3*v-4634*y^4*v+9344*x^3*z*v-F(126)/121*x^2*y*z*v+F(23)/81*x*y^2*z*v+5086*y^3*z*v-4215*x^2*z^2*v+2442*x*y*z^2*v-9336*y^2*z^2*v-F(43)/44*x*z^3*v+F(114)/109*y*z^3*v+10963*z^4*v-F(2)/111*x^3*u*v-F(5)/126*x^2*y*u*v-F(41)/56*x*y^2*u*v+F(47)/79*y^3*u*v+74*x^2*z*u*v-15011*x*y*z*u*v+11187*y^2*z*u*v-F(104)/115*x*z^2*u*v-F(94)/67*y*z^2*u*v+6674*z^3*u*v-F(5)/78*x^2*u^2*v-F(123)/79*x*y*u^2*v+F(7)/36*y^2*u^2*v+F(79)/97*x*z*u^2*v-F(55)/62*y*z*u^2*v-F(121)/3*z^2*u^2*v+F(109)/4*x*u^3*v+9102*y*u^3*v-9526*z*u^3*v+8388*u^4*v-F(47)/98*x^3*v^2+14967*x^2*y*v^2+5139*x*y^2*v^2+F(83)/16*y^3*v^2-15926*x^2*z*v^2+8971*x*y*z*v^2-15679*y^2*z*v^2+8686*x*z^2*v^2+13783*y*z^2*v^2-F(57)/103*z^3*v^2-F(73)/100*x^2*u*v^2-F(106)/119*x*y*u*v^2-F(98)/113*y^2*u*v^2+12096*x*z*u*v^2+14898*y*z*u*v^2+F(115)/61*z^2*u*v^2-F(125)/119*x*u^2*v^2+6048*y*u^2*v^2+F(1)/102*z*u^2*v^2+F(31)/125*u^3*v^2+F(7)/27*x^2*v^3+F(64)/123*x*y*v^3+2569*y^2*v^3+F(113)/96*x*z*v^3-7216*y*z*v^3+2236*z^2*v^3-7127*x*u*v^3-F(38)/17*y*u*v^3+F(94)/73*z*u*v^3+708*u^2*v^3+F(87)/88*x*v^4-15428*y*v^4-F(41)/64*z*v^4-F(55)/86*u*v^4-F(41)/89*v^5,F(109)/21*x^3*y*z-8388*x^2*y^2*z-F(17)/10*x*y^3*z-14404*y^4*z+12672*x^3*z^2+F(73)/52*x^2*y*z^2+13291*x*y^2*z^2+F(118)/57*y^3*z^2+6159*x^2*z^3+4294*x*y*z^3-F(35)/92*y^2*z^3-F(55)/116*x*z^4+F(83)/64*y*z^4-F(40)/81*z^5-F(21)/88*x^3*y*u-F(23)/58*x^2*y^2*u+F(83)/118*x*y^3*u+12766*y^4*u-F(45)/101*x^3*z*u-F(38)/71*x^2*y*z*u+F(13)/14*x*y^2*z*u+50*y^3*z*u-2123*x^2*z^2*u-F(61)/102*x*y*z^2*u+F(7)/111*y^2*z^2*u+F(48)/121*x*z^3*u-15030*y*z^3*u-F(16)/21*z^4*u-F(34)/121*x^3*u^2+F(34)/111*x^2*y*u^2+F(7)/48*x*y^2*u^2+1018*y^3*u^2-F(90)/71*x^2*z*u^2+10585*x*y*z*u^2-F(117)/56*y^2*z*u^2+15739*x*z^2*u^2-F(37)/123*y*z^2*u^2+4841*z^3*u^2-3527*x^2*u^3+F(107)/77*x*y*u^3+14476*y^2*u^3-F(53)/14*x*z*u^3-F(44)/41*y*z*u^3+F(83)/99*z^2*u^3+F(57)/83*x*u^4+1763*y*u^4+3291*z*u^4-2214*u^5+4710*x^3*y*v+4019*x^2*y^2*v+F(106)/103*x*y^3*v-F(33)/67*y^4*v-F(65)/74*x^3*z*v-F(19)/36*x^2*y*z*v+F(83)/97*x*y^2*z*v+3597*y^3*z*v+F(76)/25*x^2*z^2*v-2159*x*y*z^2*v+F(73)/67*y^2*z^2*v-F(68)/55*x*z^3*v+F(36)/29*y*z^3*v+F(96)/55*z^4*v-7809*x^3*u*v-F(101)/51*x^2*y*u*v+F(107)/99*x*y^2*u*v+3452*y^3*u*v-F(32)/63*x^2*z*u*v-F(61)/125*x*y*z*u*v+700*y^2*z*u*v+F(14)/45*x*z^2*u*v-F(10)/27*y*z^2*u*v-F(64)/109*z^3*u*v-12507*x^2*u^2*v-3286*x*y*u^2*v-8787*y^2*u^2*v+11244*x*z*u^2*v-7142*y*z*u^2*v+F(51)/52*z^2*u^2*v-F(11)/15*x*u^3*v+F(14)/15*y*u^3*v-F(71)/78*z*u^3*v+12840*u^4*v+1676*x^3*v^2-F(7)/22*x^2*y*v^2+13461*x*y^2*v^2+F(91)/17*y^3*v^2-211*x^2*z*v^2+2520*x*y*z*v^2+F(37)/91*y^2*z*v^2+F(57)/43*x*z^2*v^2+F(79)/61*y*z^2*v^2+F(126)/5*z^3*v^2+15390*x^2*u*v^2+2521*x*y*u*v^2-9728*y^2*u*v^2-F(31)/100*x*z*u*v^2-F(67)/120*y*z*u*v^2-F(2)/89*z^2*u*v^2+F(3)/40*x*u^2*v^2-F(123)/5*y*u^2*v^2+F(95)/44*z*u^2*v^2+F(55)/82*u^3*v^2+F(61)/73*x^2*v^3+4656*x*y*v^3-F(59)/61*y^2*v^3+F(53)/5*x*z*v^3+F(81)/19*y*z*v^3+7838*z^2*v^3-30*x*u*v^3-F(24)/7*y*u*v^3-F(88)/67*z*u*v^3-F(97)/109*u^2*v^3+F(1)/32*x*v^4-7809*y*v^4+F(115)/33*z*v^4+F(95)/9*u*v^4-F(51)/31*v^5,10723*x^3*y*u-F(16)/79*x^2*y^2*u-F(62)/49*x*y^3*u+5204*y^4*u+F(49)/3*x^3*z*u-F(91)/71*x^2*y*z*u-F(77)/3*x*y^2*z*u+F(88)/43*y^3*z*u+13949*x^2*z^2*u+9045*x*y*z^2*u-F(109)/16*y^2*z^2*u+F(89)/62*x*z^3*u+15674*y*z^3*u+10313*z^4*u+12767*x^3*u^2-3679*x^2*y*u^2+F(115)/94*x*y^2*u^2-F(95)/72*y^3*u^2+F(83)/126*x^2*z*u^2+3070*x*y*z*u^2+F(44)/91*y^2*z*u^2+F(105)/53*x*z^2*u^2-12618*y*z^2*u^2-13345*z^3*u^2+F(61)/47*x^2*u^3+F(108)/25*x*y*u^3-F(3)/17*y^2*u^3-F(76)/71*x*z*u^3-36*y*z*u^3+F(96)/47*z^2*u^3-7291*x*u^4+7566*y*u^4+1445*z*u^4-F(48)/79*u^5-4731*x^3*y*v+F(35)/38*x^2*y^2*v+F(26)/47*x*y^3*v-6365*y^4*v-15522*x^3*z*v+6045*x^2*y*z*v+F(121)/111*x*y^2*z*v-15013*y^3*z*v+4541*x^2*z^2*v+12683*x*y*z^2*v+F(21)/61*y^2*z^2*v+F(96)/29*x*z^3*v+F(98)/57*y*z^3*v+F(57)/112*z^4*v+1027*x^3*u*v+220*x^2*y*u*v+F(63)/122*x*y^2*u*v-6828*y^3*u*v+F(21)/92*x^2*z*u*v-13076*x*y*z*u*v+4327*y^2*z*u*v+F(65)/107*x*z^2*u*v-F(109)/120*y*z^2*u*v+F(46)/87*z^3*u*v-F(13)/102*x^2*u^2*v+F(20)/61*x*y*u^2*v-F(41)/47*y^2*u^2*v-7345*x*z*u^2*v+F(125)/42*y*z*u^2*v+F(124)/49*z^2*u^2*v-F(118)/115*x*u^3*v+F(53)/36*y*u^3*v-F(15)/56*z*u^3*v+F(107)/76*u^4*v-13479*x^3*v^2+F(104)/73*x^2*y*v^2+5583*x*y^2*v^2+12198*y^3*v^2-10858*x^2*z*v^2+244*x*y*z*v^2+15876*y^2*z*v^2-F(38)/123*x*z^2*v^2+F(93)/122*y*z^2*v^2+F(82)/15*z^3*v^2-35*x^2*u*v^2-F(90)/103*x*y*u*v^2-F(16)/73*y^2*u*v^2+5638*x*z*u*v^2-F(39)/35*y*z*u*v^2-F(82)/39*z^2*u*v^2+F(39)/106*x*u^2*v^2-3530*y*u^2*v^2+7014*z*u^2*v^2-F(9)/16*u^3*v^2+F(105)/46*x^2*v^3+F(1)/9*x*y*v^3-10361*y^2*v^3+3039*x*z*v^3-5956*y*z*v^3+750*z^2*v^3-2053*x*u*v^3+F(27)/112*y*u*v^3+F(4)/121*z*u*v^3-F(13)/108*u^2*v^3-F(70)/33*x*v^4-F(115)/108*y*v^4-F(59)/61*z*v^4-10442*u*v^4-F(17)/39*v^5,-F(80)/61*x^3*y^2+F(68)/57*x^2*y^3-F(35)/101*x*y^4+F(19)/75*y^5+635*x^3*y*z+287*x^2*y^2*z-F(31)/49*x*y^3*z+F(89)/93*y^4*z-F(20)/41*x^3*z^2-10544*x^2*y*z^2+14618*x*y^2*z^2-F(92)/49*y^3*z^2+F(33)/86*x^2*z^3-F(1)/121*x*y*z^3-9325*y^2*z^3+F(2)/67*x*z^4-F(49)/106*y*z^4+15003*z^5+F(67)/49*x^3*y*u-15884*x^2*y^2*u-F(11)/9*x*y^3*u+9349*y^4*u+F(63)/20*x^3*z*u+F(79)/25*x^2*y*z*u-F(23)/77*x*y^2*z*u+F(99)/20*y^3*z*u-8233*x^2*z^2*u-F(5)/84*x*y*z^2*u+F(71)/97*y^2*z^2*u+F(124)/43*x*z^3*u-F(64)/101*y*z^3*u-13567*z^4*u-F(59)/84*x^3*u^2+F(55)/94*x^2*y*u^2+7962*x*y^2*u^2+F(36)/95*y^3*u^2+F(5)/8*x^2*z*u^2+F(104)/27*x*y*z*u^2+F(59)/12*y^2*z*u^2+8231*x*z^2*u^2-138*y*z^2*u^2-8432*z^3*u^2+F(25)/37*x^2*u^3+F(19)/97*x*y*u^3+F(55)/97*y^2*u^3+12659*x*z*u^3+F(27)/119*y*z*u^3-F(101)/6*z^2*u^3+F(69)/103*x*u^4-F(111)/124*y*u^4+F(96)/107*z*u^4-F(97)/5*u^5+11725*x^3*y*v+F(16)/105*x^2*y^2*v-F(79)/16*x*y^3*v-F(43)/107*y^4*v+F(17)/54*x^3*z*v-12757*x^2*y*z*v+F(7)/124*x*y^2*z*v-F(93)/73*y^3*z*v+F(69)/22*x^2*z^2*v+F(73)/40*x*y*z^2*v+15350*y^2*z^2*v+F(108)/65*x*z^3*v+190*y*z^3*v+4395*z^4*v+14431*x^3*u*v+8*x^2*y*u*v+6264*x*y^2*u*v+F(38)/41*y^3*u*v-15601*x^2*z*u*v-6559*x*y*z*u*v-4074*y^2*z*u*v+8619*x*z^2*u*v+F(83)/28*y*z^2*u*v+15701*z^3*u*v-F(35)/37*x^2*u^2*v+F(75)/62*x*y*u^2*v+9293*y^2*u^2*v-F(32)/123*x*z*u^2*v-F(64)/15*y*z*u^2*v+F(101)/106*z^2*u^2*v-F(13)/111*x*u^3*v+F(89)/37*y*u^3*v+F(49)/36*z*u^3*v-F(78)/125*u^4*v+F(32)/87*x^3*v^2-15591*x^2*y*v^2-637*x*y^2*v^2+F(31)/61*y^3*v^2+F(39)/41*x^2*z*v^2-15874*x*y*z*v^2-12988*y^2*z*v^2+F(37)/109*x*z^2*v^2-F(18)/119*y*z^2*v^2-7257*z^3*v^2+F(97)/16*x^2*u*v^2+F(21)/29*x*y*u*v^2+F(81)/95*y^2*u*v^2-F(109)/34*x*z*u*v^2-F(35)/27*y*z*u*v^2+13439*z^2*u*v^2-F(55)/81*x*u^2*v^2-F(23)/90*y*u^2*v^2-F(79)/37*z*u^2*v^2-F(60)/103*u^3*v^2+F(41)/24*x^2*v^3-F(95)/54*x*y*v^3-F(104)/25*y^2*v^3-F(27)/97*x*z*v^3+F(43)/52*y*z*v^3+F(82)/65*z^2*v^3+F(15)/82*x*u*v^3+F(1)/44*y*u*v^3+F(65)/119*z*u*v^3-F(5)/32*u^2*v^3-1039*x*v^4+F(39)/31*y*v^4-2847*z*v^4-F(35)/123*u*v^4+F(59)/20*v^5,F(80)/61*x^4*y-F(116)/11*x^3*y^2-F(79)/103*x^2*y^3-10762*x*y^4-F(67)/101*y^5+F(83)/38*x^4*z+F(61)/10*x^3*y*z+F(31)/8*x^2*y^2*z-F(107)/103*x*y^3*z-F(85)/113*y^4*z-F(3)/97*x^3*z^2+6296*x^2*y*z^2+F(47)/86*x*y^2*z^2+15041*y^3*z^2+14353*x^2*z^3+1990*x*y*z^3+15780*y^2*z^3-F(92)/43*x*z^4+F(64)/47*y*z^4-F(108)/7*z^5+F(88)/49*x^4*u+F(98)/57*x^3*y*u-F(41)/44*x^2*y^2*u+F(92)/119*x*y^3*u+F(43)/71*y^4*u+14335*x^3*z*u+F(58)/7*x^2*y*z*u-F(79)/39*x*y^2*z*u-3487*y^3*z*u+F(56)/83*x^2*z^2*u-F(75)/73*x*y*z^2*u-7173*y^2*z^2*u-F(99)/64*x*z^3*u-F(51)/65*y*z^3*u+11138*z^4*u+8299*x^3*u^2+F(47)/116*x^2*y*u^2+F(98)/95*x*y^2*u^2+F(117)/53*y^3*u^2-F(123)/46*x^2*z*u^2+F(125)/74*x*y*z*u^2-F(82)/19*y^2*z*u^2-4508*x*z^2*u^2+F(49)/29*y*z^2*u^2-3337*z^3*u^2-F(109)/7*x^2*u^3+F(103)/41*x*y*u^3+F(28)/13*y^2*u^3-6790*x*z*u^3-8289*y*z*u^3+F(59)/41*z^2*u^3+15888*x*u^4-9252*y*u^4+F(10)/99*z*u^4+1485*u^5-F(101)/23*x^4*v-6626*x^3*y*v+F(19)/25*x^2*y^2*v-F(73)/49*x*y^3*v-F(17)/14*y^4*v-F(33)/46*x^3*z*v+F(109)/58*x^2*y*z*v+F(1)/44*x*y^2*z*v+F(8)/65*y^3*z*v+5700*x^2*z^2*v+F(16)/69*x*y*z^2*v+F(18)/37*y^2*z^2*v+3905*x*z^3*v+F(84)/95*y*z^3*v+F(83)/65*z^4*v-700*x^3*u*v-F(8)/7*x^2*y*u*v+3184*x*y^2*u*v-8373*y^3*u*v+F(106)/115*x^2*z*u*v+F(68)/125*x*y*z*u*v-2829*y^2*z*u*v-F(38)/41*x*z^2*u*v+F(101)/19*y*z^2*u*v+F(66)/97*z^3*u*v-F(47)/7*x^2*u^2*v+F(21)/97*x*y*u^2*v+F(9)/74*y^2*u^2*v+F(57)/55*x*z*u^2*v-F(107)/89*y*z*u^2*v-14157*z^2*u^2*v-1693*x*u^3*v+F(9)/10*y*u^3*v-F(53)/114*z*u^3*v-10049*u^4*v+9871*x^3*v^2+14318*x^2*y*v^2-8447*x*y^2*v^2+10730*y^3*v^2-F(98)/87*x^2*z*v^2-3822*x*y*z*v^2+F(38)/39*y^2*z*v^2+F(111)/59*x*z^2*v^2-4603*y*z^2*v^2+F(35)/69*z^3*v^2-161*x^2*u*v^2-F(73)/48*x*y*u*v^2-F(58)/79*y^2*u*v^2+F(71)/63*x*z*u*v^2+1832*y*z*u*v^2+7891*z^2*u*v^2-13900*x*u^2*v^2+F(113)/31*y*u^2*v^2-7626*z*u^2*v^2-1223*u^3*v^2-F(33)/94*x^2*v^3-F(49)/20*x*y*v^3+F(20)/113*y^2*v^3-F(47)/15*x*z*v^3+F(109)/32*y*z*v^3-F(49)/95*z^2*v^3+F(92)/17*x*u*v^3-F(16)/99*y*u*v^3-10817*z*u*v^3+3754*u^2*v^3-F(94)/9*x*v^4-F(67)/79*y*v^4+12659*z*v^4+F(101)/117*u*v^4+F(113)/17*v^5,-3756*x^3*y*z-F(24)/35*x^2*y^2*z-F(29)/112*x*y^3*z+7944*y^4*z-8968*x^3*z^2+F(62)/115*x^2*y*z^2-F(38)/101*x*y^2*z^2+F(28)/39*y^3*z^2+F(61)/49*x^2*z^3-F(88)/119*x*y*z^3-F(119)/87*y^2*z^3-F(7)/123*x*z^4-4935*y*z^4-1239*z^5+8*x^3*y*u-F(68)/7*x^2*y^2*u+11910*x*y^3*u+5043*y^4*u+7293*x^3*z*u-F(121)/47*x^2*y*z*u+F(8)/77*x*y^2*z*u-F(53)/109*y^3*z*u+15439*x^2*z^2*u-6359*x*y*z^2*u-F(81)/109*y^2*z^2*u-3423*x*z^3*u+12246*y*z^3*u-1382*z^4*u-2355*x^3*u^2+12015*x^2*y*u^2-F(79)/38*x*y^2*u^2-14798*y^3*u^2+623*x^2*z*u^2+F(47)/113*x*y*z*u^2+11457*y^2*z*u^2+5127*x*z^2*u^2-F(9)/16*y*z^2*u^2-F(70)/41*z^3*u^2+F(107)/112*x^2*u^3-F(73)/49*x*y*u^3-F(38)/15*y^2*u^3+8748*x*z*u^3-5150*y*z*u^3-F(94)/67*z^2*u^3+10449*x*u^4-993*y*u^4-F(53)/6*z*u^4+F(46)/79*u^5+F(59)/46*x^3*y*v+F(7)/44*x^2*y^2*v+11979*x*y^3*v+F(78)/61*y^4*v+F(69)/109*x^3*z*v-424*x^2*y*z*v+3267*x*y^2*z*v-F(82)/17*y^3*z*v+F(15)/92*x^2*z^2*v-F(106)/121*x*y*z^2*v-F(32)/37*y^2*z^2*v+15746*x*z^3*v+F(97)/30*y*z^3*v-F(7)/29*z^4*v-9484*x^3*u*v+F(64)/45*x^2*y*u*v-75*x*y^2*u*v+14513*y^3*u*v-F(11)/103*x^2*z*u*v-F(82)/3*x*y*z*u*v+8740*y^2*z*u*v-F(67)/101*x*z^2*u*v+584*y*z^2*u*v+F(15)/121*z^3*u*v+11669*x^2*u^2*v+F(51)/112*x*y*u^2*v+F(124)/101*y^2*u^2*v+F(106)/17*x*z*u^2*v+3108*y*z*u^2*v-F(67)/118*z^2*u^2*v+6866*x*u^3*v+5705*y*u^3*v-F(68)/77*z*u^3*v-F(89)/98*u^4*v+14035*x^3*v^2-1896*x^2*y*v^2+F(107)/99*x*y^2*v^2+F(46)/49*y^3*v^2+F(109)/29*x^2*z*v^2-14831*x*y*z*v^2-13871*y^2*z*v^2+F(61)/62*x*z^2*v^2-F(35)/9*y*z^2*v^2+F(61)/40*z^3*v^2+F(101)/115*x^2*u*v^2-14723*x*y*u*v^2+13811*y^2*u*v^2+F(66)/13*x*z*u*v^2-9031*y*z*u*v^2-F(118)/77*z^2*u*v^2-F(86)/121*x*u^2*v^2-10616*y*u^2*v^2-F(60)/107*z*u^2*v^2+F(123)/85*u^3*v^2-F(92)/47*x^2*v^3+F(23)/98*x*y*v^3-5986*y^2*v^3-8636*x*z*v^3+F(63)/89*y*z*v^3+1899*z^2*v^3+F(117)/19*x*u*v^3-F(65)/41*y*u*v^3+2817*z*u*v^3-1578*u^2*v^3-F(35)/82*x*v^4+2155*y*v^4+7844*z*v^4-F(40)/67*u*v^4-F(16)/111*v^5,-4394*x^3*y^2-8801*x^2*y^3+12722*x*y^4+15341*y^5+F(85)/73*x^3*y*z+3786*x^2*y^2*z-156*x*y^3*z-11841*y^4*z-F(93)/47*x^3*z^2+F(79)/20*x^2*y*z^2+F(79)/99*x*y^2*z^2+10343*y^3*z^2-631*x^2*z^3+7178*x*y*z^3-4119*y^2*z^3+15660*x*z^4+14318*y*z^4+F(1)/69*z^5-F(9)/85*x^3*y*u-F(59)/33*x^2*y^2*u-4250*x*y^3*u-9233*y^4*u-F(103)/75*x^3*z*u-F(72)/53*x^2*y*z*u+F(50)/7*x*y^2*z*u+6476*y^3*z*u-F(37)/114*x^2*z^2*u-F(5)/7*x*y*z^2*u-F(13)/93*y^2*z^2*u+7262*x*z^3*u+F(40)/93*y*z^3*u-5837*z^4*u-F(13)/116*x^3*u^2+12622*x^2*y*u^2+2615*x*y^2*u^2-235*y^3*u^2+F(99)/115*x^2*z*u^2+F(101)/9*x*y*z*u^2+F(83)/28*y^2*z*u^2+14490*x*z^2*u^2+F(119)/96*y*z^2*u^2-11229*z^3*u^2+F(41)/86*x^2*u^3+14636*x*y*u^3-F(88)/45*y^2*u^3-9311*x*z*u^3-8967*y*z*u^3-F(71)/105*z^2*u^3+F(56)/53*x*u^4+14093*y*u^4+115*z*u^4+F(118)/67*u^5-F(95)/54*x^3*y*v-11922*x^2*y^2*v-F(101)/117*x*y^3*v-15245*y^4*v-3849*x^3*z*v+100*x^2*y*z*v+F(101)/64*x*y^2*z*v+F(82)/5*y^3*z*v-F(53)/52*x^2*z^2*v-F(26)/67*x*y*z^2*v+F(17)/72*y^2*z^2*v+F(33)/41*x*z^3*v-F(91)/82*y*z^3*v-F(29)/12*z^4*v+F(73)/126*x^3*u*v+F(109)/73*x^2*y*u*v-F(28)/55*x*y^2*u*v-F(51)/29*y^3*u*v+F(17)/35*x^2*z*u*v+F(108)/67*x*y*z*u*v+F(81)/2*y^2*z*u*v+F(95)/101*x*z^2*u*v-F(57)/8*y*z^2*u*v-7142*z^3*u*v-8230*x^2*u^2*v-F(91)/107*x*y*u^2*v-F(51)/11*y^2*u^2*v+F(67)/125*x*z*u^2*v-F(59)/17*y*z*u^2*v+F(23)/121*z^2*u^2*v+14520*x*u^3*v+2632*y*u^3*v+10602*z*u^3*v+F(19)/75*u^4*v-F(98)/115*x^3*v^2-13270*x^2*y*v^2+F(23)/28*x*y^2*v^2-F(97)/24*y^3*v^2+F(18)/37*x^2*z*v^2+2394*x*y*z*v^2+9634*y^2*z*v^2-12683*x*z^2*v^2-4723*y*z^2*v^2+F(93)/11*z^3*v^2+F(76)/15*x^2*u*v^2-7066*x*y*u*v^2-3496*y^2*u*v^2-F(31)/33*x*z*u*v^2+F(23)/114*y*z*u*v^2+F(75)/4*z^2*u*v^2+6637*x*u^2*v^2+F(5)/31*y*u^2*v^2-220*z*u^2*v^2-15145*u^3*v^2-F(16)/57*x^2*v^3+994*x*y*v^3+F(13)/111*y^2*v^3-F(35)/111*x*z*v^3-1460*y*z*v^3-13818*z^2*v^3-12748*x*u*v^3+F(6)/19*y*u*v^3+3481*z*u*v^3+4768*u^2*v^3+F(31)/83*x*v^4-F(21)/34*y*v^4-F(117)/95*z*v^4+7890*u*v^4-5747*v^5,4394*x^4*y+F(25)/118*x^3*y^2+F(27)/122*x^2*y^3-F(1)/20*x*y^4-F(21)/58*y^5+7367*x^4*z+11281*x^3*y*z+2518*x^2*y^2*z-F(55)/92*x*y^3*z+F(7)/17*y^4*z-6716*x^3*z^2+F(116)/11*x^2*y*z^2+F(67)/35*x*y^2*z^2+14506*y^3*z^2-F(70)/17*x^2*z^3-F(92)/125*x*y*z^3-11235*y^2*z^3+F(28)/79*x*z^4+6770*y*z^4+13311*z^5-12323*x^4*u-36*x^3*y*u-F(74)/35*x^2*y^2*u+F(28)/11*x*y^3*u-14140*y^4*u+F(29)/76*x^3*z*u+10357*x^2*y*z*u-13657*x*y^2*z*u-F(65)/9*y^3*z*u+F(8)/49*x^2*z^2*u-186*x*y*z^2*u+7854*y^2*z^2*u+F(20)/111*x*z^3*u+F(53)/20*y*z^3*u-4420*z^4*u-49*x^3*u^2+F(106)/73*x^2*y*u^2-F(57)/112*x*y^2*u^2-1767*y^3*u^2-F(67)/36*x^2*z*u^2+F(97)/31*x*y*z*u^2-F(4)/15*y^2*z*u^2-F(27)/119*x*z^2*u^2-6837*y*z^2*u^2+F(81)/88*z^3*u^2-803*x^2*u^3-14237*x*y*u^3+F(116)/45*y^2*u^3+F(74)/53*x*z*u^3-F(44)/23*y*z*u^3+7957*z^2*u^3+2551*x*u^4-5262*y*u^4-3014*z*u^4+F(115)/98*u^5+F(46)/39*x^4*v+F(71)/2*x^3*y*v+13841*x^2*y^2*v-F(12)/7*x*y^3*v+7491*y^4*v+F(19)/67*x^3*z*v-F(21)/82*x^2*y*z*v+F(29)/92*x*y^2*z*v+F(124)/123*y^3*z*v+4436*x^2*z^2*v+4318*x*y*z^2*v-F(14)/111*y^2*z^2*v+F(105)/118*x*z^3*v+12883*y*z^3*v-F(105)/22*z^4*v-F(124)/109*x^3*u*v-F(107)/103*x^2*y*u*v+F(7)/4*x*y^2*u*v-F(114)/121*y^3*u*v-F(74)/69*x^2*z*u*v-5025*x*y*z*u*v-F(114)/97*y^2*z*u*v-5872*x*z^2*u*v+F(24)/95*y*z^2*u*v-F(49)/88*z^3*u*v-F(21)/29*x^2*u^2*v+F(121)/52*x*y*u^2*v+721*y^2*u^2*v+F(119)/67*x*z*u^2*v+F(85)/48*y*z*u^2*v+F(50)/67*z^2*u^2*v+F(24)/53*x*u^3*v+F(32)/115*y*u^3*v-2809*z*u^3*v+2966*u^4*v+5211*x^3*v^2-190*x^2*y*v^2+2551*x*y^2*v^2+2895*y^3*v^2+F(124)/63*x^2*z*v^2-6160*x*y*z*v^2+F(71)/122*y^2*z*v^2+8635*x*z^2*v^2-F(9)/28*y*z^2*v^2+193*z^3*v^2+F(9)/119*x^2*u*v^2-F(110)/101*x*y*u*v^2-11713*y^2*u*v^2-6363*x*z*u*v^2+F(61)/93*y*z*u*v^2-10139*z^2*u*v^2+F(89)/96*x*u^2*v^2+F(94)/103*y*u^2*v^2+F(61)/93*z*u^2*v^2-F(61)/78*u^3*v^2-F(67)/22*x^2*v^3+15674*x*y*v^3-F(121)/5*y^2*v^3+F(121)/76*x*z*v^3+15523*y*z*v^3-F(93)/88*z^2*v^3+8667*x*u*v^3-11344*y*u*v^3-F(97)/108*z*u*v^3-14490*u^2*v^3+10228*x*v^4-F(18)/107*y*v^4-F(54)/37*z*v^4-2193*u*v^4+14370*v^5,-F(103)/79*x^3*y*z-F(16)/41*x^2*y^2*z+F(114)/83*x*y^3*z-F(104)/85*y^4*z+F(14)/37*x^3*z^2-3023*x^2*y*z^2+723*x*y^2*z^2-F(7)/55*y^3*z^2-F(82)/85*x^2*z^3-F(33)/83*x*y*z^3+F(109)/119*y^2*z^3-13119*x*z^4-4627*y*z^4-F(35)/37*z^5-11662*x^3*y*u+F(121)/57*x^2*y^2*u-7212*x*y^3*u+F(59)/83*y^4*u+2149*x^3*z*u-F(30)/13*x^2*y*z*u+F(125)/119*x*y^2*z*u+F(79)/23*y^3*z*u-F(3)/88*x^2*z^2*u+7168*x*y*z^2*u-10717*y^2*z^2*u+F(112)/33*x*z^3*u-10509*y*z^3*u+F(121)/56*z^4*u+F(125)/88*x^3*u^2-F(115)/121*x^2*y*u^2+F(49)/4*x*y^2*u^2-F(27)/88*y^3*u^2-2867*x^2*z*u^2-F(43)/85*x*y*z*u^2-4734*y^2*z*u^2-F(37)/114*x*z^2*u^2-F(2)/43*y*z^2*u^2+F(106)/95*z^3*u^2-F(108)/35*x^2*u^3+4531*x*y*u^3-F(19)/44*y^2*u^3-F(73)/27*x*z*u^3+F(59)/117*y*z*u^3+F(25)/9*z^2*u^3-F(65)/89*x*u^4+F(81)/20*y*u^4+5365*z*u^4+1733*u^5-2363*x^3*y*v-10326*x^2*y^2*v-F(5)/24*x*y^3*v-F(17)/106*y^4*v-14412*x^3*z*v+6361*x^2*y*z*v+10187*x*y^2*z*v-F(76)/101*y^3*z*v+4991*x^2*z^2*v+14252*x*y*z^2*v-F(107)/18*y^2*z^2*v+4857*x*z^3*v-13162*y*z^3*v+1088*z^4*v-F(3)/64*x^3*u*v-F(14)/113*x^2*y*u*v+F(96)/41*x*y^2*u*v-F(76)/103*y^3*u*v+8924*x^2*z*u*v-F(11)/30*x*y*z*u*v-4493*y^2*z*u*v+3070*x*z^2*u*v+F(10)/41*y*z^2*u*v-F(35)/51*z^3*u*v+4318*x^2*u^2*v+15309*x*y*u^2*v+F(107)/101*y^2*u^2*v-1748*x*z*u^2*v+11082*y*z*u^2*v+F(29)/90*z^2*u^2*v-F(19)/8*x*u^3*v+F(107)/64*y*u^3*v+3039*z*u^3*v-F(21)/59*u^4*v+F(14)/87*x^3*v^2+F(16)/59*x^2*y*v^2+F(77)/9*x*y^2*v^2+3679*y^3*v^2+F(53)/113*x^2*z*v^2+F(97)/80*x*y*z*v^2-899*y^2*z*v^2+10016*x*z^2*v^2-10049*y*z^2*v^2+F(29)/10*z^3*v^2+F(43)/7*x^2*u*v^2-5919*x*y*u*v^2-F(7)/34*y^2*u*v^2+F(42)/85*x*z*u*v^2+F(102)/23*y*z*u*v^2-8300*z^2*u*v^2+F(125)/2*x*u^2*v^2-1700*y*u^2*v^2-F(107)/21*z*u^2*v^2+12942*u^3*v^2-3054*x^2*v^3-F(87)/53*x*y*v^3+4745*y^2*v^3+F(52)/83*x*z*v^3-11747*y*z*v^3-F(119)/64*z^2*v^3-5987*x*u*v^3+2016*y*u*v^3-2889*z*u*v^3+4535*u^2*v^3-1588*x*v^4-F(27)/5*y*v^4+8541*z*v^4-15581*u*v^4-F(3)/119*v^5,-x^3*y^2-7492*x^2*y^3+F(118)/35*x*y^4-F(43)/5*y^5-2479*x^3*y*z-8522*x^2*y^2*z-F(105)/26*x*y^3*z-F(103)/18*y^4*z+5868*x^3*z^2-10040*x^2*y*z^2-F(50)/81*x*y^2*z^2+3378*y^3*z^2-F(71)/44*x^2*z^3-12323*x*y*z^3-F(25)/44*y^2*z^3+12979*x*z^4-15146*y*z^4+5962*z^5+8852*x^3*y*u-F(39)/77*x^2*y^2*u-9554*x*y^3*u-14590*y^4*u+F(61)/84*x^3*z*u-2295*x^2*y*z*u-5969*x*y^2*z*u+14241*y^3*z*u-F(29)/74*x^2*z^2*u+F(107)/17*x*y*z^2*u+107*y^2*z^2*u-F(14)/99*x*z^3*u+F(109)/93*y*z^3*u-F(73)/41*z^4*u-9170*x^3*u^2+8877*x^2*y*u^2-F(109)/118*x*y^2*u^2+F(65)/36*y^3*u^2+F(122)/93*x^2*z*u^2-15020*x*y*z*u^2+F(33)/50*y^2*z*u^2-F(11)/46*x*z^2*u^2-F(91)/116*y*z^2*u^2-F(47)/10*z^3*u^2-F(34)/57*x^2*u^3-F(90)/79*x*y*u^3+12760*y^2*u^3+F(81)/71*x*z*u^3+1436*y*z*u^3+14839*z^2*u^3+6598*x*u^4-12418*y*u^4-F(47)/59*z*u^4-1357*u^5+F(6)/91*x^3*y*v-F(46)/21*x^2*y^2*v+F(100)/53*x*y^3*v-F(69)/125*y^4*v+98*x^3*z*v-F(63)/23*x^2*y*z*v+F(96)/121*x*y^2*z*v+F(12)/11*y^3*z*v-F(37)/120*x^2*z^2*v-F(37)/106*x*y*z^2*v-F(25)/122*y^2*z^2*v-13657*x*z^3*v+F(79)/47*y*z^3*v+15519*z^4*v+F(45)/46*x^3*u*v+F(73)/100*x^2*y*u*v-5361*x*y^2*u*v-F(39)/101*y^3*u*v+1161*x^2*z*u*v-F(105)/22*x*y*z*u*v+F(79)/34*y^2*z*u*v+F(96)/31*x*z^2*u*v-F(20)/51*y*z^2*u*v-2683*z^3*u*v-F(122)/125*x^2*u^2*v-F(66)/109*x*y*u^2*v-6357*y^2*u^2*v-3021*x*z*u^2*v+F(86)/33*y*z*u^2*v-F(105)/8*z^2*u^2*v+F(95)/22*x*u^3*v-9046*y*u^3*v+F(69)/83*z*u^3*v-3166*u^4*v+F(91)/115*x^3*v^2+9661*x^2*y*v^2-8757*x*y^2*v^2+F(79)/23*y^3*v^2+F(125)/107*x^2*z*v^2+6133*x*y*z*v^2+8975*y^2*z*v^2-F(38)/103*x*z^2*v^2-F(120)/43*y*z^2*v^2+F(59)/124*z^3*v^2-9252*x^2*u*v^2+F(27)/64*x*y*u*v^2+F(99)/80*y^2*u*v^2+F(125)/28*x*z*u*v^2-F(60)/109*y*z*u*v^2-F(73)/107*z^2*u*v^2-15162*x*u^2*v^2+F(91)/71*y*u^2*v^2-F(89)/122*z*u^2*v^2-F(1)/9*u^3*v^2+F(40)/117*x^2*v^3-F(71)/41*x*y*v^3+10502*y^2*v^3-5618*x*z*v^3-995*y*z*v^3-F(117)/112*z^2*v^3-15185*x*u*v^3-4470*y*u*v^3-F(13)/4*z*u*v^3-14631*u^2*v^3-5804*x*v^4-15527*y*v^4-F(47)/87*z*v^4+5084*u*v^4-F(64)/11*v^5,x^4*y+F(46)/47*x^3*y^2+F(13)/47*x^2*y^3-13553*x*y^4-F(5)/59*y^5-F(15)/76*x^4*z+370*x^3*y*z+F(15)/56*x^2*y^2*z-8199*x*y^3*z+F(17)/37*y^4*z+F(32)/47*x^3*z^2-F(26)/29*x^2*y*z^2-F(1)/109*x*y^2*z^2+F(68)/77*y^3*z^2-F(88)/123*x^2*z^3-1397*x*y*z^3-90*y^2*z^3+F(117)/80*x*z^4+14919*y*z^4+11248*z^5+1742*x^4*u-6060*x^3*y*u+F(97)/38*x^2*y^2*u+F(54)/53*x*y^3*u-F(26)/45*y^4*u-315*x^3*z*u+F(9)/76*x^2*y*z*u+F(113)/14*x*y^2*z*u-F(28)/79*y^3*z*u-F(109)/14*x^2*z^2*u+F(53)/54*x*y*z^2*u-213*y^2*z^2*u-F(18)/11*x*z^3*u-12129*y*z^3*u+6321*z^4*u-13361*x^3*u^2+F(109)/81*x^2*y*u^2+F(37)/84*x*y^2*u^2-F(19)/69*y^3*u^2+F(82)/31*x^2*z*u^2-F(20)/41*x*y*z*u^2-13135*y^2*z*u^2-F(52)/99*x*z^2*u^2-12942*y*z^2*u^2-8237*z^3*u^2-12377*x^2*u^3-F(41)/43*x*y*u^3+13143*y^2*u^3-6701*x*z*u^3-F(98)/53*y*z*u^3+F(97)/43*z^2*u^3+1570*x*u^4-F(41)/5*y*u^4-F(7)/86*z*u^4+306*u^5-7920*x^4*v+F(76)/53*x^3*y*v-F(91)/73*x^2*y^2*v-F(49)/64*x*y^3*v+12976*y^4*v-1787*x^3*z*v-F(95)/123*x^2*y*z*v-10033*x*y^2*z*v-F(83)/100*y^3*z*v-F(17)/112*x^2*z^2*v+1060*x*y*z^2*v-F(121)/54*y^2*z^2*v+F(25)/77*x*z^3*v+F(23)/73*y*z^3*v-3576*z^4*v+F(41)/98*x^3*u*v+15265*x^2*y*u*v+9014*x*y^2*u*v-F(86)/33*y^3*u*v-F(83)/51*x^2*z*u*v-15747*x*y*z*u*v+F(11)/16*y^2*z*u*v-12110*x*z^2*u*v-15554*y*z^2*u*v-7654*z^3*u*v+F(68)/53*x^2*u^2*v+1673*x*y*u^2*v-F(123)/64*y^2*u^2*v-F(3)/19*x*z*u^2*v+F(29)/34*y*z*u^2*v-5457*z^2*u^2*v+F(51)/58*x*u^3*v-4242*y*u^3*v-F(70)/67*z*u^3*v+7159*u^4*v+8620*x^3*v^2-9381*x^2*y*v^2+F(52)/79*x*y^2*v^2-F(73)/113*y^3*v^2+F(83)/22*x^2*z*v^2-1445*x*y*z*v^2+6524*y^2*z*v^2-122*x*z^2*v^2-F(107)/85*y*z^2*v^2-1240*z^3*v^2+1710*x^2*u*v^2-F(101)/59*x*y*u*v^2-2950*y^2*u*v^2-12343*x*z*u*v^2-F(79)/27*y*z*u*v^2-F(5)/26*z^2*u*v^2+F(89)/68*x*u^2*v^2-14985*y*u^2*v^2+8382*z*u^2*v^2-5109*u^3*v^2-12322*x^2*v^3-F(97)/106*x*y*v^3-F(86)/23*y^2*v^3+F(115)/11*x*z*v^3-F(45)/2*y*z*v^3-F(79)/110*z^2*v^3+7472*x*u*v^3-F(17)/71*y*u*v^3+1659*z*u*v^3-4468*u^2*v^3-F(85)/47*x*v^4+F(11)/73*y*v^4-1700*z*v^4-F(32)/51*u*v^4+7855*v^5,F(62)/49*x^4-F(125)/36*x^3*y+2*x^2*y^2+F(56)/39*x*y^3+7307*y^4-F(42)/43*x^3*z+F(83)/42*x^2*y*z-11648*x*y^2*z-F(43)/106*y^3*z-F(49)/109*x^2*z^2-F(95)/28*x*y*z^2-194*y^2*z^2+F(71)/95*x*z^3-F(83)/59*y*z^3-F(66)/95*z^4-F(1)/51*x^3*u-F(82)/21*x^2*y*u+F(121)/103*x*y^2*u-11106*y^3*u-F(55)/91*x^2*z*u+F(55)/119*x*y*z*u-F(123)/79*y^2*z*u-F(20)/11*x*z^2*u+6943*y*z^2*u+69*z^3*u+5908*x^2*u^2-F(56)/89*x*y*u^2-13119*y^2*u^2+F(32)/115*x*z*u^2+11412*y*z*u^2+F(63)/50*z^2*u^2+F(73)/85*x*u^3-12527*y*u^3+1991*z*u^3+F(26)/33*u^4+F(79)/91*x^3*v-F(10)/27*x^2*y*v+F(20)/117*x*y^2*v-F(49)/102*y^3*v+F(98)/33*x^2*z*v+F(115)/76*x*y*z*v+12412*y^2*z*v-F(47)/48*x*z^2*v+F(121)/113*y*z^2*v+F(105)/26*z^3*v-10954*x^2*u*v+10730*x*y*u*v+15721*y^2*u*v-F(101)/63*x*z*u*v-F(32)/19*y*z*u*v+F(67)/71*z^2*u*v+F(77)/45*x*u^2*v+F(27)/98*y*u^2*v-670*z*u^2*v-13396*u^3*v-F(55)/119*x^2*v^2-7890*x*y*v^2-F(101)/7*y^2*v^2-9563*x*z*v^2+F(3)/95*y*z*v^2+4857*z^2*v^2+9397*x*u*v^2+F(3)/116*y*u*v^2-12257*z*u*v^2-F(79)/49*u^2*v^2+11697*x*v^3+F(26)/19*y*v^3+4885*z*v^3-F(53)/22*u*v^3-8355*v^4]))
end


@doc raw"""
    elliptic_d11_pi12()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `11` and sectional genus `12`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d11_pi12()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-7669*x^4*y+F(20)/81*x^3*y^2+F(115)/48*x^2*y^3+10579*x*y^4-F(74)/3*y^5-11693*x^4*z+9169*x^3*y*z-12238*x^2*y^2*z+F(106)/31*x*y^3*z-F(15)/34*y^4*z-6909*x^3*z^2+4700*x^2*y*z^2-F(126)/107*x*y^2*z^2-F(102)/125*y^3*z^2+F(99)/59*x^2*z^3-F(27)/77*x*y*z^3-F(2)/23*y^2*z^3+F(11)/86*x*z^4-1770*y*z^4-7864*z^5+F(60)/67*x^4*u-F(92)/73*x^3*y*u+137*x^2*y^2*u-F(71)/17*x*y^3*u-F(46)/93*y^4*u-F(108)/11*x^3*z*u+F(59)/115*x^2*y*z*u+15708*x*y^2*z*u+5578*y^3*z*u+15068*x^2*z^2*u+13741*x*y*z^2*u-F(41)/95*y^2*z^2*u+F(65)/27*x*z^3*u-1224*y*z^3*u-15409*z^4*u-9417*x^3*u^2-3101*x^2*y*u^2-F(3)/85*x*y^2*u^2+13243*y^3*u^2+F(81)/59*x^2*z*u^2-7697*x*y*z*u^2+F(50)/123*y^2*z*u^2+F(3)/100*x*z^2*u^2-4898*y*z^2*u^2-10206*z^3*u^2+F(31)/66*x^2*u^3-F(73)/121*x*y*u^3-15533*y^2*u^3+F(35)/31*x*z*u^3-F(89)/94*y*z*u^3-F(65)/7*z^2*u^3+F(74)/9*x*u^4-F(61)/8*y*u^4+F(94)/19*z*u^4-F(122)/63*x^4*v-3630*x^3*y*v+7084*x^2*y^2*v-4537*x*y^3*v-F(7)/51*y^4*v-10552*x^3*z*v-8389*x^2*y*z*v-5655*x*y^2*z*v+F(125)/11*y^3*z*v-F(66)/83*x^2*z^2*v+6828*x*y*z^2*v+F(35)/89*y^2*z^2*v+F(73)/11*x*z^3*v+F(82)/109*y*z^3*v-F(118)/27*z^4*v+F(23)/103*x^3*u*v+F(40)/83*x^2*y*u*v+F(31)/19*x*y^2*u*v-F(55)/103*y^3*u*v-6770*x^2*z*u*v-F(1)/32*x*y*z*u*v+F(41)/110*y^2*z*u*v-13184*x*z^2*u*v-F(118)/101*y*z^2*u*v+F(73)/83*z^3*u*v-15753*x^2*u^2*v+F(71)/42*x*y*u^2*v-F(113)/22*y^2*u^2*v-6072*x*z*u^2*v+F(65)/114*y*z*u^2*v-F(47)/15*z^2*u^2*v+15923*x*u^3*v+202*y*u^3*v+10621*z*u^3*v+F(119)/26*x^3*v^2-F(42)/115*x^2*y*v^2+F(101)/75*x*y^2*v^2-F(23)/121*y^3*v^2-F(5)/34*x^2*z*v^2+F(45)/32*x*y*z*v^2+F(92)/9*y^2*z*v^2+15397*x*z^2*v^2-5165*y*z^2*v^2-10611*z^3*v^2-F(87)/2*x^2*u*v^2-3057*x*y*u*v^2-F(58)/23*y^2*u*v^2-1606*x*z*u*v^2+10707*y*z*u*v^2+F(56)/55*z^2*u*v^2+47*x*u^2*v^2+963*y*u^2*v^2-F(5)/83*z*u^2*v^2-8406*x^2*v^3+10169*x*y*v^3-8071*y^2*v^3+F(101)/119*x*z*v^3-F(9)/29*y*z*v^3+F(125)/8*z^2*v^3-F(26)/47*x*u*v^3+3975*y*u*v^3+F(8)/85*z*u*v^3-14916*x*v^4-11692*y*v^4-F(29)/20*z*v^4,-F(38)/77*x^4*y-F(95)/3*x^3*y^2-F(99)/56*x^2*y^3+F(25)/82*x*y^4-7423*x^4*z+F(41)/53*x^3*y*z-F(112)/113*x^2*y^2*z+12120*x*y^3*z-5248*y^4*z-9172*x^3*z^2-F(74)/11*x^2*y*z^2-F(109)/120*x*y^2*z^2+F(41)/36*y^3*z^2-13653*x^2*z^3+F(95)/28*x*y*z^3+13367*y^2*z^3+9882*x*z^4+11494*y*z^4-584*z^5-9062*x^4*u-6086*x^3*y*u-F(2)/83*x^2*y^2*u+F(17)/59*x*y^3*u+F(74)/3*y^4*u+13568*x^3*z*u-F(105)/116*x^2*y*z*u+F(23)/51*x*y^2*z*u-13011*y^3*z*u+F(2)/95*x^2*z^2*u-F(8)/25*x*y*z^2*u-2701*y^2*z^2*u-1658*x*z^3*u-14959*y*z^3*u-11946*z^4*u+F(55)/106*x^3*u^2+F(69)/103*x^2*y*u^2+F(6)/47*x*y^2*u^2+F(46)/93*y^3*u^2+5835*x^2*z*u^2+F(95)/104*x*y*z*u^2-F(69)/34*y^2*z*u^2+5762*x*z^2*u^2-5589*y*z^2*u^2+F(83)/38*z^3*u^2-F(7)/55*x^2*u^3-F(45)/122*x*y*u^3-13243*y^2*u^3-F(85)/32*x*z*u^3-F(22)/79*y*z*u^3-5204*z^2*u^3+F(95)/22*x*u^4+15533*y*u^4-3267*z*u^4+F(61)/8*u^5-4216*x^4*v+F(89)/73*x^3*y*v-F(61)/114*x^2*y^2*v+F(18)/109*x*y^3*v-6290*x^3*z*v-F(123)/34*x^2*y*z*v-F(109)/85*x*y^2*z*v-14348*y^3*z*v-12290*x^2*z^2*v+F(16)/69*x*y*z^2*v+1735*y^2*z^2*v+1695*x*z^3*v-F(121)/116*y*z^3*v-F(116)/73*z^4*v+F(125)/24*x^3*u*v-F(115)/39*x^2*y*u*v+3872*x*y^2*u*v+F(7)/51*y^3*u*v+F(13)/110*x^2*z*u*v+F(25)/29*x*y*z*u*v+6187*y^2*z*u*v+F(87)/100*x*z^2*u*v-11304*y*z^2*u*v-F(88)/29*z^3*u*v-F(113)/126*x^2*u^2*v+F(124)/49*x*y*u^2*v+F(55)/103*y^2*u^2*v-3150*x*z*u^2*v+9311*y*z*u^2*v-F(31)/88*z^2*u^2*v-F(20)/11*x*u^3*v+F(113)/22*y*u^3*v+4853*z*u^3*v-202*u^4*v+F(108)/103*x^3*v^2-F(43)/46*x^2*y*v^2+F(114)/71*x*y^2*v^2+15243*x^2*z*v^2+7950*x*y*z*v^2+F(83)/67*y^2*z*v^2-F(13)/75*x*z^2*v^2-F(1)/88*y*z^2*v^2-11838*z^3*v^2+F(61)/65*x^2*u*v^2-11404*x*y*u*v^2+F(23)/121*y^2*u*v^2+F(113)/68*x*z*u*v^2+F(85)/37*y*z*u*v^2+F(47)/59*z^2*u*v^2+55*x*u^2*v^2+F(58)/23*y*u^2*v^2+5060*z*u^2*v^2-963*u^3*v^2-13443*x^2*v^3-4841*x*y*v^3+F(59)/105*x*z*v^3-8163*y*z*v^3+F(27)/55*z^2*v^3+F(63)/4*x*u*v^3+8071*y*u*v^3+F(3)/98*z*u*v^3-3975*u^2*v^3-1217*x*v^4+10500*z*v^4+11692*u*v^4,F(38)/77*x^3*y^2+F(95)/3*x^2*y^3+F(99)/56*x*y^4-F(25)/82*y^5+F(113)/24*x^3*y*z-F(104)/99*x^2*y^2*z-14298*x*y^3*z+4427*y^4*z-14157*x^3*z^2-6744*x^2*y*z^2+1497*x*y^2*z^2-15309*y^3*z^2+228*x^2*z^3+F(23)/57*x*y*z^3+14306*y^2*z^3-F(100)/63*x*z^4+F(58)/15*y*z^4+F(39)/67*z^5-15260*x^3*y*u+4111*x^2*y^2*u+12195*x*y^3*u-F(9)/92*y^4*u-10886*x^3*z*u+F(89)/98*x^2*y*z*u-F(107)/31*x*y^2*z*u-9526*y^3*z*u-F(39)/113*x^2*z^2*u-F(102)/43*x*y*z^2*u+3608*y^2*z^2*u+11248*x*z^3*u-F(19)/54*y*z^3*u-F(91)/80*z^4*u-F(60)/67*x^3*u^2-F(38)/77*x^2*y*u^2-F(59)/120*x*y^2*u^2-10366*y^3*u^2-F(11)/61*x^2*z*u^2-F(52)/101*x*y*z*u^2-7688*y^2*z*u^2+10709*x*z^2*u^2+F(126)/109*y*z^2*u^2+10725*z^3*u^2+9417*x^2*u^3-F(34)/33*x*y*u^3-2193*y^2*u^3-F(14)/111*x*z*u^3+12748*y*z*u^3+F(3)/55*z^2*u^3-F(31)/66*x*u^4-11769*y*u^4+F(47)/10*z*u^4-F(74)/9*u^5+4216*x^3*y*v-F(89)/73*x^2*y^2*v+F(61)/114*x*y^3*v-F(18)/109*y^4*v-F(79)/67*x^3*z*v-F(23)/31*x^2*y*z*v-F(105)/92*x*y^2*z*v-F(23)/109*y^3*z*v-6548*x^2*z^2*v+10591*x*y*z^2*v+2090*y^2*z^2*v-9787*x*z^3*v+2427*y*z^3*v-F(38)/73*z^4*v+F(122)/63*x^3*u*v-3040*x^2*y*u*v+13426*x*y^2*u*v-F(71)/48*y^3*u*v+F(74)/19*x^2*z*u*v-F(9)/47*x*y*z*u*v+F(91)/99*y^2*z*u*v+F(27)/31*x*z^2*u*v-1534*y*z^2*u*v+F(21)/94*z^3*u*v-F(23)/103*x^2*u^2*v-F(59)/20*x*y*u^2*v+F(18)/103*y^2*u^2*v+F(35)/96*x*z*u^2*v+F(3)/112*y*z*u^2*v+9882*z^2*u^2*v+15753*x*u^3*v-7963*y*u^3*v+5218*z*u^3*v-15923*u^4*v-F(108)/103*x^2*y*v^2+F(43)/46*x*y^2*v^2-F(114)/71*y^3*v^2+F(17)/44*x^2*z*v^2+F(68)/37*x*y*z*v^2+F(119)/73*y^2*z*v^2-15545*x*z^2*v^2-8277*y*z^2*v^2+F(9)/119*z^3*v^2-F(119)/26*x^2*u*v^2+F(64)/75*x*y*u*v^2-F(76)/53*y^2*u*v^2-F(39)/103*x*z*u*v^2-13165*y*z*u*v^2-12430*z^2*u*v^2+F(87)/2*x*u^2*v^2+F(91)/32*y*u^2*v^2+F(124)/23*z*u^2*v^2-47*u^3*v^2+13443*x*y*v^3+4841*y^2*v^3+F(41)/120*x*z*v^3+4891*y*z*v^3+F(33)/50*z^2*v^3+8406*x*u*v^3+F(49)/117*y*u*v^3+555*z*u*v^3+F(26)/47*u^2*v^3+1217*y*v^4-F(83)/124*z*v^4+14916*u*v^4,-15242*x^4*y+7085*x^3*y^2+993*x^2*y^3+F(2)/29*x*y^4+5248*y^5+14157*x^4*z+15916*x^3*y*z+1418*x^2*y^2*z-419*x*y^3*z-F(41)/36*y^4*z-228*x^3*z^2+F(117)/16*x^2*y*z^2+F(59)/101*x*y^2*z^2-13367*y^3*z^2+F(100)/63*x^2*z^3+F(48)/107*x*y*z^3-11494*y^2*z^3-F(39)/67*x*z^4+584*y*z^4-F(49)/17*x^4*u+F(2)/55*x^3*y*u-10541*x^2*y^2*u-F(2)/87*x*y^3*u-14275*y^4*u+13987*x^3*z*u+F(35)/78*x^2*y*z*u+F(18)/91*x*y^2*z*u-F(36)/59*y^3*z*u-F(31)/75*x^2*z^2*u-F(95)/98*x*y*z^2*u-F(43)/45*y^2*z^2*u+F(83)/73*x*z^3*u+F(39)/7*y*z^3*u+7864*z^4*u+F(64)/97*x^3*u^2+15384*x^2*y*u^2+F(12)/119*x*y^2*u^2-15926*y^3*u^2+6214*x^2*z*u^2-F(56)/51*x*y*z*u^2-3166*y^2*z*u^2-5988*x*z^2*u^2-6355*y*z^2*u^2+15409*z^3*u^2-436*x^2*u^3+7948*x*y*u^3+F(49)/3*y^2*u^3-8056*x*z*u^3-F(8)/19*y*z*u^3+10206*z^2*u^3-13731*x*u^4-1837*y*u^4+F(65)/7*z*u^4-F(94)/19*u^5+F(79)/67*x^4*v-F(73)/114*x^3*y*v-F(6)/89*x^2*y^2*v+10115*x*y^3*v+14348*y^4*v+6548*x^3*z*v+F(41)/113*x^2*y*z*v+15528*x*y^2*z*v-1735*y^3*z*v+9787*x^2*z^2*v-4122*x*y*z^2*v+F(121)/116*y^2*z^2*v+F(38)/73*x*z^3*v+F(116)/73*y*z^3*v+F(99)/4*x^3*u*v+13098*x^2*y*u*v+F(61)/89*x*y^2*u*v-F(57)/23*y^3*u*v+F(79)/106*x^2*z*u*v-F(16)/89*x*y*z*u*v+15617*y^2*z*u*v+F(7)/8*x*z^2*u*v+F(13)/19*y*z^2*u*v+F(118)/27*z^3*u*v+3104*x^2*u^2*v-F(44)/119*x*y*u^2*v-F(49)/10*y^2*u^2*v+3302*x*z*u^2*v-F(87)/28*y*z*u^2*v-F(73)/83*z^2*u^2*v+F(68)/75*x*u^3*v+F(33)/74*y*u^3*v+F(47)/15*z*u^3*v-10621*u^4*v-F(17)/44*x^3*v^2+F(25)/106*x^2*y*v^2-4884*x*y^2*v^2-F(83)/67*y^3*v^2+15545*x^2*z*v^2-F(102)/71*x*y*z*v^2+F(1)/88*y^2*z*v^2-F(9)/119*x*z^2*v^2+11838*y*z^2*v^2+F(83)/71*x^2*u*v^2-14889*x*y*u*v^2+2101*y^2*u*v^2+F(120)/97*x*z*u*v^2-F(1)/124*y*z*u*v^2+10611*z^2*u*v^2-F(125)/79*x*u^2*v^2-15767*y*u^2*v^2-F(56)/55*z*u^2*v^2+F(5)/83*u^3*v^2-F(41)/120*x^2*v^3-8243*x*y*v^3+8163*y^2*v^3-F(33)/50*x*z*v^3-F(27)/55*y*z*v^3-3513*x*u*v^3-2645*y*u*v^3-F(125)/8*z*u*v^3-F(8)/85*u^2*v^3+F(83)/124*x*v^4-10500*y*v^4+F(29)/20*u*v^4,-F(9)/23*x^4*y-2301*x^3*y^2+F(51)/83*x^2*y^3-2497*x*y^4-F(7)/29*x^4*z-11136*x^3*y*z-F(26)/53*x^2*y^2*z-F(86)/11*x*y^3*z+F(71)/14*x^3*z^2-F(113)/74*x^2*y*z^2+F(15)/92*x*y^2*z^2+F(37)/12*x^2*z^3+F(80)/39*x*y*z^3-13143*x*z^4-F(91)/60*x^4*u-648*x^3*y*u-F(21)/10*x^2*y^2*u-F(55)/79*x*y^3*u+73*y^4*u+F(67)/28*x^3*z*u-75*x^2*y*z*u-2070*x*y^2*z*u+F(25)/48*y^3*z*u+F(93)/65*x^2*z^2*u-10715*x*y*z^2*u-F(91)/6*y^2*z^2*u+F(25)/62*x*z^3*u+F(107)/81*y*z^3*u-13804*z^4*u-F(65)/19*x^3*u^2+F(53)/91*x^2*y*u^2-F(98)/31*x*y^2*u^2-F(105)/104*y^3*u^2+8335*x^2*z*u^2+F(61)/32*x*y*z*u^2-15094*y^2*z*u^2+F(57)/88*x*z^2*u^2+F(86)/89*y*z^2*u^2+F(111)/47*z^3*u^2+F(17)/74*x^2*u^3+4282*x*y*u^3+9896*y^2*u^3+F(122)/105*x*z*u^3-F(43)/32*y*z*u^3+14673*z^2*u^3+F(13)/50*x*u^4+F(112)/43*y*u^4-F(101)/53*z*u^4+F(77)/115*u^5+F(62)/111*x^4*v-F(55)/114*x^3*y*v-12342*x^2*y^2*v-15263*x*y^3*v+5248*y^4*v+45*x^3*z*v+F(23)/27*x^2*y*z*v+13375*x*y^2*z*v-F(41)/36*y^3*z*v+F(85)/86*x^2*z^2*v-F(112)/43*x*y*z^2*v-13367*y^2*z^2*v-F(108)/121*x*z^3*v-11494*y*z^3*v+584*z^4*v-F(53)/63*x^3*u*v-F(2)/19*x^2*y*u*v-F(7)/66*x*y^2*u*v-8299*y^3*u*v-F(43)/49*x^2*z*u*v-10771*x*y*z*u*v-F(83)/12*y^2*z*u*v+F(104)/37*x*z^2*u*v-F(82)/73*y*z^2*u*v+6749*z^3*u*v-F(53)/89*x^2*u^2*v+715*x*y*u^2*v+F(85)/66*y^2*u^2*v-3149*x*z*u^2*v-15772*y*z*u^2*v+3421*z^2*u^2*v+F(9)/74*x*u^3*v+11222*y*u^3*v+F(116)/101*z*u^3*v-F(39)/58*u^4*v+F(25)/83*x^3*v^2+1703*x^2*y*v^2+5533*x*y^2*v^2+14348*y^3*v^2+F(101)/105*x^2*z*v^2-7610*x*y*z*v^2-1735*y^2*z*v^2-8588*x*z^2*v^2+F(121)/116*y*z^2*v^2+F(116)/73*z^3*v^2-F(31)/90*x^2*u*v^2+8092*x*y*u*v^2-F(1)/19*y^2*u*v^2-F(23)/29*x*z*u*v^2+F(107)/93*y*z*u*v^2+F(7)/93*x*u^2*v^2+F(82)/55*y*u^2*v^2+F(70)/27*z*u^2*v^2-F(73)/120*u^3*v^2+4769*x^2*v^3+F(66)/59*x*y*v^3-F(83)/67*y^2*v^3+F(84)/79*x*z*v^3+F(1)/88*y*z*v^3+11838*z^2*v^3+F(106)/71*x*u*v^3-7158*y*u*v^3+8707*z*u*v^3+F(68)/125*u^2*v^3-5371*x*v^4+8163*y*v^4-F(27)/55*z*v^4-F(25)/83*u*v^4-10500*v^5,F(9)/23*x^3*y^2+2301*x^2*y^3-F(51)/83*x*y^4+2497*y^5+F(7)/29*x^3*y*z+11136*x^2*y^2*z+F(26)/53*x*y^3*z+F(86)/11*y^4*z-F(71)/14*x^2*y*z^2+F(113)/74*x*y^2*z^2-F(15)/92*y^3*z^2-F(37)/12*x*y*z^3-F(80)/39*y^2*z^3+13143*y*z^4+F(6)/89*x^3*y*u-F(116)/117*x^2*y^2*u+12021*x*y^3*u-2479*y^4*u-10428*x^3*z*u-F(73)/27*x^2*y*z*u+F(61)/27*x*y^2*z*u-242*y^3*z*u-10712*x^2*z^2*u-F(1)/80*x*y*z^2*u+F(67)/26*y^2*z^2*u-12042*x*z^3*u-F(123)/112*y*z^3*u-F(47)/34*z^4*u+F(113)/76*x^3*u^2+F(3)/110*x^2*y*u^2-11842*x*y^2*u^2+F(53)/11*y^3*u^2-2951*x^2*z*u^2+402*x*y*z*u^2+F(65)/83*y^2*z*u^2+14636*x*z^2*u^2+F(77)/25*y*z^2*u^2-F(111)/59*z^3*u^2+30*x^2*u^3-F(107)/11*x*y*u^3-F(71)/124*y^2*u^3-10082*x*z*u^3+F(12)/43*y*z*u^3+F(33)/106*z^2*u^3-F(126)/67*x*u^4-F(48)/5*y*u^4+F(25)/18*z*u^4-F(25)/22*u^5-F(18)/115*x^3*y*v-11155*x^2*y^2*v+F(65)/12*x*y^3*v-F(109)/25*y^4*v+14157*x^3*z*v+15871*x^2*y*z*v+2602*x*y^2*z*v+F(31)/109*y^3*z*v-228*x^2*z^2*v+F(3)/74*x*y*z^2*v+1609*y^2*z^2*v+F(100)/63*x*z^3*v+9292*y*z^3*v-F(39)/67*z^4*v+F(11)/10*x^3*u*v+F(74)/91*x^2*y*u*v-F(96)/55*x*y^2*u*v-F(102)/101*y^3*u*v+F(49)/13*x^2*z*u*v+F(71)/103*x*y*z*u*v+F(83)/24*y^2*z*u*v+F(38)/47*x*z^2*u*v+F(51)/89*y*z^2*u*v+F(83)/23*z^3*u*v+F(13)/64*x^2*u^2*v+6975*x*y*u^2*v+F(9)/5*y^2*u^2*v+13411*x*z*u^2*v-F(55)/72*y*z*u^2*v+F(123)/91*z^2*u^2*v-F(73)/11*x*u^3*v-141*y*u^3*v-F(55)/98*z*u^3*v+8830*u^4*v+F(79)/67*x^3*v^2+F(71)/79*x^2*y*v^2-12846*x*y^2*v^2+F(83)/7*y^3*v^2+6548*x^2*z*v^2+F(6)/89*x*y*z*v^2-8853*y^2*z*v^2+9787*x*z^2*v^2+F(92)/43*y*z^2*v^2+F(38)/73*z^3*v^2+F(18)/59*x^2*u*v^2-F(116)/85*x*y*u*v^2-15677*y^2*u*v^2-F(21)/34*x*z*u*v^2-F(19)/48*y*z*u*v^2-12559*z^2*u*v^2+2925*x*u^2*v^2-4769*y*u^2*v^2-F(104)/101*z*u^2*v^2+5112*u^3*v^2-F(17)/44*x^2*v^3+F(101)/67*x*y*v^3-7054*y^2*v^3+15545*x*z*v^3+F(9)/58*y*z*v^3-F(9)/119*z^2*v^3+3536*x*u*v^3-F(47)/56*y*u*v^3+11186*z*u*v^3+F(10)/33*u^2*v^3-F(41)/120*x*v^4-F(79)/78*y*v^4-F(33)/50*z*v^4+F(106)/113*u*v^4+F(83)/124*v^5,F(9)/23*x^3*y*z+2301*x^2*y^2*z-F(51)/83*x*y^3*z+2497*y^4*z+F(7)/29*x^3*z^2+11136*x^2*y*z^2+F(26)/53*x*y^2*z^2+F(86)/11*y^3*z^2-F(71)/14*x^2*z^3+F(113)/74*x*y*z^3-F(15)/92*y^2*z^3-F(37)/12*x*z^4-F(80)/39*y*z^4+13143*z^5-F(5)/12*x^3*y*u-F(17)/55*x^2*y^2*u+F(85)/24*x*y^3*u-F(47)/99*y^4*u+F(17)/53*x^3*z*u+15419*x^2*y*z*u+15544*x*y^2*z*u-1098*y^3*z*u-4342*x^2*z^2*u+F(91)/66*x*y*z^2*u-F(97)/44*y^2*z^2*u+F(12)/37*x*z^3*u+F(113)/50*y*z^3*u+F(16)/15*z^4*u+F(109)/108*x^3*u^2+F(122)/97*x^2*y*u^2+2432*x*y^2*u^2-13459*y^3*u^2+F(56)/27*x^2*z*u^2+15590*x*y*z*u^2+F(114)/115*y^2*z*u^2+F(92)/81*x*z^2*u^2+F(57)/55*y*z^2*u^2+4277*z^3*u^2-F(114)/91*x^2*u^3+F(48)/11*x*y*u^3+F(69)/47*y^2*u^3+F(26)/63*x*z*u^3+F(82)/55*y*z*u^3+F(19)/56*z^2*u^3+5173*x*u^4+7345*y*u^4+F(31)/51*z*u^4-10858*u^5+F(38)/77*x^3*y*v+F(95)/3*x^2*y^2*v+F(99)/56*x*y^3*v-F(25)/82*y^4*v-F(49)/86*x^3*z*v+9525*x^2*y*z*v-963*x*y^2*z*v+F(115)/112*y^3*z*v-F(93)/7*x^2*z^2*v-F(94)/39*x*y*z^2*v+2888*y^2*z^2*v+F(40)/73*x*z^3*v+15915*y*z^3*v-F(123)/103*z^4*v+F(31)/10*x^3*u*v-F(47)/84*x^2*y*u*v-F(77)/73*x*y^2*u*v+1110*y^3*u*v+F(94)/3*x^2*z*u*v+F(83)/97*x*y*z*u*v-F(87)/40*y^2*z*u*v+3166*x*z^2*u*v+F(84)/19*y*z^2*u*v+F(4)/9*z^3*u*v-F(47)/73*x^2*u^2*v+6796*x*y*u^2*v+F(1)/35*y^2*u^2*v-11593*x*z*u^2*v-10264*y*z*u^2*v-F(85)/113*z^2*u^2*v-F(27)/5*x*u^3*v+F(117)/110*y*u^3*v+13527*z*u^3*v-F(125)/29*u^4*v+4216*x^3*v^2-F(89)/73*x^2*y*v^2+F(61)/114*x*y^2*v^2-F(18)/109*y^3*v^2-F(21)/116*x^2*z*v^2-F(37)/72*x*y*z*v^2-8919*y^2*z*v^2-10257*x*z^2*v^2-F(83)/123*y*z^2*v^2+6893*z^3*v^2+9701*x^2*u*v^2-2817*x*y*u*v^2+F(121)/125*y^2*u*v^2-9257*x*z*u*v^2+F(51)/104*y*z*u*v^2+4509*z^2*u*v^2+F(17)/95*x*u^2*v^2-F(123)/17*y*u^2*v^2-14563*z*u^2*v^2-7370*u^3*v^2-F(108)/103*x^2*v^3+F(43)/46*x*y*v^3-F(114)/71*y^2*v^3+11979*x*z*v^3-F(39)/98*y*z*v^3-F(96)/29*z^2*v^3+F(41)/100*x*u*v^3-F(61)/31*y*u*v^3-F(97)/77*z*u*v^3+F(45)/32*u^2*v^3+13443*x*v^4+4841*y*v^4+2019*z*v^4-6297*u*v^4+1217*v^5,-F(5)/12*x^4*y-F(17)/55*x^3*y^2+F(85)/24*x^2*y^3-F(47)/99*x*y^4-F(65)/88*x^4*z+F(77)/13*x^3*y*z-F(119)/99*x^2*y^2*z-F(47)/24*x*y^3*z+73*y^4*z+F(71)/35*x^3*z^2-11222*x^2*y*z^2+109*x*y^2*z^2+F(25)/48*y^3*z^2+F(109)/100*x^2*z^3-6234*x*y*z^3-F(91)/6*y^2*z^3+F(117)/103*x*z^4+F(107)/81*y*z^4-13804*z^5+F(109)/108*x^4*u+F(122)/97*x^3*y*u+2432*x^2*y^2*u-13459*x*y^3*u-5302*x^3*z*u+4341*x^2*y*z*u-2102*x*y^2*z*u-F(105)/104*y^3*z*u-F(85)/101*x^2*z^2*u+F(117)/83*x*y*z^2*u-15094*y^2*z^2*u+13366*x*z^3*u+F(86)/89*y*z^3*u+F(111)/47*z^4*u-F(114)/91*x^3*u^2+F(48)/11*x^2*y*u^2+F(69)/47*x*y^2*u^2+F(57)/46*x^2*z*u^2-F(95)/27*x*y*z*u^2+9896*y^2*z*u^2+13293*x*z^2*u^2-F(43)/32*y*z^2*u^2+14673*z^3*u^2+5173*x^2*u^3+7345*x*y*u^3-F(110)/119*x*z*u^3+F(112)/43*y*z*u^3-F(101)/53*z^2*u^3-10858*x*u^4+F(77)/115*z*u^4+F(25)/107*x^4*v-14846*x^3*y*v+F(91)/38*x^2*y^2*v+F(27)/43*x*y^3*v+F(74)/3*y^4*v+F(109)/58*x^3*z*v+F(104)/29*x^2*y*z*v-F(18)/115*x*y^2*z*v+F(52)/3*y^3*z*v+F(63)/113*x^2*z^2*v+F(7)/68*x*y*z^2*v-42*y^2*z^2*v-14528*x*z^3*v+F(77)/6*y*z^3*v+F(123)/80*z^4*v-F(113)/11*x^3*u*v-F(53)/10*x^2*y*u*v-F(46)/79*x*y^2*u*v+F(46)/93*y^3*u*v-F(103)/111*x^2*z*u*v-4934*x*y*z*u*v-F(103)/62*y^2*z*u*v-8712*x*z^2*u*v-F(101)/3*y*z^2*u*v+F(99)/32*z^3*u*v-9312*x^2*u^2*v+F(96)/97*x*y*u^2*v-13243*y^2*u^2*v+15659*x*z*u^2*v-F(95)/32*y*z*u^2*v-F(48)/113*z^2*u^2*v+1755*x*u^3*v+15533*y*u^3*v+6109*z*u^3*v+F(61)/8*u^4*v-15620*x^3*v^2-F(11)/48*x^2*y*v^2-F(21)/52*x*y^2*v^2+F(7)/51*y^3*v^2-F(49)/41*x^2*z*v^2+15041*x*y*z*v^2-548*y^2*z*v^2-F(11)/52*x*z^2*v^2-6487*y*z^2*v^2-F(88)/29*z^3*v^2+F(47)/76*x^2*u*v^2+10979*x*y*u*v^2+F(55)/103*y^2*u*v^2-F(114)/23*x*z*u*v^2-F(34)/91*y*z*u*v^2+13830*z^2*u*v^2+F(90)/71*x*u^2*v^2+F(113)/22*y*u^2*v^2+10984*z*u^2*v^2-202*u^3*v^2-4502*x^2*v^3+F(54)/37*x*y*v^3+F(23)/121*y^2*v^3-F(24)/5*x*z*v^3+4949*y*z*v^3+F(77)/120*z^2*v^3+F(72)/19*x*u*v^3+F(58)/23*y*u*v^3-F(7)/31*z*u*v^3-963*u^2*v^3-14279*x*v^4+8071*y*v^4-F(79)/103*z*v^4-3975*u*v^4+11692*v^5,F(5)/12*x^3*y^2+F(17)/55*x^2*y^3-F(85)/24*x*y^4+F(47)/99*y^5-1533*x^3*y*z+F(59)/45*x^2*y^2*z-F(115)/109*x*y^3*z-1381*y^4*z-10428*x^3*z^2+F(75)/62*x^2*y*z^2+F(97)/49*x*y^2*z^2-2421*y^3*z^2-10712*x^2*z^3-F(109)/99*x*y*z^3-3248*y^2*z^3-12042*x*z^4-F(123)/92*y*z^4-F(47)/34*z^5-F(109)/108*x^3*y*u-F(122)/97*x^2*y^2*u-2432*x*y^3*u+13459*y^4*u+F(113)/76*x^3*z*u+8712*x^2*y*z*u-F(78)/7*x*y^2*z*u+F(98)/19*y^3*z*u-2951*x^2*z^2*u+F(19)/70*x*y*z^2*u+14443*y^2*z^2*u+14636*x*z^3*u-435*y*z^3*u-F(111)/59*z^4*u+F(114)/91*x^2*y*u^2-F(48)/11*x*y^2*u^2-F(69)/47*y^3*u^2+30*x^2*z*u^2-F(83)/122*x*y*z*u^2-F(57)/89*y^2*z*u^2-10082*x*z^2*u^2-F(41)/19*y*z^2*u^2+F(33)/106*z^3*u^2-5173*x*y*u^3-7345*y^2*u^3-F(126)/67*x*z*u^3+F(54)/7*y*z*u^3+F(25)/18*z^2*u^3+10858*y*u^4-F(25)/22*z*u^4-F(111)/61*x^3*y*v+F(92)/87*x^2*y^2*v+F(89)/40*x*y^3*v+1324*y^4*v-14084*x^3*z*v-F(17)/67*x^2*y*z*v-F(82)/65*x*y^2*z*v+F(121)/101*y^3*z*v+F(73)/104*x^2*z^2*v+4496*x*y*z^2*v-3619*y^2*z^2*v-7129*x*z^3*v-F(15)/97*y*z^3*v+F(124)/15*z^4*v-F(60)/67*x^3*u*v+F(64)/85*x^2*y*u*v-9729*x*y^2*u*v-F(5)/44*y^3*u*v-F(105)/67*x^2*z*u*v-F(23)/66*x*y*z*u*v+8976*y^2*z*u*v-F(25)/126*x*z^2*u*v-13797*y*z^2*u*v-F(98)/19*z^3*u*v+9417*x^2*u^2*v-F(95)/67*x*y*u^2*v+2750*y^2*u^2*v+F(5)/69*x*z*u^2*v-920*y*z*u^2*v+F(38)/7*z^2*u^2*v-F(31)/66*x*u^3*v-6249*y*u^3*v-13559*z*u^3*v-F(74)/9*u^4*v+F(122)/63*x^3*v^2-12741*x^2*y*v^2+F(92)/65*x*y^2*v^2+F(7)/32*y^3*v^2+10392*x^2*z*v^2-8746*x*y*z*v^2-7626*y^2*z*v^2-10714*x*z^2*v^2+F(6)/59*y*z^2*v^2-F(79)/18*z^3*v^2-F(23)/103*x^2*u*v^2-8506*x*y*u*v^2+F(6)/23*y^2*u*v^2+6591*x*z*u*v^2+12936*y*z*u*v^2+F(27)/26*z^2*u*v^2+15753*x*u^2*v^2-F(31)/54*y*u^2*v^2-F(41)/96*z*u^2*v^2-15923*u^3*v^2-F(119)/26*x^2*v^3+3946*x*y*v^3+4401*y^2*v^3-3608*x*z*v^3-F(39)/71*y*z*v^3-1244*z^2*v^3+F(87)/2*x*u*v^3+F(23)/16*y*u*v^3-F(81)/82*z*u*v^3-47*u^2*v^3+8406*x*v^4+F(116)/109*y*v^4-F(24)/85*z*v^4+F(26)/47*u*v^4+14916*v^5,-F(72)/95*x^4*y-1265*x^3*y^2+F(105)/103*x^2*y^3+50*x*y^4-73*y^5+10428*x^4*z-5797*x^3*y*z-F(21)/16*x^2*y^2*z-F(50)/83*x*y^3*z-F(25)/48*y^4*z+10712*x^3*z^2+F(107)/33*x^2*y*z^2+F(86)/27*x*y^2*z^2+F(91)/6*y^3*z^2+12042*x^2*z^3-12042*x*y*z^3-F(107)/81*y^2*z^3+F(47)/34*x*z^4+13804*y*z^4-F(113)/76*x^4*u-3410*x^3*y*u-8900*x^2*y^2*u+F(6)/53*x*y^3*u+F(105)/104*y^4*u+2951*x^3*z*u-8737*x^2*y*z*u-5194*x*y^2*z*u+15094*y^3*z*u-14636*x^2*z^2*u+F(72)/47*x*y*z^2*u-F(86)/89*y^2*z^2*u+F(111)/59*x*z^3*u-F(111)/47*y*z^3*u-30*x^3*u^2-7890*x^2*y*u^2-F(85)/98*x*y^2*u^2-9896*y^3*u^2+10082*x^2*z*u^2-F(110)/37*x*y*z*u^2+F(43)/32*y^2*z*u^2-F(33)/106*x*z^2*u^2-14673*y*z^2*u^2+F(126)/67*x^2*u^3+F(42)/73*x*y*u^3-F(112)/43*y^2*u^3-F(25)/18*x*z*u^3+F(101)/53*y*z*u^3+F(25)/22*x*u^4-F(77)/115*y*u^4-6214*x^4*v+F(49)/33*x^3*y*v+F(115)/31*x^2*y^2*v-2301*x*y^3*v+F(31)/91*y^4*v-F(90)/19*x^3*z*v-15708*x^2*y*z*v-5301*x*y^2*z*v+13607*y^3*z*v-F(122)/33*x^2*z^2*v+F(47)/13*x*y*z^2*v+6710*y^2*z^2*v-F(76)/77*x*z^3*v+6967*y*z^3*v+7864*z^4*v-7281*x^3*u*v+5534*x^2*y*u*v+F(111)/115*x*y^2*u*v+F(89)/115*y^3*u*v+F(39)/40*x^2*z*u*v+8768*x*y*z*u*v+F(115)/33*y^2*z*u*v-F(12)/109*x*z^2*u*v-F(35)/36*y*z^2*u*v+15409*z^3*u*v-F(57)/34*x^2*u^2*v-7042*x*y*u^2*v+F(13)/59*y^2*u^2*v-7729*x*z*u^2*v+F(40)/17*y*z*u^2*v+10206*z^2*u^2*v+F(102)/95*x*u^3*v+F(33)/97*y*u^3*v+F(65)/7*z*u^3*v-F(94)/19*u^4*v+160*x^3*v^2+F(97)/23*x^2*y*v^2+F(106)/109*x*y^2*v^2+15078*y^3*v^2+15340*x^2*z*v^2-4732*x*y*z*v^2+10800*y^2*z*v^2+F(2)/71*x*z^2*v^2+F(13)/19*y*z^2*v^2+F(118)/27*z^3*v^2+179*x^2*u*v^2+F(97)/76*x*y*u*v^2-F(83)/104*y^2*u*v^2-F(1)/7*x*z*u*v^2-F(19)/85*y*z*u*v^2-F(73)/83*z^2*u*v^2+F(112)/15*x*u^2*v^2-3969*y*u^2*v^2+F(47)/15*z*u^2*v^2-10621*u^3*v^2-F(21)/103*x^2*v^3-7608*x*y*v^3-F(59)/38*y^2*v^3-14153*x*z*v^3-8965*y*z*v^3+10611*z^2*v^3+7459*x*u*v^3+F(118)/101*y*u*v^3-F(56)/55*z*u*v^3+F(5)/83*u^2*v^3+F(39)/121*x*v^4-F(9)/101*y*v^4-F(125)/8*z*v^4-F(8)/85*u*v^4+F(29)/20*v^5,-3268*x^4*y-F(1)/124*x^3*y^2-8651*x^2*y^3+F(109)/110*x*y^4-F(77)/81*x^4*z+10718*x^3*y*z+F(83)/45*x^2*y^2*z+F(52)/51*x*y^3*z-F(80)/119*x^3*z^2-8440*x^2*y*z^2+F(12)/49*x*y^2*z^2-4021*x^2*z^3-12292*x*y*z^3-F(117)/83*x*z^4+7472*x^4*u+1191*x^3*y*u+F(48)/23*x^2*y^2*u-F(41)/10*x*y^3*u+F(23)/106*x^3*z*u+F(44)/125*x^2*y*z*u+F(22)/113*x*y^2*z*u-F(46)/55*x^2*z^2*u+12579*x*y*z^2*u-F(18)/121*x*z^3*u-8140*x^3*u^2+F(14)/25*x^2*y*u^2+8444*x*y^2*u^2+F(32)/39*x^2*z*u^2+14202*x*y*z*u^2-F(11)/90*x*z^2*u^2+F(26)/103*x^2*u^3+F(43)/100*x*y*u^3+F(98)/93*x*z*u^3+F(77)/116*x*u^4-F(112)/111*x^4*v-1972*x^3*y*v+F(34)/49*x^2*y^2*v-F(35)/114*x*y^3*v-F(117)/70*y^4*v+13583*x^3*z*v+F(48)/91*x^2*y*z*v-F(61)/81*x*y^2*z*v-F(37)/61*y^3*z*v-3607*x^2*z^2*v-5880*x*y*z^2*v+F(5)/69*y^2*z^2*v+F(109)/29*x*z^3*v+F(63)/64*y*z^3*v-F(112)/43*z^4*v-F(9)/2*x^3*u*v-F(60)/37*x^2*y*u*v+F(59)/110*x*y^2*u*v+10799*y^3*u*v+12121*x^2*z*u*v+F(115)/96*x*y*z*u*v+6357*y^2*z*u*v+F(110)/19*x*z^2*u*v-F(96)/25*y*z^2*u*v+F(31)/65*z^3*u*v-F(17)/97*x^2*u^2*v+F(87)/122*x*y*u^2*v-F(77)/90*y^2*u^2*v-F(22)/81*x*z*u^2*v-6130*y*z*u^2*v-F(88)/43*z^2*u^2*v+9369*x*u^3*v-F(29)/95*y*u^3*v+2930*z*u^3*v+F(28)/15*u^4*v-F(122)/89*x^3*v^2-6064*x^2*y*v^2-15273*x*y^2*v^2+10771*y^3*v^2+448*x^2*z*v^2-8588*x*y*z*v^2+F(107)/51*y^2*z*v^2+F(63)/62*x*z^2*v^2-F(20)/119*y*z^2*v^2+13232*z^3*v^2-F(109)/49*x^2*u*v^2+F(52)/69*x*y*u*v^2+F(76)/47*y^2*u*v^2+F(118)/31*x*z*u*v^2+F(119)/99*y*z*u*v^2-F(104)/59*z^2*u*v^2-F(57)/85*x*u^2*v^2+F(67)/110*y*u^2*v^2-3872*z*u^2*v^2+F(30)/11*u^3*v^2+2738*x^2*v^3+F(83)/41*x*y*v^3+F(35)/41*y^2*v^3-15785*x*z*v^3-F(53)/118*y*z*v^3+F(107)/35*z^2*v^3+F(109)/61*x*u*v^3-F(75)/2*y*u*v^3+F(55)/91*z*u*v^3-F(73)/25*u^2*v^3-F(79)/10*x*v^4-F(71)/106*y*v^4-11735*z*v^4+F(82)/95*u*v^4+F(61)/70*v^5,3268*x^3*y^2+F(1)/124*x^2*y^3+8651*x*y^4-F(109)/110*y^5+F(77)/81*x^3*y*z-10718*x^2*y^2*z-F(83)/45*x*y^3*z-F(52)/51*y^4*z+F(80)/119*x^2*y*z^2+8440*x*y^2*z^2-F(12)/49*y^3*z^2+4021*x*y*z^3+12292*y^2*z^3+F(117)/83*y*z^4-7472*x^3*y*u-1191*x^2*y^2*u-F(48)/23*x*y^3*u+F(41)/10*y^4*u-F(23)/106*x^2*y*z*u-F(44)/125*x*y^2*z*u-F(22)/113*y^3*z*u+F(46)/55*x*y*z^2*u-12579*y^2*z^2*u+F(18)/121*y*z^3*u+8140*x^2*y*u^2-F(14)/25*x*y^2*u^2-8444*y^3*u^2-F(32)/39*x*y*z*u^2-14202*y^2*z*u^2+F(11)/90*y*z^2*u^2-F(26)/103*x*y*u^3-F(43)/100*y^2*u^3-F(98)/93*y*z*u^3-F(77)/116*y*u^4+F(1)/111*x^3*y*v+F(31)/43*x^2*y^2*v+F(25)/101*x*y^3*v+14620*y^4*v+F(20)/77*x^3*z*v+1550*x^2*y*z*v+6771*x*y^2*z*v-1051*y^3*z*v+F(19)/99*x^2*z^2*v+F(82)/101*x*y*z^2*v+7448*y^2*z^2*v-F(45)/116*x*z^3*v+1697*y*z^3*v+F(71)/87*z^4*v+F(31)/52*x^3*u*v-F(82)/35*x^2*y*u*v+F(19)/50*x*y^2*u*v-F(79)/88*y^3*u*v+F(36)/17*x^2*z*u*v-F(72)/115*x*y*z*u*v-F(64)/91*y^2*z*u*v-F(113)/78*x*z^2*u*v+F(37)/35*y*z^2*u*v-F(43)/87*z^3*u*v-F(29)/81*x^2*u^2*v-14745*x*y*u^2*v-F(3)/56*y^2*u^2*v-14839*x*z*u^2*v+F(67)/57*y*z*u^2*v+13868*z^2*u^2*v+F(85)/19*x*u^3*v-14300*y*u^3*v+F(21)/44*z*u^3*v+F(109)/41*u^4*v-F(17)/76*x^3*v^2-11367*x^2*y*v^2-3073*x*y^2*v^2-F(29)/35*y^3*v^2+6510*x^2*z*v^2+9282*x*y*z*v^2+4732*y^2*z*v^2+11908*x*z^2*v^2+2251*y*z^2*v^2-3527*z^3*v^2-F(17)/114*x^2*u*v^2-F(39)/23*x*y*u*v^2-9949*y^2*u*v^2-F(64)/93*x*z*u*v^2+F(109)/104*y*z*u*v^2-2149*z^2*u*v^2-234*x*u^2*v^2+F(49)/86*y*u^2*v^2-14794*z*u^2*v^2+F(111)/20*u^3*v^2+F(23)/51*x^2*v^3-F(79)/112*x*y*v^3+9231*y^2*v^3+F(29)/62*x*z*v^3+F(50)/53*y*z*v^3-F(86)/125*z^2*v^3-F(123)/19*x*u*v^3-F(102)/97*y*u*v^3-13143*z*u*v^3-F(1)/117*u^2*v^3+F(112)/47*x*v^4-F(37)/41*y*v^4+10229*z*v^4-F(62)/47*u*v^4-F(118)/81*v^5,3268*x^3*y*z+F(1)/124*x^2*y^2*z+8651*x*y^3*z-F(109)/110*y^4*z+F(77)/81*x^3*z^2-10718*x^2*y*z^2-F(83)/45*x*y^2*z^2-F(52)/51*y^3*z^2+F(80)/119*x^2*z^3+8440*x*y*z^3-F(12)/49*y^2*z^3+4021*x*z^4+12292*y*z^4+F(117)/83*z^5-7472*x^3*z*u-1191*x^2*y*z*u-F(48)/23*x*y^2*z*u+F(41)/10*y^3*z*u-F(23)/106*x^2*z^2*u-F(44)/125*x*y*z^2*u-F(22)/113*y^2*z^2*u+F(46)/55*x*z^3*u-12579*y*z^3*u+F(18)/121*z^4*u+8140*x^2*z*u^2-F(14)/25*x*y*z*u^2-8444*y^2*z*u^2-F(32)/39*x*z^2*u^2-14202*y*z^2*u^2+F(11)/90*z^3*u^2-F(26)/103*x*z*u^3-F(43)/100*y*z*u^3-F(98)/93*z^2*u^3-F(77)/116*z*u^4-F(101)/15*x^3*y*v+9577*x^2*y^2*v+F(69)/19*x*y^3*v-F(95)/107*y^4*v+10248*x^3*z*v+F(82)/109*x^2*y*z*v+13478*x*y^2*z*v-14455*y^3*z*v+F(3)/112*x^2*z^2*v-F(55)/101*x*y*z^2*v+9757*y^2*z^2*v+F(93)/22*x*z^3*v-F(107)/56*y*z^3*v-F(52)/125*z^4*v-6496*x^3*u*v-F(61)/50*x^2*y*u*v+F(32)/79*x*y^2*u*v-F(69)/103*y^3*u*v-7018*x^2*z*u*v+F(95)/48*x*y*z*u*v+F(77)/43*y^2*z*u*v-F(123)/109*x*z^2*u*v+F(106)/5*y*z^2*u*v+F(119)/104*z^3*u*v+899*x^2*u^2*v+F(85)/126*x*y*u^2*v+F(109)/110*y^2*u^2*v+F(40)/61*x*z*u^2*v-F(59)/82*y*z*u^2*v+F(34)/89*z^2*u^2*v+F(55)/61*x*u^3*v-F(1)/49*y*u^3*v+F(87)/119*z*u^3*v+F(73)/122*u^4*v-F(89)/82*x^3*v^2-5039*x^2*y*v^2+11775*x*y^2*v^2+3982*y^3*v^2-F(1)/77*x^2*z*v^2+F(1)/88*x*y*z*v^2-F(103)/92*y^2*z*v^2-F(83)/59*x*z^2*v^2-F(73)/42*y*z^2*v^2-F(85)/86*z^3*v^2-F(112)/95*x^2*u*v^2+574*x*y*u*v^2-9922*y^2*u*v^2-F(121)/120*x*z*u*v^2-1497*y*z*u*v^2+F(25)/41*z^2*u*v^2-1252*x*u^2*v^2+12336*y*u^2*v^2-3698*z*u^2*v^2-14349*u^3*v^2+10339*x^2*v^3+F(27)/64*x*y*v^3-F(69)/82*y^2*v^3+F(17)/31*x*z*v^3+F(68)/53*y*z*v^3-8353*z^2*v^3+388*x*u*v^3-12452*y*u*v^3-F(76)/27*z*u*v^3-14264*u^2*v^3+F(70)/9*x*v^4+F(61)/82*y*v^4-F(53)/100*z*v^4+F(107)/85*u*v^4-8650*v^5,3268*x^3*y*u+F(1)/124*x^2*y^2*u+8651*x*y^3*u-F(109)/110*y^4*u+F(77)/81*x^3*z*u-10718*x^2*y*z*u-F(83)/45*x*y^2*z*u-F(52)/51*y^3*z*u+F(80)/119*x^2*z^2*u+8440*x*y*z^2*u-F(12)/49*y^2*z^2*u+4021*x*z^3*u+12292*y*z^3*u+F(117)/83*z^4*u-7472*x^3*u^2-1191*x^2*y*u^2-F(48)/23*x*y^2*u^2+F(41)/10*y^3*u^2-F(23)/106*x^2*z*u^2-F(44)/125*x*y*z*u^2-F(22)/113*y^2*z*u^2+F(46)/55*x*z^2*u^2-12579*y*z^2*u^2+F(18)/121*z^3*u^2+8140*x^2*u^3-F(14)/25*x*y*u^3-8444*y^2*u^3-F(32)/39*x*z*u^3-14202*y*z*u^3+F(11)/90*z^2*u^3-F(26)/103*x*u^4-F(43)/100*y*u^4-F(98)/93*z*u^4-F(77)/116*u^5+F(33)/31*x^3*y*v-12827*x^2*y^2*v-F(94)/85*x*y^3*v+F(71)/90*y^4*v+F(109)/78*x^3*z*v+1790*x^2*y*z*v-F(87)/61*x*y^2*z*v+13354*y^3*z*v+F(25)/4*x^2*z^2*v-F(115)/72*x*y*z^2*v+F(124)/3*y^2*z^2*v+6945*x*z^3*v-F(35)/27*y*z^3*v-F(49)/99*z^4*v-F(113)/14*x^3*u*v-F(107)/122*x^2*y*u*v-F(58)/49*x*y^2*u*v+14139*y^3*u*v-F(74)/109*x^2*z*u*v-15164*x*y*z*u*v+F(89)/55*y^2*z*u*v+F(79)/66*x*z^2*u*v-14011*y*z^2*u*v+12712*z^3*u*v-7174*x^2*u^2*v-756*x*y*u^2*v+F(17)/7*y^2*u^2*v-F(106)/75*x*z*u^2*v-F(76)/17*y*z*u^2*v-3812*z^2*u^2*v+4937*x*u^3*v+F(41)/105*y*u^3*v+9367*z*u^3*v+10920*u^4*v+F(24)/107*x^3*v^2-F(88)/75*x^2*y*v^2-7398*x*y^2*v^2-8587*y^3*v^2+2122*x^2*z*v^2-2365*x*y*z*v^2-F(85)/117*y^2*z*v^2-F(126)/109*x*z^2*v^2+F(47)/26*y*z^2*v^2-F(57)/85*z^3*v^2+F(7)/99*x^2*u*v^2-8*x*y*u*v^2-3391*y^2*u*v^2+15883*x*z*u*v^2+14102*y*z*u*v^2+4097*z^2*u*v^2-F(108)/101*x*u^2*v^2+F(16)/61*y*u^2*v^2-F(37)/87*z*u^2*v^2-14366*u^3*v^2-F(52)/125*x^2*v^3+F(26)/103*x*y*v^3+5932*y^2*v^3+13345*x*z*v^3+F(5)/21*y*z*v^3-F(25)/64*z^2*v^3+F(81)/86*x*u*v^3+F(44)/111*y*u*v^3-F(81)/37*z*u*v^3+2928*u^2*v^3-F(47)/17*x*v^4-F(101)/40*y*v^4+12929*z*v^4+F(91)/124*u*v^4-12748*v^5,F(33)/31*x^4*y-12827*x^3*y^2-F(94)/85*x^2*y^3+F(71)/90*x*y^4+F(109)/78*x^4*z+1790*x^3*y*z-F(87)/61*x^2*y^2*z+13354*x*y^3*z+F(25)/4*x^3*z^2-F(115)/72*x^2*y*z^2+F(124)/3*x*y^2*z^2+6945*x^2*z^3-F(35)/27*x*y*z^3-F(49)/99*x*z^4-F(89)/3*x^4*u+1436*x^3*y*u-F(24)/49*x^2*y^2*u-F(89)/7*x*y^3*u-F(117)/70*y^4*u+F(49)/15*x^3*z*u-F(22)/53*x^2*y*z*u+F(50)/91*x*y^2*z*u-F(37)/61*y^3*z*u+F(69)/109*x^2*z^2*u+12100*x*y*z^2*u+F(5)/69*y^2*z^2*u-F(117)/89*x*z^3*u+F(63)/64*y*z^3*u-F(112)/43*z^4*u+8817*x^3*u^2+107*x^2*y*u^2+F(62)/119*x*y^2*u^2+10799*y^3*u^2-F(65)/94*x^2*z*u^2+3976*x*y*z*u^2+6357*y^2*z*u^2-F(108)/77*x*z^2*u^2-F(96)/25*y*z^2*u^2+F(31)/65*z^3*u^2+F(101)/119*x^2*u^3-9781*x*y*u^3-F(77)/90*y^2*u^3-8801*x*z*u^3-6130*y*z*u^3-F(88)/43*z^2*u^3+F(83)/41*x*u^4-F(29)/95*y*u^4+2930*z*u^4+F(28)/15*u^5+F(24)/107*x^4*v-F(88)/75*x^3*y*v-7398*x^2*y^2*v-8587*x*y^3*v+2122*x^3*z*v-2365*x^2*y*z*v-F(85)/117*x*y^2*z*v-F(126)/109*x^2*z^2*v+F(47)/26*x*y*z^2*v-F(57)/85*x*z^3*v-F(95)/101*x^3*u*v-6072*x^2*y*u*v-F(31)/12*x*y^2*u*v+10771*y^3*u*v-15660*x^2*z*u*v-F(49)/29*x*y*z*u*v+F(107)/51*y^2*z*u*v-F(9)/104*x*z^2*u*v-F(20)/119*y*z^2*u*v+13232*z^3*u*v+11839*x^2*u^2*v+2266*x*y*u^2*v+F(76)/47*y^2*u^2*v+F(1)/88*x*z*u^2*v+F(119)/99*y*z*u^2*v-F(104)/59*z^2*u^2*v-2323*x*u^3*v+F(67)/110*y*u^3*v-3872*z*u^3*v+F(30)/11*u^4*v-F(52)/125*x^3*v^2+F(26)/103*x^2*y*v^2+5932*x*y^2*v^2+13345*x^2*z*v^2+F(5)/21*x*y*z*v^2-F(25)/64*x*z^2*v^2+879*x^2*u*v^2+4438*x*y*u*v^2+F(35)/41*y^2*u*v^2-224*x*z*u*v^2-F(53)/118*y*z*u*v^2+F(107)/35*z^2*u*v^2-F(19)/121*x*u^2*v^2-F(75)/2*y*u^2*v^2+F(55)/91*z*u^2*v^2-F(73)/25*u^3*v^2-F(47)/17*x^2*v^3-F(101)/40*x*y*v^3+12929*x*z*v^3-F(53)/101*x*u*v^3-F(71)/106*y*u*v^3-11735*z*u*v^3+F(82)/95*u^2*v^3-12748*x*v^4+F(61)/70*u*v^4,-F(33)/31*x^3*y^2+12827*x^2*y^3+F(94)/85*x*y^4-F(71)/90*y^5-F(109)/78*x^3*y*z-1790*x^2*y^2*z+F(87)/61*x*y^3*z-13354*y^4*z-F(25)/4*x^2*y*z^2+F(115)/72*x*y^2*z^2-F(124)/3*y^3*z^2-6945*x*y*z^3+F(35)/27*y^2*z^3+F(49)/99*y*z^4+F(86)/3*x^3*y*u-F(93)/83*x^2*y^2*u-F(14)/67*x*y^3*u+481*y^4*u+F(20)/77*x^3*z*u+F(21)/32*x^2*y*z*u-F(59)/35*x*y^2*z*u+1274*y^3*z*u+F(19)/99*x^2*z^2*u+F(4)/27*x*y*z^2*u-10532*y^2*z^2*u-F(45)/116*x*z^3*u-F(104)/61*y*z^3*u+F(71)/87*z^4*u+F(31)/52*x^3*u^2+F(75)/97*x^2*y*u^2-F(109)/32*x*y^2*u^2-3379*y^3*u^2+F(36)/17*x^2*z*u^2+10887*x*y*z*u^2-7751*y^2*z*u^2-F(113)/78*x*z^2*u^2+1985*y*z^2*u^2-F(43)/87*z^3*u^2-F(29)/81*x^2*u^3+F(62)/13*x*y*u^3-15120*y^2*u^3-14839*x*z*u^3-F(71)/22*y*z*u^3+13868*z^2*u^3+F(85)/19*x*u^4+6771*y*u^4+F(21)/44*z*u^4+F(109)/41*u^5-F(24)/107*x^3*y*v+F(88)/75*x^2*y^2*v+7398*x*y^3*v+8587*y^4*v-2122*x^2*y*z*v+2365*x*y^2*z*v+F(85)/117*y^3*z*v+F(126)/109*x*y*z^2*v-F(47)/26*y^2*z^2*v+F(57)/85*y*z^3*v-F(17)/76*x^3*u*v+4790*x^2*y*u*v-3065*x*y^2*u*v-F(121)/107*y^3*u*v+6510*x^2*z*u*v+F(20)/63*x*y*z*u*v+F(109)/99*y^2*z*u*v+11908*x*z^2*u*v-F(19)/52*y*z^2*u*v-3527*z^3*u*v-F(17)/114*x^2*u^2*v-F(62)/77*x*y*u^2*v-F(34)/67*y^2*u^2*v-F(64)/93*x*z*u^2*v-F(71)/55*y*z*u^2*v-2149*z^2*u^2*v-234*x*u^3*v-F(32)/53*y*u^3*v-14794*z*u^3*v+F(111)/20*u^4*v+F(52)/125*x^2*y*v^2-F(26)/103*x*y^2*v^2-5932*y^3*v^2-13345*x*y*z*v^2-F(5)/21*y^2*z*v^2+F(25)/64*y*z^2*v^2+F(23)/51*x^2*u*v^2-F(34)/65*x*y*u*v^2-6909*y^2*u*v^2+F(29)/62*x*z*u*v^2+F(38)/59*y*z*u*v^2-F(86)/125*z^2*u*v^2-F(123)/19*x*u^2*v^2-F(13)/67*y*u^2*v^2-13143*z*u^2*v^2-F(1)/117*u^3*v^2+F(47)/17*x*y*v^3+F(101)/40*y^2*v^3-12929*y*z*v^3+F(112)/47*x*u*v^3+F(77)/76*y*u*v^3+10229*z*u*v^3-F(62)/47*u^2*v^3+12748*y*v^4-F(118)/81*u*v^4,-F(33)/31*x^3*y*z+12827*x^2*y^2*z+F(94)/85*x*y^3*z-F(71)/90*y^4*z-F(109)/78*x^3*z^2-1790*x^2*y*z^2+F(87)/61*x*y^2*z^2-13354*y^3*z^2-F(25)/4*x^2*z^3+F(115)/72*x*y*z^3-F(124)/3*y^2*z^3-6945*x*z^4+F(35)/27*y*z^4+F(49)/99*z^5-F(101)/15*x^3*y*u+9577*x^2*y^2*u+F(69)/19*x*y^3*u-F(95)/107*y^4*u-F(107)/4*x^3*z*u+F(74)/103*x^2*y*z*u+F(107)/104*x*y^2*z*u-F(31)/113*y^3*z*u+14589*x^2*z^2*u-F(64)/41*x*y*z^2*u+12082*y^2*z^2*u+F(100)/33*x*z^3*u+F(68)/71*y*z^3*u-7082*z^4*u-6496*x^3*u^2-F(61)/50*x^2*y*u^2+F(32)/79*x*y^2*u^2-F(69)/103*y^3*u^2+156*x^2*z*u^2-15904*x*y*z*u^2+7014*y^2*z*u^2+F(59)/56*x*z^2*u^2-12018*y*z^2*u^2+F(23)/33*z^3*u^2+899*x^2*u^3+F(85)/126*x*y*u^3+F(109)/110*y^2*u^3-F(101)/95*x*z*u^3-F(75)/91*y*z*u^3-F(122)/23*z^2*u^3+F(55)/61*x*u^4-F(1)/49*y*u^4+1447*z*u^4+F(73)/122*u^5-F(24)/107*x^3*z*v+F(88)/75*x^2*y*z*v+7398*x*y^2*z*v+8587*y^3*z*v-2122*x^2*z^2*v+2365*x*y*z^2*v+F(85)/117*y^2*z^2*v+F(126)/109*x*z^3*v-F(47)/26*y*z^3*v+F(57)/85*z^4*v-F(89)/82*x^3*u*v-5039*x^2*y*u*v+11775*x*y^2*u*v+3982*y^3*u*v-F(47)/10*x^2*z*u*v+F(13)/47*x*y*z*u*v+F(78)/19*y^2*z*u*v-5040*x*z^2*u*v+F(108)/83*y*z^2*u*v+F(114)/103*z^3*u*v-F(112)/95*x^2*u^2*v+574*x*y*u^2*v-9922*y^2*u^2*v+F(73)/115*x*z*u^2*v-F(44)/27*y*z*u^2*v-5416*z^2*u^2*v-1252*x*u^3*v+12336*y*u^3*v+F(13)/3*z*u^3*v-14349*u^4*v+F(52)/125*x^2*z*v^2-F(26)/103*x*y*z*v^2-5932*y^2*z*v^2-13345*x*z^2*v^2-F(5)/21*y*z^2*v^2+F(25)/64*z^3*v^2+10339*x^2*u*v^2+F(27)/64*x*y*u*v^2-F(69)/82*y^2*u*v^2-F(84)/61*x*z*u*v^2-F(86)/87*y*z*u*v^2+8077*z^2*u*v^2+388*x*u^2*v^2-12452*y*u^2*v^2+6548*z*u^2*v^2-14264*u^3*v^2+F(47)/17*x*z*v^3+F(101)/40*y*z*v^3-12929*z^2*v^3+F(70)/9*x*u*v^3+F(61)/82*y*u*v^3-F(107)/52*z*u*v^3+F(107)/85*u^2*v^3+12748*z*v^4-8650*u*v^4,-F(101)/15*x^4*y+9577*x^3*y^2+F(69)/19*x^2*y^3-F(95)/107*x*y^4-13386*x^4*z+14171*x^3*y*z-F(109)/54*x^2*y^2*z+F(119)/50*x*y^3*z-F(117)/70*y^4*z+F(107)/44*x^3*z^2+15677*x^2*y*z^2+F(29)/30*x*y^2*z^2-F(37)/61*y^3*z^2+F(22)/23*x^2*z^3+15255*x*y*z^3+F(5)/69*y^2*z^3-F(44)/37*x*z^4+F(63)/64*y*z^4-F(112)/43*z^5-6496*x^4*u-F(61)/50*x^3*y*u+F(32)/79*x^2*y^2*u-F(69)/103*x*y^3*u-F(7)/82*x^3*z*u+F(83)/81*x^2*y*z*u+11047*x*y^2*z*u+10799*y^3*z*u+F(27)/41*x^2*z^2*u-8042*x*y*z^2*u+6357*y^2*z^2*u-2632*x*z^3*u-F(96)/25*y*z^3*u+F(31)/65*z^4*u+899*x^3*u^2+F(85)/126*x^2*y*u^2+F(109)/110*x*y^2*u^2-5125*x^2*z*u^2+F(115)/19*x*y*z*u^2-F(77)/90*y^2*z*u^2-F(115)/124*x*z^2*u^2-6130*y*z^2*u^2-F(88)/43*z^3*u^2+F(55)/61*x^2*u^3-F(1)/49*x*y*u^3-F(115)/78*x*z*u^3-F(29)/95*y*z*u^3+2930*z^2*u^3+F(73)/122*x*u^4+F(28)/15*z*u^4-F(89)/82*x^4*v-5039*x^3*y*v+11775*x^2*y^2*v+3982*x*y^3*v-F(105)/86*x^3*z*v-F(6)/25*x^2*y*z*v-F(84)/37*x*y^2*z*v+10771*y^3*z*v+F(1)/17*x^2*z^2*v-F(75)/22*x*y*z^2*v+F(107)/51*y^2*z^2*v-F(23)/36*x*z^3*v-F(20)/119*y*z^3*v+13232*z^4*v-F(112)/95*x^3*u*v+574*x^2*y*u*v-9922*x*y^2*u*v-F(41)/80*x^2*z*u*v+F(25)/101*x*y*z*u*v+F(76)/47*y^2*z*u*v-10869*x*z^2*u*v+F(119)/99*y*z^2*u*v-F(104)/59*z^3*u*v-1252*x^2*u^2*v+12336*x*y*u^2*v-F(11)/23*x*z*u^2*v+F(67)/110*y*z*u^2*v-3872*z^2*u^2*v-14349*x*u^3*v+F(30)/11*z*u^3*v+10339*x^3*v^2+F(27)/64*x^2*y*v^2-F(69)/82*x*y^2*v^2-11709*x^2*z*v^2-F(53)/73*x*y*z*v^2+F(35)/41*y^2*z*v^2+F(73)/110*x*z^2*v^2-F(53)/118*y*z^2*v^2+F(107)/35*z^3*v^2+388*x^2*u*v^2-12452*x*y*u*v^2+12100*x*z*u*v^2-F(75)/2*y*z*u*v^2+F(55)/91*z^2*u*v^2-14264*x*u^2*v^2-F(73)/25*z*u^2*v^2+F(70)/9*x^2*v^3+F(61)/82*x*y*v^3+F(8)/37*x*z*v^3-F(71)/106*y*z*v^3-11735*z^2*v^3+F(107)/85*x*u*v^3+F(82)/95*z*u*v^3-8650*x*v^4+F(61)/70*z*v^4,F(101)/15*x^3*y^2-9577*x^2*y^3-F(69)/19*x*y^4+F(95)/107*y^5+F(99)/98*x^3*y*z+6921*x^2*y^2*z-F(113)/95*x*y^3*z-F(85)/11*y^4*z+F(20)/77*x^3*z^2-1592*x^2*y*z^2+8672*x*y^2*z^2-F(17)/74*y^3*z^2+F(19)/99*x^2*z^3-5604*x*y*z^3-13687*y^2*z^3-F(45)/116*x*z^4+F(39)/122*y*z^4+F(71)/87*z^5+6496*x^3*y*u+F(61)/50*x^2*y^2*u-F(32)/79*x*y^3*u+F(69)/103*y^4*u+F(31)/52*x^3*z*u-14007*x^2*y*z*u+10902*x*y^2*z*u-10393*y^3*z*u+F(36)/17*x^2*z^2*u+4602*x*y*z^2*u+F(23)/15*y^2*z^2*u-F(113)/78*x*z^3*u+5862*y*z^3*u-F(43)/87*z^4*u-899*x^2*y*u^2-F(85)/126*x*y^2*u^2-F(109)/110*y^3*u^2-F(29)/81*x^2*z*u^2-11599*x*y*z*u^2+F(117)/92*y^2*z*u^2-14839*x*z^2*u^2+1962*y*z^2*u^2+13868*z^3*u^2-F(55)/61*x*y*u^3+F(1)/49*y^2*u^3+F(85)/19*x*z*u^3-F(47)/6*y*z*u^3+F(21)/44*z^2*u^3-F(73)/122*y*u^4+F(109)/41*z*u^4+F(89)/82*x^3*y*v+5039*x^2*y^2*v-11775*x*y^3*v-3982*y^4*v-F(17)/76*x^3*z*v-F(79)/20*x^2*y*z*v+F(61)/121*x*y^2*z*v-15568*y^3*z*v+6510*x^2*z^2*v-F(19)/41*x*y*z^2*v-F(81)/74*y^2*z^2*v+11908*x*z^3*v-F(31)/17*y*z^3*v-3527*z^4*v+F(112)/95*x^2*y*u*v-574*x*y^2*u*v+9922*y^3*u*v-F(17)/114*x^2*z*u*v+1935*x*y*z*u*v-F(82)/53*y^2*z*u*v-F(64)/93*x*z^2*u*v-8545*y*z^2*u*v-2149*z^3*u*v+1252*x*y*u^2*v-12336*y^2*u^2*v-234*x*z*u^2*v-F(37)/89*y*z*u^2*v-14794*z^2*u^2*v+14349*y*u^3*v+F(111)/20*z*u^3*v-10339*x^2*y*v^2-F(27)/64*x*y^2*v^2+F(69)/82*y^3*v^2+F(23)/51*x^2*z*v^2+F(24)/47*x*y*z*v^2-F(86)/27*y^2*z*v^2+F(29)/62*x*z^2*v^2+F(77)/68*y*z^2*v^2-F(86)/125*z^3*v^2-388*x*y*u*v^2+12452*y^2*u*v^2-F(123)/19*x*z*u*v^2-F(68)/3*y*z*u*v^2-13143*z^2*u*v^2+14264*y*u^2*v^2-F(1)/117*z*u^2*v^2-F(70)/9*x*y*v^3-F(61)/82*y^2*v^3+F(112)/47*x*z*v^3-9387*y*z*v^3+10229*z^2*v^3-F(107)/85*y*u*v^3-F(62)/47*z*u*v^3+8650*y*v^4-F(118)/81*z*v^4,x^4*y+F(88)/91*x^3*y^2-F(45)/89*x^2*y^3+8672*x*y^4+F(117)/70*y^5-F(20)/77*x^4*z-15133*x^3*y*z+7642*x^2*y^2*z-F(19)/104*x*y^3*z+F(37)/61*y^4*z-F(19)/99*x^3*z^2-F(91)/33*x^2*y*z^2+F(19)/102*x*y^2*z^2-F(5)/69*y^3*z^2+F(45)/116*x^2*z^3+F(45)/11*x*y*z^3-F(63)/64*y^2*z^3-F(71)/87*x*z^4+F(112)/43*y*z^4-F(31)/52*x^4*u+5034*x^3*y*u+4895*x^2*y^2*u-F(55)/49*x*y^3*u-10799*y^4*u-F(36)/17*x^3*z*u-F(5)/79*x^2*y*z*u+3775*x*y^2*z*u-6357*y^3*z*u+F(113)/78*x^2*z^2*u-3230*x*y*z^2*u+F(96)/25*y^2*z^2*u+F(43)/87*x*z^3*u-F(31)/65*y*z^3*u+F(29)/81*x^3*u^2+F(63)/44*x^2*y*u^2-7090*x*y^2*u^2+F(77)/90*y^3*u^2+14839*x^2*z*u^2-4283*x*y*z*u^2+6130*y^2*z*u^2-13868*x*z^2*u^2+F(88)/43*y*z^2*u^2-F(85)/19*x^2*u^3+F(121)/13*x*y*u^3+F(29)/95*y^2*u^3-F(21)/44*x*z*u^3-2930*y*z*u^3-F(109)/41*x*u^4-F(28)/15*y*u^4+F(17)/76*x^4*v+F(20)/83*x^3*y*v-F(23)/7*x^2*y^2*v-11233*x*y^3*v-10771*y^4*v-6510*x^3*z*v-9730*x^2*y*z*v+3856*x*y^2*z*v-F(107)/51*y^3*z*v-11908*x^2*z^2*v+F(47)/104*x*y*z^2*v+F(20)/119*y^2*z^2*v+3527*x*z^3*v-13232*y*z^3*v+F(17)/114*x^3*u*v+F(121)/125*x^2*y*u*v-12770*x*y^2*u*v-F(76)/47*y^3*u*v+F(64)/93*x^2*z*u*v+F(77)/117*x*y*z*u*v-F(119)/99*y^2*z*u*v+2149*x*z^2*u*v+F(104)/59*y*z^2*u*v+234*x^2*u^2*v+F(5)/93*x*y*u^2*v-F(67)/110*y^2*u^2*v+14794*x*z*u^2*v+3872*y*z*u^2*v-F(111)/20*x*u^3*v-F(30)/11*y*u^3*v-F(23)/51*x^3*v^2-5308*x^2*y*v^2+2471*x*y^2*v^2-F(35)/41*y^3*v^2-F(29)/62*x^2*z*v^2-F(94)/19*x*y*z*v^2+F(53)/118*y^2*z*v^2+F(86)/125*x*z^2*v^2-F(107)/35*y*z^2*v^2+F(123)/19*x^2*u*v^2-F(34)/83*x*y*u*v^2+F(75)/2*y^2*u*v^2+13143*x*z*u*v^2-F(55)/91*y*z*u*v^2+F(1)/117*x*u^2*v^2+F(73)/25*y*u^2*v^2-F(112)/47*x^2*v^3+F(94)/55*x*y*v^3+F(71)/106*y^2*v^3-10229*x*z*v^3+11735*y*z*v^3+F(62)/47*x*u*v^3-F(82)/95*y*u*v^3+F(118)/81*x*v^4-F(61)/70*y*v^4,-F(125)/3*x^4-F(118)/115*x^3*y-F(11)/67*x^2*y^2+F(91)/73*x*y^3-F(110)/87*y^4+1398*x^3*z+13653*x^2*y*z+F(16)/117*x*y^2*z+F(121)/4*y^3*z+F(124)/9*x^2*z^2+9171*x*y*z^2-14940*y^2*z^2+F(124)/69*x*z^3+F(74)/65*y*z^3-F(105)/68*z^4+F(103)/109*x^3*u+F(34)/45*x^2*y*u+F(99)/53*x*y^2*u+F(19)/66*y^3*u-F(121)/80*x^2*z*u+F(2)/95*x*y*z*u-F(79)/62*y^2*z*u+1924*x*z^2*u+6993*y*z^2*u-F(50)/13*z^3*u+F(103)/61*x^2*u^2-12842*x*y*u^2+15120*y^2*u^2-F(103)/101*x*z*u^2+15344*y*z*u^2-F(88)/21*z^2*u^2-F(14)/99*x*u^3+12698*y*u^3+F(50)/19*z*u^3-F(60)/47*u^4-14513*x^3*v-F(59)/55*x^2*y*v+F(28)/67*x*y^2*v+F(99)/26*y^3*v+F(25)/19*x^2*z*v+8484*x*y*z*v+899*y^2*z*v-15273*x*z^2*v-6310*y*z^2*v+15672*z^3*v+982*x^2*u*v-F(64)/69*x*y*u*v-F(117)/10*y^2*u*v+13638*x*z*u*v-F(103)/72*y*z*u*v+F(82)/77*z^2*u*v-F(12)/121*x*u^2*v+F(87)/19*y*u^2*v+3503*z*u^2*v+430*u^3*v+12828*x^2*v^2+1635*x*y*v^2-F(69)/62*y^2*v^2-116*x*z*v^2+F(11)/120*y*z*v^2-F(11)/65*z^2*v^2-10187*x*u*v^2+F(105)/88*y*u*v^2+F(65)/68*z*u*v^2+2255*u^2*v^2+F(103)/113*x*v^3+F(53)/6*y*v^3+8791*z*v^3-F(100)/57*u*v^3+F(111)/68*v^4]))
end


@doc raw"""
    elliptic_d12_pi13()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `12` and sectional genus `13`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d12_pi13()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(125)/108*x^4*y^2-12139*x^3*y^3-13772*x^2*y^4-6791*x*y^5-15438*y^6+F(119)/94*x^4*y*z+F(31)/85*x^3*y^2*z-F(47)/121*x^2*y^3*z-F(122)/103*x*y^4*z-F(77)/75*y^5*z+13042*x^4*z^2-F(17)/7*x^3*y*z^2-F(9)/38*x^2*y^2*z^2-F(116)/97*x*y^3*z^2+F(108)/29*y^4*z^2-3812*x^3*z^3-11723*x^2*y*z^3+F(33)/64*x*y^2*z^3-F(31)/27*y^3*z^3+F(120)/23*x^2*z^4-15250*x*y*z^4-F(26)/121*y^2*z^4+F(30)/53*x*z^5-1751*y*z^5-3260*z^6-12042*x^4*y*u-F(19)/35*x^3*y^2*u-F(79)/94*x^2*y^3*u-6780*x*y^4*u+9472*y^5*u+F(115)/53*x^4*z*u-3103*x^3*y*z*u-F(13)/33*x^2*y^2*z*u-15590*x*y^3*z*u-F(103)/126*y^4*z*u+5212*x^3*z^2*u+F(36)/73*x^2*y*z^2*u+12342*x*y^2*z^2*u-F(108)/65*y^3*z^2*u-7281*x^2*z^3*u+F(48)/101*x*y*z^3*u+F(29)/41*y^2*z^3*u-11951*x*z^4*u-F(39)/50*y*z^4*u+66*z^5*u+9824*x^4*u^2-5623*x^3*y*u^2-F(18)/53*x^2*y^2*u^2+12729*x*y^3*u^2+2652*y^4*u^2-F(64)/49*x^3*z*u^2+F(38)/31*x^2*y*z*u^2+14724*x*y^2*z*u^2-F(15)/124*y^3*z*u^2-F(95)/97*x^2*z^2*u^2+F(65)/24*x*y*z^2*u^2-F(51)/121*y^2*z^2*u^2-15768*x*z^3*u^2-F(11)/53*y*z^3*u^2+F(31)/112*z^4*u^2-F(71)/34*x^3*u^3+13770*x^2*y*u^3+F(96)/17*x*y^2*u^3-F(1)/65*y^3*u^3+14572*x^2*z*u^3-F(19)/48*x*y*z*u^3+F(105)/104*y^2*z*u^3+F(47)/59*x*z^2*u^3+F(27)/35*y*z^2*u^3+9801*z^3*u^3+F(17)/124*x^2*u^4-F(122)/89*x*y*u^4-F(24)/55*y^2*u^4-F(35)/101*x*z*u^4-F(29)/50*y*z*u^4+945*z^2*u^4+F(67)/28*x*u^5+4908*y*u^5+448*z*u^5-F(49)/45*u^6+F(107)/2*x^4*y*v-309*x^3*y^2*v+F(51)/65*x^2*y^3*v+15437*x*y^4*v-F(66)/97*y^5*v+F(22)/59*x^4*z*v-F(49)/107*x^3*y*z*v+3220*x^2*y^2*z*v-F(79)/10*x*y^3*z*v-15727*y^4*z*v+F(81)/47*x^3*z^2*v-F(48)/109*x^2*y*z^2*v+9645*x*y^2*z^2*v-12561*y^3*z^2*v+F(91)/120*x^2*z^3*v+9906*x*y*z^3*v-F(24)/121*y^2*z^3*v+8621*x*z^4*v-15786*y*z^4*v-F(85)/82*z^5*v+F(105)/34*x^4*u*v+F(125)/114*x^3*y*u*v-1168*x^2*y^2*u*v+F(1)/83*x*y^3*u*v-F(39)/107*y^4*u*v+F(73)/112*x^3*z*u*v+1445*x^2*y*z*u*v-F(118)/93*x*y^2*z*u*v-F(43)/101*y^3*z*u*v-623*x^2*z^2*u*v-F(109)/110*x*y*z^2*u*v-F(43)/69*y^2*z^2*u*v+2956*x*z^3*u*v-5708*y*z^3*u*v-F(77)/69*z^4*u*v+3931*x^3*u^2*v+F(107)/52*x^2*y*u^2*v+1374*x*y^2*u^2*v+9236*y^3*u^2*v+1763*x^2*z*u^2*v-F(111)/68*x*y*z*u^2*v+15822*y^2*z*u^2*v+F(37)/94*x*z^2*u^2*v+F(73)/108*y*z^2*u^2*v+F(125)/44*z^3*u^2*v-F(9)/17*x^2*u^3*v+F(3)/109*x*y*u^3*v+3535*y^2*u^3*v+8898*x*z*u^3*v-F(7)/92*y*z*u^3*v+9170*z^2*u^3*v-F(107)/90*x*u^4*v+9487*y*u^4*v+F(9)/49*z*u^4*v-522*u^5*v-F(67)/77*x^4*v^2-178*x^3*y*v^2+F(16)/113*x^2*y^2*v^2-904*x*y^3*v^2-F(61)/54*y^4*v^2+F(1)/99*x^3*z*v^2+5555*x^2*y*z*v^2+F(126)/53*x*y^2*z*v^2+F(33)/82*y^3*z*v^2+F(59)/84*x^2*z^2*v^2-5455*x*y*z^2*v^2-F(71)/102*y^2*z^2*v^2-F(4)/11*x*z^3*v^2+F(41)/103*y*z^3*v^2+F(81)/34*z^4*v^2-12975*x^3*u*v^2+F(9)/65*x^2*y*u*v^2-13762*x*y^2*u*v^2-10486*y^3*u*v^2-9028*x^2*z*u*v^2+F(108)/13*x*y*z*u*v^2+9166*y^2*z*u*v^2+7946*x*z^2*u*v^2-15721*y*z^2*u*v^2-3214*z^3*u*v^2+9634*x^2*u^2*v^2-F(61)/117*x*y*u^2*v^2-F(103)/35*y^2*u^2*v^2-F(112)/67*x*z*u^2*v^2-F(70)/59*y*z*u^2*v^2+F(41)/76*z^2*u^2*v^2-F(85)/36*x*u^3*v^2-11798*y*u^3*v^2+F(37)/3*z*u^3*v^2-F(13)/42*u^4*v^2+F(59)/102*x^3*v^3+F(116)/117*x^2*y*v^3-F(23)/88*x*y^2*v^3+F(110)/21*y^3*v^3+F(101)/110*x^2*z*v^3+F(57)/101*x*y*z*v^3-F(111)/109*y^2*z*v^3-7035*x*z^2*v^3-F(58)/3*y*z^2*v^3+F(23)/102*z^3*v^3+10722*x^2*u*v^3+F(28)/121*x*y*u*v^3+F(104)/41*y^2*u*v^3-F(59)/64*x*z*u*v^3+270*y*z*u*v^3+F(66)/71*z^2*u*v^3+F(25)/87*x*u^2*v^3-F(123)/86*y*u^2*v^3+F(115)/112*z*u^2*v^3+F(93)/61*u^3*v^3+5302*x^2*v^4-F(63)/95*x*y*v^4-11037*y^2*v^4-F(73)/23*x*z*v^4-F(61)/97*y*z*v^4-F(77)/86*z^2*v^4-15553*x*u*v^4+F(38)/21*y*u*v^4-F(59)/39*z*u*v^4-8588*u^2*v^4-2235*x*v^5-F(3)/124*y*v^5+7692*z*v^5+F(105)/26*u*v^5+F(76)/69*v^6,-4204*x^4*y^2+F(31)/70*x^3*y^3-F(23)/33*x^2*y^4-F(59)/75*x*y^5+F(94)/41*y^6-6905*x^4*y*z+F(19)/75*x^3*y^2*z+F(79)/108*x^2*y^3*z-495*x*y^4*z+F(15)/17*y^5*z+F(1)/65*x^4*z^2-4842*x^3*y*z^2+F(85)/113*x^2*y^2*z^2+F(107)/118*x*y^3*z^2-F(125)/31*y^4*z^2-748*x^3*z^3+F(122)/71*x^2*y*z^3+F(1)/3*x*y^2*z^3-13840*y^3*z^3+F(101)/119*x^2*z^4+F(45)/82*x*y*z^4+F(6)/7*y^2*z^4+10422*x*z^5-10371*y*z^5+F(14)/115*z^6+11618*x^4*y*u-F(53)/47*x^3*y^2*u+F(90)/97*x^2*y^3*u-F(113)/69*x*y^4*u-1419*y^5*u+F(71)/17*x^4*z*u+12172*x^3*y*z*u-F(23)/77*x^2*y^2*z*u-F(89)/40*x*y^3*z*u+F(47)/38*y^4*z*u+F(4)/77*x^3*z^2*u-5404*x^2*y*z^2*u+F(87)/65*x*y^2*z^2*u+F(25)/52*y^3*z^2*u+F(121)/47*x^2*z^3*u-5540*x*y*z^3*u+F(14)/71*y^2*z^3*u-15001*x*z^4*u+F(54)/31*y*z^4*u+F(75)/76*z^5*u+F(82)/5*x^4*u^2+F(67)/18*x^3*y*u^2-7772*x^2*y^2*u^2+4748*x*y^3*u^2-F(19)/5*y^4*u^2-F(96)/121*x^3*z*u^2+F(94)/109*x^2*y*z*u^2-F(66)/53*x*y^2*z*u^2+F(113)/114*y^3*z*u^2+F(113)/23*x^2*z^2*u^2+15224*x*y*z^2*u^2-F(67)/10*y^2*z^2*u^2-6354*x*z^3*u^2+F(81)/92*y*z^3*u^2+F(100)/31*z^4*u^2-F(21)/94*x^3*u^3-F(103)/77*x^2*y*u^3-12226*x*y^2*u^3+9937*y^3*u^3-2406*x^2*z*u^3+9360*x*y*z*u^3+13686*y^2*z*u^3+F(13)/54*x*z^2*u^3+F(75)/2*y*z^2*u^3-14739*z^3*u^3-13868*x^2*u^4-F(63)/89*x*y*u^4+F(109)/96*y^2*u^4+F(120)/91*x*z*u^4-F(74)/79*y*z*u^4-F(97)/47*z^2*u^4+15909*x*u^5-F(61)/87*y*u^5-F(109)/76*z*u^5+6138*u^6-13163*x^4*y*v-12888*x^3*y^2*v-12343*x^2*y^3*v-F(53)/73*x*y^4*v-F(121)/4*y^5*v-15890*x^4*z*v-F(37)/64*x^3*y*z*v+3249*x^2*y^2*z*v+5368*x*y^3*z*v+F(126)/125*y^4*z*v+F(29)/20*x^3*z^2*v+10928*x^2*y*z^2*v-F(29)/114*x*y^2*z^2*v-F(28)/73*y^3*z^2*v+545*x^2*z^3*v-F(123)/43*x*y*z^3*v+F(101)/109*y^2*z^3*v+F(75)/43*x*z^4*v+6237*y*z^4*v-9638*z^5*v-10841*x^4*u*v-F(7)/71*x^3*y*u*v-2920*x^2*y^2*u*v+F(41)/53*x*y^3*u*v-6871*y^4*u*v+F(61)/57*x^3*z*u*v-8658*x^2*y*z*u*v-11221*x*y^2*z*u*v-F(80)/43*y^3*z*u*v+F(99)/13*x^2*z^2*u*v-F(116)/15*x*y*z^2*u*v-F(121)/111*y^2*z^2*u*v+F(89)/26*x*z^3*u*v+F(20)/9*y*z^3*u*v+F(18)/89*z^4*u*v+8660*x^3*u^2*v-F(76)/29*x^2*y*u^2*v+F(94)/53*x*y^2*u^2*v-6510*y^3*u^2*v-F(107)/86*x^2*z*u^2*v+F(91)/75*x*y*z*u^2*v-F(58)/73*y^2*z*u^2*v+909*x*z^2*u^2*v+F(107)/3*y*z^2*u^2*v-F(119)/78*z^3*u^2*v+F(11)/124*x^2*u^3*v+F(79)/111*x*y*u^3*v-F(72)/55*y^2*u^3*v+F(125)/84*x*z*u^3*v+2421*y*z*u^3*v-F(8)/15*z^2*u^3*v-F(125)/49*x*u^4*v+12089*y*u^4*v+F(13)/81*z*u^4*v+F(1)/2*u^5*v+4420*x^4*v^2+67*x^3*y*v^2+7938*x^2*y^2*v^2-F(81)/17*x*y^3*v^2-F(124)/35*y^4*v^2-F(5)/48*x^3*z*v^2+F(21)/110*x^2*y*z*v^2-F(117)/40*x*y^2*z*v^2-F(101)/70*y^3*z*v^2+1630*x^2*z^2*v^2-F(61)/75*x*y*z^2*v^2-6627*y^2*z^2*v^2-F(87)/113*x*z^3*v^2+F(103)/36*y*z^3*v^2-F(67)/104*z^4*v^2+F(110)/41*x^3*u*v^2-4590*x^2*y*u*v^2+4791*x*y^2*u*v^2-F(11)/104*y^3*u*v^2+11769*x^2*z*u*v^2+F(62)/125*x*y*z*u*v^2+F(88)/51*y^2*z*u*v^2-F(83)/54*x*z^2*u*v^2+F(102)/11*y*z^2*u*v^2-5139*z^3*u*v^2+F(22)/9*x^2*u^2*v^2+1630*x*y*u^2*v^2-F(55)/63*y^2*u^2*v^2-F(105)/118*x*z*u^2*v^2-F(64)/73*y*z*u^2*v^2+F(98)/87*z^2*u^2*v^2+F(113)/3*x*u^3*v^2+F(83)/27*y*u^3*v^2+F(17)/93*z*u^3*v^2-3452*u^4*v^2+F(93)/101*x^3*v^3+F(55)/117*x^2*y*v^3-F(2)/5*x*y^2*v^3+F(51)/107*y^3*v^3-F(2)/95*x^2*z*v^3+15438*x*y*z*v^3+F(76)/43*y^2*z*v^3+6728*x*z^2*v^3-2568*y*z^2*v^3+10315*z^3*v^3+F(17)/8*x^2*u*v^3-F(55)/6*x*y*u*v^3+F(21)/4*y^2*u*v^3+F(53)/50*x*z*u*v^3+14198*y*z*u*v^3-F(118)/123*z^2*u*v^3+4934*x*u^2*v^3+F(53)/108*y*u^2*v^3+13547*z*u^2*v^3-15186*u^3*v^3+F(13)/61*x^2*v^4-7060*x*y*v^4-8863*y^2*v^4+F(29)/30*x*z*v^4+F(39)/34*y*z*v^4+F(75)/19*z^2*v^4-F(121)/23*x*u*v^4-F(85)/44*y*u*v^4-11748*z*u*v^4+F(88)/123*u^2*v^4-11333*x*v^5+F(7)/59*y*v^5-7963*z*v^5+F(56)/59*u*v^5+11084*v^6,F(82)/33*x^4*y^2-F(61)/113*x^3*y^3-13286*x^2*y^4+15628*x*y^5+F(49)/85*y^6-10229*x^4*y*z+6712*x^3*y^2*z+F(122)/93*x^2*y^3*z-11922*x*y^4*z-F(103)/16*y^5*z-F(6)/29*x^4*z^2-F(70)/123*x^3*y*z^2+9163*x^2*y^2*z^2-F(52)/109*x*y^3*z^2+11878*y^4*z^2-13886*x^3*z^3+F(124)/67*x^2*y*z^3-7179*x*y^2*z^3-F(1)/105*y^3*z^3-2789*x^2*z^4+F(70)/41*x*y*z^4+F(52)/97*y^2*z^4+F(49)/53*x*z^5-F(1)/7*y*z^5-F(26)/3*z^6+13741*x^4*y*u+F(118)/31*x^3*y^2*u+F(121)/65*x^2*y^3*u-F(113)/55*x*y^4*u+F(112)/45*y^5*u-F(43)/22*x^4*z*u-F(93)/70*x^3*y*z*u-9524*x^2*y^2*z*u+9958*x*y^3*z*u+F(100)/7*y^4*z*u+F(82)/125*x^3*z^2*u+F(118)/67*x^2*y*z^2*u-9121*x*y^2*z^2*u+F(10)/33*y^3*z^2*u-F(23)/88*x^2*z^3*u-F(83)/2*x*y*z^3*u-F(3)/116*y^2*z^3*u-F(65)/92*x*z^4*u+F(90)/119*y*z^4*u+3706*z^5*u+F(43)/31*x^4*u^2+15519*x^3*y*u^2+F(85)/97*x^2*y^2*u^2-13840*x*y^3*u^2-101*y^4*u^2+F(98)/115*x^3*z*u^2+F(95)/6*x^2*y*z*u^2+F(83)/64*x*y^2*z*u^2+F(54)/5*y^3*z*u^2+F(23)/18*x^2*z^2*u^2+F(118)/27*x*y*z^2*u^2-F(23)/30*y^2*z^2*u^2-12760*x*z^3*u^2-F(61)/87*y*z^3*u^2-F(52)/55*z^4*u^2-F(106)/87*x^3*u^3+F(69)/25*x^2*y*u^3+F(11)/24*x*y^2*u^3+F(7)/78*y^3*u^3-F(44)/75*x^2*z*u^3-F(89)/2*x*y*z*u^3-F(2)/15*y^2*z*u^3+9831*x*z^2*u^3-50*y*z^2*u^3-14297*z^3*u^3-3453*x^2*u^4+5146*x*y*u^4+6350*y^2*u^4+10848*x*z*u^4+5298*y*z*u^4-10949*z^2*u^4-F(101)/109*x*u^5-F(20)/9*y*u^5-F(29)/66*z*u^5+F(74)/85*u^6+2335*x^4*y*v+1425*x^3*y^2*v+F(84)/13*x^2*y^3*v-7028*x*y^4*v+F(59)/126*y^5*v-9622*x^4*z*v+F(83)/97*x^3*y*z*v-9717*x^2*y^2*z*v-14483*x*y^3*z*v+13854*y^4*z*v-F(47)/29*x^3*z^2*v+10207*x^2*y*z^2*v+F(76)/83*x*y^2*z^2*v+8853*y^3*z^2*v-F(104)/93*x^2*z^3*v-12128*x*y*z^3*v+F(80)/101*y^2*z^3*v+12129*x*z^4*v-F(19)/96*y*z^4*v-F(41)/126*z^5*v+9062*x^4*u*v+F(6)/101*x^3*y*u*v-F(38)/3*x^2*y^2*u*v+8088*x*y^3*u*v+F(46)/39*y^4*u*v-F(41)/21*x^3*z*u*v-F(17)/42*x^2*y*z*u*v-3652*x*y^2*z*u*v+10824*y^3*z*u*v-133*x^2*z^2*u*v-6445*x*y*z^2*u*v+2737*y^2*z^2*u*v+11259*x*z^3*u*v+15375*y*z^3*u*v-F(106)/123*z^4*u*v+1374*x^3*u^2*v-8037*x^2*y*u^2*v-11527*x*y^2*u^2*v+F(83)/63*y^3*u^2*v+F(44)/15*x^2*z*u^2*v+F(37)/17*x*y*z*u^2*v+8831*y^2*z*u^2*v-7086*x*z^2*u^2*v-F(101)/114*y*z^2*u^2*v-F(100)/89*z^3*u^2*v+F(101)/79*x^2*u^3*v-F(113)/43*x*y*u^3*v+F(109)/84*y^2*u^3*v-1062*x*z*u^3*v+F(22)/119*y*z*u^3*v+F(87)/46*z^2*u^3*v+12168*x*u^4*v-14978*y*u^4*v+13165*z*u^4*v-13652*u^5*v-F(7)/52*x^4*v^2+F(113)/74*x^3*y*v^2+14193*x^2*y^2*v^2+F(24)/55*x*y^3*v^2+F(94)/31*y^4*v^2-F(19)/75*x^3*z*v^2-14456*x^2*y*z*v^2-F(94)/41*x*y^2*z*v^2-1158*y^3*z*v^2+F(79)/83*x^2*z^2*v^2+F(117)/19*x*y*z^2*v^2-F(67)/104*y^2*z^2*v^2+F(114)/37*x*z^3*v^2+14370*y*z^3*v^2+F(43)/50*z^4*v^2+F(95)/103*x^3*u*v^2-2448*x^2*y*u*v^2+F(17)/59*x*y^2*u*v^2+F(109)/72*y^3*u*v^2-714*x^2*z*u*v^2-419*x*y*z*u*v^2+F(115)/28*y^2*z*u*v^2-F(124)/47*x*z^2*u*v^2-F(38)/109*y*z^2*u*v^2+3532*z^3*u*v^2+5987*x^2*u^2*v^2+F(61)/33*x*y*u^2*v^2+F(103)/27*y^2*u^2*v^2+2117*x*z*u^2*v^2+15686*y*z*u^2*v^2+8546*z^2*u^2*v^2+F(25)/124*x*u^3*v^2-7345*y*u^3*v^2+F(116)/81*z*u^3*v^2+F(13)/53*u^4*v^2+F(61)/116*x^3*v^3+4217*x^2*y*v^3+F(113)/110*x*y^2*v^3+F(107)/112*y^3*v^3-3218*x^2*z*v^3-3574*x*y*z*v^3+12256*y^2*z*v^3+F(118)/77*x*z^2*v^3+F(52)/113*y*z^2*v^3+F(59)/35*z^3*v^3-5617*x^2*u*v^3+2930*x*y*u*v^3-F(47)/72*y^2*u*v^3+F(90)/43*x*z*u*v^3-7687*y*z*u*v^3+13184*z^2*u*v^3+F(62)/71*x*u^2*v^3-4460*y*u^2*v^3+F(19)/4*z*u^2*v^3-4671*u^3*v^3-F(49)/75*x^2*v^4+F(102)/53*x*y*v^4+4940*y^2*v^4+5932*x*z*v^4+F(91)/45*y*z*v^4+15336*z^2*v^4+F(110)/113*x*u*v^4-F(18)/119*y*u*v^4-7177*z*u*v^4+8216*u^2*v^4-F(74)/7*x*v^5-4778*y*v^5-F(50)/103*z*v^5+F(66)/53*u*v^5-F(58)/69*v^6,-14193*x^4*y^2+F(96)/7*x^3*y^3+F(53)/20*x^2*y^4-14520*x*y^5+F(123)/43*y^6-F(51)/124*x^4*y*z-F(92)/43*x^3*y^2*z+F(85)/39*x^2*y^3*z-F(15)/26*x*y^4*z-F(61)/75*y^5*z+F(56)/65*x^4*z^2+F(41)/107*x^3*y*z^2+10426*x^2*y^2*z^2-F(67)/111*x*y^3*z^2+F(91)/32*y^4*z^2+6427*x^3*z^3+F(34)/125*x^2*y*z^3+F(1)/114*x*y^2*z^3+F(85)/113*y^3*z^3-F(37)/64*x^2*z^4-3420*x*y*z^4-13426*y^2*z^4+5873*x*z^5+4304*y*z^5+F(113)/16*z^6+F(65)/57*x^4*y*u+F(113)/82*x^3*y^2*u-F(72)/91*x^2*y^3*u-4023*x*y^4*u+F(5)/24*y^5*u+1932*x^4*z*u-1236*x^3*y*z*u-F(41)/115*x^2*y^2*z*u+F(92)/43*x*y^3*z*u-F(119)/73*y^4*z*u-8965*x^3*z^2*u-F(57)/4*x^2*y*z^2*u-F(12)/95*x*y^2*z^2*u+15364*y^3*z^2*u+6178*x^2*z^3*u+F(108)/13*x*y*z^3*u+F(109)/39*y^2*z^3*u+2895*x*z^4*u-11145*y*z^4*u+10582*z^5*u-F(116)/55*x^4*u^2-F(122)/39*x^3*y*u^2-F(48)/89*x^2*y^2*u^2+F(47)/95*x*y^3*u^2-12757*y^4*u^2-F(124)/63*x^3*z*u^2-7869*x^2*y*z*u^2-4138*x*y^2*z*u^2-F(19)/88*y^3*z*u^2-F(93)/101*x^2*z^2*u^2+F(37)/16*x*y*z^2*u^2-8071*y^2*z^2*u^2+5154*x*z^3*u^2+12499*y*z^3*u^2+14275*z^4*u^2+F(32)/125*x^3*u^3-F(122)/45*x^2*y*u^3+7192*x*y^2*u^3-F(97)/7*y^3*u^3-F(5)/93*x^2*z*u^3-2979*x*y*z*u^3-15752*y^2*z*u^3+F(65)/124*x*z^2*u^3+5084*y*z^2*u^3+4911*z^3*u^3-1493*x^2*u^4-836*x*y*u^4+F(89)/112*y^2*u^4-8172*x*z*u^4+F(76)/67*y*z*u^4-F(117)/94*z^2*u^4-F(60)/7*x*u^5+F(125)/57*y*u^5+F(45)/52*z*u^5-F(56)/71*u^6-F(13)/88*x^4*y*v+F(51)/67*x^3*y^2*v+F(103)/56*x^2*y^3*v-F(52)/71*x*y^4*v+3897*y^5*v-F(97)/93*x^4*z*v+F(9)/23*x^3*y*z*v+F(13)/118*x^2*y^2*z*v-6821*x*y^3*z*v+F(125)/68*y^4*z*v-12598*x^3*z^2*v+F(65)/111*x^2*y*z^2*v+607*x*y^2*z^2*v-F(36)/17*y^3*z^2*v+4088*x^2*z^3*v-10774*x*y*z^3*v+7536*y^2*z^3*v+5655*x*z^4*v-F(2)/21*y*z^4*v+13275*z^5*v+7952*x^4*u*v-6748*x^3*y*u*v+F(97)/5*x^2*y^2*u*v+F(123)/50*x*y^3*u*v+11378*y^4*u*v+F(43)/3*x^3*z*u*v+6533*x^2*y*z*u*v+8033*x*y^2*z*u*v-F(43)/40*y^3*z*u*v-F(3)/67*x^2*z^2*u*v-11580*x*y*z^2*u*v-2403*y^2*z^2*u*v+F(37)/116*x*z^3*u*v+1460*y*z^3*u*v-220*z^4*u*v-F(11)/87*x^3*u^2*v+F(94)/75*x^2*y*u^2*v+F(29)/117*x*y^2*u^2*v-F(119)/118*y^3*u^2*v+5386*x^2*z*u^2*v+F(95)/101*x*y*z*u^2*v+F(119)/62*y^2*z*u^2*v-F(65)/59*x*z^2*u^2*v+12261*y*z^2*u^2*v+F(42)/19*z^3*u^2*v+F(22)/39*x^2*u^3*v+11652*x*y*u^3*v-5987*y^2*u^3*v+459*x*z*u^3*v+15920*y*z*u^3*v-8314*z^2*u^3*v-11134*x*u^4*v-F(124)/49*y*u^4*v+6993*z*u^4*v-F(35)/58*u^5*v-F(43)/26*x^4*v^2-F(24)/101*x^3*y*v^2+10437*x^2*y^2*v^2-F(107)/98*x*y^3*v^2+10963*y^4*v^2-F(105)/121*x^3*z*v^2+F(43)/68*x^2*y*z*v^2+8605*x*y^2*z*v^2+F(94)/99*y^3*z*v^2-F(4)/125*x^2*z^2*v^2+F(65)/4*x*y*z^2*v^2-12458*y^2*z^2*v^2-F(67)/98*x*z^3*v^2+F(58)/67*y*z^3*v^2+12535*z^4*v^2-F(10)/17*x^3*u*v^2+F(125)/38*x^2*y*u*v^2-9338*x*y^2*u*v^2-F(81)/4*y^3*u*v^2+F(123)/23*x^2*z*u*v^2-1954*x*y*z*u*v^2+F(103)/102*y^2*z*u*v^2-5794*x*z^2*u*v^2+F(64)/49*y*z^2*u*v^2-F(31)/47*z^3*u*v^2+F(65)/68*x^2*u^2*v^2-9526*x*y*u^2*v^2+F(2)/95*y^2*u^2*v^2+F(107)/36*x*z*u^2*v^2+F(79)/97*y*z*u^2*v^2-F(95)/91*z^2*u^2*v^2+F(101)/30*x*u^3*v^2+F(45)/17*y*u^3*v^2-38*z*u^3*v^2+F(17)/32*u^4*v^2-F(75)/8*x^3*v^3+519*x^2*y*v^3-4550*x*y^2*v^3+4447*y^3*v^3+14609*x^2*z*v^3-15223*x*y*z*v^3-F(27)/47*y^2*z*v^3+F(7)/101*x*z^2*v^3-8519*y*z^2*v^3+148*z^3*v^3-F(23)/61*x^2*u*v^3+F(41)/89*x*y*u*v^3+F(97)/121*y^2*u*v^3-F(3)/100*x*z*u*v^3-F(37)/107*y*z*u*v^3-F(112)/19*z^2*u*v^3+9163*x*u^2*v^3+F(119)/80*y*u^2*v^3+F(88)/13*z*u^2*v^3-F(28)/95*u^3*v^3-9716*x^2*v^4+F(74)/117*x*y*v^4+12762*y^2*v^4-F(103)/85*x*z*v^4+F(1)/69*y*z*v^4+F(116)/13*z^2*v^4+6138*x*u*v^4-F(9)/4*y*u*v^4+F(113)/32*z*u*v^4+14559*u^2*v^4+F(29)/89*x*v^5-4294*y*v^5+2890*z*v^5-F(96)/29*u*v^5-F(47)/95*v^6,F(125)/92*x^4*y^2-F(16)/11*x^3*y^3-28*x^2*y^4+F(101)/16*x*y^5+F(5)/39*y^6-6077*x^4*y*z-F(9)/100*x^3*y^2*z+1945*x^2*y^3*z-F(16)/93*x*y^4*z+F(103)/25*y^5*z-5918*x^4*z^2-6352*x^3*y*z^2+F(79)/117*x^2*y^2*z^2-15582*x*y^3*z^2+10291*y^4*z^2+F(58)/37*x^3*z^3+F(97)/91*x^2*y*z^3+F(1)/34*x*y^2*z^3+F(71)/52*y^3*z^3+12763*x^2*z^4+F(33)/59*x*y*z^4-F(67)/8*y^2*z^4-F(108)/115*x*z^5-15617*y*z^5+F(103)/11*z^6-8092*x^4*y*u-7306*x^3*y^2*u+5442*x^2*y^3*u-F(25)/76*x*y^4*u-F(7)/16*y^5*u-4686*x^4*z*u-F(115)/81*x^3*y*z*u-F(46)/101*x^2*y^2*z*u+F(125)/41*x*y^3*z*u+5572*y^4*z*u-15459*x^3*z^2*u+F(73)/30*x^2*y*z^2*u-9460*x*y^2*z^2*u-860*y^3*z^2*u-3487*x^2*z^3*u+11887*x*y*z^3*u-F(79)/105*y^2*z^3*u+5416*x*z^4*u+F(24)/91*y*z^4*u+F(4)/19*z^5*u-F(66)/101*x^4*u^2-9634*x^3*y*u^2-F(5)/47*x^2*y^2*u^2+7814*x*y^3*u^2-F(97)/86*y^4*u^2+F(53)/16*x^3*z*u^2-F(4)/119*x^2*y*z*u^2+F(26)/125*x*y^2*z*u^2+F(100)/67*y^3*z*u^2+4590*x^2*z^2*u^2-107*x*y*z^2*u^2+F(123)/106*y^2*z^2*u^2+F(108)/67*x*z^3*u^2+12938*y*z^3*u^2-F(59)/38*z^4*u^2+315*x^3*u^3+F(117)/10*x^2*y*u^3-F(14)/125*x*y^2*u^3+5542*y^3*u^3+F(101)/40*x^2*z*u^3-9506*x*y*z*u^3+F(25)/84*y^2*z*u^3+F(84)/61*x*z^2*u^3-F(9)/43*y*z^2*u^3-F(100)/47*z^3*u^3-9676*x^2*u^4-921*x*y*u^4+1935*y^2*u^4+F(17)/89*x*z*u^4+6161*y*z*u^4-F(19)/99*z^2*u^4+F(102)/61*x*u^5+F(25)/28*y*u^5-F(73)/65*z*u^5+F(115)/93*u^6+14433*x^4*y*v+F(29)/113*x^3*y^2*v+F(29)/101*x^2*y^3*v+4019*x*y^4*v-F(35)/47*y^5*v+F(117)/34*x^4*z*v-F(97)/105*x^3*y*z*v+F(45)/11*x^2*y^2*z*v-F(76)/45*x*y^3*z*v+1369*y^4*z*v+F(87)/70*x^3*z^2*v-3805*x^2*y*z^2*v+F(33)/34*x*y^2*z^2*v-F(105)/73*y^3*z^2*v-204*x^2*z^3*v+F(93)/13*x*y*z^3*v-F(14)/75*y^2*z^3*v+F(103)/39*x*z^4*v-14252*y*z^4*v+15205*z^5*v-F(113)/14*x^4*u*v-4342*x^3*y*u*v-2750*x^2*y^2*u*v-15610*x*y^3*u*v+F(23)/72*y^4*u*v-F(83)/48*x^3*z*u*v-F(47)/49*x^2*y*z*u*v-F(32)/109*x*y^2*z*u*v-F(65)/14*y^3*z*u*v-F(45)/68*x^2*z^2*u*v+F(7)/75*x*y*z^2*u*v+10443*y^2*z^2*u*v+12712*x*z^3*u*v+F(58)/61*y*z^3*u*v-3497*z^4*u*v-F(62)/33*x^3*u^2*v-F(85)/56*x^2*y*u^2*v-3177*x*y^2*u^2*v+F(112)/85*y^3*u^2*v-F(67)/87*x^2*z*u^2*v-F(63)/71*x*y*z*u^2*v-6868*y^2*z*u^2*v-8948*x*z^2*u^2*v-F(56)/73*y*z^2*u^2*v-F(104)/113*z^3*u^2*v+2215*x^2*u^3*v-F(50)/21*x*y*u^3*v-F(121)/104*y^2*u^3*v+13689*x*z*u^3*v-12091*y*z*u^3*v+F(63)/38*z^2*u^3*v+F(67)/60*x*u^4*v-F(114)/49*y*u^4*v+11842*z*u^4*v+9068*u^5*v+F(22)/93*x^4*v^2+F(87)/74*x^3*y*v^2+F(69)/70*x^2*y^2*v^2+F(79)/94*x*y^3*v^2+F(69)/79*y^4*v^2-F(64)/67*x^3*z*v^2+12973*x^2*y*z*v^2-12943*x*y^2*z*v^2-1891*y^3*z*v^2+F(59)/37*x^2*z^2*v^2-F(14)/3*x*y*z^2*v^2+5501*y^2*z^2*v^2-12266*x*z^3*v^2-10972*y*z^3*v^2-5736*z^4*v^2-F(123)/17*x^3*u*v^2-F(67)/112*x^2*y*u*v^2+14086*x*y^2*u*v^2-13818*y^3*u*v^2+F(121)/124*x^2*z*u*v^2+5132*x*y*z*u*v^2-F(62)/49*y^2*z*u*v^2-78*x*z^2*u*v^2+F(3)/59*y*z^2*u*v^2-F(105)/83*z^3*u*v^2+F(83)/39*x^2*u^2*v^2-12488*x*y*u^2*v^2-8698*y^2*u^2*v^2+F(32)/81*x*z*u^2*v^2-F(71)/43*y*z*u^2*v^2+F(89)/27*z^2*u^2*v^2+13515*x*u^3*v^2-F(79)/8*y*u^3*v^2+F(26)/87*z*u^3*v^2+F(59)/51*u^4*v^2+F(103)/3*x^3*v^3+F(95)/84*x^2*y*v^3-F(57)/86*x*y^2*v^3+F(71)/83*y^3*v^3+10984*x^2*z*v^3+F(125)/39*x*y*z*v^3+F(27)/100*y^2*z*v^3-14813*x*z^2*v^3-F(55)/53*y*z^2*v^3-F(93)/8*z^3*v^3-4640*x^2*u*v^3+F(40)/39*x*y*u*v^3-F(59)/72*y^2*u*v^3-3390*x*z*u*v^3+F(83)/27*y*z*u*v^3+F(74)/7*z^2*u*v^3-F(117)/50*x*u^2*v^3+F(2)/49*y*u^2*v^3+4790*z*u^2*v^3-F(91)/23*u^3*v^3-F(91)/66*x^2*v^4+F(29)/102*x*y*v^4-F(12)/13*y^2*v^4+14793*x*z*v^4-11344*y*z*v^4+7041*z^2*v^4-F(67)/126*x*u*v^4-2201*y*u*v^4+F(21)/95*z*u*v^4+F(86)/79*u^2*v^4+F(117)/8*x*v^5-F(9)/44*y*v^5-5534*z*v^5+F(49)/68*u*v^5-F(59)/102*v^6,F(104)/99*x^4*y^2-F(37)/2*x^3*y^3+8767*x^2*y^4+F(37)/53*x*y^5-F(79)/102*y^6+F(1)/24*x^4*y*z+6742*x^3*y^2*z-14236*x^2*y^3*z-9563*x*y^4*z+F(112)/17*y^5*z+F(18)/113*x^4*z^2+F(31)/11*x^3*y*z^2-4675*x^2*y^2*z^2+F(107)/112*x*y^3*z^2+4039*y^4*z^2-8713*x^3*z^3-F(7)/88*x^2*y*z^3+2922*x*y^2*z^3+F(43)/7*y^3*z^3-13042*x^2*z^4-F(69)/83*x*y*z^4-1382*y^2*z^4-F(29)/92*x*z^5-F(93)/50*y*z^5-F(77)/81*z^6-F(13)/122*x^4*y*u+1786*x^3*y^2*u+F(54)/91*x^2*y^3*u+F(76)/103*x*y^4*u+3487*y^5*u+F(64)/67*x^4*z*u+F(38)/123*x^3*y*z*u-F(59)/35*x^2*y^2*z*u-F(109)/107*x*y^3*z*u+F(1)/24*y^4*z*u+F(55)/17*x^3*z^2*u-4144*x^2*y*z^2*u+F(14)/81*x*y^2*z^2*u-F(86)/97*y^3*z^2*u-F(57)/65*x^2*z^3*u-F(42)/85*x*y*z^3*u-13603*y^2*z^3*u-15517*x*z^4*u-F(77)/24*y*z^4*u-F(32)/27*z^5*u-5231*x^4*u^2-2152*x^3*y*u^2-F(50)/23*x^2*y^2*u^2-8502*x*y^3*u^2+F(35)/32*y^4*u^2+F(97)/11*x^3*z*u^2+1971*x^2*y*z*u^2+13803*x*y^2*z*u^2-F(32)/105*y^3*z*u^2+F(112)/45*x^2*z^2*u^2-2145*x*y*z^2*u^2+104*y^2*z^2*u^2+15452*x*z^3*u^2+F(92)/35*y*z^3*u^2-F(73)/79*z^4*u^2+F(77)/90*x^3*u^3-F(33)/13*x^2*y*u^3+2213*x*y^2*u^3-F(67)/48*y^3*u^3+F(75)/97*x^2*z*u^3+F(60)/29*x*y*z*u^3-8230*y^2*z*u^3-14234*x*z^2*u^3+F(84)/71*y*z^2*u^3-2166*z^3*u^3+7326*x^2*u^4-F(7)/13*x*y*u^4+1987*y^2*u^4+F(17)/35*x*z*u^4+10882*y*z*u^4-F(121)/60*z^2*u^4-15185*x*u^5-F(37)/24*y*u^5-F(37)/54*z*u^5-F(29)/17*u^6-11137*x^4*y*v+10288*x^3*y^2*v-F(114)/121*x^2*y^3*v-15029*x*y^4*v-7395*y^5*v+14632*x^4*z*v-F(13)/14*x^3*y*z*v+10585*x^2*y^2*z*v+F(123)/115*x*y^3*z*v-F(93)/100*y^4*z*v-F(63)/43*x^3*z^2*v-F(59)/12*x^2*y*z^2*v+8589*x*y^2*z^2*v+2972*y^3*z^2*v-F(44)/27*x^2*z^3*v+1365*x*y*z^3*v-F(107)/80*y^2*z^3*v-F(84)/59*x*z^4*v+F(113)/68*y*z^4*v-F(91)/58*z^5*v+F(31)/42*x^4*u*v-14270*x^3*y*u*v+F(64)/47*x^2*y^2*u*v-10451*x*y^3*u*v-F(49)/122*y^4*u*v+5804*x^3*z*u*v-F(53)/2*x^2*y*z*u*v-F(55)/83*x*y^2*z*u*v-7608*y^3*z*u*v+8686*x^2*z^2*u*v+F(79)/39*x*y*z^2*u*v-1617*y^2*z^2*u*v+3814*x*z^3*u*v-13079*y*z^3*u*v+10596*z^4*u*v+11556*x^3*u^2*v+1644*x^2*y*u^2*v-F(108)/113*x*y^2*u^2*v-15317*y^3*u^2*v-8165*x^2*z*u^2*v-1260*x*y*z*u^2*v-11011*y^2*z*u^2*v-F(47)/122*x*z^2*u^2*v-F(61)/11*y*z^2*u^2*v-F(21)/37*z^3*u^2*v+F(101)/98*x^2*u^3*v+5364*x*y*u^3*v+F(81)/95*y^2*u^3*v-F(3)/125*x*z*u^3*v+8572*y*z*u^3*v+6299*z^2*u^3*v-F(34)/81*x*u^4*v-903*y*u^4*v+F(43)/94*z*u^4*v-F(121)/40*u^5*v+8992*x^4*v^2-F(3)/73*x^3*y*v^2+F(9)/13*x^2*y^2*v^2+F(11)/64*x*y^3*v^2+8231*y^4*v^2+4712*x^3*z*v^2-14376*x^2*y*z*v^2-15336*x*y^2*z*v^2-F(85)/79*y^3*z*v^2-11606*x^2*z^2*v^2-F(112)/61*x*y*z^2*v^2-F(63)/101*y^2*z^2*v^2+F(45)/92*x*z^3*v^2-F(61)/20*y*z^3*v^2+F(57)/73*z^4*v^2-F(41)/28*x^3*u*v^2-8490*x^2*y*u*v^2-3471*x*y^2*u*v^2-1273*y^3*u*v^2-F(69)/50*x^2*z*u*v^2-F(101)/19*x*y*z*u*v^2+F(62)/113*y^2*z*u*v^2+F(17)/11*x*z^2*u*v^2+2310*y*z^2*u*v^2-F(46)/33*z^3*u*v^2+F(97)/122*x^2*u^2*v^2+10615*x*y*u^2*v^2-F(34)/61*y^2*u^2*v^2-7153*x*z*u^2*v^2+F(32)/77*y*z*u^2*v^2+F(115)/11*z^2*u^2*v^2-F(85)/28*x*u^3*v^2-F(18)/83*y*u^3*v^2+F(112)/69*z*u^3*v^2+4671*u^4*v^2-F(89)/22*x^3*v^3-F(97)/101*x^2*y*v^3-12912*x*y^2*v^3+9582*y^3*v^3+F(11)/98*x^2*z*v^3+F(71)/106*x*y*z*v^3+14781*y^2*z*v^3-F(64)/51*x*z^2*v^3-12534*y*z^2*v^3-F(26)/79*z^3*v^3+F(4)/15*x^2*u*v^3-F(115)/31*x*y*u*v^3-13144*y^2*u*v^3+F(83)/95*x*z*u*v^3+F(22)/47*y*z*u*v^3-F(24)/41*z^2*u*v^3+F(28)/69*x*u^2*v^3-4135*y*u^2*v^3+F(47)/97*z*u^2*v^3+F(69)/26*u^3*v^3+7815*x^2*v^4-F(46)/47*x*y*v^4-F(7)/88*y^2*v^4+F(54)/13*x*z*v^4+F(67)/74*y*z*v^4+15450*z^2*v^4+F(117)/28*x*u*v^4-9120*y*u*v^4-F(99)/86*z*u*v^4-F(72)/101*u^2*v^4+F(122)/55*x*v^5+F(11)/102*y*v^5+F(85)/74*z*v^5+F(9)/17*u*v^5-F(86)/83*v^6,15618*x^4*y^2-9825*x^3*y^3+2887*x^2*y^4+F(43)/95*x*y^5-2809*y^6-4419*x^4*y*z+F(29)/7*x^3*y^2*z+4286*x^2*y^3*z+F(117)/19*x*y^4*z+12651*y^5*z+9716*x^4*z^2-F(121)/104*x^3*y*z^2+F(26)/87*x^2*y^2*z^2+4254*x*y^3*z^2-22*y^4*z^2-7398*x^3*z^3-F(70)/59*x^2*y*z^3-3261*x*y^2*z^3+1899*y^3*z^3+F(54)/101*x^2*z^4-F(17)/55*x*y*z^4-F(76)/33*y^2*z^4+3004*x*z^5+2053*y*z^5-1448*z^6-F(49)/67*x^4*y*u+10785*x^3*y^2*u-8641*x^2*y^3*u+F(7)/13*x*y^4*u+F(17)/101*y^5*u-11611*x^4*z*u+F(96)/77*x^3*y*z*u+F(90)/107*x^2*y^2*z*u-6048*x*y^3*z*u+F(84)/95*y^4*z*u-F(113)/35*x^3*z^2*u+F(57)/79*x^2*y*z^2*u-F(40)/13*x*y^2*z^2*u+F(125)/11*y^3*z^2*u+11526*x^2*z^3*u+8288*x*y*z^3*u-F(103)/15*y^2*z^3*u-1692*x*z^4*u-F(1)/37*y*z^4*u+F(63)/80*z^5*u+F(117)/31*x^4*u^2-F(109)/90*x^3*y*u^2+5370*x^2*y^2*u^2+F(93)/50*x*y^3*u^2+F(113)/9*y^4*u^2+14562*x^3*z*u^2-F(98)/39*x^2*y*z*u^2-F(72)/65*x*y^2*z*u^2-F(3)/97*y^3*z*u^2-2155*x^2*z^2*u^2+8608*x*y*z^2*u^2-3786*y^2*z^2*u^2+F(119)/95*x*z^3*u^2-2147*y*z^3*u^2+9855*z^4*u^2-F(22)/57*x^3*u^3-10786*x^2*y*u^3-F(93)/73*x*y^2*u^3-F(119)/114*y^3*u^3+F(79)/90*x^2*z*u^3+F(115)/98*x*y*z*u^3-152*y^2*z*u^3+F(50)/113*x*z^2*u^3+15397*y*z^2*u^3+4163*z^3*u^3+F(13)/41*x^2*u^4-F(83)/61*x*y*u^4-F(86)/27*y^2*u^4-F(29)/27*x*z*u^4+F(61)/8*y*z*u^4-F(92)/41*z^2*u^4-F(115)/89*x*u^5+11235*y*u^5+8942*z*u^5+F(8)/23*u^6-12388*x^4*y*v+F(35)/57*x^3*y^2*v+F(36)/25*x^2*y^3*v+F(14)/23*x*y^4*v+F(95)/82*y^5*v-F(31)/91*x^4*z*v+8651*x^3*y*z*v-10114*x^2*y^2*z*v-F(92)/21*x*y^3*z*v+5618*y^4*z*v-F(125)/72*x^3*z^2*v+6842*x^2*y*z^2*v+11588*x*y^2*z^2*v+6197*y^3*z^2*v+F(32)/79*x^2*z^3*v+F(66)/53*x*y*z^3*v-F(49)/73*y^2*z^3*v+7345*x*z^4*v+F(70)/67*y*z^4*v-F(107)/69*z^5*v+7254*x^4*u*v+F(23)/59*x^3*y*u*v-F(34)/125*x^2*y^2*u*v-F(82)/75*x*y^3*u*v-3772*y^4*u*v-F(58)/41*x^3*z*u*v-F(17)/100*x^2*y*z*u*v+F(9)/113*x*y^2*z*u*v+F(5)/43*y^3*z*u*v+F(111)/107*x^2*z^2*u*v-F(96)/119*x*y*z^2*u*v-F(124)/59*y^2*z^2*u*v+F(49)/86*x*z^3*u*v+F(95)/78*y*z^3*u*v+F(70)/73*z^4*u*v-9067*x^3*u^2*v-F(99)/92*x^2*y*u^2*v-15001*x*y^2*u^2*v-F(119)/25*y^3*u^2*v+F(82)/73*x^2*z*u^2*v-7763*x*y*z*u^2*v+F(89)/23*y^2*z*u^2*v+F(111)/47*x*z^2*u^2*v+F(36)/65*y*z^2*u^2*v-F(11)/100*z^3*u^2*v+3579*x^2*u^3*v-F(115)/66*x*y*u^3*v+F(81)/83*y^2*u^3*v-15891*x*z*u^3*v-2273*y*z*u^3*v+13381*z^2*u^3*v-F(78)/89*x*u^4*v+F(17)/16*y*u^4*v+F(17)/47*z*u^4*v+F(91)/61*u^5*v-4910*x^4*v^2-1611*x^3*y*v^2-11504*x^2*y^2*v^2-F(115)/104*x*y^3*v^2-F(77)/59*y^4*v^2+F(15)/101*x^3*z*v^2-12579*x^2*y*z*v^2-5679*x*y^2*z*v^2-3399*y^3*z*v^2+7663*x^2*z^2*v^2+F(37)/56*x*y*z^2*v^2-F(3)/38*y^2*z^2*v^2-13792*x*z^3*v^2+8912*y*z^3*v^2+7809*z^4*v^2-F(28)/107*x^3*u*v^2-6514*x^2*y*u*v^2-12528*x*y^2*u*v^2-13342*y^3*u*v^2-14285*x^2*z*u*v^2-F(31)/2*x*y*z*u*v^2+645*y^2*z*u*v^2-F(31)/81*x*z^2*u*v^2-8521*y*z^2*u*v^2-F(22)/95*z^3*u*v^2-10392*x^2*u^2*v^2-F(83)/54*x*y*u^2*v^2+2211*y^2*u^2*v^2-8121*x*z*u^2*v^2-60*y*z*u^2*v^2+F(77)/18*z^2*u^2*v^2-237*x*u^3*v^2+5390*y*u^3*v^2-13675*z*u^3*v^2+6488*u^4*v^2+13426*x^3*v^3-F(24)/119*x^2*y*v^3-F(113)/105*x*y^2*v^3-F(100)/51*y^3*v^3+7823*x^2*z*v^3-12924*x*y*z*v^3-F(48)/97*y^2*z*v^3+F(31)/15*x*z^2*v^3-2700*y*z^2*v^3-F(118)/89*z^3*v^3+5727*x^2*u*v^3+F(11)/113*x*y*u*v^3+1736*y^2*u*v^3-13245*x*z*u*v^3+F(73)/2*y*z*u*v^3-F(78)/115*z^2*u*v^3-F(67)/56*x*u^2*v^3+6630*y*u^2*v^3-F(25)/78*z*u^2*v^3-10256*u^3*v^3-10197*x^2*v^4+F(83)/55*x*y*v^4-F(47)/125*y^2*v^4+15111*x*z*v^4+F(15)/67*y*z*v^4-F(7)/116*z^2*v^4-14085*x*u*v^4+F(13)/37*y*u*v^4+F(81)/95*z*u*v^4-12015*u^2*v^4+F(108)/31*x*v^5-9344*y*v^5+F(13)/41*z*v^5+F(77)/13*u*v^5+F(88)/45*v^6,-4278*x^4*y^2+13114*x^3*y^3+13494*x^2*y^4-F(111)/64*x*y^5+99*y^6-F(80)/21*x^4*y*z-F(14)/61*x^3*y^2*z-F(77)/69*x^2*y^3*z-F(53)/65*x*y^4*z-F(65)/76*y^5*z-F(56)/125*x^4*z^2+9936*x^3*y*z^2-F(83)/113*x^2*y^2*z^2+10345*x*y^3*z^2-F(58)/51*y^4*z^2+F(33)/125*x^3*z^3+F(84)/83*x^2*y*z^3+9324*x*y^2*z^3-F(87)/13*y^3*z^3-12218*x^2*z^4-F(59)/112*x*y*z^4-F(103)/59*y^2*z^4+F(68)/61*x*z^5+F(111)/4*y*z^5+10202*z^6-4053*x^4*y*u+F(87)/89*x^3*y^2*u+F(107)/125*x^2*y^3*u-F(99)/46*x*y^4*u-F(21)/73*y^5*u+F(13)/47*x^4*z*u+F(41)/116*x^3*y*z*u-F(8)/89*x^2*y^2*z*u-F(14)/59*x*y^3*z*u+2199*y^4*z*u+F(126)/41*x^3*z^2*u+F(123)/29*x^2*y*z^2*u+8547*x*y^2*z^2*u-F(31)/43*y^3*z^2*u+9865*x^2*z^3*u+F(61)/113*x*y*z^3*u+F(118)/57*y^2*z^3*u+F(83)/10*x*z^4*u-2143*y*z^4*u+11472*z^5*u-F(7)/19*x^4*u^2-F(87)/14*x^3*y*u^2+F(101)/3*x^2*y^2*u^2-2149*x*y^3*u^2+13787*y^4*u^2-1443*x^3*z*u^2+15907*x^2*y*z*u^2-F(67)/60*x*y^2*z*u^2+8171*y^3*z*u^2+9501*x^2*z^2*u^2+1224*x*y*z^2*u^2+6873*y^2*z^2*u^2+F(47)/52*x*z^3*u^2-F(86)/45*y*z^3*u^2+F(38)/53*z^4*u^2+F(71)/15*x^3*u^3-F(55)/16*x^2*y*u^3+F(95)/118*x*y^2*u^3-4778*y^3*u^3+8288*x^2*z*u^3-F(38)/125*x*y*z*u^3-F(29)/112*y^2*z*u^3+15161*x*z^2*u^3-9618*y*z^2*u^3-F(69)/59*z^3*u^3+8679*x^2*u^4-F(43)/25*x*y*u^4+15753*y^2*u^4+F(47)/44*x*z*u^4-7963*y*z*u^4-F(88)/101*z^2*u^4-F(47)/120*x*u^5+F(22)/17*y*u^5+F(77)/29*z*u^5-F(53)/80*u^6+15506*x^4*y*v+12598*x^3*y^2*v-F(5)/12*x^2*y^3*v-3224*x*y^4*v-F(124)/79*y^5*v+F(49)/41*x^4*z*v-F(58)/23*x^3*y*z*v+F(11)/73*x^2*y^2*z*v+F(47)/93*x*y^3*z*v+F(71)/69*y^4*z*v-F(1)/78*x^3*z^2*v-15242*x^2*y*z^2*v-F(71)/67*x*y^2*z^2*v+F(117)/121*y^3*z^2*v+F(21)/40*x^2*z^3*v+14680*x*y*z^3*v-F(11)/71*y^2*z^3*v+F(85)/114*x*z^4*v-F(107)/41*y*z^4*v+F(25)/4*z^5*v+F(75)/107*x^4*u*v-8302*x^3*y*u*v-2749*x^2*y^2*u*v+5909*x*y^3*u*v+7336*y^4*u*v+13638*x^3*z*u*v+F(1)/82*x^2*y*z*u*v-F(7)/20*x*y^2*z*u*v-11756*y^3*z*u*v+F(38)/11*x^2*z^2*u*v-226*x*y*z^2*u*v+F(3)/32*y^2*z^2*u*v+3314*x*z^3*u*v+5295*y*z^3*u*v-F(111)/56*z^4*u*v+F(9)/2*x^3*u^2*v+9563*x^2*y*u^2*v+F(29)/51*x*y^2*u^2*v+F(70)/67*y^3*u^2*v+2224*x^2*z*u^2*v-F(17)/69*x*y*z*u^2*v+13901*y^2*z*u^2*v+F(17)/42*x*z^2*u^2*v-15712*y*z^2*u^2*v-7228*z^3*u^2*v+F(54)/13*x^2*u^3*v-F(29)/32*x*y*u^3*v+12060*y^2*u^3*v+F(86)/107*x*z*u^3*v-14941*y*z*u^3*v+6673*z^2*u^3*v-F(45)/101*x*u^4*v+F(21)/79*y*u^4*v-5755*z*u^4*v+F(59)/90*u^5*v-F(62)/79*x^4*v^2+F(57)/116*x^3*y*v^2-F(77)/107*x^2*y^2*v^2+191*x*y^3*v^2+15093*y^4*v^2-F(55)/58*x^3*z*v^2-F(83)/59*x^2*y*z*v^2+F(79)/94*x*y^2*z*v^2+F(69)/109*y^3*z*v^2+F(49)/115*x^2*z^2*v^2-F(39)/113*x*y*z^2*v^2+F(48)/101*y^2*z^2*v^2-8977*x*z^3*v^2-4937*y*z^3*v^2-F(63)/31*z^4*v^2+8803*x^3*u*v^2-F(11)/105*x^2*y*u*v^2-3529*x*y^2*u*v^2-13297*y^3*u*v^2-F(71)/99*x^2*z*u*v^2-F(15)/86*x*y*z*u*v^2+F(125)/37*y^2*z*u*v^2+F(3)/2*x*z^2*u*v^2+F(29)/45*y*z^2*u*v^2-10288*z^3*u*v^2-F(105)/109*x^2*u^2*v^2+11908*x*y*u^2*v^2-15919*y^2*u^2*v^2-6747*x*z*u^2*v^2+8037*y*z*u^2*v^2+F(104)/45*z^2*u^2*v^2-5860*x*u^3*v^2-F(42)/59*y*u^3*v^2+F(89)/10*z*u^3*v^2+F(31)/94*u^4*v^2+10721*x^3*v^3-2014*x^2*y*v^3+7407*x*y^2*v^3+F(35)/52*y^3*v^3+12953*x^2*z*v^3+6656*x*y*z*v^3+9050*y^2*z*v^3-F(48)/115*x*z^2*v^3+F(121)/94*y*z^2*v^3-4503*z^3*v^3+F(41)/13*x^2*u*v^3-F(89)/47*x*y*u*v^3+F(101)/36*y^2*u*v^3+5416*x*z*u*v^3-10443*y*z*u*v^3+F(44)/107*z^2*u*v^3+3852*x*u^2*v^3+4354*y*u^2*v^3+F(95)/116*z*u^2*v^3+13732*u^3*v^3-F(103)/59*x^2*v^4-F(21)/122*x*y*v^4-F(43)/80*y^2*v^4+F(52)/7*x*z*v^4+F(11)/18*y*z*v^4-F(59)/56*z^2*v^4-F(59)/63*x*u*v^4-3132*y*u*v^4-F(92)/35*z*u*v^4-F(5)/72*u^2*v^4+F(47)/104*x*v^5+F(28)/69*y*v^5+1136*z*v^5-F(65)/7*u*v^5+14199*v^6,F(41)/59*x^4*y^2+F(100)/117*x^3*y^3-7059*x^2*y^4-F(74)/55*x*y^5-F(70)/83*y^6-4257*x^4*y*z-F(5)/79*x^3*y^2*z+15260*x^2*y^3*z+10256*x*y^4*z+F(26)/9*y^5*z+F(119)/124*x^4*z^2+F(6)/5*x^3*y*z^2-F(54)/31*x^2*y^2*z^2-11175*x*y^3*z^2+10228*y^4*z^2+125*x^3*z^3+F(99)/43*x^2*y*z^3-8950*x*y^2*z^3-F(111)/68*y^3*z^3-F(81)/62*x^2*z^4-F(108)/11*x*y*z^4+9885*y^2*z^4+11259*x*z^5-F(22)/25*y*z^5+F(13)/113*z^6-10713*x^4*y*u-14920*x^3*y^2*u-F(11)/61*x^2*y^3*u-13955*x*y^4*u+12534*y^5*u-F(101)/59*x^4*z*u-1870*x^3*y*z*u+F(42)/83*x^2*y^2*z*u-F(90)/37*x*y^3*z*u-15003*y^4*z*u-12024*x^3*z^2*u-F(38)/99*x^2*y*z^2*u-F(39)/83*x*y^2*z^2*u+F(84)/13*y^3*z^2*u+F(117)/70*x^2*z^3*u+5478*x*y*z^3*u+8250*y^2*z^3*u-9200*x*z^4*u-F(112)/65*y*z^4*u-F(99)/68*z^5*u+F(12)/97*x^4*u^2-F(61)/84*x^3*y*u^2+10500*x^2*y^2*u^2+F(121)/50*x*y^3*u^2-F(69)/56*y^4*u^2+9279*x^3*z*u^2-6614*x^2*y*z*u^2-201*x*y^2*z*u^2-F(122)/79*y^3*z*u^2+F(19)/56*x^2*z^2*u^2-4519*x*y*z^2*u^2-4380*y^2*z^2*u^2-F(64)/87*x*z^3*u^2-F(107)/69*y*z^3*u^2+F(49)/60*z^4*u^2-10463*x^3*u^3+5025*x^2*y*u^3+15924*x*y^2*u^3-3256*y^3*u^3-F(22)/119*x^2*z*u^3+F(17)/114*x*y*z*u^3+F(102)/97*y^2*z*u^3-3407*x*z^2*u^3-5779*y*z^2*u^3+F(97)/119*z^3*u^3-F(53)/94*x^2*u^4-1502*x*y*u^4+F(104)/19*y^2*u^4+F(83)/43*x*z*u^4+9675*y*z*u^4-F(47)/38*z^2*u^4+F(73)/55*x*u^5-1973*y*u^5-12248*z*u^5+F(73)/102*u^6-1741*x^4*y*v+14403*x^3*y^2*v-5111*x^2*y^3*v-15366*x*y^4*v-F(29)/88*y^5*v-1577*x^4*z*v-F(125)/81*x^3*y*z*v+F(24)/113*x^2*y^2*z*v+F(57)/112*x*y^3*z*v+F(106)/79*y^4*z*v-F(25)/21*x^3*z^2*v+5144*x^2*y*z^2*v+F(63)/101*x*y^2*z^2*v-11617*y^3*z^2*v+F(83)/96*x^2*z^3*v+3128*x*y*z^3*v+5879*y^2*z^3*v+11479*x*z^4*v+4603*y*z^4*v+F(98)/51*z^5*v+14529*x^4*u*v+13010*x^3*y*u*v-78*x^2*y^2*u*v+14754*x*y^3*u*v-13804*y^4*u*v+F(12)/61*x^3*z*u*v-F(59)/86*x^2*y*z*u*v+F(25)/48*x*y^2*z*u*v-F(1)/11*y^3*z*u*v-8695*x^2*z^2*u*v+F(5)/26*x*y*z^2*u*v-F(126)/73*y^2*z^2*u*v+13805*x*z^3*u*v+F(23)/11*y*z^3*u*v-6070*z^4*u*v+F(17)/120*x^3*u^2*v-F(83)/63*x^2*y*u^2*v+F(95)/8*x*y^2*u^2*v+5589*y^3*u^2*v+4634*x^2*z*u^2*v-F(103)/31*x*y*z*u^2*v-F(21)/109*y^2*z*u^2*v-F(12)/29*x*z^2*u^2*v+7029*y*z^2*u^2*v+14357*z^3*u^2*v+13559*x^2*u^3*v+F(56)/73*x*y*u^3*v-F(101)/14*y^2*u^3*v+F(62)/29*x*z*u^3*v+2186*y*z*u^3*v-777*z^2*u^3*v-F(10)/33*x*u^4*v-F(56)/57*y*u^4*v+3260*z*u^4*v-F(48)/79*u^5*v-F(36)/7*x^4*v^2+7906*x^3*y*v^2+F(59)/32*x^2*y^2*v^2+F(76)/77*x*y^3*v^2+F(5)/66*y^4*v^2+F(6)/125*x^3*z*v^2+F(7)/62*x^2*y*z*v^2-F(123)/50*x*y^2*z*v^2+F(63)/40*y^3*z*v^2+F(14)/69*x^2*z^2*v^2-F(112)/25*x*y*z^2*v^2+F(61)/107*y^2*z^2*v^2+185*x*z^3*v^2-F(41)/26*y*z^3*v^2-9070*z^4*v^2-10868*x^3*u*v^2+2037*x^2*y*u*v^2+11967*x*y^2*u*v^2+F(115)/12*y^3*u*v^2+F(70)/27*x^2*z*u*v^2+F(52)/63*x*y*z*u*v^2+F(122)/9*y^2*z*u*v^2+11913*x*z^2*u*v^2-F(111)/65*y*z^2*u*v^2+F(73)/84*z^3*u*v^2+14523*x^2*u^2*v^2+F(35)/13*x*y*u^2*v^2-F(63)/109*y^2*u^2*v^2+6349*x*z*u^2*v^2-123*y*z*u^2*v^2+6790*z^2*u^2*v^2-F(41)/68*x*u^3*v^2-9876*y*u^3*v^2-F(19)/55*z*u^3*v^2+14298*u^4*v^2+F(25)/116*x^3*v^3-7087*x^2*y*v^3+F(104)/125*x*y^2*v^3+F(12)/55*y^3*v^3-F(55)/34*x^2*z*v^3+11941*x*y*z*v^3+F(19)/48*y^2*z*v^3-2235*x*z^2*v^3+F(20)/69*y*z^2*v^3+F(49)/50*z^3*v^3+F(64)/87*x^2*u*v^3-6070*x*y*u*v^3-F(60)/119*y^2*u*v^3+F(92)/61*x*z*u*v^3+3285*y*z*u*v^3+F(95)/51*z^2*u*v^3+12479*x*u^2*v^3-8506*y*u^2*v^3-7300*z*u^2*v^3+467*u^3*v^3+15173*x^2*v^4+F(74)/51*x*y*v^4-9620*y^2*v^4-9872*x*z*v^4-F(28)/53*y*z*v^4+13478*z^2*v^4+F(10)/101*x*u*v^4+F(74)/83*y*u*v^4-F(57)/118*z*u*v^4+2121*u^2*v^4-749*x*v^5+44*y*v^5+F(121)/123*z*v^5-F(111)/83*u*v^5-11504*v^6,-F(71)/50*x^4*y^2-F(57)/46*x^3*y^3-15914*x^2*y^4-11670*x*y^5-F(67)/21*y^6-4281*x^4*y*z+F(106)/45*x^3*y^2*z-F(41)/114*x^2*y^3*z-F(49)/25*x*y^4*z+15112*y^5*z+10721*x^4*z^2-F(113)/55*x^3*y*z^2-F(95)/68*x^2*y^2*z^2-6290*x*y^3*z^2-F(49)/6*y^4*z^2-F(83)/95*x^3*z^3-5795*x^2*y*z^3-F(18)/101*x*y^2*z^3-F(55)/52*y^3*z^3-13956*x^2*z^4+13770*x*y*z^4-F(103)/85*y^2*z^4+13746*x*z^5-F(81)/92*y*z^5-12574*z^6+277*x^4*y*u-14010*x^3*y^2*u+6748*x^2*y^3*u+12041*x*y^4*u-8984*y^5*u-F(94)/75*x^4*z*u-F(15)/23*x^3*y*z*u-833*x^2*y^2*z*u-6296*x*y^3*z*u-F(49)/33*y^4*z*u-F(87)/34*x^3*z^2*u+893*x^2*y*z^2*u+F(87)/38*x*y^2*z^2*u-F(42)/115*y^3*z^2*u-F(35)/18*x^2*z^3*u+F(114)/25*x*y*z^3*u+14557*y^2*z^3*u-14822*x*z^4*u+F(31)/3*y*z^4*u+F(85)/101*z^5*u+F(27)/110*x^4*u^2-F(124)/123*x^3*y*u^2-F(62)/69*x^2*y^2*u^2+4538*x*y^3*u^2+F(39)/43*y^4*u^2-F(65)/67*x^3*z*u^2+F(73)/101*x^2*y*z*u^2+3898*x*y^2*z*u^2+F(123)/52*y^3*z*u^2-15652*x^2*z^2*u^2+F(99)/56*x*y*z^2*u^2-6446*y^2*z^2*u^2-F(45)/71*x*z^3*u^2+F(71)/22*y*z^3*u^2+F(95)/23*z^4*u^2+11279*x^3*u^3-F(71)/11*x^2*y*u^3-4028*x*y^2*u^3-F(33)/40*y^3*u^3-F(79)/26*x^2*z*u^3+F(68)/107*x*y*z*u^3+10267*y^2*z*u^3-F(42)/97*x*z^2*u^3-F(14)/87*y*z^2*u^3-2495*z^3*u^3+F(58)/73*x^2*u^4+F(88)/113*x*y*u^4-F(96)/83*y^2*u^4-9717*x*z*u^4+F(124)/37*y*z*u^4-F(29)/12*z^2*u^4-3452*x*u^5-3535*y*u^5-F(20)/21*z*u^5+F(102)/77*u^6-12840*x^4*y*v+45*x^3*y^2*v-F(37)/95*x^2*y^3*v+F(104)/119*x*y^4*v-F(91)/67*y^5*v-F(35)/37*x^4*z*v+F(13)/123*x^3*y*z*v+4786*x^2*y^2*z*v-13577*x*y^3*z*v+F(85)/12*y^4*z*v+7814*x^3*z^2*v-12945*x^2*y*z^2*v+F(97)/3*x*y^2*z^2*v+8768*y^3*z^2*v+F(93)/113*x^2*z^3*v-F(105)/46*x*y*z^3*v+F(29)/107*y^2*z^3*v-7645*x*z^4*v+7583*y*z^4*v+15920*z^5*v-F(79)/117*x^4*u*v+F(3)/52*x^3*y*u*v+F(94)/111*x^2*y^2*u*v-7084*x*y^3*u*v-F(47)/53*y^4*u*v+F(74)/63*x^3*z*u*v-4220*x^2*y*z*u*v+F(61)/32*x*y^2*z*u*v-F(3)/10*y^3*z*u*v-F(7)/55*x^2*z^2*u*v+F(82)/9*x*y*z^2*u*v-10912*y^2*z^2*u*v+F(37)/87*x*z^3*u*v+5862*y*z^3*u*v+F(70)/123*z^4*u*v+F(94)/77*x^3*u^2*v+3871*x^2*y*u^2*v+15527*x*y^2*u^2*v-F(37)/90*y^3*u^2*v+101*x^2*z*u^2*v+8952*x*y*z*u^2*v+F(3)/23*y^2*z*u^2*v-6866*x*z^2*u^2*v+F(38)/45*y*z^2*u^2*v-2551*z^3*u^2*v+F(106)/61*x^2*u^3*v-3471*x*y*u^3*v-6064*y^2*u^3*v+F(9)/32*x*z*u^3*v-12391*y*z*u^3*v+7645*z^2*u^3*v-F(118)/125*x*u^4*v+F(16)/11*y*u^4*v+F(84)/79*z*u^4*v+13576*u^5*v-F(121)/57*x^4*v^2-13593*x^3*y*v^2+3531*x^2*y^2*v^2+7129*x*y^3*v^2+2212*y^4*v^2-F(124)/95*x^3*z*v^2-F(53)/83*x^2*y*z*v^2-2888*x*y^2*z*v^2-F(44)/65*y^3*z*v^2+14432*x^2*z^2*v^2+15365*x*y*z^2*v^2-F(55)/64*y^2*z^2*v^2+F(2)/25*x*z^3*v^2-F(77)/6*y*z^3*v^2+F(41)/89*z^4*v^2+F(24)/125*x^3*u*v^2-14601*x^2*y*u*v^2+13616*x*y^2*u*v^2-F(79)/52*y^3*u*v^2-13162*x^2*z*u*v^2+F(45)/34*x*y*z*u*v^2-3377*y^2*z*u*v^2+3742*x*z^2*u*v^2-F(29)/12*y*z^2*u*v^2+6161*z^3*u*v^2-590*x^2*u^2*v^2+919*x*y*u^2*v^2-6687*y^2*u^2*v^2-F(36)/103*x*z*u^2*v^2+11698*y*z*u^2*v^2-F(87)/115*z^2*u^2*v^2+5608*x*u^3*v^2+4671*y*u^3*v^2+F(79)/63*z*u^3*v^2+F(13)/96*u^4*v^2-8971*x^3*v^3+F(104)/119*x^2*y*v^3+F(126)/113*x*y^2*v^3-F(5)/118*y^3*v^3-F(97)/9*x^2*z*v^3+7943*x*y*z*v^3+F(48)/29*y^2*z*v^3-F(41)/26*x*z^2*v^3+14875*y*z^2*v^3-12929*z^3*v^3+F(114)/79*x^2*u*v^3+F(58)/35*x*y*u*v^3+F(98)/47*y^2*u*v^3+F(73)/37*x*z*u*v^3-F(113)/40*y*z*u*v^3+F(1)/110*z^2*u*v^3-F(74)/101*x*u^2*v^3+47*y*u^2*v^3-F(91)/24*z*u^2*v^3-9746*u^3*v^3+1971*x^2*v^4+F(123)/118*x*y*v^4-F(33)/46*y^2*v^4+F(44)/57*x*z*v^4-F(1)/123*y*z*v^4+F(12)/29*z^2*v^4-F(1)/84*x*u*v^4+6072*y*u*v^4+1899*z*u*v^4-13431*u^2*v^4+5502*x*v^5-7411*y*v^5+9533*z*v^5-1038*u*v^5-7787*v^6,F(77)/10*x^4*y^2-F(111)/49*x^3*y^3+15718*x^2*y^4+F(46)/97*x*y^5-13243*y^6-5297*x^4*y*z+7092*x^3*y^2*z+12531*x^2*y^3*z+F(94)/9*x*y^4*z+8187*y^5*z+9757*x^4*z^2-2951*x^3*y*z^2+F(27)/74*x^2*y^2*z^2-860*x*y^3*z^2+2075*y^4*z^2+F(9)/70*x^3*z^3-F(33)/53*x^2*y*z^3-F(35)/124*x*y^2*z^3+F(46)/93*y^3*z^3+F(109)/105*x^2*z^4-F(81)/56*x*y*z^4-15636*y^2*z^4+F(46)/51*x*z^5+F(75)/106*y*z^5-F(104)/85*z^6+11887*x^4*y*u-14500*x^3*y^2*u+13899*x^2*y^3*u+F(31)/66*x*y^4*u+2345*y^5*u+11138*x^4*z*u+F(35)/87*x^3*y*z*u+F(77)/97*x^2*y^2*z*u+F(11)/48*x*y^3*z*u+F(25)/79*y^4*z*u-F(101)/86*x^3*z^2*u-2959*x^2*y*z^2*u+208*x*y^2*z^2*u+F(55)/114*y^3*z^2*u-F(37)/99*x^2*z^3*u-1566*x*y*z^3*u+F(82)/43*y^2*z^3*u-12041*x*z^4*u-F(114)/103*y*z^4*u-15128*z^5*u+F(14)/101*x^4*u^2+F(67)/51*x^3*y*u^2+F(124)/83*x^2*y^2*u^2-5478*x*y^3*u^2+F(87)/43*y^4*u^2+10357*x^3*z*u^2+F(5)/26*x^2*y*z*u^2-F(53)/75*x*y^2*z*u^2-756*y^3*z*u^2-230*x^2*z^2*u^2+7595*x*y*z^2*u^2-F(71)/22*y^2*z^2*u^2+F(121)/9*x*z^3*u^2+7515*y*z^3*u^2+7281*z^4*u^2+F(38)/99*x^3*u^3+3006*x^2*y*u^3+F(118)/81*x*y^2*u^3+F(71)/61*y^3*u^3-F(56)/15*x^2*z*u^3-F(33)/4*x*y*z*u^3-F(107)/74*y^2*z*u^3+F(73)/46*x*z^2*u^3-F(16)/19*y*z^2*u^3-737*z^3*u^3-15711*x^2*u^4-3857*x*y*u^4-F(77)/72*y^2*u^4-F(61)/104*x*z*u^4+F(64)/83*y*z*u^4-11759*z^2*u^4+F(61)/114*x*u^5+7685*y*u^5+6010*z*u^5+387*u^6-3979*x^4*y*v+F(121)/20*x^3*y^2*v+7134*x^2*y^3*v+6299*x*y^4*v+F(47)/55*y^5*v+F(1)/21*x^4*z*v+F(13)/106*x^3*y*z*v+2495*x^2*y^2*z*v+F(81)/50*x*y^3*z*v+F(86)/7*y^4*z*v-13094*x^3*z^2*v-F(68)/75*x^2*y*z^2*v+12180*x*y^2*z^2*v-F(44)/117*y^3*z^2*v+F(17)/75*x^2*z^3*v+F(103)/100*x*y*z^3*v+F(34)/67*y^2*z^3*v-F(30)/17*x*z^4*v+F(5)/2*y*z^4*v-10609*z^5*v-F(90)/7*x^4*u*v-F(26)/121*x^3*y*u*v+5578*x^2*y^2*u*v-6524*x*y^3*u*v-8484*y^4*u*v-F(57)/47*x^3*z*u*v-5910*x^2*y*z*u*v+9162*x*y^2*z*u*v-224*y^3*z*u*v+F(119)/66*x^2*z^2*u*v-F(52)/121*x*y*z^2*u*v+3038*y^2*z^2*u*v+F(31)/30*x*z^3*u*v-F(22)/111*y*z^3*u*v+F(37)/40*z^4*u*v-F(72)/121*x^3*u^2*v+F(107)/4*x^2*y*u^2*v+F(19)/91*x*y^2*u^2*v-1516*y^3*u^2*v-F(121)/89*x^2*z*u^2*v+F(95)/48*x*y*z*u^2*v-F(89)/25*y^2*z*u^2*v-F(40)/37*x*z^2*u^2*v-9018*y*z^2*u^2*v+F(113)/71*z^3*u^2*v+9283*x^2*u^3*v+F(91)/61*x*y*u^3*v-F(101)/121*y^2*u^3*v+F(101)/122*x*z*u^3*v+3005*y*z*u^3*v-15095*z^2*u^3*v-7293*x*u^4*v+3433*y*u^4*v+1609*z*u^4*v+F(67)/47*u^5*v-4873*x^4*v^2+252*x^3*y*v^2-7153*x^2*y^2*v^2+10847*x*y^3*v^2-F(88)/19*y^4*v^2+14719*x^3*z*v^2+F(26)/3*x^2*y*z*v^2-F(105)/11*x*y^2*z*v^2+F(55)/76*y^3*z*v^2+7228*x^2*z^2*v^2+F(52)/103*x*y*z^2*v^2-F(40)/91*y^2*z^2*v^2-6443*x*z^3*v^2+F(31)/101*y*z^3*v^2-F(18)/91*z^4*v^2+F(47)/18*x^3*u*v^2-15101*x^2*y*u*v^2-F(96)/29*x*y^2*u*v^2-8501*y^3*u*v^2+F(94)/103*x^2*z*u*v^2-F(52)/103*x*y*z*u*v^2-15289*y^2*z*u*v^2+F(115)/24*x*z^2*u*v^2-F(97)/70*y*z^2*u*v^2+15617*z^3*u*v^2-F(117)/10*x^2*u^2*v^2-F(58)/115*x*y*u^2*v^2+F(44)/25*y^2*u^2*v^2-F(121)/69*x*z*u^2*v^2+F(125)/72*y*z*u^2*v^2-12667*z^2*u^2*v^2-F(42)/5*x*u^3*v^2+F(41)/32*y*u^3*v^2+F(121)/54*z*u^3*v^2-F(45)/91*u^4*v^2+F(100)/91*x^3*v^3+F(28)/117*x^2*y*v^3-3113*x*y^2*v^3-5110*y^3*v^3-F(38)/37*x^2*z*v^3-F(16)/27*x*y*z*v^3-808*y^2*z*v^3-F(101)/23*x*z^2*v^3-F(121)/91*y*z^2*v^3-1906*z^3*v^3-F(88)/31*x^2*u*v^3+F(106)/37*x*y*u*v^3-F(23)/106*y^2*u*v^3+2193*x*z*u*v^3+F(43)/104*y*z*u*v^3+5043*z^2*u*v^3+F(50)/17*x*u^2*v^3-8951*y*u^2*v^3-F(75)/11*z*u^2*v^3+F(74)/67*u^3*v^3-14233*x^2*v^4-13733*x*y*v^4+F(107)/119*y^2*v^4+F(21)/79*x*z*v^4+15366*y*z*v^4+7179*z^2*v^4+7788*x*u*v^4-F(93)/13*y*u*v^4+14525*z*u*v^4-3509*u^2*v^4-F(125)/94*x*v^5+F(94)/15*y*v^5-14049*z*v^5+F(111)/83*u*v^5-11102*v^6,x^4*y^2-F(83)/15*x^3*y^3+F(75)/7*x^2*y^4-F(97)/14*x*y^5-F(49)/19*y^6+F(29)/101*x^4*y*z-7336*x^3*y^2*z-4370*x^2*y^3*z+F(95)/79*x*y^4*z-F(93)/103*y^5*z+F(57)/58*x^4*z^2-F(43)/19*x^3*y*z^2+F(123)/115*x^2*y^2*z^2-F(28)/55*x*y^3*z^2+F(45)/64*y^4*z^2-F(82)/107*x^3*z^3+F(108)/77*x^2*y*z^3-4541*x*y^2*z^3-F(46)/21*y^3*z^3-6822*x^2*z^4-F(79)/5*x*y*z^4+F(42)/47*y^2*z^4-6946*x*z^5-9747*y*z^5-F(12)/109*z^6-F(118)/55*x^4*y*u+1893*x^3*y^2*u+4988*x^2*y^3*u-F(91)/43*x*y^4*u-F(97)/111*y^5*u-F(95)/3*x^4*z*u+F(97)/122*x^3*y*z*u+7187*x^2*y^2*z*u-F(27)/25*x*y^3*z*u-F(7)/74*y^4*z*u+3872*x^3*z^2*u+965*x^2*y*z^2*u-4592*x*y^2*z^2*u-F(104)/43*y^3*z^2*u+2677*x^2*z^3*u+F(79)/47*x*y*z^3*u-F(9)/2*y^2*z^3*u+F(29)/59*x*z^4*u+1515*y*z^4*u+7949*z^5*u+F(123)/70*x^4*u^2+F(73)/3*x^3*y*u^2-F(36)/7*x^2*y^2*u^2-F(53)/16*x*y^3*u^2-4029*y^4*u^2-6935*x^3*z*u^2-10276*x^2*y*z*u^2-5232*x*y^2*z*u^2-15177*y^3*z*u^2+F(67)/125*x^2*z^2*u^2+F(27)/76*x*y*z^2*u^2+F(118)/49*y^2*z^2*u^2+F(62)/41*x*z^3*u^2+F(60)/89*y*z^3*u^2+3539*z^4*u^2+F(57)/64*x^3*u^3-6086*x^2*y*u^3+F(85)/19*x*y^2*u^3+F(63)/82*y^3*u^3-F(35)/62*x^2*z*u^3-F(48)/49*x*y*z*u^3+F(43)/94*y^2*z*u^3+7687*x*z^2*u^3+F(119)/81*y*z^2*u^3-8492*z^3*u^3-F(40)/63*x^2*u^4-F(109)/89*x*y*u^4-251*y^2*u^4-14989*x*z*u^4+8817*y*z*u^4-F(4)/7*z^2*u^4-F(45)/74*x*u^5-5747*y*u^5+F(118)/25*z*u^5+F(77)/47*u^6-F(52)/101*x^4*y*v-14711*x^3*y^2*v-15470*x^2*y^3*v+F(79)/32*x*y^4*v-14376*y^5*v-1916*x^4*z*v+2853*x^3*y*z*v-F(116)/67*x^2*y^2*z*v+F(38)/125*x*y^3*z*v+F(118)/89*y^4*z*v-5467*x^3*z^2*v-F(109)/115*x^2*y*z^2*v-F(67)/80*x*y^2*z^2*v-11808*y^3*z^2*v+1972*x^2*z^3*v+F(99)/91*x*y*z^3*v-F(118)/111*y^2*z^3*v-14180*x*z^4*v+4857*y*z^4*v+F(5)/14*z^5*v-11220*x^4*u*v-1868*x^3*y*u*v-3823*x^2*y^2*u*v-7135*x*y^3*u*v+F(40)/121*y^4*u*v+14153*x^3*z*u*v+F(13)/18*x^2*y*z*u*v+F(123)/43*x*y^2*z*u*v-6366*y^3*z*u*v-F(119)/90*x^2*z^2*u*v-12583*x*y*z^2*u*v+F(29)/19*y^2*z^2*u*v-F(26)/37*x*z^3*u*v+1397*y*z^3*u*v-F(29)/80*z^4*u*v-277*x^3*u^2*v-F(124)/75*x^2*y*u^2*v-F(119)/87*x*y^2*u^2*v-F(59)/99*y^3*u^2*v-F(19)/69*x^2*z*u^2*v+F(40)/49*x*y*z*u^2*v+F(35)/94*y^2*z*u^2*v-9576*x*z^2*u^2*v-F(61)/85*y*z^2*u^2*v-F(71)/79*z^3*u^2*v-F(92)/95*x^2*u^3*v-F(21)/11*x*y*u^3*v+F(1)/37*y^2*u^3*v+F(82)/23*x*z*u^3*v-1497*y*z*u^3*v+F(4)/97*z^2*u^3*v+F(71)/28*x*u^4*v+11244*y*u^4*v+F(106)/91*z*u^4*v+F(99)/14*u^5*v+619*x^4*v^2-10275*x^3*y*v^2+8817*x^2*y^2*v^2-F(29)/55*x*y^3*v^2+9506*y^4*v^2+F(51)/19*x^3*z*v^2+9700*x^2*y*z*v^2+3889*x*y^2*z*v^2-1895*y^3*z*v^2+2122*x^2*z^2*v^2+F(62)/17*x*y*z^2*v^2-F(123)/37*y^2*z^2*v^2+F(51)/10*x*z^3*v^2+F(43)/109*y*z^3*v^2+F(114)/31*z^4*v^2-12673*x^3*u*v^2+9283*x^2*y*u*v^2+F(74)/47*x*y^2*u*v^2-9266*y^3*u*v^2+F(86)/45*x^2*z*u*v^2+F(32)/39*x*y*z*u*v^2-F(119)/24*y^2*z*u*v^2+14198*x*z^2*u*v^2+F(26)/51*y*z^2*u*v^2+14582*z^3*u*v^2+13016*x^2*u^2*v^2-F(1)/41*x*y*u^2*v^2+F(108)/35*y^2*u^2*v^2-5608*x*z*u^2*v^2-F(56)/15*y*z*u^2*v^2+F(14)/43*z^2*u^2*v^2-F(57)/58*x*u^3*v^2-F(15)/59*y*u^3*v^2+F(67)/26*z*u^3*v^2+F(115)/106*u^4*v^2+14358*x^3*v^3+2553*x^2*y*v^3+2432*x*y^2*v^3+F(9)/22*y^3*v^3+3132*x^2*z*v^3+F(13)/25*x*y*z*v^3-F(82)/31*y^2*z*v^3+3697*x*z^2*v^3+14039*y*z^2*v^3+F(98)/109*z^3*v^3+F(80)/33*x^2*u*v^3+2894*x*y*u*v^3+F(101)/125*y^2*u*v^3-F(79)/126*x*z*u*v^3-F(92)/111*y*z*u*v^3+F(7)/125*z^2*u*v^3-13951*x*u^2*v^3+F(63)/53*y*u^2*v^3+14595*z*u^2*v^3-F(15)/56*u^3*v^3-9111*x^2*v^4-F(43)/21*x*y*v^4-F(59)/64*y^2*v^4-F(24)/7*x*z*v^4+4096*y*z*v^4-F(51)/55*z^2*v^4+F(64)/99*x*u*v^4+F(30)/41*y*u*v^4+1787*z*u*v^4-F(124)/105*u^2*v^4-F(69)/50*x*v^5+9832*y*v^5+F(8)/47*z*v^5-1400*u*v^5-F(27)/124*v^6,-9295*x^5+F(79)/123*x^4*y-F(118)/47*x^3*y^2-F(98)/33*x^2*y^3-3632*x*y^4+5592*y^5-10720*x^4*z+F(57)/97*x^3*y*z+4352*x^2*y^2*z+F(119)/99*x*y^3*z-4309*y^4*z+11979*x^3*z^2+F(105)/124*x^2*y*z^2-8035*x*y^2*z^2+6262*y^3*z^2+F(106)/47*x^2*z^3+F(71)/24*x*y*z^3-F(57)/49*y^2*z^3+F(116)/11*x*z^4-F(1)/72*y*z^4+6357*z^5-11251*x^4*u-F(13)/123*x^3*y*u-F(94)/79*x^2*y^2*u+11151*x*y^3*u-F(103)/83*y^4*u+F(109)/99*x^3*z*u-F(9)/46*x^2*y*z*u+12509*x*y^2*z*u+13128*y^3*z*u+14820*x^2*z^2*u-12076*x*y*z^2*u+5987*y^2*z^2*u+5617*x*z^3*u+4*y*z^3*u+7003*z^4*u-3267*x^3*u^2+4065*x^2*y*u^2+F(45)/44*x*y^2*u^2+6773*y^3*u^2+F(112)/15*x^2*z*u^2+F(63)/50*x*y*z*u^2+3577*y^2*z*u^2+8597*x*z^2*u^2-12154*y*z^2*u^2+F(62)/83*z^3*u^2+F(35)/62*x^2*u^3+F(33)/7*x*y*u^3+F(93)/43*y^2*u^3+F(83)/28*x*z*u^3-F(71)/33*y*z*u^3+F(86)/7*z^2*u^3+F(125)/82*x*u^4+F(40)/79*y*u^4-3597*z*u^4+F(31)/9*u^5-8097*x^4*v-F(92)/27*x^3*y*v+F(117)/56*x^2*y^2*v+F(19)/25*x*y^3*v+F(3)/22*y^4*v-2849*x^3*z*v-10613*x^2*y*z*v+F(62)/119*x*y^2*z*v-4329*y^3*z*v-3472*x^2*z^2*v-10258*x*y*z^2*v+F(99)/2*y^2*z^2*v+F(78)/31*x*z^3*v+F(101)/39*y*z^3*v-F(111)/77*z^4*v+5405*x^3*u*v-F(102)/73*x^2*y*u*v+10724*x*y^2*u*v+F(34)/59*y^3*u*v-5145*x^2*z*u*v-F(29)/74*x*y*z*u*v-142*y^2*z*u*v+F(1)/16*x*z^2*u*v+4183*y*z^2*u*v+F(107)/101*z^3*u*v+4474*x^2*u^2*v+F(74)/35*x*y*u^2*v-F(109)/29*y^2*u^2*v+F(35)/104*x*z*u^2*v-F(121)/76*y*z*u^2*v+F(53)/7*z^2*u^2*v-15772*x*u^3*v-F(77)/46*y*u^3*v-F(8)/55*z*u^3*v+104*u^4*v-12141*x^3*v^2-F(91)/102*x^2*y*v^2+F(79)/38*x*y^2*v^2-F(11)/70*y^3*v^2-F(14)/13*x^2*z*v^2-3969*x*y*z*v^2-F(125)/14*y^2*z*v^2-F(78)/7*x*z^2*v^2-2270*y*z^2*v^2+F(43)/119*z^3*v^2-F(115)/68*x^2*u*v^2+11341*x*y*u*v^2-8314*y^2*u*v^2+724*x*z*u*v^2+F(41)/113*y*z*u*v^2-5603*z^2*u*v^2+F(114)/61*x*u^2*v^2+F(123)/38*y*u^2*v^2+F(124)/69*z*u^2*v^2-F(105)/32*u^3*v^2-F(115)/84*x^2*v^3+F(21)/97*x*y*v^3-F(38)/45*y^2*v^3-F(81)/59*x*z*v^3-5183*y*z*v^3-99*z^2*v^3+4713*x*u*v^3+F(67)/39*y*u*v^3+F(34)/19*z*u*v^3-5305*u^2*v^3+F(45)/44*x*v^4+F(53)/77*y*v^4-F(11)/15*z*v^4-F(4)/123*u*v^4+15646*v^5,F(44)/101*x^5-1444*x^4*y+F(82)/111*x^3*y^2-11724*x^2*y^3-F(82)/83*x*y^4-4417*y^5+7366*x^4*z+7400*x^3*y*z+14482*x^2*y^2*z-F(48)/17*x*y^3*z+15517*y^4*z-10602*x^3*z^2-5295*x^2*y*z^2+F(1)/111*x*y^2*z^2-F(110)/57*y^3*z^2+F(113)/107*x^2*z^3+F(99)/101*x*y*z^3+F(24)/23*y^2*z^3+3013*x*z^4+495*y*z^4-12377*z^5-4545*x^4*u+F(1)/30*x^3*y*u-1382*x^2*y^2*u+F(28)/95*x*y^3*u-F(19)/34*y^4*u-F(11)/90*x^3*z*u-705*x^2*y*z*u+15499*x*y^2*z*u-F(1)/5*y^3*z*u+9689*x^2*z^2*u-F(43)/31*x*y*z^2*u+F(113)/101*y^2*z^2*u-8057*x*z^3*u-10092*y*z^3*u+8071*z^4*u+F(103)/51*x^3*u^2+F(77)/117*x^2*y*u^2+F(52)/71*x*y^2*u^2+751*y^3*u^2-F(21)/67*x^2*z*u^2-F(49)/27*x*y*z*u^2-F(67)/36*y^2*z*u^2-F(13)/95*x*z^2*u^2-9922*y*z^2*u^2+F(104)/67*z^3*u^2+F(40)/51*x^2*u^3+13283*x*y*u^3+9212*y^2*u^3+14524*x*z*u^3+7352*y*z*u^3-F(43)/8*z^2*u^3+F(74)/29*x*u^4-459*y*u^4+46*z*u^4+F(38)/31*u^5+F(77)/81*x^4*v+F(102)/107*x^3*y*v+F(109)/102*x^2*y^2*v+8965*x*y^3*v-F(44)/39*y^4*v-F(33)/49*x^3*z*v-3791*x^2*y*z*v-15829*x*y^2*z*v+2928*y^3*z*v-F(33)/86*x^2*z^2*v-F(125)/54*x*y*z^2*v-1637*y^2*z^2*v-F(86)/55*x*z^3*v-14300*y*z^3*v-F(103)/124*z^4*v-11682*x^3*u*v-F(25)/68*x^2*y*u*v+F(21)/104*x*y^2*u*v+F(119)/41*y^3*u*v+F(79)/15*x^2*z*u*v+13093*x*y*z*u*v+8454*y^2*z*u*v+1955*x*z^2*u*v-1008*y*z^2*u*v-F(105)/22*z^3*u*v-F(119)/31*x^2*u^2*v-F(112)/19*x*y*u^2*v-F(125)/82*y^2*u^2*v-F(46)/123*x*z*u^2*v-F(1)/69*y*z*u^2*v-F(110)/87*z^2*u^2*v-F(79)/42*x*u^3*v-F(13)/123*y*u^3*v-F(108)/115*z*u^3*v+F(113)/33*u^4*v+13078*x^3*v^2-F(113)/18*x^2*y*v^2+8312*x*y^2*v^2+F(119)/123*y^3*v^2-F(118)/75*x^2*z*v^2-F(112)/81*x*y*z*v^2-F(23)/8*y^2*z*v^2-F(63)/85*x*z^2*v^2-4749*y*z^2*v^2-F(107)/76*z^3*v^2+F(104)/21*x^2*u*v^2+442*x*y*u*v^2-F(7)/45*y^2*u*v^2-4782*x*z*u*v^2+F(49)/103*y*z*u*v^2-F(60)/43*z^2*u*v^2-F(64)/29*x*u^2*v^2+F(80)/61*y*u^2*v^2+F(57)/43*z*u^2*v^2-2490*u^3*v^2-F(120)/7*x^2*v^3+3615*x*y*v^3-F(85)/114*y^2*v^3-14430*x*z*v^3+F(65)/51*y*z*v^3+5727*z^2*v^3-7938*x*u*v^3-3849*y*u*v^3+6928*z*u*v^3-F(94)/63*u^2*v^3-F(27)/92*x*v^4+F(4)/105*y*v^4+F(8)/35*z*v^4+F(81)/4*u*v^4+F(31)/88*v^5,13251*x^5-F(23)/107*x^4*y-F(113)/46*x^3*y^2-5303*x^2*y^3-F(57)/91*x*y^4-5573*y^5+F(100)/101*x^4*z-14804*x^3*y*z+F(81)/107*x^2*y^2*z+8950*x*y^3*z-F(72)/41*y^4*z+1871*x^3*z^2+F(45)/67*x^2*y*z^2+F(74)/57*x*y^2*z^2-8896*y^3*z^2+F(93)/11*x^2*z^3-F(13)/23*x*y*z^3+F(91)/54*y^2*z^3+F(113)/21*x*z^4+F(97)/14*y*z^4+F(98)/121*z^5+13734*x^4*u+6538*x^3*y*u+F(65)/34*x^2*y^2*u-F(63)/40*x*y^3*u-F(58)/99*y^4*u+14238*x^3*z*u-1652*x^2*y*z*u+1179*x*y^2*z*u-F(110)/91*y^3*z*u-F(11)/17*x^2*z^2*u+1309*x*y*z^2*u+12864*y^2*z^2*u-F(99)/10*x*z^3*u-8894*y*z^3*u-F(29)/108*z^4*u+F(104)/49*x^3*u^2+F(87)/112*x^2*y*u^2-F(43)/14*x*y^2*u^2+4232*y^3*u^2-F(7)/125*x^2*z*u^2+15385*x*y*z*u^2+F(116)/59*y^2*z*u^2-F(97)/57*x*z^2*u^2+F(107)/120*y*z^2*u^2-F(84)/29*z^3*u^2-216*x^2*u^3-15063*x*y*u^3+F(11)/47*y^2*u^3+F(121)/29*x*z*u^3-F(1)/86*y*z*u^3-F(56)/123*z^2*u^3-7666*x*u^4-F(103)/62*y*u^4+6426*z*u^4+15385*u^5+F(121)/37*x^4*v-238*x^3*y*v-9338*x^2*y^2*v+8140*x*y^3*v-F(52)/123*y^4*v+F(93)/79*x^3*z*v+F(102)/37*x^2*y*z*v+8389*x*y^2*z*v-F(4)/21*y^3*z*v-F(89)/60*x^2*z^2*v-15907*x*y*z^2*v-F(11)/119*y^2*z^2*v-F(5)/38*x*z^3*v-F(70)/97*y*z^3*v-6636*z^4*v+F(44)/47*x^3*u*v-921*x^2*y*u*v-4417*x*y^2*u*v+F(87)/40*y^3*u*v-F(85)/8*x^2*z*u*v-13256*x*y*z*u*v+F(83)/17*y^2*z*u*v+2335*x*z^2*u*v+F(34)/75*y*z^2*u*v-874*z^3*u*v-F(108)/109*x^2*u^2*v+8739*x*y*u^2*v-5309*y^2*u^2*v-F(121)/72*x*z*u^2*v+F(85)/101*y*z*u^2*v+F(2)/83*z^2*u^2*v+F(21)/59*x*u^3*v-F(31)/82*y*u^3*v-F(126)/67*z*u^3*v-F(77)/2*u^4*v+8924*x^3*v^2-14154*x^2*y*v^2+12847*x*y^2*v^2+7185*y^3*v^2-6743*x^2*z*v^2+F(41)/23*x*y*z*v^2-F(49)/106*y^2*z*v^2+F(8)/89*x*z^2*v^2-14560*y*z^2*v^2+12527*z^3*v^2-1097*x^2*u*v^2+15133*x*y*u*v^2-F(24)/95*y^2*u*v^2-10725*x*z*u*v^2-F(59)/22*y*z*u*v^2-15409*z^2*u*v^2-F(119)/27*x*u^2*v^2+8852*y*u^2*v^2+F(53)/37*z*u^2*v^2+1709*u^3*v^2-F(35)/71*x^2*v^3+F(99)/56*x*y*v^3-F(62)/79*y^2*v^3-F(97)/108*x*z*v^3-F(39)/77*y*z*v^3+15200*z^2*v^3-6270*x*u*v^3+F(6)/85*y*u*v^3+3186*z*u*v^3-F(81)/92*u^2*v^3-F(12)/13*x*v^4-F(15)/79*y*v^4-F(118)/117*z*v^4+F(119)/92*u*v^4-F(101)/32*v^5,-F(19)/69*x^5+2727*x^4*y-F(102)/113*x^3*y^2+F(121)/34*x^2*y^3+F(2)/77*x*y^4+7625*y^5-F(89)/20*x^4*z+14070*x^3*y*z+F(121)/29*x^2*y^2*z-13886*x*y^3*z-F(92)/61*y^4*z-F(103)/97*x^3*z^2-F(93)/17*x^2*y*z^2-8950*x*y^2*z^2-F(103)/78*y^3*z^2-14171*x^2*z^3+F(60)/119*x*y*z^3-F(41)/19*y^2*z^3-1906*x*z^4-9120*y*z^4-F(45)/32*z^5-601*x^4*u-4058*x^3*y*u-F(75)/29*x^2*y^2*u+F(27)/28*x*y^3*u-9215*y^4*u-F(29)/48*x^3*z*u+11229*x^2*y*z*u+4193*x*y^2*z*u+F(18)/35*y^3*z*u-F(98)/75*x^2*z^2*u-7843*x*y*z^2*u+F(119)/43*y^2*z^2*u+F(121)/94*x*z^3*u+F(26)/3*y*z^3*u+F(23)/49*z^4*u+3081*x^3*u^2-F(103)/85*x^2*y*u^2-3803*x*y^2*u^2-F(55)/16*y^3*u^2+F(82)/69*x^2*z*u^2-F(51)/40*x*y*z*u^2+7518*y^2*z*u^2-8199*x*z^2*u^2+F(96)/115*y*z^2*u^2-14467*z^3*u^2-F(93)/56*x^2*u^3-F(98)/27*x*y*u^3+F(98)/17*y^2*u^3-7864*x*z*u^3+F(44)/25*y*z*u^3-34*z^2*u^3-F(90)/77*x*u^4+5748*y*u^4+13957*z*u^4-12964*u^5+F(82)/33*x^4*v-F(57)/25*x^3*y*v-F(91)/8*x^2*y^2*v+F(123)/109*x*y^3*v+F(82)/93*y^4*v-F(120)/53*x^3*z*v+F(108)/61*x^2*y*z*v+4438*x*y^2*z*v-F(3)/31*y^3*z*v-7278*x^2*z^2*v-F(77)/82*x*y*z^2*v-F(59)/7*y^2*z^2*v+F(39)/112*x*z^3*v+F(18)/71*y*z^3*v-210*z^4*v-F(103)/86*x^3*u*v+F(71)/110*x^2*y*u*v+8510*x*y^2*u*v+13583*y^3*u*v-F(120)/79*x^2*z*u*v-1046*x*y*z*u*v-1126*y^2*z*u*v-8740*x*z^2*u*v-15628*y*z^2*u*v+F(76)/105*z^3*u*v+F(27)/14*x^2*u^2*v-F(92)/71*x*y*u^2*v-95*y^2*u^2*v+6827*x*z*u^2*v-10369*y*z*u^2*v-2271*z^2*u^2*v+F(43)/30*x*u^3*v+F(113)/83*y*u^3*v-F(25)/3*z*u^3*v-F(59)/29*u^4*v-F(121)/97*x^3*v^2+F(72)/29*x^2*y*v^2+6964*x*y^2*v^2+F(73)/90*y^3*v^2+174*x^2*z*v^2+11187*x*y*z*v^2+F(49)/20*y^2*z*v^2-F(47)/93*x*z^2*v^2-4823*y*z^2*v^2-7706*z^3*v^2+F(115)/48*x^2*u*v^2-15568*x*y*u*v^2-918*y^2*u*v^2-14386*x*z*u*v^2-2995*y*z*u*v^2-1038*z^2*u*v^2-F(80)/93*x*u^2*v^2+F(75)/29*y*u^2*v^2-8894*z*u^2*v^2-F(53)/58*u^3*v^2+6118*x^2*v^3-F(83)/28*x*y*v^3+6872*y^2*v^3-F(35)/19*x*z*v^3+F(25)/118*y*z*v^3-481*z^2*v^3-F(100)/61*x*u*v^3-F(53)/17*y*u*v^3-12292*z*u*v^3-4437*u^2*v^3+F(11)/115*x*v^4-193*y*v^4-13857*z*v^4-8475*u*v^4+F(108)/77*v^5,-4251*x^5+F(46)/41*x^4*y+F(23)/15*x^3*y^2+F(89)/7*x^2*y^3+F(83)/24*x*y^4-F(4)/99*y^5-F(95)/91*x^4*z-189*x^3*y*z+F(112)/115*x^2*y^2*z-5948*x*y^3*z+F(70)/109*y^4*z+F(7)/27*x^3*z^2-F(106)/49*x^2*y*z^2-7174*x*y^2*z^2+7053*y^3*z^2+14322*x^2*z^3-6046*x*y*z^3+F(36)/109*y^2*z^3+7535*x*z^4-F(94)/35*y*z^4+F(35)/103*z^5-13878*x^4*u-F(91)/103*x^3*y*u+126*x^2*y^2*u+7519*x*y^3*u-F(50)/99*y^4*u-F(112)/43*x^3*z*u+10290*x^2*y*z*u-14316*x*y^2*z*u+4388*y^3*z*u-10006*x^2*z^2*u+3697*x*y*z^2*u+F(53)/93*y^2*z^2*u-F(92)/43*x*z^3*u-9768*y*z^3*u-F(3)/113*z^4*u+F(28)/81*x^3*u^2+F(125)/9*x^2*y*u^2-12548*x*y^2*u^2+12175*y^3*u^2+F(13)/38*x^2*z*u^2-3242*x*y*z*u^2-7457*y^2*z*u^2-F(53)/22*x*z^2*u^2+F(11)/36*y*z^2*u^2-3569*z^3*u^2-F(35)/31*x^2*u^3-F(29)/91*x*y*u^3-F(104)/45*y^2*u^3-F(67)/35*x*z*u^3+F(13)/95*y*z*u^3-F(79)/27*z^2*u^3-F(118)/125*x*u^4+F(95)/56*y*u^4-F(56)/69*z*u^4+3378*u^5+8739*x^4*v+15310*x^3*y*v-10604*x^2*y^2*v+F(47)/32*x*y^3*v+12274*y^4*v+F(51)/113*x^3*z*v+F(48)/41*x^2*y*z*v+F(101)/23*x*y^2*z*v-608*y^3*z*v-13770*x^2*z^2*v-14233*x*y*z^2*v-F(81)/113*y^2*z^2*v+6041*x*z^3*v-F(115)/27*y*z^3*v+4590*z^4*v+F(67)/38*x^3*u*v+14861*x^2*y*u*v-F(2)/5*x*y^2*u*v+F(43)/46*y^3*u*v+1052*x^2*z*u*v-9359*x*y*z*u*v-F(109)/3*y^2*z*u*v+F(73)/120*x*z^2*u*v+14279*y*z^2*u*v+F(75)/17*z^3*u*v+F(77)/38*x^2*u^2*v+1491*x*y*u^2*v+F(61)/29*y^2*u^2*v-F(51)/25*x*z*u^2*v-1651*y*z*u^2*v+1008*z^2*u^2*v-12499*x*u^3*v+F(119)/90*y*u^3*v+F(4)/5*z*u^3*v+F(83)/95*u^4*v+6869*x^3*v^2+F(87)/52*x^2*y*v^2+1503*x*y^2*v^2-F(21)/80*y^3*v^2-2772*x^2*z*v^2-12467*x*y*z*v^2+F(58)/75*y^2*z*v^2-F(50)/59*x*z^2*v^2-4213*y*z^2*v^2-8739*z^3*v^2-F(31)/121*x^2*u*v^2+F(32)/109*x*y*u*v^2+1516*y^2*u*v^2-F(75)/86*x*z*u*v^2-F(57)/47*y*z*u*v^2+3151*z^2*u*v^2-F(126)/97*x*u^2*v^2-F(37)/53*y*u^2*v^2-9747*z*u^2*v^2+F(7)/103*u^3*v^2-F(22)/103*x^2*v^3+F(31)/119*x*y*v^3-F(9)/104*y^2*v^3-F(125)/117*x*z*v^3-F(64)/21*y*z*v^3-13259*z^2*v^3-1045*x*u*v^3+10806*y*u*v^3+2869*z*u*v^3-80*u^2*v^3+F(55)/79*x*v^4-F(53)/22*y*v^4+2268*z*v^4-F(35)/38*u*v^4+4059*v^5,-F(22)/95*x^5-F(3)/82*x^4*y-F(81)/103*x^3*y^2-F(103)/4*x^2*y^3+F(43)/101*x*y^4+11907*y^5-9538*x^4*z-9616*x^3*y*z-5682*x^2*y^2*z+F(7)/13*x*y^3*z-F(49)/90*y^4*z+F(32)/21*x^3*z^2+3977*x^2*y*z^2+F(85)/54*x*y^2*z^2-F(37)/97*y^3*z^2-3530*x^2*z^3-F(79)/70*x*y*z^3+F(27)/94*y^2*z^3-F(125)/53*x*z^4+F(109)/60*y*z^4+F(37)/66*z^5+F(12)/85*x^4*u-F(97)/52*x^3*y*u+F(11)/106*x^2*y^2*u+10110*x*y^3*u-F(7)/3*y^4*u-F(65)/109*x^3*z*u+F(86)/27*x^2*y*z*u-2929*x*y^2*z*u+F(35)/74*y^3*z*u+F(91)/106*x^2*z^2*u-F(20)/107*x*y*z^2*u+F(36)/77*y^2*z^2*u-6687*x*z^3*u-F(32)/89*y*z^3*u+F(61)/65*z^4*u-F(7)/17*x^3*u^2-2159*x^2*y*u^2-10729*x*y^2*u^2+F(100)/101*y^3*u^2+192*x^2*z*u^2+7957*x*y*z*u^2+F(111)/71*y^2*z*u^2+F(71)/50*x*z^2*u^2-F(65)/116*y*z^2*u^2+247*z^3*u^2+5572*x^2*u^3-12671*x*y*u^3-148*y^2*u^3-F(70)/39*x*z*u^3-F(109)/41*y*z*u^3+10024*z^2*u^3+2227*x*u^4-F(53)/110*y*u^4+F(116)/21*z*u^4+9448*u^5+F(3)/47*x^4*v+F(111)/124*x^3*y*v-4832*x^2*y^2*v-F(119)/59*x*y^3*v-7777*y^4*v+F(97)/109*x^3*z*v+F(82)/51*x^2*y*z*v+F(87)/17*x*y^2*z*v+F(116)/79*y^3*z*v-F(13)/2*x^2*z^2*v-F(80)/13*x*y*z^2*v-4960*y^2*z^2*v-1990*x*z^3*v-14244*y*z^3*v+F(108)/59*z^4*v+10251*x^3*u*v+5974*x^2*y*u*v+F(92)/47*x*y^2*u*v-14711*y^3*u*v+13560*x^2*z*u*v+F(98)/97*x*y*z*u*v-F(116)/23*y^2*z*u*v+F(123)/106*x*z^2*u*v+14630*y*z^2*u*v-F(74)/45*z^3*u*v+F(105)/17*x^2*u^2*v-2598*x*y*u^2*v+F(126)/37*y^2*u^2*v-F(84)/85*x*z*u^2*v-F(84)/71*y*z*u^2*v+7776*z^2*u^2*v+F(7)/81*x*u^3*v+F(47)/14*y*u^3*v+F(92)/29*z*u^3*v+14477*u^4*v+F(92)/67*x^3*v^2-F(45)/97*x^2*y*v^2+2249*x*y^2*v^2+8057*y^3*v^2-3833*x^2*z*v^2+F(14)/61*x*y*z*v^2+F(35)/94*y^2*z*v^2-F(84)/113*x*z^2*v^2+F(122)/57*y*z^2*v^2+F(21)/115*z^3*v^2-3530*x^2*u*v^2-F(73)/36*x*y*u*v^2+F(1)/75*y^2*u*v^2-F(104)/89*x*z*u*v^2+F(100)/73*y*z*u*v^2-9575*z^2*u*v^2-F(95)/107*x*u^2*v^2-F(44)/81*y*u^2*v^2+10619*z*u^2*v^2+F(126)/97*u^3*v^2+6076*x^2*v^3+F(25)/2*x*y*v^3-15029*y^2*v^3-992*x*z*v^3+F(99)/74*y*z*v^3+F(38)/59*z^2*v^3+F(77)/32*x*u*v^3+836*y*u*v^3-F(98)/125*z*u*v^3-F(74)/75*u^2*v^3+F(29)/84*x*v^4-F(7)/33*y*v^4-F(19)/93*z*v^4-F(76)/13*u*v^4+F(94)/71*v^5,-F(73)/83*x^5-F(52)/31*x^4*y+F(51)/67*x^3*y^2-F(91)/18*x^2*y^3-F(9)/25*x*y^4-9266*y^5+F(71)/99*x^4*z-F(3)/89*x^3*y*z+F(107)/6*x^2*y^2*z-F(79)/99*x*y^3*z+F(17)/52*y^4*z+13073*x^3*z^2-3782*x^2*y*z^2+11509*x*y^2*z^2-9495*y^3*z^2-F(53)/110*x^2*z^3-F(106)/87*x*y*z^3+193*y^2*z^3+F(86)/45*x*z^4+8353*y*z^4+F(49)/115*z^5+9036*x^4*u+F(7)/55*x^3*y*u+8222*x^2*y^2*u-F(70)/123*x*y^3*u+F(76)/85*y^4*u-1764*x^3*z*u-10171*x^2*y*z*u+15790*x*y^2*z*u-3346*y^3*z*u-6102*x^2*z^2*u+F(93)/16*x*y*z^2*u+F(32)/123*y^2*z^2*u+7959*x*z^3*u+F(60)/49*y*z^3*u+F(104)/7*z^4*u+F(4)/15*x^3*u^2-F(97)/73*x^2*y*u^2-F(46)/71*x*y^2*u^2-13985*y^3*u^2+F(59)/58*x^2*z*u^2-9118*x*y*z*u^2-9435*y^2*z*u^2+5127*x*z^2*u^2-2447*y*z^2*u^2-F(1)/24*z^3*u^2-5939*x^2*u^3-F(103)/4*x*y*u^3-F(120)/29*y^2*u^3+F(89)/91*x*z*u^3-5917*y*z*u^3+F(57)/64*z^2*u^3-12038*x*u^4+F(13)/107*y*u^4+F(79)/4*z*u^4-5466*u^5-F(83)/40*x^4*v+F(83)/5*x^3*y*v-F(77)/61*x^2*y^2*v+F(101)/44*x*y^3*v-F(101)/110*y^4*v-F(56)/113*x^3*z*v-F(52)/87*x^2*y*z*v+F(38)/123*x*y^2*z*v+6504*y^3*z*v-8395*x^2*z^2*v-F(107)/49*x*y*z^2*v+4545*y^2*z^2*v+F(33)/8*x*z^3*v+8712*y*z^3*v+5879*z^4*v+14862*x^3*u*v-F(103)/6*x^2*y*u*v-F(19)/41*x*y^2*u*v-14433*y^3*u*v+F(10)/101*x^2*z*u*v-F(47)/114*x*y*z*u*v-F(47)/31*y^2*z*u*v-4877*x*z^2*u*v+F(95)/17*y*z^2*u*v+F(115)/123*z^3*u*v+F(58)/59*x^2*u^2*v-1080*x*y*u^2*v+14306*y^2*u^2*v+F(41)/126*x*z*u^2*v+9175*y*z*u^2*v+F(97)/20*z^2*u^2*v+10477*x*u^3*v-F(78)/7*y*u^3*v+15282*z*u^3*v+12089*u^4*v-10985*x^3*v^2-F(47)/81*x^2*y*v^2+F(49)/16*x*y^2*v^2-F(53)/79*y^3*v^2-2022*x^2*z*v^2+10614*x*y*z*v^2-8335*y^2*z*v^2-3233*x*z^2*v^2-716*y*z^2*v^2-6658*z^3*v^2+9045*x^2*u*v^2-3220*x*y*u*v^2-F(83)/80*y^2*u*v^2+9638*x*z*u*v^2+F(105)/106*y*z*u*v^2-F(93)/11*z^2*u*v^2+15370*x*u^2*v^2+F(81)/56*y*u^2*v^2+13888*z*u^2*v^2+F(13)/2*u^3*v^2-F(33)/29*x^2*v^3-4791*x*y*v^3-5255*y^2*v^3+F(33)/10*x*z*v^3+F(43)/105*y*z*v^3-7814*z^2*v^3+2053*x*u*v^3+F(103)/88*y*u*v^3-453*z*u*v^3-F(106)/93*u^2*v^3+8037*x*v^4+9573*y*v^4+F(80)/117*z*v^4+15199*u*v^4+2014*v^5,F(1)/71*x^4*y+F(21)/10*x^3*y^2-F(26)/51*x^2*y^3+F(103)/35*x*y^4+4251*y^5-13639*x^4*z+F(87)/26*x^3*y*z-9729*x^2*y^2*z+14618*x*y^3*z-F(34)/25*y^4*z+F(4)/87*x^3*z^2-F(79)/27*x^2*y*z^2+F(30)/91*x*y^2*z^2-8047*y^3*z^2-F(105)/94*x^2*z^3-2890*x*y*z^3-F(39)/79*y^2*z^3-F(89)/93*x*z^4+F(29)/124*y*z^4+F(126)/67*z^5+8173*x^4*u-F(71)/121*x^3*y*u+F(49)/122*x^2*y^2*u+12038*x*y^3*u-F(114)/11*y^4*u+F(55)/23*x^3*z*u+F(99)/112*x^2*y*z*u+F(123)/44*x*y^2*z*u+F(39)/71*y^3*z*u-2809*x^2*z^2*u+7234*x*y*z^2*u-F(49)/74*y^2*z^2*u-6903*x*z^3*u+F(18)/13*y*z^3*u-3169*z^4*u-F(123)/19*x^3*u^2-F(71)/92*x^2*y*u^2+15227*x*y^2*u^2+F(80)/47*y^3*u^2+2804*x^2*z*u^2-F(117)/31*x*y*z*u^2-F(77)/73*y^2*z*u^2+F(1)/15*x*z^2*u^2-F(71)/91*y*z^2*u^2-F(34)/11*z^3*u^2-F(98)/39*x^2*u^3+2708*x*y*u^3+F(15)/44*y^2*u^3+12475*x*z*u^3+F(34)/99*y*z*u^3-F(115)/66*z^2*u^3-10858*x*u^4-10231*y*u^4+F(94)/55*z*u^4-F(109)/41*u^5+5704*x^4*v+10913*x^3*y*v-1733*x^2*y^2*v-F(79)/117*x*y^3*v+11909*y^4*v-14059*x^3*z*v+F(36)/17*x^2*y*z*v+1109*x*y^2*z*v+F(107)/47*y^3*z*v-3969*x^2*z^2*v-F(47)/121*x*y*z^2*v+1622*y^2*z^2*v+66*x*z^3*v+13732*y*z^3*v+9983*z^4*v+F(22)/111*x^3*u*v+F(99)/85*x^2*y*u*v+F(74)/87*x*y^2*u*v+F(107)/48*y^3*u*v+F(23)/67*x^2*z*u*v-F(92)/81*x*y*z*u*v-F(101)/27*y^2*z*u*v-F(97)/103*x*z^2*u*v+6803*y*z^2*u*v-F(47)/54*z^3*u*v-F(109)/112*x^2*u^2*v-7584*x*y*u^2*v+F(27)/56*y^2*u^2*v+F(84)/109*x*z*u^2*v+F(59)/100*y*z*u^2*v-F(39)/10*z^2*u^2*v+9336*x*u^3*v+F(62)/35*y*u^3*v-F(44)/53*z*u^3*v-F(103)/106*u^4*v+4043*x^3*v^2+F(93)/14*x^2*y*v^2-7218*x*y^2*v^2-9102*y^3*v^2+7065*x^2*z*v^2-3973*x*y*z*v^2+F(68)/37*y^2*z*v^2+F(96)/11*x*z^2*v^2-14299*y*z^2*v^2-F(13)/47*z^3*v^2+12528*x^2*u*v^2-14102*x*y*u*v^2-12707*y^2*u*v^2+F(56)/33*x*z*u*v^2-F(59)/96*y*z*u*v^2-F(124)/33*z^2*u*v^2+8407*x*u^2*v^2+F(82)/125*y*u^2*v^2+F(83)/110*z*u^2*v^2-F(29)/83*u^3*v^2-7065*x^2*v^3+12141*x*y*v^3-13552*y^2*v^3-12917*x*z*v^3-F(76)/119*y*z*v^3+11838*z^2*v^3+F(109)/64*x*u*v^3-7095*y*u*v^3-3845*z*u*v^3+F(66)/43*u^2*v^3+F(105)/73*x*v^4-F(113)/45*y*v^4+15928*z*v^4-F(55)/61*u*v^4-6225*v^5,9295*x^4*y+F(121)/120*x^3*y^2-9748*x^2*y^3+11659*x*y^4-F(113)/99*y^5+F(115)/26*x^4*z+2335*x^3*y*z+F(36)/29*x^2*y^2*z-F(113)/6*x*y^3*z-F(103)/76*y^4*z+F(78)/37*x^3*z^2-F(47)/38*x^2*y*z^2-F(70)/13*x*y^2*z^2-568*y^3*z^2+11765*x^2*z^3-F(94)/73*x*y*z^3+5800*y^2*z^3-8448*x*z^4+13583*y*z^4+F(53)/70*z^5-F(5)/117*x^4*u+F(89)/65*x^3*y*u-15063*x^2*y^2*u+8038*x*y^3*u+F(41)/29*y^4*u-F(81)/49*x^3*z*u+5778*x^2*y*z*u+F(123)/73*x*y^2*z*u-F(51)/16*y^3*z*u+F(85)/14*x^2*z^2*u-F(93)/89*x*y*z^2*u+F(29)/101*y^2*z^2*u+F(8)/39*x*z^3*u-F(44)/53*y*z^3*u+F(31)/87*z^4*u+F(121)/119*x^3*u^2-7340*x^2*y*u^2+6048*x*y^2*u^2+F(71)/68*y^3*u^2+F(91)/23*x^2*z*u^2-F(61)/75*x*y*z*u^2+8502*y^2*z*u^2-8031*x*z^2*u^2-F(109)/41*y*z^2*u^2+14375*z^3*u^2-3378*x^2*u^3-5193*x*y*u^3+F(44)/31*y^2*u^3+5547*x*z*u^3-3626*y*z*u^3+8407*z^2*u^3+F(24)/25*x*u^4-F(43)/36*y*u^4+7777*z*u^4-F(39)/40*u^5+87*x^4*v+F(108)/55*x^3*y*v-F(37)/101*x^2*y^2*v-8636*x*y^3*v+12378*y^4*v-F(42)/19*x^3*z*v-4731*x^2*y*z*v-F(17)/81*x*y^2*z*v+F(125)/106*y^3*z*v-1607*x^2*z^2*v+F(121)/7*x*y*z^2*v-F(121)/26*y^2*z^2*v+3360*x*z^3*v+11888*y*z^3*v+F(87)/119*z^4*v+F(39)/14*x^3*u*v-14107*x^2*y*u*v+F(9)/40*x*y^2*u*v+1445*y^3*u*v+1119*x^2*z*u*v+F(113)/83*x*y*z*u*v-F(118)/27*y^2*z*u*v+2418*x*z^2*u*v+F(108)/19*y*z^2*u*v-F(4)/123*z^3*u*v+F(102)/67*x^2*u^2*v+F(12)/49*x*y*u^2*v-F(32)/47*y^2*u^2*v+F(115)/106*x*z*u^2*v-F(19)/123*y*z*u^2*v+4676*z^2*u^2*v+9336*x*u^3*v+F(105)/67*y*u^3*v-7730*z*u^3*v+F(101)/3*u^4*v-F(47)/98*x^3*v^2-F(63)/41*x^2*y*v^2+1742*x*y^2*v^2+16*y^3*v^2+F(111)/98*x^2*z*v^2+7216*x*y*z*v^2+F(31)/87*y^2*z*v^2-F(93)/101*x*z^2*v^2-F(23)/51*y*z^2*v^2+15913*z^3*v^2-2451*x^2*u*v^2+F(52)/55*x*y*u*v^2+12474*y^2*u*v^2+F(85)/66*x*z*u*v^2-F(56)/117*y*z*u*v^2-F(68)/67*z^2*u*v^2+7964*x*u^2*v^2+9668*y*u^2*v^2+F(72)/67*z*u^2*v^2-F(7)/86*u^3*v^2-5241*x^2*v^3+F(79)/89*x*y*v^3-2197*y^2*v^3-F(38)/47*x*z*v^3+F(99)/97*y*z*v^3+F(62)/121*z^2*v^3+137*x*u*v^3+F(103)/24*y*u*v^3+F(100)/83*z*u*v^3+11913*u^2*v^3-F(2)/97*x*v^4-14526*y*v^4+6582*z*v^4-F(12)/49*u*v^4+10973*v^5,12014*x^4*y+F(73)/54*x^3*y^2-1545*x^2*y^3+F(125)/39*x*y^4-6780*y^5+13602*x^4*z-F(93)/77*x^3*y*z+9465*x^2*y^2*z+795*x*y^3*z-F(26)/27*y^4*z-14845*x^3*z^2+F(49)/60*x^2*y*z^2+F(118)/95*x*y^2*z^2-295*y^3*z^2+10013*x^2*z^3+F(74)/29*x*y*z^3-14119*y^2*z^3-F(3)/43*x*z^4-F(6)/97*y*z^4+F(111)/86*z^5-F(28)/83*x^4*u+F(43)/88*x^3*y*u+F(13)/28*x^2*y^2*u+14442*x*y^3*u-F(17)/15*y^4*u-F(3)/112*x^3*z*u+F(107)/55*x^2*y*z*u-F(64)/39*x*y^2*z*u-4536*y^3*z*u+2726*x^2*z^2*u+9668*x*y*z^2*u+F(89)/31*y^2*z^2*u-F(39)/125*x*z^3*u-F(92)/5*y*z^3*u+13919*z^4*u-15272*x^3*u^2-7278*x^2*y*u^2-9325*x*y^2*u^2-F(103)/108*y^3*u^2-F(81)/11*x^2*z*u^2+F(63)/73*x*y*z*u^2+3920*y^2*z*u^2+F(13)/40*x*z^2*u^2+15421*y*z^2*u^2+F(17)/104*z^3*u^2-15913*x^2*u^3-12845*x*y*u^3+F(121)/108*y^2*u^3-F(95)/22*x*z*u^3-F(93)/34*y*z*u^3+F(109)/11*z^2*u^3-F(58)/3*x*u^4-3920*y*u^4+F(23)/62*z*u^4+5603*u^5-F(81)/28*x^4*v-7502*x^3*y*v-F(84)/71*x^2*y^2*v-F(123)/101*x*y^3*v-F(91)/124*y^4*v-3698*x^3*z*v+6843*x^2*y*z*v+8232*x*y^2*z*v+7240*y^3*z*v-F(69)/94*x^2*z^2*v+F(30)/67*x*y*z^2*v+14784*y^2*z^2*v-38*x*z^3*v-F(65)/109*y*z^3*v+10515*z^4*v+F(61)/78*x^3*u*v-F(3)/103*x^2*y*u*v-F(27)/61*x*y^2*u*v-649*y^3*u*v+F(47)/13*x^2*z*u*v+6681*x*y*z*u*v+6526*y^2*z*u*v-8543*x*z^2*u*v+F(113)/7*y*z^2*u*v-F(125)/34*z^3*u*v+F(41)/28*x^2*u^2*v-13231*x*y*u^2*v-F(22)/27*y^2*u^2*v+15913*x*z*u^2*v+F(76)/15*y*z*u^2*v+F(82)/77*z^2*u^2*v-F(63)/22*x*u^3*v+15917*y*u^3*v-6249*z*u^3*v+8136*u^4*v+F(2)/119*x^3*v^2+F(54)/103*x^2*y*v^2-1673*x*y^2*v^2+F(13)/30*y^3*v^2-3724*x^2*z*v^2-5371*x*y*z*v^2-F(73)/82*y^2*z*v^2+F(82)/57*x*z^2*v^2+F(107)/20*y*z^2*v^2+F(116)/73*z^3*v^2+9220*x^2*u*v^2+F(71)/49*x*y*u*v^2+157*y^2*u*v^2+F(15)/37*x*z*u*v^2+13529*y*z*u*v^2-15810*z^2*u*v^2+1533*x*u^2*v^2+F(115)/54*y*u^2*v^2-F(94)/11*z*u^2*v^2+10016*u^3*v^2-F(114)/35*x^2*v^3-9121*x*y*v^3+11939*y^2*v^3-9797*x*z*v^3+F(33)/82*y*z*v^3+10501*z^2*v^3+14595*x*u*v^3-2677*y*u*v^3+7028*z*u*v^3-F(83)/8*u^2*v^3+F(28)/27*x*v^4+738*y*v^4+F(17)/98*z*v^4-F(95)/34*u*v^4-F(121)/51*v^5,-F(44)/101*x^4*y+F(52)/69*x^3*y^2+F(29)/45*x^2*y^3-F(43)/85*x*y^4-F(90)/23*y^5-F(11)/20*x^4*z-F(11)/47*x^3*y*z-F(40)/29*x^2*y^2*z-14687*x*y^3*z-F(91)/20*y^4*z-F(82)/105*x^3*z^2+F(122)/35*x^2*y*z^2-12828*x*y^2*z^2+F(35)/54*y^3*z^2-F(111)/70*x^2*z^3-10610*x*y*z^3+13231*y^2*z^3-5495*x*z^4+9037*y*z^4-F(31)/16*z^5-14961*x^4*u+8952*x^3*y*u-F(65)/89*x^2*y^2*u+9583*x*y^3*u-F(53)/88*y^4*u-F(78)/43*x^3*z*u-F(93)/118*x^2*y*z*u+F(93)/47*x*y^2*z*u-1310*y^3*z*u+F(98)/69*x^2*z^2*u-F(50)/21*x*y*z^2*u-F(76)/77*y^2*z^2*u+3287*x*z^3*u+F(53)/20*y*z^3*u+1675*z^4*u-F(90)/53*x^3*u^2-12197*x^2*y*u^2-F(12)/49*x*y^2*u^2-15894*y^3*u^2+12152*x^2*z*u^2-8791*x*y*z*u^2+3126*y^2*z*u^2+2893*x*z^2*u^2-5296*y*z^2*u^2+F(39)/85*z^3*u^2-14477*x^2*u^3+F(23)/93*x*y*u^3-F(85)/53*y^2*u^3-F(83)/89*x*z*u^3-F(33)/98*y*z*u^3-F(61)/109*z^2*u^3+F(98)/99*x*u^4+F(56)/45*y*u^4+3268*z*u^4-3443*u^5-5521*x^4*v-F(94)/109*x^3*y*v-F(107)/102*x^2*y^2*v+4459*x*y^3*v-14749*y^4*v+F(11)/105*x^3*z*v+F(38)/37*x^2*y*z*v-2300*x*y^2*z*v-F(52)/105*y^3*z*v+F(28)/57*x^2*z^2*v+14876*x*y*z^2*v-F(73)/40*y^2*z^2*v-5671*x*z^3*v+5748*y*z^3*v+F(52)/35*z^4*v+F(32)/81*x^3*u*v+F(57)/22*x^2*y*u*v+F(75)/58*x*y^2*u*v-F(22)/115*y^3*u*v-F(95)/44*x^2*z*u*v-F(71)/61*x*y*z*u*v+7207*y^2*z*u*v-F(115)/57*x*z^2*u*v-488*y*z^2*u*v+10172*z^3*u*v+11827*x^2*u^2*v-F(39)/76*x*y*u^2*v+F(49)/46*y^2*u^2*v+F(31)/74*x*z*u^2*v-4024*y*z*u^2*v-11575*z^2*u^2*v+4187*x*u^3*v-F(49)/62*y*u^3*v-2083*z*u^3*v-366*u^4*v+11436*x^3*v^2+F(65)/111*x^2*y*v^2-F(37)/24*x*y^2*v^2+F(55)/96*y^3*v^2-F(71)/107*x^2*z*v^2-15790*x*y*z*v^2+F(17)/125*y^2*z*v^2+F(25)/94*x*z^2*v^2+F(55)/19*y*z^2*v^2+F(79)/9*z^3*v^2+14386*x^2*u*v^2-F(124)/125*x*y*u*v^2-2142*y^2*u*v^2+5449*x*z*u*v^2-7370*y*z*u*v^2+F(33)/16*z^2*u*v^2+F(80)/71*x*u^2*v^2+5656*y*u^2*v^2-F(93)/77*z*u^2*v^2-F(2)/71*u^3*v^2+F(43)/69*x^2*v^3-F(51)/52*x*y*v^3-F(67)/108*y^2*v^3+F(49)/12*x*z*v^3+F(38)/53*y*z*v^3-6432*z^2*v^3+F(43)/44*x*u*v^3+F(10)/101*y*u*v^3+F(45)/119*z*u*v^3-F(116)/23*u^2*v^3-F(71)/92*x*v^4-9225*y*v^4+F(9)/46*z*v^4-15012*u*v^4-F(71)/50*v^5,-F(17)/25*x^4*y+13563*x^3*y^2-F(24)/95*x^2*y^3-8301*x*y^4-2750*y^5-F(15)/92*x^4*z-F(119)/9*x^3*y*z-15065*x^2*y^2*z+F(106)/5*x*y^3*z-F(36)/97*y^4*z+F(107)/84*x^3*z^2+9051*x^2*y*z^2-F(61)/101*x*y^2*z^2+13818*y^3*z^2-F(106)/91*x^2*z^3+11649*x*y*z^3+3290*y^2*z^3+2038*x*z^4-988*y*z^4+5081*z^5+117*x^4*u-F(49)/108*x^3*y*u-F(10)/79*x^2*y^2*u-11827*x*y^3*u+12462*y^4*u+F(43)/13*x^3*z*u-9862*x^2*y*z*u+F(110)/27*x*y^2*z*u-9*y^3*z*u+F(79)/44*x^2*z^2*u-F(71)/75*x*y*z^2*u-F(43)/74*y^2*z^2*u+F(120)/41*x*z^3*u-F(85)/93*y*z^3*u+7704*z^4*u+F(22)/89*x^3*u^2+5362*x^2*y*u^2-F(27)/52*x*y^2*u^2-F(67)/44*y^3*u^2+F(83)/32*x^2*z*u^2-7411*x*y*z*u^2+168*y^2*z*u^2+10594*x*z^2*u^2-3846*y*z^2*u^2+1577*z^3*u^2+F(58)/91*x^2*u^3+F(101)/98*x*y*u^3-7029*y^2*u^3-7445*x*z*u^3-9921*y*z*u^3-F(60)/47*z^2*u^3+14124*x*u^4-10579*y*u^4-F(111)/106*z*u^4+4543*u^5-2016*x^4*v-F(120)/61*x^3*y*v+4734*x^2*y^2*v-8541*x*y^3*v-F(61)/77*y^4*v+7352*x^3*z*v-11479*x^2*y*z*v+8043*x*y^2*z*v-F(21)/20*y^3*z*v+F(121)/54*x^2*z^2*v-F(77)/37*x*y*z^2*v-F(41)/105*y^2*z^2*v-796*x*z^3*v-1963*y*z^3*v+F(63)/20*z^4*v+F(98)/117*x^3*u*v+F(17)/82*x^2*y*u*v+F(104)/95*x*y^2*u*v+F(59)/26*y^3*u*v-2406*x^2*z*u*v-4256*x*y*z*u*v+F(91)/99*y^2*z*u*v-919*x*z^2*u*v+12500*y*z^2*u*v+F(1)/72*z^3*u*v+8311*x^2*u^2*v+6234*x*y*u^2*v-F(70)/83*y^2*u^2*v-F(113)/67*x*z*u^2*v+5003*y*z*u^2*v+8834*z^2*u^2*v-F(28)/61*x*u^3*v+7299*y*u^3*v-F(89)/54*z*u^3*v-F(97)/58*u^4*v-F(115)/121*x^3*v^2-F(23)/25*x^2*y*v^2+2323*x*y^2*v^2-F(118)/73*y^3*v^2-F(89)/98*x^2*z*v^2-F(53)/23*x*y*z*v^2-11279*y^2*z*v^2+F(56)/79*x*z^2*v^2-5636*y*z^2*v^2-11209*z^3*v^2-6104*x^2*u*v^2-F(5)/103*x*y*u*v^2+F(77)/18*y^2*u*v^2+15398*x*z*u*v^2-14703*y*z*u*v^2+F(112)/41*z^2*u*v^2+F(56)/125*x*u^2*v^2+918*y*u^2*v^2+14385*z*u^2*v^2+1382*u^3*v^2+F(82)/57*x^2*v^3-11558*x*y*v^3+F(87)/119*y^2*v^3+10275*x*z*v^3-1374*y*z*v^3-F(93)/85*z^2*v^3-F(67)/106*x*u*v^3+F(17)/110*y*u*v^3-14520*z*u*v^3+F(91)/9*u^2*v^3+F(96)/67*x*v^4+11048*y*v^4+F(92)/117*z*v^4-F(83)/60*u*v^4-F(115)/123*v^5,-6700*x^4*y+6186*x^3*y^2-F(49)/75*x^2*y^3-F(75)/47*x*y^4-1414*y^5+F(19)/17*x^4*z+6725*x^3*y*z+10186*x^2*y^2*z+919*x*y^3*z+F(77)/19*y^4*z-11594*x^3*z^2+5496*x^2*y*z^2-F(59)/72*x*y^2*z^2-F(83)/8*y^3*z^2+50*x^2*z^3+5004*x*y*z^3-F(89)/11*y^2*z^3+F(10)/91*x*z^4+11334*y*z^4-3729*z^5-3079*x^4*u+F(53)/57*x^3*y*u+F(41)/47*x^2*y^2*u-14601*x*y^3*u-F(43)/44*y^4*u-F(78)/47*x^3*z*u-15643*x^2*y*z*u-F(11)/91*x*y^2*z*u+F(101)/37*y^3*z*u+F(1)/9*x^2*z^2*u-13443*x*y*z^2*u+61*y^2*z^2*u-4221*x*z^3*u-F(98)/13*y*z^3*u+5531*z^4*u+F(19)/10*x^3*u^2+8198*x^2*y*u^2-F(70)/107*x*y^2*u^2-F(102)/121*y^3*u^2+8608*x^2*z*u^2-F(29)/28*x*y*z*u^2-5736*y^2*z*u^2+F(59)/22*x*z^2*u^2-13317*y*z^2*u^2+6011*z^3*u^2-3396*x^2*u^3+10885*x*y*u^3-F(78)/97*y^2*u^3-120*x*z*u^3-11135*y*z*u^3+F(52)/23*z^2*u^3+F(5)/22*x*u^4-F(121)/37*y*u^4-F(17)/49*z*u^4-F(63)/23*u^5-F(114)/11*x^4*v-12139*x^3*y*v-F(73)/104*x^2*y^2*v+F(91)/38*x*y^3*v-F(5)/72*y^4*v+2377*x^3*z*v+13528*x^2*y*z*v+7153*x*y^2*z*v-F(101)/12*y^3*z*v-F(115)/18*x^2*z^2*v+F(93)/41*x*y*z^2*v-8597*y^2*z^2*v-15415*x*z^3*v-F(115)/47*y*z^3*v-F(89)/121*z^4*v+7839*x^3*u*v+15627*x^2*y*u*v+F(103)/34*x*y^2*u*v+F(109)/107*y^3*u*v+F(19)/93*x^2*z*u*v+7557*x*y*z*u*v-3269*y^2*z*u*v+F(94)/31*x*z^2*u*v+12929*y*z^2*u*v-F(77)/114*z^3*u*v+F(96)/125*x^2*u^2*v+F(61)/93*x*y*u^2*v+4961*y^2*u^2*v+8740*x*z*u^2*v+F(72)/19*y*z*u^2*v-8462*z^2*u^2*v-7125*x*u^3*v+2029*y*u^3*v-F(104)/15*z*u^3*v+F(57)/31*u^4*v+10188*x^3*v^2-F(93)/70*x^2*y*v^2-1461*x*y^2*v^2+11346*y^3*v^2-F(121)/37*x^2*z*v^2-F(67)/93*x*y*z*v^2+F(71)/10*y^2*z*v^2+10412*x*z^2*v^2+8427*y*z^2*v^2+11748*z^3*v^2+F(122)/3*x^2*u*v^2-10058*x*y*u*v^2+3509*y^2*u*v^2-8427*x*z*u*v^2+9701*y*z*u*v^2-F(83)/60*z^2*u*v^2-F(41)/54*x*u^2*v^2+F(112)/51*y*u^2*v^2-F(22)/81*z*u^2*v^2+2502*u^3*v^2+F(1)/93*x^2*v^3-13028*x*y*v^3-F(73)/72*y^2*v^3-F(15)/106*x*z*v^3+6177*y*z*v^3-F(27)/19*z^2*v^3+14200*x*u*v^3-12499*y*u*v^3-8762*z*u*v^3-F(38)/25*u^2*v^3+F(53)/67*x*v^4-F(104)/21*y*v^4+14513*z*v^4-F(88)/13*u*v^4+F(29)/63*v^5,-15568*x^4*y+F(116)/63*x^3*y^2-F(125)/26*x^2*y^3+F(20)/83*x*y^4+F(107)/126*y^5-F(83)/122*x^4*z-15340*x^3*y*z-F(37)/63*x^2*y^2*z+F(54)/53*x*y^3*z-F(101)/77*y^4*z-F(73)/71*x^3*z^2-13354*x^2*y*z^2+42*x*y^2*z^2+F(105)/41*y^3*z^2+F(119)/93*x^2*z^3+F(58)/23*x*y*z^3-12481*y^2*z^3+F(87)/77*x*z^4+11081*y*z^4+F(97)/35*z^5+F(19)/47*x^4*u-12504*x^3*y*u-6448*x^2*y^2*u+F(123)/125*x*y^3*u-11180*y^4*u-F(87)/43*x^3*z*u+F(27)/52*x^2*y*z*u-F(83)/4*x*y^2*z*u+3626*y^3*z*u+F(75)/77*x^2*z^2*u-F(97)/31*x*y*z^2*u+F(36)/17*y^2*z^2*u+15465*x*z^3*u-4676*y*z^3*u-15916*z^4*u+15878*x^3*u^2+F(47)/12*x^2*y*u^2+1531*x*y^2*u^2-F(47)/116*y^3*u^2+10333*x^2*z*u^2+F(26)/11*x*y*z*u^2+9224*y^2*z*u^2-F(23)/53*x*z^2*u^2-F(47)/38*y*z^2*u^2+F(35)/94*z^3*u^2-10231*x^2*u^3-F(60)/101*x*y*u^3-F(93)/110*y^2*u^3+9892*x*z*u^3-8037*y*z*u^3-F(96)/13*z^2*u^3+2248*x*u^4-9118*y*u^4+F(47)/70*z*u^4+F(92)/45*u^5+F(9)/94*x^4*v-F(114)/101*x^3*y*v-F(23)/18*x^2*y^2*v+F(57)/94*x*y^3*v+1949*y^4*v+F(123)/64*x^3*z*v-F(69)/95*x^2*y*z*v-6305*x*y^2*z*v-F(82)/79*y^3*z*v-F(89)/46*x^2*z^2*v-10*x*y*z^2*v+F(59)/92*y^2*z^2*v+F(64)/63*x*z^3*v-F(123)/43*y*z^3*v-F(65)/116*z^4*v+7515*x^3*u*v+F(85)/67*x^2*y*u*v+F(2)/97*x*y^2*u*v+11304*y^3*u*v+F(93)/41*x^2*z*u*v+7869*x*y*z*u*v-1005*y^2*z*u*v+4469*x*z^2*u*v-F(61)/11*y*z^2*u*v-F(13)/60*z^3*u*v-F(76)/21*x^2*u^2*v+F(109)/76*x*y*u^2*v-6772*y^2*u^2*v+15045*x*z*u^2*v-8261*y*z*u^2*v-8619*z^2*u^2*v+F(64)/75*x*u^3*v-F(114)/103*y*u^3*v-10713*z*u^3*v-1373*u^4*v+F(66)/107*x^3*v^2+F(102)/79*x^2*y*v^2+15568*x*y^2*v^2-F(44)/95*y^3*v^2-F(13)/28*x^2*z*v^2+F(51)/92*x*y*z*v^2+14730*y^2*z*v^2+F(61)/4*x*z^2*v^2+F(43)/10*y*z^2*v^2-14468*z^3*v^2+F(104)/115*x^2*u*v^2-3302*x*y*u*v^2+F(62)/89*y^2*u*v^2-F(102)/29*x*z*u*v^2+F(117)/17*y*z*u*v^2-8164*z^2*u*v^2+F(79)/120*x*u^2*v^2+F(2)/83*y*u^2*v^2-F(5)/57*z*u^2*v^2+F(99)/50*u^3*v^2-F(89)/28*x^2*v^3-F(53)/93*x*y*v^3-F(100)/99*y^2*v^3+F(68)/49*x*z*v^3-F(67)/37*y*z*v^3+2650*z^2*v^3+6991*x*u*v^3+F(28)/83*y*u*v^3-13426*z*u*v^3+F(119)/44*u^2*v^3+F(113)/20*x*v^4-707*y*v^4+F(7)/39*z*v^4+6761*u*v^4-F(25)/104*v^5,-F(75)/19*x^4*y-F(15)/121*x^3*y^2+5570*x^2*y^3-14367*x*y^4+15920*y^5+2949*x^4*z+F(1)/26*x^3*y*z-10228*x^2*y^2*z+F(31)/83*x*y^3*z+221*y^4*z+13762*x^3*z^2+9283*x^2*y*z^2-F(65)/86*x*y^2*z^2+F(24)/125*y^3*z^2-F(49)/101*x^2*z^3-F(125)/81*x*y*z^3-F(92)/35*y^2*z^3-F(81)/47*x*z^4+F(29)/71*y*z^4+F(105)/83*z^5-15428*x^4*u+4278*x^3*y*u-F(93)/37*x^2*y^2*u-F(44)/41*x*y^3*u-F(61)/63*y^4*u-8640*x^3*z*u+F(21)/32*x^2*y*z*u-3242*x*y^2*z*u+15620*y^3*z*u+F(95)/98*x^2*z^2*u+7570*x*y*z^2*u+F(11)/28*y^2*z^2*u+F(111)/55*x*z^3*u+F(75)/112*y*z^3*u-F(1)/23*z^4*u+F(59)/70*x^3*u^2-F(30)/67*x^2*y*u^2-F(123)/38*x*y^2*u^2-335*y^3*u^2+2687*x^2*z*u^2+10895*x*y*z*u^2+F(74)/99*y^2*z*u^2-F(103)/120*x*z^2*u^2+4248*y*z^2*u^2+F(15)/118*z^3*u^2-1224*x^2*u^3+14244*x*y*u^3-F(87)/107*y^2*u^3-F(58)/101*x*z*u^3-1922*y*z*u^3+F(25)/81*z^2*u^3-F(101)/14*x*u^4-F(43)/69*y*u^4-706*z*u^4-9796*u^5+270*x^4*v+6887*x^3*y*v+F(23)/24*x^2*y^2*v+8709*x*y^3*v-5837*y^4*v-F(17)/70*x^3*z*v-F(125)/104*x^2*y*z*v-13792*x*y^2*z*v+F(83)/105*y^3*z*v+F(69)/88*x^2*z^2*v-F(93)/112*x*y*z^2*v+F(105)/118*y^2*z^2*v+F(71)/25*x*z^3*v-F(125)/69*y*z^3*v-F(98)/33*z^4*v-F(27)/71*x^3*u*v+6427*x^2*y*u*v-3220*x*y^2*u*v+12076*y^3*u*v+F(83)/57*x^2*z*u*v-F(71)/81*x*y*z*u*v-F(81)/2*y^2*z*u*v+F(22)/19*x*z^2*u*v-6298*y*z^2*u*v-13119*z^3*u*v+8230*x^2*u^2*v-8856*x*y*u^2*v+15664*y^2*u^2*v+F(47)/88*x*z*u^2*v-F(4)/41*y*z*u^2*v-F(8)/83*z^2*u^2*v+9252*x*u^3*v+3651*y*u^3*v+14297*z*u^3*v-F(7)/107*u^4*v+F(44)/113*x^3*v^2+F(67)/27*x^2*y*v^2-F(123)/79*x*y^2*v^2+7670*y^3*v^2+F(28)/107*x^2*z*v^2-5542*x*y*z*v^2-F(115)/42*y^2*z*v^2-F(8)/25*x*z^2*v^2-F(11)/3*y*z^2*v^2-8672*z^3*v^2+1080*x^2*u*v^2+5227*x*y*u*v^2-10058*y^2*u*v^2-8510*x*z*u*v^2-5570*y*z*u*v^2-10584*z^2*u*v^2+F(119)/24*x*u^2*v^2+F(10)/7*y*u^2*v^2-9279*z*u^2*v^2+13742*u^3*v^2-10090*x^2*v^3-15493*x*y*v^3+F(13)/96*y^2*v^3+1693*x*z*v^3+F(19)/105*y*z*v^3+13939*z^2*v^3-F(49)/101*x*u*v^3-F(73)/72*y*u*v^3+F(76)/29*z*u*v^3-12323*u^2*v^3+14191*x*v^4+F(7)/61*y*v^4-F(25)/21*z*v^4+11963*u*v^4+135*v^5]))
end


@doc raw"""
    elliptic_d12_pi14_ss_0()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `12`, sectional genus `14`, and no 6-secant.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d12_pi14_ss_0()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^5-F(107)/16*x^4*y-F(37)/86*x^3*y^2-F(17)/14*x^2*y^3+F(97)/111*x*y^4-F(99)/83*x^4*z+F(46)/109*x^3*y*z+F(81)/83*x^2*y^2*z+F(65)/8*x*y^3*z-F(101)/11*y^4*z+13771*x^3*z^2+F(20)/117*x^2*y*z^2-7473*x*y^2*z^2-F(11)/94*y^3*z^2+F(47)/108*x^2*z^3-8936*x*y*z^3-F(80)/97*y^2*z^3-F(21)/10*x*z^4-62*y*z^4-3503*z^5+9036*x^4*u+F(51)/115*x^3*y*u+F(13)/95*x^2*y^2*u-10606*x*y^3*u-F(55)/21*y^4*u-F(33)/89*x^3*z*u+13540*x^2*y*z*u+F(123)/83*x*y^2*z*u+F(107)/58*y^3*z*u-14834*x^2*z^2*u-F(44)/41*x*y*z^2*u+13736*y^2*z^2*u-F(109)/12*x*z^3*u+F(121)/107*y*z^3*u+12432*z^4*u-F(58)/13*x^3*u^2-14966*x^2*y*u^2-F(36)/113*x*y^2*u^2-F(23)/27*y^3*u^2-15870*x^2*z*u^2+F(97)/7*x*y*z*u^2+11910*y^2*z*u^2-F(86)/91*x*z^2*u^2+12828*y*z^2*u^2-12902*z^3*u^2+15213*x^2*u^3+F(122)/109*x*y*u^3+F(2)/125*y^2*u^3-F(60)/43*x*z*u^3-F(64)/69*y*z*u^3+F(31)/41*z^2*u^3-F(48)/59*x*u^4-F(109)/45*y*u^4-5704*z*u^4-F(64)/101*u^5+F(104)/53*x^4*v+F(47)/71*x^3*y*v+F(117)/14*x^2*y^2*v+5509*x*y^3*v+4768*y^4*v-F(13)/36*x^3*z*v-10095*x^2*y*z*v-15900*x*y^2*z*v-4508*y^3*z*v-7583*x^2*z^2*v-F(79)/111*x*y*z^2*v-5592*y^2*z^2*v+F(2)/41*x*z^3*v+F(51)/73*y*z^3*v+4792*z^4*v+F(71)/18*x^3*u*v-F(5)/34*x^2*y*u*v-F(101)/117*x*y^2*u*v-413*y^3*u*v+F(80)/37*x^2*z*u*v+15920*x*y*z*u*v-F(111)/55*y^2*z*u*v+6103*x*z^2*u*v+F(27)/43*y*z^2*u*v+3399*z^3*u*v+F(84)/85*x^2*u^2*v-4024*x*y*u^2*v-F(100)/77*y^2*u^2*v-F(31)/38*x*z*u^2*v+F(20)/67*y*z*u^2*v+F(112)/45*z^2*u^2*v-F(19)/40*x*u^3*v+F(123)/25*y*u^3*v-F(11)/34*z*u^3*v-F(19)/41*u^4*v-F(109)/10*x^3*v^2+F(103)/32*x^2*y*v^2+F(69)/113*x*y^2*v^2-13118*y^3*v^2-11390*x^2*z*v^2-7571*x*y*z*v^2-5610*y^2*z*v^2+F(16)/117*x*z^2*v^2-F(38)/75*y*z^2*v^2+F(79)/93*z^3*v^2-12026*x^2*u*v^2+F(37)/34*x*y*u*v^2+8163*y^2*u*v^2-473*x*z*u*v^2+9279*y*z*u*v^2-12474*z^2*u*v^2-F(79)/62*x*u^2*v^2+F(39)/10*y*u^2*v^2-11066*z*u^2*v^2-3897*u^3*v^2-F(75)/106*x^2*v^3+F(47)/65*x*y*v^3+F(9)/4*y^2*v^3+8472*x*z*v^3-F(106)/105*y*z*v^3+466*z^2*v^3+F(65)/103*x*u*v^3+13593*y*u*v^3-F(77)/106*z*u*v^3-10275*u^2*v^3+F(81)/19*x*v^4+F(67)/101*y*v^4+6576*z*v^4-3922*u*v^4+F(39)/124*v^5,x^4*y-F(107)/16*x^3*y^2-F(37)/86*x^2*y^3-F(17)/14*x*y^4+F(97)/111*y^5-F(36)/35*x^4*z+8761*x^3*y*z-F(7)/61*x^2*y^2*z-F(121)/118*x*y^3*z+F(97)/69*y^4*z+F(31)/125*x^3*z^2+12906*x^2*y*z^2-15415*x*y^2*z^2-F(43)/10*y^3*z^2-F(35)/39*x^2*z^3-3669*x*y*z^3-F(105)/34*y^2*z^3-4891*x*z^4+8757*y*z^4+12057*z^5+11215*x^4*u+F(37)/98*x^3*y*u+919*x^2*y^2*u-F(110)/29*x*y^3*u-F(43)/52*y^4*u-15702*x^3*z*u+F(52)/45*x^2*y*z*u-F(81)/44*x*y^2*z*u+F(124)/19*y^3*z*u+5124*x^2*z^2*u+F(36)/31*x*y*z^2*u-F(86)/31*y^2*z^2*u-F(101)/126*x*z^3*u-14421*y*z^3*u-F(46)/79*z^4*u-F(92)/13*x^3*u^2+F(17)/113*x^2*y*u^2-F(36)/13*x*y^2*u^2+F(35)/51*y^3*u^2-F(61)/112*x^2*z*u^2+7937*x*y*z*u^2+6576*y^2*z*u^2-4604*x*z^2*u^2+F(53)/112*y*z^2*u^2-7706*z^3*u^2-F(2)/75*x^2*u^3+10115*x*y*u^3+F(101)/10*y^2*u^3+F(82)/111*x*z*u^3-5834*y*z*u^3-F(88)/53*z^2*u^3+14026*x*u^4-1925*y*u^4+F(1)/117*z*u^4-1417*u^5-F(33)/112*x^4*v+10168*x^3*y*v+7557*x^2*y^2*v+7892*x*y^3*v-9424*y^4*v-F(125)/114*x^3*z*v-3180*x^2*y*z*v+F(53)/108*x*y^2*z*v+F(86)/119*y^3*z*v-4935*x^2*z^2*v+5632*x*y*z^2*v+988*y^2*z^2*v+F(14)/51*x*z^3*v-F(53)/60*y*z^3*v-F(11)/26*z^4*v-F(107)/93*x^3*u*v-F(68)/13*x^2*y*u*v+F(80)/69*x*y^2*u*v+F(95)/64*y^3*u*v-F(29)/81*x^2*z*u*v-F(79)/35*x*y*z*u*v+F(11)/114*y^2*z*u*v-F(56)/107*x*z^2*u*v+11682*y*z^2*u*v+15773*z^3*u*v+F(124)/83*x^2*u^2*v-5860*x*y*u^2*v-5369*y^2*u^2*v+14991*x*z*u^2*v+F(58)/95*y*z*u^2*v+3133*z^2*u^2*v-F(83)/108*x*u^3*v+F(107)/15*y*u^3*v-3482*z*u^3*v+2070*u^4*v-F(109)/89*x^3*v^2-15903*x^2*y*v^2+F(119)/13*x*y^2*v^2+F(71)/72*y^3*v^2-F(47)/23*x^2*z*v^2+F(83)/89*x*y*z*v^2-2334*y^2*z*v^2-F(11)/31*x*z^2*v^2+F(83)/79*y*z^2*v^2-F(35)/122*z^3*v^2+F(77)/109*x^2*u*v^2+F(42)/41*x*y*u*v^2+F(123)/5*y^2*u*v^2-F(104)/37*x*z*u*v^2+F(109)/120*y*z*u*v^2+F(96)/103*z^2*u*v^2-F(27)/118*x*u^2*v^2-F(96)/17*y*u^2*v^2-15240*z*u^2*v^2+F(40)/119*u^3*v^2-F(113)/50*x^2*v^3+3221*x*y*v^3-3679*y^2*v^3+F(25)/48*x*z*v^3-F(33)/17*y*z*v^3+F(11)/7*z^2*v^3+F(50)/17*x*u*v^3+6501*y*u*v^3+F(19)/92*z*u*v^3+4238*u^2*v^3+F(64)/107*x*v^4+4079*y*v^4+F(101)/32*z*v^4+14655*u*v^4-F(87)/56*v^5,x^3*y^2+F(114)/107*x^2*y^3-F(46)/57*x*y^4-F(25)/84*y^5+11980*x^4*z+F(92)/39*x^3*y*z-F(62)/57*x^2*y^2*z+3345*x*y^3*z-12228*y^4*z-5511*x^3*z^2-11246*x^2*y*z^2-F(5)/86*x*y^2*z^2-F(13)/108*y^3*z^2+15727*x^2*z^3-F(92)/15*x*y*z^3+6962*y^2*z^3+7571*x*z^4-F(68)/111*y*z^4-F(7)/61*z^5-403*x^4*u-F(113)/70*x^3*y*u+6827*x^2*y^2*u-11262*x*y^3*u+12699*y^4*u-F(63)/26*x^3*z*u+F(53)/92*x^2*y*z*u+14183*x*y^2*z*u-F(83)/52*y^3*z*u-F(99)/7*x^2*z^2*u+155*x*y*z^2*u+14279*y^2*z^2*u-15200*x*z^3*u+F(14)/109*y*z^3*u-F(31)/56*z^4*u+6047*x^3*u^2-12717*x^2*y*u^2-F(35)/53*x*y^2*u^2+15829*y^3*u^2+751*x^2*z*u^2-F(123)/56*x*y*z*u^2-F(5)/3*y^2*z*u^2+F(17)/80*x*z^2*u^2-11979*y*z^2*u^2-F(60)/89*z^3*u^2-F(75)/59*x^2*u^3-F(45)/107*x*y*u^3+F(35)/29*y^2*u^3+13164*x*z*u^3+F(59)/48*y*z*u^3+F(43)/27*z^2*u^3+11056*x*u^4+F(91)/71*y*u^4+12573*z*u^4-30*u^5+F(36)/119*x^4*v+9379*x^3*y*v+11796*x^2*y^2*v-15894*x*y^3*v+F(111)/112*y^4*v-7760*x^3*z*v-15140*x^2*y*z*v+11692*x*y^2*z*v+12953*y^3*z*v-11271*x^2*z^2*v+168*x*y*z^2*v-F(7)/67*y^2*z^2*v-F(24)/119*x*z^3*v-9730*y*z^3*v-F(110)/69*z^4*v+F(109)/44*x^3*u*v+F(89)/105*x^2*y*u*v+F(89)/11*x*y^2*u*v+F(89)/68*y^3*u*v-F(117)/106*x^2*z*u*v+F(70)/3*x*y*z*u*v-1381*y^2*z*u*v+F(83)/73*x*z^2*u*v+9523*y*z^2*u*v+F(50)/81*z^3*u*v+F(125)/34*x^2*u^2*v+F(17)/49*x*y*u^2*v+F(75)/32*y^2*u^2*v-3350*x*z*u^2*v-F(125)/123*y*z*u^2*v-F(63)/46*z^2*u^2*v-3798*x*u^3*v-F(61)/94*y*u^3*v+15349*z*u^3*v+F(119)/94*u^4*v+3893*x^3*v^2-7340*x^2*y*v^2-F(103)/91*x*y^2*v^2+6141*y^3*v^2-14883*x^2*z*v^2-5390*x*y*z*v^2+F(103)/75*y^2*z*v^2-4309*x*z^2*v^2-F(88)/91*y*z^2*v^2+F(19)/118*z^3*v^2+1675*x^2*u*v^2+F(115)/126*x*y*u*v^2-F(78)/23*y^2*u*v^2+F(83)/59*x*z*u*v^2-F(23)/86*y*z*u*v^2-15796*z^2*u*v^2+14528*x*u^2*v^2-F(95)/118*y*u^2*v^2+F(59)/88*z*u^2*v^2-F(106)/29*u^3*v^2-6*x^2*v^3+F(11)/95*x*y*v^3+8178*y^2*v^3-F(11)/105*x*z*v^3+9923*y*z*v^3+F(29)/81*z^2*v^3-F(94)/45*x*u*v^3-4401*y*u*v^3+9160*z*u*v^3-F(91)/109*u^2*v^3-F(110)/37*x*v^4-F(93)/14*y*v^4+2748*z*v^4-F(19)/110*u*v^4-F(82)/55*v^5,x^2*y^3+F(59)/69*x*y^4-F(81)/74*y^5+10024*x^4*z-F(71)/48*x^3*y*z+9310*x^2*y^2*z-F(33)/125*x*y^3*z+F(2)/113*y^4*z-8669*x^3*z^2+F(112)/61*x^2*y*z^2+F(43)/124*x*y^2*z^2-7767*y^3*z^2-12391*x^2*z^3+F(21)/16*x*y*z^3+F(22)/49*y^2*z^3-14859*x*z^4-F(90)/109*y*z^4-F(105)/34*z^5-6198*x^4*u+F(63)/53*x^3*y*u+F(27)/53*x^2*y^2*u-F(32)/91*x*y^3*u+7231*y^4*u-F(4)/15*x^3*z*u+7513*x^2*y*z*u+10127*x*y^2*z*u+F(59)/102*y^3*z*u-3967*x^2*z^2*u-12336*x*y*z^2*u+13916*y^2*z^2*u+F(111)/101*x*z^3*u-13502*y*z^3*u-F(117)/53*z^4*u+13229*x^3*u^2+F(103)/21*x^2*y*u^2-F(1)/78*x*y^2*u^2-10435*y^3*u^2+358*x^2*z*u^2-F(35)/3*x*y*z*u^2+F(122)/125*y^2*z*u^2+4396*x*z^2*u^2+11856*y*z^2*u^2-F(91)/2*z^3*u^2+F(14)/103*x^2*u^3-15919*x*y*u^3-F(31)/23*y^2*u^3+F(90)/91*x*z*u^3-7401*y*z*u^3+F(71)/106*z^2*u^3-F(46)/91*x*u^4+F(117)/83*y*u^4-F(121)/34*z*u^4+5212*u^5+F(90)/71*x^4*v-10058*x^3*y*v+F(124)/41*x^2*y^2*v+F(104)/9*x*y^3*v-F(51)/32*y^4*v+4550*x^3*z*v-F(101)/30*x^2*y*z*v+F(1)/18*x*y^2*z*v-F(91)/125*y^3*z*v+F(21)/76*x^2*z^2*v-F(82)/117*x*y*z^2*v+9690*y^2*z^2*v+F(50)/107*x*z^3*v+4535*y*z^3*v+F(32)/89*z^4*v+F(61)/57*x^3*u*v+F(40)/59*x^2*y*u*v+F(44)/21*x*y^2*u*v+8630*y^3*u*v+123*x^2*z*u*v-F(77)/123*x*y*z*u*v+F(31)/6*y^2*z*u*v-F(91)/73*x*z^2*u*v+F(47)/53*y*z^2*u*v+F(28)/73*z^3*u*v+4016*x^2*u^2*v+2444*x*y*u^2*v-F(57)/37*y^2*u^2*v+F(121)/63*x*z*u^2*v-6770*y*z*u^2*v-10067*z^2*u^2*v+F(115)/26*x*u^3*v-F(22)/9*y*u^3*v+8971*z*u^3*v-440*u^4*v+F(83)/15*x^3*v^2-F(117)/67*x^2*y*v^2+F(87)/13*x*y^2*v^2+F(120)/61*y^3*v^2+2684*x^2*z*v^2+3884*x*y*z*v^2-F(38)/119*y^2*z*v^2-F(69)/35*x*z^2*v^2+6604*y*z^2*v^2-F(62)/83*z^3*v^2-2684*x^2*u*v^2+F(119)/74*x*y*u*v^2+3799*y^2*u*v^2+7086*x*z*u*v^2+15739*y*z*u*v^2-F(23)/122*z^2*u*v^2+F(46)/77*x*u^2*v^2+F(5)/64*y*u^2*v^2-8876*z*u^2*v^2+1591*u^3*v^2+F(85)/13*x^2*v^3+1545*x*y*v^3-F(53)/56*y^2*v^3-10008*x*z*v^3-F(97)/13*y*z*v^3-13027*z^2*v^3+3968*x*u*v^3+F(44)/51*y*u*v^3-7582*z*u*v^3-F(17)/23*u^2*v^3-4432*x*v^4+F(94)/65*y*v^4+1053*z*v^4+F(30)/17*u*v^4+F(103)/12*v^5,x^4*z+3830*x^3*y*z-4731*x^2*y^2*z+F(23)/120*x*y^3*z+F(45)/113*y^4*z+15898*x^3*z^2+F(50)/117*x^2*y*z^2+6534*x*y^2*z^2-F(79)/6*y^3*z^2-10743*x^2*z^3-F(67)/34*x*y*z^3+2859*y^2*z^3-F(22)/13*x*z^4+F(35)/53*y*z^4+F(41)/124*z^5-F(125)/6*x^4*u-13199*x^3*y*u-5797*x^2*y^2*u-F(9)/100*x*y^3*u-F(73)/53*y^4*u+7637*x^3*z*u-4961*x^2*y*z*u-5637*x*y^2*z*u-F(47)/100*y^3*z*u-F(112)/67*x^2*z^2*u-F(13)/79*x*y*z^2*u-12127*y^2*z^2*u-F(111)/68*x*z^3*u+F(1)/35*y*z^3*u-14416*z^4*u-F(123)/80*x^3*u^2-F(53)/89*x^2*y*u^2-F(109)/52*x*y^2*u^2-F(32)/113*y^3*u^2-824*x^2*z*u^2+1160*x*y*z*u^2-4352*y^2*z*u^2+14430*x*z^2*u^2+F(53)/27*y*z^2*u^2+F(4)/115*z^3*u^2-F(11)/72*x^2*u^3-9709*x*y*u^3+71*y^2*u^3-13136*x*z*u^3+9687*y*z*u^3+10858*z^2*u^3+7086*x*u^4+F(85)/48*y*u^4-10932*z*u^4-5307*u^5-F(37)/97*x^4*v+15485*x^3*y*v+13682*x^2*y^2*v-F(43)/120*x*y^3*v-F(29)/16*y^4*v+F(79)/115*x^3*z*v-12652*x^2*y*z*v-14456*x*y^2*z*v+F(56)/55*y^3*z*v-F(27)/44*x^2*z^2*v+F(93)/103*x*y*z^2*v-F(109)/39*y^2*z^2*v+F(64)/35*x*z^3*v-5419*y*z^3*v-F(5)/119*z^4*v-3504*x^3*u*v-F(53)/80*x^2*y*u*v-F(87)/64*x*y^2*u*v+F(121)/51*y^3*u*v+F(66)/95*x^2*z*u*v+11247*x*y*z*u*v-F(43)/69*y^2*z*u*v-F(29)/102*x*z^2*u*v-F(31)/48*y*z^2*u*v+4213*z^3*u*v+F(29)/4*x^2*u^2*v-3055*x*y*u^2*v-F(49)/54*y^2*u^2*v+5450*x*z*u^2*v+F(69)/17*y*z*u^2*v-F(115)/61*z^2*u^2*v-11676*x*u^3*v+7401*y*u^3*v-F(17)/37*z*u^3*v-219*u^4*v-13758*x^3*v^2-F(67)/75*x^2*y*v^2+9322*x*y^2*v^2+F(39)/97*y^3*v^2+6888*x^2*z*v^2-F(85)/83*x*y*z*v^2-F(53)/119*y^2*z*v^2+F(78)/83*x*z^2*v^2+F(126)/71*y*z^2*v^2-6354*z^3*v^2-1988*x^2*u*v^2-6803*x*y*u*v^2+F(33)/10*y^2*u*v^2+4823*x*z*u*v^2+F(81)/50*y*z*u*v^2-F(85)/54*z^2*u*v^2+8521*x*u^2*v^2-14505*y*u^2*v^2-8042*z*u^2*v^2+11064*u^3*v^2+3526*x^2*v^3-F(31)/13*x*y*v^3+F(15)/37*y^2*v^3+4857*x*z*v^3+649*y*z*v^3+F(117)/67*z^2*v^3+5136*x*u*v^3+F(84)/83*y*u*v^3+F(67)/65*z*u*v^3+F(12)/125*u^2*v^3-F(101)/57*x*v^4+11648*y*v^4+F(22)/39*z*v^4+6291*u*v^4+9233*v^5,x*y^4-F(41)/22*y^5-F(120)/107*x^3*y*z-9002*x^2*y^2*z-F(27)/26*x*y^3*z+F(107)/111*y^4*z+4280*x^3*z^2+12573*x^2*y*z^2-F(63)/29*x*y^2*z^2-F(107)/99*y^3*z^2-121*x^2*z^3-F(65)/97*x*y*z^3-1138*y^2*z^3+F(119)/107*x*z^4+7460*y*z^4+F(87)/32*z^5-8691*x^4*u+9892*x^3*y*u-10314*x^2*y^2*u-6869*x*y^3*u-6705*y^4*u-15506*x^3*z*u-F(84)/109*x^2*y*z*u-F(41)/89*x*y^2*z*u+F(55)/118*y^3*z*u-F(116)/57*x^2*z^2*u-F(53)/45*x*y*z^2*u-F(126)/61*y^2*z^2*u+F(64)/73*x*z^3*u-619*y*z^3*u+F(6)/25*z^4*u-F(9)/53*x^3*u^2+F(83)/107*x^2*y*u^2-1610*x*y^2*u^2-14582*y^3*u^2+F(33)/46*x^2*z*u^2+4018*x*y*z*u^2+F(96)/77*y^2*z*u^2+F(76)/49*x*z^2*u^2-F(109)/85*y*z^2*u^2+F(92)/49*z^3*u^2-F(106)/69*x^2*u^3+748*x*y*u^3+3487*y^2*u^3-F(43)/44*x*z*u^3-946*y*z*u^3-F(103)/120*z^2*u^3+F(59)/48*x*u^4-F(95)/44*y*u^4-F(3)/91*z*u^4+F(15)/94*u^5+3947*x^4*v+F(49)/74*x^3*y*v+6039*x^2*y^2*v+F(25)/81*x*y^3*v+F(23)/88*y^4*v+15364*x^3*z*v-F(17)/95*x^2*y*z*v-F(125)/123*x*y^2*z*v-3528*y^3*z*v+14407*x^2*z^2*v+F(90)/97*x*y*z^2*v-F(65)/51*y^2*z^2*v-F(121)/118*x*z^3*v+10979*y*z^3*v+F(79)/126*z^4*v-F(33)/38*x^3*u*v-3434*x^2*y*u*v+F(10)/89*x*y^2*u*v+F(43)/72*y^3*u*v+F(119)/103*x^2*z*u*v+8311*x*y*z*u*v-F(5)/19*y^2*z*u*v+F(113)/66*x*z^2*u*v-13986*y*z^2*u*v-F(38)/73*z^3*u*v-F(115)/36*x^2*u^2*v-2299*x*y*u^2*v-2491*y^2*u^2*v+8651*x*z*u^2*v-11146*y*z*u^2*v+184*z^2*u^2*v-5144*x*u^3*v+F(39)/97*y*u^3*v-F(97)/21*z*u^3*v-2100*u^4*v-F(31)/9*x^3*v^2+F(8)/117*x^2*y*v^2-F(97)/67*x*y^2*v^2+F(10)/53*y^3*v^2-F(55)/72*x^2*z*v^2-F(1)/44*x*y*z*v^2+9337*y^2*z*v^2+F(101)/121*x*z^2*v^2-F(101)/106*y*z^2*v^2-F(42)/47*z^3*v^2-F(43)/31*x^2*u*v^2-5086*x*y*u*v^2-11579*y^2*u*v^2+F(5)/46*x*z*u*v^2-F(80)/61*y*z*u*v^2-F(34)/5*z^2*u*v^2+F(28)/111*x*u^2*v^2+F(10)/93*y*u^2*v^2-F(2)/37*z*u^2*v^2+6205*u^3*v^2-F(17)/9*x^2*v^3+9920*x*y*v^3-F(67)/65*y^2*v^3+F(52)/23*x*z*v^3+13792*y*z*v^3-F(25)/24*z^2*v^3-13396*x*u*v^3-F(63)/74*y*u*v^3-10421*z*u*v^3+F(2)/103*u^2*v^3+12840*x*v^4+2038*y*v^4+4420*z*v^4+2350*u*v^4-F(13)/60*v^5,y^5-8521*x^3*y*z+F(66)/71*x^2*y^2*z+F(46)/73*x*y^3*z+10593*y^4*z-5855*x^3*z^2+F(117)/79*x^2*y*z^2+F(29)/85*x*y^2*z^2-20*y^3*z^2-12023*x^2*z^3-6994*x*y*z^3+F(9)/55*y^2*z^3+F(121)/35*x*z^4+8400*y*z^4+F(11)/71*z^5-F(105)/73*x^4*u-F(77)/120*x^3*y*u-1120*x^2*y^2*u-2159*x*y^3*u+F(99)/106*y^4*u-672*x^3*z*u+14252*x^2*y*z*u+14945*x*y^2*z*u-7445*y^3*z*u-F(119)/37*x^2*z^2*u-F(95)/42*x*y*z^2*u-F(67)/16*y^2*z^2*u+F(19)/84*x*z^3*u+F(115)/76*y*z^3*u+F(87)/109*z^4*u-F(89)/67*x^3*u^2+166*x^2*y*u^2-3102*x*y^2*u^2-F(43)/36*y^3*u^2+F(74)/43*x^2*z*u^2+F(11)/73*x*y*z*u^2+F(109)/21*y^2*z*u^2+F(72)/101*x*z^2*u^2+14386*y*z^2*u^2+F(97)/54*z^3*u^2-F(108)/43*x^2*u^3+12212*x*y*u^3-F(77)/37*y^2*u^3-F(21)/65*x*z*u^3-F(94)/23*y*z*u^3-F(98)/11*z^2*u^3-F(46)/45*x*u^4-F(61)/45*y*u^4+6430*z*u^4+F(13)/55*u^5-F(107)/77*x^4*v+43*x^3*y*v+F(31)/104*x^2*y^2*v+F(55)/51*x*y^3*v-3914*y^4*v+15310*x^3*z*v-F(106)/57*x^2*y*z*v-F(124)/37*x*y^2*z*v+F(41)/15*y^3*z*v+F(73)/62*x^2*z^2*v+15192*x*y*z^2*v-1474*y^2*z^2*v-F(61)/40*x*z^3*v+F(59)/27*y*z^3*v-F(61)/79*z^4*v-15557*x^3*u*v+F(4)/11*x^2*y*u*v+12111*x*y^2*u*v-10989*y^3*u*v+13738*x^2*z*u*v-14730*x*y*z*u*v-15044*y^2*z*u*v+10896*x*z^2*u*v-8429*y*z^2*u*v-8077*z^3*u*v-4891*x^2*u^2*v+13740*x*y*u^2*v-F(19)/53*y^2*u^2*v+9256*x*z*u^2*v-3578*y*z*u^2*v+6280*z^2*u^2*v-F(8)/61*x*u^3*v+6437*y*u^3*v-F(108)/91*z*u^3*v-F(69)/106*u^4*v+2749*x^3*v^2-1815*x^2*y*v^2-F(97)/79*x*y^2*v^2-F(40)/69*y^3*v^2+F(91)/99*x^2*z*v^2-F(74)/47*x*y*z*v^2-F(94)/95*y^2*z*v^2+F(11)/35*x*z^2*v^2-F(95)/32*y*z^2*v^2+F(105)/31*z^3*v^2+F(58)/9*x^2*u*v^2+2117*x*y*u*v^2-F(68)/31*y^2*u*v^2+F(27)/104*x*z*u*v^2-3852*y*z*u*v^2-5168*z^2*u*v^2+2095*x*u^2*v^2+F(94)/45*y*u^2*v^2-F(102)/107*z*u^2*v^2+10256*u^3*v^2-F(116)/5*x^2*v^3+1582*x*y*v^3+2536*y^2*v^3+F(59)/11*x*z*v^3+F(121)/43*y*z*v^3-3249*z^2*v^3+F(109)/9*x*u*v^3+F(41)/110*y*u*v^3-6059*z*u*v^3+F(23)/104*u^2*v^3-F(13)/92*x*v^4-936*y*v^4+9094*z*v^4+F(5)/56*u*v^4-F(11)/72*v^5,x^3*y*z-11545*x^2*y^2*z+F(113)/77*x*y^3*z-F(64)/3*y^4*z-F(109)/40*x^3*z^2+14719*x^2*y*z^2-6495*x*y^2*z^2+435*y^3*z^2+F(55)/102*x^2*z^3-11612*x*y*z^3-4633*y^2*z^3+F(39)/86*x*z^4-F(23)/98*y*z^4+6364*z^5+F(97)/50*x^4*u-937*x^3*y*u-11092*x^2*y^2*u-3679*x*y^3*u+F(115)/126*y^4*u+8747*x^3*z*u-F(64)/111*x^2*y*z*u+F(23)/100*x*y^2*z*u-F(61)/45*y^3*z*u-8672*x^2*z^2*u-F(59)/26*x*y*z^2*u+F(76)/105*y^2*z^2*u+1273*x*z^3*u+F(14)/5*y*z^3*u-F(78)/25*z^4*u-F(25)/51*x^3*u^2-11358*x^2*y*u^2-15924*x*y^2*u^2-1610*y^3*u^2-F(91)/92*x^2*z*u^2-F(73)/63*x*y*z*u^2-F(63)/109*y^2*z*u^2-F(19)/119*x*z^2*u^2+13825*y*z^2*u^2+F(51)/80*z^3*u^2-F(25)/67*x^2*u^3-F(76)/43*x*y*u^3-9401*y^2*u^3-F(118)/13*x*z*u^3-9097*y*z*u^3-F(87)/118*z^2*u^3+F(23)/25*x*u^4+F(73)/33*y*u^4-11456*z*u^4+1953*u^5+977*x^4*v-F(67)/49*x^3*y*v-F(107)/116*x^2*y^2*v-F(49)/90*x*y^3*v-11612*y^4*v+12099*x^3*z*v+F(58)/73*x^2*y*z*v-14529*x*y^2*z*v-8249*y^3*z*v+70*x^2*z^2*v+F(14)/31*x*y*z^2*v-F(52)/57*y^2*z^2*v+F(119)/29*x*z^3*v+F(113)/26*y*z^3*v+3015*z^4*v+F(101)/120*x^3*u*v-3174*x^2*y*u*v+F(113)/90*x*y^2*u*v+F(55)/38*y^3*u*v-F(101)/64*x^2*z*u*v+7005*x*y*z*u*v-11665*y^2*z*u*v-F(48)/101*x*z^2*u*v-3596*y*z^2*u*v+F(82)/57*z^3*u*v+2311*x^2*u^2*v-F(92)/123*x*y*u^2*v+F(5)/24*y^2*u^2*v-7583*x*z*u^2*v+F(83)/22*y*z*u^2*v+6923*z^2*u^2*v+871*x*u^3*v-F(101)/47*y*u^3*v-F(5)/114*z*u^3*v+F(104)/115*u^4*v+F(47)/38*x^3*v^2-F(4)/29*x^2*y*v^2+6072*x*y^2*v^2+F(15)/76*y^3*v^2+F(73)/7*x^2*z*v^2+14692*x*y*z*v^2-F(103)/93*y^2*z*v^2+F(116)/123*x*z^2*v^2+12770*y*z^2*v^2+F(61)/72*z^3*v^2+9756*x^2*u*v^2+F(74)/97*x*y*u*v^2+15250*y^2*u*v^2+F(59)/73*x*z*u*v^2+F(115)/91*y*z*u*v^2-F(76)/105*z^2*u*v^2-12895*x*u^2*v^2-F(52)/75*y*u^2*v^2+F(89)/29*z*u^2*v^2-219*u^3*v^2+F(9)/88*x^2*v^3-F(47)/28*x*y*v^3+F(6)/125*y^2*v^3+10274*x*z*v^3-F(86)/23*y*z*v^3+F(75)/53*z^2*v^3-F(16)/73*x*u*v^3-F(75)/53*y*u*v^3-14637*z*u*v^3+F(39)/58*u^2*v^3+F(98)/19*x*v^4-F(117)/97*y*v^4+35*z*v^4+F(14)/75*u*v^4+F(126)/83*v^5]))
end


@doc raw"""
    elliptic_d12_pi14_ss_inf()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `12`, sectional genus `14`, and infinitely many 6-secants.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d12_pi14_ss_inf()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^6+F(114)/49*x^5*y+F(4)/43*x^4*y^2+F(31)/27*x^3*y^3-3454*x^2*y^4-F(65)/23*x*y^5-F(11)/57*x^5*z+4236*x^4*y*z+3813*x^3*y^2*z-F(13)/55*x^2*y^3*z+F(85)/53*x*y^4*z-5228*x^4*z^2+F(11)/115*x^3*y*z^2+1019*x^2*y^2*z^2-14005*x*y^3*z^2-796*x^3*z^3+F(62)/15*x^2*y*z^3+11580*x*y^2*z^3+4459*x^2*z^4-11755*x*y*z^4+11648*x*z^5+7084*x^5*u+9967*x^4*y*u-F(93)/76*x^3*y^2*u-F(47)/125*x^2*y^3*u-2296*x*y^4*u+F(91)/113*x^4*z*u+F(108)/29*x^3*y*z*u+15885*x^2*y^2*z*u-F(3)/20*x*y^3*z*u-F(101)/76*x^3*z^2*u-F(62)/39*x^2*y*z^2*u+13461*x*y^2*z^2*u-F(27)/71*y^3*z^2*u+7490*x^2*z^3*u+11155*x*y*z^3*u+F(19)/36*y^2*z^3*u+607*x*z^4*u-10133*y*z^4*u-F(24)/109*z^5*u-F(111)/124*x^4*u^2-F(90)/83*x^3*y*u^2+8554*x^2*y^2*u^2-F(19)/5*x*y^3*u^2-15917*x^3*z*u^2+F(63)/86*x^2*y*z*u^2-F(1)/15*x*y^2*z*u^2+F(70)/31*x^2*z^2*u^2-F(109)/96*x*y*z^2*u^2-F(72)/83*y^2*z^2*u^2-8107*x*z^3*u^2-9934*y*z^3*u^2+F(28)/9*z^4*u^2+F(23)/117*x^3*u^3-F(45)/23*x^2*y*u^3+F(10)/91*x*y^2*u^3-826*x^2*z*u^3+F(83)/45*x*y*z*u^3-5935*x*z^2*u^3+F(25)/28*y*z^2*u^3+F(83)/67*z^3*u^3-F(67)/25*x^2*u^4+F(76)/115*x*y*u^4-948*x*z*u^4+11180*z^2*u^4+F(95)/61*x^5*v-F(94)/95*x^4*y*v-10746*x^3*y^2*v+12622*x^2*y^3*v+F(61)/42*x*y^4*v-15913*x^4*z*v+4852*x^3*y*z*v-488*x^2*y^2*z*v-12500*x*y^3*z*v-F(13)/60*x^3*z^2*v-1241*x^2*y*z^2*v+14085*x*y^2*z^2*v-F(20)/97*y^3*z^2*v-6046*x^2*z^3*v+F(83)/7*x*y*z^3*v-F(104)/49*y^2*z^3*v+F(88)/97*x*z^4*v+700*y*z^4*v+F(102)/73*z^5*v-F(54)/31*x^4*u*v+3745*x^3*y*u*v+F(30)/61*x^2*y^2*u*v-F(107)/14*x*y^3*u*v-F(91)/118*x^3*z*u*v+F(80)/7*x^2*y*z*u*v+F(66)/107*x*y^2*z*u*v+F(71)/62*x^2*z^2*u*v+F(26)/123*x*y*z^2*u*v-12504*y^2*z^2*u*v-14027*x*z^3*u*v+F(17)/52*y*z^3*u*v+134*z^4*u*v-11907*x^3*u^2*v+11568*x^2*y*u^2*v+F(105)/124*x*y^2*u^2*v+7698*x^2*z*u^2*v-F(50)/67*x*y*z*u^2*v+15094*x*z^2*u^2*v-F(103)/74*y*z^2*u^2*v-F(107)/104*z^3*u^2*v+F(64)/27*x^2*u^3*v-F(43)/66*x*y*u^3*v+F(35)/93*x*z*u^3*v+6496*z^2*u^3*v-F(14)/33*x^4*v^2-12089*x^3*y*v^2-6488*x^2*y^2*v^2+F(81)/119*x*y^3*v^2-F(59)/88*x^3*z*v^2+976*x^2*y*z*v^2-F(64)/55*x*y^2*z*v^2-574*x^2*z^2*v^2+8587*x*y*z^2*v^2+F(113)/53*y^2*z^2*v^2+5420*x*z^3*v^2-F(103)/14*y*z^3*v^2-8285*z^4*v^2-F(25)/23*x^3*u*v^2-F(32)/117*x^2*y*u*v^2-F(59)/107*x*y^2*u*v^2+F(21)/74*x^2*z*u*v^2+15212*x*y*z*u*v^2-F(123)/43*x*z^2*u*v^2+F(51)/7*y*z^2*u*v^2-7961*z^3*u*v^2-F(71)/70*x^2*u^2*v^2+1383*x*y*u^2*v^2+7014*x*z*u^2*v^2+F(9)/76*z^2*u^2*v^2+F(73)/98*x^3*v^3-F(11)/106*x^2*y*v^3+F(72)/103*x*y^2*v^3+14672*x^2*z*v^3-9225*x*y*z*v^3+50*x*z^2*v^3+F(36)/121*y*z^2*v^3+F(123)/34*z^3*v^3-F(106)/87*x^2*u*v^3-14830*x*y*u*v^3-F(96)/37*x*z*u*v^3+6815*z^2*u*v^3-8769*x^2*v^4+2307*x*y*v^4-F(42)/23*x*z*v^4-F(83)/124*z^2*v^4,-x^5*y-F(114)/49*x^4*y^2-F(4)/43*x^3*y^3-F(31)/27*x^2*y^4+3454*x*y^5+F(65)/23*y^6+F(11)/57*x^4*y*z-4236*x^3*y^2*z-3813*x^2*y^3*z+F(13)/55*x*y^4*z-F(85)/53*y^5*z+5228*x^3*y*z^2-F(11)/115*x^2*y^2*z^2-1019*x*y^3*z^2+14005*y^4*z^2+796*x^2*y*z^3-F(62)/15*x*y^2*z^3-11580*y^3*z^3-4459*x*y*z^4+11755*y^2*z^4-11648*y*z^5-7084*x^4*y*u-9967*x^3*y^2*u+F(93)/76*x^2*y^3*u+F(47)/125*x*y^4*u+2296*y^5*u-F(91)/113*x^3*y*z*u-F(108)/29*x^2*y^2*z*u-15885*x*y^3*z*u+F(3)/20*y^4*z*u+F(59)/15*x^3*z^2*u+F(121)/23*x^2*y*z^2*u-F(124)/89*x*y^2*z^2*u-F(121)/9*y^3*z^2*u+10267*x^2*z^3*u+F(41)/113*x*y*z^3*u-F(59)/126*y^2*z^3*u-F(78)/115*x*z^4*u+6993*y*z^4*u-F(5)/113*z^5*u+F(111)/124*x^3*y*u^2+F(90)/83*x^2*y^2*u^2-8554*x*y^3*u^2+F(19)/5*y^4*u^2+15917*x^2*y*z*u^2-F(63)/86*x*y^2*z*u^2+F(1)/15*y^3*z*u^2+F(61)/98*x^2*z^2*u^2-1074*x*y*z^2*u^2-F(25)/107*y^2*z^2*u^2-12390*x*z^3*u^2+F(37)/96*y*z^3*u^2+1225*z^4*u^2-F(23)/117*x^2*y*u^3+F(45)/23*x*y^2*u^3-F(10)/91*y^3*u^3+826*x*y*z*u^3-F(83)/45*y^2*z*u^3+8037*x*z^2*u^3+F(104)/107*y*z^2*u^3+F(37)/117*z^3*u^3+F(67)/25*x*y*u^4-F(76)/115*y^2*u^4+948*y*z*u^4+6599*z^2*u^4-F(95)/61*x^4*y*v+F(94)/95*x^3*y^2*v+10746*x^2*y^3*v-12622*x*y^4*v-F(61)/42*y^5*v+15913*x^3*y*z*v-4852*x^2*y^2*z*v+488*x*y^3*z*v+12500*y^4*z*v+F(61)/126*x^3*z^2*v-F(71)/85*x^2*y*z^2*v-13899*x*y^2*z^2*v-8410*y^3*z^2*v-910*x^2*z^3*v-F(81)/94*x*y*z^3*v-F(88)/119*y^2*z^3*v-9398*x*z^4*v+7326*y*z^4*v+F(96)/41*z^5*v+F(54)/31*x^3*y*u*v-3745*x^2*y^2*u*v-F(30)/61*x*y^3*u*v+F(107)/14*y^4*u*v+F(91)/118*x^2*y*z*u*v-F(80)/7*x*y^2*z*u*v-F(66)/107*y^3*z*u*v-6439*x^2*z^2*u*v+F(125)/103*x*y*z^2*u*v+F(91)/44*y^2*z^2*u*v-F(90)/53*x*z^3*u*v+F(25)/68*y*z^3*u*v-F(23)/71*z^4*u*v+11907*x^2*y*u^2*v-11568*x*y^2*u^2*v-F(105)/124*y^3*u^2*v-7698*x*y*z*u^2*v+F(50)/67*y^2*z*u^2*v-F(67)/109*x*z^2*u^2*v-F(6)/41*y*z^2*u^2*v+F(87)/31*z^3*u^2*v-F(64)/27*x*y*u^3*v+F(43)/66*y^2*u^3*v-F(35)/93*y*z*u^3*v+11613*z^2*u^3*v+F(14)/33*x^3*y*v^2+12089*x^2*y^2*v^2+6488*x*y^3*v^2-F(81)/119*y^4*v^2+F(59)/88*x^2*y*z*v^2-976*x*y^2*z*v^2+F(64)/55*y^3*z*v^2+F(113)/58*x^2*z^2*v^2-6488*x*y*z^2*v^2-F(89)/90*y^2*z^2*v^2+11206*x*z^3*v^2-F(48)/79*y*z^3*v^2-F(21)/104*z^4*v^2+F(25)/23*x^2*y*u*v^2+F(32)/117*x*y^2*u*v^2+F(59)/107*y^3*u*v^2-F(21)/74*x*y*z*u*v^2-15212*y^2*z*u*v^2-F(88)/107*x*z^2*u*v^2-F(91)/25*y*z^2*u*v^2+11659*z^3*u*v^2+F(71)/70*x*y*u^2*v^2-1383*y^2*u^2*v^2-7014*y*z*u^2*v^2+F(1)/116*z^2*u^2*v^2-F(73)/98*x^2*y*v^3+F(11)/106*x*y^2*v^3-F(72)/103*y^3*v^3-14672*x*y*z*v^3+9225*y^2*z*v^3+F(55)/57*x*z^2*v^3+F(37)/103*y*z^2*v^3+F(98)/17*z^3*v^3+F(106)/87*x*y*u*v^3+14830*y^2*u*v^3+F(96)/37*y*z*u*v^3-1080*z^2*u*v^3+8769*x*y*v^4-2307*y^2*v^4+F(42)/23*y*z*v^4-F(47)/118*z^2*v^4,-10816*x^4*u+F(47)/69*x^3*y*u+11233*x^2*y^2*u+F(118)/91*x*y^3*u-F(18)/47*y^4*u+1899*x^3*z*u+F(11)/124*x^2*y*z*u+F(33)/52*x*y^2*z*u+F(102)/29*y^3*z*u+F(16)/123*x^2*z^2*u-F(76)/65*x*y*z^2*u+F(106)/21*y^2*z^2*u-909*x*z^3*u+F(101)/40*y*z^3*u-F(4)/11*x^3*u^2+F(13)/99*x^2*y*u^2+F(111)/82*x*y^2*u^2+F(111)/10*y^3*u^2-F(113)/7*x^2*z*u^2-12319*x*y*z*u^2+15066*y^2*z*u^2+F(67)/112*x*z^2*u^2-F(74)/13*y*z^2*u^2-5889*z^3*u^2+F(13)/69*x^2*u^3-7362*x*y*u^3-F(79)/101*y^2*u^3+12751*x*z*u^3+F(24)/5*y*z*u^3+F(89)/93*z^2*u^3-12705*x*u^4+F(7)/50*y*u^4-13076*z*u^4+F(31)/119*x^4*v+F(54)/95*x^3*y*v+8277*x^2*y^2*v-5241*x*y^3*v+F(104)/97*y^4*v-F(93)/35*x^3*z*v-F(37)/59*x^2*y*z*v-F(121)/92*x*y^2*z*v+2155*y^3*z*v-F(7)/74*x^2*z^2*v-F(7)/19*x*y*z^2*v-F(117)/122*y^2*z^2*v+F(92)/19*x*z^3*v+F(101)/66*y*z^3*v+6993*x^3*u*v-F(89)/23*x^2*y*u*v+F(8)/3*x*y^2*u*v-6204*y^3*u*v-F(32)/71*x^2*z*u*v-F(23)/99*x*y*z*u*v-2119*y^2*z*u*v+F(61)/85*x*z^2*u*v+F(13)/9*y*z^2*u*v-7207*z^3*u*v+F(24)/79*x^2*u^2*v-F(112)/65*x*y*u^2*v+F(31)/78*y^2*u^2*v+F(75)/28*x*z*u^2*v+F(47)/93*y*z*u^2*v+F(31)/77*z^2*u^2*v+F(18)/115*x*u^3*v+5594*y*u^3*v-F(79)/64*z*u^3*v+F(116)/101*x^3*v^2-F(99)/113*x^2*y*v^2-F(38)/85*x*y^2*v^2+F(76)/43*y^3*v^2+F(4)/53*x^2*z*v^2+824*x*y*z*v^2-F(1)/22*y^2*z*v^2+F(71)/32*x*z^2*v^2-F(33)/100*y*z^2*v^2-2238*z^3*v^2-4249*x^2*u*v^2-15361*x*y*u*v^2-F(53)/113*y^2*u*v^2+F(84)/103*x*z*u*v^2-F(108)/83*y*z*u*v^2+F(98)/27*z^2*u*v^2+F(13)/7*x*u^2*v^2-13553*y*u^2*v^2+15439*z*u^2*v^2+F(53)/30*x^2*v^3+7153*x*y*v^3+9688*y^2*v^3-F(48)/23*x*z*v^3+F(97)/91*y*z*v^3-6525*z^2*v^3+F(81)/28*x*u*v^3-F(76)/3*y*u*v^3+F(121)/102*z*u*v^3-6426*x*v^4+F(44)/67*y*v^4-F(119)/111*z*v^4,-F(73)/17*x^4*u+83*x^3*y*u-F(121)/46*x^2*y^2*u+7547*x*y^3*u-F(119)/76*x^3*z*u-F(91)/68*x^2*y*z*u-F(107)/46*x*y^2*z*u-F(27)/85*y^3*z*u+F(53)/45*x^2*z^2*u+F(61)/36*x*y*z^2*u-1703*y^2*z^2*u-F(55)/89*x*z^3*u+F(44)/23*y*z^3*u+6705*x^3*u^2+F(10)/113*x^2*y*u^2-15029*x*y^2*u^2+F(18)/47*y^3*u^2-F(23)/125*x^2*z*u^2+F(96)/37*x*y*z*u^2-F(23)/9*y^2*z*u^2+F(43)/105*x*z^2*u^2-F(9)/53*y*z^2*u^2-F(35)/76*z^3*u^2+F(5)/71*x^2*u^3+1339*x*y*u^3-F(111)/10*y^2*u^3-F(96)/25*x*z*u^3+9248*y*z*u^3+F(64)/27*z^2*u^3+F(16)/11*x*u^4+F(79)/101*y*u^4+F(56)/25*z*u^4-F(7)/50*u^5-F(47)/55*x^4*v-F(1)/38*x^3*y*v-F(1)/22*x^2*y^2*v-14510*x*y^3*v-7707*x^3*z*v-F(26)/41*x^2*y*z*v-F(77)/43*x*y^2*z*v-F(112)/93*y^3*z*v-12660*x^2*z^2*v-15881*x*y*z^2*v+10804*y^2*z^2*v-F(118)/25*x*z^3*v-6840*y*z^3*v-4699*x^3*u*v+13051*x^2*y*u*v-F(92)/75*x*y^2*u*v-F(104)/97*y^3*u*v+F(89)/16*x^2*z*u*v-F(23)/97*x*y*z*u*v-4712*y^2*z*u*v+15739*x*z^2*u*v+F(80)/79*y*z^2*u*v-10357*z^3*u*v-12726*x^2*u^2*v+F(19)/121*x*y*u^2*v+6204*y^2*u^2*v-12840*x*z*u^2*v+F(53)/7*y*z*u^2*v+8389*z^2*u^2*v+7901*x*u^3*v-F(31)/78*y*u^3*v-9896*z*u^3*v-5594*u^4*v+F(103)/76*x^3*v^2-3893*x^2*y*v^2-F(95)/26*x*y^2*v^2+F(1)/6*x^2*z*v^2+8744*x*y*z*v^2+F(51)/43*y^2*z*v^2+F(101)/44*x*z^2*v^2-F(15)/116*y*z^2*v^2+F(124)/103*z^3*v^2-F(85)/62*x^2*u*v^2+4546*x*y*u*v^2-F(76)/43*y^2*u*v^2+123*x*z*u*v^2-F(105)/94*y*z*u*v^2+F(88)/105*z^2*u*v^2-F(16)/115*x*u^2*v^2+F(53)/113*y*u^2*v^2+F(22)/61*z*u^2*v^2+13553*u^3*v^2+F(57)/94*x^2*v^3+6104*x*y*v^3-F(83)/89*x*z*v^3+F(83)/3*y*z*v^3+6967*z^2*v^3+937*x*u*v^3-9688*y*u*v^3-F(69)/79*z*u*v^3+F(76)/3*u^2*v^3+F(73)/13*x*v^4+8660*z*v^4-F(44)/67*u*v^4,F(73)/17*x^3*y*u-83*x^2*y^2*u+F(121)/46*x*y^3*u-7547*y^4*u-F(83)/23*x^3*z*u-F(22)/101*x^2*y*z*u+F(71)/91*x*y^2*z*u+F(84)/11*y^3*z*u+F(66)/35*x^2*z^2*u-2015*x*y*z^2*u-F(97)/53*y^2*z^2*u-4861*x*z^3*u+F(11)/5*y*z^3*u+10816*x^3*u^2+9058*x^2*y*u^2-F(25)/96*x*y^2*u^2-F(29)/28*y^3*u^2-F(86)/73*x^2*z*u^2+13552*x*y*z*u^2-4662*y^2*z*u^2+11391*x*z^2*u^2-F(90)/83*y*z^2*u^2-11010*z^3*u^2+F(4)/11*x^2*u^3+F(25)/34*x*y*u^3-F(108)/19*y^2*u^3+F(22)/17*x*z*u^3+F(12)/23*y*z*u^3+F(123)/55*z^2*u^3-F(13)/69*x*u^4+1544*y*u^4+6842*z*u^4+12705*u^5+F(47)/55*x^3*y*v+F(1)/38*x^2*y^2*v+F(1)/22*x*y^3*v+14510*y^4*v+F(86)/67*x^3*z*v-1018*x^2*y*z*v+F(96)/77*x*y^2*z*v-14084*y^3*z*v+8639*x^2*z^2*v-147*x*y*z^2*v-F(11)/75*y^2*z^2*v+3355*x*z^3*v-2733*y*z^3*v-F(31)/119*x^3*u*v-4057*x^2*y*u*v-F(2)/3*x*y^2*u*v-10540*y^3*u*v+8173*x^2*z*u*v+5935*x*y*z*u*v+F(81)/41*y^2*z*u*v-11180*x*z^2*u*v-F(24)/41*y*z^2*u*v+13119*z^3*u*v-6993*x^2*u^2*v-F(121)/6*x*y*u^2*v-F(118)/73*y^2*u^2*v+13568*x*z*u^2*v+F(21)/113*y*z*u^2*v-3893*z^2*u^2*v-F(24)/79*x*u^3*v-F(23)/41*y*u^3*v-F(79)/45*z*u^3*v-F(18)/115*u^4*v-F(103)/76*x^2*y*v^2+3893*x*y^2*v^2+F(95)/26*y^3*v^2-F(83)/114*x^2*z*v^2-F(7)/11*x*y*z*v^2+F(27)/16*y^2*z*v^2-177*x*z^2*v^2-F(103)/73*y*z^2*v^2+13129*z^3*v^2-F(116)/101*x^2*u*v^2-14459*x*y*u*v^2-F(73)/67*y^2*u*v^2-12753*x*z*u*v^2-7892*y*z*u*v^2-F(91)/43*z^2*u*v^2+4249*x*u^2*v^2-1608*y*u^2*v^2-F(52)/89*z*u^2*v^2-F(13)/7*u^3*v^2-F(57)/94*x*y*v^3-6104*y^2*v^3+1944*x*z*v^3-6010*y*z*v^3-1089*z^2*v^3-F(53)/30*x*u*v^3-F(28)/87*y*u*v^3-5678*z*u*v^3-F(81)/28*u^2*v^3-F(73)/13*y*v^4+F(83)/49*z*v^4+6426*u*v^4,F(83)/23*x^4*u-F(24)/73*x^3*y*u+F(43)/40*x^2*y^2*u-F(1)/24*x*y^3*u+F(27)/85*y^4*u-F(66)/35*x^3*z*u-F(23)/54*x^2*y*z*u+15610*x*y^2*z*u+1703*y^3*z*u+4861*x^2*z^2*u-F(61)/84*x*y*z^2*u-F(44)/23*y^2*z^2*u+9058*x^3*u^2+2037*x^2*y*u^2+14286*x*y^2*u^2+8579*y^3*u^2-F(41)/14*x^2*z*u^2-15012*x*y*z*u^2-F(11)/126*y^2*z*u^2+F(40)/51*x*z^2*u^2-F(121)/79*y*z^2*u^2+2972*x^2*u^3+F(53)/121*x*y*u^3-F(21)/25*y^2*u^3+F(73)/97*x*z*u^3+8115*y*z*u^3+5889*z^2*u^3+F(119)/80*x*u^4+14069*y*u^4-F(89)/93*z*u^4+13076*u^5-F(86)/67*x^4*v+F(2)/11*x^3*y*v-F(124)/37*x^2*y^2*v+F(97)/13*x*y^3*v+F(112)/93*y^4*v-8639*x^3*z*v+F(53)/5*x^2*y*z*v-F(61)/77*x*y^2*z*v-10804*y^3*z*v-3355*x^2*z^2*v+5297*x*y*z^2*v+6840*y^2*z^2*v-F(97)/80*x^3*u*v+F(91)/48*x^2*y*u*v-F(100)/27*x*y^2*u*v-F(57)/25*y^3*u*v-12597*x^2*z*u*v+6438*x*y*z*u*v+12092*y^2*z*u*v+F(125)/122*x*z^2*u*v+F(31)/121*y*z^2*u*v+F(99)/7*x^2*u^2*v+F(19)/58*x*y*u^2*v-11599*y^2*u^2*v+505*x*z*u^2*v-11945*y*z*u^2*v+7207*z^2*u^2*v-F(102)/83*x*u^3*v+F(38)/51*y*u^3*v-F(31)/77*z*u^3*v+F(79)/64*u^4*v+F(83)/114*x^3*v^2+F(31)/66*x^2*y*v^2-F(37)/115*x*y^2*v^2-F(51)/43*y^3*v^2+177*x^2*z*v^2-F(13)/71*x*y*z*v^2+F(15)/116*y^2*z*v^2-13129*x*z^2*v^2-F(124)/103*y*z^2*v^2-F(2)/125*x^2*u*v^2+6945*x*y*u*v^2-10580*y^2*u*v^2-F(59)/122*x*z*u*v^2-F(65)/8*y*z*u*v^2+2238*z^2*u*v^2-F(67)/3*x*u^2*v^2+F(43)/39*y*u^2*v^2-F(98)/27*z*u^2*v^2-15439*u^3*v^2-1944*x^2*v^3-5132*x*y*v^3-F(83)/3*y^2*v^3+1089*x*z*v^3-6967*y*z*v^3-F(66)/95*x*u*v^3+4788*y*u*v^3+6525*z*u*v^3-F(121)/102*u^2*v^3-F(83)/49*x*v^4-8660*y*v^4+F(119)/111*u*v^4,-12488*x^4*u+9830*x^3*y*u-9515*x^2*y^2*u+F(125)/52*x*y^3*u+F(51)/70*x^3*z*u-14191*x^2*y*z*u-F(42)/43*x*y^2*z*u+F(7)/48*x^2*z^2*u-F(11)/67*x*y*z^2*u+F(16)/113*x^3*u^2+F(47)/60*x^2*y*u^2-F(125)/86*x*y^2*u^2+5479*y^3*u^2+5930*x^2*z*u^2+4132*x*y*z*u^2+F(33)/31*y^2*z*u^2-F(44)/17*x*z^2*u^2+F(44)/43*y*z^2*u^2+9105*x^2*u^3-F(73)/105*x*y*u^3-14506*y^2*u^3+15241*x*z*u^3-F(99)/59*y*z*u^3-9854*z^2*u^3+F(121)/119*x*u^4+12922*y*u^4-8501*z*u^4+F(3)/91*u^5+F(93)/100*x^4*v-F(12)/49*x^3*y*v+F(9)/83*x^2*y^2*v-F(1)/40*x*y^3*v+F(96)/29*x^3*z*v-7902*x^2*y*z*v-F(7)/103*x*y^2*z*v+9294*x^2*z^2*v-F(104)/47*x*y*z^2*v+15428*x^3*u*v+6012*x^2*y*u*v-F(112)/113*x*y^2*u*v+F(27)/94*y^3*u*v+F(81)/71*x^2*z*u*v-558*x*y*z*u*v-F(28)/17*y^2*z*u*v-F(30)/91*x*z^2*u*v+F(94)/37*y*z^2*u*v+F(124)/29*x^2*u^2*v+F(65)/114*x*y*u^2*v-F(53)/52*y^2*u^2*v-14643*x*z*u^2*v+12096*y*z*u^2*v-F(5)/121*z^2*u^2*v-2512*x*u^3*v-1383*y*u^3*v+F(37)/51*z*u^3*v+3101*u^4*v-F(56)/101*x^3*v^2-8911*x^2*y*v^2-10735*x*y^2*v^2+F(112)/93*y^3*v^2-F(108)/61*x^2*z*v^2+8965*x*y*z*v^2-10804*y^2*z*v^2+6611*x*z^2*v^2+6840*y*z^2*v^2+13229*x^2*u*v^2+F(78)/25*x*y*u*v^2-F(56)/65*y^2*u*v^2-5367*x*z*u*v^2-F(28)/73*y*z*u*v^2+F(79)/121*z^2*u*v^2+4395*x*u^2*v^2+10920*y*u^2*v^2+14028*z*u^2*v^2+F(55)/93*u^3*v^2+F(19)/115*x^2*v^3+F(42)/29*x*y*v^3-F(51)/43*y^2*v^3-F(109)/2*x*z*v^3+F(15)/116*y*z*v^3-F(124)/103*z^2*v^3+3222*x*u*v^3+2940*y*u*v^3-F(102)/13*z*u*v^3+F(94)/45*u^2*v^3-F(77)/24*x*v^4-F(83)/3*y*v^4-6967*z*v^4+F(71)/88*u*v^4-8660*v^5,12488*x^3*y*u-9830*x^2*y^2*u+9515*x*y^3*u-F(125)/52*y^4*u-F(51)/70*x^2*y*z*u+14191*x*y^2*z*u+F(42)/43*y^3*z*u-F(7)/48*x*y*z^2*u+F(11)/67*y^2*z^2*u+12038*x^3*u^2-1516*x^2*y*u^2+F(99)/47*x*y^2*u^2+1873*y^3*u^2-F(38)/49*x^2*z*u^2-11913*x*y*z*u^2+4245*y^2*z*u^2-F(121)/106*x*z^2*u^2-5987*y*z^2*u^2+F(41)/103*x^2*u^3-F(67)/37*x*y*u^3+5547*y^2*u^3+F(57)/56*x*z*u^3+F(79)/43*y*z*u^3-5226*z^2*u^3-F(102)/97*x*u^4+F(112)/69*y*u^4-11764*z*u^4-F(76)/37*u^5-F(93)/100*x^3*y*v+F(12)/49*x^2*y^2*v-F(9)/83*x*y^3*v+F(1)/40*y^4*v-F(96)/29*x^2*y*z*v+7902*x*y^2*z*v+F(7)/103*y^3*z*v-9294*x*y*z^2*v+F(104)/47*y^2*z^2*v+F(124)/17*x^3*u*v+6843*x^2*y*u*v-F(38)/45*x*y^2*u*v+4700*y^3*u*v-11504*x^2*z*u*v-6893*x*y*z*u*v+7035*y^2*z*u*v-F(38)/49*x*z^2*u*v-F(5)/52*y*z^2*u*v-15908*x^2*u^2*v+453*x*y*u^2*v+F(31)/27*y^2*u^2*v-13560*x*z*u^2*v+14389*y*z*u^2*v-8811*z^2*u^2*v-F(29)/70*x*u^3*v-11913*y*u^3*v-12227*z*u^3*v-6726*u^4*v-F(86)/67*x^3*v^2+F(107)/28*x^2*y*v^2+14960*x*y^2*v^2-F(97)/30*y^3*v^2-8639*x^2*z*v^2+10711*x*y*z*v^2-12705*y^2*z*v^2-3355*x*z^2*v^2+F(51)/73*y*z^2*v^2+F(29)/36*x^2*u*v^2-F(96)/85*x*y*u*v^2-15876*y^2*u*v^2+F(53)/18*x*z*u*v^2-F(23)/21*y*z*u*v^2-1566*z^2*u*v^2-F(78)/79*x*u^2*v^2+F(119)/89*y*u^2*v^2+10016*z*u^2*v^2-F(51)/106*u^3*v^2+F(83)/114*x^2*v^3+F(10)/19*x*y*v^3+12833*y^2*v^3+177*x*z*v^3-F(66)/61*y*z*v^3-13129*z^2*v^3+10909*x*u*v^3+F(120)/37*y*u*v^3-F(107)/26*z*u*v^3-F(88)/39*u^2*v^3-1944*x*v^4+F(54)/125*y*v^4+1089*z*v^4+F(91)/27*u*v^4-F(83)/49*v^5,12488*x^3*z*u-9830*x^2*y*z*u+9515*x*y^2*z*u-F(125)/52*y^3*z*u-F(51)/70*x^2*z^2*u+14191*x*y*z^2*u+F(42)/43*y^2*z^2*u-F(7)/48*x*z^3*u+F(11)/67*y*z^3*u-F(16)/15*x^3*u^2-4599*x^2*y*u^2-F(19)/59*x*y^2*u^2+183*y^3*u^2-14202*x^2*z*u^2-F(59)/40*x*y*z*u^2-F(34)/37*y^2*z*u^2-F(2)/111*x*z^2*u^2-1985*y*z^2*u^2+F(37)/104*z^3*u^2+F(22)/59*x^2*u^3+F(83)/32*x*y*u^3-F(23)/94*y^2*u^3+F(77)/62*x*z*u^3-F(26)/81*y*z*u^3-6215*z^2*u^3-5572*x*u^4-F(83)/22*y*u^4+7636*z*u^4+F(23)/101*u^5-F(93)/100*x^3*z*v+F(12)/49*x^2*y*z*v-F(9)/83*x*y^2*z*v+F(1)/40*y^3*z*v-F(96)/29*x^2*z^2*v+7902*x*y*z^2*v+F(7)/103*y^2*z^2*v-9294*x*z^3*v+F(104)/47*y*z^3*v+F(33)/68*x^3*u*v-4275*x^2*y*u*v-9872*x*y^2*u*v-F(33)/26*y^3*u*v-14751*x^2*z*u*v+F(85)/58*x*y*z*u*v-5679*y^2*z*u*v+F(125)/43*x*z^2*u*v+F(92)/53*y*z^2*u*v-F(68)/97*z^3*u*v-3472*x^2*u^2*v-F(114)/73*x*y*u^2*v+15629*y^2*u^2*v+F(9)/89*x*z*u^2*v+9184*y*z*u^2*v+F(33)/68*z^2*u^2*v+7215*x*u^3*v+13183*y*u^3*v+10346*z*u^3*v-12846*u^4*v+F(47)/55*x^3*v^2+F(1)/38*x^2*y*v^2+F(1)/22*x*y^2*v^2+14510*y^3*v^2-F(103)/17*x^2*z*v^2+F(57)/107*x*y*z*v^2+F(3)/95*y^2*z*v^2+F(99)/106*x*z^2*v^2-F(36)/37*y*z^2*v^2-4047*z^3*v^2+F(17)/126*x^2*u*v^2-F(5)/79*x*y*u*v^2+171*y^2*u*v^2-F(77)/95*x*z*u*v^2-F(89)/106*y*z*u*v^2-F(77)/79*z^2*u*v^2+407*x*u^2*v^2-F(67)/91*y*u^2*v^2+11686*z*u^2*v^2+F(124)/55*u^3*v^2-F(103)/76*x^2*v^3+3893*x*y*v^3+F(95)/26*y^2*v^3-11174*x*z*v^3-F(51)/124*y*z*v^3+F(56)/15*z^2*v^3+F(66)/41*x*u*v^3+F(24)/121*y*u*v^3+F(113)/35*z*u*v^3-F(64)/109*u^2*v^3-F(57)/94*x*v^4-6104*y*v^4-4474*z*v^4+F(53)/21*u*v^4-F(73)/13*v^5,-F(16)/15*x^4*u-4599*x^3*y*u-F(19)/59*x^2*y^2*u+183*x*y^3*u+6748*x^3*z*u-F(83)/120*x^2*y*z*u-F(47)/41*x*y^2*z*u+5479*y^3*z*u+F(61)/89*x^2*z^2*u+2147*x*y*z^2*u+F(33)/31*y^2*z^2*u+6747*x*z^3*u+F(44)/43*y*z^3*u+F(22)/59*x^3*u^2+F(83)/32*x^2*y*u^2-F(23)/94*x*y^2*u^2-15145*x^2*z*u^2+F(47)/87*x*y*z*u^2-14506*y^2*z*u^2+F(113)/39*x*z^2*u^2-F(99)/59*y*z^2*u^2-9854*z^3*u^2-5572*x^2*u^3-F(83)/22*x*y*u^3-F(99)/53*x*z*u^3+12922*y*z*u^3-8501*z^2*u^3+F(23)/101*x*u^4+F(3)/91*z*u^4+F(101)/97*x^4*v-4192*x^3*y*v+F(98)/115*x^2*y^2*v-F(114)/37*x*y^3*v+F(38)/63*x^3*z*v+F(91)/6*x^2*y*z*v+F(68)/103*x*y^2*z*v+11383*y^3*z*v+9265*x^2*z^2*v-10732*x*y*z^2*v+2059*y^2*z^2*v-F(69)/5*x*z^3*v-F(25)/121*y*z^3*v+3233*x^3*u*v+9915*x^2*y*u*v+600*x*y^2*u*v+F(18)/47*y^3*u*v+12081*x^2*z*u*v-5610*x*y*z*u*v+15240*y^2*z*u*v-F(75)/86*x*z^2*u*v-F(81)/22*y*z^2*u*v+5249*z^3*u*v+12622*x^2*u^2*v+14522*x*y*u^2*v-F(111)/10*y^2*u^2*v-F(92)/33*x*z*u^2*v-F(100)/61*y*z*u^2*v+F(82)/49*z^2*u^2*v-7028*x*u^3*v+F(79)/101*y*u^3*v+14620*z*u^3*v-F(7)/50*u^4*v-1906*x^3*v^2+4547*x^2*y*v^2-F(87)/2*x*y^2*v^2-F(104)/97*y^3*v^2+F(35)/47*x^2*z*v^2-F(119)/69*x*y*z*v^2+701*y^2*z*v^2+F(12)/59*x*z^2*v^2-8115*y*z^2*v^2-F(52)/7*z^3*v^2-12319*x^2*u*v^2+F(85)/89*x*y*u*v^2+6204*y^2*u*v^2+F(111)/79*x*z*u*v^2+F(109)/87*y*z*u*v^2-9574*z^2*u*v^2-F(5)/97*x*u^2*v^2-F(31)/78*y*u^2*v^2+F(35)/39*z*u^2*v^2-5594*u^3*v^2-F(56)/29*x^2*v^3+F(65)/73*x*y*v^3-F(76)/43*y^2*v^3-F(58)/113*x*z*v^3+F(53)/27*y*z*v^3+F(115)/7*z^2*v^3+11392*x*u*v^3+F(53)/113*y*u*v^3-F(121)/41*z*u*v^3+13553*u^2*v^3-5154*x*v^4-9688*y*v^4+13736*z*v^4+F(76)/3*u*v^4-F(44)/67*v^5,F(16)/15*x^3*y*u+4599*x^2*y^2*u+F(19)/59*x*y^3*u-183*y^4*u+12038*x^3*z*u-F(5)/58*x^2*y*z*u-12980*x*y^2*z*u+13114*y^3*z*u-F(38)/49*x^2*z^2*u-F(104)/19*x*y*z^2*u+6230*y^2*z^2*u-F(121)/106*x*z^3*u-F(13)/69*y*z^3*u-F(22)/59*x^2*y*u^2-F(83)/32*x*y^2*u^2+F(23)/94*y^3*u^2+F(41)/103*x^2*z*u^2+F(121)/70*x*y*z*u^2+F(58)/69*y^2*z*u^2+F(57)/56*x*z^2*u^2-F(118)/95*y*z^2*u^2-5226*z^3*u^2+5572*x*y*u^3+F(83)/22*y^2*u^3-F(102)/97*x*z*u^3+F(37)/32*y*z*u^3-11764*z^2*u^3-F(23)/101*y*u^4-F(76)/37*z*u^4-F(101)/97*x^3*y*v+4192*x^2*y^2*v-F(98)/115*x*y^3*v+F(114)/37*y^4*v+14731*x^3*z*v-F(61)/39*x^2*y*z*v+F(1)/52*x*y^2*z*v-F(4)/77*y^3*z*v+F(45)/53*x^2*z^2*v-5935*x*y*z^2*v-F(11)/20*y^2*z^2*v+F(77)/96*x*z^3*v-4017*y*z^3*v+10816*x^3*u*v+F(23)/120*x^2*y*u*v-F(83)/59*x*y^2*u*v-F(107)/106*y^3*u*v+5126*x^2*z*u*v+13286*x*y*z*u*v-12660*y^2*z*u*v-F(7)/59*x*z^2*u*v-F(65)/22*y*z^2*u*v-F(45)/92*z^3*u*v+F(4)/11*x^2*u^2*v-F(31)/51*x*y*u^2*v+F(67)/114*y^2*u^2*v+5727*x*z*u^2*v-F(89)/23*y*z*u^2*v+15113*z^2*u^2*v-F(13)/69*x*u^3*v-F(119)/20*y*u^3*v+116*z*u^3*v+12705*u^4*v-F(31)/119*x^3*v^2+F(73)/14*x^2*y*v^2-12824*x*y^2*v^2-10711*y^3*v^2+F(59)/71*x^2*z*v^2+F(123)/47*x*y*z*v^2-10507*y^2*z*v^2+8373*x*z^2*v^2+F(19)/111*y*z^2*v^2+F(25)/36*z^3*v^2-6993*x^2*u*v^2-F(31)/50*x*y*u*v^2-F(83)/56*y^2*u*v^2-10325*x*z*u*v^2-F(77)/9*y*z*u*v^2+6123*z^2*u*v^2-F(24)/79*x*u^2*v^2-F(115)/61*y*u^2*v^2+F(67)/60*z*u^2*v^2-F(18)/115*u^3*v^2-F(116)/101*x^2*v^3-9779*x*y*v^3+F(33)/100*y^2*v^3+F(85)/52*x*z*v^3-F(36)/97*y*z*v^3+2426*z^2*v^3+4249*x*u*v^3+3969*y*u*v^3+F(125)/102*z*u*v^3-F(13)/7*u^2*v^3-F(53)/30*x*v^4+F(7)/16*y*v^4+13283*z*v^4-F(81)/28*u*v^4+6426*v^5,-12038*x^4*u-F(23)/107*x^3*y*u+9515*x^2*y^2*u+F(107)/119*x*y^3*u-5479*y^4*u+F(38)/49*x^3*z*u+F(116)/123*x^2*y*z*u+F(67)/42*x*y^2*z*u-F(33)/31*y^3*z*u+F(121)/106*x^2*z^2*u-F(53)/60*x*y*z^2*u-F(44)/43*y^2*z^2*u-F(41)/103*x^3*u^2+F(29)/104*x^2*y*u^2+14867*x*y^2*u^2+14506*y^3*u^2-F(57)/56*x^2*z*u^2+F(71)/51*x*y*z*u^2+F(99)/59*y^2*z*u^2+5226*x*z^2*u^2+9854*y*z^2*u^2+F(102)/97*x^2*u^3+F(100)/61*x*y*u^3-12922*y^2*u^3+11764*x*z*u^3+8501*y*z*u^3+F(76)/37*x*u^4-F(3)/91*y*u^4-14731*x^4*v+14102*x^3*y*v-F(121)/116*x^2*y^2*v+12653*x*y^3*v-11383*y^4*v-F(45)/53*x^3*z*v+F(115)/48*x^2*y*z*v-F(51)/7*x*y^2*z*v-2059*y^3*z*v-F(77)/96*x^2*z^2*v+10429*x*y*z^2*v+F(25)/121*y^2*z^2*v-F(106)/41*x^3*u*v+3786*x^2*y*u*v-11876*x*y^2*u*v+F(89)/72*y^3*u*v-F(107)/105*x^2*z*u*v+F(92)/13*x*y*z*u*v-F(87)/13*y^2*z*u*v-F(21)/125*x*z^2*u*v-F(77)/9*y*z^2*u*v+F(51)/28*x^2*u^2*v+10724*x*y*u^2*v+F(68)/113*y^2*u^2*v+25*x*z*u^2*v-F(12)/47*y*z*u^2*v+5889*z^2*u^2*v-F(97)/92*x*u^3*v-F(12)/35*y*u^3*v-F(89)/93*z*u^3*v+13076*u^4*v-1646*x^3*v^2+F(57)/34*x^2*y*v^2+F(45)/91*x*y^2*v^2+F(19)/56*y^3*v^2-159*x^2*z*v^2-F(3)/28*x*y*z*v^2-3684*y^2*z*v^2+F(29)/37*x*z^2*v^2-F(48)/121*y*z^2*v^2-12654*x^2*u*v^2-F(79)/99*x*y*u*v^2+9472*y^2*u*v^2-F(6)/37*x*z*u*v^2-F(11)/101*y*z*u*v^2+7207*z^2*u*v^2+F(103)/13*x*u^2*v^2+F(73)/80*y*u^2*v^2-F(31)/77*z*u^2*v^2+F(79)/64*u^3*v^2+13916*x^2*v^3+5449*x*y*v^3-13520*y^2*v^3-F(37)/112*x*z*v^3-F(29)/104*y*z*v^3+2238*z^2*v^3+14745*x*u*v^3+F(32)/23*y*u*v^3-F(98)/27*z*u*v^3-15439*u^2*v^3+F(52)/113*x*v^4+8059*y*v^4+6525*z*v^4-F(121)/102*u*v^4+F(119)/111*v^5,-11692*x^4*u+F(125)/4*x^3*y*u-8745*x^2*y^2*u-4809*x*y^3*u-237*x^3*z*u+13677*x^2*y*z*u-F(55)/79*x*y^2*z*u+F(53)/89*x^2*z^2*u+11441*x*y*z^2*u+15569*x^3*u^2-F(44)/89*x^2*y*u^2+F(5)/3*x*y^2*u^2-12573*x^2*z*u^2+F(52)/25*x*y*z*u^2-F(37)/11*x*z^2*u^2+F(47)/93*x^2*u^3-F(49)/101*x*y*u^3+F(17)/80*x*z*u^3+9769*x*u^4+10712*x^4*v-F(116)/35*x^3*y*v-F(89)/31*x^2*y^2*v-F(17)/120*x*y^3*v+4676*x^3*z*v-2606*x^2*y*z*v-F(112)/123*x*y^2*z*v-5522*x^2*z^2*v+F(88)/53*x*y*z^2*v+11648*x*z^3*v+F(114)/61*x^3*u*v-12976*x^2*y*u*v+8743*x*y^2*u*v-F(27)/71*y^3*u*v+F(117)/49*x^2*z*u*v+F(50)/67*x*y*z*u*v+F(19)/36*y^2*z*u*v-1447*x*z^2*u*v-10133*y*z^2*u*v-F(24)/109*z^3*u*v-F(19)/11*x^2*u^2*v+F(117)/38*x*y*u^2*v-F(72)/83*y^2*u^2*v-F(33)/4*x*z*u^2*v-9934*y*z*u^2*v+F(28)/9*z^2*u^2*v-F(23)/99*x*u^3*v+F(25)/28*y*u^3*v+F(83)/67*z*u^3*v+11180*u^4*v+F(46)/35*x^3*v^2+10091*x^2*y*v^2+F(121)/97*x*y^2*v^2-F(20)/97*y^3*v^2-F(47)/85*x^2*z*v^2+F(10)/99*x*y*z*v^2-F(104)/49*y^2*z*v^2+41*x*z^2*v^2+700*y*z^2*v^2+F(102)/73*z^3*v^2+151*x^2*u*v^2+995*x*y*u*v^2-12504*y^2*u*v^2-10306*x*z*u*v^2+F(17)/52*y*z*u*v^2+134*z^2*u*v^2+F(79)/87*x*u^2*v^2-F(103)/74*y*u^2*v^2-F(107)/104*z*u^2*v^2+6496*u^3*v^2+9018*x^2*v^3-4971*x*y*v^3+F(113)/53*y^2*v^3-F(70)/117*x*z*v^3-F(103)/14*y*z*v^3-8285*z^2*v^3-F(82)/83*x*u*v^3+F(51)/7*y*u*v^3-7961*z*u*v^3+F(9)/76*u^2*v^3-F(109)/110*x*v^4+F(36)/121*y*v^4+F(123)/34*z*v^4+6815*u*v^4-F(83)/124*v^5,-11692*x^3*y*u+F(125)/4*x^2*y^2*u-8745*x*y^3*u-4809*y^4*u-237*x^2*y*z*u+13677*x*y^2*z*u-F(55)/79*y^3*z*u+F(53)/89*x*y*z^2*u+11441*y^2*z^2*u+15569*x^2*y*u^2-F(44)/89*x*y^2*u^2+F(5)/3*y^3*u^2-12573*x*y*z*u^2+F(52)/25*y^2*z*u^2-F(37)/11*y*z^2*u^2+F(47)/93*x*y*u^3-F(49)/101*y^2*u^3+F(17)/80*y*z*u^3+9769*y*u^4+10712*x^3*y*v-F(116)/35*x^2*y^2*v-F(89)/31*x*y^3*v-F(17)/120*y^4*v+4676*x^2*y*z*v-2606*x*y^2*z*v-F(112)/123*y^3*z*v-5522*x*y*z^2*v+F(88)/53*y^2*z^2*v+11648*y*z^3*v-F(59)/15*x^3*u*v-7839*x^2*y*u*v-13185*x*y^2*u*v-1150*y^3*u*v-10267*x^2*z*u*v-12451*x*y*z*u*v+F(113)/72*y^2*z*u*v+F(78)/115*x*z^2*u*v+F(95)/99*y*z^2*u*v+F(5)/113*z^3*u*v-F(61)/98*x^2*u^2*v-10094*x*y*u^2*v-F(5)/11*y^2*u^2*v+12390*x*z*u^2*v+11764*y*z*u^2*v-1225*z^2*u^2*v-8037*x*u^3*v+F(90)/41*y*u^3*v-F(37)/117*z*u^3*v-6599*u^4*v-F(61)/126*x^3*v^2-F(74)/35*x^2*y*v^2-F(95)/71*x*y^2*v^2+F(11)/49*y^3*v^2+910*x^2*z*v^2-4592*x*y*z*v^2+27*y^2*z*v^2+9398*x*z^2*v^2-F(119)/124*y*z^2*v^2-F(96)/41*z^3*v^2+6439*x^2*u*v^2+2959*x*y*u*v^2-3482*y^2*u*v^2+F(90)/53*x*z*u*v^2+5132*y*z*u*v^2+F(23)/71*z^2*u*v^2+F(67)/109*x*u^2*v^2-9936*y*u^2*v^2-F(87)/31*z*u^2*v^2-11613*u^3*v^2-F(113)/58*x^2*v^3-15911*x*y*v^3+F(68)/63*y^2*v^3-11206*x*z*v^3-6576*y*z*v^3+F(21)/104*z^2*v^3+F(88)/107*x*u*v^3+F(25)/98*y*u*v^3-11659*z*u*v^3-F(1)/116*u^2*v^3-F(55)/57*x*v^4-F(88)/119*y*v^4-F(98)/17*z*v^4+1080*u*v^4+F(47)/118*v^5,11692*x^3*u^2-F(125)/4*x^2*y*u^2+8745*x*y^2*u^2+4809*y^3*u^2+237*x^2*z*u^2-13677*x*y*z*u^2+F(55)/79*y^2*z*u^2-F(53)/89*x*z^2*u^2-11441*y*z^2*u^2-15569*x^2*u^3+F(44)/89*x*y*u^3-F(5)/3*y^2*u^3+12573*x*z*u^3-F(52)/25*y*z*u^3+F(37)/11*z^2*u^3-F(47)/93*x*u^4+F(49)/101*y*u^4-F(17)/80*z*u^4-9769*u^5-8037*x^3*u*v+F(37)/10*x^2*y*u*v+F(123)/85*x*y^2*u*v+F(123)/19*y^3*u*v+8286*x^2*z*u*v-F(97)/87*x*y*z*u*v-F(114)/89*y^2*z*u*v-F(35)/48*x*z^2*u*v+F(49)/65*y*z^2*u*v-F(71)/77*x^2*u^2*v+F(90)/109*x*y*u^2*v+F(37)/77*y^2*u^2*v-F(95)/14*x*z*u^2*v+F(30)/77*y*z*u^2*v+F(19)/70*z^2*u^2*v-9937*x*u^3*v-8506*y*u^3*v+F(113)/94*z*u^3*v+13312*u^4*v-F(57)/47*x^3*v^2+14524*x^2*y*v^2+448*x*y^2*v^2-F(79)/18*y^3*v^2-F(89)/74*x^2*z*v^2+F(71)/52*x*y*z*v^2-F(9)/35*y^2*z*v^2+3022*x*z^2*v^2-7869*y*z^2*v^2+F(31)/13*x^2*u*v^2+14867*x*y*u*v^2-6700*y^2*u*v^2+243*x*z*u*v^2-3914*y*z*u*v^2-2718*z^2*u*v^2+F(43)/17*x*u^2*v^2+F(77)/27*y*u^2*v^2+11899*z*u^2*v^2-F(25)/93*u^3*v^2+13520*x^2*v^3+2295*x*y*v^3-13623*y^2*v^3+F(38)/39*x*z*v^3-15347*y*z*v^3+12990*z^2*v^3-F(103)/96*x*u*v^3+F(73)/122*y*u*v^3-3608*z*u*v^3-F(101)/56*u^2*v^3+F(104)/87*x*v^4-F(119)/73*y*v^4-F(75)/92*z*v^4-F(47)/3*u*v^4-F(24)/53*v^5,F(109)/12*x^4*u+F(27)/70*x^3*y*u+6761*x^2*y^2*u-F(10)/107*x*y^3*u+F(15)/116*x^3*z*u-F(43)/79*x^2*y*z*u-F(27)/107*x*y^2*z*u-F(53)/112*x^2*z^2*u+5602*x*y*z^2*u+11648*x*z^3*u-11169*x^3*u^2+F(95)/118*x^2*y*u^2-F(108)/61*x*y^2*u^2-F(27)/71*y^3*u^2-F(21)/79*x^2*z*u^2+F(47)/25*x*y*z*u^2+F(19)/36*y^2*z*u^2+F(29)/60*x*z^2*u^2-10133*y*z^2*u^2-F(24)/109*z^3*u^2-12847*x^2*u^3-F(1)/20*x*y*u^3-F(72)/83*y^2*u^3-F(4)/73*x*z*u^3-9934*y*z*u^3+F(28)/9*z^2*u^3-6723*x*u^4+F(25)/28*y*u^4+F(83)/67*z*u^4+11180*u^5-F(57)/47*x^4*v+14524*x^3*y*v+448*x^2*y^2*v-F(79)/18*x*y^3*v-F(89)/74*x^3*z*v+F(71)/52*x^2*y*z*v-F(9)/35*x*y^2*z*v+3022*x^2*z^2*v-7869*x*y*z^2*v-F(52)/119*x^3*u*v-7033*x^2*y*u*v+F(16)/93*x*y^2*u*v-F(20)/97*y^3*u*v+F(125)/89*x^2*z*u*v-F(71)/118*x*y*z*u*v-F(104)/49*y^2*z*u*v-2677*x*z^2*u*v+700*y*z^2*u*v+F(102)/73*z^3*u*v+5799*x^2*u^2*v-187*x*y*u^2*v-12504*y^2*u^2*v+1593*x*z*u^2*v+F(17)/52*y*z*u^2*v+134*z^2*u^2*v+F(63)/80*x*u^3*v-F(103)/74*y*u^3*v-F(107)/104*z*u^3*v+6496*u^4*v+13520*x^3*v^2+2295*x^2*y*v^2-13623*x*y^2*v^2+F(38)/39*x^2*z*v^2-15347*x*y*z*v^2+12990*x*z^2*v^2+14682*x^2*u*v^2-F(22)/87*x*y*u*v^2+F(113)/53*y^2*u*v^2-F(85)/123*x*z*u*v^2-F(103)/14*y*z*u*v^2-8285*z^2*u*v^2-F(112)/13*x*u^2*v^2+F(51)/7*y*u^2*v^2-7961*z*u^2*v^2+F(9)/76*u^3*v^2+F(104)/87*x^2*v^3-F(119)/73*x*y*v^3-F(75)/92*x*z*v^3-F(77)/57*x*u*v^3+F(36)/121*y*u*v^3+F(123)/34*z*u*v^3+6815*u^2*v^3-F(24)/53*x*v^4-F(83)/124*u*v^4,F(109)/12*x^3*y*u+F(27)/70*x^2*y^2*u+6761*x*y^3*u-F(10)/107*y^4*u+F(15)/116*x^2*y*z*u-F(43)/79*x*y^2*z*u-F(27)/107*y^3*z*u-F(53)/112*x*y*z^2*u+5602*y^2*z^2*u+11648*y*z^3*u-F(59)/15*x^3*u^2-F(97)/29*x^2*y*u^2+7654*x*y^2*u^2-F(104)/119*y^3*u^2-10267*x^2*z*u^2+F(123)/53*x*y*z*u^2+F(66)/125*y^2*z*u^2+F(78)/115*x*z^2*u^2-F(4)/83*y*z^2*u^2+F(5)/113*z^3*u^2-F(61)/98*x^2*u^3+F(80)/107*x*y*u^3+F(89)/67*y^2*u^3+12390*x*z*u^3+F(41)/116*y*z*u^3-1225*z^2*u^3-8037*x*u^4-F(8)/47*y*u^4-F(37)/117*z*u^4-6599*u^5-F(57)/47*x^3*y*v+14524*x^2*y^2*v+448*x*y^3*v-F(79)/18*y^4*v-F(89)/74*x^2*y*z*v+F(71)/52*x*y^2*z*v-F(9)/35*y^3*z*v+3022*x*y*z^2*v-7869*y^2*z^2*v-F(61)/126*x^3*u*v-6187*x^2*y*u*v+8107*x*y^2*u*v-171*y^3*u*v+910*x^2*z*u*v-F(73)/103*x*y*z*u*v-F(26)/107*y^2*z*u*v+9398*x*z^2*u*v-F(93)/112*y*z^2*u*v-F(96)/41*z^3*u*v+6439*x^2*u^2*v+8607*x*y*u^2*v+F(65)/48*y^2*u^2*v+F(90)/53*x*z*u^2*v-14960*y*z*u^2*v+F(23)/71*z^2*u^2*v+F(67)/109*x*u^3*v+F(90)/107*y*u^3*v-F(87)/31*z*u^3*v-11613*u^4*v+13520*x^2*y*v^2+2295*x*y^2*v^2-13623*y^3*v^2+F(38)/39*x*y*z*v^2-15347*y^2*z*v^2+12990*y*z^2*v^2-F(113)/58*x^2*u*v^2-10247*x*y*u*v^2+2678*y^2*u*v^2-11206*x*z*u*v^2-F(111)/22*y*z*u*v^2+F(21)/104*z^2*u*v^2+F(88)/107*x*u^2*v^2-15589*y*u^2*v^2-11659*z*u^2*v^2-F(1)/116*u^3*v^2+F(104)/87*x*y*v^3-F(119)/73*y^2*v^3-F(75)/92*y*z*v^3-F(55)/57*x*u*v^3+F(19)/48*y*u*v^3-F(98)/17*z*u*v^3+1080*u^2*v^3-F(24)/53*y*v^4+F(47)/118*u*v^4,-F(59)/15*x^4*u-2072*x^3*y*u-209*x^2*y^2*u+F(109)/97*x*y^3*u+F(27)/71*y^4*u-10267*x^3*z*u-F(9)/94*x^2*y*z*u+2302*x*y^2*z*u-F(19)/36*y^3*z*u+F(78)/115*x^2*z^2*u-7600*x*y*z^2*u+10133*y^2*z^2*u+F(5)/113*x*z^3*u+F(24)/109*y*z^3*u-F(61)/98*x^3*u^2-F(115)/49*x^2*y*u^2+73*x*y^2*u^2+F(72)/83*y^3*u^2+12390*x^2*z*u^2+F(25)/89*x*y*z*u^2+9934*y^2*z*u^2-1225*x*z^2*u^2-F(28)/9*y*z^2*u^2-8037*x^2*u^3-9613*x*y*u^3-F(25)/28*y^2*u^3-F(37)/117*x*z*u^3-F(83)/67*y*z*u^3-6599*x*u^4-11180*y*u^4-F(61)/126*x^4*v-F(24)/7*x^3*y*v+15140*x^2*y^2*v+F(51)/62*x*y^3*v+F(20)/97*y^4*v+910*x^3*z*v-F(69)/31*x^2*y*z*v+9398*x*y^2*z*v+F(104)/49*y^3*z*v+9398*x^2*z^2*v+F(123)/77*x*y*z^2*v-700*y^2*z^2*v-F(96)/41*x*z^3*v-F(102)/73*y*z^3*v+6439*x^3*u*v+F(101)/57*x^2*y*u*v+F(87)/50*x*y^2*u*v+12504*y^3*u*v+F(90)/53*x^2*z*u*v+15438*x*y*z*u*v-F(17)/52*y^2*z*u*v+F(23)/71*x*z^2*u*v-134*y*z^2*u*v+F(67)/109*x^2*u^2*v-F(26)/3*x*y*u^2*v+F(103)/74*y^2*u^2*v-F(87)/31*x*z*u^2*v+F(107)/104*y*z*u^2*v-11613*x*u^3*v-6496*y*u^3*v-F(113)/58*x^3*v^2-F(73)/77*x^2*y*v^2-4676*x*y^2*v^2-F(113)/53*y^3*v^2-11206*x^2*z*v^2+F(71)/57*x*y*z*v^2+F(103)/14*y^2*z*v^2+F(21)/104*x*z^2*v^2+8285*y*z^2*v^2+F(88)/107*x^2*u*v^2-5737*x*y*u*v^2-F(51)/7*y^2*u*v^2-11659*x*z*u*v^2+7961*y*z*u*v^2-F(1)/116*x*u^2*v^2-F(9)/76*y*u^2*v^2-F(55)/57*x^2*v^3+9578*x*y*v^3-F(36)/121*y^2*v^3-F(98)/17*x*z*v^3-F(123)/34*y*z*v^3+1080*x*u*v^3-6815*y*u*v^3+F(47)/118*x*v^4+F(83)/124*y*v^4,-F(67)/90*x^4*u+9076*x^3*y*u-3219*x^2*y^2*u-6681*x*y^3*u-F(65)/23*y^4*u+F(94)/39*x^3*z*u+F(58)/99*x^2*y*z*u-934*x*y^2*z*u+1664*y^3*z*u-9359*x^2*z^2*u+F(55)/73*x*y*z^2*u+F(75)/74*y^2*z^2*u-F(120)/31*x*z^3*u+F(1)/2*y*z^3*u-9014*x^3*u^2+F(13)/75*x^2*y*u^2-F(3)/46*x*y^2*u^2-2296*y^3*u^2+F(87)/25*x^2*z*u^2-9252*x*y*z*u^2+F(99)/25*y^2*z*u^2+F(11)/52*x*z^2*u^2-F(77)/51*y*z^2*u^2+2598*z^3*u^2-F(91)/16*x^2*u^3-3175*x*y*u^3-F(19)/5*y^2*u^3-F(1)/22*x*z*u^3-F(86)/3*y*z*u^3+1693*z^2*u^3-F(13)/105*x*u^4+F(10)/91*y*u^4+F(91)/121*z*u^4+F(76)/115*u^5+F(103)/62*x^4*v+13395*x^3*y*v+F(19)/41*x^2*y^2*v-F(59)/67*x*y^3*v-11746*x^3*z*v+11574*x^2*y*z*v+8495*x*y^2*z*v+F(17)/13*y^3*z*v-F(15)/67*x^2*z^2*v-13888*x*y*z^2*v-8432*y^2*z^2*v-15924*x*z^3*v+F(47)/97*y*z^3*v-F(87)/103*x^3*u*v-6114*x^2*y*u*v-12640*x*y^2*u*v+F(61)/42*y^3*u*v+6433*x^2*z*u*v-F(23)/121*x*y*z*u*v-F(31)/48*y^2*z*u*v-2497*x*z^2*u*v-F(71)/13*y*z^2*u*v+F(61)/36*z^3*u*v-13011*x^2*u^2*v-8131*x*y*u^2*v-F(107)/14*y^2*u^2*v-F(121)/70*x*z*u^2*v-F(107)/58*y*z*u^2*v+F(18)/113*z^2*u^2*v+F(82)/25*x*u^3*v+F(105)/124*y*u^3*v+5679*z*u^3*v-F(43)/66*u^4*v+3224*x^3*v^2+F(7)/60*x^2*y*v^2-1972*x*y^2*v^2+F(20)/3*x^2*z*v^2-F(22)/73*x*y*z*v^2-10245*y^2*z*v^2-13638*x*z^2*v^2+F(99)/35*y*z^2*v^2-F(117)/20*z^3*v^2+F(97)/48*x^2*u*v^2+F(83)/69*x*y*u*v^2+F(81)/119*y^2*u*v^2+F(88)/59*x*z*u*v^2-F(121)/51*y*z*u*v^2-F(94)/65*z^2*u*v^2+15213*x*u^2*v^2-F(59)/107*y*u^2*v^2+12379*z*u^2*v^2+1383*u^3*v^2+F(83)/88*x^2*v^3+F(101)/11*x*y*v^3-7556*x*z*v^3+F(46)/45*y*z*v^3-4278*z^2*v^3+F(120)/23*x*u*v^3+F(72)/103*y*u*v^3-F(25)/26*z*u*v^3-14830*u^2*v^3-F(62)/7*x*v^4-F(81)/65*z*v^4+2307*u*v^4,x^4*u+6365*x^3*y*u-6100*x^2*y^2*u+4405*x*y^3*u+F(121)/119*y^4*u-161*x^3*z*u+F(118)/109*x^2*y*z*u-F(97)/49*x*y^2*z*u+1841*y^3*z*u-F(87)/110*x^2*z^2*u+F(5)/103*x*y*z^2*u+13094*y^2*z^2*u-F(111)/86*x*z^3*u+F(113)/122*y*z^3*u+7084*x^3*u^2-13010*x^2*y*u^2-F(25)/42*x*y^2*u^2+595*y^3*u^2-14807*x^2*z*u^2-4111*x*y*z*u^2+F(121)/120*y^2*z*u^2-13349*x*z^2*u^2-F(69)/125*y*z^2*u^2+F(11)/14*z^3*u^2-F(111)/124*x^2*u^3+F(89)/124*x*y*u^3-F(31)/30*y^2*u^3+F(8)/103*x*z*u^3+F(91)/67*y*z*u^3-9618*z^2*u^3+F(23)/117*x*u^4+F(68)/93*y*u^4-F(112)/17*z*u^4-F(67)/25*u^5-F(103)/62*x^3*y*v-13395*x^2*y^2*v-F(19)/41*x*y^3*v+F(59)/67*y^4*v-F(76)/71*x^3*z*v+F(27)/56*x^2*y*z*v+2490*x*y^2*z*v+F(3)/55*y^3*z*v+8407*x^2*z^2*v+7346*x*y*z^2*v+7729*y^2*z^2*v-F(22)/113*x*z^3*v+F(35)/38*y*z^3*v+F(95)/61*x^3*u*v-4865*x^2*y*u*v-4632*x*y^2*u*v+F(113)/19*y^3*u*v+3436*x^2*z*u*v-F(12)/31*x*y*z*u*v-F(125)/121*y^2*z*u*v+11136*x*z^2*u*v-F(23)/95*y*z^2*u*v+F(52)/119*z^3*u*v-F(54)/31*x^2*u^2*v-F(25)/21*x*y*u^2*v-F(7)/97*y^2*u^2*v+F(14)/33*x*z*u^2*v-5654*y*z*u^2*v+F(90)/77*z^2*u^2*v-11907*x*u^3*v+14124*y*u^3*v+F(19)/28*z*u^3*v+F(64)/27*u^4*v-3224*x^2*y*v^2-F(7)/60*x*y^2*v^2+1972*y^3*v^2+F(54)/49*x^2*z*v^2-F(31)/123*x*y*z*v^2-F(8)/39*y^2*z*v^2+F(26)/67*x*z^2*v^2+672*y*z^2*v^2-F(68)/91*z^3*v^2-F(14)/33*x^2*u*v^2-F(126)/79*x*y*u*v^2+F(119)/69*y^2*u*v^2-13461*x*z*u*v^2+230*y*z*u*v^2-F(37)/40*z^2*u*v^2-F(25)/23*x*u^2*v^2-12479*y*u^2*v^2+F(41)/73*z*u^2*v^2-F(71)/70*u^3*v^2-F(83)/88*x*y*v^3-F(101)/11*y^2*v^3+F(59)/81*x*z*v^3+F(91)/25*y*z*v^3+F(45)/124*z^2*v^3+F(73)/98*x*u*v^3+F(19)/100*y*u*v^3+F(43)/92*z*u*v^3-F(106)/87*u^2*v^3+F(62)/7*y*v^4+F(62)/33*z*v^4-8769*u*v^4,7457*x^4*u+F(89)/21*x^3*y*u-F(116)/11*x^2*y^2*u-F(77)/111*x*y^3*u-2266*y^4*u+F(98)/103*x^3*z*u+10090*x^2*y*z*u-F(97)/47*x*y^2*z*u+26*y^3*z*u+F(93)/122*x^2*z^2*u+F(3)/22*x*y*z^2*u+F(113)/84*y^2*z^2*u+F(19)/85*x*z^3*u-F(17)/94*y*z^3*u-F(35)/16*x^3*u^2+F(2)/111*x^2*y*u^2+F(49)/53*x*y^2*u^2-F(13)/81*y^3*u^2-9579*x^2*z*u^2+11939*x*y*z*u^2-F(46)/99*y^2*z*u^2-F(4)/113*x*z^2*u^2+F(33)/19*y*z^2*u^2+10737*z^3*u^2-F(49)/37*x^2*u^3+F(112)/101*x*y*u^3+12825*y^2*u^3-F(104)/101*x*z*u^3+F(1)/86*y*z*u^3+12490*z^2*u^3-15874*x*u^4+F(31)/23*y*u^4+788*z*u^4-948*u^5+F(76)/71*x^4*v-F(113)/5*x^3*y*v+F(117)/116*x^2*y^2*v+4883*x*y^3*v-F(17)/13*y^4*v+12014*x^3*z*v+14896*x^2*y*z*v-F(47)/28*x*y^2*z*v+F(41)/71*y^3*z*v+8705*x^2*z^2*v-F(32)/43*x*y*z^2*v-F(125)/11*y^2*z^2*v-3022*x*z^3*v+7869*y*z^3*v+F(49)/124*x^3*u*v+F(65)/77*x^2*y*u*v+42*x*y^2*u*v+F(26)/49*y^3*u*v-F(58)/35*x^2*z*u*v+7279*x*y*z*u*v-F(53)/2*y^2*z*u*v+3174*x*z^2*u*v-F(77)/76*y*z^2*u*v-5897*z^3*u*v+F(76)/67*x^2*u^2*v+F(9)/50*x*y*u^2*v+F(17)/9*y^2*u^2*v+15879*x*z*u^2*v-F(67)/22*y*z*u^2*v-15620*z^2*u^2*v+13410*x*u^3*v-10932*y*u^3*v+12294*z*u^3*v+F(35)/93*u^4*v-F(54)/49*x^3*v^2+F(49)/114*x^2*y*v^2+15260*x*y^2*v^2+10245*y^3*v^2-11133*x^2*z*v^2+F(22)/3*x*y*z*v^2+8136*y^2*z*v^2-F(119)/8*x*z^2*v^2+F(102)/77*y*z^2*v^2-12990*z^3*v^2-F(86)/67*x^2*u*v^2-F(73)/5*x*y*u*v^2+F(82)/115*y^2*u*v^2+F(110)/79*x*z*u*v^2-F(57)/47*y*z*u*v^2+15044*z^2*u*v^2-2529*x*u^2*v^2+2833*y*u^2*v^2+F(47)/38*z*u^2*v^2+7014*u^3*v^2-F(59)/81*x^2*v^3+8832*x*y*v^3-F(46)/45*y^2*v^3+F(1)/26*x*z*v^3+1212*y*z*v^3+F(75)/92*z^2*v^3-F(13)/5*x*u*v^3+14154*y*u*v^3+12960*z*u*v^3-F(96)/37*u^2*v^3-F(62)/33*x*v^4+F(81)/65*y*v^4+F(24)/53*z*v^4-F(42)/23*u*v^4,15143*x^4*u+F(39)/94*x^3*y*u-3215*x^2*y^2*u-F(23)/29*x*y^3*u-F(62)/65*x^3*z*u+14671*x^2*y*z*u+F(102)/7*x*y^2*z*u-F(121)/23*x^2*z^2*u+F(33)/101*x*y*z^2*u-F(29)/123*x^3*u^2-15869*x^2*y*u^2-F(45)/106*x*y^2*u^2-F(4)/11*y^3*u^2-4216*x^2*z*u^2-F(1)/70*x*y*z*u^2-9417*y^2*z*u^2-F(105)/16*x*z^2*u^2-F(61)/62*y*z^2*u^2+15317*x^2*u^3+13185*x*y*u^3+2363*y^2*u^3+3856*x*z*u^3-6010*y*z*u^3+F(86)/115*z^2*u^3+39*x*u^4+F(119)/45*y*u^4+9176*z*u^4-10800*u^5+F(23)/100*x^4*v+F(20)/41*x^3*y*v+F(103)/116*x^2*y^2*v+2922*x*y^3*v+F(90)/23*x^3*z*v+F(77)/114*x^2*y*z*v-F(125)/8*x*y^2*z*v-13181*x^2*z^2*v+10868*x*y*z^2*v+F(11)/39*x^3*u*v-F(37)/53*x^2*y*u*v+15383*x*y^2*u*v+F(2)/13*y^3*u*v+4119*x^2*z*u*v+F(101)/107*x*y*z*u*v+F(113)/13*y^2*z*u*v+F(77)/104*x*z^2*u*v+F(60)/101*y*z^2*u*v-F(122)/29*x^2*u^2*v-F(87)/53*x*y*u^2*v-F(109)/62*y^2*u^2*v+F(92)/47*x*z*u^2*v+F(62)/9*y*z*u^2*v+15850*z^2*u^2*v+8233*x*u^3*v-6525*y*u^3*v-F(5)/99*z*u^3*v-10264*u^4*v-10600*x^3*v^2-F(103)/98*x^2*y*v^2-F(71)/122*x*y^2*v^2-F(17)/13*y^3*v^2-F(79)/53*x^2*z*v^2+F(65)/46*x*y*z*v^2+8432*y^2*z*v^2+F(117)/32*x*z^2*v^2-F(47)/97*y*z^2*v^2+F(15)/26*x^2*u*v^2-F(26)/37*x*y*u*v^2-F(93)/112*y^2*u*v^2-F(109)/43*x*z*u*v^2+4483*y*z*u*v^2+F(47)/87*z^2*u*v^2+3454*x*u^2*v^2-F(65)/62*y*u^2*v^2+71*z*u^2*v^2+F(115)/68*u^3*v^2-F(74)/45*x^2*v^3+2700*x*y*v^3+10245*y^2*v^3-F(19)/26*x*z*v^3-F(99)/35*y*z*v^3+F(117)/20*z^2*v^3-F(35)/107*x*u*v^3-3775*y*u*v^3+1127*z*u*v^3+F(19)/43*u^2*v^3+F(59)/85*x*v^4-F(46)/45*y*v^4+4278*z*v^4-F(71)/23*u*v^4+F(81)/65*v^5,-15143*x^3*y*u-F(39)/94*x^2*y^2*u+3215*x*y^3*u+F(23)/29*y^4*u+F(62)/65*x^2*y*z*u-14671*x*y^2*z*u-F(102)/7*y^3*z*u+F(121)/23*x*y*z^2*u-F(33)/101*y^2*z^2*u-F(17)/122*x^3*u^2+10090*x^2*y*u^2+7014*x*y^2*u^2+F(13)/33*y^3*u^2-F(105)/2*x^2*z*u^2+F(61)/120*x*y*z*u^2-F(77)/62*y^2*z*u^2-F(71)/39*x*z^2*u^2-F(102)/107*y*z^2*u^2-F(32)/123*x^2*u^3+3036*x*y*u^3+4968*y^2*u^3-707*x*z*u^3+14459*y*z*u^3+3579*z^2*u^3-10722*x*u^4-F(101)/114*y*u^4+F(89)/3*z*u^4-14067*u^5-F(23)/100*x^3*y*v-F(20)/41*x^2*y^2*v-F(103)/116*x*y^3*v-2922*y^4*v-F(90)/23*x^2*y*z*v-F(77)/114*x*y^2*z*v+F(125)/8*y^3*z*v+13181*x*y*z^2*v-10868*y^2*z^2*v-F(94)/79*x^3*u*v-F(23)/88*x^2*y*u*v+6237*x*y^2*u*v-F(53)/97*y^3*u*v-9755*x^2*z*u*v-2443*x*y*z*u*v-26*y^2*z*u*v+F(17)/66*x*z^2*u*v+3182*y*z^2*u*v-13576*x^2*u^2*v+F(106)/99*x*y*u^2*v+6487*y^2*u^2*v+6644*x*z*u^2*v+F(75)/124*y*z*u^2*v-12733*z^2*u^2*v+2301*x*u^3*v-F(73)/102*y*u^3*v+1929*z*u^3*v+F(17)/30*u^4*v+F(76)/71*x^3*v^2-2219*x^2*y*v^2+F(53)/23*x*y^2*v^2+12488*y^3*v^2-8407*x^2*z*v^2+F(88)/15*x*y*z*v^2-8447*y^2*z*v^2+F(22)/113*x*z^2*v^2-2970*y*z^2*v^2+F(99)/31*x^2*u*v^2-F(41)/20*x*y*u*v^2+F(95)/39*y^2*u*v^2-F(91)/38*x*z*u*v^2-F(112)/103*y*z*u*v^2-2302*z^2*u*v^2+4991*x*u^2*v^2+14238*y*u^2*v^2+3674*z*u^2*v^2-F(44)/115*u^3*v^2-F(54)/49*x^2*v^3+F(29)/23*x*y*v^3+12560*y^2*v^3-F(26)/67*x*z*v^3-15333*y*z*v^3+F(68)/91*z^2*v^3+11980*x*u*v^3+F(29)/120*y*u*v^3-5540*z*u*v^3-F(51)/44*u^2*v^3-F(59)/81*x*v^4-F(107)/79*y*v^4-F(45)/124*z*v^4-9770*u*v^4-F(62)/33*v^5,-15143*x^3*z*u-F(39)/94*x^2*y*z*u+3215*x*y^2*z*u+F(23)/29*y^3*z*u+F(62)/65*x^2*z^2*u-14671*x*y*z^2*u-F(102)/7*y^2*z^2*u+F(121)/23*x*z^3*u-F(33)/101*y*z^3*u-F(44)/13*x^3*u^2-F(27)/79*x^2*y*u^2+F(83)/46*x*y^2*u^2+F(109)/115*y^3*u^2-F(14)/103*x^2*z*u^2+F(29)/119*x*y*z*u^2-5917*y^2*z*u^2-F(71)/32*x*z^2*u^2+6147*y*z^2*u^2-2576*z^3*u^2-F(75)/97*x^2*u^3-F(13)/46*x*y*u^3+F(104)/105*y^2*u^3-F(15)/34*x*z*u^3+F(3)/89*y*z*u^3-2107*z^2*u^3-9175*x*u^4+15213*y*u^4+169*z*u^4-F(89)/9*u^5-F(23)/100*x^3*z*v-F(20)/41*x^2*y*z*v-F(103)/116*x*y^2*z*v-2922*y^3*z*v-F(90)/23*x^2*z^2*v-F(77)/114*x*y*z^2*v+F(125)/8*y^2*z^2*v+13181*x*z^3*v-10868*y*z^3*v-8394*x^3*u*v-F(35)/72*x^2*y*u*v+11038*x*y^2*u*v+F(88)/15*y^3*u*v+11612*x^2*z*u*v-6427*x*y*z*u*v-6727*y^2*z*u*v-F(28)/113*x*z^2*u*v+12051*y*z^2*u*v-3326*z^3*u*v+F(75)/32*x^2*u^2*v-12032*x*y*u^2*v-F(104)/11*y^2*u^2*v+F(99)/29*x*z*u^2*v+12596*y*z*u^2*v+234*z^2*u^2*v+8077*x*u^3*v-F(31)/21*y*u^3*v+10985*z*u^3*v-8180*u^4*v-F(103)/62*x^3*v^2-13395*x^2*y*v^2-F(19)/41*x*y^2*v^2+F(59)/67*y^3*v^2-9645*x^2*z*v^2-8635*x*y*z*v^2-F(49)/36*y^2*z*v^2+F(87)/56*x*z^2*v^2+F(80)/89*y*z^2*v^2+2924*z^3*v^2-12422*x^2*u*v^2+F(16)/25*x*y*u*v^2+F(57)/53*y^2*u*v^2-F(38)/39*x*z*u*v^2-F(64)/107*y*z*u*v^2-F(71)/106*z^2*u*v^2-7456*x*u^2*v^2-9688*y*u^2*v^2-11188*z*u^2*v^2+10354*u^3*v^2-3224*x^2*v^3-F(7)/60*x*y*v^3+1972*y^2*v^3+7815*x*z*v^3-1385*y*z*v^3+F(37)/24*z^2*v^3+F(126)/11*x*u*v^3+F(39)/2*y*u*v^3-93*z*u*v^3-F(46)/111*u^2*v^3-F(83)/88*x*v^4-F(101)/11*y*v^4+F(79)/70*z*v^4+F(89)/35*u*v^4+F(62)/7*v^5,-F(44)/13*x^4*u-F(27)/79*x^3*y*u+F(83)/46*x^2*y^2*u+F(109)/115*x*y^3*u+F(2)/15*x^3*z*u+F(85)/59*x^2*y*z*u+F(87)/16*x*y^2*z*u-F(4)/11*y^3*z*u-F(97)/5*x^2*z^2*u+6604*x*y*z^2*u-9417*y^2*z^2*u-F(83)/7*x*z^3*u-F(61)/62*y*z^3*u-F(75)/97*x^3*u^2-F(13)/46*x^2*y*u^2+F(104)/105*x*y^2*u^2+10612*x^2*z*u^2+2761*x*y*z*u^2+2363*y^2*z*u^2+1749*x*z^2*u^2-6010*y*z^2*u^2+F(86)/115*z^3*u^2-9175*x^2*u^3+15213*x*y*u^3+208*x*z*u^3+F(119)/45*y*z*u^3+9176*z^2*u^3-F(89)/9*x*u^4-10800*z*u^4+F(19)/68*x^4*v-F(109)/84*x^3*y*v-F(46)/45*x^2*y^2*v-F(54)/31*x*y^3*v-F(65)/23*y^4*v-F(120)/43*x^3*z*v+F(97)/64*x^2*y*z*v+F(1)/29*x*y^2*z*v-F(89)/31*y^3*z*v+6084*x^2*z^2*v-F(4)/103*x*y*z^2*v+F(7)/11*y^2*z^2*v+F(62)/59*x*z^3*v-F(61)/89*y*z^3*v+9983*x^3*u*v-9046*x^2*y*u*v-14867*x*y^2*u*v-2296*y^3*u*v+5386*x^2*z*u*v+F(101)/125*x*y*z*u*v-10173*y^2*z*u*v+F(37)/43*x*z^2*u*v+F(76)/53*y*z^2*u*v-13543*z^3*u*v-F(53)/108*x^2*u^2*v-9270*x*y*u^2*v-F(19)/5*y^2*u^2*v-F(89)/46*x*z*u^2*v+F(116)/109*y*z*u^2*v-9617*z^2*u^2*v-14883*x*u^3*v+F(10)/91*y*u^3*v+6922*z*u^3*v+F(76)/115*u^4*v+F(87)/103*x^3*v^2-7393*x^2*y*v^2+7280*x*y^2*v^2+F(61)/42*y^3*v^2-F(79)/105*x^2*z*v^2-10948*x*y*z*v^2-F(31)/21*y^2*z*v^2+F(78)/125*x*z^2*v^2+F(36)/11*y*z^2*v^2-F(15)/7*z^3*v^2+11524*x^2*u*v^2-F(97)/79*x*y*u*v^2-F(107)/14*y^2*u*v^2+15572*x*z*u*v^2-F(11)/53*y*z*u*v^2+F(88)/45*z^2*u*v^2+7798*x*u^2*v^2+F(105)/124*y*u^2*v^2-13608*z*u^2*v^2-F(43)/66*u^3*v^2-F(107)/64*x^2*v^3+8598*x*y*v^3+F(81)/119*y^2*v^3+F(83)/103*x*z*v^3-F(59)/50*y*z*v^3-F(31)/66*z^2*v^3+F(95)/77*x*u*v^3-F(59)/107*y*u*v^3-F(7)/111*z*u*v^3+1383*u^2*v^3+2114*x*v^4+F(72)/103*y*v^4+7218*z*v^4-14830*u*v^4+2307*v^5,F(44)/13*x^3*y*u+F(27)/79*x^2*y^2*u-F(83)/46*x*y^3*u-F(109)/115*y^4*u-F(17)/122*x^3*z*u+F(31)/68*x^2*y*z*u-7772*x*y^2*z*u+F(49)/87*y^3*z*u-F(105)/2*x^2*z^2*u-F(89)/114*x*y*z^2*u-13888*y^2*z^2*u-F(71)/39*x*z^3*u-12673*y*z^3*u+F(75)/97*x^2*y*u^2+F(13)/46*x*y^2*u^2-F(104)/105*y^3*u^2-F(32)/123*x^2*z*u^2+F(77)/62*x*y*z*u^2+F(11)/106*y^2*z*u^2-707*x*z^2*u^2-F(43)/56*y*z^2*u^2+3579*z^3*u^2+9175*x*y*u^3-15213*y^2*u^3-10722*x*z*u^3+F(20)/61*y*z*u^3+F(89)/3*z^2*u^3+F(89)/9*y*u^4-14067*z*u^4+x^4*v-F(79)/13*x^3*y*v-1212*x^2*y^2*v-F(59)/82*x*y^3*v-11708*y^4*v+6317*x^3*z*v-8873*x^2*y*z*v-F(33)/43*x*y^2*z*v+4280*y^3*z*v+F(57)/29*x^2*z^2*v-F(2)/117*x*y*z^2*v+F(89)/63*y^2*z^2*v-15602*x*z^3*v-14731*y*z^3*v+7084*x^3*u*v-16*x^2*y*u*v+F(113)/58*x*y^2*u*v+F(114)/5*y^3*u*v+3608*x^2*z*u*v+15030*x*y*z*u*v+F(10)/23*y^2*z*u*v-6705*x*z^2*u*v-F(29)/23*y*z^2*u*v-F(54)/49*z^3*u*v-F(111)/124*x^2*u^2*v+14885*x*y*u^2*v+F(31)/70*y^2*u^2*v+F(5)/71*x*z*u^2*v-F(11)/125*y*z*u^2*v+F(119)/104*z^2*u^2*v+F(23)/117*x*u^3*v-419*y*u^3*v+7584*z*u^3*v-F(67)/25*u^4*v+F(95)/61*x^3*v^2+7557*x^2*y*v^2+F(111)/124*x*y^2*v^2+F(61)/6*y^3*v^2-F(65)/93*x^2*z*v^2+F(71)/60*x*y*z*v^2-7826*y^2*z*v^2+8608*x*z^2*v^2+10342*y*z^2*v^2-F(61)/20*z^3*v^2-F(54)/31*x^2*u*v^2+F(96)/37*x*y*u*v^2-536*y^2*u*v^2+4022*x*z*u*v^2-12219*y*z*u*v^2-F(11)/91*z^2*u*v^2-11907*x*u^2*v^2+F(108)/17*y*u^2*v^2+964*z*u^2*v^2+F(64)/27*u^3*v^2-F(14)/33*x^2*v^3-2590*x*y*v^3-F(100)/123*y^2*v^3+F(7)/108*x*z*v^3-F(70)/27*y*z*v^3+F(115)/2*z^2*v^3-F(25)/23*x*u*v^3+F(125)/49*y*u*v^3+7539*z*u*v^3-F(71)/70*u^2*v^3+F(73)/98*x*v^4-F(79)/53*y*v^4-14290*z*v^4-F(106)/87*u*v^4-8769*v^5,F(17)/122*x^4*u-F(110)/27*x^3*y*u+F(39)/112*x^2*y^2*u+3832*x*y^3*u+F(4)/11*y^4*u-4251*x^3*z*u-F(56)/109*x^2*y*z*u+F(67)/2*x*y^2*z*u+F(70)/9*y^3*z*u-7964*x^2*z^2*u+3578*x*y*z^2*u-2944*y^2*z^2*u-F(53)/89*x*z^3*u-11441*y*z^3*u+F(32)/123*x^3*u^2+13638*x^2*y*u^2+13838*x*y^2*u^2-2363*y^3*u^2-14862*x^2*z*u^2+10082*x*y*z*u^2+F(45)/71*y^2*z*u^2-F(111)/32*x*z^2*u^2+3391*y*z^2*u^2+F(37)/11*z^3*u^2+10722*x^2*u^3+1365*x*y*u^3-F(119)/45*y^2*u^3-5534*x*z*u^3+F(118)/75*y*z*u^3-F(17)/80*z^2*u^3+14067*x*u^4+10800*y*u^4-9769*z*u^4-F(31)/98*x^4*v+F(89)/5*x^3*y*v-F(67)/125*x^2*y^2*v+F(21)/74*x*y^3*v-F(93)/88*y^4*v-8220*x^3*z*v+F(67)/103*x^2*y*z*v+F(79)/112*x*y^2*z*v+1578*y^3*z*v-F(29)/60*x^2*z^2*v+F(97)/34*x*y*z^2*v+2496*y^2*z^2*v-F(24)/125*x*z^3*v-F(75)/61*y*z^3*v-F(25)/19*x^3*u*v+F(32)/123*x^2*y*u*v-1052*x*y^2*u*v-F(36)/125*y^3*u*v+10366*x^2*z*u*v-F(82)/95*x*y*z*u*v+10149*y^2*z*u*v-15362*x*z^2*u*v-F(26)/51*y*z^2*u*v-F(51)/109*z^3*u*v+F(10)/3*x^2*u^2*v-10172*x*y*u^2*v+F(75)/124*y^2*u^2*v-F(81)/95*x*z*u^2*v+3176*y*z*u^2*v-101*z^2*u^2*v-8410*x*u^3*v+15829*y*u^3*v+14100*z*u^3*v-948*u^4*v+6447*x^3*v^2-F(93)/4*x^2*y*v^2+F(38)/7*x*y^2*v^2-F(34)/5*y^3*v^2-F(17)/7*x^2*z*v^2-13229*x*y*z*v^2+4786*y^2*z*v^2+5719*x*z^2*v^2+F(73)/75*y*z^2*v^2-F(53)/26*z^3*v^2+F(116)/15*x^2*u*v^2+14939*x*y*u*v^2+8660*y^2*u*v^2-F(105)/43*x*z*u*v^2-11435*y*z*u*v^2-F(48)/43*z^2*u*v^2-F(18)/19*x*u^2*v^2+8355*y*u^2*v^2+F(71)/26*z*u^2*v^2+F(35)/93*u^3*v^2-F(4)/97*x^2*v^3+13482*x*y*v^3+F(59)/51*y^2*v^3+F(82)/121*x*z*v^3-F(41)/111*y*z*v^3+11436*z^2*v^3-F(111)/101*x*u*v^3-11303*y*u*v^3+F(59)/2*z*u*v^3+7014*u^2*v^3-3029*x*v^4+F(104)/107*y*v^4-8383*z*v^4-F(96)/37*u*v^4-F(42)/23*v^5]))
end


@doc raw"""
    elliptic_d7_pi6()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `7` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d7_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([4971*x*y^3+F(3)/101*y^4-12318*x*y^2*z-12835*y^3*z+F(97)/98*x*y*z^2+63*y^2*z^2-8056*x*z^3+F(23)/91*y*z^3-9662*z^4-7398*x*y^2*u+F(69)/71*y^3*u-F(53)/68*x*y*z*u-F(49)/67*y^2*z*u-F(113)/122*x*z^2*u-F(9)/61*y*z^2*u+F(71)/88*z^3*u+11358*x*y*u^2-F(38)/29*y^2*u^2-10232*x*z*u^2+14490*y*z*u^2+2274*z^2*u^2+3501*x*u^3+10427*y*u^3-F(109)/38*z*u^3-F(99)/5*u^4-6605*x*y^2*v-1555*y^3*v-648*x*y*z*v-2083*y^2*z*v-F(61)/41*x*z^2*v+F(75)/17*y*z^2*v-F(69)/55*z^3*v-6104*x*y*u*v-9582*y^2*u*v+F(69)/2*x*z*u*v-12551*y*z*u*v+F(47)/49*z^2*u*v-F(118)/13*x*u^2*v+F(34)/105*y*u^2*v+F(105)/41*z*u^2*v+6533*u^3*v+F(122)/25*x*y*v^2+F(2)/43*y^2*v^2+F(16)/61*x*z*v^2+11524*y*z*v^2+F(113)/99*z^2*v^2-F(71)/26*x*u*v^2+7809*y*u*v^2-4865*z*u*v^2-2122*u^2*v^2+F(53)/118*x*v^3-13209*y*v^3-11106*z*v^3-F(49)/79*u*v^3+3006*v^4,x*y^3+15492*y^4-13742*x*y^2*z+F(112)/117*y^3*z+F(6)/47*x*y*z^2+F(28)/41*y^2*z^2+F(71)/111*x*z^3+F(49)/57*y*z^3-F(61)/44*z^4-11759*x*y^2*u+4242*y^3*u-F(109)/18*x*y*z*u+2260*y^2*z*u-6873*x*z^2*u-F(41)/112*y*z^2*u+12574*z^3*u-10939*x*y*u^2+F(119)/38*y^2*u^2-F(62)/33*x*z*u^2-3699*y*z*u^2+2651*z^2*u^2-13194*x*u^3-15185*y*u^3-F(11)/116*z*u^3-F(61)/83*u^4-10094*x*y^2*v+F(13)/4*y^3*v-F(74)/73*x*y*z*v+F(43)/20*y^2*z*v-11547*x*z^2*v+F(53)/43*y*z^2*v-F(92)/93*z^3*v+F(32)/41*x*y*u*v+F(118)/33*y^2*u*v-F(121)/39*x*z*u*v-15913*y*z*u*v+F(53)/11*z^2*u*v+F(97)/76*x*u^2*v+F(85)/29*y*u^2*v-5183*z*u^2*v+8520*u^3*v+F(121)/28*x*y*v^2+F(64)/51*y^2*v^2-15810*x*z*v^2+F(1)/43*y*z*v^2-6160*z^2*v^2+13988*x*u*v^2+F(9)/40*y*u*v^2+F(123)/4*z*u*v^2+15024*u^2*v^2+F(73)/95*x*v^3+F(80)/97*y*v^3+F(57)/25*z*v^3-F(109)/81*u*v^3-F(121)/87*v^4,-4971*x^2+14389*x*y+1607*y^2+F(59)/119*x*z+12020*y*z+F(103)/122*z^2+8894*x*u+7091*y*u+F(54)/19*z*u-F(50)/77*u^2+F(28)/25*x*v-F(113)/56*y*v+F(68)/29*z*v-14620*u*v+F(79)/107*v^2]))
end


@doc raw"""
    elliptic_d8_pi7()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `8` and sectional genus `7`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d8_pi7()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^2*y^2-F(47)/69*x*y^3+6059*y^4+F(78)/85*x^2*y*z+F(55)/124*x*y^2*z+13641*y^3*z+F(8)/17*x^2*z^2+7817*x*y*z^2-2746*y^2*z^2+F(85)/124*x*z^3+87*y*z^3+13182*z^4+F(109)/93*x^2*y*u-F(69)/17*x*y^2*u+12089*y^3*u+8769*x^2*z*u-F(53)/36*x*y*z*u-14834*y^2*z*u+F(123)/23*x*z^2*u+F(103)/77*y*z^2*u-2344*z^3*u-F(43)/104*x^2*u^2-6198*x*y*u^2+F(47)/115*y^2*u^2-F(39)/19*x*z*u^2-F(29)/24*y*z*u^2+F(51)/89*z^2*u^2-F(65)/37*x*u^3-F(95)/94*y*u^3+11302*z*u^3-F(53)/57*u^4-2874*x^2*y*v+4347*x*y^2*v-F(25)/77*y^3*v+13819*x^2*z*v+F(29)/34*x*y*z*v+474*y^2*z*v+F(33)/107*x*z^2*v-3517*y*z^2*v+10617*z^3*v+1834*x^2*u*v+F(54)/113*x*y*u*v-8751*y^2*u*v+F(111)/70*x*z*u*v-F(66)/61*y*z*u*v+9195*z^2*u*v-14289*x*u^2*v-F(13)/110*y*u^2*v+F(103)/9*z*u^2*v+5113*u^3*v+F(116)/89*x^2*v^2+15142*x*y*v^2+13078*y^2*v^2-F(38)/41*x*z*v^2-F(13)/113*y*z*v^2-12824*z^2*v^2-F(57)/11*x*u*v^2-F(114)/17*y*u*v^2-F(125)/31*z*u*v^2+11939*u^2*v^2+F(44)/13*x*v^3+F(56)/69*y*v^3+F(12)/125*z*v^3+643*u*v^3+3530*v^4,-3454*x^2*y-1285*x*y^2-6182*y^3-F(8)/69*x^2*z+F(9)/19*x*y*z+F(64)/49*y^2*z+F(98)/67*x*z^2-13809*y*z^2+F(21)/44*z^3+F(77)/47*x^2*u+748*x*y*u-F(41)/77*y^2*u+7318*x*z*u+4217*y*z*u+12562*z^2*u-F(98)/69*x*u^2-F(14)/85*y*u^2+F(119)/46*z*u^2-F(61)/121*u^3+5582*x^2*v+F(108)/77*x*y*v-F(93)/4*y^2*v-F(65)/49*x*z*v-4135*y*z*v+2477*z^2*v+11114*x*u*v+F(85)/14*y*u*v+F(51)/125*z*u*v-7572*u^2*v-F(115)/52*x*v^2-7647*y*v^2+4647*z*v^2-5684*u*v^2-F(1)/55*v^3,3454*x^3-6645*x^2*y-F(43)/34*x*y^2+14590*y^3+F(8)/11*x^2*z-F(117)/112*x*y*z+F(109)/54*y^2*z+6566*x*z^2+F(23)/57*y*z^2-13078*z^3+F(95)/61*x^2*u+F(67)/40*x*y*u-4544*y^2*u-F(95)/72*x*z*u-F(8)/103*y*z*u+F(100)/77*z^2*u+F(23)/63*x*u^2+F(69)/61*y*u^2-F(94)/105*z*u^2+8619*u^3+F(68)/123*x^2*v+F(8)/117*x*y*v+F(101)/77*y^2*v+F(124)/125*x*z*v+F(17)/84*y*z*v+F(23)/67*z^2*v+F(18)/59*x*u*v+3216*y*u*v-F(77)/59*z*u*v-F(9)/50*u^2*v+F(96)/109*x*v^2-2491*y*v^2+14089*z*v^2+14067*u*v^2-F(56)/113*v^3]))
end


@doc raw"""
    elliptic_d9_pi7()

Return a smooth elliptic surface in $\mathbb P^4$ with degree `9` and sectional genus `7`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function elliptic_d9_pi7()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(48)/67*x^4*y-F(86)/109*x^3*y^2-9824*x^2*y^3-F(113)/77*x*y^4+F(98)/79*x^4*z+F(34)/111*x^3*y*z-F(44)/51*x^2*y^2*z-F(10)/101*x*y^3*z+F(97)/89*x^3*z^2-692*x^2*y*z^2+8389*x*y^2*z^2-F(81)/85*x^2*z^3+15885*x*y*z^3+F(59)/30*x*z^4+F(60)/59*x^4*u+F(101)/8*x^3*y*u-F(85)/83*x^2*y^2*u-4329*x*y^3*u-2311*x^3*z*u+F(99)/19*x^2*y*z*u-7476*x*y^2*z*u+F(77)/43*x^2*z^2*u+15926*x*y*z^2*u-F(121)/96*x*z^3*u-346*x^3*u^2+14335*x^2*y*u^2+10491*x*y^2*u^2+F(97)/114*x^2*z*u^2+F(25)/74*x*y*z*u^2-F(89)/5*x*z^2*u^2+F(73)/83*x^2*u^3+15145*x*y*u^3+13747*x*z*u^3+6427*x*u^4-F(112)/87*x^4*v+7959*x^3*y*v+14416*x^2*y^2*v+F(103)/112*x*y^3*v+569*y^4*v+10328*x^3*z*v-13311*x^2*y*z*v-F(64)/5*x*y^2*z*v+7637*y^3*z*v-13741*x^2*z^2*v-F(18)/17*x*y*z^2*v-F(121)/61*y^2*z^2*v-10339*x*z^3*v-1703*y*z^3*v-12316*z^4*v+F(99)/106*x^3*u*v+F(32)/39*x^2*y*u*v+F(123)/61*x*y^2*u*v+F(66)/13*y^3*u*v-F(53)/17*x^2*z*u*v-9865*x*y*z*u*v-4372*y^2*z*u*v-14523*x*z^2*u*v+F(125)/51*y*z^2*u*v-7890*z^3*u*v+F(111)/85*x^2*u^2*v-F(91)/102*x*y*u^2*v+5038*y^2*u^2*v+10948*x*z*u^2*v+F(56)/79*y*z*u^2*v-14124*z^2*u^2*v-7452*x*u^3*v+F(74)/65*y*u^3*v+12643*z*u^3*v-F(62)/59*u^4*v+F(83)/35*x^3*v^2+9418*x^2*y*v^2+15761*x*y^2*v^2+11448*y^3*v^2+F(89)/59*x^2*z*v^2-F(123)/124*x*y*z*v^2+F(29)/30*y^2*z*v^2-F(23)/14*x*z^2*v^2-6215*y*z^2*v^2-F(20)/111*z^3*v^2+F(51)/95*x^2*u*v^2+5511*x*y*u*v^2-F(33)/122*y^2*u*v^2-5700*x*z*u*v^2+F(8)/11*y*z*u*v^2-6841*z^2*u*v^2+F(1)/95*x*u^2*v^2+F(63)/32*y*u^2*v^2-F(88)/37*z*u^2*v^2-3971*u^3*v^2+F(2)/65*x^2*v^3+14128*x*y*v^3-1408*y^2*v^3+F(123)/77*x*z*v^3-14264*y*z*v^3-F(116)/21*z^2*v^3+F(104)/29*x*u*v^3+F(73)/54*y*u*v^3-1700*z*u*v^3-F(13)/60*u^2*v^3-2980*x*v^4-F(5)/77*y*v^4+F(64)/91*z*v^4-8778*u*v^4-F(29)/120*v^5,-F(48)/67*x^3*y^2+F(86)/109*x^2*y^3+9824*x*y^4+F(113)/77*y^5-F(98)/79*x^3*y*z-F(34)/111*x^2*y^2*z+F(44)/51*x*y^3*z+F(10)/101*y^4*z-F(97)/89*x^2*y*z^2+692*x*y^2*z^2-8389*y^3*z^2+F(81)/85*x*y*z^3-15885*y^2*z^3-F(59)/30*y*z^4-F(60)/59*x^3*y*u-F(101)/8*x^2*y^2*u+F(85)/83*x*y^3*u+4329*y^4*u+2311*x^2*y*z*u-F(99)/19*x*y^2*z*u+7476*y^3*z*u-F(77)/43*x*y*z^2*u-15926*y^2*z^2*u+F(121)/96*y*z^3*u+346*x^2*y*u^2-14335*x*y^2*u^2-10491*y^3*u^2-F(97)/114*x*y*z*u^2-F(25)/74*y^2*z*u^2+F(89)/5*y*z^2*u^2-F(73)/83*x*y*u^3-15145*y^2*u^3-13747*y*z*u^3-6427*y*u^4-F(27)/82*x^3*y*v-4600*x^2*y^2*v-F(124)/101*x*y^3*v-1624*y^4*v-F(52)/89*x^3*z*v+F(25)/87*x^2*y*z*v+5172*x*y^2*z*v-F(1)/42*y^3*z*v-14889*x^2*z^2*v-11175*x*y*z^2*v-13985*y^2*z^2*v+F(7)/37*x*z^3*v+F(85)/82*y*z^3*v+8247*z^4*v-9745*x^3*u*v+F(63)/62*x^2*y*u*v+7647*x*y^2*u*v-10467*y^3*u*v-11245*x^2*z*u*v-9885*x*y*z*u*v-13493*y^2*z*u*v+12527*x*z^2*u*v-F(44)/5*y*z^2*u*v+8874*z^3*u*v+11027*x^2*u^2*v-12822*x*y*u^2*v-F(16)/115*y^2*u^2*v-F(92)/73*x*z*u^2*v+15178*y*z*u^2*v+6083*z^2*u^2*v-F(5)/102*x*u^3*v-4906*y*u^3*v+14346*z*u^3*v-12757*u^4*v+1823*x^3*v^2+10794*x^2*y*v^2-F(106)/81*x*y^2*v^2+F(81)/89*y^3*v^2+F(19)/63*x^2*z*v^2-F(109)/59*x*y*z*v^2+F(107)/105*y^2*z*v^2+10333*x*z^2*v^2-747*y*z^2*v^2-9473*z^3*v^2+F(121)/31*x^2*u*v^2-11724*x*y*u*v^2+F(107)/32*y^2*u*v^2-F(64)/125*x*z*u*v^2+F(114)/49*y*z*u*v^2+F(89)/8*z^2*u*v^2-F(37)/61*x*u^2*v^2-11442*y*u^2*v^2+11899*z*u^2*v^2+7808*u^3*v^2+171*x^2*v^3-F(22)/113*x*y*v^3-9861*y^2*v^3-1492*x*z*v^3-11522*y*z*v^3+F(52)/31*z^2*v^3+F(86)/121*x*u*v^3+F(113)/17*y*u*v^3-F(37)/109*z*u*v^3+F(21)/11*u^2*v^3+10112*x*v^4+3896*y*v^4-9941*z*v^4-F(104)/85*u*v^4+F(38)/3*v^5,-F(48)/67*x^3*y*z+F(86)/109*x^2*y^2*z+9824*x*y^3*z+F(113)/77*y^4*z-F(98)/79*x^3*z^2-F(34)/111*x^2*y*z^2+F(44)/51*x*y^2*z^2+F(10)/101*y^3*z^2-F(97)/89*x^2*z^3+692*x*y*z^3-8389*y^2*z^3+F(81)/85*x*z^4-15885*y*z^4-F(59)/30*z^5-F(60)/59*x^3*z*u-F(101)/8*x^2*y*z*u+F(85)/83*x*y^2*z*u+4329*y^3*z*u+2311*x^2*z^2*u-F(99)/19*x*y*z^2*u+7476*y^2*z^2*u-F(77)/43*x*z^3*u-15926*y*z^3*u+F(121)/96*z^4*u+346*x^2*z*u^2-14335*x*y*z*u^2-10491*y^2*z*u^2-F(97)/114*x*z^2*u^2-F(25)/74*y*z^2*u^2+F(89)/5*z^3*u^2-F(73)/83*x*z*u^3-15145*y*z*u^3-13747*z^2*u^3-6427*z*u^4-F(25)/68*x^3*y*v-9266*x^2*y^2*v+15250*x*y^3*v-11084*y^4*v+10454*x^3*z*v-F(29)/27*x^2*y*z*v+F(90)/59*x*y^2*z*v-F(10)/71*y^3*z*v+F(16)/33*x^2*z^2*v+3215*x*y*z^2*v+F(79)/72*y^2*z^2*v+F(124)/33*x*z^3*v+F(103)/10*y*z^3*v+6993*z^4*v-9037*x^3*u*v-F(47)/15*x^2*y*u*v+15162*x*y^2*u*v+F(9)/92*y^3*u*v-10008*x^2*z*u*v+11462*x*y*z*u*v+F(74)/105*y^2*z*u*v-2853*x*z^2*u*v-F(71)/95*y*z^2*u*v-F(26)/77*z^3*u*v-F(19)/26*x^2*u^2*v-F(87)/113*x*y*u^2*v+F(86)/13*y^2*u^2*v-8548*x*z*u^2*v-F(67)/86*y*z*u^2*v-F(49)/124*z^2*u^2*v-4905*x*u^3*v+403*y*u^3*v+F(8)/67*z*u^3*v+8369*u^4*v-F(122)/49*x^3*v^2+F(119)/106*x^2*y*v^2+6010*x*y^2*v^2+F(10)/31*y^3*v^2+F(101)/88*x^2*z*v^2-F(37)/30*x*y*z*v^2+F(60)/53*y^2*z*v^2+F(22)/103*x*z^2*v^2+F(43)/120*y*z^2*v^2-F(117)/23*z^3*v^2+F(31)/45*x^2*u*v^2+F(87)/34*x*y*u*v^2-6059*y^2*u*v^2+15127*x*z*u*v^2+F(51)/77*y*z*u*v^2+F(122)/31*z^2*u*v^2+13563*x*u^2*v^2-8216*y*u^2*v^2+F(59)/19*z*u^2*v^2-F(43)/14*u^3*v^2-10868*x^2*v^3+4449*x*y*v^3+9038*y^2*v^3-7219*x*z*v^3-11252*y*z*v^3+F(123)/58*z^2*v^3+8968*x*u*v^3-1060*y*u*v^3-F(107)/19*z*u*v^3-F(77)/43*u^2*v^3+F(57)/44*x*v^4-F(11)/89*y*v^4-7514*z*v^4-F(9)/53*u*v^4+F(41)/32*v^5,-F(48)/67*x^3*y*u+F(86)/109*x^2*y^2*u+9824*x*y^3*u+F(113)/77*y^4*u-F(98)/79*x^3*z*u-F(34)/111*x^2*y*z*u+F(44)/51*x*y^2*z*u+F(10)/101*y^3*z*u-F(97)/89*x^2*z^2*u+692*x*y*z^2*u-8389*y^2*z^2*u+F(81)/85*x*z^3*u-15885*y*z^3*u-F(59)/30*z^4*u-F(60)/59*x^3*u^2-F(101)/8*x^2*y*u^2+F(85)/83*x*y^2*u^2+4329*y^3*u^2+2311*x^2*z*u^2-F(99)/19*x*y*z*u^2+7476*y^2*z*u^2-F(77)/43*x*z^2*u^2-15926*y*z^2*u^2+F(121)/96*z^3*u^2+346*x^2*u^3-14335*x*y*u^3-10491*y^2*u^3-F(97)/114*x*z*u^3-F(25)/74*y*z*u^3+F(89)/5*z^2*u^3-F(73)/83*x*u^4-15145*y*u^4-13747*z*u^4-6427*u^5+1765*x^3*y*v+F(101)/82*x^2*y^2*v+1718*x*y^3*v+F(99)/8*y^4*v-817*x^3*z*v-2804*x^2*y*z*v-1244*x*y^2*z*v-9270*y^3*z*v-F(32)/13*x^2*z^2*v-F(68)/117*x*y*z^2*v+F(71)/125*y^2*z^2*v-F(109)/119*x*z^3*v+F(117)/116*y*z^3*v-F(9)/23*z^4*v-8448*x^3*u*v-10010*x^2*y*u*v+F(64)/61*x*y^2*u*v+11826*y^3*u*v+15244*x^2*z*u*v-F(8)/67*x*y*z*u*v-F(82)/101*y^2*z*u*v+1119*x*z^2*u*v-F(31)/87*y*z^2*u*v-12917*z^3*u*v-F(32)/21*x^2*u^2*v-F(44)/21*x*y*u^2*v-F(41)/95*y^2*u^2*v+F(107)/109*x*z*u^2*v+661*y*z*u^2*v-F(74)/53*z^2*u^2*v-7960*x*u^3*v-F(57)/110*y*u^3*v-F(61)/91*z*u^3*v+7825*u^4*v-10874*x^3*v^2+F(57)/97*x^2*y*v^2+F(32)/81*x*y^2*v^2-F(97)/46*y^3*v^2+F(61)/88*x^2*z*v^2-5738*x*y*z*v^2-F(86)/15*y^2*z*v^2-F(86)/27*x*z^2*v^2+F(71)/83*y*z^2*v^2+F(31)/34*z^3*v^2+5181*x^2*u*v^2+13070*x*y*u*v^2-F(49)/38*y^2*u*v^2-F(77)/47*x*z*u*v^2+8589*y*z*u*v^2+F(31)/57*z^2*u*v^2-13543*x*u^2*v^2+F(23)/126*y*u^2*v^2+F(113)/95*z*u^2*v^2-11300*u^3*v^2+F(34)/31*x^2*v^3+1949*x*y*v^3+12902*y^2*v^3+9729*x*z*v^3-10112*y*z*v^3+F(118)/55*z^2*v^3-F(39)/98*x*u*v^3+15439*y*u*v^3+F(31)/99*z*u*v^3-1046*u^2*v^3-F(11)/42*x*v^4+10463*y*v^4-1474*z*v^4-9621*u*v^4+6592*v^5,1765*x^4*y+F(101)/82*x^3*y^2+1718*x^2*y^3+F(99)/8*x*y^4-817*x^4*z-2804*x^3*y*z-1244*x^2*y^2*z-9270*x*y^3*z-F(32)/13*x^3*z^2-F(68)/117*x^2*y*z^2+F(71)/125*x*y^2*z^2-F(109)/119*x^2*z^3+F(117)/116*x*y*z^3-F(9)/23*x*z^4-8817*x^4*u-F(23)/78*x^3*y*u-F(122)/75*x^2*y^2*u+F(67)/40*x*y^3*u+569*y^4*u-F(104)/5*x^3*z*u-5194*x^2*y*z*u+F(33)/86*x*y^2*z*u+7637*y^3*z*u-12622*x^2*z^2*u-F(37)/72*x*y*z^2*u-F(121)/61*y^2*z^2*u+F(112)/11*x*z^3*u-1703*y*z^3*u-12316*z^4*u-F(57)/121*x^3*u^2-F(116)/91*x^2*y*u^2+13477*x*y^2*u^2+F(66)/13*y^3*u^2+8233*x^2*z*u^2-F(81)/73*x*y*z*u^2-4372*y^2*z*u^2-15128*x*z^2*u^2+F(125)/51*y*z^2*u^2-7890*z^3*u^2+12365*x^2*u^3+F(3)/103*x*y*u^3+5038*y^2*u^3+F(122)/31*x*z*u^3+F(56)/79*y*z*u^3-14124*z^2*u^3+F(87)/86*x*u^4+F(74)/65*y*u^4+12643*z*u^4-F(62)/59*u^5-10874*x^4*v+F(57)/97*x^3*y*v+F(32)/81*x^2*y^2*v-F(97)/46*x*y^3*v+F(61)/88*x^3*z*v-5738*x^2*y*z*v-F(86)/15*x*y^2*z*v-F(86)/27*x^2*z^2*v+F(71)/83*x*y*z^2*v+F(31)/34*x*z^3*v-6699*x^3*u*v-F(73)/101*x^2*y*u*v+F(93)/121*x*y^2*u*v+11448*y^3*u*v-10833*x^2*z*u*v-F(121)/47*x*y*z*u*v+F(29)/30*y^2*z*u*v+F(28)/11*x*z^2*u*v-6215*y*z^2*u*v-F(20)/111*z^3*u*v-F(84)/29*x^2*u^2*v-3883*x*y*u^2*v-F(33)/122*y^2*u^2*v+F(103)/69*x*z*u^2*v+F(8)/11*y*z*u^2*v-6841*z^2*u^2*v-F(11)/45*x*u^3*v+F(63)/32*y*u^3*v-F(88)/37*z*u^3*v-3971*u^4*v+F(34)/31*x^3*v^2+1949*x^2*y*v^2+12902*x*y^2*v^2+9729*x^2*z*v^2-10112*x*y*z*v^2+F(118)/55*x*z^2*v^2-F(61)/2*x^2*u*v^2-F(29)/66*x*y*u*v^2-1408*y^2*u*v^2+F(71)/13*x*z*u*v^2-14264*y*z*u*v^2-F(116)/21*z^2*u*v^2+2267*x*u^2*v^2+F(73)/54*y*u^2*v^2-1700*z*u^2*v^2-F(13)/60*u^3*v^2-F(11)/42*x^2*v^3+10463*x*y*v^3-1474*x*z*v^3+F(50)/33*x*u*v^3-F(5)/77*y*u*v^3+F(64)/91*z*u*v^3-8778*u^2*v^3+6592*x*v^4-F(29)/120*u*v^4,-1765*x^3*y^2-F(101)/82*x^2*y^3-1718*x*y^4-F(99)/8*y^5+817*x^3*y*z+2804*x^2*y^2*z+1244*x*y^3*z+9270*y^4*z+F(32)/13*x^2*y*z^2+F(68)/117*x*y^2*z^2-F(71)/125*y^3*z^2+F(109)/119*x*y*z^3-F(117)/116*y^2*z^3+F(9)/23*y*z^4-F(75)/64*x^3*y*u+5410*x^2*y^2*u+F(77)/46*x*y^3*u+F(67)/88*y^4*u-F(52)/89*x^3*z*u-14876*x^2*y*z*u+F(117)/76*x*y^2*z*u-F(61)/47*y^3*z*u-14889*x^2*z^2*u-12294*x*y*z^2*u-F(37)/35*y^2*z^2*u+F(7)/37*x*z^3*u-F(112)/69*y*z^3*u+8247*z^4*u-9745*x^3*u^2-5575*x^2*y*u^2-F(107)/44*x*y^2*u^2+12769*y^3*u^2-11245*x^2*z*u^2-8712*x*y*z*u^2-14154*y^2*z*u^2+12527*x*z^2*u^2-5802*y*z^2*u^2+8874*z^3*u^2+11027*x^2*u^3+F(79)/125*x*y*u^3+F(45)/52*y^2*u^3-F(92)/73*x*z*u^3-3805*y*z*u^3+6083*z^2*u^3-F(5)/102*x*u^4+F(11)/98*y*u^4+14346*z*u^4-12757*u^5+10874*x^3*y*v-F(57)/97*x^2*y^2*v-F(32)/81*x*y^3*v+F(97)/46*y^4*v-F(61)/88*x^2*y*z*v+5738*x*y^2*z*v+F(86)/15*y^3*z*v+F(86)/27*x*y*z^2*v-F(71)/83*y^2*z^2*v-F(31)/34*y*z^3*v+1823*x^3*u*v+F(31)/57*x^2*y*u*v+F(73)/123*x*y^2*u*v-F(13)/93*y^3*u*v+F(19)/63*x^2*z*u*v+9206*x*y*z*u*v+12130*y^2*z*u*v+10333*x*z^2*u*v-6360*y*z^2*u*v-9473*z^3*u*v+F(121)/31*x^2*u^2*v+F(117)/88*x*y*u^2*v-F(1)/80*y^2*u^2*v-F(64)/125*x*z*u^2*v-11840*y*z*u^2*v+F(89)/8*z^2*u^2*v-F(37)/61*x*u^3*v-142*y*u^3*v+11899*z*u^3*v+7808*u^4*v-F(34)/31*x^2*y*v^2-1949*x*y^2*v^2-12902*y^3*v^2-9729*x*y*z*v^2+10112*y^2*z*v^2-F(118)/55*y*z^2*v^2+171*x^2*u*v^2+5307*x*y*u*v^2+6691*y^2*u*v^2-1492*x*z*u*v^2-F(29)/96*y*z*u*v^2+F(52)/31*z^2*u*v^2+F(86)/121*x*u^2*v^2+F(6)/59*y*u^2*v^2-F(37)/109*z*u^2*v^2+F(21)/11*u^3*v^2+F(11)/42*x*y*v^3-10463*y^2*v^3+1474*y*z*v^3+10112*x*u*v^3-F(23)/71*y*u*v^3-9941*z*u*v^3-F(104)/85*u^2*v^3-6592*y*v^4+F(38)/3*u*v^4,-1765*x^3*y*z-F(101)/82*x^2*y^2*z-1718*x*y^3*z-F(99)/8*y^4*z+817*x^3*z^2+2804*x^2*y*z^2+1244*x*y^2*z^2+9270*y^3*z^2+F(32)/13*x^2*z^3+F(68)/117*x*y*z^3-F(71)/125*y^2*z^3+F(109)/119*x*z^4-F(117)/116*y*z^4+F(9)/23*z^5-F(25)/68*x^3*y*u-9266*x^2*y^2*u+15250*x*y^3*u-11084*y^4*u-F(39)/22*x^3*z*u-6579*x^2*y*z*u+F(110)/7*x*y^2*z*u-F(14)/65*y^3*z*u+F(50)/109*x^2*z^2*u-F(19)/124*x*y*z^2*u+F(126)/13*y^2*z^2*u+8579*x*z^3*u+6225*y*z^3*u-12081*z^4*u-9037*x^3*u^2-F(47)/15*x^2*y*u^2+15162*x*y^2*u^2+F(9)/92*y^3*u^2+15891*x^2*z*u^2-F(77)/57*x*y*z*u^2-9973*y^2*z*u^2+F(71)/19*x*z^2*u^2-325*y*z^2*u^2+2682*z^3*u^2-F(19)/26*x^2*u^3-F(87)/113*x*y*u^3+F(86)/13*y^2*u^3-F(110)/109*x*z*u^3+7940*y*z*u^3+366*z^2*u^3-4905*x*u^4+403*y*u^4+F(107)/2*z*u^4+8369*u^5+10874*x^3*z*v-F(57)/97*x^2*y*z*v-F(32)/81*x*y^2*z*v+F(97)/46*y^3*z*v-F(61)/88*x^2*z^2*v+5738*x*y*z^2*v+F(86)/15*y^2*z^2*v+F(86)/27*x*z^3*v-F(71)/83*y*z^3*v-F(31)/34*z^4*v-F(122)/49*x^3*u*v+F(119)/106*x^2*y*u*v+6010*x*y^2*u*v+F(10)/31*y^3*u*v-F(68)/45*x^2*z*u*v+F(110)/19*x*y*z*u*v-77*y^2*z*u*v+F(87)/22*x*z^2*u*v+F(17)/69*y*z^2*u*v-7009*z^3*u*v+F(31)/45*x^2*u^2*v+F(87)/34*x*y*u^2*v-6059*y^2*u^2*v-F(125)/106*x*z*u^2*v+11472*y*z*u^2*v+5684*z^2*u^2*v+13563*x*u^3*v-8216*y*u^3*v-7218*z*u^3*v-F(43)/14*u^4*v-F(34)/31*x^2*z*v^2-1949*x*y*z*v^2-12902*y^2*z*v^2-9729*x*z^2*v^2+10112*y*z^2*v^2-F(118)/55*z^3*v^2-10868*x^2*u*v^2+4449*x*y*u*v^2+9038*y^2*u*v^2-F(80)/11*x*z*u*v^2+5300*y*z*u*v^2-11102*z^2*u*v^2+8968*x*u^2*v^2-1060*y*u^2*v^2+F(83)/81*z*u^2*v^2-F(77)/43*u^3*v^2+F(11)/42*x*z*v^3-10463*y*z*v^3+1474*z^2*v^3+F(57)/44*x*u*v^3-F(11)/89*y*u*v^3+2107*z*u*v^3-F(9)/53*u^2*v^3-6592*z*v^4+F(41)/32*u*v^4,-F(25)/68*x^4*y-9266*x^3*y^2+15250*x^2*y^3-11084*x*y^4+F(81)/92*x^4*z-8630*x^3*y*z+F(1)/39*x^2*y^2*z+F(10)/49*x*y^3*z+569*y^4*z-F(68)/113*x^3*z^2+F(122)/19*x^2*y*z^2+F(125)/126*x*y^2*z^2+7637*y^3*z^2-4043*x^2*z^3-F(63)/95*x*y*z^3-F(121)/61*y^2*z^3-3346*x*z^4-1703*y*z^4-12316*z^5-9037*x^4*u-F(47)/15*x^3*y*u+15162*x^2*y^2*u+F(9)/92*x*y^3*u-F(42)/115*x^3*z*u+3260*x^2*y*z*u+3504*x*y^2*z*u+F(66)/13*y^3*z*u+F(52)/83*x^2*z^2*u+F(11)/47*x*y*z^2*u-4372*y^2*z^2*u-F(91)/18*x*z^3*u+F(125)/51*y*z^3*u-7890*z^4*u-F(19)/26*x^3*u^2-F(87)/113*x^2*y*u^2+F(86)/13*x*y^2*u^2+11777*x^2*z*u^2+F(112)/29*x*y*z*u^2+5038*y^2*z*u^2-1694*x*z^2*u^2+F(56)/79*y*z^2*u^2-14124*z^3*u^2-4905*x^2*u^3+403*x*y*u^3-15569*x*z*u^3+F(74)/65*y*z*u^3+12643*z^2*u^3+8369*x*u^4-F(62)/59*z*u^4-F(122)/49*x^4*v+F(119)/106*x^3*y*v+6010*x^2*y^2*v+F(10)/31*x*y^3*v+13205*x^3*z*v-F(101)/36*x^2*y*z*v-5364*x*y^2*z*v+11448*y^3*z*v+F(68)/67*x^2*z^2*v+F(23)/113*x*y*z^2*v+F(29)/30*y^2*z^2*v-12823*x*z^3*v-6215*y*z^3*v-F(20)/111*z^4*v+F(31)/45*x^3*u*v+F(87)/34*x^2*y*u*v-6059*x*y^2*u*v-F(41)/79*x^2*z*u*v-F(123)/59*x*y*z*u*v-F(33)/122*y^2*z*u*v-7760*x*z^2*u*v+F(8)/11*y*z^2*u*v-6841*z^3*u*v+13563*x^2*u^2*v-8216*x*y*u^2*v+14820*x*z*u^2*v+F(63)/32*y*z*u^2*v-F(88)/37*z^2*u^2*v-F(43)/14*x*u^3*v-3971*z*u^3*v-10868*x^3*v^2+4449*x^2*y*v^2+9038*x*y^2*v^2-F(19)/39*x^2*z*v^2+F(36)/89*x*y*z*v^2-1408*y^2*z*v^2-F(85)/112*x*z^2*v^2-14264*y*z^2*v^2-F(116)/21*z^3*v^2+8968*x^2*u*v^2-1060*x*y*u*v^2+F(121)/26*x*z*u*v^2+F(73)/54*y*z*u*v^2-1700*z^2*u*v^2-F(77)/43*x*u^2*v^2-F(13)/60*z*u^2*v^2+F(57)/44*x^2*v^3-F(11)/89*x*y*v^3-F(119)/125*x*z*v^3-F(5)/77*y*z*v^3+F(64)/91*z^2*v^3-F(9)/53*x*u*v^3-8778*z*u*v^3+F(41)/32*x*v^4-F(29)/120*z*v^4,F(25)/68*x^3*y^2+9266*x^2*y^3-15250*x*y^4+11084*y^5-F(16)/49*x^3*y*z-F(61)/8*x^2*y^2*z-F(38)/123*x*y^3*z+F(103)/94*y^4*z-F(52)/89*x^3*z^2-F(107)/34*x^2*y*z^2-F(80)/49*x*y^2*z^2+8314*y^3*z^2-14889*x^2*z^3+11118*x*y*z^3+F(61)/80*y^2*z^3+F(7)/37*x*z^4+7443*y*z^4+8247*z^5+9037*x^3*y*u+F(47)/15*x^2*y^2*u-15162*x*y^3*u-F(9)/92*y^4*u-9745*x^3*z*u+F(125)/76*x^2*y*z*u+F(48)/109*x*y^2*z*u+F(117)/83*y^3*z*u-11245*x^2*z^2*u-F(92)/91*x*y*z^2*u-13829*y^2*z^2*u+12527*x*z^3*u-8484*y*z^3*u+8874*z^4*u+F(19)/26*x^2*y*u^2+F(87)/113*x*y^2*u^2-F(86)/13*y^3*u^2+11027*x^2*z*u^2-4274*x*y*z*u^2+F(1)/42*y^2*z*u^2-F(92)/73*x*z^2*u^2+F(40)/23*y*z^2*u^2+6083*z^3*u^2+4905*x*y*u^3-403*y^2*u^3-F(5)/102*x*z*u^3+F(119)/10*y*z*u^3+14346*z^2*u^3-8369*y*u^4-12757*z*u^4+F(122)/49*x^3*y*v-F(119)/106*x^2*y^2*v-6010*x*y^3*v-F(10)/31*y^4*v+1823*x^3*z*v-14291*x^2*y*z*v+F(116)/33*x*y^2*z*v-4395*y^3*z*v+F(19)/63*x^2*z^2*v-12610*x*y*z^2*v+F(37)/56*y^2*z^2*v+10333*x*z^3*v+649*y*z^3*v-9473*z^4*v-F(31)/45*x^2*y*u*v-F(87)/34*x*y^2*u*v+6059*y^3*u*v+F(121)/31*x^2*z*u*v-F(79)/56*x*y*z*u*v-F(104)/121*y^2*z*u*v-F(64)/125*x*z^2*u*v+14467*y*z^2*u*v+F(89)/8*z^3*u*v-13563*x*y*u^2*v+8216*y^2*u^2*v-F(37)/61*x*z*u^2*v+F(111)/104*y*z*u^2*v+11899*z^2*u^2*v+F(43)/14*y*u^3*v+7808*z*u^3*v+10868*x^2*y*v^2-4449*x*y^2*v^2-9038*y^3*v^2+171*x^2*z*v^2+2406*x*y*z*v^2+F(2)/23*y^2*z*v^2-1492*x*z^2*v^2+F(88)/39*y*z^2*v^2+F(52)/31*z^3*v^2-8968*x*y*u*v^2+1060*y^2*u*v^2+F(86)/121*x*z*u*v^2-F(27)/22*y*z*u*v^2-F(37)/109*z^2*u*v^2+F(77)/43*y*u^2*v^2+F(21)/11*z*u^2*v^2-F(57)/44*x*y*v^3+F(11)/89*y^2*v^3+10112*x*z*v^3+11410*y*z*v^3-9941*z^2*v^3+F(9)/53*y*u*v^3-F(104)/85*z*u*v^3-F(41)/32*y*v^4+F(38)/3*z*v^4,F(125)/69*x^4*y-3359*x^3*y^2-15365*x^2*y^3+11049*x*y^4-569*y^5+F(52)/89*x^4*z-F(97)/3*x^3*y*z+F(107)/114*x^2*y^2*z-15678*x*y^3*z-7637*y^4*z+14889*x^3*z^2-F(7)/104*x^2*y*z^2+F(13)/81*x*y^2*z^2+F(121)/61*y^3*z^2-F(7)/37*x^2*z^3-4097*x*y*z^3+1703*y^2*z^3-8247*x*z^4+12316*y*z^4+9745*x^4*u-11081*x^3*y*u+555*x^2*y^2*u+F(118)/39*x*y^3*u-F(66)/13*y^4*u+11245*x^3*z*u+F(102)/67*x^2*y*z*u-F(32)/63*x*y^2*z*u+4372*y^3*z*u-12527*x^2*z^2*u-11061*x*y*z^2*u-F(125)/51*y^2*z^2*u-8874*x*z^3*u+7890*y*z^3*u-11027*x^3*u^2+F(86)/81*x^2*y*u^2-F(125)/93*x*y^2*u^2-5038*y^3*u^2+F(92)/73*x^2*z*u^2-F(1)/60*x*y*z*u^2-F(56)/79*y^2*z*u^2-6083*x*z^2*u^2+14124*y*z^2*u^2+F(5)/102*x^2*u^3-F(95)/44*x*y*u^3-F(74)/65*y^2*u^3-14346*x*z*u^3-12643*y*z*u^3+12757*x*u^4+F(62)/59*y*u^4-1823*x^4*v+F(92)/59*x^3*y*v+F(75)/122*x^2*y^2*v-F(57)/59*x*y^3*v-11448*y^4*v-F(19)/63*x^3*z*v+F(20)/59*x^2*y*z*v+F(104)/61*x*y^2*z*v-F(29)/30*y^3*z*v-10333*x^2*z^2*v+F(49)/113*x*y*z^2*v+6215*y^2*z^2*v+9473*x*z^3*v+F(20)/111*y*z^3*v-F(121)/31*x^3*u*v+7009*x^2*y*u*v+F(29)/124*x*y^2*u*v+F(33)/122*y^3*u*v+F(64)/125*x^2*z*u*v+F(49)/62*x*y*z*u*v-F(8)/11*y^2*z*u*v-F(89)/8*x*z^2*u*v+6841*y*z^2*u*v+F(37)/61*x^2*u^2*v+10095*x*y*u^2*v-F(63)/32*y^2*u^2*v-11899*x*z*u^2*v+F(88)/37*y*z*u^2*v-7808*x*u^3*v+3971*y*u^3*v-171*x^3*v^2+10719*x^2*y*v^2-F(23)/15*x*y^2*v^2+1408*y^3*v^2+1492*x^2*z*v^2+10274*x*y*z*v^2+14264*y^2*z*v^2-F(52)/31*x*z^2*v^2+F(116)/21*y*z^2*v^2-F(86)/121*x^2*u*v^2-F(23)/72*x*y*u*v^2-F(73)/54*y^2*u*v^2+F(37)/109*x*z*u*v^2+1700*y*z*u*v^2-F(21)/11*x*u^2*v^2+F(13)/60*y*u^2*v^2-10112*x^2*v^3-F(69)/35*x*y*v^3+F(5)/77*y^2*v^3+9941*x*z*v^3-F(64)/91*y*z*v^3+F(104)/85*x*u*v^3+8778*y*u*v^3-F(38)/3*x*v^4+F(29)/120*y*v^4,1610*x^4*y+5103*x^3*y^2-F(48)/103*x^2*y^3-7704*x*y^4+F(89)/99*x^4*z+F(85)/36*x^3*y*z-F(3)/8*x^2*y^2*z-14268*x*y^3*z+F(21)/122*x^3*z^2-F(23)/70*x^2*y*z^2-F(101)/107*x*y^2*z^2-13526*x^2*z^3+12762*x*y*z^3-9424*x*z^4-F(67)/80*x^4*u-14459*x^3*y*u-F(59)/114*x^2*y^2*u-11900*x*y^3*u+15870*x^3*z*u+F(85)/19*x^2*y*z*u+9435*x*y^2*z*u-F(41)/54*x^2*z^2*u-F(41)/36*x*y*z^2*u-F(100)/83*x*z^3*u+1834*x^3*u^2+10771*x^2*y*u^2+F(30)/11*x*y^2*u^2-F(2)/49*x^2*z*u^2+6297*x*y*z*u^2-F(31)/54*x*z^2*u^2-F(5)/99*x^2*u^3-F(103)/79*x*y*u^3-F(96)/89*x*z*u^3-F(21)/124*x*u^4+F(109)/124*x^4*v+F(41)/117*x^3*y*v+F(113)/34*x^2*y^2*v+F(71)/5*x*y^3*v-F(121)/126*y^4*v-14318*x^3*z*v-F(10)/31*x^2*y*z*v+1736*x*y^2*z*v+15895*y^3*z*v-5703*x^2*z^2*v-F(94)/113*x*y*z^2*v-F(98)/43*y^2*z^2*v+3*x*z^3*v+424*y*z^3*v+F(9)/92*z^4*v-F(99)/100*x^3*u*v+F(115)/94*x^2*y*u*v+3152*x*y^2*u*v+10277*y^3*u*v-F(27)/107*x^2*z*u*v+F(15)/67*x*y*z*u*v+9280*y^2*z*u*v+13440*x*z^2*u*v-8276*y*z^2*u*v+11616*z^3*u*v+F(74)/15*x^2*u^2*v+F(5)/21*x*y*u^2*v+7185*y^2*u^2*v+5660*x*z*u^2*v+13929*y*z*u^2*v-F(116)/39*z^2*u^2*v+F(54)/41*x*u^3*v+10463*y*u^3*v-1317*z*u^3*v+F(53)/25*u^4*v+1755*x^3*v^2-F(107)/39*x^2*y*v^2+F(115)/76*x*y^2*v^2+11323*y^3*v^2-F(47)/35*x^2*z*v^2+14895*x*y*z*v^2-F(99)/59*y^2*z*v^2+14938*x*z^2*v^2+2013*y*z^2*v^2+9910*z^3*v^2-6063*x^2*u*v^2-F(116)/107*x*y*u*v^2-11416*y^2*u*v^2-9280*x*z*u*v^2-10275*y*z*u*v^2+103*z^2*u*v^2+2684*x*u^2*v^2-8501*y*u^2*v^2+4242*z*u^2*v^2+F(77)/53*u^3*v^2-F(112)/97*x^2*v^3+5247*x*y*v^3+15*y^2*v^3-F(122)/113*x*z*v^3-13730*y*z*v^3-F(122)/71*z^2*v^3-F(23)/99*x*u*v^3+60*y*u*v^3+F(119)/72*z*u*v^3+F(85)/74*u^2*v^3-F(13)/125*x*v^4+7626*y*v^4+F(61)/123*z*v^4+F(96)/23*u*v^4+F(1)/10*v^5,-1610*x^3*y^2-5103*x^2*y^3+F(48)/103*x*y^4+7704*y^5-F(89)/99*x^3*y*z-F(85)/36*x^2*y^2*z+F(3)/8*x*y^3*z+14268*y^4*z-F(21)/122*x^2*y*z^2+F(23)/70*x*y^2*z^2+F(101)/107*y^3*z^2+13526*x*y*z^3-12762*y^2*z^3+9424*y*z^4+F(67)/80*x^3*y*u+14459*x^2*y^2*u+F(59)/114*x*y^3*u+11900*y^4*u-15870*x^2*y*z*u-F(85)/19*x*y^2*z*u-9435*y^3*z*u+F(41)/54*x*y*z^2*u+F(41)/36*y^2*z^2*u+F(100)/83*y*z^3*u-1834*x^2*y*u^2-10771*x*y^2*u^2-F(30)/11*y^3*u^2+F(2)/49*x*y*z*u^2-6297*y^2*z*u^2+F(31)/54*y*z^2*u^2+F(5)/99*x*y*u^3+F(103)/79*y^2*u^3+F(96)/89*y*z*u^3+F(21)/124*y*u^4-F(46)/123*x^3*y*v+F(117)/37*x^2*y^2*v+F(41)/72*x*y^3*v-1019*y^4*v-11526*x^3*z*v+F(7)/18*x^2*y*z*v-F(41)/75*x*y^2*z*v+14176*y^3*z*v+7743*x^2*z^2*v-F(98)/59*x*y*z^2*v+15698*y^2*z^2*v-F(16)/89*x*z^3*v-F(31)/59*y*z^3*v+F(73)/94*z^4*v+F(92)/3*x^3*u*v+F(7)/46*x^2*y*u*v+F(75)/109*x*y^2*u*v+F(79)/119*y^3*u*v-3350*x^2*z*u*v+2859*x*y*z*u*v+4546*y^2*z*u*v+F(29)/38*x*z^2*u*v-F(32)/71*y*z^2*u*v+F(7)/4*z^3*u*v+F(2)/103*x^2*u^2*v-F(97)/107*x*y*u^2*v+8234*y^2*u^2*v-13987*x*z*u^2*v+F(31)/15*y*z*u^2*v+10582*z^2*u^2*v+F(45)/47*x*u^3*v-5929*y*u^3*v-F(37)/98*z*u^3*v-6364*u^4*v-F(77)/58*x^3*v^2-F(22)/37*x^2*y*v^2-F(125)/32*x*y^2*v^2-12288*y^3*v^2+7168*x^2*z*v^2+7788*x*y*z*v^2+F(120)/19*y^2*z*v^2+F(28)/103*x*z^2*v^2-15664*y*z^2*v^2-F(116)/9*z^3*v^2+F(113)/56*x^2*u*v^2+F(102)/65*x*y*u*v^2+F(64)/15*y^2*u*v^2+F(60)/119*x*z*u*v^2+12151*y*z*u*v^2+F(89)/24*z^2*u*v^2-F(39)/64*x*u^2*v^2+F(12)/29*y*u^2*v^2+F(32)/107*z*u^2*v^2-F(39)/8*u^3*v^2+F(2)/15*x^2*v^3-378*x*y*v^3+11793*y^2*v^3+9505*x*z*v^3+F(101)/107*y*z*v^3+F(121)/87*z^2*v^3+11378*x*u*v^3+F(67)/27*y*u*v^3-F(37)/124*z*u*v^3+961*u^2*v^3-F(24)/101*x*v^4+F(85)/18*y*v^4+F(83)/25*z*v^4-15412*u*v^4+F(17)/54*v^5,-1610*x^3*y*z-5103*x^2*y^2*z+F(48)/103*x*y^3*z+7704*y^4*z-F(89)/99*x^3*z^2-F(85)/36*x^2*y*z^2+F(3)/8*x*y^2*z^2+14268*y^3*z^2-F(21)/122*x^2*z^3+F(23)/70*x*y*z^3+F(101)/107*y^2*z^3+13526*x*z^4-12762*y*z^4+9424*z^5+F(67)/80*x^3*z*u+14459*x^2*y*z*u+F(59)/114*x*y^2*z*u+11900*y^3*z*u-15870*x^2*z^2*u-F(85)/19*x*y*z^2*u-9435*y^2*z^2*u+F(41)/54*x*z^3*u+F(41)/36*y*z^3*u+F(100)/83*z^4*u-1834*x^2*z*u^2-10771*x*y*z*u^2-F(30)/11*y^2*z*u^2+F(2)/49*x*z^2*u^2-6297*y*z^2*u^2+F(31)/54*z^3*u^2+F(5)/99*x*z*u^3+F(103)/79*y*z*u^3+F(96)/89*z^2*u^3+F(21)/124*z*u^4+F(13)/114*x^3*y*v-F(54)/59*x^2*y^2*v-5248*x*y^3*v+4142*y^4*v-F(121)/73*x^3*z*v-F(57)/68*x^2*y*z*v+5956*x*y^2*z*v+5015*y^3*z*v-11103*x^2*z^2*v+F(74)/21*x*y*z^2*v-F(93)/5*y^2*z^2*v-F(77)/104*x*z^3*v-3287*y*z^3*v+F(86)/55*z^4*v+F(14)/97*x^3*u*v-11054*x^2*y*u*v-F(31)/106*x*y^2*u*v-12826*y^3*u*v-10806*x^2*z*u*v+F(26)/113*x*y*z*u*v+F(118)/111*y^2*z*u*v-F(15)/88*x*z^2*u*v-250*y*z^2*u*v-11676*z^3*u*v-8071*x^2*u^2*v-13271*x*y*u^2*v+10089*y^2*u^2*v-F(93)/104*x*z*u^2*v-11603*y*z*u^2*v+F(49)/96*z^2*u^2*v+8303*x*u^3*v+34*y*u^3*v-F(32)/105*z*u^3*v+F(68)/53*u^4*v-F(67)/90*x^3*v^2-F(89)/21*x^2*y*v^2-15470*x*y^2*v^2+3974*y^3*v^2+3176*x^2*z*v^2-F(73)/9*x*y*z*v^2+9857*y^2*z*v^2+1591*x*z^2*v^2-5437*y*z^2*v^2+F(111)/28*z^3*v^2-F(97)/28*x^2*u*v^2+F(16)/113*x*y*u*v^2+F(47)/117*y^2*u*v^2+15012*x*z*u*v^2-15924*y*z*u*v^2+F(20)/73*z^2*u*v^2+F(65)/33*x*u^2*v^2-3931*y*u^2*v^2+F(109)/126*z*u^2*v^2-1788*u^3*v^2+F(52)/95*x^2*v^3-F(119)/67*x*y*v^3-10832*y^2*v^3-F(3)/32*x*z*v^3+F(52)/89*y*z*v^3+F(97)/105*z^2*v^3+F(28)/47*x*u*v^3-F(91)/106*y*u*v^3-F(97)/44*z*u*v^3-11614*u^2*v^3+F(65)/86*x*v^4+F(71)/53*y*v^4+5271*z*v^4-F(107)/23*u*v^4-6867*v^5,-1610*x^3*y*u-5103*x^2*y^2*u+F(48)/103*x*y^3*u+7704*y^4*u-F(89)/99*x^3*z*u-F(85)/36*x^2*y*z*u+F(3)/8*x*y^2*z*u+14268*y^3*z*u-F(21)/122*x^2*z^2*u+F(23)/70*x*y*z^2*u+F(101)/107*y^2*z^2*u+13526*x*z^3*u-12762*y*z^3*u+9424*z^4*u+F(67)/80*x^3*u^2+14459*x^2*y*u^2+F(59)/114*x*y^2*u^2+11900*y^3*u^2-15870*x^2*z*u^2-F(85)/19*x*y*z*u^2-9435*y^2*z*u^2+F(41)/54*x*z^2*u^2+F(41)/36*y*z^2*u^2+F(100)/83*z^3*u^2-1834*x^2*u^3-10771*x*y*u^3-F(30)/11*y^2*u^3+F(2)/49*x*z*u^3-6297*y*z*u^3+F(31)/54*z^2*u^3+F(5)/99*x*u^4+F(103)/79*y*u^4+F(96)/89*z*u^4+F(21)/124*u^5-12965*x^3*y*v+F(32)/59*x^2*y^2*v-F(86)/59*x*y^3*v-F(62)/79*y^4*v+F(34)/19*x^3*z*v+F(13)/2*x^2*y*z*v+2809*x*y^2*z*v-9116*y^3*z*v+5450*x^2*z^2*v+10903*x*y*z^2*v+F(16)/43*y^2*z^2*v-4166*x*z^3*v-225*y*z^3*v+F(103)/39*z^4*v-F(25)/74*x^3*u*v+F(104)/51*x^2*y*u*v-F(109)/47*x*y^2*u*v+F(81)/83*y^3*u*v-F(47)/20*x^2*z*u*v-F(113)/14*x*y*z*u*v-F(8)/49*y^2*z*u*v-F(7)/38*x*z^2*u*v-14754*y*z^2*u*v+F(40)/31*z^3*u*v+192*x^2*u^2*v-F(21)/109*x*y*u^2*v+10167*y^2*u^2*v-5429*x*z*u^2*v+F(22)/21*y*z*u^2*v-8966*z^2*u^2*v+F(69)/76*x*u^3*v-F(89)/115*y*u^3*v+F(117)/88*z*u^3*v+9423*u^4*v-5458*x^3*v^2+F(53)/126*x^2*y*v^2+F(63)/61*x*y^2*v^2-1358*y^3*v^2-722*x^2*z*v^2+7831*x*y*z*v^2+11764*y^2*z*v^2-F(10)/93*x*z^2*v^2-15707*y*z^2*v^2-F(13)/77*z^3*v^2+11234*x^2*u*v^2+F(111)/104*x*y*u*v^2-1222*y^2*u*v^2+F(49)/51*x*z*u*v^2-F(55)/43*y*z*u*v^2+15127*z^2*u*v^2+F(44)/3*x*u^2*v^2+F(120)/37*y*u^2*v^2-F(65)/19*z*u^2*v^2-F(52)/57*u^3*v^2-15084*x^2*v^3+15767*x*y*v^3+F(54)/37*y^2*v^3+F(67)/70*x*z*v^3-10781*y*z*v^3-6724*z^2*v^3+2754*x*u*v^3+F(95)/63*y*u*v^3-15691*z*u*v^3+7950*u^2*v^3+14464*x*v^4+7372*y*v^4+F(13)/41*z*v^4+F(74)/123*u*v^4+F(65)/118*v^5,-12965*x^4*y+F(32)/59*x^3*y^2-F(86)/59*x^2*y^3-F(62)/79*x*y^4+F(34)/19*x^4*z+F(13)/2*x^3*y*z+2809*x^2*y^2*z-9116*x*y^3*z+5450*x^3*z^2+10903*x^2*y*z^2+F(16)/43*x*y^2*z^2-4166*x^2*z^3-225*x*y*z^3+F(103)/39*x*z^4-10354*x^4*u-F(67)/46*x^3*y*u-F(109)/31*x^2*y^2*u+F(5)/107*x*y^3*u-F(121)/126*y^4*u+12872*x^3*z*u-8043*x^2*y*z*u+F(79)/111*x*y^2*z*u+15895*y^3*z*u-F(38)/53*x^2*z^2*u-F(17)/111*x*y*z^2*u-F(98)/43*y^2*z^2*u+9292*x*z^3*u+424*y*z^3*u+F(9)/92*z^4*u-F(117)/125*x^3*u^2+9115*x^2*y*u^2+13319*x*y^2*u^2+10277*y^3*u^2+F(60)/49*x^2*z*u^2-7411*x*y*z*u^2+9280*y^2*z*u^2+4474*x*z^2*u^2-8276*y*z^2*u^2+11616*z^3*u^2+3682*x^2*u^3+F(54)/97*x*y*u^3+7185*y^2*u^3+F(45)/77*x*z*u^3+13929*y*z*u^3-F(116)/39*z^2*u^3+F(14)/13*x*u^4+10463*y*u^4-1317*z*u^4+F(53)/25*u^5-5458*x^4*v+F(53)/126*x^3*y*v+F(63)/61*x^2*y^2*v-1358*x*y^3*v-722*x^3*z*v+7831*x^2*y*z*v+11764*x*y^2*z*v-F(10)/93*x^2*z^2*v-15707*x*y*z^2*v-F(13)/77*x*z^3*v+12989*x^3*u*v-F(107)/125*x^2*y*u*v+F(69)/121*x*y^2*u*v+11323*y^3*u*v+F(24)/31*x^2*z*u*v+F(93)/59*x*y*z*u*v-F(99)/59*y^2*z*u*v+F(97)/83*x*z^2*u*v+2013*y*z^2*u*v+9910*z^3*u*v-F(19)/67*x^2*u^2*v+12923*x*y*u^2*v-11416*y^2*u^2*v-F(50)/39*x*z*u^2*v-10275*y*z*u^2*v+103*z^2*u^2*v-F(26)/23*x*u^3*v-8501*y*u^3*v+4242*z*u^3*v+F(77)/53*u^4*v-15084*x^3*v^2+15767*x^2*y*v^2+F(54)/37*x*y^2*v^2+F(67)/70*x^2*z*v^2-10781*x*y*z*v^2-6724*x*z^2*v^2+F(107)/73*x^2*u*v^2-F(100)/53*x*y*u*v^2+15*y^2*u*v^2-F(29)/124*x*z*u*v^2-13730*y*z*u*v^2-F(122)/71*z^2*u*v^2+F(22)/45*x*u^2*v^2+60*y*u^2*v^2+F(119)/72*z*u^2*v^2+F(85)/74*u^3*v^2+14464*x^2*v^3+7372*x*y*v^3+F(13)/41*x*z*v^3+F(19)/34*x*u*v^3+7626*y*u*v^3+F(61)/123*z*u*v^3+F(96)/23*u^2*v^3+F(65)/118*x*v^4+F(1)/10*u*v^4,12965*x^3*y^2-F(32)/59*x^2*y^3+F(86)/59*x*y^4+F(62)/79*y^5-F(34)/19*x^3*y*z-F(13)/2*x^2*y^2*z-2809*x*y^3*z+9116*y^4*z-5450*x^2*y*z^2-10903*x*y^2*z^2-F(16)/43*y^3*z^2+4166*x*y*z^3+225*y^2*z^3-F(103)/39*y*z^4-F(65)/48*x^3*y*u-8679*x^2*y^2*u+F(99)/31*x*y^3*u-F(98)/53*y^4*u-11526*x^3*z*u-F(49)/47*x^2*y*z*u-F(34)/121*x*y^2*z*u+F(49)/73*y^3*z*u+7743*x^2*z^2*u-F(65)/73*x*y*z^2*u-F(101)/104*y^2*z^2*u-F(16)/89*x*z^3*u+F(63)/52*y*z^3*u+F(73)/94*z^4*u+F(92)/3*x^3*u^2+F(72)/25*x^2*y*u^2+F(96)/109*x*y^2*u^2-F(121)/3*y^3*u^2-3350*x^2*z*u^2+8288*x*y*z*u^2-F(114)/65*y^2*z*u^2+F(29)/38*x*z^2*u^2-46*y*z^2*u^2+F(7)/4*z^3*u^2+F(2)/103*x^2*u^3-10714*x*y*u^3-8178*y^2*u^3-13987*x*z*u^3-F(13)/81*y*z*u^3+10582*z^2*u^3+F(45)/47*x*u^4+F(92)/25*y*u^4-F(37)/98*z*u^4-6364*u^5+5458*x^3*y*v-F(53)/126*x^2*y^2*v-F(63)/61*x*y^3*v+1358*y^4*v+722*x^2*y*z*v-7831*x*y^2*z*v-11764*y^3*z*v+F(10)/93*x*y*z^2*v+15707*y^2*z^2*v+F(13)/77*y*z^3*v-F(77)/58*x^3*u*v-3453*x^2*y*u*v-F(13)/49*x*y^2*u*v-11066*y^3*u*v+7168*x^2*z*u*v-6013*x*y*z*u*v-14637*y^2*z*u*v+F(28)/103*x*z^2*u*v+F(27)/80*y*z^2*u*v-F(116)/9*z^3*u*v+F(113)/56*x^2*u^2*v+F(42)/97*x*y*u^2*v+F(117)/58*y^2*u^2*v+F(60)/119*x*z*u^2*v+F(101)/78*y*z*u^2*v+F(89)/24*z^2*u^2*v-F(39)/64*x*u^3*v+F(92)/11*y*u^3*v+F(32)/107*z*u^3*v-F(39)/8*u^4*v+15084*x^2*y*v^2-15767*x*y^2*v^2-F(54)/37*y^3*v^2-F(67)/70*x*y*z*v^2+10781*y^2*z*v^2+6724*y*z^2*v^2+F(2)/15*x^2*u*v^2-3132*x*y*u*v^2-F(49)/81*y^2*u*v^2+9505*x*z*u*v^2-F(67)/80*y*z*u*v^2+F(121)/87*z^2*u*v^2+11378*x*u^2*v^2-F(14)/41*y*u^2*v^2-F(37)/124*z*u^2*v^2+961*u^3*v^2-14464*x*y*v^3-7372*y^2*v^3-F(13)/41*y*z*v^3-F(24)/101*x*u*v^3-8102*y*u*v^3+F(83)/25*z*u*v^3-15412*u^2*v^3-F(65)/118*y*v^4+F(17)/54*u*v^4,12965*x^3*y*z-F(32)/59*x^2*y^2*z+F(86)/59*x*y^3*z+F(62)/79*y^4*z-F(34)/19*x^3*z^2-F(13)/2*x^2*y*z^2-2809*x*y^2*z^2+9116*y^3*z^2-5450*x^2*z^3-10903*x*y*z^3-F(16)/43*y^2*z^3+4166*x*z^4+225*y*z^4-F(103)/39*z^5+F(13)/114*x^3*y*u-F(54)/59*x^2*y^2*u-5248*x*y^3*u+4142*y^4*u-F(5)/107*x^3*z*u-11137*x^2*y*z*u+4597*x*y^2*z*u+13879*y^3*z*u-F(49)/66*x^2*z^2*u-F(8)/65*x*y*z^2*u-F(77)/75*y^2*z^2*u+F(27)/97*x*z^3*u-F(78)/53*y*z^3*u+1764*z^4*u+F(14)/97*x^3*u^2-11054*x^2*y*u^2-F(31)/106*x*y^2*u^2-12826*y^3*u^2-F(35)/32*x^2*z*u^2-F(81)/5*x*y*z*u^2-4690*y^2*z*u^2+F(79)/117*x*z^2*u^2-F(63)/67*y*z^2*u^2+F(65)/59*z^3*u^2-8071*x^2*u^3-13271*x*y*u^3+10089*y^2*u^3+F(79)/46*x*z*u^3+3976*y*z*u^3-F(113)/27*z^2*u^3+8303*x*u^4+34*y*u^4+10990*z*u^4+F(68)/53*u^5+5458*x^3*z*v-F(53)/126*x^2*y*z*v-F(63)/61*x*y^2*z*v+1358*y^3*z*v+722*x^2*z^2*v-7831*x*y*z^2*v-11764*y^2*z^2*v+F(10)/93*x*z^3*v+15707*y*z^3*v+F(13)/77*z^4*v-F(67)/90*x^3*u*v-F(89)/21*x^2*y*u*v-15470*x*y^2*u*v+3974*y^3*u*v-8058*x^2*z*u*v+25*x*y*z*u*v+11079*y^2*z*u*v-12210*x*z^2*u*v+F(114)/55*y*z^2*u*v+2015*z^3*u*v-F(97)/28*x^2*u^2*v+F(16)/113*x*y*u^2*v+F(47)/117*y^2*u^2*v-6330*x*z*u^2*v-14198*y*z*u^2*v-F(17)/114*z^2*u^2*v+F(65)/33*x*u^3*v-3931*y*u^3*v-F(94)/105*z*u^3*v-1788*u^4*v+15084*x^2*z*v^2-15767*x*y*z*v^2-F(54)/37*y^2*z*v^2-F(67)/70*x*z^2*v^2+10781*y*z^2*v^2+6724*z^3*v^2+F(52)/95*x^2*u*v^2-F(119)/67*x*y*u*v^2-10832*y^2*u*v^2-F(55)/114*x*z*u*v^2+13293*y*z*u*v^2+F(11)/10*z^2*u*v^2+F(28)/47*x*u^2*v^2-F(91)/106*y*u^2*v^2-5771*z*u^2*v^2-11614*u^3*v^2-14464*x*z*v^3-7372*y*z*v^3-F(13)/41*z^2*v^3+F(65)/86*x*u*v^3+F(71)/53*y*u*v^3+F(113)/104*z*u*v^3-F(107)/23*u^2*v^3-F(65)/118*z*v^4-6867*u*v^4,F(13)/114*x^4*y-F(54)/59*x^3*y^2-5248*x^2*y^3+4142*x*y^4+F(107)/85*x^4*z-10443*x^3*y*z+F(11)/51*x^2*y^2*z-1369*x*y^3*z-F(121)/126*y^4*z+F(47)/112*x^3*z^2+15581*x^2*y*z^2-11079*x*y^2*z^2+15895*y^3*z^2+F(73)/18*x^2*z^3-F(19)/5*x*y*z^3-F(98)/43*y^2*z^3+11056*x*z^4+424*y*z^4+F(9)/92*z^5+F(14)/97*x^4*u-11054*x^3*y*u-F(31)/106*x^2*y^2*u-12826*x*y^3*u-F(112)/67*x^3*z*u+F(87)/64*x^2*y*z*u+8629*x*y^2*z*u+10277*y^3*z*u-2219*x^2*z^2*u+F(101)/99*x*y*z^2*u+9280*y^2*z^2*u+1764*x*z^3*u-8276*y*z^3*u+11616*z^4*u-8071*x^3*u^2-13271*x^2*y*u^2+10089*x*y^2*u^2-F(78)/67*x^2*z*u^2-F(23)/86*x*y*z*u^2+7185*y^2*z*u^2-F(73)/16*x*z^2*u^2+13929*y*z^2*u^2-F(116)/39*z^3*u^2+8303*x^2*u^3+34*x*y*u^3-3774*x*z*u^3+10463*y*z*u^3-1317*z^2*u^3+F(68)/53*x*u^4+F(53)/25*z*u^4-F(67)/90*x^4*v-F(89)/21*x^3*y*v-15470*x^2*y^2*v+3974*x*y^3*v+F(121)/13*x^3*z*v+536*x^2*y*z*v-12451*x*y^2*z*v+11323*y^3*z*v+F(84)/107*x^2*z^2*v-F(24)/115*x*y*z^2*v-F(99)/59*y^2*z^2*v+89*x*z^3*v+2013*y*z^3*v+9910*z^4*v-F(97)/28*x^3*u*v+F(16)/113*x^2*y*u*v+F(47)/117*x*y^2*u*v+8949*x^2*z*u*v+F(116)/25*x*y*z*u*v-11416*y^2*z*u*v-2268*x*z^2*u*v-10275*y*z^2*u*v+103*z^3*u*v+F(65)/33*x^2*u^2*v-3931*x*y*u^2*v-108*x*z*u^2*v-8501*y*z*u^2*v+4242*z^2*u^2*v-1788*x*u^3*v+F(77)/53*z*u^3*v+F(52)/95*x^3*v^2-F(119)/67*x^2*y*v^2-10832*x*y^2*v^2-F(54)/35*x^2*z*v^2-F(107)/93*x*y*z*v^2+15*y^2*z*v^2-F(49)/3*x*z^2*v^2-13730*y*z^2*v^2-F(122)/71*z^3*v^2+F(28)/47*x^2*u*v^2-F(91)/106*x*y*u*v^2-F(24)/43*x*z*u*v^2+60*y*z*u*v^2+F(119)/72*z^2*u*v^2-11614*x*u^2*v^2+F(85)/74*z*u^2*v^2+F(65)/86*x^2*v^3+F(71)/53*x*y*v^3+F(79)/103*x*z*v^3+7626*y*z*v^3+F(61)/123*z^2*v^3-F(107)/23*x*u*v^3+F(96)/23*z*u*v^3-6867*x*v^4+F(1)/10*z*v^4,-F(13)/114*x^3*y^2+F(54)/59*x^2*y^3+5248*x*y^4-4142*y^5-10584*x^3*y*z-F(37)/13*x^2*y^2*z-F(31)/20*x*y^3*z+F(108)/53*y^4*z-11526*x^3*z^2-F(20)/101*x^2*y*z^2-F(37)/54*x*y^2*z^2-F(45)/32*y^3*z^2+7743*x^2*z^3-F(16)/91*x*y*z^3+F(121)/91*y^2*z^3-F(16)/89*x*z^4-2378*y*z^4+F(73)/94*z^5-F(14)/97*x^3*y*u+11054*x^2*y^2*u+F(31)/106*x*y^3*u+12826*y^4*u+F(92)/3*x^3*z*u+F(33)/119*x^2*y*z*u+F(71)/121*x*y^2*z*u-6014*y^3*z*u-3350*x^2*z^2*u-14954*x*y*z^2*u-F(53)/20*y^2*z^2*u+F(29)/38*x*z^3*u-F(23)/12*y*z^3*u+F(7)/4*z^4*u+8071*x^2*y*u^2+13271*x*y^2*u^2-10089*y^3*u^2+F(2)/103*x^2*z*u^2-14193*x*y*z*u^2-12154*y^2*z*u^2-13987*x*z^2*u^2-F(77)/79*y*z^2*u^2+10582*z^3*u^2-8303*x*y*u^3-34*y^2*u^3+F(45)/47*x*z*u^3+F(60)/17*y*z*u^3-F(37)/98*z^2*u^3-F(68)/53*y*u^4-6364*z*u^4+F(67)/90*x^3*y*v+F(89)/21*x^2*y^2*v+15470*x*y^3*v-3974*y^4*v-F(77)/58*x^3*z*v+4605*x^2*y*z*v+F(89)/74*x*y^2*z*v+F(34)/13*y^3*z*v+7168*x^2*z^2*v+6197*x*y*z^2*v-F(23)/30*y^2*z^2*v+F(28)/103*x*z^3*v-815*y*z^3*v-F(116)/9*z^4*v+F(97)/28*x^2*y*u*v-F(16)/113*x*y^2*u*v-F(47)/117*y^3*u*v+F(113)/56*x^2*z*u*v+F(19)/26*x*y*z*u*v+F(97)/62*y^2*z*u*v+F(60)/119*x*z^2*u*v+5139*y*z^2*u*v+F(89)/24*z^3*u*v-F(65)/33*x*y*u^2*v+3931*y^2*u^2*v-F(39)/64*x*z*u^2*v-F(63)/62*y*z*u^2*v+F(32)/107*z^2*u^2*v+1788*y*u^3*v-F(39)/8*z*u^3*v-F(52)/95*x^2*y*v^2+F(119)/67*x*y^2*v^2+10832*y^3*v^2+F(2)/15*x^2*z*v^2+10619*x*y*z*v^2-10134*y^2*z*v^2+9505*x*z^2*v^2-F(31)/16*y*z^2*v^2+F(121)/87*z^3*v^2-F(28)/47*x*y*u*v^2+F(91)/106*y^2*u*v^2+11378*x*z*u*v^2+F(13)/43*y*z*u*v^2-F(37)/124*z^2*u*v^2+11614*y*u^2*v^2+961*z*u^2*v^2-F(65)/86*x*y*v^3-F(71)/53*y^2*v^3-F(24)/101*x*z*v^3+F(51)/55*y*z*v^3+F(83)/25*z^2*v^3+F(107)/23*y*u*v^3-15412*z*u*v^3+6867*y*v^4+F(17)/54*z*v^4,F(122)/45*x^4*y-F(51)/4*x^3*y^2+F(10)/31*x^2*y^3+F(15)/121*x*y^4+F(121)/126*y^5+11526*x^4*z-8787*x^3*y*z-F(12)/97*x^2*y^2*z-15912*x*y^3*z-15895*y^4*z-7743*x^3*z^2+F(73)/102*x^2*y*z^2-12583*x*y^2*z^2+F(98)/43*y^3*z^2+F(16)/89*x^2*z^3-8678*x*y*z^3-424*y^2*z^3-F(73)/94*x*z^4-F(9)/92*y*z^4-F(92)/3*x^4*u-6912*x^3*y*u-1485*x^2*y^2*u-2615*x*y^3*u-10277*y^4*u+3350*x^3*z*u-F(106)/95*x^2*y*z*u-9321*x*y^2*z*u-9280*y^3*z*u-F(29)/38*x^2*z^2*u+F(99)/65*x*y*z^2*u+8276*y^2*z^2*u-F(7)/4*x*z^3*u-11616*y*z^3*u-F(2)/103*x^3*u^2+F(92)/91*x^2*y*u^2-11281*x*y^2*u^2-7185*y^3*u^2+13987*x^2*z*u^2-14193*x*y*z*u^2-13929*y^2*z*u^2-10582*x*z^2*u^2+F(116)/39*y*z^2*u^2-F(45)/47*x^2*u^3+F(116)/17*x*y*u^3-10463*y^2*u^3+F(37)/98*x*z*u^3+1317*y*z*u^3+6364*x*u^4-F(53)/25*y*u^4+F(77)/58*x^4*v-F(23)/104*x^3*y*v+F(99)/65*x^2*y^2*v+F(93)/86*x*y^3*v-11323*y^4*v-7168*x^3*z*v-F(48)/29*x^2*y*z*v+F(58)/107*x*y^2*z*v+F(99)/59*y^3*z*v-F(28)/103*x^2*z^2*v-F(47)/44*x*y*z^2*v-2013*y^2*z^2*v+F(116)/9*x*z^3*v-9910*y*z^3*v-F(113)/56*x^3*u*v+F(75)/124*x^2*y*u*v+15205*x*y^2*u*v+11416*y^3*u*v-F(60)/119*x^2*z*u*v-F(1)/78*x*y*z*u*v+10275*y^2*z*u*v-F(89)/24*x*z^2*u*v-103*y*z^2*u*v+F(39)/64*x^2*u^2*v-F(116)/51*x*y*u^2*v+8501*y^2*u^2*v-F(32)/107*x*z*u^2*v-4242*y*z*u^2*v+F(39)/8*x*u^3*v-F(77)/53*y*u^3*v-F(2)/15*x^3*v^2+F(65)/8*x^2*y*v^2-F(121)/92*x*y^2*v^2-15*y^3*v^2-9505*x^2*z*v^2-F(27)/35*x*y*z*v^2+13730*y^2*z*v^2-F(121)/87*x*z^2*v^2+F(122)/71*y*z^2*v^2-11378*x^2*u*v^2+F(11)/43*x*y*u*v^2-60*y^2*u*v^2+F(37)/124*x*z*u*v^2-F(119)/72*y*z*u*v^2-961*x*u^2*v^2-F(85)/74*y*u^2*v^2+F(24)/101*x^2*v^3+F(48)/5*x*y*v^3-7626*y^2*v^3-F(83)/25*x*z*v^3-F(61)/123*y*z*v^3+15412*x*u*v^3-F(96)/23*y*u*v^3-F(17)/54*x*v^4-F(1)/10*y*v^4,-5572*x^4*y-F(30)/103*x^3*y^2+F(38)/9*x^2*y^3+15913*x*y^4+F(28)/75*x^4*z+F(104)/47*x^3*y*z-13244*x^2*y^2*z-F(97)/55*x*y^3*z+F(97)/72*x^3*z^2-F(58)/63*x^2*y*z^2-F(21)/11*x*y^2*z^2-F(125)/82*x^2*z^3-F(44)/13*x*y*z^3-F(23)/104*x*z^4+F(96)/113*x^4*u-F(99)/10*x^3*y*u+4408*x^2*y^2*u+F(71)/84*x*y^3*u-F(124)/109*x^3*z*u-F(124)/5*x^2*y*z*u-F(111)/29*x*y^2*z*u-4355*x^2*z^2*u-F(106)/61*x*y*z^2*u-F(39)/121*x*z^3*u+6447*x^3*u^2-3113*x^2*y*u^2-4220*x*y^2*u^2-F(22)/101*x^2*z*u^2-9184*x*y*z*u^2-F(115)/103*x*z^2*u^2-F(105)/83*x^2*u^3-8463*x*y*u^3+F(29)/79*x*z*u^3+F(80)/31*x*u^4-F(111)/85*x^4*v-F(96)/103*x^3*y*v+F(35)/78*x^2*y^2*v-F(90)/73*x*y^3*v+F(74)/69*y^4*v-12944*x^3*z*v+F(92)/55*x^2*y*z*v-F(44)/9*x*y^2*z*v-F(112)/71*y^3*z*v+F(121)/74*x^2*z^2*v-166*x*y*z^2*v-6064*y^2*z^2*v-13423*x*z^3*v-2588*y*z^3*v-F(58)/21*z^4*v-F(17)/55*x^3*u*v+F(11)/2*x^2*y*u*v-2490*x*y^2*u*v+792*y^3*u*v+F(41)/102*x^2*z*u*v-F(23)/52*x*y*z*u*v-F(65)/46*y^2*z*u*v+F(77)/117*x*z^2*u*v-F(12)/77*y*z^2*u*v-F(97)/86*z^3*u*v-11973*x^2*u^2*v+700*x*y*u^2*v+F(13)/44*y^2*u^2*v-F(91)/72*x*z*u^2*v+3535*y*z*u^2*v+F(125)/102*z^2*u^2*v-6461*x*u^3*v+12751*y*u^3*v-8263*z*u^3*v+F(67)/75*u^4*v-F(95)/46*x^3*v^2+15753*x^2*y*v^2+F(106)/3*x*y^2*v^2-F(79)/80*y^3*v^2-F(41)/26*x^2*z*v^2+7066*x*y*z*v^2+F(95)/82*y^2*z*v^2-F(53)/111*x*z^2*v^2-F(58)/53*y*z^2*v^2+F(67)/75*z^3*v^2+F(20)/99*x^2*u*v^2+5855*x*y*u*v^2+7456*y^2*u*v^2-F(107)/90*x*z*u*v^2-F(2)/35*y*z*u*v^2-15326*z^2*u*v^2+4122*x*u^2*v^2+9045*y*u^2*v^2+F(67)/42*z*u^2*v^2+15503*u^3*v^2+F(91)/123*x^2*v^3-F(59)/80*x*y*v^3-1074*y^2*v^3-F(6)/73*x*z*v^3-F(107)/85*y*z*v^3-F(68)/65*z^2*v^3+F(8)/39*x*u*v^3-7347*y*u*v^3-F(111)/89*z*u*v^3+F(62)/65*u^2*v^3-12467*x*v^4-F(10)/49*y*v^4+F(26)/55*z*v^4-4021*u*v^4-F(67)/30*v^5,5572*x^3*y^2+F(30)/103*x^2*y^3-F(38)/9*x*y^4-15913*y^5-F(28)/75*x^3*y*z-F(104)/47*x^2*y^2*z+13244*x*y^3*z+F(97)/55*y^4*z-F(97)/72*x^2*y*z^2+F(58)/63*x*y^2*z^2+F(21)/11*y^3*z^2+F(125)/82*x*y*z^3+F(44)/13*y^2*z^3+F(23)/104*y*z^4-F(96)/113*x^3*y*u+F(99)/10*x^2*y^2*u-4408*x*y^3*u-F(71)/84*y^4*u+F(124)/109*x^2*y*z*u+F(124)/5*x*y^2*z*u+F(111)/29*y^3*z*u+4355*x*y*z^2*u+F(106)/61*y^2*z^2*u+F(39)/121*y*z^3*u-6447*x^2*y*u^2+3113*x*y^2*u^2+4220*y^3*u^2+F(22)/101*x*y*z*u^2+9184*y^2*z*u^2+F(115)/103*y*z^2*u^2+F(105)/83*x*y*u^3+8463*y^2*u^3-F(29)/79*y*z*u^3-F(80)/31*y*u^4+F(26)/85*x^3*y*v+14235*x^2*y^2*v-9312*x*y^3*v-4438*y^4*v+F(111)/92*x^3*z*v+F(89)/62*x^2*y*z*v-F(61)/12*x*y^2*z*v+F(121)/49*y^3*z*v-F(39)/112*x^2*z^2*v-238*x*y*z^2*v+F(61)/70*y^2*z^2*v+F(53)/123*x*z^3*v+9751*y*z^3*v-8559*z^4*v-F(103)/23*x^3*u*v+F(19)/31*x^2*y*u*v-5456*x*y^2*u*v+F(116)/39*y^3*u*v-F(19)/115*x^2*z*u*v-F(104)/89*x*y*z*u*v+15190*y^2*z*u*v+F(62)/73*x*z^2*u*v-F(63)/113*y*z^2*u*v+F(30)/97*z^3*u*v+F(85)/42*x^2*u^2*v-F(77)/114*x*y*u^2*v-F(39)/16*y^2*u^2*v-162*x*z*u^2*v-10830*y*z*u^2*v-F(52)/27*z^2*u^2*v-5804*x*u^3*v+15582*y*u^3*v-F(10)/39*z*u^3*v-F(106)/31*u^4*v+F(31)/107*x^3*v^2-F(1)/114*x^2*y*v^2+F(95)/27*x*y^2*v^2+3535*y^3*v^2+1833*x^2*z*v^2+F(108)/95*x*y*z*v^2-F(60)/119*y^2*z*v^2+12953*x*z^2*v^2-F(86)/85*y*z^2*v^2+925*z^3*v^2+F(83)/77*x^2*u*v^2+F(122)/37*x*y*u*v^2-F(69)/5*y^2*u*v^2+F(15)/61*x*z*u*v^2+5218*y*z*u*v^2+F(53)/31*z^2*u*v^2-F(11)/4*x*u^2*v^2-F(16)/45*y*u^2*v^2+F(122)/27*z*u^2*v^2-F(5)/59*u^3*v^2-F(109)/90*x^2*v^3-2148*x*y*v^3+3817*y^2*v^3+13395*x*z*v^3+F(57)/104*y*z*v^3-F(92)/71*z^2*v^3+F(115)/83*x*u*v^3+F(121)/119*y*u*v^3-14991*z*u*v^3-7159*u^2*v^3+13247*x*v^4-15166*y*v^4+3517*z*v^4+14608*u*v^4+F(117)/19*v^5,5572*x^3*y*z+F(30)/103*x^2*y^2*z-F(38)/9*x*y^3*z-15913*y^4*z-F(28)/75*x^3*z^2-F(104)/47*x^2*y*z^2+13244*x*y^2*z^2+F(97)/55*y^3*z^2-F(97)/72*x^2*z^3+F(58)/63*x*y*z^3+F(21)/11*y^2*z^3+F(125)/82*x*z^4+F(44)/13*y*z^4+F(23)/104*z^5-F(96)/113*x^3*z*u+F(99)/10*x^2*y*z*u-4408*x*y^2*z*u-F(71)/84*y^3*z*u+F(124)/109*x^2*z^2*u+F(124)/5*x*y*z^2*u+F(111)/29*y^2*z^2*u+4355*x*z^3*u+F(106)/61*y*z^3*u+F(39)/121*z^4*u-6447*x^2*z*u^2+3113*x*y*z*u^2+4220*y^2*z*u^2+F(22)/101*x*z^2*u^2+9184*y*z^2*u^2+F(115)/103*z^3*u^2+F(105)/83*x*z*u^3+8463*y*z*u^3-F(29)/79*z^2*u^3-F(80)/31*z*u^4+F(68)/81*x^3*y*v-F(39)/28*x^2*y^2*v-4371*x*y^3*v-F(107)/104*y^4*v-F(82)/51*x^3*z*v+F(53)/107*x^2*y*z*v-F(70)/41*x*y^2*z*v-4395*y^3*z*v-F(119)/76*x^2*z^2*v-4229*x*y*z^2*v+15730*y^2*z^2*v-4022*x*z^3*v-7054*y*z^3*v-7287*z^4*v+F(87)/124*x^3*u*v-F(1)/31*x^2*y*u*v-F(28)/101*x*y^2*u*v-85*y^3*u*v-12882*x^2*z*u*v+F(123)/110*x*y*z*u*v+F(58)/5*y^2*z*u*v-F(27)/58*x*z^2*u*v-F(10)/119*y*z^2*u*v-F(43)/108*z^3*u*v-5524*x^2*u^2*v-15519*x*y*u^2*v-1238*y^2*u^2*v+F(113)/67*x*z*u^2*v+10346*y*z*u^2*v-F(7)/24*z^2*u^2*v-F(79)/119*x*u^3*v+11367*y*u^3*v-F(77)/101*z*u^3*v+F(117)/86*u^4*v-5298*x^3*v^2+F(117)/37*x^2*y*v^2+F(31)/36*x*y^2*v^2+F(61)/101*y^3*v^2+F(71)/34*x^2*z*v^2+F(109)/13*x*y*z*v^2+F(43)/17*y^2*z*v^2-F(51)/118*x*z^2*v^2+F(55)/107*y*z^2*v^2+F(12)/101*z^3*v^2-12110*x^2*u*v^2+F(73)/7*x*y*u*v^2-F(73)/102*y^2*u*v^2+6014*x*z*u*v^2-F(55)/96*y*z*u*v^2+14028*z^2*u*v^2-F(80)/19*x*u^2*v^2+12634*y*u^2*v^2+10153*z*u^2*v^2-F(125)/116*u^3*v^2-F(71)/35*x^2*v^3-F(79)/113*x*y*v^3-F(79)/124*y^2*v^3+1735*x*z*v^3+F(20)/23*y*z*v^3-F(123)/23*z^2*v^3+F(1)/20*x*u*v^3-9877*y*u*v^3-F(73)/78*z*u*v^3-F(61)/84*u^2*v^3-11440*x*v^4+11452*y*v^4-F(55)/114*z*v^4+2090*u*v^4+12917*v^5,5572*x^3*y*u+F(30)/103*x^2*y^2*u-F(38)/9*x*y^3*u-15913*y^4*u-F(28)/75*x^3*z*u-F(104)/47*x^2*y*z*u+13244*x*y^2*z*u+F(97)/55*y^3*z*u-F(97)/72*x^2*z^2*u+F(58)/63*x*y*z^2*u+F(21)/11*y^2*z^2*u+F(125)/82*x*z^3*u+F(44)/13*y*z^3*u+F(23)/104*z^4*u-F(96)/113*x^3*u^2+F(99)/10*x^2*y*u^2-4408*x*y^2*u^2-F(71)/84*y^3*u^2+F(124)/109*x^2*z*u^2+F(124)/5*x*y*z*u^2+F(111)/29*y^2*z*u^2+4355*x*z^2*u^2+F(106)/61*y*z^2*u^2+F(39)/121*z^3*u^2-6447*x^2*u^3+3113*x*y*u^3+4220*y^2*u^3+F(22)/101*x*z*u^3+9184*y*z*u^3+F(115)/103*z^2*u^3+F(105)/83*x*u^4+8463*y*u^4-F(29)/79*z*u^4-F(80)/31*u^5+F(53)/108*x^3*y*v+F(119)/73*x^2*y^2*v-7204*x*y^3*v-15791*y^4*v+F(45)/74*x^3*z*v+F(55)/31*x^2*y*z*v+9057*x*y^2*z*v+F(7)/120*y^3*z*v-F(29)/51*x^2*z^2*v+9962*x*y*z^2*v-1223*y^2*z^2*v+F(8)/55*x*z^3*v+F(115)/116*y*z^3*v+F(71)/105*z^4*v+13096*x^3*u*v-F(45)/4*x^2*y*u*v-F(23)/48*x*y^2*u*v+559*y^3*u*v-F(103)/31*x^2*z*u*v+2185*x*y*z*u*v-F(50)/77*y^2*z*u*v-F(82)/45*x*z^2*u*v-F(122)/105*y*z^2*u*v+F(121)/58*z^3*u*v-9187*x^2*u^2*v-F(77)/57*x*y*u^2*v-F(106)/39*y^2*u^2*v-F(79)/109*x*z*u^2*v-F(99)/61*y*z*u^2*v-5419*z^2*u^2*v+F(43)/19*x*u^3*v-F(83)/2*y*u^3*v+F(81)/70*z*u^3*v+11411*u^4*v-15019*x^3*v^2+F(97)/113*x^2*y*v^2+4470*x*y^2*v^2+F(113)/70*y^3*v^2+F(64)/81*x^2*z*v^2-F(104)/11*x*y*z*v^2-F(63)/82*y^2*z*v^2+1038*x*z^2*v^2+F(43)/101*y*z^2*v^2+F(77)/25*z^3*v^2+F(7)/18*x^2*u*v^2+F(13)/121*x*y*u*v^2+F(115)/34*y^2*u*v^2+F(93)/35*x*z*u*v^2-8356*y*z*u*v^2+11133*z^2*u*v^2-F(123)/118*x*u^2*v^2-F(89)/17*y*u^2*v^2-1722*z*u^2*v^2-F(89)/98*u^3*v^2+F(44)/65*x^2*v^3+5801*x*y*v^3-4748*y^2*v^3-F(83)/41*x*z*v^3-F(4)/59*y*z*v^3-F(83)/9*z^2*v^3+2707*x*u*v^3-F(53)/5*y*u*v^3+F(48)/79*z*u*v^3-F(73)/89*u^2*v^3-7042*x*v^4+7958*y*v^4+F(83)/118*z*v^4-F(11)/16*u*v^4+4731*v^5,F(53)/108*x^4*y+F(119)/73*x^3*y^2-7204*x^2*y^3-15791*x*y^4+F(45)/74*x^4*z+F(55)/31*x^3*y*z+9057*x^2*y^2*z+F(7)/120*x*y^3*z-F(29)/51*x^3*z^2+9962*x^2*y*z^2-1223*x*y^2*z^2+F(8)/55*x^2*z^3+F(115)/116*x*y*z^3+F(71)/105*x*z^4-7229*x^4*u-F(93)/65*x^3*y*u+5178*x^2*y^2*u-F(119)/32*x*y^3*u+F(74)/69*y^4*u-F(9)/11*x^3*z*u-F(48)/71*x^2*y*z*u+F(13)/56*x*y^2*z*u-F(112)/71*y^3*z*u-11980*x^2*z^2*u+F(41)/100*x*y*z^2*u-6064*y^2*z^2*u+F(52)/87*x*z^3*u-2588*y*z^3*u-F(58)/21*z^4*u-F(105)/4*x^3*u^2+3091*x^2*y*u^2-F(104)/29*x*y^2*u^2+792*y^3*u^2+4123*x^2*z*u^2-6487*x*y*z*u^2-F(65)/46*y^2*z*u^2-F(17)/16*x*z^2*u^2-F(12)/77*y*z^2*u^2-F(97)/86*z^3*u^2-F(117)/28*x^2*u^3+F(84)/73*x*y*u^3+F(13)/44*y^2*u^3+F(32)/57*x*z*u^3+3535*y*z*u^3+F(125)/102*z^2*u^3-F(83)/84*x*u^4+12751*y*u^4-8263*z*u^4+F(67)/75*u^5-15019*x^4*v+F(97)/113*x^3*y*v+4470*x^2*y^2*v+F(113)/70*x*y^3*v+F(64)/81*x^3*z*v-F(104)/11*x^2*y*z*v-F(63)/82*x*y^2*z*v+1038*x^2*z^2*v+F(43)/101*x*y*z^2*v+F(77)/25*x*z^3*v+F(2)/91*x^3*u*v+13638*x^2*y*u*v+4116*x*y^2*u*v-F(79)/80*y^3*u*v+F(60)/23*x^2*z*u*v-F(5)/124*x*y*z*u*v+F(95)/82*y^2*z*u*v+F(76)/77*x*z^2*u*v-F(58)/53*y*z^2*u*v+F(67)/75*z^3*u*v-F(54)/61*x^2*u^2*v+F(20)/83*x*y*u^2*v+7456*y^2*u^2*v+765*x*z*u^2*v-F(2)/35*y*z*u^2*v-15326*z^2*u^2*v+F(24)/85*x*u^3*v+9045*y*u^3*v+F(67)/42*z*u^3*v+15503*u^4*v+F(44)/65*x^3*v^2+5801*x^2*y*v^2-4748*x*y^2*v^2-F(83)/41*x^2*z*v^2-F(4)/59*x*y*z*v^2-F(83)/9*x*z^2*v^2-F(83)/26*x^2*u*v^2-1211*x*y*u*v^2-1074*y^2*u*v^2-10040*x*z*u*v^2-F(107)/85*y*z*u*v^2-F(68)/65*z^2*u*v^2+1004*x*u^2*v^2-7347*y*u^2*v^2-F(111)/89*z*u^2*v^2+F(62)/65*u^3*v^2-7042*x^2*v^3+7958*x*y*v^3+F(83)/118*x*z*v^3+F(43)/123*x*u*v^3-F(10)/49*y*u*v^3+F(26)/55*z*u*v^3-4021*u^2*v^3+4731*x*v^4-F(67)/30*u*v^4,-F(53)/108*x^3*y^2-F(119)/73*x^2*y^3+7204*x*y^4+15791*y^5-F(45)/74*x^3*y*z-F(55)/31*x^2*y^2*z-9057*x*y^3*z-F(7)/120*y^4*z+F(29)/51*x^2*y*z^2-9962*x*y^2*z^2+1223*y^3*z^2-F(8)/55*x*y*z^3-F(115)/116*y^2*z^3-F(71)/105*y*z^4+7228*x^3*y*u-9747*x^2*y^2*u+F(92)/109*x*y^3*u+F(51)/32*y^4*u+F(111)/92*x^3*z*u-F(83)/91*x^2*y*z*u-F(25)/112*x*y^2*z*u-F(88)/105*y^3*z*u-F(39)/112*x^2*z^2*u-1658*x*y*z^2*u+F(61)/30*y^2*z^2*u+F(53)/123*x*z^3*u-F(19)/77*y*z^3*u-8559*z^4*u-F(103)/23*x^3*u^2+F(31)/10*x^2*y*u^2-F(22)/103*x*y^2*u^2+F(74)/13*y^3*u^2-F(19)/115*x^2*z*u^2+2519*x*y*z*u^2+F(93)/86*y^2*z*u^2+F(62)/73*x*z^2*u^2+F(83)/117*y*z^2*u^2+F(30)/97*z^3*u^2+F(85)/42*x^2*u^3+F(119)/55*x*y*u^3-13957*y^2*u^3-162*x*z*u^3-5804*y*z*u^3-F(52)/27*z^2*u^3-5804*x*u^4-F(40)/23*y*u^4-F(10)/39*z*u^4-F(106)/31*u^5+15019*x^3*y*v-F(97)/113*x^2*y^2*v-4470*x*y^3*v-F(113)/70*y^4*v-F(64)/81*x^2*y*z*v+F(104)/11*x*y^2*z*v+F(63)/82*y^3*z*v-1038*x*y*z^2*v-F(43)/101*y^2*z^2*v-F(77)/25*y*z^3*v+F(31)/107*x^3*u*v-8232*x^2*y*u*v-9730*x*y^2*u*v+10118*y^3*u*v+1833*x^2*z*u*v-F(115)/44*x*y*z*u*v+F(50)/89*y^2*z*u*v+12953*x*z^2*u*v-6994*y*z^2*u*v+925*z^3*u*v+F(83)/77*x^2*u^2*v-F(104)/25*x*y*u^2*v-F(69)/52*y^2*u^2*v+F(15)/61*x*z*u^2*v+6940*y*z*u^2*v+F(53)/31*z^2*u^2*v-F(11)/4*x*u^3*v-F(17)/48*y*u^3*v+F(122)/27*z*u^3*v-F(5)/59*u^4*v-F(44)/65*x^2*y*v^2-5801*x*y^2*v^2+4748*y^3*v^2+F(83)/41*x*y*z*v^2+F(4)/59*y^2*z*v^2+F(83)/9*y*z^2*v^2-F(109)/90*x^2*u*v^2+F(87)/112*x*y*u*v^2+F(125)/102*y^2*u*v^2+13395*x*z*u*v^2-2091*y*z*u*v^2-F(92)/71*z^2*u*v^2+F(115)/83*x*u^2*v^2+F(47)/93*y*u^2*v^2-14991*z*u^2*v^2-7159*u^3*v^2+7042*x*y*v^3-7958*y^2*v^3-F(83)/118*y*z*v^3+13247*x*u*v^3-9167*y*u*v^3+3517*z*u*v^3+14608*u^2*v^3-4731*y*v^4+F(117)/19*u*v^4,-F(53)/108*x^3*y*z-F(119)/73*x^2*y^2*z+7204*x*y^3*z+15791*y^4*z-F(45)/74*x^3*z^2-F(55)/31*x^2*y*z^2-9057*x*y^2*z^2-F(7)/120*y^3*z^2+F(29)/51*x^2*z^3-9962*x*y*z^3+1223*y^2*z^3-F(8)/55*x*z^4-F(115)/116*y*z^4-F(71)/105*z^5+F(68)/81*x^3*y*u-F(39)/28*x^2*y^2*u-4371*x*y^3*u-F(107)/104*y^4*u+8857*x^3*z*u-63*x^2*y*z*u-F(13)/90*x*y^2*z*u-4954*y^3*z*u-F(31)/21*x^2*z^2*u-F(79)/5*x*y*z^2*u-7951*y^2*z^2*u-5442*x*z^3*u-F(49)/102*y*z^3*u+F(77)/83*z^4*u+F(87)/124*x^3*u^2-F(1)/31*x^2*y*u^2-F(28)/101*x*y^2*u^2-85*y^3*u^2-F(97)/26*x^2*z*u^2-5411*x*y*z*u^2+F(6)/115*y^2*z*u^2+F(69)/110*x*z^2*u^2+3276*y*z^2*u^2-F(6)/17*z^3*u^2-5524*x^2*u^3-15519*x*y*u^3-1238*y^2*u^3-8369*x*z*u^3-5608*y*z*u^3+F(77)/52*z^2*u^3-F(79)/119*x*u^4+11367*y*u^4+F(91)/25*z*u^4+F(117)/86*u^5+15019*x^3*z*v-F(97)/113*x^2*y*z*v-4470*x*y^2*z*v-F(113)/70*y^3*z*v-F(64)/81*x^2*z^2*v+F(104)/11*x*y*z^2*v+F(63)/82*y^2*z^2*v-1038*x*z^3*v-F(43)/101*y*z^3*v-F(77)/25*z^4*v-5298*x^3*u*v+F(117)/37*x^2*y*u*v+F(31)/36*x*y^2*u*v+F(61)/101*y^3*u*v+9829*x^2*z*u*v+F(57)/29*x*y*z*u*v-F(29)/34*y^2*z*u*v-F(95)/116*x*z^2*u*v+F(74)/55*y*z^2*u*v-6065*z^3*u*v-12110*x^2*u^2*v+F(73)/7*x*y*u^2*v-F(73)/102*y^2*u^2*v-13776*x*z*u^2*v+2847*y*z*u^2*v+F(38)/65*z^2*u^2*v-F(80)/19*x*u^3*v+12634*y*u^3*v+F(29)/96*z*u^3*v-F(125)/116*u^4*v-F(44)/65*x^2*z*v^2-5801*x*y*z*v^2+4748*y^2*z*v^2+F(83)/41*x*z^2*v^2+F(4)/59*y*z^2*v^2+F(83)/9*z^3*v^2-F(71)/35*x^2*u*v^2-F(79)/113*x*y*u*v^2-F(79)/124*y^2*u*v^2-F(85)/33*x*z*u*v^2-F(69)/91*y*z*u*v^2-10429*z^2*u*v^2+F(1)/20*x*u^2*v^2-9877*y*u^2*v^2-F(8)/109*z*u^2*v^2-F(61)/84*u^3*v^2+7042*x*z*v^3-7958*y*z*v^3-F(83)/118*z^2*v^3-11440*x*u*v^3+11452*y*u*v^3+F(112)/33*z*u*v^3+2090*u^2*v^3-4731*z*v^4+12917*u*v^4,F(68)/81*x^4*y-F(39)/28*x^3*y^2-4371*x^2*y^3-F(107)/104*x*y^4+F(79)/59*x^4*z-F(27)/115*x^3*y*z+F(95)/4*x^2*y^2*z-F(34)/97*x*y^3*z+F(74)/69*y^4*z-F(29)/73*x^3*z^2+12059*x^2*y*z^2+F(79)/26*x*y^2*z^2-F(112)/71*y^3*z^2-F(117)/101*x^2*z^3+F(117)/31*x*y*z^3-6064*y^2*z^3+11281*x*z^4-2588*y*z^4-F(58)/21*z^5+F(87)/124*x^4*u-F(1)/31*x^3*y*u-F(28)/101*x^2*y^2*u-85*x*y^3*u+F(48)/101*x^3*z*u-F(125)/69*x^2*y*z*u-F(52)/31*x*y^2*z*u+792*y^3*z*u+10231*x^2*z^2*u-F(119)/10*x*y*z^2*u-F(65)/46*y^2*z^2*u+10231*x*z^3*u-F(12)/77*y*z^3*u-F(97)/86*z^4*u-5524*x^3*u^2-15519*x^2*y*u^2-1238*x*y^2*u^2+F(65)/12*x^2*z*u^2+F(125)/84*x*y*z*u^2+F(13)/44*y^2*z*u^2-F(14)/9*x*z^2*u^2+3535*y*z^2*u^2+F(125)/102*z^3*u^2-F(79)/119*x^2*u^3+11367*x*y*u^3+3674*x*z*u^3+12751*y*z*u^3-8263*z^2*u^3+F(117)/86*x*u^4+F(67)/75*z*u^4-5298*x^4*v+F(117)/37*x^3*y*v+F(31)/36*x^2*y^2*v+F(61)/101*x*y^3*v-F(80)/79*x^3*z*v+5918*x^2*y*z*v-15644*x*y^2*z*v-F(79)/80*y^3*z*v+F(97)/14*x^2*z^2*v-707*x*y*z^2*v+F(95)/82*y^2*z^2*v+168*x*z^3*v-F(58)/53*y*z^3*v+F(67)/75*z^4*v-12110*x^3*u*v+F(73)/7*x^2*y*u*v-F(73)/102*x*y^2*u*v-F(37)/93*x^2*z*u*v-4476*x*y*z*u*v+7456*y^2*z*u*v+F(109)/31*x*z^2*u*v-F(2)/35*y*z^2*u*v-15326*z^3*u*v-F(80)/19*x^2*u^2*v+12634*x*y*u^2*v+14275*x*z*u^2*v+9045*y*z*u^2*v+F(67)/42*z^2*u^2*v-F(125)/116*x*u^3*v+15503*z*u^3*v-F(71)/35*x^3*v^2-F(79)/113*x^2*y*v^2-F(79)/124*x*y^2*v^2-F(34)/77*x^2*z*v^2-F(25)/74*x*y*z*v^2-1074*y^2*z*v^2+11522*x*z^2*v^2-F(107)/85*y*z^2*v^2-F(68)/65*z^3*v^2+F(1)/20*x^2*u*v^2-9877*x*y*u*v^2-F(19)/26*x*z*u*v^2-7347*y*z*u*v^2-F(111)/89*z^2*u*v^2-F(61)/84*x*u^2*v^2+F(62)/65*z*u^2*v^2-11440*x^2*v^3+11452*x*y*v^3+5773*x*z*v^3-F(10)/49*y*z*v^3+F(26)/55*z^2*v^3+2090*x*u*v^3-4021*z*u*v^3+12917*x*v^4-F(67)/30*z*v^4,-F(68)/81*x^3*y^2+F(39)/28*x^2*y^3+4371*x*y^4+F(107)/104*y^5-1629*x^3*y*z-9684*x^2*y^2*z+3174*x*y^3*z-43*y^4*z+F(111)/92*x^3*z^2+F(22)/39*x^2*y*z^2-F(104)/41*x*y^2*z^2+F(68)/77*y^3*z^2-F(39)/112*x^2*z^3+F(11)/93*x*y*z^3+11168*y^2*z^3+F(53)/123*x*z^4-F(63)/92*y*z^4-8559*z^5-F(87)/124*x^3*y*u+F(1)/31*x^2*y^2*u+F(28)/101*x*y^3*u+85*y^4*u-F(103)/23*x^3*z*u-F(55)/64*x^2*y*z*u-F(87)/92*x*y^2*z*u-F(103)/69*y^3*z*u-F(19)/115*x^2*z^2*u-3589*x*y*z^2*u-671*y^2*z^2*u+F(62)/73*x*z^3*u+F(61)/12*y*z^3*u+F(30)/97*z^4*u+5524*x^2*y*u^2+15519*x*y^2*u^2+1238*y^3*u^2+F(85)/42*x^2*z*u^2+228*x*y*z*u^2-F(81)/23*y^2*z*u^2-162*x*z^2*u^2+F(37)/64*y*z^2*u^2-F(52)/27*z^3*u^2+F(79)/119*x*y*u^3-11367*y^2*u^3-5804*x*z*u^3+F(81)/47*y*z*u^3-F(10)/39*z^2*u^3-F(117)/86*y*u^4-F(106)/31*z*u^4+5298*x^3*y*v-F(117)/37*x^2*y^2*v-F(31)/36*x*y^3*v-F(61)/101*y^4*v+F(31)/107*x^3*z*v-F(97)/62*x^2*y*z*v-2010*x*y^2*z*v-F(41)/106*y^3*z*v+1833*x^2*z^2*v+13988*x*y*z^2*v+F(27)/124*y^2*z^2*v+12953*x*z^3*v-F(119)/69*y*z^3*v+925*z^4*v+12110*x^2*y*u*v-F(73)/7*x*y^2*u*v+F(73)/102*y^3*u*v+F(83)/77*x^2*z*u*v+F(10)/21*x*y*z*u*v+F(85)/49*y^2*z*u*v+F(15)/61*x*z^2*u*v-F(61)/69*y*z^2*u*v+F(53)/31*z^3*u*v+F(80)/19*x*y*u^2*v-12634*y^2*u^2*v-F(11)/4*x*z*u^2*v-F(21)/32*y*z*u^2*v+F(122)/27*z^2*u^2*v+F(125)/116*y*u^3*v-F(5)/59*z*u^3*v+F(71)/35*x^2*y*v^2+F(79)/113*x*y^2*v^2+F(79)/124*y^3*v^2-F(109)/90*x^2*z*v^2-3883*x*y*z*v^2-10093*y^2*z*v^2+13395*x*z^2*v^2+8338*y*z^2*v^2-F(92)/71*z^3*v^2-F(1)/20*x*y*u*v^2+9877*y^2*u*v^2+F(115)/83*x*z*u*v^2+F(7)/23*y*z*u*v^2-14991*z^2*u*v^2+F(61)/84*y*u^2*v^2-7159*z*u^2*v^2+11440*x*y*v^3-11452*y^2*v^3+13247*x*z*v^3+F(60)/113*y*z*v^3+3517*z^2*v^3-2090*y*u*v^3+14608*z*u*v^3-12917*y*v^4+F(117)/19*z*v^4,x^4*y+F(87)/25*x^3*y^2+F(27)/20*x^2*y^3+F(17)/8*x*y^4-F(74)/69*y^5-F(111)/92*x^4*z+F(40)/97*x^3*y*z-13617*x^2*y^2*z-723*x*y^3*z+F(112)/71*y^4*z+F(39)/112*x^3*z^2+13638*x^2*y*z^2+F(122)/81*x*y^2*z^2+6064*y^3*z^2-F(53)/123*x^2*z^3+F(55)/61*x*y*z^3+2588*y^2*z^3+8559*x*z^4+F(58)/21*y*z^4+F(103)/23*x^4*u+11220*x^3*y*u-10545*x^2*y^2*u+F(96)/35*x*y^3*u-792*y^4*u+F(19)/115*x^3*z*u-F(125)/53*x^2*y*z*u+3882*x*y^2*z*u+F(65)/46*y^3*z*u-F(62)/73*x^2*z^2*u-12902*x*y*z^2*u+F(12)/77*y^2*z^2*u-F(30)/97*x*z^3*u+F(97)/86*y*z^3*u-F(85)/42*x^3*u^2-13563*x^2*y*u^2+F(86)/83*x*y^2*u^2-F(13)/44*y^3*u^2+162*x^2*z*u^2+F(51)/109*x*y*z*u^2-3535*y^2*z*u^2+F(52)/27*x*z^2*u^2-F(125)/102*y*z^2*u^2+5804*x^2*u^3-9121*x*y*u^3-12751*y^2*u^3+F(10)/39*x*z*u^3+8263*y*z*u^3+F(106)/31*x*u^4-F(67)/75*y*u^4-F(31)/107*x^4*v-5830*x^3*y*v-3908*x^2*y^2*v-14234*x*y^3*v+F(79)/80*y^4*v-1833*x^3*z*v+15711*x^2*y*z*v+F(5)/46*x*y^2*z*v-F(95)/82*y^3*z*v-12953*x^2*z^2*v-F(29)/42*x*y*z^2*v+F(58)/53*y^2*z^2*v-925*x*z^3*v-F(67)/75*y*z^3*v-F(83)/77*x^3*u*v+6634*x^2*y*u*v+F(73)/115*x*y^2*u*v-7456*y^3*u*v-F(15)/61*x^2*z*u*v-7705*x*y*z*u*v+F(2)/35*y^2*z*u*v-F(53)/31*x*z^2*u*v+15326*y*z^2*u*v+F(11)/4*x^2*u^2*v+F(41)/25*x*y*u^2*v-9045*y^2*u^2*v-F(122)/27*x*z*u^2*v-F(67)/42*y*z*u^2*v+F(5)/59*x*u^3*v-15503*y*u^3*v+F(109)/90*x^3*v^2+15932*x^2*y*v^2+F(93)/110*x*y^2*v^2+1074*y^3*v^2-13395*x^2*z*v^2-F(102)/29*x*y*z*v^2+F(107)/85*y^2*z*v^2+F(92)/71*x*z^2*v^2+F(68)/65*y*z^2*v^2-F(115)/83*x^2*u*v^2+14819*x*y*u*v^2+7347*y^2*u*v^2+14991*x*z*u*v^2+F(111)/89*y*z*u*v^2+7159*x*u^2*v^2-F(62)/65*y*u^2*v^2-13247*x^2*v^3+F(97)/22*x*y*v^3+F(10)/49*y^2*v^3-3517*x*z*v^3-F(26)/55*y*z*v^3-14608*x*u*v^3+4021*y*u*v^3-F(117)/19*x*v^4+F(67)/30*y*v^4,-F(55)/32*x^3*y-F(72)/13*x^2*y^2-10928*x*y^3-10990*y^4-F(2)/33*x^3*z+F(123)/88*x^2*y*z+10799*x*y^2*z-F(21)/116*y^3*z-F(71)/33*x^2*z^2+F(107)/34*x*y*z^2-F(6)/29*y^2*z^2+F(55)/89*x*z^3+F(43)/6*y*z^3-F(80)/47*z^4+F(117)/76*x^3*u-1399*x^2*y*u+F(91)/86*x*y^2*u+3527*y^3*u-15111*x^2*z*u+3389*x*y*z*u-8976*y^2*z*u-F(15)/104*x*z^2*u-F(29)/53*y*z^2*u+F(34)/93*z^3*u+F(104)/33*x^2*u^2+F(49)/67*x*y*u^2-F(41)/86*y^2*u^2-4436*x*z*u^2+F(78)/107*y*z*u^2-14752*z^2*u^2+F(65)/18*x*u^3+F(97)/17*y*u^3-F(124)/111*z*u^3+8605*u^4-F(63)/44*x^3*v+6756*x^2*y*v+F(122)/31*x*y^2*v-F(12)/95*y^3*v-F(69)/10*x^2*z*v-F(1)/45*x*y*z*v-F(7)/75*y^2*z*v+5139*x*z^2*v+F(52)/111*y*z^2*v+F(23)/9*z^3*v-10595*x^2*u*v-F(115)/13*x*y*u*v+F(107)/8*y^2*u*v-F(5)/18*x*z*u*v-F(29)/99*y*z*u*v+F(71)/123*z^2*u*v-F(20)/81*x*u^2*v+12041*y*u^2*v-15145*z*u^2*v+366*u^3*v-F(118)/53*x^2*v^2+3812*x*y*v^2+F(64)/41*y^2*v^2+883*x*z*v^2-F(79)/26*y*z*v^2-860*z^2*v^2-7923*x*u*v^2-13093*y*u*v^2+12706*z*u*v^2-2801*u^2*v^2-F(73)/60*x*v^3-10127*y*v^3-F(5)/17*z*v^3+6169*u*v^3+F(83)/105*v^4,2121*x^4+F(81)/34*x^3*y-F(7)/94*x^2*y^2+F(70)/83*x*y^3-F(13)/58*y^4+F(97)/61*x^3*z+2726*x^2*y*z-F(34)/15*x*y^2*z+947*y^3*z+1382*x^2*z^2+F(93)/83*x*y*z^2+9074*y^2*z^2+2506*x*z^3-1154*y*z^3+F(31)/118*z^4-F(109)/23*x^3*u+F(81)/10*x^2*y*u-F(89)/69*x*y^2*u-F(119)/5*y^3*u-6445*x^2*z*u-721*x*y*z*u-F(107)/120*y^2*z*u-4335*x*z^2*u-F(108)/73*y*z^2*u-F(92)/57*z^3*u+F(53)/80*x^2*u^2+F(81)/97*x*y*u^2+F(5)/51*y^2*u^2-13956*x*z*u^2+F(8)/111*y*z*u^2+F(1)/53*z^2*u^2+F(58)/25*x*u^3+F(25)/19*y*u^3+2311*z*u^3-11940*u^4+F(28)/57*x^3*v-F(83)/23*x^2*y*v+11389*x*y^2*v-F(27)/64*y^3*v-10830*x^2*z*v-F(25)/101*x*y*z*v-F(23)/82*y^2*z*v-F(70)/37*x*z^2*v+F(104)/115*y*z^2*v-F(31)/16*z^3*v-9344*x^2*u*v+F(93)/35*x*y*u*v+F(106)/81*y^2*u*v+F(17)/39*x*z*u*v-F(87)/101*y*z*u*v-F(64)/29*z^2*u*v+9634*x*u^2*v-F(103)/48*y*u^2*v+F(118)/71*z*u^2*v-10011*u^3*v+13742*x^2*v^2+10598*x*y*v^2-9239*y^2*v^2+14876*x*z*v^2+F(95)/36*y*z*v^2+15263*z^2*v^2+4401*x*u*v^2+1013*y*u*v^2-F(53)/28*z*u*v^2+F(83)/32*u^2*v^2+F(23)/68*x*v^3+F(116)/75*y*v^3-F(87)/49*z*v^3+F(15)/94*u*v^3+101*v^4]))
end


@doc raw"""
    quintic_elliptic_scroll()

Return a smooth ruled surface in $\mathbb P^4$ with degree `5` and sectional genus `1`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function  quintic_elliptic_scroll()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-11546*x^3+9484*x^2*y+2143*x*y^2-13251*y^3-F(71)/42*x^2*z+F(110)/117*x*y*z-13130*y^2*z-5109*x*z^2-F(6)/59*y*z^2-8900*z^3+14290*x^2*u+F(87)/28*x*y*u-F(67)/20*y^2*u+F(111)/59*x*z*u+F(97)/24*y*z*u-3390*z^2*u+14572*x*u^2-F(1)/2*y*u^2+5604*z*u^2-12534*x^2*v-8121*x*y*v-F(83)/104*y^2*v+2572*x*z*v+2648*y*z*v+6137*z^2*v-7958*x*u*v-42*y*u*v+F(117)/11*z*u*v-8911*x*v^2+F(25)/12*y*v^2+F(43)/16*z*v^2,9224*x^3+F(113)/93*x^2*y+F(32)/89*x*y^2-14415*x^2*z+13096*x*y*z+11534*y^2*z+F(5)/38*x*z^2+161*y*z^2+F(93)/49*z^3+F(89)/109*x^2*u+12762*x*y*u+13251*y^2*u-4116*x*z*u-F(49)/65*y*z*u-F(58)/45*z^2*u+F(123)/64*x*u^2+F(67)/20*y*u^2-F(121)/61*z*u^2+F(1)/2*u^3+12741*x^2*v-F(36)/35*x*y*v+F(68)/103*x*z*v-F(116)/27*y*z*v-5030*z^2*v-F(33)/43*x*u*v+F(83)/104*y*u*v+8249*z*u*v+42*u^2*v+10267*x*v^2-F(19)/18*z*v^2-F(25)/12*u*v^2,-9224*x^2*y-F(113)/93*x*y^2-F(32)/89*y^3-13874*x^2*z+F(64)/67*x*y*z-F(125)/19*y^2*z+8558*x*z^2+F(21)/52*y*z^2-F(85)/69*z^3+11546*x^2*u+2255*x*y*u-F(9)/88*y^2*u-3215*x*z*u-4087*y*z*u+F(5)/23*z^2*u-14290*x*u^2+1923*y*u^2+F(89)/114*z*u^2-14572*u^3-12741*x*y*v+F(36)/35*y^2*v-9170*x*z*v-92*y*z*v-F(9)/77*z^2*v+12534*x*u*v+F(63)/47*y*u*v-13678*z*u*v+7958*u^2*v-10267*y*v^2-6256*z*v^2+8911*u*v^2,13874*x^3+F(79)/102*x^2*y-F(88)/75*x*y^2-11534*y^3-8558*x^2*z-11301*x*y*z-161*y^2*z+F(85)/69*x*z^2-F(93)/49*y*z^2-F(76)/13*x^2*u-F(89)/117*x*y*u+F(115)/71*y^2*u-F(34)/49*x*z*u+F(51)/80*y*z*u+8900*z^2*u+14233*x*u^2-F(98)/37*y*u^2+3390*z*u^2-5604*u^3+9170*x^2*v+5682*x*y*v+F(116)/27*y^2*v+F(9)/77*x*z*v+5030*y*z*v+11106*x*u*v+F(94)/91*y*u*v-6137*z*u*v-F(117)/11*u^2*v+6256*x*v^2+F(19)/18*y*v^2-F(43)/16*u*v^2,-13903*x^3+11973*x^2*y+F(101)/76*x*y^2+F(103)/90*x^2*z+F(100)/11*x*y*z-5302*x*z^2+12099*x^2*u-15900*x*y*u+F(66)/107*y^2*u+F(57)/112*x*z*u+F(101)/92*y*z*u+F(15)/79*z^2*u+F(6)/25*x*u^2-F(31)/105*y*u^2+9615*z*u^2+11827*u^3+9771*x^2*v-F(12)/17*x*y*v-11534*y^2*v+F(81)/2*x*z*v-161*y*z*v-F(93)/49*z^2*v-F(116)/89*x*u*v+15306*y*u*v+8845*z*u*v+9969*u^2*v-2896*x*v^2+F(116)/27*y*v^2+5030*z*v^2+F(52)/61*u*v^2+F(19)/18*v^3,13903*x^2*y-11973*x*y^2-F(101)/76*y^3-F(103)/90*x*y*z-F(100)/11*y^2*z+5302*y*z^2-x^2*u+F(124)/85*x*y*u-9612*y^2*u-F(43)/26*x*z*u-F(2)/13*y*z*u-F(47)/40*z^2*u+10411*x*u^2-F(61)/42*y*u^2+F(18)/73*z*u^2+F(12)/125*u^3+13874*x^2*v-F(17)/40*x*y*v+4541*y^2*v-8558*x*z*v+F(42)/55*y*z*v+F(85)/69*z^2*v+9246*x*u*v-F(43)/110*y*u*v+106*z*u*v-F(86)/93*u^2*v+9170*x*v^2+8578*y*v^2+F(9)/77*z*v^2-F(85)/78*u*v^2+6256*v^3,13903*x^2*z-11973*x*y*z-F(101)/76*y^2*z-F(103)/90*x*z^2-F(100)/11*y*z^2+5302*z^3+F(37)/98*x^2*u-15200*x*y*u-7292*y^2*u-10910*x*z*u-F(72)/101*y*z*u+8520*z^2*u-F(48)/121*x*u^2+9435*y*u^2+F(53)/32*z*u^2-9573*u^3-9224*x^2*v-F(113)/93*x*y*v-F(32)/89*y^2*v+F(9)/62*x*z*v-F(100)/23*y*z*v+15113*z^2*v+F(116)/89*x*u*v+815*y*u*v-F(41)/88*z*u*v-F(110)/43*u^2*v-12741*x*v^2+F(36)/35*y*v^2-F(69)/49*z*v^2-3345*u*v^2-10267*v^3,F(37)/98*x^3-15200*x^2*y-7292*x*y^2+F(112)/27*x^2*z-14317*x*y*z+F(66)/107*y^2*z+4236*x*z^2+F(101)/92*y*z^2+F(15)/79*z^3-F(48)/121*x^2*u+9435*x*y*u-7476*x*z*u-F(31)/105*y*z*u+9615*z^2*u-9573*x*u^2+11827*z*u^2-9581*x^2*v+13577*x*y*v+13251*y^2*v+F(87)/53*x*z*v+48*y*z*v-F(114)/11*z^2*v-361*x*u*v+F(67)/20*y*u*v+14687*z*u*v+F(1)/2*u^2*v-F(106)/125*x*v^2+F(83)/104*y*v^2+F(37)/100*z*v^2+42*u*v^2-F(25)/12*v^3,-F(37)/98*x^2*y+15200*x*y^2+7292*y^3-x^2*z-F(53)/17*x*y*z+F(37)/20*y^2*z-F(43)/26*x*z^2+F(66)/67*y*z^2-F(47)/40*z^3+F(48)/121*x*y*u-9435*y^2*u+10411*x*z*u-9191*y*z*u+F(18)/73*z^2*u+9573*y*u^2+F(12)/125*z*u^2+11546*x^2*v+97*x*y*v-F(33)/116*y^2*v-F(11)/122*x*z*v-14484*y*z*v-12412*z^2*v-14290*x*u*v-12210*y*u*v-F(71)/53*z*u*v-14572*u^2*v+12534*x*v^2+4027*y*v^2-1785*z*v^2+7958*u*v^2+8911*v^3,x^3-10595*x^2*y+F(15)/79*x*y^2-F(66)/107*y^3+F(43)/26*x^2*z+6745*x*y*z-F(101)/92*y^2*z+F(47)/40*x*z^2-F(15)/79*y*z^2-10411*x^2*u-F(69)/119*x*y*u+F(31)/105*y^2*u-F(18)/73*x*z*u-9615*y*z*u-F(12)/125*x*u^2-11827*y*u^2-6791*x^2*v+F(91)/38*x*y*v-F(115)/22*y^2*v+F(89)/42*x*z*v+14749*y*z*v+8900*z^2*v+11826*x*u*v-F(57)/2*y*u*v+3390*z*u*v-5604*u^2*v-F(41)/122*x*v^2+F(89)/78*y*v^2-6137*z*v^2-F(117)/11*u*v^2-F(43)/16*v^3]))
end


@doc raw"""
    enriques_d10_pi8()

Return a smooth Enriques surface in $\mathbb P^4$ with degree `10` and sectional genus `8`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function enriques_d10_pi8()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^5*y+F(43)/90*x^4*y^2+F(36)/115*x^3*y^3-2728*x^2*y^4-F(28)/71*x*y^5-14688*x^4*y*z+7046*x^3*y^2*z-F(33)/35*x^2*y^3*z+1695*x*y^4*z-F(117)/40*x^3*y*z^2-F(111)/89*x^2*y^2*z^2-3214*x*y^3*z^2+4841*x^2*y*z^3-545*x*y^2*z^3-F(72)/67*x*y*z^4-F(121)/58*x^4*y*u+F(48)/53*x^3*y^2*u+10094*x^2*y^3*u-F(88)/115*x*y^4*u-11574*x^3*y*z*u+2791*x^2*y^2*z*u+F(38)/43*x*y^3*z*u-F(16)/75*x^2*y*z^2*u-F(43)/87*x*y^2*z^2*u-11410*x*y*z^3*u-F(103)/15*x^3*y*u^2-9164*x^2*y^2*u^2+F(63)/32*x*y^3*u^2-F(31)/22*x^2*y*z*u^2+14702*x*y^2*z*u^2+13869*x*y*z^2*u^2-F(97)/47*x^2*y*u^3-F(65)/107*x*y^2*u^3+4164*x*y*z*u^3+9690*x^4*y*v+F(44)/89*x^3*y^2*v+F(44)/43*x^2*y^3*v+7334*x*y^4*v+13209*x^3*y*z*v-13039*x^2*y^2*z*v+1749*x*y^3*z*v+F(62)/77*x^2*y*z^2*v+F(101)/118*x*y^2*z^2*v-14941*x*y*z^3*v-F(62)/61*x^4*u*v-F(55)/41*x^3*y*u*v-F(60)/37*x^2*y^2*u*v+F(17)/41*x*y^3*u*v+6432*y^4*u*v+F(13)/110*x^2*y*z*u*v+9112*x*y^2*z*u*v+F(10)/67*x*y*z^2*u*v-5021*x^2*y*u^2*v+4043*x*y^2*u^2*v+5843*x*y*z*u^2*v+5617*x^2*u^3*v-F(56)/95*x*y*u^3*v-F(111)/58*y^2*u^3*v-F(18)/61*x^3*y*v^2+4087*x^2*y^2*v^2-F(59)/103*x*y^3*v^2+F(99)/2*x^2*y*z*v^2+559*x*y^2*z*v^2+F(22)/81*x*y*z^2*v^2-2079*x^2*y*u*v^2-F(19)/18*x*y^2*u*v^2-730*x*y*z*u*v^2-13077*x*y*u^2*v^2-357*x^2*y*v^3-F(52)/87*x*y^2*v^3-F(55)/73*x*y*z*v^3-14793*x^2*u*v^3-F(51)/40*x*y*u*v^3-F(55)/18*y^2*u*v^3,F(123)/46*x*y*u^4-F(52)/79*x^2*z^2*u*v+F(109)/33*x*y*z^2*u*v-1736*y^2*z^2*u*v+F(121)/7*x*z^3*u*v-F(114)/101*y*z^3*u*v-8832*z^4*u*v+F(49)/96*x^2*z*u^2*v-F(93)/95*x*y*z*u^2*v-F(52)/31*y^2*z*u^2*v-F(98)/11*x*z^2*u^2*v-F(121)/86*y*z^2*u^2*v+F(79)/74*z^3*u^2*v-F(88)/31*x^2*u^3*v-F(56)/107*x*y*u^3*v+6843*y^2*u^3*v+F(59)/18*x*z*u^3*v+11212*y*z*u^3*v+755*z^2*u^3*v-F(56)/115*x*u^4*v-F(59)/74*y*u^4*v-F(110)/27*z*u^4*v+4419*u^5*v-F(103)/74*x*y*z^2*v^2-F(107)/104*x^2*z*u*v^2-F(71)/56*x*y*z*u*v^2+F(18)/101*y^2*z*u*v^2+10712*x*z^2*u*v^2-F(9)/16*y*z^2*u*v^2-F(47)/43*z^3*u*v^2-4809*x^2*u^2*v^2-F(101)/96*x*y*u^2*v^2+F(121)/123*y^2*u^2*v^2-F(57)/46*x*z*u^2*v^2-F(49)/68*y*z*u^2*v^2-9437*z^2*u^2*v^2+F(35)/43*x*u^3*v^2-2350*y*u^3*v^2-F(46)/101*z*u^3*v^2-8704*u^4*v^2+9191*x^2*u*v^3-9399*x*y*u*v^3+10171*y^2*u*v^3-F(61)/71*x*z*u*v^3-3724*y*z*u*v^3+14739*z^2*u*v^3-F(85)/57*x*u^2*v^3-4791*y*u^2*v^3+F(38)/93*z*u^2*v^3+F(44)/31*u^3*v^3-9824*x*y*v^4-8374*x*u*v^4+F(63)/25*y*u*v^4-F(32)/91*z*u*v^4-F(101)/121*u^2*v^4+1469*u*v^5,10011*x^3*z*u-F(33)/113*x^2*y*z*u+F(56)/73*x*y^2*z*u-F(91)/16*x^2*z^2*u-F(41)/20*x*y*z^2*u-6612*x*z^3*u-F(93)/28*x^3*u^2-F(89)/20*x^2*y*u^2+F(107)/112*x*y^2*u^2-7799*x^2*z*u^2+F(11)/84*x*y*z*u^2+14867*x*z^2*u^2+4903*x^2*u^3-4407*x*y*u^3+F(53)/54*x*z*u^3+9447*x^3*z*v+F(59)/115*x^2*y*z*v-8661*x*y^2*z*v+12021*x^2*z^2*v+11548*x*y*z^2*v+10726*x*z^3*v-F(30)/121*x^3*u*v+934*x^2*y*u*v-490*x*y^2*u*v+F(67)/95*x^2*z*u*v+F(113)/67*x*y*z*u*v-134*x*z^2*u*v-F(89)/94*y*z^2*u*v+12855*x^2*u^2*v-2238*x*y*u^2*v+6446*x*z*u^2*v-3380*x*u^3*v+8282*y*u^3*v+F(75)/73*x^3*v^2+F(10)/39*x^2*y*v^2-F(29)/79*x*y^2*v^2-F(10)/29*x^2*z*v^2-F(46)/99*x*y*z*v^2-F(111)/32*x*z^2*v^2+F(13)/22*x^2*u*v^2-4191*x*y*u*v^2-F(118)/85*x*z*u*v^2-993*x*u^2*v^2-F(100)/121*x^2*v^3+7193*x*y*v^3-14252*x*z*v^3+F(45)/116*x*u*v^3-F(98)/3*y*u*v^3,F(7)/46*x^2*y*z*u+F(116)/21*x*y^2*z*u+F(35)/71*y^3*z*u-9941*x*y*z^2*u-F(13)/74*y^2*z^2*u-419*y*z^3*u-9486*x^2*y*u^2-F(81)/40*x*y^2*u^2+2506*y^3*u^2+3813*x*y*z*u^2+F(9)/65*y^2*z*u^2-1013*y*z^2*u^2+F(13)/107*x*y*u^3+15273*y^2*u^3-F(31)/2*y*z*u^3+176*x^2*y*z*v+F(20)/113*x*y^2*z*v+F(20)/51*y^3*z*v+F(68)/105*x*y*z^2*v-7946*y^2*z^2*v+F(122)/107*y*z^3*v+2797*x^2*y*u*v-F(25)/91*x*y^2*u*v+4025*y^3*u*v-6171*x*y*z*u*v+F(24)/113*y^2*z*u*v-F(99)/116*x*z^2*u*v-F(28)/47*y*z^2*u*v-F(121)/47*x*y*u^2*v-F(47)/46*y^2*u^2*v+10928*y*z*u^2*v+8516*x*u^3*v+11970*y*u^3*v-14990*x^2*y*v^2-F(70)/53*x*y^2*v^2+15272*y^3*v^2+F(10)/63*x*y*z*v^2-8337*y^2*z*v^2-757*y*z^2*v^2-9687*x*y*u*v^2+775*y^2*u*v^2+F(67)/13*y*z*u*v^2+15012*y*u^2*v^2+4687*x*y*v^3+F(67)/89*y^2*v^3-F(79)/80*y*z*v^3+F(119)/117*x*u*v^3+F(30)/19*y*u*v^3,-x^4*z-F(43)/90*x^3*y*z-F(36)/115*x^2*y^2*z+2728*x*y^3*z+F(28)/71*y^4*z+14688*x^3*z^2-7046*x^2*y*z^2+F(33)/35*x*y^2*z^2-1695*y^3*z^2+F(117)/40*x^2*z^3+F(111)/89*x*y*z^3+3214*y^2*z^3-4841*x*z^4+545*y*z^4+F(72)/67*z^5+F(63)/4*x^4*u-2929*x^3*y*u+11778*x^2*y^2*u+F(109)/43*x*y^3*u+8403*y^4*u-1870*x^3*z*u+F(45)/22*x^2*y*z*u+F(44)/109*x*y^2*z*u-13271*y^3*z*u+F(83)/81*x^2*z^2*u-2791*x*y*z^2*u-7340*y^2*z^2*u+F(16)/75*x*z^3*u+F(43)/87*y*z^3*u+11410*z^4*u-F(126)/55*x^3*u^2+F(45)/124*x^2*y*u^2+7953*x*y^2*u^2-578*y^3*u^2+4669*x^2*z*u^2+9164*x*y*z*u^2+5784*y^2*z*u^2+F(31)/22*x*z^2*u^2-14702*y*z^2*u^2-13869*z^3*u^2-F(38)/7*x^2*u^3+937*x*y*u^3-2013*y^2*u^3-F(111)/58*x*z*u^3-7154*y*z*u^3+F(75)/68*z^2*u^3-7810*x*u^4-F(48)/47*y*u^4-F(7)/83*z*u^4-F(27)/17*x^4*v+8630*x^3*y*v+11268*x^2*y^2*v+F(61)/106*x*y^3*v+F(62)/11*y^4*v+F(62)/13*x^3*z*v-1659*x^2*y*z*v-F(125)/54*x*y^2*z*v+5261*y^3*z*v+F(22)/83*x^2*z^2*v+13039*x*y*z^2*v-F(120)/121*y^2*z^2*v-F(62)/77*x*z^3*v-F(101)/118*y*z^3*v+14941*z^4*v-F(118)/55*x^3*u*v+F(73)/26*x^2*y*u*v+F(11)/27*x*y^2*u*v-F(47)/36*y^3*u*v+F(7)/121*x^2*z*u*v+F(63)/31*x*y*z*u*v-3830*y^2*z*u*v-F(13)/110*x*z^2*u*v-9112*y*z^2*u*v-F(10)/67*z^3*u*v+F(103)/118*x^2*u^2*v+10881*x*y*u^2*v+F(106)/23*y^2*u^2*v-F(19)/17*x*z*u^2*v+14960*y*z*u^2*v-F(39)/34*z^2*u^2*v+14521*x*u^3*v-F(37)/44*y*u^3*v+F(43)/21*z*u^3*v+F(47)/122*x^3*v^2+F(117)/2*x^2*y*v^2-12769*x*y^2*v^2+F(3)/16*y^3*v^2+F(41)/105*x^2*z*v^2-4087*x*y*z*v^2-14455*y^2*z*v^2-F(99)/2*x*z^2*v^2-559*y*z^2*v^2-F(22)/81*z^3*v^2+522*x^2*u*v^2+12608*x*y*u*v^2+F(26)/105*y^2*u*v^2-F(19)/107*x*z*u*v^2+9122*y*z*u*v^2-6105*z^2*u*v^2-F(107)/125*x*u^2*v^2+F(124)/21*y*u^2*v^2-F(73)/32*z*u^2*v^2-F(47)/6*x^2*v^3+F(97)/12*x*y*v^3-F(46)/49*y^2*v^3+2814*x*z*v^3+5450*y*z*v^3-12823*z^2*v^3+F(76)/41*x*u*v^3+3257*y*u*v^3+F(11)/87*z*u*v^3-F(86)/5*x*v^4+6509*y*v^4-248*z*v^4,-F(86)/67*x^3*y*u-F(11)/124*x^2*y^2*u-12237*x*y^3*u-F(45)/16*x^3*z*u-F(78)/47*x^2*y*z*u+14703*x*y^2*z*u-F(80)/49*y^3*z*u+354*x^2*z^2*u-7706*x*y*z^2*u-3773*y^2*z^2*u-825*x*z^3*u+F(81)/100*y*z^3*u+F(76)/13*x^3*u^2+F(73)/98*x^2*y*u^2+F(35)/81*x*y^2*u^2-6994*y^3*u^2-12019*x^2*z*u^2-8088*x*y*z*u^2-F(35)/117*y^2*z*u^2-10265*x*z^2*u^2+F(78)/41*y*z^2*u^2-F(59)/122*x^2*u^3-2895*x*y*u^3-F(77)/46*y^2*u^3-10881*x*z*u^3-F(28)/123*y*z*u^3+F(1)/59*x^3*y*v-1985*x*y^3*v-3230*x*y*z^2*v+12345*x^3*u*v-F(33)/35*x^2*y*u*v+F(101)/120*x*y^2*u*v-F(61)/96*y^3*u*v-10392*x^2*z*u*v+F(89)/96*x*y*z*u*v+F(61)/15*y^2*z*u*v-F(46)/113*x*z^2*u*v+11548*y*z^2*u*v-F(75)/8*x^2*u^2*v+F(113)/20*x*y*u^2*v+F(101)/34*y^2*u^2*v-F(43)/41*x*z*u^2*v+8337*y*z*u^2*v+F(47)/65*x^2*u*v^2-F(40)/29*x*y*u*v^2+F(14)/13*y^2*u*v^2+11738*x*z*u*v^2-921*y*z*u*v^2,F(123)/46*x^3*y*u+F(43)/109*x*y^3*u+F(73)/126*x*y*z^2*u+1934*x^3*y*v+3581*x^2*y^2*v-F(66)/31*x*y^3*v-14784*x^3*z*v-15801*x^2*y*z*v+F(43)/2*x*y^2*z*v+F(47)/57*y^3*z*v-F(21)/118*x^2*z^2*v+F(63)/94*x*y*z^2*v-2059*y^2*z^2*v+F(114)/55*x*z^3*v-F(90)/13*y*z^3*v+8911*x^3*u*v+F(107)/111*x^2*y*u*v+F(79)/57*x*y^2*u*v-F(18)/119*y^3*u*v-826*x^2*z*u*v+F(104)/15*x*y*z*u*v-F(113)/51*y^2*z*u*v+F(67)/76*x*z^2*u*v-F(49)/51*y*z^2*u*v-F(55)/57*x^2*u^2*v-1724*x*y*u^2*v+F(67)/54*y^2*u^2*v-3268*x*z*u^2*v-F(35)/22*y*z*u^2*v+F(78)/113*x^3*v^2+F(65)/73*x^2*y*v^2-F(34)/87*x*y^2*v^2+F(33)/73*y^3*v^2+5986*x^2*z*v^2+F(79)/112*x*y*z*v^2-11384*y^2*z*v^2+F(115)/53*x*z^2*v^2+F(9)/44*y*z^2*v^2+148*x^2*u*v^2+F(87)/89*x*y*u*v^2-F(117)/76*y^2*u*v^2+366*x*z*u*v^2+10148*y*z*u*v^2+14611*x^2*v^3-F(12)/61*x*y*v^3-8174*y^2*v^3-F(115)/11*x*z*v^3+F(98)/79*y*z*v^3,-F(63)/4*x^3*y*u+2929*x^2*y^2*u+F(55)/21*x*y^3*u-F(10)/71*x^2*y*z*u+F(89)/100*x*y^2*z*u+F(83)/90*x*y*z^2*u+F(126)/55*x^2*y*u^2-F(45)/124*x*y^2*u^2+10267*x*y*z*u^2+F(27)/17*x^3*y*v-8630*x^2*y^2*v+4058*x*y^3*v-F(83)/16*x^2*y*z*v+F(50)/67*x*y^2*z*v-14751*x*y*z^2*v+F(118)/55*x^2*y*u*v-F(73)/26*x*y^2*u*v+F(62)/61*x^2*z*u*v+F(50)/49*x*y*z*u*v+F(103)/37*y^2*z*u*v-F(47)/122*x^2*y*v^2-F(117)/2*x*y^2*v^2+12142*x*y*z*v^2,-F(99)/122*x^3*y*u+F(54)/121*x^2*y^2*u-12508*x*y^3*u+F(119)/33*x^2*y*z*u-3250*x*y^2*z*u-F(6)/119*x*y*z^2*u+F(93)/94*x^2*y*u^2+9523*x*y^2*u^2+9547*x*y*z*u^2-F(1)/59*x^2*y^2*v-F(5)/92*x^2*y*u*v+4221*x*y^2*u*v+F(59)/54*x*y*z*u*v-3249*x^2*u^2*v-F(55)/21*x*y*u^2*v-13841*y^2*u^2*v-F(34)/65*x*y*u*v^2,-F(123)/46*x^2*y^2*u+3913*x^3*y*v+F(19)/110*x^2*y^2*v+80*x*y^3*v+F(51)/118*x^2*y*z*v-F(3)/100*x*y^2*z*v+F(106)/111*x*y*z^2*v+5540*x^2*y*u*v-F(118)/39*x*y^2*u*v+6811*x*y*z*u*v-4419*x*y*u^2*v-6521*x^2*y*v^2+F(13)/54*x*y^2*v^2+F(3)/50*x*y*z*v^2-F(58)/115*x^2*u*v^2+5474*x*y*u*v^2-F(19)/14*y^2*u*v^2,-F(99)/122*x^2*y*z*u+F(54)/121*x*y^2*z*u-12508*y^3*z*u+F(119)/33*x*y*z^2*u-3250*y^2*z^2*u-F(6)/119*y*z^3*u-2447*x^2*y*u^2+589*x*y^2*u^2+6041*y^3*u^2-15067*x*y*z*u^2-F(32)/73*y^2*z*u^2+6226*y*z^2*u^2+F(121)/51*x*y*u^3+5988*y^2*u^3-F(115)/82*y*z*u^3-F(1)/59*x*y^2*z*v+F(26)/11*x^2*y*u*v+F(57)/2*x*y^2*u*v+14010*y^3*u*v-6182*x*y*z*u*v-14459*y^2*z*u*v+9447*y*z^2*u*v+F(92)/19*x*y*u^2*v+F(91)/4*y^2*u^2*v+5608*x*z*u^2*v+F(79)/102*y*z*u^2*v-F(99)/31*x*y*u*v^2+F(7)/89*y^2*u*v^2+2895*y*z*u*v^2,-F(123)/46*x*y^2*z*u+3913*x^2*y*z*v+F(19)/110*x*y^2*z*v+80*y^3*z*v+F(51)/118*x*y*z^2*v-F(3)/100*y^2*z^2*v+F(106)/111*y*z^3*v-2143*x^2*y*u*v+12099*x*y^2*u*v-9195*y^3*u*v-10514*x*y*z*u*v-3024*y^2*z*u*v-F(22)/87*y*z^2*u*v+F(112)/75*x*y*u^2*v-F(22)/93*y^2*u^2*v+F(122)/85*y*z*u^2*v+F(88)/85*x^2*y*v^2+F(9)/119*x*y^2*v^2+F(37)/44*y^3*v^2+10419*x*y*z*v^2-F(97)/49*y^2*z*v^2+5527*y*z^2*v^2-1610*x*y*u*v^2+12430*y^2*u*v^2-251*x*z*u*v^2+2307*y*z*u*v^2+F(124)/125*x*y*v^3-F(79)/58*y^2*v^3+F(93)/71*y*z*v^3,-F(99)/122*x^3*z*u+F(54)/121*x^2*y*z*u-12508*x*y^2*z*u+F(119)/33*x^2*z^2*u-3250*x*y*z^2*u-F(6)/119*x*z^3*u+F(14)/37*x^3*u^2-F(75)/98*x^2*y*u^2+F(72)/19*x*y^2*u^2+F(37)/108*x^2*z*u^2+F(27)/88*x*y*z*u^2-13975*x*z^2*u^2-F(68)/93*x^2*u^3+F(56)/25*x*y*u^3+F(3)/44*x*z*u^3-F(1)/59*x^2*y*z*v-11647*x^3*u*v-F(104)/107*x^2*y*u*v+6147*x*y^2*u*v-F(101)/123*x^2*z*u*v-3975*x*y*z*u*v+F(20)/51*x*z^2*u*v+10592*x^2*u^2*v+3865*x*y*u^2*v-6204*x*z*u^2*v-F(62)/117*y*z*u^2*v-F(17)/35*x^2*u*v^2+F(52)/37*x*y*u*v^2-F(67)/126*x*z*u*v^2,-F(123)/46*x^2*y*z*u+3913*x^3*z*v+F(19)/110*x^2*y*z*v+80*x*y^2*z*v+F(51)/118*x^2*z^2*v-F(3)/100*x*y*z^2*v+F(106)/111*x*z^3*v-F(1)/34*x^3*u*v+F(109)/80*x^2*y*u*v-11079*x*y^2*u*v+F(107)/31*x^2*z*u*v-10900*x*y*z*u*v+F(35)/96*x*z^2*u*v+F(103)/83*x^2*u^2*v+F(65)/59*x*y*u^2*v-10344*x*z*u^2*v-F(79)/23*x^3*v^2+F(1)/113*x^2*y*v^2-F(103)/14*x*y^2*v^2+F(41)/116*x^2*z*v^2+F(9)/43*x*y*z*v^2-F(80)/97*x*z^2*v^2+252*x^2*u*v^2+5009*x*y*u*v^2+F(13)/98*x*z*u*v^2+8483*y*z*u*v^2-F(28)/123*x^2*v^3-F(13)/119*x*y*v^3+F(30)/83*x*z*v^3,-F(123)/46*x*y*z*u^2+F(125)/66*x^2*z*u*v+F(25)/113*x*y*z*u*v-F(8)/83*y^2*z*u*v-F(14)/111*x*z^2*u*v-8151*y*z^2*u*v-F(97)/52*x^2*u^2*v-F(105)/31*x*y*u^2*v+F(9)/53*y^2*u^2*v-2295*x*z*u^2*v+F(31)/35*y*z*u^2*v+3177*x*u^3*v-4539*y*u^3*v+F(103)/90*x*y*z*v^2+F(116)/67*x^2*u*v^2-12467*x*y*u*v^2-F(85)/29*y^2*u*v^2+F(118)/55*x*z*u*v^2+F(113)/37*y*z*u*v^2-6165*x*u^2*v^2+F(67)/47*y*u^2*v^2+4167*x*u*v^3-F(97)/14*y*u*v^3,-F(123)/46*x*y^2*u^2-7239*x^2*y*u*v+175*x*y^2*u*v+F(49)/30*x*y*z*u*v-F(83)/49*y^2*z*u*v-9035*y*z^2*u*v+F(73)/7*x*y*u^2*v-F(124)/57*y^2*u^2*v-1916*y*z*u^2*v-4419*y*u^3*v+8941*x*y^2*v^2+F(76)/55*x*y*u*v^2+F(49)/71*y^2*u*v^2-4459*y*z*u*v^2+8931*x*u^2*v^2+9935*y*u^2*v^2-14939*y*u*v^3,-F(123)/46*x^2*y*u^2-F(95)/39*x^2*y*u*v-F(5)/22*x*y^2*u*v-F(104)/125*x^2*z*u*v+F(25)/76*x*y*z*u*v-F(5)/32*x*z^2*u*v+14601*x^2*u^2*v-233*x*y*u^2*v+F(98)/121*x*z*u^2*v-4419*x*u^3*v+F(25)/46*x^2*y*v^2-8462*x^2*u*v^2-F(36)/103*x*y*u*v^2-F(114)/89*x*z*u*v^2-F(15)/26*x*u^2*v^2-F(110)/93*y*u^2*v^2+F(33)/53*x*u*v^3]))
end


@doc raw"""
    enriques_d11_pi10()

Return a smooth Enriques surface in $\mathbb P^4$ with degree `11` and sectional genus `10`.

The returned surface is defined over a prime field of characteristic 43.
"""
function enriques_d11_pi10()
   F = GF(43)  # SPECIAL
   R, (v,w,x,y,z) = graded_polynomial_ring(F, [:v,:w,:x,:y,:z])
   return ProjectiveScheme(R, ideal([F(1)/2*v^4*w*x+6*v^3*w^2*x-5*v^2*w^3*x+7*v*w^4*x+16*w^5*x-F(1)/2*v^3*w*x^2-18*v^2*w^2*x^2-F(2)/3*v*w^3*x^2+9*w^4*x^2-F(1)/2*v^2*w*x^3-2*v*w^2*x^3-F(1)/2*w^3*x^3+6*v*w*x^4+w^2*x^4-3*w*x^5-18*v^4*x*y-16*v^3*w*x*y+F(1)/2*v^2*w^2*x*y+F(1)/3*v*w^3*x*y-12*w^4*x*y+F(3)/2*v^3*x^2*y+F(1)/3*v^2*w*x^2*y+2*v*w^2*x^2*y-9*w^3*x^2*y+v^2*x^3*y-F(3)/4*v*w*x^3*y-3*w^2*x^3*y+F(1)/2*v*x^4*y+12*w*x^4*y+x^5*y+F(1)/3*v^3*x*y^2-F(3)/2*v^2*w*x*y^2-16*v*w^2*x*y^2-18*w^3*x*y^2-F(3)/4*v^2*x^2*y^2-12*v*w*x^2*y^2-19*w^2*x^2*y^2+12*v*x^3*y^2+F(3)/2*w*x^3*y^2+9*x^4*y^2+7*v^2*x*y^3-F(3)/4*v*w*x*y^3-2*w^2*x*y^3+F(4)/3*v*x^2*y^3+3*w*x^2*y^3-4*v*x*y^4-19*w*x*y^4+17*x^2*y^4-2*x*y^5-F(1)/4*v^4*x*z+F(1)/4*v^3*w*x*z-17*v^2*w^2*x*z-19*v*w^3*x*z+F(1)/3*w^4*x*z-F(3)/4*v^3*x^2*z-4*v^2*w*x^2*z+8*v*w^2*x^2*z+4*w^3*x^2*z+18*v^2*x^3*z+9*v*w*x^3*z+F(4)/3*w^2*x^3*z-F(3)/2*v*x^4*z-F(2)/3*w*x^4*z+F(3)/2*x^5*z+8*v^4*y*z+5*v^3*w*y*z-F(4)/3*v^2*w^2*y*z-3*v*w^3*y*z+8*w^4*y*z+F(4)/3*v^3*x*y*z+4*v^2*w*x*y*z+F(1)/3*v*w^2*x*y*z+3*w^3*x*y*z-5*v^2*x^2*y*z+F(2)/3*v*w*x^2*y*z+F(1)/3*w^2*x^2*y*z+3*v*x^3*y*z-3*x^4*y*z-v^3*y^2*z+12*v^2*w*y^2*z-16*v*w^2*y^2*z-4*w^3*y^2*z-4*v^2*x*y^2*z-3*v*w*x*y^2*z+4*w^2*x*y^2*z+v*x^2*y^2*z-w*x^2*y^2*z+F(3)/4*x^3*y^2*z+18*v^2*y^3*z+v*w*y^3*z-7*w^2*y^3*z+6*v*x*y^3*z+7*w*x*y^3*z-19*x^2*y^3*z+8*v*y^4*z+F(4)/3*w*y^4*z+18*x*y^4*z-F(3)/2*y^5*z+17*v^4*z^2+F(1)/3*v^3*w*z^2+F(1)/4*v^2*w^2*z^2+6*v*w^3*z^2-18*w^4*z^2-17*v^2*w*x*z^2+v*w^2*x*z^2-F(2)/3*w^3*x*z^2+3*v^2*x^2*z^2-F(1)/4*v*w*x^2*z^2-5*w^2*x^2*z^2+9*v*x^3*z^2+4*w*x^3*z^2+4*x^4*z^2-F(4)/3*v^3*y*z^2-8*v^2*w*y*z^2-3*v*w^2*y*z^2-F(1)/2*w^3*y*z^2+2*v^2*x*y*z^2+F(1)/4*v*w*x*y*z^2+F(3)/4*w^2*x*y*z^2-6*v*x^2*y*z^2+18*w*x^2*y*z^2-12*x^3*y*z^2-F(2)/3*v^2*y^2*z^2+5*v*w*y^2*z^2+4*w^2*y^2*z^2+16*v*x*y^2*z^2-18*w*x*y^2*z^2-18*x^2*y^2*z^2+F(1)/4*v*y^3*z^2-4*x*y^3*z^2+F(2)/3*y^4*z^2-3*v^3*z^3+F(1)/3*v^2*w*z^3+18*v*w^2*z^3+F(2)/3*w^3*z^3-2*v^2*x*z^3+16*v*w*x*z^3-12*w^2*x*z^3-F(3)/4*v*x^2*z^3+19*w*x^2*z^3-4*x^3*z^3-F(3)/2*v^2*y*z^3-17*v*w*y*z^3-w^2*y*z^3+F(3)/4*v*x*y*z^3+9*w*x*y*z^3+5*x^2*y*z^3-5*w*y^2*z^3+8*x*y^2*z^3-v^2*z^4+5*v*w*z^4+18*w^2*z^4+F(1)/4*v*x*z^4-5*w*x*z^4+x^2*z^4+16*v*y*z^4-F(1)/4*w*y*z^4+F(1)/4*x*y*z^4+5*y^2*z^4+3*v*z^5-5*w*z^5+7*x*z^5+5*y*z^5+F(1)/2*z^6,F(3)/2*v^4*x^2+18*v^3*w*x^2-F(2)/3*v^2*w^2*x^2-F(1)/2*v*w^3*x^2+5*w^4*x^2-F(3)/2*v^3*x^3-F(1)/4*v^2*w*x^3-2*v*w^2*x^3-16*w^3*x^3-F(3)/2*v^2*x^4-6*v*w*x^4-F(3)/2*w^2*x^4+18*v*x^5+3*w*x^5-9*x^6+8*v^4*w*y-F(3)/4*v^3*w^2*y+6*v^2*w^3*y-17*v*w^4*y-2*w^5*y+F(2)/3*v^4*x*y+17*v^3*w*x*y+4*v^2*w^2*x*y+9*v*w^3*x*y+18*w^4*x*y-6*v^3*x^2*y-2*v^2*w*x^2*y+F(1)/2*v*w^2*x^2*y-3*w^3*x^2*y+7*v^2*x^3*y-F(1)/2*v*w*x^3*y-7*w^2*x^3*y+19*v*x^4*y+F(3)/2*w*x^4*y-x^5*y+18*v^4*y^2-F(3)/2*v^3*w*y^2-F(1)/4*v^2*w^2*y^2+8*v*w^3*y^2+8*w^4*y^2+5*v^3*x*y^2+3*v^2*w*x*y^2+4*v*w^2*x*y^2-18*w^3*x*y^2-19*v^2*x^2*y^2+v*w*x^2*y^2+3*w^2*x^2*y^2-19*v*x^3*y^2+F(1)/4*w*x^3*y^2+9*x^4*y^2+3*v^3*y^3-9*v^2*w*y^3+F(1)/3*v*w^2*y^3-v^2*x*y^3+7*v*w*x*y^3-7*w^2*x*y^3-8*v*x^2*y^3-4*w*x^2*y^3+16*x^3*y^3-v^2*y^4-6*v*w*y^4-F(1)/4*w^2*y^4+F(1)/3*v*x*y^4+F(3)/2*w*x*y^4+F(3)/4*x^2*y^4-F(1)/4*v*y^5+F(3)/2*w*y^5+8*x*y^5-2*y^6+F(1)/3*v^4*w*z-8*v^3*w^2*z-v^2*w^3*z-F(1)/2*v*w^4*z+7*w^5*z-16*v^4*x*z-9*v^3*w*x*z-F(1)/2*v^2*w^2*x*z+16*v*w^3*x*z+8*w^4*x*z+F(1)/2*v^3*x^2*z+F(4)/3*v^2*w*x^2*z+9*w^3*x^2*z+8*v^2*x^3*z+F(3)/4*v*w*x^3*z-17*w^2*x^3*z+F(2)/3*v*x^4*z+3*w*x^4*z-7*x^5*z-F(1)/4*v^4*y*z+F(1)/4*v^3*w*y*z+3*v^2*w^2*y*z-6*v*w^3*y*z+2*w^4*y*z-3*v^3*x*y*z-F(1)/4*v^2*w*x*y*z-12*v*w^2*x*y*z-17*w^3*x*y*z+F(3)/2*v^2*x^2*y*z-F(2)/3*v*w*x^2*y*z-F(1)/2*w^2*x^2*y*z+3*v*x^3*y*z+7*w*x^3*y*z-2*x^4*y*z+2*v^3*y^2*z-3*v^2*w*y^2*z-8*v*w^2*y^2*z-3*w^3*y^2*z+4*v^2*x*y^2*z+v*w*x*y^2*z+19*w^2*x*y^2*z+F(2)/3*v*x^2*y^2*z-17*w*x^2*y^2*z-5*x^3*y^2*z-3*v^2*y^3*z+12*v*w*y^3*z+5*w^2*y^3*z-6*v*x*y^3*z+8*w*x*y^3*z-F(2)/3*x^2*y^3*z-F(1)/2*v*y^4*z-F(1)/4*w*y^4*z-F(1)/3*x*y^4*z+6*y^5*z+12*v^4*z^2-F(2)/3*v^3*w*z^2+17*v^2*w^2*z^2-9*v*w^3*z^2+16*w^4*z^2+8*v^3*x*z^2-6*v^2*w*x*z^2+4*v*w^2*x*z^2-F(1)/3*w^3*x*z^2-17*v^2*x^2*z^2-4*v*w*x^2*z^2+F(3)/4*w^2*x^2*z^2+19*v*x^3*z^2-6*w*x^3*z^2+3*x^4*z^2-F(2)/3*v^3*y*z^2+F(1)/2*v^2*w*y*z^2+9*v*w^2*y*z^2-16*w^3*y*z^2+F(1)/4*v^2*x*y*z^2+F(1)/4*w^2*x*y*z^2-F(1)/3*v*x^2*y*z^2-12*w*x^2*y*z^2-F(1)/4*x^3*y*z^2-7*v^2*y^2*z^2+9*v*w*y^2*z^2-2*w^2*y^2*z^2-F(3)/2*v*x*y^2*z^2+F(1)/4*w*x*y^2*z^2+19*x^2*y^2*z^2+F(1)/3*v*y^3*z^2-3*w*y^3*z^2-19*x*y^3*z^2-19*y^4*z^2-5*v^3*z^3-F(2)/3*v^2*w*z^3+7*v*w^2*z^3-9*w^3*z^3+F(1)/2*v^2*x*z^3-v*w*x*z^3+18*w^2*x*z^3+9*v*x^2*z^3-F(1)/3*w*x^2*z^3-8*x^3*z^3+4*v^2*y*z^3-3*v*w*y*z^3+7*w^2*y*z^3+19*v*x*y*z^3+3*w*x*y*z^3-F(3)/4*x^2*y*z^3+19*v*y^2*z^3-17*w*y^2*z^3+3*x*y^2*z^3+F(4)/3*y^3*z^3-F(2)/3*v^2*z^4+3*v*w*z^4+9*w^2*z^4-16*v*x*z^4+9*w*x*z^4+18*x^2*z^4+9*v*y*z^4+6*w*y*z^4-9*x*y*z^4+18*y^2*z^4+F(1)/3*v*z^5+F(1)/2*w*z^5-18*x*z^5+F(1)/4*y*z^5+12*z^6,-16*v^4*x^2+F(3)/4*v^3*w*x^2+17*v^2*w^2*x^2+6*v*w^3*x^2-16*w^4*x^2-16*v^3*x^3-9*v^2*w*x^3-5*v*w^2*x^3-w^3*x^3+4*v^2*x^4+3*v*w*x^4+19*w^2*x^4-4*v*x^5-16*w*x^5+F(1)/4*x^6-3*v^4*x*y+16*v^3*w*x*y+3*v^2*w^2*x*y+9*v*w^3*x*y-w^4*x*y+F(3)/2*v^3*x^2*y+F(1)/2*v^2*w*x^2*y+8*v*w^2*x^2*y-F(4)/3*w^3*x^2*y+3*v^2*x^3*y-3*v*w*x^3*y-F(2)/3*w^2*x^3*y-F(4)/3*v*x^4*y+18*w*x^4*y-4*x^5*y+F(1)/2*v^4*y^2+9*v^3*w*y^2+F(1)/3*v^2*w^2*y^2+8*v*w^3*y^2-3*w^4*y^2+5*v^3*x*y^2-16*v^2*w*x*y^2-5*v*w^2*x*y^2+17*w^3*x*y^2+4*v^2*x^2*y^2+F(3)/2*v*w*x^2*y^2+7*w^2*x^2*y^2+4*v*x^3*y^2+12*w*x^3*y^2-F(2)/3*x^4*y^2+3*v^3*y^3+16*v^2*w*y^3+F(1)/4*v*w^2*y^3-18*w^3*y^3-4*v^2*x*y^3-6*v*w*x*y^3+w^2*x*y^3-F(1)/2*v*x^2*y^3-6*w*x^2*y^3+2*x^3*y^3-F(2)/3*v^2*y^4-F(2)/3*v*w*y^4-5*w^2*y^4+8*v*x*y^4-5*w*x*y^4-12*x^2*y^4+F(1)/4*v*y^5-18*w*y^5+F(2)/3*x*y^5+F(1)/2*y^6-8*v^4*w*z+F(3)/4*v^3*w^2*z-6*v^2*w^3*z+17*v*w^4*z+2*w^5*z-F(4)/3*v^4*x*z+v^3*w*x*z+5*v^2*w^2*x*z+F(1)/2*w^4*x*z+F(3)/2*v^3*x^2*z-17*v^2*w*x^2*z+16*v*w^2*x^2*z-17*w^3*x^2*z+4*v^2*x^3*z-7*v*w*x^3*z+F(2)/3*w^2*x^3*z+F(1)/2*v*x^4*z-12*w*x^4*z+F(1)/2*x^5*z+4*v^4*y*z-v^3*w*y*z+F(1)/2*v^2*w^2*y*z-2*v*w^3*y*z+12*w^4*y*z+12*v^3*x*y*z-5*v^2*w*x*y*z-3*v*w^2*x*y*z+F(1)/4*w^3*x*y*z+17*v^2*x^2*y*z+4*v*w*x^2*y*z+F(1)/3*w^2*x^2*y*z+F(4)/3*v*x^3*y*z+F(1)/2*w*x^3*y*z+19*x^4*y*z+6*v^3*y^2*z-8*v^2*w*y^2*z-F(1)/3*v*w^2*y^2*z-18*w^3*y^2*z-2*v^2*x*y^2*z-9*v*w*x*y^2*z+F(4)/3*w^2*x*y^2*z+17*v*x^2*y^2*z+3*w*x^2*y^2*z+9*x^3*y^2*z+8*v^2*y^3*z+17*v*w*y^3*z+F(1)/4*w^2*y^3*z+6*v*x*y^3*z+2*w*x*y^3*z+F(1)/4*x^2*y^3*z+2*v*y^4*z-F(2)/3*w*y^4*z+16*x*y^4*z-18*y^5*z+16*v^4*z^2+F(3)/4*v^3*w*z^2-F(1)/3*v^2*w^2*z^2-F(4)/3*v*w^3*z^2+F(1)/4*w^4*z^2-17*v^3*x*z^2-5*v^2*w*x*z^2-F(3)/4*v*w^2*x*z^2+6*w^3*x*z^2+9*v^2*x^2*z^2+5*v*w*x^2*z^2+F(3)/4*w^2*x^2*z^2+8*v*x^3*z^2+8*w*x^3*z^2+9*x^4*z^2+F(3)/4*v^3*y*z^2+F(2)/3*v^2*w*y*z^2-4*v*w^2*y*z^2-w^3*y*z^2+F(1)/2*v^2*x*y*z^2+19*v*w*x*y*z^2-F(4)/3*w^2*x*y*z^2+6*v*x^2*y*z^2+17*w*x^2*y*z^2-F(4)/3*x^3*y*z^2+8*v^2*y^2*z^2+19*v*w*y^2*z^2-4*w^2*y^2*z^2+7*v*x*y^2*z^2-F(1)/3*w*x*y^2*z^2+F(3)/2*x^2*y^2*z^2-F(3)/4*v*y^3*z^2-F(1)/4*w*y^3*z^2+8*x*y^3*z^2-5*y^4*z^2+v^3*z^3+16*v^2*w*z^3-2*v*w^2*z^3+5*w^3*z^3+F(3)/2*v^2*x*z^3+F(2)/3*v*w*x*z^3-7*w^2*x*z^3+F(3)/4*v*x^2*z^3+F(2)/3*w*x^2*z^3-x^3*z^3-F(4)/3*v^2*y*z^3+7*v*w*y*z^3+7*v*x*y*z^3+9*w*x*y*z^3-5*x^2*y*z^3+7*v*y^2*z^3+18*w*y^2*z^3+x*y^2*z^3-2*y^3*z^3+16*v^2*z^4-F(2)/3*v*w*z^4+F(3)/4*w^2*z^4-12*v*x*z^4+F(4)/3*w*x*z^4+18*v*y*z^4-F(3)/4*w*y*z^4+F(1)/3*x*y*z^4+12*y^2*z^4-17*v*z^5-2*w*z^5-F(2)/3*x*z^5-17*y*z^5-z^6,-16*v^4*x*y+F(3)/4*v^3*w*x*y+17*v^2*w^2*x*y+6*v*w^3*x*y-16*w^4*x*y-16*v^3*x^2*y-9*v^2*w*x^2*y-5*v*w^2*x^2*y-w^3*x^2*y+4*v^2*x^3*y+3*v*w*x^3*y+19*w^2*x^3*y-4*v*x^4*y-16*w*x^4*y+F(1)/4*x^5*y+9*v^4*y^2-12*v^3*w*y^2+12*v^2*w^2*y^2+5*v*w^3*y^2-4*w^4*y^2+F(1)/4*v^3*x*y^2-F(3)/2*v^2*w*x*y^2-8*v*w^2*x*y^2-F(1)/3*w^3*x*y^2-9*v^2*x^2*y^2-8*v*w*x^2*y^2+16*w^2*x^2*y^2-F(2)/3*v*x^3*y^2-w*x^3*y^2-F(3)/4*x^4*y^2+9*v^3*y^3+F(1)/2*v^2*w*y^3-19*w^3*y^3+v^2*x*y^3-6*v*w*x*y^3-F(2)/3*w^2*x*y^3+16*v*x^2*y^3+12*w*x^2*y^3+16*x^3*y^3-F(2)/3*v^2*y^4-F(4)/3*v*w*y^4-7*w^2*y^4+F(1)/4*v*x*y^4-F(2)/3*w*x*y^4+F(3)/2*x^2*y^4+F(1)/2*v*y^5-7*w*y^5-16*x*y^5-12*y^6+3*v^4*x*z-7*v^3*w*x*z-F(4)/3*v^2*w^2*x*z-v*w^3*x*z-F(3)/4*w^4*x*z-3*v^3*x^2*z-F(1)/2*v^2*w*x^2*z-4*v*w^2*x^2*z+F(1)/4*w^3*x^2*z-3*v^2*x^3*z-12*v*w*x^3*z-3*w^2*x^3*z-7*v*x^4*z+6*w*x^4*z-18*x^5*z-F(2)/3*v^4*y*z-3*v^3*w*y*z+F(1)/4*v*w^3*y*z-F(1)/4*w^4*y*z-F(2)/3*v^3*x*y*z-8*v^2*w*x*y*z-17*v*w^2*x*y*z+F(3)/2*w^3*x*y*z-17*v^2*x^2*y*z+19*v*w*x^2*y*z+F(3)/4*w^2*x^2*y*z+v*x^3*y*z+F(1)/2*w*x^3*y*z+F(1)/2*x^4*y*z-4*v^3*y^2*z+F(2)/3*v^2*w*y^2*z-F(4)/3*v*w^2*y^2*z-F(3)/4*w^3*y^2*z+17*v^2*x*y^2*z+5*v*w*x*y^2*z-12*w^2*x*y^2*z-12*v*x^2*y^2*z+F(2)/3*w*x^2*y^2*z+8*x^3*y^2*z-F(2)/3*v^2*y^3*z+2*v*w*y^3*z-F(4)/3*v*x*y^3*z+16*w*x*y^3*z+2*x^2*y^3*z-17*v*y^4*z-12*w*y^4*z-6*x*y^4*z-9*y^5*z-v^4*z^2+5*v^3*w*z^2-F(1)/2*v^2*w^2*z^2-12*v*w^3*z^2+3*w^4*z^2+2*v^3*x*z^2-2*v^2*w*x*z^2-v*w^2*x*z^2-F(1)/2*w^3*x*z^2-12*v^2*x^2*z^2-F(4)/3*v*w*x^2*z^2-F(1)/2*w^2*x^2*z^2+16*v*x^3*z^2+F(1)/4*w*x^3*z^2+x^4*z^2+9*v^3*y*z^2+F(1)/2*v^2*w*y*z^2+7*v*w^2*y*z^2-8*w^3*y*z^2-5*v^2*x*y*z^2-19*v*w*x*y*z^2-w^2*x*y*z^2-9*v*x^2*y*z^2+F(1)/2*w*x^2*y*z^2-F(1)/3*x^3*y*z^2-17*v^2*y^2*z^2+8*v*w*y^2*z^2+16*w^2*y^2*z^2+F(4)/3*v*x*y^2*z^2+19*w*x*y^2*z^2-9*x^2*y^2*z^2-16*v*y^3*z^2-19*w*y^3*z^2+17*y^4*z^2+F(1)/2*v^3*z^3+17*v^2*w*z^3+F(2)/3*v*w^2*z^3+2*w^3*z^3+F(4)/3*v^2*x*z^3+F(1)/3*v*w*x*z^3-F(3)/2*w^2*x*z^3-17*v*x^2*z^3-x^3*z^3-F(4)/3*v^2*y*z^3+F(3)/2*v*w*y*z^3-F(1)/4*w^2*y*z^3+4*v*x*y*z^3+19*w*x*y*z^3-2*x^2*y*z^3-F(1)/4*v*y^2*z^3-8*w*y^2*z^3+4*x*y^2*z^3+5*y^3*z^3+v^2*z^4+18*v*w*z^4-3*w^2*z^4+3*v*x*z^4-7*w*x*z^4+9*x^2*z^4-7*v*y*z^4+17*w*y*z^4+F(2)/3*x*y*z^4+2*y^2*z^4-4*v*z^5+12*w*z^5-19*x*z^5-12*y*z^5+F(3)/2*z^6,12*v^4*w^2+F(2)/3*v^3*w^3+9*v^2*w^4-4*v*w^5-3*w^6+F(2)/3*v^4*w*x+12*v^3*w^2*x+7*v^2*w^3*x-F(1)/3*v*w^4*x-F(3)/2*w^5*x-F(1)/4*v^4*x^2-F(3)/4*v^3*w*x^2-5*v^2*w^2*x^2-12*v*w^3*x^2-17*v^3*x^3-F(1)/2*v^2*w*x^3+F(1)/2*v*w^2*x^3-F(3)/4*w^3*x^3+3*v^2*x^4+F(1)/4*w^2*x^4+F(3)/2*v*x^5+F(3)/4*w*x^5+3*x^6+17*v^4*w*y-6*v^3*w^2*y-F(1)/2*v*w^4*y-w^5*y-6*v^4*x*y-2*v^3*w*x*y-F(3)/2*v^2*w^2*x*y+12*v*w^3*x*y+4*w^4*x*y+2*v^3*x^2*y-F(4)/3*v^2*w*x^2*y-6*v*w^2*x^2*y-5*v^2*x^3*y-v*w*x^3*y-F(3)/2*w^2*x^3*y-2*v*x^4*y-F(1)/2*w*x^4*y+F(3)/4*x^5*y-F(1)/2*v^4*y^2-v^3*w*y^2-4*v^2*w^2*y^2+F(4)/3*v*w^3*y^2+4*w^4*y^2-v^3*x*y^2-18*v^2*w*x*y^2+16*v*w^2*x*y^2-F(3)/2*v^2*x^2*y^2-7*v*w*x^2*y^2-17*w^2*x^2*y^2-4*v*x^3*y^2-F(2)/3*w*x^3*y^2-F(3)/4*x^4*y^2-8*v^3*y^3-2*v^2*w*y^3+F(3)/2*v*w^2*y^3+17*w^3*y^3+5*v*w*x*y^3-17*w^2*x*y^3-18*v*x^2*y^3+6*w*x^2*y^3-F(3)/4*x^3*y^3+F(2)/3*v^2*y^4-7*v*w*y^4-9*w^2*y^4-4*v*x*y^4-17*w*x*y^4+F(1)/2*x^2*y^4-F(1)/2*v*y^5+5*w*y^5-12*x*y^5-12*y^6-6*v^4*w*z+6*v^3*w^2*z+F(1)/2*v^2*w^3*z+17*v*w^4*z+8*w^5*z-4*v^4*x*z+F(1)/3*v^3*w*x*z+19*v^2*w^2*x*z+F(1)/4*v*w^3*x*z-19*w^4*x*z+2*v^3*x^2*z+v^2*w*x^2*z+v*w^2*x^2*z-8*w^3*x^2*z+4*v^2*x^3*z-F(1)/3*v*w*x^3*z-F(1)/2*w^2*x^3*z+F(2)/3*v*x^4*z+6*w*x^4*z-17*x^5*z+F(3)/2*v^4*y*z-9*v^3*w*y*z+18*v^2*w^2*y*z-3*v*w^3*y*z+19*w^4*y*z-6*v^3*x*y*z+4*v^2*w*x*y*z+v*w^2*x*y*z+F(4)/3*w^3*x*y*z-F(4)/3*v^2*x^2*y*z+17*v*w*x^2*y*z-F(1)/3*w^2*x^2*y*z+18*v*x^3*y*z-F(1)/3*w*x^3*y*z+2*x^4*y*z-9*v^3*y^2*z+F(1)/4*v^2*w*y^2*z+F(1)/3*v*w^2*y^2*z+8*w^3*y^2*z+F(2)/3*v^2*x*y^2*z+16*v*w*x*y^2*z-7*w^2*x*y^2*z-3*v*x^2*y^2*z+F(3)/4*w*x^2*y^2*z+7*x^3*y^2*z+F(1)/4*v^2*y^3*z+16*v*w*y^3*z+2*w^2*y^3*z-F(3)/4*v*x*y^3*z+F(1)/2*w*x*y^3*z+2*x^2*y^3*z+F(1)/2*v*y^4*z-12*w*y^4*z+F(4)/3*x*y^4*z-16*y^5*z-F(3)/2*v^4*z^2-18*v^3*w*z^2-7*v^2*w^2*z^2-3*v*w^3*z^2+3*w^4*z^2+F(2)/3*v^3*x*z^2+F(3)/4*v^2*w*x*z^2+v*w^2*x*z^2-18*w^3*x*z^2-F(3)/4*v^2*x^2*z^2-17*v*w*x^2*z^2+2*w^2*x^2*z^2-3*v*x^3*z^2-17*w*x^3*z^2-F(3)/4*v^3*y*z^2-F(1)/3*v^2*w*y*z^2+3*v*w^2*y*z^2+8*w^3*y*z^2-3*v^2*x*y*z^2-18*v*w*x*y*z^2+2*w^2*x*y*z^2-v*x^2*y*z^2-F(1)/4*w*x^2*y*z^2+x^3*y*z^2+9*v^2*y^2*z^2-8*v*w*y^2*z^2+5*v*x*y^2*z^2-F(4)/3*w*x*y^2*z^2-9*x^2*y^2*z^2+17*v*y^3*z^2+5*w*y^3*z^2+F(2)/3*x*y^3*z^2+2*y^4*z^2+F(3)/2*v^3*z^3+2*v^2*w*z^3+17*v*w^2*z^3-F(1)/4*w^3*z^3-3*v^2*x*z^3+16*v*w*x*z^3-F(3)/4*w^2*x*z^3+F(3)/2*v*x^2*z^3+F(1)/2*w*x^2*z^3+7*x^3*z^3+F(4)/3*v^2*y*z^3-19*v*w*y*z^3-3*v*x*y*z^3-F(4)/3*w*x*y*z^3-4*x^2*y*z^3-18*v*y^2*z^3+9*w*y^2*z^3-6*x*y^2*z^3+3*y^3*z^3-7*v^2*z^4-F(4)/3*v*w*z^4+12*w^2*z^4-F(1)/3*v*x*z^4-12*w*x*z^4-2*x^2*z^4+7*v*y*z^4-5*w*y*z^4-F(2)/3*x*y*z^4-F(1)/4*y^2*z^4-19*v*z^5+9*w*z^5+F(1)/3*x*z^5+4*y*z^5-F(2)/3*z^6,F(3)/2*v^4*x*y+18*v^3*w*x*y-F(2)/3*v^2*w^2*x*y-F(1)/2*v*w^3*x*y+5*w^4*x*y-F(3)/2*v^3*x^2*y-F(1)/4*v^2*w*x^2*y-2*v*w^2*x^2*y-16*w^3*x^2*y-F(3)/2*v^2*x^3*y-6*v*w*x^3*y-F(3)/2*w^2*x^3*y+18*v*x^4*y+3*w*x^4*y-9*x^5*y+8*v^4*y^2+F(2)/3*v^3*w*y^2-9*v^2*w^2*y^2-v*w^3*y^2-5*w^4*y^2+F(1)/2*v^3*x*y^2-6*v^2*w*x*y^2+6*v*w^2*x*y^2+17*w^3*x*y^2+5*v^2*x^2*y^2-9*v*w*x^2*y^2+8*w^2*x^2*y^2+F(3)/2*v*x^3*y^2-F(3)/2*x^4*y^2+5*v^3*y^3-2*v^2*w*y^3+4*v*w^2*y^3-F(4)/3*w^3*y^3+4*v^2*x*y^3-3*v*w*x*y^3-w^2*x*y^3-12*v*x^2*y^3+F(2)/3*w*x^2*y^3+8*x^3*y^3+18*v^2*y^4+18*v*w*y^4+6*w^2*y^4-9*v*x*y^4-F(1)/4*w*x*y^4-F(1)/2*x^2*y^4+4*v*y^5-F(4)/3*w*y^5+2*x*y^5+8*y^6+v^4*w*z+12*v^3*w^2*z+F(3)/4*v^2*w^3*z-F(1)/3*v*w^4*z-F(1)/4*w^5*z-v^3*w*x*z+7*v^2*w^2*x*z-F(4)/3*v*w^3*x*z+18*w^4*x*z-v^2*w*x^2*z-4*v*w^2*x^2*z-w^3*x^2*z+12*v*w*x^3*z+2*w^2*x^3*z-6*w*x^4*z-F(2)/3*v^4*y*z-8*v^3*w*y*z+F(3)/4*v^2*w^2*y*z+9*v*w^3*y*z-w^4*y*z+4*v^3*x*y*z-F(1)/3*v^2*w*x*y*z-18*v*w^2*x*y*z-F(1)/3*w^3*x*y*z-4*v^2*x^2*y*z+5*v*w*x^2*y*z-17*w^2*x^2*y*z+9*v*x^3*y*z+8*w*x^3*y*z-19*x^4*y*z+v^3*y^2*z+F(1)/2*v^2*w*y^2*z-F(1)/2*v*w^2*y^2*z+12*w^3*y^2*z-17*v^2*x*y^2*z+17*v*w*x*y^2*z-F(4)/3*w^2*x*y^2*z+16*v*x^2*y^2*z+w*x^2*y^2*z-18*x^3*y^2*z+F(4)/3*v^2*y^3*z-12*v*w*y^3*z-w^2*y^3*z-12*v*x*y^3*z+2*w*x*y^3*z+x^2*y^3*z+3*v*y^4*z-F(3)/2*w*y^4*z+12*y^5*z+16*v^4*z^2+3*v^3*w*z^2+9*v^2*w^2*z^2+8*v*w^3*z^2+7*w^4*z^2-8*v^3*x*z^2+6*v^2*w*x*z^2+17*v*w^2*x*z^2+12*w^3*x*z^2+F(3)/2*v^2*x^2*z^2-12*v*w*x^2*z^2-9*w^2*x^2*z^2-9*v*x^3*z^2+F(3)/2*w*x^3*z^2+6*x^4*z^2+F(1)/3*v^3*y*z^2+6*v^2*w*y*z^2+5*v*w^2*y*z^2+F(2)/3*w^3*y*z^2+16*v^2*x*y*z^2+6*v*w*x*y*z^2-6*w^2*x*y*z^2+F(1)/3*v*x^2*y*z^2-17*w*x^2*y*z^2-3*x^3*y*z^2-7*v^2*y^2*z^2+3*v*w*y^2*z^2-16*w^2*y^2*z^2+3*v*x*y^2*z^2-9*w*x*y^2*z^2+19*x^2*y^2*z^2-5*v*y^3*z^2-16*w*y^3*z^2+2*x*y^3*z^2-F(3)/4*y^4*z^2-F(1)/4*v^3*z^3+3*v^2*w*z^3+8*v*w^2*z^3+F(2)/3*w^3*z^3+9*v^2*x*z^3-5*v*w*x*z^3+7*w^2*x*z^3+17*v*x^2*z^3+F(1)/4*w*x^2*z^3-F(1)/4*x^3*z^3-F(1)/4*v^2*y*z^3-F(3)/2*v*w*y*z^3+2*w^2*y*z^3-9*v*x*y*z^3-8*w*x*y*z^3+12*x^2*y*z^3+17*v*y^2*z^3+F(1)/4*w*y^2*z^3+F(3)/4*x*y^2*z^3+9*y^3*z^3-9*v^2*z^4+5*v*w*z^4+5*w^2*z^4+v*x*z^4+F(1)/3*w*x*z^4-F(1)/2*x^2*z^4-2*w*y*z^4+F(4)/3*x*y*z^4-3*y^2*z^4-2*w*z^5-F(2)/3*x*z^5+16*y*z^5-5*z^6,-3*v^4*w*y+F(1)/3*v^3*w^2*y+9*v^2*w^3*y-17*v*w^4*y+F(3)/2*w^5*y-F(1)/4*v^4*x*y-4*v^3*w*x*y-F(4)/3*v^2*w^2*x*y+17*v*w^3*x*y+6*w^4*x*y-17*v^3*x^2*y+F(1)/2*v^2*w*x^2*y+17*v*w^2*x^2*y-8*w^3*x^2*y+3*v^2*x^3*y-2*v*w*x^3*y+F(1)/4*w^2*x^3*y+F(3)/2*v*x^4*y+w*x^4*y+3*x^5*y+F(3)/4*v^4*y^2-5*v^3*w*y^2+19*v^2*w^2*y^2+2*v*w^3*y^2-18*w^4*y^2-F(1)/3*v^3*x*y^2-F(2)/3*v^2*w*x*y^2-5*v*w^2*x*y^2-2*w^3*x*y^2+19*v^2*x^2*y^2-F(3)/2*v*w*x^2*y^2-17*w^2*x^2*y^2+19*v*x^3*y^2+19*w*x^3*y^2+x^4*y^2-17*v^3*y^3-2*v^2*w*y^3-v*w^2*y^3+7*w^3*y^3+F(3)/4*v^2*x*y^3-F(1)/3*v*w*x*y^3-4*w^2*x*y^3+17*v*x^2*y^3-F(1)/2*w*x^2*y^3+5*x^3*y^3+F(3)/2*v^2*y^4-5*v*w*y^4-8*w^2*y^4+F(1)/3*v*x*y^4-F(2)/3*w*x*y^4-18*x^2*y^4+4*v*y^5-F(2)/3*w*y^5-17*x*y^5-F(2)/3*y^6+18*v^4*x*z-9*v^3*w*x*z+F(2)/3*v*w^3*x*z+3*w^4*x*z-F(1)/4*v^3*x^2*z-16*v^2*w*x^2*z+5*v*w^2*x^2*z-F(3)/2*w^3*x^2*z-F(1)/2*v^2*x^3*z+F(1)/2*v*w*x^3*z-w^2*x^3*z-F(4)/3*v*x^4*z+7*w*x^4*z+F(2)/3*x^5*z-16*v^4*y*z-v^3*w*y*z+18*v^2*w^2*y*z-18*v*w^3*y*z-9*w^4*y*z-F(1)/4*v^3*x*y*z-8*v^2*w*x*y*z+F(4)/3*v*w^2*x*y*z+9*w^3*x*y*z-v^2*x^2*y*z+17*v*w*x^2*y*z-18*w^2*x^2*y*z-4*v*x^3*y*z+12*w*x^3*y*z+F(1)/3*x^4*y*z+19*v^3*y^2*z+4*v^2*w*y^2*z-2*v*w^2*y^2*z+F(2)/3*w^3*y^2*z-2*v^2*x*y^2*z+6*v*w*x*y^2*z+F(3)/4*w^2*x*y^2*z+3*v*x^2*y^2*z+F(3)/2*w*x^2*y^2*z-F(1)/4*x^3*y^2*z+F(1)/4*v^2*y^3*z-16*v*w*y^3*z+12*w^2*y^3*z+7*v*x*y^3*z-2*w*x*y^3*z+17*x^2*y^3*z-F(3)/2*v*y^4*z-2*w*y^4*z-F(1)/3*x*y^4*z-F(1)/4*y^5*z+2*v^4*z^2+F(1)/4*v^3*w*z^2+19*v^2*w^2*z^2-17*v*w^3*z^2+8*w^4*z^2-12*v^3*x*z^2-F(4)/3*v^2*w*x*z^2+F(4)/3*v*w^2*x*z^2-F(3)/4*w^3*x*z^2+F(2)/3*v^2*x^2*z^2-v*w*x^2*z^2-w^2*x^2*z^2-5*v*x^3*z^2+F(2)/3*w*x^3*z^2-9*x^4*z^2+v^3*y*z^2-F(3)/4*v*w^2*y*z^2+F(4)/3*v^2*x*y*z^2-8*v*w*x*y*z^2-F(2)/3*w^2*x*y*z^2-17*v*x^2*y*z^2-16*w*x^2*y*z^2+12*x^3*y*z^2-3*v^2*y^2*z^2-16*v*w*y^2*z^2-F(1)/3*w^2*y^2*z^2-F(1)/2*v*x*y^2*z^2+7*w*x*y^2*z^2+2*x^2*y^2*z^2-17*v*y^3*z^2+F(2)/3*w*y^3*z^2-F(1)/3*x*y^3*z^2+y^4*z^2+F(4)/3*v^3*z^3+F(1)/4*v^2*w*z^3-8*v*w^2*z^3+F(1)/2*w^3*z^3+18*v^2*x*z^3-6*v*w*x*z^3+16*w^2*x*z^3-9*v*x^2*z^3-2*w*x^2*z^3+16*x^3*z^3-F(1)/2*v^2*y*z^3-F(1)/3*v*w*y*z^3+F(2)/3*w^2*y*z^3-F(1)/2*v*x*y*z^3+F(2)/3*w*x*y*z^3+F(1)/3*x^2*y*z^3+18*v*y^2*z^3+F(2)/3*w*y^2*z^3-F(1)/2*x*y^2*z^3-F(3)/2*y^3*z^3-F(3)/4*v^2*z^4-7*v*w*z^4-8*w^2*z^4+19*v*x*z^4-3*w*x*z^4+x^2*z^4+9*v*y*z^4-2*w*y*z^4+6*x*y*z^4+8*y^2*z^4-F(4)/3*v*z^5+7*w*z^5-9*x*z^5-F(1)/3*y*z^5-5*z^6,-F(3)/2*v^4*w*y-18*v^3*w^2*y+F(2)/3*v^2*w^3*y+F(1)/2*v*w^4*y-5*w^5*y+F(3)/2*v^3*w*x*y+F(1)/4*v^2*w^2*x*y+2*v*w^3*x*y+16*w^4*x*y+F(3)/2*v^2*w*x^2*y+6*v*w^2*x^2*y+F(3)/2*w^3*x^2*y-18*v*w*x^3*y-3*w^2*x^3*y+9*w*x^4*y+F(1)/4*v^4*y^2+5*v^3*w*y^2-F(3)/2*v^2*w^2*y^2-v*w^3*y^2-7*w^4*y^2+17*v^3*x*y^2-v^2*w*x*y^2-6*v*w^2*x*y^2-16*w^3*x*y^2-3*v^2*x^2*y^2-F(4)/3*v*w*x^2*y^2+9*w^2*x^2*y^2-F(3)/2*v*x^3*y^2+7*w*x^3*y^2-3*x^4*y^2-v^3*y^3-17*v^2*w*y^3+5*v*w^2*y^3+F(1)/4*w^3*y^3-F(4)/3*v^2*x*y^3-7*v*w*x*y^3-F(1)/3*w^2*x*y^3+7*v*x^2*y^3+17*w*x^2*y^3+16*x^3*y^3+F(1)/2*v^2*y^4-F(4)/3*v*w*y^4+6*w^2*y^4-4*v*x*y^4-9*w*x*y^4+12*v*y^5-F(1)/3*w*y^5-8*x*y^5+6*y^6+7*v^4*w*z+4*v^3*w^2*z+F(1)/2*v^2*w^3*z+F(1)/4*v*w^4*z+18*w^5*z-3*v^4*x*z-5*v^3*w*x*z-16*v^2*w^2*x*z-F(1)/4*v*w^3*x*z+F(1)/3*w^4*x*z+F(1)/4*v^3*x^2*z+6*v^2*w*x^2*z-F(1)/4*v*w^2*x^2*z+F(3)/4*w^3*x^2*z-7*v^2*x^3*z+19*v*w*x^3*z+3*w^2*x^3*z+18*v*x^4*z+12*w*x^4*z-7*x^5*z+F(1)/2*v^4*y*z+3*v^3*w*y*z-F(1)/2*v^2*w^2*y*z+7*v*w^3*y*z+9*w^4*y*z-17*v^3*x*y*z+17*v^2*w*x*y*z+6*v*w^2*x*y*z+F(3)/2*w^3*x*y*z-7*v^2*x^2*y*z+7*v*w*x^2*y*z-F(1)/4*w^2*x^2*y*z+F(2)/3*v*x^3*y*z+12*w*x^3*y*z-2*x^4*y*z-F(4)/3*v^3*y^2*z-7*v^2*w*y^2*z+9*v*w^2*y^2*z+5*w^3*y^2*z+3*v^2*x*y^2*z+17*v*w*x*y^2*z+2*w^2*x*y^2*z-v*x^2*y^2*z-F(1)/3*w*x^2*y^2*z+x^3*y^2*z+16*v^2*y^3*z-F(1)/3*v*w*y^3*z+5*w^2*y^3*z-F(1)/2*v*x*y^3*z+6*w*x*y^3*z+18*x^2*y^3*z-16*v*y^4*z-8*w*y^4*z+2*x*y^4*z+9*y^5*z-6*v^4*z^2+F(1)/3*v^3*w*z^2-F(4)/3*v^2*w^2*z^2-17*v*w^3*z^2-8*w^4*z^2+F(3)/2*v^3*x*z^2+18*v^2*w*x*z^2-9*v*w^2*x*z^2+F(2)/3*w^3*x*z^2+16*v^2*x^2*z^2-F(3)/2*v*w*x^2*z^2-F(3)/4*w^2*x^2*z^2-3*v*x^3*z^2+3*w*x^3*z^2+9*x^4*z^2-17*v^3*y*z^2-16*v^2*w*y*z^2+F(3)/2*v*w^2*y*z^2-7*w^3*y*z^2+2*v^2*x*y*z^2-F(3)/4*v*w*x*y*z^2-4*w^2*x*y*z^2-8*v*x^2*y*z^2-F(3)/4*w*x^2*y*z^2-7*x^3*y*z^2-3*v^2*y^2*z^2+F(1)/2*v*w*y^2*z^2+F(1)/4*w^2*y^2*z^2+8*v*x*y^2*z^2-18*w*x*y^2*z^2+12*x^2*y^2*z^2-3*v*y^3*z^2+7*w*y^3*z^2+F(4)/3*x*y^3*z^2+16*y^4*z^2-19*v^3*z^3+F(1)/3*v^2*w*z^3+18*v*w^2*z^3-17*w^3*z^3+5*v*w*x*z^3+w^2*x*z^3+7*v*x^2*z^3+2*w*x^2*z^3+17*x^3*z^3-7*v^2*y*z^3-18*v*w*y*z^3+5*w^2*y*z^3-16*v*x*y*z^3+F(4)/3*w*x*y*z^3-F(1)/2*x^2*y*z^3+3*w*y^2*z^3-16*x*y^2*z^3-F(3)/2*y^3*z^3+F(1)/3*v^2*z^4-F(3)/2*v*w*z^4+17*w^2*z^4+7*v*x*z^4-F(3)/2*w*x*z^4-3*x^2*z^4-18*v*y*z^4+3*w*y*z^4+F(3)/4*x*y*z^4-6*y^2*z^4+7*v*z^5-F(1)/4*w*z^5+4*x*z^5+19*y*z^5-6*z^6,v^4*w^2-17*v^3*w^3+5*v*w^5-5*w^6-4*v^3*w^2*x-F(1)/3*v^2*w^3*x-F(4)/3*v*w^4*x+F(1)/3*w^5*x-12*v^2*w^2*x^2+9*v*w^3*x^2-F(1)/3*w^4*x^2-19*v*w^2*x^3+4*w^3*x^3-3*w^2*x^4+v^4*w*y-F(3)/2*v^3*w^2*y+12*v^2*w^3*y+8*v*w^4*y-F(1)/2*w^5*y-F(2)/3*v^3*w*x*y+F(1)/3*v^2*w^2*x*y+16*v*w^3*x*y-18*w^4*x*y-3*v^2*w*x^2*y+v*w^2*x^2*y+18*w^3*x^2*y-17*v*w*x^3*y-F(3)/2*w^2*x^3*y-7*w*x^4*y+17*v^4*y^2-16*v^3*w*y^2+19*v^2*w^2*y^2-F(1)/2*v*w^3*y^2+8*w^4*y^2-17*v^3*x*y^2-F(1)/3*v^2*w*x*y^2-F(4)/3*v*w^2*x*y^2+16*w^3*x*y^2-17*v^2*x^2*y^2-5*v*w*x^2*y^2+16*w^2*x^2*y^2-F(1)/4*v*x^3*y^2+18*w*x^3*y^2-16*x^4*y^2+F(2)/3*v^3*y^3-18*v^2*w*y^3-7*v*w^2*y^3+F(3)/4*w^3*y^3+F(3)/4*v^2*x*y^3+17*w^2*x*y^3-5*v*x^2*y^3+3*w*x^2*y^3-F(1)/4*x^3*y^3-F(1)/2*v^2*y^4-F(1)/3*v*w*y^4-7*w^2*y^4-5*v*x*y^4-F(3)/2*w*x*y^4+8*x^2*y^4-18*v*y^5+6*w*y^5+F(3)/4*x*y^5+8*y^6+F(1)/2*v^4*w*z+16*v^3*w^2*z+9*v^2*w^3*z-F(1)/3*v*w^4*z-18*w^5*z+F(3)/2*v^4*x*z-3*v^3*w*x*z-9*v^2*w^2*x*z+7*v*w^3*x*z+F(4)/3*w^4*x*z-F(3)/2*v^3*x^2*z+5*v^2*w*x^2*z+7*v*w^2*x^2*z-2*w^3*x^2*z-F(3)/2*v^2*x^3*z-F(1)/2*v*w*x^3*z+F(1)/2*w^2*x^3*z+18*v*x^4*z+4*w*x^4*z-9*x^5*z-F(1)/4*v^4*y*z-F(3)/2*v^3*w*y*z-12*v^2*w^2*y*z-F(3)/2*v*w^3*y*z-16*w^4*y*z+4*v^3*x*y*z+12*v^2*w*x*y*z+8*v*w^2*x*y*z-12*w^3*x*y*z+5*v^2*x^2*y*z+12*v*w*x^2*y*z-w^2*x^2*y*z-18*v*x^3*y*z-F(2)/3*w*x^3*y*z-F(1)/2*x^4*y*z+F(1)/4*v^3*y^2*z-3*v^2*w*y^2*z+F(1)/3*v*w^2*y^2*z-19*w^3*y^2*z+F(1)/4*v^2*x*y^2*z-F(1)/2*v*w*x*y^2*z-8*w^2*x*y^2*z+4*v*x^2*y^2*z-16*w*x^2*y^2*z-F(3)/2*x^3*y^2*z-2*v^2*y^3*z-19*v*w*y^3*z+9*w^2*y^3*z+F(2)/3*v*x*y^3*z+5*w*x*y^3*z+8*x^2*y^3*z+F(1)/4*v*y^4*z-w*y^4*z+F(4)/3*x*y^4*z-16*y^5*z-v^4*z^2+F(1)/2*v^3*w*z^2-v^2*w^2*z^2-F(4)/3*v*w^3*z^2+9*w^4*z^2-F(3)/4*v^3*x*z^2+F(1)/4*v^2*w*x*z^2-9*v*w^2*x*z^2-6*w^3*x*z^2-4*v*w*x^2*z^2+F(1)/2*w^2*x^2*z^2+F(1)/4*v*x^3*z^2-w*x^3*z^2-F(2)/3*x^4*z^2+F(3)/2*v^3*y*z^2-v^2*w*y*z^2-F(4)/3*v*w^2*y*z^2+2*w^3*y*z^2-18*v^2*x*y*z^2-F(3)/4*v*w*x*y*z^2-F(1)/4*w^2*x*y*z^2+8*v*x^2*y*z^2+F(1)/2*w*x^2*y*z^2-F(1)/3*x^3*y*z^2-2*v^2*y^2*z^2+19*v*w*y^2*z^2-18*w^2*y^2*z^2+9*v*x*y^2*z^2-3*w*x*y^2*z^2-8*x^2*y^2*z^2-9*w*y^3*z^2-F(1)/4*x*y^3*z^2-F(1)/4*y^4*z^2+F(3)/2*v^3*z^3+17*v^2*w*z^3-16*v*w^2*z^3-F(4)/3*w^3*z^3-F(1)/2*v^2*x*z^3+12*v*w*x*z^3+F(4)/3*w^2*x*z^3+2*v*x^2*z^3+F(1)/3*w*x^2*z^3+3*x^3*z^3+7*v^2*y*z^3+F(3)/4*v*w*y*z^3-8*w^2*y*z^3+F(4)/3*v*x*y*z^3-8*w*x*y*z^3-F(3)/4*x^2*y*z^3+7*v*y^2*z^3-8*w*y^2*z^3+F(1)/4*x*y^2*z^3-18*y^3*z^3-2*v^2*z^4-4*v*w*z^4-9*w^2*z^4-17*v*x*z^4+18*w*x*z^4-6*x^2*z^4+12*v*y*z^4+5*w*y*z^4-7*x*y*z^4+F(4)/3*y^2*z^4-4*v*z^5-3*w*z^5+8*x*z^5+6*y*z^5+F(2)/3*z^6,v^4*x*y-F(1)/2*v^3*w*x*y+8*v*w^3*x*y-7*w^4*x*y-3*v^3*x^2*y+F(3)/2*v^2*w*x^2*y+17*v*w^2*x^2*y-18*w^3*x^2*y-6*v^2*x^3*y+6*v*w*x^3*y-12*w^2*x^3*y-16*v*x^4*y-2*w*x^4*y+8*x^5*y+16*v^4*y^2-F(3)/4*v^3*w*y^2-17*v^2*w^2*y^2-6*v*w^3*y^2+16*w^4*y^2+4*v^3*x*y^2+5*v^2*w*x*y^2+F(1)/3*v*w^2*x*y^2-18*w^3*x*y^2+F(1)/3*v^2*x^2*y^2+v*w*x^2*y^2-9*w^2*x^2*y^2-16*v*x^3*y^2-w*x^3*y^2+12*x^4*y^2-2*v^3*y^3-19*v^2*w*y^3+F(1)/4*v*w^2*y^3-8*w^3*y^3+5*v^2*x*y^3+F(1)/3*v*w*x*y^3+18*w^2*x*y^3-v*x^2*y^3-7*w*x^2*y^3+7*x^3*y^3-7*v^2*y^4+2*v*w*y^4+F(1)/3*w^2*y^4-2*v*x*y^4-F(1)/3*w*x*y^4-x^2*y^4+16*v*y^5+17*w*y^5+F(3)/4*x*y^5-3*y^6-12*v^4*x*z-F(1)/3*v^3*w*x*z+2*v^2*w^2*x*z-17*v*w^3*x*z-12*w^4*x*z-12*v^3*x^2*z+4*v^2*w*x^2*z+7*v*w^2*x^2*z-F(3)/4*w^3*x^2*z+3*v^2*x^3*z-F(4)/3*v*w*x^3*z-18*w^2*x^3*z-3*v*x^4*z-12*w*x^4*z+19*x^5*z+9*v^4*y*z-9*v^3*w*y*z+3*v^2*w^2*y*z+F(1)/2*v*w^3*y*z-w^4*y*z+F(2)/3*v^3*x*y*z+8*v^2*w*x*y*z-4*v*w^2*x*y*z-3*w^3*x*y*z+F(3)/2*v^2*x^2*y*z-17*v*w*x^2*y*z+F(1)/4*w^2*x^2*y*z+v*x^3*y*z-4*w*x^3*y*z+F(4)/3*x^4*y*z+v^3*y^2*z-F(2)/3*v^2*w*y^2*z-6*v*w^2*y^2*z+4*w^3*y^2*z-17*v^2*x*y^2*z-18*v*w*x*y^2*z-19*w^2*x*y^2*z+F(1)/4*v*x^2*y^2*z+16*w*x^2*y^2*z-6*x^3*y^2*z-17*v^2*y^3*z-7*v*w*y^3*z-6*w^2*y^3*z+3*v*x*y^3*z+9*w*x*y^3*z-17*x^2*y^3*z+F(3)/2*v*y^4*z+F(1)/3*w*y^4*z-F(2)/3*x*y^4*z+6*y^5*z+v^4*z^2-7*v^3*w*z^2-8*v^2*w^2*z^2+2*v*w^3*z^2+7*w^4*z^2-F(3)/2*v^3*x*z^2-F(3)/2*v^2*w*x*z^2-F(1)/3*v*w^2*x*z^2+7*w^3*x*z^2+F(1)/2*v^2*x^2*z^2-12*v*w*x^2*z^2+19*w^2*x^2*z^2+6*w*x^3*z^2-F(1)/4*x^4*z^2+5*v^2*w*y*z^2-F(4)/3*v*w^2*y*z^2+9*w^3*y*z^2+18*v^2*x*y*z^2+8*v*w*x*y*z^2+2*w^2*x*y*z^2+F(1)/4*v*x^2*y*z^2+F(3)/2*w*x^2*y*z^2+7*x^3*y*z^2-17*v^2*y^2*z^2+F(1)/3*v*w*y^2*z^2-F(1)/4*w^2*y^2*z^2-5*v*x*y^2*z^2-6*x^2*y^2*z^2+3*v*y^3*z^2+F(1)/4*w*y^3*z^2+2*x*y^3*z^2-F(4)/3*y^4*z^2-v^3*z^3-F(1)/2*v^2*w*z^3-5*v*w^2*z^3-16*w^3*z^3+6*v^2*x*z^3+16*v*w*x*z^3-19*w^2*x*z^3-3*v*x^2*z^3-F(1)/2*w*x^2*z^3-12*x^3*z^3+F(4)/3*v^2*y*z^3+F(1)/4*v*w*y*z^3+F(2)/3*w^2*y*z^3-F(3)/4*v*x*y*z^3-17*w*x*y*z^3+19*x^2*y*z^3+F(1)/3*v*y^2*z^3-F(3)/4*w*y^2*z^3-16*x*y^2*z^3+6*y^3*z^3+19*v^2*z^4-F(1)/4*v*w*z^4-F(1)/3*w^2*z^4+3*v*x*z^4+F(2)/3*w*x*z^4-8*x^2*z^4+3*v*y*z^4+2*w*y*z^4+17*x*y*z^4+y^2*z^4-16*v*z^5-12*w*z^5+9*x*z^5+y*z^5-F(3)/4*z^6,-v^5+17*v^4*w-5*v^2*w^3+5*v*w^4-F(1)/3*v^4*x-19*v^3*w*x+F(4)/3*v^2*w^2*x+8*v*w^3*x+16*w^4*x-18*v^3*x^2+6*v^2*w*x^2-16*v*w^2*x^2-8*w^3*x^2+2*v^2*x^3-F(4)/3*v*w*x^3+9*w^2*x^3+F(2)/3*v*x^4+F(3)/2*w*x^4-6*x^5+9*v^4*y+F(4)/3*v^3*w*y+2*v^2*w^2*y+12*v*w^3*y+2*w^4*y+12*v^3*x*y-F(4)/3*v^2*w*x*y+F(1)/4*v*w^2*x*y-F(1)/2*v^2*x^2*y-v*w*x^2*y+9*w^2*x^2*y-16*v*x^3*y-F(3)/2*w*x^3*y+F(1)/2*x^4*y+5*v^3*y^2-12*v^2*w*y^2+F(1)/3*v*w^2*y^2+18*w^3*y^2-17*v^2*x*y^2-5*v*w*x*y^2+F(3)/4*w^2*x*y^2-12*v*x^2*y^2+2*w*x^2*y^2-F(1)/4*x^3*y^2+F(3)/2*v^2*y^3-6*v*w*y^3-F(1)/2*w^2*y^3+16*v*x*y^3-F(1)/4*w*x*y^3-9*x^2*y^3-12*v*y^4-F(4)/3*w*y^4+x*y^4+F(2)/3*y^5-18*v^4*z+F(4)/3*v^3*w*z-F(3)/4*v^2*w^2*z-12*v*w^3*z-F(3)/4*v^3*x*z+F(1)/4*v^2*w*x*z-v*w^2*x*z+F(2)/3*w^3*x*z-16*v^2*x^2*z+19*v*w*x^2*z-4*w^2*x^2*z+6*v*x^3*z-F(3)/2*w*x^3*z+3*x^4*z+F(3)/2*v^3*y*z-F(3)/2*v^2*w*y*z-16*v*w^2*y*z-3*w^3*y*z+7*v^2*x*y*z+F(3)/4*w^2*x*y*z+7*v*x^2*y*z+w*x^2*y*z-5*x^3*y*z+16*v^2*y^2*z-F(3)/4*v*w*y^2*z-18*w^2*y^2*z+8*v*x*y^2*z-F(4)/3*w*x*y^2*z-F(3)/2*x^2*y^2*z-F(2)/3*v*y^3*z-8*w*y^3*z+5*x*y^3*z-2*y^4*z-8*v^3*z^2+F(1)/3*v^2*w*z^2+3*v*w^2*z^2+F(1)/4*w^3*z^2+16*v^2*x*z^2+F(1)/3*v*w*x*z^2-2*w^2*x*z^2+18*v*x^2*z^2-3*w*x^2*z^2+F(1)/4*x^3*z^2-5*v*w*y*z^2-7*w^2*y*z^2-F(1)/2*v*x*y*z^2+16*w*x*y*z^2-F(2)/3*x^2*y*z^2+v*y^2*z^2-2*w*y^2*z^2+6*x*y^2*z^2-F(1)/4*y^3*z^2-2*v^2*z^3-F(1)/4*v*w*z^3-F(2)/3*w^2*z^3+F(2)/3*v*x*z^3-9*w*x*z^3-8*x^2*z^3+F(3)/2*v*y*z^3+8*w*y*z^3-F(1)/2*x*y*z^3+4*y^2*z^3+17*v*z^4+8*w*z^4+F(3)/2*x*z^4-F(1)/2*y*z^4-8*z^5,-F(3)/2*v^5-19*v^4*w-F(1)/4*v^3*w^2+F(1)/2*v^2*w^3+F(3)/4*v*w^4+5*w^5+F(3)/2*v^4*x+F(2)/3*v^3*w*x-12*v^2*w^2*x+3*v*w^3*x-F(1)/3*w^4*x+F(3)/2*v^3*x^2+18*v^2*w*x^2-F(1)/3*v*w^2*x^2+F(1)/3*w^3*x^2-18*v^2*x^3+16*v*w*x^3-4*w^2*x^3+9*v*x^4+3*w*x^4-16*v^4*y-18*v^3*w*y-3*v^2*w^2*y+F(3)/2*v*w^3*y+F(1)/4*w^4*y-9*v^3*x*y+F(1)/4*v^2*w*x*y+F(2)/3*v*w^2*x*y-F(2)/3*w^3*x*y-9*v^2*x^2*y-6*v*w*x^2*y-18*w^2*x^2*y-F(2)/3*v*x^3*y+7*w*x^3*y-3*x^4*y-F(3)/4*v^3*y^2+F(3)/2*v^2*w*y^2+9*v*w^2*y^2+17*w^3*y^2-8*v^2*x*y^2+19*v*w*x*y^2+6*w^2*x*y^2-3*v*x^2*y^2-F(2)/3*w*x^2*y^2-F(1)/4*x^3*y^2-5*v^2*y^3+17*v*w*y^3-8*w^2*y^3-19*v*x*y^3+F(1)/3*w*x*y^3-3*x^2*y^3-9*v*y^4+3*w*y^4+7*x*y^4+F(3)/4*y^5+16*v^4*z+5*v^3*w*z-16*v^2*w^2*z-6*v*w^3*z-5*w^4*z+F(3)/4*v^3*x*z-v^2*w*x*z-18*v*w^2*x*z-F(1)/4*w^3*x*z-F(3)/4*v^2*x^2*z-3*v*w*x^2*z-F(4)/3*w^2*x^2*z+F(1)/3*v*x^3*z-19*w*x^3*z-19*x^4*z-4*v^3*y*z+F(1)/4*v^2*w*y*z+19*v*w^2*y*z+F(2)/3*w^3*y*z-F(1)/2*v^2*x*y*z+17*w^2*x*y*z-F(3)/4*v*x^2*y*z+12*w*x^2*y*z+7*x^3*y*z+9*v^2*y^2*z+5*v*w*y^2*z+18*w^2*y^2*z-3*v*x*y^2*z-w*x*y^2*z-8*x^2*y^2*z-F(2)/3*v*y^3*z-w*y^3*z-F(1)/3*x*y^3*z-7*y^4*z+9*v^3*z^2-3*v^2*w*z^2+17*v*w^2*z^2-19*w^3*z^2-18*v^2*x*z^2+9*v*w*x*z^2+5*w^2*x*z^2+19*v*x^2*z^2-6*w*x^2*z^2-6*x^3*z^2+7*v^2*y*z^2+F(4)/3*v*w*y*z^2-w^2*y*z^2-4*v*x*y*z^2+F(1)/4*w*x*y*z^2-7*x^2*y*z^2-8*v*y^2*z^2+F(1)/4*w*y^2*z^2-F(1)/2*x*y^2*z^2-8*y^3*z^2-F(1)/4*v^2*z^3-F(4)/3*v*w*z^3+F(1)/3*w^2*z^3-8*v*x*z^3-9*w*x*z^3-19*x^2*z^3-F(1)/3*v*y*z^3-2*w*y*z^3-4*x*y*z^3-8*y^2*z^3+F(4)/3*v*z^4-16*w*z^4-F(4)/3*x*z^4-F(4)/3*y*z^4-F(2)/3*z^5,16*v^5+12*v^4*w+7*v^3*w^2+17*v^2*w^3+v*w^4-F(1)/2*w^5+16*v^4*x+7*v^3*w*x+19*v^2*w^2*x-16*v*w^3*x-7*w^4*x-4*v^3*x^2-5*v^2*w*x^2+16*v*w^2*x^2-2*w^3*x^2+4*v^2*x^3-3*v*w*x^3+4*w^2*x^3-F(1)/4*v*x^4-12*w*x^4+F(1)/4*v^4*y+8*v^3*w*y-9*v^2*w^2*y-F(3)/4*v*w^3*y-2*w^4*y-9*v^3*x*y+4*v^2*w*x*y-8*v*w^2*x*y-7*w^3*x*y-17*v^2*x^2*y-19*v*w*x^2*y+6*w^2*x^2*y-6*v*x^3*y+w*x^3*y-x^4*y+12*v^3*y^2-7*v^2*w*y^2-6*v*w^2*y^2+8*w^3*y^2+F(3)/4*v^2*x*y^2+v*w*x*y^2+6*w^2*x*y^2+16*v*x^2*y^2+F(4)/3*w*x^2*y^2+3*x^3*y^2+F(2)/3*v^2*y^3+F(4)/3*v*w*y^3+5*w^2*y^3+F(2)/3*v*x*y^3-F(1)/3*w*x*y^3-18*x^2*y^3+F(1)/2*v*y^4+F(1)/2*w*y^4-19*x*y^4+F(1)/3*y^5+v^4*z-3*v^3*w*z-6*v^2*w^2*z-v*w^3*z-w^4*z-8*v^3*x*z+4*v^2*w*x*z-3*v*w^2*x*z+w^3*x*z+9*v^2*x^2*z-9*v*w*x^2*z+F(3)/2*w^2*x^2*z+9*v*x^3*z+6*w*x^3*z+19*x^4*z+6*v^3*y*z-7*v^2*w*y*z-F(1)/4*v*w^2*y*z-3*w^3*y*z-3*v^2*x*y*z+F(1)/4*v*x^2*y*z+F(1)/4*w*x^2*y*z+F(1)/3*x^3*y*z+7*v^2*y^2*z+F(2)/3*v*w*y^2*z-19*w^2*y^2*z+F(1)/4*v*x*y^2*z+F(2)/3*w*x*y^2*z+F(4)/3*x^2*y^2*z-12*v*y^3*z-F(2)/3*w*y^3*z+5*x*y^3*z-F(4)/3*y^4*z-19*v^3*z^2-F(1)/3*v*w^2*z^2+9*w^3*z^2-F(2)/3*v^2*x*z^2-v*w*x*z^2+5*w^2*x*z^2+5*v*x^2*z^2+16*w*x^2*z^2+x^3*z^2-4*v^2*y*z^2-9*v*w*y*z^2+F(1)/2*w^2*y*z^2-8*v*x*y*z^2+F(3)/4*w*x*y*z^2-F(1)/2*x^2*y*z^2-F(3)/4*v*y^2*z^2-F(2)/3*w*y^2*z^2+F(1)/4*x*y^2*z^2-18*y^3*z^2+3*v^2*z^3-F(1)/2*v*w*z^3-17*w^2*z^3-16*v*x*z^3+16*w*x*z^3+2*x^2*z^3-F(4)/3*v*y*z^3+9*w*y*z^3+F(1)/3*x*y*z^3+F(3)/2*y^2*z^3+16*v*z^4-F(3)/4*w*z^4+17*x*z^4+12*y*z^4-F(3)/2*z^5,-v^5-F(3)/4*v^4*w-F(1)/3*v^3*w^2-6*v^2*w^3+F(3)/4*v*w^4-12*w^5+18*v^4*x+16*v^3*w*x+9*v^2*w^2*x-F(3)/2*v*w^3*x+17*w^4*x-9*v^3*x^2+16*v^2*w*x^2+5*v*w^2*x^2-6*w^3*x^2+v^2*x^3-18*v*w*x^3+16*w^2*x^3+6*w*x^4-4*x^5+F(1)/2*v^4*y+F(1)/3*v^3*w*y+v^2*w^2*y+7*v*w^3*y-w^4*y+6*v^3*x*y-F(1)/4*v^2*w*x*y+17*v*w^2*x*y+5*w^3*x*y+5*v^2*x^2*y-19*v*w*x^2*y-F(1)/3*w^2*x^2*y+F(3)/4*v*x^3*y+F(1)/4*w*x^3*y-F(1)/2*x^4*y+v^3*y^2-19*v^2*w*y^2-F(1)/3*v*w^2*y^2-F(1)/4*w^3*y^2-F(1)/3*v^2*x*y^2-v*w*x*y^2-5*w^2*x*y^2-F(4)/3*v*x^2*y^2+19*w*x^2*y^2-6*x^3*y^2-F(2)/3*v^2*y^3-v*w*y^3-8*w^2*y^3+8*v*x*y^3+F(1)/4*w*x*y^3-18*x^2*y^3+F(1)/3*v*y^4+16*w*y^4+16*x*y^4+F(1)/2*y^5+16*v^4*z-2*v^3*w*z+F(3)/2*v^2*w^2*z-F(2)/3*v*w^3*z-5*w^4*z-16*v^3*x*z-2*v^2*w*x*z-2*v*w^2*x*z+F(2)/3*w^3*x*z+6*v^2*x^2*z-5*v*w*x^2*z-3*w^2*x^2*z-8*v*x^3*z-F(2)/3*w*x^3*z-6*x^4*z+6*v^3*y*z+F(1)/4*v^2*w*y*z+F(1)/3*v*w^2*y*z-F(1)/4*w^3*y*z+3*v^2*x*y*z+6*w^2*x*y*z+12*v*x^2*y*z+17*w*x^2*y*z+19*x^3*y*z+3*v^2*y^2*z-7*v*w*y^2*z+9*w^2*y^2*z+9*v*x*y^2*z+3*w*x*y^2*z+F(1)/3*x^2*y^2*z-18*v*y^3*z-F(1)/4*w*y^3*z+F(1)/4*x*y^3*z-6*y^4*z+F(3)/4*v^3*z^2+6*v^2*w*z^2-17*v*w^2*z^2+7*w^3*z^2+16*v^2*x*z^2-17*v*w*x*z^2-F(1)/3*w^2*x*z^2+F(1)/2*v*x^2*z^2+12*w*x^2*z^2-6*x^3*z^2+2*v^2*y*z^2-19*v*w*y*z^2-6*w^2*y*z^2+17*v*x*y*z^2+F(3)/2*w*x*y*z^2+3*x^2*y*z^2-F(4)/3*v*y^2*z^2+16*w*y^2*z^2-F(1)/3*x*y^2*z^2+8*y^3*z^2-19*v^2*z^3+3*v*w*z^3-2*w^2*z^3-v*x*z^3-F(1)/3*w*x*z^3-18*x^2*z^3+3*v*y*z^3+16*w*y*z^3+12*x*y*z^3+19*y^2*z^3+12*v*z^4+3*w*z^4+F(1)/4*x*z^4+F(1)/3*y*z^4-5*z^5,F(1)/4*v^5+F(3)/2*v^4*w-5*v^3*w^2+F(3)/4*v^2*w^3+F(1)/3*v*w^4+F(3)/4*w^5+v^4*x+4*v^3*w*x-3*v^2*w^2*x-F(3)/4*v*w^3*x+6*w^4*x-19*v^3*x^2-2*v^2*w*x^2-16*v*w^2*x^2+6*w^3*x^2-19*v^2*x^3-6*v*w*x^3+F(1)/3*w^2*x^3-7*v*x^4+18*w*x^4+F(1)/4*x^5+8*v^4*y+16*v^3*w*y+8*v^2*w^2*y-5*v*w^3*y-F(2)/3*w^4*y-19*v^3*x*y-F(3)/4*v^2*w*x*y-8*v*w^2*x*y-16*w^3*x*y-2*v^2*x^2*y+18*v*w*x^2*y+F(3)/4*w^2*x^2*y+v*x^3*y+9*w*x^3*y-F(3)/4*x^4*y-12*v^3*y^2+v^2*w*y^2+F(2)/3*v*w^2*y^2-12*w^3*y^2-3*v^2*x*y^2-F(4)/3*v*w*x*y^2+16*w^2*x*y^2+16*v*x^2*y^2+3*w*x^2*y^2+16*x^3*y^2-3*v^2*y^3+18*v*w*y^3+3*v*x*y^3+6*w*x*y^3+F(3)/2*x^2*y^3-F(2)/3*v*y^4+6*w*y^4-16*x*y^4-12*y^5-6*v^4*z+16*v^3*w*z-F(4)/3*v^2*w^2*z-18*v*w^3*z+F(2)/3*w^4*z+18*v^3*x*z+4*v^2*w*x*z-F(1)/4*v*w^2*x*z+w^3*x*z-19*v^2*x^2*z-F(3)/2*v*w*x^2*z-17*w^2*x^2*z-6*v*x^3*z+F(1)/3*w*x^3*z-16*x^4*z-6*v^3*y*z-F(1)/2*v^2*w*y*z+6*v*w^2*y*z-w^3*y*z+4*v^2*x*y*z+6*w^2*x*y*z+17*v*x^2*y*z+w*x^2*y*z+F(4)/3*x^3*y*z+2*v^2*y^2*z+F(1)/4*v*w*y^2*z+19*w^2*y^2*z+F(1)/4*v*x*y^2*z-17*w*x*y^2*z+F(3)/4*x^2*y^2*z-6*v*y^3*z+F(1)/3*w*y^3*z+6*x*y^3*z-19*y^4*z+12*v^3*z^2+5*v^2*w*z^2+19*v*w^2*z^2-F(1)/2*w^3*z^2-3*v^2*x*z^2-3*v*w*x*z^2-12*w^2*x*z^2-5*v*x^2*z^2-3*w*x^2*z^2-F(4)/3*x^3*z^2+19*v^2*y*z^2+F(1)/4*v*w*y*z^2-2*w^2*y*z^2-5*v*x*y*z^2-w*x*y*z^2+8*x^2*y*z^2+F(1)/2*v*y^2*z^2-4*w*y^2*z^2+19*x*y^2*z^2-3*y^3*z^2+v^2*z^3+F(1)/4*v*w*z^3-18*w^2*z^3-8*v*x*z^3+8*w*x*z^3+12*x^2*z^3+2*v*y*z^3+12*w*y*z^3-5*x*y*z^3-F(1)/2*y^2*z^3+9*v*z^4-F(4)/3*w*z^4-16*x*z^4-3*y*z^4,3*v^5-F(1)/3*v^4*w-9*v^3*w^2+17*v^2*w^3-F(3)/2*v*w^4+16*v^4*x-4*v^3*w*x-7*v^2*w^2*x+F(4)/3*v*w^3*x+F(3)/2*w^4*x-12*v^3*x^2-9*v^2*w*x^2-3*v*w^2*x^2-7*w^3*x^2-5*v*w*x^3+3*w^2*x^3+7*v*x^4+6*w*x^4-12*x^5-18*v^4*y-2*v^3*w*y+F(1)/4*v^2*w^2*y-9*v*w^3*y-F(2)/3*w^4*y+2*v^3*x*y-4*v^2*w*x*y-16*v*w^2*x*y+F(3)/2*w^3*x*y+5*v^2*x^2*y+9*v*w*x^2*y-5*w^2*x^2*y+F(4)/3*v*x^3*y+12*w*x^3*y+6*x^4*y+F(4)/3*v^3*y^2-F(2)/3*v^2*w*y^2+F(1)/3*v*w^2*y^2-5*w^3*y^2-6*v^2*x*y^2-7*v*w*x*y^2+17*w^2*x*y^2-9*v*x^2*y^2-2*w*x^2*y^2-F(1)/2*x^3*y^2+9*v^2*y^3-4*v*w*y^3-F(1)/3*w^2*y^3-8*v*x*y^3+F(1)/3*w*x*y^3-8*x^2*y^3+9*v*y^4-18*w*y^4-F(4)/3*x*y^4-9*y^5+F(1)/4*v^4*z+7*v^3*w*z+4*v^2*w^2*z+19*v*w^3*z-F(1)/2*w^4*z-2*v^3*x*z-F(4)/3*v^2*w*x*z-F(2)/3*v*w^2*x*z-F(1)/3*w^3*x*z+18*v^2*x^2*z-3*v*w*x^2*z+4*w^2*x^2*z-18*v*x^3*z-F(1)/2*w*x^3*z-F(4)/3*x^4*z-12*v^3*y*z-6*v^2*w*y*z-F(2)/3*v*w^2*y*z+18*w^3*y*z+8*v^2*x*y*z+17*w^2*x*y*z-16*v*x^2*y*z-5*w*x^2*y*z+7*x^3*y*z+4*v^2*y^2*z-6*v*w*y^2*z-F(1)/4*w^2*y^2*z+3*v*x*y^2*z-2*w*x*y^2*z-F(1)/4*x^2*y^2*z-4*v*y^3*z+F(4)/3*w*y^3*z+7*x*y^3*z+4*y^4*z-F(4)/3*v^3*z^2-F(1)/4*v^2*w*z^2+16*v*w^2*z^2-F(1)/2*w^3*z^2-6*v^2*x*z^2-5*v*w*x*z^2-16*w^2*x*z^2-9*v*x^2*z^2+16*w*x^2*z^2-7*x^3*z^2-7*v^2*y*z^2+19*v*w*y*z^2-w^2*y*z^2-7*v*x*y*z^2+4*w*x*y*z^2+F(2)/3*x^2*y*z^2-7*v*y^2*z^2+6*y^3*z^2+12*v^2*z^3-F(1)/2*v*w*z^3-9*w^2*z^3-3*v*x*z^3+2*w*x*z^3-F(1)/2*x^2*z^3+F(4)/3*v*y*z^3+4*w*y*z^3-2*x*y*z^3+F(1)/3*y^2*z^3-8*v*z^4-2*w*z^4+F(2)/3*x*z^4+3*y*z^4+18*z^5,3*v^5-18*v^4*w-12*v^3*w^2-9*v^2*w^3-9*v*w^4-F(3)/4*w^5+F(3)/2*v^4*x+18*v^2*w^2*x-19*v*w^3*x-18*w^4*x+12*v^3*x^2-F(3)/4*v^2*w*x^2-F(4)/3*v*w^2*x^2+F(1)/3*w^3*x^2+8*v^2*x^3-7*v*w*x^3+17*w^2*x^3+12*v*x^4-F(3)/4*w*x^4+5*x^5-F(3)/4*v^4*y-7*v^3*w*y+F(3)/4*v^2*w^2*y-5*v*w^3*y+8*w^4*y-F(3)/2*v^3*x*y+18*v^2*w*x*y+F(2)/3*v*w^2*x*y-19*w^3*x*y-v^2*x^2*y-F(2)/3*v*w*x^2*y-5*w^2*x^2*y-2*v*x^3*y+4*w*x^3*y+F(3)/2*x^4*y+12*v^3*y^2+F(1)/3*v^2*w*y^2+F(4)/3*v*w^2*y^2-F(3)/4*w^3*y^2-v^2*x*y^2+9*v*w*x*y^2-6*v*x^2*y^2+9*w*x^2*y^2-F(3)/4*x^3*y^2+F(2)/3*v^2*y^3+2*v*w*y^3+F(1)/4*w^2*y^3+5*v*x*y^3+12*w*x*y^3-9*x^2*y^3+F(3)/4*v*y^4+F(3)/4*w*y^4+F(2)/3*x*y^4-19*y^5-5*v^4*z-3*v^3*w*z-F(1)/2*v^2*w^2*z+F(3)/2*v*w^3*z-19*w^4*z-19*v^3*x*z+18*v^2*w*x*z+6*v*w^2*x*z+F(1)/2*w^3*x*z-12*v^2*x^2*z+18*v*w*x^2*z+F(3)/2*w^2*x^2*z-F(3)/2*v*x^3*z-w*x^3*z-2*x^4*z+4*v^3*y*z+19*v^2*w*y*z+F(3)/4*v*w^2*y*z+F(1)/2*w^3*y*z+18*v^2*x*y*z-19*w^2*x*y*z+6*v*x^2*y*z-8*w*x^2*y*z+18*x^3*y*z-12*v^2*y^2*z-F(4)/3*v*w*y^2*z-w^2*y^2*z+F(3)/4*v*x*y^2*z-6*w*x*y^2*z+F(1)/3*x^2*y^2*z+F(3)/4*v*y^3*z-F(1)/4*w*y^3*z-F(1)/4*x*y^3*z+F(1)/4*y^4*z-5*v^3*z^2-3*v^2*w*z^2-2*v*w^2*z^2-6*w^3*z^2-18*v^2*x*z^2+F(2)/3*v*w*x*z^2-F(1)/2*w^2*x*z^2-5*v*x^2*z^2+w*x^2*z^2+2*x^3*z^2+F(2)/3*v^2*y*z^2+F(1)/4*v*w*y*z^2-12*w^2*y*z^2+18*v*x*y*z^2+F(1)/3*w*x*y*z^2+4*x^2*y*z^2+5*v*y^2*z^2+F(1)/4*w*y^2*z^2-2*x*y^2*z^2-4*y^3*z^2-9*v^2*z^3+3*v*w*z^3-F(1)/4*w^2*z^3+v*x*z^3+F(1)/2*w*x*z^3+F(1)/3*x^2*z^3+F(1)/3*v*y*z^3+F(1)/3*w*y*z^3-6*x*y*z^3-F(3)/2*y^2*z^3-v*z^4-18*w*z^4+F(1)/2*y*z^4+z^5,F(3)/2*v^4*w+18*v^3*w^2-F(2)/3*v^2*w^3-F(1)/2*v*w^4+5*w^5-F(4)/3*v^4*x-F(1)/3*v^3*w*x-F(1)/4*v^2*w^2*x-F(3)/2*v*w^3*x+5*w^4*x-9*v^3*x^2+F(4)/3*v^2*w*x^2-9*v*w^2*x^2-F(3)/4*w^3*x^2+16*v^2*x^3+6*v*w*x^3-F(4)/3*w^2*x^3+F(1)/4*v*x^4+4*x^5+F(1)/3*v^4*y-F(3)/4*v^3*w*y+8*v*w^3*y+16*w^4*y+6*v^3*x*y-16*v^2*w*x*y+8*v*w^2*x*y+17*w^3*x*y-8*v^2*x^2*y+F(1)/3*v*w*x^2*y-F(3)/2*w^2*x^2*y-F(1)/4*v*x^3*y+16*w*x^3*y+12*x^4*y+F(1)/4*v^3*y^2+8*v^2*w*y^2-4*v*w^2*y^2-F(1)/3*w^3*y^2+17*v^2*x*y^2-7*v*w*x*y^2-8*w^2*x*y^2-F(4)/3*v*x^2*y^2+19*w*x^2*y^2-4*x^3*y^2-F(3)/2*v^2*y^3-8*v*w*y^3+F(3)/4*w^2*y^3+19*v*x*y^3+6*w*x*y^3+F(3)/4*x^2*y^3-18*v*y^4+F(1)/2*w*y^4-18*x*y^4+19*y^5-F(3)/4*v^4*z+F(3)/4*v^3*w*z-8*v^2*w^2*z+F(1)/3*v*w^3*z+w^4*z-16*v^3*x*z-5*v^2*w*x*z+F(3)/2*v*w^2*x*z+8*w^3*x*z-2*v^2*x^2*z+19*v*w*x^2*z-6*w^2*x^2*z-F(1)/2*v*x^3*z+9*w*x^3*z-19*x^4*z+3*v^3*y*z-F(2)/3*v^2*w*y*z+17*v*w^2*y*z-8*w^3*y*z-F(4)/3*v^2*x*y*z-F(3)/4*w^2*x*y*z+5*v*x^2*y*z+17*w*x^2*y*z+16*x^3*y*z+F(3)/2*v^2*y^2*z+12*v*w*y^2*z-F(1)/4*w^2*y^2*z-F(3)/2*v*x*y^2*z-2*w*x*y^2*z-4*x^2*y^2*z+18*v*y^3*z-w*y^3*z+12*y^4*z-17*v^3*z^2+3*v^2*w*z^2-w^3*z^2-9*v^2*x*z^2+16*v*w*x*z^2-3*w^2*x*z^2+F(1)/2*v*x^2*z^2+F(2)/3*w*x^2*z^2+16*x^3*z^2+F(4)/3*v^2*y*z^2-F(4)/3*v*w*y*z^2+17*w^2*y*z^2+7*v*x*y*z^2+F(1)/2*w*x*y*z^2-2*x^2*y*z^2+9*v*y^2*z^2+2*w*y^2*z^2+4*x*y^2*z^2+F(3)/2*y^3*z^2+F(3)/2*v^2*z^3-F(3)/2*v*w*z^3+w^2*z^3+3*v*x*z^3-F(1)/4*w*x*z^3+2*x^2*z^3+17*v*y*z^3-F(4)/3*w*y*z^3+12*x*y*z^3+2*y^2*z^3-F(4)/3*v*z^4-12*w*z^4+18*x*z^4+12*y*z^4+F(2)/3*z^5,16*v^4*w-F(3)/4*v^3*w^2-17*v^2*w^3-6*v*w^4+16*w^5-17*v^4*x-16*v^3*w*x+7*v^2*w^2*x-18*v*w^3*x+16*w^4*x+17*v^3*x^2+6*v^2*w*x^2-9*v*w^2*x^2+19*w^3*x^2+17*v^2*x^3+F(1)/3*v*w*x^3+F(3)/4*w^2*x^3+F(1)/4*v*x^4-2*w*x^4+16*x^5+16*v^4*y-F(2)/3*v^3*w*y+6*v^2*w^2*y+9*v*w^3*y-18*w^4*y-4*v^3*x*y+12*v^2*w*x*y+6*v*w^2*x*y-16*w^3*x*y-v^2*x^2*y-F(1)/2*v*w*x^2*y+12*w^2*x^2*y+19*v*x^3*y-F(3)/4*w*x^3*y+F(2)/3*x^4*y-F(3)/4*v^3*y^2-F(1)/4*v^2*w*y^2-F(3)/4*v*w^2*y^2+12*w^3*y^2+v^2*x*y^2+F(1)/2*v*w*x*y^2+F(1)/4*w^2*x*y^2-8*v*x^2*y^2-F(4)/3*w*x^2*y^2-x^3*y^2-7*v^2*y^3+9*v*w*y^3+F(1)/3*w^2*y^3-12*v*x*y^3+8*w*x*y^3-12*x^2*y^3+8*v*y^4+F(3)/2*w*y^4+F(4)/3*x*y^4+16*y^5-2*v^4*z+8*v^3*w*z-17*v^2*w^2*z-F(1)/2*v*w^3*z+12*w^4*z+12*v^3*x*z-19*v^2*w*x*z+7*v*w^2*x*z-F(1)/3*w^3*x*z-12*v^2*x^2*z-2*v*w*x^2*z-4*w^2*x^2*z-16*v*x^3*z+4*w*x^3*z+F(1)/3*x^4*z+F(3)/4*v^3*y*z-F(1)/2*v^2*w*y*z-7*v*w^2*y*z+17*w^3*y*z+9*v^2*x*y*z-5*w^2*x*y*z+18*v*x^2*y*z+F(1)/4*w*x^2*y*z-17*x^3*y*z-5*v^2*y^2*z+12*v*w*y^2*z+3*w^2*y^2*z-F(1)/3*v*x*y^2*z-F(1)/2*w*x*y^2*z-5*x^2*y^2*z-3*v*y^3*z+F(4)/3*w*y^3*z+F(1)/2*x*y^3*z+17*y^4*z+2*v^3*z^2-F(3)/2*v^2*w*z^2+18*v*w^2*z^2-w^3*z^2+8*v^2*x*z^2+17*v*w*x*z^2+2*w^2*x*z^2-F(2)/3*v*x^2*z^2+7*w*x^2*z^2+2*x^3*z^2+v^2*y*z^2+F(2)/3*v*w*y*z^2+8*w^2*y*z^2+12*v*x*y*z^2+6*w*x*y*z^2-F(4)/3*x^2*y*z^2+F(3)/4*v*y^2*z^2-2*w*y^2*z^2+17*x*y^2*z^2+5*y^3*z^2-v^2*z^3-F(1)/2*v*w*z^3-17*w^2*z^3+F(1)/3*v*x*z^3-6*w*x*z^3+3*x^2*z^3-F(1)/2*v*y*z^3+w*y*z^3+3*x*y*z^3+9*y^2*z^3-8*v*z^4+3*w*z^4-F(1)/4*x*z^4-16*y*z^4-5*z^5,F(3)/4*v^4*w+5*v^3*w^2+6*v*w^4-16*w^5-8*v^4*x-18*v^3*w*x+F(2)/3*v^2*w^2*x+5*v*w^3*x-8*v^3*x^2-9*v^2*w*x^2+2*v*w^2*x^2+12*w^3*x^2+2*v^2*x^3+F(1)/4*v*w*x^3+8*w^2*x^3-2*v*x^4-2*w*x^4-16*x^5-2*v^4*y-7*v^3*w*y-18*v^2*w^2*y-6*v*w^3*y+F(3)/2*w^4*y+12*v^3*x*y+F(1)/2*v*w^2*x*y+5*w^3*x*y+2*v^2*x^2*y+7*v*w*x^2*y-9*w^2*x^2*y+F(1)/2*v*x^3*y-7*w*x^3*y+x^4*y+19*v^3*y^2+F(1)/3*v^2*w*y^2-6*v*w^2*y^2-F(3)/2*w^3*y^2-2*v^2*x*y^2-F(3)/4*v*w*x*y^2+w^2*x*y^2-4*v*x^2*y^2-7*w*x^2*y^2+16*x^3*y^2+F(4)/3*v^2*y^3-16*v*w*y^3+8*w^2*y^3+F(2)/3*v*x*y^3-17*w*x*y^3-8*x^2*y^3-F(2)/3*v*y^4-12*w*y^4+F(1)/4*x*y^4-4*y^5-5*v^4*z+7*v^3*w*z-v^2*w^2*z+6*v*w^3*z-18*w^4*z-2*v^2*w*x*z-18*v*w^2*x*z-17*w^3*x*z+17*v^2*x^2*z-F(2)/3*v*w*x^2*z-4*w^2*x^2*z+F(3)/4*v*x^3*z-4*w*x^3*z+F(4)/3*x^4*z+2*v^2*w*y*z-17*v*w^2*y*z-w^3*y*z-v^2*x*y*z+F(4)/3*w^2*x*y*z+5*v*x^2*y*z-F(1)/4*w*x^2*y*z+5*x^3*y*z-F(2)/3*v^2*y^2*z-3*v*w*y^2*z-7*w^2*y^2*z+5*v*x*y^2*z+F(1)/4*w*x*y^2*z-16*x^2*y^2*z-F(4)/3*v*y^3*z+F(1)/4*w*y^3*z-F(3)/4*x*y^3*z+F(1)/3*y^4*z+5*v^3*z^2+4*v^2*w*z^2-6*v*w^2*z^2+17*w^3*z^2+12*v^2*x*z^2+F(3)/4*v*w*x*z^2+F(2)/3*w^2*x*z^2+v*x^2*z^2-F(1)/2*w*x^2*z^2-3*x^3*z^2+F(1)/2*v^2*y*z^2-18*v*w*y*z^2-3*w^2*y*z^2+18*v*x*y*z^2-19*w*x*y*z^2+F(1)/3*x^2*y*z^2-9*v*y^2*z^2-F(1)/3*w*y^2*z^2+18*x*y^2*z^2+F(1)/3*y^3*z^2-9*v^2*z^3+9*v*w*z^3-F(2)/3*w^2*z^3+7*v*x*z^3+F(2)/3*w*x*z^3+5*x^2*z^3-F(2)/3*v*y*z^3-F(1)/4*w*y*z^3+F(1)/4*x*y*z^3-F(1)/4*y^2*z^3-6*v*z^4+F(4)/3*w*z^4+16*x*z^4-5*y*z^4-7*z^5,F(4)/3*v^4*w+F(1)/2*v^3*w^2+F(2)/3*v^2*w^3+16*v*w^4-4*w^5-v^4*x+6*v^3*w*x-F(3)/4*v^2*w^2*x+F(2)/3*v*w^3*x-18*w^4*x-9*v^3*x^2+7*v^2*w*x^2-8*v*w^2*x^2+F(1)/3*w^3*x^2+9*v^2*x^3-F(2)/3*v*w*x^3-16*w^2*x^3+F(2)/3*v*x^4-F(3)/2*w*x^4-F(3)/2*x^5+16*v^4*y-4*v^3*w*y+F(4)/3*v^2*w^2*y-16*v*w^3*y-19*w^4*y-8*v^3*x*y-8*v^2*w*x*y+9*v*w^2*x*y+2*w^3*x*y-3*v^2*x^2*y+F(1)/4*v*w*x^2*y-w^2*x^2*y-F(1)/2*v*x^3*y+6*w*x^3*y-F(4)/3*x^4*y-2*v^3*y^2-9*v^2*w*y^2+4*v*w^2*y^2-F(1)/4*w^3*y^2+v^2*x*y^2-18*v*w*x*y^2+6*w^2*x*y^2+F(3)/4*v*x^2*y^2+F(3)/4*w*x^2*y^2+19*x^3*y^2-7*v^2*y^3+19*v*w*y^3-F(4)/3*w^2*y^3+16*v*x*y^3+6*w*x*y^3-F(1)/3*x^2*y^3+16*v*y^4-18*w*y^4+F(4)/3*x*y^4-3*y^5-12*v^4*z+5*v^3*w*z-F(1)/2*v^2*w^2*z-4*v*w^3*z-F(1)/2*w^4*z-F(2)/3*v^3*x*z+F(3)/2*v^2*w*x*z-F(1)/3*v*w^2*x*z+2*w^3*x*z-F(1)/4*v^2*x^2*z+F(3)/2*v*w*x^2*z-9*w^2*x^2*z+12*v*x^3*z-16*w*x^3*z-F(3)/4*x^4*z-F(4)/3*v^3*y*z-19*v^2*w*y*z+3*v*w^2*y*z-w^3*y*z-F(3)/2*v^2*x*y*z-3*w^2*x*y*z+F(3)/4*v*x^2*y*z-F(3)/4*w*x^2*y*z+F(1)/4*x^3*y*z-F(1)/2*v^2*y^2*z-2*v*w*y^2*z-18*w^2*y^2*z+6*v*x*y^2*z+7*w*x*y^2*z+F(1)/2*x^2*y^2*z-6*v*y^3*z+5*w*y^3*z-8*x*y^3*z+y^4*z-8*v^3*z^2+F(3)/4*v^2*w*z^2-19*v*w^2*z^2+w^3*z^2-F(2)/3*v^2*x*z^2+9*v*w*x*z^2+5*w^2*x*z^2+F(2)/3*v*x^2*z^2+F(4)/3*w*x^2*z^2-12*x^3*z^2+17*v^2*y*z^2-F(4)/3*w^2*y*z^2+3*v*x*y*z^2+3*w*x*y*z^2-F(2)/3*x^2*y*z^2+v*y^2*z^2+4*w*y^2*z^2-18*x*y^2*z^2-12*y^3*z^2-17*v^2*z^3+F(3)/4*v*w*z^3-F(1)/4*w^2*z^3+F(2)/3*v*x*z^3-F(1)/2*w*x*z^3-F(3)/2*x^2*z^3-18*v*y*z^3+F(4)/3*w*y*z^3+12*x*y*z^3+18*y^2*z^3+8*v*z^4-w*z^4-9*x*z^4+8*y*z^4+F(4)/3*z^5,18*v^4*w-9*v^3*w^2+F(1)/3*v^2*w^3-12*v*w^4+17*w^5-2*v^4*x+F(2)/3*v^3*w*x+4*v^2*w^2*x-6*v*w^3*x+19*w^4*x-7*v^3*x^2+16*v^2*w*x^2-2*v*w^2*x^2+8*w^3*x^2-19*v^2*x^3+3*v*w*x^3-2*w^2*x^3+12*v*x^4+F(1)/4*w*x^4-19*x^5+8*v^4*y+3*v^3*w*y+F(1)/4*v^2*w^2*y-F(1)/4*v*w^3*y+F(1)/3*w^4*y-F(1)/4*v^3*x*y+6*v^2*w*x*y-5*v*w^2*x*y-9*w^3*x*y+8*v^2*x^2*y+5*v*w*x^2*y-18*w^2*x^2*y-8*v*x^3*y-3*w*x^3*y-4*x^4*y+5*v^3*y^2-2*v^2*w*y^2+F(3)/2*v*w^2*y^2+19*w^3*y^2-5*v^2*x*y^2-12*v*w*x*y^2+F(4)/3*w^2*x*y^2-F(4)/3*v*x^2*y^2-2*w*x^2*y^2-4*x^3*y^2+16*v^2*y^3+F(1)/3*v*w*y^3-F(1)/2*w^2*y^3-17*v*x*y^3-17*w*x*y^3+16*x^2*y^3-F(1)/3*v*y^4+19*w*y^4-F(1)/2*x*y^4+12*y^5+F(3)/4*v^4*z-v^3*w*z-3*v^2*w^2*z-17*v*w^3*z-5*v^3*x*z+F(2)/3*v^2*w*x*z-F(4)/3*v*w^2*x*z+F(1)/3*w^3*x*z+5*v^2*x^2*z+F(4)/3*v*w*x^2*z-12*w^2*x^2*z+18*v*x^3*z+16*w*x^3*z-6*x^4*z+7*v^3*y*z-F(1)/2*v^2*w*y*z-5*v*w^2*y*z-F(4)/3*w^3*y*z+7*v^2*x*y*z+9*w^2*x*y*z-8*v*x^2*y*z+6*w*x^2*y*z-F(1)/2*x^3*y*z-F(4)/3*v^2*y^2*z-F(1)/3*v*w*y^2*z+F(2)/3*w^2*y^2*z-F(4)/3*v*x*y^2*z+17*w*x*y^2*z-F(3)/4*x^2*y^2*z+F(4)/3*v*y^3*z-6*w*y^3*z+16*x*y^3*z+12*y^4*z-3*v^3*z^2+F(1)/4*v^2*w*z^2-F(1)/4*v*w^2*z^2+3*w^3*z^2+F(2)/3*v^2*x*z^2+6*v*w*x*z^2-F(1)/3*w^2*x*z^2+8*v*x^2*z^2-17*w*x^2*z^2-9*x^3*z^2+7*v^2*y*z^2+F(4)/3*v*w*y*z^2+F(2)/3*w^2*y*z^2-5*v*x*y*z^2+7*w*x*y*z^2-5*x^2*y*z^2+F(3)/4*v*y^2*z^2+16*w*y^2*z^2-19*x*y^2*z^2+16*y^3*z^2-9*v^2*z^3+18*v*w*z^3-19*w^2*z^3-5*v*x*z^3-6*w*x*z^3+19*x^2*z^3-6*v*y*z^3-19*w*y*z^3-3*x*y*z^3-5*y^2*z^3-17*v*z^4+2*w*z^4-F(3)/4*y*z^4+F(3)/4*z^5,F(4)/3*v^4*w+12*v^3*w^2-F(4)/3*v^2*w^3-4*v*w^4-F(3)/4*w^5+F(4)/3*v^4*x+16*v^3*w*x+9*v^2*w^2*x+F(3)/2*v*w^3*x-F(1)/2*w^4*x-3*v^3*x^2+2*v^2*w*x^2+16*v*w^2*x^2+F(2)/3*w^3*x^2+F(1)/3*v^2*x^3+F(2)/3*v*w*x^3-F(1)/3*w^2*x^3-12*v*x^4-2*w*x^4-2*x^5+18*v^4*y+8*v^3*w*y+5*v^2*w^2*y-F(3)/2*v*w^3*y-9*w^4*y+4*v^3*x*y-F(3)/2*v^2*w*x*y+F(3)/4*v*w^2*x*y-4*w^3*x*y+2*v^2*x^2*y+F(1)/2*v*w*x^2*y-16*v*x^3*y+F(1)/4*w*x^3*y-F(1)/4*x^4*y-18*v^3*y^2+F(3)/4*v^2*w*y^2+2*v*w^2*y^2-F(3)/2*w^3*y^2+7*v^2*x*y^2-12*v*w*x*y^2+5*w^2*x*y^2-3*v*x^2*y^2-5*w*x^2*y^2+6*x^3*y^2+F(4)/3*v^2*y^3-19*v*w*y^3-w^2*y^3-2*v*x*y^3+3*w*x*y^3+F(4)/3*x^2*y^3+19*v*y^4+6*w*y^4+3*x*y^4+6*y^5+F(2)/3*v^4*z+19*v^3*w*z+F(3)/4*v^2*w^2*z-5*v*w^3*z-F(2)/3*w^4*z+F(2)/3*v^3*x*z-v^2*w*x*z+F(3)/2*v*w^2*x*z-F(2)/3*w^3*x*z-F(1)/4*v^2*x^2*z-8*v*w*x^2*z-5*w^2*x^2*z+F(1)/3*v*x^3*z+18*w*x^3*z-F(4)/3*x^4*z+19*v^2*w*y*z-F(1)/4*v*w^2*y*z+F(3)/2*w^3*y*z-F(1)/2*v^2*x*y*z-F(3)/2*w^2*x*y*z+12*v*x^2*y*z-18*w*x^2*y*z+6*x^3*y*z-F(1)/2*v^2*y^2*z-v*w*y^2*z-5*w^2*y^2*z-F(1)/3*v*x*y^2*z+4*w*x*y^2*z+F(1)/4*x^2*y^2*z+7*v*y^3*z-17*w*y^3*z-F(1)/2*x*y^3*z-16*y^4*z-6*v^3*z^2+v^2*w*z^2-6*v*w^2*z^2+17*w^3*z^2+17*v^2*x*z^2-19*v*w*x*z^2+9*w^2*x*z^2+4*v*x^2*z^2-12*w*x^2*z^2-19*x^3*z^2+8*v^2*y*z^2-18*v*w*y*z^2+16*w^2*y*z^2+F(3)/4*v*x*y*z^2-16*w*x*y*z^2-F(1)/2*x^2*y*z^2-F(1)/4*v*y^2*z^2+F(1)/3*w*y^2*z^2-18*x*y^2*z^2+6*y^3*z^2+16*v^2*z^3+F(3)/2*v*w*z^3-17*w^2*z^3+19*v*x*z^3+6*w*x*z^3+19*x^2*z^3+12*v*y*z^3-2*w*y*z^3-3*x*y*z^3+3*y^2*z^3+8*v*z^4-3*w*z^4-4*x*z^4+5*y*z^4+F(1)/3*z^5,-16*v^5+5*v^4*w-12*v^3*w^2-9*v^2*w^3+F(1)/4*v*w^4+3*w^5+17*v^4*x+F(2)/3*v^3*w*x+4*v^2*w^2*x+16*v*w^3*x-F(2)/3*w^4*x+7*v^3*x^2-9*v^2*w*x^2+F(1)/2*v*w^2*x^2-9*w^3*x^2-v^2*x^3-F(3)/4*v*w*x^3+F(4)/3*w^2*x^3+16*v*x^4+3*w*x^4-9*v^4*y+v^3*w*y+7*v^2*w^2*y-2*v*w^3*y+12*w^4*y-18*v^3*x*y-12*v^2*w*x*y-F(1)/2*v*w^2*x*y-12*w^3*x*y-5*v^2*x^2*y+3*v*w*x^2*y-5*w^2*x^2*y-2*v*x^3*y+F(3)/4*x^4*y-F(1)/3*v^3*y^2-17*v^2*w*y^2+17*v*w^2*y^2-F(2)/3*w^3*y^2+7*v^2*x*y^2-5*v*w*x*y^2-12*w^2*x*y^2-16*v*x^2*y^2-F(4)/3*w*x^2*y^2-12*x^3*y^2-F(3)/4*v^2*y^3+7*v*w*y^3+F(2)/3*w^2*y^3+19*v*x*y^3-12*x^2*y^3-6*v*y^4-8*w*y^4-2*x*y^4+F(1)/2*y^5-17*v^4*z+F(3)/4*v^3*w*z-17*v^2*w^2*z-F(1)/4*v*w^3*z-F(1)/2*w^4*z+F(3)/4*v^3*x*z-F(1)/3*v^2*w*x*z+F(3)/4*v*w^2*x*z-4*w^3*x*z-F(3)/2*v^2*x^2*z+F(1)/4*v*w*x^2*z+5*v*x^3*z-12*w*x^3*z-9*v^3*y*z+17*v^2*w*y*z-F(1)/3*v*w^2*y*z-4*w^3*y*z-19*v^2*x*y*z-3*w^2*x*y*z-5*v*x^2*y*z-12*w*x^2*y*z+19*x^3*y*z+F(1)/2*v^2*y^2*z+F(3)/2*v*w*y^2*z+16*w^2*y^2*z+F(2)/3*v*x*y^2*z+2*w*x*y^2*z-6*x^2*y^2*z+F(2)/3*v*y^3*z-3*w*y^3*z-F(1)/3*x*y^3*z+F(1)/4*y^4*z-F(3)/4*v^3*z^2-9*v^2*w*z^2-8*v*w^2*z^2-F(1)/4*w^3*z^2+9*v^2*x*z^2+2*v*w*x*z^2-5*w^2*x*z^2+5*v*x^2*z^2+4*w*x^2*z^2-2*x^3*z^2+F(4)/3*v^2*y*z^2+F(1)/4*v*w*y*z^2+F(3)/2*w^2*y*z^2-F(1)/3*v*x*y*z^2-19*w*x*y*z^2+7*x^2*y*z^2-F(1)/3*v*y^2*z^2+2*x*y^2*z^2+5*y^3*z^2-4*v^2*z^3-F(1)/2*v*w*z^3+18*w^2*z^3+5*v*x*z^3-F(3)/4*w*x*z^3-8*x^2*z^3+F(3)/2*v*y*z^3+19*w*y*z^3+3*x*y*z^3-17*y^2*z^3-7*v*z^4-5*w*z^4-6*x*z^4+8*y*z^4-9*z^5,F(2)/3*v^5+F(1)/4*v^4*w-F(1)/2*v^3*w^2+F(1)/3*v^2*w^3-17*v*w^4-F(1)/3*w^5-F(3)/4*v^4*x-F(3)/4*v^3*w*x+8*v*w^3*x-12*w^4*x+F(1)/2*v^3*x^2+17*v^2*w*x^2+17*v*w^2*x^2+5*w^3*x^2-4*v^2*x^3+9*v*w*x^3-18*w^2*x^3+19*v*x^4-F(3)/2*w*x^4-4*x^5-F(4)/3*v^4*y+7*v^3*w*y+6*v^2*w^2*y-16*v*w^3*y-3*w^4*y-3*v^3*x*y-17*v^2*w*x*y+F(4)/3*w^3*x*y-2*v^2*x^2*y+18*v*w*x^2*y-6*w^2*x^2*y+F(4)/3*v*x^3*y-5*w*x^3*y+x^4*y+12*v^3*y^2-F(1)/3*v^2*w*y^2+17*v*w^2*y^2-6*w^3*y^2+18*v^2*x*y^2-F(3)/4*v*w*x*y^2+17*w^2*x*y^2-F(1)/3*v*x^2*y^2+9*w*x^2*y^2-8*x^3*y^2-F(4)/3*v^2*y^3+F(2)/3*v*w*y^3-3*w^2*y^3+3*v*x*y^3+2*w*x*y^3-2*x^2*y^3+4*v*y^4-F(1)/2*w*y^4+F(3)/4*x*y^4+18*y^5+F(3)/4*v^4*z+12*v^3*w*z+12*v^2*w^2*z+12*v*w^3*z+7*w^4*z-9*v^3*x*z-18*v^2*w*x*z-4*v*w^2*x*z+4*w^3*x*z+F(3)/4*v^2*x^2*z+18*v*w*x^2*z-12*w^2*x^2*z+5*v*x^3*z-F(3)/4*w*x^3*z-F(3)/2*x^4*z-18*v^3*y*z+12*v^2*w*y*z-9*v*w^2*y*z+F(1)/2*w^3*y*z-16*v^2*x*y*z+F(3)/2*w^2*x*y*z+3*v*x^2*y*z+7*x^3*y*z-18*v^2*y^2*z+F(2)/3*v*w*y^2*z+F(1)/4*w^2*y^2*z+F(2)/3*v*x*y^2*z+5*w*x*y^2*z+12*x^2*y^2*z-4*v*y^3*z+F(1)/4*w*y^3*z+F(2)/3*x*y^3*z-F(1)/2*y^4*z-F(1)/3*v^3*z^2+8*v^2*w*z^2-F(4)/3*v*w^2*z^2-F(4)/3*w^3*z^2+5*v^2*x*z^2-F(3)/2*v*w*x*z^2+F(3)/4*w^2*x*z^2-6*v*x^2*z^2-F(1)/2*w*x^2*z^2-2*x^3*z^2-9*v^2*y*z^2+9*v*w*y*z^2+w^2*y*z^2-F(3)/2*v*x*y*z^2+F(3)/2*w*x*y*z^2-F(1)/3*x^2*y*z^2+3*v*y^2*z^2+F(4)/3*w*y^2*z^2+F(2)/3*x*y^2*z^2-5*y^3*z^2+17*v^2*z^3-12*v*w*z^3-9*w^2*z^3+6*v*x*z^3+4*w*x*z^3-F(3)/2*x^2*z^3-F(4)/3*v*y*z^3-9*w*y*z^3+9*x*y*z^3+7*y^2*z^3+19*v*z^4-F(4)/3*w*z^4-12*x*z^4+8*y*z^4]))
end


@doc raw"""
    enriques_d13_pi16()

Return a smooth Enriques surface in $\mathbb P^4$ with degree `13` and sectional genus `16`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function enriques_d13_pi16()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^2*y^4-F(97)/65*x*y^5-F(79)/11*y^6+F(2)/75*x^4*y*z+F(4)/3*x^3*y^2*z+F(88)/109*x^2*y^3*z-F(89)/78*x*y^4*z+10437*y^5*z-1211*x^4*z^2+F(49)/5*x^3*y*z^2+10298*x^2*y^2*z^2+11155*x*y^3*z^2-F(82)/21*y^4*z^2-F(59)/108*x^3*z^3-F(99)/35*x^2*y*z^3+F(88)/111*x*y^2*z^3-635*y^3*z^3-F(5)/116*x^2*z^4+F(57)/83*x*y*z^4+F(49)/50*y^2*z^4+7423*x*z^5-F(91)/44*y*z^5+F(19)/123*z^6+11826*x^4*y*u-F(10)/63*x^3*y^2*u+F(17)/58*x^2*y^3*u-F(17)/94*x*y^4*u+F(91)/97*y^5*u-F(32)/89*x^4*z*u-F(85)/64*x^3*y*z*u+2472*x^2*y^2*z*u-F(43)/41*x*y^3*z*u+874*y^4*z*u-12151*x^3*z^2*u+F(10)/61*x^2*y*z^2*u-F(114)/109*x*y^2*z^2*u+F(27)/109*y^3*z^2*u+F(125)/111*x^2*z^3*u+8922*x*y*z^3*u-F(87)/5*y^2*z^3*u-F(77)/67*x*z^4*u+F(15)/91*y*z^4*u+F(57)/86*z^5*u+F(40)/93*x^4*u^2+F(117)/34*x^3*y*u^2-F(77)/71*x^2*y^2*u^2+F(59)/107*x*y^3*u^2+F(33)/83*y^4*u^2+107*x^3*z*u^2-4016*x^2*y*z*u^2-F(119)/87*x*y^2*z*u^2+5885*y^3*z*u^2+F(36)/5*x^2*z^2*u^2-F(76)/25*x*y*z^2*u^2+F(49)/101*y^2*z^2*u^2-15627*x*z^3*u^2-F(7)/13*y*z^3*u^2+F(107)/125*z^4*u^2-F(76)/7*x^3*u^3-F(32)/27*x^2*y*u^3-F(17)/39*x*y^2*u^3+104*y^3*u^3-F(119)/103*x^2*z*u^3-F(107)/87*x*y*z*u^3-2405*y^2*z*u^3-15917*x*z^2*u^3+F(111)/104*y*z^2*u^3+F(123)/4*z^3*u^3-10405*x^2*u^4+F(17)/25*x*y*u^4+182*y^2*u^4-8607*x*z*u^4-3358*y*z*u^4+13247*z^2*u^4+12027*x*u^5+F(114)/35*y*u^5-F(89)/14*z*u^5+12247*u^6+F(7)/88*x^4*y*v-5455*x^3*y^2*v+7061*x^2*y^3*v-963*x*y^4*v-F(58)/91*y^5*v-F(105)/118*x^4*z*v-F(101)/8*x^3*y*z*v-15903*x^2*y^2*z*v-F(115)/64*x*y^3*z*v-636*y^4*z*v+F(118)/91*x^3*z^2*v-F(47)/7*x^2*y*z^2*v-F(61)/117*x*y^2*z^2*v+9858*y^3*z^2*v+F(6)/115*x^2*z^3*v+F(23)/72*x*y*z^3*v-15177*y^2*z^3*v+F(6)/83*x*z^4*v-4058*y*z^4*v-F(59)/90*z^5*v-11207*x^4*u*v-F(7)/41*x^3*y*u*v+F(113)/23*x^2*y^2*u*v-F(101)/68*x*y^3*u*v-F(57)/112*y^4*u*v+F(124)/65*x^3*z*u*v+12267*x^2*y*z*u*v-15364*x*y^2*z*u*v+2767*y^3*z*u*v-F(13)/60*x^2*z^2*u*v+14871*x*y*z^2*u*v+F(91)/37*y^2*z^2*u*v+6177*x*z^3*u*v+F(39)/32*y*z^3*u*v-5707*z^4*u*v-10895*x^3*u^2*v-8495*x^2*y*u^2*v-13286*x*y^2*u^2*v-6566*y^3*u^2*v+F(51)/29*x^2*z*u^2*v-F(122)/57*x*y*z*u^2*v-2788*y^2*z*u^2*v-F(91)/15*x*z^2*u^2*v+5296*y*z^2*u^2*v+9989*z^3*u^2*v+9547*x^2*u^3*v+10913*x*y*u^3*v+F(6)/17*y^2*u^3*v-F(74)/99*x*z*u^3*v+F(7)/52*y*z*u^3*v-13229*z^2*u^3*v-4089*x*u^4*v-5542*y*u^4*v-F(31)/87*z*u^4*v+F(65)/88*u^5*v-F(59)/31*x^4*v^2-F(116)/77*x^3*y*v^2+13741*x^2*y^2*v^2-F(58)/19*x*y^3*v^2-10344*y^4*v^2+F(77)/32*x^3*z*v^2-F(28)/11*x^2*y*z*v^2-F(109)/6*x*y^2*z*v^2-1659*y^3*z*v^2-8172*x^2*z^2*v^2-F(11)/122*x*y*z^2*v^2+F(63)/22*y^2*z^2*v^2-F(103)/67*x*z^3*v^2+12488*y*z^3*v^2-11778*z^4*v^2+10885*x^3*u*v^2-F(77)/24*x^2*y*u*v^2-F(10)/101*x*y^2*u*v^2+7005*y^3*u*v^2+12766*x^2*z*u*v^2-F(97)/21*x*y*z*u*v^2+3532*y^2*z*u*v^2-F(8)/69*x*z^2*u*v^2+F(113)/41*y*z^2*u*v^2+4518*z^3*u*v^2-F(68)/123*x^2*u^2*v^2-F(67)/7*x*y*u^2*v^2+8302*y^2*u^2*v^2-3830*x*z*u^2*v^2-F(109)/64*y*z*u^2*v^2+10725*z^2*u^2*v^2+F(67)/41*x*u^3*v^2+F(17)/49*y*u^3*v^2+549*z*u^3*v^2-F(121)/111*u^4*v^2+F(31)/5*x^3*v^3+F(59)/97*x^2*y*v^3+12973*x*y^2*v^3-F(11)/104*y^3*v^3-1073*x^2*z*v^3+6912*x*y*z*v^3-F(36)/47*y^2*z*v^3+F(41)/19*x*z^2*v^3-F(103)/9*y*z^2*v^3+F(43)/69*z^3*v^3-7646*x^2*u*v^3-F(125)/27*x*y*u*v^3+12092*y^2*u*v^3+10762*x*z*u*v^3-F(106)/97*y*z*u*v^3+247*z^2*u*v^3+10187*x*u^2*v^3+7844*y*u^2*v^3-6744*z*u^2*v^3+F(11)/4*u^3*v^3-F(91)/92*x^2*v^4-F(59)/96*x*y*v^4-F(87)/38*y^2*v^4-221*x*z*v^4-7627*y*z*v^4+F(89)/46*z^2*v^4-5144*x*u*v^4-F(108)/59*y*u*v^4-4678*z*u*v^4-7763*u^2*v^4+F(41)/26*x*v^5+F(73)/41*y*v^5+F(17)/23*z*v^5+F(1)/71*u*v^5+F(124)/57*v^6,1635*x^5-F(22)/109*x^4*y+F(124)/57*x^3*y^2-F(94)/79*x^2*y^3+F(29)/22*x*y^4-13315*y^5+F(23)/10*x^4*z-F(29)/8*x^3*y*z+9967*x^2*y^2*z-4542*x*y^3*z+14693*y^4*z+F(74)/71*x^3*z^2-F(9)/49*x^2*y*z^2+F(113)/71*x*y^2*z^2-14011*y^3*z^2-5410*x^2*z^3+F(54)/49*x*y*z^3+12974*y^2*z^3-F(94)/59*x*z^4+2315*y*z^4+14959*z^5+9464*x^4*u+6657*x^3*y*u+F(113)/70*x^2*y^2*u-F(83)/53*x*y^3*u-F(123)/49*y^4*u-F(30)/107*x^3*z*u+13879*x^2*y*z*u+908*x*y^2*z*u-5981*y^3*z*u+F(67)/48*x^2*z^2*u-15893*x*y*z^2*u-F(26)/85*y^2*z^2*u-F(54)/121*x*z^3*u+15774*y*z^3*u+14981*z^4*u+F(47)/65*x^3*u^2-6427*x^2*y*u^2+F(45)/41*x*y^2*u^2+F(99)/122*y^3*u^2+F(125)/7*x^2*z*u^2-F(91)/62*x*y*z*u^2+48*y^2*z*u^2-F(57)/34*x*z^2*u^2+870*y*z^2*u^2+9505*z^3*u^2+5718*x^2*u^3-5109*x*y*u^3+9715*y^2*u^3+14559*x*z*u^3-6673*y*z*u^3+13511*z^2*u^3+F(7)/79*x*u^4-5359*y*u^4+8408*z*u^4+F(66)/37*u^5-F(43)/79*x^4*v+F(106)/115*x^3*y*v+F(121)/23*x^2*y^2*v+F(12)/13*x*y^3*v+15796*y^4*v-F(16)/51*x^3*z*v-F(91)/76*x^2*y*z*v+F(117)/43*x*y^2*z*v-11589*y^3*z*v+7799*x^2*z^2*v-F(53)/38*x*y*z^2*v-5705*y^2*z^2*v+F(71)/85*x*z^3*v+F(68)/43*y*z^3*v-5070*z^4*v-F(46)/87*x^3*u*v-2406*x^2*y*u*v-F(35)/3*x*y^2*u*v+2010*y^3*u*v+1692*x^2*z*u*v-882*x*y*z*u*v-F(41)/49*y^2*z*u*v+F(105)/19*x*z^2*u*v+F(19)/7*y*z^2*u*v-4256*z^3*u*v-F(49)/124*x^2*u^2*v-11557*x*y*u^2*v-F(88)/9*y^2*u^2*v-F(40)/29*x*z*u^2*v+5832*y*z*u^2*v+F(111)/73*z^2*u^2*v-F(34)/47*x*u^3*v-F(43)/98*y*u^3*v-F(80)/97*z*u^3*v+F(17)/93*u^4*v-10900*x^3*v^2-F(111)/10*x^2*y*v^2+F(125)/17*x*y^2*v^2+5855*y^3*v^2-3977*x^2*z*v^2-F(7)/40*x*y*z*v^2+F(66)/49*y^2*z*v^2+175*x*z^2*v^2+F(114)/49*y*z^2*v^2+1954*z^3*v^2+9907*x^2*u*v^2-14794*x*y*u*v^2-1916*y^2*u*v^2+F(88)/39*x*z*u*v^2+11685*y*z*u*v^2+4355*z^2*u*v^2+F(119)/15*x*u^2*v^2+F(59)/94*y*u^2*v^2+8971*z*u^2*v^2-F(43)/89*u^3*v^2+F(93)/17*x^2*v^3-F(5)/29*x*y*v^3+1565*y^2*v^3+8558*x*z*v^3-F(117)/101*y*z*v^3+F(85)/36*z^2*v^3+1192*x*u*v^3+F(37)/7*y*u*v^3-F(109)/33*z*u*v^3-F(125)/102*u^2*v^3-4187*x*v^4+F(57)/77*y*v^4+F(22)/3*z*v^4+4687*u*v^4+F(33)/73*v^5,-1635*x^4*y-F(77)/86*x^3*y^2+F(103)/45*x^2*y^3-F(101)/90*x*y^4-F(72)/125*y^5-F(125)/34*x^4*z-F(45)/2*x^3*y*z-F(69)/121*x^2*y^2*z+F(58)/103*x*y^3*z+4785*y^4*z+F(105)/82*x^3*z^2-8993*x^2*y*z^2-496*x*y^2*z^2-12526*y^3*z^2+F(5)/71*x^2*z^3+5308*x*y*z^3+F(23)/100*y^2*z^3+F(42)/125*x*z^4+F(19)/73*y*z^4-F(52)/99*z^5-3270*x^4*u+130*x^3*y*u-F(64)/59*x^2*y^2*u+F(33)/95*x*y^3*u-4640*y^4*u-F(17)/42*x^3*z*u-F(17)/126*x^2*y*z*u-F(83)/87*x*y^2*z*u+F(69)/82*y^3*z*u-F(20)/81*x^2*z^2*u-F(89)/99*x*y*z^2*u-F(61)/121*y^2*z^2*u+F(39)/112*x*z^3*u+F(44)/73*y*z^3*u+11615*z^4*u+2814*x^3*u^2+F(24)/121*x^2*y*u^2-F(71)/25*x*y^2*u^2+F(77)/86*y^3*u^2+11164*x^2*z*u^2+14586*x*y*z*u^2+7751*y^2*z*u^2+F(82)/27*x*z^2*u^2-F(101)/119*y*z^2*u^2-11540*z^3*u^2-F(121)/41*x^2*u^3-F(15)/119*x*y*u^3-F(39)/112*y^2*u^3-F(103)/73*x*z*u^3+5070*y*z*u^3+12524*z^2*u^3+F(64)/39*x*u^4-6254*y*u^4+F(117)/16*z*u^4+F(17)/25*u^5-F(25)/9*x^4*v-F(55)/41*x^3*y*v-F(60)/41*x^2*y^2*v-F(103)/6*x*y^3*v-6357*y^4*v+4369*x^3*z*v+6145*x^2*y*z*v+156*x*y^2*z*v+F(36)/53*y^3*z*v+F(117)/23*x^2*z^2*v-F(41)/18*x*y*z^2*v+10744*y^2*z^2*v+F(47)/50*x*z^3*v-F(51)/65*y*z^3*v+F(114)/7*z^4*v-F(34)/87*x^3*u*v-11613*x^2*y*u*v-13246*x*y^2*u*v+F(27)/76*y^3*u*v-12945*x^2*z*u*v-F(85)/16*x*y*z*u*v+F(115)/9*y^2*z*u*v-569*x*z^2*u*v-14493*y*z^2*u*v+11064*z^3*u*v-12417*x^2*u^2*v-6147*x*y*u^2*v-F(87)/62*y^2*u^2*v-10610*x*z*u^2*v+F(77)/81*y*z*u^2*v+F(121)/13*z^2*u^2*v+3186*x*u^3*v-F(57)/116*y*u^3*v-F(69)/31*z*u^3*v-13502*u^4*v+F(8)/105*x^3*v^2+1564*x^2*y*v^2-F(78)/103*x*y^2*v^2-F(27)/122*y^3*v^2+60*x^2*z*v^2-F(12)/125*x*y*z*v^2+F(73)/70*y^2*z*v^2-F(76)/3*x*z^2*v^2+F(59)/13*y*z^2*v^2-F(109)/65*z^3*v^2-14619*x^2*u*v^2-F(5)/38*x*y*u*v^2+F(49)/52*y^2*u*v^2+F(107)/105*x*z*u*v^2-393*y*z*u*v^2+F(26)/93*z^2*u*v^2-4810*x*u^2*v^2-6580*y*u^2*v^2-F(59)/52*z*u^2*v^2-F(102)/11*u^3*v^2+F(71)/56*x^2*v^3+F(68)/91*x*y*v^3-8299*y^2*v^3-1385*x*z*v^3-F(59)/29*y*z*v^3+F(75)/71*z^2*v^3+12985*x*u*v^3-12556*y*u*v^3-2336*z*u*v^3-F(92)/119*u^2*v^3+9323*x*v^4-F(80)/89*y*v^4+F(11)/95*z*v^4-F(46)/123*u*v^4+8903*v^5,-6636*x^2*y^3+F(88)/51*x*y^4+4519*y^5-F(9)/71*x^4*z+F(7)/44*x^3*y*z+F(5)/91*x^2*y^2*z+F(37)/104*x*y^3*z+F(16)/123*y^4*z+F(97)/89*x^3*z^2-F(63)/125*x^2*y*z^2+670*x*y^2*z^2+F(88)/39*y^3*z^2-F(33)/41*x^2*z^3-F(15)/86*x*y*z^3-F(25)/4*y^2*z^3-F(97)/103*x*z^4+1443*y*z^4+7059*z^5-14981*x^4*u+F(26)/17*x^3*y*u+F(97)/126*x^2*y^2*u+10339*x*y^3*u+F(113)/36*y^4*u+10715*x^3*z*u-1343*x^2*y*z*u+F(43)/15*x*y^2*z*u-F(68)/55*y^3*z*u+6137*x^2*z^2*u-213*x*y*z^2*u-F(43)/10*y^2*z^2*u+15620*x*z^3*u-14724*y*z^3*u+F(67)/3*z^4*u+F(27)/65*x^3*u^2-11024*x^2*y*u^2-3162*x*y^2*u^2-1373*y^3*u^2-F(13)/50*x^2*z*u^2+15*x*y*z*u^2+F(6)/31*y^2*z*u^2+F(99)/26*x*z^2*u^2-F(36)/79*y*z^2*u^2-F(121)/63*z^3*u^2-F(18)/83*x^2*u^3-9044*x*y*u^3+3178*y^2*u^3+F(33)/124*x*z*u^3-F(19)/107*y*z*u^3-12163*z^2*u^3-5855*x*u^4+F(73)/41*y*u^4+F(106)/47*z*u^4+F(114)/121*u^5+F(26)/45*x^4*v-F(38)/61*x^3*y*v+15651*x^2*y^2*v+F(119)/54*x*y^3*v-4712*y^4*v+F(17)/24*x^3*z*v+9973*x^2*y*z*v-10327*x*y^2*z*v+11081*y^3*z*v-13941*x^2*z^2*v-F(18)/7*x*y*z^2*v+F(116)/121*y^2*z^2*v+9882*x*z^3*v+F(20)/23*y*z^3*v-11133*z^4*v-F(117)/92*x^3*u*v+F(113)/61*x^2*y*u*v-8031*x*y^2*u*v-F(3)/32*y^3*u*v-F(13)/68*x^2*z*u*v-F(59)/69*x*y*z*u*v+F(32)/25*y^2*z*u*v+F(1)/2*x*z^2*u*v-F(25)/113*y*z^2*u*v+13482*z^3*u*v+F(106)/25*x^2*u^2*v+F(3)/26*x*y*u^2*v-F(65)/58*y^2*u^2*v-F(22)/85*x*z*u^2*v-F(22)/65*y*z*u^2*v+F(61)/49*z^2*u^2*v-4065*x*u^3*v+11685*y*u^3*v-F(79)/35*z*u^3*v-251*u^4*v+F(100)/77*x^3*v^2-F(66)/67*x^2*y*v^2-F(48)/85*x*y^2*v^2-F(53)/115*y^3*v^2-F(37)/28*x^2*z*v^2-F(113)/94*x*y*z*v^2-F(103)/117*y^2*z*v^2+F(104)/55*x*z^2*v^2-3577*y*z^2*v^2-9748*z^3*v^2-13919*x^2*u*v^2-F(85)/99*x*y*u*v^2+2269*y^2*u*v^2+F(6)/67*x*z*u*v^2-F(65)/67*y*z*u*v^2+7470*z^2*u*v^2+F(122)/51*x*u^2*v^2-F(35)/86*y*u^2*v^2+F(100)/27*z*u^2*v^2-14264*u^3*v^2+11156*x^2*v^3+F(75)/88*x*y*v^3-F(97)/101*y^2*v^3+F(73)/34*x*z*v^3+F(116)/43*y*z*v^3-F(71)/98*z^2*v^3+F(57)/104*x*u*v^3+F(38)/67*y*u*v^3-F(111)/47*z*u*v^3+10812*u^2*v^3+F(58)/63*x*v^4-F(35)/23*y*v^4-F(109)/44*z*v^4+F(83)/69*u*v^4+F(25)/92*v^5,-F(5)/58*x^2*y^3-F(7)/78*x*y^4-F(98)/75*y^5-F(44)/57*x^4*z-13567*x^3*y*z+9232*x^2*y^2*z+F(27)/67*x*y^3*z+14688*y^4*z+F(18)/61*x^3*z^2-7646*x^2*y*z^2+F(33)/7*x*y^2*z^2+F(7)/25*y^3*z^2-F(107)/54*x^2*z^3+F(34)/125*x*y*z^3+4911*y^2*z^3-14194*x*z^4+13938*y*z^4-194*z^5+F(31)/32*x^4*u+F(96)/115*x^3*y*u-5220*x^2*y^2*u-90*x*y^3*u-8071*y^4*u-F(91)/37*x^3*z*u-F(12)/29*x^2*y*z*u-5653*x*y^2*z*u+F(88)/103*y^3*z*u-F(50)/89*x^2*z^2*u-3249*x*y*z^2*u-2178*y^2*z^2*u-F(72)/61*x*z^3*u+F(113)/78*y*z^3*u+F(13)/93*z^4*u-F(53)/89*x^3*u^2+510*x^2*y*u^2-14798*x*y^2*u^2+F(49)/31*y^3*u^2+F(103)/53*x^2*z*u^2+F(75)/37*x*y*z*u^2+F(45)/97*y^2*z*u^2-15892*x*z^2*u^2+5112*y*z^2*u^2-F(125)/79*z^3*u^2+12976*x^2*u^3-1894*x*y*u^3+10230*y^2*u^3+5855*x*z*u^3-F(13)/76*y*z*u^3+899*z^2*u^3-F(60)/101*x*u^4-F(35)/87*y*u^4+F(61)/99*z*u^4-F(34)/89*u^5-1061*x^4*v-F(53)/2*x^3*y*v-6627*x^2*y^2*v+8861*x*y^3*v+2971*y^4*v-11594*x^3*z*v+8922*x^2*y*z*v-F(89)/93*x*y^2*z*v-12430*y^3*z*v-4439*x^2*z^2*v+12763*x*y*z^2*v+8587*y^2*z^2*v-1080*x*z^3*v-F(91)/67*y*z^3*v-15191*z^4*v-F(37)/9*x^3*u*v-F(118)/125*x^2*y*u*v-F(56)/73*x*y^2*u*v-5084*y^3*u*v+13939*x^2*z*u*v-F(37)/60*x*y*z*u*v-1366*y^2*z*u*v-F(19)/110*x*z^2*u*v+F(77)/58*y*z^2*u*v+F(71)/40*z^3*u*v+F(51)/125*x^2*u^2*v+F(20)/81*x*y*u^2*v+F(60)/89*y^2*u^2*v+9310*x*z*u^2*v-F(56)/51*y*z*u^2*v-F(118)/65*z^2*u^2*v+F(73)/59*x*u^3*v-F(115)/102*y*u^3*v-F(57)/122*z*u^3*v+F(43)/41*u^4*v+F(13)/84*x^3*v^2-12211*x^2*y*v^2-F(99)/26*x*y^2*v^2+9859*y^3*v^2-7686*x^2*z*v^2-F(80)/61*x*y*z*v^2-F(83)/27*y^2*z*v^2+F(100)/31*x*z^2*v^2-F(101)/47*y*z^2*v^2-F(89)/56*z^3*v^2+F(53)/4*x^2*u*v^2+193*x*y*u*v^2-F(89)/43*y^2*u*v^2-5127*x*z*u*v^2+F(49)/47*y*z*u*v^2-F(93)/8*z^2*u*v^2+F(39)/8*x*u^2*v^2-6904*y*u^2*v^2-F(108)/97*z*u^2*v^2-F(42)/23*u^3*v^2-2687*x^2*v^3-F(99)/107*x*y*v^3+10544*y^2*v^3-9861*x*z*v^3-6913*y*z*v^3+F(4)/53*z^2*v^3+F(94)/109*x*u*v^3-F(92)/79*y*u*v^3+F(106)/79*z*u*v^3+F(3)/4*u^2*v^3+F(55)/17*x*v^4+F(71)/90*y*v^4+F(59)/115*z*v^4+6746*u*v^4-159*v^5,F(113)/106*x^3*y^2+F(48)/41*x^2*y^3-12672*x*y^4-F(23)/56*y^5-12543*x^4*z+F(99)/59*x^3*y*z-F(71)/11*x^2*y^2*z-6801*x*y^3*z+F(20)/89*y^4*z+11945*x^3*z^2+F(43)/19*x^2*y*z^2+F(107)/41*x*y^2*z^2-F(126)/85*y^3*z^2+12081*x^2*z^3+F(36)/41*x*y*z^3-F(121)/45*y^2*z^3-2746*x*z^4+F(12)/41*y*z^4-F(121)/81*z^5+F(59)/65*x^4*u-5219*x^3*y*u+6330*x^2*y^2*u-F(120)/7*x*y^3*u-5139*y^4*u-F(124)/3*x^3*z*u-23*x^2*y*z*u-5656*x*y^2*z*u+F(53)/40*y^3*z*u-F(82)/95*x^2*z^2*u+F(4)/113*x*y*z^2*u-11530*y^2*z^2*u+11756*x*z^3*u+F(56)/11*y*z^3*u+12846*z^4*u-F(67)/65*x^3*u^2+F(39)/17*x^2*y*u^2+F(53)/98*x*y^2*u^2+F(100)/119*y^3*u^2+F(80)/43*x^2*z*u^2+1544*x*y*z*u^2+F(37)/25*y^2*z*u^2-106*x*z^2*u^2+F(110)/41*y*z^2*u^2-F(101)/66*z^3*u^2+12622*x^2*u^3+12402*x*y*u^3+3813*y^2*u^3-F(8)/61*x*z*u^3+5632*y*z*u^3+6264*z^2*u^3-F(113)/44*x*u^4+F(95)/33*y*u^4-F(86)/13*z*u^4-F(116)/7*u^5-14896*x^4*v+7400*x^3*y*v-2993*x^2*y^2*v-F(29)/92*x*y^3*v+F(75)/44*y^4*v-185*x^3*z*v+F(36)/13*x^2*y*z*v-7826*x*y^2*z*v+2212*y^3*z*v-F(87)/106*x^2*z^2*v+618*x*y*z^2*v+F(55)/68*y^2*z^2*v-F(19)/21*x*z^3*v-6262*y*z^3*v+1592*z^4*v-F(109)/70*x^3*u*v+11686*x^2*y*u*v-F(31)/107*x*y^2*u*v-1127*y^3*u*v+4724*x^2*z*u*v-10214*x*y*z*u*v-13770*y^2*z*u*v+F(11)/120*x*z^2*u*v+F(47)/84*y*z^2*u*v-F(111)/77*z^3*u*v-F(77)/83*x^2*u^2*v+7370*x*y*u^2*v-7728*y^2*u^2*v+F(113)/65*x*z*u^2*v+11842*y*z*u^2*v-F(83)/61*z^2*u^2*v+F(27)/38*x*u^3*v-F(5)/124*y*u^3*v+6581*z*u^3*v+F(21)/124*u^4*v-F(55)/72*x^3*v^2+14299*x^2*y*v^2+F(85)/47*x*y^2*v^2-F(47)/39*y^3*v^2-F(124)/115*x^2*z*v^2+F(43)/83*x*y*z*v^2+F(91)/64*y^2*z*v^2+13579*x*z^2*v^2-F(125)/116*y*z^2*v^2+8151*z^3*v^2-F(82)/67*x^2*u*v^2+F(30)/7*x*y*u*v^2-12490*y^2*u*v^2-13269*x*z*u*v^2-2647*y*z*u*v^2-2428*z^2*u*v^2+F(116)/85*x*u^2*v^2-977*y*u^2*v^2-1038*z*u^2*v^2+F(43)/36*u^3*v^2+F(107)/38*x^2*v^3-F(85)/89*x*y*v^3-F(29)/14*y^2*v^3+F(25)/3*x*z*v^3-12209*y*z*v^3-F(70)/113*z^2*v^3-3251*x*u*v^3-F(92)/27*y*u*v^3-1217*z*u*v^3+F(97)/34*u^2*v^3+F(67)/97*x*v^4-623*y*v^4-12448*z*v^4+F(100)/11*u*v^4-F(74)/23*v^5]))
end


@doc raw"""
    enriques_d13_pi16_two()

Return a smooth Enriques surface in $\mathbb P^4$ with degree `13` and sectional genus `16`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function enriques_d13_pi16_two()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([4255*x^2*y^4+F(71)/105*x*y^5-F(67)/40*y^6+F(63)/62*x^4*y*z+F(104)/7*x^3*y^2*z+F(106)/67*x^2*y^3*z+4276*x*y^4*z+F(68)/53*y^5*z+F(123)/67*x^4*z^2-F(104)/97*x^3*y*z^2+1608*x^2*y^2*z^2-579*x*y^3*z^2+F(93)/8*y^4*z^2-F(98)/109*x^3*z^3-F(58)/123*x^2*y*z^3-8545*x*y^2*z^3+4699*y^3*z^3-F(47)/67*x^2*z^4+F(107)/22*x*y*z^4-F(95)/4*y^2*z^4+F(11)/106*x*z^5+3849*y*z^5-F(48)/17*z^6-3755*x^4*y*u-F(32)/23*x^3*y^2*u-F(103)/3*x^2*y^3*u+F(62)/35*x*y^4*u-F(74)/75*y^5*u-2582*x^4*z*u-6678*x^3*y*z*u-11655*x^2*y^2*z*u-F(109)/44*x*y^3*z*u-F(61)/50*y^4*z*u-F(53)/115*x^3*z^2*u-3531*x^2*y*z^2*u-6790*x*y^2*z^2*u-3183*y^3*z^2*u-15243*x^2*z^3*u-F(61)/43*x*y*z^3*u-F(19)/65*y^2*z^3*u+F(126)/97*x*z^4*u-F(35)/46*y*z^4*u-F(71)/73*z^5*u+11717*x^4*u^2-14477*x^3*y*u^2-6476*x^2*y^2*u^2+12357*x*y^3*u^2+F(99)/58*y^4*u^2+441*x^3*z*u^2+F(113)/26*x^2*y*z*u^2-F(91)/71*x*y^2*z*u^2-F(55)/29*y^3*z*u^2-F(57)/122*x^2*z^2*u^2-F(2)/71*x*y*z^2*u^2+F(95)/14*y^2*z^2*u^2-F(121)/70*x*z^3*u^2-5747*y*z^3*u^2-35*z^4*u^2-13387*x^3*u^3-F(103)/42*x^2*y*u^3-2403*x*y^2*u^3-F(58)/81*y^3*u^3-3517*x^2*z*u^3+3921*x*y*z*u^3-4953*y^2*z*u^3-5294*x*z^2*u^3+4427*y*z^2*u^3+F(61)/126*z^3*u^3-15333*x^2*u^4-13001*x*y*u^4-F(9)/88*y^2*u^4-F(17)/13*x*z*u^4-F(65)/4*y*z*u^4+F(53)/85*z^2*u^4+F(88)/57*x*u^5-F(64)/51*y*u^5-F(95)/61*z*u^5-F(9)/49*u^6+10338*x^4*y*v+15752*x^3*y^2*v+3597*x^2*y^3*v-F(121)/39*x*y^4*v-15744*y^5*v+747*x^4*z*v+741*x^3*y*z*v-F(58)/69*x^2*y^2*z*v+14967*x*y^3*z*v+4057*y^4*z*v+13610*x^3*z^2*v+10811*x^2*y*z^2*v-F(55)/3*x*y^2*z^2*v+F(82)/47*y^3*z^2*v+F(49)/96*x^2*z^3*v-F(86)/9*x*y*z^3*v-6945*y^2*z^3*v+F(53)/51*x*z^4*v+3287*y*z^4*v+F(71)/56*z^5*v+F(93)/92*x^4*u*v+F(119)/18*x^3*y*u*v-F(13)/37*x^2*y^2*u*v-13900*x*y^3*u*v+F(121)/124*y^4*u*v+F(4)/65*x^3*z*u*v-F(35)/78*x^2*y*z*u*v+6048*x*y^2*z*u*v+F(109)/7*y^3*z*u*v-F(98)/87*x^2*z^2*u*v-5218*x*y*z^2*u*v-10466*y^2*z^2*u*v-F(113)/22*x*z^3*u*v+2310*y*z^3*u*v+F(7)/18*z^4*u*v+5616*x^3*u^2*v+1357*x^2*y*u^2*v-F(122)/55*x*y^2*u^2*v+F(113)/75*y^3*u^2*v+1842*x^2*z*u^2*v-F(105)/71*x*y*z*u^2*v+F(41)/92*y^2*z*u^2*v+2015*x*z^2*u^2*v+10344*y*z^2*u^2*v+7716*z^3*u^2*v+F(79)/64*x^2*u^3*v-F(112)/27*x*y*u^3*v+F(10)/13*y^2*u^3*v+F(89)/119*x*z*u^3*v-F(65)/53*y*z*u^3*v+5429*z^2*u^3*v+F(103)/52*x*u^4*v+F(29)/113*y*u^4*v+9937*z*u^4*v+F(96)/19*u^5*v+F(19)/115*x^4*v^2+F(119)/3*x^3*y*v^2+9532*x^2*y^2*v^2+F(87)/11*x*y^3*v^2-F(19)/120*y^4*v^2+2687*x^3*z*v^2+15768*x^2*y*z*v^2-F(23)/103*x*y^2*z*v^2-7538*y^3*z*v^2-14027*x^2*z^2*v^2+F(53)/51*x*y*z^2*v^2-963*y^2*z^2*v^2+F(89)/68*x*z^3*v^2-F(113)/88*y*z^3*v^2-F(56)/33*z^4*v^2+F(85)/88*x^3*u*v^2-F(1)/44*x^2*y*u*v^2-F(65)/44*x*y^2*u*v^2-14367*y^3*u*v^2-F(20)/103*x^2*z*u*v^2-F(93)/83*x*y*z*u*v^2-8741*y^2*z*u*v^2+14609*x*z^2*u*v^2+4786*y*z^2*u*v^2+12622*z^3*u*v^2+F(120)/89*x^2*u^2*v^2-F(8)/109*x*y*u^2*v^2+F(29)/36*y^2*u^2*v^2+F(118)/41*x*z*u^2*v^2+F(67)/46*y*z*u^2*v^2-4053*z^2*u^2*v^2-F(56)/93*x*u^3*v^2-F(125)/17*y*u^3*v^2+F(12)/107*z*u^3*v^2-10473*u^4*v^2-F(37)/98*x^3*v^3-F(121)/23*x^2*y*v^3-F(23)/48*x*y^2*v^3-F(57)/40*y^3*v^3-F(49)/96*x^2*z*v^3-F(26)/107*x*y*z*v^3-F(75)/4*y^2*z*v^3+5029*x*z^2*v^3+12401*y*z^2*v^3+F(35)/109*z^3*v^3-2726*x^2*u*v^3+12129*x*y*u*v^3-F(73)/29*y^2*u*v^3+F(72)/109*x*z*u*v^3+F(14)/103*y*z*u*v^3+F(114)/47*z^2*u*v^3-F(15)/53*x*u^2*v^3+F(41)/60*y*u^2*v^3+3496*z*u^2*v^3-F(3)/88*u^3*v^3+F(50)/71*x^2*v^4-F(43)/31*x*y*v^4-F(10)/103*y^2*v^4+F(7)/48*x*z*v^4+6882*y*z*v^4+F(124)/77*z^2*v^4-9018*x*u*v^4-F(57)/59*y*u*v^4-169*z*u*v^4-8517*u^2*v^4+F(77)/27*x*v^5-10601*y*v^5-8396*z*v^5+2375*u*v^5-1062*v^6,x^2*y^4+15452*x*y^5+F(51)/104*y^6-5778*x^4*y*z+10372*x^3*y^2*z-156*x^2*y^3*z-F(107)/62*x*y^4*z-8411*y^5*z+F(9)/10*x^4*z^2+F(57)/73*x^3*y*z^2-F(97)/45*x^2*y^2*z^2-723*x*y^3*z^2+F(65)/28*y^4*z^2+F(62)/97*x^3*z^3+4549*x^2*y*z^3+15201*x*y^2*z^3-F(37)/9*y^3*z^3+F(26)/99*x^2*z^4-F(21)/107*x*y*z^4+14316*y^2*z^4-1176*x*z^5+F(75)/53*y*z^5+F(3)/109*z^6+358*x^4*y*u-14467*x^3*y^2*u-F(9)/35*x^2*y^3*u-6201*x*y^4*u+F(84)/121*y^5*u+F(121)/53*x^4*z*u+11488*x^3*y*z*u-F(89)/27*x^2*y^2*z*u+F(89)/73*x*y^3*z*u-F(115)/21*y^4*z*u-F(105)/58*x^3*z^2*u+13869*x^2*y*z^2*u-F(118)/85*x*y^2*z^2*u-4509*y^3*z^2*u-2428*x^2*z^3*u+11878*x*y*z^3*u+6441*y^2*z^3*u+F(9)/26*x*z^4*u-F(62)/95*y*z^4*u+F(126)/113*z^5*u+2772*x^4*u^2+15144*x^3*y*u^2+1697*x^2*y^2*u^2+15567*x*y^3*u^2-5671*y^4*u^2-F(41)/125*x^3*z*u^2-F(87)/50*x^2*y*z*u^2-7609*x*y^2*z*u^2+F(77)/47*y^3*z*u^2+723*x^2*z^2*u^2-F(39)/14*x*y*z^2*u^2-F(43)/75*y^2*z^2*u^2-F(32)/103*x*z^3*u^2+F(117)/7*y*z^3*u^2-855*z^4*u^2-F(103)/49*x^3*u^3-F(86)/83*x^2*y*u^3+194*x*y^2*u^3-4192*y^3*u^3-F(120)/77*x^2*z*u^3+F(47)/3*x*y*z*u^3-1675*y^2*z*u^3-F(13)/49*x*z^2*u^3-7416*y*z^2*u^3+12068*z^3*u^3+F(125)/28*x^2*u^4-6711*x*y*u^4+9769*y^2*u^4-F(71)/79*x*z*u^4+F(18)/5*y*z*u^4-3685*z^2*u^4+13229*x*u^5-2443*y*u^5+F(106)/43*z*u^5-11876*u^6+F(49)/54*x^4*y*v-F(101)/99*x^3*y^2*v-F(80)/109*x^2*y^3*v+F(79)/110*x*y^4*v+F(49)/113*y^5*v+F(83)/52*x^4*z*v+F(4)/15*x^3*y*z*v-F(103)/83*x^2*y^2*z*v+F(7)/46*x*y^3*z*v-F(35)/18*y^4*z*v-14308*x^3*z^2*v-1168*x^2*y*z^2*v+F(64)/3*x*y^2*z^2*v-F(96)/85*y^3*z^2*v-F(74)/47*x^2*z^3*v-7515*x*y*z^3*v+F(111)/38*y^2*z^3*v+F(9)/38*x*z^4*v+11404*y*z^4*v+F(114)/43*z^5*v-F(2)/85*x^4*u*v-F(125)/18*x^3*y*u*v-49*x^2*y^2*u*v-F(77)/40*x*y^3*u*v+6359*y^4*u*v+F(55)/124*x^3*z*u*v-8546*x^2*y*z*u*v+F(68)/55*x*y^2*z*u*v-992*y^3*z*u*v+F(40)/83*x^2*z^2*u*v+F(50)/97*x*y*z^2*u*v-3*y^2*z^2*u*v-F(59)/70*x*z^3*u*v-F(107)/6*y*z^3*u*v+F(26)/19*z^4*u*v-10927*x^3*u^2*v+202*x^2*y*u^2*v-F(115)/83*x*y^2*u^2*v+13919*y^3*u^2*v+3668*x^2*z*u^2*v-1636*x*y*z*u^2*v+5603*y^2*z*u^2*v+12100*x*z^2*u^2*v+8857*y*z^2*u^2*v+4503*z^3*u^2*v+3290*x^2*u^3*v-2038*x*y*u^3*v-10857*y^2*u^3*v-F(33)/116*x*z*u^3*v-15459*y*z*u^3*v-585*z^2*u^3*v+F(107)/115*x*u^4*v-8548*y*u^4*v+F(103)/85*z*u^4*v-F(79)/118*u^5*v+13404*x^4*v^2-F(25)/11*x^3*y*v^2+3315*x^2*y^2*v^2+F(119)/106*x*y^3*v^2+F(103)/20*y^4*v^2-F(114)/113*x^3*z*v^2-F(39)/88*x^2*y*z*v^2+5310*x*y^2*z*v^2-14601*y^3*z*v^2-6723*x^2*z^2*v^2+13512*x*y*z^2*v^2+12590*y^2*z^2*v^2-4903*x*z^3*v^2-F(65)/106*y*z^3*v^2-12524*z^4*v^2-10012*x^3*u*v^2+F(81)/56*x^2*y*u*v^2+1949*x*y^2*u*v^2+F(125)/93*y^3*u*v^2-F(45)/67*x^2*z*u*v^2+7307*x*y*z*u*v^2+F(115)/42*y^2*z*u*v^2-F(88)/63*x*z^2*u*v^2-F(79)/103*y*z^2*u*v^2-F(79)/64*z^3*u*v^2+F(101)/48*x^2*u^2*v^2+F(68)/33*x*y*u^2*v^2+3682*y^2*u^2*v^2+10074*x*z*u^2*v^2-1551*y*z*u^2*v^2-F(59)/37*z^2*u^2*v^2+F(64)/61*x*u^3*v^2-F(69)/73*y*u^3*v^2+F(59)/104*z*u^3*v^2-8428*u^4*v^2+8174*x^3*v^3-15029*x^2*y*v^3+F(101)/87*x*y^2*v^3+F(10)/109*y^3*v^3-F(43)/53*x^2*z*v^3-11611*x*y*z*v^3-F(113)/56*y^2*z*v^3+8473*x*z^2*v^3-F(41)/109*y*z^2*v^3+F(89)/48*z^3*v^3-F(97)/24*x^2*u*v^3+10086*x*y*u*v^3+F(101)/9*y^2*u*v^3-4502*x*z*u*v^3-F(30)/119*y*z*u*v^3+64*z^2*u*v^3+F(77)/65*x*u^2*v^3-8941*y*u^2*v^3-F(37)/75*z*u^2*v^3-F(37)/2*u^3*v^3-F(22)/71*x^2*v^4+15712*x*y*v^4+14175*y^2*v^4-F(91)/38*x*z*v^4-14432*y*z*v^4-15466*z^2*v^4+11686*x*u*v^4-F(44)/111*y*u*v^4+15930*z*u*v^4-F(51)/104*u^2*v^4-F(43)/113*x*v^5+7052*y*v^5-1299*z*v^5-7697*u*v^5-7538*v^6,-10372*x^5-6205*x^4*y+F(93)/101*x^3*y^2-9826*x^2*y^3-7041*x*y^4-F(124)/121*y^5-9348*x^4*z-F(78)/37*x^3*y*z+F(23)/107*x^2*y^2*z+F(119)/60*x*y^3*z-F(67)/114*y^4*z-F(11)/24*x^3*z^2-F(116)/23*x^2*y*z^2-F(64)/43*x*y^2*z^2+7042*y^3*z^2-11367*x^2*z^3-F(103)/112*x*y*z^3-4716*y^2*z^3+F(72)/43*x*z^4+F(16)/43*y*z^4-15531*z^5+10507*x^4*u-F(17)/75*x^3*y*u+F(58)/75*x^2*y^2*u-14498*x*y^3*u-F(11)/63*y^4*u+10741*x^3*z*u-F(67)/96*x^2*y*z*u-F(39)/98*x*y^2*z*u+5671*y^3*z*u+4589*x^2*z^2*u-F(95)/112*x*y*z^2*u-F(95)/109*y^2*z^2*u-F(37)/125*x*z^3*u+F(114)/103*y*z^3*u-F(59)/15*z^4*u-F(1)/93*x^3*u^2+F(92)/93*x^2*y*u^2+F(89)/14*x*y^2*u^2-F(25)/14*y^3*u^2-11280*x^2*z*u^2+F(48)/83*x*y*z*u^2+F(22)/3*y^2*z*u^2-F(67)/95*x*z^2*u^2-F(59)/57*y*z^2*u^2-F(75)/14*z^3*u^2+F(115)/84*x^2*u^3+F(49)/95*x*y*u^3-14528*y^2*u^3-F(83)/125*x*z*u^3+14083*y*z*u^3-F(109)/36*z^2*u^3-F(97)/12*x*u^4+F(109)/82*y*u^4+F(110)/81*z*u^4-F(107)/96*u^5-6438*x^4*v-F(78)/71*x^3*y*v+14164*x^2*y^2*v-4811*x*y^3*v+F(111)/28*y^4*v+F(100)/103*x^3*z*v-228*x^2*y*z*v+F(113)/17*x*y^2*z*v-F(48)/7*y^3*z*v+F(115)/27*x^2*z^2*v-5435*x*y*z^2*v+F(113)/56*y^2*z^2*v+1589*x*z^3*v-F(89)/119*y*z^3*v-F(20)/71*z^4*v+9336*x^3*u*v-645*x^2*y*u*v-F(115)/63*x*y^2*u*v-F(118)/61*y^3*u*v+2774*x^2*z*u*v+F(52)/11*x*y*z*u*v-F(22)/9*y^2*z*u*v-2390*x*z^2*u*v-F(117)/37*y*z^2*u*v+F(117)/35*z^3*u*v+F(114)/115*x^2*u^2*v-7317*x*y*u^2*v-F(91)/51*y^2*u^2*v+F(59)/41*x*z*u^2*v-3573*y*z*u^2*v-5403*z^2*u^2*v+F(75)/52*x*u^3*v-F(72)/91*y*u^3*v+5543*z*u^3*v+F(76)/83*u^4*v-74*x^3*v^2-9526*x^2*y*v^2+12466*x*y^2*v^2-F(122)/53*y^3*v^2-10256*x^2*z*v^2+F(86)/97*x*y*z*v^2-F(91)/30*y^2*z*v^2+4111*x*z^2*v^2-F(11)/119*y*z^2*v^2-F(83)/22*z^3*v^2-4909*x^2*u*v^2+F(47)/53*x*y*u*v^2+F(21)/5*y^2*u*v^2+1384*x*z*u*v^2+F(65)/2*y*z*u*v^2+F(95)/78*z^2*u*v^2+12494*x*u^2*v^2-8520*y*u^2*v^2-2979*z*u^2*v^2-6468*u^3*v^2-F(3)/85*x^2*v^3-8896*x*y*v^3-12974*y^2*v^3+F(101)/15*x*z*v^3+F(93)/8*y*z*v^3-F(103)/60*z^2*v^3-F(11)/58*x*u*v^3-F(55)/16*y*u*v^3+2677*z*u*v^3-776*u^2*v^3-F(27)/58*x*v^4-2646*y*v^4+F(47)/2*z*v^4-5794*u*v^4-F(1)/75*v^5,10372*x^4*y+6205*x^3*y^2-F(93)/101*x^2*y^3-1718*x*y^4+F(8)/95*y^5-F(75)/83*x^4*z-F(68)/117*x^3*y*z-F(11)/81*x^2*y^2*z-F(61)/11*x*y^3*z+F(17)/70*y^4*z+F(40)/41*x^3*z^2-F(58)/3*x^2*y*z^2+F(79)/83*x*y^2*z^2-1557*y^3*z^2+F(57)/37*x^2*z^3-2315*x*y*z^3-11980*y^2*z^3-F(49)/31*x*z^4+F(98)/113*y*z^4+12223*z^5-F(111)/118*x^4*u+F(107)/4*x^3*y*u-F(29)/120*x^2*y^2*u-12882*x*y^3*u-714*y^4*u-F(25)/32*x^3*z*u-F(77)/65*x^2*y*z*u-F(31)/71*x*y^2*z*u-F(92)/105*y^3*z*u+138*x^2*z^2*u-F(106)/75*x*y*z^2*u-F(22)/35*y^2*z^2*u-663*x*z^3*u+9501*y*z^3*u+F(97)/82*z^4*u-1198*x^3*u^2-F(98)/85*x^2*y*u^2-9076*x*y^2*u^2+F(17)/95*y^3*u^2-5042*x^2*z*u^2-14263*x*y*z*u^2-F(10)/121*y^2*z*u^2+F(11)/60*x*z^2*u^2-F(121)/60*y*z^2*u^2+12290*z^3*u^2+F(103)/53*x^2*u^3+3276*x*y*u^3-F(92)/113*y^2*u^3-F(72)/121*x*z*u^3-13411*y*z*u^3+3336*z^2*u^3-F(2)/87*x*u^4-9174*y*u^4+F(41)/11*z*u^4+15923*u^5-12219*x^4*v-14971*x^3*y*v+F(43)/9*x^2*y^2*v-F(44)/103*x*y^3*v+1658*y^4*v-F(106)/111*x^3*z*v-F(43)/79*x^2*y*z*v-11963*x*y^2*z*v+F(122)/103*y^3*z*v+12890*x^2*z^2*v-4278*x*y*z^2*v+F(67)/71*y^2*z^2*v+F(51)/32*x*z^3*v+F(52)/115*y*z^3*v-9967*z^4*v-F(103)/14*x^3*u*v-F(43)/102*x^2*y*u*v-F(92)/25*x*y^2*u*v+F(17)/64*y^3*u*v+7628*x^2*z*u*v+3706*x*y*z*u*v+F(29)/124*y^2*z*u*v+F(67)/55*x*z^2*u*v-11066*y*z^2*u*v+13828*z^3*u*v-1466*x^2*u^2*v-8681*x*y*u^2*v+F(75)/106*y^2*u^2*v+F(80)/91*x*z*u^2*v-9831*y*z*u^2*v-F(11)/90*z^2*u^2*v+3729*x*u^3*v+9443*y*u^3*v+14620*z*u^3*v+F(29)/110*u^4*v+F(76)/55*x^3*v^2+3156*x^2*y*v^2+F(75)/49*x*y^2*v^2+F(21)/10*y^3*v^2-6625*x^2*z*v^2-F(120)/41*x*y*z*v^2+F(111)/56*y^2*z*v^2+F(30)/7*x*z^2*v^2+9266*y*z^2*v^2-F(116)/13*z^3*v^2-14406*x^2*u*v^2-13939*x*y*u*v^2-4275*y^2*u*v^2-F(86)/121*x*z*u*v^2-F(62)/85*y*z*u*v^2+F(6)/61*z^2*u*v^2-F(91)/103*x*u^2*v^2+3906*y*u^2*v^2+F(11)/94*z*u^2*v^2+11606*u^3*v^2+F(43)/64*x^2*v^3+F(115)/98*x*y*v^3+F(121)/53*y^2*v^3+F(56)/123*x*z*v^3-F(23)/56*y*z*v^3-F(83)/51*z^2*v^3-12400*x*u*v^3-F(61)/24*y*u*v^3-F(2)/3*z*u*v^3-F(34)/71*u^2*v^3-9827*x*v^4+F(79)/7*y*v^4+F(105)/29*z*v^4+F(74)/123*u*v^4-11465*v^5,-12667*x^2*y^3+F(106)/23*x*y^4+F(123)/74*y^5+F(98)/23*x^4*z-F(1)/94*x^3*y*z-158*x^2*y^2*z+12493*x*y^3*z+2*y^4*z-15569*x^3*z^2-F(1)/56*x^2*y*z^2-F(75)/64*x*y^2*z^2-F(20)/13*y^3*z^2+F(29)/111*x^2*z^3-F(24)/31*x*y*z^3-F(50)/97*y^2*z^3-F(29)/52*x*z^4-F(124)/35*y*z^4-F(2)/77*z^5+F(83)/118*x^4*u-F(6)/61*x^3*y*u+F(55)/6*x^2*y^2*u-F(51)/116*x*y^3*u-5575*y^4*u+F(119)/121*x^3*z*u-F(79)/47*x^2*y*z*u-F(123)/73*x*y^2*z*u-F(69)/125*y^3*z*u+487*x^2*z^2*u+F(68)/3*x*y*z^2*u-11713*y^2*z^2*u+F(20)/57*x*z^3*u+F(42)/41*y*z^3*u-F(109)/48*z^4*u+2296*x^3*u^2+F(13)/47*x^2*y*u^2+F(69)/106*x*y^2*u^2-F(64)/45*y^3*u^2+F(84)/5*x^2*z*u^2-7885*x*y*z*u^2-F(25)/112*y^2*z*u^2+4936*x*z^2*u^2-F(117)/16*y*z^2*u^2-F(91)/80*z^3*u^2-F(69)/70*x^2*u^3+5226*x*y*u^3-8039*y^2*u^3+3573*x*z*u^3-9831*y*z*u^3-F(38)/43*z^2*u^3+F(113)/2*x*u^4-10007*y*u^4-F(77)/102*z*u^4+F(9)/16*u^5-F(39)/113*x^4*v-14198*x^3*y*v+F(13)/35*x^2*y^2*v+F(88)/13*x*y^3*v-14118*y^4*v-F(5)/38*x^3*z*v+F(24)/5*x^2*y*z*v-14907*x*y^2*z*v+F(79)/123*y^3*z*v-F(13)/84*x^2*z^2*v-12151*x*y*z^2*v+F(31)/25*y^2*z^2*v+F(17)/123*x*z^3*v-4884*y*z^3*v+F(61)/111*z^4*v+F(52)/27*x^3*u*v-F(71)/87*x^2*y*u*v+F(126)/47*x*y^2*u*v-1834*y^3*u*v+10841*x^2*z*u*v-6446*x*y*z*u*v-F(118)/47*y^2*z*u*v-2542*x*z^2*u*v+F(69)/10*y*z^2*u*v-13479*z^3*u*v-F(102)/71*x^2*u^2*v+F(115)/36*x*y*u^2*v+7540*y^2*u^2*v-F(106)/121*x*z*u^2*v-F(85)/67*y*z*u^2*v+F(59)/95*z^2*u^2*v-F(22)/9*x*u^3*v-F(121)/5*y*u^3*v+F(72)/65*z*u^3*v+14121*u^4*v-F(31)/27*x^3*v^2-F(13)/24*x^2*y*v^2-13259*x*y^2*v^2-F(109)/69*y^3*v^2+93*x^2*z*v^2-2647*x*y*z*v^2-F(19)/120*y^2*z*v^2-10058*x*z^2*v^2-3036*y*z^2*v^2+13918*z^3*v^2+10862*x^2*u*v^2-F(107)/49*x*y*u*v^2+1463*y^2*u*v^2-F(57)/2*x*z*u*v^2+F(55)/6*y*z*u*v^2+F(9)/80*z^2*u*v^2-F(14)/11*x*u^2*v^2-F(43)/69*y*u^2*v^2-F(87)/53*z*u^2*v^2+F(25)/34*u^3*v^2+F(13)/28*x^2*v^3+14525*x*y*v^3+F(94)/77*y^2*v^3-F(9)/115*x*z*v^3-F(51)/116*y*z*v^3+F(104)/7*z^2*v^3-9003*x*u*v^3+F(82)/25*y*u*v^3-F(17)/24*z*u*v^3+F(53)/66*u^2*v^3-F(97)/49*x*v^4-15473*y*v^4-5080*z*v^4-8036*u*v^4+11188*v^5,-14820*x^2*y^3-14333*x*y^4-F(118)/21*y^5+F(17)/70*x^4*z+2991*x^3*y*z+F(67)/62*x^2*y^2*z+F(33)/95*x*y^3*z-3799*y^4*z-F(106)/43*x^3*z^2+7125*x^2*y*z^2+F(77)/86*x*y^2*z^2-F(74)/9*y^3*z^2+F(34)/29*x^2*z^3-F(66)/25*x*y*z^3+15722*y^2*z^3+5295*x*z^4-F(92)/85*y*z^4-F(13)/63*z^5-3448*x^4*u+F(113)/45*x^3*y*u-F(55)/106*x^2*y^2*u+F(113)/77*x*y^3*u+F(85)/124*y^4*u+11815*x^3*z*u+F(16)/113*x^2*y*z*u+F(99)/83*x*y^2*z*u+F(49)/78*y^3*z*u-10495*x^2*z^2*u-F(90)/61*x*y*z^2*u-5622*y^2*z^2*u-F(35)/87*x*z^3*u-6672*y*z^3*u+F(106)/121*z^4*u-F(93)/49*x^3*u^2+2236*x^2*y*u^2+F(121)/16*x*y^2*u^2+5489*y^3*u^2+15127*x^2*z*u^2+15202*x*y*z*u^2-F(13)/32*y^2*z*u^2-5898*x*z^2*u^2-F(89)/56*y*z^2*u^2-4536*z^3*u^2-73*x^2*u^3-F(41)/50*x*y*u^3-15617*y^2*u^3-9857*x*z*u^3+14321*y*z*u^3+F(84)/113*z^2*u^3+F(113)/32*x*u^4+F(107)/110*y*u^4-F(81)/125*z*u^4-F(67)/18*u^5+F(124)/45*x^4*v-F(80)/63*x^3*y*v-3419*x^2*y^2*v+1789*x*y^3*v+7089*y^4*v-F(47)/109*x^3*z*v-1955*x^2*y*z*v-F(59)/77*x*y^2*z*v+8837*y^3*z*v+F(73)/68*x^2*z^2*v-F(121)/97*x*y*z^2*v+F(123)/70*y^2*z^2*v+F(51)/35*x*z^3*v-F(14)/123*y*z^3*v-F(39)/73*z^4*v-F(15)/11*x^3*u*v+F(3)/77*x^2*y*u*v-F(67)/52*x*y^2*u*v-11669*y^3*u*v+F(51)/16*x^2*z*u*v+F(21)/85*x*y*z*u*v-F(57)/91*y^2*z*u*v+9170*x*z^2*u*v+3700*y*z^2*u*v+1213*z^3*u*v+13940*x^2*u^2*v+5707*x*y*u^2*v+F(49)/62*y^2*u^2*v+F(97)/45*x*z*u^2*v-7687*y*z*u^2*v+15113*z^2*u^2*v+F(76)/123*x*u^3*v+F(67)/113*y*u^3*v+F(89)/36*z*u^3*v-F(41)/45*u^4*v+F(7)/78*x^3*v^2+F(31)/89*x^2*y*v^2+6605*x*y^2*v^2-F(95)/91*y^3*v^2+F(112)/43*x^2*z*v^2+F(22)/51*x*y*z*v^2-4501*y^2*z*v^2-F(8)/23*x*z^2*v^2+F(7)/121*y*z^2*v^2+F(23)/40*z^3*v^2-F(43)/80*x^2*u*v^2-4593*x*y*u*v^2-362*y^2*u*v^2-F(82)/45*x*z*u*v^2-F(71)/91*y*z*u*v^2+F(7)/66*z^2*u*v^2+F(5)/29*x*u^2*v^2+F(103)/55*y*u^2*v^2-1765*z*u^2*v^2+F(5)/107*u^3*v^2+F(47)/80*x^2*v^3-F(101)/60*x*y*v^3+5635*y^2*v^3-F(77)/103*x*z*v^3-4119*y*z*v^3-13108*z^2*v^3+F(13)/8*x*u*v^3-5442*y*u*v^3-F(103)/120*z*u*v^3-F(31)/11*u^2*v^3+F(109)/4*x*v^4-10861*y*v^4-9826*z*v^4-F(79)/24*u*v^4-12706*v^5,12667*x^3*y^2+F(117)/34*x^2*y^3-F(117)/28*x*y^4-F(100)/67*y^5+3158*x^4*z+14914*x^3*y*z-15876*x^2*y^2*z+2655*x*y^3*z+F(113)/99*y^4*z-3597*x^3*z^2-F(18)/83*x^2*y*z^2+4087*x*y^2*z^2+F(35)/11*y^3*z^2-F(61)/11*x^2*z^3-662*x*y*z^3+F(10)/113*y^2*z^3-F(44)/3*x*z^4+F(35)/36*y*z^4-13730*z^5-12493*x^4*u-F(103)/108*x^3*y*u+F(121)/64*x^2*y^2*u-5969*x*y^3*u+F(46)/9*y^4*u+9581*x^3*z*u-F(87)/34*x^2*y*z*u-F(29)/39*x*y^2*z*u+F(47)/14*y^3*z*u+F(7)/116*x^2*z^2*u+6921*x*y*z^2*u+F(87)/32*y^2*z^2*u-F(31)/14*x*z^3*u+F(26)/31*y*z^3*u+8903*z^4*u+F(78)/83*x^3*u^2-F(109)/107*x^2*y*u^2+F(65)/68*x*y^2*u^2-F(43)/96*y^3*u^2-8660*x^2*z*u^2+F(3)/17*x*y*z*u^2-F(82)/89*y^2*z*u^2-6199*x*z^2*u^2-1634*y*z^2*u^2-13739*z^3*u^2-8686*x^2*u^3-5762*x*y*u^3+F(79)/51*y^2*u^3-F(109)/98*x*z*u^3+3858*y*z*u^3+F(79)/66*z^2*u^3+F(11)/53*x*u^4-10401*y*u^4-15023*z*u^4+F(1)/10*u^5-F(31)/23*x^4*v-8787*x^3*y*v-F(87)/5*x^2*y^2*v+F(98)/3*x*y^3*v-F(113)/36*y^4*v+F(61)/98*x^3*z*v-15878*x^2*y*z*v+6845*x*y^2*z*v-4593*y^3*z*v-F(23)/7*x^2*z^2*v+F(73)/71*x*y*z^2*v-F(87)/23*y^2*z^2*v-14914*x*z^3*v+F(76)/121*y*z^3*v+225*z^4*v+F(19)/72*x^3*u*v+F(63)/113*x^2*y*u*v-11302*x*y^2*u*v-F(65)/99*y^3*u*v-F(39)/32*x^2*z*u*v-14349*x*y*z*u*v-F(49)/11*y^2*z*u*v+9237*x*z^2*u*v+F(109)/21*y*z^2*u*v-2009*z^3*u*v+8812*x^2*u^2*v-10110*x*y*u^2*v-F(63)/58*y^2*u^2*v-F(1)/101*x*z*u^2*v-1555*y*z*u^2*v+12139*z^2*u^2*v+7778*x*u^3*v-9280*y*u^3*v+F(107)/73*z*u^3*v+F(27)/89*u^4*v-F(106)/33*x^3*v^2+F(46)/35*x^2*y*v^2-919*x*y^2*v^2+8793*y^3*v^2-F(99)/38*x^2*z*v^2-F(125)/71*x*y*z*v^2-F(58)/103*y^2*z*v^2+2015*x*z^2*v^2-F(99)/13*y*z^2*v^2-F(93)/76*z^3*v^2+F(19)/56*x^2*u*v^2+7142*x*y*u*v^2+F(65)/27*y^2*u*v^2-F(15)/83*x*z*u*v^2+15250*y*z*u*v^2+F(29)/83*z^2*u*v^2+F(22)/47*x*u^2*v^2-F(69)/83*y*u^2*v^2-F(109)/55*z*u^2*v^2+F(29)/13*u^3*v^2+F(123)/79*x^2*v^3-2300*x*y*v^3-F(79)/121*y^2*v^3-F(73)/8*x*z*v^3+7174*y*z*v^3+F(56)/71*z^2*v^3+10920*x*u*v^3+F(107)/83*y*u*v^3+F(111)/58*z*u*v^3+5502*u^2*v^3+F(118)/83*x*v^4-F(36)/43*y*v^4+5250*z*v^4+F(91)/114*u*v^4-15498*v^5]))
end


@doc raw"""
    enriques_d9_pi6()

Return a smooth Enriques surface in $\mathbb P^4$ with degree `9` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function enriques_d9_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(48)/55*x^4*u-F(101)/85*x^3*y*u-F(1)/44*x^2*y^2*u-1061*x*y^3*u+13221*y^4*u-F(123)/79*x^3*z*u-9796*x^2*y*z*u-3977*x*y^2*z*u+3897*y^3*z*u+F(13)/102*x^2*z^2*u-12368*x*y*z^2*u+F(45)/104*y^2*z^2*u-F(18)/47*x*z^3*u+9321*y*z^3*u-F(119)/40*z^4*u+228*x^3*u^2-F(37)/16*x^2*y*u^2-F(8)/71*x*y^2*u^2-15739*y^3*u^2+1027*x^2*z*u^2+F(121)/3*x*y*z*u^2+7257*y^2*z*u^2-F(59)/107*x*z^2*u^2+5383*y*z^2*u^2-F(77)/9*z^3*u^2-F(9)/71*x^2*u^3+7186*x*y*u^3-F(118)/99*y^2*u^3-F(124)/109*x*z*u^3+12726*y*z*u^3-F(29)/76*z^2*u^3-F(9)/70*x*u^4-2363*y*u^4+7749*z*u^4+6070*u^5+F(93)/98*x^4*v-F(79)/22*x^3*y*v-F(94)/43*x^2*y^2*v-7136*x*y^3*v-11534*y^4*v-6675*x^3*z*v+F(47)/82*x^2*y*z*v-F(3)/58*x*y^2*z*v-F(56)/9*y^3*z*v-F(33)/35*x^2*z^2*v-F(23)/113*x*y*z^2*v+F(63)/52*y^2*z^2*v+F(108)/89*x*z^3*v-12748*y*z^3*v-9118*z^4*v+F(80)/47*x^3*u*v-F(115)/67*x^2*y*u*v+4842*x*y^2*u*v+8285*y^3*u*v-13888*x^2*z*u*v+F(61)/118*x*y*z*u*v-13183*y^2*z*u*v+10928*x*z^2*u*v-9257*y*z^2*u*v-F(111)/59*z^3*u*v+F(92)/63*x^2*u^2*v+F(1)/96*x*y*u^2*v-F(44)/27*y^2*u^2*v+F(7)/124*x*z*u^2*v-4746*y*z*u^2*v-F(67)/22*z^2*u^2*v-1987*x*u^3*v+14264*y*u^3*v+7280*z*u^3*v+7145*u^4*v-129*x^3*v^2-F(112)/103*x^2*y*v^2+4069*x*y^2*v^2-5009*y^3*v^2-F(43)/65*x^2*z*v^2+F(28)/87*x*y*z*v^2-F(49)/48*y^2*z*v^2+474*x*z^2*v^2-F(49)/86*y*z^2*v^2-220*z^3*v^2+F(93)/8*x^2*u*v^2-F(19)/79*x*y*u*v^2+F(101)/65*y^2*u*v^2+F(29)/66*x*z*u*v^2+2490*y*z*u*v^2-F(113)/103*z^2*u*v^2-2215*x*u^2*v^2+12562*y*u^2*v^2-F(25)/56*z*u^2*v^2+F(101)/2*u^3*v^2-F(23)/30*x^2*v^3+238*x*y*v^3-F(35)/88*y^2*v^3+F(28)/47*x*z*v^3-9086*y*z*v^3+F(73)/126*z^2*v^3+1436*x*u*v^3-F(116)/3*y*u*v^3-F(27)/38*z*u*v^3+7471*u^2*v^3+F(96)/41*x*v^4-2363*y*v^4-8080*z*v^4+8608*u*v^4-F(12)/43*v^5,-F(63)/52*x^4*u+F(122)/75*x^3*y*u+15665*x^2*y^2*u+F(33)/37*x*y^3*u+F(49)/39*y^4*u-13351*x^3*z*u+F(29)/51*x^2*y*z*u-140*x*y^2*z*u+9553*y^3*z*u-F(41)/86*x^2*z^2*u+F(113)/118*x*y*z^2*u+F(11)/120*y^2*z^2*u-7683*x*z^3*u+F(103)/118*y*z^3*u+15013*z^4*u+15905*x^3*u^2+F(30)/119*x^2*y*u^2-F(11)/50*x*y^2*u^2-F(86)/81*y^3*u^2-F(112)/99*x^2*z*u^2+7961*x*y*z*u^2-F(80)/87*y^2*z*u^2+F(119)/121*x*z^2*u^2+F(73)/13*y*z^2*u^2+8652*z^3*u^2+F(88)/119*x^2*u^3+13184*x*y*u^3+F(75)/53*y^2*u^3-F(25)/117*x*z*u^3+12172*y*z*u^3-F(14)/43*z^2*u^3-F(39)/95*x*u^4-F(113)/93*y*u^4+F(8)/91*z*u^4-9524*u^5+8540*x^4*v-12026*x^3*y*v-8703*x^2*y^2*v+1576*x*y^3*v+9898*y^4*v+F(11)/21*x^3*z*v+F(117)/82*x^2*y*z*v-7186*x*y^2*z*v+9630*y^3*z*v-F(101)/33*x^2*z^2*v+7669*x*y*z^2*v+14443*y^2*z^2*v-11312*x*z^3*v-14620*y*z^3*v-3248*z^4*v-15029*x^3*u*v-9827*x^2*y*u*v+1606*x*y^2*u*v+F(33)/109*y^3*u*v-F(110)/73*x^2*z*u*v+11724*x*y*z*u*v+F(6)/41*y^2*z*u*v+7281*x*z^2*u*v-F(108)/71*y*z^2*u*v-1299*z^3*u*v-3847*x^2*u^2*v-F(76)/85*x*y*u^2*v-F(94)/117*y^2*u^2*v-7942*x*z*u^2*v+F(85)/54*y*z*u^2*v+F(109)/73*z^2*u^2*v-2543*x*u^3*v+13878*y*u^3*v-245*z*u^3*v+F(42)/65*u^4*v+7951*x^3*v^2-F(56)/71*x^2*y*v^2+F(95)/37*x*y^2*v^2-4286*y^3*v^2-11040*x^2*z*v^2-2495*x*y*z*v^2+F(95)/89*y^2*z*v^2-11360*x*z^2*v^2-10251*y*z^2*v^2+14787*z^3*v^2+9449*x^2*u*v^2+F(47)/9*x*y*u*v^2+F(82)/97*y^2*u*v^2+F(97)/82*x*z*u*v^2+F(53)/60*y*z*u*v^2-F(62)/63*z^2*u*v^2+F(108)/71*x*u^2*v^2+F(89)/93*y*u^2*v^2+F(47)/90*z*u^2*v^2-F(107)/102*u^3*v^2+15137*x^2*v^3+F(52)/73*x*y*v^3+F(95)/102*y^2*v^3+F(29)/23*x*z*v^3-F(112)/25*y*z*v^3+12716*z^2*v^3+F(107)/119*x*u*v^3-9620*y*u*v^3+F(8)/21*z*u*v^3+F(9)/34*u^2*v^3-11694*x*v^4-9825*y*v^4+15664*z*v^4-F(111)/46*u*v^4-F(105)/4*v^5,8607*x^4*u-F(17)/116*x^3*y*u-F(16)/27*x^2*y^2*u-233*x*y^3*u-12890*y^4*u+F(59)/73*x^3*z*u+F(28)/23*x^2*y*z*u-F(19)/121*x*y^2*z*u-F(20)/59*y^3*z*u+F(106)/111*x^2*z^2*u+F(32)/109*x*y*z^2*u-F(67)/126*y^2*z^2*u-5957*x*z^3*u-F(86)/3*y*z^3*u+8500*z^4*u+10326*x^3*u^2-F(34)/103*x^2*y*u^2+3*x*y^2*u^2+11749*y^3*u^2+F(59)/54*x^2*z*u^2-F(107)/66*x*y*z*u^2+F(80)/53*y^2*z*u^2+F(95)/27*x*z^2*u^2-F(41)/126*y*z^2*u^2-F(47)/28*z^3*u^2-F(61)/78*x^2*u^3+13737*x*y*u^3-F(13)/61*y^2*u^3-F(89)/42*x*z*u^3-F(81)/32*y*z*u^3+F(101)/32*z^2*u^3+3079*x*u^4-10324*y*u^4+F(95)/117*z*u^4-F(73)/10*u^5-F(11)/16*x^4*v+13313*x^3*y*v+F(39)/80*x^2*y^2*v-1109*x*y^3*v-1272*y^4*v+F(34)/97*x^3*z*v+15506*x^2*y*z*v-F(79)/62*x*y^2*z*v-F(126)/115*y^3*z*v+2542*x^2*z^2*v-5929*x*y*z^2*v+3650*y^2*z^2*v-12827*x*z^3*v-12548*y*z^3*v-6040*z^4*v-F(71)/78*x^3*u*v+F(37)/75*x^2*y*u*v-14862*x*y^2*u*v-F(19)/74*y^3*u*v-4546*x^2*z*u*v+F(37)/62*x*y*z*u*v-8652*y^2*z*u*v-11540*x*z^2*u*v-F(89)/93*y*z^2*u*v+12711*z^3*u*v+F(113)/62*x^2*u^2*v+15885*x*y*u^2*v+F(19)/29*y^2*u^2*v-F(62)/115*x*z*u^2*v-F(89)/72*y*z*u^2*v-F(10)/77*z^2*u^2*v+F(53)/34*x*u^3*v+13858*y*u^3*v-F(117)/14*z*u^3*v-12895*u^4*v+11121*x^3*v^2+F(37)/79*x^2*y*v^2-10256*x*y^2*v^2-7587*y^3*v^2-F(19)/51*x^2*z*v^2-F(52)/53*x*y*z*v^2+F(52)/115*y^2*z*v^2-F(53)/90*x*z^2*v^2+7086*y*z^2*v^2+F(123)/55*z^3*v^2-14142*x^2*u*v^2+F(27)/59*x*y*u*v^2-8686*y^2*u*v^2-F(48)/101*x*z*u*v^2+15698*y*z*u*v^2+4592*z^2*u*v^2+F(112)/83*x*u^2*v^2-F(26)/105*y*u^2*v^2+F(88)/27*z*u^2*v^2-F(19)/15*u^3*v^2+F(51)/65*x^2*v^3+1052*x*y*v^3+F(42)/11*y^2*v^3-3679*x*z*v^3+F(61)/125*y*z*v^3+F(35)/124*z^2*v^3-3737*x*u*v^3-2532*y*u*v^3-F(34)/111*z*u*v^3-F(27)/46*u^2*v^3+10092*x*v^4+F(86)/53*y*v^4+6350*z*v^4+F(43)/100*u*v^4-F(113)/115*v^5,-x^4*y+8744*x^3*y^2+4832*x^2*y^3+8547*x*y^4-F(19)/111*y^5-226*x^3*y*z-F(25)/76*x^2*y^2*z-F(11)/37*x*y^3*z+1533*y^4*z+F(15)/121*x^2*y*z^2+3508*x*y^2*z^2+6440*y^3*z^2-13868*x*y*z^3-F(16)/41*y^2*z^3+F(36)/35*y*z^4+F(70)/89*x^4*u-F(101)/48*x^3*y*u-13344*x^2*y^2*u+F(19)/54*x*y^3*u-4059*y^4*u-11558*x^3*z*u+6635*x^2*y*z*u-10606*x*y^2*z*u-F(11)/117*y^3*z*u+F(44)/69*x^2*z^2*u+7864*x*y*z^2*u-3290*y^2*z^2*u+F(14)/67*x*z^3*u+F(49)/3*y*z^3*u+F(113)/60*z^4*u-1357*x^3*u^2-F(31)/100*x^2*y*u^2+643*x*y^2*u^2+F(27)/125*y^3*u^2-F(45)/61*x^2*z*u^2+F(30)/13*x*y*z*u^2+F(53)/109*y^2*z*u^2-8077*x*z^2*u^2+F(73)/77*y*z^2*u^2-7316*z^3*u^2+12425*x^2*u^3-3908*x*y*u^3+4327*y^2*u^3+F(52)/119*x*z*u^3-F(76)/69*y*z*u^3-12539*z^2*u^3+F(121)/74*x*u^4+5671*y*u^4+15685*z*u^4-277*u^5+13462*x^4*v+F(75)/8*x^3*y*v+F(17)/15*x^2*y^2*v-5194*x*y^3*v-4449*y^4*v+F(21)/65*x^3*z*v-5541*x^2*y*z*v-F(113)/38*x*y^2*z*v+F(118)/45*y^3*z*v-11229*x^2*z^2*v+F(107)/26*x*y*z^2*v-F(27)/10*y^2*z^2*v+F(110)/27*x*z^3*v-8426*y*z^3*v-12838*z^4*v-11648*x^3*u*v+F(4)/21*x^2*y*u*v-F(11)/3*x*y^2*u*v-F(26)/15*y^3*u*v+3698*x^2*z*u*v+7692*x*y*z*u*v-F(46)/87*y^2*z*u*v-13430*x*z^2*u*v-803*y*z^2*u*v+8867*z^3*u*v-F(31)/96*x^2*u^2*v+F(24)/121*x*y*u^2*v-3303*y^2*u^2*v+F(7)/99*x*z*u^2*v-131*y*z*u^2*v+F(47)/69*z^2*u^2*v+F(43)/37*x*u^3*v+11976*y*u^3*v+F(87)/112*z*u^3*v+F(77)/52*u^4*v-F(91)/37*x^3*v^2-F(54)/47*x^2*y*v^2-F(104)/55*x*y^2*v^2-15528*y^3*v^2-F(109)/4*x^2*z*v^2-F(69)/79*x*y*z*v^2-1906*y^2*z*v^2+14270*x*z^2*v^2+F(30)/19*y*z^2*v^2-F(5)/118*z^3*v^2-F(103)/6*x^2*u*v^2+F(121)/106*x*y*u*v^2-9205*y^2*u*v^2-F(76)/15*x*z*u*v^2-5583*y*z*u*v^2-2955*z^2*u*v^2+F(20)/29*x*u^2*v^2-11967*y*u^2*v^2-F(59)/55*z*u^2*v^2-F(74)/117*u^3*v^2-F(59)/58*x^2*v^3-F(37)/43*x*y*v^3+F(113)/90*y^2*v^3-F(22)/87*x*z*v^3-F(111)/76*y*z*v^3-8321*z^2*v^3+F(19)/117*x*u*v^3-7879*y*u*v^3-F(95)/22*z*u*v^3-13622*u^2*v^3+1590*x*v^4+14669*y*v^4-F(43)/72*z*v^4-7963*u*v^4-4657*v^5,7006*x^4*y+2301*x^3*y^2-6514*x^2*y^3+6870*x*y^4-15805*y^5-F(48)/55*x^4*z+13540*x^3*y*z+F(7)/23*x^2*y^2*z-F(91)/58*x*y^3*z+F(7)/36*y^4*z+F(123)/79*x^3*z^2+1789*x^2*y*z^2-3983*x*y^2*z^2-15227*y^3*z^2-F(13)/102*x^2*z^3+F(59)/103*x*y*z^3+2396*y^2*z^3+F(18)/47*x*z^4-11043*y*z^4+F(119)/40*z^5-F(24)/43*x^4*u+13744*x^3*y*u-14245*x^2*y^2*u-2394*x*y^3*u-3830*y^4*u+15518*x^3*z*u-F(85)/69*x^2*y*z*u-F(59)/75*x*y^2*z*u-F(29)/40*y^3*z*u-10500*x^2*z^2*u-F(81)/37*x*y*z^2*u-F(121)/123*y^2*z^2*u-F(36)/65*x*z^3*u-F(113)/125*y*z^3*u+3054*z^4*u-8159*x^3*u^2-F(87)/112*x^2*y*u^2+F(20)/111*x*y^2*u^2-F(60)/101*y^3*u^2+4853*x^2*z*u^2-3152*x*y*z*u^2+7352*y^2*z*u^2+5004*x*z^2*u^2+13200*y*z^2*u^2+6305*z^3*u^2-F(63)/53*x^2*u^3+9872*x*y*u^3-F(27)/62*y^2*u^3+F(5)/66*x*z*u^3+F(35)/23*y*z*u^3+F(81)/73*z^2*u^3+13738*x*u^4+F(35)/103*y*u^4+F(9)/83*z*u^4+F(15)/97*u^5-F(71)/41*x^4*v-F(60)/119*x^3*y*v+7940*x^2*y^2*v-7777*x*y^3*v+F(121)/19*y^4*v-12109*x^3*z*v-F(28)/109*x^2*y*z*v-F(80)/41*x*y^2*z*v+F(23)/122*y^3*z*v+F(27)/53*x^2*z^2*v+13269*x*y*z^2*v-F(57)/71*y^2*z^2*v+F(77)/125*x*z^3*v+F(8)/93*y*z^3*v-F(53)/109*z^4*v+F(7)/68*x^3*u*v-F(86)/25*x^2*y*u*v-F(114)/125*x*y^2*u*v-F(53)/105*y^3*u*v+F(60)/113*x^2*z*u*v+10277*x*y*z*u*v+1288*y^2*z*u*v+F(23)/79*x*z^2*u*v-F(82)/63*y*z^2*u*v-14477*z^3*u*v+F(89)/41*x^2*u^2*v+F(43)/124*x*y*u^2*v+6226*y^2*u^2*v-F(2)/89*x*z*u^2*v+F(97)/15*y*z*u^2*v-F(8)/11*z^2*u^2*v+12609*x*u^3*v+14513*y*u^3*v+F(41)/27*z*u^3*v+F(61)/70*u^4*v-F(52)/7*x^3*v^2-F(59)/15*x^2*y*v^2-F(70)/73*x*y^2*v^2+F(27)/35*y^3*v^2+F(16)/77*x^2*z*v^2+2406*x*y*z*v^2-F(5)/113*y^2*z*v^2+F(47)/55*x*z^2*v^2+F(106)/75*y*z^2*v^2-2615*z^3*v^2-F(122)/13*x^2*u*v^2+F(41)/6*x*y*u*v^2+423*y^2*u*v^2+8506*x*z*u*v^2-F(101)/81*y*z*u*v^2+35*z^2*u*v^2+F(59)/56*x*u^2*v^2-F(17)/66*y*u^2*v^2-F(81)/107*z*u^2*v^2-2854*u^3*v^2-15437*x^2*v^3+F(89)/88*x*y*v^3+13350*y^2*v^3-14781*x*z*v^3-F(82)/59*y*z*v^3-1765*z^2*v^3+2951*x*u*v^3+F(47)/105*y*u*v^3-F(61)/94*z*u*v^3-F(13)/19*u^2*v^3-5145*x*v^4-12357*y*v^4-F(104)/53*z*v^4+7304*u*v^4+F(77)/19*v^5,F(58)/83*x^4*y-F(109)/71*x^3*y^2-963*x^2*y^3+11713*x*y^4+965*y^5+4628*x^4*z+F(49)/47*x^3*y*z+14128*x^2*y^2*z-496*x*y^3*z-7410*y^4*z-F(121)/87*x^3*z^2+F(121)/124*x^2*y*z^2-11457*x*y^2*z^2-F(34)/37*y^3*z^2-F(121)/5*x^2*z^3-7070*x*y*z^3-F(5)/122*y^2*z^3-F(15)/61*x*z^4+F(44)/123*y*z^4-F(2)/13*z^5-F(85)/111*x^4*u-107*x^3*y*u+F(92)/15*x^2*y^2*u+F(27)/88*x*y^3*u+3662*y^4*u-F(107)/118*x^3*z*u-6699*x^2*y*z*u+F(79)/68*x*y^2*z*u+F(106)/97*y^3*z*u-F(50)/97*x^2*z^2*u-F(81)/85*x*y*z^2*u+2848*y^2*z^2*u+F(77)/62*x*z^3*u+13511*y*z^3*u-15365*z^4*u+F(97)/98*x^3*u^2-9860*x^2*y*u^2-F(46)/25*x*y^2*u^2+F(122)/43*y^3*u^2+11165*x^2*z*u^2-F(99)/89*x*y*z*u^2+3399*y^2*z*u^2-F(80)/17*x*z^2*u^2-F(2)/3*y*z^2*u^2-F(108)/101*z^3*u^2+15010*x^2*u^3-1953*x*y*u^3+F(97)/110*y^2*u^3-F(83)/110*x*z*u^3-10832*y*z*u^3-F(49)/66*z^2*u^3-7316*x*u^4-F(17)/59*y*u^4-3724*z*u^4-F(117)/28*u^5+F(71)/45*x^4*v+F(33)/122*x^3*y*v-1446*x^2*y^2*v+750*x*y^3*v+F(76)/35*y^4*v-F(29)/102*x^3*z*v-13503*x^2*y*z*v+F(2)/33*x*y^2*z*v+9009*y^3*z*v+F(25)/51*x^2*z^2*v-F(59)/92*x*y*z^2*v+F(50)/9*y^2*z^2*v-F(123)/62*x*z^3*v+4589*y*z^3*v+8136*z^4*v-F(80)/83*x^3*u*v-9455*x^2*y*u*v+F(113)/7*x*y^2*u*v+F(111)/4*y^3*u*v-F(51)/10*x^2*z*u*v+F(14)/93*x*y*z*u*v-F(58)/59*y^2*z*u*v+7706*x*z^2*u*v+F(43)/82*y*z^2*u*v-F(95)/79*z^3*u*v+F(44)/57*x^2*u^2*v+F(79)/16*x*y*u^2*v+F(5)/81*y^2*u^2*v+F(85)/94*x*z*u^2*v+F(94)/101*y*z*u^2*v-7742*z^2*u^2*v-F(52)/7*x*u^3*v+4167*y*u^3*v+F(7)/19*z*u^3*v-F(112)/83*u^4*v-F(113)/20*x^3*v^2+4022*x^2*y*v^2-F(80)/59*x*y^2*v^2+F(5)/121*y^3*v^2-4502*x^2*z*v^2+2922*x*y*z*v^2+8862*y^2*z*v^2-15818*x*z^2*v^2+F(47)/9*y*z^2*v^2+15173*z^3*v^2-6468*x^2*u*v^2-F(3)/95*x*y*u*v^2+408*y^2*u*v^2-F(61)/36*x*z*u*v^2+F(93)/106*y*z*u*v^2+F(117)/94*z^2*u*v^2-F(81)/94*x*u^2*v^2-12872*y*u^2*v^2+F(62)/77*z*u^2*v^2+F(106)/67*u^3*v^2+13118*x^2*v^3-10029*x*y*v^3+9070*y^2*v^3-F(57)/49*x*z*v^3+F(29)/63*y*z*v^3-12659*z^2*v^3+2269*x*u*v^3-F(44)/117*y*u*v^3-F(34)/121*z*u*v^3+12609*u^2*v^3+11003*x*v^4+6658*y*v^4+132*z*v^4+15489*u*v^4+9304*v^5,-8432*x^4*y-F(71)/93*x^3*y^2+F(106)/61*x^2*y^3+12021*x*y^4+F(107)/70*y^5-F(100)/21*x^4*z-F(64)/81*x^3*y*z-5861*x^2*y^2*z-F(47)/72*x*y^3*z+F(121)/125*y^4*z+F(79)/24*x^3*z^2-596*x^2*y*z^2-F(101)/41*x*y^2*z^2-F(1)/51*y^3*z^2-1137*x^2*z^3+F(53)/93*x*y*z^3-3073*y^2*z^3-354*x*z^4-F(21)/83*y*z^4+4395*z^5+1436*x^4*u+F(75)/124*x^3*y*u+4437*x^2*y^2*u+F(41)/104*x*y^3*u-F(17)/113*y^4*u-F(31)/3*x^3*z*u+14242*x^2*y*z*u-F(15)/106*x*y^2*z*u+13185*y^3*z*u-5364*x^2*z^2*u-10338*x*y*z^2*u+4181*y^2*z^2*u+F(66)/7*x*z^3*u-9823*y*z^3*u-15040*z^4*u+F(1)/45*x^3*u^2+F(77)/73*x^2*y*u^2-10585*x*y^2*u^2+F(117)/73*y^3*u^2+6772*x^2*z*u^2-F(14)/39*x*y*z*u^2-1555*y^2*z*u^2+F(43)/103*x*z^2*u^2-5881*y*z^2*u^2-124*z^3*u^2+2749*x^2*u^3-10313*x*y*u^3-12146*y^2*u^3+F(5)/27*x*z*u^3-F(9)/26*y*z*u^3+F(47)/43*z^2*u^3+14619*x*u^4-F(13)/63*y*u^4-F(71)/109*z*u^4-F(62)/65*u^5+F(13)/30*x^4*v-F(95)/108*x^3*y*v+F(8)/47*x^2*y^2*v+335*x*y^3*v-F(77)/102*y^4*v+F(102)/53*x^3*z*v-F(59)/86*x^2*y*z*v+F(24)/79*x*y^2*z*v-F(101)/5*y^3*z*v-F(95)/73*x^2*z^2*v+11509*x*y*z^2*v+11530*y^2*z^2*v-F(33)/14*x*z^3*v+F(5)/61*y*z^3*v+F(37)/105*z^4*v-F(49)/19*x^3*u*v-2809*x^2*y*u*v-F(6)/125*x*y^2*u*v-4166*y^3*u*v+F(1)/118*x^2*z*u*v-F(28)/13*x*y*z*u*v+F(87)/124*y^2*z*u*v-F(109)/19*x*z^2*u*v-F(5)/84*y*z^2*u*v-13460*z^3*u*v-F(101)/107*x^2*u^2*v-F(24)/41*x*y*u^2*v-10122*y^2*u^2*v-F(58)/65*x*z*u^2*v+8573*y*z*u^2*v+13293*z^2*u^2*v-F(122)/85*x*u^3*v+3751*y*u^3*v-3772*z*u^3*v-F(71)/20*u^4*v-9076*x^3*v^2+9187*x^2*y*v^2+14455*x*y^2*v^2-13939*y^3*v^2-F(109)/118*x^2*z*v^2+F(92)/111*x*y*z*v^2-F(93)/59*y^2*z*v^2-F(19)/42*x*z^2*v^2+6612*y*z^2*v^2+F(126)/17*z^3*v^2+F(115)/107*x^2*u*v^2-11233*x*y*u*v^2+F(8)/83*y^2*u*v^2+F(89)/75*x*z*u*v^2+F(17)/87*y*z*u*v^2-3162*z^2*u*v^2-15558*x*u^2*v^2+F(57)/119*y*u^2*v^2+F(61)/4*z*u^2*v^2+F(13)/10*u^3*v^2-F(65)/4*x^2*v^3+13244*x*y*v^3-F(111)/49*y^2*v^3+1492*x*z*v^3-11968*y*z*v^3+F(73)/92*z^2*v^3-F(123)/40*x*u*v^3+F(32)/77*y*u*v^3-F(47)/9*z*u*v^3-1082*u^2*v^3-7019*x*v^4-10463*y*v^4-10401*z*v^4+F(7)/33*u*v^4-5496*v^5,10589*x^4*y+13983*x^3*y^2-10186*x^2*y^3-F(91)/88*x*y^4-F(17)/43*y^5-5748*x^4*z-F(35)/48*x^3*y*z+F(43)/105*x^2*y^2*z-9105*x*y^3*z+F(13)/75*y^4*z+F(92)/47*x^3*z^2+4818*x^2*y*z^2-10401*x*y^2*z^2+7167*y^3*z^2-4633*x^2*z^3-13347*x*y*z^3-F(46)/7*y^2*z^3-F(118)/17*x*z^4+143*y*z^4+F(107)/91*z^5+F(89)/37*x^4*u+7538*x^3*y*u-F(47)/46*x^2*y^2*u-7369*x*y^3*u-13809*y^4*u+F(33)/47*x^3*z*u+F(27)/124*x^2*y*z*u-F(25)/34*x*y^2*z*u+F(18)/109*y^3*z*u-10805*x^2*z^2*u+5706*x*y*z^2*u+F(109)/10*y^2*z^2*u-F(13)/11*x*z^3*u-F(15)/11*y*z^3*u+F(89)/52*z^4*u-209*x^3*u^2-F(13)/7*x^2*y*u^2-7581*x*y^2*u^2+6792*y^3*u^2+F(116)/123*x^2*z*u^2+F(1)/123*x*y*z*u^2-F(10)/11*y^2*z*u^2-F(27)/119*x*z^2*u^2+12667*y*z^2*u^2-F(58)/77*z^3*u^2+11797*x^2*u^3-F(123)/37*x*y*u^3+F(98)/15*y^2*u^3-3237*x*z*u^3-F(47)/56*y*z*u^3+F(8)/95*z^2*u^3-8991*x*u^4-F(119)/26*y*u^4-F(62)/47*z*u^4+F(38)/5*u^5-5309*x^4*v+9201*x^3*y*v-F(35)/41*x^2*y^2*v+15000*x*y^3*v+1435*y^4*v-5166*x^3*z*v-F(83)/67*x^2*y*z*v+F(29)/11*x*y^2*z*v-F(4)/61*y^3*z*v+F(109)/72*x^2*z^2*v+F(4)/101*x*y*z^2*v+13732*y^2*z^2*v+1120*x*z^3*v+F(78)/47*y*z^3*v-10091*z^4*v+F(20)/41*x^3*u*v-F(58)/33*x^2*y*u*v-F(22)/19*x*y^2*u*v+1223*y^3*u*v-F(61)/71*x^2*z*u*v-F(4)/27*x*y*z*u*v-F(70)/17*y^2*z*u*v+11119*x*z^2*u*v+13423*y*z^2*u*v-F(119)/13*z^3*u*v+5081*x^2*u^2*v+12110*x*y*u^2*v+9*y^2*u^2*v+F(122)/65*x*z*u^2*v+F(36)/29*y*z*u^2*v+1461*z^2*u^2*v+3336*x*u^3*v+F(80)/7*y*u^3*v+F(74)/31*z*u^3*v-2869*u^4*v+F(73)/29*x^3*v^2-F(14)/59*x^2*y*v^2-F(86)/83*x*y^2*v^2+4254*y^3*v^2+F(5)/98*x^2*z*v^2-F(40)/29*x*y*z*v^2+1532*y^2*z*v^2+F(34)/75*x*z^2*v^2+2788*y*z^2*v^2+F(55)/46*z^3*v^2+F(112)/99*x^2*u*v^2-F(61)/111*x*y*u*v^2+3699*y^2*u*v^2-F(11)/48*x*z*u*v^2-F(101)/35*y*z*u*v^2+F(58)/57*z^2*u*v^2-F(5)/46*x*u^2*v^2+F(40)/69*y*u^2*v^2-14944*z*u^2*v^2-F(83)/43*u^3*v^2-F(115)/63*x^2*v^3+F(81)/79*x*y*v^3-F(103)/71*y^2*v^3+F(1)/126*x*z*v^3+3823*y*z*v^3+14370*z^2*v^3-10868*x*u*v^3-2394*y*u*v^3+F(75)/74*z*u*v^3-F(2)/77*u^2*v^3+F(117)/112*x*v^4-2889*y*v^4+220*z*v^4+F(55)/53*u*v^4+11186*v^5,12721*x^4*y-F(109)/49*x^3*y^2-F(62)/37*x^2*y^3+F(69)/71*x*y^4-F(109)/90*y^5+F(78)/71*x^4*z+F(12)/43*x^3*y*z-12768*x^2*y^2*z-14085*x*y^3*z-6133*y^4*z+F(1)/124*x^3*z^2-F(84)/79*x^2*y*z^2+10310*x*y^2*z^2+F(107)/3*y^3*z^2-4182*x^2*z^3-F(9)/100*x*y*z^3-F(106)/21*y^2*z^3+1556*x*z^4-F(27)/67*y*z^4-F(64)/17*z^5-F(42)/23*x^4*u-8745*x^3*y*u-F(44)/53*x^2*y^2*u+F(103)/86*x*y^3*u+F(50)/23*y^4*u+F(103)/66*x^3*z*u-15731*x^2*y*z*u-F(15)/101*x*y^2*z*u-4852*y^3*z*u-F(86)/3*x^2*z^2*u-F(65)/103*x*y*z^2*u-F(73)/9*y^2*z^2*u-F(41)/49*x*z^3*u+12975*y*z^3*u-3572*z^4*u-F(11)/115*x^3*u^2+F(89)/59*x^2*y*u^2+776*x*y^2*u^2-9167*y^3*u^2-607*x^2*z*u^2+F(58)/119*x*y*z*u^2+3791*y^2*z*u^2-8964*x*z^2*u^2-10604*y*z^2*u^2-F(106)/7*z^3*u^2+F(10)/97*x^2*u^3+127*x*y*u^3-12160*y^2*u^3+F(36)/31*x*z*u^3-13928*y*z*u^3-F(123)/26*z^2*u^3+F(67)/89*x*u^4-7124*y*u^4+F(88)/57*z*u^4-6362*u^5+F(59)/115*x^4*v-F(17)/15*x^3*y*v+F(119)/25*x^2*y^2*v-F(61)/109*x*y^3*v+F(118)/3*y^4*v+F(62)/11*x^3*z*v-6504*x^2*y*z*v-3614*x*y^2*z*v+6436*y^3*z*v-15910*x^2*z^2*v-F(103)/23*x*y*z^2*v+9038*y^2*z^2*v+13982*x*z^3*v+F(121)/111*y*z^3*v+F(59)/38*z^4*v+F(17)/37*x^3*u*v-F(95)/8*x^2*y*u*v-F(99)/29*x*y^2*u*v+F(31)/85*y^3*u*v+1677*x^2*z*u*v+F(103)/17*x*y*z*u*v+F(3)/85*y^2*z*u*v-F(117)/53*x*z^2*u*v+11262*y*z^2*u*v-F(24)/41*z^3*u*v+F(34)/35*x^2*u^2*v+F(116)/111*x*y*u^2*v-8278*y^2*u^2*v-4355*x*z*u^2*v-F(113)/28*y*z*u^2*v-F(123)/26*z^2*u^2*v+243*x*u^3*v-F(53)/20*y*u^3*v+F(97)/75*z*u^3*v-4594*u^4*v-13142*x^3*v^2-F(29)/96*x^2*y*v^2-F(113)/28*x*y^2*v^2-F(59)/39*y^3*v^2+7669*x^2*z*v^2+F(119)/46*x*y*z*v^2+9319*y^2*z*v^2+7595*x*z^2*v^2-3772*y*z^2*v^2+F(120)/7*z^3*v^2+1991*x^2*u*v^2+11209*x*y*u*v^2+F(125)/37*y^2*u*v^2-15829*x*z*u*v^2+F(45)/59*y*z*u*v^2+F(113)/44*z^2*u*v^2-F(29)/77*x*u^2*v^2-F(126)/59*y*u^2*v^2-15465*z*u^2*v^2-F(118)/33*u^3*v^2-F(53)/101*x^2*v^3+10720*x*y*v^3-4542*y^2*v^3-14094*x*z*v^3-3906*y*z*v^3-F(14)/37*z^2*v^3+F(122)/91*x*u*v^3-14522*y*u*v^3+F(86)/123*z*u*v^3+15499*u^2*v^3-F(19)/52*x*v^4-9173*y*v^4-836*z*v^4+F(95)/122*u*v^4+F(97)/68*v^5,13246*x^4*y+1724*x^3*y^2+F(39)/70*x^2*y^3+F(45)/29*x*y^4+9937*y^5-F(71)/8*x^4*z-F(9)/52*x^3*y*z-558*x^2*y^2*z+F(111)/19*x*y^3*z+15885*y^4*z-F(58)/13*x^3*z^2+F(31)/57*x^2*y*z^2+F(13)/70*x*y^2*z^2-1402*y^3*z^2+F(41)/124*x^2*z^3-15924*x*y*z^3+F(22)/119*y^2*z^3-F(51)/41*x*z^4+F(67)/77*y*z^4-F(122)/103*z^5+3925*x^4*u-F(57)/5*x^3*y*u+F(85)/61*x^2*y^2*u-565*x*y^3*u+F(7)/108*y^4*u-6870*x^3*z*u+F(113)/60*x^2*y*z*u-F(63)/62*x*y^2*z*u+3682*y^3*z*u-3038*x^2*z^2*u+F(61)/63*x*y*z^2*u+8221*y^2*z^2*u+F(34)/65*x*z^3*u+F(49)/57*y*z^3*u+F(121)/52*z^4*u+10587*x^3*u^2-F(55)/113*x^2*y*u^2-F(77)/2*x*y^2*u^2+13617*y^3*u^2-F(91)/16*x^2*z*u^2-F(94)/95*x*y*z*u^2-8140*y^2*z*u^2-2114*x*z^2*u^2+F(67)/33*y*z^2*u^2-11818*z^3*u^2+F(29)/85*x^2*u^3-F(13)/103*x*y*u^3+6547*y^2*u^3+F(29)/78*x*z*u^3-F(50)/23*y*z*u^3+244*z^2*u^3-8502*x*u^4+15567*y*u^4+F(72)/113*z*u^4-F(43)/42*u^5+F(27)/115*x^4*v+F(73)/55*x^3*y*v+F(62)/109*x^2*y^2*v-6269*x*y^3*v+13345*y^4*v-F(120)/13*x^3*z*v-2268*x^2*y*z*v-F(41)/30*x*y^2*z*v+1096*y^3*z*v-2848*x^2*z^2*v-7178*x*y*z^2*v-F(61)/23*y^2*z^2*v+3184*x*z^3*v+F(121)/100*y*z^3*v-13592*z^4*v+F(126)/85*x^3*u*v+3023*x^2*y*u*v-F(103)/75*x*y^2*u*v+F(90)/79*y^3*u*v+5605*x^2*z*u*v-48*x*y*z*u*v+F(79)/44*y^2*z*u*v-F(103)/69*x*z^2*u*v+F(86)/91*y*z^2*u*v-F(99)/97*z^3*u*v+6063*x^2*u^2*v+F(61)/111*x*y*u^2*v+F(103)/5*y^2*u^2*v-F(28)/37*x*z*u^2*v-9987*y*z*u^2*v-3662*z^2*u^2*v+F(101)/104*x*u^3*v+F(49)/100*y*u^3*v-12219*z*u^3*v+F(89)/37*u^4*v+2255*x^3*v^2-F(12)/25*x^2*y*v^2-F(13)/124*x*y^2*v^2-F(90)/53*y^3*v^2-6070*x^2*z*v^2+F(28)/43*x*y*z*v^2-4690*y^2*z*v^2-F(113)/102*x*z^2*v^2+6033*y*z^2*v^2+F(2)/55*z^3*v^2+F(96)/89*x^2*u*v^2+F(102)/23*x*y*u*v^2-F(113)/40*y^2*u*v^2-F(107)/19*x*z*u*v^2+F(43)/7*y*z*u*v^2-4309*z^2*u*v^2+F(27)/16*x*u^2*v^2+F(23)/52*y*u^2*v^2+F(7)/23*z*u^2*v^2+F(40)/63*u^3*v^2-F(91)/60*x^2*v^3-F(110)/117*x*y*v^3-F(83)/18*y^2*v^3+F(91)/67*x*z*v^3-F(26)/67*y*z*v^3+F(33)/53*z^2*v^3-3242*x*u*v^3+F(61)/82*y*u*v^3-F(38)/75*z*u*v^3+F(98)/69*u^2*v^3+4851*x*v^4-F(14)/113*y*v^4+4122*z*v^4+F(13)/103*u*v^4+F(67)/122*v^5,-F(64)/45*x^4*z+F(10)/17*x^3*y*z-F(126)/121*x^2*y^2*z+F(86)/111*x*y^3*z+F(40)/23*y^4*z+12479*x^3*z^2-F(59)/47*x^2*y*z^2+F(4)/51*x*y^2*z^2+977*y^3*z^2+F(3)/17*x^2*z^3-11180*x*y*z^3-F(43)/61*y^2*z^3+F(7)/82*x*z^4+13027*y*z^4-F(30)/11*z^5+F(71)/84*x^4*u-F(16)/77*x^3*y*u-4408*x^2*y^2*u-F(91)/68*x*y^3*u-14525*y^4*u-13691*x^3*z*u+F(35)/101*x^2*y*z*u-F(6)/77*x*y^2*z*u-F(108)/49*y^3*z*u-F(42)/121*x^2*z^2*u-71*x*y*z^2*u-F(24)/79*y^2*z^2*u+10972*x*z^3*u-3303*y*z^3*u-F(126)/109*z^4*u+F(101)/15*x^3*u^2-F(29)/7*x^2*y*u^2+7082*x*y^2*u^2+F(1)/42*y^3*u^2-F(28)/55*x^2*z*u^2+F(3)/38*x*y*z*u^2-F(98)/79*y^2*z*u^2+F(87)/79*x*z^2*u^2-F(17)/110*y*z^2*u^2+F(49)/125*z^3*u^2-F(99)/95*x^2*u^3-8057*x*y*u^3+F(91)/100*y^2*u^3+F(111)/59*x*z*u^3+F(5)/102*y*z*u^3+F(101)/91*z^2*u^3-F(101)/110*x*u^4-F(85)/28*y*u^4+F(37)/72*z*u^4-2011*u^5-F(9)/14*x^4*v+F(23)/75*x^3*y*v-F(83)/94*x^2*y^2*v-F(81)/67*x*y^3*v+8658*y^4*v+F(33)/14*x^3*z*v-8230*x^2*y*z*v+F(60)/43*x*y^2*z*v-F(23)/64*y^3*z*v-F(113)/90*x^2*z^2*v+F(16)/5*x*y*z^2*v-F(72)/79*y^2*z^2*v-F(46)/37*x*z^3*v-1987*y*z^3*v+5291*z^4*v-631*x^3*u*v-7092*x^2*y*u*v+F(29)/100*x*y^2*u*v-F(61)/72*y^3*u*v-11040*x^2*z*u*v+F(10)/93*x*y*z*u*v-7760*y^2*z*u*v-4604*x*z^2*u*v+F(121)/73*y*z^2*u*v+4080*z^3*u*v+3212*x^2*u^2*v-F(64)/25*x*y*u^2*v+F(49)/113*y^2*u^2*v-7742*x*z*u^2*v-15137*y*z*u^2*v+F(46)/31*z^2*u^2*v-F(71)/80*x*u^3*v+12748*y*u^3*v-F(19)/103*z*u^3*v-7042*u^4*v-F(39)/10*x^3*v^2-F(16)/5*x^2*y*v^2-F(1)/47*x*y^2*v^2-12068*y^3*v^2+6571*x^2*z*v^2+F(111)/97*x*y*z*v^2-12364*y^2*z*v^2-838*x*z^2*v^2+F(13)/92*y*z^2*v^2-F(27)/98*z^3*v^2+F(9)/110*x^2*u*v^2-11899*x*y*u*v^2-9862*y^2*u*v^2-3832*x*z*u*v^2+F(16)/113*y*z*u*v^2-4058*z^2*u*v^2+2475*x*u^2*v^2-F(105)/92*y*u^2*v^2-F(79)/52*z*u^2*v^2+10731*u^3*v^2+14236*x^2*v^3+F(81)/110*x*y*v^3+1409*y^2*v^3-F(28)/71*x*z*v^3+F(51)/4*y*z*v^3-F(33)/41*z^2*v^3+F(124)/7*x*u*v^3+F(6)/97*y*u*v^3+15779*z*u*v^3+F(122)/91*u^2*v^3-F(29)/54*x*v^4+9266*y*v^4-7460*z*v^4+10305*u*v^4-8336*v^5,F(67)/99*x^4*y-F(124)/107*x^3*y^2-12237*x^2*y^3-8605*x*y^4+F(116)/33*y^5-F(20)/69*x^4*z-1383*x^3*y*z-F(113)/16*x^2*y^2*z+14793*x*y^3*z-15458*y^4*z+14885*x^3*z^2+F(7)/101*x^2*y*z^2+F(91)/57*x*y^2*z^2+F(54)/119*y^3*z^2+F(17)/126*x^2*z^3-F(15)/17*x*y*z^3+145*y^2*z^3-8679*x*z^4+8757*y*z^4+F(108)/101*z^5-F(34)/93*x^4*u-14889*x^3*y*u+10928*x^2*y^2*u+F(1)/52*x*y^3*u+11738*y^4*u+F(83)/58*x^3*z*u-F(58)/89*x^2*y*z*u+F(61)/99*x*y^2*z*u-F(83)/92*y^3*z*u-F(35)/58*x^2*z^2*u+F(13)/49*x*y*z^2*u-F(10)/79*y^2*z^2*u-5947*x*z^3*u-10595*y*z^3*u-14468*z^4*u-F(126)/73*x^3*u^2-5988*x^2*y*u^2-F(123)/17*x*y^2*u^2-3346*y^3*u^2+F(50)/41*x^2*z*u^2-F(67)/28*x*y*z*u^2+10973*y^2*z*u^2+F(1)/75*x*z^2*u^2-6424*y*z^2*u^2+F(81)/14*z^3*u^2+14896*x^2*u^3-1611*x*y*u^3-9486*y^2*u^3-F(4)/117*x*z*u^3+F(23)/59*y*z*u^3+7066*z^2*u^3+8948*x*u^4-12834*y*u^4-F(32)/55*z*u^4+F(79)/71*u^5-F(115)/31*x^4*v+F(49)/58*x^3*y*v+7470*x^2*y^2*v+11815*x*y^3*v-14703*y^4*v-F(89)/53*x^3*z*v+164*x^2*y*z*v+F(81)/125*x*y^2*z*v-3684*y^3*z*v-14687*x^2*z^2*v+F(49)/111*x*y*z^2*v-737*y^2*z^2*v+10604*x*z^3*v-8336*y*z^3*v+F(83)/54*z^4*v-F(73)/115*x^3*u*v-F(92)/65*x^2*y*u*v+F(24)/71*x*y^2*u*v+11234*y^3*u*v+8483*x^2*z*u*v-6653*x*y*z*u*v-2276*y^2*z*u*v+F(37)/6*x*z^2*u*v+F(101)/23*y*z^2*u*v+F(9)/70*z^3*u*v+F(51)/122*x^2*u^2*v+7483*x*y*u^2*v-F(30)/103*y^2*u^2*v+F(35)/44*x*z*u^2*v+F(87)/112*y*z*u^2*v-1902*z^2*u^2*v-8988*x*u^3*v+9074*y*u^3*v-F(83)/105*z*u^3*v+F(21)/89*u^4*v+F(101)/74*x^3*v^2+F(123)/110*x^2*y*v^2-F(25)/49*x*y^2*v^2+549*y^3*v^2-5425*x^2*z*v^2+F(110)/111*x*y*z*v^2-11002*y^2*z*v^2+F(34)/21*x*z^2*v^2+F(109)/55*y*z^2*v^2+F(70)/53*z^3*v^2-F(39)/4*x^2*u*v^2+F(107)/35*x*y*u*v^2-F(61)/6*y^2*u*v^2-F(71)/96*x*z*u*v^2+F(117)/98*y*z*u*v^2-6443*z^2*u*v^2-F(68)/27*x*u^2*v^2+F(28)/51*y*u^2*v^2-F(65)/12*z*u^2*v^2+225*u^3*v^2-F(10)/79*x^2*v^3-14101*x*y*v^3-F(111)/52*y^2*v^3+F(65)/51*x*z*v^3+F(34)/43*y*z*v^3-F(63)/79*z^2*v^3-F(18)/119*x*u*v^3+F(29)/52*y*u*v^3+2256*z*u*v^3-3296*u^2*v^3+F(90)/31*x*v^4+10247*y*v^4+F(35)/3*z*v^4+F(21)/52*u*v^4-F(21)/83*v^5,x^5-F(107)/112*x^4*y+2450*x^3*y^2-F(48)/113*x^2*y^3+14807*x*y^4-F(77)/8*x^4*z+2117*x^3*y*z+12991*x^2*y^2*z-F(91)/64*x*y^3*z-8096*y^4*z+F(61)/11*x^3*z^2-F(104)/51*x^2*y*z^2-8642*x*y^2*z^2+F(73)/49*y^3*z^2-F(119)/40*x^2*z^3-F(115)/62*x*y*z^3-9797*y^2*z^3+F(13)/113*x*z^4-F(77)/102*y*z^4-F(95)/9*z^5+F(31)/83*x^4*u-F(22)/107*x^3*y*u-F(119)/36*x^2*y^2*u-F(124)/49*x*y^3*u-F(89)/72*y^4*u-F(75)/106*x^3*z*u-F(20)/117*x^2*y*z*u+14629*x*y^2*z*u+F(29)/124*y^3*z*u+5003*x^2*z^2*u+13144*x*y*z^2*u-12895*y^2*z^2*u-F(111)/61*x*z^3*u-F(41)/121*y*z^3*u-F(47)/97*z^4*u+F(80)/21*x^3*u^2-F(20)/109*x^2*y*u^2+14469*x*y^2*u^2-9368*y^3*u^2-F(124)/39*x^2*z*u^2-F(34)/109*x*y*z*u^2-3421*y^2*z*u^2+F(29)/92*x*z^2*u^2-F(15)/43*y*z^2*u^2-14415*z^3*u^2+1110*x^2*u^3-F(90)/7*x*y*u^3+F(103)/26*y^2*u^3-15785*x*z*u^3-F(97)/75*y*z*u^3+F(82)/43*z^2*u^3-12705*x*u^4-F(97)/31*y*u^4-F(36)/95*z*u^4+F(76)/49*u^5-6746*x^4*v+F(21)/68*x^3*y*v+13494*x^2*y^2*v+14199*x*y^3*v+13803*y^4*v-F(1)/35*x^3*z*v+11003*x^2*y*z*v+F(81)/10*x*y^2*z*v-F(17)/12*y^3*z*v-F(24)/73*x^2*z^2*v-9790*x*y*z^2*v-F(46)/25*y^2*z^2*v+F(65)/102*x*z^3*v+F(12)/61*y*z^3*v+13269*z^4*v+F(35)/67*x^3*u*v-3360*x^2*y*u*v+F(71)/77*x*y^2*u*v+4487*y^3*u*v-15078*x^2*z*u*v+5805*x*y*z*u*v+F(43)/80*y^2*z*u*v-F(69)/14*x*z^2*u*v+2336*y*z^2*u*v-F(100)/123*z^3*u*v-F(25)/126*x^2*u^2*v+12096*x*y*u^2*v+12763*y^2*u^2*v+F(31)/78*x*z*u^2*v+4334*y*z*u^2*v+649*z^2*u^2*v+F(23)/101*x*u^3*v-F(43)/122*y*u^3*v+F(112)/97*z*u^3*v-F(44)/19*u^4*v+14643*x^3*v^2+F(99)/47*x^2*y*v^2-F(122)/21*x*y^2*v^2-14729*y^3*v^2-F(113)/18*x^2*z*v^2+2776*x*y*z*v^2+15921*y^2*z*v^2-14777*x*z^2*v^2-F(73)/57*y*z^2*v^2-10963*z^3*v^2-10455*x^2*u*v^2-6865*x*y*u*v^2-10520*y^2*u*v^2+F(33)/73*x*z*u*v^2-8036*y*z*u*v^2+3833*z^2*u*v^2+5916*x*u^2*v^2-F(23)/12*y*u^2*v^2+5037*z*u^2*v^2-F(58)/73*u^3*v^2-F(4)/81*x^2*v^3-F(96)/53*x*y*v^3-F(21)/100*y^2*v^3-7565*x*z*v^3+F(124)/79*y*z*v^3+13365*z^2*v^3-12391*x*u*v^3+F(58)/25*y*u*v^3+15871*z*u*v^3-6239*u^2*v^3+F(46)/11*x*v^4-F(8)/111*y*v^4-F(58)/93*z*v^4+4018*u*v^4+F(92)/53*v^5,F(86)/77*x^4*z-9447*x^3*y*z-39*x^2*y^2*z-13652*x*y^3*z-F(31)/9*y^4*z-11558*x^3*z^2+9715*x^2*y*z^2+F(103)/96*x*y^2*z^2-9566*y^3*z^2-F(114)/25*x^2*z^3-8301*x*y*z^3+F(7)/111*y^2*z^3+F(31)/47*x*z^4+66*y*z^4+15850*z^5+F(39)/112*x^4*u+F(29)/63*x^3*y*u-F(59)/125*x^2*y^2*u+10738*x*y^3*u+5449*y^4*u+F(39)/61*x^3*z*u+5594*x^2*y*z*u-13188*x*y^2*z*u+6598*y^3*z*u-15652*x^2*z^2*u+8630*x*y*z^2*u-12057*y^2*z^2*u+178*x*z^3*u-F(67)/126*y*z^3*u+F(122)/5*z^4*u-F(101)/68*x^3*u^2-F(35)/92*x^2*y*u^2+F(101)/100*x*y^2*u^2+8834*y^3*u^2+9507*x^2*z*u^2+12390*x*y*z*u^2+4657*y^2*z*u^2-4060*x*z^2*u^2-1929*y*z^2*u^2+11102*z^3*u^2+11618*x^2*u^3-F(88)/79*x*y*u^3-F(117)/4*y^2*u^3+F(35)/29*x*z*u^3-15202*y*z*u^3-5305*z^2*u^3+7128*x*u^4-F(59)/125*y*u^4-F(89)/63*z*u^4+F(43)/70*u^5+F(8)/75*x^4*v-F(25)/29*x^3*y*v-2475*x^2*y^2*v+F(12)/91*x*y^3*v-F(39)/56*y^4*v+2363*x^3*z*v-963*x^2*y*z*v-F(29)/52*x*y^2*z*v-4380*y^3*z*v+F(23)/61*x^2*z^2*v-9922*x*y*z^2*v+F(120)/59*y^2*z^2*v-6592*x*z^3*v-F(32)/19*y*z^3*v+F(83)/107*z^4*v+100*x^3*u*v+4541*x^2*y*u*v-5153*x*y^2*u*v+F(97)/123*y^3*u*v+4772*x^2*z*u*v+12076*x*y*z*u*v+4593*y^2*z*u*v+12152*x*z^2*u*v-F(107)/48*y*z^2*u*v-F(21)/125*z^3*u*v-3699*x^2*u^2*v-F(49)/88*x*y*u^2*v+F(124)/83*y^2*u^2*v-4677*x*z*u^2*v-F(17)/73*y*z*u^2*v-F(93)/19*z^2*u^2*v-7340*x*u^3*v+2582*y*u^3*v-F(71)/66*z*u^3*v-936*u^4*v+F(61)/70*x^3*v^2-3140*x^2*y*v^2-F(41)/50*x*y^2*v^2-F(110)/117*y^3*v^2+F(87)/8*x^2*z*v^2+F(118)/3*x*y*z*v^2-6059*y^2*z*v^2+F(59)/92*x*z^2*v^2+15772*y*z^2*v^2-F(28)/27*z^3*v^2+F(108)/73*x^2*u*v^2-F(38)/61*x*y*u*v^2-1703*y^2*u*v^2-14164*x*z*u*v^2+4592*y*z*u*v^2-F(115)/22*z^2*u*v^2-F(36)/59*x*u^2*v^2-F(1)/15*y*u^2*v^2-F(93)/59*z*u^2*v^2+F(119)/109*u^3*v^2+9781*x^2*v^3-F(62)/125*x*y*v^3+F(54)/125*y^2*v^3-13268*x*z*v^3+F(19)/92*y*z*v^3-9431*z^2*v^3+3975*x*u*v^3+8410*y*u*v^3-F(117)/109*z*u*v^3-F(17)/116*u^2*v^3-13902*x*v^4+F(81)/28*y*v^4-700*z*v^4+F(77)/94*u*v^4-F(25)/6*v^5,-F(67)/99*x^4*z+F(124)/107*x^3*y*z+12237*x^2*y^2*z+8605*x*y^3*z-F(116)/33*y^4*z+2491*x^3*z^2-10294*x^2*y*z^2-F(57)/49*x*y^2*z^2+2972*y^3*z^2-F(82)/87*x^2*z^3+F(47)/102*x*y*z^3-F(101)/87*y^2*z^3-3058*x*z^4-F(57)/47*y*z^4+247*z^5+F(81)/13*x^4*u+F(21)/23*x^3*y*u-F(77)/114*x^2*y^2*u+F(38)/107*x*y^3*u+F(119)/36*y^4*u+F(83)/43*x^3*z*u-4373*x^2*y*z*u+F(87)/23*x*y^2*z*u+F(19)/126*y^3*z*u+F(101)/103*x^2*z^2*u-10067*x*y*z^2*u+F(65)/118*y^2*z^2*u-4690*x*z^3*u-F(20)/69*y*z^3*u+F(55)/61*z^4*u+F(97)/13*x^3*u^2-F(33)/61*x^2*y*u^2+F(89)/33*x*y^2*u^2-8547*y^3*u^2+11117*x^2*z*u^2-8794*x*y*z*u^2+5383*y^2*z*u^2-F(70)/107*x*z^2*u^2+F(79)/29*y*z^2*u^2+F(12)/83*z^3*u^2-9915*x^2*u^3-F(60)/13*x*y*u^3+F(19)/86*y^2*u^3+F(79)/44*x*z*u^3-5074*y*z*u^3-F(30)/103*z^2*u^3+F(39)/82*x*u^4+F(43)/3*y*u^4-F(9)/58*z*u^4-F(97)/42*u^5-F(65)/37*x^4*v+2719*x^3*y*v-2147*x^2*y^2*v-12286*x*y^3*v+F(99)/106*y^4*v+F(95)/101*x^3*z*v+F(123)/53*x^2*y*z*v-3664*x*y^2*z*v+F(99)/16*y^3*z*v-9117*x^2*z^2*v-F(27)/5*x*y*z^2*v-F(31)/102*y^2*z^2*v+9937*x*z^3*v-F(55)/19*y*z^3*v-2599*z^4*v-F(66)/23*x^3*u*v+F(39)/125*x^2*y*u*v-F(86)/31*x*y^2*u*v+F(59)/3*y^3*u*v-9360*x^2*z*u*v-F(7)/3*x*y*z*u*v+F(94)/65*y^2*z*u*v-F(99)/7*x*z^2*u*v-12046*y*z^2*u*v-F(109)/84*z^3*u*v+F(11)/43*x^2*u^2*v+F(55)/32*x*y*u^2*v+F(76)/61*y^2*u^2*v+F(105)/121*x*z*u^2*v+F(109)/74*y*z*u^2*v-F(27)/22*z^2*u^2*v+F(8)/29*x*u^3*v+F(89)/90*y*u^3*v+142*z*u^3*v-F(47)/117*u^4*v+6230*x^3*v^2-F(94)/11*x^2*y*v^2+9472*x*y^2*v^2-9613*y^3*v^2-F(107)/115*x^2*z*v^2-F(17)/38*x*y*z*v^2+F(73)/123*y^2*z*v^2-F(13)/63*x*z^2*v^2+F(82)/61*y*z^2*v^2+9075*z^3*v^2+F(53)/52*x^2*u*v^2+10591*x*y*u*v^2-F(66)/25*y^2*u*v^2-F(14)/11*x*z*u*v^2-F(83)/91*y*z*u*v^2+419*z^2*u*v^2-4087*x*u^2*v^2-F(99)/94*y*u^2*v^2+F(63)/58*z*u^2*v^2+F(99)/17*u^3*v^2+F(10)/7*x^2*v^3-F(9)/10*x*y*v^3-10412*y^2*v^3-F(72)/17*x*z*v^3-1733*y*z*v^3+50*z^2*v^3+11922*x*u*v^3-F(112)/9*y*u*v^3-9801*z*u*v^3+F(81)/124*u^2*v^3+F(1)/68*x*v^4-7434*y*v^4+F(6)/19*z*v^4-F(109)/93*u*v^4-15306*v^5]))
end


@doc raw"""
    k3_d10_pi9_quart_1()

Return a smooth K3 surface in $\mathbb P^4$ with degree `10` and sectional genus `9` which is contained in precisely one quartic.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d10_pi9_quart_1()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^3*y*u^2-6913*x^2*y^2*u^2+F(52)/29*x*y^3*u^2-F(25)/49*y^4*u^2+F(64)/39*x^3*z*u^2-F(78)/115*x^2*y*z*u^2+F(
   103)/101*x*y^2*z*u^2-F(31)/122*y^3*z*u^2-14017*x^2*z^2*u^2+6204*x*y*z^2*u^2-12196*y^2*z^2*u^2+F(79)/14*x*z^3*u^2-F(113)/10*y*z^3*u^2+F(89)/94*z^4*u^2+1447*x^3*u^3-F(91)/32*x^2*y*u^3-F(125)/41*x*y^2*u^3-F(48)/29*y^3*u^3+F(25)/4*x^2*z*u^3+6626*x*y*z*u^3+F(89)/23*y^2*z*u^3-6770*x*z^2*u^3+F(55)/104*y*z^2*u^3-F(104)/89*z^3*u^3+F(119)/110*x^2*u^4-F(111)/52*x*y*u^4-F(53)/73*y^2*u^4-12651*x*z*u^4-F(67)/51*y*z*u^4-4237*z^2*u^4-F(122)/109*x*u^5+F(115)/12*y*u^5-14847*z*u^5+608*u^6-3577*x^3*y*u*v+2260*x^2*y^2*u*v-F(12)/53*x*y^3*u*v+F(29)/100*y^4*u*v-1225*x^3*z*u*v-F(1)/73*x^2*y*z*u*v+10707*x*y^2*z*u*v+F(80)/3*y^3*z*u*v+F(43)/65*x^2*z^2*u*v+F(47)/16*x*y*z^2*u*v-14897*y^2*z^2*u*v-14589*x*z^3*u*v-F(85)/63*y*z^3*u*v+F(22)/75*z^4*u*v-3144*x^3*u^2*v-F(60)/119*x^2*y*u^2*v-3527*x*y^2*u^2*v-F(47)/11*y^3*u^2*v-F(121)/53*x^2*z*u^2*v+F(15)/113*x*y*z*u^2*v-10326*y^2*z*u^2*v+4256*x*z^2*u^2*v+13210*y*z^2*u^2*v+15650*z^3*u^2*v-F(97)/95*x^2*u^3*v+F(71)/30*x*y*u^3*v-F(49)/40*y^2*u^3*v+F(27)/35*x*z*u^3*v+F(112)/75*y*z*u^3*v+11215*z^2*u^3*v+F(81)/68*x*u^4*v-6361*y*u^4*v+9559*z*u^4*v+F(103)/19*u^5*v+F(30)/53*x^3*y*v^2+10782*x^2*y^2*v^2-F(21)/73*x*y^3*v^2-F(30)/91*y^4*v^2+F(81)/13*x^3*z*v^2+13975*x^2*y*z*v^2-9031*x*y^2*z*v^2-F(55)/114*y^3*z*v^2+F(4)/45*x^2*z^2*v^2+F(48)/35*x*y*z^2*v^2+177*y^2*z^2*v^2+F(77)/76*x*z^3*v^2-F(107)/47*y*z^3*v^2-429*z^4*v^2-F(118)/41*x^3*u*v^2+F(91)/114*x^2*y*u*v^2+30*x*y^2*u*v^2+13935*y^3*u*v^2-75*x^2*z*u*v^2-F(39)/107*x*y*z*u*v^2+1087*y^2*z*u*v^2-2749*x*z^2*u*v^2-F(27)/2*y*z^2*u*v^2-11793*z^3*u*v^2-F(77)/120*x^2*u^2*v^2-F(38)/99*x*y*u^2*v^2-F(2)/49*y^2*u^2*v^2+7565*x*z*u^2*v^2+776*y*z*u^2*v^2+F(113)/64*z^2*u^2*v^2-12100*x*u^3*v^2-F(109)/82*y*u^3*v^2-F(93)/34*z*u^3*v^2+F(121)/92*u^4*v^2-2034*x^3*v^3+F(79)/96*x^2*y*v^3-F(23)/104*x*y^2*v^3+10936*y^3*v^3-F(7)/40*x^2*z*v^3-6625*x*y*z*v^3+F(73)/92*y^2*z*v^3-5834*x*z^2*v^3+F(45)/71*y*z^2*v^3+F(23)/82*z^3*v^3-9017*x^2*u*v^3+F(11)/56*x*y*u*v^3+F(86)/33*y^2*u*v^3-F(61)/68*x*z*u*v^3-8695*y*z*u*v^3+F(81)/70*z^2*u*v^3+F(99)/80*x*u^2*v^3-F(125)/84*y*u^2*v^3-F(118)/89*z*u^2*v^3+F(19)/107*u^3*v^3+12417*x^2*v^4-1089*x*y*v^4+1357*y^2*v^4+5522*x*z*v^4-F(76)/49*y*z*v^4+11795*z^2*v^4+F(31)/25*x*u*v^4+F(44)/79*y*u*v^4+2828*z*u*v^4-F(32)/47*u^2*v^4+6936*x*v^5+13679*y*v^5-F(20)/111*z*v^5-F(37)/31*u*v^5-F(29)/120*v^6,-x^4*y+6913*x^3*y^2-F(52)/29*x^2*y^3+F(25)/49*x*y^4-F(64)/39*x^4*z+14845*x^3*y*z-13563*x^2*y^2*z+11488*x*y^3*z+F(15)/8*y^4*z-F(102)/53*x^3*z^2-F(121)/80*x^2*y*z^2-4508*x*y^2*z^2+F(116)/47*y^3*z^2-15103*x^2*z^3+F(83)/16*x*y*z^3+F(58)/99*y^2*z^3+F(109)/117*x*z^4+F(100)/39*y*z^4-F(51)/44*z^5-1447*x^4*u+F(58)/11*x^3*y*u+F(103)/24*x^2*y^2*u-F(91)/37*x*y^3*u+F(61)/75*y^4*u+F(28)/51*x^3*z*u-699*x^2*y*z*u-8352*x*y^2*z*u+F(81)/43*y^3*z*u+9417*x^2*z^2*u+F(20)/41*x*y*z^2*u+F(123)/68*y^2*z^2*u-F(83)/14*x*z^3*u-F(113)/40*y*z^3*u+F(39)/5*z^4*u-14863*x^3*u^2+F(75)/41*x^2*y*u^2-F(106)/67*x*y^2*u^2+F(22)/7*y^3*u^2-F(1)/81*x^2*z*u^2-15140*x*y*z*u^2+8447*y^2*z*u^2+1240*x*z^2*u^2-9936*y*z^2*u^2-F(60)/71*z^3*u^2-F(3)/83*x^2*u^3-11603*x*y*u^3-3601*y^2*u^3+F(3)/41*x*z*u^3+F(44)/15*y*z*u^3-F(125)/42*z^2*u^3-8967*x*u^4+F(49)/88*y*u^4+F(49)/55*z*u^4-7248*u^5-F(11)/111*x^4*v-F(125)/59*x^3*y*v+F(92)/71*x^2*y^2*v+F(89)/106*x*y^3*v+5622*y^4*v-F(3)/35*x^3*z*v-F(37)/75*x^2*y*z*v-4968*x*y^2*z*v-F(105)/31*y^3*z*v+F(20)/61*x^2*z^2*v+5430*x*y*z^2*v-F(13)/12*y^2*z^2*v-10873*x*z^3*v-163*y*z^3*v+F(85)/88*z^4*v-F(34)/27*x^3*u*v-7492*x^2*y*u*v-F(116)/7*x*y^2*u*v-2396*y^3*u*v-7060*x^2*z*u*v-F(72)/97*x*y*z*u*v-6046*y^2*z*u*v-F(30)/101*x*z^2*u*v-F(119)/8*y*z^2*u*v-F(105)/22*z^3*u*v-F(81)/112*x^2*u^2*v-882*x*y*u^2*v+F(95)/89*y^2*u^2*v+3742*x*z*u^2*v-69*y*z*u^2*v+8710*z^2*u^2*v+F(117)/124*x*u^3*v+14557*y*u^3*v+F(101)/114*z*u^3*v+F(24)/85*u^4*v+F(2)/53*x^3*v^2-F(97)/4*x^2*y*v^2-F(23)/17*x*y^2*v^2+3533*y^3*v^2+F(41)/106*x^2*z*v^2-F(95)/6*x*y*z*v^2-F(46)/107*y^2*z*v^2-F(87)/73*x*z^2*v^2+F(112)/17*y*z^2*v^2+11190*z^3*v^2-F(27)/4*x^2*u*v^2-357*x*y*u*v^2+F(5)/121*y^2*u*v^2-F(6)/101*x*z*u*v^2-10595*y*z*u*v^2+F(10)/123*z^2*u*v^2-4018*x*u^2*v^2-F(45)/19*y*u^2*v^2-F(23)/100*z*u^2*v^2+F(47)/79*u^3*v^2+3570*x^2*v^3-F(25)/58*x*y*v^3+F(96)/41*y^2*v^3+F(69)/85*x*z*v^3-F(19)/3*y*z*v^3+F(68)/91*z^2*v^3+F(93)/52*x*u*v^3+F(41)/14*y*u*v^3-14895*z*u*v^3-F(121)/31*u^2*v^3+2213*x*v^4-9547*y*v^4-11358*z*v^4-F(54)/107*u*v^4-10607*v^5,-15485*x^4*y-F(18)/43*x^3*y^2-11616*x^2*y^3+F(25)/42*x*y^4-F(59)/10*y^5+7400*x^4*z+F(40)/107*x^3*y*z-8324*x^2*y^2*z-11685*x*y^3*z+13114*y^4*z+14489*x^3*z^2+F(46)/111*x^2*y*z^2-F(28)/95*x*y^2*z^2+12466*y^3*z^2+5040*x^2*z^3-11677*x*y*z^3+11415*y^2*z^3-F(101)/53*x*z^4+8230*y*z^4+F(82)/33*z^5-13095*x^4*u+F(125)/16*x^3*y*u+12466*x^2*y^2*u-F(37)/98*x*y^3*u+F(44)/83*y^4*u+9962*x^3*z*u-F(8)/69*x^2*y*z*u-F(73)/21*x*y^2*z*u+9298*y^3*z*u-F(75)/106*x^2*z^2*u+F(33)/2*x*y*z^2*u+5956*y^2*z^2*u+9779*x*z^3*u-F(83)/9*y*z^3*u-F(41)/25*z^4*u+F(74)/125*x^3*u^2+F(101)/57*x^2*y*u^2-F(67)/8*x*y^2*u^2-9860*y^3*u^2+F(31)/46*x^2*z*u^2+12130*x*y*z*u^2+F(120)/37*y^2*z*u^2+907*x*z^2*u^2+F(4)/13*y*z^2*u^2+6361*z^3*u^2+10608*x^2*u^3+F(67)/111*x*y*u^3-F(47)/61*y^2*u^3+F(77)/87*x*z*u^3+F(19)/31*y*z*u^3-F(37)/44*z^2*u^3+9769*x*u^4+F(79)/36*y*u^4-F(37)/68*z*u^4-F(100)/93*u^5+F(23)/9*x^4*v-7089*x^3*y*v-F(2)/71*x^2*y^2*v+F(69)/2*x*y^3*v-F(22)/85*y^4*v+F(57)/124*x^3*z*v-1755*x^2*y*z*v+12096*x*y^2*z*v+7829*y^3*z*v-2819*x^2*z^2*v+F(121)/42*x*y*z^2*v+F(47)/20*y^2*z^2*v+F(1)/83*x*z^3*v+12322*y*z^3*v+F(64)/95*z^4*v+F(46)/53*x^3*u*v+F(86)/25*x^2*y*u*v-F(119)/93*x*y^2*u*v+F(25)/59*y^3*u*v+4597*x^2*z*u*v+F(112)/85*x*y*z*u*v-2772*y^2*z*u*v-4164*x*z^2*u*v-F(101)/76*y*z^2*u*v+F(116)/3*z^3*u*v-F(110)/101*x^2*u^2*v-F(51)/25*x*y*u^2*v+F(34)/5*y^2*u^2*v-F(23)/67*x*z*u^2*v-F(103)/53*y*z*u^2*v+F(1)/61*z^2*u^2*v+F(49)/50*x*u^3*v-F(53)/91*y*u^3*v-F(58)/9*z*u^3*v-F(29)/95*u^4*v-F(50)/21*x^3*v^2+14608*x^2*y*v^2-F(75)/22*x*y^2*v^2+F(112)/11*y^3*v^2-F(66)/95*x^2*z*v^2+2395*x*y*z*v^2-11976*y^2*z*v^2+14165*x*z^2*v^2+11095*y*z^2*v^2-7547*z^3*v^2+F(123)/53*x^2*u*v^2+F(73)/59*x*y*u*v^2-F(79)/106*y^2*u*v^2-F(96)/83*x*z*u*v^2-14611*y*z*u*v^2+F(83)/16*z^2*u*v^2-F(121)/123*x*u^2*v^2-F(15)/49*y*u^2*v^2-F(123)/64*z*u^2*v^2+F(24)/7*u^3*v^2+15023*x^2*v^3+F(82)/85*x*y*v^3-5480*y^2*v^3+F(3)/16*x*z*v^3-F(87)/44*y*z*v^3-F(122)/45*z^2*v^3+11826*x*u*v^3+F(107)/112*y*u*v^3+6227*z*u*v^3-F(124)/49*u^2*v^3-F(28)/75*x*v^4-2696*y*v^4+F(121)/116*z*v^4-F(65)/4*u*v^4-15460*v^5,13963*x^4*y-F(87)/38*x^3*y^2-F(7)/121*x^2*y^3-11804*x*y^4-9191*x^4*z+225*x^3*y*z+F(71)/119*x^2*y^2*z+F(89)/42*x*y^3*z+F(12)/125*y^4*z+4883*x^3*z^2+F(59)/125*x^2*y*z^2-14808*x*y^2*z^2+15103*y^3*z^2+F(67)/64*x^2*z^3-F(117)/8*x*y*z^3+5040*y^2*z^3+F(112)/93*x*z^4+8375*y*z^4+574*z^5-2683*x^4*u+F(79)/13*x^3*y*u+F(31)/9*x^2*y^2*u+F(126)/61*x*y^3*u+F(8)/61*y^4*u-F(53)/60*x^3*z*u-F(4)/61*x^2*y*z*u-F(8)/71*x*y^2*z*u-F(68)/61*y^3*z*u+F(1)/121*x^2*z^2*u+F(90)/43*x*y*z^2*u+1324*y^2*z^2*u-F(5)/111*x*z^3*u+F(5)/51*y*z^3*u+F(45)/47*z^4*u-14883*x^3*u^2-2817*x^2*y*u^2-4018*x*y^2*u^2+F(105)/71*y^3*u^2+F(122)/73*x^2*z*u^2-F(68)/99*x*y*z*u^2+12286*y^2*z*u^2-6747*x*z^2*u^2-6447*y*z^2*u^2-F(125)/101*z^3*u^2+12493*x^2*u^3+7253*x*y*u^3-F(76)/109*y^2*u^3-13282*x*z*u^3-6014*y*z*u^3+F(94)/91*z^2*u^3+199*x*u^4-F(59)/44*y*u^4-F(49)/89*z*u^4+F(76)/39*u^5-F(44)/103*x^4*v-3572*x^3*y*v+F(53)/40*x^2*y^2*v-F(9)/74*x*y^3*v+14671*y^4*v-F(18)/113*x^3*z*v-F(92)/45*x^2*y*z*v-F(34)/7*x*y^2*z*v-4449*y^3*z*v+7787*x^2*z^2*v+F(7)/65*x*y*z^2*v+F(49)/122*y^2*z^2*v-127*x*z^3*v+14723*y*z^3*v-F(57)/26*z^4*v-6977*x^3*u*v+F(105)/67*x^2*y*u*v-F(11)/80*x*y^2*u*v-F(31)/101*y^3*u*v+5834*x^2*z*u*v-5127*x*y*z*u*v+F(23)/86*y^2*z*u*v+13493*x*z^2*u*v+F(123)/11*y*z^2*u*v-F(85)/6*z^3*u*v-F(35)/52*x^2*u^2*v+635*x*y*u^2*v-F(72)/7*y^2*u^2*v+F(45)/119*x*z*u^2*v+F(71)/43*y*z*u^2*v+7085*z^2*u^2*v+F(69)/5*x*u^3*v+6820*y*u^3*v-10187*z*u^3*v-F(64)/49*u^4*v-F(61)/73*x^3*v^2+F(78)/23*x^2*y*v^2-F(86)/21*x*y^2*v^2+F(124)/41*y^3*v^2+F(109)/90*x^2*z*v^2+8605*x*y*z*v^2+1408*y^2*z*v^2-12711*x*z^2*v^2+F(97)/67*y*z^2*v^2-9973*z^3*v^2+F(98)/53*x^2*u*v^2-6086*x*y*u*v^2+F(45)/31*y^2*u*v^2+F(38)/23*x*z*u*v^2-3091*y*z*u*v^2-F(17)/111*z^2*u*v^2+15437*x*u^2*v^2-F(62)/107*y*u^2*v^2-F(1)/89*z*u^2*v^2+5303*u^3*v^2-6962*x^2*v^3-F(19)/83*x*y*v^3-7191*y^2*v^3-F(46)/63*x*z*v^3-15002*y*z*v^3-8409*z^2*v^3+F(86)/71*x*u*v^3+15142*y*u*v^3+7729*z*u*v^3-F(73)/21*u^2*v^3+F(41)/126*x*v^4+F(71)/14*y*v^4-8767*z*v^4+F(109)/18*u*v^4+5305*v^5,9831*x^3*y^2+F(19)/74*x^2*y^3+F(87)/115*x*y^4-F(35)/23*y^5+2201*x^3*y*z+20*x^2*y^2*z-7202*x*y^3*z-F(115)/61*y^4*z+7961*x^3*z^2+F(26)/125*x^2*y*z^2+8857*x*y^2*z^2-8837*y^3*z^2+F(35)/26*x^2*z^3-F(43)/78*x*y*z^3+10520*y^2*z^3-F(41)/54*x*z^4+F(1)/78*y*z^4+F(83)/38*z^5-15652*x^3*y*u+9176*x^2*y^2*u+F(79)/122*x*y^3*u-F(89)/81*y^4*u+6297*x^3*z*u-37*x^2*y*z*u+F(41)/111*x*y^2*z*u+F(31)/10*y^3*z*u-11204*x^2*z^2*u+5801*x*y*z^2*u+5659*y^2*z^2*u+F(80)/107*x*z^3*u-F(19)/37*y*z^3*u-12527*z^4*u-F(53)/30*x^3*u^2+F(28)/81*x^2*y*u^2+F(121)/102*x*y^2*u^2-11775*y^3*u^2+F(39)/61*x^2*z*u^2+3023*x*y*z*u^2-F(84)/107*y^2*z*u^2-F(1)/36*x*z^2*u^2+9620*y*z^2*u^2+4807*z^3*u^2+F(112)/13*x^2*u^3+3257*x*y*u^3-12894*y^2*u^3-F(13)/63*x*z*u^3-3181*y*z*u^3-9666*z^2*u^3-F(1)/21*x*u^4-14581*y*u^4-F(7)/83*z*u^4+2403*u^5-2336*x^3*y*v+F(64)/57*x^2*y^2*v-F(34)/83*x*y^3*v-747*y^4*v+F(93)/20*x^3*z*v-3391*x^2*y*z*v+F(71)/49*x*y^2*z*v-13225*y^3*z*v+F(31)/82*x^2*z^2*v-F(82)/79*x*y*z^2*v+F(48)/55*y^2*z^2*v-F(35)/124*x*z^3*v-6659*y*z^3*v+15686*z^4*v-F(17)/95*x^3*u*v+F(3)/79*x^2*y*u*v+2445*x*y^2*u*v+13786*y^3*u*v-584*x^2*z*u*v-27*x*y*z*u*v-14347*y^2*z*u*v-2274*x*z^2*u*v-7964*y*z^2*u*v-15580*z^3*u*v+15326*x^2*u^2*v-4791*x*y*u^2*v+F(122)/99*y^2*u^2*v+9689*x*z*u^2*v+F(75)/14*y*z*u^2*v+13205*z^2*u^2*v+F(43)/28*x*u^3*v-F(67)/23*y*u^3*v-3452*z*u^3*v+F(77)/124*u^4*v+F(68)/81*x^3*v^2+F(73)/61*x^2*y*v^2-F(83)/12*x*y^2*v^2-F(41)/19*y^3*v^2-F(46)/47*x^2*z*v^2-F(41)/57*x*y*z*v^2+F(13)/73*y^2*z*v^2+F(57)/91*x*z^2*v^2-13527*y*z^2*v^2+F(61)/125*z^3*v^2+9882*x^2*u*v^2-8212*x*y*u*v^2-15881*y^2*u*v^2-F(39)/83*x*z*u*v^2-F(19)/3*y*z*u*v^2-F(43)/56*z^2*u*v^2+F(69)/43*x*u^2*v^2+F(29)/55*y*u^2*v^2-7445*z*u^2*v^2-5271*u^3*v^2+F(121)/67*x^2*v^3+F(25)/92*x*y*v^3-F(92)/81*y^2*v^3+F(12)/47*x*z*v^3+F(47)/41*y*z*v^3+13375*z^2*v^3+5006*x*u*v^3+F(17)/12*y*u*v^3+F(19)/109*z*u*v^3-5578*u^2*v^3-F(120)/61*x*v^4+F(90)/77*y*v^4+701*z*v^4+F(27)/79*u*v^4+11411*v^5,-9831*x^4*y-F(11)/15*x^3*y^2+F(60)/37*x^2*y^3+15889*x*y^4-9655*y^5-F(125)/9*x^4*z-F(88)/69*x^3*y*z-F(45)/107*x^2*y^2*z+F(25)/107*x*y^3*z+12843*y^4*z+F(37)/105*x^3*z^2+F(27)/4*x^2*y*z^2-F(29)/118*x*y^2*z^2-F(53)/15*y^3*z^2-7305*x^2*z^3-F(43)/39*x*y*z^3+F(67)/11*y^2*z^3+F(104)/41*x*z^4-F(91)/87*y*z^4+F(28)/53*z^5+12128*x^4*u-F(1)/20*x^3*y*u-4698*x^2*y^2*u+5059*x*y^3*u+F(49)/67*y^4*u+1223*x^3*z*u-F(16)/103*x^2*y*z*u-F(117)/124*x*y^2*z*u-9076*y^3*z*u-F(51)/76*x^2*z^2*u-F(35)/114*x*y*z^2*u+F(65)/109*y^2*z^2*u-F(110)/27*x*z^3*u+F(47)/63*y*z^3*u-1949*z^4*u-15645*x^3*u^2+F(42)/55*x^2*y*u^2-F(84)/41*x*y^2*u^2-13073*y^3*u^2-F(28)/39*x^2*z*u^2+12640*x*y*z*u^2+12992*y^2*z*u^2+F(71)/30*x*z^2*u^2-10612*y*z^2*u^2+8232*z^3*u^2+817*x^2*u^3+F(82)/5*x*y*u^3-F(99)/118*y^2*u^3-10507*x*z*u^3-2802*y*z*u^3-378*z^2*u^3-F(79)/40*x*u^4-8042*y*u^4+F(115)/27*z*u^4+F(8)/115*u^5-12350*x^4*v-F(7)/97*x^3*y*v-F(11)/14*x^2*y^2*v+F(78)/19*x*y^3*v+461*y^4*v-F(36)/103*x^3*z*v-F(10)/17*x^2*y*z*v-4509*x*y^2*z*v+F(123)/26*y^3*z*v-13583*x^2*z^2*v-F(24)/77*x*y*z^2*v+4482*y^2*z^2*v-F(52)/33*x*z^3*v-9165*y*z^3*v-6593*z^4*v+F(73)/93*x^3*u*v-F(5)/17*x^2*y*u*v+F(72)/7*x*y^2*u*v+F(89)/115*y^3*u*v+127*x^2*z*u*v-F(10)/63*x*y*z*u*v+F(56)/41*y^2*z*u*v+F(25)/8*x*z^2*u*v+F(43)/71*y*z^2*u*v+3971*z^3*u*v+F(119)/113*x^2*u^2*v+12991*x*y*u^2*v-F(111)/14*y^2*u^2*v+F(28)/125*x*z*u^2*v+1718*y*z*u^2*v+2147*z^2*u^2*v-F(71)/11*x*u^3*v+F(66)/109*y*u^3*v-15363*z*u^3*v-F(56)/27*u^4*v-F(10)/97*x^3*v^2-F(7)/101*x^2*y*v^2+4150*x*y^2*v^2-F(109)/99*y^3*v^2+F(75)/124*x^2*z*v^2+15310*x*y*z*v^2+F(5)/48*y^2*z*v^2+993*x*z^2*v^2-F(72)/13*y*z^2*v^2-F(117)/41*z^3*v^2+F(18)/65*x^2*u*v^2+F(121)/47*x*y*u*v^2+F(8)/49*y^2*u*v^2+F(65)/99*x*z*u*v^2+F(125)/29*y*z*u*v^2-12619*z^2*u*v^2-F(5)/31*x*u^2*v^2-F(55)/74*y*u^2*v^2+F(23)/82*z*u^2*v^2+F(123)/103*u^3*v^2-F(47)/66*x^2*v^3+F(69)/52*x*y*v^3-F(123)/112*y^2*v^3-F(89)/53*x*z*v^3-12504*y*z*v^3+F(49)/120*z^2*v^3-F(27)/118*x*u*v^3+F(107)/76*y*u*v^3+F(112)/85*z*u*v^3-5435*u^2*v^3-F(59)/74*x*v^4-14*y*v^4+2019*z*v^4-10514*u*v^4+F(1)/54*v^5,F(99)/124*x^3*y*z+F(23)/71*x^2*y^2*z-13770*x*y^3*z+F(21)/26*y^4*z-4841*x^3*z^2+F(41)/90*x^2*y*z^2-F(11)/21*x*y^2*z^2+5572*y^3*z^2+F(50)/79*x^2*z^3+11145*x*y*z^3-F(67)/30*y^2*z^3+F(59)/54*x*z^4-13502*y*z^4+F(79)/21*z^5+2324*x^3*y*u-6330*x^2*y^2*u+10786*x*y^3*u-10326*y^4*u-11946*x^3*z*u+F(37)/28*x^2*y*z*u-12756*x*y^2*z*u+1349*y^3*z*u+F(47)/105*x^2*z^2*u-F(71)/34*x*y*z^2*u-F(52)/99*y^2*z^2*u+5129*x*z^3*u+F(77)/39*y*z^3*u-F(79)/90*z^4*u+3773*x^3*u^2+F(74)/49*x^2*y*u^2+8520*x*y^2*u^2+F(66)/43*y^3*u^2-15427*x^2*z*u^2+F(39)/113*x*y*z*u^2+12246*y^2*z*u^2+15341*x*z^2*u^2-F(11)/5*y*z^2*u^2+10729*z^3*u^2+9494*x^2*u^3-F(45)/64*x*y*u^3+F(62)/41*y^2*u^3+15415*x*z*u^3-F(37)/119*y*z*u^3-F(69)/107*z^2*u^3-F(92)/107*x*u^4+F(73)/113*y*u^4-4229*z*u^4+F(4)/95*u^5+5595*x^3*y*v-2117*x^2*y^2*v-F(12)/83*x*y^3*v+F(107)/123*y^4*v+11002*x^3*z*v+3741*x^2*y*z*v+F(116)/47*x*y^2*z*v+F(76)/91*y^3*z*v+F(111)/67*x^2*z^2*v+7490*x*y*z^2*v-F(9)/29*y^2*z^2*v-F(18)/95*x*z^3*v+2299*y*z^3*v+243*z^4*v-1718*x^3*u*v+F(68)/53*x^2*y*u*v-F(107)/64*x*y^2*u*v+1565*y^3*u*v-F(63)/23*x^2*z*u*v-F(37)/62*x*y*z*u*v-F(79)/104*y^2*z*u*v+F(123)/14*x*z^2*u*v+F(97)/5*y*z^2*u*v+F(109)/20*z^3*u*v+F(6)/83*x^2*u^2*v+F(68)/39*x*y*u^2*v-6182*y^2*u^2*v+F(7)/109*x*z*u^2*v-10595*y*z*u^2*v+F(125)/98*z^2*u^2*v+F(71)/91*x*u^3*v+F(115)/119*y*u^3*v+F(43)/114*z*u^3*v+F(106)/17*u^4*v+11614*x^3*v^2-15415*x^2*y*v^2+F(122)/107*x*y^2*v^2-F(37)/19*y^3*v^2-1060*x^2*z*v^2-190*x*y*z*v^2-5307*y^2*z*v^2+1514*x*z^2*v^2-F(32)/63*y*z^2*v^2+F(103)/105*z^3*v^2+13073*x^2*u*v^2-F(107)/25*x*y*u*v^2-F(46)/41*y^2*u*v^2-F(31)/24*x*z*u*v^2-3972*y*z*u*v^2-1766*z^2*u*v^2-F(107)/80*x*u^2*v^2+F(28)/61*y*u^2*v^2+6630*z*u^2*v^2-14778*u^3*v^2-F(21)/17*x^2*v^3+7830*x*y*v^3-F(121)/86*y^2*v^3-1127*x*z*v^3+1239*y*z*v^3+9094*z^2*v^3-14867*x*u*v^3-12195*y*u*v^3+F(55)/9*z*u*v^3+15428*u^2*v^3+F(83)/17*x*v^4-3433*y*v^4+F(1)/3*z*v^4+F(37)/51*u*v^4-F(19)/77*v^5,F(35)/74*x^4*y-287*x^3*y^2+F(69)/80*x^2*y^3+F(7)/102*x*y^4+F(31)/78*x^4*z-F(83)/38*x^3*y*z-F(35)/99*x^2*y^2*z-F(59)/52*x*y^3*z-13776*y^4*z+F(51)/104*x^3*z^2-F(113)/4*x^2*y*z^2+F(2)/85*x*y^2*z^2+10277*y^3*z^2-6580*x^2*z^3-F(121)/61*x*y*z^3+10588*y^2*z^3+F(86)/105*x*z^4-F(103)/73*y*z^4-3799*z^5+2814*x^4*u+600*x^3*y*u-8625*x^2*y^2*u-F(94)/89*x*y^3*u-F(21)/65*y^4*u-F(17)/18*x^3*z*u+15890*x^2*y*z*u+F(5)/89*x*y^2*z*u+8852*y^3*z*u+F(13)/100*x^2*z^2*u-F(39)/68*x*y*z^2*u-F(117)/44*y^2*z^2*u-F(24)/83*x*z^3*u+7027*y*z^3*u+8212*z^4*u-F(14)/39*x^3*u^2-F(18)/35*x^2*y*u^2+F(32)/39*x*y^2*u^2+9486*y^3*u^2-F(7)/76*x^2*z*u^2+8794*x*y*z*u^2+F(107)/80*y^2*z*u^2-184*x*z^2*u^2+1629*y*z^2*u^2+F(5)/8*z^3*u^2-3217*x^2*u^3+39*x*y*u^3-F(23)/8*y^2*u^3+11675*x*z*u^3-F(69)/79*y*z*u^3-F(71)/38*z^2*u^3-14009*x*u^4-10482*y*u^4+F(28)/73*z*u^4-F(71)/27*u^5-F(26)/113*x^4*v+708*x^3*y*v+11212*x^2*y^2*v-F(72)/83*x*y^3*v-F(64)/59*y^4*v-7017*x^3*z*v+F(33)/80*x^2*y*z*v+F(97)/2*x*y^2*z*v+F(45)/26*y^3*z*v+12846*x^2*z^2*v-6791*x*y*z^2*v+8269*y^2*z^2*v-F(36)/73*x*z^3*v+F(58)/109*y*z^3*v+11878*z^4*v+F(14)/57*x^3*u*v+7609*x^2*y*u*v-F(67)/117*x*y^2*u*v+F(49)/62*y^3*u*v-F(103)/45*x^2*z*u*v-F(125)/121*x*y*z*u*v-F(41)/120*y^2*z*u*v-12671*x*z^2*u*v-12255*y*z^2*u*v-F(13)/51*z^3*u*v-1039*x^2*u^2*v+F(3)/38*x*y*u^2*v+F(93)/100*y^2*u^2*v+F(115)/51*x*z*u^2*v+14668*y*z*u^2*v+F(49)/96*z^2*u^2*v+F(85)/111*x*u^3*v+F(87)/106*y*u^3*v+F(68)/25*z*u^3*v+6176*u^4*v+F(95)/14*x^3*v^2-F(49)/94*x^2*y*v^2-F(89)/105*x*y^2*v^2-F(7)/43*y^3*v^2-6435*x^2*z*v^2+11356*x*y*z*v^2-14780*y^2*z*v^2+F(81)/56*x*z^2*v^2+F(41)/45*y*z^2*v^2-F(78)/53*z^3*v^2-F(76)/45*x^2*u*v^2+14729*x*y*u*v^2+F(119)/57*y^2*u*v^2+F(94)/27*x*z*u*v^2-F(11)/13*y*z*u*v^2+7696*z^2*u*v^2-F(106)/59*x*u^2*v^2+13098*y*u^2*v^2-F(97)/77*z*u^2*v^2+F(124)/5*u^3*v^2-F(47)/52*x^2*v^3+F(76)/45*x*y*v^3-F(51)/23*y^2*v^3+11189*x*z*v^3-15568*y*z*v^3-15774*z^2*v^3-11442*x*u*v^3-F(61)/35*y*u*v^3+6634*z*u*v^3+F(95)/68*u^2*v^3+F(123)/109*x*v^4-F(43)/70*y*v^4-F(103)/51*z*v^4-1674*u*v^4-4895*v^5,-612*x^4*y-F(35)/113*x^3*y^2+F(77)/116*x^2*y^3+F(19)/112*x*y^4+F(55)/54*y^5+F(115)/68*x^4*z+F(13)/35*x^3*y*z-F(29)/22*x^2*y^2*z-F(81)/94*x*y^3*z-F(58)/97*y^4*z-14235*x^3*z^2-F(10)/63*x^2*y*z^2+12711*x*y^2*z^2-F(93)/125*y^3*z^2+12322*x^2*z^3+5796*x*y*z^3+14316*y^2*z^3-13740*x*z^4+F(39)/29*y*z^4+F(124)/55*x^4*u+564*x^3*y*u+F(89)/52*x^2*y^2*u-F(123)/91*x*y^3*u+F(62)/53*y^4*u-F(75)/118*x^3*z*u-F(71)/15*x^2*y*z*u-F(21)/100*x*y^2*z*u+F(101)/73*y^3*z*u+5832*x^2*z^2*u-F(97)/77*x*y*z^2*u-F(82)/97*y^2*z^2*u+4861*x*z^3*u+1288*y*z^3*u+F(3)/86*z^4*u+F(106)/89*x^3*u^2+7936*x^2*y*u^2+F(106)/101*x*y^2*u^2-F(71)/35*y^3*u^2+F(62)/95*x^2*z*u^2+9667*x*y*z*u^2-5572*y^2*z*u^2+F(110)/93*x*z^2*u^2+11336*y*z^2*u^2-6059*z^3*u^2+15438*x^2*u^3+981*x*y*u^3+F(89)/73*y^2*u^3+F(61)/110*x*z*u^3-4911*y*z*u^3-4980*z^2*u^3+F(23)/85*x*u^4+8042*y*u^4+F(73)/123*z*u^4-F(97)/120*u^5-5510*x^4*v-F(116)/11*x^3*y*v-F(61)/16*x^2*y^2*v-F(23)/84*x*y^3*v+2185*y^4*v-F(55)/9*x^3*z*v-F(55)/19*x^2*y*z*v-F(111)/29*x*y^2*z*v-4021*y^3*z*v+F(59)/54*x^2*z^2*v+9876*x*y*z^2*v-6790*y^2*z^2*v+F(37)/65*x*z^3*v-12321*y*z^3*v-15557*z^4*v-13482*x^3*u*v-9487*x^2*y*u*v+F(55)/63*x*y^2*u*v-F(41)/32*y^3*u*v+F(59)/92*x^2*z*u*v-F(113)/32*x*y*z*u*v-F(69)/8*y^2*z*u*v+F(63)/94*x*z^2*u*v-F(95)/22*y*z^2*u*v-F(3)/122*z^3*u*v+14581*x^2*u^2*v-14601*x*y*u^2*v+10725*y^2*u^2*v+F(41)/100*x*z*u^2*v+F(65)/82*y*z*u^2*v+6761*z^2*u^2*v-F(9)/56*x*u^3*v-F(9)/122*y*u^3*v-15674*z*u^3*v-10246*u^4*v-F(61)/93*x^3*v^2+4467*x^2*y*v^2+15620*x*y^2*v^2+F(119)/52*y^3*v^2-F(37)/112*x^2*z*v^2+7070*x*y*z*v^2+F(35)/31*y^2*z*v^2+F(29)/113*x*z^2*v^2-1502*y*z^2*v^2-F(79)/124*z^3*v^2-3925*x^2*u*v^2-48*x*y*u*v^2-10716*y^2*u*v^2-12205*x*z*u*v^2+F(14)/3*y*z*u*v^2-919*z^2*u*v^2-F(52)/47*x*u^2*v^2+7949*y*u^2*v^2-10067*z*u^2*v^2+F(49)/83*u^3*v^2+F(67)/105*x^2*v^3+F(35)/6*x*y*v^3+F(91)/90*y^2*v^3-F(17)/118*x*z*v^3-F(122)/101*y*z*v^3+F(83)/48*z^2*v^3+F(54)/95*x*u*v^3-3220*y*u*v^3+26*z*u*v^3+F(119)/99*u^2*v^3-4347*x*v^4+4733*y*v^4+8923*z*v^4-1735*u*v^4-11247*v^5,-10060*x^4*y+F(4)/99*x^3*y^2-F(9)/122*x^2*y^3-F(115)/38*x*y^4-595*y^5+14163*x^4*z-F(4)/115*x^3*y*z+14384*x^2*y^2*z+F(31)/50*x*y^3*z-F(10)/41*y^4*z-F(81)/97*x^3*z^2-10168*x^2*y*z^2-F(4)/23*x*y^2*z^2-7662*y^3*z^2+10811*x^2*z^3+10984*x*y*z^3+F(95)/14*y^2*z^3+13762*x*z^4-F(18)/23*y*z^4+2185*x^4*u-F(51)/29*x^3*y*u+3922*x^2*y^2*u+7958*x*y^3*u-F(116)/115*y^4*u+9566*x^3*z*u-4768*x^2*y*z*u+F(62)/69*x*y^2*z*u+F(43)/110*y^3*z*u+F(37)/105*x^2*z^2*u-F(88)/75*x*y*z^2*u+F(2)/97*y^2*z^2*u+11036*x*z^3*u+F(51)/49*y*z^3*u+3748*z^4*u-F(113)/107*x^3*u^2+F(103)/30*x^2*y*u^2-7278*x*y^2*u^2+F(76)/91*y^3*u^2+757*x^2*z*u^2+F(78)/53*x*y*z*u^2-F(50)/71*y^2*z*u^2+F(119)/52*x*z^2*u^2+13342*y*z^2*u^2+F(91)/30*z^3*u^2-F(75)/58*x^2*u^3+F(3)/67*x*y*u^3-183*y^2*u^3+7696*x*z*u^3+3884*y*z*u^3-10467*z^2*u^3-7483*x*u^4-F(73)/60*y*u^4+F(65)/58*z*u^4+424*u^5-F(65)/71*x^4*v-6216*x^3*y*v+F(28)/125*x^2*y^2*v+F(24)/7*x*y^3*v+15630*y^4*v-5127*x^3*z*v-11886*x^2*y*z*v+402*x*y^2*z*v-5661*y^3*z*v-8794*x^2*z^2*v-F(33)/70*x*y*z^2*v+F(109)/71*y^2*z^2*v+F(81)/110*x*z^3*v+F(41)/50*y*z^3*v+F(19)/14*z^4*v-67*x^3*u*v+5367*x^2*y*u*v-7761*x*y^2*u*v-F(52)/81*y^3*u*v+F(11)/126*x^2*z*u*v+3753*x*y*z*u*v-2270*y^2*z*u*v+F(107)/84*x*z^2*u*v+5947*y*z^2*u*v+F(45)/64*z^3*u*v-F(59)/124*x^2*u^2*v-15872*x*y*u^2*v+F(124)/41*y^2*u^2*v+F(74)/33*x*z*u^2*v+F(74)/75*y*z*u^2*v-6820*z^2*u^2*v-12990*x*u^3*v+F(62)/63*y*u^3*v+F(71)/107*z*u^3*v+F(1)/28*u^4*v+F(75)/109*x^3*v^2-2956*x^2*y*v^2-F(51)/95*x*y^2*v^2-F(89)/75*y^3*v^2+12751*x^2*z*v^2+F(79)/124*x*y*z*v^2-F(6)/89*y^2*z*v^2+1426*x*z^2*v^2+15307*y*z^2*v^2-F(8)/107*z^3*v^2-F(68)/45*x^2*u*v^2-F(14)/19*x*y*u*v^2-F(19)/18*y^2*u*v^2+F(83)/62*x*z*u*v^2+F(58)/71*y*z*u*v^2-F(46)/65*z^2*u*v^2+1072*x*u^2*v^2+F(43)/59*y*u^2*v^2+1748*z*u^2*v^2+12906*u^3*v^2-7160*x^2*v^3-15325*x*y*v^3-F(100)/57*y^2*v^3+F(87)/115*x*z*v^3+F(29)/44*y*z*v^3-F(88)/13*z^2*v^3-F(13)/20*x*u*v^3+4594*y*u*v^3+9638*z*u*v^3+F(63)/5*u^2*v^3+7217*x*v^4-F(91)/76*y*v^4+F(10)/57*z*v^4-2426*u*v^4-F(103)/14*v^5,F(61)/15*x^4*y-4047*x^3*y^2-2833*x^2*y^3+F(5)/121*x*y^4-F(45)/34*y^5-F(27)/121*x^4*z-6635*x^3*y*z+F(74)/107*x^2*y^2*z-F(62)/73*x*y^3*z-685*y^4*z+919*x^3*z^2-F(86)/103*x^2*y*z^2+F(66)/41*x*y^2*z^2+F(121)/27*y^3*z^2+F(71)/118*x^2*z^3+2708*x*y*z^3+77*y^2*z^3-5294*x*z^4+F(37)/90*y*z^4+F(25)/101*x^4*u-F(55)/104*x^3*y*u+F(99)/10*x^2*y^2*u+3913*x*y^3*u+F(61)/56*y^4*u-6009*x^3*z*u+15559*x^2*y*z*u-F(44)/93*x*y^2*z*u-4710*y^3*z*u+F(121)/61*x^2*z^2*u-F(50)/79*x*y*z^2*u+F(29)/34*y^2*z^2*u+F(33)/53*x*z^3*u+F(29)/73*y*z^3*u+4249*z^4*u+11977*x^3*u^2-F(101)/31*x^2*y*u^2+F(41)/15*x*y^2*u^2+1399*y^3*u^2+8952*x^2*z*u^2-15146*x*y*z*u^2-5838*y^2*z*u^2-F(97)/61*x*z^2*u^2-F(71)/86*y*z^2*u^2-F(5)/104*z^3*u^2-9303*x^2*u^3-13314*x*y*u^3+578*y^2*u^3-7514*x*z*u^3+F(87)/76*y*z*u^3+9727*z^2*u^3+F(47)/126*x*u^4-7472*y*u^4-9045*z*u^4-F(119)/78*u^5+F(88)/125*x^4*v-F(61)/74*x^3*y*v+F(58)/27*x^2*y^2*v-F(40)/33*x*y^3*v+F(105)/104*y^4*v+F(116)/117*x^3*z*v+4604*x^2*y*z*v-F(18)/71*x*y^2*z*v+F(126)/53*y^3*z*v+3619*x^2*z^2*v+F(73)/68*x*y*z^2*v-F(9)/74*y^2*z^2*v-13365*x*z^3*v+10567*y*z^3*v-8747*z^4*v-15678*x^3*u*v+F(115)/33*x^2*y*u*v-5695*x*y^2*u*v-F(43)/103*y^3*u*v-11259*x^2*z*u*v+11534*x*y*z*u*v+15881*y^2*z*u*v+F(97)/78*x*z^2*u*v+F(85)/67*y*z^2*u*v-12080*z^3*u*v+F(50)/91*x^2*u^2*v-6204*x*y*u^2*v-F(29)/24*y^2*u^2*v+5455*x*z*u^2*v+3156*y*z*u^2*v+F(52)/55*z^2*u^2*v-5737*x*u^3*v-F(12)/73*y*u^3*v-F(119)/86*z*u^3*v+F(106)/19*u^4*v+F(118)/111*x^3*v^2-F(101)/120*x^2*y*v^2+F(73)/46*x*y^2*v^2+F(79)/63*y^3*v^2-4017*x^2*z*v^2-13411*x*y*z*v^2-F(115)/94*y^2*z*v^2+F(123)/110*x*z^2*v^2+1259*y*z^2*v^2+F(21)/79*z^3*v^2+F(85)/87*x^2*u*v^2-F(32)/23*x*y*u*v^2-11268*y^2*u*v^2+8667*x*z*u*v^2-F(78)/121*y*z*u*v^2+F(2)/61*z^2*u*v^2-7763*x*u^2*v^2-F(71)/72*y*u^2*v^2-F(71)/28*z*u^2*v^2-8785*u^3*v^2-F(58)/81*x^2*v^3+F(113)/36*x*y*v^3-F(53)/3*y^2*v^3+15567*x*z*v^3+F(11)/81*y*z*v^3+14059*z^2*v^3+F(49)/4*x*u*v^3+F(3)/77*y*u*v^3-12574*z*u*v^3+1446*u^2*v^3+F(79)/10*x*v^4+12633*y*v^4+F(2)/89*z*v^4+7516*u*v^4-12683*v^5,-F(49)/87*x^4*y+F(86)/93*x^3*y^2+977*x^2*y^3-F(23)/14*x*y^4-F(67)/17*y^5-F(83)/8*x^4*z-F(104)/11*x^3*y*z+13345*x^2*y^2*z+F(77)/20*x*y^3*z-F(17)/61*y^4*z-F(42)/61*x^3*z^2-F(91)/93*x^2*y*z^2-7536*x*y^2*z^2-15890*y^3*z^2+F(17)/116*x^2*z^3+3183*x*y*z^3-5458*y^2*z^3+11855*x*z^4-F(13)/41*y*z^4+F(75)/32*x^4*u+F(125)/107*x^3*y*u+10167*x^2*y^2*u+F(11)/96*x*y^3*u+119*y^4*u-F(119)/53*x^3*z*u-F(74)/23*x^2*y*z*u+F(2)/73*x*y^2*z*u-9615*y^3*z*u+4961*x^2*z^2*u+15045*x*y*z^2*u-335*y^2*z^2*u-F(87)/118*x*z^3*u-15065*y*z^3*u+8454*z^4*u-F(18)/119*x^3*u^2-F(112)/5*x^2*y*u^2+F(13)/40*x*y^2*u^2-189*y^3*u^2-F(97)/95*x^2*z*u^2-7556*x*y*z*u^2-10910*y^2*z*u^2-F(84)/89*x*z^2*u^2-F(46)/109*y*z^2*u^2-11915*z^3*u^2-F(8)/37*x^2*u^3+F(101)/55*x*y*u^3-F(112)/85*y^2*u^3-F(35)/109*x*z*u^3+F(3)/41*y*z*u^3+8394*z^2*u^3-F(65)/49*x*u^4-10872*y*u^4-F(90)/113*z*u^4-12466*u^5+F(116)/27*x^4*v-F(87)/112*x^3*y*v+F(47)/6*x^2*y^2*v-F(20)/103*x*y^3*v-F(108)/55*y^4*v+F(97)/90*x^3*z*v+F(9)/13*x^2*y*z*v-F(98)/29*x*y^2*z*v-11860*y^3*z*v+F(103)/80*x^2*z^2*v+12401*x*y*z^2*v-11248*y^2*z^2*v+F(78)/73*x*z^3*v-F(1)/54*y*z^3*v-F(88)/107*z^4*v-F(112)/25*x^3*u*v+F(73)/30*x^2*y*u*v-F(63)/68*x*y^2*u*v+1750*y^3*u*v+9986*x^2*z*u*v+3454*x*y*z*u*v-F(119)/96*y^2*z*u*v-F(97)/101*x*z^2*u*v+F(25)/78*y*z^2*u*v+6599*z^3*u*v+1202*x^2*u^2*v+F(99)/82*x*y*u^2*v+F(51)/47*y^2*u^2*v+F(65)/2*x*z*u^2*v-F(91)/75*y*z*u^2*v+F(114)/119*z^2*u^2*v-10267*x*u^3*v-25*y*u^3*v+F(36)/61*z*u^3*v+F(11)/12*u^4*v+F(50)/113*x^3*v^2+2011*x^2*y*v^2-14141*x*y^2*v^2-6700*y^3*v^2-F(11)/2*x^2*z*v^2+607*x*y*z*v^2+2102*y^2*z*v^2-F(67)/116*x*z^2*v^2-1286*y*z^2*v^2+F(9)/68*z^3*v^2+12356*x^2*u*v^2-F(5)/114*x*y*u*v^2-F(37)/75*y^2*u*v^2+F(81)/28*x*z*u*v^2+10435*y*z*u*v^2+F(38)/33*z^2*u*v^2+F(95)/103*x*u^2*v^2-F(23)/111*y*u^2*v^2-14560*z*u^2*v^2-F(114)/73*u^3*v^2+F(61)/5*x^2*v^3+10762*x*y*v^3-F(106)/123*y^2*v^3-F(81)/73*x*z*v^3+F(13)/55*y*z*v^3-F(93)/79*z^2*v^3+F(109)/60*x*u*v^3+11548*y*u*v^3+15739*z*u*v^3-F(25)/122*u^2*v^3+F(58)/67*x*v^4+15503*y*v^4-F(22)/51*z*v^4+F(103)/37*u*v^4+919*v^5,-F(11)/2*x^4*y-5302*x^3*y^2-F(77)/68*x^2*y^3-F(65)/9*x*y^4-F(87)/98*y^5+2551*x^4*z+F(64)/69*x^3*y*z+F(72)/29*x^2*y^2*z+5060*x*y^3*z-8976*y^4*z+7351*x^3*z^2+1126*x^2*y*z^2+F(44)/101*x*y^2*z^2-6306*y^3*z^2+F(47)/119*x^2*z^3+10902*x*y*z^3+F(11)/56*y^2*z^3-F(115)/81*x*z^4+9684*y*z^4+F(70)/121*z^5+F(14)/59*x^4*u+F(77)/12*x^3*y*u-11066*x^2*y^2*u-10715*x*y^3*u+F(99)/64*y^4*u+14520*x^3*z*u+F(51)/97*x^2*y*z*u+F(4)/27*x*y^2*z*u+F(84)/107*y^3*z*u+3038*x^2*z^2*u-2390*x*y*z^2*u+F(1)/11*y^2*z^2*u+15128*x*z^3*u-1691*y*z^3*u-F(49)/87*z^4*u+F(103)/17*x^3*u^2-F(125)/54*x^2*y*u^2-F(76)/25*x*y^2*u^2+F(55)/78*y^3*u^2+F(95)/62*x^2*z*u^2-10831*x*y*z*u^2+F(4)/73*y^2*z*u^2+F(121)/101*x*z^2*u^2-3286*y*z^2*u^2+F(19)/81*z^3*u^2+8237*x^2*u^3+3396*x*y*u^3+F(2)/107*y^2*u^3-6782*x*z*u^3-F(65)/111*y*z*u^3+F(67)/56*z^2*u^3+F(31)/121*x*u^4+F(86)/83*y*u^4-F(91)/43*z*u^4-F(89)/125*u^5+F(43)/65*x^4*v-F(2)/55*x^3*y*v-F(38)/23*x^2*y^2*v+15306*x*y^3*v+5659*y^4*v-11815*x^3*z*v+F(13)/97*x^2*y*z*v-F(121)/39*x*y^2*z*v-3256*y^3*z*v+2715*x^2*z^2*v-F(15)/112*x*y*z^2*v-F(48)/17*y^2*z^2*v+F(50)/77*x*z^3*v-F(23)/64*y*z^3*v+F(90)/121*z^4*v-F(83)/5*x^3*u*v+F(96)/109*x^2*y*u*v+15309*x*y^2*u*v+F(121)/74*y^3*u*v+F(31)/98*x^2*z*u*v-11302*x*y*z*u*v+F(31)/56*y^2*z*u*v-F(21)/95*x*z^2*u*v-F(25)/12*y*z^2*u*v+9176*z^3*u*v-14264*x^2*u^2*v+350*x*y*u^2*v-3249*y^2*u^2*v+F(1)/85*x*z*u^2*v+F(17)/30*y*z*u^2*v+11951*z^2*u^2*v-13553*x*u^3*v-F(120)/53*y*u^3*v-8833*z*u^3*v-F(65)/101*u^4*v+F(71)/47*x^3*v^2+12746*x^2*y*v^2-F(89)/6*x*y^2*v^2-10422*y^3*v^2+F(109)/45*x^2*z*v^2+F(95)/21*x*y*z*v^2-F(98)/69*y^2*z*v^2+F(87)/73*x*z^2*v^2+5267*y*z^2*v^2-3601*z^3*v^2-F(3)/115*x^2*u*v^2+4405*x*y*u*v^2-F(55)/46*y^2*u*v^2-F(11)/106*x*z*u*v^2-F(13)/126*y*z*u*v^2-F(21)/5*z^2*u*v^2-F(38)/63*x*u^2*v^2+13094*y*u^2*v^2+F(76)/29*z*u^2*v^2+F(113)/92*u^3*v^2+3391*x^2*v^3+F(65)/102*x*y*v^3-F(71)/81*y^2*v^3-2472*x*z*v^3+F(22)/89*y*z*v^3+F(82)/67*z^2*v^3-13607*x*u*v^3+14747*y*u*v^3-737*z*u*v^3+F(44)/113*u^2*v^3-F(7)/110*x*v^4+10734*y*v^4-F(12)/49*z*v^4+1515*u*v^4+F(97)/28*v^5,-11523*x^3*y^2-F(81)/31*x^2*y^3+15362*x*y^4+F(41)/115*y^5-10939*x^3*y*z-F(93)/82*x^2*y^2*z+F(35)/76*x*y^3*z-F(53)/62*y^4*z+F(39)/101*x^3*z^2+F(50)/7*x^2*y*z^2-2828*x*y^2*z^2+F(73)/99*y^3*z^2-F(113)/45*x^2*z^3-1869*x*y*z^3+4327*y^2*z^3-F(9)/106*x*z^4-13293*y*z^4-F(109)/87*z^5+F(19)/47*x^3*y*u+F(22)/13*x^2*y^2*u-11236*x*y^3*u+13188*y^4*u+F(43)/34*x^3*z*u+F(47)/53*x^2*y*z*u+9559*x*y^2*z*u+F(118)/63*y^3*z*u+F(3)/40*x^2*z^2*u+F(74)/69*x*y*z^2*u-13854*y^2*z^2*u-F(61)/33*x*z^3*u-F(107)/94*y*z^3*u-F(76)/53*z^4*u-F(30)/41*x^3*u^2-F(25)/93*x^2*y*u^2-F(41)/65*x*y^2*u^2+F(118)/63*y^3*u^2+F(20)/11*x^2*z*u^2+F(1)/39*x*y*z*u^2+F(32)/121*y^2*z*u^2-3005*x*z^2*u^2-F(95)/109*y*z^2*u^2+F(73)/84*z^3*u^2-F(111)/97*x^2*u^3-6118*x*y*u^3-F(4)/73*y^2*u^3-11079*x*z*u^3+F(110)/87*y*z*u^3-137*z^2*u^3-F(118)/55*x*u^4+1943*y*u^4-F(51)/113*z*u^4+F(99)/19*u^5+13128*x^3*y*v-7767*x^2*y^2*v-9716*x*y^3*v+F(72)/49*y^4*v+F(101)/26*x^3*z*v+7154*x^2*y*z*v-F(110)/61*x*y^2*z*v-F(118)/39*y^3*z*v+14580*x^2*z^2*v-7452*x*y*z^2*v+15910*y^2*z^2*v-F(71)/112*x*z^3*v+6841*y*z^3*v+10508*z^4*v+9807*x^3*u*v-F(33)/113*x^2*y*u*v+5543*x*y^2*u*v+7452*y^3*u*v-F(99)/19*x^2*z*u*v+F(25)/121*x*y*z*u*v-8518*y^2*z*u*v-F(100)/9*x*z^2*u*v-13985*y*z^2*u*v+15523*z^3*u*v+F(87)/101*x^2*u^2*v-13108*x*y*u^2*v+2250*y^2*u^2*v-F(25)/101*x*z*u^2*v-8222*y*z*u^2*v+F(37)/100*z^2*u^2*v-12768*x*u^3*v+F(40)/19*y*u^3*v-1177*z*u^3*v+F(27)/16*u^4*v-F(111)/94*x^3*v^2-5603*x^2*y*v^2+F(12)/109*x*y^2*v^2-6748*y^3*v^2-F(98)/117*x^2*z*v^2-F(57)/17*x*y*z*v^2+F(118)/31*y^2*z*v^2-F(46)/123*x*z^2*v^2+8056*y*z^2*v^2-9460*z^3*v^2+F(82)/77*x^2*u*v^2-7855*x*y*u*v^2-717*y^2*u*v^2+F(5)/94*x*z*u*v^2+F(87)/4*y*z*u*v^2+F(126)/113*z^2*u*v^2-15201*x*u^2*v^2+12746*y*u^2*v^2-9516*z*u^2*v^2+F(103)/61*u^3*v^2+7743*x^2*v^3-F(118)/53*x*y*v^3-F(29)/71*y^2*v^3-F(57)/43*x*z*v^3-F(6)/29*y*z*v^3-6429*z^2*v^3-12368*x*u*v^3-1724*y*u*v^3-6504*z*u*v^3+F(40)/91*u^2*v^3-F(39)/74*x*v^4+7506*y*v^4+14941*z*v^4-13641*u*v^4-7034*v^5,8430*x^3*y^2-15879*x^2*y^3+6227*x*y^4+13160*y^5-9500*x^3*y*z-11612*x^2*y^2*z-F(35)/46*x*y^3*z-F(39)/34*y^4*z-12706*x^2*y*z^2+10991*x*y^2*z^2+11267*y^3*z^2-F(77)/92*x*y*z^3-188*y^2*z^3+F(81)/119*y*z^4-F(27)/118*x^3*y*u-15918*x^2*y^2*u+F(43)/108*x*y^3*u+F(41)/112*y^4*u-14357*x^3*z*u-F(111)/61*x^2*y*z*u+F(56)/65*x*y^2*z*u-F(3)/13*y^3*z*u-14347*x^2*z^2*u+13526*x*y*z^2*u+6909*y^2*z^2*u+F(8)/91*x*z^3*u-F(78)/79*y*z^3*u+12194*z^4*u-F(9)/83*x^3*u^2+3357*x^2*y*u^2+F(97)/103*x*y^2*u^2-F(45)/56*y^3*u^2+11392*x^2*z*u^2-F(75)/98*x*y*z*u^2-475*y^2*z*u^2-2513*x*z^2*u^2+F(53)/113*y*z^2*u^2+F(23)/73*z^3*u^2+F(97)/82*x^2*u^3-F(114)/89*x*y*u^3+F(71)/89*y^2*u^3+F(36)/103*x*z*u^3-15711*y*z*u^3-F(73)/35*z^2*u^3+7334*x*u^4-F(65)/59*y*u^4-F(23)/70*z*u^4-F(100)/101*u^5-F(5)/106*x^3*y*v+F(32)/17*x^2*y^2*v+F(104)/87*x*y^3*v-F(117)/67*y^4*v+6627*x^3*z*v-F(92)/37*x^2*y*z*v+F(63)/80*x*y^2*z*v-F(94)/81*y^3*z*v+15533*x^2*z^2*v-F(109)/38*x*y*z^2*v-F(5)/124*y^2*z^2*v-F(32)/17*x*z^3*v+F(6)/119*y*z^3*v-F(82)/79*z^4*v+F(58)/113*x^3*u*v-4418*x^2*y*u*v+9949*x*y^2*u*v-F(103)/48*y^3*u*v+F(4)/23*x^2*z*u*v+F(65)/82*x*y*z*u*v-6634*y^2*z*u*v-F(6)/23*x*z^2*u*v+12922*y*z^2*u*v+10158*z^3*u*v-F(51)/118*x^2*u^2*v+F(24)/29*x*y*u^2*v-F(47)/25*y^2*u^2*v+F(57)/4*x*z*u^2*v+8786*y*z*u^2*v-F(29)/109*z^2*u^2*v+F(23)/11*x*u^3*v+F(2)/55*y*u^3*v+F(93)/22*z*u^3*v-F(119)/74*u^4*v+F(81)/116*x^3*v^2+10872*x^2*y*v^2-F(116)/21*x*y^2*v^2+10202*y^3*v^2-14386*x^2*z*v^2+8335*x*y*z*v^2+6439*y^2*z*v^2+F(107)/102*x*z^2*v^2-5103*y*z^2*v^2-F(60)/97*z^3*v^2-F(25)/123*x^2*u*v^2+F(91)/9*x*y*u*v^2-F(29)/20*y^2*u*v^2+3274*x*z*u*v^2-F(71)/24*y*z*u*v^2+F(67)/10*z^2*u*v^2-F(93)/25*x*u^2*v^2+F(35)/67*y*u^2*v^2-F(15)/101*z*u^2*v^2+691*u^3*v^2-F(89)/113*x^2*v^3+F(18)/19*x*y*v^3-15069*y^2*v^3-F(99)/26*x*z*v^3+F(17)/38*y*z*v^3+F(95)/99*z^2*v^3+F(59)/77*x*u*v^3+847*y*u*v^3-F(37)/64*z*u*v^3-F(20)/91*u^2*v^3-F(39)/116*x*v^4-F(87)/110*y*v^4+F(49)/30*z*v^4+9105*u*v^4-342*v^5,F(123)/101*x^4*y+F(5)/93*x^3*y^2+F(34)/39*x^2*y^3+F(74)/23*x*y^4-12699*y^5+5062*x^4*z-F(122)/17*x^3*y*z-12160*x^2*y^2*z+70*x*y^3*z+F(41)/3*y^4*z-F(115)/36*x^3*z^2-F(86)/65*x^2*y*z^2-13344*x*y^2*z^2-8191*y^3*z^2+9923*x^2*z^3+F(13)/36*x*y*z^3-F(38)/93*y^2*z^3-7501*x*z^4-F(116)/101*y*z^4-496*x^4*u+6983*x^3*y*u+F(84)/65*x^2*y^2*u-F(61)/119*x*y^3*u-F(59)/71*y^4*u+15047*x^3*z*u-13282*x^2*y*z*u-13*x*y^2*z*u+6538*y^3*z*u-F(36)/59*x^2*z^2*u-871*x*y*z^2*u+F(61)/96*y^2*z^2*u+8521*x*z^3*u+F(1)/29*y*z^3*u+F(14)/107*z^4*u-5125*x^3*u^2+F(26)/15*x^2*y*u^2-9504*x*y^2*u^2+F(109)/58*y^3*u^2+F(61)/36*x^2*z*u^2-15023*x*y*z*u^2+6705*y^2*z*u^2-F(91)/87*x*z^2*u^2-F(67)/47*y*z^2*u^2+F(19)/36*z^3*u^2-5129*x^2*u^3-12219*x*y*u^3+F(103)/72*y^2*u^3+F(104)/21*x*z*u^3-F(7)/76*y*z*u^3+F(51)/53*z^2*u^3+F(97)/89*x*u^4+F(29)/83*y*u^4-F(51)/8*z*u^4-F(23)/108*u^5-F(59)/103*x^4*v+F(7)/41*x^3*y*v-13540*x^2*y^2*v-F(102)/109*x*y^3*v-F(44)/57*y^4*v+F(19)/70*x^3*z*v-F(101)/63*x^2*y*z*v+F(75)/71*x*y^2*z*v+F(40)/13*y^3*z*v-F(36)/97*x^2*z^2*v-11408*x*y*z^2*v-F(22)/19*y^2*z^2*v-F(115)/14*x*z^3*v-F(66)/53*y*z^3*v-F(7)/65*z^4*v-2450*x^3*u*v+11864*x^2*y*u*v+12025*x*y^2*u*v-F(110)/27*y^3*u*v-F(85)/22*x^2*z*u*v-F(109)/73*x*y*z*u*v-F(47)/97*y^2*z*u*v-F(68)/87*x*z^2*u*v+10147*y*z^2*u*v+F(90)/77*z^3*u*v+788*x^2*u^2*v-F(39)/40*x*y*u^2*v-F(62)/23*y^2*u^2*v-F(39)/107*x*z*u^2*v-F(27)/38*y*z*u^2*v+13578*z^2*u^2*v+11526*x*u^3*v+11914*y*u^3*v+F(88)/87*z*u^3*v-13396*u^4*v+F(19)/96*x^3*v^2+F(61)/73*x^2*y*v^2+14353*x*y^2*v^2-2346*y^3*v^2-F(91)/47*x^2*z*v^2-13448*x*y*z*v^2-F(115)/2*y^2*z*v^2+14735*x*z^2*v^2-7492*y*z^2*v^2+F(77)/115*z^3*v^2-F(3)/110*x^2*u*v^2-6253*x*y*u*v^2+6699*y^2*u*v^2-F(2)/5*x*z*u*v^2-F(89)/12*y*z*u*v^2+F(33)/25*z^2*u*v^2-10291*x*u^2*v^2+F(121)/60*y*u^2*v^2+F(26)/61*z*u^2*v^2+F(87)/16*u^3*v^2-1326*x^2*v^3-F(61)/91*x*y*v^3-F(83)/92*y^2*v^3+F(31)/90*x*z*v^3-F(121)/115*y*z*v^3+F(56)/9*z^2*v^3+F(106)/31*x*u*v^3+14506*y*u*v^3-7471*z*u*v^3-1097*u^2*v^3+8369*x*v^4+F(67)/87*y*v^4+F(43)/90*z*v^4+13981*u*v^4+F(81)/43*v^5,-F(11)/27*x^4+F(64)/7*x^3*y+14235*x^2*y^2-2447*x*y^3+2677*y^4+F(16)/37*x^3*z+F(69)/34*x^2*y*z+F(5)/113*x*y^2*z-5557*y^3*z-2445*x^2*z^2+3619*x*y*z^2-F(87)/52*y^2*z^2+F(112)/25*x*z^3+F(71)/35*y*z^3+F(19)/120*z^4-2495*x^3*u-F(122)/75*x^2*y*u-1095*x*y^2*u+F(7)/30*y^3*u-F(107)/123*x^2*z*u-12176*x*y*z*u-13553*y^2*z*u+F(10)/61*x*z^2*u+13683*y*z^2*u-510*z^3*u-F(61)/52*x^2*u^2-F(14)/125*x*y*u^2+F(55)/86*y^2*u^2-F(54)/25*x*z*u^2-F(107)/60*y*z*u^2-13736*z^2*u^2+2270*x*u^3+F(5)/58*y*u^3+F(62)/55*z*u^3+231*u^4-F(70)/107*x^3*v+F(122)/43*x^2*y*v+F(19)/87*x*y^2*v-F(33)/56*y^3*v-F(46)/11*x^2*z*v-F(105)/29*x*y*z*v-F(65)/61*y^2*z*v-9920*x*z^2*v-1465*y*z^2*v-F(111)/38*z^3*v-F(35)/89*x^2*u*v+F(86)/7*x*y*u*v+202*y^2*u*v-F(109)/38*x*z*u*v+2810*y*z*u*v+F(68)/27*z^2*u*v-F(47)/78*x*u^2*v-F(61)/44*y*u^2*v+F(71)/55*z*u^2*v+F(28)/39*u^3*v-F(119)/20*x^2*v^2+F(5)/71*x*y*v^2-8709*y^2*v^2-2518*x*z*v^2-10267*y*z*v^2-9465*z^2*v^2+F(11)/14*x*u*v^2+5227*y*u*v^2+2514*z*u*v^2+8669*u^2*v^2-F(94)/39*x*v^3+F(56)/107*y*v^3-7483*z*v^3-894*u*v^3+F(15)/71*v^4]))
end


@doc raw"""
    k3_d10_pi9_quart_2()

Return a smooth K3 surface in $\mathbb P^4$ with degree `10` and sectional genus `9` which is contained in a pencil of quartics.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d10_pi9_quart_2()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^3*y*z^2+F(31)/15*x^2*y^2*z^2-7231*x*y^3*z^2+F(99)/37*y^4*z^2+F(28)/95*x^3*z^3+F(97)/32*x^2*y*z^3+13247*x*y^2*z^3+12717*y^3*z^3-F(113)/31*x^2*z^4-F(61)/30*x*y*z^4-6844*y^2*z^4+F(104)/3*x*z^5-13849*y*z^5+F(43)/39*z^6+13061*x^3*y*z*u-8463*x^2*y^2*z*u+F(94)/69*x*y^3*z*u-F(8)/61*y^4*z*u-13297*x^3*z^2*u+7217*x^2*y*z^2*u-7830*x*y^2*z^2*u-F(75)/14*y^3*z^2*u+2839*x^2*z^3*u-14657*x*y*z^3*u-F(52)/7*y^2*z^3*u-F(6)/89*x*z^4*u-6169*y*z^4*u+F(44)/7*z^5*u-F(98)/33*x^3*y*u^2+F(41)/30*x^2*y^2*u^2-F(65)/98*x*y^3*u^2+F(122)/13*y^4*u^2+9906*x^3*z*u^2-11587*x^2*y*z*u^2+F(17)/53*x*y^2*z*u^2+6504*y^3*z*u^2+F(49)/106*x^2*z^2*u^2+11480*x*y*z^2*u^2+F(97)/71*y^2*z^2*u^2+12560*x*z^3*u^2-F(114)/83*y*z^3*u^2-13761*z^4*u^2-F(67)/112*x^3*u^3-F(18)/49*x^2*y*u^3+F(21)/67*x*y^2*u^3-F(44)/43*y^3*u^3+F(123)/116*x^2*z*u^3+4459*x*y*z*u^3-13841*y^2*z*u^3-805*x*z^2*u^3-1382*y*z^2*u^3-5293*z^3*u^3+133*x^2*u^4-F(122)/79*x*y*u^4+9724*y^2*u^4+F(61)/24*x*z*u^4+F(113)/119*y*z*u^4-F(19)/108*z^2*u^4+15893*x*u^5+F(57)/22*y*u^5+4600*z*u^5-618*u^6-F(27)/53*x^3*y*z*v+F(44)/103*x^2*y^2*z*v-142*x*y^3*z*v+F(19)/84*y^4*z*v+F(105)/8*x^3*z^2*v+10532*x^2*y*z^2*v-F(75)/74*x*y^2*z^2*v-F(70)/19*y^3*z^2*v+F(31)/80*x^2*z^3*v-481*x*y*z^3*v+F(47)/30*y^2*z^3*v+14318*x*z^4*v+F(51)/28*y*z^4*v-F(15)/113*z^5*v-F(46)/17*x^3*y*u*v-F(99)/100*x^2*y^2*u*v-F(106)/5*x*y^3*u*v+14384*y^4*u*v+F(7)/100*x^3*z*u*v-F(15)/64*x^2*y*z*u*v-6976*x*y^2*z*u*v+12051*y^3*z*u*v-F(67)/42*x^2*z^2*u*v-2627*x*y*z^2*u*v-F(49)/104*y^2*z^2*u*v+F(77)/16*x*z^3*u*v+15766*y*z^3*u*v+F(85)/117*z^4*u*v-F(107)/101*x^3*u^2*v-6699*x^2*y*u^2*v+2443*x*y^2*u^2*v-F(27)/28*y^3*u^2*v+11945*x^2*z*u^2*v-14467*x*y*z*u^2*v-4873*y^2*z*u^2*v-F(63)/124*x*z^2*u^2*v-8270*y*z^2*u^2*v+11900*z^3*u^2*v+F(47)/14*x^2*u^3*v+F(53)/8*x*y*u^3*v-F(10)/51*y^2*u^3*v-F(87)/119*x*z*u^3*v+F(114)/73*y*z*u^3*v+F(86)/57*z^2*u^3*v+F(52)/63*x*u^4*v-11587*y*u^4*v+F(1)/18*z*u^4*v-F(121)/109*u^5*v+F(116)/11*x^3*y*v^2+F(19)/108*x^2*y^2*v^2-F(31)/3*x*y^3*v^2-F(43)/9*y^4*v^2-F(81)/100*x^3*z*v^2-7728*x^2*y*z*v^2-1037*x*y^2*z*v^2+F(24)/101*y^3*z*v^2-F(61)/103*x^2*z^2*v^2-F(8)/51*x*y*z^2*v^2+F(117)/109*y^2*z^2*v^2+F(98)/23*x*z^3*v^2+1646*y*z^3*v^2-3356*z^4*v^2+F(105)/59*x^3*u*v^2+F(117)/31*x^2*y*u*v^2+519*x*y^2*u*v^2+12633*y^3*u*v^2+F(25)/6*x^2*z*u*v^2-963*x*y*z*u*v^2-F(49)/23*y^2*z*u*v^2-F(116)/25*x*z^2*u*v^2+14146*y*z^2*u*v^2+11480*z^3*u*v^2-F(95)/8*x^2*u^2*v^2-10928*x*y*u^2*v^2-F(51)/23*y^2*u^2*v^2-12770*x*z*u^2*v^2-F(92)/91*y*z*u^2*v^2+3872*z^2*u^2*v^2+3183*x*u^3*v^2+6871*y*u^3*v^2+F(90)/37*z*u^3*v^2+10019*u^4*v^2-F(69)/88*x^3*v^3-1398*x^2*y*v^3-F(97)/72*x*y^2*v^3-F(46)/97*y^3*v^3+F(107)/14*x^2*z*v^3-F(20)/89*x*y*z*v^3-11367*y^2*z*v^3+F(120)/29*x*z^2*v^3-F(86)/81*y*z^2*v^3+F(107)/69*z^3*v^3-F(39)/17*x^2*u*v^3+F(83)/11*x*y*u*v^3+169*y^2*u*v^3-F(11)/71*x*z*u*v^3-F(22)/17*y*z*u*v^3-14862*z^2*u*v^3-13009*x*u^2*v^3-F(101)/12*y*u^2*v^3+10617*z*u^2*v^3+2567*u^3*v^3-F(23)/85*x^2*v^4+F(27)/50*x*y*v^4+F(113)/51*y^2*v^4+F(97)/16*x*z*v^4+4438*y*z*v^4-11857*z^2*v^4+14580*x*u*v^4-6426*y*u*v^4+9421*z*u*v^4-10585*u^2*v^4+14670*x*v^5+1807*y*v^5+10298*z*v^5-F(116)/53*u*v^5+7869*v^6,x^3*y*z*u+F(31)/15*x^2*y^2*z*u-7231*x*y^3*z*u+F(99)/37*y^4*z*u+F(28)/95*x^3*z^2*u+F(97)/32*x^2*y*z^2*u+13247*x*y^2*z^2*u+12717*y^3*z^2*u-F(113)/31*x^2*z^3*u-F(61)/30*x*y*z^3*u-6844*y^2*z^3*u+F(104)/3*x*z^4*u-13849*y*z^4*u+F(43)/39*z^5*u-F(124)/43*x^3*y*u^2-F(90)/13*x^2*y^2*u^2-13244*x*y^3*u^2-F(78)/73*y^4*u^2+F(118)/43*x^3*z*u^2+F(37)/67*x^2*y*z*u^2-10426*x*y^2*z*u^2+2412*y^3*z*u^2-F(32)/113*x^2*z^2*u^2+F(35)/104*x*y*z^2*u^2+3952*y^2*z^2*u^2+9028*x*z^3*u^2-1990*y*z^3*u^2-F(59)/109*z^4*u^2+15499*x^3*u^3+F(116)/23*x^2*y*u^3+F(95)/58*x*y^2*u^3+F(8)/47*y^3*u^3+F(59)/109*x^2*z*u^3-29*x*y*z*u^3+12412*y^2*z*u^3+F(20)/81*x*z^2*u^3+2200*y*z^2*u^3-13809*z^3*u^3+3889*x^2*u^4-8136*x*y*u^4+8922*y^2*u^4-F(4)/121*x*z*u^4+F(82)/113*y*z*u^4-F(65)/23*z^2*u^4+F(101)/53*x*u^5+F(103)/113*y*u^5-F(99)/118*z*u^5-9524*u^6-2749*x^3*y*z*v-7814*x^2*y^2*z*v+F(73)/113*x*y^3*z*v+9937*y^4*z*v-F(59)/62*x^3*z^2*v-F(23)/12*x^2*y*z^2*v-10245*x*y^2*z^2*v+7130*y^3*z^2*v-4427*x^2*z^3*v+6656*x*y*z^3*v+3448*y^2*z^3*v-F(46)/79*x*z^4*v+1611*y*z^4*v+8453*z^5*v+12013*x^3*y*u*v+F(49)/17*x^2*y^2*u*v-F(4)/115*x*y^3*u*v-F(121)/91*y^4*u*v-F(63)/29*x^3*z*u*v+F(64)/7*x^2*y*z*u*v-8785*x*y^2*z*u*v-F(87)/14*y^3*z*u*v+F(36)/121*x^2*z^2*u*v+9525*x*y*z^2*u*v+4215*y^2*z^2*u*v-F(17)/13*x*z^3*u*v-F(117)/125*y*z^3*u*v+F(101)/122*z^4*u*v+F(42)/37*x^3*u^2*v-8747*x^2*y*u^2*v-F(105)/79*x*y^2*u^2*v+10799*y^3*u^2*v-F(58)/49*x^2*z*u^2*v-F(8)/75*x*y*z*u^2*v-F(67)/49*y^2*z*u^2*v-F(38)/11*x*z^2*u^2*v+F(53)/27*y*z^2*u^2*v+F(113)/52*z^3*u^2*v+F(18)/59*x^2*u^3*v+F(71)/106*x*y*u^3*v+F(47)/2*y^2*u^3*v-4594*x*z*u^3*v+F(95)/4*y*z*u^3*v-F(121)/46*z^2*u^3*v-F(55)/62*x*u^4*v-F(101)/72*y*u^4*v+F(40)/53*z*u^4*v+15227*u^5*v-15553*x^3*y*v^2+F(29)/94*x^2*y^2*v^2-4076*x*y^3*v^2-7133*y^4*v^2+F(27)/125*x^3*z*v^2+F(33)/29*x^2*y*z*v^2-F(63)/95*x*y^2*z*v^2+9166*y^3*z*v^2-480*x^2*z^2*v^2+9941*x*y*z^2*v^2+F(107)/46*y^2*z^2*v^2+13018*x*z^3*v^2+F(53)/98*y*z^3*v^2+F(92)/35*z^4*v^2+F(17)/30*x^3*u*v^2+F(77)/95*x^2*y*u*v^2+F(11)/67*x*y^2*u*v^2+8262*y^3*u*v^2+F(65)/11*x^2*z*u*v^2+2567*x*y*z*u*v^2-F(33)/94*y^2*z*u*v^2+F(85)/92*x*z^2*u*v^2+F(103)/25*y*z^2*u*v^2-F(27)/100*z^3*u*v^2+13210*x^2*u^2*v^2-F(109)/90*x*y*u^2*v^2+141*y^2*u^2*v^2-F(124)/51*x*z*u^2*v^2-F(3)/109*y*z*u^2*v^2-4910*z^2*u^2*v^2+205*x*u^3*v^2+14357*y*u^3*v^2+F(85)/57*z*u^3*v^2-F(109)/28*u^4*v^2-F(68)/39*x^3*v^3+10545*x^2*y*v^3-2176*x*y^2*v^3-8743*y^3*v^3+15111*x^2*z*v^3+F(25)/119*x*y*z*v^3+F(8)/103*y^2*z*v^3-6046*x*z^2*v^3+8658*y*z^2*v^3+F(106)/5*z^3*v^3-F(31)/126*x^2*u*v^3-7762*x*y*u*v^3+2315*y^2*u*v^3+F(124)/67*x*z*u*v^3-F(77)/104*y*z*u*v^3+F(95)/71*z^2*u*v^3+F(69)/119*x*u^2*v^3+13069*y*u^2*v^3-8620*z*u^2*v^3+F(105)/41*u^3*v^3-15772*x^2*v^4-11212*x*y*v^4-F(61)/36*y^2*v^4+F(38)/125*x*z*v^4-15860*y*z*v^4+F(8)/63*z^2*v^4+7519*x*u*v^4-F(94)/41*y*u*v^4+F(45)/32*z*u*v^4+9417*u^2*v^4-F(71)/35*x*v^5-6287*y*v^5+6481*z*v^5+F(106)/99*u*v^5+F(3)/41*v^6,-8036*x^3*y*u^2+7966*x^2*y^2*u^2-151*x*y^3*u^2-F(14)/111*y^4*u^2-F(111)/76*x^3*z*u^2-F(102)/11*x^2*y*z*u^2+7956*x*y^2*z*u^2-7397*y^3*z*u^2-F(113)/16*x^2*z^2*u^2-8049*x*y*z^2*u^2+7230*y^2*z^2*u^2+3978*x*z^3*u^2-F(36)/113*y*z^3*u^2-8147*z^4*u^2-F(107)/83*x^3*u^3+F(78)/97*x^2*y*u^3+12700*x*y^2*u^3+F(11)/72*y^3*u^3+F(88)/31*x^2*z*u^3-F(63)/40*x*y*z*u^3+F(101)/35*y^2*z*u^3-220*x*z^2*u^3+F(3)/103*y*z^2*u^3-F(49)/45*z^3*u^3-F(21)/113*x^2*u^4+F(104)/123*x*y*u^4+F(98)/47*y^2*u^4-F(56)/61*x*z*u^4-F(87)/50*y*z*u^4+5913*z^2*u^4-F(120)/17*x*u^5+F(64)/11*y*u^5-F(109)/80*z*u^5+10371*u^6-F(118)/25*x^3*y*z*v+F(58)/99*x^2*y^2*z*v-F(5)/64*x*y^3*z*v+F(7)/46*y^4*z*v-F(49)/103*x^3*z^2*v-F(77)/106*x^2*y*z^2*v-F(44)/7*x*y^2*z^2*v-7559*y^3*z^2*v-F(17)/35*x^2*z^3*v+948*x*y*z^3*v-15043*y^2*z^3*v-3576*x*z^4*v-F(2)/109*y*z^4*v+F(74)/11*z^5*v+6436*x^3*y*u*v+7316*x^2*y^2*u*v+F(29)/5*x*y^3*u*v-1326*y^4*u*v+F(34)/49*x^3*z*u*v-F(122)/27*x^2*y*z*u*v-632*x*y^2*z*u*v+F(46)/49*y^3*z*u*v-13463*x^2*z^2*u*v-808*x*y*z^2*u*v-F(17)/32*y^2*z^2*u*v-13149*x*z^3*u*v-F(117)/88*y*z^3*u*v-F(45)/79*z^4*u*v-F(65)/94*x^3*u^2*v+F(6)/67*x^2*y*u^2*v+F(34)/39*x*y^2*u^2*v-14026*y^3*u^2*v+F(42)/107*x^2*z*u^2*v-3287*x*y*z*u^2*v-F(70)/43*y^2*z*u^2*v+F(29)/104*x*z^2*u^2*v-F(47)/18*y*z^2*u^2*v-11038*z^3*u^2*v+6262*x^2*u^3*v-5255*x*y*u^3*v-F(7)/10*y^2*u^3*v+7065*x*z*u^3*v+5608*y*z*u^3*v+4675*z^2*u^3*v-F(73)/90*x*u^4*v-15822*y*u^4*v-F(71)/63*z*u^4*v+F(110)/97*u^5*v-F(69)/5*x^3*y*v^2+4315*x^2*y^2*v^2-F(124)/45*x*y^3*v^2-F(79)/16*y^4*v^2-10739*x^3*z*v^2-F(93)/46*x^2*y*z*v^2+12499*x*y^2*z*v^2-F(73)/86*y^3*z*v^2+6367*x^2*z^2*v^2-12876*x*y*z^2*v^2-306*y^2*z^2*v^2-89*x*z^3*v^2-F(70)/51*y*z^3*v^2+13120*z^4*v^2+F(61)/57*x^3*u*v^2+14782*x^2*y*u*v^2-F(91)/9*x*y^2*u*v^2-2625*y^3*u*v^2+14747*x^2*z*u*v^2-5899*x*y*z*u*v^2-12944*y^2*z*u*v^2-F(47)/14*x*z^2*u*v^2-4551*y*z^2*u*v^2-F(99)/101*z^3*u*v^2-12618*x^2*u^2*v^2+1507*x*y*u^2*v^2-11951*y^2*u^2*v^2+F(68)/49*x*z*u^2*v^2+F(49)/39*y*z*u^2*v^2-F(56)/103*z^2*u^2*v^2-F(31)/85*x*u^3*v^2-F(32)/49*y*u^3*v^2-F(65)/14*z*u^3*v^2+F(15)/7*u^4*v^2+5749*x^3*v^3-3667*x^2*y*v^3-F(107)/29*x*y^2*v^3+11301*y^3*v^3+F(95)/18*x^2*z*v^3-F(121)/74*x*y*z*v^3+F(75)/26*y^2*z*v^3+F(101)/98*x*z^2*v^3-F(111)/76*y*z^2*v^3-11335*z^3*v^3-15923*x^2*u*v^3-F(36)/83*x*y*u*v^3-4134*y^2*u*v^3-F(87)/118*x*z*u*v^3-F(41)/11*y*z*u*v^3+F(104)/61*z^2*u*v^3+12583*x*u^2*v^3-F(50)/23*y*u^2*v^3-F(31)/44*z*u^2*v^3-F(29)/23*u^3*v^3+F(108)/107*x^2*v^4-8216*x*y*v^4-5009*y^2*v^4+F(101)/26*x*z*v^4-9779*y*z*v^4+F(71)/74*z^2*v^4-3358*x*u*v^4+F(83)/84*y*u*v^4-F(34)/39*z*u*v^4+F(44)/47*u^2*v^4-F(112)/83*x*v^5+F(113)/74*y*v^5+F(82)/79*z*v^5-F(115)/99*u*v^5+F(12)/109*v^6,-x^4*y-F(31)/15*x^3*y^2+7231*x^2*y^3-F(99)/37*x*y^4-F(28)/95*x^4*z-F(53)/107*x^3*y*z-4623*x^2*y^2*z+5300*x*y^3*z-F(41)/111*y^4*z+12205*x^3*z^2+F(113)/120*x^2*y*z^2+F(54)/49*x*y^2*z^2-F(85)/63*y^3*z^2+F(104)/89*x^2*z^3-F(52)/121*x*y*z^3-F(22)/49*y^2*z^3+14367*x*z^4+F(71)/93*y*z^4+F(55)/56*z^5-F(5)/81*x^4*u-F(67)/81*x^3*y*u-F(83)/13*x^2*y^2*u+F(98)/55*x*y^3*u+15289*y^4*u-F(94)/111*x^3*z*u+F(40)/29*x^2*y*z*u-F(16)/59*x*y^2*z*u-F(107)/14*y^3*z*u+2965*x^2*z^2*u-459*x*y*z^2*u-F(2)/47*y^2*z^2*u+F(35)/22*x*z^3*u+F(119)/39*y*z^3*u-12180*z^4*u-13679*x^3*u^2+1534*x^2*y*u^2+11305*x*y^2*u^2-F(62)/9*y^3*u^2-F(68)/39*x^2*z*u^2+F(11)/90*x*y*z*u^2-F(36)/101*y^2*z*u^2-2896*x*z^2*u^2-15114*y*z^2*u^2-F(49)/114*z^3*u^2+F(19)/16*x^2*u^3-11401*x*y*u^3-F(109)/3*y^2*u^3+F(67)/80*x*z*u^3+F(53)/92*y*z*u^3+2894*z^2*u^3+F(119)/74*x*u^4+407*y*u^4-F(65)/53*z*u^4+F(95)/94*u^5-9309*x^4*v+F(21)/40*x^3*y*v+1436*x^2*y^2*v+2194*x*y^3*v+6994*y^4*v-F(116)/81*x^3*z*v+F(13)/2*x^2*y*z*v-F(12)/13*x*y^2*z*v-F(23)/84*y^3*z*v-F(61)/83*x^2*z^2*v+2023*x*y*z^2*v+F(19)/40*y^2*z^2*v+F(43)/26*x*z^3*v-F(59)/113*y*z^3*v-F(47)/53*z^4*v+15580*x^3*u*v+21*x^2*y*u*v+F(113)/97*x*y^2*u*v-15419*y^3*u*v-15243*x^2*z*u*v+5128*x*y*z*u*v-F(34)/47*y^2*z*u*v+13206*x*z^2*u*v-4833*y*z^2*u*v+F(107)/91*z^3*u*v-1693*x^2*u^2*v+F(54)/53*x*y*u^2*v-F(86)/67*y^2*u^2*v+F(98)/9*x*z*u^2*v+F(86)/17*y*z*u^2*v+F(64)/89*z^2*u^2*v+F(25)/113*x*u^3*v+7884*y*u^3*v+14089*z*u^3*v-12027*u^4*v-9471*x^3*v^2-F(36)/85*x^2*y*v^2-F(21)/13*x*y^2*v^2+15888*y^3*v^2+F(76)/109*x^2*z*v^2+4547*x*y*z*v^2+F(115)/12*y^2*z*v^2-F(11)/107*x*z^2*v^2+6764*y*z^2*v^2-8321*z^3*v^2+F(84)/101*x^2*u*v^2-202*x*y*u*v^2+3251*y^2*u*v^2+F(91)/4*x*z*u*v^2+7124*y*z*u*v^2-F(53)/81*z^2*u*v^2+F(47)/84*x*u^2*v^2-8833*y*u^2*v^2+F(117)/14*z*u^2*v^2-F(3)/113*u^3*v^2+F(126)/97*x^2*v^3-F(78)/115*x*y*v^3+F(68)/63*y^2*v^3-F(34)/109*x*z*v^3+5913*y*z*v^3+6226*z^2*v^3-2365*x*u*v^3+F(91)/120*y*u*v^3+14120*z*u*v^3-F(69)/8*u^2*v^3+F(71)/12*x*v^4-13094*y*v^4-7262*z*v^4-33*u*v^4+5367*v^5,-9533*x^4*y-318*x^3*y^2+F(8)/49*x^2*y^3+F(83)/29*x*y^4+13129*y^5+221*x^4*z+F(115)/48*x^3*y*z+12508*x^2*y^2*z+F(97)/52*x*y^3*z+11479*y^4*z+8941*x^3*z^2+F(104)/109*x^2*y*z^2+9191*x*y^2*z^2+F(103)/64*y^3*z^2+10584*x^2*z^3-7728*x*y*z^3+3979*y^2*z^3+F(15)/82*x*z^4+5409*y*z^4-1326*z^5+3756*x^4*u-F(57)/62*x^3*y*u+F(63)/47*x^2*y^2*u-14600*x*y^3*u+159*y^4*u-F(11)/4*x^3*z*u-F(113)/57*x^2*y*z*u-F(26)/125*x*y^2*z*u-F(32)/87*y^3*z*u-F(10)/21*x^2*z^2*u+12927*x*y*z^2*u-F(73)/62*y^2*z^2*u+F(115)/99*x*z^3*u-F(13)/3*y*z^3*u-F(126)/25*z^4*u-3969*x^3*u^2-F(122)/57*x^2*y*u^2-5003*x*y^2*u^2-F(100)/117*y^3*u^2-F(71)/30*x^2*z*u^2+7356*x*y*z*u^2-2211*y^2*z*u^2+F(31)/40*x*z^2*u^2-6722*y*z^2*u^2-139*z^3*u^2+4426*x^2*u^3+F(1)/115*x*y*u^3-F(72)/85*y^2*u^3+15260*x*z*u^3+7938*y*z*u^3+F(4)/115*z^2*u^3-F(33)/89*x*u^4+F(31)/108*y*u^4-F(50)/83*z*u^4+F(14)/107*u^5+F(24)/95*x^4*v-F(113)/17*x^3*y*v+F(81)/14*x^2*y^2*v-9957*x*y^3*v-10075*y^4*v-F(122)/113*x^3*z*v+F(65)/118*x^2*y*z*v-F(96)/29*x*y^2*z*v-F(19)/41*y^3*z*v+F(113)/35*x^2*z^2*v+F(121)/31*x*y*z^2*v-F(9)/68*y^2*z^2*v+F(91)/45*x*z^3*v-F(23)/116*y*z^3*v-F(67)/99*z^4*v-5355*x^3*u*v-3112*x^2*y*u*v-12824*x*y^2*u*v-F(58)/123*y^3*u*v-F(13)/22*x^2*z*u*v-F(19)/85*x*y*z*u*v-F(121)/24*y^2*z*u*v-14093*x*z^2*u*v+F(99)/95*y*z^2*u*v+F(89)/50*z^3*u*v+13096*x^2*u^2*v-F(109)/120*x*y*u^2*v+F(121)/61*y^2*u^2*v+F(80)/41*x*z*u^2*v-39*y*z*u^2*v-F(8)/99*z^2*u^2*v+F(5)/17*x*u^3*v+F(112)/69*y*u^3*v+14346*z*u^3*v-7173*u^4*v+F(125)/13*x^3*v^2+F(43)/53*x^2*y*v^2-F(78)/103*x*y^2*v^2-F(109)/111*y^3*v^2+F(33)/13*x^2*z*v^2-15333*x*y*z*v^2+F(87)/49*y^2*z*v^2-7212*x*z^2*v^2+7729*y*z^2*v^2-F(86)/123*z^3*v^2-F(119)/103*x^2*u*v^2-F(71)/122*x*y*u*v^2-F(81)/113*y^2*u*v^2+6133*x*z*u*v^2+F(55)/72*y*z*u*v^2+F(69)/31*z^2*u*v^2+12828*x*u^2*v^2+F(94)/15*y*u^2*v^2-7588*z*u^2*v^2+F(21)/41*u^3*v^2-8712*x^2*v^3+F(74)/9*x*y*v^3-F(11)/87*y^2*v^3+1446*x*z*v^3-F(3)/95*y*z*v^3-F(87)/55*z^2*v^3-717*x*u*v^3-F(110)/97*y*u*v^3-F(13)/113*z*u*v^3-F(95)/81*u^2*v^3-F(37)/68*x*v^4+5112*y*v^4-F(56)/11*z*v^4-F(6)/115*u*v^4+7910*v^5,F(25)/42*x^4*y-F(42)/79*x^3*y^2-F(59)/21*x^2*y^3+2736*x*y^4-F(107)/115*x^4*z-203*x^3*y*z+F(47)/101*x^2*y^2*z+7686*x*y^3*z-F(63)/64*y^4*z+F(103)/57*x^3*z^2-12082*x^2*y*z^2+F(11)/102*x*y^2*z^2-F(83)/43*y^3*z^2+F(13)/49*x^2*z^3-2685*x*y*z^3+F(123)/44*y^2*z^3+F(31)/12*x*z^4+F(126)/83*y*z^4+14745*z^5+F(83)/37*x^4*u+7362*x^3*y*u-14615*x^2*y^2*u-14109*x*y^3*u+F(49)/47*y^4*u+1929*x^3*z*u+F(83)/71*x^2*y*z*u-13640*x*y^2*z*u-F(97)/58*y^3*z*u-11141*x^2*z^2*u-F(61)/49*x*y*z^2*u-3745*y^2*z^2*u-F(74)/21*x*z^3*u+3493*y*z^3*u-7540*z^4*u-F(103)/118*x^3*u^2-F(43)/32*x^2*y*u^2-9200*x*y^2*u^2-F(23)/65*y^3*u^2+15895*x^2*z*u^2-13924*x*y*z*u^2-14291*y^2*z*u^2-11039*x*z^2*u^2-F(31)/37*y*z^2*u^2-F(101)/93*z^3*u^2-F(39)/83*x^2*u^3-4536*x*y*u^3-F(78)/47*y^2*u^3+F(75)/44*x*z*u^3-F(24)/121*y*z*u^3-F(81)/113*z^2*u^3-F(81)/89*x*u^4+15825*y*u^4-4111*z*u^4+5850*u^5-12534*x^4*v-F(69)/94*x^3*y*v-10076*x^2*y^2*v+3952*x*y^3*v+F(25)/12*y^4*v+F(21)/34*x^3*z*v+11002*x^2*y*z*v-54*x*y^2*z*v+F(20)/23*y^3*z*v+4991*x^2*z^2*v+549*x*y*z^2*v+2687*y^2*z^2*v-F(110)/9*x*z^3*v+11359*y*z^3*v+F(49)/24*z^4*v+F(62)/107*x^3*u*v-F(27)/41*x^2*y*u*v-F(17)/52*x*y^2*u*v-10972*y^3*u*v+F(12)/103*x^2*z*u*v-318*x*y*z*u*v-F(77)/40*y^2*z*u*v-F(114)/53*x*z^2*u*v+F(17)/28*y*z^2*u*v-8084*z^3*u*v+F(85)/36*x^2*u^2*v+F(7)/100*x*y*u^2*v-5772*y^2*u^2*v-F(89)/114*x*z*u^2*v-F(40)/121*y*z*u^2*v+3340*z^2*u^2*v+F(36)/113*x*u^3*v-F(38)/93*y*u^3*v+2519*z*u^3*v-7084*u^4*v+8136*x^3*v^2-F(55)/23*x^2*y*v^2+F(27)/7*x*y^2*v^2+F(74)/39*y^3*v^2+F(63)/16*x^2*z*v^2-8661*x*y*z*v^2+F(2)/91*y^2*z*v^2+3773*x*z^2*v^2-F(75)/122*y*z^2*v^2+447*z^3*v^2-F(59)/109*x^2*u*v^2-F(119)/9*x*y*u*v^2-F(67)/49*y^2*u*v^2-11334*x*z*u*v^2-10482*y*z*u*v^2-F(60)/91*z^2*u*v^2+F(94)/65*x*u^2*v^2-F(108)/17*y*u^2*v^2-F(69)/70*z*u^2*v^2-F(23)/20*u^3*v^2+F(8)/115*x^2*v^3+F(29)/41*x*y*v^3+F(8)/15*y^2*v^3-F(95)/6*x*z*v^3-9714*y*z*v^3+2550*z^2*v^3-F(121)/80*x*u*v^3+F(67)/18*y*u*v^3+F(43)/5*z*u*v^3+F(23)/124*u^2*v^3-12509*x*v^4-F(104)/79*y*v^4-F(73)/21*z*v^4-1238*u*v^4+9038*v^5,F(94)/107*x^4*y+F(47)/14*x^3*y^2-6362*x^2*y^3-F(20)/59*x*y^4-F(43)/120*y^5-3028*x^4*z-15141*x^3*y*z-2028*x^2*y^2*z+F(84)/115*x*y^3*z-3024*y^4*z+2811*x^3*z^2+F(47)/45*x^2*y*z^2+F(121)/101*x*y^2*z^2-F(100)/57*y^3*z^2+F(8)/115*x^2*z^3+F(1)/101*x*y*z^3-F(13)/112*y^2*z^3+3618*x*z^4+F(88)/67*y*z^4-F(52)/63*z^5+F(102)/97*x^4*u-F(12)/89*x^3*y*u-102*x^2*y^2*u-3846*x*y^3*u-F(61)/86*y^4*u+F(85)/54*x^3*z*u+F(78)/29*x^2*y*z*u-13381*x*y^2*z*u-F(49)/95*y^3*z*u-F(77)/2*x^2*z^2*u-5784*x*y*z^2*u+1557*y^2*z^2*u-9163*x*z^3*u-F(114)/121*y*z^3*u-F(57)/103*z^4*u+F(36)/31*x^3*u^2-9062*x^2*y*u^2-F(23)/111*x*y^2*u^2+7362*y^3*u^2-7671*x^2*z*u^2+14945*x*y*z*u^2+7901*y^2*z*u^2+F(51)/5*x*z^2*u^2-F(109)/48*y*z^2*u^2+7696*z^3*u^2+11280*x^2*u^3-F(44)/57*x*y*u^3-13736*y^2*u^3-13458*x*z*u^3-14723*y*z*u^3-707*z^2*u^3+899*x*u^4-10381*y*u^4+F(99)/25*z*u^4-7788*u^5-237*x^4*v+F(45)/43*x^3*y*v-7666*x^2*y^2*v-F(4)/109*x*y^3*v+4303*y^4*v-13107*x^3*z*v-F(108)/91*x^2*y*z*v-7707*x*y^2*z*v-F(73)/47*y^3*z*v+F(61)/118*x^2*z^2*v-F(11)/65*x*y*z^2*v+2970*y^2*z^2*v-F(104)/37*x*z^3*v-15408*y*z^3*v-F(64)/55*z^4*v+F(47)/113*x^3*u*v+2185*x^2*y*u*v+7941*x*y^2*u*v-F(61)/37*y^3*u*v+6482*x^2*z*u*v-F(11)/70*x*y*z*u*v+F(83)/110*y^2*z*u*v-F(109)/83*x*z^2*u*v-F(86)/95*y*z^2*u*v-7583*z^3*u*v+F(83)/45*x^2*u^2*v+F(89)/38*x*y*u^2*v-F(2)/11*y^2*u^2*v+3577*x*z*u^2*v+F(124)/125*y*z*u^2*v-1151*z^2*u^2*v+F(109)/85*x*u^3*v+F(70)/13*y*u^3*v+F(37)/104*z*u^3*v-210*u^4*v+F(51)/29*x^3*v^2-F(104)/111*x^2*y*v^2+F(105)/58*x*y^2*v^2-13459*y^3*v^2-F(80)/79*x^2*z*v^2-3006*x*y*z*v^2-F(115)/16*y^2*z*v^2+8208*x*z^2*v^2+F(35)/38*y*z^2*v^2+F(49)/27*z^3*v^2-1647*x^2*u*v^2+10482*x*y*u*v^2-F(34)/93*y^2*u*v^2+F(97)/18*x*z*u*v^2+F(101)/20*y*z*u*v^2+1711*z^2*u*v^2+F(91)/36*x*u^2*v^2-F(96)/23*y*u^2*v^2+7006*z*u^2*v^2+F(86)/31*u^3*v^2-10734*x^2*v^3-F(43)/18*x*y*v^3-4597*y^2*v^3-11174*x*z*v^3-7334*y*z*v^3+F(7)/96*z^2*v^3+F(4)/97*x*u*v^3-F(5)/82*y*u*v^3-15600*z*u*v^3-F(69)/94*u^2*v^3-F(71)/25*x*v^4+F(21)/97*y*v^4+F(117)/23*z*v^4-6557*u*v^4-F(67)/83*v^5,8164*x^4*y+F(19)/73*x^3*y^2-1592*x^2*y^3-F(28)/87*x*y^4-F(63)/103*x^4*z+F(11)/42*x^3*y*z-F(52)/67*x^2*y^2*z-13766*x*y^3*z+11378*y^4*z+F(10)/37*x^3*z^2+F(115)/41*x^2*y*z^2+F(11)/100*x*y^2*z^2-F(49)/40*y^3*z^2+F(86)/111*x^2*z^3+F(124)/5*x*y*z^3-F(25)/79*y^2*z^3-14525*x*z^4+11380*y*z^4-F(53)/42*z^5-12169*x^4*u-F(14)/51*x^3*y*u+F(68)/33*x^2*y^2*u-F(3)/62*x*y^3*u-F(31)/22*y^4*u-F(74)/93*x^3*z*u+12924*x^2*y*z*u-F(103)/123*x*y^2*z*u-F(74)/97*y^3*z*u-2789*x^2*z^2*u-F(95)/32*x*y*z^2*u+F(45)/13*y^2*z^2*u+F(40)/71*x*z^3*u+F(49)/110*y*z^3*u+F(34)/75*z^4*u+9829*x^3*u^2-F(59)/92*x^2*y*u^2+F(106)/65*x*y^2*u^2+F(123)/86*y^3*u^2+7133*x^2*z*u^2-F(73)/46*x*y*z*u^2-F(7)/29*y^2*z*u^2-937*x*z^2*u^2-F(65)/67*y*z^2*u^2-F(88)/111*z^3*u^2-F(61)/119*x^2*u^3+975*x*y*u^3-F(54)/7*y^2*u^3-F(37)/33*x*z*u^3+F(61)/59*y*z*u^3+F(51)/115*z^2*u^3+F(117)/43*x*u^4+8506*y*u^4+13941*z*u^4-14945*u^5-F(115)/63*x^4*v-14237*x^3*y*v-F(74)/87*x^2*y^2*v+F(104)/47*x*y^3*v-F(95)/104*y^4*v+11535*x^3*z*v-F(119)/75*x^2*y*z*v-44*x*y^2*z*v+11299*y^3*z*v-F(21)/113*x^2*z^2*v-2852*x*y*z^2*v+F(95)/77*y^2*z^2*v-F(75)/19*x*z^3*v-4864*y*z^3*v-F(79)/88*z^4*v+139*x^3*u*v-10068*x^2*y*u*v+2049*x*y^2*u*v+7515*y^3*u*v+F(97)/56*x^2*z*u*v+F(109)/113*x*y*z*u*v+7778*y^2*z*u*v-F(71)/11*x*z^2*u*v-F(80)/19*y*z^2*u*v+F(55)/59*z^3*u*v-F(69)/98*x^2*u^2*v-15679*x*y*u^2*v+F(114)/11*y^2*u^2*v+F(69)/65*x*z*u^2*v+879*y*z*u^2*v+F(45)/104*z^2*u^2*v+F(47)/97*x*u^3*v-1373*y*u^3*v+15885*z*u^3*v+11121*u^4*v-5042*x^3*v^2+F(4)/25*x^2*y*v^2-8607*x*y^2*v^2-F(25)/33*y^3*v^2+F(93)/55*x^2*z*v^2+68*x*y*z*v^2-4167*y^2*z*v^2+14180*x*z^2*v^2-F(115)/47*y*z^2*v^2-F(81)/67*z^3*v^2-12099*x^2*u*v^2+F(34)/107*x*y*u*v^2+F(122)/59*y^2*u*v^2+775*x*z*u*v^2-91*y*z*u*v^2-F(85)/96*z^2*u*v^2-F(59)/95*x*u^2*v^2+174*y*u^2*v^2+F(11)/16*z*u^2*v^2+F(66)/37*u^3*v^2-F(121)/36*x^2*v^3+6070*x*y*v^3-F(83)/52*y^2*v^3-F(121)/59*x*z*v^3-F(55)/12*y*z*v^3+8088*z^2*v^3-F(20)/29*x*u*v^3+F(76)/125*y*u*v^3-10858*z*u*v^3+1833*u^2*v^3-F(103)/50*x*v^4+F(76)/93*y*v^4-F(119)/18*z*v^4+F(37)/114*u*v^4+F(51)/7*v^5,F(85)/56*x^4*y-7839*x^3*y^2+F(12)/37*x^2*y^3+6558*x*y^4-8191*x^4*z+F(115)/7*x^3*y*z+F(81)/23*x^2*y^2*z-4121*x*y^3*z-1131*y^4*z-F(23)/37*x^3*z^2-F(71)/32*x^2*y*z^2+F(30)/97*x*y^2*z^2+5070*y^3*z^2-F(49)/123*x^2*z^3+F(103)/88*x*y*z^3-F(45)/19*y^2*z^3+5132*x*z^4+7277*y*z^4+1896*z^5-F(103)/75*x^4*u-12020*x^3*y*u+12337*x^2*y^2*u+6248*x*y^3*u+14290*y^4*u-F(87)/44*x^3*z*u-5364*x^2*y*z*u-11801*x*y^2*z*u-F(59)/37*y^3*z*u+F(34)/109*x^2*z^2*u-14482*x*y*z^2*u-10338*y^2*z^2*u+F(118)/73*x*z^3*u+F(7)/8*y*z^3*u+158*z^4*u+10590*x^3*u^2-5182*x^2*y*u^2+F(83)/62*x*y^2*u^2+11557*y^3*u^2-F(92)/119*x^2*z*u^2-F(37)/94*x*y*z*u^2+5383*y^2*z*u^2-365*x*z^2*u^2+F(7)/62*y*z^2*u^2-7965*z^3*u^2-F(10)/43*x^2*u^3+F(119)/101*x*y*u^3-F(113)/83*y^2*u^3-F(121)/41*x*z*u^3+F(61)/104*y*z*u^3+F(37)/60*z^2*u^3-F(74)/95*x*u^4-F(113)/66*y*u^4-205*z*u^4+4787*u^5-F(94)/93*x^4*v+14871*x^3*y*v-14723*x^2*y^2*v+10730*x*y^3*v+F(112)/17*y^4*v-F(35)/19*x^3*z*v-3487*x^2*y*z*v-F(65)/43*x*y^2*z*v-7445*y^3*z*v-F(79)/124*x^2*z^2*v+7423*x*y*z^2*v+F(91)/2*y^2*z^2*v+F(91)/34*x*z^3*v-6970*y*z^3*v-F(50)/113*z^4*v+F(75)/43*x^3*u*v-127*x^2*y*u*v+11978*x*y^2*u*v+F(48)/113*y^3*u*v+F(113)/62*x^2*z*u*v-8941*x*y*z*u*v-F(101)/112*y^2*z*u*v-5737*x*z^2*u*v-F(31)/123*y*z^2*u*v+9490*z^3*u*v+F(19)/92*x^2*u^2*v-F(107)/73*x*y*u^2*v-F(23)/121*y^2*u^2*v+F(38)/65*x*z*u^2*v-672*y*z*u^2*v+F(13)/77*z^2*u^2*v+F(46)/119*x*u^3*v-F(103)/18*y*u^3*v+F(107)/59*z*u^3*v-F(52)/21*u^4*v-F(94)/87*x^3*v^2-F(74)/31*x^2*y*v^2-F(9)/22*x*y^2*v^2-2896*y^3*v^2+F(113)/3*x^2*z*v^2-5386*x*y*z*v^2-11391*y^2*z*v^2+F(42)/97*x*z^2*v^2+F(77)/64*y*z^2*v^2-1610*z^3*v^2-F(102)/43*x^2*u*v^2+F(124)/39*x*y*u*v^2+14829*y^2*u*v^2+F(88)/113*x*z*u*v^2-10411*y*z*u*v^2-F(51)/43*z^2*u*v^2-F(36)/121*x*u^2*v^2+9487*y*u^2*v^2-5589*z*u^2*v^2+4335*u^3*v^2-F(5)/91*x^2*v^3+6084*x*y*v^3-F(56)/39*y^2*v^3-F(84)/101*x*z*v^3-F(81)/85*y*z*v^3-6521*z^2*v^3-2432*x*u*v^3+14317*y*u*v^3-F(43)/82*z*u*v^3+F(121)/8*u^2*v^3+14783*x*v^4-F(92)/45*y*v^4+F(112)/27*z*v^4-8410*u*v^4+F(31)/105*v^5,-6691*x^4*y-10158*x^3*y^2-5372*x^2*y^3+4132*x*y^4+F(106)/9*y^5+15600*x^4*z-803*x^3*y*z+F(43)/29*x^2*y^2*z+F(9)/91*x*y^3*z-F(92)/61*y^4*z+4807*x^3*z^2-12562*x^2*y*z^2+14234*x*y^2*z^2-F(91)/17*y^3*z^2-F(91)/30*x^2*z^3-10615*x*y*z^3-4206*y^2*z^3-F(29)/45*x*z^4-F(11)/86*y*z^4-F(115)/9*z^5+F(125)/112*x^4*u+F(52)/59*x^3*y*u+F(92)/49*x^2*y^2*u+F(121)/85*x*y^3*u-F(51)/14*y^4*u-F(73)/48*x^3*z*u-F(1)/110*x^2*y*z*u+F(12)/65*x*y^2*z*u+15045*y^3*z*u+12826*x^2*z^2*u-F(123)/89*x*y*z^2*u+9465*y^2*z^2*u-F(67)/31*x*z^3*u-5080*y*z^3*u-7944*z^4*u-F(107)/72*x^3*u^2+1473*x^2*y*u^2+7965*x*y^2*u^2+15753*y^3*u^2-F(95)/98*x^2*z*u^2-9827*x*y*z*u^2-F(25)/53*y^2*z*u^2-F(83)/54*x*z^2*u^2-13217*y*z^2*u^2-F(117)/110*z^3*u^2+230*x^2*u^3-12120*x*y*u^3+F(11)/36*y^2*u^3-2071*x*z*u^3+F(109)/59*y*z*u^3+6909*z^2*u^3-F(15)/64*x*u^4+F(45)/82*y*u^4-3091*z*u^4-15711*u^5+5957*x^4*v-F(45)/86*x^3*y*v+F(26)/29*x^2*y^2*v-F(40)/57*x*y^3*v+F(25)/43*y^4*v+F(126)/37*x^3*z*v-F(38)/33*x^2*y*z*v+F(65)/109*x*y^2*z*v-F(33)/68*y^3*z*v-7287*x^2*z^2*v-4842*x*y*z^2*v+F(35)/118*y^2*z^2*v+6157*x*z^3*v-F(97)/89*y*z^3*v-F(91)/50*z^4*v-F(70)/27*x^3*u*v+F(32)/9*x^2*y*u*v+F(78)/125*x*y^2*u*v+F(38)/7*y^3*u*v-3214*x^2*z*u*v-F(68)/101*x*y*z*u*v+F(87)/55*y^2*z*u*v-F(69)/98*x*z^2*u*v+5805*y*z^2*u*v+F(41)/102*z^3*u*v-F(43)/54*x^2*u^2*v-F(42)/73*x*y*u^2*v-F(13)/49*y^2*u^2*v+11864*x*z*u^2*v+F(121)/37*y*z*u^2*v-F(100)/109*z^2*u^2*v-12609*x*u^3*v-9114*y*u^3*v-8746*z*u^3*v+11659*u^4*v+3799*x^3*v^2-9581*x^2*y*v^2+F(60)/91*x*y^2*v^2+2029*y^3*v^2+12075*x^2*z*v^2+210*x*y*z*v^2-F(1)/22*y^2*z*v^2+F(17)/58*x*z^2*v^2+1212*y*z^2*v^2+F(118)/27*z^3*v^2-3571*x^2*u*v^2-3139*x*y*u*v^2-F(23)/100*y^2*u*v^2-1240*x*z*u*v^2+F(71)/49*y*z*u*v^2-F(21)/103*z^2*u*v^2-F(110)/71*x*u^2*v^2-F(40)/77*y*u^2*v^2-F(103)/29*z*u^2*v^2+10737*u^3*v^2+2828*x^2*v^3+F(14)/39*x*y*v^3+7564*y^2*v^3+F(113)/50*x*z*v^3+F(38)/79*y*z*v^3+F(59)/66*z^2*v^3+2726*x*u*v^3+F(91)/94*y*u*v^3-15730*z*u*v^3-13408*u^2*v^3-F(97)/42*x*v^4+F(54)/29*y*v^4-F(33)/73*z*v^4+4823*u*v^4+F(57)/71*v^5,-14556*x^3*y*z-9751*x^2*y^2*z-F(45)/28*x*y^3*z+F(85)/23*y^4*z+5623*x^3*z^2+5369*x^2*y*z^2-F(19)/60*x*y^2*z^2-F(36)/5*y^3*z^2-F(95)/36*x^2*z^3+5862*x*y*z^3-F(5)/93*y^2*z^3+2949*x*z^4+11357*y*z^4-5679*z^5-F(52)/45*x^3*y*u+4448*x^2*y^2*u-F(9)/22*x*y^3*u+2427*y^4*u+3296*x^3*z*u+F(16)/39*x^2*y*z*u+F(53)/57*x*y^2*z*u+F(15)/41*y^3*z*u+9473*x^2*z^2*u+37*x*y*z^2*u-F(58)/69*y^2*z^2*u-F(23)/56*x*z^3*u-F(13)/90*y*z^3*u-F(54)/29*z^4*u-F(41)/67*x^3*u^2+10258*x^2*y*u^2+F(23)/44*x*y^2*u^2-12952*y^3*u^2+2124*x^2*z*u^2-1677*x*y*z*u^2+12911*y^2*z*u^2+F(22)/45*x*z^2*u^2+F(17)/84*y*z^2*u^2+5910*z^3*u^2+4782*x^2*u^3+F(119)/39*x*y*u^3-F(17)/84*y^2*u^3-F(120)/91*x*z*u^3+F(35)/59*y*z*u^3+F(17)/77*z^2*u^3-4467*x*u^4-F(77)/4*y*u^4-F(26)/53*z*u^4-3580*u^5-11977*x^3*y*v-F(118)/77*x^2*y^2*v+6040*x*y^3*v+9724*y^4*v-F(47)/5*x^3*z*v+F(59)/101*x^2*y*z*v+1212*x*y^2*z*v-F(7)/121*y^3*z*v+F(93)/53*x^2*z^2*v-F(56)/23*x*y*z^2*v-4470*y^2*z^2*v+F(110)/111*x*z^3*v-F(41)/99*y*z^3*v-F(81)/10*z^4*v-F(71)/24*x^3*u*v+F(26)/115*x^2*y*u*v+F(59)/39*x*y^2*u*v-10029*y^3*u*v+11748*x^2*z*u*v+5749*x*y*z*u*v+6887*y^2*z*u*v+F(38)/3*x*z^2*u*v-F(116)/61*y*z^2*u*v-F(55)/118*z^3*u*v+F(105)/22*x^2*u^2*v+F(70)/87*x*y*u^2*v-F(28)/13*y^2*u^2*v-F(109)/123*x*z*u^2*v-F(102)/47*y*z*u^2*v-F(52)/71*z^2*u^2*v+F(101)/95*x*u^3*v+F(51)/16*y*u^3*v+F(15)/97*z*u^3*v-F(78)/125*u^4*v+F(35)/46*x^3*v^2-9526*x^2*y*v^2+10781*x*y^2*v^2-F(119)/44*y^3*v^2-F(23)/10*x^2*z*v^2+F(59)/29*x*y*z*v^2-15144*y^2*z*v^2+F(29)/120*x*z^2*v^2-F(53)/126*y*z^2*v^2-F(93)/85*z^3*v^2+F(53)/8*x^2*u*v^2-487*x*y*u*v^2-12143*y^2*u*v^2+13825*x*z*u*v^2+F(55)/6*y*z*u*v^2-4250*z^2*u*v^2+4237*x*u^2*v^2-F(109)/9*y*u^2*v^2+F(67)/53*z*u^2*v^2+F(82)/33*u^3*v^2+8660*x^2*v^3+15046*x*y*v^3-F(79)/84*y^2*v^3-10310*x*z*v^3+110*y*z*v^3-7636*z^2*v^3+F(57)/92*x*u*v^3-F(22)/119*y*u*v^3-F(95)/103*z*u*v^3+5138*u^2*v^3+F(123)/49*x*v^4-7587*y*v^4+F(30)/41*z*v^4-F(124)/121*u*v^4+F(54)/71*v^5,-F(29)/60*x^4*y-F(108)/77*x^3*y^2-F(109)/37*x^2*y^3-3619*x*y^4+F(109)/6*x^4*z-F(37)/67*x^3*y*z+F(53)/45*x^2*y^2*z+5291*x*y^3*z-2927*y^4*z+F(34)/5*x^3*z^2+F(87)/17*x^2*y*z^2+F(100)/89*x*y^2*z^2-F(114)/29*y^3*z^2-4057*x^2*z^3-F(1)/42*x*y*z^3-F(14)/61*y^2*z^3-398*x*z^4-F(122)/73*y*z^4+F(66)/37*z^5+F(99)/37*x^4*u-5691*x^3*y*u-8778*x^2*y^2*u+F(17)/115*x*y^3*u+F(51)/113*y^4*u-F(71)/101*x^3*z*u+F(85)/91*x^2*y*z*u-F(92)/9*x*y^2*z*u-3442*y^3*z*u+F(109)/26*x^2*z^2*u+F(50)/37*x*y*z^2*u+F(77)/94*y^2*z^2*u+F(16)/35*x*z^3*u+9985*y*z^3*u+F(5)/102*z^4*u-5932*x^3*u^2+F(89)/125*x^2*y*u^2-895*x*y^2*u^2-12455*y^3*u^2-630*x^2*z*u^2-F(64)/47*x*y*z*u^2+F(25)/9*y^2*z*u^2+7906*x*z^2*u^2+6827*y*z^2*u^2+9808*z^3*u^2-F(113)/118*x^2*u^3+F(79)/8*x*y*u^3+9484*y^2*u^3+F(62)/39*x*z*u^3+F(6)/85*y*z*u^3-F(23)/49*z^2*u^3-F(93)/115*x*u^4-F(11)/93*y*u^4-15177*z*u^4-F(13)/2*u^5-7623*x^4*v-F(103)/73*x^3*y*v-F(96)/115*x^2*y^2*v+F(39)/76*x*y^3*v+F(80)/79*y^4*v+F(43)/68*x^3*z*v+F(45)/97*x^2*y*z*v+F(101)/87*x*y^2*z*v+4632*y^3*z*v-918*x^2*z^2*v+8248*x*y*z^2*v-4276*y^2*z^2*v+8853*x*z^3*v-F(39)/61*y*z^3*v-F(121)/87*z^4*v+9968*x^3*u*v+473*x^2*y*u*v+F(117)/56*x*y^2*u*v-F(19)/21*y^3*u*v+F(121)/119*x^2*z*u*v+F(3)/98*x*y*z*u*v-F(65)/42*y^2*z*u*v-3723*x*z^2*u*v+F(7)/34*y*z^2*u*v-F(112)/87*z^3*u*v+103*x^2*u^2*v+F(25)/41*x*y*u^2*v-14459*y^2*u^2*v-F(56)/41*x*z*u^2*v-F(59)/81*y*z*u^2*v-F(109)/102*z^2*u^2*v-F(87)/16*x*u^3*v-13011*y*u^3*v+F(49)/123*z*u^3*v+F(106)/89*u^4*v-F(61)/51*x^3*v^2+14107*x^2*y*v^2+8035*x*y^2*v^2-8853*y^3*v^2+5723*x^2*z*v^2+F(123)/53*x*y*z*v^2-9727*y^2*z*v^2-F(102)/83*x*z^2*v^2+1111*y*z^2*v^2-15745*z^3*v^2+F(83)/118*x^2*u*v^2-F(57)/35*x*y*u*v^2-F(48)/73*y^2*u*v^2-F(28)/37*x*z*u*v^2-F(27)/97*y*z*u*v^2-F(27)/58*z^2*u*v^2+F(71)/93*x*u^2*v^2+F(117)/8*y*u^2*v^2+12344*z*u^2*v^2-2497*u^3*v^2-F(118)/71*x^2*v^3-F(11)/19*x*y*v^3+F(21)/104*y^2*v^3+F(32)/113*x*z*v^3+15544*y*z*v^3+F(31)/18*z^2*v^3+5909*x*u*v^3-F(67)/58*y*u*v^3+F(27)/35*z*u*v^3+F(115)/9*u^2*v^3+F(79)/13*x*v^4+6722*y*v^4-F(37)/114*z*v^4-F(71)/124*u*v^4+4657*v^5,-F(77)/61*x^4*y-F(88)/101*x^3*y^2+F(93)/88*x^2*y^3-F(11)/70*x*y^4+9806*y^5+7896*x^4*z-4699*x^3*y*z+F(55)/122*x^2*y^2*z-F(63)/122*x*y^3*z-F(125)/74*y^4*z+F(47)/45*x^3*z^2+F(101)/17*x^2*y*z^2+F(92)/47*x*y^2*z^2+F(69)/82*y^3*z^2+12402*x^2*z^3+F(113)/98*x*y*z^3-F(101)/33*y^2*z^3-15376*x*z^4+F(47)/71*y*z^4-F(73)/10*z^5+F(65)/74*x^4*u-14409*x^3*y*u-14478*x^2*y^2*u+13593*x*y^3*u+F(102)/97*y^4*u+F(39)/62*x^3*z*u-F(34)/125*x^2*y*z*u-F(83)/9*x*y^2*z*u+F(45)/113*y^3*z*u+14484*x^2*z^2*u-15293*x*y*z^2*u-F(26)/55*y^2*z^2*u-958*x*z^3*u+F(67)/35*y*z^3*u-F(93)/19*z^4*u+F(25)/16*x^3*u^2+F(107)/52*x^2*y*u^2-4599*x*y^2*u^2-F(86)/51*y^3*u^2-9885*x^2*z*u^2-F(77)/47*x*y*z*u^2+F(33)/65*y^2*z*u^2+F(90)/109*x*z^2*u^2-F(61)/26*y*z^2*u^2+6198*z^3*u^2-F(38)/37*x^2*u^3-13935*x*y*u^3-142*y^2*u^3-F(64)/5*x*z*u^3-7228*y*z*u^3+1251*z^2*u^3+1556*x*u^4+F(117)/121*y*u^4-F(92)/35*z*u^4+F(99)/92*u^5+13493*x^4*v+12654*x^3*y*v+F(32)/101*x^2*y^2*v-11118*x*y^3*v+F(43)/51*y^4*v-575*x^3*z*v+F(103)/21*x^2*y*z*v+F(85)/24*x*y^2*z*v+1788*y^3*z*v+F(85)/3*x^2*z^2*v-F(64)/25*x*y*z^2*v+F(57)/35*y^2*z^2*v+F(37)/120*x*z^3*v-F(69)/110*y*z^3*v+F(48)/49*z^4*v+F(55)/114*x^3*u*v-6439*x^2*y*u*v+F(31)/51*x*y^2*u*v-F(90)/49*y^3*u*v-F(45)/104*x^2*z*u*v-12018*x*y*z*u*v+F(6)/119*y^2*z*u*v+F(40)/63*x*z^2*u*v+F(20)/91*y*z^2*u*v+F(50)/43*z^3*u*v+F(1)/26*x^2*u^2*v-F(109)/47*x*y*u^2*v+F(99)/7*y^2*u^2*v+F(72)/83*x*z*u^2*v+F(61)/118*y*z*u^2*v+3530*z^2*u^2*v+6146*x*u^3*v+117*y*u^3*v-9921*z*u^3*v-8708*u^4*v-F(10)/47*x^3*v^2-15294*x^2*y*v^2-7336*x*y^2*v^2+F(1)/66*y^3*v^2-3057*x^2*z*v^2+F(74)/123*x*y*z*v^2+146*y^2*z*v^2-F(103)/34*x*z^2*v^2-F(117)/76*y*z^2*v^2+8472*z^3*v^2-F(7)/92*x^2*u*v^2+10033*x*y*u*v^2+F(43)/53*y^2*u*v^2+4694*x*z*u*v^2-F(49)/2*y*z*u*v^2-F(71)/73*z^2*u*v^2-F(125)/17*x*u^2*v^2-9817*y*u^2*v^2+7218*z*u^2*v^2+6897*u^3*v^2-F(19)/90*x^2*v^3+11899*x*y*v^3-11779*y^2*v^3-5456*x*z*v^3+F(17)/42*y*z*v^3+15340*z^2*v^3+F(12)/7*x*u*v^3+9580*y*u*v^3-502*z*u*v^3-14069*u^2*v^3-4371*x*v^4+14452*y*v^4-9423*z*v^4-F(117)/122*u*v^4+1126*v^5,F(49)/108*x^4-F(39)/4*x^3*y-F(67)/21*x^2*y^2-F(8)/69*x*y^3-9779*y^4+F(57)/14*x^3*z-11145*x^2*y*z+6928*x*y^2*z-7824*y^3*z+F(1)/79*x^2*z^2+5173*x*y*z^2-F(62)/15*y^2*z^2-F(123)/112*x*z^3+F(88)/79*y*z^3+F(1)/125*z^4+F(57)/23*x^3*u-11856*x^2*y*u-7444*x*y^2*u+F(115)/8*y^3*u-11133*x^2*z*u+F(71)/73*x*y*z*u-7941*y^2*z*u+F(69)/65*x*z^2*u+F(22)/75*y*z^2*u+F(65)/121*z^3*u+9471*x^2*u^2+9167*x*y*u^2+F(51)/59*y^2*u^2+12835*x*z*u^2+15047*y*z*u^2+11102*z^2*u^2-10059*x*u^3+F(19)/28*y*u^3+F(65)/21*z*u^3-F(39)/28*u^4-F(3)/73*x^3*v+F(94)/61*x^2*y*v+8778*x*y^2*v-12922*y^3*v-8711*x^2*z*v-F(37)/97*x*y*z*v+14270*y^2*z*v+4487*x*z^2*v-F(59)/112*y*z^2*v-14183*z^3*v+15553*x^2*u*v+3579*x*y*u*v+F(114)/91*y^2*u*v-F(4)/97*x*z*u*v+F(13)/85*y*z*u*v-F(89)/15*z^2*u*v+F(58)/75*x*u^2*v-F(34)/7*y*u^2*v-F(90)/61*z*u^2*v+F(90)/101*u^3*v-14673*x^2*v^2+F(90)/19*x*y*v^2-F(45)/37*y^2*v^2+F(23)/49*x*z*v^2-F(71)/11*y*z*v^2+F(119)/8*z^2*v^2+F(89)/10*x*u*v^2+F(109)/91*y*u*v^2+F(36)/49*z*u*v^2-F(7)/31*u^2*v^2-F(40)/113*x*v^3-F(21)/121*y*v^3+9910*z*v^3+F(33)/14*u*v^3-F(23)/79*v^4,F(93)/70*x^4-F(43)/125*x^3*y+9582*x^2*y^2+7565*x*y^3-11511*y^4-F(3)/79*x^3*z-F(36)/107*x^2*y*z-2038*x*y^2*z+879*y^3*z-4700*x^2*z^2+F(103)/14*x*y*z^2+F(102)/79*y^2*z^2-F(67)/68*x*z^3-F(44)/25*y*z^3+F(105)/79*z^4-F(29)/24*x^3*u-F(74)/83*x^2*y*u+F(67)/43*x*y^2*u+F(49)/12*y^3*u-F(115)/11*x^2*z*u+F(23)/67*x*y*z*u-F(61)/27*y^2*z*u+12257*x*z^2*u+14068*y*z^2*u+F(23)/15*z^3*u+607*x^2*u^2+F(73)/8*x*y*u^2+14237*y^2*u^2-F(13)/33*x*z*u^2+F(110)/71*y*z*u^2+F(41)/101*z^2*u^2+5708*x*u^3+F(88)/67*y*u^3+1460*z*u^3-2472*u^4-1629*x^3*v-F(51)/70*x^2*y*v-F(88)/73*x*y^2*v-F(36)/97*y^3*v+F(38)/11*x^2*z*v+15899*x*y*z*v+F(54)/19*y^2*z*v+9460*x*z^2*v-5150*y*z^2*v+3462*z^3*v+5522*x^2*u*v-F(19)/123*x*y*u*v+14871*y^2*u*v+F(53)/5*x*z*u*v-7535*y*z*u*v-13430*z^2*u*v+F(107)/47*x*u^2*v-8307*y*u^2*v-F(55)/79*z*u^2*v-11945*u^3*v-F(16)/83*x^2*v^2+F(115)/48*x*y*v^2+12389*y^2*v^2+11545*x*z*v^2-F(25)/26*y*z*v^2-3755*z^2*v^2+4724*x*u*v^2-F(31)/21*y*u*v^2+7872*z*u*v^2+F(89)/45*u^2*v^2+F(87)/47*x*v^3+7625*y*v^3+13494*z*v^3-15376*u*v^3-F(25)/126*v^4]))
end


@doc raw"""
    k3_d11_pi11_ss_0()

Return a smooth K3 surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and no 6-secant.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d11_pi11_ss_0()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(38)/41*x^4*y+F(116)/67*x^3*y^2+6547*x^2*y^3-F(50)/71*x*y^4+F(13)/89*y^5+F(63)/22*x^4*z-12151*x^3*y*z-3400*x^2*y^2*z-F(52)/45*x*y^3*z+9027*y^4*z+5420*x^3*z^2+6983*x^2*y*z^2-3285*x*y^2*z^2-F(47)/2*y^3*z^2-137*x^2*z^3+F(40)/51*x*y*z^3+F(111)/40*y^2*z^3-F(97)/93*x*z^4+918*y*z^4-7492*z^5-F(122)/49*x^4*u-F(123)/47*x^3*y*u-F(95)/32*x^2*y^2*u-F(83)/36*x*y^3*u-F(125)/77*y^4*u-9824*x^3*z*u-F(51)/115*x^2*y*z*u-F(83)/18*x*y^2*z*u-F(19)/20*y^3*z*u-F(117)/98*x^2*z^2*u+885*x*y*z^2*u-F(20)/97*y^2*z^2*u+F(69)/55*x*z^3*u-F(11)/82*y*z^3*u+F(93)/47*z^4*u-F(97)/113*x^3*u^2-F(83)/48*x^2*y*u^2+12868*x*y^2*u^2-4932*y^3*u^2-F(97)/114*x^2*z*u^2-F(121)/116*x*y*z*u^2-F(79)/108*y^2*z*u^2-F(27)/68*x*z^2*u^2+F(116)/19*y*z^2*u^2+10019*z^3*u^2-6268*x^2*u^3+F(5)/31*x*y*u^3+13810*y^2*u^3-F(120)/37*x*z*u^3-F(33)/124*y*z*u^3-F(31)/41*z^2*u^3+F(79)/50*x*u^4+F(71)/59*y*u^4+F(110)/81*z*u^4-F(27)/53*u^5-12764*x^4*v-F(7)/22*x^3*y*v+15253*x^2*y^2*v-F(67)/29*x*y^3*v+15620*y^4*v-1202*x^3*z*v+F(56)/57*x^2*y*z*v-99*x*y^2*z*v-F(29)/28*y^3*z*v-15010*x^2*z^2*v-F(101)/16*x*y*z^2*v+11110*y^2*z^2*v+7300*x*z^3*v+F(58)/95*y*z^3*v+10228*z^4*v+F(65)/57*x^3*u*v+F(9)/2*x^2*y*u*v+F(116)/27*x*y^2*u*v+7960*y^3*u*v-F(11)/8*x^2*z*u*v+F(59)/23*x*y*z*u*v+F(19)/26*y^2*z*u*v+14327*x*z^2*u*v-14671*y*z^2*u*v+F(126)/101*z^3*u*v+F(69)/83*x^2*u^2*v+14041*x*y*u^2*v-F(73)/90*y^2*u^2*v+F(11)/108*x*z*u^2*v-5492*y*z*u^2*v-3858*z^2*u^2*v-2840*x*u^3*v+15365*y*u^3*v+F(28)/113*z*u^3*v+149*u^4*v-F(94)/85*x^3*v^2-F(32)/123*x^2*y*v^2-1409*x*y^2*v^2-8233*y^3*v^2+851*x^2*z*v^2-7458*x*y*z*v^2-10713*y^2*z*v^2-F(18)/11*x*z^2*v^2-F(15)/47*y*z^2*v^2-F(32)/97*z^3*v^2+F(39)/68*x^2*u*v^2-10832*x*y*u*v^2+F(39)/59*y^2*u*v^2-12211*x*z*u*v^2+F(66)/71*y*z*u*v^2+F(121)/115*z^2*u*v^2-F(65)/101*x*u^2*v^2+F(82)/11*y*u^2*v^2-F(78)/103*z*u^2*v^2-10128*u^3*v^2-F(100)/33*x^2*v^3-F(29)/28*x*y*v^3+F(61)/29*y^2*v^3-5266*x*z*v^3+1953*y*z*v^3+5799*z^2*v^3+F(13)/77*x*u*v^3-2680*y*u*v^3-F(57)/98*z*u*v^3+F(56)/125*u^2*v^3+F(121)/42*x*v^4+F(47)/111*y*v^4+2590*z*v^4-F(60)/53*u*v^4+F(43)/61*v^5,8057*x^5+3453*x^4*y-F(71)/57*x^3*y^2+14017*x^2*y^3+F(89)/28*x*y^4-F(15)/44*y^5+10785*x^4*z-14385*x^3*y*z-F(112)/31*x^2*y^2*z+F(29)/42*x*y^3*z+162*y^4*z-12837*x^3*z^2+F(31)/33*x^2*y*z^2+F(105)/74*x*y^2*z^2+F(97)/59*y^3*z^2-F(23)/61*x^2*z^3-246*x*y*z^3+F(65)/93*y^2*z^3+F(1)/54*x*z^4+F(6)/25*y*z^4+96*x^4*u-F(41)/74*x^3*y*u+15120*x^2*y^2*u-1465*x*y^3*u-F(47)/42*y^4*u+F(11)/80*x^3*z*u-F(84)/55*x^2*y*z*u+12120*x*y^2*z*u+F(84)/65*y^3*z*u+496*x^2*z^2*u-F(29)/24*x*y*z^2*u+3753*y^2*z^2*u-2608*x*z^3*u+F(11)/123*y*z^3*u+7955*z^4*u-11390*x^3*u^2+F(37)/79*x^2*y*u^2+3230*x*y^2*u^2-F(72)/83*y^3*u^2-F(87)/74*x^2*z*u^2-11620*x*y*z*u^2+8276*y^2*z*u^2+608*x*z^2*u^2-7002*y*z^2*u^2-7897*z^3*u^2-F(65)/97*x^2*u^3-F(37)/3*x*y*u^3+F(7)/43*y^2*u^3+F(37)/20*x*z*u^3+F(97)/28*y*z*u^3+F(98)/19*z^2*u^3-F(47)/45*x*u^4-F(49)/109*y*u^4-F(53)/89*z*u^4-F(66)/125*u^5+7516*x^4*v-1097*x^3*y*v+15928*x^2*y^2*v-12128*x*y^3*v-12988*y^4*v-F(23)/7*x^3*z*v+5146*x^2*y*z*v+F(9)/28*x*y^2*z*v-1816*y^3*z*v+F(115)/42*x^2*z^2*v+11840*x*y*z^2*v+F(40)/119*y^2*z^2*v-F(66)/85*x*z^3*v-8269*y*z^3*v-F(59)/45*z^4*v-F(114)/37*x^3*u*v-F(126)/55*x^2*y*u*v-F(31)/24*x*y^2*u*v-F(19)/9*y^3*u*v+F(77)/3*x^2*z*u*v+14268*x*y*z*u*v-4133*y^2*z*u*v-11603*x*z^2*u*v-F(55)/83*y*z^2*u*v-F(5)/61*z^3*u*v-13069*x^2*u^2*v+4387*x*y*u^2*v+F(94)/77*y^2*u^2*v-F(111)/79*x*z*u^2*v-F(31)/61*y*z*u^2*v-F(26)/53*z^2*u^2*v-F(13)/103*x*u^3*v-F(27)/91*y*u^3*v-F(100)/57*z*u^3*v-F(104)/111*u^4*v+F(77)/15*x^3*v^2+F(100)/69*x^2*y*v^2-1664*x*y^2*v^2+14971*y^3*v^2-F(67)/45*x^2*z*v^2-F(55)/27*x*y*z*v^2+F(13)/98*y^2*z*v^2-958*x*z^2*v^2+F(3)/68*y*z^2*v^2+F(34)/65*z^3*v^2+10771*x^2*u*v^2+F(19)/106*x*y*u*v^2-F(84)/125*y^2*u*v^2+4723*x*z*u*v^2-10707*y*z*u*v^2+F(50)/103*z^2*u*v^2+1766*x*u^2*v^2+F(110)/3*y*u^2*v^2-F(58)/51*z*u^2*v^2-F(16)/105*u^3*v^2-F(70)/111*x^2*v^3+F(83)/103*x*y*v^3-F(104)/83*y^2*v^3-2394*x*z*v^3+F(91)/109*y*z*v^3-11779*z^2*v^3+F(111)/80*x*u*v^3+F(41)/109*y*u*v^3+F(71)/77*z*u*v^3+F(110)/31*u^2*v^3+8933*x*v^4-F(109)/113*y*v^4-F(56)/87*z*v^4-F(120)/7*u*v^4-F(107)/57*v^5,-F(38)/41*x^5-F(116)/67*x^4*y-6547*x^3*y^2+F(50)/71*x^2*y^3-F(13)/89*x*y^4-F(43)/79*x^4*z+1342*x^3*y*z+14316*x^2*y^2*z+737*x*y^3*z+F(15)/44*y^4*z+F(124)/79*x^3*z^2-F(67)/13*x^2*y*z^2+F(79)/104*x*y^2*z^2-162*y^3*z^2-F(109)/120*x^2*z^3+4540*x*y*z^3-F(97)/59*y^2*z^3+3772*x*z^4-F(65)/93*y*z^4-F(6)/25*z^5-13041*x^4*u-F(90)/89*x^3*y*u+F(107)/36*x^2*y^2*u+F(43)/46*x*y^3*u-11755*y^4*u+12017*x^3*z*u-F(97)/58*x^2*y*z*u+5592*x*y^2*z*u-11967*y^3*z*u+F(24)/79*x^2*z^2*u-F(63)/83*x*y*z^2*u-7367*y^2*z^2*u-F(62)/79*x*z^3*u+F(55)/64*y*z^3*u-F(67)/18*z^4*u-672*x^3*u^2+F(19)/77*x^2*y*u^2-F(100)/63*x*y^2*u^2-F(23)/56*y^3*u^2+F(61)/78*x^2*z*u^2+F(120)/67*x*y*z*u^2+F(48)/107*y^2*z*u^2+5855*x*z^2*u^2+9877*y*z^2*u^2+6940*z^3*u^2+12754*x^2*u^3+12989*x*y*u^3-F(123)/106*y^2*u^3+F(59)/88*x*z*u^3-3400*y*z*u^3-8976*z^2*u^3+F(43)/21*x*u^4-F(86)/111*y*u^4+F(2)/29*z*u^4+F(57)/31*u^5+F(79)/48*x^4*v+F(95)/56*x^3*y*v+3056*x^2*y^2*v-13681*x*y^3*v+13735*y^4*v-F(122)/91*x^3*z*v+F(79)/74*x^2*y*z*v+F(21)/47*x*y^2*z*v+12060*y^3*z*v-12314*x^2*z^2*v+F(64)/17*x*y*z^2*v-F(57)/52*y^2*z^2*v-F(57)/86*x*z^3*v+15436*y*z^3*v+9387*z^4*v+3345*x^3*u*v+F(2)/109*x^2*y*u*v-13978*x*y^2*u*v+5604*y^3*u*v+11645*x^2*z*u*v+2633*x*y*z*u*v+15505*y^2*z*u*v+F(10)/99*x*z^2*u*v+14409*y*z^2*u*v+8127*z^3*u*v+F(99)/104*x^2*u^2*v+14440*x*y*u^2*v+F(37)/43*y^2*u^2*v-9707*x*z*u^2*v+9171*y*z*u^2*v-656*z^2*u^2*v+3723*x*u^3*v+F(4)/19*y*u^3*v+F(109)/76*z*u^3*v-8740*u^4*v+2121*x^3*v^2+F(34)/11*x^2*y*v^2-F(89)/61*x*y^2*v^2-F(26)/21*y^3*v^2+F(111)/22*x^2*z*v^2+F(88)/41*x*y*z*v^2+F(119)/99*y^2*z*v^2+F(108)/77*x*z^2*v^2+9279*y*z^2*v^2-F(45)/116*z^3*v^2+F(71)/107*x^2*u*v^2+F(25)/76*x*y*u*v^2-F(112)/121*y^2*u*v^2-F(18)/71*x*z*u*v^2-F(84)/125*y*z*u*v^2+496*z^2*u*v^2+F(96)/113*x*u^2*v^2-964*y*u^2*v^2-F(73)/4*z*u^2*v^2+F(103)/95*u^3*v^2+9962*x^2*v^3+F(38)/39*x*y*v^3-15120*y^2*v^3+11747*x*z*v^3+F(2)/123*y*z*v^3+14829*z^2*v^3-6538*x*u*v^3-F(52)/63*y*u*v^3-F(83)/107*z*u*v^3+F(73)/37*u^2*v^3-4183*x*v^4+F(11)/103*y*v^4+F(83)/119*z*v^4+F(94)/31*u*v^4-F(19)/98*v^5,-12092*x^5-2411*x^4*y+3922*x^3*y^2+F(117)/41*x^2*y^3+F(95)/22*x*y^4+F(45)/19*x^4*z-5881*x^3*y*z+F(97)/80*x^2*y^2*z-4981*x*y^3*z-F(21)/67*y^4*z+9068*x^3*z^2+F(1)/11*x^2*y*z^2-F(109)/65*x*y^2*z^2+F(22)/117*y^3*z^2-7823*x^2*z^3-F(76)/75*x*y*z^3-F(34)/25*y^2*z^3-F(79)/5*x*z^4-F(60)/37*y*z^4-F(87)/100*z^5+F(58)/83*x^4*u-F(36)/61*x^3*y*u+F(79)/76*x^2*y^2*u+502*x*y^3*u+4988*y^4*u+F(43)/38*x^3*z*u-F(4)/107*x^2*y*z*u-F(48)/11*x*y^2*z*u+11685*y^3*z*u-15002*x^2*z^2*u-F(97)/17*x*y*z^2*u+F(93)/2*y^2*z^2*u-F(35)/71*x*z^3*u+F(43)/83*y*z^3*u-F(69)/26*z^4*u+F(119)/6*x^3*u^2-F(87)/14*x^2*y*u^2-F(29)/109*x*y^2*u^2-F(36)/97*y^3*u^2-F(116)/119*x^2*z*u^2+4610*x*y*z*u^2-F(33)/58*y^2*z*u^2+F(121)/25*x*z^2*u^2-F(125)/121*y*z^2*u^2-F(35)/121*z^3*u^2-8975*x^2*u^3+9014*x*y*u^3+14845*y^2*u^3-10277*x*z*u^3+F(75)/124*y*z*u^3-F(10)/83*z^2*u^3-F(73)/69*x*u^4+F(46)/85*y*u^4-4971*z*u^4-F(113)/4*u^5+9321*x^4*v+F(109)/19*x^3*y*v+F(11)/17*x^2*y^2*v+6672*x*y^3*v+F(16)/99*y^4*v+8092*x^3*z*v+1725*x^2*y*z*v+F(80)/41*x*y^2*z*v+2445*y^3*z*v+F(4)/99*x^2*z^2*v+F(69)/101*x*y*z^2*v+13182*y^2*z^2*v-10090*x*z^3*v+3817*y*z^3*v+F(106)/79*z^4*v-F(71)/89*x^3*u*v+F(110)/107*x^2*y*u*v-F(56)/19*x*y^2*u*v-708*y^3*u*v-F(108)/97*x^2*z*u*v-11889*x*y*z*u*v+9744*y^2*z*u*v-F(24)/121*x*z^2*u*v-10711*y*z^2*u*v+3182*z^3*u*v-F(22)/35*x^2*u^2*v+F(81)/44*x*y*u^2*v-F(40)/31*y^2*u^2*v-13494*x*z*u^2*v+F(47)/80*y*z*u^2*v+F(71)/52*z^2*u^2*v-6214*x*u^3*v-5144*y*u^3*v-F(115)/49*z*u^3*v-F(109)/19*u^4*v+8621*x^3*v^2-F(62)/79*x^2*y*v^2-F(102)/103*x*y^2*v^2-8174*y^3*v^2+13689*x^2*z*v^2+15544*x*y*z*v^2-F(107)/73*y^2*z*v^2+F(120)/73*x*z^2*v^2+F(19)/56*y*z^2*v^2-4544*z^3*v^2-824*x^2*u*v^2-F(2)/17*x*y*u*v^2+F(67)/78*y^2*u*v^2-F(54)/85*x*z*u*v^2+F(31)/51*y*z*u*v^2-F(59)/19*z^2*u*v^2+F(50)/7*x*u^2*v^2+5762*y*u^2*v^2+F(79)/64*z*u^2*v^2-3729*u^3*v^2+12212*x^2*v^3+1833*x*y*v^3+12543*y^2*v^3+11974*x*z*v^3-F(11)/17*y*z*v^3-F(75)/74*z^2*v^3+F(26)/3*x*u*v^3-F(37)/36*y*u*v^3-7683*z*u*v^3+14069*u^2*v^3+12261*x*v^4+12489*y*v^4+1657*z*v^4+10781*u*v^4-F(46)/3*v^5,12092*x^4*z+2411*x^3*y*z-3922*x^2*y^2*z-F(117)/41*x*y^3*z-F(95)/22*y^4*z+11164*x^3*z^2+6625*x^2*y*z^2+F(112)/75*x*y^2*z^2+1533*y^3*z^2+F(60)/101*x^2*z^3-173*x*y*z^3+15913*y^2*z^3+14954*x*z^4-5022*y*z^4+12391*z^5-F(38)/41*x^4*u-F(116)/67*x^3*y*u-6547*x^2*y^2*u+F(50)/71*x*y^3*u-F(13)/89*y^4*u-F(12)/55*x^3*z*u-10086*x^2*y*z*u+6782*x*y^2*z*u-F(111)/89*y^3*z*u-F(52)/85*x^2*z^2*u+9744*x*y*z^2*u+9553*y^2*z^2*u-15590*x*z^3*u+F(76)/13*y*z^3*u+5938*z^4*u+F(42)/11*x^3*u^2-F(25)/116*x^2*y*u^2+2554*x*y^2*u^2-12842*y^3*u^2-F(3)/88*x^2*z*u^2+F(102)/113*x*y*z*u^2-10298*y^2*z*u^2-F(32)/69*x*z^2*u^2+9709*y*z^2*u^2+8775*z^3*u^2-9937*x^2*u^3+5128*x*y*u^3-F(56)/75*y^2*u^3+12493*x*z*u^3-F(7)/39*y*z*u^3-F(41)/24*z^2*u^3-F(125)/49*x*u^4-15745*y*u^4-1005*z*u^4+F(16)/55*u^5-7566*x^3*z*v+14010*x^2*y*z*v+F(51)/23*x*y^2*z*v+F(37)/113*y^3*z*v+F(9)/14*x^2*z^2*v+2251*x*y*z^2*v+10076*y^2*z^2*v-F(106)/69*x*z^3*v-4060*y*z^3*v+3379*z^4*v-11120*x^3*u*v-9744*x^2*y*u*v+F(50)/17*x*y^2*u*v+7065*y^3*u*v-F(70)/37*x^2*z*u*v+10016*x*y*z*u*v+F(47)/86*y^2*z*u*v+6928*x*z^2*u*v-11190*y*z^2*u*v-11611*z^3*u*v+15042*x^2*u^2*v-F(52)/105*x*y*u^2*v+2185*y^2*u^2*v-F(76)/67*x*z*u^2*v-F(57)/104*y*z*u^2*v-7610*z^2*u^2*v-3912*x*u^3*v+F(1)/48*y*u^3*v-F(24)/109*z*u^3*v+3287*u^4*v+F(17)/96*x^2*z*v^2+F(29)/76*x*y*z*v^2+15768*y^2*z*v^2+8410*x*z^2*v^2-F(116)/41*y*z^2*v^2+F(89)/48*z^3*v^2+15119*x^2*u*v^2-11840*x*y*u*v^2-F(89)/43*y^2*u*v^2+10115*x*z*u*v^2+F(93)/101*y*z*u*v^2+F(33)/62*z^2*u*v^2+11864*x*u^2*v^2-14582*y*u^2*v^2+F(95)/53*z*u^2*v^2-8816*u^3*v^2+F(35)/43*x*z*v^3+F(75)/107*y*z*v^3+13515*z^2*v^3-F(38)/67*x*u*v^3+F(83)/71*y*u*v^3+F(113)/51*z*u*v^3-F(80)/111*u^2*v^3+F(49)/5*z*v^4+13850*u*v^4,14959*x^5-F(3)/13*x^4*y+F(97)/37*x^3*y^2+14586*x^2*y^3-F(71)/120*x*y^4-10158*x^4*z+F(59)/50*x^3*y*z-F(101)/96*x^2*y^2*z+F(118)/47*x*y^3*z-F(65)/83*y^4*z+F(49)/66*x^3*z^2-F(107)/18*x^2*y*z^2-F(2)/47*x*y^2*z^2+F(104)/63*y^3*z^2-F(121)/43*x^2*z^3+9552*x*y*z^3-F(113)/36*y^2*z^3+4699*x*z^4+F(55)/53*y*z^4+F(16)/109*z^5-3576*x^4*u-F(103)/52*x^3*y*u-8299*x^2*y^2*u+1585*x*y^3*u+3377*y^4*u+15336*x^3*z*u-F(119)/24*x^2*y*z*u-F(19)/68*x*y^2*z*u-F(65)/11*y^3*z*u-F(17)/117*x^2*z^2*u+788*x*y*z^2*u+F(73)/115*y^2*z^2*u-F(46)/93*x*z^3*u-F(108)/7*y*z^3*u+3774*z^4*u-14034*x^3*u^2-15420*x^2*y*u^2-5128*x*y^2*u^2+F(83)/55*y^3*u^2+F(17)/5*x^2*z*u^2+13098*x*y*z*u^2+F(96)/29*y^2*z*u^2-F(54)/115*x*z^2*u^2+F(4)/5*y*z^2*u^2+2747*z^3*u^2+F(56)/83*x^2*u^3-1545*x*y*u^3+14384*y^2*u^3+7787*x*z*u^3-F(69)/64*y*z*u^3-F(27)/20*z^2*u^3-F(17)/15*x*u^4+F(46)/13*y*u^4-F(44)/41*z*u^4-14876*u^5+10090*x^4*v+184*x^3*y*v+F(125)/44*x^2*y^2*v-13987*x*y^3*v+F(104)/123*y^4*v+F(24)/35*x^3*z*v+F(77)/95*x^2*y*z*v+162*x*y^2*z*v+F(31)/113*y^3*z*v-F(94)/51*x^2*z^2*v-14750*x*y*z^2*v-F(110)/101*y^2*z^2*v+3224*x*z^3*v-12389*y*z^3*v-F(77)/57*z^4*v-7340*x^3*u*v+F(48)/79*x^2*y*u*v-F(55)/71*x*y^2*u*v-F(65)/57*y^3*u*v+13981*x^2*z*u*v+F(76)/73*x*y*z*u*v-F(41)/22*y^2*z*u*v-10847*x*z^2*u*v+3230*y*z^2*u*v+F(37)/14*z^3*u*v+F(64)/47*x^2*u^2*v-F(89)/97*x*y*u^2*v-608*y^2*u^2*v-F(93)/112*x*z*u^2*v-22*y*z*u^2*v+7158*z^2*u^2*v-F(120)/31*x*u^3*v-13481*y*u^3*v-F(97)/37*z*u^3*v-F(113)/87*u^4*v-F(35)/19*x^3*v^2+F(79)/53*x^2*y*v^2-1037*x*y^2*v^2-F(29)/63*y^3*v^2+5990*x^2*z*v^2-3380*x*y*z*v^2-F(17)/126*y^2*z*v^2-F(40)/121*x*z^2*v^2-15041*y*z^2*v^2+3779*z^3*v^2+13583*x^2*u*v^2+F(11)/73*x*y*u*v^2+2762*y^2*u*v^2-F(16)/49*x*z*u*v^2-F(40)/93*y*z*u*v^2-F(37)/75*z^2*u*v^2+13312*x*u^2*v^2+4407*y*u^2*v^2+5449*z*u^2*v^2+3013*u^3*v^2+F(86)/73*x^2*v^3-F(37)/66*x*y*v^3-F(73)/15*y^2*v^3+F(75)/107*x*z*v^3-F(63)/31*y*z*v^3-F(97)/99*z^2*v^3+11234*x*u*v^3+F(37)/92*y*u*v^3+F(47)/27*z*u*v^3+F(121)/26*u^2*v^3-15690*x*v^4+F(73)/40*y*v^4-12281*z*v^4+6014*u*v^4-F(92)/83*v^5,-12092*x^4*y-2411*x^3*y^2+3922*x^2*y^3+F(117)/41*x*y^4+F(95)/22*y^5+14327*x^4*z+3699*x^3*y*z+F(18)/85*x^2*y^2*z-F(56)/53*x*y^3*z-F(31)/16*y^4*z+5371*x^3*z^2+F(73)/69*x^2*y*z^2+F(65)/34*x*y^2*z^2-F(59)/49*y^3*z^2+F(23)/37*x^2*z^3-9067*x*y*z^3+F(79)/37*y^2*z^3-F(17)/50*x*z^4+8412*y*z^4+F(110)/39*z^5-F(36)/125*x^4*u-F(20)/51*x^3*y*u-F(28)/123*x^2*y^2*u-4482*x*y^3*u+4217*y^4*u+F(8)/19*x^3*z*u+F(9)/77*x^2*y*z*u-10526*x*y^2*z*u+F(76)/79*y^3*z*u-12494*x^2*z^2*u-F(50)/13*x*y*z^2*u-F(98)/123*y^2*z^2*u-F(23)/19*x*z^3*u+1491*y*z^3*u-F(50)/109*z^4*u-F(67)/62*x^3*u^2+F(68)/47*x^2*y*u^2-F(38)/99*x*y^2*u^2-4891*y^3*u^2+F(68)/113*x^2*z*u^2-F(73)/35*x*y*z*u^2-2384*y^2*z*u^2+F(71)/109*x*z^2*u^2-F(68)/39*y*z^2*u^2-F(94)/125*z^3*u^2+F(120)/121*x^2*u^3-F(20)/39*x*y*u^3+435*y^2*u^3-F(23)/14*x*z*u^3-F(39)/97*y*z*u^3+F(75)/23*z^2*u^3+F(71)/30*x*u^4-8426*y*u^4+F(125)/7*z*u^4+F(11)/46*u^5-2334*x^4*v+9113*x^3*y*v-1060*x^2*y^2*v+12839*x*y^3*v+9876*y^4*v-8395*x^3*z*v-F(79)/82*x^2*y*z*v+F(89)/79*x*y^2*z*v+805*y^3*z*v-7473*x^2*z^2*v+F(80)/89*x*y*z^2*v+F(105)/52*y^2*z^2*v-F(54)/103*x*z^3*v-2102*y*z^3*v+F(19)/117*z^4*v-1326*x^3*u*v-8963*x^2*y*u*v-F(12)/13*x*y^2*u*v+11798*y^3*u*v-F(80)/7*x^2*z*u*v+519*x*y*z*u*v-F(58)/67*y^2*z*u*v+14572*x*z^2*u*v+8426*y*z^2*u*v-F(66)/47*z^3*u*v-574*x^2*u^2*v+12480*x*y*u^2*v-F(41)/89*y^2*u^2*v-F(111)/83*x*z*u^2*v-F(58)/37*y*z*u^2*v-15255*z^2*u^2*v+F(31)/97*x*u^3*v-F(89)/113*y*u^3*v+15475*z*u^3*v-3982*u^4*v+F(51)/8*x^3*v^2+9547*x^2*y*v^2-8060*x*y^2*v^2-195*y^3*v^2-F(5)/21*x^2*z*v^2-2411*x*y*z*v^2+F(65)/12*y^2*z*v^2-5410*x*z^2*v^2+3844*y*z^2*v^2+F(3)/61*z^3*v^2-F(62)/101*x^2*u*v^2+F(53)/19*x*y*u*v^2-6567*y^2*u*v^2+11302*x*z*u*v^2+F(123)/37*y*z*u*v^2-F(107)/118*z^2*u*v^2-F(29)/79*x*u^2*v^2-F(91)/40*y*u^2*v^2+F(40)/99*z*u^2*v^2-12274*u^3*v^2-F(47)/124*x^2*v^3+12752*x*y*v^3+11039*y^2*v^3-F(115)/43*x*z*v^3+F(44)/79*y*z*v^3-F(37)/28*z^2*v^3+5364*x*u*v^3+14510*y*u*v^3+3024*z*u*v^3-F(40)/3*u^2*v^3+F(71)/122*x*v^4+12024*y*v^4+F(10)/93*z*v^4-4183*u*v^4+11229*v^5,-F(51)/74*x^4*y-F(99)/61*x^3*y^2-10442*x^2*y^3+6943*x*y^4-15040*y^5+F(126)/97*x^4*z+F(101)/41*x^3*y*z+4018*x^2*y^2*z+F(98)/61*x*y^3*z+F(87)/34*y^4*z+F(75)/52*x^3*z^2-F(109)/70*x^2*y*z^2-F(81)/71*x*y^2*z^2+8231*y^3*z^2+F(78)/49*x^2*z^3+F(53)/69*x*y*z^3+F(70)/71*y^2*z^3+F(48)/23*x*z^4-6678*y*z^4+14913*z^5-F(32)/97*x^4*u+F(78)/55*x^3*y*u+11669*x^2*y^2*u-F(58)/75*x*y^3*u-F(61)/64*y^4*u+F(118)/35*x^3*z*u-F(125)/28*x^2*y*z*u-F(125)/79*x*y^2*z*u-F(49)/23*y^3*z*u-14966*x^2*z^2*u-F(34)/19*x*y*z^2*u-F(53)/126*y^2*z^2*u+13854*x*z^3*u-2177*y*z^3*u+9962*z^4*u+15886*x^3*u^2+F(47)/38*x^2*y*u^2+10992*x*y^2*u^2-F(111)/31*y^3*u^2-F(79)/21*x^2*z*u^2-14524*x*y*z*u^2-3442*y^2*z*u^2-F(45)/86*x*z^2*u^2+F(53)/3*y*z^2*u^2+8738*z^3*u^2+9062*x^2*u^3+3872*x*y*u^3+F(56)/61*y^2*u^3-6255*x*z*u^3-F(104)/107*y*z*u^3+9858*z^2*u^3+7891*x*u^4-11114*y*u^4+F(67)/37*z*u^4-15307*u^5-F(108)/7*x^4*v+11586*x^3*y*v+F(19)/4*x^2*y^2*v+14404*x*y^3*v-F(99)/116*y^4*v+F(124)/33*x^3*z*v-F(73)/17*x^2*y*z*v+F(77)/115*x*y^2*z*v-4537*y^3*z*v+9949*x^2*z^2*v+F(101)/111*x*y*z^2*v+6634*y^2*z^2*v-11818*x*z^3*v+F(23)/52*y*z^3*v-63*z^4*v+F(7)/6*x^3*u*v-13686*x^2*y*u*v+15879*x*y^2*u*v+4731*y^3*u*v+F(107)/82*x^2*z*u*v+F(104)/119*x*y*z*u*v-F(56)/67*y^2*z*u*v-F(21)/5*x*z^2*u*v-F(24)/41*y*z^2*u*v-F(125)/108*z^3*u*v+F(58)/77*x^2*u^2*v+F(79)/49*x*y*u^2*v+F(82)/19*y^2*u^2*v-F(82)/117*x*z*u^2*v-F(8)/61*y*z*u^2*v+210*z^2*u^2*v+F(27)/25*x*u^3*v+F(71)/111*y*u^3*v-14500*z*u^3*v-F(15)/2*u^4*v-F(113)/108*x^3*v^2+F(40)/31*x^2*y*v^2+F(89)/69*x*y^2*v^2-8271*y^3*v^2+F(111)/20*x^2*z*v^2+F(1)/31*x*y*z*v^2-F(51)/101*y^2*z*v^2+3967*x*z^2*v^2+F(79)/32*y*z^2*v^2-6893*z^3*v^2-1060*x^2*u*v^2+F(38)/5*x*y*u*v^2+F(21)/20*y^2*u*v^2-F(79)/34*x*z*u*v^2+F(15)/118*y*z*u*v^2+F(25)/106*z^2*u*v^2+F(97)/20*x*u^2*v^2+F(6)/67*y*u^2*v^2+F(113)/120*z*u^2*v^2+F(13)/3*u^3*v^2+1800*x^2*v^3-F(97)/15*x*y*v^3-12712*y^2*v^3+F(49)/118*x*z*v^3+F(5)/63*y*z*v^3+234*z^2*v^3-F(97)/123*x*u*v^3+F(13)/88*y*u*v^3+F(8)/103*z*u*v^3-F(38)/97*u^2*v^3-15634*x*v^4+F(18)/25*y*v^4-7517*z*v^4+F(115)/103*u*v^4-F(56)/23*v^5,-F(123)/61*x^5+F(4)/87*x^4*y-15558*x^3*y^2+F(78)/5*x^2*y^3-F(31)/83*x*y^4+9742*x^4*z+F(23)/125*x^3*y*z+1865*x^2*y^2*z+F(56)/125*x*y^3*z-F(4)/33*y^4*z-F(54)/71*x^3*z^2-F(113)/84*x^2*y*z^2+F(37)/75*x*y^2*z^2+F(6)/23*y^3*z^2-F(28)/123*x^2*z^3+F(31)/112*x*y*z^3-15736*y^2*z^3-F(106)/111*x*z^4+F(46)/109*y*z^4-F(29)/10*z^5-F(28)/117*x^4*u+15892*x^3*y*u+8096*x^2*y^2*u-F(91)/29*x*y^3*u+F(57)/92*y^4*u+F(96)/31*x^3*z*u+166*x^2*y*z*u+2654*x*y^2*z*u-11662*y^3*z*u+1565*x^2*z^2*u+9017*x*y*z^2*u+F(121)/4*y^2*z^2*u-F(29)/115*x*z^3*u+9828*y*z^3*u-15873*z^4*u-5354*x^3*u^2+F(118)/19*x^2*y*u^2+F(88)/95*x*y^2*u^2+F(68)/19*y^3*u^2-F(7)/110*x^2*z*u^2-F(109)/37*x*y*z*u^2-F(45)/74*y^2*z*u^2+12579*x*z^2*u^2-6659*y*z^2*u^2+10257*z^3*u^2+7229*x^2*u^3+F(39)/67*x*y*u^3-F(108)/19*y^2*u^3-F(58)/9*x*z*u^3-F(29)/81*y*z*u^3-1461*z^2*u^3+F(23)/43*x*u^4-F(8)/19*y*u^4-44*z*u^4+F(100)/109*u^5+F(121)/51*x^4*v-F(20)/39*x^3*y*v-F(85)/121*x^2*y^2*v+F(73)/61*x*y^3*v+F(103)/50*y^4*v+7334*x^3*z*v+F(52)/125*x^2*y*z*v-F(14)/55*x*y^2*z*v+F(106)/103*y^3*z*v-4079*x^2*z^2*v+F(109)/57*x*y*z^2*v+F(123)/28*y^2*z^2*v-F(21)/83*x*z^3*v+F(3)/31*y*z^3*v-9826*z^4*v+F(89)/77*x^3*u*v-F(79)/35*x^2*y*u*v+8767*x*y^2*u*v+F(83)/11*y^3*u*v+F(113)/36*x^2*z*u*v-F(57)/68*x*y*z*u*v+1675*y^2*z*u*v+10873*x*z^2*u*v+F(84)/11*y*z^2*u*v+F(89)/110*z^3*u*v-5834*x^2*u^2*v-F(121)/97*x*y*u^2*v+F(49)/66*y^2*u^2*v-F(103)/95*x*z*u^2*v+14863*y*z*u^2*v+F(119)/94*z^2*u^2*v+F(44)/35*x*u^3*v-15429*y*u^3*v-9302*z*u^3*v-F(73)/12*u^4*v-F(77)/86*x^3*v^2-F(121)/46*x^2*y*v^2+F(48)/13*x*y^2*v^2+F(68)/115*y^3*v^2+F(58)/63*x^2*z*v^2+F(58)/117*x*y*z*v^2-F(16)/7*y^2*z*v^2-F(76)/39*x*z^2*v^2+F(41)/103*y*z^2*v^2-8042*z^3*v^2-F(59)/5*x^2*u*v^2+3277*x*y*u*v^2-F(120)/107*y^2*u*v^2-F(5)/84*x*z*u*v^2-F(30)/103*y*z*u*v^2+F(31)/71*z^2*u*v^2-1593*x*u^2*v^2-4668*y*u^2*v^2+5749*z*u^2*v^2+F(38)/43*u^3*v^2+F(26)/17*x^2*v^3+1894*x*y*v^3+F(62)/17*y^2*v^3-4851*x*z*v^3-10816*y*z*v^3+F(62)/55*z^2*v^3+1366*x*u*v^3-F(70)/11*y*u*v^3+F(90)/43*z*u*v^3-F(115)/57*u^2*v^3-10886*x*v^4+1987*y*v^4-F(78)/25*z*v^4+F(110)/29*u*v^4+4694*v^5,F(47)/12*x^5+F(64)/29*x^4*y-F(113)/70*x^3*y^2+15185*x^2*y^3-F(52)/81*x*y^4+F(67)/28*x^4*z-4220*x^3*y*z-9239*x^2*y^2*z+F(10)/117*x*y^3*z-5609*y^4*z-4439*x^3*z^2-F(61)/91*x^2*y*z^2+15282*x*y^2*z^2+8165*y^3*z^2-F(19)/109*x^2*z^3+F(123)/94*x*y*z^3+F(62)/49*y^2*z^3+F(77)/2*x*z^4-F(1)/37*y*z^4-1202*z^5-14940*x^4*u+13396*x^3*y*u-F(6)/5*x^2*y^2*u-F(91)/50*x*y^3*u+F(115)/118*y^4*u-455*x^3*z*u-2755*x^2*y*z*u+14353*x*y^2*z*u+F(11)/53*y^3*z*u+F(115)/57*x^2*z^2*u+F(86)/49*x*y*z^2*u+43*y^2*z^2*u+5247*x*z^3*u+4438*y*z^3*u+F(109)/16*z^4*u-3113*x^3*u^2-15629*x^2*y*u^2-15664*x*y^2*u^2-F(115)/57*y^3*u^2+F(3)/103*x^2*z*u^2-3723*x*y*z*u^2+F(89)/17*y^2*z*u^2-F(59)/56*x*z^2*u^2+F(19)/123*y*z^2*u^2+F(43)/14*z^3*u^2+F(103)/14*x^2*u^3+F(106)/31*x*y*u^3-13591*y^2*u^3-F(71)/43*x*z*u^3+F(74)/119*y*z*u^3-F(36)/113*z^2*u^3-F(73)/96*x*u^4-F(31)/120*y*u^4-F(105)/76*z*u^4-F(37)/34*u^5-F(58)/23*x^4*v-11111*x^3*y*v+5888*x^2*y^2*v+14867*x*y^3*v+8608*y^4*v-2829*x^3*z*v-F(31)/120*x^2*y*z*v+6306*x*y^2*z*v+5913*y^3*z*v+F(35)/6*x^2*z^2*v+F(75)/104*x*y*z^2*v+F(33)/41*y^2*z^2*v-15317*x*z^3*v-F(31)/49*y*z^3*v-7300*z^4*v+2214*x^3*u*v-F(91)/46*x^2*y*u*v+10949*x*y^2*u*v-F(3)/82*y^3*u*v-F(16)/27*x^2*z*u*v-14175*x*y*z*u*v-F(13)/19*y^2*z*u*v-F(21)/65*x*z^2*u*v+148*y*z^2*u*v+14615*z^3*u*v-F(43)/6*x^2*u^2*v-4133*x*y*u^2*v+F(33)/49*y^2*u^2*v-1299*x*z*u^2*v+F(96)/53*y*z*u^2*v-12667*z^2*u^2*v-F(86)/113*x*u^3*v+10437*y*u^3*v-F(53)/13*z*u^3*v-15678*u^4*v-F(53)/121*x^3*v^2-F(20)/49*x^2*y*v^2+3813*x*y^2*v^2+10006*y^3*v^2+15644*x^2*z*v^2-F(99)/7*x*y*z*v^2-F(42)/13*y^2*z*v^2-F(124)/57*x*z^2*v^2+F(73)/70*y*z^2*v^2+F(110)/107*z^3*v^2-F(97)/22*x^2*u*v^2-F(125)/116*x*y*u*v^2+F(92)/93*y^2*u*v^2-12120*x*z*u*v^2+143*y*z*u*v^2-15344*z^2*u*v^2+13225*x*u^2*v^2+F(17)/104*y*u^2*v^2+1212*z*u^2*v^2-7685*u^3*v^2-F(81)/4*x^2*v^3+10328*x*y*v^3-F(21)/19*y^2*v^3+13407*x*z*v^3-F(1)/118*y*z*v^3+F(59)/88*z^2*v^3+965*x*u*v^3-F(93)/43*y*u*v^3-F(76)/85*z*u*v^3-F(81)/86*u^2*v^3-6949*x*v^4+9205*y*v^4-7777*z*v^4+15020*u*v^4+8845*v^5,3079*x^5-F(28)/11*x^4*y+F(117)/88*x^3*y^2-F(89)/115*x^2*y^3+F(40)/13*x*y^4+13407*y^5+3942*x^4*z+F(30)/121*x^3*y*z-2922*x^2*y^2*z-F(71)/74*x*y^3*z-F(103)/114*y^4*z-F(89)/59*x^3*z^2+12255*x^2*y*z^2+F(44)/109*x*y^2*z^2+13778*y^3*z^2+F(103)/110*x^2*z^3-F(73)/95*x*y*z^3+8049*y^2*z^3+10042*x*z^4-F(101)/35*y*z^4-1636*z^5-F(53)/27*x^4*u+130*x^3*y*u-5989*x^2*y^2*u+F(67)/109*x*y^3*u+4601*y^4*u-12943*x^3*z*u-F(86)/75*x^2*y*z*u+5837*x*y^2*z*u+F(107)/104*y^3*z*u+2396*x^2*z^2*u-F(25)/58*x*y*z^2*u+F(29)/110*y^2*z^2*u+10526*x*z^3*u-F(7)/45*y*z^3*u-9105*z^4*u-F(64)/35*x^3*u^2+F(64)/105*x^2*y*u^2+F(113)/24*x*y^2*u^2-5955*y^3*u^2+F(20)/91*x^2*z*u^2+F(119)/78*x*y*z*u^2+F(83)/8*y^2*z*u^2+F(125)/103*x*z^2*u^2+F(83)/101*y*z^2*u^2+14895*z^3*u^2-F(118)/93*x^2*u^3-F(19)/23*x*y*u^3+F(100)/53*y^2*u^3-F(92)/59*x*z*u^3+F(34)/89*y*z*u^3+9417*z^2*u^3-6626*x*u^4-F(36)/65*y*u^4+F(119)/82*z*u^4+8761*u^5-15340*x^4*v-F(89)/115*x^3*y*v+F(89)/85*x^2*y^2*v+F(33)/74*x*y^3*v-8088*y^4*v+F(29)/16*x^3*z*v-6581*x^2*y*z*v-F(88)/85*x*y^2*z*v-F(12)/11*y^3*z*v-F(58)/19*x^2*z^2*v+12219*x*y*z^2*v-F(100)/119*y^2*z^2*v+F(116)/53*x*z^3*v-F(67)/94*y*z^3*v+F(13)/5*z^4*v+236*x^3*u*v+F(107)/96*x^2*y*u*v+F(115)/113*x*y^2*u*v-7860*y^3*u*v+15064*x^2*z*u*v+F(29)/59*x*y*z*u*v-F(7)/103*y^2*z*u*v-F(71)/66*x*z^2*u*v-F(103)/79*y*z^2*u*v-F(83)/4*z^3*u*v-F(77)/23*x^2*u^2*v+F(7)/37*x*y*u^2*v+F(106)/103*y^2*u^2*v+F(3)/50*x*z*u^2*v+F(125)/17*y*z*u^2*v-15620*z^2*u^2*v+6424*x*u^3*v-12832*y*u^3*v+F(58)/53*z*u^3*v-3893*u^4*v+F(25)/124*x^3*v^2-F(99)/61*x^2*y*v^2-F(61)/10*x*y^2*v^2-F(103)/39*y^3*v^2-9764*x^2*z*v^2+F(123)/23*x*y*z*v^2-F(83)/126*y^2*z*v^2+13541*x*z^2*v^2+F(73)/106*y*z^2*v^2+1172*z^3*v^2-14333*x^2*u*v^2+F(7)/72*x*y*u*v^2-F(41)/25*y^2*u*v^2-F(42)/31*x*z*u*v^2+F(7)/12*y*z*u*v^2-9744*z^2*u*v^2-15518*x*u^2*v^2-F(121)/5*y*u^2*v^2-F(7)/115*z*u^2*v^2+F(75)/14*u^3*v^2-F(9)/67*x^2*v^3+10248*x*y*v^3+5738*y^2*v^3+F(71)/13*x*z*v^3-F(43)/24*y*z*v^3+F(9)/94*z^2*v^3-F(113)/16*x*u*v^3-F(27)/25*y*u*v^3-F(50)/11*z*u*v^3+F(41)/49*u^2*v^3+10896*x*v^4-F(46)/43*y*v^4-F(58)/15*z*v^4-5671*u*v^4-F(61)/9*v^5,6254*x^5-F(95)/34*x^4*y-7133*x^3*y^2+F(111)/38*x^2*y^3-F(43)/19*x*y^4-F(104)/111*y^5+F(92)/51*x^4*z+11826*x^3*y*z+3453*x^2*y^2*z+5703*x*y^3*z+13099*y^4*z-F(73)/79*x^3*z^2-F(69)/11*x^2*y*z^2-15727*x*y^2*z^2+F(98)/115*y^3*z^2-10548*x^2*z^3-14452*x*y*z^3-5138*y^2*z^3-138*x*z^4+F(46)/109*y*z^4-5419*z^5+F(73)/6*x^4*u-F(63)/5*x^3*y*u+F(31)/60*x^2*y^2*u-11963*x*y^3*u+6161*y^4*u-2311*x^3*z*u+F(104)/105*x^2*y*z*u+10873*x*y^2*z*u+8165*y^3*z*u-F(17)/83*x^2*z^2*u-F(18)/91*x*y*z^2*u+13544*y^2*z^2*u+F(36)/121*x*z^3*u-8704*y*z^3*u+F(46)/7*z^4*u+11392*x^3*u^2+133*x^2*y*u^2+7460*x*y^2*u^2+8831*y^3*u^2+144*x^2*z*u^2-F(79)/47*x*y*z*u^2-F(70)/113*y^2*z*u^2+7369*x*z^2*u^2-11234*y*z^2*u^2-F(19)/52*z^3*u^2+9830*x^2*u^3+F(13)/41*x*y*u^3+1985*y^2*u^3-F(22)/15*x*z*u^3-5449*y*z*u^3+5967*z^2*u^3+9321*x*u^4+13078*y*u^4-F(97)/3*z*u^4+4079*u^5-5700*x^4*v+5013*x^3*y*v-10774*x^2*y^2*v-13183*x*y^3*v-F(20)/109*y^4*v-F(106)/65*x^3*z*v+F(49)/117*x^2*y*z*v-F(67)/17*x*y^2*z*v-13917*y^3*z*v+9457*x^2*z^2*v+F(95)/67*x*y*z^2*v-F(108)/65*y^2*z^2*v+F(65)/77*x*z^3*v-11235*y*z^3*v+F(125)/104*z^4*v+F(17)/117*x^3*u*v+6355*x^2*y*u*v+F(5)/36*x*y^2*u*v-F(117)/31*y^3*u*v-1225*x^2*z*u*v+F(117)/53*x*y*z*u*v-F(7)/26*y^2*z*u*v+F(87)/46*x*z^2*u*v+F(33)/76*y*z^2*u*v-F(82)/73*z^3*u*v-F(34)/39*x^2*u^2*v+F(9)/22*x*y*u^2*v+F(83)/11*y^2*u^2*v+13740*x*z*u^2*v-F(108)/11*y*z*u^2*v-9627*z^2*u^2*v-F(32)/95*x*u^3*v-7966*y*u^3*v-12400*z*u^3*v-F(111)/91*u^4*v+F(47)/11*x^3*v^2-F(10)/27*x^2*y*v^2-5257*x*y^2*v^2-F(85)/19*y^3*v^2-F(3)/62*x^2*z*v^2+12660*x*y*z*v^2-F(37)/110*y^2*z*v^2+10980*x*z^2*v^2+F(73)/29*y*z^2*v^2+F(87)/92*z^3*v^2-F(73)/90*x^2*u*v^2-F(77)/25*x*y*u*v^2+F(66)/31*y^2*u*v^2-F(23)/96*x*z*u*v^2-3169*y*z*u*v^2+F(125)/71*z^2*u*v^2+F(9)/11*x*u^2*v^2+6550*y*u^2*v^2-15163*z*u^2*v^2+10810*u^3*v^2+F(96)/79*x^2*v^3+196*x*y*v^3-F(95)/94*y^2*v^3+15146*x*z*v^3+14695*y*z*v^3+F(97)/80*z^2*v^3+13047*x*u*v^3-F(44)/79*y*u*v^3-F(43)/41*z*u*v^3+F(40)/57*u^2*v^3-8369*x*v^4+F(12)/5*y*v^4+F(77)/100*z*v^4-F(53)/79*u*v^4-F(53)/86*v^5,-F(122)/49*x^5-3959*x^4*y-7277*x^3*y^2+F(41)/93*x^2*y^3+F(39)/59*x*y^4-F(91)/68*x^4*z+F(37)/11*x^3*y*z+11561*x^2*y^2*z-5297*x*y^3*z+F(53)/14*y^4*z-F(17)/122*x^3*z^2+5470*x^2*y*z^2+14889*x*y^2*z^2-F(123)/49*y^3*z^2+131*x^2*z^3+F(13)/43*x*y*z^3-F(82)/37*y^2*z^3-F(106)/91*x*z^4+11066*y*z^4-F(72)/13*z^5+15904*x^4*u+F(44)/47*x^3*y*u+10450*x^2*y^2*u+F(86)/41*x*y^3*u-F(40)/41*y^4*u+F(17)/92*x^3*z*u-F(81)/29*x^2*y*z*u-F(40)/101*x*y^2*z*u+F(103)/59*y^3*z*u-F(29)/72*x^2*z^2*u+F(17)/123*x*y*z^2*u+F(38)/3*y^2*z^2*u+3358*x*z^3*u-F(69)/97*y*z^3*u+7627*z^4*u+15451*x^3*u^2+5371*x^2*y*u^2-9009*x*y^2*u^2-F(62)/35*y^3*u^2-F(44)/75*x^2*z*u^2+13188*x*y*z*u^2+F(50)/51*y^2*z*u^2+F(53)/93*x*z^2*u^2-6796*y*z^2*u^2-F(13)/6*z^3*u^2+F(39)/62*x^2*u^3-F(22)/71*x*y*u^3-F(32)/97*y^2*u^3+F(43)/14*x*z*u^3-F(9)/37*y*z*u^3-F(48)/125*z^2*u^3+F(100)/103*x*u^4+13286*y*u^4+6246*z*u^4-F(26)/107*u^5-15634*x^4*v+F(112)/9*x^3*y*v+F(54)/109*x^2*y^2*v+F(23)/93*x*y^3*v+F(101)/13*y^4*v+F(79)/84*x^3*z*v+5290*x^2*y*z*v-F(52)/29*x*y^2*z*v-F(109)/111*y^3*z*v+F(107)/88*x^2*z^2*v+10186*x*y*z^2*v+F(122)/97*y^2*z^2*v-8370*x*z^3*v-F(108)/47*y*z^3*v+6311*z^4*v-1126*x^3*u*v+F(51)/10*x^2*y*u*v+9201*x*y^2*u*v-F(111)/119*y^3*u*v+F(105)/58*x^2*z*u*v-F(74)/23*x*y*z*u*v-130*y^2*z*u*v-F(99)/28*x*z^2*u*v+F(24)/65*y*z^2*u*v-F(111)/49*z^3*u*v-4961*x^2*u^2*v-F(23)/42*x*y*u^2*v-F(32)/101*y^2*u^2*v+F(48)/83*x*z*u^2*v-13001*y*z*u^2*v-F(31)/86*z^2*u^2*v+F(85)/91*x*u^3*v+4019*y*u^3*v-F(10)/43*z*u^3*v+F(11)/105*u^4*v+9790*x^3*v^2+F(73)/78*x^2*y*v^2+F(63)/50*x*y^2*v^2+F(35)/2*y^3*v^2-F(52)/119*x^2*z*v^2+13680*x*y*z*v^2-F(46)/55*y^2*z*v^2+F(89)/12*x*z^2*v^2-F(23)/82*y*z^2*v^2-F(107)/85*z^3*v^2-F(20)/79*x^2*u*v^2-F(85)/111*x*y*u*v^2+F(31)/95*y^2*u*v^2-13*x*z*u*v^2+5661*y*z*u*v^2+9399*z^2*u*v^2+F(73)/75*x*u^2*v^2+4782*y*u^2*v^2-5440*z*u^2*v^2+F(45)/64*u^3*v^2-1484*x^2*v^3+F(1)/54*x*y*v^3+F(43)/115*y^2*v^3+139*x*z*v^3-F(47)/85*y*z*v^3-F(103)/43*z^2*v^3+F(48)/97*x*u*v^3+F(67)/15*y*u*v^3-F(69)/95*z*u*v^3-F(67)/90*u^2*v^3+6540*x*v^4+6276*y*v^4-9756*z*v^4-F(10)/121*u*v^4+F(118)/63*v^5,x^5+F(13)/23*x^4*y-10277*x^3*y^2-F(81)/107*x^2*y^3+F(18)/97*x*y^4+193*x^4*z-206*x^3*y*z+1473*x^2*y^2*z-3907*x*y^3*z+9620*y^4*z-11603*x^3*z^2+7430*x^2*y*z^2-3745*x*y^2*z^2-F(23)/55*y^3*z^2-F(31)/13*x^2*z^3-F(115)/51*x*y*z^3+F(54)/5*y^2*z^3+F(107)/6*x*z^4+8432*y*z^4-F(23)/71*z^5-F(21)/79*x^4*u-7130*x^3*y*u-F(55)/46*x^2*y^2*u+F(61)/113*x*y^3*u+F(11)/95*y^4*u-F(31)/125*x^3*z*u+F(47)/80*x^2*y*z*u+F(12)/79*x*y^2*z*u-F(51)/112*y^3*z*u-F(28)/75*x^2*z^2*u+1962*x*y*z^2*u-12942*y^2*z^2*u-F(93)/37*x*z^3*u-F(7)/9*y*z^3*u+F(81)/7*z^4*u-F(79)/62*x^3*u^2+F(98)/29*x^2*y*u^2-F(113)/36*x*y^2*u^2-F(59)/51*y^3*u^2-5931*x^2*z*u^2+F(8)/29*x*y*z*u^2-F(22)/117*y^2*z*u^2-12146*x*z^2*u^2+12607*y*z^2*u^2-8748*z^3*u^2-11878*x^2*u^3-6*x*y*u^3-11798*y^2*u^3+F(97)/111*x*z*u^3+F(122)/3*y*z*u^3+10228*z^2*u^3-F(99)/67*x*u^4-8058*y*u^4+F(116)/43*z*u^4-6801*u^5-7565*x^4*v-F(21)/58*x^3*y*v-F(23)/90*x^2*y^2*v-F(111)/49*x*y^3*v+F(62)/3*y^4*v-F(118)/109*x^3*z*v-F(91)/27*x^2*y*z*v-2256*x*y^2*z*v-6909*y^3*z*v+F(47)/126*x^2*z^2*v+397*x*y*z^2*v-F(65)/103*y^2*z^2*v+F(106)/43*x*z^3*v-F(43)/9*y*z^3*v+4502*z^4*v+5383*x^3*u*v-F(19)/48*x^2*y*u*v+F(91)/92*x*y^2*u*v-15545*y^3*u*v+11204*x^2*z*u*v+7609*x*y*z*u*v-F(125)/44*y^2*z*u*v+F(15)/7*x*z^2*u*v-4157*y*z^2*u*v+F(27)/10*z^3*u*v+F(70)/67*x^2*u^2*v+F(5)/121*x*y*u^2*v-12337*y^2*u^2*v-12417*x*z*u^2*v+F(46)/113*y*z*u^2*v+F(27)/89*z^2*u^2*v-9419*x*u^3*v-F(59)/47*y*u^3*v+F(52)/97*z*u^3*v+F(71)/56*u^4*v+F(56)/123*x^3*v^2+F(122)/73*x^2*y*v^2+F(100)/103*x*y^2*v^2+F(43)/8*y^3*v^2+F(99)/98*x^2*z*v^2-14242*x*y*z*v^2-F(93)/38*y^2*z*v^2+F(61)/109*x*z^2*v^2-F(21)/125*y*z^2*v^2+F(80)/23*z^3*v^2+11603*x^2*u*v^2-1734*x*y*u*v^2-F(55)/73*y^2*u*v^2+F(5)/89*x*z*u*v^2+F(77)/100*y*z*u*v^2-F(71)/38*z^2*u*v^2+F(41)/47*x*u^2*v^2+981*y*u^2*v^2+F(11)/41*z*u^2*v^2-8983*u^3*v^2+F(98)/59*x^2*v^3+F(105)/97*x*y*v^3-6372*y^2*v^3+12829*x*z*v^3+F(79)/96*y*z*v^3+F(110)/57*z^2*v^3+8495*x*u*v^3-11228*y*u*v^3-2396*z*u*v^3-F(101)/9*u^2*v^3-F(71)/39*x*v^4-11746*y*v^4-F(43)/96*z*v^4+F(64)/55*u*v^4-769*v^5,F(122)/49*x^4*y+3959*x^3*y^2+7277*x^2*y^3-F(41)/93*x*y^4-F(39)/59*y^5+F(32)/97*x^4*z+F(97)/96*x^3*y*z-F(43)/116*x^2*y^2*z+F(19)/30*x*y^3*z+F(18)/17*y^4*z+F(39)/122*x^3*z^2-F(98)/99*x^2*y*z^2+F(23)/65*x*y^2*z^2-F(50)/103*y^3*z^2-11445*x^2*z^3+F(111)/119*x*y*z^3-F(82)/67*y^2*z^3+F(49)/15*x*z^4+F(59)/74*y*z^4+F(87)/124*z^5+F(97)/113*x^3*y*u+F(59)/113*x^2*y^2*u-F(17)/16*x*y^3*u-1892*y^4*u-15886*x^3*z*u+F(75)/124*x^2*y*z*u-F(13)/108*x*y^2*z*u+14203*y^3*z*u+13871*x^2*z^2*u+24*x*y*z^2*u+1483*y^2*z^2*u-F(34)/91*x*z^3*u+F(51)/29*y*z^3*u+F(88)/107*z^4*u+6268*x^2*y*u^2+13776*x*y^2*u^2+1935*y^3*u^2-9062*x^2*z*u^2-4534*x*y*z*u^2+9553*y^2*z*u^2-8429*x*z^2*u^2-11141*y*z^2*u^2-F(89)/88*z^3*u^2-F(79)/50*x*y*u^3-F(121)/15*y^2*u^3-7891*x*z*u^3-F(27)/94*y*z*u^3+F(96)/71*z^2*u^3+F(27)/53*y*u^4+15307*z*u^4-775*x^4*v-2648*x^3*y*v-2095*x^2*y^2*v+7594*x*y^3*v+12989*y^4*v-F(17)/29*x^3*z*v+11801*x^2*y*z*v+F(32)/45*x*y^2*z*v-F(55)/73*y^3*z*v+9756*x^2*z^2*v+6886*x*y*z^2*v+5939*y^2*z^2*v+F(39)/23*x*z^3*v+2632*y*z^3*v+F(77)/6*z^4*v+F(36)/121*x^3*u*v-F(76)/55*x^2*y*u*v+5296*x*y^2*u*v-F(32)/47*y^3*u*v-F(125)/84*x^2*z*u*v-F(30)/67*x*y*z*u*v-14914*y^2*z*u*v+F(73)/23*x*z^2*u*v+F(123)/101*y*z^2*u*v+F(49)/43*z^3*u*v-F(76)/71*x^2*u^2*v+F(79)/50*x*y*u^2*v+F(38)/7*y^2*u^2*v-5262*x*z*u^2*v-5211*y*z*u^2*v+F(2)/95*z^2*u^2*v-3570*x*u^3*v-9280*y*u^3*v+F(32)/91*z*u^3*v+1089*u^4*v-F(73)/33*x^3*v^2-3931*x^2*y*v^2-F(21)/113*x*y^2*v^2+191*y^3*v^2-F(93)/2*x^2*z*v^2-F(75)/47*x*y*z*v^2+F(16)/115*y^2*z*v^2+F(120)/23*x*z^2*v^2+F(23)/114*y*z^2*v^2-F(115)/108*z^3*v^2-F(103)/12*x^2*u*v^2+F(20)/107*x*y*u*v^2+F(28)/11*y^2*u*v^2+F(23)/42*x*z*u*v^2-15290*y*z*u*v^2+3651*z^2*u*v^2-9566*x*u^2*v^2-F(113)/68*y*u^2*v^2+7705*z*u^2*v^2-12261*u^3*v^2+9882*x^2*v^3-8520*x*y*v^3+14858*y^2*v^3+200*x*z*v^3+F(14)/73*y*z*v^3+F(45)/53*z^2*v^3-F(4)/113*x*u*v^3-11327*y*u*v^3+F(59)/51*z*u*v^3-13869*u^2*v^3-F(33)/29*x*v^4-13026*y*v^4+4236*z*v^4+10782*u*v^4+2817*v^5,-F(123)/61*x^4*y+F(4)/87*x^3*y^2-15558*x^2*y^3+F(78)/5*x*y^4-F(31)/83*y^5+10267*x^4*z-F(29)/21*x^3*y*z+2197*x^2*y^2*z-F(57)/98*x*y^3*z+F(121)/31*y^4*z-1310*x^3*z^2+9386*x^2*y*z^2-F(37)/50*x*y^2*z^2-1767*y^3*z^2-489*x^2*z^3-F(117)/31*x*y*z^3+13576*y^2*z^3-F(50)/9*x*z^4-F(47)/77*y*z^4+F(53)/89*z^5-F(57)/11*x^4*u-8374*x^3*y*u+F(104)/103*x^2*y^2*u+F(29)/12*x*y^3*u+7368*y^4*u+F(102)/37*x^3*z*u+10508*x^2*y*z*u+28*x*y^2*z*u-14484*y^3*z*u-240*x^2*z^2*u-F(115)/114*x*y*z^2*u+F(97)/74*y^2*z^2*u+F(105)/11*x*z^3*u+F(125)/46*y*z^3*u-6746*z^4*u+3454*x^3*u^2+14656*x^2*y*u^2-F(82)/111*x*y^2*u^2+13130*y^3*u^2+F(121)/58*x^2*z*u^2-F(42)/11*x*y*z*u^2-F(89)/81*y^2*z*u^2+F(68)/49*x*z^2*u^2+8243*y*z^2*u^2-11615*z^3*u^2-F(40)/87*x^2*u^3+F(80)/39*x*y*u^3+14735*y^2*u^3+10706*x*z*u^3-2448*y*z*u^3+F(64)/51*z^2*u^3+4514*x*u^4+658*y*u^4-F(39)/49*z*u^4+F(21)/23*u^5+F(11)/105*x^4*v-108*x^3*y*v-F(31)/98*x^2*y^2*v-F(73)/64*x*y^3*v+205*y^4*v-15348*x^3*z*v-13734*x^2*y*z*v-F(52)/11*x*y^2*z*v+5897*y^3*z*v+F(106)/27*x^2*z^2*v+9613*x*y*z^2*v+F(81)/59*y^2*z^2*v+1172*x*z^3*v+3716*y*z^3*v-3581*z^4*v+50*x^3*u*v+F(81)/38*x^2*y*u*v+F(84)/109*x*y^2*u*v-F(34)/69*y^3*u*v-F(58)/35*x^2*z*u*v-F(23)/105*x*y*z*u*v+F(43)/78*y^2*z*u*v-F(57)/31*x*z^2*u*v+F(98)/37*y*z^2*u*v+F(11)/96*z^3*u*v+147*x^2*u^2*v+11414*x*y*u^2*v+F(5)/96*y^2*u^2*v+F(19)/77*x*z*u^2*v+1274*y*z*u^2*v+F(4)/63*z^2*u^2*v-4396*x*u^3*v+7060*y*u^3*v+5044*z*u^3*v-F(102)/121*u^4*v+F(67)/25*x^3*v^2-13601*x^2*y*v^2+F(68)/65*x*y^2*v^2+F(82)/77*y^3*v^2-8394*x^2*z*v^2-13318*x*y*z*v^2+F(71)/104*y^2*z*v^2-F(75)/107*x*z^2*v^2+F(39)/100*y*z^2*v^2+11229*z^3*v^2+F(113)/84*x^2*u*v^2-2988*x*y*u*v^2-9616*y^2*u*v^2-F(41)/50*x*z*u*v^2+F(3)/34*y*z*u*v^2-12322*z^2*u*v^2-F(120)/11*x*u^2*v^2+F(4)/75*y*u^2*v^2+12562*z*u^2*v^2-F(92)/39*u^3*v^2+1497*x^2*v^3+F(80)/103*x*y*v^3+F(15)/64*y^2*v^3-F(16)/109*x*z*v^3-F(85)/81*y*z*v^3-F(52)/81*z^2*v^3-F(23)/49*x*u*v^3+1295*y*u*v^3+13460*z*u*v^3+F(52)/73*u^2*v^3-F(24)/5*x*v^4+11079*y*v^4-F(25)/57*z*v^4-F(119)/47*u*v^4-F(7)/120*v^5,F(123)/61*x^4*z-F(4)/87*x^3*y*z+15558*x^2*y^2*z-F(78)/5*x*y^3*z+F(31)/83*y^4*z-2966*x^3*z^2+F(73)/89*x^2*y*z^2+F(75)/112*x*y^2*z^2+F(51)/29*y^3*z^2-F(46)/47*x^2*z^3+3812*x*y*z^3-8592*y^2*z^3+7582*x*z^4+F(113)/92*y*z^4-11815*z^5+F(34)/111*x^3*z*u-F(25)/63*x^2*y*z*u+F(115)/79*x*y^2*z*u+F(36)/115*y^3*z*u-F(17)/38*x^2*z^2*u+7906*x*y*z^2*u-F(108)/71*y^2*z^2*u+7541*x*z^3*u-12683*y*z^3*u-F(111)/110*z^4*u-F(107)/24*x^2*z*u^2-F(70)/9*x*y*z*u^2+F(42)/113*y^2*z*u^2-75*x*z^2*u^2+F(64)/101*y*z^2*u^2+F(65)/106*z^3*u^2-F(118)/97*x*z*u^3+12960*y*z*u^3-918*z^2*u^3-F(72)/113*z*u^4+F(17)/73*x^4*v+F(15)/17*x^3*y*v-F(22)/91*x^2*y^2*v+F(78)/121*x*y^3*v+F(9)/77*y^4*v-F(91)/102*x^3*z*v+15814*x^2*y*z*v-12480*x*y^2*z*v+1177*y^3*z*v-F(7)/40*x^2*z^2*v+F(47)/69*x*y*z^2*v-2847*y^2*z^2*v+987*x*z^3*v-1954*y*z^3*v+F(43)/97*z^4*v-7692*x^3*u*v-F(109)/44*x^2*y*u*v-F(94)/111*x*y^2*u*v+F(84)/17*y^3*u*v+13682*x^2*z*u*v+F(64)/7*x*y*z*u*v-13069*y^2*z*u*v+F(92)/67*x*z^2*u*v-F(109)/84*y*z^2*u*v-F(93)/56*z^3*u*v-5368*x^2*u^2*v-F(93)/101*x*y*u^2*v-F(118)/43*y^2*u^2*v-F(40)/57*x*z*u^2*v-F(58)/55*y*z*u^2*v+F(84)/19*z^2*u^2*v+F(4)/9*x*u^3*v-540*y*u^3*v+11588*z*u^3*v-F(93)/25*u^4*v-F(85)/89*x^3*v^2-1800*x^2*y*v^2-F(40)/51*x*y^2*v^2-7293*y^3*v^2+15927*x^2*z*v^2-F(33)/106*x*y*z*v^2+F(25)/98*y^2*z*v^2-13657*x*z^2*v^2+F(43)/37*y*z^2*v^2+F(79)/115*z^3*v^2-F(74)/11*x^2*u*v^2+F(29)/114*x*y*u*v^2-F(50)/69*y^2*u*v^2+252*x*z*u*v^2-F(65)/69*y*z*u*v^2-F(88)/5*z^2*u*v^2+4418*x*u^2*v^2-6634*y*u^2*v^2-9854*z*u^2*v^2-F(20)/73*u^3*v^2+F(94)/11*x^2*v^3+1430*x*y*v^3+F(1)/41*y^2*v^3+F(21)/101*x*z*v^3-F(68)/59*y*z*v^3-9515*z^2*v^3+F(55)/8*x*u*v^3-12014*y*u*v^3+F(57)/73*z*u*v^3-F(97)/58*u^2*v^3-F(124)/91*x*v^4+5575*y*v^4-5470*z*v^4+15340*u*v^4+F(123)/7*v^5,-14327*x^4*y-14863*x^3*y^2+3913*x^2*y^3+4595*x*y^4+F(75)/77*y^5-5371*x^3*y*z+F(86)/123*x^2*y^2*z-F(84)/115*x*y^3*z-F(117)/100*y^4*z-F(23)/37*x^2*y*z^2-F(82)/125*x*y^2*z^2+2426*y^3*z^2+F(17)/50*x*y*z^3+11188*y^2*z^3-F(110)/39*y*z^4-F(126)/97*x^4*u-F(85)/41*x^3*y*u-F(22)/25*x^2*y^2*u-10619*x*y^3*u+3473*y^4*u-F(75)/52*x^3*z*u-F(121)/37*x^2*y*z*u+F(48)/125*x*y^2*z*u+7246*y^3*z*u-F(78)/49*x^2*z^2*u+F(106)/3*x*y*z^2*u-F(50)/59*y^2*z^2*u-F(48)/23*x*z^3*u+F(79)/111*y*z^3*u-14913*z^4*u+F(31)/87*x^3*u^2+F(101)/84*x^2*y*u^2-14270*x*y^2*u^2-F(116)/83*y^3*u^2-F(15)/86*x^2*z*u^2-3946*x*y*z*u^2+F(17)/23*y^2*z*u^2+F(11)/2*x*z^2*u^2-6992*y*z^2*u^2-F(118)/77*z^3*u^2+F(68)/87*x^2*u^3-F(101)/70*x*y*u^3-5015*y^2*u^3-134*x*z*u^3-13479*y*z*u^3-F(103)/19*z^2*u^3-F(24)/61*x*u^4+F(35)/83*y*u^4-15310*z*u^4-10610*u^5-F(22)/41*x^4*v-F(120)/91*x^3*y*v-10964*x^2*y^2*v-F(44)/31*x*y^3*v+F(65)/122*y^4*v-F(29)/47*x^3*z*v+2889*x^2*y*z*v+F(43)/63*x*y^2*z*v-F(47)/9*y^3*z*v+F(14)/41*x^2*z^2*v-F(58)/57*x*y*z^2*v-F(67)/9*y^2*z^2*v+F(20)/33*x*z^3*v+F(32)/87*y*z^3*v-F(113)/114*z^4*v+F(31)/19*x^3*u*v-7367*x^2*y*u*v+11612*x*y^2*u*v-F(27)/62*y^3*u*v-F(47)/2*x^2*z*u*v-F(28)/15*x*y*z*u*v-6682*y^2*z*u*v+14672*x*z^2*u*v+4043*y*z^2*u*v-2529*z^3*u*v-F(39)/115*x^2*u^2*v+F(17)/4*x*y*u^2*v+F(49)/39*y^2*u^2*v+15437*x*z*u^2*v+F(91)/40*y*z*u^2*v+14140*z^2*u^2*v-F(18)/107*x*u^3*v-F(49)/5*y*u^3*v-F(88)/115*z*u^3*v-F(77)/82*u^4*v+1398*x^3*v^2+F(7)/111*x^2*y*v^2+1081*x*y^2*v^2+6822*y^3*v^2-F(19)/100*x^2*z*v^2-F(19)/9*x*y*z*v^2+F(116)/83*y^2*z*v^2+123*x*z^2*v^2-F(37)/54*y*z^2*v^2+F(83)/32*z^3*v^2-F(23)/82*x^2*u*v^2-F(17)/45*x*y*u*v^2-10206*y^2*u*v^2+F(20)/11*x*z*u*v^2-F(29)/43*y*z*u*v^2-F(61)/77*z^2*u*v^2-15878*x*u^2*v^2-F(39)/44*y*u^2*v^2+F(19)/41*z*u^2*v^2+F(35)/88*u^3*v^2+2955*x^2*v^3-F(87)/115*x*y*v^3-F(16)/17*y^2*v^3-F(107)/34*x*z*v^3+F(80)/91*y*z*v^3+F(112)/69*z^2*v^3+F(109)/107*x*u*v^3+F(85)/28*y*u*v^3-F(68)/121*z*u*v^3+2654*u^2*v^3-F(64)/55*x*v^4-F(37)/60*y*v^4+F(69)/7*z*v^4+F(66)/85*u*v^4+5754*v^5,-F(36)/35*x^4*u+12883*x^3*y*u+F(49)/120*x^2*y^2*u-4628*x*y^3*u-F(75)/118*y^4*u+F(65)/93*x^3*z*u+5754*x^2*y*z*u-F(41)/62*x*y^2*z*u+328*y^3*z*u-F(43)/49*x^2*z^2*u+F(98)/107*x*y*z^2*u+F(9)/16*y^2*z^2*u+15504*x*z^3*u-F(94)/39*y*z^3*u-7028*z^4*u-F(74)/81*x^3*u^2+3486*x^2*y*u^2+F(110)/31*x*y^2*u^2+7609*y^3*u^2-F(17)/113*x^2*z*u^2-F(71)/78*x*y*z*u^2-F(49)/102*y^2*z*u^2-2012*x*z^2*u^2-4935*y*z^2*u^2+459*z^3*u^2+F(6)/53*x^2*u^3-12342*x*y*u^3-4432*y^2*u^3-F(95)/23*x*z*u^3+11567*y*z*u^3+F(23)/84*z^2*u^3-F(78)/19*x*u^4+F(57)/10*y*u^4+F(37)/92*z*u^4+11049*u^5-F(70)/83*x^4*v-F(34)/69*x^3*y*v-9309*x^2*y^2*v+10247*x*y^3*v-F(119)/89*y^4*v-13641*x^3*z*v+7018*x^2*y*z*v+F(33)/95*x*y^2*z*v+8545*y^3*z*v+8410*x^2*z^2*v-F(119)/33*x*y*z^2*v-F(31)/44*y^2*z^2*v-F(15)/79*x*z^3*v+F(49)/2*y*z^3*v+15796*z^4*v-F(8)/61*x^3*u*v-4248*x^2*y*u*v+1419*x*y^2*u*v-7061*y^3*u*v-12022*x^2*z*u*v+5656*x*y*z*u*v+F(101)/51*y^2*z*u*v-6255*x*z^2*u*v+8208*y*z^2*u*v-F(25)/31*z^3*u*v-F(55)/64*x^2*u^2*v+F(35)/88*x*y*u^2*v+7627*y^2*u^2*v-15213*x*z*u^2*v-F(125)/47*y*z*u^2*v+716*z^2*u^2*v-13163*x*u^3*v-12319*y*u^3*v+9945*z*u^3*v-F(122)/123*u^4*v-F(55)/112*x^3*v^2-14498*x^2*y*v^2-8389*x*y^2*v^2+F(31)/24*y^3*v^2-F(48)/61*x^2*z*v^2-10781*x*y*z*v^2-F(51)/43*y^2*z*v^2-F(47)/38*x*z^2*v^2+F(73)/88*y*z^2*v^2+3185*z^3*v^2+F(41)/11*x^2*u*v^2+F(53)/15*x*y*u*v^2+F(35)/94*y^2*u*v^2-F(2)/81*x*z*u*v^2-F(109)/55*y*z*u*v^2-12412*z^2*u*v^2+F(8)/67*x*u^2*v^2-F(79)/101*y*u^2*v^2+F(61)/76*z*u^2*v^2-F(119)/47*u^3*v^2+F(100)/7*x^2*v^3-F(56)/81*x*y*v^3+F(21)/82*y^2*v^3+F(87)/52*x*z*v^3-F(85)/57*y*z*v^3-13804*z^2*v^3-F(106)/31*x*u*v^3+12534*y*u*v^3-F(93)/7*z*u*v^3-F(14)/27*u^2*v^3+F(85)/11*x*v^4-13639*y*v^4-F(119)/76*z*v^4+F(59)/111*u*v^4-3220*v^5,975*x^4*y+F(104)/33*x^3*y^2-F(106)/65*x^2*y^3+F(66)/23*x*y^4+F(59)/86*y^5+F(73)/72*x^3*y*z+F(33)/92*x^2*y^2*z+F(41)/19*x*y^3*z-14775*y^4*z-F(99)/23*x^2*y*z^2+3753*x*y^2*z^2+F(71)/41*y^3*z^2-F(92)/5*x*y*z^3+6199*y^2*z^3+125*y*z^4-5129*x^4*u+F(112)/123*x^3*y*u+F(19)/2*x^2*y^2*u-2647*x*y^3*u-F(35)/89*y^4*u-F(31)/54*x^3*z*u+F(72)/37*x^2*y*z*u-F(121)/52*x*y^2*z*u-F(111)/68*y^3*z*u+3697*x^2*z^2*u+128*x*y*z^2*u+14200*y^2*z^2*u-F(27)/53*x*z^3*u+F(59)/78*y*z^3*u-F(22)/45*z^4*u-F(123)/38*x^3*u^2+F(102)/53*x^2*y*u^2-F(49)/57*x*y^2*u^2+5449*y^3*u^2-2572*x^2*z*u^2+F(3)/2*x*y*z*u^2+9486*y^2*z*u^2+F(6)/125*x*z^2*u^2+F(45)/2*y*z^2*u^2-F(19)/55*z^3*u^2-14525*x^2*u^3+F(83)/18*x*y*u^3+F(123)/56*y^2*u^3-12494*x*z*u^3-15240*y*z*u^3-F(28)/117*z^2*u^3-F(20)/93*x*u^4-10552*y*u^4-F(79)/50*z*u^4+F(107)/33*u^5-11945*x^4*v-F(86)/85*x^3*y*v-F(115)/62*x^2*y^2*v+1874*x*y^3*v+F(96)/13*y^4*v+11797*x^3*z*v+15569*x^2*y*z*v+6118*x*y^2*z*v-F(106)/103*y^3*z*v+F(117)/40*x^2*z^2*v+13729*x*y*z^2*v-4329*y^2*z^2*v+2102*x*z^3*v-F(77)/27*y*z^3*v-F(73)/25*z^4*v+F(79)/50*x^3*u*v+F(115)/58*x^2*y*u*v-F(1)/111*x*y^2*u*v-14589*y^3*u*v-10733*x^2*z*u*v-F(83)/65*x*y*z*u*v+1735*y^2*z*u*v-F(104)/31*x*z^2*u*v+F(99)/97*y*z^2*u*v+F(23)/60*z^3*u*v+4021*x^2*u^2*v+15801*x*y*u^2*v+4183*y^2*u^2*v-173*x*z*u^2*v-F(100)/51*y*z*u^2*v-F(79)/123*z^2*u^2*v-8262*x*u^3*v-9744*y*u^3*v+F(28)/37*z*u^3*v+F(5)/111*u^4*v-F(91)/124*x^3*v^2+F(100)/123*x^2*y*v^2+F(113)/96*x*y^2*v^2-F(67)/50*y^3*v^2+F(101)/6*x^2*z*v^2+3452*x*y*z*v^2+9281*y^2*z*v^2+F(53)/22*x*z^2*v^2-4861*y*z^2*v^2+8899*z^3*v^2+F(91)/23*x^2*u*v^2+13924*x*y*u*v^2-F(4)/75*y^2*u*v^2+F(59)/4*x*z*u*v^2-214*y*z*u*v^2-F(114)/49*z^2*u*v^2+14285*x*u^2*v^2+F(85)/27*y*u^2*v^2-F(99)/5*z*u^2*v^2-F(74)/33*u^3*v^2+8313*x^2*v^3-F(96)/71*x*y*v^3+F(41)/69*y^2*v^3-F(97)/31*x*z*v^3-6935*y*z*v^3-F(102)/29*z^2*v^3+F(67)/82*x*u*v^3+F(35)/9*y*u*v^3-F(33)/95*z*u*v^3+6316*u^2*v^3-6800*x*v^4-F(93)/4*y*v^4-6367*z*v^4+F(75)/56*u*v^4-13460*v^5]))
end


@doc raw"""
    k3_d11_pi11_ss_1()

Return a smooth K3 surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and one 6-secant.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d11_pi11_ss_1()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^4*y^2-3663*x^3*y^3-10724*x^2*y^4-3804*x*y^5+7491*y^6-F(69)/52*x^4*y*z-F(6)/13*x^3*y^2*z-F(10)/37*x^2*y^3*z-F(44)/45*x*y^4*z+F(89)/9*y^5*z-8057*x^3*y*z^2-947*x^2*y^2*z^2+F(28)/53*x*y^3*z^2+F(41)/46*y^4*z^2-F(77)/104*x^2*y*z^3+11931*x*y^2*z^3-2696*y^3*z^3-F(26)/97*x*y*z^4-1711*y^2*z^4+473*y*z^5+6446*x^4*y*u+12163*x^3*y^2*u+F(91)/47*x^2*y^3*u-11931*x*y^4*u+F(59)/46*y^5*u-F(36)/91*x^4*z*u-F(10)/17*x^3*y*z*u+F(42)/41*x^2*y^2*z*u+14006*x*y^3*z*u+1291*y^4*z*u-3793*x^3*z^2*u+F(105)/101*x^2*y*z^2*u-F(97)/12*x*y^2*z^2*u+F(72)/107*y^3*z^2*u-3531*x^2*z^3*u+F(113)/24*x*y*z^3*u-10029*y^2*z^3*u+F(15)/43*x*z^4*u-871*y*z^4*u-F(19)/52*z^5*u-8977*x^4*u^2-F(120)/43*x^3*y*u^2-F(19)/50*x^2*y^2*u^2-2147*x*y^3*u^2+2275*y^4*u^2-F(43)/36*x^3*z*u^2+F(20)/73*x^2*y*z*u^2-F(91)/2*x*y^2*z*u^2+3539*y^3*z*u^2+F(18)/95*x^2*z^2*u^2+F(121)/32*x*y*z^2*u^2+10149*y^2*z^2*u^2-9885*x*z^3*u^2+4135*y*z^3*u^2-9028*z^4*u^2-F(124)/19*x^3*u^3+F(91)/101*x^2*y*u^3-F(5)/117*x*y^2*u^3-F(56)/85*y^3*u^3-6593*x^2*z*u^3-11095*x*y*z*u^3-F(15)/71*y^2*z*u^3-F(35)/53*x*z^2*u^3-5737*y*z^2*u^3-F(31)/125*z^3*u^3+358*x^2*u^4+F(29)/7*x*y*u^4-F(47)/48*y^2*u^4+F(89)/60*x*z*u^4-F(59)/23*y*z*u^4-2626*z^2*u^4+F(86)/17*x*u^5+F(5)/29*y*u^5-F(57)/113*z*u^5+11717*u^6-F(73)/111*x^4*y*v-F(32)/11*x^3*y^2*v+F(5)/83*x^2*y^3*v+F(34)/25*x*y^4*v-5062*y^5*v+F(92)/107*x^4*z*v-F(5)/98*x^3*y*z*v+1842*x^2*y^2*z*v-F(28)/99*x*y^3*z*v-10830*y^4*z*v+F(10)/29*x^3*z^2*v-F(59)/113*x^2*y*z^2*v-12046*x*y^2*z^2*v+F(68)/97*y^3*z^2*v+F(125)/97*x^2*z^3*v+F(61)/44*x*y*z^3*v+3329*y^2*z^3*v+8432*x*z^4*v-4791*y*z^4*v-F(13)/111*z^5*v-F(25)/87*x^4*u*v-F(35)/89*x^3*y*u*v+3028*x^2*y^2*u*v-F(63)/67*x*y^3*u*v-8541*y^4*u*v+F(76)/119*x^3*z*u*v-F(51)/29*x^2*y*z*u*v-F(105)/8*x*y^2*z*u*v+F(29)/102*y^3*z*u*v+3454*x^2*z^2*u*v-F(61)/69*x*y*z^2*u*v+F(45)/103*y^2*z^2*u*v-15784*x*z^3*u*v-6842*y*z^3*u*v+F(114)/53*z^4*u*v-F(24)/5*x^3*u^2*v-5930*x^2*y*u^2*v+15904*x*y^2*u^2*v-1610*y^3*u^2*v+F(4)/37*x^2*z*u^2*v-F(9)/35*x*y*z*u^2*v-F(19)/52*y^2*z*u^2*v+F(19)/101*x*z^2*u^2*v-F(112)/31*y*z^2*u^2*v-F(16)/121*z^3*u^2*v-F(11)/6*x^2*u^3*v+F(53)/70*x*y*u^3*v-15878*y^2*u^3*v-F(71)/52*x*z*u^3*v-7484*y*z*u^3*v+3269*z^2*u^3*v-9638*x*u^4*v+F(57)/58*y*u^4*v+6*z*u^4*v-7368*u^5*v-F(86)/51*x^4*v^2+F(85)/113*x^3*y*v^2-5213*x^2*y^2*v^2+F(24)/109*x*y^3*v^2+F(8)/99*y^4*v^2-F(1)/38*x^3*z*v^2+F(49)/102*x^2*y*z*v^2-F(13)/116*x*y^2*z*v^2+F(118)/43*y^3*z*v^2-F(79)/72*x^2*z^2*v^2-2490*x*y*z^2*v^2-F(11)/92*y^2*z^2*v^2+F(57)/103*x*z^3*v^2+11068*y*z^3*v^2+F(22)/45*z^4*v^2+2008*x^3*u*v^2+F(53)/28*x^2*y*u*v^2+F(122)/49*x*y^2*u*v^2-12475*y^3*u*v^2+3882*x^2*z*u*v^2-12197*x*y*z*u*v^2+F(68)/3*y^2*z*u*v^2+F(71)/46*x*z^2*u*v^2+F(112)/83*y*z^2*u*v^2-F(34)/59*z^3*u*v^2-F(43)/37*x^2*u^2*v^2-2794*x*y*u^2*v^2-F(19)/74*y^2*u^2*v^2+9256*x*z*u^2*v^2+8056*y*z*u^2*v^2+F(103)/117*z^2*u^2*v^2+F(64)/105*x*u^3*v^2-F(12)/29*y*u^3*v^2-F(68)/125*z*u^3*v^2+11064*u^4*v^2-F(62)/53*x^3*v^3-F(101)/26*x^2*y*v^3+F(97)/18*x*y^2*v^3+9768*y^3*v^3-7028*x^2*z*v^3-6354*x*y*z*v^3+7636*y^2*z*v^3+15474*x*z^2*v^3-F(29)/53*y*z^2*v^3-489*z^3*v^3+F(119)/9*x^2*u*v^3+2120*x*y*u*v^3-F(64)/105*y^2*u*v^3-9961*x*z*u*v^3-1310*y*z*u*v^3-F(28)/81*z^2*u*v^3+14140*x*u^2*v^3+F(63)/115*y*u^2*v^3+F(57)/46*z*u^2*v^3+F(36)/67*u^3*v^3-15895*x^2*v^4+F(58)/119*x*y*v^4-2652*y^2*v^4+10744*x*z*v^4+F(113)/24*y*z*v^4-F(117)/79*z^2*v^4-F(37)/56*x*u*v^4+F(13)/50*y*u*v^4+F(122)/87*z*u*v^4+F(17)/96*u^2*v^4+F(53)/97*x*v^5+994*y*v^5-5718*z*v^5-F(73)/106*u*v^5-F(25)/63*v^6,F(69)/52*x^5+F(75)/112*x^4*y+14562*x^3*y^2+7777*x^2*y^3-2686*x*y^4-F(73)/124*y^5+8057*x^4*z+10049*x^3*y*z+F(90)/37*x^2*y^2*z-F(75)/74*x*y^3*z-F(50)/69*y^4*z+F(77)/104*x^3*z^2-F(51)/32*x^2*y*z^2-F(19)/65*x*y^2*z^2-12826*y^3*z^2+F(26)/97*x^2*z^3+13344*x*y*z^3-F(55)/87*y^2*z^3-473*x*z^4+F(63)/109*y*z^4-F(13)/8*x^4*u-F(59)/51*x^3*y*u+F(89)/21*x^2*y^2*u+9622*x*y^3*u+13131*y^4*u-F(65)/102*x^3*z*u-F(49)/117*x^2*y*z*u+9506*x*y^2*z*u-F(73)/110*y^3*z*u+28*x^2*z^2*u-F(66)/25*x*y*z^2*u-F(49)/93*y^2*z^2*u+F(81)/94*x*z^3*u-2971*y*z^3*u+12175*z^4*u+F(45)/61*x^3*u^2-11541*x^2*y*u^2+825*x*y^2*u^2+F(9)/17*y^3*u^2-F(11)/31*x^2*z*u^2-3125*x*y*z*u^2+F(116)/99*y^2*z*u^2-F(69)/65*x*z^2*u^2-13494*y*z^2*u^2-2445*z^3*u^2+F(7)/76*x^2*u^3+F(79)/113*x*y*u^3+15928*y^2*u^3+F(83)/60*x*z*u^3-F(64)/91*y*z*u^3+14202*z^2*u^3+4769*x*u^4-F(71)/114*y*u^4+13186*z*u^4+12985*u^5+1464*x^4*v+F(62)/35*x^3*y*v-F(101)/44*x^2*y^2*v-F(67)/4*x*y^3*v+F(104)/109*y^4*v+2166*x^3*z*v-F(43)/48*x^2*y*z*v+11356*x*y^2*z*v-F(46)/39*y^3*z*v-F(93)/61*x^2*z^2*v-F(69)/94*x*y*z^2*v+1060*y^2*z^2*v-15206*x*z^3*v+F(4)/107*y*z^3*v+F(53)/90*z^4*v+F(103)/2*x^3*u*v+F(17)/19*x^2*y*u*v+9984*x*y^2*u*v-402*y^3*u*v-8249*x^2*z*u*v+7556*x*y*z*u*v-F(114)/65*y^2*z*u*v-7610*x*z^2*u*v+F(16)/9*y*z^2*u*v+2059*z^3*u*v+6801*x^2*u^2*v-12430*x*y*u^2*v-F(3)/58*y^2*u^2*v+F(85)/53*x*z*u^2*v+3442*y*z*u^2*v+2212*z^2*u^2*v+129*x*u^3*v-F(89)/7*y*u^3*v+F(111)/122*z*u^3*v+2165*u^4*v-3536*x^3*v^2-F(92)/107*x^2*y*v^2+F(58)/99*x*y^2*v^2-F(11)/75*y^3*v^2+F(35)/54*x^2*z*v^2-14291*x*y*z*v^2+F(33)/10*y^2*z*v^2-13874*x*z^2*v^2+5542*y*z^2*v^2-F(87)/11*z^3*v^2-10611*x^2*u*v^2-15912*x*y*u*v^2+F(23)/109*y^2*u*v^2-F(108)/13*x*z*u*v^2+F(43)/110*y*z*u*v^2+8114*z^2*u*v^2-8207*x*u^2*v^2-7168*y*u^2*v^2+1815*z*u^2*v^2+F(37)/87*u^3*v^2-9067*x^2*v^3+4016*x*y*v^3+F(58)/111*y^2*v^3-496*x*z*v^3+F(109)/100*y*z*v^3+378*z^2*v^3-F(35)/51*x*u*v^3+F(29)/3*y*u*v^3+988*z*u*v^3-4788*u^2*v^3+7394*x*v^4-F(78)/37*y*v^4-F(111)/85*z*v^4+15068*u*v^4-11908*v^5,717*x^5-F(13)/42*x^4*y+F(3)/76*x^3*y^2-5720*x^2*y^3-F(65)/73*x*y^4+F(56)/111*y^5+F(115)/11*x^4*z+F(112)/111*x^3*y*z+F(113)/29*x^2*y^2*z-7891*x*y^3*z-F(53)/49*y^4*z-15545*x^3*z^2-6867*x^2*y*z^2-903*x*y^2*z^2-F(46)/107*y^3*z^2-4785*x^2*z^3+13039*x*y*z^3+F(121)/5*y^2*z^3+F(69)/79*x*z^4-F(109)/8*y*z^4+F(27)/112*x^4*u+8049*x^3*y*u+3080*x^2*y^2*u+15931*x*y^3*u-F(37)/28*y^4*u-F(18)/67*x^3*z*u-F(74)/11*x^2*y*z*u+F(57)/73*x*y^2*z*u-F(49)/100*y^3*z*u-513*x^2*z^2*u-11279*x*y*z^2*u-F(108)/37*y^2*z^2*u-F(107)/114*x*z^3*u+F(89)/88*y*z^3*u+F(91)/33*z^4*u-5495*x^3*u^2+8896*x^2*y*u^2-F(79)/45*x*y^2*u^2+F(71)/85*y^3*u^2-2053*x^2*z*u^2-F(107)/112*x*y*z*u^2-F(36)/91*y^2*z*u^2+3817*x*z^2*u^2+F(94)/73*y*z^2*u^2+6333*z^3*u^2+F(90)/119*x^2*u^3+13114*x*y*u^3-F(31)/66*y^2*u^3-F(103)/10*x*z*u^3+2364*y*z*u^3+3742*z^2*u^3+F(76)/111*x*u^4-978*y*u^4-7937*z*u^4-F(31)/97*u^5+7935*x^4*v+F(6)/35*x^3*y*v+10421*x^2*y^2*v+F(82)/89*x*y^3*v+F(7)/107*y^4*v-13129*x^3*z*v-F(95)/37*x^2*y*z*v-5957*x*y^2*z*v+15191*y^3*z*v-4909*x^2*z^2*v+F(118)/101*x*y*z^2*v+F(73)/79*y^2*z^2*v+F(109)/30*x*z^3*v+F(85)/79*y*z^3*v+F(51)/14*z^4*v+6366*x^3*u*v-F(19)/50*x^2*y*u*v+F(95)/27*x*y^2*u*v+12120*y^3*u*v-1908*x^2*z*u*v+F(96)/53*x*y*z*u*v-F(91)/96*y^2*z*u*v-F(11)/107*x*z^2*u*v-6821*y*z^2*u*v+F(22)/111*z^3*u*v+F(13)/25*x^2*u^2*v+F(51)/8*x*y*u^2*v-F(58)/19*y^2*u^2*v+10920*x*z*u^2*v+F(6)/23*y*z*u^2*v+825*z^2*u^2*v+F(95)/8*x*u^3*v+F(38)/35*y*u^3*v-5774*z*u^3*v+2412*u^4*v+F(64)/31*x^3*v^2+F(53)/86*x^2*y*v^2+F(115)/108*x*y^2*v^2+F(94)/37*y^3*v^2+F(122)/29*x^2*z*v^2+F(52)/17*x*y*z*v^2+536*y^2*z*v^2+F(4)/117*x*z^2*v^2+F(102)/13*y*z^2*v^2+F(11)/63*z^3*v^2-F(22)/51*x^2*u*v^2+F(117)/124*x*y*u*v^2+F(124)/37*y^2*u*v^2+3698*x*z*u*v^2-15225*y*z*u*v^2-4427*z^2*u*v^2+165*x*u^2*v^2-4238*y*u^2*v^2-F(31)/86*z*u^2*v^2-F(79)/84*u^3*v^2-F(69)/83*x^2*v^3-6842*x*y*v^3+F(29)/94*y^2*v^3-F(68)/103*x*z*v^3+F(4)/43*y*z*v^3+F(61)/53*z^2*v^3+F(5)/97*x*u*v^3-F(37)/35*y*u*v^3-6510*z*u*v^3-112*u^2*v^3+F(19)/40*x*v^4-F(40)/77*y*v^4-F(85)/12*z*v^4-F(59)/123*u*v^4+F(11)/107*v^5,-15293*x^5-F(109)/14*x^4*y+7042*x^3*y^2-F(73)/22*x^2*y^3+F(30)/71*x*y^4+13232*y^5+F(36)/73*x^4*z-15465*x^3*y*z-F(85)/8*x^2*y^2*z-732*x*y^3*z-11212*y^4*z+165*x^3*z^2-2683*x^2*y*z^2-F(65)/121*x*y^2*z^2-F(95)/64*y^3*z^2-F(80)/77*x^2*z^3+15879*x*y*z^3-9801*y^2*z^3+15766*x*z^4-F(124)/55*y*z^4+F(112)/19*z^5+F(17)/94*x^4*u+F(1)/74*x^3*y*u-F(104)/121*x^2*y^2*u-7318*x*y^3*u-10344*y^4*u-F(34)/75*x^3*z*u+3053*x^2*y*z*u-F(41)/69*x*y^2*z*u-13119*y^3*z*u+F(119)/2*x^2*z^2*u-10091*x*y*z^2*u+2255*y^2*z^2*u+12223*x*z^3*u-8287*y*z^3*u-F(73)/76*z^4*u-F(67)/51*x^3*u^2-7371*x^2*y*u^2-F(35)/108*x*y^2*u^2-15217*y^3*u^2-5214*x^2*z*u^2-2046*x*y*z*u^2+F(67)/96*y^2*z*u^2-F(5)/37*x*z^2*u^2+F(60)/59*y*z^2*u^2+1327*z^3*u^2+6643*x^2*u^3+F(115)/67*x*y*u^3+F(125)/64*y^2*u^3+14336*x*z*u^3+F(6)/121*y*z*u^3+11040*z^2*u^3-F(12)/55*x*u^4+F(23)/113*y*u^4+F(119)/72*z*u^4-F(71)/59*u^5-F(49)/109*x^4*v+103*x^3*y*v-3741*x^2*y^2*v+F(61)/120*x*y^3*v+4418*y^4*v-F(74)/65*x^3*z*v+F(103)/97*x^2*y*z*v-F(79)/57*x*y^2*z*v+3036*y^3*z*v-6840*x^2*z^2*v-F(121)/102*x*y*z^2*v+2185*y^2*z^2*v-F(79)/118*x*z^3*v-2887*y*z^3*v-5521*z^4*v+F(27)/119*x^3*u*v+6771*x^2*y*u*v-F(109)/103*x*y^2*u*v-F(20)/23*y^3*u*v+15712*x^2*z*u*v+F(111)/19*x*y*z*u*v-F(85)/103*y^2*z*u*v-F(13)/80*x*z^2*u*v-5435*y*z^2*u*v+F(87)/53*z^3*u*v+F(59)/100*x^2*u^2*v-F(61)/55*x*y*u^2*v-10508*y^2*u^2*v-F(67)/104*x*z*u^2*v-F(39)/17*y*z*u^2*v-9201*z^2*u^2*v-6297*x*u^3*v-15459*y*u^3*v-1708*z*u^3*v-F(49)/44*u^4*v+F(67)/27*x^3*v^2-F(79)/126*x^2*y*v^2-1382*x*y^2*v^2+F(101)/91*y^3*v^2-12980*x^2*z*v^2+11932*x*y*z*v^2+246*y^2*z*v^2+F(47)/37*x*z^2*v^2+F(64)/35*y*z^2*v^2-13766*z^3*v^2+2619*x^2*u*v^2-6354*x*y*u*v^2-9661*y^2*u*v^2-F(10)/31*x*z*u*v^2-F(53)/86*y*z*u*v^2+F(43)/94*z^2*u*v^2+11168*x*u^2*v^2+5595*y*u^2*v^2+1135*z*u^2*v^2+F(61)/26*u^3*v^2-F(41)/19*x^2*v^3+1223*x*y*v^3+5425*y^2*v^3+F(91)/9*x*z*v^3-F(73)/14*y*z*v^3+F(63)/124*z^2*v^3-F(1)/61*x*u*v^3-F(102)/91*y*u*v^3-F(89)/96*z*u*v^3-9764*u^2*v^3+F(86)/77*x*v^4+3214*y*v^4+F(43)/98*z*v^4-2101*u*v^4+1198*v^5,-x^5+3663*x^4*y+10724*x^3*y^2+3804*x^2*y^3-7491*x*y^4-7185*x^4*z-9374*x^3*y*z+44*x^2*y^2*z-4433*x*y^3*z+F(73)/124*y^4*z-9102*x^3*z^2+F(49)/11*x^2*y*z^2-F(103)/80*x*y^2*z^2+F(50)/69*y^3*z^2+15063*x^2*z^3+F(7)/115*x*y*z^3+12826*y^2*z^3+F(1)/11*x*z^4+F(55)/87*y*z^4-F(63)/109*z^5+F(38)/39*x^4*u+3872*x^3*y*u-5610*x^2*y^2*u+F(113)/43*x*y^3*u-F(16)/71*y^4*u-F(18)/35*x^3*z*u+F(103)/78*x^2*y*z*u+F(57)/104*x*y^2*z*u-8941*y^3*z*u-F(9)/23*x^2*z^2*u+5398*x*y*z^2*u+F(55)/2*y^2*z^2*u+3419*x*z^3*u-F(54)/107*y*z^3*u-F(100)/33*z^4*u+F(125)/4*x^3*u^2-F(71)/118*x^2*y*u^2-1326*x*y^2*u^2-F(33)/46*y^3*u^2+12151*x^2*z*u^2-F(107)/122*x*y*z*u^2+10329*y^2*z*u^2+6947*x*z^2*u^2-F(113)/20*y*z^2*u^2+F(32)/71*z^3*u^2-F(11)/123*x^2*u^3-F(81)/124*x*y*u^3-F(55)/92*y^2*u^3+38*x*z*u^3-3463*y*z*u^3-7070*z^2*u^3+F(121)/46*x*u^4-14367*y*u^4-15386*z*u^4-1150*u^5-13515*x^4*v-F(33)/52*x^3*y*v+F(43)/123*x^2*y^2*v+F(67)/118*x*y^3*v+F(27)/19*y^4*v-F(85)/4*x^3*z*v-F(67)/35*x^2*y*z*v+F(121)/47*x*y^2*z*v-F(63)/85*y^3*z*v+14039*x^2*z^2*v+F(19)/102*x*y*z^2*v-9507*y^2*z^2*v-F(38)/65*x*z^3*v+4728*y*z^3*v-F(14)/45*z^4*v-F(23)/90*x^3*u*v-F(7)/8*x^2*y*u*v-F(119)/92*x*y^2*u*v-5063*y^3*u*v-7571*x^2*z*u*v+13551*x*y*z*u*v+F(48)/49*y^2*z*u*v-F(7)/8*x*z^2*u*v-F(42)/113*y*z^2*u*v+F(20)/93*z^3*u*v-F(68)/75*x^2*u^2*v+F(82)/83*x*y*u^2*v-F(93)/5*y^2*u^2*v-14915*x*z*u^2*v-F(89)/49*y*z*u^2*v+12321*z^2*u^2*v+14556*x*u^3*v+F(116)/117*y*u^3*v-F(121)/31*z*u^3*v+3486*u^4*v+4153*x^3*v^2-12081*x^2*y*v^2+F(72)/13*x*y^2*v^2-6992*y^3*v^2-2103*x^2*z*v^2+7763*x*y*z*v^2+2039*y^2*z*v^2-F(22)/105*x*z^2*v^2+F(36)/67*y*z^2*v^2+8899*z^3*v^2-F(69)/5*x^2*u*v^2-3166*x*y*u*v^2+F(22)/105*y^2*u*v^2+F(73)/16*x*z*u*v^2-F(61)/13*y*z*u*v^2+F(91)/113*z^2*u*v^2-F(103)/24*x*u^2*v^2-4988*y*u^2*v^2+F(29)/49*z*u^2*v^2+F(109)/126*u^3*v^2-9709*x^2*v^3+F(71)/90*x*y*v^3-1755*y^2*v^3-5873*x*z*v^3-6568*y*z*v^3+F(28)/43*z^2*v^3+F(87)/4*x*u*v^3+F(113)/95*y*u*v^3-F(73)/38*z*u*v^3-F(43)/34*u^2*v^3-F(13)/123*x*v^4+F(92)/49*y*v^4-3103*z*v^4-6255*u*v^4+5831*v^5,F(55)/38*x^5+F(7)/51*x^4*y-F(57)/16*x^3*y^2-F(42)/95*x^2*y^3+F(115)/106*x*y^4-1924*y^5-F(87)/106*x^4*z-5673*x^3*y*z-F(118)/89*x^2*y^2*z-5939*x*y^3*z-7809*y^4*z+F(89)/29*x^3*z^2-F(23)/107*x^2*y*z^2+F(62)/59*x*y^2*z^2-2193*y^3*z^2-F(73)/99*x^2*z^3-F(109)/50*x*y*z^3+F(74)/119*y^2*z^3-3959*x*z^4+7262*y*z^4+3496*z^5-10090*x^4*u-9935*x^3*y*u+14298*x^2*y^2*u-F(71)/83*x*y^3*u+F(85)/118*y^4*u+F(90)/97*x^3*z*u-9892*x^2*y*z*u+5155*x*y^2*z*u+F(55)/73*y^3*z*u+13541*x^2*z^2*u-3896*x*y*z^2*u-F(71)/2*y^2*z^2*u+F(12)/83*x*z^3*u+13131*y*z^3*u+11875*z^4*u+F(35)/121*x^3*u^2-F(77)/103*x^2*y*u^2-F(101)/112*x*y^2*u^2+15185*y^3*u^2-F(100)/69*x^2*z*u^2+F(99)/125*x*y*z*u^2-13162*y^2*z*u^2-F(48)/115*x*z^2*u^2-F(116)/33*y*z^2*u^2-F(68)/109*z^3*u^2-F(126)/83*x^2*u^3+2394*x*y*u^3-5761*y^2*u^3-F(119)/12*x*z*u^3+1483*y*z*u^3+F(73)/4*z^2*u^3-F(73)/123*x*u^4-F(47)/92*y*u^4-F(58)/15*z*u^4-F(115)/101*u^5-F(55)/68*x^4*v+5749*x^3*y*v+F(107)/57*x^2*y^2*v+10593*x*y^3*v+14692*y^4*v+F(15)/76*x^3*z*v+F(44)/97*x^2*y*z*v-F(107)/123*x*y^2*z*v-F(79)/124*y^3*z*v-11717*x^2*z^2*v+F(110)/63*x*y*z^2*v-F(57)/8*y^2*z^2*v-13199*x*z^3*v-15112*y*z^3*v-F(107)/53*z^4*v+F(96)/115*x^3*u*v-4059*x^2*y*u*v-F(59)/49*x*y^2*u*v+F(31)/99*y^3*u*v-F(71)/7*x^2*z*u*v-13867*x*y*z*u*v-F(111)/110*y^2*z*u*v-F(101)/77*x*z^2*u*v+F(97)/7*y*z^2*u*v-F(72)/73*z^3*u*v-134*x^2*u^2*v-2583*x*y*u^2*v+F(45)/44*y^2*u^2*v+11967*x*z*u^2*v+10248*y*z*u^2*v-F(102)/107*z^2*u^2*v+F(55)/58*x*u^3*v+F(124)/117*y*u^3*v-10586*z*u^3*v-4698*u^4*v-F(31)/95*x^3*v^2-F(56)/79*x^2*y*v^2-F(81)/94*x*y^2*v^2+6625*y^3*v^2-F(111)/106*x^2*z*v^2-11718*x*y*z*v^2-F(101)/104*y^2*z*v^2-F(91)/114*x*z^2*v^2-7203*y*z^2*v^2+F(83)/44*z^3*v^2-F(109)/77*x^2*u*v^2+F(83)/51*x*y*u*v^2+2475*y^2*u*v^2+15664*x*z*u*v^2-F(49)/37*y*z*u*v^2+5501*z^2*u*v^2-F(11)/119*x*u^2*v^2+F(28)/55*y*u^2*v^2+F(99)/37*z*u^2*v^2+F(105)/104*u^3*v^2-F(82)/105*x^2*v^3-5974*x*y*v^3+8793*y^2*v^3-F(118)/85*x*z*v^3+F(73)/57*y*z*v^3-7717*z^2*v^3+8651*x*u*v^3-5248*y*u*v^3+F(107)/45*z*u*v^3+F(20)/77*u^2*v^3+15532*x*v^4+14920*y*v^4+F(84)/67*z*v^4-6056*u*v^4+F(45)/8*v^5,F(10)/119*x^5+13481*x^4*y+F(89)/85*x^3*y^2-F(29)/106*x^2*y^3+F(35)/111*x*y^4-3882*y^5+F(91)/99*x^4*z-5593*x^3*y*z-7581*x^2*y^2*z-6743*x*y^3*z-6495*y^4*z-F(82)/121*x^3*z^2+F(125)/113*x^2*y*z^2-15146*x*y^2*z^2+F(15)/23*y^3*z^2-F(119)/43*x^2*z^3+F(21)/38*x*y*z^3+5830*y^2*z^3+F(67)/20*x*z^4-F(13)/48*y*z^4+F(108)/95*z^5+F(45)/43*x^4*u-8052*x^3*y*u-F(90)/119*x^2*y^2*u-F(109)/71*x*y^3*u-F(101)/77*y^4*u+1074*x^3*z*u+2869*x^2*y*z*u+F(85)/121*x*y^2*z*u+1341*y^3*z*u-13407*x^2*z^2*u-10949*x*y*z^2*u-F(85)/12*y^2*z^2*u-5695*x*z^3*u+10589*y*z^3*u-F(107)/28*z^4*u-12390*x^3*u^2+9460*x^2*y*u^2-14468*x*y^2*u^2+1383*y^3*u^2+13591*x^2*z*u^2-10139*x*y*z*u^2+F(22)/61*y^2*z*u^2-F(124)/73*x*z^2*u^2-15931*y*z^2*u^2-F(62)/79*z^3*u^2+14452*x^2*u^3+F(32)/37*x*y*u^3-F(55)/114*y^2*u^3+F(8)/121*x*z*u^3+14576*y*z*u^3-F(76)/113*z^2*u^3+11614*x*u^4-5164*y*u^4+F(33)/92*z*u^4-F(111)/46*u^5+F(74)/15*x^4*v-F(14)/9*x^3*y*v-F(103)/50*x^2*y^2*v+F(65)/82*x*y^3*v+13462*y^4*v+1118*x^3*z*v+F(121)/106*x^2*y*z*v-14499*x*y^2*z*v-5673*y^3*z*v-937*x^2*z^2*v-7885*x*y*z^2*v-F(19)/74*y^2*z^2*v-F(35)/83*x*z^3*v+F(77)/59*y*z^3*v-15079*z^4*v-F(66)/89*x^3*u*v-F(20)/93*x^2*y*u*v-2890*x*y^2*u*v-8898*y^3*u*v-F(124)/73*x^2*z*u*v-5653*x*y*z*u*v+F(42)/65*y^2*z*u*v-F(59)/33*x*z^2*u*v-F(8)/43*y*z^2*u*v+F(85)/83*z^3*u*v-9170*x^2*u^2*v+10275*x*y*u^2*v-4369*y^2*u^2*v-F(111)/73*x*z*u^2*v+F(27)/107*y*z*u^2*v-F(80)/107*z^2*u^2*v-F(13)/16*x*u^3*v-F(74)/11*y*u^3*v-F(31)/99*z*u^3*v+F(28)/39*u^4*v-F(12)/71*x^3*v^2+F(31)/117*x^2*y*v^2+5247*x*y^2*v^2-F(33)/115*y^3*v^2+F(103)/110*x^2*z*v^2-F(25)/96*x*y*z*v^2+13742*y^2*z*v^2+F(23)/65*x*z^2*v^2-6904*y*z^2*v^2-15350*z^3*v^2+F(89)/98*x^2*u*v^2+F(64)/29*x*y*u*v^2+F(45)/73*y^2*u*v^2+F(121)/32*x*z*u*v^2-4493*y*z*u*v^2+F(95)/66*z^2*u*v^2-F(28)/121*x*u^2*v^2-F(47)/18*y*u^2*v^2+F(121)/29*z*u^2*v^2-F(111)/106*u^3*v^2-F(108)/91*x^2*v^3-9690*x*y*v^3+4060*y^2*v^3+F(120)/53*x*z*v^3-F(24)/23*y*z*v^3-F(100)/113*z^2*v^3-F(84)/47*x*u*v^3+8263*y*u*v^3+F(47)/96*z*u*v^3-13499*u^2*v^3+10264*x*v^4-F(14)/25*y*v^4-F(109)/80*z*v^4-3540*u*v^4-14498*v^5,-F(79)/60*x^5-F(2)/87*x^4*y-11189*x^3*y^2+F(25)/28*x^2*y^3+F(77)/123*x*y^4-F(85)/73*y^5-F(76)/105*x^4*z+8952*x^3*y*z+F(119)/71*x^2*y^2*z-F(4)/89*x*y^3*z-10139*y^4*z+F(59)/123*x^3*z^2-F(112)/121*x^2*y*z^2-F(111)/22*x*y^2*z^2+F(29)/77*y^3*z^2-4047*x^2*z^3-F(78)/43*x*y*z^3+6424*y^2*z^3+F(106)/101*x*z^4+F(61)/6*y*z^4-F(19)/52*z^5-F(67)/69*x^4*u+15286*x^3*y*u+4221*x^2*y^2*u+F(4)/83*x*y^3*u-12965*y^4*u+F(5)/114*x^3*z*u+2376*x^2*y*z*u+F(103)/117*x*y^2*z*u-F(14)/117*y^3*z*u-F(64)/71*x^2*z^2*u+3777*x*y*z^2*u+F(13)/69*y^2*z^2*u+2763*x*z^3*u-5128*y*z^3*u-F(8)/25*z^4*u+3242*x^3*u^2+7065*x^2*y*u^2+8592*x*y^2*u^2-15875*y^3*u^2-5173*x^2*z*u^2-2940*x*y*z*u^2+F(16)/47*y^2*z*u^2+F(102)/23*x*z^2*u^2-F(125)/122*y*z^2*u^2-F(55)/72*z^3*u^2-F(56)/5*x^2*u^3+F(112)/85*x*y*u^3-14049*y^2*u^3-F(104)/7*x*z*u^3-F(94)/57*y*z*u^3-14820*z^2*u^3+F(43)/17*x*u^4-14798*y*u^4-F(120)/109*z*u^4+F(20)/17*u^5+F(69)/37*x^4*v-F(59)/31*x^3*y*v+5126*x^2*y^2*v+F(33)/46*x*y^3*v+F(103)/7*y^4*v-2082*x^3*z*v-F(29)/90*x^2*y*z*v-F(95)/16*x*y^2*z*v-F(1)/73*y^3*z*v-F(65)/14*x^2*z^2*v+F(125)/32*x*y*z^2*v+F(39)/101*y^2*z^2*v-2270*x*z^3*v-F(15)/68*y*z^3*v+F(7)/83*z^4*v+F(65)/17*x^3*u*v-7278*x^2*y*u*v+1578*x*y^2*u*v+8492*y^3*u*v+F(21)/61*x^2*z*u*v+13121*x*y*z*u*v+F(65)/42*y^2*z*u*v+6299*x*z^2*u*v+F(121)/114*y*z^2*u*v-1785*z^3*u*v-166*x^2*u^2*v-F(104)/11*x*y*u^2*v-F(25)/111*y^2*u^2*v+14349*x*z*u^2*v+F(102)/97*y*z*u^2*v-F(29)/66*z^2*u^2*v+F(26)/123*x*u^3*v+F(16)/83*y*u^3*v-F(59)/3*z*u^3*v+F(49)/111*u^4*v+15886*x^3*v^2-F(106)/49*x^2*y*v^2-F(56)/57*x*y^2*v^2-11186*y^3*v^2+F(117)/58*x^2*z*v^2+F(101)/72*x*y*z*v^2-7066*y^2*z*v^2+F(33)/91*x*z^2*v^2+12448*y*z^2*v^2-F(69)/83*z^3*v^2+F(31)/55*x^2*u*v^2+F(116)/47*x*y*u*v^2+14093*y^2*u*v^2-11068*x*z*u*v^2-F(123)/29*y*z*u*v^2-12596*z^2*u*v^2+14799*x*u^2*v^2+9323*y*u^2*v^2-F(77)/64*z*u^2*v^2+F(18)/55*u^3*v^2-F(68)/77*x^2*v^3-4352*x*y*v^3+F(97)/44*y^2*v^3+12412*x*z*v^3+F(7)/120*y*z*v^3+11154*z^2*v^3-F(57)/98*x*u*v^3+3345*y*u*v^3+3175*z*u*v^3-3406*u^2*v^3-2111*x*v^4-6205*y*v^4+2776*z*v^4-F(104)/111*u*v^4-6752*v^5,-F(4)/9*x^4*y+F(79)/59*x^3*y^2-F(5)/44*x^2*y^3+F(7)/82*x*y^4+F(49)/16*y^5+11557*x^4*z-F(59)/112*x^3*y*z+F(4)/5*x^2*y^2*z+14170*x*y^3*z-14981*y^4*z+658*x^3*z^2+15922*x^2*y*z^2+F(26)/109*x*y^2*z^2-9455*y^3*z^2-2059*x^2*z^3+F(46)/123*x*y*z^3-F(91)/125*y^2*z^3+F(96)/79*x*z^4+F(76)/3*y*z^4+F(20)/79*z^5-F(26)/57*x^4*u-F(85)/76*x^3*y*u-1244*x^2*y^2*u+1862*x*y^3*u+F(31)/120*y^4*u-F(17)/63*x^3*z*u+F(2)/33*x^2*y*z*u+F(89)/59*x*y^2*z*u+7240*y^3*z*u+2049*x^2*z^2*u+8364*x*y*z^2*u+F(52)/121*y^2*z^2*u-F(78)/119*x*z^3*u+F(79)/25*y*z^3*u-F(53)/8*z^4*u+5771*x^3*u^2+4538*x^2*y*u^2-F(29)/64*x*y^2*u^2-F(99)/8*y^3*u^2-F(98)/99*x^2*z*u^2-F(21)/82*x*y*z*u^2+F(13)/95*y^2*z*u^2+F(91)/16*x*z^2*u^2-F(7)/108*y*z^2*u^2+F(31)/25*z^3*u^2+6185*x^2*u^3+7961*x*y*u^3-6550*y^2*u^3-F(76)/93*x*z*u^3+F(71)/125*y*z*u^3+F(21)/82*z^2*u^3+F(53)/3*x*u^4+F(69)/80*y*u^4-15930*z*u^4-F(67)/35*u^5+4716*x^4*v-14163*x^3*y*v-F(9)/62*x^2*y^2*v-131*x*y^3*v+10231*y^4*v+9742*x^3*z*v+473*x^2*y*z*v-3697*x*y^2*z*v+F(83)/97*y^3*z*v+F(79)/65*x^2*z^2*v-7704*x*y*z^2*v-1973*y^2*z^2*v+14154*x*z^3*v-F(117)/83*y*z^3*v+F(63)/79*z^4*v+F(119)/101*x^3*u*v+1564*x^2*y*u*v-4245*x*y^2*u*v+2738*y^3*u*v+F(58)/73*x^2*z*u*v+14243*x*y*z*u*v+F(72)/7*y^2*z*u*v-10040*x*z^2*u*v+1398*y*z^2*u*v+7351*z^3*u*v+F(69)/104*x^2*u^2*v+3004*x*y*u^2*v+F(24)/49*y^2*u^2*v-14618*x*z*u^2*v+F(58)/59*y*z*u^2*v+F(17)/86*z^2*u^2*v-F(47)/57*x*u^3*v-4981*y*u^3*v-12979*z*u^3*v+4600*u^4*v+1327*x^3*v^2-F(49)/26*x^2*y*v^2-F(7)/9*x*y^2*v^2+F(41)/114*y^3*v^2-F(23)/37*x^2*z*v^2-F(27)/38*x*y*z*v^2-13041*y^2*z*v^2-9082*x*z^2*v^2-F(16)/43*y*z^2*v^2-F(122)/19*z^3*v^2+240*x^2*u*v^2-1111*x*y*u*v^2-F(73)/116*y^2*u*v^2+15428*x*z*u*v^2+F(122)/93*y*z*u*v^2-14730*z^2*u*v^2-8911*x*u^2*v^2+F(26)/45*y*u^2*v^2-F(96)/59*z*u^2*v^2-F(105)/58*u^3*v^2+F(27)/106*x^2*v^3-F(1)/50*x*y*v^3+F(63)/41*y^2*v^3+F(83)/124*x*z*v^3-F(79)/15*y*z*v^3+232*z^2*v^3-F(27)/122*x*u*v^3-F(29)/80*y*u*v^3+F(77)/104*z*u*v^3-11619*u^2*v^3+F(31)/64*x*v^4-F(28)/15*y*v^4+F(109)/53*z*v^4+F(87)/124*u*v^4-5605*v^5,-F(10)/119*x^4*y-12700*x^3*y^2+2955*x^2*y^3-5572*x*y^4+F(67)/47*y^5-717*x^4*z-14858*x^3*y*z-14107*x^2*y^2*z+F(41)/13*x*y^3*z+F(21)/26*y^4*z-F(115)/11*x^3*z^2-F(89)/13*x^2*y*z^2-F(77)/37*x*y^2*z^2-8177*y^3*z^2+15545*x^2*z^3+12771*x*y*z^3-9558*y^2*z^3+4785*x*z^4-F(80)/71*y*z^4-F(69)/79*z^5-5172*x^4*u+F(28)/73*x^3*y*u-F(100)/111*x^2*y^2*u-F(83)/38*x*y^3*u+F(85)/106*y^4*u-F(54)/115*x^3*z*u-F(45)/58*x^2*y*z*u+10171*x*y^2*z*u+8745*y^3*z*u-F(47)/67*x^2*z^2*u+8817*x*y*z^2*u+6145*y^2*z^2*u-F(101)/98*x*z^3*u+14581*y*z^3*u+F(101)/25*z^4*u-14704*x^3*u^2+F(33)/5*x^2*y*u^2-F(83)/113*x*y^2*u^2+F(65)/19*y^3*u^2+F(79)/5*x^2*z*u^2-F(47)/105*x*y*z*u^2-F(55)/61*y^2*z*u^2-13270*x*z^2*u^2+F(124)/119*y*z^2*u^2-F(119)/40*z^3*u^2+F(123)/92*x^2*u^3+9159*x*y*u^3+F(21)/61*y^2*u^3+F(94)/43*x*z*u^3+9266*y*z*u^3+F(76)/101*z^2*u^3+3274*x*u^4+2602*y*u^4-F(29)/64*z*u^4-F(22)/79*u^5+F(118)/13*x^3*y*v-F(93)/11*x^2*y^2*v-F(125)/48*x*y^3*v+5228*y^4*v+7559*x^3*z*v-8269*x^2*y*z*v+14443*x*y^2*z*v+F(17)/78*y^3*z*v+F(56)/83*x^2*z^2*v+F(50)/9*x*y*z^2*v-3651*y^2*z^2*v-14421*x*z^3*v+F(91)/17*y*z^3*v+F(18)/77*z^4*v+8321*x^3*u*v-F(65)/114*x^2*y*u*v+619*x*y^2*u*v+F(52)/67*y^3*u*v+1984*x^2*z*u*v-F(50)/27*x*y*z*u*v+F(115)/33*y^2*z*u*v-F(93)/112*x*z^2*u*v+F(107)/33*y*z^2*u*v+4353*z^3*u*v+536*x^2*u^2*v+6045*x*y*u^2*v-F(59)/123*y^2*u^2*v-11080*x*z*u^2*v-4522*y*z*u^2*v+F(78)/101*z^2*u^2*v+14050*x*u^3*v+12462*y*u^3*v+5355*z*u^3*v-4865*u^4*v-F(85)/93*x^2*y*v^2-3486*x*y^2*v^2-F(95)/51*y^3*v^2+13357*x^2*z*v^2-F(109)/119*x*y*z*v^2-F(43)/33*y^2*z*v^2+14478*x*z^2*v^2-F(121)/46*y*z^2*v^2-F(118)/25*z^3*v^2-F(72)/43*x^2*u*v^2+F(46)/29*x*y*u*v^2+7831*y^2*u*v^2+4352*x*z*u*v^2+F(31)/81*y*z*u*v^2-15185*z^2*u*v^2+F(2)/35*x*u^2*v^2-1497*y*u^2*v^2-F(56)/79*z*u^2*v^2+4330*u^3*v^2-F(53)/102*x*y*v^3+F(25)/87*y^2*v^3-6033*x*z*v^3+F(77)/23*y*z*v^3-F(79)/99*z^2*v^3-13542*x*u*v^3+F(126)/37*y*u*v^3-F(121)/78*z*u*v^3-14557*u^2*v^3+F(45)/73*y*v^4-7230*z*v^4-12756*u*v^4,-9114*x^4*z+F(11)/62*x^3*y*z+3415*x^2*y^2*z+F(107)/76*x*y^3*z+F(118)/15*y^4*z+F(94)/85*x^3*z^2-F(86)/45*x^2*y*z^2+1154*x*y^2*z^2-F(109)/84*y^3*z^2-F(97)/47*x^2*z^3-2079*x*y*z^3-7238*y^2*z^3-F(4)/99*x*z^4+3144*y*z^4+F(47)/24*z^5-F(52)/19*x^4*u-468*x^3*y*u-F(17)/83*x^2*y^2*u-7492*x*y^3*u-F(47)/3*y^4*u-F(103)/123*x^3*z*u+1220*x^2*y*z*u-F(53)/77*x*y^2*z*u+F(67)/120*y^3*z*u-F(11)/81*x^2*z^2*u+3753*x*y*z^2*u+F(117)/85*y^2*z^2*u+F(126)/41*x*z^3*u-F(1)/68*y*z^3*u+8299*z^4*u-F(4)/87*x^3*u^2-7458*x^2*y*u^2+3218*x*y^2*u^2-F(13)/22*y^3*u^2+F(121)/17*x^2*z*u^2-F(29)/75*x*y*z*u^2+14858*y^2*z*u^2+1710*x*z^2*u^2+F(38)/47*y*z^2*u^2-F(41)/106*z^3*u^2+F(73)/41*x^2*u^3+F(15)/112*x*y*u^3-F(30)/91*y^2*u^3-F(92)/103*x*z*u^3+F(53)/2*y*z*u^3-F(101)/51*z^2*u^3-692*x*u^4+4785*y*u^4+F(87)/125*z*u^4-5956*u^5-F(100)/77*x^3*z*v-1427*x^2*y*z*v+F(74)/23*x*y^2*z*v+9019*y^3*z*v+4687*x^2*z^2*v-F(23)/73*x*y*z^2*v+F(101)/8*y^2*z^2*v+1676*x*z^3*v+F(111)/89*y*z^3*v+F(45)/52*z^4*v+F(68)/11*x^3*u*v-F(12)/121*x^2*y*u*v-F(60)/37*x*y^2*u*v+9579*y^3*u*v-F(75)/83*x^2*z*u*v+F(15)/41*x*y*z*u*v+F(11)/43*y^2*z*u*v-F(32)/119*x*z^2*u*v-F(31)/89*y*z^2*u*v-F(78)/49*z^3*u*v-7594*x^2*u^2*v+F(115)/24*x*y*u^2*v-7714*y^2*u^2*v+F(121)/111*x*z*u^2*v+5250*y*z*u^2*v+F(46)/93*z^2*u^2*v+F(123)/74*x*u^3*v-F(37)/19*y*u^3*v-848*z*u^3*v+F(70)/97*u^4*v+F(3)/71*x^2*z*v^2-F(107)/7*x*y*z*v^2-F(120)/73*y^2*z*v^2+8686*x*z^2*v^2+F(57)/8*y*z^2*v^2-13183*z^3*v^2+F(100)/23*x^2*u*v^2+10049*x*y*u*v^2-4133*y^2*u*v^2+F(22)/75*x*z*u*v^2-F(9)/100*y*z*u*v^2-F(113)/54*z^2*u*v^2-F(31)/73*x*u^2*v^2+2014*y*u^2*v^2-F(94)/17*z*u^2*v^2-F(14)/3*u^3*v^2-F(99)/65*x*z*v^3-F(49)/111*y*z*v^3+F(27)/14*z^2*v^3+F(1)/21*x*u*v^3+12722*y*u*v^3-F(115)/27*z*u*v^3+8396*u^2*v^3-F(35)/107*z*v^4+F(58)/101*u*v^4,-F(107)/87*x^4*u-12111*x^3*y*u-F(61)/19*x^2*y^2*u+12692*x*y^3*u-F(83)/44*y^4*u+7091*x^3*z*u+15628*x^2*y*z*u+F(117)/62*x*y^2*z*u-F(1)/23*y^3*z*u+F(60)/31*x^2*z^2*u+13983*x*y*z^2*u-F(118)/103*y^2*z^2*u+10063*x*z^3*u-10874*y*z^3*u+F(44)/43*z^4*u-F(45)/26*x^3*u^2+F(108)/79*x^2*y*u^2-F(41)/36*x*y^2*u^2-F(101)/33*y^3*u^2-7434*x^2*z*u^2+F(83)/58*x*y*z*u^2+F(19)/70*y^2*z*u^2-F(124)/21*x*z^2*u^2+162*y*z^2*u^2+5165*z^3*u^2+6722*x^2*u^3-2654*x*y*u^3-1222*y^2*u^3-F(38)/49*x*z*u^3+206*y*z*u^3-F(98)/93*z^2*u^3-2833*x*u^4+11645*y*u^4+F(23)/39*z*u^4-F(90)/107*u^5+717*x^4*v-F(120)/43*x^3*y*v+F(76)/99*x^2*y^2*v+F(108)/107*x*y^3*v-F(69)/82*y^4*v+F(115)/11*x^3*z*v+13602*x^2*y*z*v-F(64)/13*x*y^2*z*v+F(3)/65*y^3*z*v-15545*x^2*z^2*v-F(80)/91*x*y*z^2*v+F(7)/36*y^2*z^2*v-4785*x*z^3*v-F(71)/108*y*z^3*v+F(69)/79*z^4*v+F(64)/125*x^3*u*v-4745*x^2*y*u*v+F(97)/94*x*y^2*u*v+F(58)/33*y^3*u*v+11892*x^2*z*u*v+6447*x*y*z*u*v+F(43)/109*y^2*z*u*v+13297*x*z^2*u*v+F(115)/7*y*z^2*u*v-9057*z^3*u*v-1694*x^2*u^2*v-8459*x*y*u^2*v-F(1)/20*y^2*u^2*v-F(115)/56*x*z*u^2*v+F(109)/24*y*z*u^2*v-F(17)/28*z^2*u^2*v-7761*x*u^3*v-15473*y*u^3*v-F(69)/10*z*u^3*v+4021*u^4*v-7559*x^3*v^2-F(29)/73*x^2*y*v^2-F(3)/124*x*y^2*v^2-F(113)/23*y^3*v^2-F(56)/83*x^2*z*v^2+F(16)/81*x*y*z*v^2-F(53)/113*y^2*z*v^2+14421*x*z^2*v^2+F(21)/79*y*z^2*v^2-F(18)/77*z^3*v^2+F(97)/99*x^2*u*v^2-F(73)/105*x*y*u*v^2-F(58)/83*y^2*u*v^2-3378*x*z*u*v^2+11175*y*z*u*v^2-11838*z^2*u*v^2-F(111)/119*x*u^2*v^2+5182*y*u^2*v^2-15227*z*u^2*v^2+F(89)/95*u^3*v^2-13357*x^2*v^3-1634*x*y*v^3-102*y^2*v^3-14478*x*z*v^3-F(59)/117*y*z*v^3+F(118)/25*z^2*v^3+F(70)/37*x*u*v^3-15255*y*u*v^3+F(81)/86*z*u*v^3+10713*u^2*v^3+6033*x*v^4+F(121)/4*y*v^4+F(79)/99*z*v^4+F(38)/41*u*v^4+7230*v^5,-F(14)/57*x^5-F(110)/19*x^4*y-F(7)/10*x^3*y^2+5488*x^2*y^3+1924*x*y^4-7352*x^4*z+F(79)/32*x^3*y*z-F(116)/105*x^2*y^2*z+F(116)/103*x*y^3*z+F(97)/67*x^3*z^2-10718*x^2*y*z^2-F(45)/88*x*y^2*z^2-F(61)/110*x^2*z^3-F(56)/75*x*y*z^3-F(77)/47*x*z^4-F(101)/9*x^4*u+F(123)/64*x^3*y*u+F(35)/92*x^2*y^2*u-3275*x*y^3*u-14779*x^3*z*u+3021*x^2*y*z*u+F(116)/63*x*y^2*z*u+5301*x^2*z^2*u-13217*x*y*z^2*u+12453*x*z^3*u-F(89)/98*x^3*u^2+F(32)/125*x^2*y*u^2+13344*x*y^2*u^2-1265*x^2*z*u^2+F(89)/70*x*y*z*u^2-F(95)/32*x*z^2*u^2-1763*x^2*u^3+10812*x*y*u^3+F(53)/43*x*z*u^3-F(53)/22*x*u^4-6290*x^4*v+2640*x^3*y*v+F(115)/62*x^2*y^2*v-F(69)/25*x*y^3*v-F(37)/21*y^4*v+10805*x^3*z*v+F(73)/60*x^2*y*z*v+F(83)/105*x*y^2*z*v+13982*y^3*z*v+F(9)/79*x^2*z^2*v-F(45)/91*x*y*z^2*v+10949*y^2*z^2*v+F(42)/95*x*z^3*v-F(61)/123*y*z^3*v-F(42)/95*z^4*v+F(73)/61*x^3*u*v-F(23)/9*x^2*y*u*v-F(10)/63*x*y^2*u*v-F(103)/118*y^3*u*v-F(21)/113*x^2*z*u*v+12120*x*y*z*u*v-F(3)/104*y^2*z*u*v+3126*x*z^2*u*v-F(1)/39*y*z^2*u*v-F(61)/124*z^3*u*v+10171*x^2*u^2*v+F(29)/10*x*y*u^2*v+F(27)/25*y^2*u^2*v+F(1)/93*x*z*u^2*v+F(83)/4*y*z*u^2*v+12889*z^2*u^2*v-F(65)/116*x*u^3*v+F(91)/123*y*u^3*v-2228*z*u^3*v+11384*u^4*v-F(64)/15*x^3*v^2-F(63)/116*x^2*y*v^2-F(75)/68*x*y^2*v^2+F(81)/91*y^3*v^2-F(123)/49*x^2*z*v^2-F(104)/79*x*y*z*v^2+F(124)/41*y^2*z*v^2-F(37)/126*x*z^2*v^2-F(107)/71*y*z^2*v^2+2980*z^3*v^2+4536*x^2*u*v^2-2542*x*y*u*v^2+F(107)/36*y^2*u*v^2-11164*x*z*u*v^2-15523*y*z*u*v^2+2102*z^2*u*v^2+F(4)/93*x*u^2*v^2-11356*y*u^2*v^2+F(7)/75*z*u^2*v^2+F(37)/86*u^3*v^2+F(85)/18*x^2*v^3+F(73)/23*x*y*v^3-F(3)/52*y^2*v^3-F(46)/7*x*z*v^3+6678*y*z*v^3-F(86)/69*z^2*v^3-2890*x*u*v^3+F(113)/109*y*u*v^3-10012*z*u*v^3-F(82)/43*u^2*v^3+9905*x*v^4+3597*y*v^4+F(29)/12*z*v^4+F(80)/79*u*v^4+F(83)/53*v^5,F(107)/87*x^5+12111*x^4*y+F(61)/19*x^3*y^2-12692*x^2*y^3+F(83)/44*x*y^4-F(10)/89*x^4*z-F(13)/30*x^3*y*z-318*x^2*y^2*z-F(117)/59*x*y^3*z-2859*y^4*z-F(14)/3*x^3*z^2-6985*x^2*y*z^2-F(5)/51*x*y^2*z^2+F(89)/100*y^3*z^2-9374*x^2*z^3-F(23)/21*x*y*z^3+3778*y^2*z^3-7596*x*z^4+12872*y*z^4+4420*z^5+F(97)/92*x^4*u-F(118)/53*x^3*y*u+12462*x^2*y^2*u+F(7)/73*x*y^3*u-F(35)/107*y^4*u-5804*x^3*z*u+F(53)/90*x^2*y*z*u-12831*x*y^2*z*u-12556*y^3*z*u-5880*x^2*z^2*u+F(83)/55*x*y*z^2*u-F(119)/72*y^2*z^2*u+15128*x*z^3*u+F(87)/28*y*z^3*u+F(103)/97*z^4*u-F(115)/97*x^3*u^2+15917*x^2*y*u^2-12640*x*y^2*u^2+F(89)/63*y^3*u^2+F(64)/49*x^2*z*u^2+2200*x*y*z*u^2+F(119)/64*y^2*z*u^2-10741*x*z^2*u^2+F(118)/57*y*z^2*u^2-5365*z^3*u^2+F(26)/99*x^2*u^3-F(113)/102*x*y*u^3-9487*y^2*u^3-8592*x*z*u^3-2215*y*z*u^3-838*z^2*u^3+3443*x*u^4+F(67)/112*y*u^4+13315*z*u^4+5435*u^5-F(64)/41*x^4*v-2227*x^3*y*v-F(29)/96*x^2*y^2*v-F(65)/96*x*y^3*v+5111*y^4*v-F(39)/14*x^3*z*v-3579*x^2*y*z*v-F(82)/25*x*y^2*z*v-F(41)/69*y^3*z*v+14297*x^2*z^2*v+F(74)/85*x*y*z^2*v-3775*y^2*z^2*v+F(3)/46*x*z^3*v-2502*y*z^3*v+F(8)/67*z^4*v+9485*x^3*u*v+F(32)/61*x^2*y*u*v-13187*x*y^2*u*v-F(80)/9*y^3*u*v-F(125)/126*x^2*z*u*v+F(41)/40*x*y*z*u*v-12624*y^2*z*u*v+F(72)/107*x*z^2*u*v-758*y*z^2*u*v+F(47)/43*z^3*u*v+3625*x^2*u^2*v+F(89)/107*x*y*u^2*v+4198*y^2*u^2*v-8877*x*z*u^2*v+4535*y*z*u^2*v+6430*z^2*u^2*v-3315*x*u^3*v+F(42)/47*y*u^3*v-F(48)/95*z*u^3*v+F(39)/86*u^4*v+F(109)/8*x^3*v^2-9639*x^2*y*v^2-6056*x*y^2*v^2-F(87)/23*y^3*v^2+3699*x^2*z*v^2-6447*x*y*z*v^2-F(9)/109*y^2*z*v^2-F(75)/107*x*z^2*v^2-F(9)/62*y*z^2*v^2-F(5)/99*z^3*v^2+F(19)/99*x^2*u*v^2-F(106)/105*x*y*u*v^2-F(10)/37*y^2*u*v^2-F(39)/14*x*z*u*v^2+10148*y*z*u*v^2+6501*z^2*u*v^2-2378*x*u^2*v^2-5219*y*u^2*v^2+F(61)/120*z*u^2*v^2-13078*u^3*v^2+358*x^2*v^3+F(97)/11*x*y*v^3+930*y^2*v^3+F(112)/47*x*z*v^3+F(2)/13*y*z*v^3+F(47)/90*z^2*v^3-57*x*u*v^3-12905*y*u*v^3+F(106)/11*z*u*v^3-F(125)/86*u^2*v^3-F(46)/113*x*v^4+13618*y*v^4+F(79)/21*z*v^4+F(39)/110*u*v^4+1483*v^5,12328*x^4*y+F(76)/125*x^3*y^2+F(47)/57*x^2*y^3-14322*x*y^4-7959*y^5-12923*x^4*z-F(35)/104*x^3*y*z+F(59)/86*x^2*y^2*z-F(52)/73*x*y^3*z+5085*y^4*z+13737*x^3*z^2-F(11)/70*x^2*y*z^2+4087*x*y^2*z^2-F(62)/57*y^3*z^2+5064*x^2*z^3+12417*x*y*z^3+776*y^2*z^3-F(75)/119*x*z^4+F(125)/18*y*z^4-5541*z^5+F(98)/27*x^4*u-F(49)/40*x^3*y*u-F(49)/31*x^2*y^2*u-7777*x*y^3*u-F(17)/126*y^4*u-F(7)/27*x^3*z*u+F(120)/53*x^2*y*z*u+F(36)/125*x*y^2*z*u-107*y^3*z*u+F(49)/94*x^2*z^2*u+F(63)/5*x*y*z^2*u-10113*y^2*z^2*u-13131*x*z^3*u-6*y*z^3*u+F(56)/13*z^4*u+F(20)/93*x^3*u^2+F(37)/100*x^2*y*u^2-F(25)/46*x*y^2*u^2+661*y^3*u^2-F(77)/10*x^2*z*u^2+F(59)/79*x*y*z*u^2-F(89)/70*y^2*z*u^2+F(33)/86*x*z^2*u^2+1258*y*z^2*u^2+7126*z^3*u^2+F(86)/101*x^2*u^3+14731*x*y*u^3+F(94)/61*y^2*u^3+F(91)/19*x*z*u^3+F(64)/47*y*z*u^3+12152*z^2*u^3+11835*x*u^4-F(34)/41*y*u^4-F(113)/83*z*u^4-F(115)/24*u^5+13135*x^4*v-F(82)/77*x^3*y*v+F(91)/19*x^2*y^2*v+F(121)/113*x*y^3*v+F(7)/65*y^4*v-F(16)/111*x^3*z*v-F(70)/61*x^2*y*z*v-F(40)/13*x*y^2*z*v-7692*y^3*z*v+F(117)/83*x^2*z^2*v+110*x*y*z^2*v-F(49)/109*y^2*z^2*v-1204*x*z^3*v+F(121)/102*y*z^3*v-F(10)/117*z^4*v+6372*x^3*u*v-14978*x^2*y*u*v-11700*x*y^2*u*v-F(105)/82*y^3*u*v-F(55)/126*x^2*z*u*v-F(80)/17*x*y*z*u*v-F(77)/90*y^2*z*u*v-F(30)/49*x*z^2*u*v+F(97)/8*y*z^2*u*v-F(89)/124*z^3*u*v-F(85)/107*x^2*u^2*v-F(125)/101*x*y*u^2*v-F(121)/9*y^2*u^2*v-F(49)/72*x*z*u^2*v+190*y*z*u^2*v+F(126)/101*z^2*u^2*v-10718*x*u^3*v-4387*y*u^3*v-F(45)/4*z*u^3*v-F(16)/111*u^4*v+F(81)/68*x^3*v^2-F(14)/23*x^2*y*v^2+9832*x*y^2*v^2-F(45)/47*y^3*v^2+F(23)/17*x^2*z*v^2-565*x*y*z*v^2+3597*y^2*z*v^2+F(79)/106*x*z^2*v^2-6472*y*z^2*v^2+F(81)/19*z^3*v^2+F(22)/15*x^2*u*v^2-8744*x*y*u*v^2+4213*y^2*u*v^2-F(110)/43*x*z*u*v^2-F(67)/121*y*z*u*v^2+2009*z^2*u*v^2+F(34)/125*x*u^2*v^2-F(58)/77*y*u^2*v^2-F(81)/34*z*u^2*v^2+1244*u^3*v^2+F(53)/23*x^2*v^3+F(57)/25*x*y*v^3-F(5)/79*y^2*v^3-F(59)/100*x*z*v^3+F(122)/75*y*z*v^3-F(103)/53*z^2*v^3-F(115)/57*x*u*v^3-15745*y*u*v^3+14890*z*u*v^3-F(53)/75*u^2*v^3-F(122)/25*x*v^4-F(43)/80*y*v^4+F(16)/49*z*v^4-2802*u*v^4+6645*v^5,F(125)/118*x^5-10590*x^4*y+F(16)/55*x^3*y^2+12163*x^2*y^3+2589*x*y^4+F(90)/37*y^5+F(103)/2*x^4*z-F(18)/53*x^3*y*z+5060*x^2*y^2*z-F(41)/21*x*y^3*z+F(7)/23*y^4*z+F(73)/119*x^3*z^2+9106*x^2*y*z^2+F(121)/37*x*y^2*z^2-3433*y^3*z^2+7053*x^2*z^3-F(63)/76*x*y*z^3-208*y^2*z^3-F(113)/68*x*z^4-F(9)/41*y*z^4-F(25)/51*z^5-F(76)/45*x^4*u-15093*x^3*y*u+2707*x^2*y^2*u-F(92)/101*x*y^3*u-F(73)/33*y^4*u-F(122)/25*x^3*z*u-F(117)/49*x^2*y*z*u+F(60)/101*x*y^2*z*u-F(73)/62*y^3*z*u-F(81)/56*x^2*z^2*u+F(37)/65*x*y*z^2*u+F(34)/117*y^2*z^2*u+14121*x*z^3*u+F(101)/72*y*z^3*u+F(58)/49*z^4*u-F(116)/125*x^3*u^2+7218*x^2*y*u^2-F(25)/109*x*y^2*u^2+F(31)/106*y^3*u^2-11383*x^2*z*u^2+F(87)/44*x*y*z*u^2+F(87)/37*y^2*z*u^2-F(19)/83*x*z^2*u^2+F(9)/112*y*z^2*u^2+7737*z^3*u^2+7808*x^2*u^3-3088*x*y*u^3-6489*y^2*u^3+303*x*z*u^3+11979*y*z*u^3+7958*z^2*u^3+1109*x*u^4-14459*y*u^4-F(105)/44*z*u^4-F(91)/83*u^5-F(68)/31*x^4*v-F(22)/113*x^3*y*v-F(92)/67*x^2*y^2*v+5005*x*y^3*v-2445*y^4*v+F(51)/112*x^3*z*v+F(26)/111*x^2*y*z*v+F(114)/5*x*y^2*z*v-F(11)/101*y^3*z*v+F(59)/75*x^2*z^2*v-12060*x*y*z^2*v+F(87)/98*y^2*z^2*v+F(71)/117*x*z^3*v-1555*y*z^3*v+F(109)/92*z^4*v-F(76)/13*x^3*u*v-6872*x^2*y*u*v+F(109)/121*x*y^2*u*v+F(43)/108*y^3*u*v+F(21)/85*x^2*z*u*v+F(119)/67*x*y*z*u*v-11741*y^2*z*u*v+F(61)/72*x*z^2*u*v-7692*y*z^2*u*v-11662*z^3*u*v+2684*x^2*u^2*v-F(35)/41*x*y*u^2*v+7263*y^2*u^2*v+6269*x*z*u^2*v+F(78)/59*y*z*u^2*v-F(37)/89*z^2*u^2*v-F(77)/64*x*u^3*v-3337*y*u^3*v-F(52)/27*z*u^3*v+F(95)/109*u^4*v+F(1)/27*x^3*v^2+F(107)/73*x^2*y*v^2-F(97)/66*x*y^2*v^2-F(47)/107*y^3*v^2+F(7)/54*x^2*z*v^2+F(82)/33*x*y*z*v^2-F(44)/91*y^2*z*v^2-F(116)/55*x*z^2*v^2-F(108)/91*y*z^2*v^2-F(102)/95*z^3*v^2+14672*x^2*u*v^2-F(89)/72*x*y*u*v^2-5855*y^2*u*v^2+3962*x*z*u*v^2+F(53)/116*y*z*u*v^2+7291*z^2*u*v^2+F(7)/67*x*u^2*v^2+7088*y*u^2*v^2+F(39)/4*z*u^2*v^2-13778*u^3*v^2-13736*x^2*v^3+F(12)/11*x*y*v^3+F(95)/47*y^2*v^3+F(23)/13*x*z*v^3+F(77)/38*y*z*v^3+F(1)/28*z^2*v^3-F(69)/89*x*u*v^3-F(99)/62*y*u*v^3+F(37)/116*z*u*v^3+5457*u^2*v^3-F(39)/83*x*v^4-F(61)/4*y*v^4+F(54)/115*z*v^4+5691*u*v^4+11764*v^5,F(19)/54*x^5-612*x^4*y+F(83)/101*x^3*y^2+12943*x^2*y^3-11877*x*y^4-11436*x^4*z+12700*x^3*y*z-9654*x^2*y^2*z+F(109)/106*x*y^3*z-5404*y^4*z+14238*x^3*z^2-3537*x^2*y*z^2+F(47)/55*x*y^2*z^2+F(65)/53*y^3*z^2-F(19)/40*x^2*z^3+7736*x*y*z^3+F(16)/17*y^2*z^3+F(53)/123*x*z^4-11499*y*z^4+F(123)/46*z^5+4087*x^4*u+F(49)/111*x^3*y*u+8992*x^2*y^2*u+F(73)/114*x*y^3*u-6839*y^4*u-246*x^3*z*u-11079*x^2*y*z*u-7029*x*y^2*z*u+14918*y^3*z*u+705*x^2*z^2*u+F(121)/19*x*y*z^2*u+F(42)/113*y^2*z^2*u+F(100)/87*x*z^3*u-5037*y*z^3*u+4448*z^4*u+F(112)/25*x^3*u^2+15240*x^2*y*u^2+5671*x*y^2*u^2+7742*y^3*u^2+12733*x^2*z*u^2+13433*x*y*z*u^2+F(54)/43*y^2*z*u^2+739*x*z^2*u^2+F(12)/79*y*z^2*u^2+F(53)/64*z^3*u^2+6613*x^2*u^3-F(40)/121*x*y*u^3-F(101)/38*y^2*u^3-15291*x*z*u^3-F(93)/53*y*z*u^3+10616*z^2*u^3+F(22)/67*x*u^4+F(57)/83*y*u^4+F(45)/7*z*u^4+5908*u^5-13077*x^4*v+F(57)/32*x^3*y*v+F(102)/79*x^2*y^2*v-15766*x*y^3*v-2019*y^4*v+F(47)/97*x^3*z*v+F(76)/119*x^2*y*z*v+F(113)/67*x*y^2*z*v+F(99)/101*y^3*z*v+F(38)/53*x^2*z^2*v+F(97)/45*x*y*z^2*v-F(116)/43*y^2*z^2*v+F(19)/74*x*z^3*v-2344*y*z^3*v+6436*z^4*v-1052*x^3*u*v-3509*x^2*y*u*v+F(118)/117*x*y^2*u*v+2532*y^3*u*v-58*x^2*z*u*v-F(51)/94*x*y*z*u*v-F(85)/44*y^2*z*u*v-3219*x*z^2*u*v+F(76)/9*y*z^2*u*v+180*z^3*u*v-F(87)/101*x^2*u^2*v-F(16)/5*x*y*u^2*v-9830*y^2*u^2*v+F(95)/109*x*z*u^2*v-F(114)/85*y*z*u^2*v-F(32)/43*z^2*u^2*v+F(37)/34*x*u^3*v+F(60)/11*y*u^3*v+32*z*u^3*v-4890*u^4*v-F(111)/76*x^3*v^2-F(39)/119*x^2*y*v^2+F(124)/15*x*y^2*v^2+2650*y^3*v^2-F(37)/84*x^2*z*v^2-F(25)/49*x*y*z*v^2+F(11)/34*y^2*z*v^2+F(51)/59*x*z^2*v^2+F(55)/26*y*z^2*v^2+12342*z^3*v^2+F(89)/48*x^2*u*v^2-F(31)/15*x*y*u*v^2-F(55)/89*y^2*u*v^2-7423*x*z*u*v^2+F(75)/79*y*z*u*v^2+F(18)/65*z^2*u*v^2-F(57)/82*x*u^2*v^2+F(107)/94*y*u^2*v^2+F(63)/8*z*u^2*v^2+F(82)/97*u^3*v^2-13915*x^2*v^3+F(38)/111*x*y*v^3-1833*y^2*v^3+2648*x*z*v^3+F(117)/73*y*z*v^3-F(97)/108*z^2*v^3-F(19)/81*x*u*v^3+F(31)/37*y*u*v^3-F(111)/109*z*u*v^3+F(77)/100*u^2*v^3+672*x*v^4+7851*y*v^4+5801*z*v^4-6533*u*v^4+15332*v^5,-F(19)/54*x^4*y+612*x^3*y^2-F(83)/101*x^2*y^3-12943*x*y^4+11877*y^5-7167*x^4*z+6801*x^3*y*z+F(101)/11*x^2*y^2*z-2536*x*y^3*z-10206*y^4*z-F(13)/43*x^3*z^2-12716*x^2*y*z^2+F(83)/102*x*y^2*z^2-11899*y^3*z^2+F(91)/76*x^2*z^3-7729*x*y*z^3+934*y^2*z^3+F(31)/41*x*z^4+F(47)/85*y*z^4-15079*z^5-1973*x^4*u-3935*x^3*y*u-F(10)/123*x^2*y^2*u-F(83)/86*x*y^3*u-F(47)/123*y^4*u-F(14)/99*x^3*z*u-10372*x^2*y*z*u+F(29)/121*x*y^2*z*u-838*y^3*z*u-F(92)/111*x^2*z^2*u-F(2)/21*x*y*z^2*u+2266*y^2*z^2*u+F(8)/89*x*z^3*u-14270*y*z^3*u+F(121)/34*z^4*u-9120*x^3*u^2+11326*x^2*y*u^2-F(73)/107*x*y^2*u^2-6358*y^3*u^2-F(19)/4*x^2*z*u^2-F(29)/85*x*y*z*u^2+6643*y^2*z*u^2+F(114)/73*x*z^2*u^2-F(115)/114*y*z^2*u^2+4753*z^3*u^2+F(7)/92*x^2*u^3+F(1)/56*x*y*u^3+F(66)/49*y^2*u^3+F(29)/34*x*z*u^3-14408*y*z*u^3+2486*z^2*u^3+F(21)/121*x*u^4-F(29)/48*y*u^4+2015*z*u^4-407*u^5+F(100)/49*x^4*v-F(26)/79*x^3*y*v-4595*x^2*y^2*v+14107*x*y^3*v+F(12)/59*y^4*v-1578*x^3*z*v-14430*x^2*y*z*v-15128*x*y^2*z*v+F(121)/42*y^3*z*v+4469*x^2*z^2*v-11396*x*y*z^2*v+F(53)/33*y^2*z^2*v-F(26)/113*x*z^3*v+F(118)/101*y*z^3*v-129*z^4*v+4809*x^3*u*v+894*x^2*y*u*v+F(90)/103*x*y^2*u*v-F(89)/2*y^3*u*v-F(82)/97*x^2*z*u*v-5766*x*y*z*u*v+12474*y^2*z*u*v+F(58)/81*x*z^2*u*v+F(7)/54*y*z^2*u*v+F(95)/84*z^3*u*v-F(61)/17*x^2*u^2*v-F(109)/84*x*y*u^2*v+F(55)/87*y^2*u^2*v-5232*x*z*u^2*v+F(122)/47*y*z*u^2*v-6815*z^2*u^2*v-F(107)/3*x*u^3*v+F(77)/3*y*u^3*v-F(80)/13*z*u^3*v+1221*u^4*v+11244*x^3*v^2+F(101)/86*x^2*y*v^2+F(20)/29*x*y^2*v^2+10817*y^3*v^2+77*x^2*z*v^2-F(5)/124*x*y*z*v^2+F(83)/34*y^2*z*v^2+4890*x*z^2*v^2-F(40)/109*y*z^2*v^2-F(20)/21*z^3*v^2+5014*x^2*u*v^2+9169*x*y*u*v^2-3969*y^2*u*v^2+F(73)/49*x*z*u*v^2-F(62)/53*y*z*u*v^2-4877*z^2*u*v^2-F(80)/77*x*u^2*v^2-F(18)/19*y*u^2*v^2+F(69)/97*z*u^2*v^2+F(59)/69*u^3*v^2-10567*x^2*v^3-F(16)/71*x*y*v^3+F(46)/57*y^2*v^3+F(55)/39*x*z*v^3+F(37)/82*y*z*v^3-10327*z^2*v^3-15043*x*u*v^3+7946*y*u*v^3+F(5)/86*z*u*v^3+F(79)/16*u^2*v^3+3707*x*v^4-2514*y*v^4-F(4)/97*z*v^4+F(43)/6*u*v^4-F(73)/47*v^5,-F(55)/38*x^4*y-F(92)/65*x^3*y^2+7060*x^2*y^3-F(85)/73*x*y^4+8092*y^5+F(79)/60*x^4*z+15112*x^3*y*z+1288*x^2*y^2*z+F(71)/117*x*y^3*z-10605*y^4*z-F(125)/119*x^3*z^2+F(69)/59*x^2*y*z^2-F(124)/27*x*y^2*z^2+12210*y^3*z^2-F(81)/74*x^2*z^3-13679*x*y*z^3-15250*y^2*z^3+F(1)/62*x*z^4-13460*y*z^4-F(62)/99*z^5+F(13)/47*x^3*y*u-F(89)/122*x^2*y^2*u-F(31)/120*x*y^3*u-F(5)/16*y^4*u-F(117)/22*x^3*z*u+142*x^2*y*z*u-8363*x*y^2*z*u+F(79)/115*y^3*z*u-15317*x^2*z^2*u-F(8)/23*x*y*z^2*u+F(90)/119*y^2*z^2*u-1622*x*z^3*u-F(96)/29*y*z^3*u-15636*z^4*u+F(52)/111*x^2*y*u^2-920*x*y^2*u^2+F(122)/37*y^3*u^2+F(34)/117*x^2*z*u^2+F(81)/98*x*y*z*u^2+4098*y^2*z*u^2+F(7)/23*x*z^2*u^2+12042*y*z^2*u^2-F(37)/103*z^3*u^2+8247*x*y*u^3-15003*y^2*u^3+F(66)/85*x*z*u^3-3089*y*z*u^3+2959*z^2*u^3-F(41)/109*y*u^4+F(37)/50*z*u^4+F(85)/26*x^3*y*v-F(116)/79*x^2*y^2*v+F(97)/42*x*y^3*v+4143*y^4*v-F(63)/17*x^3*z*v-6681*x^2*y*z*v+14124*x*y^2*z*v+F(103)/25*y^3*z*v+F(15)/73*x^2*z^2*v+F(111)/53*x*y*z^2*v-F(95)/61*y^2*z^2*v-F(34)/19*x*z^3*v-F(77)/114*y*z^3*v-2719*z^4*v+F(31)/34*x^2*y*u*v+F(44)/73*x*y^2*u*v-F(1)/103*y^3*u*v-F(87)/89*x^2*z*u*v-F(65)/46*x*y*z*u*v+2211*y^2*z*u*v+9298*x*z^2*u*v+F(38)/123*y*z^2*u*v-F(52)/101*z^3*u*v+F(84)/53*x*y*u^2*v+F(102)/11*y^2*u^2*v+F(50)/49*x*z*u^2*v+F(47)/68*y*z*u^2*v+13678*z^2*u^2*v-F(37)/36*y*u^3*v-13552*z*u^3*v-F(42)/5*x^2*y*v^2-14680*x*y^2*v^2+4274*y^3*v^2-13078*x^2*z*v^2+F(103)/15*x*y*z*v^2+12683*y^2*z*v^2-F(7)/15*x*z^2*v^2-F(101)/71*y*z^2*v^2+F(83)/75*z^3*v^2-F(121)/126*x*y*u*v^2+F(43)/63*y^2*u*v^2-F(29)/114*x*z*u*v^2+4501*y*z*u*v^2-10244*z^2*u*v^2-F(59)/107*y*u^2*v^2-F(105)/118*z*u^2*v^2+6782*x*y*v^3-F(68)/33*y^2*v^3+1425*x*z*v^3+F(11)/109*y*z*v^3+F(111)/107*z^2*v^3+F(106)/33*y*u*v^3+F(87)/71*z*u*v^3-F(4)/47*y*v^4-F(97)/103*z*v^4,-F(107)/87*x^4*z-12111*x^3*y*z-F(61)/19*x^2*y^2*z+12692*x*y^3*z-F(83)/44*y^4*z+7091*x^3*z^2+15628*x^2*y*z^2+F(117)/62*x*y^2*z^2-F(1)/23*y^3*z^2+F(60)/31*x^2*z^3+13983*x*y*z^3-F(118)/103*y^2*z^3+10063*x*z^4-10874*y*z^4+F(44)/43*z^5-F(45)/26*x^3*z*u+F(108)/79*x^2*y*z*u-F(41)/36*x*y^2*z*u-F(101)/33*y^3*z*u-7434*x^2*z^2*u+F(83)/58*x*y*z^2*u+F(19)/70*y^2*z^2*u-F(124)/21*x*z^3*u+162*y*z^3*u+5165*z^4*u+6722*x^2*z*u^2-2654*x*y*z*u^2-1222*y^2*z*u^2-F(38)/49*x*z^2*u^2+206*y*z^2*u^2-F(98)/93*z^3*u^2-2833*x*z*u^3+11645*y*z*u^3+F(23)/39*z^2*u^3-F(90)/107*z*u^4+F(55)/38*x^4*v+F(92)/65*x^3*y*v-7060*x^2*y^2*v+F(85)/73*x*y^3*v-8092*y^4*v-11908*x^3*z*v+15527*x^2*y*z*v+8030*x*y^2*z*v-F(63)/86*y^3*z*v-6682*x^2*z^2*v+F(36)/97*x*y*z^2*v+F(32)/27*y^2*z^2*v-F(78)/71*x*z^3*v-F(45)/116*y*z^3*v+2242*z^4*v-F(13)/47*x^3*u*v+F(89)/122*x^2*y*u*v+F(31)/120*x*y^2*u*v+F(5)/16*y^3*u*v-F(35)/12*x^2*z*u*v+F(59)/106*x*y*z*u*v-11795*y^2*z*u*v-423*x*z^2*u*v-5247*y*z^2*u*v-F(23)/18*z^3*u*v-F(52)/111*x^2*u^2*v+920*x*y*u^2*v-F(122)/37*y^2*u^2*v+F(13)/17*x*z*u^2*v-F(107)/50*y*z*u^2*v+5981*z^2*u^2*v-8247*x*u^3*v+15003*y*u^3*v+1892*z*u^3*v+F(41)/109*u^4*v-F(85)/26*x^3*v^2+F(116)/79*x^2*y*v^2-F(97)/42*x*y^2*v^2-4143*y^3*v^2-14970*x^2*z*v^2-3856*x*y*z*v^2+9280*y^2*z*v^2-F(120)/43*x*z^2*v^2+F(12)/101*y*z^2*v^2-F(31)/117*z^3*v^2-F(31)/34*x^2*u*v^2-F(44)/73*x*y*u*v^2+F(1)/103*y^2*u*v^2+F(1)/47*x*z*u*v^2+15325*y*z*u*v^2+1013*z^2*u*v^2-F(84)/53*x*u^2*v^2-F(102)/11*y*u^2*v^2-250*z*u^2*v^2+F(37)/36*u^3*v^2+F(42)/5*x^2*v^3+14680*x*y*v^3-4274*y^2*v^3+F(82)/67*x*z*v^3+4601*y*z*v^3-F(34)/59*z^2*v^3+F(121)/126*x*u*v^3-F(43)/63*y*u*v^3-F(25)/57*z*u*v^3+F(59)/107*u^2*v^3-6782*x*v^4+F(68)/33*y*v^4-F(114)/13*z*v^4-F(106)/33*u*v^4+F(4)/47*v^5,-F(107)/87*x^4*y-12111*x^3*y^2-F(61)/19*x^2*y^3+12692*x*y^4-F(83)/44*y^5+7091*x^3*y*z+15628*x^2*y^2*z+F(117)/62*x*y^3*z-F(1)/23*y^4*z+F(60)/31*x^2*y*z^2+13983*x*y^2*z^2-F(118)/103*y^3*z^2+10063*x*y*z^3-10874*y^2*z^3+F(44)/43*y*z^4-F(45)/26*x^3*y*u+F(108)/79*x^2*y^2*u-F(41)/36*x*y^3*u-F(101)/33*y^4*u-7434*x^2*y*z*u+F(83)/58*x*y^2*z*u+F(19)/70*y^3*z*u-F(124)/21*x*y*z^2*u+162*y^2*z^2*u+5165*y*z^3*u+6722*x^2*y*u^2-2654*x*y^2*u^2-1222*y^3*u^2-F(38)/49*x*y*z*u^2+206*y^2*z*u^2-F(98)/93*y*z^2*u^2-2833*x*y*u^3+11645*y^2*u^3+F(23)/39*y*z*u^3-F(90)/107*y*u^4+F(79)/60*x^4*v+F(49)/10*x^3*y*v-F(61)/78*x^2*y^2*v-F(71)/2*x*y^3*v+F(55)/78*y^4*v-F(125)/119*x^3*z*v+10128*x^2*y*z*v-F(83)/92*x*y^2*z*v+F(70)/59*y^3*z*v-F(81)/74*x^2*z^2*v-F(118)/81*x*y*z^2*v+F(34)/13*y^2*z^2*v+F(1)/62*x*z^3*v-F(29)/77*y*z^3*v-F(62)/99*z^4*v-F(117)/22*x^3*u*v-F(70)/57*x^2*y*u*v+F(37)/45*x*y^2*u*v+F(123)/107*y^3*u*v-15317*x^2*z*u*v-5987*x*y*z*u*v-F(27)/107*y^2*z*u*v-1622*x*z^2*u*v+F(59)/94*y*z^2*u*v-15636*z^3*u*v+F(34)/117*x^2*u^2*v-F(101)/21*x*y*u^2*v-10620*y^2*u^2*v+F(7)/23*x*z*u^2*v-F(7)/71*y*z*u^2*v-F(37)/103*z^2*u^2*v+F(66)/85*x*u^3*v-F(115)/107*y*u^3*v+2959*z*u^3*v+F(37)/50*u^4*v-F(63)/17*x^3*v^2-F(52)/99*x^2*y*v^2-F(58)/81*x*y^2*v^2-953*y^3*v^2+F(15)/73*x^2*z*v^2-8395*x*y*z*v^2+4542*y^2*z*v^2-F(34)/19*x*z^2*v^2-1807*y*z^2*v^2-2719*z^3*v^2-F(87)/89*x^2*u*v^2+F(109)/28*x*y*u*v^2-14455*y^2*u*v^2+9298*x*z*u*v^2+F(77)/26*y*z*u*v^2-F(52)/101*z^2*u*v^2+F(50)/49*x*u^2*v^2+12453*y*u^2*v^2+13678*z*u^2*v^2-13552*u^3*v^2-13078*x^2*v^3+F(37)/99*x*y*v^3-14707*y^2*v^3-F(7)/15*x*z*v^3-F(94)/5*y*z*v^3+F(83)/75*z^2*v^3-F(29)/114*x*u*v^3-F(95)/44*y*u*v^3-10244*z*u*v^3-F(105)/118*u^2*v^3+1425*x*v^4+F(107)/71*y*v^4+F(111)/107*z*v^4+F(87)/71*u*v^4-F(97)/103*v^5]))
end


@doc raw"""
    k3_d11_pi11_ss_2()

Return a smooth K3 surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and two 6-secants.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d11_pi11_ss_2()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^5*y-F(95)/29*x^4*y^2-F(77)/58*x^5*z-F(58)/91*x^4*y*z+F(37)/66*x^3*y^2*z-F(3)/106*x^2*y^3*z-F(9)/125*x^4*z^2-F(13)/64*x^3*y*z^2+10739*x^2*y^2*z^2-5110*x*y^3*z^2-F(61)/28*x^3*z^3-8411*x^2*y*z^3+F(124)/81*x*y^2*z^3-F(12)/35*x^2*z^4-F(3)/98*x*y*z^4+6988*x*z^5+F(41)/18*x^5*u+F(37)/47*x^4*y*u-F(9)/94*x^3*y^2*u+4165*x^4*z*u-F(7)/46*x^3*y*z*u-F(15)/98*x^2*y^2*z*u+7434*x*y^3*z*u+F(121)/14*x^3*z^2*u-9689*x^2*y*z^2*u-F(25)/21*x*y^2*z^2*u-8339*y^3*z^2*u-F(17)/18*x^2*z^3*u-F(79)/122*x*y*z^3*u-F(49)/99*y^2*z^3*u-5846*x*z^4*u+F(112)/3*y*z^4*u-F(116)/65*z^5*u+F(100)/29*x^4*u^2+F(35)/44*x^3*y*u^2-6039*x^2*y^2*u^2-F(43)/85*x^3*z*u^2+F(89)/42*x^2*y*z*u^2-9238*x*y^2*z*u^2-3482*y^3*z*u^2+F(93)/28*x^2*z^2*u^2-12761*x*y*z^2*u^2+15143*y^2*z^2*u^2+2345*x*z^3*u^2+8518*y*z^3*u^2+15690*z^4*u^2+F(113)/57*x^3*u^3-F(113)/55*x^2*y*u^3+13504*x^2*z*u^3-F(82)/13*x*y*z*u^3+F(72)/55*y^2*z*u^3-F(95)/82*x*z^2*u^3+F(55)/34*y*z^2*u^3-9067*z^3*u^3-8115*x^2*u^4+F(21)/4*x*z*u^4-F(118)/83*y*z*u^4-F(67)/94*z^2*u^4-14576*z*u^5-9102*x^5*v+F(25)/91*x^4*y*v+7945*x^3*y^2*v-F(66)/61*x^4*z*v+F(29)/75*x^3*y*z*v+F(58)/109*x^2*y^2*z*v+F(106)/43*x*y^3*z*v+1722*x^3*z^2*v+F(54)/37*x^2*y*z^2*v-F(37)/105*x*y^2*z^2*v-F(79)/107*y^3*z^2*v-F(55)/27*x^2*z^3*v+F(107)/90*x*y*z^3*v+15666*y^2*z^3*v-F(86)/113*x*z^4*v+F(57)/16*y*z^4*v+F(19)/11*z^5*v-F(87)/52*x^4*u*v-F(89)/96*x^3*y*u*v-F(44)/123*x^2*y^2*u*v-F(16)/55*x^3*z*u*v-13639*x^2*y*z*u*v+F(54)/89*x*y^2*z*u*v+F(121)/114*y^3*z*u*v+F(4)/123*x^2*z^2*u*v-F(102)/35*x*y*z^2*u*v-14412*y^2*z^2*u*v-F(112)/53*x*z^3*u*v-F(2)/59*y*z^3*u*v+F(123)/16*z^4*u*v+98*x^3*u^2*v+8545*x^2*y*u^2*v+13731*x^2*z*u^2*v+F(85)/33*x*y*z*u^2*v+F(119)/31*y^2*z*u^2*v+F(53)/26*x*z^2*u^2*v+10074*y*z^2*u^2*v+F(91)/110*z^3*u^2*v+1120*x^2*u^3*v+F(74)/87*x*z*u^3*v+F(100)/59*y*z*u^3*v+9579*z^2*u^3*v+6554*z*u^4*v+F(40)/113*x^4*v^2-F(58)/115*x^3*y*v^2-13135*x^2*y^2*v^2+8541*x^3*z*v^2-F(50)/43*x^2*y*z*v^2+F(40)/19*x*y^2*z*v^2-2896*y^3*z*v^2+2572*x^2*z^2*v^2+F(29)/10*x*y*z^2*v^2+F(65)/122*y^2*z^2*v^2+F(101)/116*x*z^3*v^2+F(125)/21*y*z^3*v^2+F(118)/93*z^4*v^2-13478*x^3*u*v^2-11618*x^2*y*u*v^2-F(87)/14*x^2*z*u*v^2+F(119)/37*x*y*z*u*v^2+F(104)/45*y^2*z*u*v^2+F(70)/43*x*z^2*u*v^2+8179*y*z^2*u*v^2+F(8)/69*z^3*u*v^2-F(77)/61*x^2*u^2*v^2-F(8)/95*x*z*u^2*v^2+F(22)/7*y*z*u^2*v^2+1608*z^2*u^2*v^2+F(1)/67*z*u^3*v^2-9689*x^3*v^3-2011*x^2*y*v^3+F(81)/55*x^2*z*v^3-F(1)/68*x*y*z*v^3+F(44)/3*y^2*z*v^3+F(29)/119*x*z^2*v^3+11480*y*z^2*v^3-14141*z^3*v^3-F(103)/36*x^2*u*v^3-F(47)/80*x*z*u*v^3+F(21)/10*y*z*u*v^3+15809*z^2*u*v^3+F(97)/8*z*u^2*v^3-F(62)/67*x^2*v^4-4791*x*z*v^4-F(15)/64*y*z*v^4-F(79)/85*z^2*v^4+F(113)/54*z*u*v^4+4193*z*v^5,F(49)/4*x^2*y^4-805*x*y^5+12829*x^2*y^3*z-7662*x*y^4*z-F(53)/24*y^5*z+F(107)/36*x^2*y^2*z^2-11341*x*y^3*z^2-F(77)/72*y^4*z^2+F(44)/17*x*y^2*z^3-7501*y^3*z^3-F(102)/101*y^2*z^4+F(37)/47*x^2*y^3*u-F(65)/2*x*y^4*u+F(73)/10*y^5*u-2620*x^2*y^2*z*u+F(112)/117*x*y^3*z*u+F(57)/91*y^4*z*u-F(16)/5*x*y^2*z^2*u-14068*y^3*z^2*u-1540*y^2*z^3*u+F(52)/27*x^2*y^2*u^2-13159*x*y^3*u^2-1211*y^4*u^2-F(78)/125*x*y^2*z*u^2-F(37)/20*y^3*z*u^2+4634*y^2*z^2*u^2+F(93)/116*x*y^2*u^3-F(104)/73*y^3*u^3+F(88)/87*y^2*z*u^3-12450*y^2*u^4+2172*x^3*y^2*v-F(35)/3*x^2*y^3*v-F(45)/77*x*y^4*v+F(114)/41*y^5*v+F(45)/73*x^3*y*z*v-14813*x^2*y^2*z*v-8812*x*y^3*z*v-3589*y^4*z*v+15112*x^2*y*z^2*v-14180*x*y^2*z^2*v-F(115)/11*y^3*z^2*v+F(79)/34*x*y*z^3*v-5113*y^2*z^3*v-15079*y*z^4*v-14255*x^3*y*u*v-824*x^2*y^2*u*v-F(122)/85*x*y^3*u*v-14499*y^4*u*v-7278*x^3*z*u*v-F(16)/39*x^2*y*z*u*v-10315*x*y^2*z*u*v-2747*y^3*z*u*v-2582*x^2*z^2*u*v-F(1)/81*x*y*z^2*u*v+5962*y^2*z^2*u*v+8667*x*z^3*u*v+F(92)/41*y*z^3*u*v-14083*z^4*u*v+F(37)/106*x^3*u^2*v-13622*x^2*y*u^2*v-F(47)/60*x*y^2*u^2*v+F(110)/59*y^3*u^2*v-F(73)/50*x^2*z*u^2*v+F(100)/109*x*y*z*u^2*v-F(112)/61*y^2*z*u^2*v-5661*x*z^2*u^2*v+F(15)/37*y*z^2*u^2*v+11169*z^3*u^2*v+F(7)/18*x^2*u^3*v-F(5)/19*x*y*u^3*v+12721*y^2*u^3*v-F(87)/119*x*z*u^3*v-9337*y*z*u^3*v-F(21)/74*z^2*u^3*v-14164*x*u^4*v+F(64)/49*y*u^4*v+1891*z*u^4*v+13871*u^5*v+F(111)/5*x^3*y*v^2+F(23)/107*x^2*y^2*v^2+7582*x*y^3*v^2-5168*y^4*v^2+9172*x^3*z*v^2+F(107)/9*x^2*y*z*v^2-F(12)/55*x*y^2*z*v^2+F(52)/93*y^3*z*v^2+F(69)/50*x^2*z^2*v^2-11809*x*y*z^2*v^2+14576*y^2*z^2*v^2+F(87)/121*x*z^3*v^2+F(72)/119*y*z^3*v^2-F(7)/122*z^4*v^2+8136*x^3*u*v^2+F(7)/95*x^2*y*u*v^2-12746*x*y^2*u*v^2-8695*y^3*u*v^2+F(76)/37*x^2*z*u*v^2+F(46)/107*x*y*z*u*v^2-9238*y^2*z*u*v^2-11973*x*z^2*u*v^2+F(88)/51*y*z^2*u*v^2+5449*z^3*u*v^2+F(35)/78*x^2*u^2*v^2-F(99)/68*x*y*u^2*v^2-15289*y^2*u^2*v^2-10333*x*z*u^2*v^2-1935*y*z*u^2*v^2-F(38)/53*z^2*u^2*v^2-4030*x*u^3*v^2+964*y*u^3*v^2-13527*z*u^3*v^2+F(49)/39*u^4*v^2-F(81)/7*x^3*v^3+8315*x^2*y*v^3+F(63)/83*x*y^2*v^3-F(41)/39*y^3*v^3-1651*x^2*z*v^3+F(21)/107*x*y*z*v^3-F(6)/107*y^2*z*v^3+211*x*z^2*v^3-6831*y*z^2*v^3-F(96)/107*z^3*v^3-5718*x^2*u*v^3-14140*x*y*u*v^3-12343*y^2*u*v^3-6100*x*z*u*v^3+4971*y*z*u*v^3+13291*z^2*u*v^3+5720*x*u^2*v^3+F(88)/103*y*u^2*v^3+F(65)/113*z*u^2*v^3-F(74)/57*u^3*v^3+F(107)/2*x^2*v^4+F(41)/59*x*y*v^4-F(43)/106*y^2*v^4-F(53)/54*x*z*v^4+F(35)/93*y*z*v^4+11080*z^2*v^4-12181*x*u*v^4-F(22)/47*y*u*v^4-F(25)/6*z*u*v^4+F(7)/8*u^2*v^4+5022*x*v^5-13652*y*v^5-F(83)/37*z*v^5+5978*u*v^5-F(120)/103*v^6,F(42)/89*x^4*y-2980*x^3*y^2+F(33)/74*x^2*y^3-F(83)/107*x*y^4-12479*x^4*z-F(121)/48*x^3*y*z-F(62)/69*x^2*y^2*z+F(5)/42*x*y^3*z+8339*y^4*z+F(100)/89*x^3*z^2-F(25)/11*x^2*y*z^2-10599*x*y^2*z^2+F(49)/99*y^3*z^2+F(116)/19*x^2*z^3-F(59)/123*x*y*z^3-F(112)/3*y^2*z^3+8455*x*z^4+F(116)/65*y*z^4-8845*x^4*u-F(8)/87*x^3*y*u+F(57)/50*x^2*y^2*u-F(23)/67*x*y^3*u+3482*y^4*u-F(49)/5*x^3*z*u-6145*x^2*y*z*u+F(37)/122*x*y^2*z*u-15143*y^3*z*u-F(24)/113*x^2*z^2*u+F(80)/119*x*y*z^2*u-8518*y^2*z^2*u+F(118)/99*x*z^3*u-15690*y*z^3*u+F(75)/122*x^3*u^2+F(57)/70*x^2*y*u^2-F(81)/34*x*y^2*u^2-F(72)/55*y^3*u^2-F(12)/97*x^2*z*u^2-F(79)/42*x*y*z*u^2-F(55)/34*y^2*z*u^2-10709*x*z^2*u^2+9067*y*z^2*u^2-F(113)/52*x^2*u^3+F(37)/105*x*y*u^3+F(118)/83*y^2*u^3+F(29)/26*x*z*u^3+F(67)/94*y*z*u^3+8966*x*u^4+14576*y*u^4-5390*x^4*v+F(104)/31*x^3*y*v+F(71)/35*x^2*y^2*v+F(125)/44*x*y^3*v-F(96)/115*y^4*v+15727*x^3*z*v+2729*x^2*y*z*v+14101*x*y^2*z*v+15375*y^3*z*v-11117*x^2*z^2*v+12771*x*y*z^2*v-F(1)/57*y^2*z^2*v-7372*x*z^3*v+11451*y*z^3*v-F(49)/46*x^3*u*v+2869*x^2*y*u*v+F(53)/81*x*y^2*u*v-F(26)/123*y^3*u*v-13578*x^2*z*u*v+9312*x*y*z*u*v+8621*y^2*z*u*v-F(101)/103*x*z^2*u*v+F(101)/74*y*z^2*u*v+F(68)/19*x^2*u^2*v-141*x*y*u^2*v+5919*y^2*u^2*v-F(22)/119*x*z*u^2*v+1799*y*z*u^2*v+309*x*u^3*v-7088*y*u^3*v+F(49)/73*x^3*v^2+F(50)/27*x^2*y*v^2+2177*x*y^2*v^2+9824*y^3*v^2-11940*x^2*z*v^2-F(91)/120*x*y*z*v^2+14581*y^2*z*v^2-F(66)/71*x*z^2*v^2-14196*y*z^2*v^2-F(94)/21*x^2*u*v^2-F(34)/49*x*y*u*v^2+F(102)/47*y^2*u*v^2-8286*x*z*u*v^2+F(52)/77*y*z*u*v^2+F(88)/49*x*u^2*v^2+F(106)/83*y*u^2*v^2-F(13)/49*x^2*v^3-F(44)/79*x*y*v^3-5610*y^2*v^3-4418*x*z*v^3+F(83)/44*y*z*v^3+F(63)/79*x*u*v^3+F(110)/7*y*u*v^3-F(85)/116*x*v^4-F(58)/69*y*v^4,F(62)/69*x^4*y-F(107)/40*x^3*y^2-F(28)/51*x^2*y^3-F(63)/40*x*y^4-5*x^4*z-5805*x^3*y*z-F(85)/2*x^2*y^2*z+6594*x*y^3*z-12448*y^4*z+F(30)/71*x^3*z^2+F(98)/39*x^2*y*z^2+1893*x*y^2*z^2+6234*y^3*z^2+F(111)/106*x^2*z^3-F(40)/69*x*y*z^3-12160*y^2*z^3+F(24)/89*x*z^4-F(16)/95*y*z^4-F(54)/29*z^5-F(103)/24*x^4*u+F(88)/103*x^3*y*u+F(112)/3*x^2*y^2*u-4053*x*y^3*u+691*y^4*u+F(58)/113*x^3*z*u+11910*x^2*y*z*u-9621*x*y^2*z*u+F(34)/65*y^3*z*u+14814*x^2*z^2*u+F(59)/32*x*y*z^2*u+F(76)/117*y^2*z^2*u-F(106)/21*x*z^3*u-F(84)/61*y*z^3*u-8429*z^4*u+4591*x^3*u^2+F(77)/123*x^2*y*u^2+5673*x*y^2*u^2-F(13)/90*y^3*u^2-6943*x^2*z*u^2-F(85)/106*x*y*z*u^2-7748*y^2*z*u^2-F(42)/23*x*z^2*u^2-F(17)/66*y*z^2*u^2+8558*z^3*u^2-F(91)/95*x^2*u^3-F(77)/85*x*y*u^3+6176*y^2*u^3+F(115)/93*x*z*u^3+6945*y*z*u^3+15549*z^2*u^3+2582*x*u^4-F(111)/106*y*u^4+F(107)/110*z*u^4-F(64)/103*u^5+3028*x^4*v+F(22)/15*x^3*y*v+4540*x^2*y^2*v-F(43)/102*x*y^3*v+F(77)/57*y^4*v+7623*x^3*z*v-9859*x^2*y*z*v+F(45)/43*x*y^2*z*v-F(82)/33*y^3*z*v-F(23)/80*x^2*z^2*v-F(9)/70*x*y*z^2*v-3472*y^2*z^2*v+F(32)/37*x*z^3*v+14876*y*z^3*v+F(101)/30*z^4*v+3176*x^3*u*v+F(109)/113*x^2*y*u*v-1635*x*y^2*u*v-F(3)/26*y^3*u*v+8336*x^2*z*u*v+3217*x*y*z*u*v+F(27)/115*y^2*z*u*v+6263*x*z^2*u*v+F(117)/28*y*z^2*u*v+1193*z^3*u*v+13657*x^2*u^2*v+14939*x*y*u^2*v-6039*y^2*u^2*v+F(115)/13*x*z*u^2*v-6224*y*z*u^2*v+5900*z^2*u^2*v+F(55)/97*x*u^3*v+F(87)/22*y*u^3*v+F(19)/85*z*u^3*v+F(61)/78*u^4*v-12448*x^3*v^2-F(85)/43*x^2*y*v^2-F(3)/31*x*y^2*v^2-8270*y^3*v^2+F(102)/71*x^2*z*v^2-F(22)/97*x*y*z*v^2+7065*y^2*z*v^2-F(35)/61*x*z^2*v^2+F(15)/88*y*z^2*v^2+12197*z^3*v^2+F(71)/66*x^2*u*v^2-F(39)/58*x*y*u*v^2+F(55)/106*y^2*u*v^2+3741*x*z*u*v^2+15029*y*z*u*v^2+F(99)/26*z^2*u*v^2+15394*x*u^2*v^2+28*y*u^2*v^2+F(18)/23*z*u^2*v^2+F(83)/28*u^3*v^2+F(41)/14*x^2*v^3+F(104)/93*x*y*v^3-F(48)/85*y^2*v^3-14719*x*z*v^3-10841*y*z*v^3-F(101)/43*z^2*v^3+196*x*u*v^3+7228*y*u*v^3-F(99)/4*z*u*v^3+F(13)/86*u^2*v^3+128*x*v^4+F(68)/63*y*v^4+4877*z*v^4+F(74)/49*u*v^4-7004*v^5,10597*x^4*y-F(98)/61*x^3*y^2-F(96)/73*x^2*y^3+F(3)/106*x*y^4-661*x^4*z-9058*x^3*y*z+1767*x^2*y^2*z-15568*x*y^3*z+5110*y^4*z-F(95)/37*x^3*z^2+F(32)/63*x^2*y*z^2+F(71)/44*x*y^2*z^2-F(124)/81*y^3*z^2+F(21)/16*x^2*z^3+2166*x*y*z^3+F(3)/98*y^2*z^3-3434*x*z^4-6988*y*z^4+F(55)/92*x^4*u-13314*x^3*y*u-F(43)/5*x^2*y^2*u-13689*x*y^3*u-11021*y^4*u+F(125)/91*x^3*z*u-F(55)/24*x^2*y*z*u+F(111)/26*x*y^2*z*u+F(15)/14*y^3*z*u-F(105)/62*x^2*z^2*u-F(17)/33*x*y*z^2*u+5093*y^2*z^2*u-F(91)/75*x*z^3*u-F(58)/81*y*z^3*u-8455*z^4*u+F(119)/116*x^3*u^2+F(61)/100*x^2*y*u^2+F(113)/123*x*y^2*u^2+F(111)/38*y^3*u^2+F(97)/101*x^2*z*u^2-F(55)/113*x*y*z*u^2+F(83)/124*y^2*z*u^2+F(39)/7*x*z^2*u^2-3421*y*z^2*u^2-F(118)/99*z^3*u^2+F(85)/7*x^2*u^3+58*x*y*u^3-F(75)/58*y^2*u^3-F(111)/50*x*z*u^3-F(123)/94*y*z*u^3+10709*z^2*u^3-F(31)/98*x*u^4-7394*y*u^4-F(29)/26*z*u^4-8966*u^5-F(73)/23*x^4*v-F(67)/47*x^3*y*v-F(103)/80*x^2*y^2*v-F(25)/97*x*y^3*v+F(118)/83*y^4*v+7202*x^3*z*v-F(97)/81*x^2*y*z*v+13070*x*y^2*z*v-12510*y^3*z*v-F(44)/47*x^2*z^2*v-4890*x*y*z^2*v+6444*y^2*z^2*v+4883*x*z^3*v+F(47)/96*y*z^3*v-F(44)/23*z^4*v-14170*x^3*u*v+14941*x^2*y*u*v-F(88)/23*x*y^2*u*v-3464*y^3*u*v-5164*x^2*z*u*v+F(62)/109*x*y*z*u*v+F(70)/123*y^2*z*u*v+1136*x*z^2*u*v-F(41)/58*y*z^2*u*v+3967*z^3*u*v-12729*x^2*u^2*v+F(22)/41*x*y*u^2*v+11875*y^2*u^2*v-15883*x*z*u^2*v-15439*y*z*u^2*v+F(3)/80*z^2*u^2*v+3882*x*u^3*v-15701*y*u^3*v-2684*z*u^3*v-F(43)/70*u^4*v+F(29)/114*x^3*v^2-F(19)/115*x^2*y*v^2-F(45)/8*x*y^2*v^2-F(80)/79*y^3*v^2-F(73)/40*x^2*z*v^2+F(119)/37*x*y*z*v^2+F(35)/121*y^2*z*v^2-41*x*z^2*v^2-F(19)/115*y*z^2*v^2-3224*z^3*v^2+F(57)/22*x^2*u*v^2-3970*x*y*u*v^2-F(17)/106*y^2*u*v^2-F(116)/59*x*z*u*v^2+F(106)/117*y*z*u*v^2-F(71)/90*z^2*u*v^2+2994*x*u^2*v^2-143*y*u^2*v^2-F(32)/57*z*u^2*v^2+12024*u^3*v^2-9175*x^2*v^3+F(75)/49*x*y*v^3-F(37)/4*y^2*v^3-F(83)/34*x*z*v^3+F(43)/80*y*z*v^3-F(123)/59*z^2*v^3+F(5)/53*x*u*v^3-7470*y*u*v^3-F(106)/17*z*u*v^3+F(2)/103*u^2*v^3-F(65)/59*x*v^4+F(73)/98*y*v^4+F(15)/107*z*v^4-F(113)/35*u*v^4-F(19)/21*v^5,-F(112)/81*x^4*y-F(49)/67*x^3*y^2+F(89)/67*x^2*y^3+F(110)/17*x*y^4-F(67)/36*x^4*z+8055*x^3*y*z-235*x^2*y^2*z+F(73)/106*x*y^3*z-F(25)/2*y^4*z-9354*x^3*z^2-11208*x^2*y*z^2+F(87)/34*x*y^2*z^2-644*y^3*z^2-15411*x^2*z^3-7003*x*y*z^3+F(74)/117*y^2*z^3+2773*x*z^4-12069*y*z^4-6020*z^5-F(61)/66*x^4*u+130*x^3*y*u+F(19)/39*x^2*y^2*u-7844*x*y^3*u-8096*y^4*u+5909*x^3*z*u+F(65)/2*x^2*y*z*u-1062*x*y^2*z*u+3487*y^3*z*u+F(7)/79*x^2*z^2*u-11445*x*y*z^2*u-9252*y^2*z^2*u-F(32)/105*x*z^3*u+F(123)/52*y*z^3*u-1006*z^4*u-F(94)/73*x^3*u^2-F(107)/32*x^2*y*u^2-6761*x*y^2*u^2+F(95)/73*y^3*u^2+7885*x^2*z*u^2-15374*x*y*z*u^2-F(117)/124*y^2*z*u^2-F(7)/109*x*z^2*u^2+946*y*z^2*u^2+F(116)/95*z^3*u^2+F(42)/43*x^2*u^3-14021*x*y*u^3+F(122)/43*y^2*u^3-F(90)/83*x*z*u^3-9282*y*z*u^3+F(53)/72*z^2*u^3-8352*x*u^4+4069*y*u^4+F(89)/77*z*u^4+4459*u^5-F(49)/87*x^4*v-14680*x^3*y*v+F(115)/9*x^2*y^2*v+14*x*y^3*v-4081*y^4*v-212*x^3*z*v+F(41)/10*x^2*y*z*v+7054*x*y^2*z*v-F(51)/70*y^3*z*v-7831*x^2*z^2*v+3883*x*y*z^2*v+F(100)/123*y^2*z^2*v+F(103)/94*x*z^3*v-7262*y*z^3*v+F(43)/71*z^4*v-8660*x^3*u*v+5616*x^2*y*u*v+F(39)/106*x*y^2*u*v-F(96)/67*y^3*u*v-F(3)/10*x^2*z*u*v-12528*x*y*z*u*v+14286*y^2*z*u*v-F(42)/5*x*z^2*u*v+14366*y*z^2*u*v-8738*z^3*u*v+F(62)/99*x^2*u^2*v-13600*x*y*u^2*v-4712*y^2*u^2*v-F(33)/10*x*z*u^2*v-2645*y*z*u^2*v+11615*z^2*u^2*v-2814*x*u^3*v-F(83)/84*y*u^3*v+3774*z*u^3*v+9323*u^4*v+12888*x^3*v^2-8208*x^2*y*v^2+F(106)/69*x*y^2*v^2-F(25)/12*y^3*v^2-F(93)/100*x^2*z*v^2+F(53)/65*x*y*z*v^2+F(89)/17*y^2*z*v^2+10586*x*z^2*v^2-F(63)/22*y*z^2*v^2-2197*z^3*v^2-9937*x^2*u*v^2+F(68)/125*x*y*u*v^2-F(94)/81*y^2*u*v^2+F(97)/37*x*z*u*v^2-F(42)/115*y*z*u*v^2+2718*z^2*u*v^2+F(97)/46*x*u^2*v^2-4814*y*u^2*v^2+F(33)/79*z*u^2*v^2-F(11)/21*u^3*v^2-F(87)/122*x^2*v^3+F(111)/23*x*y*v^3-14813*y^2*v^3-F(116)/31*x*z*v^3+F(113)/47*y*z*v^3-F(61)/35*z^2*v^3+7959*x*u*v^3+F(123)/95*y*u*v^3-F(35)/26*z*u*v^3+F(97)/64*u^2*v^3-F(29)/70*x*v^4-F(57)/5*y*v^4-10713*z*v^4-13633*u*v^4,6592*x^2*y^3+F(41)/122*x^3*y*z-8922*x^2*y^2*z-14867*x*y^3*z+F(77)/90*x^3*z^2+15202*x^2*y*z^2+F(81)/71*x*y^2*z^2-F(56)/45*y^3*z^2-F(102)/91*x^2*z^3-11764*x*y*z^3+F(38)/45*y^2*z^3-11306*x*z^4+12759*y*z^4-F(30)/109*z^5-F(122)/9*x^3*y*u-F(39)/31*x^2*y^2*u+10356*x*y^3*u-1985*x^3*z*u+2324*x^2*y*z*u+6305*x*y^2*z*u-F(6)/41*y^3*z*u+F(55)/6*x^2*z^2*u-F(46)/21*x*y*z^2*u-F(71)/34*y^2*z^2*u-F(45)/59*x*z^3*u-F(113)/102*y*z^3*u-4203*z^4*u+F(77)/109*x^3*u^2-F(80)/111*x^2*y*u^2-F(111)/28*x*y^2*u^2+11065*y^3*u^2-F(20)/7*x^2*z*u^2+F(110)/63*x*y*z*u^2-F(26)/103*y^2*z*u^2-F(47)/50*x*z^2*u^2+F(75)/67*y*z^2*u^2+F(109)/15*z^3*u^2+F(73)/39*x^2*u^3+F(25)/97*x*y*u^3-1611*y^2*u^3-6476*x*z*u^3+717*y*z*u^3+3337*z^2*u^3+F(41)/94*x*u^4+10710*y*u^4+F(37)/29*z*u^4+11922*u^5+8459*x^3*y*v-4238*x^2*y^2*v+F(20)/57*x*y^3*v+F(106)/61*x^3*z*v-10148*x^2*y*z*v-559*x*y^2*z*v+F(55)/83*y^3*z*v-F(7)/125*x^2*z^2*v-3748*x*y*z^2*v+3608*y^2*z^2*v+F(104)/29*x*z^3*v+7084*y*z^3*v+F(19)/78*z^4*v+F(11)/37*x^3*u*v+F(19)/119*x^2*y*u*v-4274*x*y^2*u*v+F(1)/31*y^3*u*v+1935*x^2*z*u*v+F(83)/43*x*y*z*u*v+F(45)/37*y^2*z*u*v+F(93)/64*x*z^2*u*v+F(104)/29*y*z^2*u*v+F(1)/86*z^3*u*v+4772*x^2*u^2*v+2271*x*y*u^2*v-8640*y^2*u^2*v+F(25)/14*x*z*u^2*v+11735*y*z*u^2*v+F(49)/73*z^2*u^2*v+F(12)/119*x*u^3*v+F(25)/37*y*u^3*v-14556*z*u^3*v-F(77)/30*u^4*v-F(81)/7*x^3*v^2-F(97)/119*x^2*y*v^2-F(53)/52*x*y^2*v^2+F(72)/47*y^3*v^2+F(28)/13*x^2*z*v^2+F(99)/28*x*y*z*v^2+5182*y^2*z*v^2+F(80)/93*x*z^2*v^2-2972*y*z^2*v^2+F(81)/112*z^3*v^2-F(54)/107*x^2*u*v^2+F(65)/29*x*y*u*v^2+2186*y^2*u*v^2-10919*x*z*u*v^2+F(41)/94*y*z*u*v^2-F(107)/111*z^2*u*v^2-F(12)/83*x*u^2*v^2-F(69)/125*y*u^2*v^2+F(48)/107*z*u^2*v^2+F(97)/74*u^3*v^2+F(107)/2*x^2*v^3-8566*x*y*v^3-6013*y^2*v^3-F(32)/115*x*z*v^3+F(78)/25*y*z*v^3+7247*z^2*v^3+F(67)/102*x*u*v^3+F(65)/121*y*u*v^3-F(79)/88*z*u*v^3+F(102)/89*u^2*v^3+5022*x*v^4-F(61)/74*y*v^4+F(14)/41*z*v^4-F(82)/7*u*v^4-F(120)/103*v^5,-11909*x^4*y+361*x^3*y^2+F(82)/111*x^2*y^3-F(89)/60*x*y^4-3390*x^4*z+14086*x^3*y*z-F(69)/119*x^2*y^2*z-F(32)/13*x*y^3*z+5880*y^4*z-3971*x^3*z^2+F(107)/56*x^2*y*z^2-6823*x*y^2*z^2+F(102)/13*y^3*z^2-F(47)/11*x^2*z^3-F(15)/68*x*y*z^3-6539*y^2*z^3+F(66)/113*x*z^4-2727*y*z^4-F(65)/44*x^4*u-F(107)/67*x^3*y*u-13981*x^2*y^2*u+F(123)/106*x*y^3*u-11367*y^4*u+27*x^3*z*u+F(121)/31*x^2*y*z*u-1972*x*y^2*z*u-2111*y^3*z*u-F(79)/105*x^2*z^2*u-F(41)/78*x*y*z^2*u+F(108)/53*y^2*z^2*u+F(33)/101*x*z^3*u+F(125)/119*y*z^3*u-F(12)/101*z^4*u-4732*x^3*u^2+15185*x^2*y*u^2-12590*x*y^2*u^2+15752*y^3*u^2-7502*x^2*z*u^2+6802*x*y*z*u^2+3452*y^2*z*u^2+F(112)/103*x*z^2*u^2+12632*y*z^2*u^2-13187*z^3*u^2-8713*x^2*u^3-7017*x*y*u^3+F(119)/108*y^2*u^3+F(38)/81*x*z*u^3-F(65)/38*y*z*u^3+1832*z^2*u^3+8251*x*u^4+8515*y*u^4-F(15)/59*z*u^4-5798*u^5-F(84)/23*x^4*v+F(8)/45*x^3*y*v-F(46)/31*x^2*y^2*v+F(18)/115*x*y^3*v-F(45)/79*y^4*v-F(14)/5*x^3*z*v+6264*x^2*y*z*v-F(80)/19*x*y^2*z*v-F(53)/16*y^3*z*v-F(41)/60*x^2*z^2*v+1824*x*y*z^2*v-15701*y^2*z^2*v-10231*x*z^3*v-F(29)/10*y*z^3*v+7855*z^4*v-F(107)/83*x^3*u*v+6109*x^2*y*u*v+11535*x*y^2*u*v-F(79)/3*y^3*u*v-13512*x^2*z*u*v-F(33)/106*x*y*z*u*v+12160*y^2*z*u*v-F(67)/59*x*z^2*u*v-F(85)/18*y*z^2*u*v+F(13)/93*z^3*u*v+1750*x^2*u^2*v-1192*x*y*u^2*v+13899*y^2*u^2*v+F(1)/32*x*z*u^2*v-12562*y*z*u^2*v-F(50)/103*z^2*u^2*v-7775*x*u^3*v+7685*y*u^3*v-F(73)/82*z*u^3*v+F(14)/73*u^4*v-F(2)/15*x^3*v^2+F(97)/117*x^2*y*v^2+5754*x*y^2*v^2+F(17)/28*y^3*v^2+14871*x^2*z*v^2-15064*x*y*z*v^2-F(28)/5*y^2*z*v^2+1236*x*z^2*v^2-F(11)/86*y*z^2*v^2+F(56)/123*z^3*v^2-F(35)/13*x^2*u*v^2-F(19)/46*x*y*u*v^2-513*y^2*u*v^2+F(59)/99*x*z*u*v^2-5275*y*z*u*v^2+F(51)/76*z^2*u*v^2+13691*x*u^2*v^2-F(62)/15*y*u^2*v^2+F(68)/45*z*u^2*v^2-F(109)/108*u^3*v^2+F(67)/104*x^2*v^3-F(92)/27*x*y*v^3+F(33)/85*y^2*v^3+F(103)/78*x*z*v^3-F(54)/119*y*z*v^3+5638*z^2*v^3-3729*x*u*v^3-6526*y*u*v^3+15871*z*u*v^3+F(31)/95*u^2*v^3+F(82)/73*x*v^4+2727*y*v^4-F(106)/9*z*v^4+F(29)/118*u*v^4+F(75)/83*v^5,-F(96)/65*x^4*y+6041*x^3*y^2+F(79)/15*x^2*y^3-F(117)/10*x*y^4+F(113)/120*x^4*z+F(69)/103*x^3*y*z-4254*x^2*y^2*z-F(75)/103*x*y^3*z-F(49)/30*y^4*z-F(102)/25*x^3*z^2+12944*x^2*y*z^2-F(11)/8*x*y^2*z^2-F(48)/79*y^3*z^2+11794*x^2*z^3-9061*x*y*z^3-1873*y^2*z^3+4199*x*z^4-F(58)/19*y*z^4+F(115)/66*z^5-F(106)/75*x^4*u+F(2)/57*x^3*y*u+14991*x^2*y^2*u-11971*x*y^3*u+F(100)/57*y^4*u+13149*x^3*z*u+F(91)/43*x^2*y*z*u-F(67)/86*x*y^2*z*u-11804*y^3*z*u+F(3)/67*x^2*z^2*u+13010*x*y*z^2*u+F(73)/99*y^2*z^2*u-F(54)/43*x*z^3*u+F(101)/43*y*z^3*u-F(23)/65*z^4*u+F(56)/97*x^3*u^2+13460*x^2*y*u^2+F(104)/79*x*y^2*u^2-F(122)/27*y^3*u^2-F(43)/99*x^2*z*u^2+F(47)/11*x*y*z*u^2+5306*y^2*z*u^2+F(62)/101*x*z^2*u^2-7316*y*z^2*u^2-F(88)/63*z^3*u^2+F(117)/74*x^2*u^3-F(25)/51*x*y*u^3+F(83)/30*y^2*u^3-F(115)/64*x*z*u^3-F(41)/71*y*z*u^3-15721*z^2*u^3-F(57)/43*x*u^4+F(41)/11*y*u^4+F(107)/90*z*u^4+1511*u^5-357*x^4*v+F(25)/39*x^3*y*v+13260*x^2*y^2*v-497*x*y^3*v+F(81)/64*y^4*v+15291*x^3*z*v-5802*x^2*y*z*v-F(19)/10*x*y^2*z*v+F(124)/89*y^3*z*v-14670*x^2*z^2*v-F(38)/65*x*y*z^2*v-56*y^2*z^2*v+F(42)/115*x*z^3*v-F(9)/59*y*z^3*v-F(99)/104*z^4*v-F(87)/124*x^3*u*v+F(101)/102*x^2*y*u*v-F(83)/14*x*y^2*u*v-F(101)/69*y^3*u*v-71*x^2*z*u*v-F(68)/11*x*y*z*u*v+1343*y^2*z*u*v+7059*x*z^2*u*v+12610*y*z^2*u*v-F(82)/97*z^3*u*v+F(104)/63*x^2*u^2*v-3535*x*y*u^2*v+F(91)/95*y^2*u^2*v+F(90)/13*x*z*u^2*v-F(85)/109*y*z*u^2*v-F(21)/44*z^2*u^2*v-F(47)/97*x*u^3*v-1191*y*u^3*v-F(33)/64*z*u^3*v+F(16)/91*u^4*v+9232*x^3*v^2-F(99)/26*x^2*y*v^2+F(15)/38*x*y^2*v^2+1374*y^3*v^2+F(1)/49*x^2*z*v^2-8058*x*y*z*v^2-F(47)/112*y^2*z*v^2-12960*x*z^2*v^2-F(21)/20*y*z^2*v^2+F(67)/28*z^3*v^2-F(9)/62*x^2*u*v^2+15678*x*y*u*v^2-F(71)/85*y^2*u*v^2-F(14)/45*x*z*u*v^2+F(124)/61*y*z*u*v^2+9344*z^2*u*v^2-F(16)/5*x*u^2*v^2+F(101)/50*y*u^2*v^2-F(65)/119*z*u^2*v^2+5440*u^3*v^2+F(64)/17*x^2*v^3+F(120)/73*x*y*v^3-14808*y^2*v^3-F(13)/16*x*z*v^3-111*y*z*v^3+3233*z^2*v^3+F(23)/108*x*u*v^3+F(67)/16*y*u*v^3-F(89)/72*z*u*v^3-1842*u^2*v^3-F(113)/29*x*v^4+F(3)/88*y*v^4+15659*z*v^4+F(20)/113*u*v^4-F(11)/90*v^5,-F(69)/122*x^2*y^3+F(112)/75*x^3*y*z+F(43)/42*x^2*y^2*z+10806*x*y^3*z-3517*x^3*z^2+F(77)/108*x^2*y*z^2-F(23)/25*x*y^2*z^2+F(19)/47*y^3*z^2-12830*x^2*z^3-F(107)/25*x*y*z^3+5064*y^2*z^3-F(122)/109*x*z^4-F(53)/52*y*z^4-F(11)/91*z^5+F(97)/10*x^3*y*u-F(5)/33*x^2*y^2*u+F(18)/101*x*y^3*u-F(125)/126*x^3*z*u+F(48)/119*x^2*y*z*u-F(79)/67*x*y^2*z*u+F(19)/37*y^3*z*u+F(35)/13*x^2*z^2*u-F(39)/32*x*y*z^2*u-9937*y^2*z^2*u-F(71)/119*x*z^3*u+4788*y*z^3*u-F(59)/120*z^4*u+618*x^3*u^2-F(103)/55*x^2*y*u^2-F(55)/2*x*y^2*u^2+12323*y^3*u^2-227*x^2*z*u^2-11939*x*y*z*u^2-5300*y^2*z*u^2-5873*x*z^2*u^2+2155*y*z^2*u^2+3908*z^3*u^2-9257*x^2*u^3+F(33)/14*x*y*u^3+F(83)/3*y^2*u^3+7399*x*z*u^3+F(71)/40*y*z*u^3+15768*z^2*u^3-14754*x*u^4-4640*y*u^4-11442*z*u^4+12770*u^5+14846*x^3*y*v-F(56)/111*x^2*y^2*v-F(52)/33*x*y^3*v-346*x^3*z*v+F(107)/59*x^2*y*z*v-9019*x*y^2*z*v-F(109)/83*y^3*z*v+F(47)/41*x^2*z^2*v+8704*x*y*z^2*v+11306*y^2*z^2*v-F(31)/53*x*z^3*v+F(117)/112*y*z^3*v+F(112)/115*z^4*v-14562*x^3*u*v+3502*x^2*y*u*v+F(93)/25*x*y^2*u*v+F(4)/31*y^3*u*v-F(32)/41*x^2*z*u*v+1652*x*y*z*u*v-8604*y^2*z*u*v-F(11)/34*x*z^2*u*v+F(101)/126*y*z^2*u*v-F(1)/122*z^3*u*v+F(79)/77*x^2*u^2*v+2502*x*y*u^2*v-F(8)/71*y^2*u^2*v-18*x*z*u^2*v+F(83)/101*y*z*u^2*v+F(67)/12*z^2*u^2*v+100*x*u^3*v-6533*y*u^3*v+10713*z*u^3*v-10584*u^4*v+7564*x^3*v^2+F(58)/121*x^2*y*v^2-F(71)/119*x*y^2*v^2+1225*y^3*v^2+F(2)/25*x^2*z*v^2+F(91)/16*x*y*z*v^2-F(93)/52*y^2*z*v^2+5682*x*z^2*v^2+F(67)/30*y*z^2*v^2+9983*z^3*v^2+23*x^2*u*v^2-3574*x*y*u*v^2+F(75)/38*y^2*u*v^2-F(13)/97*x*z*u*v^2-2394*y*z*u*v^2+14713*z^2*u*v^2+3750*x*u^2*v^2+F(77)/113*y*u^2*v^2+F(98)/23*z*u^2*v^2-F(96)/79*u^3*v^2+F(58)/77*x^2*v^3-F(113)/73*x*y*v^3-F(27)/59*y^2*v^3-F(73)/80*x*z*v^3-5746*y*z*v^3+5695*z^2*v^3+F(100)/27*x*u*v^3+13225*y*u*v^3-8288*z*u*v^3+12409*u^2*v^3+F(92)/121*x*v^4-9515*y*v^4+F(7)/19*z*v^4-F(13)/48*u*v^4+F(23)/103*v^5,-x^3*y^2+F(95)/29*x^2*y^3-F(31)/105*x^3*y*z-F(20)/63*x^2*y^2*z+F(67)/36*x*y^3*z+661*x^3*z^2+F(13)/79*x^2*y*z^2-15263*x*y^2*z^2+F(9)/53*y^3*z^2+F(95)/37*x^2*z^3+1652*x*y*z^3+F(63)/92*y^2*z^3-F(21)/16*x*z^4-F(81)/95*y*z^4+3434*z^5-F(41)/18*x^3*y*u-F(37)/47*x^2*y^2*u+F(9)/94*x*y^3*u-F(55)/92*x^3*z*u-F(83)/46*x^2*y*z*u-5774*x*y^2*z*u-11650*y^3*z*u-F(55)/97*x^2*z^2*u+F(29)/67*x*y*z^2*u-F(9)/88*y^2*z^2*u+7091*x*z^3*u+4043*y*z^3*u-F(27)/56*z^4*u-F(100)/29*x^2*y*u^2-F(35)/44*x*y^2*u^2+6039*y^3*u^2-12943*x^2*z*u^2-F(66)/109*x*y*z*u^2+F(23)/66*y^2*z*u^2-13231*x*z^2*u^2-3534*y*z^2*u^2-F(17)/56*z^3*u^2-F(113)/57*x*y*u^3+F(113)/55*y^2*u^3-F(111)/41*x*z*u^3+F(111)/41*y*z*u^3-F(59)/48*z^2*u^3+8115*y*u^4-F(113)/116*z*u^4+9102*x^3*y*v-F(25)/91*x^2*y^2*v-7945*x*y^3*v+F(73)/23*x^3*z*v-7608*x^2*y*z*v+549*x*y^2*z*v+F(30)/59*y^3*z*v-F(107)/36*x^2*z^2*v+F(19)/96*x*y*z^2*v+F(69)/118*y^2*z^2*v+12750*x*z^3*v-3908*y*z^3*v+6756*z^4*v+F(87)/52*x^2*y*u*v+F(89)/96*x*y^2*u*v+F(44)/123*y^3*u*v+8818*x^2*z*u*v+F(51)/116*x*y*z*u*v+4531*y^2*z*u*v-7238*x*z^2*u*v+F(85)/18*y*z^2*u*v+751*z^3*u*v-98*x*y*u^2*v-8545*y^2*u^2*v+F(82)/43*x*z*u^2*v-F(88)/71*y*z*u^2*v-4938*z^2*u^2*v-1120*y*u^3*v-9120*z*u^3*v-F(40)/113*x^2*y*v^2+F(58)/115*x*y^2*v^2+13135*y^3*v^2-F(52)/17*x^2*z*v^2-7228*x*y*z*v^2+F(14)/9*y^2*z*v^2+1868*x*z^2*v^2-1241*y*z^2*v^2-12198*z^3*v^2+13478*x*y*u*v^2+11618*y^2*u*v^2-F(89)/36*x*z*u*v^2-F(57)/107*y*z*u*v^2-11411*z^2*u*v^2+F(77)/61*y*u^2*v^2-F(40)/29*z*u^2*v^2+9689*x*y*v^3+2011*y^2*v^3-F(36)/91*x*z*v^3+F(44)/73*y*z*v^3+F(103)/20*z^2*v^3+F(103)/36*y*u*v^3+F(13)/19*z*u*v^3+F(62)/67*y*v^4-F(59)/95*z*v^4,454*x^3*y*z-F(84)/109*x^2*y^2*z-F(98)/69*x^3*z^2-F(35)/36*x^2*y*z^2+1892*x*y^2*z^2-F(45)/37*x^2*z^3-F(113)/19*x*y*z^3-F(101)/103*y^2*z^3-F(72)/5*x*z^4+730*y*z^4-13009*z^5-F(60)/109*x^3*y*u-F(41)/39*x^2*y^2*u+F(117)/2*x^3*z*u+11186*x^2*y*z*u+F(109)/3*x*y^2*z*u+F(71)/97*x^2*z^2*u+9796*x*y*z^2*u+4397*y^2*z^2*u-F(73)/63*x*z^3*u-2071*y*z^3*u-3674*z^4*u+5086*x^3*u^2-F(9)/64*x^2*y*u^2+F(21)/43*x*y^2*u^2+15658*x^2*z*u^2-F(103)/66*x*y*z*u^2-F(103)/54*y^2*z*u^2-F(123)/17*x*z^2*u^2-F(121)/96*y*z^2*u^2+F(15)/37*z^3*u^2-F(31)/11*x^2*u^3-7658*x*y*u^3-F(65)/79*y^2*u^3+3805*x*z*u^3+F(71)/28*y*z*u^3+824*z^2*u^3-F(13)/111*x*u^4-F(40)/59*y*u^4+F(41)/97*z*u^4+2706*u^5+F(50)/99*x^3*z*v-11436*x^2*y*z*v-13387*x*y^2*z*v+F(86)/69*x^2*z^2*v-2039*x*y*z^2*v-F(116)/53*y^2*z^2*v-14430*x*z^3*v-F(104)/25*y*z^3*v-13592*z^4*v+F(10)/99*x^3*u*v-F(115)/14*x^2*y*u*v+2377*x*y^2*u*v+F(59)/103*x^2*z*u*v-2984*x*y*z*u*v+15206*y^2*z*u*v+F(8)/109*x*z^2*u*v+F(23)/24*y*z^2*u*v+F(106)/109*z^3*u*v+7248*x^2*u^2*v+F(22)/115*x*y*u^2*v+895*y^2*u^2*v+F(61)/121*x*z*u^2*v-F(98)/41*y*z*u^2*v-F(21)/17*z^2*u^2*v-F(75)/83*x*u^3*v+F(7)/37*y*u^3*v+F(107)/46*z*u^3*v-1497*u^4*v+F(72)/43*x^2*z*v^2-F(21)/50*x*y*z*v^2+383*y^2*z*v^2-9620*x*z^2*v^2+F(14)/41*y*z^2*v^2-15766*z^3*v^2-F(45)/31*x^2*u*v^2-F(28)/83*x*y*u*v^2-F(6)/85*y^2*u*v^2-F(87)/86*x*z*u*v^2-15129*y*z*u*v^2-1046*z^2*u*v^2-F(21)/68*x*u^2*v^2+F(113)/41*y*u^2*v^2-3290*z*u^2*v^2+F(11)/92*u^3*v^2-12175*x*z*v^3-F(113)/86*y*z*v^3-F(77)/125*z^2*v^3-1417*x*u*v^3-F(55)/49*y*u*v^3-F(49)/94*z*u*v^3+15459*u^2*v^3+F(70)/39*z*v^4-F(49)/50*u*v^4,F(96)/65*x^3*y*z-6041*x^2*y^2*z-F(113)/120*x^3*z^2+F(103)/82*x^2*y*z^2+F(5)/101*x*y^2*z^2-13629*x^2*z^3+F(83)/124*x*y*z^3-F(81)/64*y^2*z^3-12896*x*z^4+F(80)/77*y*z^4+15128*z^5+x^3*y*u-F(95)/29*x^2*y^2*u+5126*x^3*z*u-12672*x^2*y*z*u+10006*x*y^2*z*u+F(94)/51*x^2*z^2*u+F(39)/94*x*y*z^2*u-11040*y^2*z^2*u-8966*x*z^3*u-F(32)/9*y*z^3*u-F(74)/5*z^4*u+F(41)/18*x^3*u^2+F(37)/47*x^2*y*u^2-F(9)/94*x*y^2*u^2+3357*x^2*z*u^2-F(109)/96*x*y*z*u^2+F(56)/89*y^2*z*u^2-F(104)/57*x*z^2*u^2-F(50)/97*y*z^2*u^2+11244*z^3*u^2+F(100)/29*x^2*u^3+F(35)/44*x*y*u^3-6039*y^2*u^3+4486*x*z*u^3+F(102)/41*y*z*u^3+F(101)/96*z^2*u^3+F(113)/57*x*u^4-F(113)/55*y*u^4-F(123)/49*z*u^4-8115*u^5+F(109)/60*x^3*y*v-5062*x^2*y^2*v+F(37)/28*x^3*z*v-8896*x^2*y*z*v+1769*x*y^2*z*v-6672*x^2*z^2*v-9860*x*y*z^2*v-F(119)/60*y^2*z^2*v-F(23)/116*x*z^3*v+F(119)/93*y*z^3*v+1088*z^4*v+F(105)/32*x^3*u*v-F(108)/109*x^2*y*u*v-13547*x*y^2*u*v+F(29)/20*x^2*z*u*v+99*x*y*z*u*v+F(17)/14*y^2*z*u*v+F(112)/43*x*z^2*u*v+F(61)/50*y*z^2*u*v-F(106)/17*z^3*u*v+F(14)/97*x^2*u^2*v+F(65)/67*x*y*u^2*v-F(98)/101*y^2*u^2*v+1585*x*z*u^2*v+F(67)/56*y*z*u^2*v+F(3)/5*z^2*u^2*v+F(103)/65*x*u^3*v+F(16)/25*y*u^3*v+7368*z*u^3*v+10371*u^4*v-F(16)/83*x^3*v^2-13915*x^2*y*v^2-4228*x*y^2*v^2-F(125)/47*x^2*z*v^2-F(113)/119*x*y*z*v^2-F(65)/83*y^2*z*v^2-15133*x*z^2*v^2+F(47)/34*y*z^2*v^2+F(79)/87*z^3*v^2+13952*x^2*u*v^2+F(105)/82*x*y*u*v^2-F(19)/55*y^2*u*v^2+F(59)/121*x*z*u*v^2+8861*y*z*u*v^2+F(44)/41*z^2*u*v^2+6961*x*u^2*v^2+F(118)/13*y*u^2*v^2-2*z*u^2*v^2+F(11)/46*u^3*v^2-F(59)/55*x^2*v^3+2512*x*y*v^3+10795*y^2*v^3-F(14)/125*x*z*v^3+F(89)/99*y*z*v^3+837*z^2*v^3+9547*x*u*v^3-F(22)/59*y*u*v^3+7179*z*u*v^3+F(13)/2*u^2*v^3-1073*x*v^4+11809*y*v^4-F(99)/28*z*v^4-F(109)/13*u*v^4+8577*v^5,454*x^3*y^2-F(84)/109*x^2*y^3-F(98)/69*x^3*y*z-F(35)/36*x^2*y^2*z+1892*x*y^3*z-F(45)/37*x^2*y*z^2-F(113)/19*x*y^2*z^2-F(101)/103*y^3*z^2-F(72)/5*x*y*z^3+730*y^2*z^3-13009*y*z^4-F(66)/85*x^3*y*u+F(93)/25*x^2*y^2*u-1530*x*y^3*u+9028*x^3*z*u-13919*x^2*y*z*u+F(102)/109*x*y^2*z*u-F(55)/29*y^3*z*u+10731*x^2*z^2*u-F(87)/71*x*y*z^2*u+F(73)/47*y^2*z^2*u+F(116)/59*x*z^3*u-14054*y*z^3*u-F(76)/105*z^4*u-3981*x^3*u^2+F(95)/73*x^2*y*u^2-F(49)/47*x*y^2*u^2+F(45)/41*y^3*u^2+11725*x^2*z*u^2-F(73)/87*x*y*z*u^2-F(118)/113*y^2*z*u^2-F(86)/11*x*z^2*u^2-F(106)/79*y*z^2*u^2-F(95)/37*z^3*u^2-F(89)/68*x^2*u^3-15078*x*y*u^3+F(122)/111*y^2*u^3+14914*x*z*u^3-F(87)/59*y*z*u^3+F(110)/31*z^2*u^3+F(104)/89*x*u^4+F(56)/69*y*u^4+F(116)/63*z*u^4+F(12)/79*u^5+F(50)/99*x^3*y*v-11436*x^2*y^2*v-13387*x*y^3*v+F(86)/69*x^2*y*z*v-2039*x*y^2*z*v-F(116)/53*y^3*z*v-14430*x*y*z^2*v-F(104)/25*y^2*z^2*v-13592*y*z^3*v-F(55)/68*x^3*u*v-F(109)/69*x^2*y*u*v+15791*x*y^2*u*v-11378*y^3*u*v-1955*x^2*z*u*v-F(77)/102*x*y*z*u*v-F(18)/73*y^2*z*u*v+F(13)/12*x*z^2*u*v-F(109)/83*y*z^2*u*v+2738*z^3*u*v+F(23)/24*x^2*u^2*v+133*x*y*u^2*v-15396*y^2*u^2*v-13919*x*z*u^2*v-F(65)/119*y*z*u^2*v-F(89)/106*z^2*u^2*v+5746*x*u^3*v-3959*y*u^3*v-F(47)/118*z*u^3*v+F(59)/21*u^4*v+F(72)/43*x^2*y*v^2-F(21)/50*x*y^2*v^2+383*y^3*v^2-9620*x*y*z*v^2+F(14)/41*y^2*z*v^2-15766*y*z^2*v^2-1414*x^2*u*v^2+F(79)/57*x*y*u*v^2+F(103)/115*y^2*u*v^2+14527*x*z*u*v^2-F(59)/22*y*z*u*v^2+F(114)/5*z^2*u*v^2-F(31)/121*x*u^2*v^2+F(65)/43*y*u^2*v^2-7513*z*u^2*v^2+F(62)/55*u^3*v^2-12175*x*y*v^3-F(113)/86*y^2*v^3-F(77)/125*y*z*v^3+5496*x*u*v^3+F(100)/91*y*u*v^3-F(7)/106*z*u*v^3+F(107)/3*u^2*v^3+F(70)/39*y*v^4-13485*u*v^4,F(96)/65*x^3*y^2-6041*x^2*y^3-F(113)/120*x^3*y*z+F(103)/82*x^2*y^2*z+F(5)/101*x*y^3*z-13629*x^2*y*z^2+F(83)/124*x*y^2*z^2-F(81)/64*y^3*z^2-12896*x*y*z^3+F(80)/77*y^2*z^3+15128*y*z^4+3907*x^3*y*u-6071*x^2*y^2*u+F(70)/69*x*y^3*u+661*x^3*z*u+F(49)/60*x^2*y*z*u-7435*x*y^2*z*u+F(87)/103*y^3*z*u+F(95)/37*x^2*z^2*u-F(62)/35*x*y*z^2*u-F(61)/102*y^2*z^2*u-F(21)/16*x*z^3*u+F(55)/18*y*z^3*u+3434*z^4*u-F(55)/92*x^3*u^2-1513*x^2*y*u^2+13886*x*y^2*u^2+7042*y^3*u^2-F(55)/97*x^2*z*u^2-10975*x*y*z*u^2-5671*y^2*z*u^2+7091*x*z^2*u^2-F(83)/113*y*z^2*u^2-F(27)/56*z^3*u^2-12943*x^2*u^3-F(17)/113*x*y*u^3-F(62)/49*y^2*u^3-13231*x*z*u^3-F(73)/120*y*z*u^3-F(17)/56*z^2*u^3-F(111)/41*x*u^4-F(78)/89*y*u^4-F(59)/48*z*u^4-F(113)/116*u^5+F(105)/82*x^3*y*v+F(112)/97*x^2*y^2*v+14317*x*y^3*v+1734*x^3*z*v-F(124)/121*x^2*y*z*v+14131*x*y^2*z*v-13623*y^3*z*v-F(27)/56*x^2*z^2*v+F(57)/23*x*y*z^2*v-F(79)/55*y^2*z^2*v+9295*x*z^3*v+12418*y*z^3*v-F(52)/7*z^4*v-F(5)/67*x^3*u*v-F(101)/84*x^2*y*u*v+F(37)/61*x*y^2*u*v-14914*y^3*u*v-14039*x^2*z*u*v+F(26)/43*x*y*z*u*v-F(35)/69*y^2*z*u*v-F(119)/65*x*z^2*u*v+2521*y*z^2*u*v-F(72)/5*z^3*u*v-F(73)/125*x^2*u^2*v-F(57)/11*x*y*u^2*v+8856*y^2*u^2*v-F(31)/87*x*z*u^2*v+11358*y*z*u^2*v+F(18)/67*z^2*u^2*v-F(110)/123*x*u^3*v+9002*y*u^3*v-F(12)/43*z*u^3*v-F(19)/96*u^4*v-13602*x^3*v^2-6745*x^2*y*v^2+15473*x*y^2*v^2-6157*y^3*v^2+15294*x^2*z*v^2-F(94)/3*x*y*z*v^2-F(51)/31*y^2*z*v^2+7167*x*z^2*v^2+35*y*z^2*v^2+F(88)/61*z^3*v^2+F(101)/35*x^2*u*v^2-13855*x*y*u*v^2-1923*y^2*u*v^2-F(112)/71*x*z*u*v^2-10721*y*z*u*v^2+5695*z^2*u*v^2-F(19)/85*x*u^2*v^2-7159*y*u^2*v^2+2012*z*u^2*v^2-F(57)/52*u^3*v^2+F(81)/8*x^2*v^3-14240*x*y*v^3-F(121)/8*y^2*v^3-F(58)/71*x*z*v^3+8713*y*z*v^3+234*z^2*v^3+2378*x*u*v^3+9524*y*u*v^3-F(117)/106*z*u*v^3-F(97)/117*u^2*v^3+F(43)/38*x*v^4+585*y*v^4-F(33)/119*z*v^4+F(66)/61*u*v^4-2016*v^5,F(47)/90*x^3*y*u+F(113)/80*x^2*y^2*u+F(28)/81*x^3*z*u-F(29)/53*x^2*y*z*u-4283*x*y^2*z*u-F(77)/106*x^2*z^2*u-F(83)/8*x*y*z^2*u+10185*y^2*z^2*u-F(43)/71*x*z^3*u+1592*y*z^3*u-14954*z^4*u-F(67)/28*x^3*u^2+9787*x^2*y*u^2+9164*x*y^2*u^2+F(22)/125*x^2*z*u^2+F(48)/71*x*y*z*u^2-F(16)/105*y^2*z*u^2-F(29)/69*x*z^2*u^2-F(7)/48*y*z^2*u^2+F(67)/115*z^3*u^2+10462*x^2*u^3-F(34)/71*x*y*u^3+F(111)/22*y^2*u^3+11797*x*z*u^3-F(8)/25*y*z*u^3+4873*z^2*u^3-F(43)/34*x*u^4+12355*y*u^4-14107*z*u^4-F(63)/55*u^5-F(125)/27*x^3*y*v-F(7)/5*x^2*y^2*v+F(64)/89*x^3*z*v-15627*x^2*y*z*v-F(9)/103*x*y^2*z*v-F(7)/101*x^2*z^2*v+F(29)/109*x*y*z^2*v+F(42)/113*y^2*z^2*v-7704*x*z^3*v+6658*y*z^3*v+F(43)/9*z^4*v-F(95)/118*x^3*u*v-6509*x^2*y*u*v+2427*x*y^2*u*v-F(48)/35*x^2*z*u*v-F(17)/103*x*y*z*u*v+9368*y^2*z*u*v-F(13)/89*x*z^2*u*v+F(41)/71*y*z^2*u*v+F(31)/20*z^3*u*v-2616*x^2*u^2*v-F(94)/57*x*y*u^2*v-F(78)/121*y^2*u^2*v+13218*x*z*u^2*v-6158*y*z*u^2*v-4181*z^2*u^2*v+F(25)/61*x*u^3*v-3463*y*u^3*v+F(123)/104*z*u^3*v+8687*u^4*v+F(119)/71*x^3*v^2+F(75)/4*x^2*y*v^2+F(107)/65*x*y^2*v^2-F(62)/75*x^2*z*v^2-F(80)/111*x*y*z*v^2-F(104)/97*y^2*z*v^2+F(25)/72*x*z^2*v^2-F(16)/115*y*z^2*v^2-9763*z^3*v^2+F(94)/105*x^2*u*v^2+F(64)/13*x*y*u*v^2+F(101)/18*y^2*u*v^2+F(47)/71*x*z*u*v^2+F(11)/115*y*z*u*v^2-F(87)/59*z^2*u*v^2+F(97)/91*x*u^2*v^2-F(29)/59*y*u^2*v^2-8431*z*u^2*v^2-9199*u^3*v^2-F(91)/19*x^2*v^3-F(23)/114*x*y*v^3+F(49)/79*y^2*v^3+F(25)/108*x*z*v^3-F(65)/21*y*z*v^3+F(10)/71*z^2*v^3+F(47)/57*x*u*v^3-F(21)/11*y*u*v^3+11694*z*u*v^3+824*u^2*v^3+12539*x*v^4+413*y*v^4+4789*z*v^4+F(89)/31*u*v^4-5602*v^5,-F(89)/67*x^3*y^2-F(110)/17*x^2*y^3+7547*x^3*y*z+F(88)/37*x^2*y^2*z+5879*x*y^3*z-F(111)/97*x^3*z^2-F(5)/29*x^2*y*z^2+11935*x*y^2*z^2-5880*y^3*z^2+11575*x^2*z^3+6141*x*y*z^3-F(102)/13*y^2*z^3+F(57)/82*x*z^4+6539*y*z^4+2727*z^5+10588*x^3*y*u+8794*x^2*y^2*u-F(101)/7*x*y^3*u+F(21)/88*x^3*z*u-F(23)/111*x^2*y*z*u+F(87)/95*x*y^2*z*u-F(47)/90*y^3*z*u-F(61)/87*x^2*z^2*u-F(109)/78*x*y*z^2*u-F(101)/83*y^2*z^2*u-13940*x*z^3*u+F(67)/97*y*z^3*u-F(107)/56*z^4*u+7*x^3*u^2+F(63)/52*x^2*y*u^2-F(109)/80*x*y^2*u^2-691*y^3*u^2+11185*x^2*z*u^2+1858*x*y*z*u^2-8636*y^2*z*u^2-13382*x*z^2*u^2-F(3)/19*y*z^2*u^2+F(49)/110*z^3*u^2-F(13)/2*x^2*u^3+13422*x*y*u^3-F(82)/119*y^2*u^3+9933*x*z*u^3-F(101)/27*y*z*u^3+8299*z^2*u^3-F(92)/121*x*u^4-F(39)/43*y*u^4-8269*z*u^4+F(1)/80*u^5-1220*x^3*y*v-14*x^2*y^2*v+4081*x*y^3*v-4981*x^3*z*v+F(30)/41*x^2*y*z*v+4884*x*y^2*z*v+F(45)/79*y^3*z*v+7130*x^2*z^2*v-F(86)/111*x*y*z^2*v+F(27)/101*y^2*z^2*v-F(49)/115*x*z^3*v+F(31)/63*y*z^3*v+1408*z^4*v+15308*x^3*u*v+F(42)/103*x^2*y*u*v+6678*x*y^2*u*v-F(77)/57*y^3*u*v-F(87)/104*x^2*z*u*v+F(39)/86*x*y*z*u*v-564*y^2*z*u*v-1852*x*z^2*u*v-F(16)/67*y*z^2*u*v-F(122)/63*z^3*u*v+F(113)/21*x^2*u^2*v+F(57)/107*x*y*u^2*v-F(117)/62*y^2*u^2*v-5212*x*z*u^2*v+F(115)/21*y*z*u^2*v+F(59)/122*z^2*u^2*v-F(31)/63*x*u^3*v-12548*y*u^3*v-F(119)/22*z*u^3*v+6871*u^4*v-F(115)/116*x^3*v^2-F(19)/35*x^2*y*v^2+F(25)/12*x*y^2*v^2+4940*x^2*z*v^2+833*x*y*z*v^2-F(97)/109*y^2*z*v^2+F(115)/111*x*z^2*v^2+F(120)/101*y*z^2*v^2-F(40)/53*z^3*v^2+10442*x^2*u*v^2+3055*x*y*u*v^2+F(3)/23*y^2*u*v^2-F(111)/10*x*z*u*v^2-F(47)/90*y*z*u*v^2+13939*z^2*u*v^2+F(97)/17*x*u^2*v^2-F(113)/60*y*u^2*v^2-12151*z*u^2*v^2-F(125)/31*u^3*v^2+F(59)/55*x^2*v^3+14813*x*y*v^3-1039*x*z*v^3-8271*y*z*v^3-F(29)/4*z^2*v^3+F(122)/41*x*u*v^3-11478*y*u*v^3+F(56)/53*z*u*v^3+14783*u^2*v^3+F(57)/5*x*v^4-F(67)/21*z*v^4-F(115)/87*u*v^4,-F(103)/15*x^2*y^2*z-F(94)/65*x*y^3*z-15244*x^2*y*z^2+F(73)/94*x*y^2*z^2-5571*y^3*z^2-8619*x^2*z^3-14954*x*y*z^3-3224*y^2*z^3-1718*x*z^4+F(32)/91*y*z^4+4111*z^5-2029*x^2*y^2*u-F(82)/91*x*y^3*u-13209*x^2*y*z*u+F(3)/65*x*y^2*z*u-F(11)/12*y^3*z*u+3631*x^2*z^2*u-F(46)/57*x*y*z^2*u+11347*y^2*z^2*u-6114*x*z^3*u-F(17)/78*y*z^3*u-6254*z^4*u+F(21)/40*x^2*y*u^2-4814*x*y^2*u^2+15375*y^3*u^2-10442*x^2*z*u^2+F(92)/119*x*y*z*u^2-2687*y^2*z*u^2+6521*x*z^2*u^2-F(125)/2*y*z^2*u^2-F(62)/107*z^3*u^2+F(119)/96*x^2*u^3+F(67)/3*x*y*u^3+F(49)/111*y^2*u^3-F(2)/39*x*z*u^3+F(113)/23*y*z*u^3-F(74)/9*z^2*u^3-F(19)/52*x*u^4-F(71)/92*y*u^4-4053*z*u^4+11359*u^5-13230*x^2*y*z*v-F(67)/20*x*y^2*z*v+F(43)/34*y^3*z*v+F(115)/66*x^2*z^2*v-F(107)/59*x*y*z^2*v+7083*y^2*z^2*v-F(19)/106*x*z^3*v-F(21)/121*y*z^3*v-F(82)/125*z^4*v-F(125)/108*x^2*y*u*v-F(45)/73*x*y^2*u*v-F(14)/89*y^3*u*v-F(64)/103*x^2*z*u*v-F(40)/59*x*y*z*u*v-82*y^2*z*u*v-F(32)/57*x*z^2*u*v-14846*y*z^2*u*v+10328*z^3*u*v+F(76)/83*x^2*u^2*v+F(85)/116*x*y*u^2*v+11039*y^2*u^2*v+F(57)/110*x*z*u^2*v-14749*y*z*u^2*v-13797*z^2*u^2*v-F(9)/62*x*u^3*v-F(102)/31*y*u^3*v-F(66)/119*z*u^3*v+F(98)/41*u^4*v+10545*x^2*z*v^2-F(34)/79*x*y*z*v^2-10583*y^2*z*v^2+F(41)/105*x*z^2*v^2+F(91)/101*y*z^2*v^2-F(101)/67*z^3*v^2-11215*x^2*u*v^2-F(61)/97*x*y*u*v^2+F(121)/12*y^2*u*v^2+4700*x*z*u*v^2+F(26)/93*y*z*u*v^2+220*z^2*u*v^2-F(81)/56*x*u^2*v^2+1220*y*u^2*v^2-F(81)/49*z*u^2*v^2-F(27)/70*u^3*v^2+7351*x*z*v^3-F(23)/2*y*z*v^3-730*z^2*v^3-9989*x*u*v^3-F(113)/105*y*u*v^3-15601*z*u*v^3-F(68)/53*u^2*v^3-F(47)/110*z*v^4-F(23)/7*u*v^4,-F(49)/4*x^2*y^2*z+805*x*y^3*z-12829*x^2*y*z^2+7662*x*y^2*z^2+F(53)/24*y^3*z^2-F(107)/36*x^2*z^3+11341*x*y*z^3+F(77)/72*y^2*z^3-F(44)/17*x*z^4+7501*y*z^4+F(102)/101*z^5-F(37)/47*x^2*y*z*u+F(65)/2*x*y^2*z*u-F(73)/10*y^3*z*u+2620*x^2*z^2*u-F(112)/117*x*y*z^2*u-F(57)/91*y^2*z^2*u+F(16)/5*x*z^3*u+14068*y*z^3*u+1540*z^4*u-F(52)/27*x^2*z*u^2+13159*x*y*z*u^2+1211*y^2*z*u^2+F(78)/125*x*z^2*u^2+F(37)/20*y*z^2*u^2-4634*z^3*u^2-F(93)/116*x*z*u^3+F(104)/73*y*z*u^3-F(88)/87*z^2*u^3+12450*z*u^4-2029*x^2*y^2*v-F(82)/91*x*y^3*v-6843*x^2*y*z*v-F(1)/102*x*y^2*z*v+F(46)/5*y^3*z*v-F(103)/101*x^2*z^2*v-F(59)/50*x*y*z^2*v+F(37)/55*y^2*z^2*v-8521*x*z^3*v-F(56)/43*y*z^3*v-15421*z^4*v+F(21)/40*x^2*y*u*v-4814*x*y^2*u*v+15375*y^3*u*v+F(55)/32*x^2*z*u*v-F(102)/29*x*y*z*u*v-F(48)/107*y^2*z*u*v+15*x*z^2*u*v+F(19)/91*y*z^2*u*v-F(23)/104*z^3*u*v+F(119)/96*x^2*u^2*v+F(67)/3*x*y*u^2*v+F(49)/111*y^2*u^2*v-9961*x*z*u^2*v-860*y*z*u^2*v+F(83)/86*z^2*u^2*v-F(19)/52*x*u^3*v-F(71)/92*y*u^3*v+F(46)/13*z*u^3*v+11359*u^4*v-F(125)/108*x^2*y*v^2-F(45)/73*x*y^2*v^2-F(14)/89*y^3*v^2-9367*x^2*z*v^2+7707*x*y*z*v^2+2273*y^2*z*v^2+F(21)/116*x*z^2*v^2+F(56)/31*y*z^2*v^2+F(97)/10*z^3*v^2+F(76)/83*x^2*u*v^2+F(85)/116*x*y*u*v^2+11039*y^2*u*v^2-F(44)/59*x*z*u*v^2+F(114)/65*y*z*u*v^2+3871*z^2*u*v^2-F(9)/62*x*u^2*v^2-F(102)/31*y*u^2*v^2-350*z*u^2*v^2+F(98)/41*u^3*v^2-11215*x^2*v^3-F(61)/97*x*y*v^3+F(121)/12*y^2*v^3-F(55)/111*x*z*v^3-F(77)/115*y*z*v^3-8710*z^2*v^3-F(81)/56*x*u*v^3+1220*y*u*v^3+6653*z*u*v^3-F(27)/70*u^2*v^3-9989*x*v^4-F(113)/105*y*v^4+F(61)/100*z*v^4-F(68)/53*u*v^4-F(23)/7*v^5,F(49)/92*x^3*y^2+F(112)/9*x^2*y^3+F(84)/107*x^3*y*z-7707*x^2*y^2*z+F(5)/67*x*y^3*z+5919*x^3*z^2+F(6)/121*x^2*y*z^2+11612*x*y^2*z^2-F(107)/36*x^2*z^3-F(89)/93*x*y*z^3+3540*x*z^4+F(103)/57*x^3*y*u+14498*x^2*y^2*u+1149*x*y^3*u-F(1)/32*x^3*z*u+F(68)/57*x^2*y*z*u+F(24)/47*x*y^2*z*u+F(89)/102*y^3*z*u-F(75)/79*x^2*z^2*u-12711*x*y*z^2*u+15636*y^2*z^2*u+961*x*z^3*u-F(25)/61*y*z^3*u-14778*z^4*u+1126*x^3*u^2+F(18)/47*x^2*y*u^2+952*x*y^2*u^2-F(24)/109*y^3*u^2-F(85)/109*x^2*z*u^2+12322*x*y*z*u^2+F(102)/13*y^2*z*u^2-F(39)/14*x*z^2*u^2+F(115)/14*y*z^2*u^2-F(78)/59*z^3*u^2-3054*x^2*u^3-191*x*y*u^3-F(29)/32*y^2*u^3+F(46)/37*x*z*u^3+13351*y*z*u^3+F(94)/99*z^2*u^3-F(22)/91*x*u^4-F(62)/95*y*u^4+F(109)/119*z*u^4+14255*u^5+3872*x^3*y*v+F(31)/113*x^2*y^2*v-8845*x*y^3*v+10288*x^3*z*v-F(73)/112*x^2*y*z*v+F(65)/93*x*y^2*z*v-F(4)/97*y^3*z*v-4467*x^2*z^2*v-6264*x*y*z^2*v+12846*y^2*z^2*v-1053*x*z^3*v-5833*y*z^3*v-F(88)/9*z^4*v-F(121)/15*x^3*u*v+4401*x^2*y*u*v+12152*x*y^2*u*v-F(76)/35*y^3*u*v+F(86)/49*x^2*z*u*v+F(6)/19*x*y*z*u*v-F(64)/105*y^2*z*u*v-F(65)/33*x*z^2*u*v-12175*y*z^2*u*v-F(66)/13*z^3*u*v+F(22)/41*x^2*u^2*v+13840*x*y*u^2*v+15162*y^2*u^2*v-4539*x*z*u^2*v+2773*y*z*u^2*v-F(11)/18*z^2*u^2*v-4711*x*u^3*v+8544*y*u^3*v-11471*z*u^3*v+15253*u^4*v+F(58)/61*x^3*v^2-5510*x^2*y*v^2-12494*x*y^2*v^2-F(59)/23*y^3*v^2-8641*x^2*z*v^2+F(83)/103*x*y*z*v^2+2059*y^2*z*v^2-11941*x*z^2*v^2+3884*y*z^2*v^2+F(111)/17*z^3*v^2-12351*x^2*u*v^2-F(49)/37*x*y*u*v^2-F(67)/99*y^2*u*v^2-F(73)/89*x*z*u*v^2+5250*y*z*u*v^2+13819*z^2*u*v^2-F(19)/2*x*u^2*v^2+F(67)/122*y*u^2*v^2-F(19)/125*z*u^2*v^2+F(12)/113*u^3*v^2+F(42)/5*x^2*v^3+F(59)/52*x*y*v^3+F(116)/91*y^2*v^3+4237*x*z*v^3+F(1)/47*y*z*v^3-F(61)/79*z^2*v^3+F(114)/43*x*u*v^3-5450*y*u*v^3-F(12)/7*z*u*v^3-F(54)/61*u^2*v^3-F(12)/115*x*v^4-29*y*v^4-4596*z*v^4+15691*u*v^4-4544*v^5,-F(49)/4*x^3*y^2+805*x^2*y^3-12829*x^3*y*z+7662*x^2*y^2*z+F(53)/24*x*y^3*z-F(107)/36*x^3*z^2+11341*x^2*y*z^2+F(77)/72*x*y^2*z^2-F(44)/17*x^2*z^3+7501*x*y*z^3+F(102)/101*x*z^4-F(37)/47*x^3*y*u+F(65)/2*x^2*y^2*u-F(73)/10*x*y^3*u+2620*x^3*z*u-F(112)/117*x^2*y*z*u-F(57)/91*x*y^2*z*u+F(16)/5*x^2*z^2*u+14068*x*y*z^2*u+1540*x*z^3*u-F(52)/27*x^3*u^2+13159*x^2*y*u^2+1211*x*y^2*u^2+F(78)/125*x^2*z*u^2+F(37)/20*x*y*z*u^2-4634*x*z^2*u^2-F(93)/116*x^2*u^3+F(104)/73*x*y*u^3-F(88)/87*x*z*u^3+12450*x*u^4-F(43)/126*x^3*y*v-F(101)/109*x^2*y^2*v+1842*x*y^3*v+2028*x^3*z*v+F(105)/26*x^2*y*z*v+5543*x*y^2*z*v-12927*y^3*z*v-F(67)/30*x^2*z^2*v-10462*x*y*z^2*v+13766*y^2*z^2*v+6082*x*z^3*v-9797*y*z^3*v+F(113)/48*z^4*v-F(23)/87*x^3*u*v+8080*x^2*y*u*v-F(124)/107*x*y^2*u*v+F(13)/41*y^3*u*v-219*x^2*z*u*v-F(17)/74*x*y*z*u*v-F(110)/27*y^2*z*u*v+14885*x*z^2*u*v-F(59)/15*y*z^2*u*v-F(115)/82*z^3*u*v+F(47)/60*x^2*u^2*v-9727*x*y*u^2*v+15246*y^2*u^2*v-F(73)/47*x*z*u^2*v+F(87)/31*y*z*u^2*v-F(2)/25*z^2*u^2*v+F(83)/73*x*u^3*v+13952*y*u^3*v+F(83)/57*z*u^3*v+F(33)/86*u^4*v-12344*x^3*v^2+9781*x^2*y*v^2-F(18)/83*x*y^2*v^2+F(103)/74*y^3*v^2+F(117)/17*x^2*z*v^2-9028*x*y*z*v^2+F(105)/31*y^2*z*v^2-4248*x*z^2*v^2+F(29)/68*y*z^2*v^2+F(39)/16*z^3*v^2-F(24)/125*x^2*u*v^2+14048*x*y*u*v^2+2198*y^2*u*v^2-F(68)/77*x*z*u*v^2+F(13)/96*y*z*u*v^2-F(56)/109*z^2*u*v^2-F(97)/46*x*u^2*v^2-14694*y*u^2*v^2-F(13)/87*z*u^2*v^2+10050*u^3*v^2-F(18)/77*x^2*v^3+8589*x*y*v^3-F(57)/10*y^2*v^3-2829*x*z*v^3+1735*y*z*v^3+F(9)/2*z^2*v^3+9581*x*u*v^3+F(35)/38*y*u*v^3+83*z*u*v^3-F(11)/79*u^2*v^3-F(7)/54*x*v^4+2529*y*v^4-6255*z*v^4+F(37)/32*u*v^4-F(2)/105*v^5,-F(49)/4*x^2*y^2*u+805*x*y^3*u-12829*x^2*y*z*u+7662*x*y^2*z*u+F(53)/24*y^3*z*u-F(107)/36*x^2*z^2*u+11341*x*y*z^2*u+F(77)/72*y^2*z^2*u-F(44)/17*x*z^3*u+7501*y*z^3*u+F(102)/101*z^4*u-F(37)/47*x^2*y*u^2+F(65)/2*x*y^2*u^2-F(73)/10*y^3*u^2+2620*x^2*z*u^2-F(112)/117*x*y*z*u^2-F(57)/91*y^2*z*u^2+F(16)/5*x*z^2*u^2+14068*y*z^2*u^2+1540*z^3*u^2-F(52)/27*x^2*u^3+13159*x*y*u^3+1211*y^2*u^3+F(78)/125*x*z*u^3+F(37)/20*y*z*u^3-4634*z^2*u^3-F(93)/116*x*u^4+F(104)/73*y*u^4-F(88)/87*z*u^4+12450*u^5+F(103)/15*x^2*y^2*v+F(94)/65*x*y^3*v+15244*x^2*y*z*v-F(73)/94*x*y^2*z*v+5571*y^3*z*v+8619*x^2*z^2*v+14954*x*y*z^2*v+3224*y^2*z^2*v+1718*x*z^3*v-F(32)/91*y*z^3*v-4111*z^4*v+6366*x^2*y*u*v+F(62)/13*x*y^2*u*v+F(3)/53*y^3*u*v+6187*x^2*z*u*v+6656*x*y*z*u*v+4940*y^2*z*u*v+F(86)/93*x*z^2*u*v-F(64)/103*y*z^2*u*v-9167*z^3*u*v-F(101)/105*x^2*u^2*v-8301*x*y*u^2*v+F(55)/57*y^2*u^2*v+F(38)/59*x*z*u^2*v+F(19)/90*y*z*u^2*v-F(49)/120*z^2*u^2*v-F(8)/21*x*u^3*v+F(95)/61*y*u^3*v-15325*z*u^3*v-3326*u^4*v+13230*x^2*y*v^2+F(67)/20*x*y^2*v^2-F(43)/34*y^3*v^2-F(115)/66*x^2*z*v^2+F(107)/59*x*y*z*v^2-7083*y^2*z*v^2+F(19)/106*x*z^2*v^2+F(21)/121*y*z^2*v^2+F(82)/125*z^3*v^2+F(46)/31*x^2*u*v^2+F(121)/108*x*y*u*v^2+2355*y^2*u*v^2+412*x*z*u*v^2-F(33)/85*y*z*u*v^2-721*z^2*u*v^2-F(34)/23*x*u^2*v^2-1983*y*u^2*v^2+F(89)/67*z*u^2*v^2-F(77)/101*u^3*v^2-10545*x^2*v^3+F(34)/79*x*y*v^3+10583*y^2*v^3-F(41)/105*x*z*v^3-F(91)/101*y*z*v^3+F(101)/67*z^2*v^3+F(62)/9*x*u*v^3-F(108)/35*y*u*v^3-F(98)/43*z*u*v^3-527*u^2*v^3-7351*x*v^4+F(23)/2*y*v^4+730*z*v^4-F(26)/9*u*v^4+F(47)/110*v^5]))
end


@doc raw"""
    k3_d11_pi11_ss_3()

Return a smooth K3 surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and three 6-secants.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d11_pi11_ss_3()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-x^5*z-F(32)/121*x^4*y*z-F(6)/49*x^3*y^2*z+F(107)/4*x^2*y^3*z-F(125)/116*x*y^4*z+12128*x^4*z^2-7957*x^3*y*z^2+F(85)/94*x^2*y^2*z^2+F(115)/96*x*y^3*z^2+F(10)/119*x^3*z^3+11392*x^2*y*z^3+F(2)/19*x*y^2*z^3+5102*x^2*z^4+F(70)/33*x*y*z^4+F(22)/21*x*z^5+F(76)/81*x^5*u-F(4)/107*x^4*y*u+F(112)/85*x^3*y^2*u+F(74)/83*x^4*z*u-30*x^3*y*z*u+F(100)/43*x^2*y^2*z*u-5420*x*y^3*z*u-F(55)/31*y^4*z*u-F(71)/89*x^3*z^2*u+F(8)/15*x^2*y*z^2*u-F(94)/93*x*y^2*z^2*u-5578*y^3*z^2*u+F(17)/43*x^2*z^3*u+F(49)/22*x*y*z^3*u-F(53)/87*y^2*z^3*u-F(119)/125*x*z^4*u+F(57)/35*y*z^4*u+3243*z^5*u+F(19)/82*x^4*u^2+1824*x^3*y*u^2-F(71)/32*x^2*y^2*u^2+F(103)/62*x^3*z*u^2-3664*x^2*y*z*u^2+F(125)/34*x*y^2*z*u^2+F(97)/67*y^3*z*u^2+11206*x^2*z^2*u^2-F(75)/17*x*y*z^2*u^2-F(121)/98*y^2*z^2*u^2+3340*x*z^3*u^2-F(79)/12*y*z^3*u^2-221*z^4*u^2-F(8)/27*x^3*u^3-11950*x^2*y*u^3-F(55)/102*x^2*z*u^3+F(109)/17*x*y*z*u^3-F(89)/125*y^2*z*u^3-61*x*z^2*u^3+15002*y*z^2*u^3-F(55)/122*z^3*u^3+4433*x^2*u^4-6306*x*z*u^4-F(62)/43*y*z*u^4-F(35)/122*z^2*u^4-F(121)/12*x^5*v-8439*x^4*y*v+F(3)/95*x^3*y^2*v+7717*x^4*z*v-2620*x^3*y*z*v+F(109)/62*x^2*y^2*z*v+15880*x*y^3*z*v-F(21)/31*y^4*z*v+3287*x^3*z^2*v-F(90)/107*x^2*y*z^2*v-F(7)/67*x*y^2*z^2*v-F(61)/99*y^3*z^2*v-F(31)/34*x^2*z^3*v-F(23)/19*x*y*z^3*v-F(97)/121*y^2*z^3*v+7956*x*z^4*v+9107*y*z^4*v+F(95)/96*z^5*v+10326*x^4*u*v+F(105)/46*x^3*y*u*v-10781*x^2*y^2*u*v+7588*x^3*z*u*v-F(124)/19*x^2*y*z*u*v-3422*x*y^2*z*u*v-F(89)/17*y^3*z*u*v-F(13)/6*x^2*z^2*u*v-F(32)/13*x*y*z^2*u*v-6086*y^2*z^2*u*v+F(105)/38*x*z^3*u*v-F(33)/76*y*z^3*u*v+3615*z^4*u*v+5442*x^3*u^2*v+15433*x^2*y*u^2*v+2427*x^2*z*u^2*v+11489*x*y*z*u^2*v+F(114)/83*y^2*z*u^2*v-F(100)/41*x*z^2*u^2*v-F(58)/53*y*z^2*u^2*v-6114*z^3*u^2*v+F(49)/13*x^2*u^3*v-F(123)/8*x*z*u^3*v-F(89)/49*y*z*u^3*v-7535*z^2*u^3*v-F(35)/24*x^4*v^2+F(104)/41*x^3*y*v^2+2931*x^2*y^2*v^2-F(103)/39*x^3*z*v^2+3596*x^2*y*z*v^2+F(87)/22*x*y^2*z*v^2-F(107)/37*y^3*z*v^2-F(34)/43*x^2*z^2*v^2+F(83)/56*x*y*z^2*v^2-F(79)/17*y^2*z^2*v^2+6065*x*z^3*v^2+F(3)/47*y*z^3*v^2+F(119)/74*z^4*v^2+7786*x^3*u*v^2-F(44)/71*x^2*y*u*v^2-F(33)/20*x^2*z*u*v^2+F(102)/95*x*y*z*u*v^2-F(26)/101*y^2*z*u*v^2+13361*x*z^2*u*v^2-F(49)/99*y*z^2*u*v^2-F(24)/35*z^3*u*v^2+14048*x^2*u^2*v^2-F(82)/91*x*z*u^2*v^2+F(34)/71*y*z*u^2*v^2-F(49)/55*z^2*u^2*v^2-4152*x^3*v^3-8876*x^2*y*v^3+F(73)/3*x^2*z*v^3+11437*x*y*z*v^3-11316*y^2*z*v^3-F(50)/73*x*z^2*v^3+145*y*z^2*v^3-3021*z^3*v^3-F(67)/124*x^2*u*v^3+5061*x*z*u*v^3-F(107)/56*y*z*u*v^3+153*z^2*u*v^3-F(79)/7*x^2*v^4-3080*x*z*v^4-F(35)/16*y*z*v^4-5480*z^2*v^4,x^4*y*z+F(32)/121*x^3*y^2*z+F(6)/49*x^2*y^3*z-F(107)/4*x*y^4*z+F(125)/116*y^5*z-12128*x^3*y*z^2+7957*x^2*y^2*z^2-F(85)/94*x*y^3*z^2-F(115)/96*y^4*z^2-F(10)/119*x^2*y*z^3-11392*x*y^2*z^3-F(2)/19*y^3*z^3-5102*x*y*z^4-F(70)/33*y^2*z^4-F(22)/21*y*z^5+F(115)/116*x^2*y^3*u+F(108)/101*x*y^4*u+F(57)/23*y^5*u+F(56)/51*x^4*z*u+F(53)/83*x^3*y*z*u-F(64)/3*x^2*y^2*z*u-9797*x*y^3*z*u-12990*y^4*z*u-12256*x^3*z^2*u-F(49)/2*x^2*y*z^2*u+623*x*y^2*z^2*u+F(97)/114*y^3*z^2*u+F(9)/46*x^2*z^3*u-F(34)/69*x*y*z^3*u+F(47)/62*y^2*z^3*u+F(55)/84*x*z^4*u+F(5)/6*y*z^4*u+15474*z^5*u-F(38)/47*x^2*y^2*u^2+14385*x*y^3*u^2+F(62)/125*y^4*u^2-F(103)/77*x^3*z*u^2+F(120)/113*x^2*y*z*u^2-8652*x*y^2*z*u^2-F(107)/20*y^3*z*u^2-F(90)/11*x^2*z^2*u^2-F(25)/61*x*y*z^2*u^2-F(3)/88*y^2*z^2*u^2-F(13)/6*x*z^3*u^2+F(41)/60*y*z^3*u^2+F(8)/35*z^4*u^2+F(58)/19*x*y^2*u^3+8738*y^3*u^3-F(109)/110*x^2*z*u^3-398*x*y*z*u^3+6701*y^2*z*u^3+14409*x*z^2*u^3-F(77)/9*y*z^2*u^3-F(85)/52*z^3*u^3-13298*y^2*u^4-4016*x*z*u^4+6306*y*z*u^4-10115*z^2*u^4+F(83)/109*x^2*y^3*v+4541*x*y^4*v-8277*y^5*v+F(25)/116*x^4*z*v+F(115)/113*x^3*y*z*v-4143*x^2*y^2*z*v-2522*x*y^3*z*v+9354*y^4*z*v+13440*x^3*z^2*v-F(38)/47*x^2*y*z^2*v-F(72)/19*x*y^2*z^2*v-8681*y^3*z^2*v+9937*x^2*z^3*v-F(47)/76*x*y*z^3*v-10954*y^2*z^3*v+13261*x*z^4*v-F(83)/33*y*z^4*v-F(38)/23*z^5*v+12490*x^2*y^2*u*v+14095*x*y^3*u*v-F(50)/121*y^4*u*v-F(29)/92*x^3*z*u*v+11229*x^2*y*z*u*v-F(47)/75*x*y^2*z*u*v-F(61)/10*y^3*z*u*v+F(46)/93*x^2*z^2*u*v-F(109)/86*x*y*z^2*u*v-270*y^2*z^2*u*v-F(3)/97*x*z^3*u*v+F(11)/51*y*z^3*u*v-F(83)/86*z^4*u*v-F(115)/49*x*y^2*u^2*v+F(9)/25*y^3*u^2*v+F(74)/79*x^2*z*u^2*v-F(113)/66*x*y*z*u^2*v+F(12)/13*y^2*z*u^2*v-15374*x*z^2*u^2*v-6104*y*z^2*u^2*v+F(27)/43*z^3*u^2*v-1700*y^2*u^3*v+F(89)/12*x*z*u^3*v+F(123)/8*y*z*u^3*v+F(85)/82*z^2*u^3*v-F(79)/69*x^2*y^2*v^2-12943*x*y^3*v^2-F(60)/17*y^4*v^2-F(91)/4*x^3*z*v^2+F(107)/66*x^2*y*z*v^2-4842*x*y^2*z*v^2-F(41)/89*y^3*z*v^2-F(91)/111*x^2*z^2*v^2-8993*x*y*z^2*v^2-F(88)/7*y^2*z^2*v^2-F(112)/71*x*z^3*v^2+F(49)/86*y*z^3*v^2-F(85)/14*z^4*v^2-F(50)/117*x*y^2*u*v^2-F(43)/66*y^3*u*v^2+F(44)/41*x^2*z*u*v^2-13691*x*y*z*u*v^2+F(18)/17*y^2*z*u*v^2-435*x*z^2*u*v^2-F(89)/37*y*z^2*u*v^2+F(86)/97*z^3*u*v^2+F(102)/95*y^2*u^2*v^2-F(67)/63*x*z*u^2*v^2+F(82)/91*y*z*u^2*v^2-F(28)/73*z^2*u^2*v^2-F(76)/35*x*y^2*v^3-6357*y^3*v^3-7772*x^2*z*v^3-2736*x*y*z*v^3+164*y^2*z*v^3+2418*x*z^2*v^3-F(79)/122*y*z^2*v^3-14300*z^3*v^3-14198*y^2*u*v^3+10811*x*z*u*v^3-5061*y*z*u*v^3+F(77)/46*z^2*u*v^3-F(108)/95*y^2*v^4-6281*x*z*v^4+3080*y*z*v^4+101*z^2*v^4,x^5*y+F(32)/121*x^4*y^2+F(6)/49*x^3*y^3-F(107)/4*x^2*y^4+F(125)/116*x*y^5-12128*x^4*y*z+7957*x^3*y^2*z-F(85)/94*x^2*y^3*z-F(115)/96*x*y^4*z-F(10)/119*x^3*y*z^2-11392*x^2*y^2*z^2-F(2)/19*x*y^3*z^2-5102*x^2*y*z^3-F(70)/33*x*y^2*z^3-F(22)/21*x*y*z^4+F(56)/51*x^5*u+F(53)/83*x^4*y*u+11580*x^3*y^2*u-10076*x^2*y^3*u+5420*x*y^4*u+F(55)/31*y^5*u-12256*x^4*z*u+796*x^3*y*z*u-F(93)/64*x^2*y^2*z*u+F(94)/93*x*y^3*z*u+5578*y^4*z*u+15142*x^3*z^2*u+F(113)/37*x^2*y*z^2*u-F(49)/22*x*y^2*z^2*u+F(53)/87*y^3*z^2*u+F(119)/125*x*y*z^3*u-F(57)/35*y^2*z^3*u-3243*y*z^4*u-F(103)/77*x^4*u^2+F(120)/113*x^3*y*u^2+F(79)/94*x^2*y^2*u^2-F(125)/34*x*y^3*u^2-F(97)/67*y^4*u^2+3572*x^3*z*u^2+361*x^2*y*z*u^2+F(75)/17*x*y^2*z*u^2+F(121)/98*y^3*z*u^2+F(123)/50*x^2*z^2*u^2-3340*x*y*z^2*u^2+F(79)/12*y^2*z^2*u^2+221*y*z^3*u^2-F(109)/110*x^3*u^3-398*x^2*y*u^3-F(109)/17*x*y^2*u^3+F(89)/125*y^3*u^3+F(55)/27*x^2*z*u^3+61*x*y*z*u^3-15002*y^2*z*u^3+F(55)/122*y*z^2*u^3-4016*x^2*u^4+6306*x*y*u^4+F(62)/43*y^2*u^4+F(35)/122*y*z*u^4+F(25)/116*x^5*v+F(115)/113*x^4*y*v-F(93)/44*x^3*y^2*v-F(31)/27*x^2*y^3*v-15880*x*y^4*v+F(21)/31*y^5*v+13440*x^4*z*v+1660*x^3*y*z*v+7844*x^2*y^2*z*v+F(7)/67*x*y^3*z*v+F(61)/99*y^4*z*v-F(55)/32*x^3*z^2*v+9524*x^2*y*z^2*v+F(23)/19*x*y^2*z^2*v+F(97)/121*y^3*z^2*v-7956*x*y*z^3*v-9107*y^2*z^3*v-F(95)/96*y*z^4*v-F(29)/92*x^4*u*v+11229*x^3*y*u*v+F(16)/41*x^2*y^2*u*v+3422*x*y^3*u*v+F(89)/17*y^4*u*v+F(67)/117*x^3*z*u*v+14991*x^2*y*z*u*v+F(32)/13*x*y^2*z*u*v+6086*y^3*z*u*v+F(103)/70*x^2*z^2*u*v-F(105)/38*x*y*z^2*u*v+F(33)/76*y^2*z^2*u*v-3615*y*z^3*u*v+F(74)/79*x^3*u^2*v-F(113)/66*x^2*y*u^2*v-11489*x*y^2*u^2*v-F(114)/83*y^3*u^2*v-4823*x^2*z*u^2*v+F(100)/41*x*y*z*u^2*v+F(58)/53*y^2*z*u^2*v+6114*y*z^2*u^2*v+F(89)/12*x^2*u^3*v+F(123)/8*x*y*u^3*v+F(89)/49*y^2*u^3*v+7535*y*z*u^3*v-F(91)/4*x^4*v^2+F(107)/66*x^3*y*v^2+F(51)/112*x^2*y^2*v^2-F(87)/22*x*y^3*v^2+F(107)/37*y^4*v^2+F(117)/38*x^3*z*v^2+13527*x^2*y*z*v^2-F(83)/56*x*y^2*z*v^2+F(79)/17*y^3*z*v^2-F(71)/86*x^2*z^2*v^2-6065*x*y*z^2*v^2-F(3)/47*y^2*z^2*v^2-F(119)/74*y*z^3*v^2+F(44)/41*x^3*u*v^2-13691*x^2*y*u*v^2-F(102)/95*x*y^2*u*v^2+F(26)/101*y^3*u*v^2+12912*x^2*z*u*v^2-13361*x*y*z*u*v^2+F(49)/99*y^2*z*u*v^2+F(24)/35*y*z^2*u*v^2-F(67)/63*x^2*u^2*v^2+F(82)/91*x*y*u^2*v^2-F(34)/71*y^2*u^2*v^2+F(49)/55*y*z*u^2*v^2-7772*x^3*v^3-2736*x^2*y*v^3-11437*x*y^2*v^3+11316*y^3*v^3+F(106)/25*x^2*z*v^3+F(50)/73*x*y*z*v^3-145*y^2*z*v^3+3021*y*z^2*v^3+10811*x^2*u*v^3-5061*x*y*u*v^3+F(107)/56*y^2*u*v^3-153*y*z*u*v^3-6281*x^2*v^4+3080*x*y*v^4+F(35)/16*y^2*v^4+5480*y*z*v^4,F(94)/119*x^3*y*u-7642*x^2*y^2*u-14954*x*y^3*u+F(115)/119*x^3*z*u-3182*x^2*y*z*u-F(18)/55*x*y^2*z*u-F(25)/37*y^3*z*u-618*x^2*z^2*u+F(102)/101*x*y*z^2*u-F(117)/4*y^2*z^2*u-F(51)/49*x*z^3*u+F(120)/53*y*z^3*u-F(41)/124*x^3*u^2-F(3)/116*x^2*y*u^2+97*x*y^2*u^2-7410*y^3*u^2-F(109)/13*x^2*z*u^2-F(104)/45*x*y*z*u^2+F(94)/71*y^2*z*u^2-13966*x*z^2*u^2-15760*y*z^2*u^2-2867*z^3*u^2+5490*x^2*u^3-6065*x*y*u^3+7647*y^2*u^3-7215*x*z*u^3-2365*y*z*u^3+F(101)/18*z^2*u^3-2654*x*u^4-3100*y*u^4-11698*z*u^4-6306*u^5-F(104)/29*x^3*y*v-12826*x*y^3*v-F(27)/8*x^3*z*v-5226*x^2*y*z*v-F(45)/89*x*y^2*z*v-F(86)/57*y^3*z*v-11777*x*y*z^2*v+F(11)/82*x*z^3*v-12219*y*z^3*v+F(105)/31*x^3*u*v+15465*x^2*y*u*v+454*x*y^2*u*v+1110*y^3*u*v+F(7)/20*x^2*z*u*v-F(83)/61*x*y*z*u*v-F(4)/55*y^2*z*u*v+4534*x*z^2*u*v+12705*y*z^2*u*v-2123*z^3*u*v+13431*x^2*u^2*v+8942*x*y*u^2*v+F(17)/121*y^2*u^2*v+4380*x*z*u^2*v+F(98)/9*y*z*u^2*v-F(52)/103*z^2*u^2*v-365*x*u^3*v-F(2)/27*y*u^3*v+11079*z*u^3*v-F(123)/8*u^4*v-11676*x^3*v^2+F(41)/49*x^2*y*v^2+15554*x*y^2*v^2+13120*y^3*v^2+F(97)/17*x^2*z*v^2-F(25)/22*x*y*z*v^2+F(14)/107*y^2*z*v^2-4716*x*z^2*v^2+F(75)/101*y*z^2*v^2-1134*z^3*v^2-F(5)/9*x^2*u*v^2+4*x*y*u*v^2+F(84)/17*y^2*u*v^2+F(26)/31*x*z*u*v^2+F(105)/113*y*z*u*v^2-13963*z^2*u*v^2+F(51)/22*x*u^2*v^2-3735*y*u^2*v^2+F(29)/18*z*u^2*v^2-F(82)/91*u^3*v^2+F(84)/121*x^2*v^3-F(55)/81*x*y*v^3-9987*y^2*v^3+9420*x*z*v^3+9471*y*z*v^3-F(74)/7*z^2*v^3-8407*x*u*v^3+F(101)/99*y*u*v^3-F(80)/101*z*u*v^3+5061*u^2*v^3-F(99)/29*x*v^4+F(81)/47*y*v^4+F(95)/22*z*v^4-3080*u*v^4,F(101)/96*x^3*y*u+4988*x*y^3*u+F(47)/82*x^3*z*u-13431*x^2*y*z*u-F(51)/56*x*y^2*z*u-12762*y^3*z*u-F(13)/11*x*y*z^2*u-5177*x*z^3*u+F(21)/17*y*z^3*u-F(117)/109*x^3*u^2+F(121)/74*x^2*y*u^2+4251*x*y^2*u^2+12337*y^3*u^2-F(51)/89*x^2*z*u^2-F(69)/112*x*y*z*u^2+F(17)/23*y^2*z*u^2+F(67)/12*x*z^2*u^2-F(23)/78*y*z^2*u^2+F(21)/41*z^3*u^2+14822*x^2*u^3+F(63)/16*x*y*u^3+10314*y^2*u^3-F(116)/119*x*z*u^3+3504*y*z*u^3+71*z^2*u^3-8113*x*u^4-15310*y*u^4+3091*z*u^4+F(9)/2*x^3*y*v-7642*x^2*y^2*v+6983*x*y^3*v+F(111)/31*x^3*z*v+F(53)/109*x^2*y*z*v-F(118)/33*x*y^2*z*v+11305*y^3*z*v-618*x^2*z^2*v+F(2)/97*x*y*z^2*v-F(117)/4*y^2*z^2*v-F(67)/6*x*z^3*v+F(9)/65*y*z^3*v+13568*x^3*u*v+F(57)/4*x^2*y*u*v-11407*x*y^2*u*v+F(5)/111*y^3*u*v-909*x^2*z*u*v+F(103)/102*x*y*z*u*v+F(5)/16*y^2*z*u*v+F(71)/70*x*z^2*u*v-701*y*z^2*u*v+2924*z^3*u*v+F(19)/63*x^2*u^2*v-F(113)/18*x*y*u^2*v+F(40)/91*y^2*u^2*v+F(79)/25*x*z*u^2*v-4540*y*z*u^2*v-F(61)/45*z^2*u^2*v+F(122)/25*x*u^3*v+10588*y*u^3*v-F(39)/85*z*u^3*v-6306*u^4*v+4632*x^3*v^2+7959*x^2*y*v^2-F(69)/11*x*y^2*v^2-F(83)/51*y^3*v^2-F(117)/17*x^2*z*v^2-6351*x*y*z*v^2-13657*y^2*z*v^2-10735*x*z^2*v^2+7059*y*z^2*v^2+F(104)/81*z^3*v^2-1907*x^2*u*v^2+7368*x*y*u*v^2-F(2)/33*y^2*u*v^2+F(20)/79*x*z*u*v^2-F(118)/95*y*z*u*v^2+F(59)/75*z^2*u*v^2-11735*x*u^2*v^2-F(29)/44*y*u^2*v^2+F(37)/15*z*u^2*v^2-F(123)/8*u^3*v^2-F(85)/114*x^2*v^3+1717*x*y*v^3+13915*y^2*v^3-5490*x*z*v^3-F(98)/5*y*z*v^3+F(91)/64*z^2*v^3-F(71)/73*x*u*v^3-12963*y*u*v^3-F(23)/106*z*u*v^3-F(82)/91*u^2*v^3-11190*x*v^4-2767*y*v^4-F(76)/33*z*v^4+5061*u*v^4-3080*v^5,-F(76)/81*x^3*y*u+F(4)/107*x^2*y^2*u-F(112)/85*x*y^3*u-F(56)/51*x^3*z*u-3085*x^2*y*z*u+F(83)/36*x*y^2*z*u-342*y^3*z*u+12256*x^2*z^2*u-F(46)/43*x*y*z^2*u-9763*y^2*z^2*u-15142*x*z^3*u-F(79)/122*y*z^3*u-F(19)/82*x^2*y*u^2-1824*x*y^2*u^2+F(71)/32*y^3*u^2+F(103)/77*x^2*z*u^2+F(113)/96*x*y*z*u^2+F(17)/59*y^2*z*u^2-3572*x*z^2*u^2-11567*y*z^2*u^2-F(123)/50*z^3*u^2+F(8)/27*x*y*u^3+11950*y^2*u^3+F(109)/110*x*z*u^3+F(5)/37*y*z*u^3-F(55)/27*z^2*u^3-4433*y*u^4+4016*z*u^4+F(121)/12*x^3*y*v+8439*x^2*y^2*v-F(3)/95*x*y^3*v-F(25)/116*x^3*z*v+F(74)/41*x^2*y*z*v+F(102)/53*x*y^2*z*v+8924*y^3*z*v-13440*x^2*z^2*v+F(6)/97*x*y*z^2*v-F(83)/57*y^2*z^2*v+F(55)/32*x*z^3*v+F(37)/107*y*z^3*v-10326*x^2*y*u*v-F(105)/46*x*y^2*u*v+10781*y^3*u*v+F(29)/92*x^2*z*u*v+F(21)/17*x*y*z*u*v-F(110)/49*y^2*z*u*v-F(67)/117*x*z^2*u*v+F(35)/53*y*z^2*u*v-F(103)/70*z^3*u*v-5442*x*y*u^2*v-15433*y^2*u^2*v-F(74)/79*x*z*u^2*v-F(19)/11*y*z*u^2*v+4823*z^2*u^2*v-F(49)/13*y*u^3*v-F(89)/12*z*u^3*v+F(35)/24*x^2*y*v^2-F(104)/41*x*y^2*v^2-2931*y^3*v^2+F(91)/4*x^2*z*v^2+4438*x*y*z*v^2-F(31)/78*y^2*z*v^2-F(117)/38*x*z^2*v^2+11769*y*z^2*v^2+F(71)/86*z^3*v^2-7786*x*y*u*v^2+F(44)/71*y^2*u*v^2-F(44)/41*x*z*u*v^2+8894*y*z*u*v^2-12912*z^2*u*v^2-14048*y*u^2*v^2+F(67)/63*z*u^2*v^2+4152*x*y*v^3+8876*y^2*v^3+7772*x*z*v^3-7952*y*z*v^3-F(106)/25*z^2*v^3+F(67)/124*y*u*v^3-10811*z*u*v^3+F(79)/7*y*v^4+6281*z*v^4,-x^3*y*u+F(9)/2*x^2*y^2*u-13268*x*y^3*u+F(111)/31*x^2*y*z*u-F(15)/67*x*y^2*z*u-F(53)/58*x*y*z^2*u-F(56)/51*x^3*u^2+F(103)/39*x^2*y*u^2-F(125)/49*x*y^2*u^2-F(73)/113*y^3*u^2+12256*x^2*z*u^2+F(112)/27*x*y*z*u^2+10058*y^2*z*u^2-15142*x*z^2*u^2+F(88)/21*y*z^2*u^2+F(103)/77*x^2*u^3+F(71)/12*x*y*u^3-F(110)/97*y^2*u^3-3572*x*z*u^3+15410*y*z*u^3-F(123)/50*z^2*u^3+F(109)/110*x*u^4+F(110)/21*y*u^4-F(55)/27*z*u^4+4016*u^5+F(104)/29*x^2*y^2*v+8710*x*y^3*v+F(27)/8*x^2*y*z*v+5226*x*y^2*z*v-F(118)/61*x*y*z^2*v-F(25)/116*x^3*u*v-7009*x^2*y*u*v+F(109)/125*x*y^2*u*v+F(98)/87*y^3*u*v-13440*x^2*z*u*v+F(41)/57*x*y*z*u*v+10332*y^2*z*u*v+F(55)/32*x*z^2*u*v+F(74)/5*y*z^2*u*v+F(29)/92*x^2*u^2*v-F(13)/48*x*y*u^2*v-F(91)/93*y^2*u^2*v-F(67)/117*x*z*u^2*v+156*y*z*u^2*v-F(103)/70*z^2*u^2*v-F(74)/79*x*u^3*v-118*y*u^3*v+4823*z*u^3*v-F(89)/12*u^4*v+11676*x^2*y*v^2+F(26)/31*x*y^2*v^2-F(17)/38*y^3*v^2+F(91)/59*x*y*z*v^2+F(67)/81*y^2*z*v^2+F(47)/55*y*z^2*v^2+F(91)/4*x^2*u*v^2-14704*x*y*u*v^2-F(43)/42*y^2*u*v^2-F(117)/38*x*z*u*v^2-F(67)/52*y*z*u*v^2+F(71)/86*z^2*u*v^2-F(44)/41*x*u^2*v^2+F(99)/5*y*u^2*v^2-12912*z*u^2*v^2+F(67)/63*u^3*v^2-F(84)/121*x*y*v^3-F(66)/25*y^2*v^3+1870*y*z*v^3+7772*x*u*v^3+F(6)/89*y*u*v^3-F(106)/25*z*u*v^3-10811*u^2*v^3+F(99)/29*y*v^4+6281*u*v^4,-F(101)/96*x^2*y^2*u-F(108)/31*x*y^3*u-F(47)/82*x^2*y*z*u+13431*x*y^2*z*u-F(89)/70*x*y*z^2*u+F(117)/109*x^2*y*u^2-F(15)/104*x*y^2*u^2-F(97)/6*y^3*u^2+F(19)/16*x*y*z*u^2+F(109)/4*y^2*z*u^2+F(41)/56*y*z^2*u^2-14822*x*y*u^3-F(27)/100*y^2*u^3-12548*y*z*u^3+8113*y*u^4-x^3*y*v+F(94)/119*x^2*y^2*v-F(119)/30*x*y^3*v+F(115)/119*x^2*y*z*v+F(74)/113*x*y^2*z*v+F(82)/25*x*y*z^2*v-F(56)/51*x^3*u*v-8558*x^2*y*u*v-F(45)/124*x*y^2*u*v+F(74)/101*y^3*u*v+12256*x^2*z*u*v-3040*x*y*z*u*v+F(51)/71*y^2*z*u*v-15142*x*z^2*u*v-F(109)/4*y*z^2*u*v+F(103)/77*x^2*u^2*v-5295*x*y*u^2*v+F(33)/125*y^2*u^2*v-3572*x*z*u^2*v+F(47)/51*y*z*u^2*v-F(123)/50*z^2*u^2*v+F(109)/110*x*u^3*v-F(8)/13*y*u^3*v-F(55)/27*z*u^3*v+4016*u^4*v-F(25)/116*x^3*v^2+F(87)/43*x^2*y*v^2-F(117)/97*x*y^2*v^2+7393*y^3*v^2-13440*x^2*z*v^2+F(121)/57*x*y*z*v^2+4841*y^2*z*v^2+F(55)/32*x*z^2*v^2+4753*y*z^2*v^2+F(29)/92*x^2*u*v^2-9322*x*y*u*v^2+9730*y^2*u*v^2-F(67)/117*x*z*u*v^2-F(17)/100*y*z*u*v^2-F(103)/70*z^2*u*v^2-F(74)/79*x*u^2*v^2+11252*y*u^2*v^2+4823*z*u^2*v^2-F(89)/12*u^3*v^2+F(91)/4*x^2*v^3+F(55)/112*x*y*v^3-5968*y^2*v^3-F(117)/38*x*z*v^3+F(67)/106*y*z*v^3+F(71)/86*z^2*v^3-F(44)/41*x*u*v^3+F(17)/33*y*u*v^3-12912*z*u*v^3+F(67)/63*u^2*v^3+7772*x*v^4+F(43)/85*y*v^4-F(106)/25*z*v^4-10811*u*v^4+6281*v^5,-x^3*z*u+F(9)/2*x^2*y*z*u-13268*x*y^2*z*u+F(111)/31*x^2*z^2*u-F(15)/67*x*y*z^2*u-F(53)/58*x*z^3*u+F(76)/81*x^3*u^2-F(4)/107*x^2*y*u^2+F(112)/85*x*y^2*u^2-13318*x^2*z*u^2-F(45)/13*x*y*z*u^2-F(111)/71*y^2*z*u^2+3422*x*z^2*u^2+F(45)/92*y*z^2*u^2-F(101)/23*z^3*u^2+F(19)/82*x^2*u^3+1824*x*y*u^3-F(71)/32*y^2*u^3-F(101)/49*x*z*u^3+F(83)/91*y*z*u^3-5014*z^2*u^3-F(8)/27*x*u^4-11950*y*u^4+F(69)/70*z*u^4+4433*u^5+F(104)/29*x^2*y*z*v+8710*x*y^2*z*v+F(27)/8*x^2*z^2*v+5226*x*y*z^2*v-F(118)/61*x*z^3*v-F(121)/12*x^3*u*v-8439*x^2*y*u*v+F(3)/95*x*y^2*u*v-4670*x^2*z*u*v-5419*x*y*z*u*v+F(83)/33*y^2*z*u*v+1019*x*z^2*u*v+F(62)/61*y*z^2*u*v-15114*z^3*u*v+10326*x^2*u^2*v+F(105)/46*x*y*u^2*v-10781*y^2*u^2*v-5843*x*z*u^2*v-F(71)/16*y*z*u^2*v+9813*z^2*u^2*v+5442*x*u^3*v+15433*y*u^3*v-F(109)/126*z*u^3*v+F(49)/13*u^4*v+11676*x^2*z*v^2+F(26)/31*x*y*z*v^2-F(17)/38*y^2*z*v^2+F(91)/59*x*z^2*v^2+F(67)/81*y*z^2*v^2+F(47)/55*z^3*v^2-F(35)/24*x^2*u*v^2+F(104)/41*x*y*u*v^2+2931*y^2*u*v^2+F(19)/122*x*z*u*v^2-F(57)/91*y*z*u*v^2-F(117)/62*z^2*u*v^2+7786*x*u^2*v^2-F(44)/71*y*u^2*v^2-2476*z*u^2*v^2+14048*u^3*v^2-F(84)/121*x*z*v^3-F(66)/25*y*z*v^3+1870*z^2*v^3-4152*x*u*v^3-8876*y*u*v^3-12896*z*u*v^3-F(67)/124*u^2*v^3+F(99)/29*z*v^4-F(79)/7*u*v^4,-F(101)/96*x^2*y*z*u-F(108)/31*x*y^2*z*u-F(47)/82*x^2*z^2*u+13431*x*y*z^2*u-F(89)/70*x*z^3*u+F(117)/109*x^2*z*u^2-F(15)/104*x*y*z*u^2-F(97)/6*y^2*z*u^2+F(19)/16*x*z^2*u^2+F(109)/4*y*z^2*u^2+F(41)/56*z^3*u^2-14822*x*z*u^3-F(27)/100*y*z*u^3-12548*z^2*u^3+8113*z*u^4-x^3*z*v+F(94)/119*x^2*y*z*v-F(119)/30*x*y^2*z*v+F(115)/119*x^2*z^2*v+F(74)/113*x*y*z^2*v+F(82)/25*x*z^3*v+F(76)/81*x^3*u*v-F(4)/107*x^2*y*u*v+F(112)/85*x*y^2*u*v-F(65)/76*x^2*z*u*v-60*x*y*z*u*v-F(112)/81*y^2*z*u*v-F(60)/119*x*z^2*u*v-11864*y*z^2*u*v+F(95)/26*z^3*u*v+F(19)/82*x^2*u^2*v+1824*x*y*u^2*v-F(71)/32*y^2*u^2*v-F(37)/27*x*z*u^2*v-F(85)/107*y*z*u^2*v+7177*z^2*u^2*v-F(8)/27*x*u^3*v-11950*y*u^3*v+F(53)/18*z*u^3*v+4433*u^4*v-F(121)/12*x^3*v^2-8439*x^2*y*v^2+F(3)/95*x*y^2*v^2+3085*x^2*z*v^2+F(71)/29*x*y*z*v^2-1531*y^2*z*v^2-11327*x*z^2*v^2+F(72)/83*y*z^2*v^2+15217*z^3*v^2+10326*x^2*u*v^2+F(105)/46*x*y*u*v^2-10781*y^2*u*v^2+9495*x*z*u*v^2+11038*y*z*u*v^2-F(29)/64*z^2*u*v^2+5442*x*u^2*v^2+15433*y*u^2*v^2+F(125)/61*z*u^2*v^2+F(49)/13*u^3*v^2-F(35)/24*x^2*v^3+F(104)/41*x*y*v^3+2931*y^2*v^3-153*x*z*v^3+F(16)/89*y*z*v^3+F(101)/53*z^2*v^3+7786*x*u*v^3-F(44)/71*y*u*v^3+10495*z*u*v^3+14048*u^2*v^3-4152*x*v^4-8876*y*v^4-10113*z*v^4-F(67)/124*u*v^4-F(79)/7*v^5,-F(115)/116*x^3*y*u-F(108)/101*x^2*y^2*u-F(57)/23*x*y^3*u-F(92)/125*x^3*z*u-F(94)/115*x^2*y*z*u+F(9)/106*x*y^2*z*u+F(55)/31*y^3*z*u-172*x^2*z^2*u-15893*x*y*z^2*u+5578*y^2*z^2*u-F(8)/97*x*z^3*u-10030*y*z^3*u+F(38)/47*x^3*u^2-14385*x^2*y*u^2-F(62)/125*x*y^2*u^2+F(67)/24*x^2*z*u^2-13077*x*y*z*u^2-F(97)/67*y^2*z*u^2+F(57)/124*x*z^2*u^2-5773*y*z^2*u^2-F(98)/9*z^3*u^2-F(58)/19*x^2*u^3-8738*x*y*u^3+10229*x*z*u^3+F(89)/125*y*z*u^3+15450*z^2*u^3+13298*x*u^4+F(62)/43*z*u^4-F(83)/109*x^3*y*v-4541*x^2*y^2*v+8277*x*y^3*v+F(125)/6*x^3*z*v+F(73)/24*x^2*y*z*v-F(118)/71*x*y^2*z*v+F(21)/31*y^3*z*v+F(109)/29*x^2*z^2*v-F(123)/92*x*y*z^2*v+F(61)/99*y^2*z^2*v-F(67)/92*x*z^3*v+F(41)/26*y*z^3*v-12490*x^3*u*v-14095*x^2*y*u*v+F(50)/121*x*y^2*u*v+F(74)/83*x^2*z*u*v+229*x*y*z*u*v+F(89)/17*y^2*z*u*v+9487*x*z^2*u*v+F(5)/7*y*z^2*u*v-5399*z^3*u*v+F(115)/49*x^2*u^2*v-F(9)/25*x*y*u^2*v+5736*x*z*u^2*v-F(114)/83*y*z*u^2*v-10060*z^2*u^2*v+1700*x*u^3*v+F(89)/49*z*u^3*v+F(79)/69*x^3*v^2+12943*x^2*y*v^2+F(60)/17*x*y^2*v^2-F(123)/95*x^2*z*v^2+F(86)/111*x*y*z*v^2+F(107)/37*y^2*z*v^2-F(70)/69*x*z^2*v^2+9221*y*z^2*v^2-F(19)/103*z^3*v^2+F(50)/117*x^2*u*v^2+F(43)/66*x*y*u*v^2-F(49)/88*x*z*u*v^2+F(26)/101*y*z*u*v^2-F(97)/35*z^2*u*v^2-F(102)/95*x*u^2*v^2-F(34)/71*z*u^2*v^2+F(76)/35*x^2*v^3+6357*x*y*v^3+F(12)/91*x*z*v^3+11316*y*z*v^3+F(38)/103*z^2*v^3+14198*x*u*v^3+F(107)/56*z*u*v^3+F(108)/95*x*v^4+F(35)/16*z*v^4,-F(119)/30*x^3*y*u+6983*x^2*y^2*u-F(125)/116*x*y^3*u+F(17)/54*x^2*y*z*u+11305*x*y^2*z*u+F(67)/34*x*y*z^2*u-2848*x^3*u^2-F(52)/123*x^2*y*u^2+1990*x*y^2*u^2-F(55)/31*y^3*u^2-8911*x^2*z*u^2+12594*x*y*z*u^2-5578*y^2*z*u^2+F(41)/85*x*z^2*u^2+10030*y*z^2*u^2-F(13)/6*x^2*u^3-F(78)/25*x*y*u^3+F(97)/67*y^2*u^3+F(26)/45*x*z*u^3+5773*y*z*u^3+F(98)/9*z^2*u^3-13830*x*u^4-F(89)/125*y*u^4-15450*z*u^4-F(62)/43*u^5-8710*x^3*y*v+12826*x^2*y^2*v+F(45)/89*x^2*y*z*v+F(86)/57*x*y^2*z*v+F(37)/79*x*y*z^2*v-12666*x^3*u*v+4408*x^2*y*u*v+F(64)/13*x*y^2*u*v-F(21)/31*y^3*u*v+10217*x^2*z*u*v+F(64)/41*x*y*z*u*v-F(61)/99*y^2*z*u*v-F(79)/126*x*z^2*u*v-F(41)/26*y*z^2*u*v+F(29)/38*x^2*u^2*v+11648*x*y*u^2*v-F(89)/17*y^2*u^2*v-F(85)/52*x*z*u^2*v-F(5)/7*y*z*u^2*v+5399*z^2*u^2*v-3914*x*u^3*v+F(114)/83*y*u^3*v+10060*z*u^3*v-F(89)/49*u^4*v+F(95)/69*x^3*v^2+F(19)/61*x^2*y*v^2-13120*x*y^2*v^2+9081*x^2*z*v^2+F(19)/49*x*y*z*v^2+2336*x*z^2*v^2-11524*x^2*u*v^2+1111*x*y*u*v^2-F(107)/37*y^2*u*v^2-F(41)/59*x*z*u*v^2-9221*y*z*u*v^2+F(19)/103*z^2*u*v^2+13165*x*u^2*v^2-F(26)/101*y*u^2*v^2+F(97)/35*z*u^2*v^2+F(34)/71*u^3*v^2+12152*x^2*v^3+9987*x*y*v^3-11390*x*z*v^3+6912*x*u*v^3-11316*y*u*v^3-F(38)/103*z*u*v^3-F(107)/56*u^2*v^3-F(81)/47*x*v^4-F(35)/16*u*v^4,F(108)/31*x^3*y*u-4988*x^2*y^2*u+F(51)/56*x^2*y*z*u+12762*x*y^2*z*u-F(98)/101*x*y*z^2*u-F(75)/28*x^3*u^2+1097*x^2*y*u^2-12337*x*y^2*u^2+7424*x^2*z*u^2-11410*x*y*z*u^2+568*x*z^2*u^2-13040*x^2*u^3-10314*x*y*u^3-12090*x*z*u^3+15310*x*u^4-13268*x^3*y*v-14954*x^2*y^2*v-F(125)/116*x*y^3*v-F(81)/74*x^2*y*z*v-F(25)/37*x*y^2*z*v+11145*x*y*z^2*v+F(63)/4*x^3*u*v+894*x^2*y*u*v+F(122)/95*x*y^2*u*v-F(55)/31*y^3*u*v+F(64)/39*x^2*z*u*v-8911*x*y*z*u*v-5578*y^2*z*u*v-F(59)/51*x*z^2*u*v+10030*y*z^2*u*v+F(20)/11*x^2*u^2*v+F(124)/99*x*y*u^2*v+F(97)/67*y^2*u^2*v-5290*x*z*u^2*v+5773*y*z*u^2*v+F(98)/9*z^2*u^2*v+F(106)/93*x*u^3*v-F(89)/125*y*u^3*v-15450*z*u^3*v-F(62)/43*u^4*v+F(57)/32*x^3*v^2-9109*x^2*y*v^2-12973*x*y^2*v^2-F(21)/31*y^3*v^2+F(2)/27*x^2*z*v^2+15370*x*y*z*v^2-F(61)/99*y^2*z*v^2-F(110)/97*x*z^2*v^2-F(41)/26*y*z^2*v^2+F(52)/47*x^2*u*v^2-F(103)/104*x*y*u*v^2-F(89)/17*y^2*u*v^2-F(100)/43*x*z*u*v^2-F(5)/7*y*z*u*v^2+5399*z^2*u*v^2+F(109)/55*x*u^2*v^2+F(114)/83*y*u^2*v^2+10060*z*u^2*v^2-F(89)/49*u^3*v^2+14819*x^2*v^3+7901*x*y*v^3-F(107)/37*y^2*v^3-5480*x*z*v^3-9221*y*z*v^3+F(19)/103*z^2*v^3-F(7)/10*x*u*v^3-F(26)/101*y*u*v^3+F(97)/35*z*u*v^3+F(34)/71*u^2*v^3+14204*x*v^4-11316*y*v^4-F(38)/103*z*v^4-F(107)/56*u*v^4-F(35)/16*v^5,F(119)/30*x^2*y*z*u-6983*x*y^2*z*u+F(125)/116*y^3*z*u-F(17)/54*x*y*z^2*u-11305*y^2*z^2*u-F(67)/34*y*z^3*u+F(115)/116*x^2*y*u^2+F(108)/101*x*y^2*u^2+F(57)/23*y^3*u^2+12574*x^2*z*u^2+F(58)/3*x*y*z*u^2+F(4)/69*y^2*z*u^2-F(70)/81*x*z^2*u^2+F(93)/97*y*z^2*u^2-F(101)/10*z^3*u^2-F(38)/47*x^2*u^3+14385*x*y*u^3+F(62)/125*y^2*u^3-F(5)/8*x*z*u^3-F(29)/45*y*z*u^3+F(19)/48*z^2*u^3+F(58)/19*x*u^4+8738*y*u^4+3601*z*u^4-13298*u^5+8710*x^2*y*z*v-12826*x*y^2*z*v-F(45)/89*x*y*z^2*v-F(86)/57*y^2*z^2*v-F(37)/79*y*z^3*v+F(83)/109*x^2*y*u*v+4541*x*y^2*u*v-8277*y^3*u*v+F(116)/105*x^2*z*u*v-F(79)/39*x*y*z*u*v-F(37)/107*y^2*z*u*v+11842*x*z^2*u*v-9338*y*z^2*u*v-6186*z^3*u*v+12490*x^2*u^2*v+14095*x*y*u^2*v-F(50)/121*y^2*u^2*v+6216*x*z*u^2*v-11877*y*z*u^2*v-F(41)/48*z^2*u^2*v-F(115)/49*x*u^3*v+F(9)/25*y*u^3*v-1822*z*u^3*v-1700*u^4*v-F(95)/69*x^2*z*v^2-F(19)/61*x*y*z*v^2+13120*y^2*z*v^2-9081*x*z^2*v^2-F(19)/49*y*z^2*v^2-2336*z^3*v^2-F(79)/69*x^2*u*v^2-12943*x*y*u*v^2-F(60)/17*y^2*u*v^2-F(5)/102*x*z*u*v^2-F(51)/83*y*z*u*v^2+3035*z^2*u*v^2-F(50)/117*x*u^2*v^2-F(43)/66*y*u^2*v^2+7557*z*u^2*v^2+F(102)/95*u^3*v^2-12152*x*z*v^3-9987*y*z*v^3+11390*z^2*v^3-F(76)/35*x*u*v^3-6357*y*u*v^3+F(77)/116*z*u*v^3-14198*u^2*v^3+F(81)/47*z*v^4-F(108)/95*u*v^4,-F(108)/31*x^2*y*z*u+4988*x*y^2*z*u-F(51)/56*x*y*z^2*u-12762*y^2*z^2*u+F(98)/101*y*z^3*u+F(75)/28*x^2*z*u^2-1097*x*y*z*u^2+12337*y^2*z*u^2-7424*x*z^2*u^2+11410*y*z^2*u^2-568*z^3*u^2+13040*x*z*u^3+10314*y*z*u^3+12090*z^2*u^3-15310*z*u^4+13268*x^2*y*z*v+14954*x*y^2*z*v+F(125)/116*y^3*z*v+F(81)/74*x*y*z^2*v+F(25)/37*y^2*z^2*v-11145*y*z^3*v+F(115)/116*x^2*y*u*v+F(108)/101*x*y^2*u*v+F(57)/23*y^3*u*v-F(73)/56*x^2*z*u*v+F(11)/52*x*y*z*u*v-F(65)/73*y^2*z*u*v-15415*x*z^2*u*v-7187*y*z^2*u*v-F(49)/117*z^3*u*v-F(38)/47*x^2*u^2*v+14385*x*y*u^2*v+F(62)/125*y^2*u^2*v+15385*x*z*u^2*v+F(37)/39*y*z*u^2*v-F(117)/17*z^2*u^2*v+F(58)/19*x*u^3*v+8738*y*u^3*v-14702*z*u^3*v-13298*u^4*v+F(83)/109*x^2*y*v^2+4541*x*y^2*v^2-8277*y^3*v^2-11686*x^2*z*v^2-F(55)/107*x*y*z*v^2+6216*y^2*z*v^2-F(87)/76*x*z^2*v^2-F(40)/107*y*z^2*v^2+F(121)/13*z^3*v^2+12490*x^2*u*v^2+14095*x*y*u*v^2-F(50)/121*y^2*u*v^2+13685*x*z*u*v^2-10379*y*z*u*v^2-F(80)/103*z^2*u*v^2-F(115)/49*x*u^2*v^2+F(9)/25*y*u^2*v^2+F(62)/23*z*u^2*v^2-1700*u^3*v^2-F(79)/69*x^2*v^3-12943*x*y*v^3-F(60)/17*y^2*v^3-F(78)/95*x*z*v^3+7085*y*z*v^3+F(13)/84*z^2*v^3-F(50)/117*x*u*v^3-F(43)/66*y*u*v^3-1671*z*u*v^3+F(102)/95*u^2*v^3-F(76)/35*x*v^4-6357*y*v^4+F(79)/86*z*v^4-14198*u*v^4-F(108)/95*v^5,F(82)/25*x^3*z*u+F(34)/105*x^2*y*z*u+11145*x*y^2*z*u-F(67)/6*x^2*z^2*u+F(9)/65*x*y*z^2*u+F(22)/21*x*z^3*u-11820*x^3*u^2+11842*x^2*y*u^2+F(65)/111*x*y^2*u^2-F(77)/36*x^2*z*u^2+F(33)/95*x*y*z*u^2+13503*y^2*z*u^2-7627*x*z^2*u^2+F(57)/35*y*z^2*u^2+3243*z^3*u^2-F(23)/86*x^2*u^3-F(76)/97*x*y*u^3+6304*y^2*u^3+F(19)/121*x*z*u^3+F(65)/98*y*z*u^3-221*z^2*u^3+F(43)/11*x*u^4-F(101)/104*y*u^4-F(55)/122*z*u^4-F(35)/122*u^5+F(118)/61*x^3*z*v+11777*x^2*y*z*v-F(37)/79*x*y^2*z*v-F(11)/82*x^2*z^2*v+12219*x*y*z^2*v-2531*x^3*u*v+11466*x^2*y*u*v+F(1)/58*x*y^2*u*v+F(81)/25*x^2*z*u*v-13494*x*y*z*u*v+F(28)/23*y^2*z*u*v-F(26)/73*x*z^2*u*v+9107*y*z^2*u*v+F(95)/96*z^3*u*v-F(2)/77*x^2*u^2*v+F(88)/45*x*y*u^2*v+3055*y^2*u^2*v+5880*x*z*u^2*v+11017*y*z*u^2*v+3615*z^2*u^2*v+7645*x*u^3*v+5029*y*u^3*v-6114*z*u^3*v-7535*u^4*v+F(16)/77*x^3*v^2+6843*x^2*y*v^2+F(49)/82*x*y^2*v^2-F(59)/93*x^2*z*v^2+F(46)/95*x*y*z*v^2+1134*x*z^2*v^2-F(107)/97*x^2*u*v^2-F(104)/123*x*y*u*v^2+12980*y^2*u*v^2-11963*x*z*u*v^2+6747*y*z*u*v^2+F(119)/74*z^2*u*v^2+F(106)/93*x*u^2*v^2+F(32)/75*y*u^2*v^2-F(24)/35*z*u^2*v^2-F(49)/55*u^3*v^2+F(16)/17*x^2*v^3-F(23)/50*x*y*v^3+F(74)/7*x*z*v^3+5593*x*u*v^3-1097*y*u*v^3-3021*z*u*v^3+153*u^2*v^3-F(95)/22*x*v^4-5480*u*v^4,-F(82)/25*x^2*y*z*u-F(34)/105*x*y^2*z*u-11145*y^3*z*u+F(67)/6*x*y*z^2*u-F(9)/65*y^2*z^2*u-F(22)/21*y*z^3*u+F(83)/32*x^2*y*u^2+15103*x*y^2*u^2+12090*y^3*u^2-F(125)/8*x^2*z*u^2+F(57)/50*x*y*z*u^2+5303*y^2*z*u^2+F(55)/84*x*z^2*u^2-8198*y*z^2*u^2+15474*z^3*u^2-F(10)/13*x^2*u^3+F(72)/37*x*y*u^3-F(59)/24*y^2*u^3-F(50)/103*x*z*u^3-7636*y*z*u^3+F(8)/35*z^2*u^3+6113*x*u^4-8152*y*u^4-F(85)/52*z*u^4-10115*u^5-F(118)/61*x^2*y*z*v-11777*x*y^2*z*v+F(37)/79*y^3*z*v+F(11)/82*x*y*z^2*v-12219*y^2*z^2*v-F(97)/118*x^2*y*u*v-14433*x*y^2*u*v-2833*y^3*u*v+F(23)/68*x^2*z*u*v-12205*x*y*z*u*v+7937*y^2*z*u*v+13261*x*z^2*u*v+10477*y*z^2*u*v-F(38)/23*z^3*u*v-F(85)/72*x^2*u^2*v-4021*x*y*u^2*v-13810*y^2*u^2*v-F(104)/41*x*z*u^2*v-116*y*z*u^2*v-F(83)/86*z^2*u^2*v-F(73)/94*x*u^3*v-F(62)/45*y*u^3*v+F(27)/43*z*u^3*v+F(85)/82*u^4*v-F(16)/77*x^2*y*v^2-6843*x*y^2*v^2-F(49)/82*y^3*v^2+F(59)/93*x*y*z*v^2-F(46)/95*y^2*z*v^2-1134*y*z^2*v^2-F(95)/73*x^2*u*v^2-F(89)/26*x*y*u*v^2+F(1)/121*y^2*u*v^2+F(67)/106*x*z*u*v^2-F(7)/15*y*z*u*v^2-F(85)/14*z^2*u*v^2-13347*x*u^2*v^2+F(25)/29*y*u^2*v^2+F(86)/97*z*u^2*v^2-F(28)/73*u^3*v^2-F(16)/17*x*y*v^3+F(23)/50*y^2*v^3-F(74)/7*y*z*v^3+F(16)/123*x*u*v^3+F(85)/69*y*u*v^3-14300*z*u*v^3+F(77)/46*u^2*v^3+F(95)/22*y*v^4+101*u*v^4,-13640*x^2*z*u^2-F(87)/100*x*y*z*u^2+F(44)/111*y^2*z*u^2-F(103)/120*x*z^2*u^2+14669*y*z^2*u^2-F(27)/32*x^2*u^3+F(24)/59*x*y*u^3+1178*y^2*u^3-F(40)/31*x*z*u^3-F(105)/31*y*z*u^3+5302*z^2*u^3-F(79)/5*x*u^4+6525*y*u^4+F(61)/20*z*u^4+F(72)/17*u^5-F(97)/62*x^2*z*u*v-F(89)/56*x*y*z*u*v-14452*y^2*z*u*v+F(96)/55*x*z^2*u*v+F(7)/93*y*z^2*u*v+3029*x^2*u^2*v-5295*x*y*u^2*v+10596*y^2*u^2*v-3956*x*z*u^2*v+F(121)/39*y*z*u^2*v-6983*z^2*u^2*v+F(83)/18*x*u^3*v-F(65)/73*y*u^3*v-6204*z*u^3*v+10187*u^4*v-F(36)/109*x^2*z*v^2+F(125)/56*x*y*z*v^2-4019*y^2*z*v^2-F(41)/11*x*z^2*v^2+11136*y*z^2*v^2-F(96)/101*x^2*u*v^2+F(5)/74*x*y*u*v^2-F(77)/17*y^2*u*v^2-7959*x*z*u*v^2-4933*y*z*u*v^2+F(51)/67*z^2*u*v^2+F(101)/94*x*u^2*v^2-14239*y*u^2*v^2+F(122)/41*z*u^2*v^2+F(63)/52*u^3*v^2+F(107)/56*x^2*v^3+F(77)/71*x*y*v^3+F(37)/50*y^2*v^3-F(70)/53*x*z*v^3+4419*y*z*v^3-2069*z^2*v^3-F(1)/76*x*u*v^3-15126*y*u*v^3+8234*z*u*v^3-F(91)/69*u^2*v^3+F(63)/116*x*v^4-F(110)/117*y*v^4+F(82)/53*z*v^4+7743*u*v^4-9662*v^5,F(17)/78*x^3*y*u-F(99)/19*x^2*y^2*u-12667*x*y^3*u+F(125)/8*x^3*z*u+392*x^2*y*z*u+F(88)/9*x*y^2*z*u-13503*y^3*z*u-F(55)/84*x^2*z^2*u+15825*x*y*z^2*u-F(57)/35*y^2*z^2*u-15474*x*z^3*u-3243*y*z^3*u+F(10)/13*x^3*u^2-F(57)/10*x^2*y*u^2-F(21)/2*x*y^2*u^2-6304*y^3*u^2+F(50)/103*x^2*z*u^2+13188*x*y*z*u^2-F(65)/98*y^2*z*u^2-F(8)/35*x*z^2*u^2+221*y*z^2*u^2-6113*x^2*u^3-F(84)/101*x*y*u^3+F(101)/104*y^2*u^3+F(85)/52*x*z*u^3+F(55)/122*y*z*u^3+10115*x*u^4+F(35)/122*y*u^4-F(109)/111*x^3*y*v-F(120)/97*x^2*y^2*v+F(123)/31*x*y^3*v-F(23)/68*x^3*z*v+685*x^2*y*z*v+5557*x*y^2*z*v-F(28)/23*y^3*z*v-13261*x^2*z^2*v+11435*x*y*z^2*v-9107*y^2*z^2*v+F(38)/23*x*z^3*v-F(95)/96*y*z^3*v+F(85)/72*x^3*u*v-F(67)/72*x^2*y*u*v-1832*x*y^2*u*v-3055*y^3*u*v+F(104)/41*x^2*z*u*v+F(16)/111*x*y*z*u*v-11017*y^2*z*u*v+F(83)/86*x*z^2*u*v-3615*y*z^2*u*v+F(73)/94*x^2*u^2*v+F(33)/109*x*y*u^2*v-5029*y^2*u^2*v-F(27)/43*x*z*u^2*v+6114*y*z*u^2*v-F(85)/82*x*u^3*v+7535*y*u^3*v+F(95)/73*x^3*v^2-9471*x^2*y*v^2+F(107)/2*x*y^2*v^2-12980*y^3*v^2-F(67)/106*x^2*z*v^2+7698*x*y*z*v^2-6747*y^2*z*v^2+F(85)/14*x*z^2*v^2-F(119)/74*y*z^2*v^2+13347*x^2*u*v^2-F(64)/109*x*y*u*v^2-F(32)/75*y^2*u*v^2-F(86)/97*x*z*u*v^2+F(24)/35*y*z*u*v^2+F(28)/73*x*u^2*v^2+F(49)/55*y*u^2*v^2-F(16)/123*x^2*v^3+12024*x*y*v^3+1097*y^2*v^3+14300*x*z*v^3+3021*y*z*v^3-F(77)/46*x*u*v^3-153*y*u*v^3-101*x*v^4+5480*y*v^4,F(89)/70*x^3*z*u+F(13)/11*x^2*y*z*u+F(98)/101*x*y^2*z*u+5177*x^2*z^2*u-F(21)/17*x*y*z^2*u-7257*x^3*u^2+F(123)/125*x^2*y*u^2+15582*x*y^2*u^2+12752*x^2*z*u^2+1483*x*y*z*u^2-F(21)/41*x*z^2*u^2+F(110)/83*x^2*u^3+F(125)/41*x*y*u^3-71*x*z*u^3-3091*x*u^4-F(53)/58*x^3*z*v+13041*x^2*y*z*v+F(67)/34*x*y^2*z*v-F(51)/49*x^2*z^2*v+F(120)/53*x*y*z^2*v+F(22)/21*x*z^3*v-F(74)/71*x^3*u*v-F(55)/8*x^2*y*u*v-F(124)/123*x*y^2*u*v-5356*x^2*z*u*v-3601*x*y*z*u*v+13503*y^2*z*u*v-F(75)/31*x*z^2*u*v+F(57)/35*y*z^2*u*v+3243*z^3*u*v-3652*x^2*u^2*v+F(31)/104*x*y*u^2*v+6304*y^2*u^2*v+6185*x*z*u^2*v+F(65)/98*y*z*u^2*v-221*z^2*u^2*v-1566*x*u^3*v-F(101)/104*y*u^3*v-F(55)/122*z*u^3*v-F(35)/122*u^4*v-7961*x^3*v^2+9069*x^2*y*v^2-F(19)/46*x*y^2*v^2+F(113)/9*x^2*z*v^2+3269*x*y*z*v^2+F(28)/23*y^2*z*v^2-F(116)/83*x*z^2*v^2+9107*y*z^2*v^2+F(95)/96*z^3*v^2-F(86)/107*x^2*u*v^2-F(59)/79*x*y*u*v^2+3055*y^2*u*v^2+6591*x*z*u*v^2+11017*y*z*u*v^2+3615*z^2*u*v^2-9004*x*u^2*v^2+5029*y*u^2*v^2-6114*z*u^2*v^2-7535*u^3*v^2+F(14)/11*x^2*v^3+F(28)/103*x*y*v^3+12980*y^2*v^3-F(43)/7*x*z*v^3+6747*y*z*v^3+F(119)/74*z^2*v^3-F(56)/3*x*u*v^3+F(32)/75*y*u*v^3-F(24)/35*z*u*v^3-F(49)/55*u^2*v^3-F(38)/75*x*v^4-1097*y*v^4-3021*z*v^4+153*u*v^4-5480*v^5,-F(89)/70*x^2*y*z*u-F(13)/11*x*y^2*z*u-F(98)/101*y^3*z*u-5177*x*y*z^2*u+F(21)/17*y^2*z^2*u+7257*x^2*y*u^2-F(123)/125*x*y^2*u^2-15582*y^3*u^2-12752*x*y*z*u^2-1483*y^2*z*u^2+F(21)/41*y*z^2*u^2-F(110)/83*x*y*u^3-F(125)/41*y^2*u^3+71*y*z*u^3+3091*y*u^4+F(53)/58*x^2*y*z*v-13041*x*y^2*z*v-F(67)/34*y^3*z*v+F(51)/49*x*y*z^2*v-F(120)/53*y^2*z^2*v-F(22)/21*y*z^3*v+12022*x^2*y*u*v-9038*x*y^2*u*v+F(91)/118*y^3*u*v-F(125)/8*x^2*z*u*v-F(7)/58*x*y*z*u*v+F(110)/3*y^2*z*u*v+F(55)/84*x*z^2*u*v+F(86)/93*y*z^2*u*v+15474*z^3*u*v-F(10)/13*x^2*u^2*v+F(111)/70*x*y*u^2*v-F(47)/116*y^2*u^2*v-F(50)/103*x*z*u^2*v+12618*y*z*u^2*v+F(8)/35*z^2*u^2*v+6113*x*u^3*v-F(4)/19*y*u^3*v-F(85)/52*z*u^3*v-10115*u^4*v-F(27)/35*x^2*y*v^2+F(22)/101*x*y^2*v^2+F(91)/88*y^3*v^2+F(23)/68*x^2*z*v^2+F(37)/56*x*y*z*v^2-F(26)/29*y^2*z*v^2+13261*x*z^2*v^2-9121*y*z^2*v^2-F(38)/23*z^3*v^2-F(85)/72*x^2*u*v^2-F(82)/3*x*y*u*v^2-192*y^2*u*v^2-F(104)/41*x*z*u*v^2+F(41)/116*y*z*u*v^2-F(83)/86*z^2*u*v^2-F(73)/94*x*u^2*v^2+F(49)/108*y*u^2*v^2+F(27)/43*z*u^2*v^2+F(85)/82*u^3*v^2-F(95)/73*x^2*v^3+12378*x*y*v^3-F(61)/82*y^2*v^3+F(67)/106*x*z*v^3+F(73)/60*y*z*v^3-F(85)/14*z^2*v^3-13347*x*u*v^3-F(12)/101*y*u*v^3+F(86)/97*z*u*v^3-F(28)/73*u^2*v^3+F(16)/123*x*v^4-F(39)/59*y*v^4-14300*z*v^4+F(77)/46*u*v^4+101*v^5,-F(108)/31*x^2*y*u^2+4988*x*y^2*u^2-F(51)/56*x*y*z*u^2-12762*y^2*z*u^2+F(98)/101*y*z^2*u^2+F(75)/28*x^2*u^3-1097*x*y*u^3+12337*y^2*u^3-7424*x*z*u^3+11410*y*z*u^3-568*z^2*u^3+13040*x*u^4+10314*y*u^4+12090*z*u^4-15310*u^5+1534*x^2*y*u*v+F(11)/35*x*y^2*u*v-F(53)/76*x*y*z*u*v-7716*y^2*z*u*v+F(20)/121*y*z^2*u*v-F(29)/81*x^2*u^2*v-F(120)/17*x*y*u^2*v-F(108)/67*y^2*u^2*v-F(43)/111*x*z*u^2*v-10486*y*z*u^2*v-F(73)/32*z^2*u^2*v+3389*x*u^3*v-F(112)/67*y*u^3*v-6084*z*u^3*v+13688*u^4*v-8710*x^2*y*v^2+12826*x*y^2*v^2+F(45)/89*x*y*z*v^2+F(86)/57*y^2*z*v^2+F(37)/79*y*z^2*v^2+F(76)/55*x^2*u*v^2-F(23)/71*x*y*u*v^2-4248*y^2*u*v^2-6371*x*z*u*v^2+3358*y*z*u*v^2+11117*z^2*u*v^2+7469*x*u^2*v^2-F(101)/64*y*u^2*v^2-F(73)/60*z*u^2*v^2+12952*u^3*v^2+F(95)/69*x^2*v^3+F(19)/61*x*y*v^3-13120*y^2*v^3+9081*x*z*v^3+F(19)/49*y*z*v^3+2336*z^2*v^3+F(43)/17*x*u*v^3-6790*y*u*v^3-2654*z*u*v^3+F(9)/52*u^2*v^3+12152*x*v^4+9987*y*v^4-11390*z*v^4-7292*u*v^4-F(81)/47*v^5,-F(101)/96*x^2*y*u^2-F(108)/31*x*y^2*u^2-F(47)/82*x^2*z*u^2+13431*x*y*z*u^2-F(89)/70*x*z^2*u^2+F(117)/109*x^2*u^3-F(15)/104*x*y*u^3-F(97)/6*y^2*u^3+F(19)/16*x*z*u^3+F(109)/4*y*z*u^3+F(41)/56*z^2*u^3-14822*x*u^4-F(27)/100*y*u^4-12548*z*u^4+8113*u^5+7658*x^2*y*u*v+1534*x*y^2*u*v+6961*x^2*z*u*v-2302*x*y*z*u*v+F(31)/60*x*z^2*u*v-F(98)/53*x^2*u^2*v+7326*x*y*u^2*v+206*y^2*u^2*v-4229*x*z*u^2*v+306*y*z*u^2*v+F(35)/11*z^2*u^2*v-F(38)/85*x*u^3*v-4447*y*u^3*v+F(83)/21*z*u^3*v-12896*u^4*v-F(104)/29*x^2*y*v^2-8710*x*y^2*v^2-F(27)/8*x^2*z*v^2-5226*x*y*z*v^2+F(118)/61*x*z^2*v^2-F(13)/33*x^2*u*v^2-F(122)/13*x*y*u*v^2-F(125)/26*y^2*u*v^2+F(80)/57*x*z*u*v^2-5491*y*z*u*v^2-1660*z^2*u*v^2-F(11)/73*x*u^2*v^2+F(95)/46*y*u^2*v^2-4315*z*u^2*v^2+11370*u^3*v^2-11676*x^2*v^3-F(26)/31*x*y*v^3+F(17)/38*y^2*v^3-F(91)/59*x*z*v^3-F(67)/81*y*z*v^3-F(47)/55*z^2*v^3-13002*x*u*v^3+3935*y*u*v^3-F(76)/39*z*u*v^3+F(62)/37*u^2*v^3+F(84)/121*x*v^4+F(66)/25*y*v^4-1870*z*v^4+F(27)/23*u*v^4-F(99)/29*v^5]))
end


@doc raw"""
    k3_d11_pi12()

Return a smooth K3 surface in $\mathbb P^4$ with degree `11` and sectional genus `12`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d11_pi12()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-11613*x^4*y-F(124)/113*x^3*y^2-12025*x^2*y^3-F(101)/11*x*y^4-F(95)/112*y^5+F(11)/67*x^4*z+67*x^3*y*z-5658*x^2*y^2*z-F(19)/24*x*y^3*z-F(16)/45*y^4*z+8299*x^3*z^2+F(117)/92*x^2*y*z^2+1309*x*y^2*z^2+1118*y^3*z^2-9168*x^2*z^3-F(19)/123*x*y*z^3+F(66)/53*y^2*z^3-F(90)/113*x*z^4-F(23)/36*y*z^4+14004*z^5-F(94)/99*x^4*u-F(32)/111*x^3*y*u+F(73)/23*x^2*y^2*u-F(53)/23*x*y^3*u-1178*y^4*u-F(31)/104*x^3*z*u-13315*x^2*y*z*u-8115*x*y^2*z*u+F(95)/34*y^3*z*u-F(48)/103*x^2*z^2*u+F(85)/86*x*y*z^2*u-F(33)/62*y^2*z^2*u-F(42)/79*x*z^3*u+F(33)/92*y*z^3*u+F(96)/23*z^4*u+12474*x^3*u^2+8140*x^2*y*u^2-13*x*y^2*u^2-F(17)/43*y^3*u^2+F(91)/47*x^2*z*u^2-F(2)/93*x*y*z*u^2-F(101)/49*y^2*z*u^2+F(4)/55*x*z^2*u^2+10824*y*z^2*u^2+1945*z^3*u^2+F(119)/15*x^2*u^3-F(62)/67*x*y*u^3-F(96)/23*y^2*u^3+11277*x*z*u^3+13131*y*z*u^3+F(14)/99*z^2*u^3+F(76)/105*x*u^4-3730*y*u^4-F(89)/121*z*u^4-12847*x^4*v+13578*x^3*y*v-15001*x^2*y^2*v-F(69)/34*x*y^3*v+F(107)/44*y^4*v-15074*x^3*z*v-14054*x^2*y*z*v-6101*x*y^2*z*v-6626*y^3*z*v+13743*x^2*z^2*v-10580*x*y*z^2*v+F(12)/5*y^2*z^2*v+3772*x*z^3*v+F(101)/11*y*z^3*v-11558*z^4*v-F(68)/107*x^3*u*v+1018*x^2*y*u*v-F(81)/79*x*y^2*u*v-623*y^3*u*v-F(37)/65*x^2*z*u*v-F(85)/69*x*y*z*u*v-F(112)/71*y^2*z*u*v-F(85)/79*x*z^2*u*v+F(11)/94*y*z^2*u*v-F(4)/113*z^3*u*v-F(100)/21*x^2*u^2*v+F(79)/64*x*y*u^2*v+F(31)/54*y^2*u^2*v-7890*x*z*u^2*v-F(118)/61*y*z*u^2*v-F(36)/43*z^2*u^2*v-15889*x*u^3*v-F(55)/102*y*u^3*v+F(115)/104*z*u^3*v+F(52)/83*x^3*v^2+F(47)/56*x^2*y*v^2-F(106)/37*x*y^2*v^2+3589*y^3*v^2-F(45)/7*x^2*z*v^2-F(75)/103*x*y*z*v^2-9583*y^2*z*v^2-F(106)/77*x*z^2*v^2+7830*y*z^2*v^2-F(3)/121*z^3*v^2-13738*x^2*u*v^2-F(55)/26*x*y*u*v^2+671*y^2*u*v^2+8447*x*z*u*v^2+F(103)/35*y*z*u*v^2+3753*z^2*u*v^2+9958*x*u^2*v^2+F(125)/18*y*u^2*v^2-F(71)/82*z*u^2*v^2-F(39)/5*x^2*v^3-F(111)/16*x*y*v^3-8312*y^2*v^3+F(86)/9*x*z*v^3+F(105)/97*y*z*v^3+F(107)/46*z^2*v^3+328*x*u*v^3-F(117)/116*y*u*v^3-F(121)/13*z*u*v^3+1872*x*v^4+F(99)/56*y*v^4-8768*z*v^4,10372*x^4*y+2346*x^3*y^2+5436*x^2*y^3-14668*x*y^4-F(57)/2*x^4*z+F(45)/88*x^3*y*z+F(32)/81*x^2*y^2*z-F(22)/81*x*y^3*z+1533*y^4*z+5802*x^3*z^2-13233*x^2*y*z^2+F(49)/45*x*y^2*z^2-F(26)/51*y^3*z^2-F(35)/9*x^2*z^3-11653*x*y*z^3-3536*y^2*z^3+F(47)/34*x*z^4-F(45)/64*y*z^4-F(109)/26*z^5+6321*x^4*u-F(67)/36*x^3*y*u+F(23)/7*x^2*y^2*u-8122*x*y^3*u+F(95)/112*y^4*u-F(67)/54*x^3*z*u-F(98)/47*x^2*y*z*u-13129*x*y^2*z*u-F(29)/100*y^3*z*u+F(86)/109*x^2*z^2*u+9582*x*y*z^2*u-F(19)/81*y^2*z^2*u+708*x*z^3*u+3055*y*z^3*u+F(16)/53*z^4*u-F(55)/68*x^3*u^2-F(98)/61*x^2*y*u^2-1265*x*y^2*u^2+1178*y^3*u^2-F(7)/103*x^2*z*u^2+F(9)/104*x*y*z*u^2+7516*y^2*z*u^2+5261*x*z^2*u^2+1786*y*z^2*u^2+6268*z^3*u^2+F(17)/77*x^2*u^3+13652*x*y*u^3+F(17)/43*y^2*u^3-11765*x*z*u^3-1857*y*z*u^3-F(113)/86*z^2*u^3-F(97)/6*x*u^4+F(96)/23*y*u^4+F(103)/108*z*u^4+3730*u^5-F(31)/68*x^4*v-F(103)/109*x^3*y*v-F(91)/86*x^2*y^2*v-2211*x*y^3*v-F(9)/28*x^3*z*v+2009*x^2*y*z*v-F(13)/110*x*y^2*z*v+F(69)/17*y^3*z*v+10370*x^2*z^2*v-F(14)/75*x*y*z^2*v-9465*y^2*z^2*v+F(21)/92*x*z^3*v-F(35)/29*y*z^3*v+F(2)/9*z^4*v+F(92)/81*x^3*u*v-F(7)/31*x^2*y*u*v+14914*x*y^2*u*v-F(107)/44*y^3*u*v+F(63)/65*x^2*z*u*v+F(74)/109*x*y*z*u*v+13182*y^2*z*u*v-3534*x*z^2*u*v-F(58)/25*y*z^2*u*v+F(13)/114*z^3*u*v-F(25)/76*x^2*u^2*v+F(121)/20*x*y*u^2*v+623*y^2*u^2*v+F(77)/76*x*z*u^2*v-F(17)/54*y*z*u^2*v-F(41)/78*z^2*u^2*v-1251*x*u^3*v-F(31)/54*y*u^3*v+F(22)/37*z*u^3*v+F(55)/102*u^4*v-F(118)/79*x^3*v^2-F(117)/70*x^2*y*v^2+F(35)/31*x*y^2*v^2-F(59)/16*x^2*z*v^2-5480*x*y*z*v^2+965*y^2*z*v^2-741*x*z^2*v^2+F(67)/95*y*z^2*v^2+8745*z^3*v^2-F(77)/115*x^2*u*v^2-11016*x*y*u*v^2-3589*y^2*u*v^2-F(16)/107*x*z*u*v^2+12929*y*z*u*v^2-14059*z^2*u*v^2-11110*x*u^2*v^2-671*y*u^2*v^2-F(77)/45*z*u^2*v^2-F(125)/18*u^3*v^2-F(41)/5*x^2*v^3+11204*x*y*v^3-14748*x*z*v^3+F(95)/77*y*z*v^3-2748*z^2*v^3-F(9)/74*x*u*v^3+8312*y*u*v^3-F(106)/55*z*u*v^3+F(117)/116*u^2*v^3-6624*x*v^4+6349*z*v^4-F(99)/56*u*v^4,-10372*x^3*y^2-2346*x^2*y^3-5436*x*y^4+14668*y^5-F(57)/40*x^3*y*z+4547*x^2*y^2*z+362*x*y^3*z+F(37)/58*y^4*z-F(42)/79*x^3*z^2+F(87)/20*x^2*y*z^2-1607*x*y^2*z^2+2682*y^3*z^2+F(102)/61*x^2*z^3-14813*x*y*z^3+11175*y^2*z^3+2090*x*z^4-F(71)/43*y*z^4+9265*z^5+5292*x^3*y*u-F(4)/97*x^2*y^2*u-F(83)/65*x*y^3*u+F(49)/93*y^4*u+F(94)/35*x^3*z*u-F(119)/53*x^2*y*z*u-9116*x*y^2*z*u+14107*y^3*z*u-F(109)/117*x^2*z^2*u+F(76)/17*x*y*z^2*u+7207*y^2*z^2*u-F(50)/29*x*z^3*u+66*y*z^3*u+2727*z^4*u+F(94)/99*x^3*u^2-8234*x^2*y*u^2-F(113)/43*x*y^2*u^2-9860*y^3*u^2-2038*x^2*z*u^2-4256*x*y*z*u^2-F(107)/12*y^2*z*u^2-F(62)/41*x*z^2*u^2-4747*y*z^2*u^2-F(42)/29*z^3*u^2-12474*x^2*u^3+F(103)/35*x*y*u^3-13639*y^2*u^3+F(98)/57*x*z*u^3-480*y*z*u^3-748*z^2*u^3-F(119)/15*x*u^4+14421*y*u^4-12711*z*u^4-F(76)/105*u^5+F(31)/68*x^3*y*v+F(103)/109*x^2*y^2*v+F(91)/86*x*y^3*v+2211*y^4*v+F(7)/16*x^3*z*v+F(37)/26*x^2*y*z*v-F(91)/118*x*y^2*z*v-13741*y^3*z*v-10278*x^2*z^2*v+15645*x*y*z^2*v+7174*y^2*z^2*v-F(40)/103*x*z^3*v-F(112)/103*y*z^3*v-F(63)/67*z^4*v+12847*x^3*u*v-F(65)/24*x^2*y*u*v-F(31)/95*x*y^2*u*v-F(113)/19*y^3*u*v-4623*x^2*z*u*v+F(44)/35*x*y*z*u*v+2403*y^2*z*u*v-F(31)/77*x*z^2*u*v-4746*y*z^2*u*v+7316*z^3*u*v+F(68)/107*x^2*u^2*v+1087*x*y*u^2*v-F(22)/89*y^2*u^2*v-12833*x*z*u^2*v+F(19)/10*y*z*u^2*v+11586*z^2*u^2*v+F(100)/21*x*u^3*v-10247*y*u^3*v+F(17)/79*z*u^3*v+15889*u^4*v+F(118)/79*x^2*y*v^2+F(117)/70*x*y^2*v^2-F(35)/31*y^3*v^2-1862*x^2*z*v^2+F(3)/23*x*y*z*v^2+1202*y^2*z*v^2+F(74)/49*x*z^2*v^2+F(15)/86*y*z^2*v^2+8338*z^3*v^2-F(52)/83*x^2*u*v^2+7168*x*y*u*v^2+F(120)/19*y^2*u*v^2-F(100)/103*x*z*u*v^2+5042*y*z*u*v^2+F(26)/19*z^2*u*v^2+13738*x*u^2*v^2+4960*y*u^2*v^2+12896*z*u^2*v^2-9958*u^3*v^2+F(41)/5*x*y*v^3-11204*y^2*v^3+F(31)/54*x*z*v^3+3102*y*z*v^3+F(21)/17*z^2*v^3+F(39)/5*x*u*v^3-F(37)/56*y*u*v^3+F(52)/111*z*u*v^3-328*u^2*v^3+6624*y*v^4+F(1)/108*z*v^4-1872*u*v^4,10427*x^4*y+F(5)/53*x^3*y^2-F(6)/109*x^2*y^3+3275*x*y^4-1533*y^5+F(42)/79*x^4*z-10605*x^3*y*z-F(115)/97*x^2*y^2*z-F(67)/66*x*y^3*z+F(26)/51*y^4*z-F(102)/61*x^3*z^2-F(37)/89*x^2*y*z^2+F(35)/67*x*y^2*z^2+3536*y^3*z^2-2090*x^2*z^3-F(20)/7*x*y*z^3+F(45)/64*y^2*z^3-9265*x*z^4+F(109)/26*y*z^4-2895*x^4*u-F(14)/81*x^3*y*u+F(95)/99*x^2*y^2*u-7642*x*y^3*u+F(93)/89*y^4*u+F(101)/42*x^3*z*u-13199*x^2*y*z*u+13893*x*y^2*z*u+F(54)/95*y^3*z*u+F(64)/45*x^2*z^2*u+F(29)/126*x*y*z^2*u+4187*y^2*z^2*u+11146*x*z^3*u+15409*y*z^3*u-14004*z^4*u+F(81)/10*x^3*u^2+12291*x^2*y*u^2+F(41)/54*x*y^2*u^2+F(17)/2*y^3*u^2-F(109)/10*x^2*z*u^2-143*x*y*z*u^2+15242*y^2*z*u^2+F(109)/48*x*z^2*u^2-F(37)/4*y*z^2*u^2-F(96)/23*z^3*u^2-F(116)/35*x^2*u^3+F(22)/47*x*y*u^3-13810*y^2*u^3+F(3)/35*x*z*u^3+F(52)/41*y*z*u^3-1945*z^2*u^3+F(105)/67*x*u^4+13231*y*u^4-F(14)/99*z*u^4+F(89)/121*u^5-F(7)/16*x^4*v+F(59)/106*x^3*y*v-3906*x^2*y^2*v-F(76)/7*x*y^3*v-F(69)/17*y^4*v+10278*x^3*z*v-F(31)/91*x^2*y*z*v+F(115)/18*x*y^2*z*v+9465*y^3*z*v+F(40)/103*x^2*z^2*v-F(25)/77*x*y*z^2*v+F(35)/29*y^2*z^2*v+F(63)/67*x*z^3*v-F(2)/9*y*z^3*v-12294*x^3*u*v-15619*x^2*y*u*v-7749*x*y^2*u*v-F(57)/122*y^3*u*v-14989*x^2*z*u*v-13131*x*y*z*u*v-F(2)/25*y^2*z*u*v-11088*x*z^2*u*v+F(13)/48*y*z^2*u*v+11558*z^3*u*v-455*x^2*u^2*v-14505*x*y*u^2*v+F(43)/58*y^2*u^2*v-F(10)/51*x*z*u^2*v+10507*y*z*u^2*v+F(4)/113*z^2*u^2*v-F(33)/35*x*u^3*v+F(94)/49*y*u^3*v+F(36)/43*z*u^3*v-F(115)/104*u^4*v+1862*x^3*v^2-12080*x^2*y*v^2+4278*x*y^2*v^2-965*y^3*v^2-F(74)/49*x^2*z*v^2-F(3)/119*x*y*z*v^2-F(67)/95*y^2*z*v^2-8338*x*z^2*v^2-8745*y*z^2*v^2-6870*x^2*u*v^2-13520*x*y*u*v^2-3346*y^2*u*v^2+F(88)/65*x*z*u*v^2+F(75)/113*y*z*u*v^2+F(3)/121*z^2*u*v^2-F(47)/3*x*u^2*v^2+3858*y*u^2*v^2-3753*z*u^2*v^2+F(71)/82*u^3*v^2-F(31)/54*x^2*v^3+11646*x*y*v^3-F(95)/77*y^2*v^3-F(21)/17*x*z*v^3+2748*y*z*v^3-F(81)/56*x*u*v^3+F(111)/103*y*u*v^3-F(107)/46*z*u*v^3+F(121)/13*u^2*v^3-F(1)/108*x*v^4-6349*y*v^4+8768*u*v^4,-F(79)/27*x^4*y+F(113)/68*x^3*y^2+F(38)/5*x^2*y^3-F(27)/26*x*y^4+F(121)/119*x^4*z-8834*x^3*y*z-F(114)/37*x^2*y^2*z-1418*x*y^3*z+F(86)/103*x^3*z^2-F(31)/67*x^2*y*z^2-F(37)/119*x*y^2*z^2+8071*x^2*z^3-F(87)/38*x*y*z^3-F(9)/77*x*z^4-13003*x^4*u-9610*x^3*y*u+F(107)/62*x^2*y^2*u+13159*x*y^3*u+F(67)/64*y^4*u-4678*x^3*z*u-8059*x^2*y*z*u-F(23)/41*x*y^2*z*u-1005*y^3*z*u-F(77)/75*x^2*z^2*u+11207*x*y*z^2*u+F(8)/121*y^2*z^2*u+F(119)/118*x*z^3*u-F(63)/17*y*z^3*u+F(29)/122*z^4*u-F(11)/28*x^3*u^2-F(5)/87*x^2*y*u^2+F(42)/121*x*y^2*u^2-F(7)/41*y^3*u^2+F(25)/123*x^2*z*u^2-F(23)/14*x*y*z*u^2-F(28)/29*y^2*z*u^2+5634*x*z^2*u^2+4865*y*z^2*u^2+2165*z^3*u^2-F(109)/43*x^2*u^3+12561*x*y*u^3+150*y^2*u^3+F(71)/28*x*z*u^3-F(91)/37*y*z*u^3+F(116)/119*z^2*u^3+F(15)/4*x*u^4-5718*y*u^4-13886*z*u^4+11510*u^5+F(53)/68*x^4*v-F(65)/126*x^3*y*v-F(49)/3*x^2*y^2*v+F(21)/121*x*y^3*v-1533*y^4*v+F(106)/89*x^3*z*v+F(13)/115*x^2*y*z*v+9176*x*y^2*z*v+F(26)/51*y^3*z*v-F(9)/14*x^2*z^2*v+8874*x*y*z^2*v+3536*y^2*z^2*v+F(21)/125*x*z^3*v+F(45)/64*y*z^3*v+F(109)/26*z^4*v+F(7)/71*x^3*u*v-F(77)/38*x^2*y*u*v+13978*x*y^2*u*v+F(91)/3*y^3*u*v+15822*x^2*z*u*v+14236*x*y*z*u*v-F(105)/118*y^2*z*u*v-2194*x*z^2*u*v+F(77)/65*y*z^2*u*v+F(59)/79*z^3*u*v-14175*x^2*u^2*v+10247*x*y*u^2*v+12266*y^2*u^2*v-9755*x*z*u^2*v+F(26)/69*y*z*u^2*v-F(86)/91*z^2*u^2*v-2738*x*u^3*v+1512*y*u^3*v-F(43)/81*z*u^3*v-F(27)/89*u^4*v+5126*x^3*v^2+F(89)/40*x^2*y*v^2-F(97)/82*x*y^2*v^2-F(69)/17*y^3*v^2+11741*x^2*z*v^2-F(64)/89*x*y*z*v^2+9465*y^2*z*v^2+10811*x*z^2*v^2+F(35)/29*y*z^2*v^2-F(2)/9*z^3*v^2+13811*x^2*u*v^2-5416*x*y*u*v^2+14011*y^2*u*v^2+F(51)/62*x*z*u*v^2+11510*y*z*u*v^2-F(85)/92*z^2*u*v^2-F(11)/42*x*u^2*v^2-F(42)/101*y*u^2*v^2-F(59)/110*z*u^2*v^2-4419*u^3*v^2-F(51)/77*x^2*v^3-F(109)/10*x*y*v^3-965*y^2*v^3-5063*x*z*v^3-F(67)/95*y*z*v^3-8745*z^2*v^3-6446*x*u*v^3+13889*y*u*v^3-F(55)/19*z*u*v^3-F(33)/16*u^2*v^3+F(40)/103*x*v^4-F(95)/77*y*v^4+2748*z*v^4-F(121)/46*u*v^4-6349*v^5,F(79)/27*x^3*y^2-F(113)/68*x^2*y^3-F(38)/5*x*y^4+F(27)/26*y^5-F(121)/119*x^3*y*z+8834*x^2*y^2*z+F(114)/37*x*y^3*z+1418*y^4*z-F(86)/103*x^2*y*z^2+F(31)/67*x*y^2*z^2+F(37)/119*y^3*z^2-8071*x*y*z^3+F(87)/38*y^2*z^3+F(9)/77*y*z^4-F(46)/59*x^3*y*u-F(31)/98*x^2*y^2*u-F(34)/97*x*y^3*u+7896*y^4*u-F(46)/31*x^3*z*u+7086*x^2*y*z*u+12024*x*y^2*z*u-F(91)/12*y^3*z*u+12666*x^2*z^2*u-9860*x*y*z^2*u-15260*y^2*z^2*u+F(25)/113*x*z^3*u+14467*y*z^3*u+F(109)/88*z^4*u-2365*x^3*u^2-F(119)/18*x^2*y*u^2+F(105)/4*x*y^2*u^2-F(47)/32*y^3*u^2-F(29)/88*x^2*z*u^2+F(11)/49*x*y*z*u^2-4353*y^2*z*u^2-F(104)/31*x*z^2*u^2-F(64)/79*y*z^2*u^2+F(125)/43*z^3*u^2+10847*x^2*u^3+F(103)/70*x*y*u^3+F(95)/77*y^2*u^3+F(46)/25*x*z*u^3-7626*y*z*u^3-F(85)/27*z^2*u^3-9494*x*u^4+F(64)/15*y*u^4+F(63)/106*z*u^4-F(47)/43*u^5-3217*x^3*y*v+F(14)/125*x^2*y^2*v+7158*x*y^3*v-691*y^4*v+F(42)/79*x^3*z*v+F(87)/112*x^2*y*z*v+F(50)/41*x*y^2*z*v-F(59)/28*y^3*z*v-F(102)/61*x^2*z^2*v-F(15)/47*x*y*z^2*v-8396*y^2*z^2*v-2090*x*z^3*v-F(7)/99*y*z^3*v-9265*z^4*v+F(10)/43*x^3*u*v+F(111)/40*x^2*y*u*v+F(59)/47*x*y^2*u*v-7279*y^3*u*v-1213*x^2*z*u*v+12100*x*y*z*u*v-9957*y^2*z*u*v+F(84)/31*x*z^2*u*v+1493*y*z^2*u*v-F(38)/107*z^3*u*v+F(119)/104*x^2*u^2*v-F(5)/67*x*y*u^2*v+F(97)/36*y^2*u^2*v-F(1)/119*x*z*u^2*v-F(45)/62*y*z*u^2*v+F(7)/62*z^2*u^2*v-12488*x*u^3*v-F(63)/107*y*u^3*v+5939*z*u^3*v+F(47)/30*u^4*v-F(7)/16*x^3*v^2+1816*x^2*y*v^2-F(117)/34*x*y^2*v^2+F(69)/62*y^3*v^2+10278*x^2*z*v^2-F(95)/111*x*y*z*v^2-F(23)/67*y^2*z*v^2+F(40)/103*x*z^2*v^2-14966*y*z^2*v^2+F(63)/67*z^3*v^2-F(55)/54*x^2*u*v^2+3935*x*y*u*v^2-8250*y^2*u*v^2-F(29)/122*x*z*u*v^2-F(73)/56*y*z*u*v^2+F(49)/58*z^2*u*v^2+F(55)/111*x*u^2*v^2-72*y*u^2*v^2-F(25)/11*z*u^2*v^2-F(101)/110*u^3*v^2+1862*x^2*v^3-F(77)/16*x*y*v^3+F(35)/47*y^2*v^3-F(74)/49*x*z*v^3+224*y*z*v^3-8338*z^2*v^3-13408*x*u*v^3+6305*y*u*v^3-F(25)/24*z*u*v^3-F(89)/63*u^2*v^3-F(31)/54*x*v^4-3884*y*v^4-F(21)/17*z*v^4-441*u*v^4-F(1)/108*v^5,F(79)/27*x^3*y*z-F(113)/68*x^2*y^2*z-F(38)/5*x*y^3*z+F(27)/26*y^4*z-F(121)/119*x^3*z^2+8834*x^2*y*z^2+F(114)/37*x*y^2*z^2+1418*y^3*z^2-F(86)/103*x^2*z^3+F(31)/67*x*y*z^3+F(37)/119*y^2*z^3-8071*x*z^4+F(87)/38*y*z^4+F(9)/77*z^5-F(5)/69*x^3*y*u-F(34)/75*x^2*y^2*u+F(57)/119*x*y^3*u+12127*y^4*u+13526*x^3*z*u+F(51)/8*x^2*y*z*u-F(116)/63*x*y^2*z*u+9558*y^3*z*u+5455*x^2*z^2*u+3974*x*y*z^2*u-F(37)/47*y^2*z^2*u+F(30)/19*x*z^3*u-4022*y*z^3*u+142*z^4*u-F(5)/92*x^3*u^2-7879*x^2*y*u^2+7880*x*y^2*u^2-F(101)/56*y^3*u^2-30*x^2*z*u^2+F(11)/82*x*y*z*u^2-F(83)/53*y^2*z*u^2+9167*x*z^2*u^2+F(108)/49*y*z^2*u^2-F(101)/72*z^3*u^2+F(76)/71*x^2*u^3+F(21)/116*x*y*u^3+F(40)/43*y^2*u^3-F(42)/53*x*z*u^3-F(3)/92*y*z*u^3-F(49)/116*z^2*u^3-F(17)/74*x*u^4+F(4)/121*y*u^4-F(29)/75*z*u^4-F(43)/39*u^5-10372*x^3*y*v-2346*x^2*y^2*v-5436*x*y^3*v+14668*y^4*v+F(61)/121*x^3*z*v-F(17)/69*x^2*y*z*v-F(124)/17*x*y^2*z*v+14202*y^3*z*v-52*x^2*z^2*v-F(23)/35*x*y*z^2*v+F(7)/55*y^2*z^2*v-F(109)/88*x*z^3*v-F(65)/23*y*z^3*v-4676*z^4*v-569*x^3*u*v+15924*x^2*y*u*v+F(124)/41*x*y^2*u*v-10114*y^3*u*v+F(83)/64*x^2*z*u*v+3037*x*y*z*u*v-10950*y^2*z*u*v-F(83)/103*x*z^2*u*v-13744*y*z^2*u*v-F(122)/13*z^3*u*v-F(71)/30*x^2*u^2*v-2892*x*y*u^2*v+6204*y^2*u^2*v+F(99)/115*x*z*u^2*v+F(81)/59*y*z*u^2*v-6280*z^2*u^2*v+F(117)/32*x*u^3*v+F(6)/41*y*u^3*v-14093*z*u^3*v+527*u^4*v+F(31)/68*x^3*v^2+F(103)/109*x^2*y*v^2+F(91)/86*x*y^2*v^2+2211*y^3*v^2+F(1)/109*x^2*z*v^2-2811*x*y*z*v^2-10128*y^2*z*v^2+F(29)/68*x*z^2*v^2-F(116)/73*y*z^2*v^2+5532*z^3*v^2-F(16)/67*x^2*u*v^2-F(103)/74*x*y*u*v^2+F(95)/6*y^2*u*v^2+F(49)/76*x*z*u*v^2-F(103)/108*y*z*u*v^2+3423*z^2*u*v^2-519*x*u^2*v^2+F(107)/13*y*u^2*v^2+14895*z*u^2*v^2-2685*u^3*v^2+F(118)/79*x^2*v^3+F(117)/70*x*y*v^3-F(35)/31*y^2*v^3+8080*x*z*v^3+F(88)/81*y*z*v^3+5804*z^2*v^3+6681*x*u*v^3-12449*y*u*v^3+F(10)/107*z*u*v^3-F(37)/19*u^2*v^3+F(41)/5*x*v^4-11204*y*v^4-F(71)/41*z*v^4-F(89)/65*u*v^4+6624*v^5,-F(5)/69*x^4*y-F(34)/75*x^3*y^2+F(57)/119*x^2*y^3+12127*x*y^4-F(88)/61*x^4*z+F(123)/107*x^3*y*z-F(45)/106*x^2*y^2*z-F(86)/69*x*y^3*z+F(67)/64*y^4*z+777*x^3*z^2-F(49)/47*x^2*y*z^2+1692*x*y^2*z^2-1005*y^3*z^2-12504*x^2*z^3+7185*x*y*z^3+F(8)/121*y^2*z^3+2583*x*z^4-F(63)/17*y*z^4+F(29)/122*z^5-F(5)/92*x^4*u-7879*x^3*y*u+7880*x^2*y^2*u-F(101)/56*x*y^3*u+F(18)/37*x^3*z*u-13260*x^2*y*z*u-F(83)/64*x*y^2*z*u-F(7)/41*y^3*z*u-3057*x^2*z^2*u+F(55)/98*x*y*z^2*u-F(28)/29*y^2*z^2*u-F(103)/75*x*z^3*u+4865*y*z^3*u+2165*z^4*u+F(76)/71*x^3*u^2+F(21)/116*x^2*y*u^2+F(40)/43*x*y^2*u^2+F(13)/110*x^2*z*u^2-7955*x*y*z*u^2+150*y^2*z*u^2-F(16)/67*x*z^2*u^2-F(91)/37*y*z^2*u^2+F(116)/119*z^3*u^2-F(17)/74*x^2*u^3+F(4)/121*x*y*u^3+110*x*z*u^3-5718*y*z*u^3-13886*z^2*u^3-F(43)/39*x*u^4+11510*z*u^4+F(72)/89*x^4*v-962*x^3*y*v+F(19)/109*x^2*y^2*v-F(113)/100*x*y^3*v+F(95)/112*y^4*v-10451*x^3*z*v-F(43)/111*x^2*y*z*v+F(27)/19*x*y^2*z*v+F(2)/103*y^3*z*v+10086*x^2*z^2*v+10074*x*y*z^2*v-F(103)/41*y^2*z^2*v+F(58)/23*x*z^3*v+F(93)/82*y*z^3*v+F(123)/110*z^4*v+F(83)/43*x^3*u*v-F(48)/91*x^2*y*u*v+4939*x*y^2*u*v+1178*y^3*u*v-F(41)/103*x^2*z*u*v-14970*x*y*z*u*v-12209*y^2*z*u*v-10774*x*z^2*u*v+F(69)/101*y*z^2*u*v+9431*z^3*u*v+3029*x^2*u^2*v+7410*x*y*u^2*v+F(17)/43*y^2*u^2*v+3395*x*z*u^2*v-F(94)/93*y*z*u^2*v-F(23)/122*z^2*u^2*v-F(32)/73*x*u^3*v+F(96)/23*y*u^3*v+3472*z*u^3*v+3730*u^4*v-6672*x^3*v^2+4447*x^2*y*v^2+F(7)/10*x*y^2*v^2-F(107)/44*y^3*v^2-1749*x^2*z*v^2+402*x*y*z*v^2+F(13)/20*y^2*z*v^2-F(37)/94*x*z^2*v^2-3848*y*z^2*v^2+3812*z^3*v^2-F(109)/61*x^2*u*v^2+F(101)/76*x*y*u*v^2+623*y^2*u*v^2-F(101)/22*x*z*u*v^2+7736*y*z*u*v^2-F(35)/47*z^2*u*v^2+F(88)/65*x*u^2*v^2-F(31)/54*y*u^2*v^2-F(5)/118*z*u^2*v^2+F(55)/102*u^3*v^2-F(65)/54*x^2*v^3-F(74)/15*x*y*v^3-3589*y^2*v^3-F(49)/61*x*z*v^3-5173*y*z*v^3+F(23)/54*z^2*v^3+F(68)/95*x*u*v^3-671*y*u*v^3-F(37)/104*z*u*v^3-F(125)/18*u^2*v^3-F(125)/66*x*v^4+8312*y*v^4+1399*z*v^4+F(117)/116*u*v^4-F(99)/56*v^5,F(5)/69*x^3*y^2+F(34)/75*x^2*y^3-F(57)/119*x*y^4-12127*y^5-14069*x^3*y*z-11024*x^2*y^2*z+15340*x*y^3*z-F(10)/77*y^4*z-F(46)/31*x^3*z^2-F(117)/98*x^2*y*z^2+8050*x*y^2*z^2+F(109)/9*y^3*z^2+12666*x^2*z^3+12027*x*y*z^3+F(77)/37*y^2*z^3+F(25)/113*x*z^4+F(45)/67*y*z^4+F(109)/88*z^5+F(5)/92*x^3*y*u+7879*x^2*y^2*u-7880*x*y^3*u+F(101)/56*y^4*u-2365*x^3*z*u-8863*x^2*y*z*u-F(89)/19*x*y^2*z*u+14430*y^3*z*u-F(29)/88*x^2*z^2*u+F(42)/97*x*y*z^2*u+15884*y^2*z^2*u-F(104)/31*x*z^3*u-4257*y*z^3*u+F(125)/43*z^4*u-F(76)/71*x^2*y*u^2-F(21)/116*x*y^2*u^2-F(40)/43*y^3*u^2+10847*x^2*z*u^2+F(23)/113*x*y*z*u^2+F(54)/89*y^2*z*u^2+F(46)/25*x*z^2*u^2+6991*y*z^2*u^2-F(85)/27*z^3*u^2+F(17)/74*x*y*u^3-F(4)/121*y^2*u^3-9494*x*z*u^3-5967*y*z*u^3+F(63)/106*z^2*u^3+F(43)/39*y*u^4-F(47)/43*z*u^4+5861*x^3*y*v+F(77)/38*x^2*y^2*v-F(53)/110*x*y^3*v-5021*y^4*v+F(123)/64*x^3*z*v+8942*x^2*y*z*v-F(97)/101*x*y^2*z*v+F(47)/9*y^3*z*v+F(16)/59*x^2*z^2*v+F(37)/49*x*y*z^2*v-F(93)/32*y^2*z^2*v-F(112)/103*x*z^3*v-F(85)/58*y*z^3*v-F(64)/65*z^4*v+F(94)/99*x^3*u*v-9298*x^2*y*u*v+14793*x*y^2*u*v+15927*y^3*u*v-9727*x^2*z*u*v-12154*x*y*z*u*v+F(43)/34*y^2*z*u*v-F(73)/117*x*z^2*u*v+F(107)/59*y*z^2*u*v-F(89)/40*z^3*u*v-12474*x^2*u^2*v-11169*x*y*u^2*v-7397*y^2*u^2*v+F(109)/76*x*z*u^2*v-11801*y*z*u^2*v+F(121)/37*z^2*u^2*v-F(119)/15*x*u^3*v-F(107)/99*y*u^3*v+F(56)/111*z*u^3*v-F(76)/105*u^4*v+12847*x^3*v^2+F(101)/88*x^2*y*v^2+F(26)/97*x*y^2*v^2+F(113)/85*y^3*v^2+F(45)/62*x^2*z*v^2-14814*x*y*z*v^2-218*y^2*z*v^2-3212*x*z^2*v^2-F(124)/57*y*z^2*v^2-F(78)/7*z^3*v^2+F(68)/107*x^2*u*v^2+1606*x*y*u*v^2+14093*y^2*u*v^2-F(12)/61*x*z*u*v^2+F(70)/87*y*z*u*v^2+F(41)/117*z^2*u*v^2+F(100)/21*x*u^2*v^2-F(27)/55*y*u^2*v^2+F(35)/34*z*u^2*v^2+15889*u^3*v^2-F(52)/83*x^2*v^3+487*x*y*v^3-11117*y^2*v^3+11749*x*z*v^3+F(112)/13*y*z*v^3+F(111)/125*z^2*v^3+13738*x*u*v^3+F(77)/18*y*u*v^3-F(118)/35*z*u*v^3-9958*u^2*v^3+F(39)/5*x*v^4-9746*y*v^4+F(103)/5*z*v^4-328*u*v^4-1872*v^5,F(29)/111*x^4*y-F(94)/63*x^3*y^2+F(37)/93*x^2*y^3+10936*x*y^4-F(67)/64*y^5+F(46)/31*x^4*z-F(7)/93*x^3*y*z+F(100)/121*x^2*y^2*z+5405*x*y^3*z+1005*y^4*z-12666*x^3*z^2-F(32)/67*x^2*y*z^2+4053*x*y^2*z^2-F(8)/121*y^3*z^2-F(25)/113*x^2*z^3+F(107)/70*x*y*z^3+F(63)/17*y^2*z^3-F(109)/88*x*z^4-F(29)/122*y*z^4+2365*x^4*u+12321*x^3*y*u-F(37)/33*x^2*y^2*u+15063*x*y^3*u+F(7)/41*y^4*u+F(29)/88*x^3*z*u+5695*x^2*y*z*u+15780*x*y^2*z*u+F(28)/29*y^3*z*u+F(104)/31*x^2*z^2*u+846*x*y*z^2*u-4865*y^2*z^2*u-F(125)/43*x*z^3*u-2165*y*z^3*u-10847*x^3*u^2+203*x^2*y*u^2-F(31)/123*x*y^2*u^2-150*y^3*u^2-F(46)/25*x^2*z*u^2+F(98)/73*x*y*z*u^2+F(91)/37*y^2*z*u^2+F(85)/27*x*z^2*u^2-F(116)/119*y*z^2*u^2+9494*x^2*u^3-F(36)/71*x*y*u^3+5718*y^2*u^3-F(63)/106*x*z*u^3+13886*y*z*u^3+F(47)/43*x*u^4-11510*y*u^4+F(36)/65*x^4*v+F(107)/111*x^3*y*v+F(36)/115*x^2*y^2*v-F(6)/29*x*y^3*v-13928*y^4*v-F(33)/10*x^3*z*v+F(72)/7*x^2*y*z*v+9614*x*y^2*z*v+9028*y^3*z*v+F(7)/48*x^2*z^2*v+11111*x*y*z^2*v+F(13)/109*y^2*z^2*v-15164*x*z^3*v-14558*y*z^3*v-14004*z^4*v-F(47)/64*x^3*u*v+F(86)/103*x^2*y*u*v+F(1)/30*x*y^2*u*v-F(93)/77*y^3*u*v+F(51)/20*x^2*z*u*v-F(33)/38*x*y*z*u*v+F(10)/103*y^2*z*u*v+F(125)/38*x*z^2*u*v+F(92)/63*y*z^2*u*v-F(96)/23*z^3*u*v-F(5)/44*x^2*u^2*v+9094*x*y*u^2*v-15322*y^2*u^2*v-8681*x*z*u^2*v+11465*y*z*u^2*v-1945*z^2*u^2*v+8897*x*u^3*v-F(6)/79*y*u^3*v-F(14)/99*z*u^3*v+F(89)/121*u^4*v-8146*x^3*v^2-1374*x^2*y*v^2+5917*x*y^2*v^2-F(19)/14*y^3*v^2-10531*x^2*z*v^2+6827*x*y*z*v^2-15349*y^2*z*v^2-F(1)/96*x*z^2*v^2-F(26)/51*y*z^2*v^2+11558*z^3*v^2-F(31)/107*x^2*u*v^2-1484*x*y*u*v^2-F(54)/43*y^2*u*v^2-F(65)/106*x*z*u*v^2+F(59)/94*y*z*u*v^2+F(4)/113*z^2*u*v^2+F(122)/107*x*u^2*v^2-6678*y*u^2*v^2+F(36)/43*z*u^2*v^2-F(115)/104*u^3*v^2+6538*x^2*v^3-F(52)/55*x*y*v^3-F(118)/13*y^2*v^3-2602*x*z*v^3-7238*y*z*v^3+F(3)/121*z^2*v^3+11665*x*u*v^3-F(24)/101*y*u*v^3-3753*z*u*v^3+F(71)/82*u^2*v^3+F(52)/69*x*v^4+F(103)/117*y*v^4-F(107)/46*z*v^4+F(121)/13*u*v^4+8768*v^5,F(9)/40*x^4*y-F(121)/118*x^3*y^2-F(61)/25*x^2*y^3+F(86)/3*x*y^4-F(26)/29*y^5+F(27)/32*x^4*z-F(99)/107*x^3*y*z-F(11)/75*x^2*y^2*z+6823*x*y^3*z+11282*y^4*z+F(58)/3*x^3*z^2-11692*x^2*y*z^2-15225*x*y^2*z^2+F(16)/87*y^3*z^2+2496*x^2*z^3+F(93)/53*x*y*z^3+F(120)/19*y^2*z^3-F(113)/64*x*z^4-F(9)/68*y*z^4+F(95)/88*z^5-F(43)/26*x^4*u-F(74)/5*x^3*y*u+4184*x^2*y^2*u-F(57)/28*x*y^3*u+F(109)/92*y^4*u+F(77)/61*x^3*z*u+F(82)/83*x^2*y*z*u-4843*x*y^2*z*u-F(19)/80*y^3*z*u-12376*x^2*z^2*u-F(69)/103*x*y*z^2*u-F(34)/29*y^2*z^2*u+8409*x*z^3*u-F(66)/13*y*z^3*u-F(43)/71*z^4*u-F(103)/31*x^3*u^2+4841*x^2*y*u^2+225*x*y^2*u^2+3941*y^3*u^2+F(67)/41*x^2*z*u^2+F(121)/114*x*y*z*u^2+F(17)/6*y^2*z*u^2+F(19)/80*x*z^2*u^2+F(123)/71*y*z^2*u^2+11025*z^3*u^2+14006*x^2*u^3-F(11)/48*x*y*u^3-F(60)/23*y^2*u^3-F(49)/111*x*z*u^3-15779*y*z*u^3+F(101)/56*z^2*u^3-12916*x*u^4-F(29)/76*y*u^4-12651*z*u^4+F(15)/13*x^4*v+15822*x^3*y*v+10324*x^2*y^2*v+F(7)/50*x*y^3*v-F(11)/27*y^4*v+F(93)/100*x^3*z*v-F(67)/77*x^2*y*z*v+15042*x*y^2*z*v+F(55)/8*y^3*z*v-F(51)/20*x^2*z^2*v-F(53)/114*x*y*z^2*v+F(57)/92*y^2*z^2*v+F(31)/6*x*z^3*v+F(79)/120*y*z^3*v+F(102)/97*z^4*v+6524*x^3*u*v+11390*x^2*y*u*v-12877*x*y^2*u*v+F(59)/14*y^3*u*v+11685*x^2*z*u*v+131*x*y*z*u*v-F(65)/66*y^2*z*u*v+12721*x*z^2*u*v-F(27)/13*y*z^2*u*v+4058*z^3*u*v+11575*x^2*u^2*v-7564*x*y*u^2*v+F(93)/89*y^2*u^2*v-F(68)/63*x*z*u^2*v-F(47)/41*y*z*u^2*v-F(8)/15*z^2*u^2*v-F(83)/65*x*u^3*v-F(104)/7*y*u^3*v-5*z*u^3*v+F(84)/37*x^3*v^2+15350*x^2*y*v^2-F(59)/126*x*y^2*v^2+F(13)/98*y^3*v^2+F(4)/87*x^2*z*v^2-F(65)/116*x*y*z*v^2-F(115)/63*y^2*z*v^2+F(53)/85*x*z^2*v^2-F(59)/68*y*z^2*v^2-F(97)/52*z^3*v^2+13578*x^2*u*v^2-2737*x*y*u*v^2+4459*y^2*u*v^2-F(34)/93*x*z*u*v^2+10771*y*z*u*v^2-F(43)/71*z^2*u*v^2+4632*x*u^2*v^2-F(79)/107*y*u^2*v^2-F(73)/19*z*u^2*v^2-11588*x^2*v^3-F(96)/47*x*y*v^3+12945*y^2*v^3+8588*x*z*v^3+F(57)/116*y*z*v^3-15664*z^2*v^3-7238*x*u*v^3+F(28)/45*y*u*v^3+10720*z*u*v^3+3906*x*v^4-4309*y*v^4+12027*z*v^4,-F(122)/49*x^4*y+126*x^3*y^2-4670*x^2*y^3-F(65)/23*x*y^4+645*x^4*z+F(27)/67*x^3*y*z+F(111)/61*x^2*y^2*z-F(67)/106*x*y^3*z-14746*y^4*z-F(59)/52*x^3*z^2+11190*x^2*y*z^2+F(111)/83*x*y^2*z^2+4734*y^3*z^2+F(23)/31*x^2*z^3+F(108)/71*x*y*z^3+11765*y^2*z^3-F(3)/47*x*z^4+F(109)/11*y*z^4-F(15)/29*z^5-2346*x^4*u+F(46)/113*x^3*y*u-F(89)/104*x^2*y^2*u-F(66)/115*x*y^3*u+F(26)/29*y^4*u+F(53)/36*x^3*z*u+F(101)/22*x^2*y*z*u+F(113)/59*x*y^2*z*u+6144*y^3*z*u-907*x^2*z^2*u-F(67)/94*x*y*z^2*u-1943*y^2*z^2*u-4150*x*z^3*u-F(106)/47*y*z^3*u+F(13)/58*z^4*u-F(111)/70*x^3*u^2-15142*x^2*y*u^2+F(32)/13*x*y^2*u^2-F(109)/92*y^3*u^2+F(78)/37*x^2*z*u^2+5365*x*y*z*u^2+F(23)/9*y^2*z*u^2+F(22)/117*x*z^2*u^2-F(25)/118*y*z^2*u^2+F(64)/29*z^3*u^2+F(106)/77*x*y*u^3-3941*y^2*u^3+F(126)/83*x*z*u^3-870*y*z*u^3+F(80)/69*z^2*u^3+F(93)/52*x*u^4+F(60)/23*y*u^4-11334*z*u^4+F(29)/76*u^5-F(41)/109*x^4*v-F(29)/89*x^3*y*v-F(121)/38*x^2*y^2*v-F(26)/47*x*y^3*v-7962*x^3*z*v-1138*x^2*y*z*v+6882*x*y^2*z*v+F(61)/107*y^3*z*v+F(11)/97*x^2*z^2*v+F(89)/21*x*y*z^2*v+3224*y^2*z^2*v+F(54)/125*x*z^3*v-F(53)/82*y*z^3*v-14358*z^4*v+F(8)/51*x^3*u*v-F(43)/37*x^2*y*u*v-13210*x*y^2*u*v+F(11)/27*y^3*u*v-F(25)/33*x^2*z*u*v+F(114)/11*x*y*z*u*v-F(48)/53*y^2*z*u*v-F(65)/126*x*z^2*u*v-8382*y*z^2*u*v+F(85)/7*z^3*u*v+F(96)/35*x^2*u^2*v-8030*x*y*u^2*v-F(59)/14*y^2*u^2*v-F(77)/15*x*z*u^2*v-F(105)/73*y*z*u^2*v+F(41)/110*z^2*u^2*v-F(31)/126*x*u^3*v-F(93)/89*y*u^3*v-F(71)/97*z*u^3*v+F(104)/7*u^4*v+7457*x^3*v^2+F(42)/19*x^2*y*v^2-F(9)/4*x*y^2*v^2+F(13)/118*x^2*z*v^2+F(13)/61*x*y*z*v^2+F(19)/78*y^2*z*v^2-F(104)/77*x*z^2*v^2+F(115)/114*y*z^2*v^2+F(75)/62*z^3*v^2-F(75)/91*x^2*u*v^2-F(53)/109*x*y*u*v^2-F(13)/98*y^2*u*v^2-15074*x*z*u*v^2-10090*y*z*u*v^2-14919*z^2*u*v^2+F(13)/118*x*u^2*v^2-4459*y*u^2*v^2+2646*z*u^2*v^2+F(79)/107*u^3*v^2-F(111)/38*x^2*v^3+F(47)/69*x*y*v^3+2148*x*z*v^3+F(17)/78*y*z*v^3-F(42)/61*z^2*v^3-F(23)/118*x*u*v^3-12945*y*u*v^3+F(1)/86*z*u*v^3-F(28)/45*u^2*v^3+9559*x*v^4+F(89)/96*z*v^4+4309*u*v^4,F(122)/49*x^3*y^2-126*x^2*y^3+4670*x*y^4+F(65)/23*y^5+F(41)/35*x^3*y*z+10579*x^2*y^2*z+F(55)/69*x*y^3*z+2706*y^4*z-11540*x^3*z^2-F(115)/72*x^2*y*z^2-5058*x*y^2*z^2+3156*y^3*z^2+F(3)/103*x^2*z^3-F(109)/64*x*y*z^3-F(118)/33*y^2*z^3-3218*x*z^4+F(39)/121*y*z^4+F(81)/56*z^5-F(121)/62*x^3*y*u-F(107)/52*x^2*y^2*u+F(78)/5*x*y^3*u-F(19)/103*y^4*u+F(90)/37*x^3*z*u-14695*x^2*y*z*u-F(18)/7*x*y^2*z*u-F(31)/29*y^3*z*u-F(89)/42*x^2*z^2*u-11357*x*y*z^2*u-6362*y^2*z^2*u+F(92)/11*x*z^3*u-F(93)/49*y*z^3*u+5800*z^4*u+F(43)/26*x^3*u^2-12323*x^2*y*u^2+10958*x*y^2*u^2+7470*y^3*u^2+10833*x^2*z*u^2-F(71)/126*x*y*z*u^2+F(81)/82*y^2*z*u^2-F(16)/47*x*z^2*u^2-11968*y*z^2*u^2-48*z^3*u^2+F(103)/31*x^2*u^3-4841*x*y*u^3-F(8)/123*y^2*u^3+13131*x*z*u^3+F(85)/11*y*z*u^3-F(108)/71*z^2*u^3-14006*x*u^4-9281*y*u^4+F(72)/53*z*u^4+12916*u^5+F(41)/109*x^3*y*v+F(29)/89*x^2*y^2*v+F(121)/38*x*y^3*v+F(26)/47*y^4*v+11277*x^3*z*v-12163*x^2*y*z*v+2331*x*y^2*z*v+6041*y^3*z*v+F(5)/48*x^2*z^2*v-F(72)/95*x*y*z^2*v+y^2*z^2*v+F(32)/23*x*z^3*v+F(65)/108*y*z^3*v+12707*z^4*v-F(15)/13*x^3*u*v+F(58)/13*x^2*y*u*v-812*x*y^2*u*v-F(46)/85*y^3*u*v+860*x^2*z*u*v-F(85)/22*x*y*z*u*v+F(11)/90*y^2*z*u*v-F(53)/66*x*z^2*u*v+6828*y*z^2*u*v+F(49)/40*z^3*u*v-6524*x^2*u^2*v+12372*x*y*u^2*v-11084*y^2*u^2*v+F(16)/109*x*z*u^2*v-9167*y*z*u^2*v+2213*z^2*u^2*v-11575*x*u^3*v+3248*y*u^3*v+3502*z*u^3*v+F(83)/65*u^4*v-7457*x^2*y*v^2-F(42)/19*x*y^2*v^2+F(9)/4*y^3*v^2-F(8)/53*x^2*z*v^2+F(113)/26*x*y*z*v^2-F(83)/73*y^2*z*v^2-F(32)/91*x*z^2*v^2-F(101)/53*y*z^2*v^2+F(92)/9*z^3*v^2-F(84)/37*x^2*u*v^2+119*x*y*u*v^2-F(77)/117*y^2*u*v^2-8286*x*z*u*v^2+1236*y*z*u*v^2+8768*z^2*u*v^2-13578*x*u^2*v^2+F(35)/117*y*u^2*v^2+3378*z*u^2*v^2-4632*u^3*v^2+F(111)/38*x*y*v^3-F(47)/69*y^2*v^3+15736*x*z*v^3-F(10)/81*y*z*v^3-F(23)/8*z^2*v^3+11588*x*u*v^3-3776*y*u*v^3+F(30)/41*z*u*v^3+7238*u^2*v^3-9559*y*v^4+F(1)/109*z*v^4-3906*u*v^4,-F(116)/119*x^4*y+F(103)/5*x^3*y^2+12508*x^2*y^3-F(72)/49*x*y^4+14746*y^5+11540*x^4*z+11931*x^3*y*z-F(47)/120*x^2*y^2*z+F(71)/46*x*y^3*z-4734*y^4*z-F(3)/103*x^3*z^2+F(47)/116*x^2*y*z^2-F(104)/69*x*y^2*z^2-11765*y^3*z^2+3218*x^2*z^3-F(4)/59*x*y*z^3-F(109)/11*y^2*z^3-F(81)/56*x*z^4+F(15)/29*y*z^4+14290*x^4*u-F(108)/73*x^3*y*u-10275*x^2*y^2*u-F(92)/31*x*y^3*u-F(1)/123*y^4*u+6838*x^3*z*u-8035*x^2*y*z*u+F(67)/107*x*y^2*z*u+F(97)/75*y^3*z*u+F(79)/101*x^2*z^2*u+F(47)/107*x*y*z^2*u-15910*y^2*z^2*u-F(95)/37*x*z^3*u-795*y*z^3*u-F(95)/88*z^4*u+9619*x^3*u^2-F(109)/37*x^2*y*u^2+F(116)/5*x*y^2*u^2+7951*y^3*u^2-F(27)/49*x^2*z*u^2+4647*x*y*z*u^2+F(117)/44*y^2*z*u^2+F(25)/88*x*z^2*u^2+2888*y*z^2*u^2+F(43)/71*z^3*u^2-F(19)/59*x^2*u^3-F(121)/112*x*y*u^3+6199*y^2*u^3-F(23)/83*x*z*u^3-6533*y*z*u^3-11025*z^2*u^3-F(109)/107*x*u^4+F(117)/59*y*u^4-F(101)/56*z*u^4+12651*u^5-11277*x^4*v+F(101)/62*x^3*y*v-1193*x^2*y^2*v-12923*x*y^3*v-F(61)/107*y^4*v-F(5)/48*x^3*z*v-F(83)/119*x^2*y*z*v-F(110)/21*x*y^2*z*v-3224*y^3*z*v-F(32)/23*x^2*z^2*v+7646*x*y*z^2*v+F(53)/82*y^2*z^2*v-12707*x*z^3*v+14358*y*z^3*v-F(75)/64*x^3*u*v+F(15)/91*x^2*y*u*v+10508*x*y^2*u*v+F(37)/57*y^3*u*v-F(25)/7*x^2*z*u*v+5948*x*y*z*u*v+F(119)/13*y^2*z*u*v-6138*x*z^2*u*v-3707*y*z^2*u*v-F(102)/97*z^3*u*v-F(49)/85*x^2*u^2*v-5888*x*y*u^2*v+1430*y^2*u^2*v-F(73)/15*x*z*u^2*v+F(33)/85*y*z*u^2*v-4058*z^2*u^2*v-13149*x*u^3*v+F(51)/40*y*u^3*v+F(8)/15*z*u^3*v+5*u^4*v+F(8)/53*x^3*v^2-13268*x^2*y*v^2+6129*x*y^2*v^2-F(19)/78*y^3*v^2+F(32)/91*x^2*z*v^2-F(65)/33*x*y*z*v^2-F(115)/114*y^2*z*v^2-F(92)/9*x*z^2*v^2-F(75)/62*y*z^2*v^2-2010*x^2*u*v^2-4639*x*y*u*v^2+F(9)/95*y^2*u*v^2-F(71)/118*x*z*u*v^2+F(107)/90*y*z*u*v^2+F(97)/52*z^2*u*v^2+F(17)/50*x*u^2*v^2-F(44)/31*y*u^2*v^2+F(43)/71*z*u^2*v^2+F(73)/19*u^3*v^2-15736*x^2*v^3-F(97)/69*x*y*v^3-F(17)/78*y^2*v^3+F(23)/8*x*z*v^3+F(42)/61*y*z*v^3-9369*x*u*v^3-F(104)/41*y*u*v^3+15664*z*u*v^3-10720*u^2*v^3-F(1)/109*x*v^4-F(89)/96*y*v^4-12027*u*v^4,-F(47)/88*x^4*y-9610*x^3*y^2+3005*x^2*y^3-10610*x*y^4-1737*x^4*z-F(17)/114*x^3*y*z-13934*x^2*y^2*z-F(64)/9*x*y^3*z-F(107)/13*x^3*z^2-F(10)/9*x^2*y*z^2-F(41)/71*x*y^2*z^2+F(118)/53*x^2*z^3-F(25)/111*x*y*z^3+F(93)/28*x*z^4-F(24)/59*x^4*u+F(9)/112*x^3*y*u+F(14)/95*x^2*y^2*u+F(61)/81*x*y^3*u+F(12)/59*y^4*u-F(11)/101*x^3*z*u+F(9)/110*x^2*y*z*u+F(118)/121*x*y^2*z*u+5541*y^3*z*u-F(48)/11*x^2*z^2*u+F(8)/79*x*y*z^2*u+4541*y^2*z^2*u+F(19)/97*x*z^3*u+423*y*z^3*u-F(16)/55*z^4*u+F(112)/57*x^3*u^2-1008*x^2*y*u^2-F(34)/65*x*y^2*u^2-F(19)/9*y^3*u^2-15364*x^2*z*u^2-F(79)/108*x*y*z*u^2+F(76)/45*y^2*z*u^2+F(6)/35*x*z^2*u^2+F(95)/111*y*z^2*u^2+2747*z^3*u^2+F(76)/85*x^2*u^3-9797*x*y*u^3-510*y^2*u^3-F(71)/62*x*z*u^3-3358*y*z*u^3+14026*z^2*u^3+10128*x*u^4+14238*y*u^4-8120*z*u^4-15341*u^5-F(99)/70*x^4*v+F(85)/27*x^3*y*v-F(79)/58*x^2*y^2*v+2788*x*y^3*v+14746*y^4*v-F(69)/13*x^3*z*v-13924*x^2*y*z*v-14049*x*y^2*z*v-4734*y^3*z*v-1061*x^2*z^2*v-4179*x*y*z^2*v-11765*y^2*z^2*v+F(42)/17*x*z^3*v-F(109)/11*y*z^3*v+F(15)/29*z^4*v+F(36)/95*x^3*u*v-4778*x^2*y*u*v-F(54)/47*x*y^2*u*v-15780*y^3*u*v+8605*x^2*z*u*v+8971*x*y*z*u*v-1222*y^2*z*u*v-F(5)/62*x*z^2*u*v-12499*y*z^2*u*v+F(108)/85*z^3*u*v+F(89)/7*x^2*u^2*v-F(29)/121*x*y*u^2*v+15870*y^2*u^2*v-9374*x*z*u^2*v-F(116)/117*y*z*u^2*v+F(37)/93*z^2*u^2*v+14105*x*u^3*v+1167*y*u^3*v-10548*z*u^3*v-F(96)/109*u^4*v-F(113)/61*x^3*v^2-1448*x^2*y*v^2+11840*x*y^2*v^2-F(61)/107*y^3*v^2+F(37)/107*x^2*z*v^2-1087*x*y*z*v^2-3224*y^2*z*v^2+4372*x*z^2*v^2+F(53)/82*y*z^2*v^2+14358*z^3*v^2-10076*x^2*u*v^2-F(120)/107*x*y*u*v^2+F(95)/73*y^2*u*v^2+636*x*z*u*v^2+3776*y*z*u*v^2-4590*z^2*u*v^2+9051*x*u^2*v^2-1513*y*u^2*v^2-6086*z*u^2*v^2+F(59)/84*u^3*v^2+F(121)/79*x^2*v^3+F(71)/33*x*y*v^3-F(19)/78*y^2*v^3+2219*x*z*v^3-F(115)/114*y*z*v^3-F(75)/62*z^2*v^3-F(35)/31*x*u*v^3-F(68)/7*y*u*v^3-F(53)/92*z*u*v^3-4852*u^2*v^3+F(4)/81*x*v^4-F(17)/78*y*v^4+F(42)/61*z*v^4-F(90)/101*u*v^4-F(89)/96*v^5,F(47)/88*x^3*y^2+9610*x^2*y^3-3005*x*y^4+10610*y^5+1737*x^3*y*z+F(17)/114*x^2*y^2*z+13934*x*y^3*z+F(64)/9*y^4*z+F(107)/13*x^2*y*z^2+F(10)/9*x*y^2*z^2+F(41)/71*y^3*z^2-F(118)/53*x*y*z^3+F(25)/111*y^2*z^3-F(93)/28*y*z^4-F(35)/59*x^3*y*u-F(83)/35*x^2*y^2*u+F(54)/119*x*y^3*u-10016*y^4*u-F(37)/110*x^3*z*u-F(71)/42*x^2*y*z*u+14431*x*y^2*z*u-F(38)/43*y^3*z*u-15166*x^2*z^2*u+6369*x*y*z^2*u+2714*y^2*z^2*u-2119*x*z^3*u-F(79)/22*y*z^3*u-F(8)/35*z^4*u+F(75)/112*x^3*u^2-3707*x^2*y*u^2+F(8)/17*x*y^2*u^2+15825*y^3*u^2+F(77)/120*x^2*z*u^2-14620*x*y*z*u^2+F(101)/26*y^2*z*u^2+F(24)/121*x*z^2*u^2+14335*y*z^2*u^2+F(75)/53*z^3*u^2+F(37)/65*x^2*u^3+F(59)/81*x*y*u^3-10919*y^2*u^3+F(82)/51*x*z*u^3+505*y*z*u^3+F(46)/39*z^2*u^3+3736*x*u^4-671*y*u^4-14275*z*u^4+13499*u^5+F(89)/19*x^3*y*v-F(45)/11*x^2*y^2*v-F(65)/94*x*y^3*v+F(119)/3*y^4*v+11540*x^3*z*v-15133*x^2*y*z*v-F(53)/78*x*y^2*z*v+14746*y^3*z*v-F(3)/103*x^2*z^2*v+F(56)/111*x*y*z^2*v+F(45)/107*y^2*z^2*v+3218*x*z^3*v-F(115)/83*y*z^3*v-F(81)/56*z^4*v+6205*x^3*u*v+F(67)/122*x^2*y*u*v+F(55)/101*x*y^2*u*v-F(105)/13*y^3*u*v-755*x^2*z*u*v+F(116)/53*x*y*z*u*v+F(67)/9*y^2*z*u*v-8158*x*z^2*u*v+F(35)/113*y*z^2*u*v+F(117)/70*z^3*u*v+F(7)/64*x^2*u^2*v-F(103)/23*x*y*u^2*v+F(65)/83*y^2*u^2*v-3972*x*z*u^2*v-F(78)/107*y*z*u^2*v+78*z^2*u^2*v-F(117)/53*x*u^3*v-F(1)/77*y*u^3*v+F(76)/83*z*u^3*v-851*u^4*v-11277*x^3*v^2+9638*x^2*y*v^2-F(116)/125*x*y^2*v^2+7228*y^3*v^2-F(5)/48*x^2*z*v^2+4549*x*y*z*v^2-F(73)/114*y^2*z*v^2-F(32)/23*x*z^2*v^2+3274*y*z^2*v^2-12707*z^3*v^2-3517*x^2*u*v^2+9907*x*y*u*v^2-F(33)/26*y^2*u*v^2+6046*x*z*u*v^2+F(6)/121*y*z*u*v^2-F(5)/57*z^2*u*v^2+F(99)/91*x*u^2*v^2-F(13)/120*y*u^2*v^2-F(108)/115*z*u^2*v^2+F(65)/109*u^3*v^2+F(8)/53*x^2*v^3-F(55)/21*x*y*v^3-F(109)/84*y^2*v^3+F(32)/91*x*z*v^3-F(36)/7*y*z*v^3-F(92)/9*z^2*v^3+F(9)/17*x*u*v^3-15294*y*u*v^3+F(59)/34*z*u*v^3-F(29)/73*u^2*v^3-15736*x*v^4+14440*y*v^4+F(23)/8*z*v^4-F(32)/69*u*v^4-F(1)/109*v^5,F(47)/88*x^3*y*z+9610*x^2*y^2*z-3005*x*y^3*z+10610*y^4*z+1737*x^3*z^2+F(17)/114*x^2*y*z^2+13934*x*y^2*z^2+F(64)/9*y^3*z^2+F(107)/13*x^2*z^3+F(10)/9*x*y*z^3+F(41)/71*y^2*z^3-F(118)/53*x*z^4+F(25)/111*y*z^4-F(93)/28*z^5-F(111)/29*x^3*y*u-F(51)/82*x^2*y^2*u-F(95)/68*x*y^3*u-F(106)/93*y^4*u-F(73)/41*x^3*z*u+F(101)/92*x^2*y*z*u+5654*x*y^2*z*u+F(69)/37*y^3*z*u-F(25)/37*x^2*z^2*u-F(6)/61*x*y*z^2*u-11416*y^2*z^2*u-11303*x*z^3*u+4670*y*z^3*u-F(120)/17*z^4*u+F(79)/56*x^3*u^2-1385*x^2*y*u^2-1544*x*y^2*u^2-12915*y^3*u^2-F(72)/83*x^2*z*u^2+F(41)/66*x*y*z*u^2+11317*y^2*z*u^2+F(124)/71*x*z^2*u^2+F(44)/37*y*z^2*u^2+5292*z^3*u^2-6745*x^2*u^3+F(100)/87*x*y*u^3-3434*y^2*u^3-10509*x*z*u^3-13766*y*z*u^3-8559*z^2*u^3+2616*x*u^4+F(111)/83*y*u^4-11580*z*u^4-F(87)/76*u^5+F(122)/49*x^3*y*v-126*x^2*y^2*v+4670*x*y^3*v+F(65)/23*y^4*v+F(89)/122*x^3*z*v-F(76)/121*x^2*y*z*v+F(41)/108*x*y^2*z*v+F(57)/101*y^3*z*v+F(87)/109*x^2*z^2*v-F(32)/117*x*y*z^2*v-14089*y^2*z^2*v+9316*x*z^3*v-F(97)/25*y*z^3*v+11769*z^4*v+F(97)/94*x^3*u*v-F(45)/8*x^2*y*u*v+11839*x*y^2*u*v+F(41)/61*y^3*u*v+F(113)/24*x^2*z*u*v+15900*x*y*z*u*v+F(62)/35*y^2*z*u*v-4121*x*z^2*u*v-F(62)/15*y*z^2*u*v+F(111)/119*z^3*u*v-F(99)/119*x^2*u^2*v+F(52)/5*x*y*u^2*v+F(61)/87*y^2*u^2*v+F(8)/63*x*z*u^2*v+F(125)/4*y*z*u^2*v-5637*z^2*u^2*v+F(83)/80*x*u^3*v+11698*y*u^3*v+F(123)/77*z*u^3*v-7129*u^4*v+F(41)/109*x^3*v^2+F(29)/89*x^2*y*v^2+F(121)/38*x*y^2*v^2+F(26)/47*y^3*v^2+F(23)/38*x^2*z*v^2+F(86)/99*x*y*z*v^2+13269*y^2*z*v^2+3538*x*z^2*v^2+F(41)/114*y*z^2*v^2-5908*z^3*v^2+F(69)/5*x^2*u*v^2-F(119)/5*x*y*u*v^2-F(91)/99*y^2*u*v^2-F(40)/57*x*z*u*v^2-1824*y*z*u*v^2-11647*z^2*u*v^2+5358*x*u^2*v^2-F(40)/73*y*u^2*v^2+F(25)/78*z*u^2*v^2-F(39)/100*u^3*v^2-7457*x^2*v^3-F(42)/19*x*y*v^3+F(9)/4*y^2*v^3+12013*x*z*v^3+F(16)/47*y*z*v^3-10527*z^2*v^3+4670*x*u*v^3+F(79)/120*y*u*v^3+11137*z*u*v^3+F(99)/62*u^2*v^3+F(111)/38*x*v^4-F(47)/69*y*v^4-2543*z*v^4-9159*u*v^4-9559*v^5,-F(111)/29*x^4*y-F(51)/82*x^3*y^2-F(95)/68*x^2*y^3-F(106)/93*x*y^4+1479*x^4*z+6993*x^3*y*z-F(45)/77*x^2*y^2*z+F(29)/7*x*y^3*z+F(12)/59*y^4*z+F(59)/23*x^3*z^2-F(49)/80*x^2*y*z^2+6299*x*y^2*z^2+5541*y^3*z^2-F(93)/89*x^2*z^3+5480*x*y*z^3+4541*y^2*z^3+F(12)/35*x*z^4+423*y*z^4-F(16)/55*z^5+F(79)/56*x^4*u-1385*x^3*y*u-1544*x^2*y^2*u-12915*x*y^3*u+F(81)/43*x^3*z*u-F(102)/41*x^2*y*z*u+15746*x*y^2*z*u-F(19)/9*y^3*z*u+F(67)/9*x^2*z^2*u-3418*x*y*z^2*u+F(76)/45*y^2*z^2*u-192*x*z^3*u+F(95)/111*y*z^3*u+2747*z^4*u-6745*x^3*u^2+F(100)/87*x^2*y*u^2-3434*x*y^2*u^2+F(105)/116*x^2*z*u^2+8428*x*y*z*u^2-510*y^2*z*u^2-F(86)/63*x*z^2*u^2-3358*y*z^2*u^2+14026*z^3*u^2+2616*x^2*u^3+F(111)/83*x*y*u^3+F(47)/22*x*z*u^3+14238*y*z*u^3-8120*z^2*u^3-F(87)/76*x*u^4-15341*z*u^4-F(80)/89*x^4*v-15753*x^3*y*v+F(55)/54*x^2*y^2*v+11588*x*y^3*v+F(26)/29*y^4*v+F(70)/67*x^3*z*v+15489*x^2*y*z*v+612*x*y^2*z*v-F(13)/83*y^3*z*v+3577*x^2*z^2*v-F(41)/23*x*y*z^2*v-F(96)/91*y^2*z^2*v-F(95)/44*x*z^3*v-5014*y*z^3*v-F(44)/27*z^4*v-F(56)/101*x^3*u*v+F(41)/63*x^2*y*u*v+F(29)/36*x*y^2*u*v-F(109)/92*y^3*u*v+14988*x^2*z*u*v-F(77)/86*x*y*z*u*v+12318*y^2*z*u*v-F(107)/85*x*z^2*u*v+4895*y*z^2*u*v+F(22)/13*z^3*u*v+F(83)/80*x^2*u^2*v-9074*x*y*u^2*v-3941*y^2*u^2*v-1990*x*z*u^2*v+F(85)/108*y*z*u^2*v+F(11)/12*z^2*u^2*v-F(63)/113*x*u^3*v+F(60)/23*y*u^3*v+F(21)/95*z*u^3*v+F(29)/76*u^4*v-F(69)/40*x^3*v^2+F(123)/2*x^2*y*v^2+4885*x*y^2*v^2+F(11)/27*y^3*v^2-F(73)/40*x^2*z*v^2-F(81)/125*x*y*z*v^2-F(79)/72*y^2*z*v^2-12281*x*z^2*v^2+F(88)/125*y*z^2*v^2-F(54)/7*z^3*v^2+13587*x^2*u*v^2+9937*x*y*u*v^2-F(59)/14*y^2*u*v^2-223*x*z*u*v^2-F(15)/101*y*z*u*v^2-F(125)/21*z^2*u*v^2+13593*x*u^2*v^2-F(93)/89*y*u^2*v^2-F(78)/115*z*u^2*v^2+F(104)/7*u^3*v^2-10799*x^2*v^3+F(33)/7*x*y*v^3-F(13)/98*y^2*v^3-F(124)/119*x*z*v^3+12751*y*z*v^3+F(1)/85*z^2*v^3+F(64)/17*x*u*v^3-4459*y*u*v^3+F(8)/29*z*u*v^3+F(79)/107*u^2*v^3-F(5)/74*x*v^4-12945*y*v^4-F(26)/17*z*v^4-F(28)/45*u*v^4+4309*v^5,F(111)/29*x^3*y^2+F(51)/82*x^2*y^3+F(95)/68*x*y^4+F(106)/93*y^5+F(115)/108*x^3*y*z+F(47)/38*x^2*y^2*z-14525*x*y^3*z-7424*y^4*z-F(37)/110*x^3*z^2+13730*x^2*y*z^2-F(67)/110*x*y^2*z^2-F(49)/93*y^3*z^2-15166*x^2*z^3+F(82)/105*x*y*z^3-1956*y^2*z^3-2119*x*z^4+F(37)/60*y*z^4-F(8)/35*z^5-F(79)/56*x^3*y*u+1385*x^2*y^2*u+1544*x*y^3*u+12915*y^4*u+F(75)/112*x^3*z*u+F(87)/50*x^2*y*z*u+F(51)/40*x*y^2*z*u+4508*y^3*z*u+F(77)/120*x^2*z^2*u-1555*x*y*z^2*u-F(72)/109*y^2*z^2*u+F(24)/121*x*z^3*u+F(95)/46*y*z^3*u+F(75)/53*z^4*u+6745*x^2*y*u^2-F(100)/87*x*y^2*u^2+3434*y^3*u^2+F(37)/65*x^2*z*u^2-7658*x*y*z*u^2+2847*y^2*z*u^2+F(82)/51*x*z^2*u^2-F(7)/60*y*z^2*u^2+F(46)/39*z^3*u^2-2616*x*y*u^3-F(111)/83*y^2*u^3+3736*x*z*u^3+10909*y*z*u^3-14275*z^2*u^3+F(87)/76*y*u^4+13499*z*u^4+F(113)/21*x^3*y*v-F(90)/61*x^2*y^2*v+F(118)/33*x*y^3*v-953*y^4*v+F(100)/101*x^3*z*v-7204*x^2*y*z*v+4279*x*y^2*z*v-F(17)/4*y^3*z*v+F(32)/7*x^2*z^2*v-F(47)/59*x*y*z^2*v-F(19)/74*y^2*z^2*v+F(25)/86*x*z^3*v+F(97)/90*y*z^3*v-15678*z^4*v+F(43)/26*x^3*u*v+F(113)/94*x^2*y*u*v-8247*x*y^2*u*v+10411*y^3*u*v-F(82)/89*x^2*z*u*v+9102*x*y*z*u*v+F(50)/121*y^2*z*u*v+F(13)/55*x*z^2*u*v+F(47)/16*y*z^2*u*v+30*z^3*u*v+F(103)/31*x^2*u^2*v+F(47)/45*x*y*u^2*v+F(20)/47*y^2*u^2*v+14336*x*z*u^2*v+F(80)/81*y*z*u^2*v+F(65)/63*z^2*u^2*v-14006*x*u^3*v-2152*y*u^3*v+F(121)/105*z*u^3*v+12916*u^4*v-F(15)/13*x^3*v^2+975*x^2*y*v^2+5610*x*y^2*v^2+12390*y^3*v^2+F(107)/12*x^2*z*v^2+2276*x*y*z*v^2-F(109)/93*y^2*z*v^2-13828*x*z^2*v^2-F(59)/117*y*z^2*v^2+F(3)/89*z^3*v^2-6524*x^2*u*v^2+7014*x*y*u*v^2+2940*y^2*u*v^2+1659*x*z*u*v^2+5167*y*z*u*v^2-F(67)/5*z^2*u*v^2-11575*x*u^2*v^2-F(118)/69*y*u^2*v^2+9666*z*u^2*v^2+F(83)/65*u^3*v^2-F(84)/37*x^2*v^3-4551*x*y*v^3-F(124)/89*y^2*v^3-2640*x*z*v^3+6796*y*z*v^3+5947*z^2*v^3-13578*x*u*v^3+15906*y*u*v^3+6007*z*u*v^3-4632*u^2*v^3+11588*x*v^4+5383*y*v^4+4026*z*v^4+7238*u*v^4-3906*v^5,x^4*y+F(64)/3*x^3*y^2-9987*x^2*y^3-F(53)/8*x*y^4-F(12)/59*y^5+F(37)/110*x^4*z-F(29)/118*x^3*y*z+5636*x^2*y^2*z-F(3)/100*x*y^3*z-5541*y^4*z+15166*x^3*z^2-F(100)/101*x^2*y*z^2+F(51)/118*x*y^2*z^2-4541*y^3*z^2+2119*x^2*z^3+F(122)/111*x*y*z^3-423*y^2*z^3+F(8)/35*x*z^4+F(16)/55*y*z^4-F(75)/112*x^4*u-F(126)/121*x^3*y*u+6653*x^2*y^2*u+11737*x*y^3*u+F(19)/9*y^4*u-F(77)/120*x^3*z*u-F(121)/16*x^2*y*z*u+13691*x*y^2*z*u-F(76)/45*y^3*z*u-F(24)/121*x^2*z^2*u-F(114)/47*x*y*z^2*u-F(95)/111*y^2*z^2*u-F(75)/53*x*z^3*u-2747*y*z^3*u-F(37)/65*x^3*u^2+12897*x^2*y*u^2+F(63)/122*x*y^2*u^2+510*y^3*u^2-F(82)/51*x^2*z*u^2-F(1)/85*x*y*z*u^2+3358*y^2*z*u^2-F(46)/39*x*z^2*u^2-14026*y*z^2*u^2-3736*x^2*u^3-9457*x*y*u^3-14238*y^2*u^3+14275*x*z*u^3+8120*y*z*u^3-13499*x*u^4+15341*y*u^4-F(58)/91*x^4*v+F(34)/19*x^3*y*v-7398*x^2*y^2*v+F(89)/113*x*y^3*v-1646*y^4*v+F(113)/59*x^3*z*v+F(87)/26*x^2*y*z*v+13479*x*y^2*z*v-F(49)/69*y^3*z*v-11796*x^2*z^2*v+F(123)/103*x*y*z^2*v+F(103)/75*y^2*z^2*v+4183*x*z^3*v-F(65)/62*y*z^3*v-F(95)/88*z^4*v+2121*x^3*u*v-3663*x^2*y*u*v+F(28)/23*x*y^2*u*v-F(44)/101*y^3*u*v+F(95)/62*x^2*z*u*v-F(65)/71*x*y*z*u*v+1725*y^2*z*u*v-8439*x*z^2*u*v+F(44)/13*y*z^2*u*v+F(43)/71*z^3*u*v+F(72)/125*x^2*u^2*v+2227*x*y*u^2*v-F(26)/89*y^2*u^2*v-F(99)/83*x*z*u^2*v+4015*y*z*u^2*v-11025*z^2*u^2*v-F(33)/58*x*u^3*v-12508*y*u^3*v-F(101)/56*z*u^3*v+12651*u^4*v-F(66)/115*x^3*v^2+9661*x^2*y*v^2-F(29)/115*x*y^2*v^2-F(52)/9*y^3*v^2-F(29)/34*x^2*z*v^2-F(79)/117*x*y*z*v^2-F(12)/49*y^2*z*v^2-F(109)/44*x*z^2*v^2+883*y*z^2*v^2-F(102)/97*z^3*v^2-13344*x^2*u*v^2-5075*x*y*u*v^2+F(113)/87*y^2*u*v^2+6487*x*z*u*v^2+F(79)/112*y*z*u*v^2-4058*z^2*u*v^2+F(123)/53*x*u^2*v^2-F(61)/93*y*u^2*v^2+F(8)/15*z*u^2*v^2+5*u^3*v^2-F(4)/117*x^2*v^3+14784*x*y*v^3-10718*y^2*v^3+F(37)/107*x*z*v^3-F(59)/41*y*z*v^3+F(97)/52*z^2*v^3+F(77)/45*x*u*v^3-8565*y*u*v^3+F(43)/71*z*u*v^3+F(73)/19*u^2*v^3-12614*x*v^4+F(7)/38*y*v^4+15664*z*v^4-10720*u*v^4-12027*v^5,-2543*x^3*y-F(59)/83*x^2*y^2+F(3)/16*x*y^3-F(18)/37*y^4-2072*x^3*z+F(63)/22*x^2*y*z+11452*x*y^2*z-13315*y^3*z-14845*x^2*z^2-F(99)/2*x*y*z^2+11841*y^2*z^2+F(54)/89*x*z^3-F(27)/53*y*z^3-F(95)/104*z^4-F(87)/113*x^3*u-12090*x^2*y*u+F(25)/106*x*y^2*u+1120*y^3*u-173*x^2*z*u-1735*x*y*z*u-F(9)/112*y^2*z*u+4895*x*z^2*u-11886*y*z^2*u-F(120)/17*z^3*u+F(45)/121*x^2*u^2-1327*x*y*u^2+F(23)/85*y^2*u^2-3422*x*z*u^2+2034*y*z*u^2-2829*z^2*u^2+5830*x*u^3-F(93)/65*y*u^3+F(21)/19*z*u^3-F(101)/100*u^4-140*x^3*v+12757*x^2*y*v-3848*x*y^2*v-F(71)/86*y^3*v-11134*x^2*z*v+F(33)/53*x*y*z*v+F(37)/61*y^2*z*v-11686*x*z^2*v-F(37)/76*y*z^2*v-3316*z^3*v+F(32)/3*x^2*u*v-F(37)/3*x*y*u*v-F(43)/125*y^2*u*v+F(111)/23*x*z*u*v+F(32)/97*y*z*u*v-7884*z^2*u*v+8055*x*u^2*v+F(31)/52*y*u^2*v-14317*z*u^2*v+9666*u^3*v-F(65)/72*x^2*v^2-12481*x*y*v^2+F(7)/109*y^2*v^2-F(71)/115*x*z*v^2-9788*y*z*v^2+1955*z^2*v^2-F(43)/20*x*u*v^2+F(1)/21*y*u*v^2-F(2)/19*z*u*v^2-F(13)/123*u^2*v^2+F(7)/79*x*v^3-F(64)/97*y*v^3+F(13)/19*z*v^3-7458*u*v^3+3682*v^4,-F(59)/83*x^4+F(33)/98*x^3*y+F(53)/22*x^2*y^2+F(107)/85*x*y^3-F(29)/82*y^4+3534*x^3*z-F(108)/37*x^2*y*z+5635*x*y^2*z-F(118)/121*y^3*z-F(117)/35*x^2*z^2-8619*x*y*z^2-F(82)/101*y^2*z^2-F(4)/73*x*z^3-5967*y*z^3-26*z^4-14917*x^3*u+F(73)/52*x^2*y*u+F(103)/92*x*y^2*u-F(100)/99*y^3*u-F(103)/123*x^2*z*u+12512*x*y*z*u-F(20)/71*y^2*z*u-6846*x*z^2*u-4514*y*z^2*u+F(121)/34*z^3*u+F(19)/16*x^2*u^2+7645*x*y*u^2-15042*y^2*u^2+6886*x*z*u^2-F(11)/30*y*z*u^2-2299*z^2*u^2+F(46)/49*x*u^3-F(90)/37*y*u^3+F(67)/13*z*u^3+F(67)/63*u^4+F(45)/68*x^3*v-F(61)/105*x^2*y*v+F(73)/126*x*y^2*v-F(77)/23*y^3*v+F(122)/57*x^2*z*v+12923*x*y*z*v-2926*y^2*z*v-1126*x*z^2*v-F(16)/5*y*z^2*v+F(111)/58*z^3*v-10774*x^2*u*v+5603*x*y*u*v-975*y^2*u*v-8680*x*z*u*v+3973*y*z*u*v-5795*z^2*u*v+2486*x*u^2*v+8136*y*u^2*v-F(34)/87*z*u^2*v-11205*u^3*v-10372*x^2*v^2-9501*x*y*v^2+2121*y^2*v^2+15744*x*z*v^2+455*y*z*v^2-F(97)/107*z^2*v^2-F(25)/118*x*u*v^2-10580*y*u*v^2+F(91)/6*z*u*v^2-11835*u^2*v^2-F(68)/93*x*v^3-9566*y*v^3+F(117)/14*z*v^3+13917*u*v^3-F(67)/44*v^4]))
end


@doc raw"""
    k3_d12_pi14()

Return a smooth K3 surface in $\mathbb P^4$ with degree `12` and sectional genus `14`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d12_pi14()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(25)/104*x^5-F(62)/21*x^4*y-F(69)/94*x^3*y^2-F(70)/87*x^2*y^3-F(100)/23*x*y^4-F(113)/115*y^5+F(89)/62*x^4*z+12922*x^3*y*z-9727*x^2*y^2*z-13903*x*y^3*z+12129*y^4*z+F(73)/98*x^3*z^2+F(101)/68*x^2*y*z^2-F(97)/65*x*y^2*z^2+F(69)/118*y^3*z^2+5366*x^2*z^3+F(106)/93*x*y*z^3+F(19)/47*y^2*z^3-F(35)/68*x*z^4+8408*y*z^4-F(71)/90*z^5+12228*x^4*u-F(100)/103*x^3*y*u-F(103)/22*x^2*y^2*u+F(58)/79*x*y^3*u-F(43)/20*y^4*u+F(85)/53*x^3*z*u-F(81)/65*x^2*y*z*u+F(65)/21*x*y^2*z*u+7356*y^3*z*u+F(64)/103*x^2*z^2*u-F(9)/11*x*y*z^2*u+F(111)/38*y^2*z^2*u+F(121)/83*x*z^3*u-F(40)/29*y*z^3*u+8877*z^4*u+8171*x^3*u^2-F(91)/15*x^2*y*u^2-F(89)/97*x*y^2*u^2-F(109)/84*y^3*u^2+F(9)/46*x^2*z*u^2-F(83)/96*x*y*z*u^2+F(17)/46*y^2*z*u^2+3472*x*z^2*u^2-F(39)/85*y*z^2*u^2-F(50)/59*z^3*u^2-714*x^2*u^3+F(26)/73*x*y*u^3-F(111)/14*y^2*u^3+12937*x*z*u^3-F(109)/73*y*z*u^3+37*z^2*u^3+F(88)/49*x*u^4+F(101)/2*y*u^4-1052*z*u^4-F(106)/93*x^4*v-F(19)/79*x^3*y*v-F(115)/94*x^2*y^2*v+F(9)/55*x*y^3*v+2576*y^4*v+2143*x^3*z*v-F(17)/66*x^2*y*z*v-F(101)/4*x*y^2*z*v+11407*y^3*z*v+F(23)/28*x^2*z^2*v-8074*x*y*z^2*v+F(23)/58*y^2*z^2*v+F(12)/89*x*z^3*v-F(125)/93*y*z^3*v-8092*z^4*v-F(106)/101*x^3*u*v+9339*x^2*y*u*v+5241*x*y^2*u*v+F(78)/77*y^3*u*v-F(89)/5*x^2*z*u*v+12535*x*y*z*u*v+F(95)/11*y^2*z*u*v+15701*x*z^2*u*v+15254*y*z^2*u*v+12528*z^3*u*v-F(125)/117*x^2*u^2*v-137*x*y*u^2*v-1401*y^2*u^2*v+F(12)/13*x*z*u^2*v-9443*y*z*u^2*v+F(89)/33*z^2*u^2*v-F(43)/55*x*u^3*v-14194*y*u^3*v-F(32)/27*z*u^3*v-15609*x^3*v^2+F(103)/77*x^2*y*v^2+5309*x*y^2*v^2-F(94)/83*y^3*v^2-F(57)/89*x^2*z*v^2+F(63)/92*x*y*z*v^2-F(32)/9*y^2*z*v^2+F(86)/99*x*z^2*v^2+3395*y*z^2*v^2+F(109)/116*z^3*v^2+F(125)/27*x^2*u*v^2+F(59)/81*x*y*u*v^2-F(29)/15*y^2*u*v^2+1249*x*z*u*v^2-F(75)/124*y*z*u*v^2+F(41)/37*z^2*u*v^2+F(119)/82*x*u^2*v^2-F(101)/70*y*u^2*v^2-F(29)/19*z*u^2*v^2-F(81)/119*x^2*v^3+F(31)/100*x*y*v^3-F(27)/53*y^2*v^3+8131*x*z*v^3+13432*y*z*v^3-10197*z^2*v^3-F(5)/24*x*u*v^3-F(85)/21*y*u*v^3+F(31)/83*z*u*v^3+F(77)/67*x*v^4-F(11)/15*y*v^4+F(61)/120*z*v^4,F(59)/95*x^5-F(53)/104*x^4*y+13540*x^3*y^2-F(111)/70*x^2*y^3-13577*x*y^4-5357*x^4*z-8172*x^3*y*z+13449*x^2*y^2*z+F(11)/123*x*y^3*z+F(91)/67*y^4*z+1657*x^3*z^2-F(17)/117*x^2*y*z^2+707*x*y^2*z^2-F(50)/57*y^3*z^2-F(48)/113*x^2*z^3+14233*x*y*z^3-F(47)/67*y^2*z^3-15577*x*z^4+F(95)/17*y*z^4-6727*z^5+14955*x^4*u-F(123)/88*x^3*y*u+11755*x^2*y^2*u+F(83)/73*x*y^3*u+F(113)/115*y^4*u+805*x^3*z*u+5167*x^2*y*z*u-F(114)/71*x*y^2*z*u+15277*y^3*z*u+F(38)/89*x^2*z^2*u-10737*x*y*z^2*u-F(23)/31*y^2*z^2*u-F(51)/110*x*z^3*u+F(1)/39*y*z^3*u-F(25)/31*z^4*u+F(43)/40*x^3*u^2+15503*x^2*y*u^2+F(100)/73*x*y^2*u^2+F(43)/20*y^3*u^2+F(95)/2*x^2*z*u^2-6476*x*y*z*u^2+8454*y^2*z*u^2+F(70)/13*x*z^2*u^2-F(79)/41*y*z^2*u^2-388*z^3*u^2-F(109)/8*x^2*u^3+F(17)/111*x*y*u^3+F(109)/84*y^2*u^3-10213*x*z*u^3-10599*y*z*u^3+F(109)/106*z^2*u^3+13375*x*u^4+F(111)/14*y*u^4-F(104)/15*z*u^4-F(101)/2*u^5-5125*x^4*v+11510*x^3*y*v-15364*x^2*y^2*v-F(5)/112*x*y^3*v+8151*x^3*z*v+F(31)/100*x^2*y*z*v-F(107)/61*x*y^2*z*v-F(68)/35*y^3*z*v-10894*x^2*z^2*v-7490*x*y*z^2*v+F(73)/50*y^2*z^2*v+5521*x*z^3*v-F(114)/83*y*z^3*v-F(41)/35*z^4*v+5932*x^3*u*v-F(59)/80*x^2*y*u*v-10473*x*y^2*u*v-2576*y^3*u*v-F(59)/36*x^2*z*u*v+7207*x*y*z*u*v+F(39)/122*y^2*z*u*v-13410*x*z^2*u*v+12401*y*z^2*u*v+12024*z^3*u*v-3224*x^2*u^2*v+F(95)/17*x*y*u^2*v-F(78)/77*y^2*u^2*v-F(27)/35*x*z*u^2*v-F(52)/105*y*z*u^2*v-9051*z^2*u^2*v+F(101)/24*x*u^3*v+1401*y*u^3*v+F(17)/25*z*u^3*v+14194*u^4*v+F(86)/13*x^3*v^2+5835*x^2*y*v^2-F(37)/122*x*y^2*v^2+7136*x^2*z*v^2+569*x*y*z*v^2+5300*y^2*z*v^2+F(67)/3*x*z^2*v^2-1025*y*z^2*v^2-F(59)/11*z^3*v^2+F(58)/23*x^2*u*v^2-F(59)/109*x*y*u*v^2+F(94)/83*y^2*u*v^2+585*x*z*u*v^2+F(61)/25*y*z*u*v^2+8490*z^2*u*v^2+2023*x*u^2*v^2+F(29)/15*y*u^2*v^2-1325*z*u^2*v^2+F(101)/70*u^3*v^2+8055*x^2*v^3+143*x*y*v^3+14787*x*z*v^3+F(58)/65*y*z*v^3-F(39)/79*z^2*v^3+F(75)/34*x*u*v^3+F(27)/53*y*u*v^3-6549*z*u*v^3+F(85)/21*u^2*v^3-F(125)/41*x*v^4-F(29)/27*z*v^4+F(11)/15*u*v^4,-F(59)/95*x^4*y+F(53)/104*x^3*y^2-13540*x^2*y^3+F(111)/70*x*y^4+13577*y^5+8837*x^4*z-F(71)/81*x^3*y*z+F(123)/5*x^2*y^2*z+F(121)/118*x*y^3*z-F(15)/47*y^4*z-F(124)/13*x^3*z^2-F(56)/45*x^2*y*z^2+F(107)/74*x*y^2*z^2-11259*y^3*z^2+14670*x^2*z^3-13130*x*y*z^3+F(4)/93*y^2*z^3-11067*x*z^4+F(96)/47*y*z^4-5167*z^5+F(25)/104*x^4*u-F(28)/81*x^3*y*u+7559*x^2*y^2*u+F(61)/34*x*y^3*u-F(91)/31*y^4*u+F(26)/67*x^3*z*u+F(48)/109*x^2*y*z*u-F(59)/108*x*y^2*z*u+2236*y^3*z*u+F(38)/49*x^2*z^2*u-F(13)/23*x*y*z^2*u+F(43)/68*y^2*z^2*u-F(125)/28*x*z^3*u-F(73)/25*y*z^3*u+F(111)/19*z^4*u-12228*x^3*u^2+F(53)/13*x^2*y*u^2-F(70)/111*x*y^2*u^2+7054*y^3*u^2+F(66)/107*x^2*z*u^2-2478*x*y*z*u^2+F(83)/93*y^2*z*u^2+F(84)/5*x*z^2*u^2+F(118)/27*y*z^2*u^2+F(9)/88*z^3*u^2-8171*x^2*u^3-F(53)/65*x*y*u^3+F(107)/105*y^2*u^3-F(119)/52*x*z*u^3-3009*y*z*u^3-F(40)/69*z^2*u^3+714*x*u^4-3296*y*u^4-13732*z*u^4-F(88)/49*u^5+5125*x^3*y*v-11510*x^2*y^2*v+15364*x*y^3*v+F(5)/112*y^4*v-F(83)/118*x^3*z*v+6048*x^2*y*z*v+F(10)/19*x*y^2*z*v+8373*y^3*z*v-10168*x^2*z^2*v-11815*x*y*z^2*v+701*y^2*z^2*v-F(37)/89*x*z^3*v-F(115)/93*y*z^3*v-8565*z^4*v+F(106)/93*x^3*u*v-F(75)/8*x^2*y*u*v+F(34)/13*x*y^2*u*v-13375*y^3*u*v+F(69)/79*x^2*z*u*v-13114*x*y*z*u*v+F(87)/77*y^2*z*u*v+F(111)/88*x*z^2*u*v+F(110)/83*y*z^2*u*v+F(97)/39*z^3*u*v+F(106)/101*x^2*u^2*v+F(63)/68*x*y*u^2*v+9808*y^2*u^2*v+F(43)/101*x*z*u^2*v-F(43)/76*y*z*u^2*v-F(5)/97*z^2*u^2*v+F(125)/117*x*u^3*v-6532*y*u^3*v-F(77)/80*z*u^3*v+F(43)/55*u^4*v-F(86)/13*x^2*y*v^2-5835*x*y^2*v^2+F(37)/122*y^3*v^2-5404*x^2*z*v^2-F(11)/98*x*y*z*v^2+4278*y^2*z*v^2-F(79)/74*x*z^2*v^2+F(53)/81*y*z^2*v^2-14643*z^3*v^2+15609*x^2*u*v^2+F(38)/51*x*y*u*v^2-F(13)/12*y^2*u*v^2-F(53)/48*x*z*u*v^2-2236*y*z*u*v^2+F(121)/74*z^2*u*v^2-F(125)/27*x*u^2*v^2-F(106)/107*y*u^2*v^2-1427*z*u^2*v^2-F(119)/82*u^3*v^2-8055*x*y*v^3-143*y^2*v^3-F(124)/63*x*z*v^3-F(57)/116*y*z*v^3-F(79)/76*z^2*v^3+F(81)/119*x*u*v^3+F(97)/126*y*u*v^3+4341*z*u*v^3+F(5)/24*u^2*v^3+F(125)/41*y*v^4-3883*z*v^4-F(77)/67*u*v^4,-8837*x^5+F(89)/11*x^4*y-F(86)/117*x^3*y^2+11221*x^2*y^3+F(62)/29*x*y^4-F(91)/67*y^5+F(124)/13*x^4*z+F(82)/27*x^3*y*z-F(29)/99*x^2*y^2*z+10552*x*y^3*z+F(50)/57*y^4*z-14670*x^3*z^2+F(11)/8*x^2*y*z^2-15609*x*y^2*z^2+F(47)/67*y^3*z^2+11067*x^2*z^3-F(94)/103*x*y*z^3-F(95)/17*y^2*z^3+5167*x*z^4+6727*y*z^4-11546*x^4*u+F(50)/71*x^3*y*u-F(17)/15*x^2*y^2*u-12212*x*y^3*u+F(104)/7*y^4*u-10774*x^3*z*u+F(111)/43*x^2*y*z*u+F(65)/77*x*y^2*z*u+F(95)/48*y^3*z*u-F(37)/91*x^2*z^2*u+F(57)/70*x*y*z^2*u-12549*y^2*z^2*u-F(52)/33*x*z^3*u-F(120)/103*y*z^3*u+F(71)/90*z^4*u-9265*x^3*u^2-F(31)/111*x^2*y*u^2+2295*x*y^2*u^2-15810*y^3*u^2+F(41)/44*x^2*z*u^2-F(37)/95*x*y*z*u^2-2773*y^2*z*u^2+8605*x*z^2*u^2+F(65)/3*y*z^2*u^2-8877*z^3*u^2+F(93)/70*x^2*u^3+893*x*y*u^3-F(63)/58*y^2*u^3-F(65)/68*x*z*u^3+3021*y*z*u^3+F(50)/59*z^2*u^3+795*x*u^4+F(78)/89*y*u^4-37*z*u^4+1052*u^5+F(83)/118*x^4*v-14199*x^3*y*v+9748*x^2*y^2*v-15189*x*y^3*v+F(68)/35*y^4*v+10168*x^3*z*v-9282*x^2*y*z*v+F(100)/33*x*y^2*z*v-F(73)/50*y^3*z*v+F(37)/89*x^2*z^2*v+F(72)/125*x*y*z^2*v+F(114)/83*y^2*z^2*v+8565*x*z^3*v+F(41)/35*y*z^3*v-F(50)/99*x^3*u*v+F(47)/27*x^2*y*u*v-847*x*y^2*u*v+F(57)/107*y^3*u*v+7736*x^2*z*u*v+F(7)/72*x*y*z*u*v-5231*y^2*z*u*v-3422*x*z^2*u*v+F(37)/63*y*z^2*u*v+8092*z^3*u*v+9583*x^2*u^2*v-F(37)/20*x*y*u^2*v-F(73)/60*y^2*u^2*v-F(2)/117*x*z*u^2*v-F(65)/98*y*z*u^2*v-12528*z^2*u^2*v+6429*x*u^3*v-7193*y*u^3*v-F(89)/33*z*u^3*v+F(32)/27*u^4*v+5404*x^3*v^2+F(104)/95*x^2*y*v^2+F(4)/33*x*y^2*v^2-5300*y^3*v^2+F(79)/74*x^2*z*v^2-F(44)/105*x*y*z*v^2+1025*y^2*z*v^2+14643*x*z^2*v^2+F(59)/11*y*z^2*v^2-F(51)/121*x^2*u*v^2+F(120)/101*x*y*u*v^2-8672*y^2*u*v^2+F(10)/71*x*z*u*v^2-F(92)/35*y*z*u*v^2-F(109)/116*z^2*u*v^2+178*x*u^2*v^2-11316*y*u^2*v^2-F(41)/37*z*u^2*v^2+F(29)/19*u^3*v^2+F(124)/63*x^2*v^3-F(109)/111*x*y*v^3-F(58)/65*y^2*v^3+F(79)/76*x*z*v^3+F(39)/79*y*z*v^3-F(55)/59*x*u*v^3+F(90)/79*y*u*v^3+10197*z*u*v^3-F(31)/83*u^2*v^3+3883*x*v^4+F(29)/27*y*v^4-F(61)/120*u*v^4,-15754*x^5+F(1)/75*x^4*y+F(40)/107*x^3*y^2+11856*x^2*y^3-F(108)/47*x*y^4+15308*x^4*z-8313*x^3*y*z-F(101)/93*x^2*y^2*z-F(7)/31*x*y^3*z-F(123)/122*x^3*z^2-F(31)/101*x^2*y*z^2-F(81)/113*x*y^2*z^2+3573*x^2*z^3+2701*x*y*z^3+11969*x*z^4-8544*x^4*u-F(50)/117*x^3*y*u+F(79)/64*x^2*y^2*u+F(59)/106*x*y^3*u-F(72)/73*y^4*u-F(11)/80*x^3*z*u-F(15)/94*x^2*y*z*u+F(47)/113*x*y^2*z*u-F(98)/51*y^3*z*u-F(107)/110*x^2*z^2*u-F(33)/116*x*y*z^2*u-F(79)/10*y^2*z^2*u-F(29)/59*x*z^3*u-15200*y*z^3*u+F(111)/110*z^4*u-F(50)/23*x^3*u^2+F(1)/91*x^2*y*u^2-F(74)/91*x*y^2*u^2-F(48)/7*y^3*u^2+F(24)/85*x^2*z*u^2+F(69)/86*x*y*z*u^2-F(61)/124*y^2*z*u^2+F(23)/65*x*z^2*u^2-F(25)/124*y*z^2*u^2-F(67)/107*z^3*u^2-F(84)/37*x^2*u^3+F(87)/14*x*y*u^3-975*y^2*u^3+4686*x*z*u^3+F(25)/83*y*z*u^3+F(100)/39*z^2*u^3-F(95)/61*x*u^4+12350*y*u^4+F(96)/49*z*u^4-F(111)/47*u^5+F(57)/94*x^4*v-8107*x^3*y*v-F(97)/107*x^2*y^2*v-15660*x*y^3*v-F(91)/67*y^4*v+F(83)/23*x^3*z*v+F(89)/85*x^2*y*z*v-F(47)/86*x*y^2*z*v+F(50)/57*y^3*z*v-F(24)/13*x^2*z^2*v-13382*x*y*z^2*v+F(47)/67*y^2*z^2*v+13018*x*z^3*v-F(95)/17*y*z^3*v+6727*z^4*v+5128*x^3*u*v-5138*x^2*y*u*v-2271*x*y^2*u*v-F(81)/89*y^3*u*v-F(93)/112*x^2*z*u*v-F(87)/37*x*y*z*u*v+F(75)/38*y^2*z*u*v+F(72)/49*x*z^2*u*v+1154*y*z^2*u*v+F(92)/19*z^3*u*v+F(101)/11*x^2*u^2*v+F(5)/82*x*y*u^2*v-F(22)/101*y^2*u^2*v-F(101)/60*x*z*u^2*v-F(44)/69*y*z*u^2*v-10393*z^2*u^2*v+12322*x*u^3*v-8177*y*u^3*v+F(89)/84*z*u^3*v+1474*u^4*v+F(91)/5*x^3*v^2-F(123)/20*x^2*y*v^2-F(3)/83*x*y^2*v^2+F(68)/35*y^3*v^2-F(47)/108*x^2*z*v^2+F(83)/15*x*y*z*v^2-F(73)/50*y^2*z*v^2+13244*x*z^2*v^2+F(114)/83*y*z^2*v^2+F(41)/35*z^3*v^2-3222*x^2*u*v^2+13188*x*y*u*v^2+F(13)/98*y^2*u*v^2+F(101)/59*x*z*u*v^2+13485*y*z*u*v^2-10042*z^2*u*v^2+F(40)/111*x*u^2*v^2-F(93)/116*y*u^2*v^2-10831*z*u^2*v^2-10811*u^3*v^2+F(91)/125*x^2*v^3+958*x*y*v^3-5300*y^2*v^3+F(115)/49*x*z*v^3+1025*y*z*v^3+F(59)/11*z^2*v^3-F(59)/103*x*u*v^3-3823*y*u*v^3+6773*z*u*v^3-F(92)/107*u^2*v^3+F(126)/59*x*v^4-F(58)/65*y*v^4+F(39)/79*z*v^4+4753*u*v^4+F(29)/27*v^5,15754*x^4*y-F(1)/75*x^3*y^2-F(40)/107*x^2*y^3-11856*x*y^4+F(108)/47*y^5-15308*x^3*y*z+8313*x^2*y^2*z+F(101)/93*x*y^3*z+F(7)/31*y^4*z+F(123)/122*x^2*y*z^2+F(31)/101*x*y^2*z^2+F(81)/113*y^3*z^2-3573*x*y*z^3-2701*y^2*z^3-11969*y*z^4+11593*x^4*u-F(49)/114*x^3*y*u+6611*x^2*y^2*u+F(115)/99*x*y^3*u-F(91)/67*y^4*u+6924*x^3*z*u-2887*x^2*y*z*u+F(81)/40*x*y^2*z*u-F(18)/49*y^3*z*u-12448*x^2*z^2*u-F(8)/111*x*y*z^2*u-F(112)/9*y^2*z^2*u-12449*x*z^3*u-F(37)/29*y*z^3*u-F(103)/59*z^4*u+F(31)/115*x^3*u^2-11025*x^2*y*u^2+F(109)/116*x*y^2*u^2+9787*y^3*u^2-F(125)/16*x^2*z*u^2-F(14)/39*x*y*z*u^2+4229*y^2*z*u^2-5804*x*z^2*u^2-F(73)/51*y*z^2*u^2-15589*z^3*u^2-15161*x^2*u^3-F(18)/47*x*y*u^3+15427*y^2*u^3+F(13)/118*x*z*u^3+6254*y*z*u^3+11737*z^2*u^3+3090*x*u^4-F(50)/91*y*u^4-F(91)/113*z*u^4+F(87)/14*u^5-8837*x^4*v-F(47)/126*x^3*y*v-F(18)/37*x^2*y^2*v+9727*x*y^3*v+14559*y^4*v+F(124)/13*x^3*z*v-1443*x^2*y*z*v+14943*x*y^2*z*v-F(94)/89*y^3*z*v-14670*x^2*z^2*v-F(124)/57*x*y*z^2*v-2227*y^2*z^2*v+11067*x*z^3*v-F(76)/21*y*z^3*v+5167*z^4*v-F(109)/92*x^3*u*v-F(35)/27*x^2*y*u*v-F(13)/98*x*y^2*u*v-2551*y^3*u*v-F(116)/63*x^2*z*u*v+F(113)/18*x*y*z*u*v+14297*y^2*z*u*v-F(89)/123*x*z^2*u*v-F(55)/72*y*z^2*u*v-F(109)/64*z^3*u*v+F(8)/11*x^2*u^2*v+F(103)/111*x*y*u^2*v-F(18)/77*y^2*u^2*v+4509*x*z*u^2*v-7247*y*z*u^2*v-12895*z^2*u^2*v+9224*x*u^3*v-7019*y*u^3*v+F(81)/38*z*u^3*v+8199*u^4*v+F(83)/118*x^3*v^2+F(46)/45*x^2*y*v^2-11040*x*y^2*v^2+F(41)/93*y^3*v^2+10168*x^2*z*v^2-F(126)/121*x*y*z*v^2+2518*y^2*z*v^2+F(37)/89*x*z^2*v^2+F(17)/20*y*z^2*v^2+8565*z^3*v^2+F(123)/4*x^2*u*v^2+F(30)/89*x*y*u*v^2-F(64)/41*y^2*u*v^2-F(7)/33*x*z*u*v^2+13103*y*z*u*v^2+97*z^2*u*v^2-F(43)/20*x*u^2*v^2+F(57)/14*y*u^2*v^2+6540*z*u^2*v^2-8833*u^3*v^2+5404*x^2*v^3+5981*x*y*v^3-5805*y^2*v^3+F(79)/74*x*z*v^3+F(31)/36*y*z*v^3+14643*z^2*v^3-F(14)/39*x*u*v^3+F(73)/65*y*u*v^3-F(55)/16*z*u*v^3-F(34)/21*u^2*v^3+F(124)/63*x*v^4-F(114)/35*y*v^4+F(79)/76*z*v^4-F(29)/98*u*v^4+3883*v^5,15754*x^4*z-F(1)/75*x^3*y*z-F(40)/107*x^2*y^2*z-11856*x*y^3*z+F(108)/47*y^4*z-15308*x^3*z^2+8313*x^2*y*z^2+F(101)/93*x*y^2*z^2+F(7)/31*y^3*z^2+F(123)/122*x^2*z^3+F(31)/101*x*y*z^3+F(81)/113*y^2*z^3-3573*x*z^4-2701*y*z^4-11969*z^5-F(80)/21*x^4*u-F(2)/111*x^3*y*u+F(9)/25*x^2*y^2*u-101*x*y^3*u-F(113)/43*y^4*u-7762*x^3*z*u+F(59)/27*x^2*y*z*u-3493*x*y^2*z*u-F(121)/47*y^3*z*u-F(35)/113*x^2*z^2*u+6691*x*y*z^2*u-8876*y^2*z^2*u+F(28)/51*x*z^3*u+F(32)/33*y*z^3*u+F(91)/25*z^4*u+F(97)/11*x^3*u^2-9423*x^2*y*u^2+F(35)/19*x*y^2*u^2+F(119)/40*y^3*u^2-7838*x^2*z*u^2+F(29)/84*x*y*z*u^2-F(7)/103*y^2*z*u^2+14180*x*z^2*u^2+11606*y*z^2*u^2-F(11)/61*z^3*u^2-4229*x^2*u^3+14279*x*y*u^3-11646*y^2*u^3+6868*x*z*u^3-F(87)/110*y*z*u^3-F(37)/77*z^2*u^3+F(89)/58*x*u^4+9035*y*u^4-F(82)/65*z*u^4+15809*u^5-F(59)/95*x^4*v+F(53)/104*x^3*y*v-13540*x^2*y^2*v+F(111)/70*x*y^3*v+13577*y^4*v-F(83)/89*x^3*z*v-15712*x^2*y*z*v-14943*x*y^2*z*v+15920*y^3*z*v-11397*x^2*z^2*v-14885*x*y*z^2*v+F(101)/82*y^2*z^2*v-F(17)/98*x*z^3*v-851*y*z^3*v-F(7)/25*z^4*v-F(107)/106*x^3*u*v+F(116)/37*x^2*y*u*v+F(123)/115*x*y^2*u*v-F(115)/93*y^3*u*v-F(24)/109*x^2*z*u*v+5361*x*y*z*u*v-11011*y^2*z*u*v+F(67)/17*x*z^2*u*v+F(82)/49*y*z^2*u*v+F(91)/66*z^3*u*v+11442*x^2*u^2*v+670*x*y*u^2*v+3742*y^2*u^2*v-F(102)/67*x*z*u^2*v+F(27)/5*y*z*u^2*v+F(70)/17*z^2*u^2*v-F(92)/73*x*u^3*v+F(68)/33*y*u^3*v+F(41)/17*z*u^3*v+F(69)/64*u^4*v+5125*x^3*v^2-11510*x^2*y*v^2+15364*x*y^2*v^2+F(5)/112*y^3*v^2+F(113)/18*x^2*z*v^2-7672*x*y*z*v^2-F(117)/37*y^2*z*v^2+F(86)/53*x*z^2*v^2+3219*y*z^2*v^2-F(109)/104*z^3*v^2-1617*x^2*u*v^2-F(41)/122*x*y*u*v^2-F(37)/9*y^2*u*v^2-F(97)/108*x*z*u*v^2-3574*y*z*u*v^2-F(101)/111*z^2*u*v^2+4354*x*u^2*v^2+F(119)/20*y*u^2*v^2+F(124)/45*z*u^2*v^2-9265*u^3*v^2-F(86)/13*x^2*v^3-5835*x*y*v^3+F(37)/122*y^2*v^3+F(15)/53*x*z*v^3-F(76)/21*y*z*v^3+14121*z^2*v^3+14204*x*u*v^3-14484*y*u*v^3-8787*z*u*v^3+F(50)/47*u^2*v^3-8055*x*v^4-143*y*v^4+10153*z*v^4+F(79)/70*u*v^4+F(125)/41*v^5,-F(80)/21*x^5-F(2)/111*x^4*y+F(9)/25*x^3*y^2-101*x^2*y^3-F(113)/43*x*y^4+15685*x^4*z+F(53)/82*x^3*y*z+F(29)/4*x^2*y^2*z+F(18)/77*x*y^3*z-F(72)/73*y^4*z+6557*x^3*z^2-F(39)/115*x^2*y*z^2+F(87)/59*x*y^2*z^2-F(98)/51*y^3*z^2-7944*x^2*z^3+9486*x*y*z^3-F(79)/10*y^2*z^3+F(21)/110*x*z^4-15200*y*z^4+F(111)/110*z^5+F(97)/11*x^4*u-9423*x^3*y*u+F(35)/19*x^2*y^2*u+F(119)/40*x*y^3*u+F(125)/3*x^3*z*u+F(61)/89*x^2*y*z*u-3158*x*y^2*z*u-F(48)/7*y^3*z*u+F(71)/83*x^2*z^2*u+F(9)/109*x*y*z^2*u-F(61)/124*y^2*z^2*u+F(19)/63*x*z^3*u-F(25)/124*y*z^3*u-F(67)/107*z^4*u-4229*x^3*u^2+14279*x^2*y*u^2-11646*x*y^2*u^2-F(109)/19*x^2*z*u^2-4814*x*y*z*u^2-975*y^2*z*u^2+11333*x*z^2*u^2+F(25)/83*y*z^2*u^2+F(100)/39*z^3*u^2+F(89)/58*x^2*u^3+9035*x*y*u^3-14308*x*z*u^3+12350*y*z*u^3+F(96)/49*z^2*u^3+15809*x*u^4-F(111)/47*z*u^4-15528*x^4*v-F(39)/85*x^3*y*v-F(27)/88*x^2*y^2*v+F(22)/63*x*y^3*v+F(113)/115*y^4*v-F(22)/51*x^3*z*v+5390*x^2*y*z*v+10597*x*y^2*z*v+F(124)/109*y^3*z*v+4538*x^2*z^2*v+F(79)/73*x*y*z^2*v-11106*y^2*z^2*v+F(86)/51*x*z^3*v+6896*y*z^3*v-14009*z^4*v+1046*x^3*u*v-15818*x^2*y*u*v+6811*x*y^2*u*v+F(43)/20*y^3*u*v+F(82)/79*x^2*z*u*v-F(118)/89*x*y*z*u*v+F(123)/89*y^2*z*u*v+F(23)/106*x*z^2*u*v+F(71)/12*y*z^2*u*v-10781*z^3*u*v-13874*x^2*u^2*v-548*x*y*u^2*v+F(109)/84*y^2*u^2*v+F(4)/41*x*z*u^2*v+F(61)/46*y*z*u^2*v+F(27)/68*z^2*u^2*v-4119*x*u^3*v+F(111)/14*y*u^3*v+F(13)/16*z*u^3*v-F(101)/2*u^4*v+4315*x^3*v^2+F(91)/74*x^2*y*v^2-F(10)/19*x*y^2*v^2-2576*y^3*v^2-F(87)/50*x^2*z*v^2+F(96)/97*x*y*z*v^2+F(30)/19*y^2*z*v^2+F(3)/8*x*z^2*v^2-6105*y*z^2*v^2+F(29)/113*z^3*v^2+F(77)/85*x^2*u*v^2+2552*x*y*u*v^2-F(78)/77*y^2*u*v^2+318*x*z*u*v^2+1593*y*z*u*v^2+12109*z^2*u*v^2-F(79)/37*x*u^2*v^2+1401*y*u^2*v^2+F(93)/11*z*u^2*v^2+14194*u^3*v^2+F(59)/61*x^2*v^3+F(49)/99*x*y*v^3+F(94)/83*y^2*v^3-F(115)/18*x*z*v^3+F(79)/74*y*z*v^3+15263*z^2*v^3+8150*x*u*v^3+F(29)/15*y*u*v^3-15079*z*u*v^3+F(101)/70*u^2*v^3-F(40)/117*x*v^4+F(27)/53*y*v^4+F(111)/89*z*v^4+F(85)/21*u*v^4+F(11)/15*v^5,F(80)/21*x^4*y+F(2)/111*x^3*y^2-F(9)/25*x^2*y^3+101*x*y^4+F(113)/43*y^5+11593*x^4*z+F(87)/43*x^3*y*z+F(77)/78*x^2*y^2*z+F(76)/79*x*y^3*z+15504*y^4*z+6924*x^3*z^2-F(7)/107*x^2*y*z^2+F(76)/63*x*y^2*z^2+F(1)/37*y^3*z^2-12448*x^2*z^3-F(89)/116*x*y*z^3+3218*y^2*z^3-12449*x*z^4+3481*y*z^4-F(103)/59*z^5-F(97)/11*x^3*y*u+9423*x^2*y^2*u-F(35)/19*x*y^3*u-F(119)/40*y^4*u+F(31)/115*x^3*z*u+F(121)/10*x^2*y*z*u+F(109)/95*x*y^2*z*u+F(87)/71*y^3*z*u-F(125)/16*x^2*z^2*u-F(109)/91*x*y*z^2*u+F(72)/13*y^2*z^2*u-5804*x*z^3*u+F(39)/125*y*z^3*u-15589*z^4*u+4229*x^2*y*u^2-14279*x*y^2*u^2+11646*y^3*u^2-15161*x^2*z*u^2+F(1)/11*x*y*z*u^2+F(18)/43*y^2*z*u^2+F(13)/118*x*z^2*u^2-393*y*z^2*u^2+11737*z^3*u^2-F(89)/58*x*y*u^3-9035*y^2*u^3+3090*x*z*u^3-F(110)/43*y*z*u^3-F(91)/113*z^2*u^3-15809*y*u^4+F(87)/14*z*u^4+F(25)/104*x^4*v-F(113)/15*x^3*y*v-F(123)/58*x^2*y^2*v+958*x*y^3*v+9630*y^4*v-F(100)/119*x^3*z*v+7902*x^2*y*z*v-F(53)/115*x*y^2*z*v+F(97)/3*y^3*z*v+6165*x^2*z^2*v-F(62)/101*x*y*z^2*v-F(43)/33*y^2*z^2*v+15498*x*z^3*v-F(92)/97*y*z^3*v+F(102)/31*z^4*v-12228*x^3*u*v-F(89)/105*x^2*y*u*v-F(5)/101*x*y^2*u*v+F(75)/29*y^3*u*v-F(102)/5*x^2*z*u*v+884*x*y*z*u*v-6436*y^2*z*u*v+F(10)/41*x*z^2*u*v-F(28)/103*y*z^2*u*v+2010*z^3*u*v-8171*x^2*u^2*v-9580*x*y*u^2*v+7145*y^2*u^2*v-F(84)/13*x*z*u^2*v-F(101)/88*y*z*u^2*v-F(99)/62*z^2*u^2*v+714*x*u^3*v+14198*y*u^3*v-5533*z*u^3*v-F(88)/49*u^4*v+F(106)/93*x^3*v^2-10389*x^2*y*v^2-F(92)/3*x*y^2*v^2+11511*y^3*v^2+7017*x^2*z*v^2+3798*x*y*z*v^2-F(68)/83*y^2*z*v^2-12965*x*z^2*v^2+F(44)/19*y*z^2*v^2+F(79)/75*z^3*v^2+F(106)/101*x^2*u*v^2+F(43)/55*x*y*u*v^2-F(25)/78*y^2*u*v^2-14366*x*z*u*v^2-8748*y*z*u*v^2-F(7)/6*z^2*u*v^2+F(125)/117*x*u^2*v^2+2733*y*u^2*v^2+1963*z*u^2*v^2+F(43)/55*u^3*v^2+15609*x^2*v^3-F(44)/25*x*y*v^3+F(56)/111*y^2*v^3-F(35)/94*x*z*v^3+14919*y*z*v^3-11404*z^2*v^3-F(125)/27*x*u*v^3-F(38)/99*y*u*v^3+F(66)/65*z*u*v^3-F(119)/82*u^2*v^3+F(81)/119*x*v^4-6297*y*v^4+97*z*v^4+F(5)/24*u*v^4-F(77)/67*v^5,-11593*x^5-F(118)/37*x^4*y+F(11)/79*x^3*y^2-F(77)/108*x^2*y^3+699*x*y^4+F(72)/73*y^5-6924*x^4*z+F(66)/119*x^3*y*z+8115*x^2*y^2*z-15594*x*y^3*z+F(98)/51*y^4*z+12448*x^3*z^2-F(19)/42*x^2*y*z^2-F(115)/68*x*y^2*z^2+F(79)/10*y^3*z^2+12449*x^2*z^3-F(20)/31*x*y*z^3+15200*y^2*z^3+F(103)/59*x*z^4-F(111)/110*y*z^4-F(31)/115*x^4*u+13809*x^3*y*u-F(117)/37*x^2*y^2*u-1349*x*y^3*u+F(48)/7*y^4*u+F(125)/16*x^3*z*u-12198*x^2*y*z*u+F(121)/107*x*y^2*z*u+F(61)/124*y^3*z*u+5804*x^2*z^2*u-F(95)/82*x*y*z^2*u+F(25)/124*y^2*z^2*u+15589*x*z^3*u+F(67)/107*y*z^3*u+15161*x^3*u^2+F(101)/45*x^2*y*u^2-8578*x*y^2*u^2+975*y^3*u^2-F(13)/118*x^2*z*u^2-10940*x*y*z*u^2-F(25)/83*y^2*z*u^2-11737*x*z^2*u^2-F(100)/39*y*z^2*u^2-3090*x^2*u^3+175*x*y*u^3-12350*y^2*u^3+F(91)/113*x*z*u^3-F(96)/49*y*z*u^3-F(87)/14*x*u^4+F(111)/47*y*u^4-F(84)/61*x^4*v+F(71)/101*x^3*y*v-F(65)/106*x^2*y^2*v-F(97)/13*x*y^3*v+11056*y^4*v-F(104)/15*x^3*z*v-10189*x^2*y*z*v-9544*x*y^2*z*v+2701*y^3*z*v-F(7)/23*x^2*z^2*v+1217*x*y*z^2*v+F(52)/7*y^2*z^2*v+1530*x*z^3*v+F(103)/40*y*z^3*v+F(71)/90*z^4*v-F(49)/113*x^3*u*v-8447*x^2*y*u*v-F(92)/25*x*y^2*u*v-F(107)/54*y^3*u*v-F(27)/110*x^2*z*u*v+F(46)/125*x*y*z*u*v-F(85)/89*y^2*z*u*v-10491*x*z^2*u*v-249*y*z^2*u*v-8877*z^3*u*v+12257*x^2*u^2*v+F(74)/29*x*y*u^2*v-F(39)/23*y^2*u^2*v-F(108)/43*x*z*u^2*v+10256*y*z*u^2*v+F(50)/59*z^2*u^2*v-7404*x*u^3*v+15421*y*u^3*v-37*z*u^3*v+1052*u^4*v-9160*x^3*v^2-14576*x^2*y*v^2+F(74)/75*x*y^2*v^2+F(11)/58*y^3*v^2-8744*x^2*z*v^2-12344*x*y*z*v^2+13275*y^2*z*v^2+F(1)/100*x*z^2*v^2+9027*y*z^2*v^2+8092*z^3*v^2-11209*x^2*u*v^2-4105*x*y*u*v^2-F(82)/109*y^2*u*v^2-F(59)/108*x*z*u*v^2+4628*y*z*u*v^2-12528*z^2*u*v^2+F(26)/109*x*u^2*v^2+3618*y*u^2*v^2-F(89)/33*z*u^2*v^2+F(32)/27*u^3*v^2+F(100)/27*x^2*v^3-6761*x*y*v^3-F(62)/33*y^2*v^3+8818*x*z*v^3+F(41)/12*y*z*v^3-F(109)/116*z^2*v^3+1703*x*u*v^3+F(62)/105*y*u*v^3-F(41)/37*z*u*v^3+F(29)/19*u^2*v^3-F(61)/35*x*v^4-F(32)/11*y*v^4+10197*z*v^4-F(31)/83*u*v^4-F(61)/120*v^5,11164*x^5+F(63)/107*x^4*y+F(11)/54*x^3*y^2-15348*x^2*y^3+F(72)/91*x*y^4+F(35)/93*y^5-14454*x^4*z+F(101)/64*x^3*y*z-2869*x^2*y^2*z-13684*x*y^3*z+13591*y^4*z+144*x^3*z^2-F(18)/85*x^2*y*z^2-504*x*y^2*z^2-3776*y^3*z^2-F(91)/64*x^2*z^3-F(28)/69*x*y*z^3+5900*y^2*z^3-78*x*z^4-F(95)/7*y*z^4+F(71)/103*z^5+2411*x^4*u-F(45)/7*x^3*y*u+F(108)/31*x^2*y^2*u+F(65)/7*x*y^3*u-F(3)/110*y^4*u+1630*x^3*z*u-F(11)/64*x^2*y*z*u-11838*x*y^2*z*u+F(101)/10*y^3*z*u+F(105)/23*x^2*z^2*u+F(112)/125*x*y*z^2*u-F(17)/71*y^2*z^2*u+11594*x*z^3*u+5671*y*z^3*u-F(104)/67*z^4*u-6675*x^3*u^2-F(88)/119*x^2*y*u^2+F(79)/13*x*y^2*u^2-F(113)/43*y^3*u^2-F(97)/28*x^2*z*u^2+5399*x*y*z*u^2-1837*y^2*z*u^2+5868*x*z^2*u^2+F(40)/109*y*z^2*u^2-F(86)/67*z^3*u^2-14710*x^2*u^3+F(44)/25*x*y*u^3+15523*y^2*u^3-F(101)/104*x*z*u^3+F(71)/65*y*z*u^3+9563*z^2*u^3-F(19)/7*x*u^4-F(122)/55*y*u^4+13079*z*u^4-F(107)/77*x^4*v+11547*x^3*y*v+12041*x^2*y^2*v+8120*x*y^3*v+1894*y^4*v+F(83)/43*x^3*z*v+4275*x^2*y*z*v-12432*x*y^2*z*v+6871*y^3*z*v+10405*x^2*z^2*v-F(83)/27*x*y*z^2*v+149*y^2*z^2*v+7829*x*z^3*v-F(76)/59*y*z^3*v-F(59)/14*z^4*v+F(112)/47*x^3*u*v-5247*x^2*y*u*v+F(71)/52*x*y^2*u*v-F(111)/94*y^3*u*v+F(5)/117*x^2*z*u*v-10842*x*y*z*u*v+F(53)/64*y^2*z*u*v+F(118)/27*x*z^2*u*v+F(14)/51*y*z^2*u*v+F(14)/123*z^3*u*v-11575*x^2*u^2*v-7203*x*y*u^2*v-F(103)/10*y^2*u^2*v-F(96)/101*x*z*u^2*v+5353*y*z*u^2*v-F(38)/103*z^2*u^2*v+F(113)/82*x*u^3*v-11407*y*u^3*v-F(121)/56*z*u^3*v+F(83)/52*x^3*v^2-3380*x^2*y*v^2-F(5)/31*x*y^2*v^2+F(69)/94*y^3*v^2+F(10)/61*x^2*z*v^2-F(62)/81*x*y*z*v^2+555*y^2*z*v^2-F(44)/85*x*z^2*v^2-F(57)/70*y*z^2*v^2-2654*z^3*v^2+12944*x^2*u*v^2-7663*x*y*u*v^2-10811*y^2*u*v^2-8164*x*z*u*v^2+F(32)/31*y*z*u*v^2-F(7)/10*z^2*u*v^2+4940*x*u^2*v^2+5064*y*u^2*v^2-F(55)/103*z*u^2*v^2+F(63)/4*x^2*v^3-F(13)/98*x*y*v^3-6355*y^2*v^3+F(111)/70*x*z*v^3+F(79)/106*y*z*v^3-F(81)/112*z^2*v^3+F(48)/53*x*u*v^3-F(93)/43*y*u*v^3-F(29)/77*z*u*v^3-F(19)/25*x*v^4+F(50)/49*y*v^4-12609*z*v^4,F(24)/101*x^5+5109*x^4*y-6489*x^3*y^2+F(61)/46*x^2*y^3+F(74)/89*x*y^4+5139*x^4*z+14898*x^3*y*z+F(121)/8*x^2*y^2*z+F(47)/24*x*y^3*z-F(71)/81*y^4*z+3831*x^3*z^2+11670*x^2*y*z^2-F(19)/30*x*y^2*z^2-9225*y^3*z^2+8775*x^2*z^3-F(69)/47*x*y*z^3+1734*y^2*z^3+F(74)/77*x*z^4-F(66)/91*y*z^4+F(41)/113*z^5-4668*x^4*u+F(85)/96*x^3*y*u+F(29)/6*x^2*y^2*u+12219*x*y^3*u-F(35)/93*y^4*u+1577*x^3*z*u+F(24)/23*x^2*y*z*u-14347*x*y^2*z*u-F(25)/6*y^3*z*u+F(20)/59*x^2*z^2*u-F(35)/72*x*y*z^2*u+F(111)/109*y^2*z^2*u-F(63)/25*x*z^3*u+F(29)/110*y*z^3*u-8740*z^4*u-F(18)/71*x^3*u^2-12429*x^2*y*u^2+10355*x*y^2*u^2+F(3)/110*y^3*u^2-F(105)/43*x^2*z*u^2-14416*x*y*z*u^2+F(1)/60*y^2*z*u^2+F(122)/105*x*z^2*u^2+9553*y*z^2*u^2+2755*z^3*u^2+11588*x^2*u^3-F(121)/73*x*y*u^3+F(113)/43*y^2*u^3-8787*x*z*u^3-3852*y*z*u^3+9419*z^2*u^3+5799*x*u^4-15523*y*u^4+F(19)/40*z*u^4+F(122)/55*u^5+15888*x^4*v+F(3)/119*x^3*y*v+11969*x^2*y^2*v+1786*x*y^3*v+F(24)/65*x^3*z*v-F(103)/29*x^2*y*z*v+15286*x*y^2*z*v+F(65)/58*y^3*z*v+15534*x^2*z^2*v+15350*x*y*z^2*v+1252*y^2*z^2*v-F(96)/109*x*z^3*v+F(29)/27*y*z^3*v+13830*z^4*v-3725*x^3*u*v-F(103)/4*x^2*y*u*v-F(55)/116*x*y^2*u*v-1894*y^3*u*v+F(44)/29*x^2*z*u*v+F(72)/125*x*y*z*u*v+F(102)/61*y^2*z*u*v-F(28)/87*x*z^2*u*v-F(117)/5*y*z^2*u*v-14049*z^3*u*v-7347*x^2*u^2*v+15518*x*y*u^2*v+F(111)/94*y^2*u^2*v+F(19)/109*x*z*u^2*v-F(91)/33*y*z*u^2*v-F(7)/106*z^2*u^2*v-5112*x*u^3*v+F(103)/10*y*u^3*v-2363*z*u^3*v+11407*u^4*v-15244*x^3*v^2-8174*x^2*y*v^2+F(85)/13*x*y^2*v^2+F(30)/113*x^2*z*v^2-F(93)/110*x*y*z*v^2+F(7)/16*y^2*z*v^2-F(119)/110*x*z^2*v^2-F(103)/85*y*z^2*v^2-F(32)/45*z^3*v^2-F(13)/57*x^2*u*v^2-7019*x*y*u*v^2-F(69)/94*y^2*u*v^2+F(12)/23*x*z*u*v^2+9828*y*z*u*v^2+F(113)/65*z^2*u*v^2-F(102)/121*x*u^2*v^2+10811*y*u^2*v^2+F(73)/117*z*u^2*v^2-5064*u^3*v^2+F(79)/63*x^2*v^3-F(49)/10*x*y*v^3+F(90)/43*x*z*v^3-F(22)/39*y*z*v^3-5480*z^2*v^3+F(109)/70*x*u*v^3+6355*y*u*v^3+F(79)/47*z*u*v^3+F(93)/43*u^2*v^3-F(60)/31*x*v^4-F(84)/41*z*v^4-F(50)/49*u*v^4,-F(24)/101*x^4*y-5109*x^3*y^2+6489*x^2*y^3-F(61)/46*x*y^4-F(74)/89*y^5-14084*x^4*z+F(112)/87*x^3*y*z+6007*x^2*y^2*z+F(31)/30*x*y^3*z+F(27)/61*y^4*z+10580*x^3*z^2-14955*x^2*y*z^2-8356*x*y^2*z^2+F(113)/47*y^3*z^2+F(124)/41*x^2*z^3+4619*x*y*z^3-14746*y^2*z^3+2177*x*z^4+F(35)/102*y*z^4+2495*z^5-11164*x^4*u+11245*x^3*y*u+F(49)/57*x^2*y^2*u-11316*x*y^3*u+F(105)/97*y^4*u-F(83)/42*x^3*z*u+F(35)/51*x^2*y*z*u-F(40)/27*x*y^2*z*u+6162*y^3*z*u+F(24)/103*x^2*z^2*u+F(9)/95*x*y*z^2*u-F(124)/85*y^2*z^2*u-F(53)/80*x*z^3*u+F(112)/3*y*z^3*u-F(77)/76*z^4*u-2411*x^3*u^2-637*x^2*y*u^2-3054*x*y^2*u^2-1224*y^3*u^2+6780*x^2*z*u^2+12069*x*y*z*u^2-F(15)/68*y^2*z*u^2-F(47)/3*x*z^2*u^2+9634*y*z^2*u^2-12142*z^3*u^2+6675*x^2*u^3+F(16)/107*x*y*u^3-F(37)/16*y^2*u^3+9487*x*z*u^3-F(9)/20*y*z*u^3+F(115)/51*z^2*u^3+14710*x*u^4+F(93)/28*y*u^4+12699*z*u^4+F(19)/7*u^5-15888*x^3*y*v-F(3)/119*x^2*y^2*v-11969*x*y^3*v-1786*y^4*v+6033*x^3*z*v-2554*x^2*y*z*v+F(43)/41*x*y^2*z*v+2607*y^3*z*v-5127*x^2*z^2*v-12494*x*y*z^2*v-F(24)/29*y^2*z^2*v+F(43)/98*x*z^3*v+F(41)/63*y*z^3*v-F(73)/59*z^4*v+F(107)/77*x^3*u*v-F(89)/45*x^2*y*u*v+11978*x*y^2*u*v-F(43)/23*y^3*u*v+F(37)/67*x^2*z*u*v+5037*x*y*z*u*v-10419*y^2*z*u*v+F(88)/61*x*z^2*u*v-F(65)/41*y*z^2*u*v+F(81)/31*z^3*u*v-F(112)/47*x^2*u^2*v+12594*x*y*u^2*v+F(54)/35*y^2*u^2*v+F(91)/100*x*z*u^2*v+F(69)/121*y*z*u^2*v-13803*z^2*u^2*v+11575*x*u^3*v+12315*y*u^3*v+11392*z*u^3*v-F(113)/82*u^4*v+15244*x^2*y*v^2+8174*x*y^2*v^2-F(85)/13*y^3*v^2+F(8)/15*x^2*z*v^2-F(57)/74*x*y*z*v^2+F(9)/71*y^2*z*v^2-10*x*z^2*v^2-F(87)/109*y*z^2*v^2-F(116)/75*z^3*v^2-F(83)/52*x^2*u*v^2+574*x*y*u*v^2+12179*y^2*u*v^2+10743*x*z*u*v^2-11558*y*z*u*v^2+7254*z^2*u*v^2-12944*x*u^2*v^2+F(107)/46*y*u^2*v^2-F(8)/83*z*u^2*v^2-4940*u^3*v^2-F(79)/63*x*y*v^3+F(49)/10*y^2*v^3-F(122)/39*x*z*v^3-1180*y*z*v^3+F(87)/56*z^2*v^3-F(63)/4*x*u*v^3-132*y*u*v^3-F(43)/101*z*u*v^3-F(48)/53*u^2*v^3+F(60)/31*y*v^4+14109*z*v^4+F(19)/25*u*v^4,14084*x^5-5508*x^4*y+F(1)/101*x^3*y^2+7715*x^2*y^3+F(31)/69*x*y^4+F(71)/81*y^5-10580*x^4*z-F(76)/23*x^3*y*z-3314*x^2*y^2*z-F(1)/39*x*y^3*z+9225*y^4*z-F(124)/41*x^3*z^2-F(104)/43*x^2*y*z^2-4311*x*y^2*z^2-1734*y^3*z^2-2177*x^2*z^3-F(113)/121*x*y*z^3+F(66)/91*y^2*z^3-2495*x*z^4-F(41)/113*y*z^4-F(113)/7*x^4*u+F(47)/86*x^3*y*u-1973*x^2*y^2*u-10122*x*y^3*u+F(23)/31*y^4*u+F(9)/71*x^3*z*u+F(79)/54*x^2*y*z*u+3888*x*y^2*z*u+F(13)/123*y^3*z*u+F(21)/106*x^2*z^2*u+F(17)/110*x*y*z^2*u+5442*y^2*z^2*u+F(63)/118*x*z^3*u+F(105)/47*y*z^3*u-F(71)/103*z^4*u-8410*x^3*u^2+F(34)/65*x^2*y*u^2-F(87)/104*x*y^2*u^2-F(3)/53*y^3*u^2-3698*x^2*z*u^2+F(17)/20*x*y*z*u^2-F(101)/21*y^2*z*u^2+548*x*z^2*u^2-8426*y*z^2*u^2+F(104)/67*z^3*u^2+F(103)/23*x^2*u^3+4988*x*y*u^3+F(77)/45*y^2*u^3-12143*x*z*u^3-908*y*z*u^3+F(86)/67*z^2*u^3-F(15)/43*x*u^4+8919*y*u^4-9563*z*u^4-13079*u^5-6033*x^4*v-F(101)/105*x^3*y*v+F(107)/32*x^2*y^2*v+F(16)/59*x*y^3*v-F(65)/58*y^4*v+5127*x^3*z*v-3040*x^2*y*z*v+10023*x*y^2*z*v-1252*y^3*z*v-F(43)/98*x^2*z^2*v-11693*x*y*z^2*v-F(29)/27*y^2*z^2*v+F(73)/59*x*z^3*v-13830*y*z^3*v+11779*x^3*u*v-F(59)/34*x^2*y*u*v-11188*x*y^2*u*v-F(47)/125*y^3*u*v-9882*x^2*z*u*v-F(41)/112*x*y*z*u*v-12922*y^2*z*u*v+4552*x*z^2*u*v+1037*y*z^2*u*v+F(59)/14*z^3*u*v-F(49)/83*x^2*u^2*v-F(17)/62*x*y*u^2*v-F(11)/10*y^2*u^2*v+F(26)/31*x*z*u^2*v-9936*y*z*u^2*v-F(14)/123*z^2*u^2*v-2839*x*u^3*v-F(20)/107*y*u^3*v+F(38)/103*z*u^3*v+F(121)/56*u^4*v-F(8)/15*x^3*v^2+F(111)/38*x^2*y*v^2+F(35)/43*x*y^2*v^2-F(7)/16*y^3*v^2+10*x^2*z*v^2-988*x*y*z*v^2+F(103)/85*y^2*z*v^2+F(116)/75*x*z^2*v^2+F(32)/45*y*z^2*v^2+6039*x^2*u*v^2+F(91)/24*x*y*u*v^2+F(5)/114*y^2*u*v^2+2532*x*z*u*v^2+F(38)/73*y*z*u*v^2+2654*z^2*u*v^2-F(1)/11*x*u^2*v^2+F(27)/5*y*u^2*v^2+F(7)/10*z*u^2*v^2+F(55)/103*u^3*v^2+F(122)/39*x^2*v^3-F(67)/89*x*y*v^3+F(22)/39*y^2*v^3-F(87)/56*x*z*v^3+5480*y*z*v^3+15569*x*u*v^3+F(95)/117*y*u*v^3+F(81)/112*z*u*v^3+F(29)/77*u^2*v^3-14109*x*v^4+F(84)/41*y*v^4+12609*u*v^4,7943*x^5+F(82)/113*x^4*y-9790*x^3*y^2-397*x^2*y^3+F(76)/69*x*y^4-F(60)/71*x^4*z+8592*x^3*y*z-4546*x^2*y^2*z-8188*x*y^3*z-8271*x^3*z^2+55*x^2*y*z^2-13233*x*y^2*z^2-10140*x^2*z^3-F(81)/109*x*y*z^3+F(48)/13*x*z^4+7293*x^4*u-7707*x^3*y*u-F(61)/120*x^2*y^2*u-F(122)/47*x*y^3*u+7280*y^4*u-F(71)/111*x^3*z*u-F(84)/79*x^2*y*z*u-F(85)/99*x*y^2*z*u-4187*y^3*z*u+11220*x^2*z^2*u+11504*x*y*z^2*u-F(123)/64*y^2*z^2*u-1733*x*z^3*u+9769*y*z^3*u-F(119)/73*z^4*u+F(2)/13*x^3*u^2-3406*x^2*y*u^2-F(101)/73*x*y^2*u^2-F(99)/52*y^3*u^2-F(6)/71*x^2*z*u^2+F(18)/19*x*y*z*u^2-F(73)/61*y^2*z*u^2+F(61)/125*x*z^2*u^2+14730*y*z^2*u^2+11545*z^3*u^2+F(21)/4*x^2*u^3-4341*x*y*u^3-13552*y^2*u^3+14333*x*z*u^3-13829*y*z*u^3-12172*z^2*u^3-F(107)/62*x*u^4-F(45)/119*y*u^4+F(27)/89*z*u^4-13361*u^5-13463*x^4*v+F(115)/9*x^3*y*v+F(42)/13*x^2*y^2*v-F(113)/88*x*y^3*v+F(71)/81*y^4*v-F(126)/13*x^3*z*v+F(23)/50*x^2*y*z*v+F(38)/7*x*y^2*z*v+9225*y^3*z*v+6943*x^2*z^2*v+F(80)/81*x*y*z^2*v-1734*y^2*z^2*v-4282*x*z^3*v+F(66)/91*y*z^3*v-F(41)/113*z^4*v-2251*x^3*u*v+1870*x^2*y*u*v+F(116)/31*x*y^2*u*v+15785*y^3*u*v+F(55)/84*x^2*z*u*v-F(14)/69*x*y*z*u*v-6263*y^2*z*u*v+F(68)/47*x*z^2*u*v+F(26)/95*y*z^2*u*v-F(103)/126*z^3*u*v+F(107)/3*x^2*u^2*v+F(89)/44*x*y*u^2*v+F(119)/103*y^2*u^2*v+F(23)/18*x*z*u^2*v-15029*y*z*u^2*v-F(61)/47*z^2*u^2*v+F(100)/69*x*u^3*v-15754*y*u^3*v-F(94)/77*z*u^3*v-4030*u^4*v+F(103)/63*x^3*v^2+F(39)/94*x^2*y*v^2+F(121)/46*x*y^2*v^2-F(65)/58*y^3*v^2-F(116)/109*x^2*z*v^2-6793*x*y*z*v^2-1252*y^2*z*v^2+F(91)/43*x*z^2*v^2-F(29)/27*y*z^2*v^2-13830*z^3*v^2-F(29)/83*x^2*u*v^2-F(64)/99*x*y*u*v^2-F(75)/43*y^2*u*v^2+F(19)/104*x*z*u*v^2-F(21)/40*y*z*u*v^2+F(48)/89*z^2*u*v^2+F(89)/44*x*u^2*v^2-4315*y*u^2*v^2+F(32)/67*z*u^2*v^2+F(84)/79*u^3*v^2-F(27)/47*x^2*v^3-14889*x*y*v^3-F(7)/16*y^2*v^3+11244*x*z*v^3+F(103)/85*y*z*v^3+F(32)/45*z^2*v^3-6447*x*u*v^3-F(114)/85*y*u*v^3-2011*z*u*v^3-12700*u^2*v^3-F(93)/116*x*v^4+F(22)/39*y*v^4+5480*z*v^4-4699*u*v^4+F(84)/41*v^5,-7943*x^4*y-F(82)/113*x^3*y^2+9790*x^2*y^3+397*x*y^4-F(76)/69*y^5+F(60)/71*x^3*y*z-8592*x^2*y^2*z+4546*x*y^3*z+8188*y^4*z+8271*x^2*y*z^2-55*x*y^2*z^2+13233*y^3*z^2+10140*x*y*z^3+F(81)/109*y^2*z^3-F(48)/13*y*z^4-F(10)/29*x^4*u+F(41)/120*x^3*y*u+1672*x^2*y^2*u+12833*x*y^3*u+F(66)/49*y^4*u-14845*x^3*z*u+F(25)/122*x^2*y*z*u-F(9)/98*x*y^2*z*u+F(12)/83*y^3*z*u+8041*x^2*z^2*u+F(107)/5*x*y*z^2*u-F(43)/118*y^2*z^2*u-F(85)/53*x*z^3*u-F(14)/73*y*z^3*u+F(123)/80*z^4*u-6109*x^3*u^2+10482*x^2*y*u^2-F(113)/41*x*y^2*u^2-92*y^3*u^2-1748*x^2*z*u^2-F(67)/58*x*y*z*u^2-F(8)/7*y^2*z*u^2-10436*x*z^2*u^2-F(121)/53*y*z^2*u^2+F(111)/32*z^3*u^2+8172*x^2*u^3+11818*x*y*u^3-F(115)/54*y^2*u^3+F(89)/86*x*z*u^3-3436*y*z*u^3-7257*z^2*u^3+F(55)/108*x*u^4-14008*y*u^4-12021*z*u^4+4537*u^5+14084*x^4*v+7955*x^3*y*v+F(101)/61*x^2*y^2*v-11975*x*y^3*v+F(97)/59*y^4*v-10580*x^3*z*v+F(16)/103*x^2*y*z*v-F(94)/23*x*y^2*z*v+7963*y^3*z*v-F(124)/41*x^2*z^2*v+11654*x*y*z^2*v-12211*y^2*z^2*v-2177*x*z^3*v-F(91)/71*y*z^3*v-2495*z^4*v+F(87)/19*x^3*u*v-10880*x^2*y*u*v+15685*x*y^2*u*v+F(92)/105*y^3*u*v+F(22)/97*x^2*z*u*v+F(3)/109*x*y*z*u*v-6227*y^2*z*u*v+F(21)/104*x*z^2*u*v+6429*y*z^2*u*v-1465*z^3*u*v-F(89)/19*x^2*u^2*v+9787*x*y*u^2*v-1127*y^2*u^2*v+F(61)/48*x*z*u^2*v-F(109)/29*y*z*u^2*v+1212*z^2*u^2*v+F(46)/93*x*u^3*v-F(109)/64*y*u^3*v-7824*z*u^3*v-F(20)/17*u^4*v-6033*x^3*v^2-F(28)/89*x^2*y*v^2-F(92)/63*x*y^2*v^2+F(99)/8*y^3*v^2+5127*x^2*z*v^2+F(109)/30*x*y*z*v^2+F(17)/78*y^2*z*v^2-F(43)/98*x*z^2*v^2-15415*y*z^2*v^2+F(73)/59*z^3*v^2+F(61)/26*x^2*u*v^2+F(25)/102*x*y*u*v^2-F(109)/5*y^2*u*v^2+F(119)/97*x*z*u*v^2+F(17)/79*y*z*u*v^2+10198*z^2*u*v^2-F(1)/22*x*u^2*v^2+6873*y*u^2*v^2-F(67)/111*z*u^2*v^2+13010*u^3*v^2-F(8)/15*x^2*v^3+F(66)/65*x*y*v^3-F(45)/68*y^2*v^3+10*x*z*v^3-F(5)/34*y*z*v^3+F(116)/75*z^2*v^3+F(98)/47*x*u*v^3-15617*y*u*v^3+F(5)/69*z*u*v^3+F(34)/39*u^2*v^3+F(122)/39*x*v^4+1788*y*v^4-F(87)/56*z*v^4+2531*u*v^4-14109*v^5,-7943*x^4*z-F(82)/113*x^3*y*z+9790*x^2*y^2*z+397*x*y^3*z-F(76)/69*y^4*z+F(60)/71*x^3*z^2-8592*x^2*y*z^2+4546*x*y^2*z^2+8188*y^3*z^2+8271*x^2*z^3-55*x*y*z^3+13233*y^2*z^3+10140*x*z^4+F(81)/109*y*z^4-F(48)/13*z^5-F(3)/8*x^4*u-1555*x^3*y*u-8833*x^2*y^2*u-11526*x*y^3*u+F(29)/77*y^4*u-6045*x^3*z*u-F(59)/39*x^2*y*z*u-7750*x*y^2*z*u+3102*y^3*z*u+F(85)/88*x^2*z^2*u+13871*x*y*z^2*u-F(79)/2*y^2*z^2*u-8673*x*z^3*u-F(125)/102*y*z^3*u+F(17)/56*z^4*u-5373*x^3*u^2-F(124)/15*x^2*y*u^2-4395*x*y^2*u^2+F(50)/93*y^3*u^2-F(65)/51*x^2*z*u^2-748*x*y*z*u^2-F(88)/113*y^2*z*u^2-F(57)/89*x*z^2*u^2-F(111)/55*y*z^2*u^2-F(79)/50*z^3*u^2-12256*x^2*u^3-F(57)/46*x*y*u^3-15702*y^2*u^3+F(55)/102*x*z*u^3+F(113)/126*y*z*u^3-F(74)/27*z^2*u^3+F(10)/47*x*u^4-F(107)/66*y*u^4-15044*z*u^4+F(69)/95*u^5-F(24)/101*x^4*v-5109*x^3*y*v+6489*x^2*y^2*v-F(61)/46*x*y^3*v-F(74)/89*y^4*v+8324*x^3*z*v+F(125)/123*x^2*y*z*v+8287*x*y^2*z*v-7029*y^3*z*v-8743*x^2*z^2*v-9751*x*y*z^2*v+F(90)/13*y^2*z^2*v-15718*x*z^3*v+5034*y*z^3*v-14415*z^4*v-15263*x^3*u*v-242*x^2*y*u*v+F(64)/51*x*y^2*u*v-F(61)/27*y^3*u*v+F(68)/41*x^2*z*u*v+5370*x*y*z*u*v+15145*y^2*z*u*v-1004*x*z^2*u*v+F(11)/47*y*z^2*u*v+8353*z^3*u*v+F(39)/83*x^2*u^2*v+F(107)/51*x*y*u^2*v+F(87)/107*y^2*u^2*v-14459*x*z*u^2*v+10975*y*z*u^2*v+F(105)/26*z^2*u^2*v+F(9)/65*x*u^3*v+F(59)/20*y*u^3*v+11534*z*u^3*v+15465*u^4*v-15888*x^3*v^2-F(3)/119*x^2*y*v^2-11969*x*y^2*v^2-1786*y^3*v^2-11900*x^2*z*v^2-12237*x*y*z*v^2-F(67)/116*y^2*z*v^2+F(19)/14*x*z^2*v^2-F(109)/86*y*z^2*v^2-F(11)/31*z^3*v^2+F(16)/67*x^2*u*v^2+7193*x*y*u*v^2+6773*y^2*u*v^2-F(54)/109*x*z*u*v^2+8396*y*z*u*v^2+F(92)/83*z^2*u*v^2+F(30)/83*x*u^2*v^2-F(4)/51*y*u^2*v^2+F(94)/33*z*u^2*v^2-F(107)/30*u^3*v^2+15244*x^2*v^3+8174*x*y*v^3-F(85)/13*y^2*v^3+7855*x*z*v^3+F(27)/61*y*z*v^3+F(16)/47*z^2*v^3-F(58)/13*x*u*v^3+9324*y*u*v^3+6526*z*u*v^3-F(109)/77*u^2*v^3-F(79)/63*x*v^4+F(49)/10*y*v^4+608*z*v^4+F(10)/47*u*v^4+F(60)/31*v^5,-F(3)/8*x^5-1555*x^4*y-8833*x^3*y^2-11526*x^2*y^3+F(29)/77*x*y^4+F(123)/77*x^4*z+5416*x^3*y*z-4818*x^2*y^2*z+9906*x*y^3*z+7280*y^4*z+1949*x^3*z^2+5366*x^2*y*z^2+F(3)/88*x*y^2*z^2-4187*y^3*z^2-F(108)/113*x^2*z^3-F(36)/25*x*y*z^3-F(123)/64*y^2*z^3+12549*x*z^4+9769*y*z^4-F(119)/73*z^5-5373*x^4*u-F(124)/15*x^3*y*u-4395*x^2*y^2*u+F(50)/93*x*y^3*u+F(17)/114*x^3*z*u+F(52)/77*x^2*y*z*u+F(13)/46*x*y^2*z*u-F(99)/52*y^3*z*u+12418*x^2*z^2*u+F(104)/103*x*y*z^2*u-F(73)/61*y^2*z^2*u+6781*x*z^3*u+14730*y*z^3*u+11545*z^4*u-12256*x^3*u^2-F(57)/46*x^2*y*u^2-15702*x*y^2*u^2-F(71)/69*x^2*z*u^2-4594*x*y*z*u^2-13552*y^2*z*u^2+8406*x*z^2*u^2-13829*y*z^2*u^2-12172*z^3*u^2+F(10)/47*x^2*u^3-F(107)/66*x*y*u^3-F(64)/51*x*z*u^3-F(45)/119*y*z*u^3+F(27)/89*z^2*u^3+F(69)/95*x*u^4-13361*z*u^4+12060*x^4*v-F(75)/86*x^3*y*v+947*x^2*y^2*v-13850*x*y^3*v-F(35)/93*y^4*v+F(60)/109*x^3*z*v-8059*x^2*y*z*v+F(22)/103*x*y^2*z*v+10449*y^3*z*v-14991*x^2*z^2*v-F(31)/32*x*y*z^2*v+F(65)/44*y^2*z^2*v+F(61)/28*x*z^3*v-8311*y*z^3*v+13856*z^4*v+F(82)/103*x^3*u*v-F(103)/15*x^2*y*u*v+7366*x*y^2*u*v+F(3)/110*y^3*u*v-7234*x^2*z*u*v-F(122)/29*x*y*z*u*v+F(25)/92*y^2*z*u*v-F(113)/32*x*z^2*u*v-F(7)/111*y*z^2*u*v+6157*z^3*u*v-14989*x^2*u^2*v+F(52)/125*x*y*u^2*v+F(113)/43*y^2*u^2*v+4603*x*z*u^2*v+F(43)/31*y*z*u^2*v+F(17)/33*z^2*u^2*v-10727*x*u^3*v-15523*y*u^3*v+4768*z*u^3*v+F(122)/55*u^4*v+12032*x^3*v^2+15165*x^2*y*v^2-F(109)/28*x*y^2*v^2-1894*y^3*v^2+3577*x^2*z*v^2-6360*x*y*z*v^2+9757*y^2*z*v^2-15260*x*z^2*v^2-10421*y*z^2*v^2+F(32)/95*z^3*v^2-2336*x^2*u*v^2-F(7)/23*x*y*u*v^2+F(111)/94*y^2*u*v^2-F(62)/19*x*z*u*v^2+F(71)/121*y*z*u*v^2+10086*z^2*u*v^2+F(113)/109*x*u^2*v^2+F(103)/10*y*u^2*v^2-F(34)/125*z*u^2*v^2+11407*u^3*v^2-F(76)/7*x^2*v^3-F(73)/111*x*y*v^3-F(69)/94*y^2*v^3+F(120)/19*x*z*v^3+1923*y*z*v^3-15790*z^2*v^3+1924*x*u*v^3+10811*y*u*v^3-F(37)/34*z*u*v^3-5064*u^2*v^3+2598*x*v^4+6355*y*v^4-3336*z*v^4+F(93)/43*u*v^4-F(50)/49*v^5,F(3)/8*x^4*y+1555*x^3*y^2+8833*x^2*y^3+11526*x*y^4-F(29)/77*y^5-F(10)/29*x^4*z-2219*x^3*y*z-11451*x^2*y^2*z-11408*x*y^3*z+4081*y^4*z-14845*x^3*z^2-F(44)/3*x^2*y*z^2-6363*x*y^2*z^2-F(87)/61*y^3*z^2+8041*x^2*z^3-F(23)/39*x*y*z^3-F(21)/25*y^2*z^3-F(85)/53*x*z^4+F(21)/5*y*z^4+F(123)/80*z^5+5373*x^3*y*u+F(124)/15*x^2*y^2*u+4395*x*y^3*u-F(50)/93*y^4*u-6109*x^3*z*u+9856*x^2*y*z*u+11669*x*y^2*z*u-12831*y^3*z*u-1748*x^2*z^2*u-F(21)/101*x*y*z^2*u-10552*y^2*z^2*u-10436*x*z^3*u-F(28)/57*y*z^3*u+F(111)/32*z^4*u+12256*x^2*y*u^2+F(57)/46*x*y^2*u^2+15702*y^3*u^2+8172*x^2*z*u^2+9622*x*y*z*u^2+3213*y^2*z*u^2+F(89)/86*x*z^2*u^2+2491*y*z^2*u^2-7257*z^3*u^2-F(10)/47*x*y*u^3+F(107)/66*y^2*u^3+F(55)/108*x*z*u^3+F(125)/31*y*z*u^3-12021*z^2*u^3-F(69)/95*y*u^4+4537*z*u^4-11164*x^4*v+F(41)/35*x^3*y*v+F(71)/65*x^2*y^2*v+F(101)/20*x*y^3*v+F(28)/23*y^4*v+F(101)/8*x^3*z*v+F(41)/90*x^2*y*z*v+F(71)/125*x*y^2*z*v-15685*y^3*z*v+F(119)/29*x^2*z^2*v+F(111)/101*x*y*z^2*v+4806*y^2*z^2*v+F(43)/107*x*z^3*v+F(80)/113*y*z^3*v+F(29)/33*z^4*v-2411*x^3*u*v+2446*x^2*y*u*v-F(17)/117*x*y^2*u*v+1765*y^3*u*v-F(69)/40*x^2*z*u*v+F(33)/37*x*y*z*u*v-F(61)/83*y^2*z*u*v+3318*x*z^2*u*v+F(32)/61*y*z^2*u*v+F(31)/120*z^3*u*v+6675*x^2*u^2*v-3022*x*y*u^2*v-F(79)/91*y^2*u^2*v-F(51)/91*x*z*u^2*v-15635*y*z*u^2*v-F(65)/62*z^2*u^2*v+14710*x*u^3*v+F(67)/105*y*u^3*v+10816*z*u^3*v+F(19)/7*u^4*v+F(107)/77*x^3*v^2+8412*x^2*y*v^2+4785*x*y^2*v^2-F(19)/116*y^3*v^2-F(87)/8*x^2*z*v^2+14467*x*y*z*v^2+6756*y^2*z*v^2+F(5)/12*x*z^2*v^2+F(29)/86*y*z^2*v^2-F(79)/44*z^3*v^2-F(112)/47*x^2*u*v^2+7583*x*y*u*v^2-12974*y^2*u*v^2-F(61)/64*x*z*u*v^2+F(59)/113*y*z*u*v^2+2336*z^2*u*v^2+11575*x*u^2*v^2+4854*y*u^2*v^2+F(123)/59*z*u^2*v^2-F(113)/82*u^3*v^2-F(83)/52*x^2*v^3+7961*x*y*v^3-F(75)/56*y^2*v^3-13078*x*z*v^3-1710*y*z*v^3+13745*z^2*v^3-12944*x*u*v^3-F(116)/39*y*u*v^3-187*z*u*v^3-4940*u^2*v^3-F(63)/4*x*v^4+11439*y*v^4+F(71)/73*z*v^4-F(48)/53*u*v^4+F(19)/25*v^5,F(10)/29*x^5+F(52)/33*x^4*y-F(55)/53*x^3*y^2-F(91)/69*x^2*y^3-13987*x*y^4-7280*y^5+14845*x^4*z-12598*x^3*y*z-F(87)/32*x^2*y^2*z+F(57)/118*x*y^3*z+4187*y^4*z-8041*x^3*z^2+1555*x^2*y*z^2+F(57)/25*x*y^2*z^2+F(123)/64*y^3*z^2+F(85)/53*x^2*z^3-F(75)/26*x*y*z^3-9769*y^2*z^3-F(123)/80*x*z^4+F(119)/73*y*z^4+6109*x^4*u-12943*x^3*y*u-7515*x^2*y^2*u-F(58)/51*x*y^3*u+F(99)/52*y^4*u+1748*x^3*z*u+F(50)/9*x^2*y*z*u+2165*x*y^2*z*u+F(73)/61*y^3*z*u+10436*x^2*z^2*u+F(56)/85*x*y*z^2*u-14730*y^2*z^2*u-F(111)/32*x*z^3*u-11545*y*z^3*u-8172*x^3*u^2-F(45)/92*x^2*y*u^2+1381*x*y^2*u^2+13552*y^3*u^2-F(89)/86*x^2*z*u^2+F(94)/91*x*y*z*u^2+13829*y^2*z*u^2+7257*x*z^2*u^2+12172*y*z^2*u^2-F(55)/108*x^2*u^3+5238*x*y*u^3+F(45)/119*y^2*u^3+12021*x*z*u^3-F(27)/89*y*z*u^3-4537*x*u^4+13361*y*u^4-F(102)/121*x^4*v-F(4)/93*x^3*y*v-F(79)/77*x^2*y^2*v+4832*x*y^3*v+7951*y^4*v+F(87)/59*x^3*z*v-10435*x^2*y*z*v-F(86)/83*x*y^2*z*v-F(81)/83*y^3*z*v+11068*x^2*z^2*v-1907*x*y*z^2*v+2411*y^2*z^2*v+F(67)/114*x*z^3*v-132*y*z^3*v-F(71)/103*z^4*v+15167*x^3*u*v-F(24)/107*x^2*y*u*v+F(21)/74*x*y^2*u*v-F(119)/45*y^3*u*v+3632*x^2*z*u*v+F(18)/125*x*y*z*u*v-1733*y^2*z*u*v+F(119)/48*x*z^2*u*v+F(72)/119*y*z^2*u*v+F(104)/67*z^3*u*v+7826*x^2*u^2*v+5633*x*y*u^2*v-10548*y^2*u^2*v-4319*x*z*u^2*v-F(53)/50*y*z*u^2*v+F(86)/67*z^2*u^2*v+F(39)/100*x*u^3*v+F(11)/42*y*u^3*v-9563*z*u^3*v-13079*u^4*v-F(23)/91*x^3*v^2+F(13)/43*x^2*y*v^2-F(22)/101*x*y^2*v^2+15363*y^3*v^2+F(8)/31*x^2*z*v^2-10928*x*y*z*v^2-F(37)/109*y^2*z*v^2-F(9)/17*x*z^2*v^2-F(39)/61*y*z^2*v^2+F(59)/14*z^3*v^2+F(50)/69*x^2*u*v^2-F(43)/27*x*y*u*v^2+7513*y^2*u*v^2+F(88)/3*x*z*u*v^2-F(91)/115*y*z*u*v^2-F(14)/123*z^2*u*v^2-F(27)/109*x*u^2*v^2+F(113)/64*y*u^2*v^2+F(38)/103*z*u^2*v^2+F(121)/56*u^3*v^2+F(26)/15*x^2*v^3-F(2)/27*x*y*v^3-2478*y^2*v^3-3959*x*z*v^3+F(120)/53*y*z*v^3+2654*z^2*v^3+8351*x*u*v^3-91*y*u*v^3+F(7)/10*z*u*v^3+F(55)/103*u^2*v^3+F(125)/27*x*v^4+F(103)/124*y*v^4+F(81)/112*z*v^4+F(29)/77*u*v^4+12609*v^5,-4843*x^5+F(71)/109*x^4*y+F(33)/95*x^3*y^2+F(101)/122*x^2*y^3+14469*x*y^4+F(42)/79*y^5+6803*x^4*z+F(71)/49*x^3*y*z-F(23)/107*x^2*y^2*z+F(64)/89*x*y^3*z-1588*y^4*z+F(43)/68*x^3*z^2+F(41)/47*x^2*y*z^2+F(7)/88*x*y^2*z^2+F(28)/51*y^3*z^2-F(118)/13*x^2*z^3-F(26)/109*x*y*z^3+2529*y^2*z^3-F(115)/124*x*z^4-510*y*z^4+112*z^5+F(118)/37*x^4*u+F(124)/47*x^3*y*u-F(93)/95*x^2*y^2*u-F(43)/28*x*y^3*u-F(4)/97*y^4*u+F(92)/101*x^3*z*u+9174*x^2*y*z*u-1908*x*y^2*z*u+F(81)/119*y^3*z*u-F(33)/37*x^2*z^2*u+F(21)/68*x*y*z^2*u-F(38)/75*y^2*z^2*u-F(29)/113*x*z^3*u+F(67)/49*y*z^3*u-7699*z^4*u+F(19)/35*x^3*u^2-11378*x^2*y*u^2-F(125)/92*x*y^2*u^2-F(81)/59*y^3*u^2-88*x^2*z*u^2+F(67)/24*x*y*z*u^2-3474*y^2*z*u^2+11546*x*z^2*u^2+F(110)/93*y*z^2*u^2+558*z^3*u^2-F(65)/124*x^2*u^3-7159*x*y*u^3+1934*y^2*u^3-F(20)/13*x*z*u^3-F(32)/43*y*z*u^3+2269*z^2*u^3+F(16)/3*x*u^4+F(41)/2*y*u^4+15473*z*u^4+F(3)/37*x^4*v+F(92)/57*x^3*y*v+F(2)/115*x^2*y^2*v+F(103)/5*x*y^3*v-F(17)/108*y^4*v+F(72)/37*x^3*z*v+8146*x^2*y*z*v+152*x*y^2*z*v-39*y^3*z*v+F(67)/25*x^2*z^2*v-15829*x*y*z^2*v+9715*y^2*z^2*v+F(43)/55*x*z^3*v+F(117)/61*y*z^3*v-F(50)/123*z^4*v+6924*x^3*u*v+10730*x^2*y*u*v+F(67)/74*x*y^2*u*v-F(51)/29*y^3*u*v-F(31)/70*x^2*z*u*v+F(119)/32*x*y*z*u*v-F(51)/97*y^2*z*u*v-1408*x*z^2*u*v+F(82)/19*y*z^2*u*v+F(109)/2*z^3*u*v+F(54)/37*x^2*u^2*v+1865*x*y*u^2*v+F(68)/91*y^2*u^2*v-F(5)/37*x*z*u^2*v+1704*y*z*u^2*v+1514*z^2*u^2*v+1240*x*u^3*v+F(47)/45*y*u^3*v-F(74)/19*z*u^3*v+12847*x^3*v^2+F(11)/116*x^2*y*v^2+F(95)/9*x*y^2*v^2-F(21)/107*y^3*v^2-F(117)/79*x^2*z*v^2-F(61)/12*x*y*z*v^2+F(31)/94*y^2*z*v^2-8337*x*z^2*v^2+F(34)/59*y*z^2*v^2+4806*z^3*v^2-5474*x^2*u*v^2-F(105)/34*x*y*u*v^2+13025*y^2*u*v^2-4238*x*z*u*v^2+F(113)/27*y*z*u*v^2-7059*z^2*u*v^2-6224*x*u^2*v^2+F(73)/112*y*u^2*v^2-F(61)/94*z*u^2*v^2-8052*x^2*v^3+12584*x*y*v^3-766*y^2*v^3-7248*x*z*v^3+5137*y*z*v^3-15898*z^2*v^3+21*x*u*v^3+2330*y*u*v^3-4420*z*u*v^3+F(61)/31*x*v^4-676*y*v^4-15451*z*v^4,F(106)/115*x^5+F(29)/36*x^4*y+F(97)/84*x^3*y^2+3749*x^2*y^3-3803*x*y^4-F(65)/77*x^4*z+F(116)/25*x^3*y*z+F(119)/78*x^2*y^2*z+F(122)/3*x*y^3*z+8660*y^4*z-F(109)/55*x^3*z^2+F(45)/44*x^2*y*z^2+12238*x*y^2*z^2-F(67)/114*y^3*z^2-F(105)/4*x^2*z^3-F(56)/123*x*y*z^3-F(86)/41*y^2*z^3+3127*x*z^4+F(104)/47*y*z^4+F(97)/53*z^5-F(87)/83*x^4*u+2199*x^3*y*u-F(37)/36*x^2*y^2*u+6447*x*y^3*u-F(42)/79*y^4*u-10962*x^3*z*u+11215*x^2*y*z*u+F(87)/70*x*y^2*z*u-4214*y^3*z*u+151*x^2*z^2*u+F(61)/107*x*y*z^2*u-10936*y^2*z^2*u+F(84)/125*x*z^3*u-11138*y*z^3*u+F(7)/104*z^4*u-F(71)/34*x^3*u^2+F(72)/41*x^2*y*u^2+5989*x*y^2*u^2+F(4)/97*y^3*u^2-10847*x^2*z*u^2-F(3)/58*x*y*z*u^2+F(99)/59*y^2*z*u^2+F(6)/13*x*z^2*u^2+F(99)/101*y*z^2*u^2-F(5)/3*z^3*u^2+F(125)/116*x^2*u^3+F(97)/74*x*y*u^3+F(81)/59*y^2*u^3-9764*x*z*u^3+F(61)/7*y*z*u^3+5523*z^2*u^3-F(73)/44*x*u^4-1934*y*u^4+F(97)/95*z*u^4-F(41)/2*u^5+F(83)/66*x^4*v+203*x^3*y*v+F(77)/50*x^2*y^2*v+F(125)/69*x*y^3*v-10466*x^3*z*v+5673*x^2*y*z*v-F(66)/91*x*y^2*z*v+F(14)/123*y^3*z*v-F(117)/100*x^2*z^2*v-F(75)/47*x*y*z^2*v+2123*y^2*z^2*v+F(12)/77*x*z^3*v-12758*y*z^3*v+F(44)/5*z^4*v+F(107)/5*x^3*u*v+5897*x^2*y*u*v-F(93)/103*x*y^2*u*v+F(17)/108*y^3*u*v+F(50)/39*x^2*z*u*v+F(1)/103*x*y*z*u*v+216*y^2*z*u*v+F(25)/33*x*z^2*u*v-F(123)/107*y*z^2*u*v-F(93)/46*z^3*u*v+F(29)/81*x^2*u^2*v-F(9)/70*x*y*u^2*v+F(51)/29*y^2*u^2*v-12080*x*z*u^2*v+F(108)/119*y*z*u^2*v-8952*z^2*u^2*v-F(49)/10*x*u^3*v-F(68)/91*y*u^3*v-6845*z*u^3*v-F(47)/45*u^4*v-12988*x^3*v^2-F(23)/96*x^2*y*v^2+F(55)/92*x*y^2*v^2-F(122)/19*x^2*z*v^2+F(124)/65*x*y*z*v^2-2015*y^2*z*v^2+15119*x*z^2*v^2-F(52)/43*y*z^2*v^2-11462*z^3*v^2+F(101)/110*x^2*u*v^2+F(11)/20*x*y*u*v^2+F(21)/107*y^2*u*v^2+13796*x*z*u*v^2-F(8)/45*y*z*u*v^2+7623*z^2*u*v^2+F(20)/57*x*u^2*v^2-13025*y*u^2*v^2+F(107)/2*z*u^2*v^2-F(73)/112*u^3*v^2+F(25)/122*x^2*v^3+3572*x*y*v^3-12855*x*z*v^3+F(81)/98*y*z*v^3-2432*z^2*v^3+7941*x*u*v^3+766*y*u*v^3-F(42)/55*z*u*v^3-2330*u^2*v^3+9252*x*v^4+F(114)/65*z*v^4+676*u*v^4,-F(106)/115*x^4*y-F(29)/36*x^3*y^2-F(97)/84*x^2*y^3-3749*x*y^4+3803*y^5-F(108)/19*x^4*z-F(61)/8*x^3*y*z-1222*x^2*y^2*z-F(103)/81*x*y^3*z+F(29)/39*y^4*z+1285*x^3*z^2-9387*x^2*y*z^2+7558*x*y^2*z^2-F(107)/51*y^3*z^2+F(93)/92*x^2*z^3+F(38)/99*x*y*z^3-F(3)/67*y^2*z^3+6580*x*z^4+12425*y*z^4-F(90)/121*z^5+4843*x^4*u+8045*x^3*y*u+F(85)/24*x^2*y^2*u+F(103)/78*x*y^3*u+F(31)/26*y^4*u+F(7)/44*x^3*z*u-F(109)/41*x^2*y*z*u+1193*x*y^2*z*u-F(21)/20*y^3*z*u-7009*x^2*z^2*u+F(110)/107*x*y*z^2*u+F(13)/38*y^2*z^2*u-7352*x*z^3*u-F(66)/61*y*z^3*u-F(53)/104*z^4*u-F(118)/37*x^3*u^2+3663*x^2*y*u^2+F(96)/35*x*y^2*u^2-7130*y^3*u^2-F(81)/41*x^2*z*u^2+F(1)/6*x*y*z*u^2-5800*y^2*z*u^2-F(48)/17*x*z^2*u^2+F(83)/28*y*z^2*u^2-F(43)/99*z^3*u^2-F(19)/35*x^2*u^3+F(13)/16*x*y*u^3+F(125)/59*y^2*u^3-11233*x*z*u^3+3782*y*z*u^3+6216*z^2*u^3+F(65)/124*x*u^4-F(77)/80*y*u^4-F(61)/111*z*u^4-F(16)/3*u^5-F(83)/66*x^3*y*v-203*x^2*y^2*v-F(77)/50*x*y^3*v-F(125)/69*y^4*v-4107*x^3*z*v+F(12)/65*x^2*y*z*v+6828*x*y^2*z*v-3572*y^3*z*v+F(121)/21*x^2*z^2*v-F(43)/10*x*y*z^2*v+4938*y^2*z^2*v-F(107)/19*x*z^3*v-F(105)/97*y*z^3*v-F(75)/121*z^4*v-F(3)/37*x^3*u*v-F(117)/88*x^2*y*u*v+223*x*y^2*u*v+F(9)/91*y^3*u*v-8705*x^2*z*u*v-3707*x*y*z*u*v-1737*y^2*z*u*v+F(126)/53*x*z^2*u*v-F(13)/103*y*z^2*u*v-15610*z^3*u*v-6924*x^2*u^2*v+F(97)/40*x*y*u^2*v-F(55)/39*y^2*u^2*v-F(121)/86*x*z*u^2*v-4206*y*z*u^2*v+F(35)/108*z^2*u^2*v-F(54)/37*x*u^3*v+1339*y*u^3*v+6014*z*u^3*v-1240*u^4*v+12988*x^2*y*v^2+F(23)/96*x*y^2*v^2-F(55)/92*y^3*v^2+F(52)/73*x^2*z*v^2+6521*x*y*z*v^2+F(69)/107*y^2*z*v^2+F(33)/35*x*z^2*v^2-F(79)/31*y*z^2*v^2-F(50)/17*z^3*v^2-12847*x^2*u*v^2-F(113)/67*x*y*u*v^2+F(89)/56*y^2*u*v^2-F(65)/56*x*z*u*v^2+9239*y*z*u*v^2+F(103)/120*z^2*u*v^2+5474*x*u^2*v^2-F(109)/123*y*u^2*v^2-F(121)/108*z*u^2*v^2+6224*u^3*v^2-F(25)/122*x*y*v^3-3572*y^2*v^3+4843*x*z*v^3+F(77)/74*y*z*v^3-F(26)/19*z^2*v^3+8052*x*u*v^3+11466*y*u*v^3-F(99)/73*z*u*v^3-21*u^2*v^3-9252*y*v^4-F(107)/34*z*v^4-F(61)/31*u*v^4,F(108)/19*x^5-F(31)/27*x^4*y+2497*x^3*y^2-3570*x^2*y^3+4060*x*y^4-8660*y^5-1285*x^4*z-F(45)/77*x^3*y*z-F(77)/86*x^2*y^2*z-14745*x*y^3*z+F(67)/114*y^4*z-F(93)/92*x^3*z^2-13950*x^2*y*z^2+F(125)/118*x*y^2*z^2+F(86)/41*y^3*z^2-6580*x^2*z^3-F(59)/72*x*y*z^3-F(104)/47*y^2*z^3+F(90)/121*x*z^4-F(97)/53*y*z^4+5557*x^4*u+1807*x^3*y*u+7026*x^2*y^2*u-F(11)/74*x*y^3*u+5802*y^4*u-F(57)/40*x^3*z*u-F(44)/81*x^2*y*z*u+F(1)/63*x*y^2*z*u+F(62)/21*y^3*z*u-7404*x^2*z^2*u-F(81)/88*x*y*z^2*u-F(103)/26*y^2*z^2*u-8165*x*z^3*u-F(107)/39*y*z^3*u-112*z^4*u+15506*x^3*u^2-F(101)/35*x^2*y*u^2-12700*x*y^2*u^2-5944*y^3*u^2+2445*x^2*z*u^2+F(35)/46*x*y*z*u^2+3830*y^2*z*u^2+3275*x*z^2*u^2-12622*y*z^2*u^2+7699*z^3*u^2+F(72)/65*x^2*u^3+12644*x*y*u^3+F(51)/62*y^2*u^3+F(97)/9*x*z*u^3+F(116)/45*y*z*u^3-558*z^2*u^3-5363*x*u^4-F(53)/50*y*u^4-2269*z*u^4-15473*u^5+4107*x^4*v+F(43)/23*x^3*y*v+F(107)/87*x^2*y^2*v-12247*x*y^3*v-F(14)/123*y^4*v-F(121)/21*x^3*z*v-F(30)/53*x^2*y*z*v+F(42)/79*x*y^2*z*v-2123*y^3*z*v+F(107)/19*x^2*z^2*v-F(89)/38*x*y*z^2*v+12758*y^2*z^2*v+F(75)/121*x*z^3*v-F(44)/5*y*z^3*v-F(93)/44*x^3*u*v+F(86)/53*x^2*y*u*v-6801*x*y^2*u*v-177*y^3*u*v-F(31)/30*x^2*z*u*v-F(78)/5*x*y*z*u*v+F(103)/83*y^2*z*u*v-F(57)/92*x*z^2*u*v-F(91)/2*y*z^2*u*v+F(50)/123*z^3*u*v-F(97)/28*x^2*u^2*v-F(43)/87*x*y*u^2*v-8312*y^2*u^2*v-F(83)/98*x*z*u^2*v+F(27)/121*y*z*u^2*v-F(109)/2*z^2*u^2*v-3420*x*u^3*v-F(23)/56*y*u^3*v-1514*z*u^3*v+F(74)/19*u^4*v-F(52)/73*x^3*v^2+15374*x^2*y*v^2-F(35)/93*x*y^2*v^2+2015*y^3*v^2-F(33)/35*x^2*z*v^2+2427*x*y*z*v^2+F(52)/43*y^2*z*v^2+F(50)/17*x*z^2*v^2+11462*y*z^2*v^2-F(111)/62*x^2*u*v^2-F(67)/11*x*y*u*v^2+F(112)/109*y^2*u*v^2-F(8)/89*x*z*u*v^2+F(124)/113*y*z*u*v^2-4806*z^2*u*v^2+15199*x*u^2*v^2-F(75)/104*y*u^2*v^2+7059*z*u^2*v^2+F(61)/94*u^3*v^2-4843*x^2*v^3+F(23)/71*x*y*v^3-F(81)/98*y^2*v^3+F(26)/19*x*z*v^3+2432*y*z*v^3+F(120)/113*x*u*v^3+F(113)/66*y*u*v^3+15898*z*u*v^3+4420*u^2*v^3+F(107)/34*x*v^4-F(114)/65*y*v^4+15451*u*v^4,-F(9)/32*x^5-F(19)/91*x^4*y-14204*x^3*y^2-F(59)/32*x^2*y^3-4300*x*y^4-2298*x^4*z+F(70)/99*x^3*y*z+28*x^2*y^2*z+F(101)/7*x*y^3*z-F(124)/85*x^3*z^2+1426*x^2*y*z^2-F(46)/43*x*y^2*z^2-12128*x^2*z^3+F(29)/19*x*y*z^3-10939*x*z^4-10113*x^4*u+F(109)/122*x^3*y*u+F(78)/59*x^2*y^2*u-F(55)/26*x*y^3*u-F(17)/106*y^4*u-F(65)/63*x^3*z*u+11968*x^2*y*z*u-F(123)/59*x*y^2*z*u-F(79)/14*y^3*z*u-F(77)/46*x^2*z^2*u+8287*x*y*z^2*u-8686*y^2*z^2*u+3725*x*z^3*u+F(108)/95*y*z^3*u-8817*z^4*u+2776*x^3*u^2+F(109)/32*x^2*y*u^2-903*x*y^2*u^2+F(81)/20*y^3*u^2-8573*x^2*z*u^2-F(3)/76*x*y*z*u^2+F(71)/90*y^2*z*u^2+5695*x*z^2*u^2-12328*y*z^2*u^2-9374*z^3*u^2+F(53)/83*x^2*u^3-48*x*y*u^3+F(27)/20*y^2*u^3+F(61)/63*x*z*u^3+F(1)/49*y*z*u^3+F(29)/47*z^2*u^3+F(4)/25*x*u^4-11392*y*u^4+F(60)/29*z*u^4-F(35)/8*u^5+F(68)/105*x^4*v+F(101)/23*x^3*y*v+F(65)/96*x^2*y^2*v-F(10)/107*x*y^3*v-8660*y^4*v-F(112)/109*x^3*z*v-5085*x^2*y*z*v+14561*x*y^2*z*v+F(67)/114*y^3*z*v-F(81)/65*x^2*z^2*v-15569*x*y*z^2*v+F(86)/41*y^2*z^2*v+2577*x*z^3*v-F(104)/47*y*z^3*v-F(97)/53*z^4*v+4081*x^3*u*v+5772*x^2*y*u*v-12597*x*y^2*u*v-F(88)/65*y^3*u*v-11043*x^2*z*u*v+8030*x*y*z*u*v+3907*y^2*z*u*v-F(32)/47*x*z^2*u*v+9661*y*z^2*u*v+F(96)/91*z^3*u*v+F(116)/43*x^2*u^2*v-F(123)/89*x*y*u^2*v-F(38)/103*y^2*u^2*v+F(31)/88*x*z*u^2*v-777*y*z*u^2*v+15674*z^2*u^2*v-F(99)/14*x*u^3*v+3529*y*u^3*v-F(87)/79*z*u^3*v-F(23)/90*u^4*v+F(97)/12*x^3*v^2+10228*x^2*y*v^2-F(1)/89*x*y^2*v^2-F(14)/123*y^3*v^2+F(107)/88*x^2*z*v^2-F(40)/33*x*y*z*v^2-2123*y^2*z*v^2-4826*x*z^2*v^2+12758*y*z^2*v^2-F(44)/5*z^3*v^2-15213*x^2*u*v^2+907*x*y*u*v^2-11797*y^2*u*v^2+F(104)/105*x*z*u*v^2+F(83)/98*y*z*u*v^2+F(64)/73*z^2*u*v^2+F(119)/72*x*u^2*v^2-F(79)/30*y*u^2*v^2+F(39)/7*z*u^2*v^2-9935*u^3*v^2-14814*x^2*v^3-F(107)/47*x*y*v^3+2015*y^2*v^3+F(16)/67*x*z*v^3+F(52)/43*y*z*v^3+11462*z^2*v^3+F(59)/119*x*u*v^3-15925*y*u*v^3+F(70)/111*z*u*v^3-F(64)/29*u^2*v^3-F(3)/70*x*v^4-F(81)/98*y*v^4+2432*z*v^4-F(58)/43*u*v^4-F(114)/65*v^5,F(9)/32*x^4*y+F(19)/91*x^3*y^2+14204*x^2*y^3+F(59)/32*x*y^4+4300*y^5+2298*x^3*y*z-F(70)/99*x^2*y^2*z-28*x*y^3*z-F(101)/7*y^4*z+F(124)/85*x^2*y*z^2-1426*x*y^2*z^2+F(46)/43*y^3*z^2+12128*x*y*z^3-F(29)/19*y^2*z^3+10939*y*z^4-x^4*u-13729*x^3*y*u-F(41)/6*x^2*y^2*u-2949*x*y^3*u-13198*y^4*u-F(5)/24*x^3*z*u-F(17)/90*x^2*y*z*u-F(73)/112*x*y^2*z*u+F(67)/109*y^3*z*u-2266*x^2*z^2*u-13503*x*y*z^2*u-4873*y^2*z^2*u-4968*x*z^3*u-F(31)/118*y*z^3*u-F(37)/91*z^4*u+6748*x^3*u^2+F(24)/77*x^2*y*u^2-F(125)/83*x*y^2*u^2-F(69)/80*y^3*u^2+F(12)/13*x^2*z*u^2+F(56)/13*x*y*z*u^2+F(61)/20*y^2*z*u^2+2590*x*z^2*u^2+F(26)/107*y*z^2*u^2-F(13)/8*z^3*u^2+8661*x^2*u^3+4883*x*y*u^3+8544*y^2*u^3-F(65)/53*x*z*u^3+F(2)/3*y*z*u^3+F(95)/43*z^2*u^3+F(37)/69*x*u^4+F(82)/115*y*u^4-8500*z*u^4+F(93)/43*u^5+F(108)/19*x^4*v+F(111)/89*x^3*y*v-F(97)/34*x^2*y^2*v-1238*x*y^3*v+5555*y^4*v-1285*x^3*z*v-F(121)/28*x^2*y*z*v+8432*x*y^2*z*v+2685*y^3*z*v-F(93)/92*x^2*z^2*v+F(73)/23*x*y*z^2*v-F(59)/97*y^2*z^2*v-6580*x*z^3*v-F(23)/30*y*z^3*v+F(90)/121*z^4*v-F(19)/6*x^3*u*v-F(115)/112*x^2*y*u*v+73*x*y^2*u*v-12154*y^3*u*v-8517*x^2*z*u*v+F(73)/17*x*y*z*u*v-F(51)/41*y^2*z*u*v+F(125)/22*x*z^2*u*v-2120*y*z^2*u*v+F(108)/59*z^3*u*v-F(17)/114*x^2*u^2*v-F(121)/119*x*y*u^2*v+F(107)/96*y^2*u^2*v-F(38)/15*x*z*u^2*v-F(31)/104*y*z*u^2*v-F(110)/3*z^2*u^2*v+F(3)/97*x*u^3*v+5167*y*u^3*v-6882*z*u^3*v-12755*u^4*v+4107*x^3*v^2-F(114)/37*x^2*y*v^2+F(55)/38*x*y^2*v^2-5058*y^3*v^2-F(121)/21*x^2*z*v^2+2200*x*y*z*v^2+F(91)/17*y^2*z*v^2+F(107)/19*x*z^2*v^2+F(21)/85*y*z^2*v^2+F(75)/121*z^3*v^2-4745*x^2*u*v^2+12237*x*y*u*v^2-7939*y^2*u*v^2+F(28)/117*x*z*u*v^2+F(117)/88*y*z*u*v^2+F(56)/65*z^2*u*v^2-F(13)/114*x*u^2*v^2+F(8)/75*y*u^2*v^2-10593*z*u^2*v^2+12223*u^3*v^2-F(52)/73*x^2*v^3-F(49)/71*x*y*v^3+F(39)/86*y^2*v^3-F(33)/35*x*z*v^3-F(5)/12*y*z*v^3+F(50)/17*z^2*v^3-5909*x*u*v^3-15466*y*u*v^3+F(43)/87*z*u*v^3-F(117)/19*u^2*v^3-4843*x*v^4-F(111)/101*y*v^4+F(26)/19*z*v^4-860*u*v^4+F(107)/34*v^5,F(9)/32*x^4*z+F(19)/91*x^3*y*z+14204*x^2*y^2*z+F(59)/32*x*y^3*z+4300*y^4*z+2298*x^3*z^2-F(70)/99*x^2*y*z^2-28*x*y^2*z^2-F(101)/7*y^3*z^2+F(124)/85*x^2*z^3-1426*x*y*z^3+F(46)/43*y^2*z^3+12128*x*z^4-F(29)/19*y*z^4+10939*z^5+F(25)/6*x^4*u-F(102)/47*x^3*y*u-F(21)/25*x^2*y^2*u-F(92)/93*x*y^3*u-F(2)/119*y^4*u-F(79)/124*x^3*z*u-8207*x^2*y*z*u+F(23)/116*x*y^2*z*u-F(73)/46*y^3*z*u-F(23)/119*x^2*z^2*u-F(85)/61*x*y*z^2*u+F(75)/109*y^2*z^2*u+F(2)/37*x*z^3*u-10186*y*z^3*u+F(116)/33*z^4*u-9173*x^3*u^2-8705*x^2*y*u^2-10743*x*y^2*u^2-F(111)/44*y^3*u^2-F(109)/14*x^2*z*u^2+12834*x*y*z*u^2-F(50)/79*y^2*z*u^2-14164*x*z^2*u^2-F(31)/106*y*z^2*u^2+F(86)/125*z^3*u^2+F(83)/56*x^2*u^3+505*x*y*u^3+F(51)/95*y^2*u^3-13043*x*z*u^3+F(2)/93*y*z*u^3-F(86)/83*z^2*u^3+2053*x*u^4-7747*y*u^4+14421*z*u^4+F(33)/26*u^5-F(106)/115*x^4*v-F(29)/36*x^3*y*v-F(97)/84*x^2*y^2*v-3749*x*y^3*v+3803*y^4*v+10193*x^3*z*v-F(61)/93*x^2*y*z*v+12584*x*y^2*z*v+F(13)/66*y^3*z*v+F(103)/36*x^2*z^2*v-F(11)/2*x*y*z^2*v+5192*y^2*z^2*v+F(86)/115*x*z^3*v+F(23)/108*y*z^3*v-5704*z^4*v+7034*x^3*u*v-12827*x^2*y*u*v-F(90)/49*x*y^2*u*v-F(11)/61*y^3*u*v-F(85)/93*x^2*z*u*v+F(116)/91*x*y*z*u*v+15517*y^2*z*u*v+F(101)/28*x*z^2*u*v-3257*y*z^2*u*v+F(63)/101*z^3*u*v+F(61)/34*x^2*u^2*v+F(121)/38*x*y*u^2*v-F(70)/123*y^2*u^2*v-11675*x*z*u^2*v+1645*y*z*u^2*v+2255*z^2*u^2*v-11979*x*u^3*v-F(31)/68*y*u^3*v-F(68)/53*z*u^3*v+F(122)/109*u^4*v-F(83)/66*x^3*v^2-203*x^2*y*v^2-F(77)/50*x*y^2*v^2-F(125)/69*y^3*v^2-F(53)/78*x^2*z*v^2-15901*x*y*z*v^2-8630*y^2*z*v^2+11793*x*z^2*v^2+8707*y*z^2*v^2+15628*z^3*v^2-F(29)/99*x^2*u*v^2+701*x*y*u*v^2+7174*y^2*u*v^2-F(73)/37*x*z*u*v^2-F(106)/63*y*z*u*v^2+10738*z^2*u*v^2+4967*x*u^2*v^2-7159*y*u^2*v^2-F(83)/46*z*u^2*v^2+F(93)/115*u^3*v^2+12988*x^2*v^3+F(23)/96*x*y*v^3-F(55)/92*y^2*v^3-F(121)/61*x*z*v^3+F(84)/29*y*z*v^3-F(83)/86*z^2*v^3-F(1)/106*x*u*v^3+F(86)/65*y*u*v^3-12430*z*u*v^3+F(58)/51*u^2*v^3-F(25)/122*x*v^4-3572*y*v^4+F(2)/39*z*v^4+13016*u*v^4-9252*v^5,F(25)/6*x^5-F(102)/47*x^4*y-F(21)/25*x^3*y^2-F(92)/93*x^2*y^3-F(2)/119*x*y^4+F(117)/107*x^4*z-F(45)/116*x^3*y*z-6837*x^2*y^2*z+F(77)/37*x*y^3*z-F(17)/106*y^4*z+F(93)/46*x^3*z^2-F(49)/6*x^2*y*z^2-12368*x*y^2*z^2-F(79)/14*y^3*z^2-F(103)/110*x^2*z^3-1899*x*y*z^3-8686*y^2*z^3-8874*x*z^4+F(108)/95*y*z^4-8817*z^5-9173*x^4*u-8705*x^3*y*u-10743*x^2*y^2*u-F(111)/44*x*y^3*u-4087*x^3*z*u+F(35)/113*x^2*y*z*u+10030*x*y^2*z*u+F(81)/20*y^3*z*u+F(49)/121*x^2*z^2*u+F(33)/92*x*y*z^2*u+F(71)/90*y^2*z^2*u-6077*x*z^3*u-12328*y*z^3*u-9374*z^4*u+F(83)/56*x^3*u^2+505*x^2*y*u^2+F(51)/95*x*y^2*u^2+13938*x^2*z*u^2+F(9)/50*x*y*z*u^2+F(27)/20*y^2*z*u^2+159*x*z^2*u^2+F(1)/49*y*z^2*u^2+F(29)/47*z^3*u^2+2053*x^2*u^3-7747*x*y*u^3-9892*x*z*u^3-11392*y*z*u^3+F(60)/29*z^2*u^3+F(33)/26*x*u^4-F(35)/8*z*u^4-F(100)/3*x^4*v+F(107)/3*x^3*y*v-21*x^2*y^2*v-13482*x*y^3*v-F(42)/79*y^4*v-F(66)/31*x^3*z*v+817*x^2*y*z*v-F(29)/33*x*y^2*z*v+F(21)/115*y^3*z*v-14316*x^2*z^2*v-2103*x*y*z^2*v-7029*y^2*z^2*v-F(77)/71*x*z^3*v-F(32)/65*y*z^3*v+F(72)/25*z^4*v-F(5)/17*x^3*u*v+F(23)/67*x^2*y*u*v-F(37)/35*x*y^2*u*v+F(4)/97*y^3*u*v-200*x^2*z*u*v+F(101)/41*x*y*z*u*v-F(111)/64*y^2*z*u*v-F(91)/58*x*z^2*u*v+F(95)/46*y*z^2*u*v-5655*z^3*u*v-10599*x^2*u^2*v+39*x*y*u^2*v+F(81)/59*y^2*u^2*v+F(61)/116*x*z*u^2*v+F(123)/53*y*z*u^2*v+4712*z^2*u^2*v-F(31)/8*x*u^3*v-1934*y*u^3*v-F(9)/86*z*u^3*v-F(41)/2*u^4*v+F(99)/20*x^3*v^2+6598*x^2*y*v^2-4940*x*y^2*v^2+F(17)/108*y^3*v^2-9117*x^2*z*v^2-F(89)/107*x*y*z*v^2+F(113)/58*y^2*z*v^2+10075*x*z^2*v^2-11206*y*z^2*v^2-F(90)/37*z^3*v^2-167*x^2*u*v^2+F(16)/21*x*y*u*v^2+F(51)/29*y^2*u*v^2+F(113)/19*x*z*u*v^2+F(123)/17*y*z*u*v^2-F(29)/47*z^2*u*v^2+135*x*u^2*v^2-F(68)/91*y*u^2*v^2+15211*z*u^2*v^2-F(47)/45*u^3*v^2+F(72)/43*x^2*v^3-F(85)/86*x*y*v^3+F(21)/107*y^2*v^3-F(71)/57*x*z*v^3+14644*y*z*v^3+F(11)/20*z^2*v^3-3399*x*u*v^3-13025*y*u*v^3-F(14)/75*z*u*v^3-F(73)/112*u^2*v^3-F(76)/29*x*v^4+766*y*v^4+F(69)/89*z*v^4-2330*u*v^4+676*v^5,-F(25)/6*x^4*y+F(102)/47*x^3*y^2+F(21)/25*x^2*y^3+F(92)/93*x*y^4+F(2)/119*y^5-x^4*z+6653*x^3*y*z-F(69)/26*x^2*y^2*z+636*x*y^3*z+F(59)/84*y^4*z-F(5)/24*x^3*z^2+7596*x^2*y*z^2-5829*x*y^2*z^2-F(8)/109*y^3*z^2-2266*x^2*z^3+F(42)/55*x*y*z^3-F(113)/6*y^2*z^3-4968*x*z^4+F(3)/34*y*z^4-F(37)/91*z^5+9173*x^3*y*u+8705*x^2*y^2*u+10743*x*y^3*u+F(111)/44*y^4*u+6748*x^3*z*u-F(5)/102*x^2*y*z*u-7054*x*y^2*z*u-3336*y^3*z*u+F(12)/13*x^2*z^2*u-F(87)/62*x*y*z^2*u+F(93)/82*y^2*z^2*u+2590*x*z^3*u+15659*y*z^3*u-F(13)/8*z^4*u-F(83)/56*x^2*y*u^2-505*x*y^2*u^2-F(51)/95*y^3*u^2+8661*x^2*z*u^2+F(1)/116*x*y*z*u^2-F(82)/57*y^2*z*u^2-F(65)/53*x*z^2*u^2-F(102)/91*y*z^2*u^2+F(95)/43*z^3*u^2-2053*x*y*u^3+7747*y^2*u^3+F(37)/69*x*z*u^3-F(66)/17*y*z*u^3-8500*z^2*u^3-F(33)/26*y*u^4+F(93)/43*z*u^4+4843*x^4*v+F(72)/95*x^3*y*v-1832*x^2*y^2*v-F(121)/20*x*y^3*v-987*y^4*v+F(101)/47*x^3*z*v+F(116)/111*x^2*y*z*v-14555*x*y^2*z*v-10077*y^3*z*v-F(65)/68*x^2*z^2*v-F(80)/57*x*y*z^2*v-11877*y^2*z^2*v-F(49)/116*x*z^3*v+11654*y*z^3*v-10963*z^4*v-F(118)/37*x^3*u*v-4807*x^2*y*u*v+649*x*y^2*u*v+F(23)/97*y^3*u*v-14793*x^2*z*u*v-F(48)/65*x*y*z*u*v-11776*y^2*z*u*v-F(108)/67*x*z^2*u*v-11876*y*z^2*u*v+F(79)/24*z^3*u*v-F(19)/35*x^2*u^2*v+F(66)/115*x*y*u^2*v-F(61)/94*y^2*u^2*v+F(55)/14*x*z*u^2*v-4329*y*z*u^2*v+F(23)/48*z^2*u^2*v+F(65)/124*x*u^3*v+F(5)/91*y*u^3*v-F(103)/13*z*u^3*v-F(16)/3*u^4*v-F(3)/37*x^3*v^2-203*x^2*y*v^2-F(35)/67*x*y^2*v^2-7877*y^3*v^2+F(67)/88*x^2*z*v^2-F(70)/99*x*y*z*v^2-3073*y^2*z*v^2+F(69)/52*x*z^2*v^2-F(2)/67*y*z^2*v^2-F(12)/35*z^3*v^2-6924*x^2*u*v^2+F(7)/106*x*y*u*v^2+11259*y^2*u*v^2-11618*x*z*u*v^2-F(60)/59*y*z*u*v^2-F(49)/16*z^2*u*v^2-F(54)/37*x*u^2*v^2-F(9)/16*y*u^2*v^2-F(92)/107*z*u^2*v^2-1240*u^3*v^2-12847*x^2*v^3-F(69)/68*x*y*v^3+F(41)/121*y^2*v^3+7229*x*z*v^3+F(65)/98*y*z*v^3-15415*z^2*v^3+5474*x*u*v^3-F(118)/81*y*u*v^3-F(50)/39*z*u*v^3+6224*u^2*v^3+8052*x*v^4-1550*y*v^4+F(15)/59*z*v^4-21*u*v^4-F(61)/31*v^5,x^5-F(37)/106*x^4*y-6637*x^3*y^2+6201*x^2*y^3-F(51)/59*x*y^4+F(17)/106*y^5+F(5)/24*x^4*z+52*x^3*y*z+11169*x^2*y^2*z+F(32)/15*x*y^3*z+F(79)/14*y^4*z+2266*x^3*z^2+F(19)/110*x^2*y*z^2-F(122)/75*x*y^2*z^2+8686*y^3*z^2+4968*x^2*z^3-F(96)/125*x*y*z^3-F(108)/95*y^2*z^3+F(37)/91*x*z^4+8817*y*z^4-6748*x^4*u-13163*x^3*y*u-10782*x^2*y^2*u+F(77)/43*x*y^3*u-F(81)/20*y^4*u-F(12)/13*x^3*z*u-F(125)/114*x^2*y*z*u+F(123)/71*x*y^2*z*u-F(71)/90*y^3*z*u-2590*x^2*z^2*u-9582*x*y*z^2*u+12328*y^2*z^2*u+F(13)/8*x*z^3*u+9374*y*z^3*u-8661*x^3*u^2+127*x^2*y*u^2+F(103)/64*x*y^2*u^2-F(27)/20*y^3*u^2+F(65)/53*x^2*z*u^2-F(103)/63*x*y*z*u^2-F(1)/49*y^2*z*u^2-F(95)/43*x*z^2*u^2-F(29)/47*y*z^2*u^2-F(37)/69*x^2*u^3+F(5)/58*x*y*u^3+11392*y^2*u^3+8500*x*z*u^3-F(60)/29*y*z*u^3-F(93)/43*x*u^4+F(35)/8*y*u^4+F(68)/47*x^4*v+869*x^3*y*v-F(104)/27*x^2*y^2*v-F(33)/71*x*y^3*v-F(17)/120*y^4*v+F(11)/34*x^3*z*v+F(25)/9*x^2*y*z*v+F(94)/73*x*y^2*z*v-F(43)/112*y^3*z*v-139*x^2*z^2*v-F(80)/83*x*y*z^2*v-1052*y^2*z^2*v+8642*x*z^3*v-9730*y*z^3*v-112*z^4*v-F(7)/117*x^3*u*v+9237*x^2*y*u*v+12121*x*y^2*u*v-7186*y^3*u*v-F(94)/81*x^2*z*u*v-7963*x*y*z*u*v+F(37)/125*y^2*z*u*v-7352*x*z^2*u*v+F(97)/26*y*z^2*u*v+7699*z^3*u*v-2201*x^2*u^2*v+F(37)/80*x*y*u^2*v-F(81)/73*y^2*u^2*v-10880*x*z*u^2*v-F(89)/103*y*z*u^2*v-558*z^2*u^2*v+F(123)/13*x*u^3*v+F(73)/86*y*u^3*v-2269*z*u^3*v-15473*u^4*v-4709*x^3*v^2-644*x^2*y*v^2+231*x*y^2*v^2+11620*y^3*v^2-9872*x^2*z*v^2-F(113)/15*x*y*z*v^2+1491*y^2*z*v^2+F(55)/64*x*z^2*v^2-F(6)/71*y*z^2*v^2+F(50)/123*z^3*v^2-F(118)/113*x^2*u*v^2+F(115)/33*x*y*u*v^2+F(13)/53*y^2*u*v^2+F(99)/61*x*z*u*v^2-13187*y*z*u*v^2-F(109)/2*z^2*u*v^2-15643*x*u^2*v^2-F(19)/87*y*u^2*v^2-1514*z*u^2*v^2+F(74)/19*u^3*v^2-F(83)/71*x^2*v^3-F(11)/86*x*y*v^3+F(113)/123*y^2*v^3+F(73)/66*x*z*v^3-F(79)/22*y*z*v^3-4806*z^2*v^3+3419*x*u*v^3-11189*y*u*v^3+7059*z*u*v^3+F(61)/94*u^2*v^3-F(86)/65*x*v^4+F(43)/3*y*v^4+15898*z*v^4+4420*u*v^4+15451*v^5]))
end


@doc raw"""
    k3_d13_pi16()

Return a smooth K3 surface in $\mathbb P^4$ with degree `13` and sectional genus `16`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d13_pi16()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(85)/44*x^4*y^2-3390*x^3*y^3+F(26)/63*x^2*y^4+F(35)/37*x*y^5-F(23)/59*y^6-F(102)/59*x^4*y*z-F(106)/35*x^3*y^2*z-F(74)/55*x^2*y^3*z-14815*x*y^4*z-F(28)/23*y^5*z+10618*x^4*z^2-3422*x^3*y*z^2+F(39)/92*x^2*y^2*z^2+69*x*y^3*z^2-765*y^4*z^2+2892*x^3*z^3+F(41)/107*x^2*y*z^3-F(91)/9*x*y^2*z^3+11646*y^3*z^3+F(109)/13*x^2*z^4-F(33)/7*x*y*z^4-8738*y^2*z^4+F(23)/41*x*z^5+F(121)/28*y*z^5-F(96)/49*z^6-3493*x^4*y*u-7401*x^3*y^2*u-F(119)/33*x^2*y^3*u-107*x*y^4*u+F(91)/118*y^5*u+15308*x^4*z*u-F(87)/107*x^3*y*z*u+6644*x^2*y^2*z*u+11546*x*y^3*z*u-F(47)/62*y^4*z*u-13433*x^3*z^2*u+13730*x^2*y*z^2*u-F(84)/47*x*y^2*z^2*u-12317*y^3*z^2*u-11899*x^2*z^3*u+10949*x*y*z^3*u-11775*y^2*z^3*u-F(8)/41*x*z^4*u+F(79)/5*y*z^4*u+11121*z^5*u-F(52)/81*x^4*u^2-F(19)/26*x^3*y*u^2-F(93)/19*x^2*y^2*u^2-15243*x*y^3*u^2+13230*y^4*u^2+1823*x^3*z*u^2+F(112)/75*x^2*y*z*u^2+F(69)/76*x*y^2*z*u^2-4973*y^3*z*u^2-F(61)/13*x^2*z^2*u^2-F(69)/52*x*y*z^2*u^2-F(29)/44*y^2*z^2*u^2-5177*x*z^3*u^2-11899*y*z^3*u^2+F(103)/13*z^4*u^2+11441*x^3*u^3-F(51)/31*x^2*y*u^3-F(31)/104*x*y^2*u^3-10271*y^3*u^3-2302*x^2*z*u^3-F(9)/46*x*y*z*u^3-15888*y^2*z*u^3-15227*x*z^2*u^3-4419*y*z^2*u^3-5014*z^3*u^3+F(49)/102*x^2*u^4-F(22)/29*x*y*u^4+1644*y^2*u^4+11970*x*z*u^4-F(55)/91*y*z*u^4-F(19)/88*z^2*u^4-14198*x*u^5+10007*y*u^5+15310*z*u^5-F(126)/89*u^6+14779*x^4*y*v-F(77)/87*x^3*y^2*v-8088*x^2*y^3*v-496*x*y^4*v+F(28)/95*y^5*v+F(7)/18*x^4*z*v-F(19)/4*x^3*y*z*v+2553*x^2*y^2*z*v-F(99)/32*x*y^3*z*v+13283*y^4*z*v+F(83)/21*x^3*z^2*v+3852*x^2*y*z^2*v+F(22)/67*x*y^2*z^2*v-13688*y^3*z^2*v+F(59)/58*x^2*z^3*v-F(39)/8*x*y*z^3*v-7009*y^2*z^3*v+F(41)/35*x*z^4*v-F(100)/19*y*z^4*v+F(100)/7*z^5*v-2927*x^4*u*v+F(97)/103*x^3*y*u*v+F(113)/67*x^2*y^2*u*v+8801*x*y^3*u*v-F(80)/123*y^4*u*v-F(79)/113*x^3*z*u*v-5420*x^2*y*z*u*v+F(54)/41*x*y^2*z*u*v-F(34)/71*y^3*z*u*v+11209*x^2*z^2*u*v+5837*x*y*z^2*u*v-8044*y^2*z^2*u*v-11396*x*z^3*u*v+F(37)/9*y*z^3*u*v-F(66)/115*z^4*u*v-F(32)/51*x^3*u^2*v+F(7)/69*x^2*y*u^2*v-13638*x*y^2*u^2*v+F(122)/33*y^3*u^2*v+F(38)/107*x^2*z*u^2*v+7537*x*y*z*u^2*v+9748*y^2*z*u^2*v-F(115)/126*x*z^2*u^2*v-5042*y*z^2*u^2*v+4447*z^3*u^2*v-1118*x^2*u^3*v-12468*x*y*u^3*v+F(11)/92*y^2*u^3*v-11056*x*z*u^3*v-8852*y*z*u^3*v-F(17)/56*z^2*u^3*v-F(24)/47*x*u^4*v+1291*y*u^4*v-F(124)/73*z*u^4*v-F(53)/93*u^5*v+F(94)/63*x^4*v^2+11055*x^3*y*v^2+7126*x^2*y^2*v^2+1671*x*y^3*v^2-F(103)/82*y^4*v^2+7946*x^3*z*v^2+F(50)/29*x^2*y*z*v^2-F(95)/106*x*y^2*z*v^2-F(39)/25*y^3*z*v^2+F(102)/49*x^2*z^2*v^2+1735*x*y*z^2*v^2+F(53)/19*y^2*z^2*v^2+15920*x*z^3*v^2-F(67)/101*y*z^3*v^2+F(117)/43*z^4*v^2+539*x^3*u*v^2+F(23)/104*x^2*y*u*v^2-F(25)/16*x*y^2*u*v^2-F(74)/5*y^3*u*v^2+2849*x^2*z*u*v^2-15365*x*y*z*u*v^2+F(17)/87*y^2*z*u*v^2+F(23)/3*x*z^2*u*v^2-F(56)/121*y*z^2*u*v^2+10381*z^3*u*v^2-F(62)/61*x^2*u^2*v^2-F(17)/113*x*y*u^2*v^2-9544*y^2*u^2*v^2+611*x*z*u^2*v^2-2767*y*z*u^2*v^2-1342*z^2*u^2*v^2+F(88)/19*x*u^3*v^2+F(77)/96*y*u^3*v^2+13601*z*u^3*v^2-7469*u^4*v^2+F(63)/110*x^3*v^3+F(13)/75*x^2*y*v^3-6513*x*y^2*v^3-F(34)/105*y^3*v^3-14945*x^2*z*v^3-F(3)/59*x*y*z*v^3+F(92)/87*y^2*z*v^3+F(82)/115*x*z^2*v^3+9891*y*z^2*v^3+F(43)/88*z^3*v^3-9962*x^2*u*v^3-F(77)/122*x*y*u*v^3+F(56)/41*y^2*u*v^3-12504*x*z*u*v^3+10932*y*z*u*v^3-6443*z^2*u*v^3-F(32)/49*x*u^2*v^3-14907*y*u^2*v^3+157*z*u^2*v^3-10454*u^3*v^3+F(43)/4*x^2*v^4+11621*x*y*v^4-F(92)/111*y^2*v^4-F(3)/95*x*z*v^4+F(79)/45*y*z*v^4+F(85)/124*z^2*v^4+F(109)/84*x*u*v^4-F(85)/6*y*u*v^4+F(41)/98*z*u*v^4+2616*u^2*v^4-4647*x*v^5+825*y*v^5+F(13)/88*z*v^5-F(42)/11*u*v^5-F(119)/88*v^6,x^4*y^2-F(85)/46*x^3*y^3+F(70)/31*x^2*y^4-F(80)/109*x*y^5+6014*y^6+8335*x^4*y*z+F(71)/103*x^3*y^2*z+F(95)/39*x^2*y^3*z+F(67)/64*x*y^4*z-F(99)/41*y^5*z-6994*x^4*z^2-F(31)/5*x^3*y*z^2+F(23)/24*x^2*y^2*z^2-F(11)/31*x*y^3*z^2+12528*y^4*z^2+F(71)/23*x^3*z^3+F(74)/69*x^2*y*z^3+F(68)/37*x*y^2*z^3-3517*y^3*z^3-F(13)/126*x^2*z^4-F(110)/17*x*y*z^4-F(97)/109*y^2*z^4+F(119)/25*x*z^5-4675*y*z^5+F(113)/17*z^6+F(57)/79*x^4*y*u-F(58)/45*x^3*y^2*u+F(23)/51*x^2*y^3*u+F(63)/110*x*y^4*u-15240*y^5*u+4215*x^4*z*u-F(23)/2*x^3*y*z*u+8500*x^2*y^2*z*u-F(106)/31*x*y^3*z*u+F(43)/11*y^4*z*u+F(64)/103*x^3*z^2*u-F(62)/119*x^2*y*z^2*u-691*x*y^2*z^2*u-F(91)/85*y^3*z^2*u+F(13)/18*x^2*z^3*u+5835*x*y*z^3*u-F(97)/31*y^2*z^3*u+F(63)/19*x*z^4*u-6526*y*z^4*u+F(89)/65*z^5*u+F(92)/101*x^4*u^2-12433*x^3*y*u^2-F(123)/68*x^2*y^2*u^2+1634*x*y^3*u^2+F(13)/8*y^4*u^2-9574*x^3*z*u^2-F(103)/67*x^2*y*z*u^2-F(12)/49*x*y^2*z*u^2+F(61)/4*y^3*z*u^2+15433*x^2*z^2*u^2-F(96)/53*x*y*z^2*u^2-14618*y^2*z^2*u^2+F(56)/57*x*z^3*u^2-F(49)/95*y*z^3*u^2+F(116)/113*z^4*u^2+F(67)/66*x^3*u^3+8818*x^2*y*u^3-F(75)/44*x*y^2*u^3+14907*y^3*u^3+F(12)/55*x^2*z*u^3-14102*x*y*z*u^3+F(41)/83*y^2*z*u^3-F(125)/32*x*z^2*u^3+F(73)/36*y*z^2*u^3+F(65)/82*z^3*u^3+F(73)/105*x^2*u^4+F(55)/49*x*y*u^4-4518*y^2*u^4-F(93)/19*x*z*u^4+F(23)/11*y*z*u^4+F(124)/75*z^2*u^4-5451*x*u^5-F(41)/118*y*u^5-F(84)/71*z*u^5+1611*u^6+F(71)/13*x^4*y*v-F(97)/41*x^3*y^2*v-342*x^2*y^3*v+F(82)/25*x*y^4*v+F(95)/94*y^5*v-5129*x^4*z*v+F(33)/122*x^3*y*z*v-F(87)/8*x^2*y^2*z*v-6253*x*y^3*z*v+14147*y^4*z*v+F(69)/5*x^3*z^2*v-F(35)/116*x^2*y*z^2*v+F(59)/91*x*y^2*z^2*v-9583*y^3*z^2*v-F(80)/109*x^2*z^3*v-6427*x*y*z^3*v-F(123)/95*y^2*z^3*v-F(69)/31*x*z^4*v+F(51)/40*y*z^4*v-466*z^5*v+F(57)/52*x^4*u*v-F(62)/113*x^3*y*u*v-F(109)/17*x^2*y^2*u*v-F(18)/59*x*y^3*u*v+10708*y^4*u*v-2949*x^3*z*u*v-15920*x^2*y*z*u*v+F(106)/61*x*y^2*z*u*v-F(31)/114*y^3*z*u*v+F(112)/39*x^2*z^2*u*v+2652*x*y*z^2*u*v+8463*y^2*z^2*u*v-F(83)/11*x*z^3*u*v+6020*y*z^3*u*v+F(59)/65*z^4*u*v+F(16)/115*x^3*u^2*v-3315*x^2*y*u^2*v-15283*x*y^2*u^2*v+F(5)/26*y^3*u^2*v+15200*x^2*z*u^2*v-7228*x*y*z*u^2*v+12489*y^2*z*u^2*v-9537*x*z^2*u^2*v+F(86)/107*y*z^2*u^2*v+5667*z^3*u^2*v-7308*x^2*u^3*v-8233*x*y*u^3*v-14681*y^2*u^3*v+4677*x*z*u^3*v+F(79)/26*y*z*u^3*v-F(26)/17*z^2*u^3*v+F(63)/44*x*u^4*v-6425*y*u^4*v-F(14)/83*z*u^4*v-F(100)/9*u^5*v-F(95)/102*x^4*v^2-F(119)/71*x^3*y*v^2+1799*x^2*y^2*v^2+11003*x*y^3*v^2-F(31)/56*y^4*v^2+9827*x^3*z*v^2-10589*x^2*y*z*v^2+10585*x*y^2*z*v^2+10246*y^3*z*v^2-F(23)/32*x^2*z^2*v^2+F(70)/97*x*y*z^2*v^2-F(74)/77*y^2*z^2*v^2+F(63)/115*x*z^3*v^2+F(123)/7*y*z^3*v^2-F(115)/82*z^4*v^2+F(79)/24*x^3*u*v^2-7347*x^2*y*u*v^2+13276*x*y^2*u*v^2+14490*y^3*u*v^2+F(50)/109*x^2*z*u*v^2-F(119)/18*x*y*z*u*v^2-F(62)/29*y^2*z*u*v^2-F(98)/47*x*z^2*u*v^2-F(22)/43*y*z^2*u*v^2+11534*z^3*u*v^2+15613*x^2*u^2*v^2-F(30)/67*x*y*u^2*v^2+F(71)/115*y^2*u^2*v^2+F(93)/55*x*z*u^2*v^2-14780*y*z*u^2*v^2-F(75)/97*z^2*u^2*v^2+9962*x*u^3*v^2+F(41)/97*y*u^3*v^2+F(43)/119*z*u^3*v^2+F(18)/47*u^4*v^2-F(16)/9*x^3*v^3+1590*x^2*y*v^3+1260*x*y^2*v^3-F(93)/94*y^3*v^3-F(3)/26*x^2*z*v^3+F(107)/106*x*y*z*v^3+315*y^2*z*v^3-F(125)/36*x*z^2*v^3+F(9)/35*y*z^2*v^3+7429*z^3*v^3-F(119)/60*x^2*u*v^3+F(110)/119*x*y*u*v^3+12287*y^2*u*v^3+14040*x*z*u*v^3+F(41)/19*y*z*u*v^3-2432*z^2*u*v^3+F(85)/24*x*u^2*v^3+F(67)/104*y*u^2*v^3-F(126)/47*z*u^2*v^3+F(39)/56*u^3*v^3-7029*x^2*v^4-F(32)/35*x*y*v^4+F(69)/73*y^2*v^4-F(117)/121*x*z*v^4+F(119)/71*y*z*v^4-4274*z^2*v^4-F(67)/25*x*u*v^4+4933*y*u*v^4+374*z*u*v^4+F(24)/31*u^2*v^4+F(107)/68*x*v^5-3652*y*v^5+12143*z*v^5-F(59)/30*u*v^5-15377*v^6,-F(29)/97*x^4*y*z-F(108)/107*x^3*y^2*z+2474*x^2*y^3*z-8542*x*y^4*z-F(101)/2*y^5*z+F(44)/79*x^4*z^2+4122*x^3*y*z^2-13448*x^2*y^2*z^2-5226*x*y^3*z^2-1764*y^4*z^2-7215*x^3*z^3-F(77)/10*x^2*y*z^3-8541*x*y^2*z^3-9751*y^3*z^3+F(42)/17*x^2*z^4+F(13)/41*x*y*z^4-F(100)/47*y^2*z^4-14676*x*z^5-5250*y*z^5+9807*z^6-F(79)/59*x^4*y*u+F(47)/82*x^3*y^2*u-F(17)/7*x^2*y^3*u+F(111)/100*x*y^4*u-F(7)/54*y^5*u+F(67)/7*x^4*z*u+F(61)/112*x^3*y*z*u-13563*x^2*y^2*z*u-F(77)/74*x*y^3*z*u-11793*y^4*z*u-F(81)/115*x^3*z^2*u+F(73)/102*x^2*y*z^2*u+F(73)/75*x*y^2*z^2*u-F(54)/103*y^3*z^2*u-13025*x^2*z^3*u-F(59)/64*x*y*z^3*u-F(21)/2*y^2*z^3*u+F(89)/100*x*z^4*u-F(73)/35*y*z^4*u+10785*z^5*u+10025*x^4*u^2-7260*x^3*y*u^2+F(107)/67*x^2*y^2*u^2+F(2)/113*x*y^3*u^2-6985*y^4*u^2-12988*x^3*z*u^2-F(16)/67*x^2*y*z*u^2-F(68)/83*x*y^2*z*u^2+F(11)/82*y^3*z*u^2-F(22)/97*x^2*z^2*u^2+9958*x*y*z^2*u^2-F(50)/3*y^2*z^2*u^2+F(125)/34*x*z^3*u^2-F(39)/23*y*z^3*u^2-F(1)/102*z^4*u^2+F(123)/119*x^3*u^3+14290*x^2*y*u^3+F(103)/45*x*y^2*u^3+F(31)/120*y^3*u^3-F(19)/29*x^2*z*u^3+F(4)/81*x*y*z*u^3+F(17)/64*y^2*z*u^3-F(63)/59*x*z^2*u^3-F(21)/34*y*z^2*u^3-F(31)/2*z^3*u^3-8355*x^2*u^4-F(14)/99*x*y*u^4+F(53)/15*y^2*u^4+F(28)/43*x*z*u^4-F(51)/53*y*z*u^4+F(101)/84*z^2*u^4-14885*x*u^5+F(4)/49*y*u^5-F(62)/77*z*u^5+3971*u^6+F(62)/87*x^4*y*v+8174*x^3*y^2*v+F(39)/121*x^2*y^3*v-F(106)/27*x*y^4*v-F(125)/23*y^5*v+F(67)/79*x^4*z*v-15636*x^3*y*z*v+F(113)/89*x^2*y^2*z*v-776*x*y^3*z*v+F(30)/97*y^4*z*v-F(113)/75*x^3*z^2*v+F(10)/33*x^2*y*z^2*v+F(60)/31*x*y^2*z^2*v+F(9)/89*y^3*z^2*v+F(36)/103*x^2*z^3*v-8842*x*y*z^3*v-F(21)/89*y^2*z^3*v-F(102)/89*x*z^4*v-F(26)/99*y*z^4*v-F(59)/97*z^5*v+F(123)/64*x^4*u*v+F(52)/41*x^3*y*u*v+F(67)/27*x^2*y^2*u*v+11889*x*y^3*u*v+14669*y^4*u*v-F(49)/107*x^3*z*u*v+F(59)/47*x^2*y*z*u*v+F(31)/65*x*y^2*z*u*v+277*y^3*z*u*v+F(59)/99*x^2*z^2*u*v+F(43)/56*x*y*z^2*u*v-F(92)/55*y^2*z^2*u*v-F(87)/79*x*z^3*u*v+9532*y*z^3*u*v+F(61)/15*z^4*u*v+F(43)/125*x^3*u^2*v+9339*x^2*y*u^2*v-F(7)/118*x*y^2*u^2*v+F(85)/86*y^3*u^2*v+F(11)/124*x^2*z*u^2*v-F(99)/14*x*y*z*u^2*v-8641*y^2*z*u^2*v-F(55)/49*x*z^2*u^2*v-F(68)/21*y*z^2*u^2*v+10063*z^3*u^2*v+F(108)/47*x^2*u^3*v+F(17)/59*x*y*u^3*v-15003*y^2*u^3*v+F(26)/67*x*z*u^3*v-F(81)/49*y*z*u^3*v-5154*z^2*u^3*v+F(61)/124*x*u^4*v+F(51)/50*y*u^4*v+F(92)/49*z*u^4*v+F(12)/115*u^5*v-574*x^4*v^2+F(122)/21*x^3*y*v^2-92*x^2*y^2*v^2+F(38)/25*x*y^3*v^2+1946*y^4*v^2+961*x^3*z*v^2+F(14)/37*x^2*y*z*v^2+F(64)/111*x*y^2*z*v^2-2686*y^3*z*v^2+12143*x^2*z^2*v^2-F(61)/46*x*y*z^2*v^2-3212*y^2*z^2*v^2-F(79)/77*x*z^3*v^2+7261*y*z^3*v^2-11565*z^4*v^2-15143*x^3*u*v^2-3528*x^2*y*u*v^2+F(29)/113*x*y^2*u*v^2-F(81)/23*y^3*u*v^2-F(4)/77*x^2*z*u*v^2+14437*x*y*z*u*v^2+F(42)/19*y^2*z*u*v^2+14108*x*z^2*u*v^2+1517*y*z^2*u*v^2-8988*z^3*u*v^2+F(106)/95*x^2*u^2*v^2+F(78)/61*x*y*u^2*v^2-13276*y^2*u^2*v^2-F(91)/62*x*z*u^2*v^2-F(16)/105*y*z*u^2*v^2+F(64)/67*z^2*u^2*v^2-4732*x*u^3*v^2+F(106)/13*y*u^3*v^2-9501*z*u^3*v^2-1709*u^4*v^2-F(9)/29*x^3*v^3+F(123)/95*x^2*y*v^3-8131*x*y^2*v^3-F(73)/51*y^3*v^3-F(84)/107*x^2*z*v^3-9721*x*y*z*v^3+F(8)/35*y^2*z*v^3-8900*x*z^2*v^3-4228*y*z^2*v^3-F(67)/45*z^3*v^3-1357*x^2*u*v^3-7806*x*y*u*v^3+12990*y^2*u*v^3+15074*x*z*u*v^3+11207*y*z*u*v^3-F(61)/103*z^2*u*v^3+1564*x*u^2*v^3+F(104)/45*y*u^2*v^3+F(125)/74*z*u^2*v^3-F(43)/19*u^3*v^3+F(38)/45*x^2*v^4-F(51)/25*x*y*v^4+F(65)/28*y^2*v^4-F(82)/39*x*z*v^4+13182*y*z*v^4+F(91)/5*z^2*v^4-F(44)/103*x*u*v^4+F(30)/77*y*u*v^4-1131*z*u*v^4+8044*u^2*v^4-15559*x*v^5+F(118)/49*y*v^5-F(47)/33*z*v^5-1556*u*v^5+F(125)/79*v^6,-F(125)/112*x^4*y*z-4245*x^3*y^2*z-12478*x^2*y^3*z+F(11)/108*x*y^4*z-F(99)/86*y^5*z+F(101)/3*x^4*z^2-F(111)/100*x^3*y*z^2+11245*x^2*y^2*z^2-F(13)/101*x*y^3*z^2-F(28)/93*y^4*z^2+F(25)/11*x^3*z^3-13217*x^2*y*z^3+15532*x*y^2*z^3-F(55)/8*y^3*z^3+F(28)/29*x^2*z^4-676*x*y*z^4+F(67)/93*y^2*z^4-F(7)/111*x*z^5-14636*y*z^5-F(71)/120*z^6+14783*x^4*y*u+F(42)/5*x^3*y^2*u+F(39)/107*x^2*y^3*u-9910*x*y^4*u-7896*y^5*u+F(126)/41*x^4*z*u+723*x^3*y*z*u-1460*x^2*y^2*z*u+F(47)/104*x*y^3*z*u+1768*y^4*z*u+F(108)/107*x^3*z^2*u-11950*x^2*y*z^2*u+F(14)/39*x*y^2*z^2*u+F(111)/44*y^3*z^2*u-2572*x^2*z^3*u+F(1)/88*x*y*z^3*u+F(91)/120*y^2*z^3*u-5692*x*z^4*u-3133*y*z^4*u+F(67)/97*z^5*u-F(103)/84*x^4*u^2-9038*x^3*y*u^2-F(113)/34*x^2*y^2*u^2-9400*x*y^3*u^2-F(62)/9*y^4*u^2+F(109)/103*x^3*z*u^2+F(41)/45*x^2*y*z*u^2+3183*x*y^2*z*u^2+15213*y^3*z*u^2+15923*x^2*z^2*u^2-6481*x*y*z^2*u^2-F(72)/25*y^2*z^2*u^2+F(118)/83*x*z^3*u^2-13079*y*z^3*u^2-3539*z^4*u^2+F(75)/83*x^3*u^3-F(115)/104*x^2*y*u^3+F(61)/13*x*y^2*u^3+1624*y^3*u^3-14433*x^2*z*u^3+F(49)/60*x*y*z*u^3-F(111)/70*y^2*z*u^3-F(23)/20*x*z^2*u^3+F(87)/13*y*z^2*u^3-F(105)/106*z^3*u^3+F(117)/119*x^2*u^4+F(99)/2*x*y*u^4+9321*y^2*u^4+F(64)/93*x*z*u^4+F(123)/83*y*z*u^4+F(30)/43*z^2*u^4+F(100)/47*x*u^5+4791*y*u^5+F(15)/94*z*u^5+F(87)/115*u^6-F(119)/86*x^4*y*v+F(85)/49*x^3*y^2*v+15878*x^2*y^3*v+12840*x*y^4*v-F(8)/43*y^5*v+14456*x^4*z*v+F(3)/64*x^3*y*z*v+F(32)/55*x^2*y^2*z*v-F(19)/18*x*y^3*z*v-F(57)/67*y^4*z*v-5803*x^3*z^2*v-15872*x^2*y*z^2*v-F(61)/47*x*y^2*z^2*v+F(41)/47*y^3*z^2*v-4671*x^2*z^3*v-F(54)/107*x*y*z^3*v-F(5)/48*y^2*z^3*v-F(35)/67*x*z^4*v-11950*y*z^4*v-F(13)/54*z^5*v+F(37)/66*x^4*u*v-31*x^3*y*u*v-F(87)/80*x^2*y^2*u*v+F(7)/26*x*y^3*u*v+F(36)/89*y^4*u*v-F(23)/47*x^3*z*u*v+F(83)/86*x^2*y*z*u*v-F(61)/64*x*y^2*z*u*v-12883*y^3*z*u*v-F(1)/27*x^2*z^2*u*v-6559*x*y*z^2*u*v+4698*y^2*z^2*u*v-F(62)/57*x*z^3*u*v+F(103)/7*y*z^3*u*v+F(41)/103*z^4*u*v+5955*x^3*u^2*v-F(4)/9*x^2*y*u^2*v+F(13)/12*x*y^2*u^2*v-9986*y^3*u^2*v+14453*x^2*z*u^2*v+F(64)/75*x*y*z*u^2*v-8502*y^2*z*u^2*v-3166*x*z^2*u^2*v-11737*y*z^2*u^2*v+F(37)/29*z^3*u^2*v-10380*x^2*u^3*v+14306*x*y*u^3*v+15885*y^2*u^3*v+F(78)/95*x*z*u^3*v-F(83)/17*y*z*u^3*v-11117*z^2*u^3*v+9787*x*u^4*v+F(13)/60*y*u^4*v-715*z*u^4*v-F(33)/31*u^5*v-15010*x^4*v^2-9320*x^3*y*v^2-1544*x^2*y^2*v^2+12026*x*y^3*v^2+F(91)/94*y^4*v^2-3750*x^3*z*v^2-481*x^2*y*z*v^2-F(123)/11*x*y^2*z*v^2-F(41)/96*y^3*z*v^2-F(9)/85*x^2*z^2*v^2+12673*x*y*z^2*v^2+F(22)/87*y^2*z^2*v^2+F(101)/111*x*z^3*v^2+F(31)/100*y*z^3*v^2+15891*z^4*v^2+12068*x^3*u*v^2+3798*x^2*y*u*v^2-12160*x*y^2*u*v^2-1551*y^3*u*v^2-F(58)/11*x^2*z*u*v^2+F(89)/49*x*y*z*u*v^2+2095*y^2*z*u*v^2+F(15)/14*x*z^2*u*v^2-7565*y*z^2*u*v^2+9717*z^3*u*v^2+1436*x^2*u^2*v^2-5509*x*y*u^2*v^2-F(64)/65*y^2*u^2*v^2-F(26)/71*x*z*u^2*v^2-12368*y*z*u^2*v^2-F(24)/17*z^2*u^2*v^2+F(3)/31*x*u^3*v^2+F(106)/19*y*u^3*v^2+F(115)/18*z*u^3*v^2+F(43)/90*u^4*v^2-F(56)/97*x^3*v^3+11871*x^2*y*v^3+5364*x*y^2*v^3+F(99)/92*y^3*v^3+869*x^2*z*v^3+F(94)/17*x*y*z*v^3-775*y^2*z*v^3-F(57)/77*x*z^2*v^3+F(89)/34*y*z^2*v^3-12839*z^3*v^3-82*x^2*u*v^3-14500*x*y*u*v^3+97*y^2*u*v^3-F(51)/25*x*z*u*v^3+12973*y*z*u*v^3-4245*z^2*u*v^3-14776*x*u^2*v^3+F(39)/71*y*u^2*v^3-14367*z*u^2*v^3+13850*u^3*v^3-14939*x^2*v^4+F(31)/72*x*y*v^4+1672*y^2*v^4+F(93)/11*x*z*v^4-F(61)/108*y*z*v^4+12228*z^2*v^4-F(43)/119*x*u*v^4+7191*y*u*v^4+F(47)/80*z*u*v^4-F(107)/123*u^2*v^4+F(86)/23*x*v^5-13520*y*v^5+F(109)/95*z*v^5-F(95)/101*u*v^5+11473*v^6,6438*x^4*y*z-306*x^3*y^2*z+F(73)/65*x^2*y^3*z-3640*x*y^4*z+F(93)/100*y^5*z+4960*x^4*z^2+F(67)/18*x^3*y*z^2-1557*x^2*y^2*z^2-F(102)/13*x*y^3*z^2-10954*y^4*z^2+3685*x^3*z^3+F(27)/118*x^2*y*z^3+8791*x*y^2*z^3-8803*y^3*z^3-F(53)/110*x^2*z^4-9117*x*y*z^4+3256*y^2*z^4-F(121)/67*x*z^5+12400*y*z^5+F(31)/115*z^6-F(101)/69*x^4*y*u-F(31)/107*x^3*y^2*u+F(103)/12*x^2*y^3*u-F(77)/48*x*y^4*u-F(1)/17*y^5*u-F(79)/2*x^4*z*u-F(7)/96*x^3*y*z*u-9817*x^2*y^2*z*u-F(45)/43*x*y^3*z*u-3113*y^4*z*u+15528*x^3*z^2*u+4518*x^2*y*z^2*u-6086*x*y^2*z^2*u+5672*y^3*z^2*u+F(121)/65*x^2*z^3*u+F(29)/119*x*y*z^3*u-F(19)/87*y^2*z^3*u+F(118)/19*x*z^4*u-F(54)/95*y*z^4*u+6159*z^5*u+F(115)/56*x^4*u^2-197*x^3*y*u^2-F(105)/83*x^2*y^2*u^2-F(31)/18*x*y^3*u^2+15702*y^4*u^2+15545*x^3*z*u^2+F(49)/116*x^2*y*z*u^2-F(79)/107*x*y^2*z*u^2-6977*y^3*z*u^2+F(23)/87*x^2*z^2*u^2+F(43)/75*x*y*z^2*u^2+10735*y^2*z^2*u^2+F(55)/113*x*z^3*u^2+11136*y*z^3*u^2-F(11)/40*z^4*u^2+F(45)/64*x^3*u^3+9231*x^2*y*u^3-F(58)/3*x*y^2*u^3-F(58)/75*y^3*u^3-9558*x^2*z*u^3-14306*x*y*z*u^3-3256*y^2*z*u^3+F(122)/107*x*z^2*u^3-12610*y*z^2*u^3+F(17)/78*z^3*u^3+14107*x^2*u^4+F(23)/83*x*y*u^4-13164*y^2*u^4-F(68)/95*x*z*u^4-F(43)/75*y*z*u^4+F(37)/99*z^2*u^4+F(84)/85*x*u^5-F(19)/115*y*u^5-F(10)/43*z*u^5+F(3)/97*u^6-4906*x^4*y*v+F(49)/92*x^3*y^2*v-11111*x^2*y^3*v-F(45)/74*x*y^4*v-14898*y^5*v+11370*x^4*z*v-F(23)/6*x^3*y*z*v-F(92)/113*x^2*y^2*z*v+9035*x*y^3*z*v+F(48)/13*y^4*z*v+F(55)/4*x^3*z^2*v+12832*x^2*y*z^2*v+7394*x*y^2*z^2*v-14712*y^3*z^2*v-F(116)/91*x^2*z^3*v-F(123)/7*x*y*z^3*v+F(76)/97*y^2*z^3*v-12154*x*z^4*v-F(93)/20*y*z^4*v-243*z^5*v+6178*x^4*u*v-F(6)/91*x^3*y*u*v-F(24)/115*x^2*y^2*u*v-F(108)/43*x*y^3*u*v+F(101)/113*y^4*u*v+F(13)/30*x^3*z*u*v-1966*x^2*y*z*u*v-F(46)/71*x*y^2*z*u*v-F(53)/44*y^3*z*u*v-F(106)/63*x^2*z^2*u*v-F(110)/39*x*y*z^2*u*v-F(19)/25*y^2*z^2*u*v-3350*x*z^3*u*v+7216*y*z^3*u*v-F(54)/5*z^4*u*v-F(25)/23*x^3*u^2*v+F(67)/110*x^2*y*u^2*v+8933*x*y^2*u^2*v+F(112)/81*y^3*u^2*v-12110*x^2*z*u^2*v+7890*x*y*z*u^2*v+F(61)/20*y^2*z*u^2*v+F(31)/100*x*z^2*u^2*v-F(111)/122*y*z^2*u^2*v-2145*z^3*u^2*v+F(1)/70*x^2*u^3*v-F(67)/55*x*y*u^3*v+F(52)/61*y^2*u^3*v-F(75)/103*x*z*u^3*v+776*y*z*u^3*v+F(5)/43*z^2*u^3*v-F(85)/96*x*u^4*v-F(117)/61*y*u^4*v-F(83)/3*z*u^4*v+15532*u^5*v+F(81)/121*x^4*v^2-12974*x^3*y*v^2+F(79)/99*x^2*y^2*v^2-F(98)/69*x*y^3*v^2-F(107)/113*y^4*v^2-F(101)/125*x^3*z*v^2+3218*x^2*y*z*v^2+F(47)/115*x*y^2*z*v^2+F(12)/113*y^3*z*v^2+F(44)/25*x^2*z^2*v^2-4647*x*y*z^2*v^2-F(123)/35*y^2*z^2*v^2+4214*x*z^3*v^2-251*y*z^3*v^2-F(81)/94*z^4*v^2-F(118)/35*x^3*u*v^2-F(103)/40*x^2*y*u*v^2-7572*x*y^2*u*v^2-13638*y^3*u*v^2+F(7)/9*x^2*z*u*v^2-F(37)/106*x*y*z*u*v^2-15029*y^2*z*u*v^2+F(88)/117*x*z^2*u*v^2-F(59)/64*y*z^2*u*v^2+F(71)/38*z^3*u*v^2+F(45)/11*x^2*u^2*v^2+F(61)/11*x*y*u^2*v^2-F(107)/119*y^2*u^2*v^2+14493*x*z*u^2*v^2+F(51)/43*y*z*u^2*v^2-F(67)/76*z^2*u^2*v^2-564*x*u^3*v^2+F(101)/15*y*u^3*v^2+13186*z*u^3*v^2+1190*u^4*v^2+F(67)/25*x^3*v^3-4700*x^2*y*v^3+14005*x*y^2*v^3-15273*y^3*v^3-F(22)/35*x^2*z*v^3-F(20)/71*x*y*z*v^3+8543*y^2*z*v^3+10920*x*z^2*v^3+5021*y*z^2*v^3+F(80)/87*z^3*v^3+F(17)/52*x^2*u*v^3+6743*x*y*u*v^3+2951*y^2*u*v^3-F(83)/36*x*z*u*v^3+9823*y*z*u*v^3-F(80)/69*z^2*u*v^3-8942*x*u^2*v^3+F(94)/57*y*u^2*v^3-1018*z*u^2*v^3+F(64)/89*u^3*v^3+14388*x^2*v^4-F(124)/85*x*y*v^4+F(4)/59*y^2*v^4-5386*x*z*v^4-F(94)/123*y*z*v^4+10805*z^2*v^4-F(103)/122*x*u*v^4-F(92)/29*y*u*v^4+F(15)/58*z*u*v^4+F(16)/95*u^2*v^4+F(39)/73*x*v^5+11489*y*v^5+F(15)/49*z*v^5+14520*u*v^5+12912*v^6,-F(83)/121*x^4*y-657*x^3*y^2-F(110)/71*x^2*y^3-F(1)/108*x*y^4+7953*y^5+12099*x^4*z+F(81)/110*x^3*y*z+F(118)/43*x^2*y^2*z+F(41)/17*x*y^3*z+F(46)/65*y^4*z-F(9)/43*x^3*z^2+F(83)/39*x^2*y*z^2-F(69)/31*x*y^2*z^2-2707*y^3*z^2-2637*x^2*z^3+12431*x*y*z^3+F(75)/22*y^2*z^3+F(40)/17*x*z^4+5405*y*z^4+F(91)/29*z^5+7547*x^4*u+5772*x^3*y*u+F(1)/48*x^2*y^2*u+157*x*y^3*u+1985*y^4*u+6745*x^3*z*u-F(5)/48*x^2*y*z*u+F(111)/29*x*y^2*z*u+5165*y^3*z*u+8304*x^2*z^2*u-F(77)/60*x*y*z^2*u-15450*y^2*z^2*u-F(19)/75*x*z^3*u-1987*y*z^3*u-39*z^4*u-F(83)/121*x^3*u^2+3589*x^2*y*u^2+F(66)/47*x*y^2*u^2-F(79)/118*y^3*u^2-9164*x^2*z*u^2+F(47)/72*x*y*z*u^2-F(63)/115*y^2*z*u^2-2890*x*z^2*u^2+9781*y*z^2*u^2+F(100)/73*z^3*u^2-2472*x^2*u^3-4376*x*y*u^3-F(119)/29*y^2*u^3-F(119)/29*x*z*u^3+F(83)/35*y*z*u^3+15041*z^2*u^3+F(102)/61*x*u^4-3711*y*u^4-8375*z*u^4+11164*u^5-F(102)/47*x^4*v-8231*x^3*y*v-F(42)/83*x^2*y^2*v-F(93)/41*x*y^3*v-9201*y^4*v+14431*x^3*z*v-9865*x^2*y*z*v+9832*x*y^2*z*v-F(76)/115*y^3*z*v+F(125)/86*x^2*z^2*v+F(13)/61*x*y*z^2*v-F(4)/57*y^2*z^2*v-F(111)/119*x*z^3*v+F(61)/84*y*z^3*v+F(125)/124*z^4*v-F(103)/52*x^3*u*v+F(75)/19*x^2*y*u*v+5872*x*y^2*u*v+12835*y^3*u*v-F(123)/110*x^2*z*u*v-F(74)/103*x*y*z*u*v-F(19)/11*y^2*z*u*v-F(46)/75*x*z^2*u*v-F(89)/39*y*z^2*u*v-4910*z^3*u*v-12058*x^2*u^2*v-F(89)/18*x*y*u^2*v+F(8)/89*y^2*u^2*v+F(47)/45*x*z*u^2*v+3856*y*z*u^2*v+4482*z^2*u^2*v-204*x*u^3*v-F(21)/115*y*u^3*v-7174*z*u^3*v-F(32)/29*u^4*v+3864*x^3*v^2-F(74)/53*x^2*y*v^2-F(15)/62*x*y^2*v^2-F(44)/113*y^3*v^2+F(122)/63*x^2*z*v^2-10016*x*y*z*v^2+5044*y^2*z*v^2-3014*x*z^2*v^2+2300*y*z^2*v^2-5037*z^3*v^2-11456*x^2*u*v^2+11281*x*y*u*v^2-4841*y^2*u*v^2-F(97)/121*x*z*u*v^2-F(87)/109*y*z*u*v^2+F(2)/93*z^2*u*v^2-F(71)/80*x*u^2*v^2-F(76)/53*y*u^2*v^2+5761*z*u^2*v^2+11356*u^3*v^2+1695*x^2*v^3+F(25)/9*x*y*v^3+7216*y^2*v^3+F(23)/80*x*z*v^3-6993*y*z*v^3+3399*z^2*v^3+F(58)/97*x*u*v^3-F(28)/9*y*u*v^3+5356*z*u*v^3+F(7)/103*u^2*v^3-F(39)/100*x*v^4-F(97)/38*y*v^4+F(67)/12*z*v^4+11795*u*v^4-12507*v^5,-F(77)/61*x^4*y+7637*x^3*y^2+4436*x^2*y^3+F(37)/105*x*y^4-125*y^5+F(13)/59*x^4*z+F(70)/117*x^3*y*z-4214*x^2*y^2*z-14483*x*y^3*z-F(10)/73*y^4*z+7901*x^3*z^2-F(17)/81*x^2*y*z^2+F(43)/6*x*y^2*z^2-6846*y^3*z^2+F(58)/47*x^2*z^3-F(103)/61*x*y*z^3+F(46)/87*y^2*z^3-F(57)/64*x*z^4-F(101)/12*y*z^4-519*z^5+14782*x^4*u+F(14)/95*x^3*y*u+7672*x^2*y^2*u+13825*x*y^3*u+F(38)/93*y^4*u+F(63)/29*x^3*z*u-10501*x^2*y*z*u-F(8)/11*x*y^2*z*u+F(7)/115*y^3*z*u+14862*x^2*z^2*u+601*x*y*z^2*u+14637*y^2*z^2*u-8056*x*z^3*u-F(43)/60*y*z^3*u+F(66)/53*z^4*u+F(59)/126*x^3*u^2+F(44)/91*x^2*y*u^2+F(18)/97*x*y^2*u^2-F(120)/13*y^3*u^2-F(75)/23*x^2*z*u^2+548*x*y*z*u^2-F(74)/33*y^2*z*u^2-7705*x*z^2*u^2-13317*y*z^2*u^2-9920*z^3*u^2-5961*x^2*u^3-6948*x*y*u^3+F(87)/95*y^2*u^3+7943*x*z*u^3-9399*y*z*u^3-F(59)/27*z^2*u^3+F(71)/18*x*u^4-F(5)/83*y*u^4-F(29)/126*z*u^4-F(124)/15*u^5+F(118)/123*x^4*v-F(41)/81*x^3*y*v-15874*x^2*y^2*v+F(67)/20*x*y^3*v+13849*y^4*v+4075*x^3*z*v-13357*x^2*y*z*v+3125*x*y^2*z*v-F(59)/51*y^3*z*v+F(101)/31*x^2*z^2*v-F(50)/101*x*y*z^2*v-F(109)/71*y^2*z^2*v-3144*x*z^3*v+F(121)/32*y*z^3*v+F(74)/121*z^4*v+F(89)/36*x^3*u*v+F(122)/7*x^2*y*u*v-F(35)/18*x*y^2*u*v+1299*y^3*u*v+15679*x^2*z*u*v-F(23)/13*x*y*z*u*v-F(7)/22*y^2*z*u*v+8459*x*z^2*u*v+F(16)/97*y*z^2*u*v-F(66)/97*z^3*u*v+F(26)/37*x^2*u^2*v-F(79)/99*x*y*u^2*v-F(88)/51*y^2*u^2*v+F(11)/72*x*z*u^2*v+F(125)/101*y*z*u^2*v+F(79)/105*z^2*u^2*v+6672*x*u^3*v-F(26)/89*y*u^3*v+8952*z*u^3*v-5086*u^4*v+15672*x^3*v^2+F(39)/35*x^2*y*v^2+F(75)/29*x*y^2*v^2+F(10)/39*y^3*v^2-F(77)/23*x^2*z*v^2-5306*x*y*z*v^2+15289*y^2*z*v^2-13792*x*z^2*v^2+F(85)/84*y*z^2*v^2+6254*z^3*v^2-F(55)/4*x^2*u*v^2-5371*x*y*u*v^2-6045*y^2*u*v^2-F(97)/4*x*z*u*v^2+584*y*z*u*v^2-F(2)/73*z^2*u*v^2-14590*x*u^2*v^2+1677*y*u^2*v^2-1986*z*u^2*v^2+1019*u^3*v^2+12041*x^2*v^3-11647*x*y*v^3+F(94)/57*y^2*v^3-12944*x*z*v^3+8056*y*z*v^3-F(5)/52*z^2*v^3-F(23)/50*x*u*v^3-5354*y*u*v^3+4191*z*u*v^3+F(29)/119*u^2*v^3-14632*x*v^4+F(41)/90*y*v^4-F(89)/120*z*v^4-11451*u*v^4-2840*v^5,-F(61)/48*x^4*y+F(94)/41*x^3*y^2+12140*x^2*y^3-F(71)/95*x*y^4+F(120)/59*y^5+3448*x^4*z+13482*x^3*y*z+F(89)/10*x^2*y^2*z-10482*x*y^3*z+12475*y^4*z+7838*x^3*z^2+2715*x^2*y*z^2-F(91)/103*x*y^2*z^2+9505*y^3*z^2+F(46)/21*x^2*z^3-12751*x*y*z^3-F(69)/124*y^2*z^3+F(113)/21*x*z^4+1249*y*z^4-15658*z^5+F(43)/88*x^4*u+F(100)/33*x^3*y*u-F(47)/126*x^2*y^2*u-F(33)/95*x*y^3*u+F(37)/113*y^4*u-F(91)/118*x^3*z*u-F(95)/116*x^2*y*z*u+F(44)/81*x*y^2*z*u-F(70)/31*y^3*z*u-F(39)/119*x^2*z^2*u+F(67)/117*x*y*z^2*u+10343*y^2*z^2*u-F(121)/103*x*z^3*u-402*y*z^3*u+12765*z^4*u+F(31)/118*x^3*u^2+9082*x^2*y*u^2-8303*x*y^2*u^2-1383*y^3*u^2-F(31)/87*x^2*z*u^2-15001*x*y*z*u^2+F(72)/59*y^2*z*u^2-F(112)/73*x*z^2*u^2-12960*y*z^2*u^2+15736*z^3*u^2-F(75)/119*x^2*u^3+F(43)/33*x*y*u^3+7672*y^2*u^3+F(111)/73*x*z*u^3+958*y*z*u^3+F(52)/33*z^2*u^3-7736*x*u^4+2144*y*u^4-F(89)/21*z*u^4-8500*u^5+211*x^4*v+15286*x^3*y*v+F(59)/30*x^2*y^2*v+13871*x*y^3*v+F(117)/86*y^4*v-F(78)/95*x^3*z*v+3013*x^2*y*z*v+6106*x*y^2*z*v+5522*y^3*z*v-11796*x^2*z^2*v+F(74)/125*x*y*z^2*v+F(70)/57*y^2*z^2*v-F(45)/58*x*z^3*v-F(114)/65*y*z^3*v+F(69)/25*z^4*v-10989*x^3*u*v-F(68)/63*x^2*y*u*v+F(69)/50*x*y^2*u*v+5603*y^3*u*v+F(89)/8*x^2*z*u*v+F(55)/123*x*y*z*u*v-F(32)/87*y^2*z*u*v+F(123)/32*x*z^2*u*v-F(29)/88*y*z^2*u*v-F(121)/54*z^3*u*v+F(64)/73*x^2*u^2*v+F(11)/82*x*y*u^2*v-F(17)/3*y^2*u^2*v-F(23)/86*x*z*u^2*v+F(19)/53*y*z*u^2*v-12548*z^2*u^2*v-7219*x*u^3*v-F(115)/53*y*u^3*v+5733*z*u^3*v-F(121)/104*u^4*v-F(79)/123*x^3*v^2+10980*x^2*y*v^2+1865*x*y^2*v^2-F(7)/27*y^3*v^2-F(103)/59*x^2*z*v^2-F(19)/59*x*y*z*v^2-9076*y^2*z*v^2-F(107)/63*x*z^2*v^2+2346*y*z^2*v^2-10247*z^3*v^2-F(87)/62*x^2*u*v^2+F(7)/38*x*y*u*v^2+3028*y^2*u*v^2-F(89)/98*x*z*u*v^2+14807*y*z*u*v^2-F(59)/102*z^2*u*v^2+10013*x*u^2*v^2-F(44)/63*y*u^2*v^2-5718*z*u^2*v^2-F(59)/119*u^3*v^2+F(72)/77*x^2*v^3+F(97)/9*x*y*v^3-F(2)/107*y^2*v^3-8642*x*z*v^3+146*y*z*v^3+6025*z^2*v^3-F(97)/116*x*u*v^3-2518*y*u*v^3+7191*z*u*v^3+5411*u^2*v^3-F(121)/6*x*v^4+F(77)/122*y*v^4+F(61)/80*z*v^4+11509*u*v^4+12101*v^5,F(83)/121*x^5+F(13)/73*x^4*y+8698*x^3*y^2+F(109)/11*x^2*y^3+F(32)/15*x*y^4+9962*y^5-F(117)/77*x^4*z-3883*x^3*y*z+9806*x^2*y^2*z-1158*x*y^3*z+F(32)/21*y^4*z+5523*x^3*z^2-F(79)/46*x^2*y*z^2-1617*x*y^2*z^2+F(77)/58*y^3*z^2-F(3)/37*x^2*z^3+15869*x*y*z^3-F(56)/65*y^2*z^3+F(79)/65*x*z^4-3845*y*z^4-4427*z^5+F(113)/8*x^4*u-6752*x^3*y*u+6310*x^2*y^2*u+12115*x*y^3*u+F(100)/77*y^4*u+3150*x^3*z*u-14636*x^2*y*z*u-15674*x*y^2*z*u-F(20)/27*y^3*z*u-2302*x^2*z^2*u-F(70)/71*x*y*z^2*u+F(95)/43*y^2*z^2*u+4280*x*z^3*u-F(101)/83*y*z^3*u-F(61)/88*z^4*u+F(23)/48*x^3*u^2-12868*x^2*y*u^2+4355*x*y^2*u^2-F(34)/89*y^3*u^2-F(74)/125*x^2*z*u^2-F(61)/22*x*y*z*u^2-F(85)/117*y^2*z*u^2+F(101)/57*x*z^2*u^2-F(69)/107*y*z^2*u^2-F(33)/103*z^3*u^2+F(25)/24*x^2*u^3+7005*x*y*u^3-1734*y^2*u^3-F(117)/110*x*z*u^3+F(65)/98*y*z*u^3+3618*z^2*u^3+F(79)/87*x*u^4-F(123)/119*y*u^4-1611*z*u^4-F(77)/64*u^5+4418*x^4*v-F(63)/67*x^3*y*v-F(11)/59*x^2*y^2*v-F(99)/8*x*y^3*v+14754*y^4*v+F(108)/61*x^3*z*v+F(81)/25*x^2*y*z*v-5755*x*y^2*z*v-7787*y^3*z*v+F(103)/16*x^2*z^2*v+13462*x*y*z^2*v+4401*y^2*z^2*v+F(24)/61*x*z^3*v+F(44)/53*y*z^3*v-F(25)/28*z^4*v-F(97)/56*x^3*u*v-12761*x^2*y*u*v-F(40)/49*x*y^2*u*v-12461*y^3*u*v-F(82)/125*x^2*z*u*v+5295*x*y*z*u*v+F(19)/107*y^2*z*u*v-899*x*z^2*u*v-F(104)/29*y*z^2*u*v+F(37)/76*z^3*u*v-11524*x^2*u^2*v-4255*x*y*u^2*v+F(11)/37*y^2*u^2*v+F(117)/23*x*z*u^2*v-F(54)/125*y*z*u^2*v-F(73)/74*z^2*u^2*v-F(124)/79*x*u^3*v-4728*y*u^3*v+13423*z*u^3*v+12278*u^4*v+F(81)/16*x^3*v^2-F(116)/33*x^2*y*v^2+2298*x*y^2*v^2-F(13)/49*y^3*v^2+43*x^2*z*v^2-12576*x*y*z*v^2+F(19)/51*y^2*z*v^2+15785*x*z^2*v^2+56*y*z^2*v^2-6297*z^3*v^2-2497*x^2*u*v^2+F(113)/60*x*y*u*v^2+F(46)/117*y^2*u*v^2+3662*x*z*u*v^2-F(11)/73*y*z*u*v^2-F(7)/30*z^2*u*v^2-4081*x*u^2*v^2+8304*y*u^2*v^2+15900*z*u^2*v^2+F(2)/57*u^3*v^2-F(49)/22*x^2*v^3+F(8)/105*x*y*v^3-F(77)/20*y^2*v^3+1071*x*z*v^3-F(107)/62*y*z*v^3+F(32)/41*z^2*v^3-8762*x*u*v^3+F(15)/16*y*u*v^3+F(35)/106*z*u*v^3+F(27)/2*u^2*v^3+F(41)/102*x*v^4+F(113)/102*y*v^4-F(19)/123*z*v^4-10720*u*v^4+F(41)/98*v^5]))
end


@doc raw"""
    k3_d14_pi19()

Return a smooth K3 surface in $\mathbb P^4$ with degree `14` and sectional genus `19`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d14_pi19()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^4*y^2+F(52)/25*x^3*y^3+F(73)/79*x^2*y^4+F(33)/83*x*y^5-F(116)/65*y^6+F(23)/39*x^4*y*z-F(45)/11*x^3*y^2*z-F(45)/16*x^2*y^3*z+F(73)/8*x*y^4*z+F(107)/70*y^5*z+F(115)/8*x^4*z^2-4619*x^3*y*z^2+13504*x^2*y^2*z^2+F(9)/118*x*y^3*z^2-F(113)/64*y^4*z^2+10731*x^3*z^3-F(7)/13*x^2*y*z^3-F(26)/17*x*y^2*z^3+F(3)/59*y^3*z^3-4602*x^2*z^4+F(71)/44*x*y*z^4-F(66)/43*y^2*z^4-F(37)/70*x*z^5-6988*y*z^5-F(123)/29*z^6-6158*x^4*y*u+9171*x^3*y^2*u+F(71)/122*x^2*y^3*u-F(119)/73*x*y^4*u-15409*y^5*u+F(85)/3*x^4*z*u+F(104)/59*x^3*y*z*u-3336*x^2*y^2*z*u-F(50)/107*x*y^3*z*u-10232*y^4*z*u-8965*x^3*z^2*u-2736*x^2*y*z^2*u+F(4)/61*x*y^2*z^2*u+F(49)/92*y^3*z^2*u+13261*x^2*z^3*u+F(64)/31*x*y*z^3*u-F(89)/70*y^2*z^3*u+11080*x*z^4*u+10139*y*z^4*u+13653*z^5*u+F(34)/25*x^4*u^2-5678*x^3*y*u^2+F(43)/13*x^2*y^2*u^2+4119*x*y^3*u^2+F(72)/109*y^4*u^2-F(77)/64*x^3*z*u^2+15044*x^2*y*z*u^2+F(97)/53*x*y^2*z*u^2+9036*y^3*z*u^2-10599*x^2*z^2*u^2-F(107)/126*x*y*z^2*u^2+F(43)/12*y^2*z^2*u^2-4690*x*z^3*u^2-F(43)/75*y*z^3*u^2-15886*z^4*u^2-F(113)/7*x^3*u^3-242*x^2*y*u^3+F(85)/19*x*y^2*u^3-F(20)/57*y^3*u^3+F(109)/37*x^2*z*u^3+6103*x*y*z*u^3-F(51)/43*y^2*z*u^3-F(89)/92*x*z^2*u^3-F(14)/87*y*z^2*u^3+F(59)/111*z^3*u^3-F(40)/63*x^2*u^4+F(113)/86*x*y*u^4-F(17)/109*y^2*u^4-11039*x*z*u^4+F(107)/31*y*z*u^4+F(121)/47*z^2*u^4-F(123)/5*x*u^5+F(83)/97*y*u^5+F(110)/23*z*u^5-F(122)/83*u^6-F(15)/17*x^4*y*v+F(96)/65*x^3*y^2*v-1491*x^2*y^3*v+3885*x*y^4*v+F(26)/87*y^5*v-F(51)/89*x^4*z*v-F(123)/88*x^3*y*z*v-2315*x^2*y^2*z*v-4119*x*y^3*z*v-3390*y^4*z*v-F(68)/5*x^3*z^2*v-F(57)/67*x^2*y*z^2*v-F(116)/77*x*y^2*z^2*v+F(109)/54*y^3*z^2*v+F(23)/89*x^2*z^3*v+F(19)/77*x*y*z^3*v+F(32)/107*y^2*z^3*v-F(85)/121*x*z^4*v+9325*y*z^4*v+F(87)/32*z^5*v-F(65)/81*x^4*u*v-F(39)/19*x^3*y*u*v+4671*x^2*y^2*u*v+F(54)/77*x*y^3*u*v+F(8)/47*y^4*u*v+1194*x^3*z*u*v+F(5)/14*x^2*y*z*u*v+1151*x*y^2*z*u*v+F(90)/103*y^3*z*u*v-F(55)/8*x^2*z^2*u*v-F(44)/101*x*y*z^2*u*v-4713*y^2*z^2*u*v-F(1)/7*x*z^3*u*v+13344*y*z^3*u*v-11451*z^4*u*v-F(101)/108*x^3*u^2*v+F(42)/55*x^2*y*u^2*v+4810*x*y^2*u^2*v-F(3)/40*y^3*u^2*v+4699*x^2*z*u^2*v+F(114)/53*x*y*z*u^2*v+F(49)/116*y^2*z*u^2*v-F(9)/74*x*z^2*u^2*v-F(84)/109*y*z^2*u^2*v-F(119)/97*z^3*u^2*v+15282*x^2*u^3*v-F(109)/71*x*y*u^3*v+2490*y^2*u^3*v-F(119)/47*x*z*u^3*v+F(43)/40*y*z*u^3*v-F(105)/86*z^2*u^3*v-F(101)/70*x*u^4*v+10041*y*u^4*v-F(93)/53*z*u^4*v+4204*u^5*v+F(83)/43*x^4*v^2+15199*x^3*y*v^2+F(26)/43*x^2*y^2*v^2-6627*x*y^3*v^2+F(59)/113*y^4*v^2+10482*x^3*z*v^2-8354*x^2*y*z*v^2-F(78)/79*x*y^2*z*v^2+7126*y^3*z*v^2+F(68)/25*x^2*z^2*v^2+F(94)/59*x*y*z^2*v^2+F(83)/45*y^2*z^2*v^2-F(59)/60*x*z^3*v^2+15768*y*z^3*v^2+F(15)/29*z^4*v^2-F(43)/100*x^3*u*v^2-11939*x^2*y*u*v^2+1832*x*y^2*u*v^2+F(29)/6*y^3*u*v^2+F(115)/76*x^2*z*u*v^2+F(51)/106*x*y*z*u*v^2+F(7)/23*y^2*z*u*v^2+F(19)/107*x*z^2*u*v^2-13797*y*z^2*u*v^2+F(16)/109*z^3*u*v^2-7519*x^2*u^2*v^2+3472*x*y*u^2*v^2-F(28)/107*y^2*u^2*v^2-11283*x*z*u^2*v^2+12707*y*z*u^2*v^2+F(54)/47*z^2*u^2*v^2+1841*x*u^3*v^2-3037*y*u^3*v^2+9239*z*u^3*v^2+3729*u^4*v^2+7476*x^3*v^3+F(59)/81*x^2*y*v^3-F(49)/85*x*y^2*v^3-F(91)/60*y^3*v^3+F(20)/77*x^2*z*v^3-2761*x*y*z*v^3+4245*y^2*z*v^3+F(97)/75*x*z^2*v^3+F(42)/121*y*z^2*v^3+1822*z^3*v^3-F(97)/16*x^2*u*v^3-F(47)/10*x*y*u*v^3-11222*y^2*u*v^3-2194*x*z*u*v^3+F(116)/21*y*z*u*v^3+F(118)/47*z^2*u*v^3-F(89)/23*x*u^2*v^3+7317*y*u^2*v^3-10812*z*u^2*v^3+F(70)/13*u^3*v^3-5855*x^2*v^4-9755*x*y*v^4+14875*y^2*v^4+F(59)/117*x*z*v^4-F(33)/109*y*z*v^4+F(48)/59*z^2*v^4+F(62)/57*x*u*v^4-F(56)/73*y*u*v^4-3601*z*u*v^4+2155*u^2*v^4-F(77)/107*x*v^5+12478*y*v^5+7028*z*v^5-3652*u*v^5+F(81)/28*v^6,-F(50)/87*x^4*y*z-2298*x^3*y^2*z+9266*x^2*y^3*z-F(55)/16*x*y^4*z+F(31)/76*y^5*z+F(13)/46*x^4*z^2-F(11)/100*x^3*y*z^2+F(90)/119*x^2*y^2*z^2-F(65)/54*x*y^3*z^2-F(102)/77*y^4*z^2-F(76)/109*x^3*z^3-6558*x^2*y*z^3-F(60)/19*x*y^2*z^3-F(17)/113*y^3*z^3-F(74)/43*x^2*z^4+8092*x*y*z^4-13313*y^2*z^4-F(13)/107*x*z^5-F(47)/82*y*z^5+5501*z^6+F(39)/17*x^4*y*u-F(11)/13*x^3*y^2*u-4407*x^2*y^3*u+F(89)/117*x*y^4*u+F(118)/101*y^5*u-F(37)/57*x^4*z*u+F(100)/33*x^3*y*z*u-F(99)/46*x^2*y^2*z*u-2772*x*y^3*z*u-F(75)/94*y^4*z*u-6429*x^3*z^2*u-7628*x^2*y*z^2*u+10112*x*y^2*z^2*u+F(85)/32*y^3*z^2*u-4068*x^2*z^3*u+F(81)/95*x*y*z^3*u+F(97)/89*y^2*z^3*u-12700*x*z^4*u+F(19)/113*y*z^4*u+14306*z^5*u+13878*x^4*u^2-F(22)/21*x^3*y*u^2+F(107)/41*x^2*y^2*u^2-F(16)/97*x*y^3*u^2-2426*y^4*u^2-12247*x^3*z*u^2+F(71)/41*x^2*y*z*u^2+F(16)/69*x*y^2*z*u^2+F(37)/113*y^3*z*u^2-12195*x^2*z^2*u^2-8354*x*y*z^2*u^2-6177*y^2*z^2*u^2-10139*x*z^3*u^2+9118*y*z^3*u^2-F(113)/115*z^4*u^2-8042*x^3*u^3+F(37)/97*x^2*y*u^3+F(7)/30*x*y^2*u^3+F(90)/73*y^3*u^3-14676*x^2*z*u^3+630*x*y*z*u^3+7824*y^2*z*u^3-5947*x*z^2*u^3-12019*y*z^2*u^3+F(99)/19*z^3*u^3+F(17)/117*x^2*u^4+1943*x*y*u^4-F(47)/45*y^2*u^4-F(79)/98*x*z*u^4+F(46)/119*y*z*u^4-F(98)/25*z^2*u^4-F(20)/69*x*u^5+F(45)/28*y*u^5-F(123)/14*z*u^5-F(20)/29*u^6+F(83)/41*x^4*y*v+F(32)/19*x^3*y^2*v+F(5)/107*x^2*y^3*v+F(9)/95*x*y^4*v+F(35)/33*y^5*v+14385*x^4*z*v+2237*x^3*y*z*v+F(109)/11*x^2*y^2*z*v-6489*x*y^3*z*v-F(96)/107*y^4*z*v+1749*x^3*z^2*v+F(23)/64*x^2*y*z^2*v+F(105)/17*x*y^2*z^2*v+F(11)/70*y^3*z^2*v-4677*x^2*z^3*v-F(4)/119*x*y*z^3*v+8243*y^2*z^3*v-7538*x*z^4*v-F(45)/58*y*z^4*v+1768*z^5*v-10804*x^4*u*v-F(102)/89*x^3*y*u*v-F(80)/17*x^2*y^2*u*v-F(103)/66*x*y^3*u*v-F(14)/55*y^4*u*v-5981*x^3*z*u*v+F(2)/35*x^2*y*z*u*v+F(24)/71*x*y^2*z*u*v-F(17)/62*y^3*z*u*v-F(59)/52*x^2*z^2*u*v+468*x*y*z^2*u*v+F(61)/110*y^2*z^2*u*v-1265*x*z^3*u*v-11317*y*z^3*u*v+15610*z^4*u*v+F(61)/116*x^3*u^2*v+1972*x^2*y*u^2*v+F(26)/121*x*y^2*u^2*v+9787*y^3*u^2*v-F(91)/62*x^2*z*u^2*v+F(101)/65*x*y*z*u^2*v-F(85)/111*y^2*z*u^2*v+F(99)/95*x*z^2*u^2*v-F(125)/51*y*z^2*u^2*v+10324*z^3*u^2*v+F(95)/73*x^2*u^3*v-705*x*y*u^3*v-6165*y^2*u^3*v+2215*x*z*u^3*v-F(109)/12*y*z*u^3*v+F(72)/95*z^2*u^3*v-149*x*u^4*v+14154*y*u^4*v+412*z*u^4*v+2747*u^5*v+F(113)/25*x^4*v^2+5988*x^3*y*v^2-F(119)/107*x^2*y^2*v^2-F(70)/41*x*y^3*v^2+2891*y^4*v^2+F(85)/103*x^3*z*v^2-9398*x^2*y*z*v^2-F(116)/117*x*y^2*z*v^2-12527*y^3*z*v^2-8408*x^2*z^2*v^2-F(76)/47*x*y*z^2*v^2+13729*y^2*z^2*v^2-F(65)/77*x*z^3*v^2+F(23)/94*y*z^3*v^2+F(61)/77*z^4*v^2+F(18)/121*x^3*u*v^2-8520*x^2*y*u*v^2+F(103)/34*x*y^2*u*v^2+F(17)/40*y^3*u*v^2+F(23)/104*x^2*z*u*v^2+F(7)/29*x*y*z*u*v^2-5908*y^2*z*u*v^2-7791*x*z^2*u*v^2-12032*y*z^2*u*v^2+F(125)/82*z^3*u*v^2-F(89)/126*x^2*u^2*v^2+F(22)/75*x*y*u^2*v^2-F(118)/67*y^2*u^2*v^2+F(34)/29*x*z*u^2*v^2+F(30)/109*y*z*u^2*v^2-F(64)/37*z^2*u^2*v^2-F(113)/87*x*u^3*v^2+F(110)/109*y*u^3*v^2+F(81)/23*z*u^3*v^2-F(40)/99*u^4*v^2-14204*x^3*v^3+5939*x^2*y*v^3-7749*x*y^2*v^3+2924*y^3*v^3-3662*x^2*z*v^3-F(51)/122*x*y*z*v^3-7218*y^2*z*v^3-13482*x*z^2*v^3+12673*y*z^2*v^3-9675*z^3*v^3+6567*x^2*u*v^3-14008*x*y*u*v^3+14242*y^2*u*v^3-4310*x*z*u*v^3+757*y*z*u*v^3-9110*z^2*u*v^3+F(27)/14*x*u^2*v^3-F(81)/40*y*u^2*v^3-F(42)/29*z*u^2*v^3+7287*u^3*v^3+F(119)/94*x^2*v^4-11926*x*y*v^4-F(82)/117*y^2*v^4+4138*x*z*v^4-F(67)/70*y*z*v^4+F(79)/49*z^2*v^4-F(113)/50*x*u*v^4+F(102)/113*y*u*v^4-4791*z*u*v^4-F(11)/97*u^2*v^4-F(19)/18*x*v^5+6866*y*v^5+F(63)/101*z*v^5+4646*u*v^5-374*v^6,F(109)/29*x^4*y*z+F(94)/43*x^3*y^2*z-13879*x^2*y^3*z-F(53)/73*x*y^4*z+9074*y^5*z+846*x^4*z^2-F(107)/44*x^3*y*z^2+F(62)/71*x^2*y^2*z^2-F(121)/68*x*y^3*z^2+14200*y^4*z^2-F(78)/89*x^3*z^3+F(51)/94*x^2*y*z^3+F(123)/5*x*y^2*z^3-731*y^3*z^3-15645*x^2*z^4-F(91)/25*x*y*z^4+F(39)/17*y^2*z^4+10343*x*z^5-14671*y*z^5+F(101)/71*z^6+10467*x^4*y*u-F(3)/64*x^3*y^2*u+F(75)/31*x^2*y^3*u-F(43)/66*x*y^4*u-F(118)/55*y^5*u+F(7)/25*x^4*z*u-F(59)/95*x^3*y*z*u+11830*x^2*y^2*z*u-F(76)/121*x*y^3*z*u+F(95)/72*y^4*z*u-F(121)/53*x^3*z^2*u+F(120)/7*x^2*y*z^2*u-7033*x*y^2*z^2*u+F(89)/2*y^3*z^2*u+F(86)/47*x^2*z^3*u-F(13)/99*x*y*z^3*u-1447*y^2*z^3*u+6253*x*z^4*u-9082*y*z^4*u-4518*z^5*u+F(31)/111*x^4*u^2+F(68)/79*x^3*y*u^2-3532*x^2*y^2*u^2+F(61)/94*x*y^3*u^2+1210*y^4*u^2-11697*x^3*z*u^2-9386*x^2*y*z*u^2-8263*x*y^2*z*u^2+11759*y^3*z*u^2+F(113)/118*x^2*z^2*u^2-6488*x*y*z^2*u^2+F(41)/30*y^2*z^2*u^2-15179*x*z^3*u^2-10848*y*z^3*u^2-F(14)/51*z^4*u^2-F(13)/123*x^3*u^3-F(21)/10*x^2*y*u^3-4018*x*y^2*u^3-11558*y^3*u^3-F(68)/13*x^2*z*u^3+F(41)/119*x*y*z*u^3-F(94)/31*y^2*z*u^3+F(97)/101*x*z^2*u^3+2551*y*z^2*u^3+F(51)/73*z^3*u^3-10862*x^2*u^4-F(44)/125*x*y*u^4+F(113)/13*y^2*u^4-5704*x*z*u^4+F(65)/116*y*z*u^4-9578*z^2*u^4-F(95)/58*x*u^5+8033*y*u^5-F(79)/124*z*u^5-F(107)/76*u^6+7293*x^4*y*v+15260*x^3*y^2*v+F(32)/35*x^2*y^3*v+8519*x*y^4*v-F(50)/83*y^5*v+F(83)/40*x^4*z*v-6104*x^3*y*z*v+F(83)/121*x^2*y^2*z*v+230*x*y^3*z*v+F(9)/50*y^4*z*v+1507*x^3*z^2*v-F(75)/112*x^2*y*z^2*v-F(20)/7*x*y^2*z^2*v-12689*y^3*z^2*v-F(75)/112*x^2*z^3*v-8673*x*y*z^3*v+F(79)/69*y^2*z^3*v-12453*x*z^4*v-4805*y*z^4*v-F(125)/11*z^5*v-F(22)/97*x^4*u*v-11396*x^3*y*u*v+F(67)/88*x^2*y^2*u*v-F(75)/122*x*y^3*u*v-F(30)/29*y^4*u*v-837*x^3*z*u*v-F(71)/12*x^2*y*z*u*v+F(37)/77*x*y^2*z*u*v+F(78)/59*y^3*z*u*v-13742*x^2*z^2*u*v+13080*x*y*z^2*u*v+643*y^2*z^2*u*v-7517*x*z^3*u*v-15577*y*z^3*u*v+F(52)/75*z^4*u*v+12922*x^3*u^2*v-14629*x^2*y*u^2*v-10188*x*y^2*u^2*v-5113*y^3*u^2*v+F(67)/51*x^2*z*u^2*v-15273*x*y*z*u^2*v-13388*y^2*z*u^2*v+F(121)/109*x*z^2*u^2*v-F(64)/83*y*z^2*u^2*v+F(69)/20*z^3*u^2*v-F(117)/44*x^2*u^3*v-7091*x*y*u^3*v-F(118)/45*y^2*u^3*v+F(73)/26*x*z*u^3*v-F(117)/38*y*z*u^3*v+1448*z^2*u^3*v+F(71)/19*x*u^4*v-12698*y*u^4*v+F(107)/81*z*u^4*v+F(21)/43*u^5*v-F(94)/79*x^4*v^2-1660*x^3*y*v^2-15179*x^2*y^2*v^2-F(10)/71*x*y^3*v^2+9523*y^4*v^2+F(6)/115*x^3*z*v^2-15*x^2*y*z*v^2+15142*x*y^2*z*v^2+6157*y^3*z*v^2+F(5)/79*x^2*z^2*v^2+F(126)/47*x*y*z^2*v^2+F(14)/107*y^2*z^2*v^2+F(35)/11*x*z^3*v^2-F(65)/51*y*z^3*v^2-7246*z^4*v^2-2652*x^3*u*v^2-F(79)/69*x^2*y*u*v^2-F(21)/59*x*y^2*u*v^2-14050*y^3*u*v^2-F(89)/81*x^2*z*u*v^2+F(91)/55*x*y*z*u*v^2-7501*y^2*z*u*v^2-7688*x*z^2*u*v^2-F(73)/102*y*z^2*u*v^2-F(27)/13*z^3*u*v^2-2186*x^2*u^2*v^2+F(120)/31*x*y*u^2*v^2+F(45)/43*y^2*u^2*v^2-F(101)/16*x*z*u^2*v^2-F(69)/83*y*z*u^2*v^2-12658*z^2*u^2*v^2-7658*x*u^3*v^2-F(53)/103*y*u^3*v^2+11620*z*u^3*v^2+672*u^4*v^2-10795*x^3*v^3+10972*x^2*y*v^3-7178*x*y^2*v^3+F(81)/2*y^3*v^3+F(67)/43*x^2*z*v^3-F(113)/12*x*y*z*v^3-3947*y^2*z*v^3-F(17)/5*x*z^2*v^3-F(102)/49*y*z^2*v^3-F(67)/26*z^3*v^3+F(5)/6*x^2*u*v^3-F(95)/4*x*y*u*v^3+2582*y^2*u*v^3+F(72)/23*x*z*u*v^3+8490*y*z*u*v^3-F(46)/111*z^2*u*v^3+224*x*u^2*v^3+658*y*u^2*v^3-F(98)/89*z*u^2*v^3+7954*u^3*v^3+F(87)/35*x^2*v^4+7260*x*y*v^4+F(91)/40*y^2*v^4+11611*x*z*v^4+9076*y*z*v^4+6444*z^2*v^4-F(8)/95*x*u*v^4-12845*y*u*v^4+F(86)/61*z*u*v^4-F(59)/113*u^2*v^4-F(115)/84*x*v^5-F(116)/41*y*v^5-F(119)/62*z*v^5+15120*u*v^5-F(37)/51*v^6,F(21)/122*x^4*y*z-9662*x^3*y^2*z-11556*x^2*y^3*z+F(67)/18*x*y^4*z-10712*y^5*z-4891*x^4*z^2+F(113)/118*x^3*y*z^2-F(94)/83*x^2*y^2*z^2+F(61)/90*x*y^3*z^2+F(108)/101*y^4*z^2+9905*x^3*z^3+F(12)/29*x^2*y*z^3+13047*x*y^2*z^3-F(55)/46*y^3*z^3+F(59)/88*x^2*z^4+F(59)/78*x*y*z^4+11951*y^2*z^4-1357*x*z^5-F(75)/77*y*z^5+15673*z^6+F(40)/113*x^4*y*u+F(9)/55*x^3*y^2*u-F(62)/83*x^2*y^3*u+F(65)/111*x*y^4*u+4184*y^5*u-9578*x^4*z*u+F(124)/17*x^3*y*z*u+F(91)/85*x^2*y^2*z*u-F(1)/66*x*y^3*z*u+4184*y^4*z*u+F(101)/77*x^3*z^2*u-12238*x^2*y*z^2*u-5394*x*y^2*z^2*u+F(82)/9*y^3*z^2*u-F(73)/113*x^2*z^3*u+3736*x*y*z^3*u+F(52)/81*y^2*z^3*u+10426*x*z^4*u+F(11)/120*y*z^4*u+F(89)/123*z^5*u-F(99)/74*x^4*u^2-F(9)/46*x^3*y*u^2-F(57)/29*x^2*y^2*u^2-F(24)/125*x*y^3*u^2+F(115)/119*y^4*u^2-F(10)/103*x^3*z*u^2-F(66)/17*x^2*y*z*u^2-662*x*y^2*z*u^2-F(31)/17*y^3*z*u^2-F(56)/27*x^2*z^2*u^2-4728*x*y*z^2*u^2-F(86)/59*y^2*z^2*u^2-F(19)/34*x*z^3*u^2+F(61)/13*y*z^3*u^2-14093*z^4*u^2-9068*x^3*u^3-F(115)/94*x^2*y*u^3+15912*x*y^2*u^3-F(79)/66*y^3*u^3-3631*x^2*z*u^3+1074*x*y*z*u^3-F(113)/105*y^2*z*u^3-F(89)/109*x*z^2*u^3-F(80)/91*y*z^2*u^3+F(119)/12*z^3*u^3+F(19)/87*x^2*u^4-F(42)/25*x*y*u^4+F(116)/55*y^2*u^4+F(10)/77*x*z*u^4+F(74)/79*y*z*u^4-F(77)/81*z^2*u^4-F(90)/121*x*u^5-F(43)/31*y*u^5+F(107)/122*z*u^5+F(76)/113*u^6+F(60)/17*x^4*y*v-6269*x^3*y^2*v-F(124)/75*x^2*y^3*v+F(89)/48*x*y^4*v-F(69)/2*y^5*v+8832*x^4*z*v+13984*x^3*y*z*v+F(35)/29*x^2*y^2*z*v-F(65)/88*x*y^3*z*v+F(53)/74*y^4*z*v-F(55)/79*x^3*z^2*v-F(104)/105*x^2*y*z^2*v+F(50)/29*x*y^2*z^2*v-F(118)/119*y^3*z^2*v-F(25)/88*x^2*z^3*v+F(69)/82*x*y*z^3*v-F(56)/69*y^2*z^3*v-10495*x*z^4*v-F(73)/50*y*z^4*v+1872*z^5*v-F(62)/19*x^4*u*v+F(99)/89*x^3*y*u*v+3156*x^2*y^2*u*v+5804*x*y^3*u*v+F(85)/91*y^4*u*v+10928*x^3*z*u*v-F(4)/83*x^2*y*z*u*v+2839*x*y^2*z*u*v+F(11)/38*y^3*z*u*v+F(55)/126*x^2*z^2*u*v+15613*x*y*z^2*u*v+F(69)/28*y^2*z^2*u*v+F(75)/17*x*z^3*u*v+F(115)/51*y*z^3*u*v-F(111)/68*z^4*u*v-9781*x^3*u^2*v+F(2)/105*x^2*y*u^2*v-F(29)/10*x*y^2*u^2*v+F(90)/53*y^3*u^2*v-12840*x^2*z*u^2*v+F(85)/71*x*y*z*u^2*v-F(91)/80*y^2*z*u^2*v+15904*x*z^2*u^2*v-F(82)/69*y*z^2*u^2*v-F(32)/75*z^3*u^2*v-F(91)/2*x^2*u^3*v-F(77)/61*x*y*u^3*v-9757*y^2*u^3*v-F(97)/52*x*z*u^3*v-F(32)/9*y*z*u^3*v-7457*z^2*u^3*v-F(113)/100*x*u^4*v-13367*y*u^4*v-16*z*u^4*v+F(17)/53*u^5*v+F(90)/103*x^4*v^2-9338*x^3*y*v^2-F(42)/61*x^2*y^2*v^2+F(57)/124*x*y^3*v^2-F(17)/6*y^4*v^2+6201*x^3*z*v^2+F(75)/8*x^2*y*z*v^2+13205*x*y^2*z*v^2-F(21)/23*y^3*z*v^2+6724*x^2*z^2*v^2-1646*x*y*z^2*v^2-F(3)/41*y^2*z^2*v^2+13206*x*z^3*v^2+14595*y*z^3*v^2+3100*z^4*v^2-F(94)/107*x^3*u*v^2+F(106)/99*x^2*y*u*v^2+F(53)/24*x*y^2*u*v^2-10113*y^3*u*v^2+13103*x^2*z*u*v^2+F(121)/124*x*y*z*u*v^2-F(104)/103*y^2*z*u*v^2+F(59)/62*x*z^2*u*v^2+13343*y*z^2*u*v^2-F(73)/72*z^3*u*v^2-F(35)/123*x^2*u^2*v^2+F(91)/33*x*y*u^2*v^2+F(75)/58*y^2*u^2*v^2-F(69)/73*x*z*u^2*v^2-15760*y*z*u^2*v^2+684*z^2*u^2*v^2-12551*x*u^3*v^2-F(99)/79*y*u^3*v^2+F(74)/87*z*u^3*v^2+9255*u^4*v^2-9727*x^3*v^3+1222*x^2*y*v^3+F(31)/115*x*y^2*v^3+F(37)/50*y^3*v^3-F(86)/125*x^2*z*v^3-F(5)/82*x*y*z*v^3+F(7)/2*y^2*z*v^3+F(69)/88*x*z^2*v^3-F(25)/119*y*z^2*v^3-F(120)/101*z^3*v^3-F(48)/113*x^2*u*v^3-F(25)/97*x*y*u*v^3-14896*y^2*u*v^3+13431*x*z*u*v^3+13246*y*z*u*v^3+7556*z^2*u*v^3-F(103)/111*x*u^2*v^3+F(13)/108*y*u^2*v^3+9471*z*u^2*v^3+F(31)/114*u^3*v^3-F(121)/23*x^2*v^4-F(65)/69*x*y*v^4+F(66)/95*y^2*v^4+F(30)/59*x*z*v^4-F(111)/40*y*z*v^4+F(55)/4*z^2*v^4+F(114)/121*x*u*v^4+7610*y*u*v^4-9205*z*u*v^4+F(85)/81*u^2*v^4-F(88)/59*x*v^5-4248*y*v^5+F(95)/91*z*v^5+156*u*v^5-F(71)/90*v^6,7698*x^4*y-F(93)/32*x^3*y^2+F(95)/37*x^2*y^3+F(29)/104*x*y^4+F(10)/23*y^5-11774*x^4*z+4544*x^3*y*z-F(9)/85*x^2*y^2*z-F(45)/49*x*y^3*z+F(110)/41*y^4*z-F(44)/91*x^3*z^2-6083*x^2*y*z^2+F(116)/111*x*y^2*z^2+F(47)/68*y^3*z^2-11603*x^2*z^3-4229*x*y*z^3-13462*y^2*z^3-F(31)/19*x*z^4+4222*y*z^4-700*z^5-F(34)/11*x^4*u-F(20)/17*x^3*y*u-2471*x^2*y^2*u-11235*x*y^3*u+13259*y^4*u-111*x^3*z*u-109*x^2*y*z*u-F(89)/61*x*y^2*z*u-F(28)/11*y^3*z*u+F(74)/97*x^2*z^2*u+5554*x*y*z^2*u+F(75)/47*y^2*z^2*u-F(68)/77*x*z^3*u+15754*y*z^3*u-F(7)/51*z^4*u-F(53)/98*x^3*u^2+9699*x^2*y*u^2-F(9)/104*x*y^2*u^2+F(64)/87*y^3*u^2-F(95)/4*x^2*z*u^2-595*x*y*z*u^2+F(4)/19*y^2*z*u^2-F(18)/95*x*z^2*u^2-13449*y*z^2*u^2+2931*z^3*u^2-11155*x^2*u^3-F(83)/29*x*y*u^3+7830*y^2*u^3+F(108)/91*x*z*u^3+13161*y*z*u^3+F(37)/42*z^2*u^3-F(16)/79*x*u^4-10604*y*u^4-15832*z*u^4+F(40)/39*u^5-6020*x^4*v+910*x^3*y*v+F(13)/110*x^2*y^2*v+F(7)/86*x*y^3*v-F(97)/101*y^4*v-3286*x^3*z*v+F(80)/91*x^2*y*z*v+12467*x*y^2*z*v+F(115)/99*y^3*z*v+F(79)/60*x^2*z^2*v-F(8)/19*x*y*z^2*v+F(105)/71*y^2*z^2*v+F(60)/119*x*z^3*v-F(71)/15*y*z^3*v+15272*z^4*v-7397*x^3*u*v+F(125)/7*x^2*y*u*v-9507*x*y^2*u*v+5301*y^3*u*v-5605*x^2*z*u*v-F(32)/35*x*y*z*u*v-5523*y^2*z*u*v+F(67)/88*x*z^2*u*v+15144*y*z^2*u*v+F(3)/8*z^3*u*v-F(33)/56*x^2*u^2*v+F(37)/29*x*y*u^2*v+F(49)/19*y^2*u^2*v-10604*x*z*u^2*v+F(37)/44*y*z*u^2*v-8754*z^2*u^2*v+4184*x*u^3*v-F(56)/89*y*u^3*v-F(23)/32*z*u^3*v+F(74)/101*u^4*v-F(23)/123*x^3*v^2-13803*x^2*y*v^2+F(54)/95*x*y^2*v^2+9751*y^3*v^2+F(55)/119*x^2*z*v^2+F(75)/32*x*y*z*v^2+10091*y^2*z*v^2+1108*x*z^2*v^2-13283*y*z^2*v^2+F(98)/111*z^3*v^2+F(5)/109*x^2*u*v^2+F(28)/79*x*y*u*v^2-F(95)/6*y^2*u*v^2-7880*x*z*u*v^2-12659*y*z*u*v^2+14820*z^2*u*v^2+4279*x*u^2*v^2+F(79)/51*y*u^2*v^2-F(67)/49*z*u^2*v^2+11207*u^3*v^2+F(85)/54*x^2*v^3-F(78)/43*x*y*v^3-F(3)/95*y^2*v^3-F(86)/65*x*z*v^3+F(1)/114*y*z*v^3+F(27)/74*z^2*v^3-F(102)/31*x*u*v^3-F(11)/59*y*u*v^3-F(33)/29*z*u*v^3-F(3)/110*u^2*v^3-8455*x*v^4+F(77)/100*y*v^4-6225*z*v^4-F(70)/9*u*v^4-10939*v^5,-7698*x^5-F(16)/39*x^4*y+F(51)/61*x^3*y^2+F(59)/33*x^2*y^3+12841*x*y^4-11040*y^5+14163*x^4*z-F(79)/55*x^3*y*z-F(83)/18*x^2*y^2*z+F(121)/67*x*y^3*z+F(52)/35*y^4*z+F(26)/15*x^3*z^2+12770*x^2*y*z^2-F(67)/15*x*y^2*z^2-12895*y^3*z^2-F(11)/16*x^2*z^3-F(82)/45*x*y*z^3+2446*y^2*z^3+F(116)/9*x*z^4+F(6)/61*y*z^4+F(83)/98*z^5+9973*x^4*u+F(67)/101*x^3*y*u+F(25)/19*x^2*y^2*u+7517*x*y^3*u-F(59)/117*y^4*u-F(1)/86*x^3*z*u-F(10)/119*x^2*y*z*u+2552*x*y^2*z*u-2448*y^3*z*u-F(112)/45*x^2*z^2*u+F(35)/71*x*y*z^2*u+12328*y^2*z^2*u+F(124)/65*x*z^3*u-15531*y*z^3*u-F(39)/16*z^4*u+4678*x^3*u^2-F(44)/103*x^2*y*u^2-9303*x*y^2*u^2+F(59)/20*y^3*u^2-F(45)/97*x^2*z*u^2+2707*x*y*z*u^2+F(65)/61*y^2*z*u^2+F(75)/68*x*z^2*u^2+2853*y*z^2*u^2-12748*z^3*u^2-F(17)/18*x^2*u^3-F(115)/121*x*y*u^3+F(72)/71*y^2*u^3-12194*x*z*u^3-14204*y*z*u^3-F(63)/17*z^2*u^3+5772*x*u^4-F(99)/16*y*u^4-F(51)/43*z*u^4+F(49)/43*u^5-2588*x^4*v+F(89)/44*x^3*y*v+F(32)/107*x^2*y^2*v-F(117)/76*x*y^3*v-F(84)/115*y^4*v+F(113)/30*x^3*z*v-F(13)/68*x^2*y*z*v+15120*x*y^2*z*v-F(59)/28*y^3*z*v+F(61)/52*x^2*z^2*v+12390*x*y*z^2*v+11436*y^2*z^2*v+F(109)/40*x*z^3*v+F(40)/61*y*z^3*v+F(65)/31*z^4*v+12764*x^3*u*v+15885*x^2*y*u*v-11299*x*y^2*u*v-F(113)/66*y^3*u*v+2887*x^2*z*u*v-918*x*y*z*u*v+12579*y^2*z*u*v+F(39)/10*x*z^2*u*v-F(119)/53*y*z^2*u*v-F(62)/115*z^3*u*v-10887*x^2*u^2*v+F(115)/122*x*y*u^2*v-8863*y^2*u^2*v-F(30)/79*x*z*u^2*v-F(26)/5*y*z*u^2*v+15294*z^2*u^2*v-15701*x*u^3*v-F(11)/19*y*u^3*v+F(25)/14*z*u^3*v-F(48)/55*u^4*v+1341*x^3*v^2-4973*x^2*y*v^2+F(55)/117*x*y^2*v^2-1787*y^3*v^2-F(115)/57*x^2*z*v^2+F(28)/29*x*y*z*v^2-184*y^2*z*v^2+11738*x*z^2*v^2-8375*y*z^2*v^2-5962*z^3*v^2+F(52)/55*x^2*u*v^2+F(17)/48*x*y*u*v^2-F(103)/52*y^2*u*v^2-F(53)/25*x*z*u*v^2-F(101)/3*y*z*u*v^2-F(123)/35*z^2*u*v^2-14815*x*u^2*v^2-F(103)/14*y*u^2*v^2-F(68)/81*z*u^2*v^2-F(81)/22*u^3*v^2-F(121)/56*x^2*v^3-12609*x*y*v^3+5555*y^2*v^3+F(8)/17*x*z*v^3-741*y*z*v^3-F(73)/103*z^2*v^3-12550*x*u*v^3-F(17)/78*y*u*v^3+7817*z*u*v^3+6534*u^2*v^3-15384*x*v^4+1807*y*v^4-4677*z*v^4-F(101)/115*u*v^4-F(83)/19*v^5,-F(53)/83*x^5+F(107)/75*x^4*y+F(26)/51*x^3*y^2+F(45)/109*x^2*y^3-3009*x*y^4-F(27)/61*y^5+F(16)/85*x^4*z-14859*x^3*y*z-F(20)/27*x^2*y^2*z+6326*x*y^3*z-4508*y^4*z-10006*x^3*z^2-11979*x^2*y*z^2+8579*x*y^2*z^2+14669*y^3*z^2+F(67)/79*x^2*z^3-2551*x*y*z^3-F(61)/124*y^2*z^3+F(83)/10*x*z^4+12698*y*z^4+F(15)/113*z^5+F(123)/86*x^4*u-F(77)/6*x^3*y*u+15113*x^2*y^2*u+F(79)/117*x*y^3*u-F(115)/88*y^4*u+F(101)/95*x^3*z*u+F(56)/13*x^2*y*z*u-F(25)/62*x*y^2*z*u+1955*y^3*z*u+F(70)/33*x^2*z^2*u+7470*x*y*z^2*u-2148*y^2*z^2*u-14263*x*z^3*u-3962*y*z^3*u+F(47)/35*z^4*u+441*x^3*u^2+14944*x^2*y*u^2-F(2)/77*x*y^2*u^2+F(68)/23*y^3*u^2-F(121)/43*x^2*z*u^2-14321*x*y*z*u^2-F(35)/32*y^2*z*u^2+F(34)/21*x*z^2*u^2+11645*y*z^2*u^2+7131*z^3*u^2-3615*x^2*u^3-2748*x*y*u^3-15200*y^2*u^3+F(81)/101*x*z*u^3+F(39)/64*y*z*u^3-3967*z^2*u^3-10346*x*u^4+F(55)/18*y*u^4-F(8)/9*z*u^4+F(33)/68*u^5-13957*x^4*v+F(31)/116*x^3*y*v+F(58)/81*x^2*y^2*v+F(36)/71*x*y^3*v-5706*y^4*v-F(95)/48*x^3*z*v+3214*x^2*y*z*v+14729*x*y^2*z*v+F(71)/109*y^3*z*v+15365*x^2*z^2*v-5109*x*y*z^2*v-F(20)/107*y^2*z^2*v-F(6)/11*x*z^3*v+F(74)/55*y*z^3*v-F(76)/11*z^4*v+F(41)/72*x^3*u*v+7215*x^2*y*u*v-F(18)/59*x*y^2*u*v+1741*y^3*u*v+7698*x^2*z*u*v-7299*x*y*z*u*v+12127*y^2*z*u*v+F(7)/93*x*z^2*u*v+F(71)/8*y*z^2*u*v-F(123)/73*z^3*u*v+13657*x^2*u^2*v-F(98)/13*x*y*u^2*v+11818*y^2*u^2*v+F(22)/23*x*z*u^2*v-3038*y*z*u^2*v+F(68)/61*z^2*u^2*v-7173*x*u^3*v-7460*y*u^3*v+3540*z*u^3*v+F(27)/20*u^4*v-F(37)/41*x^3*v^2+20*x^2*y*v^2+F(107)/82*x*y^2*v^2-2237*y^3*v^2+9827*x^2*z*v^2+F(124)/27*x*y*z*v^2-F(18)/5*y^2*z*v^2-F(77)/24*x*z^2*v^2-10231*y*z^2*v^2-F(32)/7*z^3*v^2-11980*x^2*u*v^2-F(36)/35*x*y*u*v^2+8618*y^2*u*v^2+3174*x*z*u*v^2-F(123)/2*y*z*u*v^2-F(117)/38*z^2*u*v^2+F(117)/115*x*u^2*v^2+F(70)/3*y*u^2*v^2-3144*z*u^2*v^2+815*u^3*v^2-F(116)/85*x^2*v^3+F(98)/41*x*y*v^3-648*y^2*v^3-F(38)/5*x*z*v^3-F(9)/125*y*z*v^3+8710*z^2*v^3+F(48)/31*x*u*v^3+F(101)/109*y*u*v^3+F(70)/11*z*u*v^3-F(51)/4*u^2*v^3-F(76)/59*x*v^4-F(93)/52*y*v^4+15291*z*v^4+F(4)/55*u*v^4+F(64)/59*v^5,568*x^5+4355*x^4*y+F(48)/5*x^3*y^2+F(33)/7*x^2*y^3-F(53)/111*x*y^4+1749*y^5-F(23)/4*x^4*z-F(98)/69*x^3*y*z-F(47)/56*x^2*y^2*z-8519*x*y^3*z-F(5)/113*y^4*z+11488*x^3*z^2+F(79)/21*x^2*y*z^2+F(11)/89*x*y^2*z^2-F(64)/83*y^3*z^2-15697*x^2*z^3-F(67)/86*x*y*z^3+11545*y^2*z^3+3336*x*z^4-F(106)/39*y*z^4+15466*z^5+15202*x^4*u-F(34)/63*x^3*y*u-F(121)/72*x^2*y^2*u+F(1)/52*x*y^3*u+10800*y^4*u+4993*x^3*z*u-F(55)/112*x^2*y*z*u-F(26)/51*x*y^2*z*u-F(114)/125*y^3*z*u+F(113)/2*x^2*z^2*u-F(87)/88*x*y*z^2*u-F(91)/107*y^2*z^2*u-F(65)/6*x*z^3*u+15415*y*z^3*u+1373*z^4*u-F(27)/86*x^3*u^2-F(76)/93*x^2*y*u^2+F(9)/22*x*y^2*u^2+F(16)/91*y^3*u^2+10326*x^2*z*u^2-F(61)/84*x*y*z*u^2-F(28)/99*y^2*z*u^2+F(87)/14*x*z^2*u^2-F(88)/45*y*z^2*u^2+F(60)/59*z^3*u^2-F(13)/60*x^2*u^3-10824*x*y*u^3-F(121)/119*y^2*u^3+14919*x*z*u^3-F(81)/25*y*z*u^3+11233*z^2*u^3+14676*x*u^4-8474*y*u^4+12211*z*u^4+F(32)/83*u^5+F(57)/52*x^4*v+F(10)/13*x^3*y*v-277*x^2*y^2*v-6961*x*y^3*v-4594*y^4*v-13439*x^3*z*v-F(1)/30*x^2*y*z*v-F(118)/43*x*y^2*z*v-F(62)/15*y^3*z*v+F(76)/15*x^2*z^2*v+3805*x*y*z^2*v-F(26)/15*y^2*z^2*v+3081*x*z^3*v+662*y*z^3*v+13856*z^4*v+F(107)/30*x^3*u*v+6063*x^2*y*u*v+F(100)/37*x*y^2*u*v+F(110)/107*y^3*u*v-10346*x^2*z*u*v-F(67)/44*x*y*z*u*v-F(93)/29*y^2*z*u*v+F(17)/89*x*z^2*u*v-F(57)/104*y*z^2*u*v-F(68)/91*z^3*u*v+3804*x^2*u^2*v-F(75)/107*x*y*u^2*v-11842*y^2*u^2*v-F(103)/57*x*z*u^2*v-F(37)/18*y*z*u^2*v+10795*z^2*u^2*v-F(90)/31*x*u^3*v+14200*y*u^3*v+F(97)/124*z*u^3*v+5256*u^4*v+F(52)/101*x^3*v^2-F(94)/107*x^2*y*v^2-12841*x*y^2*v^2+F(77)/72*y^3*v^2+F(93)/74*x^2*z*v^2+7033*x*y*z*v^2+F(87)/76*y^2*z*v^2-15415*x*z^2*v^2-15164*y*z^2*v^2-14749*z^3*v^2+F(86)/53*x^2*u*v^2+14707*x*y*u*v^2+9443*y^2*u*v^2+F(118)/5*x*z*u*v^2-F(81)/2*y*z*u*v^2+F(43)/57*z^2*u*v^2+F(59)/83*x*u^2*v^2-F(121)/79*y*u^2*v^2+4449*z*u^2*v^2-F(50)/63*u^3*v^2+F(79)/31*x^2*v^3+F(95)/32*x*y*v^3+F(125)/107*y^2*v^3-9165*x*z*v^3+3151*y*z*v^3+5006*z^2*v^3+F(45)/19*x*u*v^3-5194*y*u*v^3-F(82)/11*z*u*v^3+F(121)/15*u^2*v^3-10265*x*v^4-F(99)/118*y*v^4-3162*z*v^4-F(16)/29*u*v^4-F(37)/4*v^5]))
end


@doc raw"""
    k3_d7_pi5()

Return a smooth K3 surface in $\mathbb P^4$ with degree `7` and sectional genus `5`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d7_pi5()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(97)/108*x^2*y-F(31)/118*x*y^2-F(73)/61*y^3-F(79)/14*x^2*z-15930*x*y*z-2324*y^2*z+1842*x*z^2+656*y*z^2-8852*z^3-F(89)/38*x^2*u-F(102)/43*x*y*u+14719*y^2*u+F(70)/67*x*z*u+7335*y*z*u+F(27)/56*z^2*u-10744*x*u^2-F(55)/83*y*u^2+F(120)/73*z*u^2+F(120)/61*u^3-F(126)/125*x^2*v+691*x*y*v-15385*y^2*v+F(117)/16*x*z*v-F(17)/97*y*z*v+F(80)/121*z^2*v-F(48)/119*x*u*v+F(21)/34*y*u*v-F(103)/65*z*u*v-F(49)/32*u^2*v-F(41)/42*x*v^2+F(11)/75*y*v^2-502*z*v^2-7583*u*v^2+F(26)/69*v^3,F(97)/108*x^3+F(77)/114*x^2*y+F(71)/21*x*y^2+13679*y^3-1645*x^2*z-F(1)/33*x*y*z-F(79)/7*y^2*z-F(52)/53*x*z^2+11940*y*z^2-5800*z^3+F(109)/13*x^2*u-F(115)/64*x*y*u-F(125)/56*y^2*u-2365*x*z*u+2103*y*z*u+F(56)/87*z^2*u-F(84)/79*x*u^2+F(107)/106*y*u^2-F(79)/70*z*u^2-419*u^3+5354*x^2*v+F(92)/53*x*y*v-F(32)/19*y^2*v+F(11)/74*x*z*v+4193*y*z*v+F(45)/79*z^2*v-F(113)/72*x*u*v+F(17)/71*y*u*v+11164*z*u*v-F(17)/33*u^2*v+F(103)/66*x*v^2+F(55)/79*y*v^2+F(118)/15*z*v^2-2646*u*v^2+F(57)/106*v^3,x^3-F(61)/113*x^2*y-F(64)/21*x*y^2-F(107)/8*y^3-F(13)/60*x^2*z+F(43)/35*x*y*z+F(41)/114*y^2*z-13683*x*z^2-5829*y*z^2+F(71)/38*z^3+F(90)/17*x^2*u-F(39)/29*x*y*u+F(42)/5*y^2*u-F(61)/55*x*z*u+F(111)/77*y*z*u-F(87)/100*z^2*u+10735*x*u^2-F(83)/91*y*u^2-4884*z*u^2-7965*u^3-F(65)/12*x^2*v+F(109)/86*x*y*v+10606*y^2*v-14164*x*z*v-6678*y*z*v+F(83)/18*z^2*v-F(93)/10*x*u*v+F(120)/49*y*u*v-1592*z*u*v-8710*u^2*v-F(73)/57*x*v^2+10762*y*v^2-2956*z*v^2-F(89)/63*u*v^2-F(12)/7*v^3]))
end


@doc raw"""
    k3_d8_pi6()

Return a smooth K3 surface in $\mathbb P^4$ with degree `8` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d8_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(69)/4*x^3*y-F(59)/80*x^2*y^2+F(6)/49*x*y^3+92*y^4-F(47)/75*x^3*z+F(87)/61*x^2*y*z-F(45)/88*x*y^2*z+F(58)/113*y^3*z+10048*x^2*z^2+F(112)/27*x*y*z^2-F(59)/3*y^2*z^2+F(40)/99*x*z^3-4769*y*z^3+9097*z^4-F(53)/17*x^3*u-F(19)/51*x^2*y*u-F(24)/89*x*y^2*u+1161*y^3*u-1986*x^2*z*u-F(96)/61*x*y*z*u+F(40)/103*y^2*z*u-7538*x*z^2*u+F(7)/5*y*z^2*u-F(49)/123*z^3*u-F(1)/25*x^2*u^2+6699*x*y*u^2-F(52)/89*y^2*u^2+F(104)/99*x*z*u^2-F(119)/37*y*z*u^2+4437*z^2*u^2+F(11)/114*x*u^3-1576*y*u^3-F(17)/99*z*u^3-F(55)/102*u^4+F(21)/62*x^3*v-F(79)/111*x^2*y*v+5754*x*y^2*v-11699*y^3*v-F(126)/67*x^2*z*v+F(111)/59*x*y*z*v+3503*y^2*z*v-F(19)/64*x*z^2*v-F(3)/116*y*z^2*v-F(57)/88*z^3*v-F(34)/113*x^2*u*v-F(99)/8*x*y*u*v+15760*y^2*u*v+F(19)/126*x*z*u*v+F(43)/81*y*z*u*v+F(14)/37*z^2*u*v-F(44)/61*x*u^2*v+F(59)/55*y*u^2*v-F(70)/11*z*u^2*v+F(11)/48*u^3*v+F(78)/79*x^2*v^2+F(48)/107*x*y*v^2+243*y^2*v^2-F(94)/83*x*z*v^2+F(101)/97*y*z*v^2+F(66)/61*z^2*v^2-F(52)/15*x*u*v^2+F(95)/44*y*u*v^2+F(51)/95*z*u*v^2-12911*u^2*v^2-F(50)/119*x*v^3-F(67)/71*y*v^3-F(12)/59*z*v^3+11392*u*v^3+14783*v^4,-15665*x^3*y-8384*x^2*y^2-8180*x*y^3-221*y^4-F(77)/61*x^2*y*z+F(87)/10*x*y^2*z+F(13)/118*y^3*z+F(17)/36*x*y*z^2-F(118)/89*y^2*z^2-F(125)/53*y*z^3-F(118)/109*x^2*y*u-F(39)/62*x*y^2*u-F(29)/66*y^3*u+11262*x*y*z*u+F(17)/49*y^2*z*u-F(125)/6*y*z^2*u-F(35)/82*x*y*u^2-14866*y^2*u^2+9169*y*z*u^2+12260*y*u^3-F(102)/101*x^3*v+F(113)/104*x^2*y*v+F(112)/39*x*y^2*v-F(74)/73*y^3*v+2337*x^2*z*v+F(123)/122*x*y*z*v-F(76)/47*y^2*z*v+2045*x*z^2*v+F(36)/85*y*z^2*v-303*z^3*v-1737*x^2*u*v+6297*x*y*u*v-F(102)/73*y^2*u*v+8793*x*z*u*v+7178*y*z*u*v-F(103)/124*z^2*u*v-10355*x*u^2*v+F(82)/17*y*u^2*v+F(104)/77*z*u^2*v+F(71)/50*u^3*v-F(63)/43*x^2*v^2-5898*x*y*v^2-F(1)/58*y^2*v^2-F(21)/17*x*z*v^2-F(26)/45*y*z*v^2-F(103)/29*z^2*v^2-F(107)/26*x*u*v^2-F(82)/69*y*u*v^2+945*z*u*v^2-F(113)/2*u^2*v^2+F(20)/93*x*v^3-14442*y*v^3-F(102)/49*z*v^3-F(73)/99*u*v^3+11234*v^4,-15665*x^3*z-8384*x^2*y*z-8180*x*y^2*z-221*y^3*z-F(77)/61*x^2*z^2+F(87)/10*x*y*z^2+F(13)/118*y^2*z^2+F(17)/36*x*z^3-F(118)/89*y*z^3-F(125)/53*z^4-F(118)/109*x^2*z*u-F(39)/62*x*y*z*u-F(29)/66*y^2*z*u+11262*x*z^2*u+F(17)/49*y*z^2*u-F(125)/6*z^3*u-F(35)/82*x*z*u^2-14866*y*z*u^2+9169*z^2*u^2+12260*z*u^3-3533*x^3*v-F(15)/86*x^2*y*v+113*x*y^2*v-1934*y^3*v-7920*x^2*z*v-F(119)/47*x*y*z*v+F(97)/123*y^2*z*v-F(1)/78*x*z^2*v-F(78)/35*y*z^2*v+F(90)/31*z^3*v-F(86)/11*x^2*u*v+8832*x*y*u*v-14164*y^2*u*v+5610*x*z*u*v-14464*y*z*u*v+F(87)/23*z^2*u*v-4204*x*u^2*v-F(49)/11*y*u^2*v+392*z*u^2*v-F(27)/110*u^3*v-F(73)/93*x^2*v^2+F(58)/11*x*y*v^2-F(4)/83*y^2*v^2+4734*x*z*v^2+F(73)/118*y*z*v^2+5683*z^2*v^2+F(51)/92*x*u*v^2-F(119)/117*y*u*v^2+2212*z*u*v^2-F(47)/41*u^2*v^2-3848*x*v^3-2679*y*v^3-1018*z*v^3-F(67)/35*u*v^3+4595*v^4,-15665*x^3*u-8384*x^2*y*u-8180*x*y^2*u-221*y^3*u-F(77)/61*x^2*z*u+F(87)/10*x*y*z*u+F(13)/118*y^2*z*u+F(17)/36*x*z^2*u-F(118)/89*y*z^2*u-F(125)/53*z^3*u-F(118)/109*x^2*u^2-F(39)/62*x*y*u^2-F(29)/66*y^2*u^2+11262*x*z*u^2+F(17)/49*y*z*u^2-F(125)/6*z^2*u^2-F(35)/82*x*u^3-14866*y*u^3+9169*z*u^3+12260*u^4+F(59)/30*x^3*v-468*x^2*y*v-F(65)/33*x*y^2*v-F(55)/87*y^3*v-F(119)/123*x^2*z*v+F(55)/107*x*y*z*v+F(72)/53*y^2*z*v-F(37)/81*x*z^2*v+F(54)/89*y*z^2*v-F(25)/71*z^3*v-6637*x^2*u*v-3496*x*y*u*v+F(125)/41*y^2*u*v+F(95)/89*x*z*u*v+4238*y*z*u*v-F(61)/105*z^2*u*v-F(33)/17*x*u^2*v+F(53)/126*y*u^2*v+5167*z*u^2*v+5308*u^3*v+F(43)/124*x^2*v^2-2926*x*y*v^2+F(7)/111*y^2*v^2-F(82)/65*x*z*v^2-F(107)/7*y*z*v^2-14027*z^2*v^2+F(121)/76*x*u*v^2+1892*y*u*v^2+1733*z*u*v^2-F(53)/41*u^2*v^2-F(80)/43*x*v^3+F(51)/4*y*v^3-F(16)/51*z*v^3-F(17)/54*u*v^3+F(31)/113*v^4,15665*x^4-F(55)/53*x^3*y+1924*x^2*y^2+F(46)/53*x*y^3-14657*y^4-5850*x^3*z-4068*x^2*y*z+10441*x*y^2*z-F(63)/59*y^3*z-F(85)/83*x^2*z^2+F(77)/65*x*y*z^2-F(21)/19*y^2*z^2+F(112)/51*x*z^3+F(59)/38*y*z^3+4405*z^4+F(41)/8*x^3*u-F(37)/94*x^2*y*u+F(96)/83*x*y^2*u+8382*y^3*u+1691*x^2*z*u-14560*x*y*z*u-671*y^2*z*u-F(23)/15*x*z^2*u-F(74)/15*y*z^2*u-F(13)/37*z^3*u-F(86)/81*x^2*u^2-F(104)/107*x*y*u^2-F(73)/102*y^2*u^2-F(98)/107*x*z*u^2+F(97)/57*y*z*u^2+9321*z^2*u^2-F(103)/60*x*u^3-13552*y*u^3+F(51)/64*z*u^3+F(62)/105*u^4-F(87)/62*x^3*v-F(96)/55*x^2*y*v+F(85)/32*x*y^2*v-13903*y^3*v-F(47)/110*x^2*z*v+5299*x*y*z*v+4342*y^2*z*v+F(125)/18*x*z^2*v+F(13)/79*y*z^2*v+F(23)/125*z^3*v+3028*x^2*u*v-F(82)/119*x*y*u*v-F(5)/76*y^2*u*v+F(89)/17*x*z*u*v+2176*y*z*u*v+14730*z^2*u*v-11510*x*u^2*v-9661*y*u^2*v-F(14)/85*z*u^2*v+15162*u^3*v+210*x^2*v^2+F(49)/90*x*y*v^2-11164*y^2*v^2+15785*x*z*v^2+15362*y*z*v^2+5257*z^2*v^2-F(39)/35*x*u*v^2-F(106)/105*y*u*v^2+F(16)/27*z*u*v^2+F(41)/31*u^2*v^2+13081*x*v^3-7663*y*v^3-3304*z*v^3-F(94)/63*u*v^3,-F(59)/30*x^3*y+468*x^2*y^2+F(65)/33*x*y^3+F(55)/87*y^4+F(119)/123*x^2*y*z-F(55)/107*x*y^2*z-F(72)/53*y^3*z+F(37)/81*x*y*z^2-F(54)/89*y^2*z^2+F(25)/71*y*z^3-F(102)/101*x^3*u-F(59)/79*x^2*y*u+6780*x*y^2*u+10257*y^3*u+2337*x^2*z*u-F(59)/51*x*y*z*u+F(34)/61*y^2*z*u+2045*x*z^2*u+2994*y*z^2*u-303*z^3*u-1737*x^2*u^2-4992*x*y*u^2+8975*y^2*u^2+8793*x*z*u^2+2011*y*z*u^2-F(103)/124*z^2*u^2-10355*x*u^3-7185*y*u^3+F(104)/77*z*u^3+F(71)/50*u^4-F(43)/124*x^2*y*v+2926*x*y^2*v-F(7)/111*y^3*v+F(82)/65*x*y*z*v+F(107)/7*y^2*z*v+14027*y*z^2*v-F(63)/43*x^2*u*v-9688*x*y*u*v-5753*y^2*u*v-F(21)/17*x*z*u*v-F(9)/73*y*z*u*v-F(103)/29*z^2*u*v-F(107)/26*x*u^2*v-F(68)/107*y*u^2*v+945*z*u^2*v-F(113)/2*u^3*v+F(80)/43*x*y*v^2-F(51)/4*y^2*v^2+F(16)/51*y*z*v^2+F(20)/93*x*u*v^2-F(68)/109*y*u*v^2-F(102)/49*z*u*v^2-F(73)/99*u^2*v^2-F(31)/113*y*v^3+11234*u*v^3,-F(59)/30*x^3*z+468*x^2*y*z+F(65)/33*x*y^2*z+F(55)/87*y^3*z+F(119)/123*x^2*z^2-F(55)/107*x*y*z^2-F(72)/53*y^2*z^2+F(37)/81*x*z^3-F(54)/89*y*z^3+F(25)/71*z^4-3533*x^3*u-F(15)/86*x^2*y*u+113*x*y^2*u-1934*y^3*u-F(84)/25*x^2*z*u-F(67)/37*x*y*z*u-13787*y^2*z*u+F(11)/105*x*z^2*u+F(108)/125*y*z^2*u-F(43)/61*z^3*u-F(86)/11*x^2*u^2+8832*x*y*u^2-14164*y^2*u^2-5679*x*z*u^2+F(8)/67*y*z*u^2+F(65)/87*z^2*u^2-4204*x*u^3-F(49)/11*y*u^3+F(74)/13*z*u^3-F(27)/110*u^4-F(43)/124*x^2*z*v+2926*x*y*z*v-F(7)/111*y^2*z*v+F(82)/65*x*z^2*v+F(107)/7*y*z^2*v+14027*z^3*v-F(73)/93*x^2*u*v+F(58)/11*x*y*u*v-F(4)/83*y^2*u*v+F(105)/34*x*z*u*v-15718*y*z*u*v+F(40)/81*z^2*u*v+F(51)/92*x*u^2*v-F(119)/117*y*u^2*v-10271*z*u^2*v-F(47)/41*u^3*v+F(80)/43*x*z*v^2-F(51)/4*y*z*v^2+F(16)/51*z^2*v^2-3848*x*u*v^2-2679*y*u*v^2+F(35)/64*z*u*v^2-F(67)/35*u^2*v^2-F(31)/113*z*v^3+4595*u*v^3,-F(105)/67*x^4+815*x^3*y-F(42)/37*x^2*y^2+13887*x*y^3-F(13)/124*y^4-F(39)/115*x^3*z+4988*x^2*y*z+F(29)/64*x*y^2*z-10188*y^3*z-F(117)/50*x^2*z^2-6576*x*y*z^2-5403*y^2*z^2-F(82)/63*x*z^3-5166*y*z^3+3614*z^4+11186*x^3*u-F(118)/53*x^2*y*u+F(55)/42*x*y^2*u-F(76)/83*y^3*u-F(41)/18*x^2*z*u+F(90)/19*x*y*z*u+15217*y^2*z*u+F(74)/121*x*z^2*u+F(77)/67*y*z^2*u-F(31)/50*z^3*u-12767*x^2*u^2+F(69)/118*x*y*u^2-F(52)/75*y^2*u^2-5416*x*z*u^2-F(58)/31*y*z*u^2-14408*z^2*u^2+F(70)/89*x*u^3+860*y*u^3+1221*z*u^3-2888*x^3*v+F(101)/71*x^2*y*v-9325*x*y^2*v-F(6)/95*y^3*v+F(79)/21*x^2*z*v-1899*x*y*z*v+1815*y^2*z*v+F(27)/35*x*z^2*v-F(7)/8*y*z^2*v+12355*z^3*v-F(119)/125*x^2*u*v-F(71)/96*x*y*u*v-F(113)/36*y^2*u*v+F(79)/121*x*z*u*v-F(125)/18*y*z*u*v+F(26)/83*z^2*u*v+10344*x*u^2*v+8071*y*u^2*v-F(31)/11*z*u^2*v-3434*x^2*v^2-F(107)/64*x*y*v^2+14380*y^2*v^2-8045*x*z*v^2+F(49)/89*y*z*v^2-F(71)/101*z^2*v^2+F(69)/40*x*u*v^2+F(63)/20*y*u*v^2+F(17)/64*z*u*v^2+3817*x*v^3-6924*y*v^3-6700*z*v^3,3533*x^3*y+F(15)/86*x^2*y^2-113*x*y^3+1934*y^4-F(102)/101*x^3*z-F(70)/29*x^2*y*z-9646*x*y^2*z-7947*y^3*z+2337*x^2*z^2+5232*x*y*z^2+F(108)/113*y^2*z^2+2045*x*z^3+14008*y*z^3-303*z^4+F(86)/11*x^2*y*u-8832*x*y^2*u+14164*y^3*u-1737*x^2*z*u-F(91)/93*x*y*z*u+F(67)/73*y^2*z*u+8793*x*z^2*u-F(73)/6*y*z^2*u-F(103)/124*z^3*u+4204*x*y*u^2+F(49)/11*y^2*u^2-10355*x*z*u^2-2269*y*z*u^2+F(104)/77*z^2*u^2+F(27)/110*y*u^3+F(71)/50*z*u^3+F(73)/93*x^2*y*v-F(58)/11*x*y^2*v+F(4)/83*y^3*v-F(63)/43*x^2*z*v+F(95)/3*x*y*z*v+F(36)/61*y^2*z*v-F(21)/17*x*z^2*v+5691*y*z^2*v-F(103)/29*z^3*v-F(51)/92*x*y*u*v+F(119)/117*y^2*u*v-F(107)/26*x*z*u*v+105*y*z*u*v+945*z^2*u*v+F(47)/41*y*u^2*v-F(113)/2*z*u^2*v+3848*x*y*v^2+2679*y^2*v^2+F(20)/93*x*z*v^2+F(89)/112*y*z*v^2-F(102)/49*z^2*v^2+F(67)/35*y*u*v^2-F(73)/99*z*u*v^2-4595*y*v^3+11234*z*v^3,x^4+F(78)/67*x^3*y-8352*x^2*y^2-7005*x*y^3-F(39)/38*y^4-F(99)/59*x^3*z-7556*x^2*y*z+F(68)/111*x*y^2*z-F(115)/124*y^3*z+12716*x^2*z^2-F(1)/24*x*y*z^2-F(30)/59*y^2*z^2-10050*x*z^3-F(31)/2*y*z^3-F(110)/53*x^3*u+13232*x^2*y*u+11810*x*y^2*u+F(25)/91*y^3*u-F(84)/95*x^2*z*u+F(90)/103*x*y*z*u-14920*y^2*z*u-11323*x*z^2*u+353*y*z^2*u+1158*x^2*u^2-F(101)/4*x*y*u^2+3442*y^2*u^2-F(31)/78*x*z*u^2-4024*y*z*u^2+F(73)/15*x*u^3-10800*y*u^3-F(33)/113*x^3*v-F(35)/86*x^2*y*v+F(101)/109*x*y^2*v-8668*y^3*v-F(37)/15*x^2*z*v-F(5)/68*x*y*z*v-F(93)/107*y^2*z*v+12257*x*z^2*v+2450*y*z^2*v-F(68)/81*x^2*u*v+1590*x*y*u*v+F(79)/33*y^2*u*v+F(88)/17*x*z*u*v-F(41)/64*y*z*u*v-12960*x*u^2*v+5746*y*u^2*v-5558*x^2*v^2+F(66)/91*x*y*v^2-F(106)/39*y^2*v^2+F(101)/49*x*z*v^2+F(77)/69*y*z*v^2+F(36)/71*x*u*v^2+F(7)/23*y*u*v^2-F(85)/7*x*v^3-14163*y*v^3]))
end


@doc raw"""
    k3_d9_pi8()

Return a smooth K3 surface in $\mathbb P^4$ with degree `9` and sectional genus `8`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function k3_d9_pi8()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-x^3*y-F(22)/93*x^2*y^2-13647*x*y^3+F(46)/33*y^4-4661*x^3*z+1593*x^2*y*z+14520*x*y^2*z+F(7)/107*y^3*z-F(76)/53*x^2*z^2-F(116)/45*x*y*z^2-F(57)/40*y^2*z^2-10824*x*z^3+F(34)/63*y*z^3+11277*z^4-F(124)/105*x^3*u+F(1)/68*x^2*y*u+F(71)/48*x*y^2*u-F(88)/109*y^3*u-1808*x^2*z*u-1291*x*y*z*u+F(102)/47*y^2*z*u+3782*x*z^2*u+F(4)/39*y*z^2*u-F(5)/3*z^3*u+3156*x^2*u^2+F(37)/119*x*y*u^2-F(84)/59*y^2*u^2+F(75)/37*x*z*u^2-F(57)/83*y*z*u^2-145*z^2*u^2+F(108)/49*x*u^3+11793*y*u^3-817*z*u^3+F(77)/76*u^4-F(119)/67*x^2*y*v+F(125)/54*x*y^2*v-361*y^3*v+F(17)/93*x^2*z*v+F(107)/28*x*y*z*v+F(11)/68*y^2*z*v+4753*x*z^2*v-F(74)/9*y*z^2*v-4448*z^3*v+F(53)/41*x^2*u*v+13966*x*y*u*v+F(12)/113*y^2*u*v+9014*x*z*u*v+F(109)/20*y*z*u*v-F(49)/111*z^2*u*v+103*x*u^2*v-F(95)/54*y*u^2*v-15712*z*u^2*v-10345*u^3*v-F(79)/80*x*y*v^2+F(45)/58*y^2*v^2+F(11)/20*x*z*v^2+F(45)/64*y*z*v^2+F(67)/95*z^2*v^2-5109*x*u*v^2-F(86)/99*y*u*v^2-8164*z*u*v^2-F(3)/94*u^2*v^2-F(94)/81*y*v^3-7886*z*v^3+F(111)/7*u*v^3,F(87)/16*x^3*y+F(61)/10*x^2*y^2+2307*x*y^3+F(71)/26*y^4+2123*x^3*z-F(63)/113*x^2*y*z+4786*x*y^2*z+4340*y^3*z+F(106)/81*x^2*z^2-F(83)/14*x*y*z^2-F(11)/16*y^2*z^2+F(86)/93*x*z^3-15628*y*z^3+1027*z^4+F(111)/77*x^3*u-6831*x^2*y*u+F(109)/20*x*y^2*u-F(15)/46*y^3*u+1239*x^2*z*u+F(63)/62*x*y*z*u+303*y^2*z*u-F(1)/16*x*z^2*u-F(1)/33*y*z^2*u-10594*z^3*u+F(123)/101*x^2*u^2+11165*x*y*u^2+F(46)/123*y^2*u^2+13395*x*z*u^2+8115*y*z*u^2-13547*z^2*u^2-F(97)/20*x*u^3+F(59)/13*y*u^3+14347*z*u^3-7727*u^4+4661*x^3*v-F(65)/88*x^2*y*v+F(28)/61*x*y^2*v-F(37)/120*y^3*v-9687*x^2*z*v+F(57)/121*x*y*z*v+F(84)/97*y^2*z*v-8740*x*z^2*v-F(47)/126*y*z^2*v-F(87)/94*z^3*v+F(121)/65*x^2*u*v+F(71)/29*x*y*u*v+F(99)/59*y^2*u*v+13552*x*z*u*v-F(125)/74*y*z*u*v+4081*z^2*u*v-1582*x*u^2*v+F(29)/118*y*u^2*v+F(122)/85*z*u^2*v-F(83)/15*u^3*v-F(17)/93*x^2*v^2+F(37)/72*x*y*v^2+F(71)/50*y^2*v^2+F(77)/15*x*z*v^2-F(43)/82*y*z*v^2-F(103)/25*z^2*v^2+F(26)/25*x*u*v^2+F(8)/75*y*u*v^2-4669*z*u*v^2-F(95)/89*u^2*v^2-F(11)/20*x*v^3+3212*y*v^3-F(100)/67*z*v^3-7815*u*v^3+7886*v^4,-F(48)/115*x^3*y+F(87)/71*x^2*y^2+F(85)/47*x*y^3+F(38)/125*y^4+F(19)/21*x^3*z-13874*x^2*y*z-F(113)/41*x*y^2*z-F(13)/70*y^3*z-15627*x^2*z^2+F(111)/20*x*y*z^2+F(54)/121*y^2*z^2+F(92)/123*x*z^3+11774*y*z^3+5575*z^4-F(61)/56*x^3*u+F(5)/2*x^2*y*u+6367*x*y^2*u+F(112)/61*y^3*u+F(3)/47*x^2*z*u+F(43)/95*x*y*z*u+F(58)/67*y^2*z*u+F(9)/92*x*z^2*u-F(41)/115*y*z^2*u-9298*z^3*u-2762*x^2*u^2+F(1)/10*x*y*u^2+F(65)/123*y^2*u^2-2497*x*z*u^2-F(14)/11*y*z*u^2+F(80)/123*z^2*u^2-5448*x*u^3+F(39)/2*y*u^3-F(47)/86*z*u^3+F(12)/67*u^4-F(5)/108*x^3*v-F(47)/8*x^2*y*v+9831*x*y^2*v+F(94)/107*y^3*v-F(111)/28*x^2*z*v-F(75)/92*x*y*z*v+F(22)/19*y^2*z*v-F(65)/123*x*z^2*v-130*y*z^2*v+12152*z^3*v-F(64)/63*x^2*u*v+F(21)/109*x*y*u*v+1873*y^2*u*v+F(86)/65*x*z*u*v-F(58)/59*y*z*u*v+F(97)/99*z^2*u*v+6887*x*u^2*v+3871*y*u^2*v-F(81)/52*z*u^2*v+F(69)/64*u^3*v+F(25)/103*x^2*v^2-6428*x*y*v^2+11478*y^2*v^2-F(88)/29*x*z*v^2+F(81)/32*y*z*v^2-15698*z^2*v^2-F(26)/19*x*u*v^2-11106*y*u*v^2-13407*z*u*v^2+14668*u^2*v^2+F(104)/23*x*v^3+10007*y*v^3-F(97)/89*z*v^3-F(77)/61*u*v^3+F(67)/87*v^4,F(124)/47*x^3*y+F(21)/43*x^2*y^2+F(73)/60*x*y^3+F(4)/111*y^4-10074*x^3*z-3177*x^2*y*z-F(53)/71*x*y^2*z-F(94)/43*y^3*z+F(9)/101*x^2*z^2+851*x*y*z^2+F(100)/37*y^2*z^2+7278*x*z^3-F(16)/37*y*z^3+9984*z^4-861*x^3*u-F(79)/45*x^2*y*u-F(31)/103*x*y^2*u-1258*y^3*u+F(96)/121*x^2*z*u-F(61)/111*x*y*z*u+136*y^2*z*u+F(1)/44*x*z^2*u+8710*y*z^2*u-F(107)/115*z^3*u-F(7)/6*x^2*u^2-6312*x*y*u^2-F(122)/25*y^2*u^2-F(42)/67*x*z*u^2+F(55)/28*y*z*u^2+F(76)/99*z^2*u^2+F(93)/77*x*u^3-F(20)/91*y*u^3+F(71)/112*z*u^3-7133*u^4+F(112)/33*x^3*v+11036*x^2*y*v-F(83)/125*x*y^2*v-15452*y^3*v-F(1)/15*x^2*z*v+F(115)/44*x*y*z*v-12979*y^2*z*v-7368*x*z^2*v-F(16)/93*y*z^2*v-F(85)/81*z^3*v-F(58)/25*x^2*u*v-3660*x*y*u*v-6200*y^2*u*v-F(87)/59*x*z*u*v-F(19)/18*y*z*u*v-F(15)/19*z^2*u*v-8247*x*u^2*v-9715*y*u^2*v-13870*z*u^2*v-F(88)/109*u^3*v-9436*x^2*v^2-F(73)/79*x*y*v^2-F(107)/73*y^2*v^2-F(106)/91*x*z*v^2+F(93)/40*y*z*v^2-F(105)/22*z^2*v^2+F(21)/23*x*u*v^2-15420*y*u*v^2-F(110)/63*z*u*v^2+F(89)/58*u^2*v^2+12846*x*v^3+9973*y*v^3+F(108)/115*z*v^3+F(100)/93*u*v^3-F(81)/94*v^4,-F(46)/119*x^3*y+15437*x^2*y^2-F(92)/113*x*y^3+F(57)/52*y^4+F(71)/13*x^3*z-F(29)/72*x^2*y*z-F(59)/9*x*y^2*z-15013*y^3*z-F(86)/51*x^2*z^2+7457*x*y*z^2-F(115)/24*y^2*z^2-14559*x*z^3-F(13)/3*y*z^3-F(93)/59*z^4-F(80)/17*x^3*u-7942*x^2*y*u-4861*x*y^2*u-15722*y^3*u+12422*x^2*z*u+F(9)/52*x*y*z*u-4228*y^2*z*u-F(80)/33*x*z^2*u-F(22)/75*y*z^2*u-F(49)/110*z^3*u-F(58)/3*x^2*u^2-F(123)/44*x*y*u^2-8650*y^2*u^2+11856*x*z*u^2-13079*y*z*u^2+F(23)/33*z^2*u^2-5359*x*u^3+F(42)/31*y*u^3+12246*z*u^3-F(115)/123*u^4-15523*x^3*v-F(100)/93*x^2*y*v+F(101)/31*x*y^2*v-F(42)/47*y^3*v-14084*x^2*z*v+8748*x*y*z*v+F(13)/83*y^2*z*v+F(58)/33*x*z^2*v+11121*y*z^2*v-12018*z^3*v-F(91)/121*x^2*u*v-14672*x*y*u*v-15887*y^2*u*v-12115*x*z*u*v-F(14)/27*y*z*u*v+F(54)/7*z^2*u*v-F(68)/29*x*u^2*v+F(46)/61*y*u^2*v+F(67)/37*z*u^2*v+F(123)/125*u^3*v-F(25)/18*x^2*v^2-4861*x*y*v^2-10074*y^2*v^2+F(39)/5*x*z*v^2-F(61)/50*y*z*v^2+15395*z^2*v^2-F(42)/67*x*u*v^2-10366*y*u*v^2-8276*z*u*v^2-F(47)/52*u^2*v^2+F(44)/101*x*v^3+F(89)/124*y*v^3-707*z*v^3-F(2)/37*u*v^3+3473*v^4,x^4+2056*x^3*y-F(109)/2*x^2*y^2+F(108)/85*x*y^3+5860*y^4-13955*x^3*z+6744*x^2*y*z+10816*x*y^2*z-8164*y^3*z+5957*x^2*z^2-11779*x*y*z^2-F(67)/96*y^2*z^2+F(53)/115*x*z^3-F(37)/50*y*z^3+F(49)/54*z^4+F(107)/96*x^3*u-7174*x^2*y*u+14452*x*y^2*u-F(13)/7*y^3*u+4150*x^2*z*u+3316*x*y*z*u+4166*y^2*z*u-2849*x*z^2*u-F(57)/100*y*z^2*u-11945*z^3*u+530*x^2*u^2-F(88)/31*x*y*u^2-F(81)/38*y^2*u^2-F(56)/111*x*z*u^2-4354*y*z*u^2+14349*z^2*u^2-5880*x*u^3+F(53)/107*y*u^3+9115*z*u^3+F(119)/59*u^4+7855*x^3*v+9577*x^2*y*v+F(38)/69*x*y^2*v-F(111)/113*y^3*v-F(59)/10*x^2*z*v-F(53)/99*x*y*z*v-F(1)/2*y^2*z*v+1755*x*z^2*v-F(51)/115*y*z^2*v-8084*z^3*v-F(107)/52*x^2*u*v+11316*x*y*u*v-F(83)/100*y^2*u*v+11451*x*z*u*v+3906*y*z*u*v+3053*z^2*u*v+F(125)/84*x*u^2*v+F(11)/85*y*u^2*v-962*z*u^2*v+F(39)/88*u^3*v-F(8)/75*x^2*v^2-1447*x*y*v^2-F(33)/26*y^2*v^2+12089*x*z*v^2+8918*y*z*v^2-4904*z^2*v^2+F(64)/9*x*u*v^2+10207*y*u*v^2+F(105)/41*z*u*v^2+8384*u^2*v^2-8639*x*v^3-4746*y*v^3+F(126)/107*z*v^3-6626*u*v^3+F(63)/109*v^4]))
end


@doc raw"""
    rational_d10_pi8()

Return a smooth rational surface in $\mathbb P^4$ with degree `10` and sectional genus `8`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d10_pi8()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-x^4*y*z-13256*x^3*y^2*z+F(108)/35*x^2*y^3*z+12890*x^4*z^2-9294*x^3*y*z^2-15882*x^2*y^2*z^2-F(44)/103*x*y^3*z^2+215*x^3*z^3-4449*x^2*y*z^3-F(58)/49*x*y^2*z^3+F(49)/74*x^2*z^4+3935*x*y*z^4-12274*x^5*u+F(19)/87*x^4*y*u-F(58)/107*x^3*y^2*u+6746*x^4*z*u-F(101)/75*x^3*y*z*u-F(49)/2*x^2*y^2*z*u-9801*x*y^3*z*u-F(35)/111*y^4*z*u-7132*x^3*z^2*u+F(85)/47*x^2*y*z^2*u+F(13)/29*x*y^2*z^2*u-14631*y^3*z^2*u+907*x^2*z^3*u-F(74)/69*x*y*z^3*u+10372*y^2*z^3*u-9865*x*z^4*u+6171*y*z^4*u-130*z^5*u+F(67)/97*x^4*u^2-215*x^3*y*u^2-F(76)/105*x^3*z*u^2-F(35)/76*x^2*y*z*u^2+F(51)/49*x*y^2*z*u^2+7798*y^3*z*u^2-F(103)/32*x^2*z^2*u^2-6205*x*y*z^2*u^2-F(121)/123*y^2*z^2*u^2+5194*x*z^3*u^2+F(57)/50*y*z^3*u^2+12700*z^4*u^2+4096*x^5*v-F(5)/46*x^4*y*v+F(89)/16*x^3*y^2*v-7202*x^4*z*v+2817*x^3*y*z*v-8034*x^2*y^2*z*v+F(63)/5*x*y^3*z*v-2241*y^4*z*v+F(73)/94*x^3*z^2*v-F(31)/83*x^2*y*z^2*v+F(49)/6*x*y^2*z^2*v-15581*y^3*z^2*v+F(122)/91*x^2*z^3*v+1799*x*y*z^3*v+F(23)/29*y^2*z^3*v+F(87)/100*x*z^4*v+10901*y*z^4*v-14673*z^5*v+5296*x^4*u*v+F(71)/125*x^3*y*u*v+F(122)/107*x^3*z*u*v+1445*x^2*y*z*u*v+F(21)/124*x*y^2*z*u*v-F(2)/115*y^3*z*u*v-9858*x^2*z^2*u*v+15348*x*y*z^2*u*v-F(79)/70*y^2*z^2*u*v-11137*x*z^3*u*v+F(1)/113*y*z^3*u*v-11718*z^4*u*v-12985*x^4*v^2+13955*x^3*y*v^2-F(93)/86*x^3*z*v^2-5547*x^2*y*z*v^2+12051*x*y^2*z*v^2-4394*y^3*z*v^2+F(61)/79*x^2*z^2*v^2-F(17)/118*x*y*z^2*v^2-4538*y^2*z^2*v^2+10247*x*z^3*v^2+F(57)/86*y*z^3*v^2+F(91)/34*z^4*v^2,-312*x^3*y^2*z-F(21)/85*x^2*y^3*z+F(9)/94*x*y^4*z-F(47)/101*x^3*y*z^2-F(46)/81*x^2*y^2*z^2+3421*x*y^3*z^2-F(50)/33*y^4*z^2+F(65)/31*x^2*y*z^3-12475*x*y^2*z^3-F(33)/16*y^3*z^3+F(69)/38*x*y*z^4-F(11)/61*y^2*z^4-12274*x^2*y^3*u+F(19)/87*x*y^4*u-F(58)/107*y^5*u-F(49)/87*x^4*z*u+F(29)/123*x^3*y*z*u+3131*x^2*y^2*z*u-F(22)/73*x*y^3*z*u-F(25)/19*y^4*z*u+F(43)/97*x^3*z^2*u-11875*x^2*y*z^2*u-12526*x*y^2*z^2*u-4698*y^3*z^2*u-8032*x^2*z^3*u-F(45)/64*x*y*z^3*u-F(103)/102*y^2*z^3*u+12052*x*z^4*u+1857*y*z^4*u-F(20)/83*z^5*u+F(67)/97*x*y^3*u^2-215*y^4*u^2+F(35)/123*x^3*z*u^2-F(17)/29*x^2*y*z*u^2-F(71)/112*x*y^2*z*u^2+F(110)/7*y^3*z*u^2-F(95)/51*x^2*z^2*u^2+14971*x*y*z^2*u^2-F(57)/125*y^2*z^2*u^2+F(109)/13*x*z^3*u^2-F(30)/67*y*z^3*u^2+F(103)/52*z^4*u^2+4096*x^2*y^3*v-F(5)/46*x*y^4*v+F(89)/16*y^5*v-1932*x^4*z*v-10747*x^3*y*z*v+8931*x^2*y^2*z*v-7961*x*y^3*z*v-F(11)/83*y^4*z*v+15517*x^3*z^2*v+F(106)/81*x^2*y*z^2*v-F(98)/115*x*y^2*z^2*v-F(33)/113*y^3*z^2*v+F(3)/118*x^2*z^3*v+F(35)/67*x*y*z^3*v-10355*y^2*z^3*v+15044*x*z^4*v+10991*y*z^4*v+F(32)/125*z^5*v+5296*x*y^3*u*v+F(71)/125*y^4*u*v-F(53)/30*x^3*z*u*v+F(1)/55*x^2*y*z*u*v-F(110)/63*x*y^2*z*u*v+F(66)/25*y^3*z*u*v+F(83)/119*x^2*z^2*u*v-F(39)/17*x*y*z^2*u*v-F(23)/104*y^2*z^2*u*v+F(5)/23*x*z^3*u*v-F(59)/86*y*z^3*u*v+14336*z^4*u*v-12985*x*y^3*v^2+13955*y^4*v^2-F(46)/51*x^3*z*v^2-F(58)/39*x^2*y*z*v^2+F(71)/27*x*y^2*z*v^2+14197*y^3*z*v^2-F(71)/115*x^2*z^2*v^2-F(41)/33*x*y*z^2*v^2-84*y^2*z^2*v^2+F(82)/99*x*z^3*v^2-F(108)/11*y*z^3*v^2+F(34)/27*z^4*v^2,x^4*y^2+13256*x^3*y^3-F(108)/35*x^2*y^4-12890*x^4*y*z+9294*x^3*y^2*z+15882*x^2*y^3*z+F(44)/103*x*y^4*z-215*x^3*y*z^2+4449*x^2*y^2*z^2+F(58)/49*x*y^3*z^2-F(49)/74*x^2*y*z^3-3935*x*y^2*z^3+F(32)/99*x^5*u+12120*x^4*y*u-F(95)/49*x^3*y^2*u+F(10)/81*x^2*y^3*u+9801*x*y^4*u+F(35)/111*y^5*u+1908*x^4*z*u-F(16)/57*x^3*y*z*u+F(61)/15*x^2*y^2*z*u-F(13)/29*x*y^3*z*u+14631*y^4*z*u+F(2)/75*x^3*z^2*u-4116*x^2*y*z^2*u+F(74)/69*x*y^2*z^2*u-10372*y^3*z^2*u+9865*x*y*z^3*u-6171*y^2*z^3*u+130*y*z^4*u-14054*x^4*u^2-F(7)/38*x^3*y*u^2+1287*x^2*y^2*u^2-F(51)/49*x*y^3*u^2-7798*y^4*u^2+F(63)/113*x^3*z*u^2+F(23)/125*x^2*y*z*u^2+6205*x*y^2*z*u^2+F(121)/123*y^3*z*u^2-5194*x*y*z^2*u^2-F(57)/50*y^2*z^2*u^2-12700*y*z^3*u^2+11080*x^5*v+11826*x^4*y*v+F(85)/57*x^3*y^2*v-F(101)/28*x^2*y^3*v-F(63)/5*x*y^4*v+2241*y^5*v+3539*x^4*z*v+10711*x^3*y*z*v-F(23)/93*x^2*y^2*z*v-F(49)/6*x*y^3*z*v+15581*y^4*z*v-F(65)/54*x^3*z^2*v-14903*x^2*y*z^2*v-1799*x*y^2*z^2*v-F(23)/29*y^3*z^2*v-F(87)/100*x*y*z^3*v-10901*y^2*z^3*v+14673*y*z^4*v-F(75)/34*x^4*u*v-14409*x^3*y*u*v+F(9)/97*x^2*y^2*u*v-F(21)/124*x*y^3*u*v+F(2)/115*y^4*u*v-F(51)/73*x^3*z*u*v+6554*x^2*y*z*u*v-15348*x*y^2*z*u*v+F(79)/70*y^3*z*u*v+11137*x*y*z^2*u*v-F(1)/113*y^2*z^2*u*v+11718*y*z^3*u*v-5939*x^4*v^2-163*x^3*y*v^2-F(97)/37*x^2*y^2*v^2-12051*x*y^3*v^2+4394*y^4*v^2-F(115)/9*x^3*z*v^2+F(61)/27*x^2*y*z*v^2+F(17)/118*x*y^2*z*v^2+4538*y^3*z*v^2-10247*x*y*z^2*v^2-F(57)/86*y^2*z^2*v^2-F(91)/34*y*z^3*v^2,F(91)/54*x*y*u^4+5862*x*z*u^4-F(57)/91*y*z*u^4-2755*x*u^5-F(79)/114*y*u^5+12478*z*u^5+F(21)/61*x*y*z*u^2*v+7415*x^2*u^3*v-11252*x*y*u^3*v+7831*y^2*u^3*v+F(47)/9*x*z*u^3*v-F(57)/97*y*z*u^3*v+11026*z^2*u^3*v-F(65)/64*x*u^4*v+F(33)/41*y*u^4*v-F(93)/35*z*u^4*v+F(126)/65*u^5*v+F(83)/11*x*y*z*u*v^2-F(52)/81*x^2*u^2*v^2+F(51)/23*x*y*u^2*v^2-9747*y^2*u^2*v^2-9582*x*z*u^2*v^2-F(37)/103*y*z*u^2*v^2+F(3)/110*z^2*u^2*v^2+F(103)/8*x*u^3*v^2-F(13)/57*y*u^3*v^2-4539*z*u^3*v^2-5102*u^4*v^2-6360*x^2*u*v^3+4058*x*y*u*v^3+12747*y^2*u*v^3-9164*x*z*u*v^3+F(42)/125*y*z*u*v^3-F(59)/26*z^2*u*v^3+2056*x*u^2*v^3+F(35)/102*y*u^2*v^3+F(59)/117*z*u^2*v^3-F(125)/83*u^3*v^3+1261*x*y*v^4-F(56)/71*x*z*v^4+F(110)/101*y*z*v^4-1260*x*u*v^4-10389*y*u*v^4+6882*z*u*v^4-F(73)/93*u^2*v^4+F(107)/91*x*v^5+7636*y*v^5-F(105)/22*z*v^5+6837*u*v^5,-F(21)/61*x^2*y*z*u-7415*x^3*u^2+7663*x^2*y*u^2-7831*x*y^2*u^2-15892*x^2*z*u^2-F(31)/105*x*y*z*u^2-11026*x*z^2*u^2+F(74)/9*x^2*u^3-10314*x*y*u^3+F(107)/60*y^2*u^3+F(71)/119*x*z*u^3-F(109)/84*y*z*u^3-9544*z^2*u^3-F(29)/55*x*u^4-F(83)/27*y*u^4+F(36)/43*z*u^4-F(83)/11*x^2*y*z*v+F(52)/81*x^3*u*v-14581*x^2*y*u*v+9747*x*y^2*u*v+7281*x^2*z*u*v+15348*x*y*z*u*v-F(3)/110*x*z^2*u*v-14384*x^2*u^2*v-4253*x*y*u^2*v-F(82)/79*y^2*u^2*v-F(19)/51*x*z*u^2*v-F(68)/27*y*z*u^2*v-1342*z^2*u^2*v-F(98)/109*x*u^3*v-3536*y*u^3*v+F(64)/67*z*u^3*v+6360*x^3*v^2+9420*x^2*y*v^2-12747*x*y^2*v^2-F(3)/112*x^2*z*v^2+F(17)/38*x*y*z*v^2+F(59)/26*x*z^2*v^2-F(21)/32*x^2*u*v^2-F(117)/44*x*y*u*v^2+F(107)/22*y^2*u*v^2+11713*x*z*u*v^2-F(86)/83*y*z*u*v^2-F(1)/20*z^2*u*v^2-11334*x*u^2*v^2+F(42)/37*y*u^2*v^2-7572*z*u^2*v^2+F(5)/99*x^2*v^3+2148*x*y*v^3+623*y^2*v^3-1466*x*z*v^3+F(69)/37*y*z*v^3-3660*z^2*v^3+F(21)/115*x*u*v^3+7806*y*u*v^3+6123*z*u*v^3-F(72)/115*x*v^4+F(77)/106*y*v^4+F(113)/88*z*v^4,-F(21)/61*x*y^2*z*u-7415*x^2*y*u^2+14822*x*y^2*u^2-7831*y^3*u^2-F(17)/114*x*y*z*u^2-F(75)/112*y^2*z*u^2-11026*y*z^2*u^2+7187*x^2*u^3+F(77)/37*x*y*u^3-F(103)/77*y^2*u^3-F(80)/79*x*z*u^3+F(112)/109*y*z*u^3-F(6)/53*z^2*u^3-F(31)/110*x*u^4-F(85)/2*y*u^4-14366*z*u^4-F(83)/11*x*y^2*z*v+F(52)/81*x^2*y*u*v-15179*x*y^2*u*v+9747*y^3*u*v-F(64)/89*x*y*z*u*v-146*y^2*z*u*v-F(3)/110*y*z^2*u*v-F(113)/67*x^2*u^2*v-10608*x*y*u^2*v+11594*y^2*u^2*v+F(124)/75*x*z*u^2*v-7207*y*z*u^2*v-F(99)/20*z^2*u^2*v-9382*x*u^3*v+8473*y*u^3*v+10719*z*u^3*v+6360*x^2*y*v^2-8604*x*y^2*v^2-12747*y^3*v^2+3952*x*y*z*v^2+14871*y^2*z*v^2+F(59)/26*y*z^2*v^2-F(59)/33*x^2*u*v^2-12658*x*y*u*v^2-F(23)/4*y^2*u*v^2+1443*x*z*u*v^2+F(53)/55*y*z*u*v^2-4079*z^2*u*v^2+7240*x*u^2*v^2+F(83)/65*y*u^2*v^2+4778*z*u^2*v^2-5302*x^2*v^3-15192*x*y*v^3-F(95)/88*y^2*v^3+1419*x*z*v^3+3081*y*z*v^3-F(89)/20*z^2*v^3+F(26)/19*x*u*v^3+11016*y*u*v^3+6728*z*u*v^3+11856*x*v^4-879*y*v^4-F(1)/99*z*v^4,-F(21)/61*x*y*z^2*u-7415*x^2*z*u^2+F(47)/114*x*y*z*u^2-7831*y^2*z*u^2+11525*x*z^2*u^2-F(113)/95*y*z^2*u^2-11026*z^3*u^2+F(69)/52*x^2*u^3-F(9)/86*x*y*u^3+1512*y^2*u^3+12291*x*z*u^3+8832*y*z*u^3+F(26)/19*z^2*u^3-4531*x*u^4-12525*y*u^4-F(95)/26*z*u^4-F(83)/11*x*y*z^2*v+F(52)/81*x^2*z*u*v+F(92)/107*x*y*z*u*v+9747*y^2*z*u*v+565*x*z^2*u*v-F(62)/107*y*z^2*u*v-F(3)/110*z^3*u*v+F(24)/11*x^2*u^2*v+F(91)/121*x*y*u^2*v-F(62)/43*y^2*u^2*v-F(66)/59*x*z*u^2*v+F(47)/71*y*z*u^2*v-F(32)/63*z^2*u^2*v-F(26)/23*x*u^3*v+F(71)/66*y*u^3*v+F(83)/88*z*u^3*v+6360*x^2*z*v^2-F(55)/56*x*y*z*v^2-12747*y^2*z*v^2+14297*x*z^2*v^2+5212*y*z^2*v^2+F(59)/26*z^3*v^2+4994*x^2*u*v^2+13677*x*y*u*v^2+F(55)/72*y^2*u*v^2+F(39)/107*x*z*u*v^2+F(47)/123*y*z*u*v^2+1291*z^2*u*v^2+F(54)/71*x*u^2*v^2+2927*y*u^2*v^2+F(11)/3*z*u^2*v^2-134*x^2*v^3-F(106)/81*x*y*v^3-14171*y^2*v^3-194*x*z*v^3-F(29)/104*y*z*v^3-5574*z^2*v^3+9574*x*u*v^3-F(109)/42*y*u*v^3+7495*z*u*v^3-4277*x*v^4+F(53)/38*y*v^4-F(8)/41*z*v^4,F(98)/75*x^3*y*u+12843*x^2*y^2*u+F(59)/92*x*y^3*u+F(23)/7*x^3*z*u+F(34)/35*x^2*y*z*u-F(113)/49*x*y^2*z*u-14347*y^3*z*u-F(94)/19*x^2*z^2*u-F(103)/27*x*y*z^2*u+3531*y^2*z^2*u-14270*x*z^3*u+F(121)/119*y*z^3*u-F(69)/44*x^3*u^2-F(119)/65*x^2*y*u^2+3745*x*y^2*u^2+3772*y^3*u^2+15931*x^2*z*u^2-F(31)/116*x*y*z*u^2+5250*y^2*z*u^2-4221*x*z^2*u^2-3833*y*z^2*u^2+F(33)/20*z^3*u^2+F(123)/2*x^2*u^3-8453*x*y*u^3-F(8)/13*y^2*u^3-F(114)/13*x*z*u^3-F(73)/32*y*z*u^3+12494*z^2*u^3-8635*x^3*y*v+F(78)/53*x*y^3*v+F(4)/19*x^3*z*v-F(57)/65*x^2*y*z*v-F(15)/14*x*y^2*z*v+F(23)/15*y^3*z*v-13099*x*y*z^2*v-F(17)/122*x*z^3*v-5144*y*z^3*v+9789*x^3*u*v+F(115)/36*x^2*y*u*v+F(113)/89*x*y^2*u*v-11378*y^3*u*v-F(17)/23*x^2*z*u*v+F(43)/95*x*y*z*u*v-F(117)/55*y^2*z*u*v+F(118)/89*x*z^2*u*v+F(5)/51*y*z^2*u*v-6723*z^3*u*v-6566*x^2*u^2*v+F(123)/88*x*y*u^2*v-F(32)/97*y^2*u^2*v+F(24)/83*x*z*u^2*v+F(91)/95*y*z*u^2*v+F(53)/59*z^2*u^2*v-F(43)/57*x^3*v^2+F(89)/14*x^2*y*v^2+13143*x*y^2*v^2-F(89)/115*y^3*v^2-3481*x^2*z*v^2+F(13)/2*y^2*z*v^2+F(106)/81*x*z^2*v^2+F(65)/58*y*z^2*v^2+8794*z^3*v^2-F(52)/97*x^2*u*v^2+F(20)/121*x*y*u*v^2-F(66)/35*y^2*u*v^2+7367*x*z*u*v^2-F(10)/69*y*z*u*v^2-F(23)/99*z^2*u*v^2,F(91)/54*x^3*y*u-15698*x*y^3*u+5862*x^3*z*u-F(57)/91*x^2*y*z*u-F(109)/88*x*y^2*z*u+F(19)/37*y^3*z*u-F(69)/47*x*y*z^2*u+F(15)/4*x*z^3*u+8605*y*z^3*u-2755*x^3*u^2-F(79)/114*x^2*y*u^2+F(100)/53*x*y^2*u^2-10160*y^3*u^2+12478*x^2*z*u^2-F(103)/25*y^2*z*u^2-F(12)/37*x*z^2*u^2+7280*y*z^2*u^2-8088*z^3*u^2+162*x^3*y*v+13317*x^2*y^2*v+F(109)/65*x*y^3*v-F(59)/78*x^3*z*v+13917*x^2*y*z*v-F(71)/9*x*y^2*z*v+15532*y^3*z*v+F(11)/47*x^2*z^2*v+14561*x*y*z^2*v+13772*y^2*z^2*v-F(11)/67*x*z^3*v+15201*y*z^3*v-F(67)/40*x^3*u*v-12069*x^2*y*u*v-F(75)/58*x*y^2*u*v+7901*y^3*u*v+F(55)/14*x^2*z*u*v-12120*x*y*z*u*v-F(61)/51*y^2*z*u*v-F(84)/17*x*z^2*u*v-F(43)/121*y*z^2*u*v-F(85)/82*z^3*u*v+F(55)/49*x^2*u^2*v+F(25)/41*x*y*u^2*v-13028*y^2*u^2*v-4619*x*z*u^2*v+F(20)/101*y*z*u^2*v-F(87)/61*z^2*u^2*v+12266*x^3*v^2-F(125)/47*x^2*y*v^2-F(119)/76*x*y^2*v^2-2920*y^3*v^2-F(29)/115*x^2*z*v^2+12823*x*y*z*v^2+F(47)/106*y^2*z*v^2+F(107)/104*x*z^2*v^2-13618*y*z^2*v^2+9973*z^3*v^2-F(123)/61*x^2*u*v^2-F(125)/106*x*y*u*v^2+15113*y^2*u*v^2+1148*x*z*u*v^2-12082*y*z*u*v^2-F(113)/71*z^2*u*v^2+7750*x^2*v^3+F(125)/19*x*y*v^3+222*y^2*v^3+6831*x*z*v^3-F(81)/83*y*z*v^3+7257*z^2*v^3,12274*x^3*y*u-F(19)/87*x^2*y^2*u+F(58)/107*x*y^3*u-F(32)/99*x^3*z*u+F(19)/39*x^2*y*z*u+F(103)/34*x*y^2*z*u-963*y^3*z*u-1908*x^2*z^2*u+11061*x*y*z^2*u-F(31)/13*y^2*z^2*u-F(2)/75*x*z^3*u+F(99)/10*y*z^3*u-F(67)/97*x^2*y*u^2+215*x*y^2*u^2+14054*x^2*z*u^2+F(100)/13*x*y*z*u^2+1660*y^2*z*u^2-F(63)/113*x*z^2*u^2-F(11)/28*y*z^2*u^2-4096*x^3*y*v+F(5)/46*x^2*y^2*v-F(89)/16*x*y^3*v-11080*x^3*z*v+F(100)/83*x^2*y*z*v-1696*x*y^2*z*v+4610*y^3*z*v-3539*x^2*z^2*v+10729*x*y*z^2*v-F(62)/47*y^2*z^2*v+F(65)/54*x*z^3*v-F(32)/95*y*z^3*v-5296*x^2*y*u*v-F(71)/125*x*y^2*u*v+F(75)/34*x^2*z*u*v-3830*x*y*z*u*v+F(106)/83*y^2*z*u*v+F(51)/73*x*z^2*u*v+3304*y*z^2*u*v+12985*x^2*y*v^2-13955*x*y^2*v^2+5939*x^2*z*v^2-F(47)/104*x*y*z*v^2+4685*y^2*z*v^2+F(115)/9*x*z^2*v^2+F(116)/31*y*z^2*v^2,-F(37)/100*x^2*y^2*u-F(33)/8*x^2*y*z*u+F(52)/63*x*y^2*z*u-F(16)/61*x^3*u^2-F(73)/118*x^2*y*u^2-2037*x*y^2*u^2+F(86)/19*y^3*u^2+9465*x^2*z*u^2+F(37)/96*x*y*z*u^2-5059*y^2*z*u^2+6566*x*z^2*u^2+2118*y*z^2*u^2+F(103)/74*x^2*u^3-F(37)/79*x*y*u^3-F(53)/85*y^2*u^3-3711*x*z*u^3-F(63)/37*y*z*u^3+8635*x^2*y^2*v-F(4)/19*x^2*y*z*v+F(57)/65*x*y^2*z*v+12511*x^3*u*v+F(106)/89*x^2*y*u*v-F(103)/85*x*y^2*u*v+2295*y^3*u*v-12574*x^2*z*u*v-F(32)/109*x*y*z*u*v+F(73)/8*y^2*z*u*v+3978*x*z^2*u*v+F(74)/125*y*z^2*u*v+F(72)/79*x^2*u^2*v-15568*x*y*u^2*v-3184*y^2*u^2*v+F(107)/24*x*z*u^2*v+12748*y*z*u^2*v+F(43)/57*x^2*y*v^2-F(89)/14*x*y^2*v^2+3481*x*y*z*v^2+10326*x^2*u*v^2+F(21)/61*x*y*u*v^2+F(26)/49*y^2*u*v^2-F(3)/25*x*z*u*v^2-F(76)/101*y*z*u*v^2,-F(91)/54*x^2*y^2*u-5862*x^2*y*z*u+F(57)/91*x*y^2*z*u+2755*x^2*y*u^2+F(79)/114*x*y^2*u^2-12478*x*y*z*u^2+3751*x^2*y^2*v-F(7)/65*x^2*y*z*v-F(31)/30*x*y^2*z*v+F(77)/102*x^3*u*v-14692*x^2*y*u*v-F(72)/25*x*y^2*u*v+F(39)/85*y^3*u*v+12101*x^2*z*u*v+250*x*y*z*u*v+2590*y^2*z*u*v+6076*x*z^2*u*v-F(51)/74*y*z^2*u*v+F(121)/124*x^2*u^2*v-F(33)/98*x*y*u^2*v+14321*y^2*u^2*v+F(100)/77*x*z*u^2*v-F(19)/58*y*z*u^2*v-F(31)/101*x^3*v^2-F(81)/107*x^2*y*v^2-F(42)/107*x*y^2*v^2-11299*y^3*v^2+F(23)/8*x^2*z*v^2-F(14)/41*x*y*z*v^2+10962*y^2*z*v^2+F(105)/44*x*z^2*v^2-F(95)/124*y*z^2*v^2+F(105)/103*x^2*u*v^2-9553*x*y*u*v^2+F(47)/43*y^2*u*v^2-F(21)/101*x*z*u*v^2-15898*y*z*u*v^2-13841*x^2*v^3-5749*x*y*v^3-F(17)/26*y^2*v^3+10428*x*z*v^3+F(109)/65*y*z*v^3,-F(37)/100*x*y^2*z*u-F(33)/8*x*y*z^2*u+F(52)/63*y^2*z^2*u-F(20)/101*x^2*y*u^2-F(57)/103*x*y^2*u^2-F(125)/21*y^3*u^2-F(117)/35*x^2*z*u^2+F(103)/113*x*y*z*u^2-F(82)/109*y^2*z*u^2-12763*x*z^2*u^2+F(39)/19*y*z^2*u^2-F(123)/98*z^3*u^2-F(47)/36*x*y*u^3+F(117)/31*y^2*u^3-F(76)/21*x*z*u^3-7424*y*z*u^3+F(32)/77*z^2*u^3+8635*x*y^2*z*v-F(4)/19*x*y*z^2*v+F(57)/65*y^2*z^2*v+F(116)/59*x^2*y*u*v-F(109)/21*x*y^2*u*v+F(69)/86*y^3*u*v+F(11)/17*x^2*z*u*v+1479*x*y*z*u*v-F(51)/32*y^2*z*u*v+F(23)/25*x*z^2*u*v+11509*y*z^2*u*v+F(42)/55*z^3*u*v+F(39)/62*x*y*u^2*v+F(61)/94*y^2*u^2*v-F(37)/77*x*z*u^2*v+F(101)/18*y*z*u^2*v-F(71)/115*z^2*u^2*v+F(43)/57*x*y*z*v^2-F(89)/14*y^2*z*v^2+3481*y*z^2*v^2+F(81)/61*x*y*u*v^2+F(25)/48*y^2*u*v^2-F(94)/11*x*z*u*v^2+F(63)/73*y*z*u*v^2+F(14)/79*z^2*u*v^2,-F(91)/54*x*y^2*z*u-5862*x*y*z^2*u+F(57)/91*y^2*z^2*u+2755*x*y*z*u^2+F(79)/114*y^2*z*u^2-12478*y*z^2*u^2+3751*x*y^2*z*v-F(7)/65*x*y*z^2*v-F(31)/30*y^2*z^2*v+14636*x^2*y*u*v-F(68)/95*x*y^2*u*v-F(43)/35*y^3*u*v+9920*x^2*z*u*v+F(107)/33*x*y*z*u*v+9319*y^2*z*u*v+12990*x*z^2*u*v-7469*y*z^2*u*v-11978*z^3*u*v-F(92)/9*x*y*u^2*v+1325*y^2*u^2*v-F(17)/103*x*z*u^2*v+9397*y*z*u^2*v-F(77)/93*z^2*u^2*v-F(103)/69*x^2*y*v^2-F(98)/71*x*y^2*v^2+F(40)/87*y^3*v^2+12861*x^2*z*v^2+F(30)/37*x*y*z*v^2-F(68)/125*y^2*z*v^2+F(54)/13*x*z^2*v^2-7352*y*z^2*v^2+F(9)/28*z^3*v^2-5111*x*y*u*v^2-F(77)/39*y^2*u*v^2+F(50)/41*x*z*u*v^2+15202*y*z*u*v^2+F(81)/16*z^2*u*v^2-F(103)/62*x*y*v^3-F(76)/37*y^2*v^3-F(49)/59*x*z*v^3-F(121)/24*y*z*v^3-F(70)/31*z^2*v^3,-F(37)/100*x^2*y*z*u-F(33)/8*x^2*z^2*u+F(52)/63*x*y*z^2*u+F(25)/126*x^3*u^2+F(39)/67*x^2*y*u^2+1617*x*y^2*u^2-13869*x^2*z*u^2-F(87)/94*x*y*z*u^2+5989*y^2*z*u^2-F(115)/19*x*z^2*u^2-F(7)/75*y*z^2*u^2-7566*z^3*u^2-15065*x^2*u^3+F(94)/19*x*y*u^3+F(50)/29*x*z*u^3+11967*y*z*u^3-F(77)/86*z^2*u^3+8635*x^2*y*z*v-F(4)/19*x^2*z^2*v+F(57)/65*x*y*z^2*v+F(7)/3*x^3*u*v+F(121)/50*x^2*y*u*v-F(95)/8*x*y^2*u*v-F(67)/57*x^2*z*u*v-8540*x*y*z*u*v+F(39)/25*y^2*z*u*v+F(1)/110*x*z^2*u*v+F(117)/19*y*z^2*u*v-F(35)/117*z^3*u*v+F(47)/57*x^2*u^2*v-F(66)/7*x*y*u^2*v+5371*x*z*u^2*v+227*y*z*u^2*v+9018*z^2*u^2*v+F(43)/57*x^2*z*v^2-F(89)/14*x*y*z*v^2+3481*x*z^2*v^2-12897*x^2*u*v^2-F(73)/74*x*y*u*v^2-14308*x*z*u*v^2-7178*y*z*u*v^2-15087*z^2*u*v^2,-F(91)/54*x^2*y*z*u-5862*x^2*z^2*u+F(57)/91*x*y*z^2*u+2755*x^2*z*u^2+F(79)/114*x*y*z*u^2-12478*x*z^2*u^2+3751*x^2*y*z*v-F(7)/65*x^2*z^2*v-F(31)/30*x*y*z^2*v-F(30)/43*x^3*u*v-F(22)/65*x^2*y*u*v-12828*x*y^2*u*v-2122*x^2*z*u*v-F(107)/25*x*y*z*u*v+5214*y^2*z*u*v+F(31)/17*x*z^2*u*v-F(115)/19*y*z^2*u*v+15912*z^3*u*v+F(8)/5*x^2*u^2*v-F(11)/47*x*y*u^2*v+4311*x*z*u^2*v-F(109)/120*y*z*u^2*v+13079*z^2*u^2*v-157*x^3*v^2+13639*x^2*y*v^2-F(53)/70*x*y^2*v^2+F(7)/118*x^2*z*v^2+11401*x*y*z*v^2-15293*y^2*z*v^2+14286*x*z^2*v^2-9855*y*z^2*v^2+F(23)/50*z^3*v^2+F(112)/37*x^2*u*v^2-F(73)/110*x*y*u*v^2+7436*x*z*u*v^2+F(58)/51*y*z*u*v^2+F(111)/112*z^2*u*v^2+8597*x^2*v^3-F(30)/47*x*y*v^3+F(51)/100*x*z*v^3-F(78)/89*y*z*v^3+F(20)/39*z^2*v^3,-F(91)/54*x*y*z*u^2-5862*x*z^2*u^2+F(57)/91*y*z^2*u^2+2755*x*z*u^3+F(79)/114*y*z*u^3-12478*z^2*u^3+6481*x*y*z*u*v-F(38)/119*x*z^2*u*v-F(31)/90*y*z^2*u*v-F(69)/52*x^2*u^2*v+F(9)/86*x*y*u^2*v-1512*y^2*u^2*v-F(95)/77*x*z*u^2*v-F(74)/69*y*z*u^2*v-4184*z^2*u^2*v+4531*x*u^3*v+12525*y*u^3*v-F(114)/77*z*u^3*v+F(104)/115*x*y*z*v^2-15102*x*z^2*v^2-F(31)/118*y*z^2*v^2-F(24)/11*x^2*u*v^2-F(91)/121*x*y*u*v^2+F(62)/43*y^2*u*v^2+F(91)/108*x*z*u*v^2-5570*y*z*u*v^2+9267*z^2*u*v^2+F(26)/23*x*u^2*v^2-F(71)/66*y*u^2*v^2+9827*z*u^2*v^2+6129*x*z*v^3-9295*y*z*v^3+F(95)/47*z^2*v^3+12475*x*u*v^3-9465*y*u*v^3+F(41)/115*z*u*v^3,-F(91)/54*x*y^2*u^2-5862*x*y*z*u^2+F(57)/91*y^2*z*u^2+2755*x*y*u^3+F(79)/114*y^2*u^3-12478*y*z*u^3-3570*x*y^2*u*v-F(123)/98*x*y*z*u*v+15558*y^2*z*u*v-7187*x^2*u^2*v+F(15)/62*x*y*u^2*v-F(47)/121*y^2*u^2*v+F(80)/79*x*z*u^2*v-F(35)/66*y*z*u^2*v+F(6)/53*z^2*u^2*v+F(31)/110*x*u^3*v-F(65)/53*y*u^3*v+14366*z*u^3*v+7943*x*y^2*v^2-70*x*y*z*v^2-F(29)/21*y^2*z*v^2+F(113)/67*x^2*u*v^2+11471*x*y*u*v^2+F(38)/39*y^2*u*v^2-F(124)/75*x*z*u*v^2-3534*y*z*u*v^2+F(99)/20*z^2*u*v^2+9382*x*u^2*v^2-F(67)/19*y*u^2*v^2-10719*z*u^2*v^2-5081*x*y*v^3+F(52)/87*y^2*v^3-F(47)/25*y*z*v^3+F(118)/101*x*u*v^3+921*y*u*v^3+F(1)/89*z*u*v^3,-F(91)/54*x^2*y*u^2-5862*x^2*z*u^2+F(57)/91*x*y*z*u^2+2755*x^2*u^3+F(79)/114*x*y*u^3-12478*x*z*u^3+3589*x^2*y*u*v-F(71)/17*x^2*z*u*v+F(79)/29*x*y*z*u*v+10712*x^2*u^2*v+12654*x*y*u^2*v-F(107)/60*y^2*u^2*v-F(83)/38*x*z*u^2*v+F(109)/84*y*z*u^2*v+9544*z^2*u^2*v+F(107)/51*x*u^3*v+F(83)/27*y*u^3*v-F(36)/43*z*u^3*v+F(21)/53*x^2*y*v^2-F(15)/52*x^2*z*v^2+F(94)/29*x*y*z*v^2-3685*x^2*u*v^2+12619*x*y*u*v^2+F(82)/79*y^2*u*v^2-F(19)/87*x*z*u*v^2+F(68)/27*y*z*u*v^2+1342*z^2*u*v^2-1354*x*u^2*v^2+3536*y*u^2*v^2-F(64)/67*z*u^2*v^2-F(37)/113*x^2*v^3+F(79)/83*x*y*v^3+F(7)/4*x*z*v^3+F(113)/27*x*u*v^3+F(119)/82*y*u*v^3+13729*z*u*v^3]))
end


@doc raw"""
    rational_d10_pi9_quart_1()

Return a smooth rational surface in $\mathbb P^4$ with degree `10` and sectional genus `9` which is contained in precisely one quartic.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d10_pi9_quart_1()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(17)/116*x^4*y+15896*x^3*y^2-F(85)/48*x^2*y^3+217*x*y^4-F(31)/38*x^4*z-3464*x^3*y*z+F(117)/16*x^2*y^2*z-F(19)/6*x*y^3*z+F(28)/27*x^3*z^2-F(88)/39*x^2*y*z^2-650*x*y^2*z^2+8976*x^2*z^3-F(13)/89*x*y*z^3+F(8)/65*x*z^4+3775*x^4*u-5153*x^3*y*u+F(112)/95*x^2*y^2*u+5542*x*y^3*u-2122*x^3*z*u+4895*x^2*y*z*u-14526*x*y^2*z*u+F(15)/73*x^2*z^2*u+F(1)/69*x*y*z^2*u+7747*x*z^3*u-F(59)/50*x^3*u^2+F(111)/109*x^2*y*u^2+F(62)/35*x*y^2*u^2-F(121)/20*x^2*z*u^2+10050*x*y*z*u^2-F(67)/94*x*z^2*u^2+10289*x^2*u^3-F(97)/37*x*y*u^3-F(61)/2*x*z*u^3+F(70)/43*x*u^4+2016*x^4*v+F(21)/79*x^3*y*v+F(104)/27*x^2*y^2*v+F(103)/97*x*y^3*v+F(69)/107*y^4*v+F(59)/19*x^3*z*v+F(59)/69*x^2*y*z*v-F(101)/9*x*y^2*z*v-F(124)/79*y^3*z*v-F(6)/119*x^2*z^2*v-F(37)/101*x*y*z^2*v+F(76)/51*y^2*z^2*v+F(41)/52*x*z^3*v+11006*y*z^3*v-13629*z^4*v-15890*x^3*u*v-F(71)/15*x^2*y*u*v+F(19)/93*x*y^2*u*v-F(92)/67*y^3*u*v-F(6)/53*x^2*z*u*v+14798*x*y*z*u*v+3250*y^2*z*u*v+1213*x*z^2*u*v-12589*y*z^2*u*v+F(49)/66*z^3*u*v-F(115)/101*x^2*u^2*v+F(85)/126*x*y*u^2*v+8712*y^2*u^2*v+1210*x*z*u^2*v+F(75)/31*y*z*u^2*v-10429*z^2*u^2*v-F(41)/77*x*u^3*v+F(57)/68*y*u^3*v+1660*z*u^3*v-F(99)/86*u^4*v-F(28)/25*x^3*v^2+6636*x^2*y*v^2+8863*x*y^2*v^2+12400*y^3*v^2-F(85)/18*x^2*z*v^2-F(80)/121*x*y*z*v^2+F(70)/23*y^2*z*v^2-2124*x*z^2*v^2-F(59)/29*y*z^2*v^2-F(44)/75*z^3*v^2-F(22)/79*x^2*u*v^2+F(84)/59*x*y*u*v^2+F(103)/20*y^2*u*v^2-1088*x*z*u*v^2+F(47)/87*y*z*u*v^2-F(19)/113*z^2*u*v^2-F(81)/104*x*u^2*v^2-2120*y*u^2*v^2-12528*z*u^2*v^2+F(120)/43*u^3*v^2+3737*x^2*v^3+F(14)/67*x*y*v^3+287*y^2*v^3+F(39)/64*x*z*v^3-F(112)/15*y*z*v^3-F(73)/103*z^2*v^3+F(72)/77*x*u*v^3+F(121)/19*y*u*v^3-F(12)/17*z*u*v^3+F(63)/23*u^2*v^3-F(50)/21*x*v^4+12980*y*v^4-F(8)/17*z*v^4+F(19)/107*u*v^4-8321*v^5,F(17)/116*x^3*y^2-15896*x^2*y^3+F(85)/48*x*y^4-217*y^5+F(31)/38*x^3*y*z+3464*x^2*y^2*z-F(117)/16*x*y^3*z+F(19)/6*y^4*z-F(28)/27*x^2*y*z^2+F(88)/39*x*y^2*z^2+650*y^3*z^2-8976*x*y*z^3+F(13)/89*y^2*z^3-F(8)/65*y*z^4-3775*x^3*y*u+5153*x^2*y^2*u-F(112)/95*x*y^3*u-5542*y^4*u+2122*x^2*y*z*u-4895*x*y^2*z*u+14526*y^3*z*u-F(15)/73*x*y*z^2*u-F(1)/69*y^2*z^2*u-7747*y*z^3*u+F(59)/50*x^2*y*u^2-F(111)/109*x*y^2*u^2-F(62)/35*y^3*u^2+F(121)/20*x*y*z*u^2-10050*y^2*z*u^2+F(67)/94*y*z^2*u^2-10289*x*y*u^3+F(97)/37*y^2*u^3+F(61)/2*y*z*u^3-F(70)/43*y*u^4+F(65)/59*x^3*y*v+10785*x^2*y^2*v-F(124)/5*x*y^3*v-F(84)/109*y^4*v-F(65)/34*x^3*z*v-F(113)/55*x^2*y*z*v+F(78)/41*x*y^2*z*v+1808*y^3*z*v-1708*x^2*z^2*v+F(86)/21*x*y*z^2*v-7539*y^2*z^2*v+7600*x*z^3*v-15137*y*z^3*v-F(109)/114*z^4*v-F(25)/99*x^3*u*v-F(73)/99*x^2*y*u*v-1869*x*y^2*u*v-9028*y^3*u*v+F(45)/37*x^2*z*u*v+7194*x*y*z*u*v+F(3)/50*y^2*z*u*v-F(121)/12*x*z^2*u*v-F(37)/113*y*z^2*u*v-1704*z^3*u*v-F(46)/15*x^2*u^2*v-F(49)/50*x*y*u^2*v-F(101)/54*y^2*u^2*v-F(37)/110*x*z*u^2*v+14671*y*z*u^2*v-F(47)/102*z^2*u^2*v-F(54)/37*x*u^3*v-6897*y*u^3*v-F(13)/50*z*u^3*v-9986*u^4*v+7514*x^3*v^2+F(112)/9*x^2*y*v^2+F(87)/55*x*y^2*v^2-F(122)/71*y^3*v^2+4167*x^2*z*v^2-F(118)/119*x*y*z*v^2-5126*y^2*z*v^2+F(111)/91*x*z^2*v^2+7257*y*z^2*v^2+5038*z^3*v^2+F(3)/5*x^2*u*v^2+F(33)/112*x*y*u*v^2-F(67)/22*y^2*u*v^2+F(17)/79*x*z*u*v^2-F(82)/19*y*z*u*v^2-F(34)/109*z^2*u*v^2-F(2)/49*x*u^2*v^2-F(29)/95*y*u^2*v^2-F(29)/24*z*u^2*v^2+F(109)/3*u^3*v^2+F(5)/116*x^2*v^3-9616*x*y*v^3+F(83)/76*y^2*v^3+F(113)/79*x*z*v^3+F(106)/67*y*z*v^3+F(48)/85*z^2*v^3-6310*x*u*v^3-F(53)/86*y*u*v^3-F(98)/23*z*u*v^3+11282*u^2*v^3-5913*x*v^4+F(116)/115*y*v^4-1609*z*v^4+F(89)/124*u*v^4+2572*v^5,F(17)/116*x^3*y*z-15896*x^2*y^2*z+F(85)/48*x*y^3*z-217*y^4*z+F(31)/38*x^3*z^2+3464*x^2*y*z^2-F(117)/16*x*y^2*z^2+F(19)/6*y^3*z^2-F(28)/27*x^2*z^3+F(88)/39*x*y*z^3+650*y^2*z^3-8976*x*z^4+F(13)/89*y*z^4-F(8)/65*z^5-3775*x^3*z*u+5153*x^2*y*z*u-F(112)/95*x*y^2*z*u-5542*y^3*z*u+2122*x^2*z^2*u-4895*x*y*z^2*u+14526*y^2*z^2*u-F(15)/73*x*z^3*u-F(1)/69*y*z^3*u-7747*z^4*u+F(59)/50*x^2*z*u^2-F(111)/109*x*y*z*u^2-F(62)/35*y^2*z*u^2+F(121)/20*x*z^2*u^2-10050*y*z^2*u^2+F(67)/94*z^3*u^2-10289*x*z*u^3+F(97)/37*y*z*u^3+F(61)/2*z^2*u^3-F(70)/43*z*u^4+4823*x^3*y*v-F(1)/17*x^2*y^2*v+104*x*y^3*v+7671*y^4*v-1061*x^3*z*v+F(49)/78*x^2*y*z*v-F(91)/88*x*y^2*z*v+F(87)/10*y^3*z*v-10290*x^2*z^2*v+11333*x*y*z^2*v-F(44)/101*y^2*z^2*v-1382*x*z^3*v-F(41)/55*y*z^3*v+F(93)/61*z^4*v-F(109)/74*x^3*u*v+6873*x^2*y*u*v+7706*x*y^2*u*v+3276*y^3*u*v-F(5)/11*x^2*z*u*v-F(72)/103*x*y*z*u*v-9906*y^2*z*u*v-F(100)/87*x*z^2*u*v+F(47)/99*y*z^2*u*v-F(44)/101*z^3*u*v+1894*x^2*u^2*v-11909*x*y*u^2*v+F(81)/44*y^2*u^2*v+6843*x*z*u^2*v+8303*y*z*u^2*v-F(8)/27*z^2*u^2*v+7458*x*u^3*v+13233*y*u^3*v-F(5)/21*z*u^3*v+F(78)/89*u^4*v-F(85)/49*x^3*v^2-F(17)/21*x^2*y*v^2-6033*x*y^2*v^2-1251*y^3*v^2+F(93)/95*x^2*z*v^2+F(28)/59*x*y*z*v^2+3535*y^2*z*v^2-12831*x*z^2*v^2-F(111)/38*y*z^2*v^2+11908*z^3*v^2+F(100)/77*x^2*u*v^2-7318*x*y*u*v^2-F(88)/109*y^2*u*v^2+226*x*z*u*v^2+11795*y*z*u*v^2+9120*z^2*u*v^2+12111*x*u^2*v^2-F(48)/103*y*u^2*v^2+9915*z*u^2*v^2+F(49)/123*u^3*v^2+3073*x^2*v^3+13957*x*y*v^3+6330*y^2*v^3+F(21)/125*x*z*v^3+F(1)/20*y*z*v^3-F(49)/41*z^2*v^3+F(49)/9*x*u*v^3+F(71)/114*y*u*v^3-F(94)/45*z*u*v^3+F(74)/77*u^2*v^3-F(11)/51*x*v^4-1551*y*v^4+F(43)/79*z*v^4-F(76)/65*u*v^4-F(12)/25*v^5,F(17)/116*x^3*y*u-15896*x^2*y^2*u+F(85)/48*x*y^3*u-217*y^4*u+F(31)/38*x^3*z*u+3464*x^2*y*z*u-F(117)/16*x*y^2*z*u+F(19)/6*y^3*z*u-F(28)/27*x^2*z^2*u+F(88)/39*x*y*z^2*u+650*y^2*z^2*u-8976*x*z^3*u+F(13)/89*y*z^3*u-F(8)/65*z^4*u-3775*x^3*u^2+5153*x^2*y*u^2-F(112)/95*x*y^2*u^2-5542*y^3*u^2+2122*x^2*z*u^2-4895*x*y*z*u^2+14526*y^2*z*u^2-F(15)/73*x*z^2*u^2-F(1)/69*y*z^2*u^2-7747*z^3*u^2+F(59)/50*x^2*u^3-F(111)/109*x*y*u^3-F(62)/35*y^2*u^3+F(121)/20*x*z*u^3-10050*y*z*u^3+F(67)/94*z^2*u^3-10289*x*u^4+F(97)/37*y*u^4+F(61)/2*z*u^4-F(70)/43*u^5+F(21)/67*x^3*y*v+F(97)/71*x^2*y^2*v+2446*x*y^3*v+F(41)/68*y^4*v+13920*x^3*z*v+F(31)/81*x^2*y*z*v+F(90)/119*x*y^2*z*v-14124*y^3*z*v+6316*x^2*z^2*v+9689*x*y*z^2*v+14831*y^2*z^2*v-F(42)/59*x*z^3*v+11870*y*z^3*v-F(97)/59*z^4*v-F(43)/20*x^3*u*v-5005*x^2*y*u*v+F(58)/119*x*y^2*u*v-3291*y^3*u*v+F(93)/119*x^2*z*u*v-F(62)/65*x*y*z*u*v-F(15)/94*y^2*z*u*v-1172*x*z^2*u*v+7395*y*z^2*u*v+F(49)/48*z^3*u*v+F(9)/112*x^2*u^2*v-5969*x*y*u^2*v-F(29)/50*y^2*u^2*v-F(125)/34*x*z*u^2*v+F(59)/66*y*z*u^2*v+F(53)/78*z^2*u^2*v+F(67)/123*x*u^3*v-F(125)/46*y*u^3*v-10369*z*u^3*v-F(8)/9*u^4*v+F(83)/72*x^3*v^2-1485*x^2*y*v^2+F(44)/109*x*y^2*v^2+F(113)/24*y^3*v^2+F(3)/65*x^2*z*v^2+5705*x*y*z*v^2+F(112)/89*y^2*z*v^2+F(106)/119*x*z^2*v^2+10482*y*z^2*v^2+2652*z^3*v^2+F(72)/91*x^2*u*v^2+F(119)/9*x*y*u*v^2-F(82)/105*y^2*u*v^2+F(113)/114*x*z*u*v^2+10477*y*z*u*v^2+F(88)/29*z^2*u*v^2-670*x*u^2*v^2+F(107)/114*y*u^2*v^2-14589*z*u^2*v^2+15255*u^3*v^2+407*x^2*v^3-10114*x*y*v^3-2696*y^2*v^3-7216*x*z*v^3+F(52)/51*y*z*v^3-F(66)/37*z^2*v^3-F(20)/121*x*u*v^3-F(45)/49*y*u*v^3+4709*z*u*v^3+994*u^2*v^3-5637*x*v^4-4690*y*v^4+F(29)/22*z*v^4-F(124)/37*u*v^4+F(16)/65*v^5,F(21)/67*x^4*y+F(97)/71*x^3*y^2+2446*x^2*y^3+F(41)/68*x*y^4+13920*x^4*z+F(31)/81*x^3*y*z+F(90)/119*x^2*y^2*z-14124*x*y^3*z+6316*x^3*z^2+9689*x^2*y*z^2+14831*x*y^2*z^2-F(42)/59*x^2*z^3+11870*x*y*z^3-F(97)/59*x*z^4-F(43)/108*x^4*u+F(11)/25*x^3*y*u-F(81)/80*x^2*y^2*u-8237*x*y^3*u+F(69)/107*y^4*u+3527*x^3*z*u-F(103)/104*x^2*y*z*u-1108*x*y^2*z*u-F(124)/79*y^3*z*u-F(101)/115*x^2*z^2*u+13096*x*y*z^2*u+F(76)/51*y^2*z^2*u-14712*x*z^3*u+11006*y*z^3*u-13629*z^4*u-F(116)/99*x^3*u^2-F(52)/25*x^2*y*u^2+F(56)/89*x*y^2*u^2-F(92)/67*y^3*u^2+227*x^2*z*u^2-8952*x*y*z*u^2+3250*y^2*z*u^2-6579*x*z^2*u^2-12589*y*z^2*u^2+F(49)/66*z^3*u^2-F(19)/45*x^2*u^3+F(66)/119*x*y*u^3+8712*y^2*u^3-9159*x*z*u^3+F(75)/31*y*z*u^3-10429*z^2*u^3+F(22)/17*x*u^4+F(57)/68*y*u^4+1660*z*u^4-F(99)/86*u^5+F(83)/72*x^4*v-1485*x^3*y*v+F(44)/109*x^2*y^2*v+F(113)/24*x*y^3*v+F(3)/65*x^3*z*v+5705*x^2*y*z*v+F(112)/89*x*y^2*z*v+F(106)/119*x^2*z^2*v+10482*x*y*z^2*v+2652*x*z^3*v+F(76)/111*x^3*u*v-F(115)/93*x^2*y*u*v+F(8)/69*x*y^2*u*v+12400*y^3*u*v+F(105)/22*x^2*z*u*v-F(37)/75*x*y*z*u*v+F(70)/23*y^2*z*u*v+F(103)/40*x*z^2*u*v-F(59)/29*y*z^2*u*v-F(44)/75*z^3*u*v+13098*x^2*u^2*v+2176*x*y*u^2*v+F(103)/20*y^2*u^2*v-15677*x*z*u^2*v+F(47)/87*y*z*u^2*v-F(19)/113*z^2*u^2*v-F(121)/40*x*u^3*v-2120*y*u^3*v-12528*z*u^3*v+F(120)/43*u^4*v+407*x^3*v^2-10114*x^2*y*v^2-2696*x*y^2*v^2-7216*x^2*z*v^2+F(52)/51*x*y*z*v^2-F(66)/37*x*z^2*v^2+F(34)/113*x^2*u*v^2+4891*x*y*u*v^2+287*y^2*u*v^2+F(31)/76*x*z*u*v^2-F(112)/15*y*z*u*v^2-F(73)/103*z^2*u*v^2+164*x*u^2*v^2+F(121)/19*y*u^2*v^2-F(12)/17*z*u^2*v^2+F(63)/23*u^3*v^2-5637*x^2*v^3-4690*x*y*v^3+F(29)/22*x*z*v^3+7570*x*u*v^3+12980*y*u*v^3-F(8)/17*z*u*v^3+F(19)/107*u^2*v^3+F(16)/65*x*v^4-8321*u*v^4,-F(21)/67*x^3*y^2-F(97)/71*x^2*y^3-2446*x*y^4-F(41)/68*y^5-13920*x^3*y*z-F(31)/81*x^2*y^2*z-F(90)/119*x*y^3*z+14124*y^4*z-6316*x^2*y*z^2-9689*x*y^2*z^2-14831*y^3*z^2+F(42)/59*x*y*z^3-11870*y^2*z^3+F(97)/59*y*z^4+F(78)/49*x^3*y*u+15790*x^2*y^2*u-F(41)/8*x*y^3*u-14026*y^4*u-F(65)/34*x^3*z*u-8669*x^2*y*z*u-8304*x*y^2*z*u+F(75)/71*y^3*z*u-1708*x^2*z^2*u+8793*x*y*z^2*u-F(73)/15*y^2*z^2*u+7600*x*z^3*u+191*y*z^3*u-F(109)/114*z^4*u-F(25)/99*x^3*u^2+F(86)/103*x^2*y*u^2-F(55)/39*x*y^2*u^2-F(57)/74*y^3*u^2+F(45)/37*x^2*z*u^2+F(36)/37*x*y*z*u^2+F(123)/85*y^2*z*u^2-F(121)/12*x*z^2*u^2+F(17)/48*y*z^2*u^2-1704*z^3*u^2-F(46)/15*x^2*u^3+9086*x*y*u^3+F(1)/62*y^2*u^3-F(37)/110*x*z*u^3+F(82)/23*y*z*u^3-F(47)/102*z^2*u^3-F(54)/37*x*u^4+213*y*u^4-F(13)/50*z*u^4-9986*u^5-F(83)/72*x^3*y*v+1485*x^2*y^2*v-F(44)/109*x*y^3*v-F(113)/24*y^4*v-F(3)/65*x^2*y*z*v-5705*x*y^2*z*v-F(112)/89*y^3*z*v-F(106)/119*x*y*z^2*v-10482*y^2*z^2*v-2652*y*z^3*v+7514*x^3*u*v+9191*x^2*y*u*v-F(77)/101*x*y^2*u*v+7307*y^3*u*v+4167*x^2*z*u*v+F(105)/41*x*y*z*u*v+F(97)/41*y^2*z*u*v+F(111)/91*x*z^2*u*v-468*y*z^2*u*v+5038*z^3*u*v+F(3)/5*x^2*u^2*v+F(111)/79*x*y*u^2*v+F(123)/110*y^2*u^2*v+F(17)/79*x*z*u^2*v-F(78)/83*y*z*u^2*v-F(34)/109*z^2*u^2*v-F(2)/49*x*u^3*v+F(5)/96*y*u^3*v-F(29)/24*z*u^3*v+F(109)/3*u^4*v-407*x^2*y*v^2+10114*x*y^2*v^2+2696*y^3*v^2+7216*x*y*z*v^2-F(52)/51*y^2*z*v^2+F(66)/37*y*z^2*v^2+F(5)/116*x^2*u*v^2+F(97)/126*x*y*u*v^2+8687*y^2*u*v^2+F(113)/79*x*z*u*v^2+7707*y*z*u*v^2+F(48)/85*z^2*u*v^2-6310*x*u^2*v^2+11281*y*u^2*v^2-F(98)/23*z*u^2*v^2+11282*u^3*v^2+5637*x*y*v^3+4690*y^2*v^3-F(29)/22*y*z*v^3-5913*x*u*v^3-9108*y*u*v^3-1609*z*u*v^3+F(89)/124*u^2*v^3-F(16)/65*y*v^4+2572*u*v^4,-F(21)/67*x^3*y*z-F(97)/71*x^2*y^2*z-2446*x*y^3*z-F(41)/68*y^4*z-13920*x^3*z^2-F(31)/81*x^2*y*z^2-F(90)/119*x*y^2*z^2+14124*y^3*z^2-6316*x^2*z^3-9689*x*y*z^3-14831*y^2*z^3+F(42)/59*x*z^4-11870*y*z^4+F(97)/59*z^5+4823*x^3*y*u-F(1)/17*x^2*y^2*u+104*x*y^3*u+7671*y^4*u+10138*x^3*z*u+F(124)/65*x^2*y*z*u-F(15)/46*x*y^2*z*u+12897*y^3*z*u-F(1)/93*x^2*z^2*u-11798*x*y*z^2*u+7396*y^2*z^2*u-210*x*z^3*u-F(114)/73*y*z^3*u+F(79)/5*z^4*u-F(109)/74*x^3*u^2+6873*x^2*y*u^2+7706*x*y^2*u^2+3276*y^3*u^2-15243*x^2*z*u^2+10006*x*y*z*u^2+F(22)/29*y^2*z*u^2-15582*x*z^2*u^2+2100*y*z^2*u^2-127*z^3*u^2+1894*x^2*u^3-11909*x*y*u^3+F(81)/44*y^2*u^3+F(4)/39*x*z*u^3-F(60)/19*y*z*u^3+F(5)/4*z^2*u^3+7458*x*u^4+13233*y*u^4+F(41)/63*z*u^4+F(78)/89*u^5-F(83)/72*x^3*z*v+1485*x^2*y*z*v-F(44)/109*x*y^2*z*v-F(113)/24*y^3*z*v-F(3)/65*x^2*z^2*v-5705*x*y*z^2*v-F(112)/89*y^2*z^2*v-F(106)/119*x*z^3*v-10482*y*z^3*v-2652*z^4*v-F(85)/49*x^3*u*v-F(17)/21*x^2*y*u*v-6033*x*y^2*u*v-1251*y^3*u*v+2931*x^2*z*u*v-9110*x*y*z*u*v-F(46)/89*y^2*z*u*v+4286*x*z^2*u*v+15618*y*z^2*u*v+4183*z^3*u*v+F(100)/77*x^2*u^2*v-7318*x*y*u^2*v-F(88)/109*y^2*u^2*v-F(101)/107*x*z*u^2*v-F(4)/35*y*z*u^2*v-8282*z^2*u^2*v+12111*x*u^3*v-F(48)/103*y*u^3*v-F(49)/6*z*u^3*v+F(49)/123*u^4*v-407*x^2*z*v^2+10114*x*y*z*v^2+2696*y^2*z*v^2+7216*x*z^2*v^2-F(52)/51*y*z^2*v^2+F(66)/37*z^3*v^2+3073*x^2*u*v^2+13957*x*y*u*v^2+6330*y^2*u*v^2-F(98)/55*x*z*u*v^2-F(4)/97*y*z*u*v^2-F(126)/59*z^2*u*v^2+F(49)/9*x*u^2*v^2+F(71)/114*y*u^2*v^2-F(61)/75*z*u^2*v^2+F(74)/77*u^3*v^2+5637*x*z*v^3+4690*y*z*v^3-F(29)/22*z^2*v^3-F(11)/51*x*u*v^3-1551*y*u*v^3+F(41)/33*z*u*v^3-F(76)/65*u^2*v^3-F(16)/65*z*v^4-F(12)/25*u*v^4,4823*x^4*y-F(1)/17*x^3*y^2+104*x^2*y^3+7671*x*y^4+F(3)/67*x^4*z-F(82)/7*x^3*y*z+15177*x^2*y^2*z+F(115)/103*x*y^3*z+F(69)/107*y^4*z+3183*x^3*z^2+F(99)/97*x^2*y*z^2-F(97)/117*x*y^2*z^2-F(124)/79*y^3*z^2-F(112)/81*x^2*z^3+F(1)/25*x*y*z^3+F(76)/51*y^2*z^3-F(109)/27*x*z^4+11006*y*z^4-13629*z^5-F(109)/74*x^4*u+6873*x^3*y*u+7706*x^2*y^2*u+3276*x*y^3*u+F(33)/28*x^3*z*u+6165*x^2*y*z*u-F(48)/19*x*y^2*z*u-F(92)/67*y^3*z*u+F(17)/25*x^2*z^2*u+F(45)/14*x*y*z^2*u+3250*y^2*z^2*u+F(111)/62*x*z^3*u-12589*y*z^3*u+F(49)/66*z^4*u+1894*x^3*u^2-11909*x^2*y*u^2+F(81)/44*x*y^2*u^2+11593*x^2*z*u^2-F(25)/102*x*y*z*u^2+8712*y^2*z*u^2-1160*x*z^2*u^2+F(75)/31*y*z^2*u^2-10429*z^3*u^2+7458*x^2*u^3+13233*x*y*u^3-3463*x*z*u^3+F(57)/68*y*z*u^3+1660*z^2*u^3+F(78)/89*x*u^4-F(99)/86*z*u^4-F(85)/49*x^4*v-F(17)/21*x^3*y*v-6033*x^2*y^2*v-1251*x*y^3*v+F(50)/123*x^3*z*v+F(8)/117*x^2*y*z*v+F(119)/80*x*y^2*z*v+12400*y^3*z*v+F(14)/81*x^2*z^2*v-F(64)/47*x*y*z^2*v+F(70)/23*y^2*z^2*v-F(126)/85*x*z^3*v-F(59)/29*y*z^3*v-F(44)/75*z^4*v+F(100)/77*x^3*u*v-7318*x^2*y*u*v-F(88)/109*x*y^2*u*v-F(33)/16*x^2*z*u*v+5832*x*y*z*u*v+F(103)/20*y^2*z*u*v+8032*x*z^2*u*v+F(47)/87*y*z^2*u*v-F(19)/113*z^3*u*v+12111*x^2*u^2*v-F(48)/103*x*y*u^2*v-F(66)/125*x*z*u^2*v-2120*y*z*u^2*v-12528*z^2*u^2*v+F(49)/123*x*u^3*v+F(120)/43*z*u^3*v+3073*x^3*v^2+13957*x^2*y*v^2+6330*x*y^2*v^2+F(71)/32*x^2*z*v^2-F(87)/125*x*y*z*v^2+287*y^2*z*v^2+F(112)/121*x*z^2*v^2-F(112)/15*y*z^2*v^2-F(73)/103*z^3*v^2+F(49)/9*x^2*u*v^2+F(71)/114*x*y*u*v^2-F(105)/83*x*z*u*v^2+F(121)/19*y*z*u*v^2-F(12)/17*z^2*u*v^2+F(74)/77*x*u^2*v^2+F(63)/23*z*u^2*v^2-F(11)/51*x^2*v^3-1551*x*y*v^3-2123*x*z*v^3+12980*y*z*v^3-F(8)/17*z^2*v^3-F(76)/65*x*u*v^3+F(19)/107*z*u*v^3-F(12)/25*x*v^4-8321*z*v^4,-4823*x^3*y^2+F(1)/17*x^2*y^3-104*x*y^4-7671*y^5+F(2)/97*x^3*y*z+F(57)/116*x^2*y^2*z+F(31)/66*x*y^3*z+F(12)/101*y^4*z-F(65)/34*x^3*z^2+F(104)/73*x^2*y*z^2-F(97)/119*x*y^2*z^2+9727*y^3*z^2-1708*x^2*z^3+9003*x*y*z^3+12238*y^2*z^3+7600*x*z^4-F(91)/36*y*z^4-F(109)/114*z^5+F(109)/74*x^3*y*u-6873*x^2*y^2*u-7706*x*y^3*u-3276*y^4*u-F(25)/99*x^3*z*u-F(28)/99*x^2*y*z*u+F(2)/65*x*y^2*z*u+F(112)/73*y^3*z*u+F(45)/37*x^2*z^2*u+F(85)/48*x*y*z^2*u+536*y^2*z^2*u-F(121)/12*x*z^3*u-4538*y*z^3*u-1704*z^4*u-1894*x^2*y*u^2+11909*x*y^2*u^2-F(81)/44*y^3*u^2-F(46)/15*x^2*z*u^2+F(47)/53*x*y*z*u^2-11267*y^2*z*u^2-F(37)/110*x*z^2*u^2-F(100)/107*y*z^2*u^2-F(47)/102*z^3*u^2-7458*x*y*u^3-13233*y^2*u^3-F(54)/37*x*z*u^3+F(83)/108*y*z*u^3-F(13)/50*z^2*u^3-F(78)/89*y*u^4-9986*z*u^4+F(85)/49*x^3*y*v+F(17)/21*x^2*y^2*v+6033*x*y^3*v+1251*y^4*v+7514*x^3*z*v+F(41)/46*x^2*y*z*v-12746*x*y^2*z*v-13900*y^3*z*v+4167*x^2*z^2*v+14443*x*y*z^2*v-F(72)/83*y^2*z^2*v+F(111)/91*x*z^3*v+F(123)/55*y*z^3*v+5038*z^4*v-F(100)/77*x^2*y*u*v+7318*x*y^2*u*v+F(88)/109*y^3*u*v+F(3)/5*x^2*z*u*v+2345*x*y*z*u*v-F(66)/115*y^2*z*u*v+F(17)/79*x*z^2*u*v+F(14)/31*y*z^2*u*v-F(34)/109*z^3*u*v-12111*x*y*u^2*v+F(48)/103*y^2*u^2*v-F(2)/49*x*z*u^2*v-F(71)/113*y*z*u^2*v-F(29)/24*z^2*u^2*v-F(49)/123*y*u^3*v+F(109)/3*z*u^3*v-3073*x^2*y*v^2-13957*x*y^2*v^2-6330*y^3*v^2+F(5)/116*x^2*z*v^2+11114*x*y*z*v^2+F(99)/95*y^2*z*v^2+F(113)/79*x*z^2*v^2-F(92)/13*y*z^2*v^2+F(48)/85*z^3*v^2-F(49)/9*x*y*u*v^2-F(71)/114*y^2*u*v^2-6310*x*z*u*v^2+12988*y*z*u*v^2-F(98)/23*z^2*u*v^2-F(74)/77*y*u^2*v^2+11282*z*u^2*v^2+F(11)/51*x*y*v^3+1551*y^2*v^3-5913*x*z*v^3+585*y*z*v^3-1609*z^2*v^3+F(76)/65*y*u*v^3+F(89)/124*z*u*v^3+F(12)/25*y*v^4+2572*z*v^4,-F(67)/46*x^4*y+11082*x^3*y^2-F(51)/122*x^2*y^3-9728*x*y^4-F(69)/107*y^5+F(65)/34*x^4*z+F(33)/56*x^3*y*z-F(19)/52*x^2*y^2*z-F(39)/2*x*y^3*z+F(124)/79*y^4*z+1708*x^3*z^2+2057*x^2*y*z^2-F(49)/87*x*y^2*z^2-F(76)/51*y^3*z^2-7600*x^2*z^3+14521*x*y*z^3-11006*y^2*z^3+F(109)/114*x*z^4+13629*y*z^4+F(25)/99*x^4*u+F(31)/120*x^3*y*u-F(125)/124*x^2*y^2*u-F(62)/77*x*y^3*u+F(92)/67*y^4*u-F(45)/37*x^3*z*u-F(3)/110*x^2*y*z*u+6316*x*y^2*z*u-3250*y^3*z*u+F(121)/12*x^2*z^2*u+11244*x*y*z^2*u+12589*y^2*z^2*u+1704*x*z^3*u-F(49)/66*y*z^3*u+F(46)/15*x^3*u^2+F(55)/14*x^2*y*u^2-11001*x*y^2*u^2-8712*y^3*u^2+F(37)/110*x^2*z*u^2-15881*x*y*z*u^2-F(75)/31*y^2*z*u^2+F(47)/102*x*z^2*u^2+10429*y*z^2*u^2+F(54)/37*x^2*u^3+F(27)/35*x*y*u^3-F(57)/68*y^2*u^3+F(13)/50*x*z*u^3-1660*y*z*u^3+9986*x*u^4+F(99)/86*y*u^4-7514*x^4*v-13803*x^3*y*v-2566*x^2*y^2*v+1502*x*y^3*v-12400*y^4*v-4167*x^3*z*v+170*x^2*y*z*v+1954*x*y^2*z*v-F(70)/23*y^3*z*v-F(111)/91*x^2*z^2*v+F(110)/81*x*y*z^2*v+F(59)/29*y^2*z^2*v-5038*x*z^3*v+F(44)/75*y*z^3*v-F(3)/5*x^3*u*v+15652*x^2*y*u*v+F(1)/107*x*y^2*u*v-F(103)/20*y^3*u*v-F(17)/79*x^2*z*u*v+F(6)/37*x*y*z*u*v-F(47)/87*y^2*z*u*v+F(34)/109*x*z^2*u*v+F(19)/113*y*z^2*u*v+F(2)/49*x^2*u^2*v-F(28)/13*x*y*u^2*v+2120*y^2*u^2*v+F(29)/24*x*z*u^2*v+12528*y*z*u^2*v-F(109)/3*x*u^3*v-F(120)/43*y*u^3*v-F(5)/116*x^3*v^2+5879*x^2*y*v^2-13578*x*y^2*v^2-287*y^3*v^2-F(113)/79*x^2*z*v^2-F(85)/49*x*y*z*v^2+F(112)/15*y^2*z*v^2-F(48)/85*x*z^2*v^2+F(73)/103*y*z^2*v^2+6310*x^2*u*v^2-11445*x*y*u*v^2-F(121)/19*y^2*u*v^2+F(98)/23*x*z*u*v^2+F(12)/17*y*z*u*v^2-11282*x*u^2*v^2-F(63)/23*y*u^2*v^2+5913*x^2*v^3-F(3)/104*x*y*v^3-12980*y^2*v^3+1609*x*z*v^3+F(8)/17*y*z*v^3-F(89)/124*x*u*v^3-F(19)/107*y*u*v^3-2572*x*v^4+8321*y*v^4,-F(49)/121*x^4*y-7483*x^3*y^2-F(88)/15*x^2*y^3-14830*x*y^4-F(3)/95*x^4*z+F(17)/33*x^3*y*z+F(10)/113*x^2*y^2*z+15910*x*y^3*z+F(125)/47*x^3*z^2+F(123)/86*x^2*y*z^2-F(53)/14*x*y^2*z^2+2215*x^2*z^3-F(123)/43*x*y*z^3+6772*x*z^4+F(38)/125*x^4*u+F(9)/16*x^3*y*u-4769*x^2*y^2*u+F(49)/17*x*y^3*u+7370*x^3*z*u-F(109)/122*x^2*y*z*u-F(16)/11*x*y^2*z*u+F(99)/76*x^2*z^2*u-824*x*y*z^2*u+12212*x*z^3*u-F(79)/112*x^3*u^2+F(23)/78*x^2*y*u^2+7896*x*y^2*u^2+F(93)/101*x^2*z*u^2-11017*x*y*z*u^2-8370*x*z^2*u^2+11163*x^2*u^3-F(107)/70*x*y*u^3+F(97)/7*x*z*u^3-F(69)/70*x*u^4-F(99)/46*x^4*v+F(18)/121*x^3*y*v-F(43)/6*x^2*y^2*v-7628*x*y^3*v+976*y^4*v-F(59)/77*x^3*z*v+14008*x^2*y*z*v+F(78)/35*x*y^2*z*v+10720*y^3*z*v+F(37)/15*x^2*z^2*v+F(47)/20*x*y*z^2*v-606*y^2*z^2*v+F(55)/81*x*z^3*v-F(125)/56*y*z^3*v+F(114)/113*z^4*v+6012*x^3*u*v+F(17)/103*x^2*y*u*v-F(66)/23*x*y^2*u*v+F(66)/47*y^3*u*v+F(36)/59*x^2*z*u*v-15685*x*y*z*u*v-13560*y^2*z*u*v+2037*x*z^2*u*v+F(109)/39*y*z^2*u*v+9484*z^3*u*v+661*x^2*u^2*v+F(101)/51*x*y*u^2*v-F(28)/87*y^2*u^2*v+F(6)/67*x*z*u^2*v+F(54)/59*y*z*u^2*v+F(11)/87*z^2*u^2*v-F(110)/53*x*u^3*v+5836*y*u^3*v-F(1)/44*z*u^3*v-948*u^4*v-14270*x^3*v^2-11826*x^2*y*v^2+F(111)/70*x*y^2*v^2-12260*y^3*v^2-F(73)/99*x^2*z*v^2-7061*x*y*z*v^2+F(122)/71*y^2*z*v^2+F(85)/103*x*z^2*v^2-F(76)/67*y*z^2*v^2+F(23)/121*z^3*v^2+15162*x^2*u*v^2-6356*x*y*u*v^2+1213*y^2*u*v^2+6370*x*z*u*v^2-4432*y*z*u*v^2-1946*z^2*u*v^2-F(119)/47*x*u^2*v^2-F(41)/116*y*u^2*v^2-11039*z*u^2*v^2-1166*u^3*v^2+F(93)/32*x^2*v^3+F(24)/113*x*y*v^3-663*y^2*v^3+F(23)/122*x*z*v^3+F(40)/93*y*z*v^3-F(72)/121*z^2*v^3-15310*x*u*v^3+F(85)/61*y*u*v^3+3144*z*u*v^3+F(7)/85*u^2*v^3-7017*x*v^4-F(57)/91*y*v^4-2337*z*v^4-12769*u*v^4+F(95)/109*v^5,F(49)/121*x^3*y^2+7483*x^2*y^3+F(88)/15*x*y^4+14830*y^5+F(3)/95*x^3*y*z-F(17)/33*x^2*y^2*z-F(10)/113*x*y^3*z-15910*y^4*z-F(125)/47*x^2*y*z^2-F(123)/86*x*y^2*z^2+F(53)/14*y^3*z^2-2215*x*y*z^3+F(123)/43*y^2*z^3-6772*y*z^4-F(38)/125*x^3*y*u-F(9)/16*x^2*y^2*u+4769*x*y^3*u-F(49)/17*y^4*u-7370*x^2*y*z*u+F(109)/122*x*y^2*z*u+F(16)/11*y^3*z*u-F(99)/76*x*y*z^2*u+824*y^2*z^2*u-12212*y*z^3*u+F(79)/112*x^2*y*u^2-F(23)/78*x*y^2*u^2-7896*y^3*u^2-F(93)/101*x*y*z*u^2+11017*y^2*z*u^2+8370*y*z^2*u^2-11163*x*y*u^3+F(107)/70*y^2*u^3-F(97)/7*y*z*u^3+F(69)/70*y*u^4+F(53)/46*x^3*y*v+11205*x^2*y^2*v+4670*x*y^3*v-F(61)/74*y^4*v-10739*x^3*z*v+F(71)/74*x^2*y*z*v+F(83)/94*x*y^2*z*v+F(15)/4*y^3*z*v-F(52)/35*x^2*z^2*v-F(113)/117*x*y*z^2*v-6104*y^2*z^2*v-13887*x*z^3*v+2553*y*z^3*v+4021*z^4*v-F(111)/20*x^3*u*v+F(1)/109*x^2*y*u*v+13016*x*y^2*u*v-8127*y^3*u*v+2008*x^2*z*u*v-F(26)/115*x*y*z*u*v+F(11)/28*y^2*z*u*v+F(18)/121*x*z^2*u*v-F(118)/79*y*z^2*u*v+F(97)/100*z^3*u*v-3471*x^2*u^2*v-F(110)/3*x*y*u^2*v-15774*y^2*u^2*v-F(7)/59*x*z*u^2*v-F(112)/95*y*z*u^2*v-F(103)/33*z^2*u^2*v-12707*x*u^3*v+10712*y*u^3*v+F(117)/79*z*u^3*v+F(79)/80*u^4*v-7029*x^3*v^2+172*x^2*y*v^2+5873*x*y^2*v^2+F(2)/93*y^3*v^2-F(105)/37*x^2*z*v^2+F(11)/71*x*y*z*v^2-F(68)/59*y^2*z*v^2-14752*x*z^2*v^2-6779*y*z^2*v^2-F(77)/115*z^3*v^2-14017*x^2*u*v^2-F(71)/62*x*y*u*v^2+F(21)/46*y^2*u*v^2-9764*x*z*u*v^2+F(7)/46*y*z*u*v^2-F(120)/23*z^2*u*v^2+2197*x*u^2*v^2-6637*y*u^2*v^2+F(43)/27*z*u^2*v^2-F(19)/104*u^3*v^2+F(99)/125*x^2*v^3-F(37)/99*x*y*v^3-F(58)/9*y^2*v^3+4027*x*z*v^3-13978*y*z*v^3+3532*z^2*v^3-F(90)/37*x*u*v^3+15554*y*u*v^3-F(26)/37*z*u*v^3+F(10)/103*u^2*v^3-F(74)/119*x*v^4-11006*y*v^4-357*z*v^4-3182*u*v^4+F(41)/111*v^5,F(49)/121*x^3*y*z+7483*x^2*y^2*z+F(88)/15*x*y^3*z+14830*y^4*z+F(3)/95*x^3*z^2-F(17)/33*x^2*y*z^2-F(10)/113*x*y^2*z^2-15910*y^3*z^2-F(125)/47*x^2*z^3-F(123)/86*x*y*z^3+F(53)/14*y^2*z^3-2215*x*z^4+F(123)/43*y*z^4-6772*z^5-F(38)/125*x^3*z*u-F(9)/16*x^2*y*z*u+4769*x*y^2*z*u-F(49)/17*y^3*z*u-7370*x^2*z^2*u+F(109)/122*x*y*z^2*u+F(16)/11*y^2*z^2*u-F(99)/76*x*z^3*u+824*y*z^3*u-12212*z^4*u+F(79)/112*x^2*z*u^2-F(23)/78*x*y*z*u^2-7896*y^2*z*u^2-F(93)/101*x*z^2*u^2+11017*y*z^2*u^2+8370*z^3*u^2-11163*x*z*u^3+F(107)/70*y*z*u^3-F(97)/7*z^2*u^3+F(69)/70*z*u^4+F(93)/40*x^3*y*v+F(53)/85*x^2*y^2*v-F(13)/118*x*y^3*v-9265*y^4*v+F(66)/59*x^3*z*v-F(75)/44*x^2*y*z*v+F(53)/47*x*y^2*z*v-F(7)/12*y^3*z*v-F(83)/88*x^2*z^2*v-9751*x*y*z^2*v+F(19)/37*y^2*z^2*v-F(63)/68*x*z^3*v+F(4)/91*y*z^3*v+F(125)/79*z^4*v+F(105)/31*x^3*u*v-10251*x^2*y*u*v+F(94)/67*x*y^2*u*v-3180*y^3*u*v+F(50)/79*x^2*z*u*v-12525*x*y*z*u*v+637*y^2*z*u*v-F(83)/111*x*z^2*u*v-F(1)/66*y*z^2*u*v-F(69)/79*z^3*u*v+F(4)/81*x^2*u^2*v+F(7)/55*x*y*u^2*v-F(61)/90*y^2*u^2*v-F(75)/118*x*z*u^2*v+F(58)/77*y*z*u^2*v-F(81)/71*z^2*u^2*v-12323*x*u^3*v+15322*y*u^3*v+F(110)/23*z*u^3*v+14284*u^4*v+F(125)/34*x^3*v^2+8432*x^2*y*v^2-F(79)/58*x*y^2*v^2-F(115)/83*y^3*v^2-1136*x^2*z*v^2-4021*x*y*z*v^2+7394*y^2*z*v^2+366*x*z^2*v^2+2726*y*z^2*v^2-F(16)/7*z^3*v^2+F(81)/19*x^2*u*v^2+10778*x*y*u*v^2+F(62)/121*y^2*u*v^2-F(73)/41*x*z*u*v^2+F(37)/22*y*z*u*v^2+F(89)/12*z^2*u*v^2+7019*x*u^2*v^2-F(100)/59*y*u^2*v^2+F(17)/86*z*u^2*v^2+F(59)/91*u^3*v^2+12430*x^2*v^3-F(100)/53*x*y*v^3-F(65)/56*y^2*v^3-7029*x*z*v^3+5935*y*z*v^3+F(75)/2*z^2*v^3-F(31)/9*x*u*v^3+F(36)/37*y*u*v^3+F(125)/46*z*u*v^3+F(83)/7*u^2*v^3+12589*x*v^4+3277*y*v^4-7318*z*v^4-9005*u*v^4+F(50)/79*v^5,F(49)/121*x^3*y*u+7483*x^2*y^2*u+F(88)/15*x*y^3*u+14830*y^4*u+F(3)/95*x^3*z*u-F(17)/33*x^2*y*z*u-F(10)/113*x*y^2*z*u-15910*y^3*z*u-F(125)/47*x^2*z^2*u-F(123)/86*x*y*z^2*u+F(53)/14*y^2*z^2*u-2215*x*z^3*u+F(123)/43*y*z^3*u-6772*z^4*u-F(38)/125*x^3*u^2-F(9)/16*x^2*y*u^2+4769*x*y^2*u^2-F(49)/17*y^3*u^2-7370*x^2*z*u^2+F(109)/122*x*y*z*u^2+F(16)/11*y^2*z*u^2-F(99)/76*x*z^2*u^2+824*y*z^2*u^2-12212*z^3*u^2+F(79)/112*x^2*u^3-F(23)/78*x*y*u^3-7896*y^2*u^3-F(93)/101*x*z*u^3+11017*y*z*u^3+8370*z^2*u^3-11163*x*u^4+F(107)/70*y*u^4-F(97)/7*z*u^4+F(69)/70*u^5+3038*x^3*y*v-F(88)/47*x^2*y^2*v+F(54)/53*x*y^3*v-F(104)/119*y^4*v+F(64)/117*x^3*z*v-F(13)/124*x^2*y*z*v+F(95)/68*x*y^2*z*v+F(45)/17*y^3*z*v-F(85)/21*x^2*z^2*v-F(40)/99*x*y*z^2*v-F(79)/60*y^2*z^2*v+F(47)/16*x*z^3*v+8058*y*z^3*v-F(89)/3*z^4*v-F(5)/3*x^3*u*v-F(46)/15*x^2*y*u*v-F(31)/89*x*y^2*u*v+F(37)/54*y^3*u*v+F(35)/92*x^2*z*u*v-F(83)/35*x*y*z*u*v-F(82)/79*y^2*z*u*v-F(53)/60*x*z^2*u*v+10029*y*z^2*u*v+41*z^3*u*v+F(111)/97*x^2*u^2*v-F(87)/55*x*y*u^2*v-F(11)/56*y^2*u^2*v+12679*x*z*u^2*v+F(125)/2*y*z*u^2*v-1564*z^2*u^2*v-44*x*u^3*v+8107*y*u^3*v-7327*z*u^3*v+F(2)/75*u^4*v-F(55)/56*x^3*v^2-F(40)/33*x^2*y*v^2+F(3)/95*x*y^2*v^2-F(58)/125*y^3*v^2+F(73)/121*x^2*z*v^2-F(109)/86*x*y*z*v^2-F(1)/15*y^2*z*v^2+3736*x*z^2*v^2+F(105)/37*y*z^2*v^2-6905*z^3*v^2+10799*x^2*u*v^2-F(4)/25*x*y*u*v^2-F(89)/95*y^2*u*v^2+F(107)/37*x*z*u*v^2+F(15)/4*y*z*u*v^2-9506*z^2*u*v^2+F(83)/39*x*u^2*v^2+15619*y*u^2*v^2+F(68)/83*z*u^2*v^2-2271*u^3*v^2-F(53)/24*x^2*v^3+F(79)/102*x*y*v^3-10590*y^2*v^3-7254*x*z*v^3+F(43)/2*y*z*v^3+13986*z^2*v^3+F(123)/124*x*u*v^3+F(16)/87*y*u*v^3-F(39)/38*z*u*v^3-10082*u^2*v^3-F(46)/15*x*v^4+F(108)/89*y*v^4-4939*z*v^4+12667*u*v^4-F(57)/4*v^5,3038*x^4*y-F(88)/47*x^3*y^2+F(54)/53*x^2*y^3-F(104)/119*x*y^4+F(64)/117*x^4*z-F(13)/124*x^3*y*z+F(95)/68*x^2*y^2*z+F(45)/17*x*y^3*z-F(85)/21*x^3*z^2-F(40)/99*x^2*y*z^2-F(79)/60*x*y^2*z^2+F(47)/16*x^2*z^3+8058*x*y*z^3-F(89)/3*x*z^4+228*x^4*u-F(72)/73*x^3*y*u+F(67)/15*x^2*y^2*u-F(58)/95*x*y^3*u+976*y^4*u+9876*x^3*z*u-6103*x^2*y*z*u+383*x*y^2*z*u+10720*y^3*z*u+F(19)/12*x^2*z^2*u+14830*x*y*z^2*u-606*y^2*z^2*u+F(2)/19*x*z^3*u-F(125)/56*y*z^3*u+F(114)/113*z^4*u-F(56)/81*x^3*u^2+F(79)/12*x^2*y*u^2-F(107)/43*x*y^2*u^2+F(66)/47*y^3*u^2+F(83)/107*x^2*z*u^2+F(87)/86*x*y*z*u^2-13560*y^2*z*u^2+473*x*z^2*u^2+F(109)/39*y*z^2*u^2+9484*z^3*u^2+F(93)/52*x^2*u^3+F(71)/81*x*y*u^3-F(28)/87*y^2*u^3+F(103)/124*x*z*u^3+F(54)/59*y*z*u^3+F(11)/87*z^2*u^3+2686*x*u^4+5836*y*u^4-F(1)/44*z*u^4-948*u^5-F(55)/56*x^4*v-F(40)/33*x^3*y*v+F(3)/95*x^2*y^2*v-F(58)/125*x*y^3*v+F(73)/121*x^3*z*v-F(109)/86*x^2*y*z*v-F(1)/15*x*y^2*z*v+3736*x^2*z^2*v+F(105)/37*x*y*z^2*v-6905*x*z^3*v-3471*x^3*u*v+F(111)/41*x^2*y*u*v+F(26)/3*x*y^2*u*v-12260*y^3*u*v-F(119)/38*x^2*z*u*v-F(7)/17*x*y*z*u*v+F(122)/71*y^2*z*u*v+F(119)/64*x*z^2*u*v-F(76)/67*y*z^2*u*v+F(23)/121*z^3*u*v+F(22)/35*x^2*u^2*v+F(93)/38*x*y*u^2*v+1213*y^2*u^2*v-12130*x*z*u^2*v-4432*y*z*u^2*v-1946*z^2*u^2*v-F(14)/3*x*u^3*v-F(41)/116*y*u^3*v-11039*z*u^3*v-1166*u^4*v-F(53)/24*x^3*v^2+F(79)/102*x^2*y*v^2-10590*x*y^2*v^2-7254*x^2*z*v^2+F(43)/2*x*y*z*v^2+13986*x*z^2*v^2-11251*x^2*u*v^2+F(19)/89*x*y*u*v^2-663*y^2*u*v^2+2925*x*z*u*v^2+F(40)/93*y*z*u*v^2-F(72)/121*z^2*u*v^2+6599*x*u^2*v^2+F(85)/61*y*u^2*v^2+3144*z*u^2*v^2+F(7)/85*u^3*v^2-F(46)/15*x^2*v^3+F(108)/89*x*y*v^3-4939*x*z*v^3+F(77)/17*x*u*v^3-F(57)/91*y*u*v^3-2337*z*u*v^3-12769*u^2*v^3-F(57)/4*x*v^4+F(95)/109*u*v^4,-3038*x^3*y^2+F(88)/47*x^2*y^3-F(54)/53*x*y^4+F(104)/119*y^5-F(64)/117*x^3*y*z+F(13)/124*x^2*y^2*z-F(95)/68*x*y^3*z-F(45)/17*y^4*z+F(85)/21*x^2*y*z^2+F(40)/99*x*y^2*z^2+F(79)/60*y^3*z^2-F(47)/16*x*y*z^3-8058*y^2*z^3+F(89)/3*y*z^4-229*x^3*y*u-F(6)/47*x^2*y^2*u+F(115)/98*x*y^3*u-F(51)/55*y^4*u-10739*x^3*z*u+F(31)/86*x^2*y*z*u-F(97)/18*x*y^2*z*u+F(93)/104*y^3*z*u-F(52)/35*x^2*z^2*u-F(113)/44*x*y*z^2*u-F(92)/117*y^2*z^2*u-13887*x*z^3*u+2512*y*z^3*u+4021*z^4*u-F(111)/20*x^3*u^2+12129*x^2*y*u^2-F(75)/118*x*y^2*u^2-F(74)/5*y^3*u^2+2008*x^2*z*u^2-F(102)/11*x*y*z*u^2-F(75)/38*y^2*z*u^2+F(18)/121*x*z^2*u^2-F(26)/57*y*z^2*u^2+F(97)/100*z^3*u^2-3471*x^2*u^3+F(22)/3*x*y*u^3-F(28)/71*y^2*u^3-F(7)/59*x*z*u^3+F(119)/76*y*z*u^3-F(103)/33*z^2*u^3-12707*x*u^4-F(71)/78*y*u^4+F(117)/79*z*u^4+F(79)/80*u^5+F(55)/56*x^3*y*v+F(40)/33*x^2*y^2*v-F(3)/95*x*y^3*v+F(58)/125*y^4*v-F(73)/121*x^2*y*z*v+F(109)/86*x*y^2*z*v+F(1)/15*y^3*z*v-3736*x*y*z^2*v-F(105)/37*y^2*z^2*v+6905*y*z^3*v-7029*x^3*u*v+F(110)/3*x^2*y*u*v+13551*x*y^2*u*v-7393*y^3*u*v-F(105)/37*x^2*z*u*v-5227*x*y*z*u*v+F(88)/65*y^2*z*u*v-14752*x*z^2*u*v+2727*y*z^2*u*v-F(77)/115*z^3*u*v-14017*x^2*u^2*v-1366*x*y*u^2*v+15677*y^2*u^2*v-9764*x*z*u^2*v+F(116)/115*y*z*u^2*v-F(120)/23*z^2*u^2*v+2197*x*u^3*v-F(79)/22*y*u^3*v+F(43)/27*z*u^3*v-F(19)/104*u^4*v+F(53)/24*x^2*y*v^2-F(79)/102*x*y^2*v^2+10590*y^3*v^2+7254*x*y*z*v^2-F(43)/2*y^2*z*v^2-13986*y*z^2*v^2+F(99)/125*x^2*u*v^2-F(75)/74*x*y*u*v^2-12754*y^2*u*v^2+4027*x*z*u*v^2+5386*y*z*u*v^2+3532*z^2*u*v^2-F(90)/37*x*u^2*v^2-6355*y*u^2*v^2-F(26)/37*z*u^2*v^2+F(10)/103*u^3*v^2+F(46)/15*x*y*v^3-F(108)/89*y^2*v^3+4939*y*z*v^3-F(74)/119*x*u*v^3+F(17)/50*y*u*v^3-357*z*u*v^3-3182*u^2*v^3+F(57)/4*y*v^4+F(41)/111*u*v^4,-3038*x^3*y*z+F(88)/47*x^2*y^2*z-F(54)/53*x*y^3*z+F(104)/119*y^4*z-F(64)/117*x^3*z^2+F(13)/124*x^2*y*z^2-F(95)/68*x*y^2*z^2-F(45)/17*y^3*z^2+F(85)/21*x^2*z^3+F(40)/99*x*y*z^3+F(79)/60*y^2*z^3-F(47)/16*x*z^4-8058*y*z^4+F(89)/3*z^5+F(93)/40*x^3*y*u+F(53)/85*x^2*y^2*u-F(13)/118*x*y^3*u-9265*y^4*u-8492*x^3*z*u+F(100)/109*x^2*y*z*u+11175*x*y^2*z*u-12146*y^3*z*u-F(83)/3*x^2*z^2*u+F(106)/105*x*y*z^2*u+F(7)/68*y^2*z^2*u+15180*x*z^3*u-F(91)/99*y*z^3*u-11378*z^4*u+F(105)/31*x^3*u^2-10251*x^2*y*u^2+F(94)/67*x*y^2*u^2-3180*y^3*u^2-F(37)/105*x^2*z*u^2+15396*x*y*z*u^2-F(87)/68*y^2*z*u^2+9224*x*z^2*u^2-F(59)/125*y*z^2*u^2-13420*z^3*u^2+F(4)/81*x^2*u^3+F(7)/55*x*y*u^3-F(61)/90*y^2*u^3+F(96)/121*x*z*u^3-F(29)/96*y*z*u^3+F(106)/55*z^2*u^3-12323*x*u^4+15322*y*u^4+F(11)/92*z*u^4+14284*u^5+F(55)/56*x^3*z*v+F(40)/33*x^2*y*z*v-F(3)/95*x*y^2*z*v+F(58)/125*y^3*z*v-F(73)/121*x^2*z^2*v+F(109)/86*x*y*z^2*v+F(1)/15*y^2*z^2*v-3736*x*z^3*v-F(105)/37*y*z^3*v+6905*z^4*v+F(125)/34*x^3*u*v+8432*x^2*y*u*v-F(79)/58*x*y^2*u*v-F(115)/83*y^3*u*v-11935*x^2*z*u*v+F(31)/35*x*y*z*u*v+F(91)/93*y^2*z*u*v+F(83)/105*x*z^2*u*v+10720*y*z^2*u*v-13347*z^3*u*v+F(81)/19*x^2*u^2*v+10778*x*y*u^2*v+F(62)/121*y^2*u^2*v-6086*x*z*u^2*v-F(16)/123*y*z*u^2*v-154*z^2*u^2*v+7019*x*u^3*v-F(100)/59*y*u^3*v+F(27)/67*z*u^3*v+F(59)/91*u^4*v+F(53)/24*x^2*z*v^2-F(79)/102*x*y*z*v^2+10590*y^2*z*v^2+7254*x*z^2*v^2-F(43)/2*y*z^2*v^2-13986*z^3*v^2+12430*x^2*u*v^2-F(100)/53*x*y*u*v^2-F(65)/56*y^2*u*v^2-6772*x*z*u*v^2+F(37)/108*y*z*u*v^2+3406*z^2*u*v^2-F(31)/9*x*u^2*v^2+F(36)/37*y*u^2*v^2+F(55)/92*z*u^2*v^2+F(83)/7*u^3*v^2+F(46)/15*x*z*v^3-F(108)/89*y*z*v^3+4939*z^2*v^3+12589*x*u*v^3+3277*y*u*v^3+F(75)/8*z*u*v^3-9005*u^2*v^3+F(57)/4*z*v^4+F(50)/79*u*v^4,F(93)/40*x^4*y+F(53)/85*x^3*y^2-F(13)/118*x^2*y^3-9265*x*y^4+F(41)/120*x^4*z+7864*x^3*y*z-F(110)/37*x^2*y^2*z-F(37)/101*x*y^3*z+976*y^4*z-11479*x^3*z^2+4257*x^2*y*z^2-F(25)/108*x*y^2*z^2+10720*y^3*z^2+1852*x^2*z^3+934*x*y*z^3-606*y^2*z^3-F(93)/107*x*z^4-F(125)/56*y*z^4+F(114)/113*z^5+F(105)/31*x^4*u-10251*x^3*y*u+F(94)/67*x^2*y^2*u-3180*x*y^3*u+F(9)/13*x^3*z*u-F(53)/108*x^2*y*z*u-13275*x*y^2*z*u+F(66)/47*y^3*z*u+5637*x^2*z^2*u-F(87)/109*x*y*z^2*u-13560*y^2*z^2*u+F(73)/42*x*z^3*u+F(109)/39*y*z^3*u+9484*z^4*u+F(4)/81*x^3*u^2+F(7)/55*x^2*y*u^2-F(61)/90*x*y^2*u^2+F(97)/10*x^2*z*u^2-2653*x*y*z*u^2-F(28)/87*y^2*z*u^2-F(28)/97*x*z^2*u^2+F(54)/59*y*z^2*u^2+F(11)/87*z^3*u^2-12323*x^2*u^3+15322*x*y*u^3-7398*x*z*u^3+5836*y*z*u^3-F(1)/44*z^2*u^3+14284*x*u^4-948*z*u^4+F(125)/34*x^4*v+8432*x^3*y*v-F(79)/58*x^2*y^2*v-F(115)/83*x*y^3*v-F(79)/27*x^3*z*v-F(106)/107*x^2*y*z*v+F(29)/110*x*y^2*z*v-12260*y^3*z*v-4805*x^2*z^2*v-4335*x*y*z^2*v+F(122)/71*y^2*z^2*v+F(124)/67*x*z^3*v-F(76)/67*y*z^3*v+F(23)/121*z^4*v+F(81)/19*x^3*u*v+10778*x^2*y*u*v+F(62)/121*x*y^2*u*v+5797*x^2*z*u*v+F(43)/54*x*y*z*u*v+1213*y^2*z*u*v+F(68)/125*x*z^2*u*v-4432*y*z^2*u*v-1946*z^3*u*v+7019*x^2*u^2*v-F(100)/59*x*y*u^2*v+F(17)/108*x*z*u^2*v-F(41)/116*y*z*u^2*v-11039*z^2*u^2*v+F(59)/91*x*u^3*v-1166*z*u^3*v+12430*x^3*v^2-F(100)/53*x^2*y*v^2-F(65)/56*x*y^2*v^2+F(7)/71*x^2*z*v^2+5369*x*y*z*v^2-663*y^2*z*v^2-F(53)/96*x*z^2*v^2+F(40)/93*y*z^2*v^2-F(72)/121*z^3*v^2-F(31)/9*x^2*u*v^2+F(36)/37*x*y*u*v^2-11830*x*z*u*v^2+F(85)/61*y*z*u*v^2+3144*z^2*u*v^2+F(83)/7*x*u^2*v^2+F(7)/85*z*u^2*v^2+12589*x^2*v^3+3277*x*y*v^3-14335*x*z*v^3-F(57)/91*y*z*v^3-2337*z^2*v^3-9005*x*u*v^3-12769*z*u*v^3+F(50)/79*x*v^4+F(95)/109*z*v^4,-F(93)/40*x^3*y^2-F(53)/85*x^2*y^3+F(13)/118*x*y^4+9265*y^5+8263*x^3*y*z-6970*x^2*y^2*z-7583*x*y^3*z+15635*y^4*z-10739*x^3*z^2-F(101)/107*x^2*y*z^2-1479*x*y^2*z^2+F(46)/81*y^3*z^2-F(52)/35*x^2*z^3+1540*x*y*z^3-2237*y^2*z^3-13887*x*z^4-F(63)/76*y*z^4+4021*z^5-F(105)/31*x^3*y*u+10251*x^2*y^2*u-F(94)/67*x*y^3*u+3180*y^4*u-F(111)/20*x^3*z*u+F(81)/25*x^2*y*z*u-F(25)/124*x*y^2*z*u+F(48)/73*y^3*z*u+2008*x^2*z^2*u-F(103)/86*x*y*z^2*u+6821*y^2*z^2*u+F(18)/121*x*z^3*u-F(49)/79*y*z^3*u+F(97)/100*z^4*u-F(4)/81*x^2*y*u^2-F(7)/55*x*y^2*u^2+F(61)/90*y^3*u^2-3471*x^2*z*u^2-F(52)/19*x*y*z*u^2+F(53)/36*y^2*z*u^2-F(7)/59*x*z^2*u^2+F(22)/45*y*z^2*u^2-F(103)/33*z^3*u^2+12323*x*y*u^3-15322*y^2*u^3-12707*x*z*u^3-F(10)/53*y*z*u^3+F(117)/79*z^2*u^3-14284*y*u^4+F(79)/80*z*u^4-F(125)/34*x^3*y*v-8432*x^2*y^2*v+F(79)/58*x*y^3*v+F(115)/83*y^4*v-7029*x^3*z*v+F(110)/49*x^2*y*z*v-F(12)/97*x*y^2*z*v+F(111)/62*y^3*z*v-F(105)/37*x^2*z^2*v-F(3)/101*x*y*z^2*v-14656*y^2*z^2*v-14752*x*z^3*v-15917*y*z^3*v-F(77)/115*z^4*v-F(81)/19*x^2*y*u*v-10778*x*y^2*u*v-F(62)/121*y^3*u*v-14017*x^2*z*u*v+F(1)/61*x*y*z*u*v+F(37)/73*y^2*z*u*v-9764*x*z^2*u*v+F(36)/11*y*z^2*u*v-F(120)/23*z^3*u*v-7019*x*y*u^2*v+F(100)/59*y^2*u^2*v+2197*x*z*u^2*v+F(101)/116*y*z*u^2*v+F(43)/27*z^2*u^2*v-F(59)/91*y*u^3*v-F(19)/104*z*u^3*v-12430*x^2*y*v^2+F(100)/53*x*y^2*v^2+F(65)/56*y^3*v^2+F(99)/125*x^2*z*v^2-12683*x*y*z*v^2+F(85)/64*y^2*z*v^2+4027*x*z^2*v^2+F(114)/97*y*z^2*v^2+3532*z^3*v^2+F(31)/9*x*y*u*v^2-F(36)/37*y^2*u*v^2-F(90)/37*x*z*u*v^2+F(102)/53*y*z*u*v^2-F(26)/37*z^2*u*v^2-F(83)/7*y*u^2*v^2+F(10)/103*z*u^2*v^2-12589*x*y*v^3-3277*y^2*v^3-F(74)/119*x*z*v^3+F(85)/26*y*z*v^3-357*z^2*v^3+9005*y*u*v^3-3182*z*u*v^3-F(50)/79*y*v^4+F(41)/111*z*v^4,x^4*y-894*x^3*y^2+F(121)/48*x^2*y^3-F(26)/3*x*y^4-976*y^5+10739*x^4*z+10583*x^3*y*z+F(88)/23*x^2*y^2*z+F(91)/23*x*y^3*z-10720*y^4*z+F(52)/35*x^3*z^2+F(65)/66*x^2*y*z^2+1303*x*y^2*z^2+606*y^3*z^2+13887*x^2*z^3+F(27)/2*x*y*z^3+F(125)/56*y^2*z^3-4021*x*z^4-F(114)/113*y*z^4+F(111)/20*x^4*u-6599*x^3*y*u-F(68)/95*x^2*y^2*u+F(34)/45*x*y^3*u-F(66)/47*y^4*u-2008*x^3*z*u+F(2)/93*x^2*y*z*u+12257*x*y^2*z*u+13560*y^3*z*u-F(18)/121*x^2*z^2*u-6085*x*y*z^2*u-F(109)/39*y^2*z^2*u-F(97)/100*x*z^3*u-9484*y*z^3*u+3471*x^3*u^2+F(50)/17*x^2*y*u^2+F(65)/119*x*y^2*u^2+F(28)/87*y^3*u^2+F(7)/59*x^2*z*u^2-11001*x*y*z*u^2-F(54)/59*y^2*z*u^2+F(103)/33*x*z^2*u^2-F(11)/87*y*z^2*u^2+12707*x^2*u^3-F(35)/66*x*y*u^3-5836*y^2*u^3-F(117)/79*x*z*u^3+F(1)/44*y*z*u^3-F(79)/80*x*u^4+948*y*u^4+7029*x^4*v+F(16)/59*x^3*y*v+F(51)/43*x^2*y^2*v+F(70)/39*x*y^3*v+12260*y^4*v+F(105)/37*x^3*z*v+6072*x^2*y*z*v-F(117)/32*x*y^2*z*v-F(122)/71*y^3*z*v+14752*x^2*z^2*v+F(126)/43*x*y*z^2*v+F(76)/67*y^2*z^2*v+F(77)/115*x*z^3*v-F(23)/121*y*z^3*v+14017*x^3*u*v+F(43)/73*x^2*y*u*v+F(126)/59*x*y^2*u*v-1213*y^3*u*v+9764*x^2*z*u*v+15189*x*y*z*u*v+4432*y^2*z*u*v+F(120)/23*x*z^2*u*v+1946*y*z^2*u*v-2197*x^2*u^2*v+F(19)/61*x*y*u^2*v+F(41)/116*y^2*u^2*v-F(43)/27*x*z*u^2*v+11039*y*z*u^2*v+F(19)/104*x*u^3*v+1166*y*u^3*v-F(99)/125*x^3*v^2-1285*x^2*y*v^2-F(27)/31*x*y^2*v^2+663*y^3*v^2-4027*x^2*z*v^2-8311*x*y*z*v^2-F(40)/93*y^2*z*v^2-3532*x*z^2*v^2+F(72)/121*y*z^2*v^2+F(90)/37*x^2*u*v^2-244*x*y*u*v^2-F(85)/61*y^2*u*v^2+F(26)/37*x*z*u*v^2-3144*y*z*u*v^2-F(10)/103*x*u^2*v^2-F(7)/85*y*u^2*v^2+F(74)/119*x^2*v^3-F(7)/71*x*y*v^3+F(57)/91*y^2*v^3+357*x*z*v^3+2337*y*z*v^3+3182*x*u*v^3+12769*y*u*v^3-F(41)/111*x*v^4-F(95)/109*y*v^4,F(93)/110*x^4-F(18)/109*x^3*y+F(46)/87*x^2*y^2+6445*x*y^3-F(124)/3*y^4-F(52)/55*x^3*z-F(87)/35*x^2*y*z-12843*x*y^2*z-F(49)/33*y^3*z-F(25)/79*x^2*z^2-F(71)/45*x*y*z^2-F(109)/85*y^2*z^2+6796*x*z^3+9580*y*z^3+F(19)/86*z^4-F(117)/85*x^3*u-9826*x^2*y*u-388*x*y^2*u+15458*y^3*u+9689*x^2*z*u-F(56)/85*x*y*z*u+9831*y^2*z*u-1383*x*z^2*u-F(23)/105*y*z^2*u+F(72)/115*z^3*u+F(58)/3*x^2*u^2+F(43)/11*x*y*u^2-F(54)/121*y^2*u^2+15814*x*z*u^2+5932*y*z*u^2-F(66)/49*z^2*u^2+F(65)/23*x*u^3-F(23)/103*y*u^3+7684*z*u^3-F(38)/115*u^4-F(97)/104*x^3*v+8705*x^2*y*v+7815*x*y^2*v+13142*y^3*v-6496*x^2*z*v+15166*x*y*z*v-F(67)/70*y^2*z*v-F(121)/114*x*z^2*v+F(55)/92*y*z^2*v+F(66)/91*z^3*v+9923*x^2*u*v+F(98)/61*x*y*u*v-11921*y^2*u*v+895*x*z*u*v+11151*y*z*u*v-424*z^2*u*v-F(37)/78*x*u^2*v-F(101)/26*y*u^2*v-F(89)/54*z*u^2*v-F(89)/47*u^3*v+9949*x^2*v^2-11279*x*y*v^2-8831*y^2*v^2-F(121)/68*x*z*v^2-F(67)/86*y*z*v^2-F(37)/26*z^2*v^2+8577*x*u*v^2+1381*y*u*v^2+1902*z*u*v^2-F(21)/29*u^2*v^2-F(101)/36*x*v^3-F(97)/8*y*v^3+12717*z*v^3-F(50)/79*u*v^3-F(93)/67*v^4]))
end


@doc raw"""
    rational_d10_pi9_quart_2()

Return a smooth rational surface in $\mathbb P^4$ with degree `10` and sectional genus `9` which is contained in a pencil of quartics.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d10_pi9_quart_2()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^3*y*u^2-F(48)/11*x^2*y^2*u^2-8356*x*y^3*u^2+F(35)/121*y^4*u^2+F(31)/66*x^3*z*u^2-F(54)/83*x^2*y*z*u^2-F(61)/18*x*y^2*z*u^2+11526*y^3*z*u^2+7372*x^2*z^2*u^2-F(91)/60*x*y*z^2*u^2-F(95)/97*y^2*z^2*u^2-F(45)/71*x*z^3*u^2+F(71)/115*y*z^3*u^2+F(25)/54*z^4*u^2-F(61)/102*x^3*u^3-12668*x^2*y*u^3+6653*x*y^2*u^3+F(41)/54*y^3*u^3+F(87)/50*x^2*z*u^3-5004*x*y*z*u^3+13924*y^2*z*u^3+2310*x*z^2*u^3-F(93)/14*y*z^2*u^3-F(2)/93*z^3*u^3-F(97)/125*x^2*u^4-F(58)/11*x*y*u^4+F(46)/73*y^2*u^4-4417*x*z*u^4+F(60)/101*y*z*u^4+F(56)/75*z^2*u^4-F(113)/118*x*u^5+F(115)/4*y*u^5-40*z*u^5-8554*u^6-F(54)/83*x^3*y*u*v-9770*x^2*y^2*u*v-590*x*y^3*u*v+F(15)/49*y^4*u*v+F(94)/69*x^3*z*u*v+F(121)/105*x^2*y*z*u*v+F(95)/88*x*y^2*z*u*v+3186*y^3*z*u*v+F(11)/6*x^2*z^2*u*v-F(44)/81*x*y*z^2*u*v+637*y^2*z^2*u*v+F(109)/121*x*z^3*u*v-33*y*z^3*u*v-F(94)/115*z^4*u*v-F(49)/95*x^3*u^2*v-F(11)/109*x^2*y*u^2*v+F(45)/113*x*y^2*u^2*v+F(97)/84*y^3*u^2*v+5257*x^2*z*u^2*v+F(99)/49*x*y*z*u^2*v+12584*y^2*z*u^2*v-4294*x*z^2*u^2*v+1137*y*z^2*u^2*v-F(58)/69*z^3*u^2*v-4749*x^2*u^3*v+F(120)/97*x*y*u^3*v-F(31)/103*y^2*u^3*v+F(62)/97*x*z*u^3*v-F(107)/74*y*z*u^3*v+F(53)/59*z^2*u^3*v+F(91)/33*x*u^4*v+1291*y*u^4*v+F(23)/34*z*u^4*v+F(58)/77*u^5*v+F(16)/17*x^3*y*v^2-750*x^2*y^2*v^2+F(86)/89*x*y^3*v^2+F(123)/46*y^4*v^2+F(53)/123*x^3*z*v^2-F(61)/99*x^2*y*z*v^2+12389*x*y^2*z*v^2+10419*y^3*z*v^2+F(43)/11*x^2*z^2*v^2-146*x*y*z^2*v^2-F(116)/51*y^2*z^2*v^2+F(13)/62*x*z^3*v^2-5524*y*z^3*v^2-F(111)/118*z^4*v^2-F(56)/55*x^3*u*v^2-3038*x^2*y*u*v^2+F(14)/27*x*y^2*u*v^2-F(43)/64*y^3*u*v^2+3385*x^2*z*u*v^2+F(25)/11*x*y*z*u*v^2+F(92)/41*y^2*z*u*v^2+F(28)/113*x*z^2*u*v^2-2049*y*z^2*u*v^2+F(89)/37*z^3*u*v^2-13094*x^2*u^2*v^2-2774*x*y*u^2*v^2+15474*y^2*u^2*v^2-15791*x*z*u^2*v^2-F(71)/116*y*z*u^2*v^2+F(77)/41*z^2*u^2*v^2-F(83)/68*x*u^3*v^2-F(33)/106*y*u^3*v^2+F(71)/37*z*u^3*v^2-F(41)/17*u^4*v^2+12052*x^3*v^3+1906*x^2*y*v^3+13825*x*y^2*v^3+F(80)/7*y^3*v^3-F(125)/96*x^2*z*v^3-9661*x*y*z*v^3+F(85)/116*y^2*z*v^3-F(72)/91*x*z^2*v^3+F(13)/112*y*z^2*v^3-F(126)/97*z^3*v^3-1637*x^2*u*v^3+F(34)/103*x*y*u*v^3+3844*y^2*u*v^3+F(77)/10*x*z*u*v^3+6359*y*z*u*v^3-11185*z^2*u*v^3-F(124)/121*x*u^2*v^3+F(66)/91*y*u^2*v^3-14636*z*u^2*v^3-1051*u^3*v^3+F(9)/64*x^2*v^4-12924*x*y*v^4-F(119)/41*y^2*v^4+F(74)/23*x*z*v^4+1622*y*z*v^4+F(73)/37*z^2*v^4-F(60)/101*x*u*v^4+F(111)/22*y*u*v^4-F(45)/124*z*u*v^4+F(59)/37*u^2*v^4-F(66)/37*x*v^5-F(71)/99*y*v^5+12409*z*v^5-F(113)/64*u*v^5-5267*v^6,-x^4*y-F(22)/79*x^3*y^2-F(125)/42*x^2*y^3-F(116)/7*x*y^4+F(98)/111*y^5-F(31)/66*x^4*z-F(118)/75*x^3*y*z+F(110)/93*x^2*y^2*z-F(43)/92*x*y^3*z-788*y^4*z-7372*x^3*z^2-2701*x^2*y*z^2-F(67)/124*x*y^2*z^2-F(117)/62*y^3*z^2+F(45)/71*x^2*z^3-8396*x*y*z^3-10343*y^2*z^3-F(25)/54*x*z^4+F(30)/59*y*z^4+F(61)/102*x^4*u+11736*x^3*y*u+12726*x^2*y^2*u+F(41)/118*x*y^3*u-15832*y^4*u-F(87)/50*x^3*z*u-130*x^2*y*z*u+F(41)/8*x*y^2*z*u-10300*y^3*z*u-2310*x^2*z^2*u-F(101)/5*x*y*z^2*u+6205*y^2*z^2*u+F(2)/93*x*z^3*u+8679*y*z^3*u+F(97)/125*x^3*u^2-F(43)/37*x^2*y*u^2-F(39)/80*x*y^2*u^2+12139*y^3*u^2+4417*x^2*z*u^2+4294*x*y*z*u^2+F(11)/58*y^2*z*u^2-F(56)/75*x*z^2*u^2+8338*y*z^2*u^2+F(113)/118*x^2*u^3-10190*x*y*u^3-F(37)/16*y^2*u^3+40*x*z*u^3+F(74)/23*y*z*u^3+8554*x*u^4+F(115)/22*y*u^4-F(39)/79*x^4*v+F(61)/72*x^3*y*v+8048*x^2*y^2*v-9201*x*y^3*v+F(16)/121*y^4*v+F(113)/93*x^3*z*v+F(109)/75*x^2*y*z*v+12700*x*y^2*z*v-10607*y^3*z*v+F(50)/11*x^2*z^2*v+1223*x*y*z^2*v-F(103)/79*y^2*z^2*v-F(123)/58*x*z^3*v+F(31)/26*y*z^3*v-F(15)/122*z^4*v+F(122)/25*x^3*u*v-F(99)/17*x^2*y*u*v+1723*x*y^2*u*v-F(38)/121*y^3*u*v+11016*x^2*z*u*v-F(25)/102*x*y*z*u*v-14970*y^2*z*u*v-F(61)/6*x*z^2*u*v-14981*y*z^2*u*v+15900*z^3*u*v+3268*x^2*u^2*v-F(75)/19*x*y*u^2*v-1436*y^2*u^2*v-1764*x*z*u^2*v-F(57)/41*y*z*u^2*v+12741*z^2*u^2*v-14615*x*u^3*v+F(119)/61*y*u^3*v-F(115)/119*z*u^3*v+10501*u^4*v-8502*x^3*v^2-F(51)/76*x^2*y*v^2-6281*x*y^2*v^2+F(17)/49*y^3*v^2-F(106)/7*x^2*z*v^2+F(63)/101*x*y*z*v^2-F(27)/95*y^2*z*v^2-1606*x*z^2*v^2+9245*y*z^2*v^2+1912*z^3*v^2+11155*x^2*u*v^2+223*x*y*u*v^2-F(13)/18*y^2*u*v^2+F(110)/43*x*z*u*v^2+F(76)/81*y*z*u*v^2-6291*z^2*u*v^2+1400*x*u^2*v^2-F(95)/23*y*u^2*v^2-9701*z*u^2*v^2+F(106)/105*u^3*v^2+F(72)/47*x^2*v^3-13118*x*y*v^3+14409*y^2*v^3+F(37)/86*x*z*v^3+F(44)/69*y*z*v^3-325*z^2*v^3+F(113)/71*x*u*v^3+F(16)/81*y*u*v^3+F(6)/19*z*u*v^3-F(119)/39*u^2*v^3-F(89)/9*x*v^4+F(72)/53*y*v^4+F(112)/55*z*v^4-8587*u*v^4-6604*v^5,-x^3*y^2+F(48)/11*x^2*y^3+8356*x*y^4-F(35)/121*y^5-12750*x^3*y*z+F(100)/111*x^2*y^2*z+F(45)/74*x*y^3*z+F(99)/74*y^4*z-F(6)/7*x^3*z^2-F(47)/67*x^2*y*z^2+11465*x*y^2*z^2-11865*y^3*z^2+7776*x^2*z^3+F(124)/45*x*y*z^3-F(98)/115*y^2*z^3+F(117)/85*x*z^4-F(59)/120*y*z^4-8748*z^5+F(61)/102*x^3*y*u+12668*x^2*y^2*u-6653*x*y^3*u-F(41)/54*y^4*u+13408*x^3*z*u-2185*x^2*y*z*u-1240*x*y^2*z*u+1161*y^3*z*u+F(44)/27*x^2*z^2*u-11164*x*y*z^2*u-13388*y^2*z^2*u-F(107)/13*x*z^3*u+F(90)/71*y*z^3*u+4204*z^4*u+F(97)/125*x^2*y*u^2+F(58)/11*x*y^2*u^2-F(46)/73*y^3*u^2+F(55)/48*x^2*z*u^2+F(121)/31*x*y*z*u^2+F(126)/61*y^2*z*u^2-F(55)/69*x*z^2*u^2+5988*y*z^2*u^2+3755*z^3*u^2+F(113)/118*x*y*u^3-F(115)/4*y^2*u^3+3390*x*z*u^3-5762*y*z*u^3+F(30)/61*z^2*u^3+8554*y*u^4-14317*z*u^4+F(99)/116*x^3*y*v-F(113)/119*x^2*y^2*v+F(50)/23*x*y^3*v-F(37)/79*y^4*v-8668*x^3*z*v+14049*x^2*y*z*v+F(111)/35*x*y^2*z*v+F(61)/28*y^3*z*v-10171*x^2*z^2*v+F(68)/21*x*y*z^2*v+2023*y^2*z^2*v-F(9)/109*x*z^3*v+8520*y*z^3*v-2683*z^4*v-13547*x^3*u*v+F(28)/65*x^2*y*u*v-5988*x*y^2*u*v+F(61)/111*y^3*u*v+12314*x^2*z*u*v+F(29)/44*x*y*z*u*v+6141*y^2*z*u*v+11280*x*z^2*u*v+F(79)/22*y*z^2*u*v-F(38)/111*z^3*u*v+F(19)/51*x^2*u^2*v+5093*x*y*u^2*v-10291*y^2*u^2*v-5009*x*z*u^2*v-F(111)/49*y*z*u^2*v+3813*z^2*u^2*v-F(61)/37*x*u^3*v+15914*y*u^3*v-3218*z*u^3*v-12915*u^4*v-F(118)/101*x^3*v^2-F(7)/57*x^2*y*v^2+13128*x*y^2*v^2+11606*y^3*v^2+F(42)/101*x^2*z*v^2-F(54)/17*x*y*z*v^2-F(43)/49*y^2*z*v^2-F(119)/110*x*z^2*v^2+9742*y*z^2*v^2-F(43)/4*z^3*v^2-F(55)/8*x^2*u*v^2-F(29)/88*x*y*u*v^2+12042*y^2*u*v^2+F(101)/37*x*z*u*v^2-F(57)/62*y*z*u*v^2+F(106)/97*z^2*u*v^2+F(38)/83*x*u^2*v^2+8152*y*u^2*v^2-5492*z*u^2*v^2-F(47)/79*u^3*v^2+15112*x^2*v^3+F(69)/44*x*y*v^3-F(6)/71*y^2*v^3+F(113)/54*x*z*v^3-13210*y*z*v^3-707*z^2*v^3-F(119)/8*x*u*v^3+3845*y*u*v^3-F(19)/20*z*u*v^3+4852*u^2*v^3+15761*x*v^4-12372*y*v^4+F(74)/69*z*v^4-2100*u*v^4-12833*v^5,-x^3*y*z+F(48)/11*x^2*y^2*z+8356*x*y^3*z-F(35)/121*y^4*z-F(31)/66*x^3*z^2+F(54)/83*x^2*y*z^2+F(61)/18*x*y^2*z^2-11526*y^3*z^2-7372*x^2*z^3+F(91)/60*x*y*z^3+F(95)/97*y^2*z^3+F(45)/71*x*z^4-F(71)/115*y*z^4-F(25)/54*z^5+F(15)/52*x^3*y*u+6039*x^2*y^2*u+F(74)/99*x*y^3*u-F(17)/40*y^4*u+F(29)/50*x^3*z*u-7775*x^2*y*z*u+6368*x*y^2*z*u+14170*y^3*z*u+F(52)/41*x^2*z^2*u+7003*x*y*z^2*u-5787*y^2*z^2*u-F(101)/37*x*z^3*u-F(23)/28*y*z^3*u-F(20)/63*z^4*u+F(41)/77*x^3*u^2+8650*x^2*y*u^2-15922*x*y^2*u^2-F(16)/83*y^3*u^2+7278*x^2*z*u^2+F(31)/30*x*y*z*u^2-F(2)/107*y^2*z*u^2+F(35)/122*x*z^2*u^2+F(85)/58*y*z^2*u^2-757*z^3*u^2+F(2)/101*x^2*u^3+F(86)/17*x*y*u^3+F(95)/59*y^2*u^3+F(123)/22*x*z*u^3-6869*y*z*u^3-9311*z^2*u^3-F(105)/97*x*u^4+5699*y*u^4+15925*z*u^4+13528*u^5-154*x^3*y*v+4187*x^2*y^2*v+F(56)/107*x*y^3*v-15932*y^4*v-5137*x^3*z*v-F(37)/56*x^2*y*z*v+9401*x*y^2*z*v+F(92)/123*y^3*z*v-F(79)/97*x^2*z^2*v+9201*x*y*z^2*v+F(19)/53*y^2*z^2*v+F(107)/20*x*z^3*v+F(17)/77*y*z^3*v-15306*z^4*v+3215*x^3*u*v-F(79)/117*x^2*y*u*v-F(9)/76*x*y^2*u*v-6352*y^3*u*v+F(93)/13*x^2*z*u*v-F(65)/89*x*y*z*u*v-F(115)/4*y^2*z*u*v-F(34)/57*x*z^2*u*v+F(39)/107*y*z^2*u*v+F(31)/9*z^3*u*v+F(107)/48*x^2*u^2*v+2632*x*y*u^2*v+F(29)/96*y^2*u^2*v-F(125)/89*x*z*u^2*v+F(29)/113*y*z*u^2*v+3940*z^2*u^2*v-F(116)/111*x*u^3*v+6145*y*u^3*v-F(105)/62*z*u^3*v+F(101)/17*u^4*v-9281*x^3*v^2-F(49)/107*x^2*y*v^2-12154*x*y^2*v^2+F(4)/19*y^3*v^2-F(114)/71*x^2*z*v^2-F(15)/118*x*y*z*v^2+4372*y^2*z*v^2+F(45)/121*x*z^2*v^2+F(46)/111*y*z^2*v^2+6614*z^3*v^2+17*x^2*u*v^2+10806*x*y*u*v^2-10617*y^2*u*v^2-F(25)/111*x*z*u*v^2-F(116)/27*y*z*u*v^2-F(7)/58*z^2*u*v^2-686*x*u^2*v^2+F(3)/13*y*u^2*v^2-F(17)/49*z*u^2*v^2-F(40)/107*u^3*v^2+F(47)/90*x^2*v^3-F(83)/43*x*y*v^3-6326*y^2*v^3+F(49)/64*x*z*v^3+F(113)/76*y*z*v^3-F(122)/73*z^2*v^3+10232*x*u*v^3-F(116)/109*y*u*v^3-1990*z*u*v^3+F(70)/51*u^2*v^3-F(118)/19*x*v^4-F(27)/55*y*v^4+F(21)/19*z*v^4-F(23)/57*u*v^4-11721*v^5,-3399*x^4*y+1849*x^3*y^2-F(3)/29*x^2*y^3+F(28)/87*x*y^4+F(10)/29*y^5-9788*x^4*z-F(49)/73*x^3*y*z+13829*x^2*y^2*z+F(118)/73*x*y^3*z+13129*y^4*z-618*x^3*z^2+F(92)/13*x^2*y*z^2+F(101)/117*x*y^2*z^2-162*y^3*z^2+F(24)/5*x^2*z^3-F(29)/74*x*y*z^3+2687*y^2*z^3-F(74)/39*x*z^4+F(2)/57*y*z^4+F(68)/73*x^4*u-13787*x^3*y*u-11659*x^2*y^2*u+14729*x*y^3*u+F(92)/53*y^4*u+F(15)/71*x^3*z*u-F(62)/15*x^2*y*z*u+F(21)/85*x*y^2*z*u+4938*y^3*z*u-F(120)/37*x^2*z^2*u-F(77)/102*x*y*z^2*u-4785*y^2*z^2*u-F(83)/70*x*z^3*u-12128*y*z^3*u-13592*z^4*u-F(123)/20*x^3*u^2+2607*x^2*y*u^2+F(40)/19*x*y^2*u^2+6361*y^3*u^2-3091*x^2*z*u^2+F(89)/113*x*y*z*u^2+149*y^2*z*u^2-2890*x*z^2*u^2-8374*y*z^2*u^2+11886*z^3*u^2-F(49)/43*x^2*u^3-9854*x*y*u^3-6943*y^2*u^3+10743*x*z*u^3-F(122)/45*y*z*u^3-13902*z^2*u^3-F(103)/19*x*u^4-F(48)/59*y*u^4+F(27)/86*z*u^4+F(46)/35*u^5-F(117)/17*x^4*v-F(15)/7*x^3*y*v+8409*x^2*y^2*v-F(83)/28*x*y^3*v+F(86)/35*y^4*v+F(37)/45*x^3*z*v+F(4)/3*x^2*y*z*v+F(35)/38*x*y^2*z*v+4015*y^3*z*v-F(49)/111*x^2*z^2*v-1260*x*y*z^2*v-F(25)/33*y^2*z^2*v+F(116)/19*x*z^3*v+F(93)/8*y*z^3*v+5755*z^4*v-F(25)/89*x^3*u*v-11669*x^2*y*u*v-F(64)/107*x*y^2*u*v+2993*y^3*u*v+7767*x^2*z*u*v-F(17)/95*x*y*z*u*v-F(103)/80*y^2*z*u*v-14576*x*z^2*u*v+F(80)/47*y*z^2*u*v+F(25)/107*z^3*u*v+F(103)/2*x^2*u^2*v+F(125)/117*x*y*u^2*v-F(2)/89*y^2*u^2*v-5298*x*z*u^2*v-F(50)/27*y*z*u^2*v-F(71)/53*z^2*u^2*v+2652*x*u^3*v+15761*y*u^3*v+2124*z*u^3*v+F(11)/82*u^4*v+F(100)/63*x^3*v^2+4180*x^2*y*v^2+F(11)/39*x*y^2*v^2-1221*y^3*v^2+F(108)/125*x^2*z*v^2+F(97)/126*x*y*z*v^2-7698*y^2*z*v^2+13984*x*z^2*v^2+1342*y*z^2*v^2-F(84)/121*z^3*v^2-F(26)/73*x^2*u*v^2-F(14)/15*x*y*u*v^2-F(22)/37*y^2*u*v^2-F(71)/82*x*z*u*v^2+12430*y*z*u*v^2+F(103)/52*z^2*u*v^2-13095*x*u^2*v^2+10114*y*u^2*v^2-F(8)/73*z*u^2*v^2-F(33)/97*u^3*v^2+F(83)/105*x^2*v^3+F(22)/45*x*y*v^3-7961*y^2*v^3-9654*x*z*v^3-F(54)/55*y*z*v^3-F(3)/71*z^2*v^3-10148*x*u*v^3-F(117)/98*y*u*v^3+F(101)/102*z*u*v^3-606*u^2*v^3+F(97)/43*x*v^4-F(68)/21*y*v^4+F(63)/16*z*v^4+F(42)/17*u*v^4+5834*v^5,-3399*x^3*y^2-F(32)/113*x^2*y^3+F(14)/99*x*y^4+15001*y^5-F(121)/115*x^3*y*z+4604*x^2*y^2*z+F(7)/2*x*y^3*z+9532*y^4*z-3267*x^3*z^2+F(97)/118*x^2*y*z^2-14238*x*y^2*z^2-F(80)/21*y^3*z^2-12332*x^2*z^3-F(19)/69*x*y*z^3+F(116)/15*y^2*z^3-F(103)/32*x*z^4+15340*y*z^4+10509*z^5+F(112)/109*x^3*y*u-97*x^2*y^2*u-F(40)/11*x*y^3*u+F(90)/29*y^4*u-F(95)/106*x^3*z*u-F(114)/67*x^2*y*z*u+F(113)/48*x*y^2*z*u+12080*y^3*z*u-44*x^2*z^2*u+F(18)/17*x*y*z^2*u-4814*y^2*z^2*u-F(103)/100*x*z^3*u-F(96)/61*y*z^3*u-205*z^4*u-F(87)/82*x^3*u^2-F(97)/108*x^2*y*u^2+3230*x*y^2*u^2+F(104)/83*y^3*u^2+F(41)/86*x^2*z*u^2+F(116)/49*x*y*z*u^2-F(59)/110*y^2*z*u^2+F(14)/59*x*z^2*u^2-6962*y*z^2*u^2-2185*z^3*u^2+F(59)/91*x^2*u^3+2497*x*y*u^3+F(3)/37*y^2*u^3-13010*x*z*u^3+F(6)/83*y*z*u^3-11448*z^2*u^3+F(13)/72*x*u^4-F(69)/62*y*u^4-2869*z*u^4+F(23)/73*u^5-F(20)/43*x^3*y*v+5074*x^2*y^2*v+F(28)/125*x*y^3*v-2706*y^4*v+13010*x^3*z*v-F(17)/109*x^2*y*z*v+F(21)/4*x*y^2*z*v+F(59)/93*y^3*z*v-2406*x^2*z^2*v+F(117)/11*x*y*z^2*v-14978*y^2*z^2*v+F(70)/89*x*z^3*v-F(33)/7*y*z^3*v-13676*z^4*v-13690*x^3*u*v+9825*x^2*y*u*v-F(117)/107*x*y^2*u*v+12760*y^3*u*v-F(93)/98*x^2*z*u*v-F(113)/64*x*y*z*u*v+F(113)/103*y^2*z*u*v-9748*x*z^2*u*v+11016*y*z^2*u*v-10729*z^3*u*v+F(90)/13*x^2*u^2*v-F(13)/47*x*y*u^2*v-F(11)/39*y^2*u^2*v+F(20)/69*x*z*u^2*v+5531*y*z*u^2*v+F(125)/49*z^2*u^2*v-11025*x*u^3*v-9621*y*u^3*v+F(113)/109*z*u^3*v+4710*u^4*v-F(107)/7*x^3*v^2+F(110)/119*x^2*y*v^2-10025*x*y^2*v^2-6644*y^3*v^2-5041*x^2*z*v^2+F(5)/96*x*y*z*v^2+11472*y^2*z*v^2-5128*x*z^2*v^2+2927*y*z^2*v^2+F(121)/18*z^3*v^2-F(125)/89*x^2*u*v^2+12936*x*y*u*v^2-F(71)/47*y^2*u*v^2+F(34)/47*x*z*u*v^2-F(75)/103*y*z*u*v^2-2654*z^2*u*v^2-2350*x*u^2*v^2-7707*y*u^2*v^2+F(47)/72*z*u^2*v^2-952*u^3*v^2-F(21)/67*x^2*v^3+F(58)/37*x*y*v^3-8757*y^2*v^3+3615*x*z*v^3+F(44)/123*y*z*v^3-13027*z^2*v^3-F(9)/10*x*u*v^3+F(75)/43*y*u*v^3+F(115)/18*z*u*v^3+8071*u^2*v^3-F(26)/3*x*v^4-F(67)/65*y*v^4+14186*z*v^4-F(41)/122*u*v^4+F(33)/28*v^5,-3399*x^3*y*z-F(32)/113*x^2*y^2*z+F(14)/99*x*y^3*z+15001*y^4*z-9788*x^3*z^2+F(37)/96*x^2*y*z^2+7743*x*y^2*z^2+F(31)/55*y^3*z^2-618*x^2*z^3-8171*x*y*z^3+F(82)/109*y^2*z^3+F(24)/5*x*z^4+F(88)/85*y*z^4-F(74)/39*z^5-13165*x^3*y*u+3407*x^2*y^2*u-12509*x*y^3*u-F(23)/45*y^4*u-11774*x^3*z*u-F(10)/67*x^2*y*z*u+F(69)/79*x*y^2*z*u-F(10)/123*y^3*z*u-7636*x^2*z^2*u+F(83)/32*x*y*z^2*u+F(51)/112*y^2*z^2*u+F(19)/8*x*z^3*u+9309*y*z^3*u-F(44)/49*z^4*u+4089*x^3*u^2-374*x^2*y*u^2-919*x*y^2*u^2+F(98)/107*y^3*u^2+2776*x^2*z*u^2+F(85)/26*x*y*z*u^2+F(31)/13*y^2*z*u^2-F(103)/82*x*z^2*u^2+F(35)/76*y*z^2*u^2+F(59)/45*z^3*u^2+2950*x^2*u^3+F(27)/44*x*y*u^3+F(88)/71*y^2*u^3+F(7)/114*x*z*u^3-F(72)/77*y*z*u^3+12917*z^2*u^3-F(34)/67*x*u^4-F(85)/82*y*u^4-F(55)/84*z*u^4+4690*u^5+F(11)/42*x^3*y*v-F(19)/125*x^2*y^2*v-8288*x*y^3*v+9199*y^4*v-12929*x^3*z*v+13357*x^2*y*z*v-4903*x*y^2*z*v-584*y^3*z*v-F(10)/33*x^2*z^2*v+F(59)/113*x*y*z^2*v+F(103)/92*y^2*z^2*v+F(101)/69*x*z^3*v+8708*y*z^3*v-F(8)/7*z^4*v+13560*x^3*u*v-F(43)/49*x^2*y*u*v-F(121)/98*x*y^2*u*v+F(75)/79*y^3*u*v-39*x^2*z*u*v-F(88)/69*x*y*z*u*v-F(89)/78*y^2*z*u*v+F(110)/67*x*z^2*u*v+F(61)/4*y*z^2*u*v-F(98)/45*z^3*u*v+F(82)/7*x^2*u^2*v-F(85)/41*x*y*u^2*v+6548*y^2*u^2*v+9367*x*z*u^2*v-F(59)/81*y*z*u^2*v-14408*z^2*u^2*v+2363*x*u^3*v-F(80)/11*y*u^3*v-F(50)/17*z*u^3*v-14799*u^4*v-F(53)/21*x^3*v^2+9437*x^2*y*v^2-F(117)/80*x*y^2*v^2+F(81)/85*y^3*v^2-F(8)/45*x^2*z*v^2-6428*x*y*z*v^2+15126*y^2*z*v^2+F(68)/89*x*z^2*v^2+F(7)/122*y*z^2*v^2+9639*z^3*v^2+F(113)/4*x^2*u*v^2-8678*x*y*u*v^2-F(104)/45*y^2*u*v^2-F(79)/90*x*z*u*v^2+F(39)/101*y*z*u*v^2-7234*z^2*u*v^2-F(28)/43*x*u^2*v^2+1251*y*u^2*v^2-F(97)/56*z*u^2*v^2+F(17)/41*u^3*v^2+F(107)/24*x^2*v^3+2747*x*y*v^3+9933*y^2*v^3-4199*x*z*v^3+F(53)/83*y*z*v^3+6364*z^2*v^3-5456*x*u*v^3+618*y*u*v^3-F(123)/55*z*u*v^3+2375*u^2*v^3+F(63)/76*x*v^4-F(115)/106*y*v^4-8811*z*v^4-F(31)/75*u*v^4+F(10)/109*v^5,F(13)/89*x^4*y+F(77)/31*x^3*y^2+F(36)/83*x^2*y^3-11411*x*y^4+6936*y^5-12223*x^4*z+7400*x^3*y*z+F(33)/118*x^2*y^2*z-12146*x*y^3*z+F(108)/79*y^4*z+F(82)/99*x^3*z^2-9877*x^2*y*z^2-F(79)/70*x*y^2*z^2-F(19)/123*y^3*z^2-1491*x^2*z^3+7953*x*y*z^3-F(43)/126*y^2*z^3+F(60)/17*x*z^4+F(98)/57*y*z^4-13317*x^4*u-F(77)/27*x^3*y*u-6811*x^2*y^2*u-F(69)/61*x*y^3*u+6144*y^4*u+5404*x^3*z*u+F(121)/120*x^2*y*z*u-F(91)/23*x*y^2*z*u-F(71)/106*y^3*z*u+1435*x^2*z^2*u-F(120)/13*x*y*z^2*u-12019*y^2*z^2*u-F(68)/7*x*z^3*u-F(113)/82*y*z^3*u+11526*z^4*u-8706*x^3*u^2-F(89)/53*x^2*y*u^2-14804*x*y^2*u^2+F(120)/107*y^3*u^2+F(71)/94*x^2*z*u^2-F(1)/70*x*y*z*u^2+1532*y^2*z*u^2+4470*x*z^2*u^2+F(13)/60*y*z^2*u^2-F(115)/102*z^3*u^2-F(82)/21*x^2*u^3+F(27)/121*x*y*u^3-4439*y^2*u^3-F(101)/47*x*z*u^3-3186*y*z*u^3-F(106)/101*z^2*u^3-10169*x*u^4+F(19)/58*y*u^4-F(96)/73*z*u^4-7959*u^5-10526*x^4*v-F(107)/92*x^3*y*v+F(47)/6*x^2*y^2*v-F(23)/43*x*y^3*v-F(69)/62*y^4*v+F(59)/65*x^3*z*v-F(28)/95*x^2*y*z*v+5479*x*y^2*z*v-F(39)/77*y^3*z*v+F(11)/69*x^2*z^2*v-11713*x*y*z^2*v+F(43)/79*y^2*z^2*v-15602*x*z^3*v+F(16)/73*y*z^3*v-13952*z^4*v+F(61)/82*x^3*u*v-2219*x^2*y*u*v-F(91)/106*x*y^2*u*v+F(5)/37*y^3*u*v-148*x^2*z*u*v+F(31)/51*x*y*z*u*v+F(18)/101*y^2*z*u*v+F(97)/68*x*z^2*u*v-F(73)/32*y*z^2*u*v+F(47)/2*z^3*u*v+F(2)/41*x^2*u^2*v-13009*x*y*u^2*v-F(7)/60*y^2*u^2*v+15779*x*z*u^2*v+F(72)/7*y*z*u^2*v-F(11)/73*z^2*u^2*v-F(119)/44*x*u^3*v-9067*y*u^3*v+3249*z*u^3*v+F(61)/51*u^4*v+12525*x^3*v^2-F(118)/9*x^2*y*v^2-3270*x*y^2*v^2-F(4)/25*y^3*v^2-5075*x^2*z*v^2+F(77)/40*x*y*z*v^2-F(89)/65*y^2*z*v^2+F(17)/58*x*z^2*v^2-15609*y*z^2*v^2+F(95)/54*z^3*v^2-F(75)/79*x^2*u*v^2-4907*x*y*u*v^2+12418*y^2*u*v^2-F(57)/17*x*z*u*v^2-8746*y*z*u*v^2+F(13)/95*z^2*u*v^2-F(124)/67*x*u^2*v^2+F(16)/13*y*u^2*v^2+F(28)/23*z*u^2*v^2-10847*u^3*v^2-645*x^2*v^3+F(106)/75*x*y*v^3+F(6)/115*y^2*v^3-8495*x*z*v^3+F(58)/35*y*z*v^3-9398*z^2*v^3-F(101)/72*x*u*v^3-F(71)/20*y*u*v^3-F(124)/65*z*u*v^3-8971*u^2*v^3+F(27)/28*x*v^4+F(12)/29*y*v^4-4276*z*v^4+10858*u*v^4+F(29)/12*v^5,F(13)/89*x^3*y^2+12068*x^2*y^3-15543*x*y^4-F(77)/79*y^5+6626*x^3*y*z+F(64)/53*x^2*y^2*z-F(6)/23*x*y^3*z-F(47)/125*y^4*z+14403*x^3*z^2-F(43)/78*x^2*y*z^2-F(31)/115*x*y^2*z^2+F(94)/59*y^3*z^2-F(118)/117*x^2*z^3-11229*x*y*z^3+2268*y^2*z^3-F(116)/85*x*z^4+F(25)/58*y*z^4+3085*z^5+F(59)/27*x^3*y*u+F(67)/82*x^2*y^2*u+F(11)/6*x*y^3*u+F(103)/47*y^4*u-F(63)/80*x^3*z*u-F(81)/47*x^2*y*z*u+7760*x*y^2*z*u-F(115)/56*y^3*z*u-F(10)/17*x^2*z^2*u+F(101)/5*x*y*z^2*u+15634*y^2*z^2*u+F(1)/107*x*z^3*u-9282*y*z^3*u+F(43)/62*z^4*u+F(62)/55*x^3*u^2+F(100)/113*x^2*y*u^2-9205*x*y^2*u^2-F(46)/13*y^3*u^2+F(43)/96*x^2*z*u^2+10159*x*y*z*u^2+692*y^2*z*u^2+859*x*z^2*u^2-F(19)/74*y*z^2*u^2+F(123)/47*z^3*u^2-F(9)/20*x^2*u^3-11391*x*y*u^3-2375*y^2*u^3+F(109)/24*x*z*u^3-F(57)/53*y*z*u^3-925*z^2*u^3-F(82)/45*x*u^4+F(97)/34*y*u^4+F(13)/82*z*u^4-F(108)/29*u^5+F(63)/10*x^3*y*v+F(38)/17*x^2*y^2*v-F(19)/115*x*y^3*v+3150*y^4*v+F(22)/69*x^3*z*v+F(26)/57*x^2*y*z*v+F(110)/27*x*y^2*z*v+F(87)/77*y^3*z*v+F(85)/18*x^2*z^2*v+F(39)/47*x*y*z^2*v-F(48)/17*y^2*z^2*v-F(7)/27*x*z^3*v-F(13)/100*y*z^3*v-11662*z^4*v-F(17)/8*x^3*u*v+F(37)/11*x^2*y*u*v+F(29)/11*x*y^2*u*v-F(109)/88*y^3*u*v-2817*x^2*z*u*v-F(61)/44*x*y*z*u*v+F(10)/31*y^2*z*u*v+10010*x*z^2*u*v+F(51)/86*y*z^2*u*v-F(97)/83*z^3*u*v-F(89)/96*x^2*u^2*v+4030*x*y*u^2*v-F(58)/77*y^2*u^2*v-F(114)/43*x*z*u^2*v-F(37)/10*y*z*u^2*v-2011*z^2*u^2*v+14483*x*u^3*v-F(109)/101*y*u^3*v+F(121)/102*z*u^3*v-F(79)/92*u^4*v+15113*x^3*v^2+10781*x^2*y*v^2-14259*x*y^2*v^2-F(113)/48*y^3*v^2-F(7)/94*x^2*z*v^2-F(17)/74*x*y*z*v^2-F(5)/117*y^2*z*v^2-F(59)/75*x*z^2*v^2+13188*y*z^2*v^2+F(103)/43*z^3*v^2+F(4)/125*x^2*u*v^2-F(52)/59*x*y*u*v^2+F(85)/92*y^2*u*v^2-F(1)/46*x*z*u*v^2-9106*y*z*u*v^2-F(83)/11*z^2*u*v^2-F(23)/94*x*u^2*v^2+6742*y*u^2*v^2-F(35)/107*z*u^2*v^2-14596*u^3*v^2-F(117)/43*x^2*v^3+1026*x*y*v^3+F(90)/19*y^2*v^3+14671*x*z*v^3-F(101)/100*y*z*v^3+6962*z^2*v^3+F(61)/68*x*u*v^3+F(108)/37*y*u*v^3-4157*z*u*v^3-3974*u^2*v^3+15677*x*v^4+8661*y*v^4+8459*z*v^4-F(16)/23*u*v^4-F(37)/119*v^5,F(13)/89*x^3*y*z+12068*x^2*y^2*z-15543*x*y^3*z-F(77)/79*y^4*z-12223*x^3*z^2-13941*x^2*y*z^2+F(115)/84*x*y^2*z^2+F(13)/98*y^3*z^2+F(82)/99*x^2*z^3+7751*x*y*z^3+F(122)/17*y^2*z^3-1491*x*z^4+1327*y*z^4+F(60)/17*z^5+15363*x^3*y*u+9780*x^2*y^2*u+F(19)/117*x*y^3*u-1924*y^4*u-14600*x^3*z*u+F(46)/41*x^2*y*z*u-5466*x*y^2*z*u-F(73)/12*y^3*z*u+10838*x^2*z^2*u-8302*x*y*z^2*u-F(89)/113*y^2*z^2*u+F(53)/69*x*z^3*u-9224*y*z^3*u+F(47)/33*z^4*u-7399*x^3*u^2+F(89)/77*x^2*y*u^2+9312*x*y^2*u^2-F(41)/80*y^3*u^2-732*x^2*z*u^2-6781*x*y*z*u^2-8608*y^2*z*u^2-9270*x*z^2*u^2-F(117)/58*y*z^2*u^2-F(115)/68*z^3*u^2-F(48)/31*x^2*u^3-9067*x*y*u^3+F(97)/107*y^2*u^3+F(73)/57*x*z*u^3-2719*y*z*u^3-F(110)/59*z^2*u^3-F(37)/86*x*u^4-15796*y*u^4-F(61)/4*z*u^4-F(115)/72*u^5+6161*x^3*y*v+4134*x^2*y^2*v+677*x*y^3*v-8375*y^4*v+1150*x^3*z*v+1551*x^2*y*z*v+4157*x*y^2*z*v+F(112)/87*y^3*z*v+8171*x^2*z^2*v+6040*x*y*z^2*v+15651*y^2*z^2*v-F(7)/66*x*z^3*v-F(47)/61*y*z^3*v+F(77)/64*z^4*v+14848*x^3*u*v+F(48)/119*x^2*y*u*v-9534*x*y^2*u*v-F(117)/95*y^3*u*v+F(5)/4*x^2*z*u*v+122*x*y*z*u*v+F(90)/31*y^2*z*u*v-F(41)/26*x*z^2*u*v+F(31)/30*y*z^2*u*v-10428*z^3*u*v-9896*x^2*u^2*v-F(71)/21*x*y*u^2*v-F(55)/38*y^2*u^2*v-F(29)/22*x*z*u^2*v-11092*y*z*u^2*v+F(39)/122*z^2*u^2*v+F(93)/73*x*u^3*v+F(22)/49*y*u^3*v-F(21)/106*z*u^3*v+56*u^4*v+8565*x^3*v^2-1695*x^2*y*v^2+F(2)/17*x*y^2*v^2+F(1)/78*y^3*v^2-F(113)/71*x^2*z*v^2-F(41)/100*x*y*z*v^2+F(55)/14*y^2*z*v^2+15286*x*z^2*v^2+F(17)/53*y*z^2*v^2+F(126)/71*z^3*v^2-F(79)/87*x^2*u*v^2+F(109)/97*x*y*u*v^2-F(28)/31*y^2*u*v^2-6533*x*z*u*v^2+F(22)/5*y*z*u*v^2-10449*z^2*u*v^2+10830*x*u^2*v^2-15516*y*u^2*v^2+F(28)/57*z*u^2*v^2-F(81)/22*u^3*v^2+4198*x^2*v^3+5667*x*y*v^3-7133*y^2*v^3-8408*x*z*v^3+11066*y*z*v^3-F(26)/125*z^2*v^3-808*x*u*v^3+F(95)/54*y*u*v^3-F(64)/17*z*u*v^3-5267*u^2*v^3-15333*x*v^4+F(42)/89*y*v^4+F(63)/85*z*v^4+F(119)/113*u*v^4-2011*v^5,5583*x^4*y+1725*x^3*y^2-8652*x^2*y^3-F(91)/25*x*y^4-8495*x^4*z-13731*x^3*y*z+9298*x^2*y^2*z-F(41)/111*x*y^3*z-15503*y^4*z-13805*x^3*z^2+3962*x^2*y*z^2-F(2)/63*x*y^2*z^2+3314*y^3*z^2+2522*x^2*z^3-F(10)/87*x*y*z^3-408*y^2*z^3+F(7)/16*x*z^4+F(69)/22*y*z^4-7254*z^5-F(59)/21*x^4*u+F(115)/7*x^3*y*u-1718*x^2*y^2*u+7851*x*y^3*u+2632*y^4*u-F(82)/3*x^3*z*u+F(37)/86*x^2*y*z*u+F(101)/113*x*y^2*z*u+6747*y^3*z*u-F(109)/113*x^2*z^2*u+7399*x*y*z^2*u+F(24)/103*y^2*z^2*u+F(89)/9*x*z^3*u-14630*y*z^3*u+15066*z^4*u-12561*x^3*u^2+F(113)/115*x^2*y*u^2+F(87)/97*x*y^2*u^2-F(126)/67*y^3*u^2-F(48)/7*x^2*z*u^2+F(123)/103*x*y*z*u^2-F(11)/107*y^2*z*u^2-2747*x*z^2*u^2+8158*y*z^2*u^2-F(3)/107*z^3*u^2+F(41)/6*x^2*u^3+12767*x*y*u^3+3873*y^2*u^3+F(74)/83*x*z*u^3-F(55)/119*y*z*u^3-F(24)/83*z^2*u^3+55*x*u^4-F(7)/95*y*u^4+F(57)/44*z*u^4+F(2)/101*u^5-6928*x^4*v-F(121)/57*x^3*y*v+F(111)/104*x^2*y^2*v+946*x*y^3*v-29*y^4*v+3057*x^3*z*v-F(14)/25*x^2*y*z*v+F(43)/31*x*y^2*z*v-F(105)/2*y^3*z*v+2336*x^2*z^2*v+F(61)/77*x*y*z^2*v-7880*y^2*z^2*v+F(5)/58*x*z^3*v+10593*y*z^3*v+7094*z^4*v+F(63)/59*x^3*u*v-F(5)/69*x^2*y*u*v-F(11)/81*x*y^2*u*v-4157*y^3*u*v+F(73)/65*x^2*z*u*v-1676*x*y*z*u*v-2376*y^2*z*u*v-F(85)/63*x*z^2*u*v-F(95)/2*y*z^2*u*v-14903*z^3*u*v-F(119)/110*x^2*u^2*v-F(115)/24*x*y*u^2*v+F(125)/9*y^2*u^2*v+F(106)/87*x*z*u^2*v-F(13)/12*y*z*u^2*v-F(4)/19*z^2*u^2*v+7838*x*u^3*v-F(43)/111*y*u^3*v+F(7)/113*z*u^3*v-12500*u^4*v+7743*x^3*v^2-2023*x^2*y*v^2-F(85)/83*x*y^2*v^2+F(49)/41*y^3*v^2+F(20)/87*x^2*z*v^2+3932*x*y*z*v^2-F(77)/6*y^2*z*v^2+F(47)/90*x*z^2*v^2-15580*y*z^2*v^2+F(39)/4*z^3*v^2-F(61)/8*x^2*u*v^2+2518*x*y*u*v^2+F(29)/98*y^2*u*v^2+11057*x*z*u*v^2-F(18)/107*y*z*u*v^2+708*z^2*u*v^2+14720*x*u^2*v^2-3175*y*u^2*v^2-F(113)/59*z*u^2*v^2-14735*u^3*v^2+F(7)/69*x^2*v^3-4029*x*y*v^3+F(54)/91*y^2*v^3+12372*x*z*v^3+F(67)/2*y*z*v^3+8856*z^2*v^3-2178*x*u*v^3+995*y*u*v^3+F(64)/95*z*u*v^3+4039*u^2*v^3-F(37)/44*x*v^4+F(23)/17*y*v^4-3035*z*v^4-F(103)/124*u*v^4+F(69)/64*v^5,-5583*x^3*y^2-1725*x^2*y^3+8652*x*y^4+F(91)/25*y^5+6201*x^3*y*z-F(73)/49*x^2*y^2*z-3844*x*y^3*z+10548*y^4*z-11057*x^3*z^2-F(105)/122*x^2*y*z^2+F(31)/53*x*y^2*z^2+F(79)/89*y^3*z^2-F(24)/101*x^2*z^3+F(107)/119*x*y*z^3-126*y^2*z^3+8164*x*z^4+F(2)/77*y*z^4-F(51)/8*z^5-14941*x^3*y*u-106*x^2*y^2*u+8695*x*y^3*u+F(125)/62*y^4*u+4328*x^3*z*u+F(29)/117*x^2*y*z*u-6249*x*y^2*z*u-2791*y^3*z*u+F(67)/49*x^2*z^2*u-F(38)/29*x*y*z^2*u+F(122)/41*y^2*z^2*u+10603*x*z^3*u-3029*y*z^3*u+5578*z^4*u+14754*x^3*u^2-F(108)/79*x^2*y*u^2+4408*x*y^2*u^2-12401*y^3*u^2-1426*x^2*z*u^2-1741*x*y*z*u^2-F(83)/86*y^2*z*u^2+F(79)/95*x*z^2*u^2+F(122)/121*y*z^2*u^2+F(81)/2*z^3*u^2-1172*x^2*u^3-F(41)/68*x*y*u^3-F(70)/3*y^2*u^3+F(24)/107*x*z*u^3+F(120)/79*y*z*u^3+F(18)/119*z^2*u^3-F(65)/122*x*u^4+1018*y*u^4+F(22)/107*z*u^4+15189*u^5+F(5)/8*x^3*y*v-12060*x^2*y^2*v+F(3)/62*x*y^3*v-227*y^4*v+F(60)/41*x^3*z*v-F(123)/115*x^2*y*z*v+F(110)/123*x*y^2*z*v+12864*y^3*z*v-F(86)/121*x^2*z^2*v-F(69)/94*x*y*z^2*v+F(14)/79*y^2*z^2*v+F(118)/45*x*z^3*v+10842*y*z^3*v-F(37)/58*z^4*v+F(100)/69*x^3*u*v-F(47)/65*x^2*y*u*v-F(7)/67*x*y^2*u*v-F(93)/100*y^3*u*v-6262*x^2*z*u*v-F(4)/75*x*y*z*u*v+2082*y^2*z*u*v-9117*x*z^2*u*v+12450*y*z^2*u*v-F(84)/67*z^3*u*v+F(123)/26*x^2*u^2*v-F(51)/89*x*y*u^2*v+F(19)/74*y^2*u^2*v-F(104)/77*x*z*u^2*v+318*y*z*u^2*v+12402*z^2*u^2*v+F(95)/8*x*u^3*v-F(81)/26*y*u^3*v-4486*z*u^3*v+3872*u^4*v+F(72)/91*x^3*v^2-F(83)/63*x^2*y*v^2+F(93)/92*x*y^2*v^2-15924*y^3*v^2-F(53)/62*x^2*z*v^2+6046*x*y*z*v^2+1408*y^2*z*v^2+F(60)/107*x*z^2*v^2-1150*y*z^2*v^2-F(126)/19*z^3*v^2-7429*x^2*u*v^2+2554*x*y*u*v^2+3602*y^2*u*v^2+10738*x*z*u*v^2-F(57)/64*y*z*u*v^2+F(86)/69*z^2*u*v^2+8172*x*u^2*v^2+F(91)/113*y*u^2*v^2+F(92)/65*z*u^2*v^2+F(118)/37*u^3*v^2+F(47)/83*x^2*v^3+12750*x*y*v^3+10851*y^2*v^3+4216*x*z*v^3+F(6)/101*y*z*v^3-108*z^2*v^3+2920*x*u*v^3-F(101)/102*y*u*v^3-157*z*u*v^3+7742*u^2*v^3-7234*x*v^4-F(2)/111*y*v^4+F(59)/33*z*v^4-F(93)/91*u*v^4+F(24)/19*v^5,1592*x^4*y+F(75)/121*x^3*y^2+F(40)/19*x^2*y^3-2651*x*y^4+9934*x^4*z+245*x^3*y*z+11665*x^2*y^2*z+F(30)/41*x*y^3*z+1823*y^4*z+F(89)/88*x^3*z^2-F(105)/46*x^2*y*z^2+F(79)/58*x*y^2*z^2-4191*y^3*z^2-F(76)/61*x^2*z^3-F(21)/32*x*y*z^3-9516*y^2*z^3-14896*x*z^4-F(85)/77*y*z^4+F(51)/109*z^5+F(61)/30*x^4*u-F(10)/101*x^3*y*u+11796*x^2*y^2*u+F(76)/101*x*y^3*u+F(123)/88*y^4*u-5932*x^3*z*u-11857*x^2*y*z*u+7128*x*y^2*z*u-F(45)/79*y^3*z*u+F(119)/18*x^2*z^2*u+F(9)/74*x*y*z^2*u+7042*y^2*z^2*u-1114*x*z^3*u-F(11)/82*y*z^3*u-1466*z^4*u-F(6)/85*x^3*u^2+F(27)/106*x^2*y*u^2+14246*x*y^2*u^2-6216*y^3*u^2+F(47)/6*x^2*z*u^2-F(45)/59*x*y*z*u^2+F(89)/41*y^2*z*u^2+F(41)/80*x*z^2*u^2-7583*y*z^2*u^2-F(75)/113*z^3*u^2-14808*x^2*u^3-10873*x*y*u^3-F(90)/67*y^2*u^3-11081*x*z*u^3-7369*y*z*u^3-7131*z^2*u^3-1402*x*u^4-15386*y*u^4-F(108)/73*z*u^4-5039*u^5+F(120)/113*x^4*v+10617*x^3*y*v-F(50)/87*x^2*y^2*v-2395*x*y^3*v-F(20)/69*y^4*v-8587*x^3*z*v+12960*x^2*y*z*v-F(41)/50*x*y^2*z*v-13844*y^3*z*v-F(65)/32*x^2*z^2*v-F(77)/122*x*y*z^2*v-F(85)/66*y^2*z^2*v+F(13)/100*x*z^3*v-F(20)/51*y*z^3*v-13676*z^4*v+F(76)/97*x^3*u*v+1046*x^2*y*u*v-8059*x*y^2*u*v-F(117)/59*y^3*u*v-F(29)/105*x^2*z*u*v+7287*x*y*z*u*v-F(107)/119*y^2*z*u*v-F(35)/118*x*z^2*u*v+F(79)/86*y*z^2*u*v-2211*z^3*u*v+5448*x^2*u^2*v+F(62)/35*x*y*u^2*v-2275*y^2*u^2*v+F(29)/121*x*z*u^2*v-1674*y*z*u^2*v-F(56)/43*z^2*u^2*v-3377*x*u^3*v-F(43)/110*y*u^3*v+F(23)/10*z*u^3*v-F(24)/61*u^4*v+F(121)/53*x^3*v^2-4745*x^2*y*v^2-F(57)/64*x*y^2*v^2+9554*y^3*v^2-12741*x^2*z*v^2+10449*x*y*z*v^2+F(37)/108*y^2*z*v^2+8621*x*z^2*v^2-F(11)/57*y*z^2*v^2+1566*z^3*v^2+F(125)/49*x^2*u*v^2-F(121)/118*x*y*u*v^2+F(109)/84*y^2*u*v^2-335*x*z*u*v^2+10167*y*z*u*v^2-F(59)/109*z^2*u*v^2-F(103)/119*x*u^2*v^2+F(43)/13*y*u^2*v^2-F(73)/87*z*u^2*v^2+2037*u^3*v^2+13002*x^2*v^3+F(83)/48*x*y*v^3-10713*y^2*v^3+1026*x*z*v^3-F(105)/64*y*z*v^3-F(37)/6*z^2*v^3+14779*x*u*v^3-6448*y*u*v^3+F(19)/69*z*u*v^3-F(1)/110*u^2*v^3+10010*x*v^4+F(79)/12*y*v^4+F(12)/19*z*v^4-F(35)/61*u*v^4-F(11)/57*v^5,-1592*x^3*y^2-F(75)/121*x^2*y^3-F(40)/19*x*y^4+2651*y^5+F(39)/121*x^3*y*z+F(122)/77*x^2*y^2*z-F(114)/31*x*y^3*z+1544*y^4*z+F(2)/3*x^3*z^2-10271*x^2*y*z^2-8373*x*y^2*z^2+F(56)/61*y^3*z^2+F(55)/48*x^2*z^3-116*x*y*z^3-F(25)/7*y^2*z^3-F(108)/113*x*z^4-F(34)/53*y*z^4+5548*z^5-122*x^3*y*u-9690*x^2*y^2*u+F(43)/87*x*y^3*u-F(5)/19*y^4*u+F(97)/54*x^3*z*u-F(17)/19*x^2*y*z*u+4355*x*y^2*z*u+F(12)/5*y^3*z*u-F(1)/100*x^2*z^2*u+12754*x*y*z^2*u+13600*y^2*z^2*u+F(17)/45*x*z^3*u-12091*y*z^3*u+5145*z^4*u-F(63)/64*x^3*u^2-F(84)/31*x^2*y*u^2-F(97)/41*x*y^2*u^2+F(7)/13*y^3*u^2-F(79)/62*x^2*z*u^2-F(80)/103*x*y*z*u^2-F(69)/14*y^2*z*u^2+F(119)/4*x*z^2*u^2-F(35)/87*y*z^2*u^2-13840*z^3*u^2+14101*x^2*u^3+7952*x*y*u^3-1857*y^2*u^3-9861*x*z*u^3+3180*y*z*u^3+F(75)/107*z^2*u^3-250*x*u^4-15134*y*u^4+4717*z*u^4-F(2)/41*u^5+F(22)/27*x^3*y*v-8983*x^2*y^2*v+10520*x*y^3*v-F(113)/2*y^4*v+F(10)/73*x^3*z*v-1986*x^2*y*z*v-F(110)/13*x*y^2*z*v+1550*y^3*z*v+F(32)/111*x^2*z^2*v-F(111)/35*x*y*z^2*v+F(101)/98*y^2*z^2*v+8045*x*z^3*v-F(2)/89*y*z^3*v+2924*z^4*v-F(79)/11*x^3*u*v-15178*x^2*y*u*v+10874*x*y^2*u*v+F(54)/11*y^3*u*v-8950*x^2*z*u*v+F(70)/53*x*y*z*u*v-2403*y^2*z*u*v-8249*x*z^2*u*v+6935*y*z^2*u*v+F(20)/89*z^3*u*v+885*x^2*u^2*v-F(76)/71*x*y*u^2*v-F(4)/17*y^2*u^2*v-F(31)/52*x*z*u^2*v-F(4)/99*y*z*u^2*v+10333*z^2*u^2*v-F(93)/104*x*u^3*v+F(82)/101*y*u^3*v-F(71)/37*z*u^3*v+9397*u^4*v-F(15)/112*x^3*v^2-6614*x^2*y*v^2+F(119)/2*x*y^2*v^2+F(88)/119*y^3*v^2+306*x^2*z*v^2+2790*x*y*z*v^2+10992*y^2*z*v^2-F(115)/74*x*z^2*v^2-14711*y*z^2*v^2+11612*z^3*v^2-1788*x^2*u*v^2-F(75)/97*x*y*u*v^2+F(79)/30*y^2*u*v^2+F(99)/59*x*z*u*v^2-11439*y*z*u*v^2-F(121)/113*z^2*u*v^2+F(108)/37*x*u^2*v^2+F(37)/36*y*u^2*v^2-F(3)/65*z*u^2*v^2-F(55)/42*u^3*v^2+F(13)/100*x^2*v^3-209*x*y*v^3-1272*y^2*v^3-F(117)/68*x*z*v^3+F(63)/94*y*z*v^3+F(32)/59*z^2*v^3+1013*x*u*v^3-3463*y*u*v^3+6946*z*u*v^3-F(37)/86*u^2*v^3+F(67)/117*x*v^4+F(85)/28*y*v^4-3024*z*v^4-F(82)/9*u*v^4-F(32)/65*v^5,-F(35)/52*x^4*y-12140*x^3*y^2+F(23)/83*x^2*y^3+F(69)/5*x*y^4-F(80)/79*y^5+F(120)/43*x^4*z-11865*x^3*y*z-3487*x^2*y^2*z+F(53)/59*x*y^3*z+F(53)/102*y^4*z-14083*x^3*z^2-14430*x^2*y*z^2-2442*x*y^2*z^2-F(33)/104*y^3*z^2-F(91)/38*x^2*z^3+F(4)/87*x*y*z^3-F(26)/57*y^2*z^3+4097*x*z^4-F(9)/122*y*z^4+6364*z^5+9634*x^4*u-F(97)/95*x^3*y*u-F(46)/99*x^2*y^2*u+3847*x*y^3*u+F(121)/106*y^4*u+12765*x^3*z*u-5292*x^2*y*z*u+1607*x*y^2*z*u-F(67)/121*y^3*z*u-F(12)/35*x^2*z^2*u+F(4)/55*x*y*z^2*u-F(17)/27*y^2*z^2*u+F(91)/122*x*z^3*u-F(23)/31*y*z^3*u+F(65)/49*z^4*u+F(73)/46*x^3*u^2-F(124)/27*x^2*y*u^2-9933*x*y^2*u^2+F(46)/75*y^3*u^2+F(53)/114*x^2*z*u^2+3503*x*y*z*u^2-14147*y^2*z*u^2-11283*x*z^2*u^2+11889*y*z^2*u^2+F(99)/104*z^3*u^2+3117*x^2*u^3+12624*x*y*u^3-10060*y^2*u^3+2193*x*z*u^3-F(80)/47*y*z*u^3-F(77)/13*z^2*u^3+F(11)/31*x*u^4-F(47)/90*y*u^4+F(49)/48*z*u^4-F(2)/105*u^5-F(92)/61*x^4*v+7443*x^3*y*v+F(35)/76*x^2*y^2*v+F(114)/67*x*y^3*v-F(73)/126*y^4*v+F(97)/107*x^3*z*v+9464*x^2*y*z*v+10869*x*y^2*z*v+15718*y^3*z*v-F(37)/33*x^2*z^2*v+F(124)/13*x*y*z^2*v-F(11)/26*y^2*z^2*v-F(61)/40*x*z^3*v+F(91)/100*y*z^3*v-F(18)/103*z^4*v+F(60)/29*x^3*u*v+F(21)/125*x^2*y*u*v-11117*x*y^2*u*v+11748*y^3*u*v-F(16)/117*x^2*z*u*v+F(18)/103*x*y*z*u*v-1711*y^2*z*u*v+1872*x*z^2*u*v-F(109)/123*y*z^2*u*v-F(18)/113*z^3*u*v-F(26)/103*x^2*u^2*v+14140*x*y*u^2*v+11065*y^2*u^2*v+8686*x*z*u^2*v-F(5)/111*y*z*u^2*v+F(30)/101*z^2*u^2*v-10501*x*u^3*v-F(36)/113*y*u^3*v-F(73)/74*z*u^3*v+12753*u^4*v-F(43)/52*x^3*v^2-F(76)/15*x^2*y*v^2-5793*x*y^2*v^2+F(18)/13*y^3*v^2+F(1)/79*x^2*z*v^2+F(84)/23*x*y*z*v^2-172*y^2*z*v^2+F(86)/77*x*z^2*v^2+F(15)/37*y*z^2*v^2+11835*z^3*v^2-6482*x^2*u*v^2+F(94)/113*x*y*u*v^2+10727*y^2*u*v^2-F(102)/41*x*z*u*v^2+15914*y*z*u*v^2-12973*z^2*u*v^2-9038*x*u^2*v^2-13107*y*u^2*v^2+1533*z*u^2*v^2+12549*u^3*v^2-13528*x^2*v^3+903*x*y*v^3+F(23)/114*y^2*v^3-F(123)/64*x*z*v^3-F(81)/5*y*z*v^3+F(111)/103*z^2*v^3+4734*x*u*v^3-F(33)/20*y*u*v^3-7954*z*u*v^3-2478*u^2*v^3+15518*x*v^4-6723*y*v^4-F(14)/31*z*v^4-3482*u*v^4+10919*v^5,-F(3)/94*x^4*y-12936*x^3*y^2+F(2)/11*x^2*y^3+F(32)/23*x*y^4-15921*y^5+F(61)/93*x^4*z+F(82)/111*x^3*y*z-F(93)/2*x^2*y^2*z-6659*x*y^3*z-F(97)/90*y^4*z+402*x^3*z^2-14586*x^2*y*z^2-F(121)/39*x*y^2*z^2+F(68)/7*y^3*z^2+1212*x^2*z^3-2980*x*y*z^3+F(49)/52*y^2*z^3-F(72)/89*x*z^4+F(92)/47*y*z^4+8478*z^5+2733*x^4*u-F(103)/89*x^3*y*u+1166*x^2*y^2*u-F(7)/53*x*y^3*u-F(106)/23*y^4*u+677*x^3*z*u+907*x^2*y*z*u+7891*x*y^2*z*u-9014*y^3*z*u+F(76)/47*x^2*z^2*u+F(49)/116*x*y*z^2*u-F(49)/78*y^2*z^2*u+12261*x*z^3*u+F(118)/105*y*z^3*u-F(126)/13*z^4*u-8812*x^3*u^2-F(97)/120*x^2*y*u^2-9534*x*y^2*u^2+F(92)/5*y^3*u^2-F(54)/71*x^2*z*u^2+F(94)/103*x*y*z*u^2+2256*y^2*z*u^2+4182*x*z^2*u^2-5798*y*z^2*u^2-F(31)/115*z^3*u^2-F(73)/98*x^2*u^3+15822*x*y*u^3+1004*y^2*u^3-578*x*z*u^3+9494*y*z*u^3-6779*z^2*u^3+14506*x*u^4+F(10)/121*y*u^4+F(58)/27*z*u^4-2817*u^5-F(19)/119*x^4*v+7128*x^3*y*v+F(75)/64*x^2*y^2*v-F(65)/109*x*y^3*v+5129*y^4*v-F(53)/55*x^3*z*v+F(54)/125*x^2*y*z*v-3009*x*y^2*z*v+6144*y^3*z*v+15601*x^2*z^2*v+F(123)/55*x*y*z^2*v-F(58)/77*y^2*z^2*v-F(56)/61*x*z^3*v+F(121)/10*y*z^3*v-F(103)/86*z^4*v-F(93)/25*x^3*u*v+F(94)/123*x^2*y*u*v-F(25)/107*x*y^2*u*v+14807*y^3*u*v+F(65)/7*x^2*z*u*v+F(87)/44*x*y*z*u*v+6605*y^2*z*u*v+F(23)/99*x*z^2*u*v-413*y*z^2*u*v-F(17)/15*z^3*u*v-F(79)/46*x^2*u^2*v+15240*x*y*u^2*v-F(42)/67*y^2*u^2*v+8932*x*z*u^2*v-5888*y*z*u^2*v-4204*z^2*u^2*v+7002*x*u^3*v-F(36)/97*y*u^3*v-1634*z*u^3*v+F(61)/102*u^4*v-F(14)/33*x^3*v^2-6520*x^2*y*v^2+9004*x*y^2*v^2-F(67)/36*y^3*v^2-F(7)/8*x^2*z*v^2-F(24)/11*x*y*z*v^2-9373*y^2*z*v^2+1556*x*z^2*v^2-F(79)/74*y*z^2*v^2-6691*z^3*v^2+108*x^2*u*v^2-F(76)/61*x*y*u*v^2+220*y^2*u*v^2-1191*x*z*u*v^2-F(4)/9*y*z*u*v^2+4546*z^2*u*v^2+12205*x*u^2*v^2+F(9)/22*y*u^2*v^2+F(64)/93*z*u^2*v^2-F(44)/125*u^3*v^2+292*x^2*v^3+F(41)/74*x*y*v^3+F(16)/79*y^2*v^3-15892*x*z*v^3+5733*y*z*v^3+6796*z^2*v^3-F(42)/55*x*u*v^3+F(71)/79*y*u*v^3-F(19)/104*z*u*v^3-F(38)/15*u^2*v^3+6436*x*v^4+F(28)/15*y*v^4+F(87)/55*z*v^4+2270*u*v^4-F(30)/41*v^5,-F(117)/4*x^3*y+F(97)/122*x^2*y^2-3618*x*y^3+6566*y^4+F(97)/113*x^3*z-12634*x^2*y*z+9865*x*y^2*z-1764*y^3*z+F(114)/31*x^2*z^2+5006*x*y*z^2+F(7)/44*y^2*z^2-15040*x*z^3+F(8)/125*y*z^3+11134*z^4-12980*x^3*u-F(79)/41*x^2*y*u-F(79)/98*x*y^2*u+F(89)/65*y^3*u-1217*x^2*z*u+F(89)/87*x*y*z*u+F(83)/66*y^2*z*u+F(115)/11*x*z^2*u+F(123)/107*y*z^2*u+10920*z^3*u-F(86)/73*x^2*u^2-F(11)/94*x*y*u^2-14054*y^2*u^2+6752*x*z*u^2-F(123)/124*y*z*u^2+12129*z^2*u^2-13310*x*u^3-F(52)/63*y*u^3+12847*z*u^3-1545*u^4-11064*x^3*v+11499*x^2*y*v-F(37)/64*x*y^2*v+F(50)/103*y^3*v+F(123)/94*x^2*z*v-126*x*y*z*v-F(111)/44*y^2*z*v+F(95)/14*x*z^2*v+F(113)/83*y*z^2*v-F(77)/103*z^3*v+F(41)/64*x^2*u*v+F(91)/90*x*y*u*v-4932*y^2*u*v+F(103)/31*x*z*u*v+F(62)/63*y*z*u*v+1161*z^2*u*v-F(99)/106*x*u^2*v-3181*y*u^2*v-11741*z*u^2*v-F(33)/8*u^3*v-F(3)/118*x^2*v^2-9369*x*y*v^2+527*y^2*v^2-F(113)/39*x*z*v^2-F(88)/49*y*z*v^2-F(113)/101*z^2*v^2+F(95)/68*x*u*v^2-5930*y*u*v^2-F(20)/43*z*u*v^2+F(7)/41*u^2*v^2+F(109)/93*x*v^3-F(107)/61*y*v^3-8352*z*v^3-5255*u*v^3+12021*v^4,-2159*x^4-F(94)/3*x^3*y-4602*x^2*y^2+1609*x*y^3+10721*y^4+F(28)/9*x^3*z-F(99)/35*x^2*y*z+F(1)/110*x*y^2*z+F(113)/114*y^3*z-F(118)/75*x^2*z^2-F(103)/93*x*y*z^2-F(68)/67*y^2*z^2+13687*x*z^3-1531*y*z^3+F(61)/107*z^4+6076*x^3*u+9004*x^2*y*u+2211*x*y^2*u+F(110)/53*y^3*u+F(47)/102*x^2*z*u+8495*x*y*z*u-9238*y^2*z*u+F(57)/121*x*z^2*u-8543*y*z^2*u+F(8)/19*z^3*u-13527*x^2*u^2-13293*x*y*u^2+1138*y^2*u^2+F(26)/115*x*z*u^2+F(78)/53*y*z*u^2-12556*z^2*u^2+7299*x*u^3+F(70)/19*y*u^3-14687*z*u^3+13559*u^4+F(113)/9*x^3*v-F(85)/126*x^2*y*v-F(83)/3*x*y^2*v-F(3)/46*y^3*v+1814*x^2*z*v+F(28)/79*x*y*z*v+F(103)/51*y^2*z*v+F(78)/31*x*z^2*v-14387*y*z^2*v+F(1)/88*z^3*v+F(116)/75*x^2*u*v-F(101)/59*x*y*u*v-F(70)/3*y^2*u*v+F(109)/71*x*z*u*v+F(13)/88*y*z*u*v-147*z^2*u*v-F(113)/76*x*u^2*v-9661*y*u^2*v+13855*z*u^2*v-6162*u^3*v-1857*x^2*v^2-8208*x*y*v^2-4634*y^2*v^2-6178*x*z*v^2-7352*y*z*v^2-8247*z^2*v^2-F(113)/15*x*u*v^2+F(99)/40*y*u*v^2+F(21)/97*z*u*v^2+F(11)/37*u^2*v^2-6605*x*v^3+8964*y*v^3+F(35)/121*z*v^3+8543*u*v^3-6008*v^4]))
end


@doc raw"""
    rational_d11_pi11_ss_0()

Return a smooth rational surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and no 6-secant.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d11_pi11_ss_0()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([1204*x^5-466*x^4*y-F(66)/89*x^3*y^2+7174*x^2*y^3+F(106)/67*x*y^4-F(122)/65*x^4*z+F(34)/21*x^3*y*z-1935*x^2*y^2*z+4204*x*y^3*z+F(123)/125*y^4*z+F(33)/7*x^3*z^2-F(45)/46*x^2*y*z^2-14907*x*y^2*z^2+10392*y^3*z^2+12828*x^2*z^3+F(96)/19*x*y*z^3+F(64)/99*y^2*z^3+F(36)/7*x*z^4+2038*y*z^4+3509*z^5-F(38)/103*x^4*u-2224*x^3*y*u-11764*x^2*y^2*u-10781*x*y^3*u-F(75)/41*x^3*z*u-9515*x^2*y*z*u-F(55)/91*x*y^2*z*u+F(89)/76*y^3*z*u-F(66)/17*x^2*z^2*u+F(47)/98*x*y*z^2*u-1327*y^2*z^2*u-6144*x*z^3*u+11379*y*z^3*u-1710*z^4*u-F(19)/4*x^3*u^2+11356*x^2*y*u^2-F(21)/71*x*y^2*u^2-F(64)/123*x^2*z*u^2-6909*x*y*z*u^2-11808*y^2*z*u^2+15887*x*z^2*u^2-F(52)/77*y*z^2*u^2-12400*z^3*u^2-12314*x^2*u^3+13494*x*y*u^3-F(3)/76*x*z*u^3+F(38)/55*y*z*u^3-15800*z^2*u^3+6924*x*u^4+10526*z*u^4+2479*x^4*v-F(118)/17*x^3*y*v-F(35)/26*x^2*y^2*v+F(20)/67*x*y^3*v-3974*y^4*v-F(39)/62*x^3*z*v+62*x^2*y*z*v-6060*x*y^2*z*v+9868*y^3*z*v-F(97)/54*x^2*z^2*v+903*x*y*z^2*v+15410*y^2*z^2*v-12091*x*z^3*v+F(97)/68*y*z^3*v+1592*z^4*v+10619*x^3*u*v+4832*x^2*y*u*v+13579*x*y^2*u*v+12467*y^3*u*v+11621*x^2*z*u*v-8988*x*y*z*u*v-3350*y^2*z*u*v-F(58)/33*x*z^2*u*v+8597*y*z^2*u*v-F(70)/69*z^3*u*v+8742*x^2*u^2*v-F(63)/68*x*y*u^2*v-12194*y^2*u^2*v+1890*x*z*u^2*v-F(83)/39*y*z*u^2*v+3407*z^2*u^2*v+13198*x*u^3*v+12096*y*u^3*v-F(35)/114*z*u^3*v+4808*u^4*v-F(114)/41*x^3*v^2+F(107)/79*x^2*y*v^2-6365*x*y^2*v^2-F(18)/89*y^3*v^2-F(88)/41*x^2*z*v^2-F(105)/52*x*y*z*v^2+F(61)/108*y^2*z*v^2+F(8)/53*x*z^2*v^2+11530*y*z^2*v^2-1765*z^3*v^2+F(37)/59*x^2*u*v^2+F(61)/95*x*y*u*v^2+882*y^2*u*v^2+10356*x*z*u*v^2+F(86)/3*y*z*u*v^2-F(46)/53*z^2*u*v^2-F(9)/11*x*u^2*v^2+F(13)/105*y*u^2*v^2-F(55)/21*z*u^2*v^2-10427*u^3*v^2+7484*x^2*v^3-F(71)/102*x*y*v^3-13462*y^2*v^3-F(5)/11*x*z*v^3-F(79)/105*y*z*v^3+F(73)/122*z^2*v^3-F(101)/19*x*u*v^3+F(121)/30*y*u*v^3-F(64)/63*z*u*v^3+F(113)/80*u^2*v^3+14004*x*v^4+14131*y*v^4+2227*z*v^4-F(64)/103*u*v^4+F(68)/43*v^5,-F(17)/9*x^4*y+F(120)/113*x^3*y^2-F(82)/109*x^2*y^3-1005*x*y^4+7410*y^5-9688*x^4*z-F(116)/25*x^3*y*z-F(9)/65*x^2*y^2*z+8455*x*y^3*z-3009*y^4*z+F(27)/41*x^3*z^2-403*x^2*y*z^2-F(101)/8*x*y^2*z^2+F(17)/32*y^3*z^2-F(103)/46*x^2*z^3-4909*x*y*z^3+F(73)/27*y^2*z^3+7942*x*z^4-12953*y*z^4-10246*z^5+13874*x^4*u-F(61)/40*x^3*y*u-F(81)/53*x^2*y^2*u+F(46)/11*x*y^3*u+14846*y^4*u+F(53)/61*x^3*z*u-F(41)/106*x^2*y*z*u-5778*x*y^2*z*u+F(119)/86*y^3*z*u-2495*x^2*z^2*u-F(65)/44*x*y*z^2*u-F(79)/32*y^2*z^2*u-F(8)/39*x*z^3*u-F(61)/2*y*z^3*u+F(126)/61*z^4*u-F(32)/41*x^3*u^2+F(46)/49*x^2*y*u^2+7307*x*y^2*u^2+F(123)/109*y^3*u^2+F(81)/32*x^2*z*u^2-12260*x*y*z*u^2+10962*y^2*z*u^2+3224*x*z^2*u^2+7517*y*z^2*u^2+7473*z^3*u^2-F(29)/33*x^2*u^3-F(30)/103*x*y*u^3-F(38)/31*y^2*u^3-F(1)/108*x*z*u^3+8548*y*z*u^3+6100*z^2*u^3-9984*x*u^4-10300*y*u^4-4437*z*u^4-F(125)/122*u^5+15916*x^4*v-11209*x^3*y*v+F(58)/61*x^2*y^2*v+F(121)/58*x*y^3*v+F(56)/111*y^4*v-F(9)/7*x^3*z*v+F(109)/100*x^2*y*z*v+F(119)/47*x*y^2*z*v+F(102)/77*y^3*z*v+8669*x^2*z^2*v+F(53)/26*x*y*z^2*v-F(111)/17*y^2*z^2*v+6912*x*z^3*v-12027*y*z^3*v+F(45)/16*z^4*v+8547*x^3*u*v+9069*x^2*y*u*v-F(97)/94*x*y^2*u*v+13734*y^3*u*v-F(57)/115*x^2*z*u*v-4205*x*y*z*u*v-F(57)/112*y^2*z*u*v+F(83)/112*x*z^2*u*v+15643*y*z^2*u*v+F(37)/7*z^3*u*v-F(125)/56*x^2*u^2*v-6653*x*y*u^2*v-F(83)/97*y^2*u^2*v-2802*x*z*u^2*v+F(87)/97*y*z*u^2*v-6326*z^2*u^2*v-F(118)/45*x*u^3*v+F(11)/83*y*u^3*v-4898*z*u^3*v-F(107)/114*u^4*v+15609*x^3*v^2-F(103)/36*x^2*y*v^2+522*x*y^2*v^2+F(116)/69*y^3*v^2+F(53)/49*x^2*z*v^2-F(62)/27*x*y*z*v^2+9856*y^2*z*v^2-136*x*z^2*v^2-F(113)/36*y*z^2*v^2+F(90)/107*z^3*v^2-F(26)/119*x^2*u*v^2-F(89)/106*x*y*u*v^2+F(33)/52*y^2*u*v^2-F(25)/126*x*z*u*v^2+F(65)/6*y*z*u*v^2+10950*z^2*u*v^2+F(22)/125*x*u^2*v^2-F(31)/21*y*u^2*v^2+1111*z*u^2*v^2+9973*u^3*v^2-4690*x^2*v^3+F(11)/51*x*y*v^3+F(39)/71*y^2*v^3+14431*x*z*v^3+F(6)/107*y*z*v^3+F(111)/53*z^2*v^3-F(1)/90*x*u*v^3+4215*y*u*v^3-1576*z*u*v^3-F(82)/59*u^2*v^3-10158*x*v^4-F(23)/2*y*v^4-357*z*v^4-F(28)/27*u*v^4-F(116)/119*v^5,-F(63)/17*x^4*y-F(18)/97*x^3*y^2-1465*x^2*y^3+15205*x*y^4+1571*y^5-15636*x^4*z-F(104)/93*x^3*y*z+F(113)/111*x^2*y^2*z+F(41)/126*x*y^3*z-6928*y^4*z-F(34)/5*x^3*z^2-9431*x^2*y*z^2-15504*x*y^2*z^2+3462*y^3*z^2-F(108)/119*x^2*z^3-5419*x*y*z^3-F(46)/13*y^2*z^3-991*x*z^4+F(115)/108*y*z^4+F(93)/107*z^5+F(97)/104*x^4*u-F(83)/49*x^3*y*u+1787*x^2*y^2*u+F(61)/6*x*y^3*u-5360*y^4*u+F(37)/60*x^3*z*u+F(31)/66*x^2*y*z*u-F(95)/112*x*y^2*z*u-F(34)/61*y^3*z*u-F(113)/18*x^2*z^2*u-F(67)/74*x*y*z^2*u-F(27)/80*y^2*z^2*u-F(87)/46*x*z^3*u+F(117)/112*y*z^3*u-F(20)/81*z^4*u+6802*x^3*u^2+749*x^2*y*u^2-8053*x*y^2*u^2+F(43)/76*y^3*u^2+1004*x^2*z*u^2+F(32)/117*x*y*z*u^2+F(23)/72*y^2*z*u^2-F(88)/81*x*z^2*u^2+8519*y*z^2*u^2+F(73)/80*z^3*u^2-F(22)/29*x^2*u^3+F(33)/5*x*y*u^3+F(125)/38*y^2*u^3+F(72)/101*x*z*u^3-11057*y*z*u^3-5879*z^2*u^3+F(35)/57*x*u^4-F(51)/109*y*u^4-10605*z*u^4+8454*u^5-F(83)/119*x^4*v-F(109)/56*x^3*y*v-F(103)/89*x^2*y^2*v-14406*x*y^3*v+F(13)/46*y^4*v+5136*x^3*z*v-4809*x^2*y*z*v-4245*x*y^2*z*v-F(78)/79*y^3*z*v-F(97)/116*x^2*z^2*v-F(73)/119*x*y*z^2*v-F(64)/117*y^2*z^2*v+5262*x*z^3*v-178*y*z^3*v+F(21)/106*z^4*v-9780*x^3*u*v-F(37)/113*x^2*y*u*v-F(70)/87*x*y^2*u*v+F(81)/2*y^3*u*v+5718*x^2*z*u*v-11114*x*y*z*u*v-F(1)/108*y^2*z*u*v-1589*x*z^2*u*v-F(40)/117*y*z^2*u*v+F(56)/81*z^3*u*v+11748*x^2*u^2*v-F(77)/124*x*y*u^2*v+10936*y^2*u^2*v+F(106)/107*x*z*u^2*v-F(31)/69*y*z*u^2*v+F(55)/79*z^2*u^2*v-2980*x*u^3*v+F(54)/37*y*u^3*v+F(77)/67*z*u^3*v+7535*u^4*v-F(61)/120*x^3*v^2+12953*x^2*y*v^2-136*x*y^2*v^2-4641*y^3*v^2+F(109)/36*x^2*z*v^2-F(66)/125*x*y*z*v^2-6197*y^2*z*v^2-F(68)/95*x*z^2*v^2+F(29)/68*y*z^2*v^2-F(20)/69*z^3*v^2+14895*x^2*u*v^2-F(35)/117*x*y*u*v^2+F(11)/68*y^2*u*v^2+13460*x*z*u*v^2+F(97)/29*y*z*u*v^2+1462*z^2*u*v^2-10257*x*u^2*v^2-15759*y*u^2*v^2+F(83)/27*z*u^2*v^2+F(93)/37*u^3*v^2+F(3)/34*x^2*v^3-F(99)/62*x*y*v^3-F(61)/82*y^2*v^3-F(79)/8*x*z*v^3+F(36)/13*y*z*v^3+F(81)/37*z^2*v^3+8077*x*u*v^3+F(95)/22*y*u*v^3+F(88)/35*z*u*v^3-6253*u^2*v^3-F(125)/3*x*v^4+F(63)/52*y*v^4+F(81)/103*z*v^4+495*u*v^4-F(37)/34*v^5,-5257*x^5+F(109)/71*x^4*y-F(103)/49*x^3*y^2+F(65)/114*x^2*y^3+3399*x*y^4+F(29)/62*x^4*z+F(114)/125*x^3*y*z+15702*x^2*y^2*z-F(101)/107*x*y^3*z+F(2)/63*y^4*z-F(80)/119*x^3*z^2+15161*x^2*y*z^2+F(56)/25*x*y^2*z^2+F(110)/21*y^3*z^2-F(114)/121*x^2*z^3-F(10)/101*x*y*z^3+F(24)/71*y^2*z^3-F(13)/112*x*z^4+F(91)/79*y*z^4-F(94)/121*z^5-4898*x^4*u+10531*x^3*y*u+F(67)/40*x^2*y^2*u-F(47)/4*x*y^3*u-4319*y^4*u+F(54)/101*x^3*z*u+10800*x^2*y*z*u+6630*x*y^2*z*u-14729*y^3*z*u-14636*x^2*z^2*u+F(98)/87*x*y*z^2*u-F(108)/91*y^2*z^2*u-F(41)/46*x*z^3*u-F(43)/108*y*z^3*u-F(22)/83*z^4*u-F(37)/79*x^3*u^2+F(18)/89*x^2*y*u^2+7594*x*y^2*u^2-531*y^3*u^2+9220*x^2*z*u^2+F(67)/27*x*y*z*u^2+4877*y^2*z*u^2-3686*x*z^2*u^2-14441*y*z^2*u^2-F(35)/32*z^3*u^2-5293*x^2*u^3+F(118)/77*x*y*u^3+15909*y^2*u^3-1985*x*z*u^3-F(71)/32*y*z*u^3+F(71)/69*z^2*u^3+F(85)/28*x*u^4+F(43)/108*y*u^4-F(18)/119*z*u^4-F(1)/33*u^5+15910*x^4*v+2685*x^3*y*v-F(83)/115*x^2*y^2*v-2747*x*y^3*v+F(28)/103*y^4*v+240*x^3*z*v-F(83)/7*x^2*y*z*v+F(9)/95*x*y^2*z*v+15264*y^3*z*v+F(51)/28*x^2*z^2*v+F(25)/11*x*y*z^2*v+247*y^2*z^2*v+F(121)/65*x*z^3*v+F(53)/37*y*z^3*v+F(87)/14*z^4*v-F(7)/51*x^3*u*v-270*x^2*y*u*v-F(103)/100*x*y^2*u*v+13367*y^3*u*v-F(1)/105*x^2*z*u*v+6356*x*y*z*u*v-F(31)/28*y^2*z*u*v+F(113)/39*x*z^2*u*v-11765*y*z^2*u*v-F(25)/98*z^3*u*v+F(9)/118*x^2*u^2*v-F(86)/121*x*y*u^2*v+F(19)/52*y^2*u^2*v-6145*x*z*u^2*v-1748*y*z*u^2*v+F(117)/50*z^2*u^2*v+F(35)/57*x*u^3*v-F(106)/17*y*u^3*v-8491*z*u^3*v+F(96)/67*u^4*v+F(23)/33*x^3*v^2-F(23)/16*x^2*y*v^2+13479*x*y^2*v^2-9714*y^3*v^2-4106*x^2*z*v^2-F(67)/84*x*y*z*v^2+F(95)/24*y^2*z*v^2-9956*x*z^2*v^2-F(63)/100*y*z^2*v^2+F(103)/2*z^3*v^2+4590*x^2*u*v^2-11914*x*y*u*v^2-F(23)/119*y^2*u*v^2+F(11)/68*x*z*u*v^2+5511*y*z*u*v^2-F(125)/94*z^2*u*v^2-F(4)/47*x*u^2*v^2+11006*y*u^2*v^2-F(113)/44*z*u^2*v^2+F(125)/81*u^3*v^2-2335*x^2*v^3-F(107)/117*x*y*v^3-12596*y^2*v^3-F(73)/103*x*z*v^3-F(97)/40*y*z*v^3+F(8)/23*z^2*v^3+F(86)/111*x*u*v^3+13188*y*u*v^3-2123*z*u*v^3-11068*u^2*v^3+F(113)/87*x*v^4+F(111)/97*y*v^4-5307*z*v^4+13984*u*v^4+F(103)/28*v^5,-5257*x^4*y+F(109)/71*x^3*y^2-F(103)/49*x^2*y^3+F(65)/114*x*y^4+3399*y^5-F(100)/119*x^4*z+15893*x^3*y*z-F(15)/38*x^2*y^2*z-F(44)/45*x*y^3*z-F(49)/90*y^4*z-F(109)/26*x^3*z^2+F(113)/120*x^2*y*z^2-12589*x*y^2*z^2-F(71)/110*y^3*z^2-F(11)/59*x^2*z^3+F(16)/49*x*y*z^3-3959*y^2*z^3+F(47)/32*x*z^4+F(110)/27*y*z^4-F(43)/25*z^5-13093*x^4*u+F(21)/44*x^3*y*u+F(57)/4*x^2*y^2*u-F(19)/94*x*y^3*u-3357*y^4*u-F(8)/101*x^3*z*u-F(116)/59*x^2*y*z*u+4871*x*y^2*z*u+F(38)/15*y^3*z*u-13430*x^2*z^2*u-7944*x*y*z^2*u+F(96)/29*y^2*z^2*u-3318*x*z^3*u-2811*y*z^3*u+F(73)/20*z^4*u-F(14)/61*x^3*u^2-F(4)/51*x^2*y*u^2+F(46)/83*x*y^2*u^2+F(78)/35*y^3*u^2-3631*x^2*z*u^2+F(82)/17*x*y*z*u^2+F(25)/17*y^2*z*u^2-F(33)/25*x*z^2*u^2-F(106)/85*y*z^2*u^2-F(49)/66*z^3*u^2-13449*x^2*u^3+F(23)/35*x*y*u^3+14197*y^2*u^3-F(43)/53*x*z*u^3+F(107)/48*y*z*u^3+F(9)/73*z^2*u^3+11229*x*u^4+F(89)/26*y*u^4-495*z*u^4-7396*u^5+F(13)/28*x^4*v+F(19)/126*x^3*y*v-10720*x^2*y^2*v-11809*x*y^3*v-6077*y^4*v+F(18)/101*x^3*z*v+F(79)/99*x^2*y*z*v+14468*x*y^2*z*v-F(37)/125*y^3*z*v+F(122)/119*x^2*z^2*v+5275*x*y*z^2*v-F(41)/86*y^2*z^2*v+8966*x*z^3*v-F(113)/111*y*z^3*v+F(13)/31*z^4*v+F(76)/9*x^3*u*v+F(6)/31*x^2*y*u*v-F(33)/8*x*y^2*u*v+4519*y^3*u*v-F(22)/3*x^2*z*u*v-F(73)/24*x*y*z*u*v-F(7)/78*y^2*z*u*v+F(71)/102*x*z^2*u*v+F(7)/68*y*z^2*u*v+F(76)/125*z^3*u*v+8988*x^2*u^2*v+F(23)/16*x*y*u^2*v+10247*y^2*u^2*v+F(51)/8*x*z*u^2*v-14866*y*z*u^2*v+F(38)/63*z^2*u^2*v-11215*x*u^3*v+1737*y*u^3*v+F(116)/83*z*u^3*v+11650*u^4*v-F(98)/111*x^3*v^2+F(22)/91*x^2*y*v^2+F(21)/97*x*y^2*v^2+F(64)/81*y^3*v^2+7059*x^2*z*v^2+5784*x*y*z*v^2-7855*y^2*z*v^2+F(120)/37*x*z^2*v^2+F(3)/112*y*z^2*v^2-5261*z^3*v^2+15891*x^2*u*v^2+14459*x*y*u*v^2-F(97)/27*y^2*u*v^2+F(103)/13*x*z*u*v^2+7034*y*z*u*v^2+F(71)/79*z^2*u*v^2+F(33)/13*x*u^2*v^2-F(52)/15*y*u^2*v^2+F(31)/28*z*u^2*v^2+5804*u^3*v^2+F(112)/59*x^2*v^3-1160*x*y*v^3-F(87)/79*y^2*v^3-10509*x*z*v^3+F(94)/121*y*z*v^3-F(34)/73*z^2*v^3+F(8)/123*x*u*v^3+F(36)/115*y*u*v^3-10980*z*u*v^3+F(40)/17*u^2*v^3-4905*x*v^4+F(3)/64*y*v^4+F(97)/47*z*v^4+5527*u*v^4-F(6)/109*v^5,5257*x^4*z-F(109)/71*x^3*y*z+F(103)/49*x^2*y^2*z-F(65)/114*x*y^3*z-3399*y^4*z-F(1)/69*x^3*z^2-55*x^2*y*z^2+8077*x*y^2*z^2+15003*y^3*z^2-9369*x^2*z^3-F(79)/88*x*y*z^3-F(70)/19*y^2*z^3+F(99)/73*x*z^4+8074*y*z^4+F(32)/55*z^5+F(17)/9*x^4*u-F(120)/113*x^3*y*u+F(82)/109*x^2*y^2*u+1005*x*y^3*u-7410*y^4*u-14671*x^3*z*u-F(99)/118*x^2*y*z*u-13641*x*y^2*z*u-15498*y^3*z*u+15189*x^2*z^2*u+F(89)/8*x*y*z^2*u+15644*y^2*z^2*u-4543*x*z^3*u+2118*y*z^3*u+2008*z^4*u+957*x^3*u^2+7958*x^2*y*u^2+F(106)/69*x*y^2*u^2-3420*y^3*u^2-F(16)/49*x^2*z*u^2+F(93)/28*x*y*z*u^2+5399*y^2*z*u^2+4768*x*z^2*u^2-F(85)/12*y*z^2*u^2+F(86)/15*z^3*u^2-9431*x^2*u^3-F(101)/39*x*y*u^3-13927*y^2*u^3+F(125)/4*x*z*u^3+F(99)/97*y*z*u^3+9745*z^2*u^3+F(94)/17*x*u^4+F(69)/16*y*u^4+7814*z*u^4+F(28)/5*u^5+4749*x^4*v-F(16)/55*x^3*y*v-F(15)/34*x^2*y^2*v+7469*x*y^3*v-F(19)/110*y^4*v-F(39)/10*x^3*z*v-F(115)/38*x^2*y*z*v-F(117)/22*x*y^2*z*v-4904*y^3*z*v-F(65)/112*x^2*z^2*v-13740*x*y*z^2*v-7945*y^2*z^2*v-9515*x*z^3*v+F(31)/67*y*z^3*v+F(47)/70*z^4*v-15246*x^3*u*v-F(88)/35*x^2*y*u*v-11510*x*y^2*u*v-F(70)/43*y^3*u*v-F(103)/40*x^2*z*u*v-F(103)/124*x*y*z*u*v-12286*y^2*z*u*v-3463*x*z^2*u*v-5364*y*z^2*u*v-F(67)/49*z^3*u*v+12984*x^2*u^2*v+4670*x*y*u^2*v-9422*y^2*u^2*v+F(115)/63*x*z*u^2*v+F(57)/5*y*z*u^2*v+8282*z^2*u^2*v+F(31)/98*x*u^3*v-F(9)/88*y*u^3*v+10948*z*u^3*v+F(66)/41*u^4*v+F(31)/105*x^3*v^2-F(7)/13*x^2*y*v^2-F(9)/58*x*y^2*v^2+F(58)/55*y^3*v^2+F(111)/85*x^2*z*v^2+11111*x*y*z*v^2+F(14)/23*y^2*z*v^2+F(23)/112*x*z^2*v^2-F(101)/125*y*z^2*v^2+F(43)/45*z^3*v^2-F(5)/99*x^2*u*v^2+F(112)/125*x*y*u*v^2+F(77)/69*y^2*u*v^2+F(88)/111*x*z*u*v^2+15191*y*z*u*v^2+F(27)/71*z^2*u*v^2+F(102)/109*x*u^2*v^2+F(47)/93*y*u^2*v^2-F(47)/52*z*u^2*v^2-F(90)/89*u^3*v^2+F(117)/125*x^2*v^3+F(63)/107*x*y*v^3-11080*y^2*v^3+5653*x*z*v^3+F(73)/35*y*z*v^3+8548*z^2*v^3-5399*x*u*v^3-F(85)/43*y*u*v^3+F(2)/123*z*u*v^3-F(8)/19*u^2*v^3+F(47)/91*x*v^4-4419*y*v^4-F(88)/73*z*v^4+13686*u*v^4+F(42)/71*v^5,-F(59)/67*x^5-5238*x^4*y+13448*x^3*y^2-5749*x^2*y^3+11756*x*y^4-F(14)/61*x^4*z-1239*x^3*y*z-6712*x^2*y^2*z-15912*x*y^3*z-15567*y^4*z-10441*x^3*z^2+685*x^2*y*z^2+F(78)/71*x*y^2*z^2+9069*y^3*z^2-3221*x^2*z^3-4143*x*y*z^3+F(68)/5*y^2*z^3+9856*x*z^4-F(95)/119*y*z^4+F(94)/111*z^5+F(56)/11*x^4*u-2443*x^3*y*u+15516*x^2*y^2*u-1340*x*y^3*u+F(41)/116*y^4*u-F(7)/58*x^3*z*u+F(15)/113*x^2*y*z*u+10851*x*y^2*z*u-15429*y^3*z*u+F(51)/89*x^2*z^2*u+F(3)/8*x*y*z^2*u+884*y^2*z^2*u-F(124)/125*x*z^3*u-14751*y*z^3*u+F(80)/11*z^4*u+F(19)/77*x^3*u^2-F(49)/71*x^2*y*u^2+F(7)/71*x*y^2*u^2+F(53)/82*y^3*u^2-14239*x^2*z*u^2+F(125)/54*x*y*z*u^2+F(112)/107*y^2*z*u^2+2991*x*z^2*u^2-15273*y*z^2*u^2+F(95)/41*z^3*u^2+F(40)/93*x^2*u^3+2542*x*y*u^3-F(119)/2*y^2*u^3-F(7)/88*x*z*u^3+F(22)/87*y*z*u^3+F(125)/102*z^2*u^3+14694*x*u^4+F(92)/25*y*u^4+6009*z*u^4+12539*u^5-6438*x^4*v+2926*x^3*y*v-F(49)/40*x^2*y^2*v+11262*x*y^3*v-F(63)/86*y^4*v-F(97)/123*x^3*z*v-F(16)/59*x^2*y*z*v-F(9)/100*x*y^2*z*v-4790*y^3*z*v-11700*x^2*z^2*v-F(77)/90*x*y*z^2*v+F(31)/47*y^2*z^2*v+F(38)/105*x*z^3*v-8742*y*z^3*v+12022*z^4*v-1624*x^3*u*v-F(99)/122*x^2*y*u*v-F(17)/13*x*y^2*u*v-4105*y^3*u*v+9471*x^2*z*u*v+1474*x*y*z*u*v-11908*y^2*z*u*v-F(102)/103*x*z^2*u*v-F(47)/64*y*z^2*u*v+F(99)/29*z^3*u*v+F(46)/89*x^2*u^2*v+F(110)/71*x*y*u^2*v+539*y^2*u^2*v-F(35)/69*x*z*u^2*v-7763*y*z*u^2*v+F(113)/83*z^2*u^2*v+F(109)/10*x*u^3*v+12142*y*u^3*v+F(31)/72*z*u^3*v+10495*u^4*v-F(62)/79*x^3*v^2-F(83)/45*x^2*y*v^2-F(97)/111*x*y^2*v^2+F(122)/81*y^3*v^2+F(79)/27*x^2*z*v^2-9730*x*y*z*v^2-11445*y^2*z*v^2-2869*x*z^2*v^2+F(120)/113*y*z^2*v^2-F(98)/17*z^3*v^2-2443*x^2*u*v^2+2390*x*y*u*v^2+F(15)/83*y^2*u*v^2-F(62)/35*x*z*u*v^2-F(45)/98*y*z*u*v^2+F(105)/86*z^2*u*v^2+F(46)/25*x*u^2*v^2-F(38)/73*y*u^2*v^2+3407*z*u^2*v^2-F(30)/109*u^3*v^2+F(85)/54*x^2*v^3-4151*x*y*v^3-F(61)/24*y^2*v^3+13375*x*z*v^3-F(41)/8*y*z*v^3+11910*z^2*v^3-F(113)/114*x*u*v^3+F(25)/74*y*u*v^3+11588*z*u*v^3-F(10)/43*u^2*v^3-9061*x*v^4-15074*y*v^4-5309*z*v^4-7174*u*v^4+F(95)/26*v^5,F(44)/15*x^4*y-F(57)/107*x^3*y^2-11794*x^2*y^3+F(59)/54*x*y^4+F(124)/39*y^5-F(17)/122*x^4*z-F(92)/123*x^3*y*z-F(55)/46*x^2*y^2*z+F(97)/20*x*y^3*z-F(77)/97*y^4*z-6865*x^3*z^2-15264*x^2*y*z^2-F(53)/79*x*y^2*z^2-F(80)/23*y^3*z^2+F(118)/15*x^2*z^3-1291*x*y*z^3+2143*y^2*z^3+F(102)/19*x*z^4+F(31)/12*y*z^4+F(73)/60*z^5-F(75)/4*x^4*u-F(25)/87*x^3*y*u+F(103)/107*x^2*y^2*u+6828*x*y^3*u+6747*y^4*u-F(51)/95*x^3*z*u-10620*x^2*y*z*u+F(83)/65*x*y^2*z*u-F(105)/68*y^3*z*u-6868*x^2*z^2*u-F(31)/118*x*y*z^2*u+F(7)/78*y^2*z^2*u+9892*x*z^3*u+3233*y*z^3*u-F(12)/61*z^4*u-9422*x^3*u^2-F(26)/83*x^2*y*u^2+F(99)/46*x*y^2*u^2-1097*y^3*u^2-460*x^2*z*u^2-8198*x*y*z*u^2-F(47)/118*y^2*z*u^2+F(29)/114*x*z^2*u^2-F(8)/51*y*z^2*u^2-350*z^3*u^2-10411*x^2*u^3+F(77)/13*x*y*u^3+F(72)/55*y^2*u^3+F(20)/93*x*z*u^3-504*y*z*u^3-F(67)/22*z^2*u^3-F(91)/76*x*u^4-F(67)/105*y*u^4-F(76)/79*z*u^4+4851*u^5+F(30)/53*x^4*v+F(101)/61*x^3*y*v+F(68)/25*x^2*y^2*v-F(4)/11*x*y^3*v-5501*y^4*v-F(77)/52*x^3*z*v-F(125)/111*x^2*y*z*v+F(23)/38*x*y^2*z*v-1833*y^3*z*v-12478*x^2*z^2*v+7920*x*y*z^2*v+1955*y^2*z^2*v-F(109)/113*x*z^3*v+740*y*z^3*v+F(67)/52*z^4*v+F(73)/109*x^3*u*v-F(47)/62*x^2*y*u*v+9494*x*y^2*u*v-8056*y^3*u*v+F(55)/104*x^2*z*u*v+8672*x*y*z*u*v+7607*y^2*z*u*v-13559*x*z^2*u*v+F(95)/24*y*z^2*u*v-F(8)/55*z^3*u*v+F(31)/99*x^2*u^2*v-F(47)/11*x*y*u^2*v-11401*y^2*u^2*v-8335*x*z*u^2*v-4220*y*z*u^2*v-F(69)/19*z^2*u^2*v-F(92)/77*x*u^3*v-F(54)/35*y*u^3*v+F(113)/65*z*u^3*v+F(63)/25*u^4*v-5292*x^3*v^2+F(49)/79*x^2*y*v^2-F(97)/19*x*y^2*v^2-F(24)/53*y^3*v^2+F(53)/78*x^2*z*v^2-F(25)/102*x*y*z*v^2-13346*y^2*z*v^2-2972*x*z^2*v^2-F(23)/125*y*z^2*v^2-10818*z^3*v^2-F(71)/26*x^2*u*v^2-F(65)/17*x*y*u*v^2-F(58)/61*y^2*u*v^2-F(56)/19*x*z*u*v^2-F(39)/98*y*z*u*v^2+F(118)/117*z^2*u*v^2-F(45)/14*x*u^2*v^2-F(109)/119*y*u^2*v^2-12888*z*u^2*v^2-F(64)/45*u^3*v^2-F(46)/25*x^2*v^3-F(109)/21*x*y*v^3-F(50)/69*y^2*v^3-F(86)/115*x*z*v^3-1630*y*z*v^3+2649*z^2*v^3+F(86)/51*x*u*v^3+7052*y*u*v^3-F(88)/67*z*u*v^3-F(17)/126*u^2*v^3+F(25)/38*x*v^4+F(58)/79*y*v^4+F(77)/86*z*v^4+F(92)/89*u*v^4+F(122)/69*v^5,-F(85)/47*x^4*z+6624*x^3*y*z+F(109)/117*x^2*y^2*z+5247*x*y^3*z-F(23)/57*y^4*z-13577*x^3*z^2-F(78)/53*x^2*y*z^2+F(74)/33*x*y^2*z^2+F(18)/55*y^3*z^2-F(125)/92*x^2*z^3+6962*x*y*z^3-F(85)/6*y^2*z^3+F(76)/47*x*z^4-F(33)/124*y*z^4-15721*z^5+F(19)/3*x^4*u-5437*x^3*y*u-F(85)/113*x^2*y^2*u+10248*x*y^3*u+7815*y^4*u-1025*x^3*z*u-F(97)/86*x^2*y*z*u+F(22)/57*x*y^2*z*u+F(56)/125*y^3*z*u+F(43)/84*x^2*z^2*u-7571*x*y*z^2*u+F(51)/53*y^2*z^2*u-F(16)/51*x*z^3*u-F(26)/35*y*z^3*u+2776*z^4*u-F(121)/51*x^3*u^2-F(37)/107*x^2*y*u^2-F(41)/49*x*y^2*u^2+10133*y^3*u^2+F(27)/97*x^2*z*u^2-7126*x*y*z*u^2+4300*y^2*z*u^2+F(29)/52*x*z^2*u^2-F(31)/5*y*z^2*u^2-4551*z^3*u^2-F(122)/79*x^2*u^3+3941*x*y*u^3+1261*y^2*u^3+F(104)/57*x*z*u^3+F(57)/22*y*z*u^3+2079*z^2*u^3-15522*x*u^4-206*y*u^4-F(73)/32*z*u^4+11765*u^5-490*x^4*v+F(116)/117*x^3*y*v-F(15)/2*x^2*y^2*v-F(97)/96*x*y^3*v-F(55)/122*y^4*v-F(90)/97*x^3*z*v-13010*x^2*y*z*v+F(117)/116*x*y^2*z*v-9666*y^3*z*v+F(77)/108*x^2*z^2*v-15730*x*y*z^2*v+10153*y^2*z^2*v-F(7)/19*x*z^3*v-6333*y*z^3*v-549*z^4*v-9757*x^3*u*v-F(119)/32*x^2*y*u*v+F(116)/89*x*y^2*u*v-F(122)/51*y^3*u*v+10746*x^2*z*u*v+11360*x*y*z*u*v+F(69)/68*y^2*z*u*v-F(32)/85*x*z^2*u*v-F(32)/33*y*z^2*u*v-F(15)/124*z^3*u*v-F(35)/88*x^2*u^2*v-F(83)/6*x*y*u^2*v+11769*y^2*u^2*v+F(117)/35*x*z*u^2*v-F(61)/110*y*z*u^2*v-F(37)/106*z^2*u^2*v+F(75)/121*x*u^3*v+9500*y*u^3*v+F(43)/85*z*u^3*v-10740*u^4*v-14321*x^3*v^2-F(13)/28*x^2*y*v^2+F(55)/67*x*y^2*v^2+F(13)/88*y^3*v^2+7130*x^2*z*v^2+15064*x*y*z*v^2-F(79)/43*y^2*z*v^2-F(79)/8*x*z^2*v^2-F(6)/79*y*z^2*v^2+11327*z^3*v^2-F(59)/71*x^2*u*v^2-F(118)/17*x*y*u*v^2+F(2)/71*y^2*u*v^2+14194*x*z*u*v^2-10402*y*z*u*v^2-F(117)/43*z^2*u*v^2+11940*x*u^2*v^2-178*y*u^2*v^2+F(33)/29*z*u^2*v^2+F(29)/38*u^3*v^2-F(85)/116*x^2*v^3+9621*x*y*v^3+12467*y^2*v^3+5547*x*z*v^3+15129*y*z*v^3-5113*z^2*v^3-F(60)/77*x*u*v^3+F(104)/119*y*u*v^3-F(35)/72*z*u*v^3-10454*u^2*v^3-F(45)/83*x*v^4-F(62)/101*y*v^4-F(67)/78*z*v^4-F(47)/42*u*v^4-F(28)/25*v^5,-F(27)/118*x^5+4753*x^4*y-F(99)/53*x^3*y^2+14193*x^2*y^3-F(43)/126*x*y^4+3379*x^4*z+15140*x^3*y*z-3100*x^2*y^2*z+6269*x*y^3*z+F(17)/64*y^4*z+F(97)/110*x^3*z^2-F(26)/41*x^2*y*z^2-F(109)/73*x*y^2*z^2+F(112)/25*y^3*z^2+F(79)/87*x^2*z^3-F(11)/101*x*y*z^3-F(92)/29*y^2*z^3-F(35)/101*x*z^4+F(107)/19*y*z^4+F(49)/95*z^5-F(114)/97*x^4*u+F(123)/52*x^3*y*u-7843*x^2*y^2*u-F(85)/44*x*y^3*u+F(17)/38*y^4*u+F(68)/87*x^3*z*u+F(94)/101*x^2*y*z*u+14720*x*y^2*z*u-13205*y^3*z*u-F(95)/58*x^2*z^2*u+F(43)/4*x*y*z^2*u+F(93)/58*y^2*z^2*u-F(57)/35*x*z^3*u+F(109)/66*y*z^3*u-3453*z^4*u+F(41)/125*x^3*u^2+F(34)/107*x^2*y*u^2-10830*x*y^2*u^2-F(91)/3*y^3*u^2-F(65)/47*x^2*z*u^2+F(73)/8*x*y*z*u^2+9224*y^2*z*u^2+F(78)/79*x*z^2*u^2-F(34)/75*y*z^2*u^2+F(106)/7*z^3*u^2-F(111)/22*x^2*u^3+F(111)/19*x*y*u^3+6747*y^2*u^3-3102*x*z*u^3+13164*y*z*u^3-F(87)/124*z^2*u^3-6441*x*u^4-9119*y*u^4-F(34)/41*z*u^4-F(75)/49*u^5+F(73)/53*x^4*v-F(103)/54*x^3*y*v+F(120)/13*x^2*y^2*v+F(32)/39*x*y^3*v-4179*y^4*v+2715*x^3*z*v-F(11)/7*x^2*y*z*v+12929*x*y^2*z*v+9622*y^3*z*v-F(50)/103*x^2*z^2*v+15437*x*y*z^2*v-F(43)/61*y^2*z^2*v+1736*x*z^3*v+F(31)/125*y*z^3*v-15460*z^4*v-F(106)/33*x^3*u*v-F(105)/118*x^2*y*u*v-15790*x*y^2*u*v+F(17)/92*y^3*u*v+F(10)/67*x^2*z*u*v-15465*x*y*z*u*v-F(41)/69*y^2*z*u*v+F(26)/5*x*z^2*u*v+F(91)/24*y*z^2*u*v-F(39)/2*z^3*u*v-F(107)/106*x^2*u^2*v+F(115)/114*x*y*u^2*v-7506*y^2*u^2*v+3184*x*z*u^2*v-F(38)/5*y*z*u^2*v+10771*z^2*u^2*v+8354*x*u^3*v-4911*y*u^3*v-14299*z*u^3*v+F(58)/101*u^4*v-F(71)/16*x^3*v^2-F(119)/11*x^2*y*v^2-7472*x*y^2*v^2+2495*y^3*v^2-F(19)/97*x^2*z*v^2-5354*x*y*z*v^2+13607*y^2*z*v^2-F(63)/16*x*z^2*v^2+F(39)/38*y*z^2*v^2-F(119)/81*z^3*v^2+F(22)/59*x^2*u*v^2-14424*x*y*u*v^2-982*y^2*u*v^2-F(1)/78*x*z*u*v^2-12314*y*z*u*v^2+9112*z^2*u*v^2-F(64)/107*x*u^2*v^2-F(49)/32*y*u^2*v^2+F(21)/43*z*u^2*v^2-F(42)/101*u^3*v^2-F(103)/81*x^2*v^3+5034*x*y*v^3-F(93)/22*y^2*v^3-F(46)/11*x*z*v^3+F(125)/57*y*z*v^3+F(67)/2*z^2*v^3+F(43)/35*x*u*v^3-F(55)/79*y*u*v^3-11452*z*u*v^3+F(49)/3*u^2*v^3+6216*x*v^4-12883*y*v^4-F(93)/5*z*v^4-F(117)/115*u*v^4+F(78)/43*v^5,10711*x^4*y-F(25)/72*x^3*y^2-F(89)/121*x^2*y^3+11441*x*y^4+4341*y^5-F(47)/48*x^4*z+F(29)/42*x^3*y*z+F(25)/83*x^2*y^2*z-15103*x*y^3*z-F(42)/11*y^4*z+1323*x^3*z^2-833*x^2*y*z^2-13042*x*y^2*z^2+7262*y^3*z^2+6008*x^2*z^3-F(71)/121*x*y*z^3-1608*y^2*z^3-882*x*z^4+10401*y*z^4-F(107)/96*z^5+F(59)/32*x^4*u+F(69)/59*x^3*y*u+F(53)/74*x^2*y^2*u-F(91)/89*x*y^3*u-12542*y^4*u-F(111)/8*x^3*z*u+F(115)/6*x^2*y*z*u+2859*x*y^2*z*u+F(29)/57*y^3*z*u+F(114)/37*x^2*z^2*u+F(107)/60*x*y*z^2*u+F(53)/71*y^2*z^2*u-F(79)/87*x*z^3*u-F(40)/89*y*z^3*u-F(58)/21*z^4*u-4717*x^3*u^2-F(85)/19*x^2*y*u^2+F(121)/101*x*y^2*u^2-F(120)/71*y^3*u^2-F(13)/86*x^2*z*u^2-F(97)/108*x*y*z*u^2+5705*y^2*z*u^2+10379*x*z^2*u^2+F(73)/22*y*z^2*u^2-14553*z^3*u^2+F(82)/121*x^2*u^3+722*x*y*u^3-F(29)/124*y^2*u^3+15876*x*z*u^3-10173*y*z*u^3+F(28)/95*z^2*u^3+3724*x*u^4-F(61)/4*y*u^4-1479*z*u^4-F(70)/89*u^5+F(71)/89*x^4*v+12059*x^3*y*v+F(71)/19*x^2*y^2*v+1398*x*y^3*v-F(22)/19*y^4*v+F(121)/16*x^3*z*v+F(34)/45*x^2*y*z*v+11370*x*y^2*z*v+F(51)/50*y^3*z*v-9109*x^2*z^2*v+11534*x*y*z^2*v+4493*y^2*z^2*v-14346*x*z^3*v-F(101)/89*y*z^3*v+14186*z^4*v+14235*x^3*u*v-F(34)/33*x^2*y*u*v-F(46)/99*x*y^2*u*v+F(84)/59*y^3*u*v-750*x^2*z*u*v+F(101)/12*x*y*z*u*v-750*y^2*z*u*v-11102*x*z^2*u*v+5398*y*z^2*u*v-F(23)/76*z^3*u*v-9117*x^2*u^2*v+F(16)/105*x*y*u^2*v-F(93)/68*y^2*u^2*v-3125*x*z*u^2*v+F(107)/9*y*z*u^2*v-8515*z^2*u^2*v+F(110)/17*x*u^3*v-1555*y*u^3*v+F(107)/38*z*u^3*v-F(49)/19*u^4*v-F(48)/59*x^3*v^2-F(91)/23*x^2*y*v^2+F(103)/109*x*y^2*v^2-F(12)/71*y^3*v^2-F(43)/36*x^2*z*v^2-F(99)/20*x*y*z*v^2-5467*y^2*z*v^2+15580*x*z^2*v^2+F(22)/17*y*z^2*v^2+F(97)/115*z^3*v^2+F(4)/13*x^2*u*v^2+14694*x*y*u*v^2+F(22)/37*y^2*u*v^2+8660*x*z*u*v^2+F(92)/25*y*z*u*v^2-F(113)/120*z^2*u*v^2+F(121)/120*x*u^2*v^2-F(21)/53*y*u^2*v^2-12770*z*u^2*v^2+F(115)/9*u^3*v^2+F(82)/67*x^2*v^3-12480*x*y*v^3-F(110)/37*y^2*v^3+F(125)/33*x*z*v^3+F(72)/23*y*z*v^3+612*z^2*v^3+3857*x*u*v^3+F(75)/29*y*u*v^3-F(115)/117*z*u*v^3+10092*u^2*v^3-171*x*v^4-7279*y*v^4-F(50)/37*z*v^4+F(77)/125*u*v^4-F(53)/17*v^5,F(116)/73*x^4*y-10586*x^3*y^2-11252*x^2*y^3-F(37)/126*x*y^4-4039*y^5+F(3)/95*x^4*z+1557*x^3*y*z-9616*x^2*y^2*z-F(16)/13*x*y^3*z+2522*y^4*z-F(121)/46*x^3*z^2+F(94)/49*x^2*y*z^2+F(116)/85*x*y^2*z^2+145*y^3*z^2+F(67)/90*x^2*z^3+147*x*y*z^3-F(14)/15*y^2*z^3+F(4)/51*x*z^4-3630*y*z^4-5354*z^5+F(109)/69*x^4*u+F(125)/41*x^3*y*u-1381*x^2*y^2*u+F(113)/44*x*y^3*u-F(17)/12*y^4*u+12343*x^3*z*u+F(12)/23*x^2*y*z*u+10266*x*y^2*z*u+14890*y^3*z*u+10450*x^2*z^2*u+3013*x*y*z^2*u-8172*y^2*z^2*u-F(1)/4*x*z^3*u+F(67)/38*y*z^3*u-8492*z^4*u-F(68)/9*x^3*u^2+F(52)/63*x^2*y*u^2+F(90)/23*x*y^2*u^2+F(54)/37*y^3*u^2+F(123)/35*x^2*z*u^2+10611*x*y*z*u^2-6425*y^2*z*u^2-F(32)/39*x*z^2*u^2+F(49)/66*y*z^2*u^2-6122*z^3*u^2+14406*x^2*u^3+F(86)/27*x*y*u^3+F(77)/93*y^2*u^3+2682*x*z*u^3-F(73)/91*y*z*u^3+F(21)/41*z^2*u^3+3749*x*u^4-F(61)/73*y*u^4-F(105)/106*z*u^4-F(2)/45*u^5-F(17)/18*x^4*v-F(84)/125*x^3*y*v+F(33)/25*x^2*y^2*v+12412*x*y^3*v+7167*y^4*v-F(73)/21*x^3*z*v-F(111)/8*x^2*y*z*v+F(37)/92*x*y^2*z*v-14915*y^3*z*v-F(57)/47*x^2*z^2*v-8084*x*y*z^2*v+9868*y^2*z^2*v-1095*x*z^3*v+F(49)/3*y*z^3*v-F(51)/97*z^4*v+12644*x^3*u*v-F(83)/5*x^2*y*u*v+F(109)/55*x*y^2*u*v-F(108)/107*y^3*u*v-F(88)/49*x^2*z*u*v+10881*x*y*z*u*v+675*y^2*z*u*v+7861*x*z^2*u*v+F(57)/13*y*z^2*u*v+7277*z^3*u*v+F(4)/35*x^2*u^2*v+14375*x*y*u^2*v+3786*y^2*u^2*v+F(17)/107*x*z*u^2*v-F(13)/92*y*z*u^2*v+6944*z^2*u^2*v-3830*x*u^3*v+6800*y*u^3*v-3983*z*u^3*v-F(69)/10*u^4*v+F(37)/35*x^3*v^2-13164*x^2*y*v^2+12389*x*y^2*v^2-F(43)/14*y^3*v^2+15034*x^2*z*v^2-3102*x*y*z*v^2+3756*y^2*z*v^2-F(89)/54*x*z^2*v^2-4165*y*z^2*v^2-13939*z^3*v^2-F(53)/121*x^2*u*v^2+F(97)/115*x*y*u*v^2+F(53)/10*y^2*u*v^2+11055*x*z*u*v^2-6599*y*z*u*v^2+F(44)/113*z^2*u*v^2-1592*x*u^2*v^2+F(31)/51*y*u^2*v^2-F(113)/31*z*u^2*v^2-F(67)/62*u^3*v^2-F(29)/6*x^2*v^3-F(19)/75*x*y*v^3-F(12)/115*y^2*v^3-3906*x*z*v^3+F(109)/51*y*z*v^3-9276*z^2*v^3-F(115)/81*x*u*v^3-2418*y*u*v^3+15917*z*u*v^3-F(21)/52*u^2*v^3-8406*x*v^4+F(17)/115*y*v^4-3433*z*v^4+F(49)/31*u*v^4+1006*v^5,-F(21)/122*x^5-F(103)/24*x^4*y-6671*x^3*y^2+F(61)/80*x^2*y^3-F(13)/125*x*y^4+F(94)/83*x^4*z+3908*x^3*y*z-F(67)/16*x^2*y^2*z+217*x*y^3*z+F(55)/91*y^4*z+502*x^3*z^2-F(62)/87*x^2*y*z^2-8672*x*y^2*z^2-12160*y^3*z^2+11457*x^2*z^3-12597*x*y*z^3-F(7)/10*y^2*z^3+F(124)/65*x*z^4+91*y*z^4+15818*z^5-8353*x^4*u+6592*x^3*y*u-F(17)/59*x^2*y^2*u-F(35)/69*x*y^3*u-F(67)/44*y^4*u+13221*x^3*z*u-F(19)/97*x^2*y*z*u+F(118)/35*x*y^2*z*u-10552*y^3*z*u+F(42)/13*x^2*z^2*u-F(17)/39*x*y*z^2*u+F(42)/65*y^2*z^2*u-3270*x*z^3*u-F(13)/49*y*z^3*u+F(65)/41*z^4*u+4069*x^3*u^2+3971*x^2*y*u^2+F(56)/97*x*y^2*u^2-14289*y^3*u^2+5904*x^2*z*u^2-1694*x*y*z*u^2+2091*y^2*z*u^2+F(97)/70*x*z^2*u^2+3575*y*z^2*u^2+F(31)/125*z^3*u^2+233*x^2*u^3+F(72)/55*x*y*u^3-F(20)/77*y^2*u^3-13221*x*z*u^3+8897*y*z*u^3+F(41)/69*z^2*u^3-F(38)/71*x*u^4+F(16)/25*y*u^4+F(53)/114*z*u^4+F(27)/56*u^5-F(33)/125*x^4*v+F(11)/82*x^3*y*v+5636*x^2*y^2*v-F(8)/3*x*y^3*v+15306*y^4*v-F(95)/16*x^3*z*v+F(7)/8*x^2*y*z*v+F(13)/106*x*y^2*z*v-1150*y^3*z*v-2228*x^2*z^2*v-F(115)/87*x*y*z^2*v+F(102)/71*y^2*z^2*v-F(74)/15*x*z^3*v-F(107)/93*y*z^3*v-12141*z^4*v+8383*x^3*u*v+1176*x^2*y*u*v+F(6)/7*x*y^2*u*v-8108*y^3*u*v-15506*x^2*z*u*v-F(12)/41*x*y*z*u*v+15244*y^2*z*u*v-6263*x*z^2*u*v+7698*y*z^2*u*v+7006*z^3*u*v-F(67)/100*x^2*u^2*v+F(97)/10*x*y*u^2*v-45*y^2*u^2*v+F(31)/35*x*z*u^2*v-F(46)/53*y*z*u^2*v-F(15)/38*z^2*u^2*v-F(28)/61*x*u^3*v+2194*y*u^3*v-13840*z*u^3*v+5903*u^4*v-10343*x^3*v^2-14164*x^2*y*v^2-13348*x*y^2*v^2-6065*y^3*v^2+2755*x^2*z*v^2-F(97)/57*x*y*z*v^2-3329*y^2*z*v^2-2840*x*z^2*v^2-F(24)/89*y*z^2*v^2-9035*z^3*v^2+11842*x^2*u*v^2-F(97)/89*x*y*u*v^2-F(70)/117*y^2*u*v^2-15893*x*z*u*v^2+F(97)/28*y*z*u*v^2+14820*z^2*u*v^2-F(69)/31*x*u^2*v^2-F(4)/91*y*u^2*v^2-5145*z*u^2*v^2+11416*u^3*v^2+1126*x^2*v^3+F(119)/50*x*y*v^3+F(39)/5*y^2*v^3-F(42)/83*x*z*v^3-6214*y*z*v^3-8652*z^2*v^3-3158*x*u*v^3-4229*y*u*v^3-8547*z*u*v^3+F(120)/119*u^2*v^3-15142*x*v^4+F(88)/69*y*v^4+10514*z*v^4+F(47)/58*u*v^4-F(27)/89*v^5,6520*x^4*y+F(18)/97*x^3*y^2+F(96)/119*x^2*y^3+F(19)/11*x*y^4+5522*y^5-F(42)/83*x^4*z+7572*x^3*y*z-9646*x^2*y^2*z+15427*x*y^3*z+F(113)/38*y^4*z+6728*x^3*z^2+F(8)/53*x^2*y*z^2+2519*x*y^2*z^2-10950*y^3*z^2+3303*x^2*z^3-F(21)/5*x*y*z^3-10094*y^2*z^3+5182*x*z^4-10581*y*z^4-F(25)/103*z^5+229*x^4*u+F(21)/65*x^3*y*u+F(43)/26*x^2*y^2*u+11694*x*y^3*u+F(62)/67*y^4*u-F(94)/21*x^3*z*u-F(99)/23*x^2*y*z*u+3822*x*y^2*z*u-F(119)/57*y^3*z*u-F(83)/77*x^2*z^2*u-F(9)/110*x*y*z^2*u-F(91)/120*y^2*z^2*u+F(2)/85*x*z^3*u-F(104)/35*y*z^3*u+F(16)/89*z^4*u-F(71)/53*x^3*u^2+F(109)/14*x^2*y*u^2+F(110)/13*x*y^2*u^2+F(24)/109*y^3*u^2+F(106)/57*x^2*z*u^2-F(19)/91*x*y*z*u^2+11510*y^2*z*u^2-F(109)/42*x*z^2*u^2-F(42)/121*y*z^2*u^2-F(65)/11*z^3*u^2+F(117)/20*x^2*u^3-F(33)/70*x*y*u^3+F(122)/63*y^2*u^3-F(25)/96*x*z*u^3-F(77)/122*y*z*u^3-F(115)/34*z^2*u^3+14944*x*u^4+F(61)/34*y*u^4-F(9)/116*z*u^4-F(58)/113*u^5+F(74)/63*x^4*v-F(87)/53*x^3*y*v+5632*x^2*y^2*v+F(39)/101*x*y^3*v-10006*y^4*v-894*x^3*z*v-F(53)/75*x^2*y*z*v-F(33)/82*x*y^2*z*v-5250*y^3*z*v-F(23)/30*x^2*z^2*v-F(88)/81*x*y*z^2*v+F(13)/44*y^2*z^2*v-F(59)/104*x*z^3*v+14102*y*z^3*v-F(13)/7*z^4*v+F(125)/43*x^3*u*v+14316*x^2*y*u*v+10313*x*y^2*u*v-F(108)/11*y^3*u*v+7335*x^2*z*u*v+F(42)/109*x*y*z*u*v-8364*y^2*z*u*v+F(30)/103*x*z^2*u*v+13761*y*z^2*u*v-F(16)/89*z^3*u*v+6161*x^2*u^2*v+F(11)/101*x*y*u^2*v-62*y^2*u^2*v-F(64)/73*x*z*u^2*v+1932*y*z*u^2*v+4543*z^2*u^2*v-F(5)/69*x*u^3*v-4087*y*u^3*v+5793*z*u^3*v-15019*u^4*v-F(103)/71*x^3*v^2+661*x^2*y*v^2-10214*x*y^2*v^2-684*y^3*v^2+F(31)/19*x^2*z*v^2+F(52)/41*x*y*z*v^2-F(25)/26*y^2*z*v^2-1511*x*z^2*v^2+F(51)/62*y*z^2*v^2+2577*z^3*v^2-F(43)/120*x^2*u*v^2-F(1)/99*x*y*u*v^2-F(17)/71*y^2*u*v^2+F(54)/71*x*z*u*v^2+10805*y*z*u*v^2+F(29)/49*z^2*u*v^2-F(95)/73*x*u^2*v^2-F(68)/73*y*u^2*v^2+F(49)/79*z*u^2*v^2-F(27)/112*u^3*v^2+8791*x^2*v^3-F(75)/58*x*y*v^3-F(31)/24*y^2*v^3+F(64)/91*x*z*v^3+F(80)/53*y*z*v^3-12032*z^2*v^3-F(6)/31*x*u*v^3+F(13)/49*y*u*v^3+F(53)/52*z*u*v^3+1945*u^2*v^3-F(12)/29*x*v^4+10008*y*v^4-12761*z*v^4-F(113)/88*u*v^4+5638*v^5,1095*x^5+F(27)/31*x^4*y+F(43)/94*x^3*y^2+8545*x^2*y^3-14897*x*y^4+F(9)/97*x^4*z+3453*x^3*y*z-F(11)/32*x^2*y^2*z+F(25)/71*x*y^3*z-F(62)/51*y^4*z-F(10)/93*x^3*z^2-F(116)/45*x^2*y*z^2+F(91)/88*x*y^2*z^2+13871*y^3*z^2-F(78)/71*x^2*z^3-F(80)/53*x*y*z^3+2637*y^2*z^3+5837*x*z^4-F(1)/6*y*z^4-9715*z^5+F(60)/49*x^4*u+F(29)/103*x^3*y*u-F(60)/77*x^2*y^2*u+1570*x*y^3*u+F(38)/81*y^4*u+F(92)/105*x^3*z*u+6821*x^2*y*z*u+F(6)/109*x*y^2*z*u+F(117)/2*y^3*z*u-8335*x^2*z^2*u+15915*x*y*z^2*u+12985*y^2*z^2*u-F(53)/72*x*z^3*u-F(46)/121*y*z^3*u-5355*z^4*u-F(104)/7*x^3*u^2-1408*x^2*y*u^2-2070*x*y^2*u^2-F(21)/85*y^3*u^2-14720*x^2*z*u^2+15643*x*y*z*u^2+F(97)/83*y^2*z*u^2+10305*x*z^2*u^2-F(101)/23*y*z^2*u^2+F(29)/106*z^3*u^2+F(63)/115*x^2*u^3-F(108)/25*x*y*u^3-F(7)/92*y^2*u^3-F(76)/111*x*z*u^3+6045*y*z*u^3-F(90)/121*z^2*u^3-F(102)/89*x*u^4+14070*y*u^4-F(119)/23*z*u^4+11620*u^5-F(7)/68*x^4*v-F(75)/118*x^3*y*v+F(78)/31*x^2*y^2*v+1110*x*y^3*v-F(9)/19*y^4*v-F(107)/120*x^3*z*v+9768*x^2*y*z*v-10138*x*y^2*z*v+F(116)/101*y^3*z*v-11565*x^2*z^2*v+F(37)/36*x*y*z^2*v+F(5)/48*y^2*z^2*v+14730*x*z^3*v+F(81)/25*y*z^3*v+11039*z^4*v+F(74)/51*x^3*u*v-F(19)/41*x^2*y*u*v-F(5)/27*x*y^2*u*v-7429*y^3*u*v+6630*x^2*z*u*v+F(115)/31*x*y*z*u*v+F(60)/49*y^2*z*u*v-9676*x*z^2*u*v-4508*y*z^2*u*v-F(70)/47*z^3*u*v-F(32)/99*x^2*u^2*v-3897*x*y*u^2*v+6513*y^2*u^2*v-14920*x*z*u^2*v-F(87)/35*y*z*u^2*v+14440*z^2*u^2*v+F(109)/83*x*u^3*v+F(102)/113*y*u^3*v+F(106)/89*z*u^3*v-F(68)/39*u^4*v+F(80)/47*x^3*v^2-15272*x^2*y*v^2-F(107)/40*x*y^2*v^2+F(86)/89*y^3*v^2+F(115)/62*x^2*z*v^2-9715*x*y*z*v^2-F(71)/8*y^2*z*v^2+4633*x*z^2*v^2-F(65)/101*y*z^2*v^2-14611*z^3*v^2-11465*x^2*u*v^2-F(73)/24*x*y*u*v^2+8351*y^2*u*v^2+1285*x*z*u*v^2+F(122)/113*y*z*u*v^2-11347*z^2*u*v^2-F(69)/104*x*u^2*v^2-F(21)/5*y*u^2*v^2+F(61)/49*z*u^2*v^2-4026*u^3*v^2+F(23)/118*x^2*v^3+F(115)/122*x*y*v^3+F(97)/78*y^2*v^3+F(58)/19*x*z*v^3-F(78)/113*y*z*v^3-3108*z^2*v^3+F(23)/42*x*u*v^3-6362*y*u*v^3-12350*z*u*v^3-F(101)/61*u^2*v^3+F(106)/117*x*v^4+7033*y*v^4+4280*z*v^4-2395*u*v^4-F(47)/41*v^5,-8876*x^4*y+F(115)/107*x^3*y^2+2365*x^2*y^3+9897*x*y^4+F(20)/111*y^5-14039*x^4*z-13283*x^3*y*z+F(84)/65*x^2*y^2*z+440*x*y^3*z+15211*y^4*z-F(111)/7*x^3*z^2+5456*x^2*y*z^2+3725*x*y^2*z^2+5956*y^3*z^2-F(73)/114*x^2*z^3-F(39)/7*x*y*z^3+F(100)/117*y^2*z^3+8510*x*z^4-6779*y*z^4-8878*z^5-8121*x^4*u+15336*x^3*y*u+F(46)/121*x^2*y^2*u+14611*x*y^3*u+F(70)/29*y^4*u+F(89)/79*x^3*z*u+F(4)/119*x^2*y*z*u-F(116)/5*x*y^2*z*u-F(26)/89*y^3*z*u-F(1)/111*x^2*z^2*u-9667*x*y*z^2*u+F(40)/79*y^2*z^2*u+9086*x*z^3*u-4370*y*z^3*u-F(79)/54*z^4*u-178*x^3*u^2+F(76)/29*x^2*y*u^2-12550*x*y^2*u^2-4043*y^3*u^2-448*x^2*z*u^2-13903*x*y*z*u^2+F(113)/73*y^2*z*u^2+F(5)/98*x*z^2*u^2-3571*y*z^2*u^2+F(49)/17*z^3*u^2-F(97)/60*x^2*u^3+83*x*y*u^3-F(115)/4*y^2*u^3-7368*x*z*u^3+F(75)/92*y*z*u^3-7369*z^2*u^3+F(35)/73*x*u^4+8473*y*u^4-F(28)/99*z*u^4+12091*u^5+F(72)/59*x^4*v-F(33)/35*x^3*y*v+9716*x^2*y^2*v+F(31)/60*x*y^3*v-14506*y^4*v+F(53)/13*x^3*z*v+F(47)/10*x^2*y*z*v-F(65)/21*x*y^2*z*v+8092*y^3*z*v+F(116)/77*x^2*z^2*v+F(43)/107*x*y*z^2*v+F(126)/101*y^2*z^2*v-F(67)/19*x*z^3*v+6645*y*z^3*v+F(83)/7*z^4*v+F(113)/85*x^3*u*v-F(19)/102*x^2*y*u*v-4016*x*y^2*u*v+F(39)/32*y^3*u*v-6947*x^2*z*u*v-F(103)/110*x*y*z*u*v-F(61)/107*y^2*z*u*v-F(31)/40*x*z^2*u*v+618*y*z^2*u*v+5139*z^3*u*v-F(43)/120*x^2*u^2*v-F(95)/41*x*y*u^2*v+F(105)/107*y^2*u^2*v+4639*x*z*u^2*v+13677*y*z*u^2*v-F(79)/54*z^2*u^2*v-2384*x*u^3*v+1657*y*u^3*v+1399*z*u^3*v-3055*u^4*v-F(35)/93*x^3*v^2+F(109)/106*x^2*y*v^2+5521*x*y^2*v^2-F(82)/107*y^3*v^2+12539*x^2*z*v^2+10532*x*y*z*v^2+F(35)/19*y^2*z*v^2-F(11)/63*x*z^2*v^2+F(13)/116*y*z^2*v^2+F(19)/26*z^3*v^2-F(109)/122*x^2*u*v^2+F(67)/38*x*y*u*v^2-F(61)/76*y^2*u*v^2-10329*x*z*u*v^2+13422*y*z*u*v^2+9004*z^2*u*v^2+F(33)/116*x*u^2*v^2-F(119)/11*y*u^2*v^2+F(66)/97*z*u^2*v^2+4251*u^3*v^2-F(83)/53*x^2*v^3-13*x*y*v^3+F(79)/28*y^2*v^3-5086*x*z*v^3+F(107)/15*y*z*v^3+F(40)/113*z^2*v^3+3675*x*u*v^3+11017*y*u*v^3-F(104)/87*z*u*v^3-F(75)/82*u^2*v^3+F(124)/33*x*v^4+3448*y*v^4+2733*z*v^4+F(109)/102*u*v^4+F(85)/77*v^5,-F(22)/95*x^5-F(23)/53*x^4*y+F(59)/33*x^3*y^2+F(29)/2*x^2*y^3-F(43)/80*x*y^4+F(13)/84*x^4*z-F(103)/69*x^3*y*z-F(124)/57*x^2*y^2*z+8191*x*y^3*z-F(25)/124*y^4*z+F(83)/97*x^3*z^2-3039*x^2*y*z^2+13*x*y^2*z^2+F(61)/94*y^3*z^2+F(119)/114*x^2*z^3-F(57)/71*x*y*z^3-13551*y^2*z^3-7910*x*z^4+F(31)/100*y*z^4+F(91)/53*z^5+F(118)/63*x^4*u-10901*x^3*y*u-F(44)/5*x^2*y^2*u-1462*x*y^3*u-F(121)/125*y^4*u+F(31)/70*x^3*z*u+F(126)/101*x^2*y*z*u+F(31)/107*x*y^2*z*u+F(4)/83*y^3*z*u+1402*x^2*z^2*u+13345*x*y*z^2*u-9855*y^2*z^2*u+F(9)/22*x*z^3*u-354*y*z^3*u-8740*z^4*u+F(91)/122*x^3*u^2-5429*x^2*y*u^2+F(91)/41*x*y^2*u^2+F(119)/76*y^3*u^2-F(103)/93*x^2*z*u^2+8136*x*y*z*u^2+13745*y^2*z*u^2-3081*x*z^2*u^2+F(97)/36*y*z^2*u^2+10333*z^3*u^2+9122*x^2*u^3+8952*x*y*u^3-F(113)/88*y^2*u^3-F(23)/86*x*z*u^3+7046*y*z*u^3+6010*z^2*u^3-8410*x*u^4-F(73)/49*y*u^4+F(41)/16*z*u^4-F(35)/67*u^5-6912*x^4*v-F(3)/101*x^3*y*v-F(40)/121*x^2*y^2*v-F(23)/59*x*y^3*v+7626*y^4*v+F(1)/24*x^3*z*v+F(39)/10*x^2*y*z*v-212*x*y^2*z*v+10075*y^3*z*v+F(96)/13*x^2*z^2*v+7042*x*y*z^2*v-8472*y^2*z^2*v+14619*x*z^3*v+F(1)/75*y*z^3*v-14242*z^4*v-F(113)/18*x^3*u*v+15363*x^2*y*u*v+14082*x*y^2*u*v-F(31)/8*y^3*u*v-F(120)/49*x^2*z*u*v+136*x*y*z*u*v+2123*y^2*z*u*v+10958*x*z^2*u*v+77*y*z^2*u*v+14895*z^3*u*v-15255*x^2*u^2*v+11854*x*y*u^2*v+12025*y^2*u^2*v+5798*x*z*u^2*v+F(35)/111*y*z*u^2*v-15825*z^2*u^2*v+F(35)/124*x*u^3*v-F(121)/48*y*u^3*v+F(61)/33*z*u^3*v+F(120)/83*u^4*v-5155*x^3*v^2-F(13)/125*x^2*y*v^2-F(65)/12*x*y^2*v^2+F(54)/101*y^3*v^2+F(69)/101*x^2*z*v^2-5127*x*y*z*v^2+7683*y^2*z*v^2+F(23)/34*x*z^2*v^2-F(60)/17*y*z^2*v^2-5683*z^3*v^2+F(27)/79*x^2*u*v^2+11138*x*y*u*v^2+F(36)/77*y^2*u*v^2+F(6)/17*x*z*u*v^2+F(113)/8*y*z*u*v^2+893*z^2*u*v^2-12059*x*u^2*v^2+F(85)/88*y*u^2*v^2-F(82)/39*z*u^2*v^2-F(45)/124*u^3*v^2-F(80)/93*x^2*v^3-13298*x*y*v^3+F(23)/102*y^2*v^3-F(55)/79*x*z*v^3-F(48)/47*y*z*v^3+F(95)/86*z^2*v^3-12223*x*u*v^3+6810*y*u*v^3+F(86)/13*z*u*v^3-14863*u^2*v^3+F(38)/99*x*v^4-14885*y*v^4+F(25)/122*z*v^4-F(8)/63*u*v^4+F(83)/71*v^5,F(59)/70*x^4*y+12868*x^3*y^2+13187*x^2*y^3-13460*x*y^4-11026*y^5+2075*x^4*z+11380*x^3*y*z-12223*x^2*y^2*z+2178*x*y^3*z-F(12)/67*y^4*z+F(3)/46*x^3*z^2-63*x^2*y*z^2+F(114)/115*x*y^2*z^2-F(28)/41*y^3*z^2+F(65)/17*x^2*z^3-F(64)/21*x*y*z^3+15410*y^2*z^3-F(71)/16*x*z^4-F(58)/75*y*z^4+F(92)/41*z^5-F(20)/59*x^4*u-F(33)/70*x^3*y*u+F(84)/73*x^2*y^2*u-F(112)/37*x*y^3*u-F(65)/11*y^4*u+F(76)/123*x^3*z*u-F(5)/67*x^2*y*z*u-5212*x*y^2*z*u-12013*y^3*z*u-F(55)/108*x^2*z^2*u-F(8)/15*x*y*z^2*u-F(88)/97*y^2*z^2*u+3661*x*z^3*u-15115*y*z^3*u-F(19)/47*z^4*u-3804*x^3*u^2-F(125)/48*x^2*y*u^2+15162*x*y^2*u^2+F(38)/17*y^3*u^2+F(33)/29*x^2*z*u^2-4871*x*y*z*u^2-F(112)/93*y^2*z*u^2+7851*x*z^2*u^2-F(91)/71*y*z^2*u^2+F(17)/18*z^3*u^2-7902*x^2*u^3+F(103)/39*x*y*u^3+F(8)/67*y^2*u^3+F(7)/54*x*z*u^3-F(67)/74*y*z*u^3+3615*z^2*u^3+F(112)/87*x*u^4+F(10)/61*y*u^4+F(55)/16*z*u^4-10342*u^5+11220*x^4*v-F(123)/58*x^3*y*v+F(67)/120*x^2*y^2*v+F(108)/17*x*y^3*v+F(34)/31*y^4*v-F(45)/17*x^3*z*v-F(55)/28*x^2*y*z*v-4407*x*y^2*z*v-F(97)/4*y^3*z*v-F(47)/68*x^2*z^2*v+F(43)/63*x*y*z^2*v-F(23)/22*y^2*z^2*v+F(120)/41*x*z^3*v-9401*y*z^3*v-F(39)/10*z^4*v-F(123)/115*x^3*u*v-F(13)/42*x^2*y*u*v-F(118)/45*x*y^2*u*v+F(94)/61*y^3*u*v+F(65)/19*x^2*z*u*v+14897*x*y*z*u*v-4329*y^2*z*u*v+F(50)/119*x*z^2*u*v+9419*y*z^2*u*v-F(61)/49*z^3*u*v+F(88)/87*x^2*u^2*v-F(71)/46*x*y*u^2*v-9580*y^2*u^2*v-F(109)/88*x*z*u^2*v+F(27)/32*y*z*u^2*v+9558*z^2*u^2*v-1250*x*u^3*v+14453*y*u^3*v-F(51)/7*z*u^3*v-F(43)/90*u^4*v+859*x^3*v^2+14026*x^2*y*v^2+F(31)/59*x*y^2*v^2-F(14)/107*y^3*v^2-F(14)/83*x^2*z*v^2-F(47)/101*x*y*z*v^2-7177*y^2*z*v^2+14615*x*z^2*v^2-6030*y*z^2*v^2+11737*z^3*v^2+F(73)/37*x^2*u*v^2-243*x*y*u*v^2-F(46)/113*y^2*u*v^2-F(41)/37*x*z*u*v^2+4060*y*z*u*v^2+F(58)/103*z^2*u*v^2-F(119)/110*x*u^2*v^2-F(39)/89*y*u^2*v^2-F(109)/52*z*u^2*v^2+F(89)/67*u^3*v^2-F(6)/19*x^2*v^3+13744*x*y*v^3+F(60)/103*y^2*v^3-12163*x*z*v^3+F(40)/119*y*z*v^3-8044*z^2*v^3+7514*x*u*v^3-F(83)/121*y*u*v^3-3103*z*u*v^3+6947*u^2*v^3+F(29)/22*x*v^4-F(111)/73*y*v^4-11876*z*v^4-F(121)/50*u*v^4+1852*v^5,F(35)/12*x^5+3423*x^4*y-1577*x^3*y^2+F(27)/77*x^2*y^3+F(26)/53*x*y^4+4205*x^4*z-F(87)/61*x^3*y*z+F(120)/109*x^2*y^2*z+F(61)/40*x*y^3*z+F(123)/101*y^4*z+13657*x^3*z^2-536*x^2*y*z^2-7131*x*y^2*z^2-F(25)/31*y^3*z^2+F(8)/87*x^2*z^3-5390*x*y*z^3+F(77)/18*y^2*z^3+F(116)/5*x*z^4-F(57)/97*y*z^4+12099*z^5+F(51)/59*x^4*u-F(56)/111*x^3*y*u+F(73)/70*x^2*y^2*u+1294*x*y^3*u+3667*y^4*u-F(55)/8*x^3*z*u-F(109)/73*x^2*y*z*u-F(33)/50*x*y^2*z*u-F(101)/102*y^3*z*u-F(2)/101*x^2*z^2*u+F(25)/13*x*y*z^2*u+15341*y^2*z^2*u+15420*x*z^3*u+F(85)/21*y*z^3*u+13479*z^4*u-F(124)/79*x^3*u^2-3336*x^2*y*u^2-F(33)/5*x*y^2*u^2+F(2)/13*y^3*u^2+F(126)/55*x^2*z*u^2-15666*x*y*z*u^2-F(47)/4*y^2*z*u^2+F(103)/39*x*z^2*u^2+11477*y*z^2*u^2+F(23)/114*z^3*u^2-F(13)/44*x^2*u^3-F(3)/112*x*y*u^3-F(9)/118*y^2*u^3+F(11)/109*x*z*u^3+7317*y*z*u^3+F(79)/37*z^2*u^3+3579*x*u^4+F(75)/4*y*u^4+F(22)/103*z*u^4+F(38)/87*u^5+F(84)/61*x^4*v+4685*x^3*y*v+F(28)/25*x^2*y^2*v+F(98)/57*x*y^3*v+F(11)/83*y^4*v+1891*x^3*z*v-F(90)/13*x^2*y*z*v+8374*x*y^2*z*v-F(57)/14*y^3*z*v+15902*x^2*z^2*v+F(99)/20*x*y*z^2*v+37*y^2*z^2*v+F(56)/9*x*z^3*v-F(93)/118*y*z^3*v+1340*z^4*v+7608*x^3*u*v-F(98)/23*x^2*y*u*v-F(8)/57*x*y^2*u*v-F(113)/26*y^3*u*v+12840*x^2*z*u*v-5705*x*y*z*u*v+13893*y^2*z*u*v+F(17)/22*x*z^2*u*v+F(117)/83*y*z^2*u*v+9865*z^3*u*v-13683*x^2*u^2*v-F(111)/98*x*y*u^2*v-675*y^2*u^2*v+F(47)/85*x*z*u^2*v-F(111)/29*y*z*u^2*v+10370*z^2*u^2*v-6727*x*u^3*v-F(39)/113*y*u^3*v-11931*z*u^3*v+7083*u^4*v-F(51)/107*x^3*v^2+F(94)/27*x^2*y*v^2-F(54)/109*x*y^2*v^2+12667*y^3*v^2+F(16)/91*x^2*z*v^2+F(56)/29*x*y*z*v^2-F(42)/115*y^2*z*v^2-12052*x*z^2*v^2-F(32)/93*y*z^2*v^2-F(80)/33*z^3*v^2+11327*x^2*u*v^2-F(57)/70*x*y*u*v^2+F(29)/103*y^2*u*v^2-3455*x*z*u*v^2-4221*y*z*u*v^2+8950*z^2*u*v^2+F(98)/113*x*u^2*v^2+F(47)/50*y*u^2*v^2+442*z*u^2*v^2+13080*u^3*v^2+F(76)/45*x^2*v^3+F(55)/117*x*y*v^3+11305*y^2*v^3+F(34)/37*x*z*v^3+F(66)/47*y*z*v^3+7830*z^2*v^3-F(115)/16*x*u*v^3-9283*y*u*v^3+9120*z*u*v^3-F(83)/117*u^2*v^3-F(19)/126*x*v^4+F(9)/22*y*v^4-F(18)/91*z*v^4-F(61)/28*u*v^4-F(91)/115*v^5,4547*x^4*y+F(75)/74*x^3*y^2+2981*x^2*y^3-9935*x*y^4-F(67)/90*y^5+F(81)/86*x^4*z+15013*x^3*y*z+F(65)/31*x^2*y^2*z+F(67)/2*x*y^3*z+F(20)/87*y^4*z-10940*x^3*z^2-F(35)/26*x^2*y*z^2+F(18)/7*x*y^2*z^2+F(43)/89*y^3*z^2-3355*x^2*z^3+F(56)/121*x*y*z^3-F(92)/87*y^2*z^3-F(85)/59*x*z^4-F(95)/92*y*z^4-F(30)/19*z^5-F(111)/67*x^4*u+F(3)/82*x^3*y*u+F(27)/61*x^2*y^2*u-F(86)/93*x*y^3*u+6082*y^4*u-F(70)/101*x^3*z*u-F(98)/109*x^2*y*z*u+F(70)/123*x*y^2*z*u+F(43)/85*y^3*z*u+F(52)/33*x^2*z^2*u-6985*x*y*z^2*u+F(108)/97*y^2*z^2*u-F(43)/26*x*z^3*u-F(98)/83*y*z^3*u-9717*z^4*u+5093*x^3*u^2-2970*x^2*y*u^2+F(102)/37*x*y^2*u^2+6317*y^3*u^2-12992*x^2*z*u^2+9684*x*y*z*u^2+F(13)/67*y^2*z*u^2-F(51)/92*x*z^2*u^2-1540*y*z^2*u^2+1956*z^3*u^2+15396*x^2*u^3+F(31)/90*x*y*u^3-F(85)/61*y^2*u^3-F(121)/71*x*z*u^3+F(101)/117*y*z*u^3+3156*z^2*u^3-9525*x*u^4+2701*y*u^4+2299*z*u^4+F(44)/69*u^5+13073*x^4*v-15350*x^3*y*v-F(61)/35*x^2*y^2*v+F(66)/107*x*y^3*v+11110*y^4*v+5861*x^3*z*v-14898*x^2*y*z*v+F(21)/44*x*y^2*z*v+1962*y^3*z*v-F(50)/121*x^2*z^2*v+F(94)/11*x*y*z^2*v-F(19)/96*y^2*z^2*v-F(87)/79*x*z^3*v-F(74)/69*y*z^3*v+233*z^4*v+F(95)/89*x^3*u*v-13742*x^2*y*u*v+11669*x*y^2*u*v+F(2)/83*y^3*u*v+F(123)/94*x^2*z*u*v-F(89)/51*x*y*z*u*v-F(117)/113*y^2*z*u*v+14914*x*z^2*u*v-13511*y*z^2*u*v+5*z^3*u*v+F(6)/53*x^2*u^2*v-F(33)/53*x*y*u^2*v-1072*y^2*u^2*v+4079*x*z*u^2*v-8856*y*z*u^2*v-F(7)/37*z^2*u^2*v-14863*x*u^3*v-F(31)/4*y*u^3*v-2965*z*u^3*v+F(65)/47*u^4*v-12076*x^3*v^2-F(89)/103*x^2*y*v^2+6803*x*y^2*v^2-F(81)/121*y^3*v^2-F(97)/50*x^2*z*v^2+F(52)/37*x*y*z*v^2+5242*y^2*z*v^2+8410*x*z^2*v^2-F(4)/21*y*z^2*v^2-F(96)/101*z^3*v^2-F(71)/88*x^2*u*v^2+F(105)/38*x*y*u*v^2-F(92)/13*y^2*u*v^2-14243*x*z*u*v^2-F(37)/14*y*z*u*v^2+F(33)/41*z^2*u*v^2+F(39)/49*x*u^2*v^2-F(57)/32*y*u^2*v^2-1485*z*u^2*v^2+F(3)/38*u^3*v^2+F(91)/120*x^2*v^3-F(81)/92*x*y*v^3+F(86)/119*y^2*v^3-F(53)/57*x*z*v^3+14500*y*z*v^3-F(61)/118*z^2*v^3-F(83)/90*x*u*v^3+F(25)/52*y*u*v^3+14067*z*u*v^3-F(81)/74*u^2*v^3-F(125)/83*x*v^4+F(119)/65*y*v^4-F(59)/87*z*v^4-8687*u*v^4-12082*v^5,5501*x^5-14749*x^4*y+1637*x^3*y^2+F(107)/22*x^2*y^3-F(83)/20*x*y^4+F(44)/65*y^5-F(94)/91*x^4*z+F(21)/10*x^3*y*z+5267*x^2*y^2*z-15192*x*y^3*z-F(110)/13*y^4*z+5917*x^3*z^2+796*x^2*y*z^2-F(77)/101*x*y^2*z^2-8680*y^3*z^2-15909*x^2*z^3+F(13)/87*x*y*z^3-4222*y^2*z^3+F(67)/107*x*z^4+F(2)/121*y*z^4+F(7)/74*z^5+F(39)/85*x^4*u+F(69)/65*x^3*y*u-1238*x^2*y^2*u+3302*x*y^3*u-F(41)/101*y^4*u+4871*x^3*z*u-F(55)/34*x^2*y*z*u+F(86)/47*x*y^2*z*u-12151*y^3*z*u-6046*x^2*z^2*u-14589*x*y*z^2*u-13731*y^2*z^2*u-F(56)/37*x*z^3*u+F(100)/9*y*z^3*u+F(120)/119*z^4*u+F(21)/40*x^3*u^2+F(1)/124*x^2*y*u^2+7940*x*y^2*u^2+4016*y^3*u^2-F(79)/12*x^2*z*u^2+12014*x*y*z*u^2+F(99)/86*y^2*z*u^2-F(70)/27*x*z^2*u^2+F(13)/42*y*z^2*u^2+F(53)/38*z^3*u^2+F(113)/74*x^2*u^3+F(65)/19*x*y*u^3-F(103)/117*y^2*u^3+3969*x*z*u^3-F(112)/71*y*z*u^3+F(47)/67*z^2*u^3-11347*x*u^4+F(93)/50*y*u^4-F(29)/17*z*u^4+10718*u^5-13737*x^4*v-F(69)/53*x^3*y*v+9003*x^2*y^2*v+F(113)/23*x*y^3*v-F(3)/5*y^4*v-F(44)/63*x^3*z*v+8894*x^2*y*z*v-F(7)/39*x*y^2*z*v+F(112)/59*y^3*z*v-13276*x^2*z^2*v-F(63)/19*x*y*z^2*v+F(17)/18*y^2*z^2*v-1448*x*z^3*v+7669*y*z^3*v+4724*z^4*v+F(60)/79*x^3*u*v-5005*x^2*y*u*v-F(43)/64*x*y^2*u*v+F(90)/91*y^3*u*v+F(120)/43*x^2*z*u*v+15553*x*y*z*u*v+F(5)/112*y^2*z*u*v+8554*x*z^2*u*v+F(64)/31*y*z^2*u*v-F(43)/16*z^3*u*v-F(46)/79*x^2*u^2*v+7263*x*y*u^2*v-1673*y^2*u^2*v+F(45)/23*x*z*u^2*v+F(71)/68*y*z*u^2*v-11968*z^2*u^2*v-11649*x*u^3*v+F(32)/21*y*u^3*v+F(70)/111*z*u^3*v+F(94)/57*u^4*v+632*x^3*v^2-F(48)/31*x^2*y*v^2+F(77)/118*x*y^2*v^2+F(36)/109*y^3*v^2+2887*x^2*z*v^2+F(13)/105*x*y*z*v^2+10594*y^2*z*v^2+F(11)/2*x*z^2*v^2+F(101)/36*y*z^2*v^2+F(61)/42*z^3*v^2-F(113)/17*x^2*u*v^2+F(13)/4*x*y*u*v^2+3162*y^2*u*v^2+15386*x*z*u*v^2+637*y*z*u*v^2-645*z^2*u*v^2-3569*x*u^2*v^2-F(69)/16*y*u^2*v^2-F(83)/52*z*u^2*v^2+22*u^3*v^2-F(89)/58*x^2*v^3+F(10)/27*x*y*v^3+F(91)/88*y^2*v^3+F(62)/113*x*z*v^3-13218*y*z*v^3-F(48)/31*z^2*v^3+F(13)/64*x*u*v^3-14643*y*u*v^3+F(97)/34*z*u*v^3-F(43)/75*u^2*v^3-F(41)/104*x*v^4+F(85)/7*y*v^4+F(38)/111*z*v^4-1358*u*v^4-F(17)/20*v^5,F(77)/113*x^5-14145*x^4*y+F(27)/79*x^3*y^2+13070*x^2*y^3-F(19)/36*x*y^4+14580*y^5+14147*x^4*z-4352*x^3*y*z-F(53)/63*x^2*y^2*z+15533*x*y^3*z-F(83)/4*y^4*z-F(18)/17*x^3*z^2+F(107)/97*x^2*y*z^2+429*x*y^2*z^2-F(55)/26*y^3*z^2-F(91)/108*x^2*z^3+F(49)/61*x*y*z^3-F(119)/104*y^2*z^3-13182*x*z^4-F(41)/47*y*z^4-F(73)/56*z^5+F(39)/29*x^4*u-3183*x^3*y*u+F(34)/115*x^2*y^2*u-14554*x*y^3*u+6362*y^4*u-F(44)/111*x^3*z*u-12574*x^2*y*z*u-F(32)/109*x*y^2*z*u-F(37)/55*y^3*z*u+F(101)/85*x^2*z^2*u+9808*x*y*z^2*u-F(98)/31*y^2*z^2*u-10305*x*z^3*u-12194*y*z^3*u-3006*z^4*u-F(87)/28*x^3*u^2-9070*x^2*y*u^2+F(113)/5*x*y^2*u^2-4206*y^3*u^2+15436*x^2*z*u^2-F(42)/95*x*y*z*u^2-14991*y^2*z*u^2-F(68)/109*x*z^2*u^2+9163*y*z^2*u^2-15103*z^3*u^2-14095*x^2*u^3+8642*x*y*u^3+F(71)/34*y^2*u^3-F(112)/103*x*z*u^3+8180*y*z*u^3+8992*z^2*u^3+F(50)/49*x*u^4-3103*y*u^4+12974*z*u^4-8238*u^5+F(44)/39*x^4*v-8635*x^3*y*v+F(119)/71*x^2*y^2*v-F(122)/55*x*y^3*v+12507*y^4*v+6047*x^3*z*v+F(86)/45*x^2*y*z*v-2102*x*y^2*z*v-F(115)/98*y^3*z*v-717*x^2*z^2*v-F(14)/107*x*y*z^2*v+14026*y^2*z^2*v-F(104)/33*x*z^3*v-48*y*z^3*v-3400*z^4*v+F(31)/22*x^3*u*v+F(11)/118*x^2*y*u*v+F(92)/63*x*y^2*u*v+F(111)/124*y^3*u*v-F(44)/5*x^2*z*u*v-F(106)/119*x*y*z*u*v-1260*y^2*z*u*v+F(38)/41*x*z^2*u*v-F(77)/123*y*z^2*u*v-F(10)/51*z^3*u*v+F(17)/84*x^2*u^2*v-F(25)/21*x*y*u^2*v+F(101)/62*y^2*u^2*v+3913*x*z*u^2*v+7159*y*z*u^2*v-3385*z^2*u^2*v+3474*x*u^3*v+10313*y*u^3*v+12323*z*u^3*v+F(29)/88*u^4*v-F(36)/73*x^3*v^2+F(23)/40*x^2*y*v^2-7501*x*y^2*v^2-1696*y^3*v^2+14898*x^2*z*v^2-F(112)/113*x*y*z*v^2-F(23)/124*y^2*z*v^2-F(89)/119*x*z^2*v^2+15307*y*z^2*v^2+F(113)/35*z^3*v^2+F(41)/115*x^2*u*v^2+F(70)/13*x*y*u*v^2+6613*y^2*u*v^2+7212*x*z*u*v^2-F(59)/108*y*z*u*v^2+6592*z^2*u*v^2+1718*x*u^2*v^2+F(91)/68*y*u^2*v^2-F(48)/109*z*u^2*v^2-10607*u^3*v^2+F(124)/51*x^2*v^3-F(102)/55*x*y*v^3+F(26)/29*y^2*v^3-F(72)/85*x*z*v^3+8403*y*z*v^3+F(106)/23*z^2*v^3+F(39)/37*x*u*v^3-F(59)/21*y*u*v^3-F(27)/22*z*u*v^3-F(36)/17*u^2*v^3+F(86)/53*x*v^4-F(24)/55*y*v^4-F(125)/56*z*v^4+14939*u*v^4-F(4)/107*v^5,-5455*x^5-F(48)/65*x^4*y-F(21)/37*x^3*y^2-F(21)/53*x^2*y^3+F(109)/2*x*y^4+F(25)/101*y^5-F(54)/5*x^4*z+F(11)/45*x^3*y*z+F(13)/11*x^2*y^2*z-F(102)/11*x*y^3*z-5588*y^4*z+F(37)/80*x^3*z^2-F(89)/13*x^2*y*z^2+F(71)/86*x*y^2*z^2-F(125)/71*y^3*z^2-5521*x^2*z^3+F(26)/121*x*y*z^3+F(1)/125*y^2*z^3+208*x*z^4-F(37)/120*y*z^4-F(101)/43*z^5+F(77)/90*x^4*u-F(62)/73*x^3*y*u-F(51)/13*x^2*y^2*u+10118*x*y^3*u+F(28)/5*y^4*u+8705*x^3*z*u-F(106)/95*x^2*y*z*u+1237*x*y^2*z*u+10147*y^3*z*u-F(24)/55*x^2*z^2*u-10831*x*y*z^2*u-10598*y^2*z^2*u+F(93)/92*x*z^3*u-F(81)/31*y*z^3*u+15142*z^4*u+F(49)/89*x^3*u^2+F(14)/117*x^2*y*u^2+9455*x*y^2*u^2+F(21)/86*y^3*u^2+F(98)/115*x^2*z*u^2-F(71)/32*x*y*z*u^2-F(83)/123*y^2*z*u^2-F(73)/83*x*z^2*u^2-F(53)/57*y*z^2*u^2-F(125)/46*z^3*u^2-F(113)/34*x^2*u^3-F(43)/64*x*y*u^3-F(121)/2*y^2*u^3+F(115)/26*x*z*u^3+F(44)/101*y*z*u^3+13675*z^2*u^3-F(115)/76*x*u^4-F(112)/9*y*u^4-3865*z*u^4+F(119)/75*u^5-F(111)/25*x^4*v-14860*x^3*y*v+F(4)/31*x^2*y^2*v+7687*x*y^3*v+5255*y^4*v+4599*x^3*z*v+10276*x^2*y*z*v+F(11)/61*x*y^2*z*v-F(106)/97*y^3*z*v-11523*x^2*z^2*v+F(21)/110*x*y*z^2*v+F(11)/17*y^2*z^2*v+F(11)/86*x*z^3*v-F(89)/50*y*z^3*v-8403*z^4*v-F(74)/65*x^3*u*v+F(12)/95*x^2*y*u*v+F(80)/71*x*y^2*u*v-4327*y^3*u*v-13079*x^2*z*u*v-F(77)/82*x*y*z*u*v+3577*y^2*z*u*v+F(23)/100*x*z^2*u*v+9292*y*z^2*u*v+F(51)/50*z^3*u*v-F(89)/67*x^2*u^2*v+F(80)/51*x*y*u^2*v+F(65)/69*y^2*u^2*v+14040*x*z*u^2*v-5298*y*z*u^2*v+11279*z^2*u^2*v+F(39)/7*x*u^3*v-F(37)/90*y*u^3*v-F(76)/33*z*u^3*v+8543*u^4*v+F(29)/79*x^3*v^2-7727*x^2*y*v^2-10887*x*y^2*v^2+F(79)/82*y^3*v^2-8383*x^2*z*v^2-F(121)/57*x*y*z*v^2+F(107)/38*y^2*z*v^2-5303*x*z^2*v^2+F(37)/42*y*z^2*v^2-F(19)/81*z^3*v^2-15294*x^2*u*v^2+F(11)/108*x*y*u*v^2-F(119)/31*y^2*u*v^2+F(55)/47*x*z*u*v^2-15111*y*z*u*v^2-4019*z^2*u*v^2+F(124)/47*x*u^2*v^2+3267*y*u^2*v^2+F(125)/99*z*u^2*v^2-F(109)/23*u^3*v^2+F(2)/121*x^2*v^3-11556*x*y*v^3-8335*y^2*v^3-F(24)/67*x*z*v^3+11011*y*z*v^3-1436*z^2*v^3-4873*x*u*v^3-F(89)/53*y*u*v^3-F(93)/106*z*u*v^3-8658*u^2*v^3-1943*x*v^4-F(63)/73*y*v^4-F(5)/73*z*v^4+F(79)/14*u*v^4-F(103)/101*v^5,-F(31)/126*x^5+F(18)/17*x^4*y-11969*x^3*y^2-5548*x^2*y^3-5794*x*y^4+3213*y^5+F(125)/6*x^4*z+F(29)/12*x^3*y*z-15590*x^2*y^2*z+F(113)/12*x*y^3*z+F(72)/101*y^4*z+F(17)/73*x^3*z^2-15686*x^2*y*z^2-3632*x*y^2*z^2-F(17)/24*y^3*z^2-F(37)/123*x^2*z^3-F(32)/93*x*y*z^3+F(13)/125*y^2*z^3+F(43)/38*x*z^4-F(52)/31*y*z^4+6743*z^5-F(59)/98*x^4*u-F(75)/88*x^3*y*u+11566*x^2*y^2*u-14083*x*y^3*u-F(17)/15*y^4*u-F(37)/12*x^3*z*u+15923*x^2*y*z*u-453*x*y^2*z*u+F(109)/67*y^3*z*u+6185*x^2*z^2*u-F(101)/59*x*y*z^2*u+F(25)/74*y^2*z^2*u-F(125)/47*x*z^3*u-11494*y*z^3*u-F(61)/22*z^4*u-F(3)/29*x^3*u^2-5037*x^2*y*u^2-F(40)/59*x*y^2*u^2-F(79)/27*y^3*u^2+F(72)/35*x^2*z*u^2+10742*x*y*z*u^2+F(56)/95*y^2*z*u^2-4967*x*z^2*u^2+F(59)/113*y*z^2*u^2-F(46)/61*z^3*u^2+F(55)/62*x^2*u^3+F(5)/97*x*y*u^3-F(2)/29*y^2*u^3+F(32)/29*x*z*u^3+14336*y*z*u^3+8464*z^2*u^3+F(119)/71*x*u^4-F(75)/47*y*u^4+F(72)/29*z*u^4+2442*u^5+F(103)/47*x^4*v+6072*x^3*y*v+F(25)/54*x^2*y^2*v-8164*x*y^3*v+F(115)/42*y^4*v-F(89)/75*x^3*z*v-F(13)/72*x^2*y*z*v+F(38)/123*x*y^2*z*v-2235*y^3*z*v-F(64)/17*x^2*z^2*v-F(74)/39*x*y*z^2*v+6105*y^2*z^2*v-15914*x*z^3*v+7129*y*z^3*v+F(1)/10*z^4*v-4937*x^3*u*v-F(19)/97*x^2*y*u*v-F(58)/123*x*y^2*u*v-4198*y^3*u*v+715*x^2*z*u*v-F(84)/103*x*y*z*u*v-F(9)/64*y^2*z*u*v-F(94)/37*x*z^2*u*v+F(53)/64*y*z^2*u*v+1354*z^3*u*v+13661*x^2*u^2*v+12831*x*y*u^2*v+F(59)/53*y^2*u^2*v-1566*x*z*u^2*v-851*y*z*u^2*v+F(49)/61*z^2*u^2*v-F(89)/110*x*u^3*v-F(85)/22*y*u^3*v+12409*z*u^3*v+F(99)/41*u^4*v+F(98)/109*x^3*v^2+5502*x^2*y*v^2-F(120)/31*x*y^2*v^2-F(5)/114*y^3*v^2-8337*x^2*z*v^2+F(19)/73*x*y*z*v^2-F(46)/85*y^2*z*v^2-F(83)/95*x*z^2*v^2-15928*y*z^2*v^2-F(51)/31*z^3*v^2-474*x^2*u*v^2-5605*x*y*u*v^2-F(88)/61*y^2*u*v^2+F(49)/86*x*z*u*v^2+F(62)/85*y*z*u*v^2-10091*z^2*u*v^2+F(35)/38*x*u^2*v^2-F(61)/32*y*u^2*v^2-F(119)/46*z*u^2*v^2-12128*u^3*v^2+F(41)/118*x^2*v^3+6793*x*y*v^3+15225*y^2*v^3+9102*x*z*v^3-10442*y*z*v^3+F(122)/93*z^2*v^3+F(34)/37*x*u*v^3+15272*y*u*v^3-F(27)/2*z*u*v^3-15006*u^2*v^3+F(29)/23*x*v^4-F(9)/47*y*v^4-F(121)/113*z*v^4-F(41)/125*u*v^4-1986*v^5,x^5-F(77)/59*x^4*y+F(21)/95*x^3*y^2-22*x^2*y^3+F(21)/79*x*y^4-11979*y^5-F(104)/113*x^4*z+F(6)/11*x^3*y*z-F(25)/33*x^2*y^2*z+F(93)/35*x*y^3*z-14102*y^4*z-F(8)/115*x^3*z^2+F(89)/81*x^2*y*z^2+F(78)/73*x*y^2*z^2+3021*y^3*z^2+1287*x^2*z^3+6869*x*y*z^3-F(91)/123*y^2*z^3+F(10)/73*x*z^4+F(39)/89*y*z^4+5479*z^5+4826*x^4*u-F(107)/30*x^3*y*u-1565*x^2*y^2*u+6118*x*y^3*u+1983*y^4*u+F(103)/46*x^3*z*u+F(124)/81*x^2*y*z*u-F(59)/115*x*y^2*z*u+15190*y^3*z*u-7132*x^2*z^2*u+F(101)/94*x*y*z^2*u-11441*y^2*z^2*u-11333*x*z^3*u-11972*y*z^3*u-F(49)/16*z^4*u-197*x^3*u^2-95*x^2*y*u^2+F(100)/111*x*y^2*u^2-F(11)/87*y^3*u^2-12659*x^2*z*u^2+10920*x*y*z*u^2+F(16)/71*y^2*z*u^2+F(41)/92*x*z^2*u^2-12365*y*z^2*u^2+F(11)/67*z^3*u^2-F(43)/72*x^2*u^3-10271*x*y*u^3+F(95)/84*y^2*u^3+F(26)/111*x*z*u^3+5155*y*z*u^3-1087*z^2*u^3+2981*x*u^4-F(41)/114*y*u^4+F(13)/7*z*u^4+F(89)/81*u^5+F(83)/90*x^4*v+6659*x^3*y*v+5164*x^2*y^2*v+F(67)/32*x*y^3*v-F(43)/34*y^4*v-15871*x^3*z*v-F(68)/43*x^2*y*z*v-F(72)/101*x*y^2*z*v+F(109)/39*y^3*z*v+7253*x^2*z^2*v-F(116)/5*x*y*z^2*v-4777*y^2*z^2*v+F(76)/35*x*z^3*v+4710*y*z^3*v-F(52)/101*z^4*v-131*x^3*u*v+11312*x^2*y*u*v-F(95)/8*x*y^2*u*v-F(125)/96*y^3*u*v-9495*x^2*z*u*v+F(73)/119*x*y*z*u*v+F(114)/121*y^2*z*u*v+F(37)/80*x*z^2*u*v+11612*y*z^2*u*v-F(51)/124*z^3*u*v-1414*x^2*u^2*v-F(88)/119*x*y*u^2*v-F(119)/76*y^2*u^2*v-F(112)/5*x*z*u^2*v+F(71)/20*y*z*u^2*v-F(76)/55*z^2*u^2*v+F(119)/76*x*u^3*v+F(20)/41*y*u^3*v+8122*z*u^3*v-102*u^4*v-4206*x^3*v^2-F(104)/89*x^2*y*v^2-684*x*y^2*v^2+F(35)/104*y^3*v^2+10841*x^2*z*v^2+F(68)/125*x*y*z*v^2-F(14)/31*y^2*z*v^2+F(16)/119*x*z^2*v^2+13409*y*z^2*v^2+1287*z^3*v^2+F(57)/22*x^2*u*v^2+5797*x*y*u*v^2+F(95)/28*y^2*u*v^2-7607*x*z*u*v^2+F(77)/48*y*z*u*v^2+F(23)/26*z^2*u*v^2+F(25)/71*x*u^2*v^2-F(37)/32*y*u^2*v^2-F(13)/110*z*u^2*v^2-F(68)/83*u^3*v^2-15361*x^2*v^3-10443*x*y*v^3+F(34)/77*y^2*v^3-F(67)/122*x*z*v^3+F(45)/41*y*z*v^3+3322*z^2*v^3-F(74)/15*x*u*v^3+15458*y*u*v^3+F(11)/59*z*u*v^3+F(58)/45*u^2*v^3+13811*x*v^4+15272*y*v^4+4502*z*v^4-10872*u*v^4+F(25)/99*v^5]))
end


@doc raw"""
    rational_d11_pi11_ss_1()

Return a smooth rational surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and one 6-secant.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d11_pi11_ss_1()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^4*u^2-F(33)/91*x^3*y*u^2-F(89)/33*x^2*y^2*u^2-6237*x*y^3*u^2+F(51)/29*y^4*u^2-F(43)/23*x^3*z*u^2+379*x^2*y*z*u^2-F(105)/44*x*y^2*z*u^2+5805*y^3*z*u^2-3573*x^2*z^2*u^2-F(8)/125*x*y*z^2*u^2-F(108)/37*y^2*z^2*u^2-F(48)/7*x*z^3*u^2-3847*y*z^3*u^2-F(77)/26*z^4*u^2-F(89)/71*x^3*u^3-F(100)/121*x^2*y*u^3+9325*x*y^2*u^3-2637*y^3*u^3+F(87)/38*x^2*z*u^3-F(87)/56*x*y*z*u^3+10823*y^2*z*u^3-F(79)/66*x*z^2*u^3+F(112)/17*y*z^2*u^3+6086*z^3*u^3-85*x^2*u^4-2255*x*y*u^4+F(16)/91*y^2*u^4-F(62)/111*x*z*u^4-7347*y*z*u^4+F(107)/99*z^2*u^4-F(25)/92*x*u^5-F(71)/13*y*u^5-F(51)/28*z*u^5+F(34)/67*u^6+328*x^4*u*v+F(123)/43*x^3*y*u*v+15499*x^2*y^2*u*v+4106*x*y^3*u*v+F(24)/73*y^4*u*v-F(73)/55*x^3*z*u*v+F(99)/122*x^2*y*z*u*v-12915*x*y^2*z*u*v+F(121)/124*y^3*z*u*v-7786*x^2*z^2*u*v-12089*x*y*z^2*u*v+F(13)/41*y^2*z^2*u*v-F(71)/27*x*z^3*u*v-10189*y*z^3*u*v+F(111)/86*z^4*u*v+1295*x^3*u^2*v+3920*x^2*y*u^2*v-F(113)/48*x*y^2*u^2*v+F(72)/77*y^3*u^2*v-6487*x^2*z*u^2*v+9310*x*y*z*u^2*v+11003*y^2*z*u^2*v-F(71)/49*x*z^2*u^2*v+F(52)/87*y*z^2*u^2*v-F(88)/81*z^3*u^2*v+F(35)/106*x^2*u^3*v+12323*x*y*u^3*v-11435*y^2*u^3*v-F(3)/101*x*z*u^3*v-12832*y*z*u^3*v-F(53)/116*z^2*u^3*v+F(6)/97*x*u^4*v-3748*y*u^4*v+F(91)/86*z*u^4*v+F(25)/117*u^5*v-11777*x^4*v^2+4397*x^3*y*v^2+F(101)/89*x^2*y^2*v^2+F(104)/95*x*y^3*v^2-8127*y^4*v^2-3464*x^3*z*v^2+2449*x^2*y*z*v^2-13372*x*y^2*z*v^2+F(23)/2*y^3*z*v^2+F(9)/31*x^2*z^2*v^2+10385*x*y*z^2*v^2+F(98)/45*y^2*z^2*v^2+5144*x*z^3*v^2-14712*y*z^3*v^2+F(51)/70*z^4*v^2+F(118)/121*x^3*u*v^2-F(25)/61*x^2*y*u*v^2-F(118)/29*x*y^2*u*v^2-12238*y^3*u*v^2-2615*x^2*z*u*v^2-13260*x*y*z*u*v^2-F(81)/77*y^2*z*u*v^2+11378*x*z^2*u*v^2+F(51)/122*y*z^2*u*v^2-F(51)/73*z^3*u*v^2+F(113)/100*x^2*u^2*v^2-F(78)/7*x*y*u^2*v^2-12246*y^2*u^2*v^2+F(95)/103*x*z*u^2*v^2-F(30)/29*y*z*u^2*v^2-4090*z^2*u^2*v^2+3291*x*u^3*v^2-5383*y*u^3*v^2-F(73)/68*z*u^3*v^2-F(53)/22*u^4*v^2-F(125)/73*x^3*v^3-F(53)/102*x^2*y*v^3+F(103)/18*x*y^2*v^3-7697*y^3*v^3+F(21)/40*x^2*z*v^3-F(47)/57*x*y*z*v^3+4687*y^2*z*v^3-F(62)/27*x*z^2*v^3-F(116)/55*y*z^2*v^3+12153*z^3*v^3+F(121)/114*x^2*u*v^3-F(115)/31*x*y*u*v^3+F(40)/91*y^2*u*v^3-4347*x*z*u*v^3+F(29)/89*y*z*u*v^3+3805*z^2*u*v^3-5862*x*u^2*v^3+F(70)/117*y*u^2*v^3-F(107)/91*z*u^2*v^3-12653*u^3*v^3+6065*x^2*v^4+8050*x*y*v^4-6592*y^2*v^4+F(5)/126*x*z*v^4-F(115)/18*y*z*v^4-F(94)/101*z^2*v^4-F(73)/78*x*u*v^4-F(52)/3*y*u*v^4-F(56)/29*z*u*v^4+10094*u^2*v^4-12142*x*v^5+F(16)/59*y*v^5+F(31)/66*z*v^5-4143*u*v^5+962*v^6,-F(26)/47*x^5-F(31)/22*x^4*y-F(121)/112*x^3*y^2-F(62)/79*x^2*y^3+F(116)/53*x*y^4+10343*y^5+F(73)/28*x^4*z+4552*x^3*y*z+15801*x^2*y^2*z-1737*x*y^3*z-F(41)/98*y^4*z+13218*x^3*z^2-6837*x^2*y*z^2-13867*x*y^2*z^2+14798*y^3*z^2-F(73)/38*x^2*z^3+F(67)/125*x*y*z^3-1645*y^2*z^3-F(53)/34*x*z^4-F(87)/79*y*z^4+10158*z^5-F(107)/22*x^4*u-9967*x^3*y*u+F(17)/46*x^2*y^2*u-F(62)/51*x*y^3*u+F(83)/79*y^4*u+12876*x^3*z*u+3729*x^2*y*z*u+F(8)/107*x*y^2*z*u+11699*y^3*z*u-F(98)/79*x^2*z^2*u+F(100)/99*x*y*z^2*u+F(49)/24*y^2*z^2*u+F(88)/37*x*z^3*u+13318*y*z^3*u+F(5)/2*z^4*u-F(58)/19*x^3*u^2+F(7)/73*x^2*y*u^2-F(9)/68*x*y^2*u^2+F(93)/47*y^3*u^2-F(56)/113*x^2*z*u^2-6443*x*y*z*u^2-11378*y^2*z*u^2-5718*x*z^2*u^2-F(125)/44*y*z^2*u^2-F(118)/47*z^3*u^2+F(107)/39*x^2*u^3-8285*x*y*u^3-1922*y^2*u^3-15189*x*z*u^3-F(50)/91*y*z*u^3+F(121)/87*z^2*u^3+3013*x*u^4+8831*y*u^4-4274*z*u^4+F(119)/20*x^4*v+F(55)/68*x^3*y*v-8074*x^2*y^2*v-F(57)/49*x*y^3*v-2159*y^4*v-F(29)/59*x^3*z*v+5603*x^2*y*z*v-F(96)/97*x*y^2*z*v+F(125)/9*y^3*z*v+1222*x^2*z^2*v-F(41)/75*x*y*z^2*v+215*y^2*z^2*v-F(109)/76*x*z^3*v+F(56)/93*y*z^3*v-F(110)/41*z^4*v+F(93)/34*x^3*u*v-F(24)/55*x^2*y*u*v-7029*x*y^2*u*v+1237*y^3*u*v+F(125)/26*x^2*z*u*v-11748*x*y*z*u*v+7278*y^2*z*u*v+F(65)/122*x*z^2*u*v+9310*y*z^2*u*v-9107*z^3*u*v+14180*x^2*u^2*v-F(19)/84*x*y*u^2*v-F(29)/39*y^2*u^2*v-F(3)/52*x*z*u^2*v+F(55)/76*y*z*u^2*v-F(80)/27*z^2*u^2*v-F(94)/105*x*u^3*v+F(33)/14*y*u^3*v+15383*z*u^3*v-2772*x^3*v^2+F(62)/45*x^2*y*v^2+F(11)/96*x*y^2*v^2-3736*y^3*v^2-F(89)/77*x^2*z*v^2+F(30)/113*x*y*z*v^2-4761*y^2*z*v^2-F(5)/84*x*z^2*v^2-F(107)/124*y*z^2*v^2+F(106)/43*z^3*v^2+4418*x^2*u*v^2-10747*x*y*u*v^2+3745*y^2*u*v^2-F(115)/57*x*z*u*v^2-6929*y*z*u*v^2+F(13)/89*z^2*u*v^2+F(20)/121*x*u^2*v^2+295*y*u^2*v^2+F(89)/44*z*u^2*v^2-6010*x^2*v^3-5292*x*y*v^3+F(31)/66*y^2*v^3+4309*x*z*v^3+F(13)/101*y*z*v^3-4536*z^2*v^3+3224*x*u*v^3-9983*y*u*v^3+F(111)/29*z*u*v^3+8120*x*v^4-F(53)/72*y*v^4-F(47)/107*z*v^4,-F(33)/4*x^5-826*x^4*y-F(9)/98*x^3*y^2-6224*x^2*y^3-F(37)/126*x*y^4-2417*x^4*z+6033*x^3*y*z+F(64)/9*x^2*y^2*z+F(100)/91*x*y^3*z-F(51)/80*y^4*z-F(19)/83*x^3*z^2+F(97)/18*x^2*y*z^2-F(113)/117*x*y^2*z^2-13344*y^3*z^2+7943*x^2*z^3-7230*x*y*z^3+2774*y^2*z^3-F(9)/2*x*z^4-F(32)/53*y*z^4-8924*z^5+14120*x^4*u+F(11)/93*x^3*y*u-F(43)/66*x^2*y^2*u-F(8)/69*x*y^3*u-10343*y^4*u+F(101)/45*x^3*z*u+F(101)/126*x^2*y*z*u-F(19)/56*x*y^2*z*u-F(21)/64*y^3*z*u+7692*x^2*z^2*u-F(72)/115*x*y*z^2*u-F(45)/112*y^2*z^2*u-F(10)/99*x*z^3*u-F(62)/19*y*z^3*u+F(53)/11*z^4*u+14914*x^3*u^2-10207*x^2*y*u^2-F(70)/83*x*y^2*u^2-F(83)/79*y^3*u^2-F(34)/21*x^2*z*u^2-F(79)/47*x*y*z*u^2+8578*y^2*z*u^2-F(38)/91*x*z^2*u^2-F(7)/27*y*z^2*u^2-F(43)/12*z^3*u^2-F(95)/78*x^2*u^3+F(111)/59*x*y*u^3-F(93)/47*y^2*u^3+F(125)/66*x*z*u^3+5449*y*z*u^3+1718*z^2*u^3+F(103)/86*x*u^4+1922*y*u^4+F(79)/48*z*u^4-8831*u^5+F(29)/76*x^4*v-9826*x^3*y*v+F(9)/52*x^2*y^2*v-F(40)/117*x*y^3*v+F(48)/79*x^3*z*v+F(121)/13*x^2*y*z*v+F(43)/73*x*y^2*z*v-14745*y^3*z*v+15376*x^2*z^2*v-F(81)/115*x*y*z^2*v-11080*y^2*z^2*v-13354*x*z^3*v-F(39)/29*y*z^3*v+5308*z^4*v+F(98)/17*x^3*u*v-F(41)/118*x^2*y*u*v-8606*x*y^2*u*v+2159*y^3*u*v-F(94)/59*x^2*z*u*v+F(62)/7*x*y*z*u*v+1087*y^2*z*u*v-F(59)/113*x*z^2*u*v-F(2)/71*y*z^2*u*v+1062*z^3*u*v-F(50)/49*x^2*u^2*v+F(33)/46*x*y*u^2*v-1237*y^2*u^2*v-12455*x*z*u^2*v-5798*y*z*u^2*v+F(57)/7*z^2*u^2*v-F(45)/7*x*u^3*v+F(29)/39*y*u^3*v+8559*z*u^3*v-F(33)/14*u^4*v+5548*x^3*v^2+12142*x^2*y*v^2+14917*x*y^2*v^2+10616*x^2*z*v^2-F(40)/87*x*y*z*v^2+13291*y^2*z*v^2-F(103)/94*x*z^2*v^2+1592*y*z^2*v^2+9855*z^3*v^2+3127*x^2*u*v^2-F(62)/77*x*y*u*v^2+3736*y^2*u*v^2-F(11)/69*x*z*u*v^2-9115*y*z*u*v^2+F(28)/25*z^2*u*v^2+F(23)/109*x*u^2*v^2-3745*y*u^2*v^2-1763*z*u^2*v^2-295*u^3*v^2+12507*x^2*v^3-F(58)/11*x*y*v^3-F(64)/53*x*z*v^3-4467*y*z*v^3+F(95)/52*z^2*v^3+F(69)/7*x*u*v^3-F(31)/66*y*u*v^3-F(88)/119*z*u*v^3+9983*u^2*v^3+F(115)/28*x*v^4-9324*z*v^4+F(53)/72*u*v^4,F(33)/4*x^4*y+826*x^3*y^2+F(9)/98*x^2*y^3+6224*x*y^4+F(37)/126*y^5-F(57)/82*x^4*z+F(33)/98*x^3*y*z+138*x^2*y^2*z+F(8)/77*x*y^3*z-7940*y^4*z-4749*x^3*z^2+F(107)/24*x^2*y*z^2-F(65)/113*x*y^2*z^2-F(32)/3*y^3*z^2-7186*x^2*z^3+9575*x*y*z^3+F(64)/113*y^2*z^3-3291*x*z^4-F(107)/30*y*z^4+F(40)/3*z^5+F(26)/47*x^4*u+F(121)/45*x^3*y*u-F(42)/17*x^2*y^2*u-F(95)/109*x*y^3*u+F(125)/3*y^4*u-10830*x^3*z*u-14808*x^2*y*z*u-F(84)/19*x*y^2*z*u+F(93)/8*y^3*z*u-7861*x^2*z^2*u-F(5)/87*x*y*z^2*u-F(101)/70*y^2*z^2*u+F(41)/63*x*z^3*u+12673*y*z^3*u+14255*z^4*u+F(107)/22*x^3*u^2+F(6)/97*x^2*y*u^2+12293*x*y^2*u^2-F(16)/95*y^3*u^2-F(80)/33*x^2*z*u^2-7306*x*y*z*u^2+F(123)/62*y^2*z*u^2+F(77)/60*x*z^2*u^2+F(101)/54*y*z^2*u^2-F(57)/56*z^3*u^2+F(58)/19*x^2*u^3+F(83)/94*x*y*u^3-12417*y^2*u^3+11778*x*z*u^3+F(82)/115*y*z*u^3-11546*z^2*u^3-F(107)/39*x*u^4+F(67)/49*y*u^4-F(22)/45*z*u^4-3013*u^5-F(29)/76*x^3*y*v+9826*x^2*y^2*v-F(9)/52*x*y^3*v+F(40)/117*y^4*v+F(77)/25*x^3*z*v+11411*x^2*y*z*v+F(65)/23*x*y^2*z*v-13903*y^3*z*v+F(2)/47*x^2*z^2*v-7559*x*y*z^2*v-F(103)/24*y^2*z^2*v-6041*x*z^3*v-F(101)/111*y*z^3*v-F(42)/47*z^4*v-F(119)/20*x^3*u*v-F(62)/81*x^2*y*u*v+12141*x*y^2*u*v-F(31)/27*y^3*u*v-F(80)/103*x^2*z*u*v+F(75)/67*x*y*z*u*v+F(39)/110*y^2*z*u*v-F(74)/87*x*z^2*u*v-2451*y*z^2*u*v+F(119)/12*z^3*u*v-F(93)/34*x^2*u^2*v+15718*x*y*u^2*v-F(32)/9*y^2*u^2*v-F(37)/51*x*z*u^2*v-F(23)/98*y*z*u^2*v+F(97)/71*z^2*u^2*v-14180*x*u^3*v-F(108)/41*y*u^3*v+F(99)/38*z*u^3*v+F(94)/105*u^4*v-5548*x^2*y*v^2-12142*x*y^2*v^2-14917*y^3*v^2+14070*x^2*z*v^2-6682*x*y*z*v^2-9676*y^2*z*v^2-9166*x*z^2*v^2-13919*y*z^2*v^2-F(101)/121*z^3*v^2+2772*x^2*u*v^2+F(123)/52*x*y*u*v^2+F(125)/87*y^2*u*v^2+4843*x*z*u*v^2+F(73)/94*y*z*u*v^2+F(5)/33*z^2*u*v^2-4418*x*u^2*v^2-2754*y*u^2*v^2+14191*z*u^2*v^2-F(20)/121*u^3*v^2-12507*x*y*v^3+F(58)/11*y^2*v^3+F(80)/103*x*z*v^3+F(15)/19*y*z*v^3+12289*z^2*v^3+6010*x*u*v^3+F(49)/45*y*u*v^3+F(63)/110*z*u*v^3-3224*u^2*v^3-F(115)/28*y*v^4-F(97)/68*z*v^4-8120*u*v^4,F(57)/82*x^5+8619*x^4*y-6171*x^3*y^2-F(65)/41*x^2*y^3+8642*x*y^4+F(51)/80*y^5+4749*x^4*z-10427*x^3*y*z-F(66)/115*x^2*y^2*z+13683*x*y^3*z+13344*y^4*z+7186*x^3*z^2+F(37)/42*x^2*y*z^2-F(84)/61*x*y^2*z^2-2774*y^3*z^2+3291*x^2*z^3+F(121)/15*x*y*z^3+F(32)/53*y^2*z^3-F(40)/3*x*z^4+8924*y*z^4+14255*x^4*u-F(85)/106*x^3*y*u+F(126)/55*x^2*y^2*u+F(60)/53*x*y^3*u-4600*y^4*u-5357*x^3*z*u+12015*x^2*y*z*u+3318*x*y^2*z*u+F(63)/94*y^3*z*u+1832*x^2*z^2*u-6177*x*y*z^2*u-F(19)/48*y^2*z^2*u-F(10)/27*x*z^3*u+F(83)/47*y*z^3*u-10158*z^4*u-F(77)/104*x^3*u^2+5102*x^2*y*u^2+F(16)/47*x*y^2*u^2-F(72)/71*y^3*u^2-F(49)/41*x^2*z*u^2+F(99)/85*x*y*z*u^2-F(64)/119*y^2*z*u^2-9435*x*z^2*u^2+F(91)/6*y*z^2*u^2-F(5)/2*z^3*u^2-F(83)/92*x^2*u^3-F(19)/47*x*y*u^3+5929*y^2*u^3-F(62)/63*x*z*u^3-F(56)/3*y*z*u^3+F(118)/47*z^2*u^3+3104*x*u^4+F(10)/51*y*u^4-F(121)/87*z*u^4+4274*u^5-F(77)/25*x^4*v+F(33)/116*x^3*y*v-2152*x^2*y^2*v-7571*x*y^3*v+14745*y^4*v-F(2)/47*x^3*z*v-7817*x^2*y*z*v-F(13)/125*x*y^2*z*v+11080*y^3*z*v+6041*x^2*z^2*v+962*x*y*z^2*v+F(39)/29*y^2*z^2*v+F(42)/47*x*z^3*v-5308*y*z^3*v-F(115)/38*x^3*u*v+F(11)/91*x^2*y*u*v+F(2)/119*x*y^2*u*v+F(9)/113*y^3*u*v-F(125)/107*x^2*z*u*v-F(73)/63*x*y*z*u*v+F(12)/23*y^2*z*u*v+F(53)/93*x*z^2*u*v+11665*y*z^2*u*v+F(110)/41*z^3*u*v+11697*x^2*u^2*v-F(77)/124*x*y*u^2*v+F(115)/108*y^2*u^2*v-F(98)/73*x*z*u^2*v-4748*y*z*u^2*v+9107*z^2*u^2*v-F(83)/58*x*u^3*v-F(83)/114*y*u^3*v+F(80)/27*z*u^3*v-15383*u^4*v-14070*x^3*v^2-F(83)/122*x^2*y*v^2+F(49)/48*x*y^2*v^2-13291*y^3*v^2+9166*x^2*z*v^2+F(79)/18*x*y*z*v^2-1592*y^2*z*v^2+F(101)/121*x*z^2*v^2-9855*y*z^2*v^2-15644*x^2*u*v^2+14034*x*y*u*v^2+F(32)/83*y^2*u*v^2-9175*x*z*u*v^2-F(83)/82*y*z*u*v^2-F(106)/43*z^2*u*v^2-F(52)/75*x*u^2*v^2-F(111)/92*y*u^2*v^2-F(13)/89*z*u^2*v^2-F(89)/44*u^3*v^2-F(80)/103*x^2*v^3+F(51)/46*x*y*v^3+4467*y^2*v^3-12289*x*z*v^3-F(95)/52*y*z*v^3-F(5)/121*x*u*v^3+F(15)/103*y*u*v^3+4536*z*u*v^3-F(111)/29*u^2*v^3+F(97)/68*x*v^4+9324*y*v^4+F(47)/107*u*v^4,6109*x^5+F(57)/110*x^4*y-2432*x^3*y^2-F(118)/19*x^2*y^3+7203*x*y^4-650*x^4*z+F(101)/116*x^3*y*z-F(77)/79*x^2*y^2*z+F(115)/79*x*y^3*z+F(81)/10*x^3*z^2-2750*x^2*y*z^2-F(37)/59*x*y^2*z^2+F(119)/121*x^2*z^3+F(25)/73*x*y*z^3-F(101)/69*x*z^4+F(35)/38*x^4*u-F(81)/20*x^3*y*u+2010*x^2*y^2*u-F(29)/15*x*y^3*u-F(95)/69*y^4*u-7394*x^3*z*u-9808*x^2*y*z*u+F(100)/71*x*y^2*z*u-F(37)/91*y^3*z*u-F(86)/103*x^2*z^2*u+F(47)/78*x*y*z^2*u+1497*y^2*z^2*u-F(39)/23*x*z^3*u+12979*y*z^3*u-F(123)/97*z^4*u+F(23)/82*x^3*u^2+F(74)/87*x^2*y*u^2+F(41)/43*x*y^2*u^2+F(19)/96*y^3*u^2-F(61)/110*x^2*z*u^2-F(83)/28*x*y*z*u^2-F(95)/4*y^2*z*u^2-F(81)/83*x*z^2*u^2-13515*y*z^2*u^2+9715*z^3*u^2-10041*x^2*u^3-7653*x*y*u^3-F(121)/98*y^2*u^3+14412*x*z*u^3-F(68)/107*y*z*u^3-4021*z^2*u^3+11316*x*u^4-F(1)/78*y*u^4-13971*z*u^4+F(103)/120*u^5+5557*x^4*v-8199*x^3*y*v+1722*x^2*y^2*v+F(4)/83*x*y^3*v+F(51)/80*y^4*v+12829*x^3*z*v-9957*x^2*y*z*v-11222*x*y^2*z*v+13344*y^3*z*v-F(53)/48*x^2*z^2*v+F(123)/113*x*y*z^2*v-2774*y^2*z^2*v+F(105)/4*x*z^3*v+F(32)/53*y*z^3*v+8924*z^4*v-F(107)/57*x^3*u*v+7149*x^2*y*u*v-F(25)/42*x*y^2*u*v-F(7)/65*y^3*u*v+F(85)/96*x^2*z*u*v+F(12)/13*x*y*z*u*v-F(86)/49*y^2*z*u*v+F(23)/70*x*z^2*u*v+10614*y*z^2*u*v-F(38)/59*z^3*u*v+F(110)/37*x^2*u^2*v+F(71)/66*x*y*u^2*v+5355*y^2*u^2*v+2649*x*z*u^2*v+14560*y*z*u^2*v-F(82)/105*z^2*u^2*v-F(19)/43*x*u^3*v-F(36)/89*y*u^3*v-F(76)/25*z*u^3*v+F(73)/39*u^4*v-F(112)/13*x^3*v^2+F(31)/10*x^2*y*v^2+F(94)/9*x*y^2*v^2+14745*y^3*v^2+F(85)/24*x^2*z*v^2+F(70)/23*x*y*z*v^2+11080*y^2*z*v^2+1072*x*z^2*v^2+F(39)/29*y*z^2*v^2-5308*z^3*v^2-F(53)/103*x^2*u*v^2-F(66)/43*x*y*u*v^2+12706*y^2*u*v^2-F(16)/5*x*z*u*v^2-2513*y*z*u*v^2+12180*z^2*u*v^2+F(113)/74*x*u^2*v^2-F(95)/24*y*u^2*v^2+F(43)/112*z*u^2*v^2+8034*u^3*v^2-F(47)/76*x^2*v^3+F(125)/58*x*y*v^3-13291*y^2*v^3-F(119)/110*x*z*v^3-1592*y*z*v^3-9855*z^2*v^3-F(1)/91*x*u*v^3+F(101)/109*y*u*v^3+F(82)/65*z*u*v^3+F(71)/17*u^2*v^3+F(109)/66*x*v^4+4467*y*v^4-F(95)/52*z*v^4-F(79)/49*u*v^4+9324*v^5,-6109*x^4*y-F(57)/110*x^3*y^2+2432*x^2*y^3+F(118)/19*x*y^4-7203*y^5+650*x^3*y*z-F(101)/116*x^2*y^2*z+F(77)/79*x*y^3*z-F(115)/79*y^4*z-F(81)/10*x^2*y*z^2+2750*x*y^2*z^2+F(37)/59*y^3*z^2-F(119)/121*x*y*z^3-F(25)/73*y^2*z^3+F(101)/69*y*z^4-F(119)/23*x^4*u+F(16)/53*x^3*y*u+11302*x^2*y^2*u-F(109)/83*x*y^3*u+F(81)/49*y^4*u+497*x^3*z*u-5440*x^2*y*z*u+F(37)/126*x*y^2*z*u+6113*y^3*z*u+12829*x^2*z^2*u-14528*x*y*z^2*u-F(60)/91*y^2*z^2*u+5165*x*z^3*u+10606*y*z^3*u-F(7)/111*z^4*u+F(111)/2*x^3*u^2-F(83)/25*x^2*y*u^2-F(123)/34*x*y^2*u^2-12479*y^3*u^2-F(2)/13*x^2*z*u^2+F(15)/113*x*y*z*u^2+2719*y^2*z*u^2-F(4)/101*x*z^2*u^2-F(110)/51*y*z^2*u^2-424*z^3*u^2+F(91)/51*x^2*u^3+F(54)/43*x*y*u^3+F(16)/17*y^2*u^3-F(110)/103*x*z*u^3+8769*y*z*u^3-1071*z^2*u^3+F(101)/103*x*u^4-2651*y*u^4+F(83)/96*z*u^4+F(8)/97*u^5+F(57)/82*x^4*v-F(91)/94*x^3*y*v+2028*x^2*y^2*v-8746*x*y^3*v+F(31)/88*y^4*v+4749*x^3*z*v+F(112)/11*x^2*y*z*v-12020*x*y^2*z*v-7086*y^3*z*v+7186*x^2*z^2*v+F(10)/53*x*y*z^2*v+14306*y^2*z^2*v+3291*x*z^3*v-F(61)/62*y*z^3*v-F(40)/3*z^4*v-7860*x^3*u*v+F(50)/33*x^2*y*u*v-7851*x*y^2*u*v+F(73)/23*y^3*u*v+15344*x^2*z*u*v+F(101)/70*x*y*z*u*v-15042*y^2*z*u*v-10050*x*z^2*u*v-F(79)/126*y*z^2*u*v+F(6)/49*z^3*u*v-F(26)/23*x^2*u^2*v-1287*x*y*u^2*v-F(90)/91*y^2*u^2*v-9170*x*z*u^2*v-F(1)/47*y*z*u^2*v-15914*z^2*u^2*v+2176*x*u^3*v-F(2)/77*y*u^3*v+F(47)/28*z*u^3*v+3004*u^4*v-F(77)/25*x^3*v^2-F(104)/5*x^2*y*v^2+F(75)/92*x*y^2*v^2-11136*y^3*v^2-F(2)/47*x^2*z*v^2+6842*x*y*z*v^2-F(86)/45*y^2*z*v^2+6041*x*z^2*v^2-110*y*z^2*v^2+F(42)/47*z^3*v^2-14637*x^2*u*v^2+15042*x*y*u*v^2+5772*y^2*u*v^2-F(58)/123*x*z*u*v^2-10862*y*z*u*v^2+F(72)/121*z^2*u*v^2+F(21)/122*x*u^2*v^2+F(81)/14*y*u^2*v^2+15903*z*u^2*v^2-F(123)/37*u^3*v^2-14070*x^2*v^3+F(117)/5*x*y*v^3+13903*y^2*v^3+9166*x*z*v^3-F(79)/21*y*z*v^3+F(101)/121*z^2*v^3+F(98)/87*x*u*v^3-309*y*u*v^3+10614*z*u*v^3+3357*u^2*v^3-F(80)/103*x*v^4+F(104)/119*y*v^4-12289*z*v^4-F(68)/83*u*v^4+F(97)/68*v^5,-6109*x^4*z-F(57)/110*x^3*y*z+2432*x^2*y^2*z+F(118)/19*x*y^3*z-7203*y^4*z+650*x^3*z^2-F(101)/116*x^2*y*z^2+F(77)/79*x*y^2*z^2-F(115)/79*y^3*z^2-F(81)/10*x^2*z^3+2750*x*y*z^3+F(37)/59*y^2*z^3-F(119)/121*x*z^4-F(25)/73*y*z^4+F(101)/69*z^5-F(87)/61*x^4*u+14779*x^3*y*u+F(34)/113*x^2*y^2*u+F(58)/65*x*y^3*u-3183*y^4*u-4024*x^3*z*u+F(100)/33*x^2*y*z*u-6571*x*y^2*z*u+F(62)/69*y^3*z*u-F(94)/43*x^2*z^2*u+15504*x*y*z^2*u+F(83)/85*y^2*z^2*u+11613*x*z^3*u+F(36)/113*y*z^3*u+F(98)/25*z^4*u+8140*x^3*u^2-F(17)/101*x^2*y*u^2+F(67)/13*x*y^2*u^2-F(26)/125*y^3*u^2+10160*x^2*z*u^2-5605*x*y*z*u^2-1622*y^2*z*u^2-12896*x*z^2*u^2-F(67)/32*y*z^2*u^2-F(33)/52*z^3*u^2-F(101)/23*x^2*u^3-12833*x*y*u^3-F(94)/51*y^2*u^3-F(14)/81*x*z*u^3-5623*y*z*u^3+F(118)/119*z^2*u^3+8051*x*u^4-F(105)/107*y*u^4+F(78)/53*z*u^4-F(86)/47*u^5+F(33)/4*x^4*v+826*x^3*y*v+F(9)/98*x^2*y^2*v+6224*x*y^3*v+F(37)/126*y^4*v-3140*x^3*z*v+2166*x^2*y*z*v+F(111)/113*x*y^2*z*v-F(33)/62*y^3*z*v-8589*x^2*z^2*v-F(103)/90*x*y*z^2*v+F(112)/9*y^2*z^2*v-F(37)/59*x*z^3*v+F(4)/15*y*z^3*v-F(87)/4*z^4*v+2653*x^3*u*v+9854*x^2*y*u*v+F(18)/95*x*y^2*u*v-F(113)/24*y^3*u*v+4598*x^2*z*u*v-455*x*y*z*u*v-15544*y^2*z*u*v-15895*x*z^2*u*v+F(108)/103*y*z^2*u*v+F(37)/115*z^3*u*v-F(106)/35*x^2*u^2*v+F(5)/79*x*y*u^2*v-F(67)/56*y^2*u^2*v+14353*x*z*u^2*v-1971*y*z*u^2*v+F(34)/103*z^2*u^2*v-F(3)/16*x*u^3*v+10862*y*u^3*v+F(29)/32*z*u^3*v-14252*u^4*v-F(29)/76*x^3*v^2+9826*x^2*y*v^2-F(9)/52*x*y^2*v^2+F(40)/117*y^3*v^2+4992*x^2*z*v^2-1735*x*y*z*v^2-F(59)/23*y^2*z*v^2-717*x*z^2*v^2+F(70)/89*y*z^2*v^2-F(29)/112*z^3*v^2+13492*x^2*u*v^2+13849*x*y*u*v^2+F(46)/7*y^2*u*v^2-13963*x*z*u*v^2-12448*y*z*u*v^2+10795*z^2*u*v^2+F(53)/94*x*u^2*v^2-558*y*u^2*v^2+F(101)/122*z*u^2*v^2+10333*u^3*v^2-5548*x^2*v^3-12142*x*y*v^3-14917*y^2*v^3-F(75)/74*x*z*v^3+F(94)/53*y*z*v^3+F(47)/50*z^2*v^3-3581*x*u*v^3-1038*y*u*v^3-F(118)/45*z*u*v^3-8388*u^2*v^3-12507*x*v^4+F(58)/11*y*v^4+F(19)/122*z*v^4+F(91)/97*u*v^4-F(115)/28*v^5,-F(87)/61*x^5+14779*x^4*y+F(34)/113*x^3*y^2+F(58)/65*x^2*y^3-3183*x*y^4+1870*x^4*z+F(124)/21*x^3*y*z+F(64)/7*x^2*y^2*z-F(119)/115*x*y^3*z-F(95)/69*y^4*z-13348*x^3*z^2-F(75)/73*x^2*y*z^2+F(14)/97*x*y^2*z^2-F(37)/91*y^3*z^2+F(42)/83*x^2*z^3-F(91)/47*x*y*z^3+1497*y^2*z^3-F(29)/62*x*z^4+12979*y*z^4-F(123)/97*z^5+8140*x^4*u-F(17)/101*x^3*y*u+F(67)/13*x^2*y^2*u-F(26)/125*x*y^3*u+3528*x^3*z*u-F(7)/104*x^2*y*z*u+F(89)/72*x*y^2*z*u+F(19)/96*y^3*z*u-F(106)/33*x^2*z^2*u+F(25)/108*x*y*z^2*u-F(95)/4*y^2*z^2*u+10709*x*z^3*u-13515*y*z^3*u+9715*z^4*u-F(101)/23*x^3*u^2-12833*x^2*y*u^2-F(94)/51*x*y^2*u^2+F(13)/7*x^2*z*u^2-13276*x*y*z*u^2-F(121)/98*y^2*z*u^2+F(18)/5*x*z^2*u^2-F(68)/107*y*z^2*u^2-4021*z^3*u^2+8051*x^2*u^3-F(105)/107*x*y*u^3-151*x*z*u^3-F(1)/78*y*z*u^3-13971*z^2*u^3-F(86)/47*x*u^4+F(103)/120*z*u^4+F(105)/103*x^4*v+13638*x^3*y*v+6311*x^2*y^2*v-13856*x*y^3*v-10343*y^4*v-1725*x^3*z*v-1176*x^2*y*z*v-4310*x*y^2*z*v-1823*y^3*z*v-F(15)/124*x^2*z^2*v-10902*x*y*z^2*v-F(39)/37*y^2*z^2*v-F(119)/87*x*z^3*v-F(31)/43*y*z^3*v-2214*z^4*v+15825*x^3*u*v-1703*x^2*y*u*v+7920*x*y^2*u*v-F(83)/79*y^3*u*v-F(36)/119*x^2*z*u*v+F(121)/115*x*y*z*u*v+F(89)/62*y^2*z*u*v-11396*x*z^2*u*v-11507*y*z^2*u*v-F(125)/81*z^3*u*v-6820*x^2*u^2*v+8695*x*y*u^2*v-F(93)/47*y^2*u^2*v+15101*x*z*u^2*v-F(86)/87*y*z*u^2*v+15791*z^2*u^2*v-F(3)/113*x*u^3*v+1922*y*u^3*v+7950*z*u^3*v-8831*u^4*v+325*x^3*v^2-F(45)/121*x^2*y*v^2+5111*x*y^2*v^2+2159*y^3*v^2+10598*x^2*z*v^2+7009*x*y*z*v^2+13793*y^2*z*v^2-80*x*z^2*v^2-F(5)/26*y*z^2*v^2+F(126)/29*z^3*v^2-4647*x^2*u*v^2+2920*x*y*u*v^2-1237*y^2*u*v^2+F(1)/9*x*z*u*v^2-4469*y*z*u*v^2-12845*z^2*u*v^2-7954*x*u^2*v^2+F(29)/39*y*u^2*v^2-15398*z*u^2*v^2-F(33)/14*u^3*v^2-454*x^2*v^3-F(3)/22*x*y*v^3+3736*y^2*v^3+12099*x*z*v^3-13810*y*z*v^3-F(77)/9*z^2*v^3+5113*x*u*v^3-3745*y*u*v^3+123*z*u*v^3-295*u^2*v^3-F(58)/47*x*v^4-F(31)/66*y*v^4+14745*z*v^4+9983*u*v^4+F(53)/72*v^5,F(87)/61*x^4*y-14779*x^3*y^2-F(34)/113*x^2*y^3-F(58)/65*x*y^4+3183*y^5-F(119)/23*x^4*z-12273*x^3*y*z+F(113)/78*x^2*y^2*z+F(21)/68*x*y^3*z+F(26)/47*y^4*z+497*x^3*z^2-F(123)/62*x^2*y*z^2-16*x*y^2*z^2+F(52)/59*y^3*z^2+12829*x^2*z^3+5850*x*y*z^3-5128*y^2*z^3+5165*x*z^4+14441*y*z^4-F(7)/111*z^5-8140*x^3*y*u+F(17)/101*x^2*y^2*u-F(67)/13*x*y^3*u+F(26)/125*y^4*u+F(111)/2*x^3*z*u+6472*x^2*y*z*u-F(43)/65*x*y^2*z*u-10857*y^3*z*u-F(2)/13*x^2*z^2*u-11451*x*y*z^2*u+F(53)/7*y^2*z^2*u-F(4)/101*x*z^3*u+F(104)/3*y*z^3*u-424*z^4*u+F(101)/23*x^2*y*u^2+12833*x*y^2*u^2+F(94)/51*y^3*u^2+F(91)/51*x^2*z*u^2-6428*x*y*z*u^2-5667*y^2*z*u^2-F(110)/103*x*z^2*u^2+10381*y*z^2*u^2-1071*z^3*u^2-8051*x*y*u^3+F(105)/107*y^2*u^3+F(101)/103*x*z*u^3+8816*y*z*u^3+F(83)/96*z^2*u^3+F(86)/47*y*u^4+F(8)/97*z*u^4+F(26)/47*x^4*v+10857*x^3*y*v-F(63)/38*x^2*y^2*v-F(67)/2*x*y^3*v-F(57)/85*y^4*v+F(122)/89*x^3*z*v-16*x^2*y*z*v-F(104)/71*x*y^2*z*v+F(111)/116*y^3*z*v+7483*x^2*z^2*v-11141*x*y*z^2*v-10816*y^2*z^2*v-5987*x*z^3*v+F(74)/17*y*z^3*v-F(44)/103*z^4*v+F(107)/22*x^3*u*v-F(35)/71*x^2*y*u*v+F(45)/76*x*y^2*u*v+12154*y^3*u*v+14875*x^2*z*u*v+9045*x*y*z*u*v-F(20)/49*y^2*z*u*v-F(87)/62*x*z^2*u*v-F(124)/43*y*z^2*u*v-7346*z^3*u*v+F(58)/19*x^2*u^2*v+F(72)/107*x*y*u^2*v+8712*y^2*u^2*v+F(45)/94*x*z*u^2*v-F(115)/109*y*z*u^2*v+F(99)/76*z^2*u^2*v-F(107)/39*x*u^3*v-15779*y*u^3*v-F(58)/53*z*u^3*v-3013*u^4*v-F(119)/20*x^3*v^2-3619*x^2*y*v^2-1708*x*y^2*v^2-14903*y^3*v^2+F(31)/7*x^2*z*v^2-11102*x*y*z*v^2+F(31)/74*y^2*z*v^2+F(79)/42*x*z^2*v^2+F(80)/69*y*z^2*v^2-11909*z^3*v^2-F(93)/34*x^2*u*v^2-F(29)/5*x*y*u*v^2+F(7)/109*y^2*u*v^2+F(78)/17*x*z*u*v^2-F(32)/85*y*z*u*v^2+F(98)/125*z^2*u*v^2-14180*x*u^2*v^2-F(87)/37*y*u^2*v^2+10443*z*u^2*v^2+F(94)/105*u^3*v^2+2772*x^2*v^3+5429*x*y*v^3-F(119)/100*y^2*v^3+1167*x*z*v^3-F(65)/124*y*z*v^3-F(34)/79*z^2*v^3-4418*x*u*v^3+5634*y*u*v^3-14443*z*u*v^3-F(20)/121*u^2*v^3+6010*x*v^4-4236*y*v^4-F(53)/16*z*v^4-3224*u*v^4-8120*v^5,F(119)/23*x^5-F(71)/123*x^4*y+F(65)/31*x^3*y^2-F(125)/63*x^2*y^3-1349*x*y^4+F(95)/69*y^5-497*x^4*z+12834*x^3*y*z+F(69)/107*x^2*y^2*z+F(4)/7*x*y^3*z+F(37)/91*y^4*z-12829*x^3*z^2-69*x^2*y*z^2+F(119)/32*x*y^2*z^2-1497*y^3*z^2-5165*x^2*z^3-F(88)/61*x*y*z^3-12979*y^2*z^3+F(7)/111*x*z^4+F(123)/97*y*z^4-F(111)/2*x^4*u-F(45)/16*x^3*y*u+8060*x^2*y^2*u+13966*x*y^3*u-F(19)/96*y^4*u+F(2)/13*x^3*z*u+F(81)/70*x^2*y*z*u-F(21)/122*x*y^2*z*u+F(95)/4*y^3*z*u+F(4)/101*x^2*z^2*u-80*x*y*z^2*u+13515*y^2*z^2*u+424*x*z^3*u-9715*y*z^3*u-F(91)/51*x^3*u^2+F(100)/69*x^2*y*u^2+F(73)/76*x*y^2*u^2+F(121)/98*y^3*u^2+F(110)/103*x^2*z*u^2+F(61)/69*x*y*z*u^2+F(68)/107*y^2*z*u^2+1071*x*z^2*u^2+4021*y*z^2*u^2-F(101)/103*x^2*u^3-F(37)/48*x*y*u^3+F(1)/78*y^2*u^3-F(83)/96*x*z*u^3+13971*y*z*u^3-F(8)/97*x*u^4-F(103)/120*y*u^4-9876*x^4*v-2811*x^3*y*v+F(35)/24*x^2*y^2*v-F(85)/108*x*y^3*v-F(95)/119*y^4*v-F(16)/17*x^3*z*v+F(20)/41*x^2*y*z*v-F(44)/89*x*y^2*z*v-963*y^3*z*v-F(13)/35*x^2*z^2*v+F(39)/85*x*y*z^2*v+F(62)/123*y^2*z^2*v-F(112)/31*x*z^3*v-F(87)/74*y*z^3*v-10158*z^4*v+F(19)/83*x^3*u*v-F(19)/101*x^2*y*u*v+F(28)/59*x*y^2*u*v+6359*y^3*u*v+6828*x^2*z*u*v+5416*x*y*z*u*v+10172*y^2*z*u*v-F(15)/79*x*z^2*u*v+F(89)/82*y*z^2*u*v-F(5)/2*z^3*u*v+F(35)/111*x^2*u^2*v-F(52)/81*x*y*u^2*v+F(15)/109*y^2*u^2*v-F(39)/110*x*z*u^2*v-F(11)/28*y*z*u^2*v+F(118)/47*z^2*u^2*v+100*x*u^3*v-8301*y*u^3*v-F(121)/87*z*u^3*v+4274*u^4*v+F(12)/115*x^3*v^2+F(70)/69*x^2*y*v^2-F(37)/112*x*y^2*v^2+F(31)/26*y^3*v^2-F(122)/53*x^2*z*v^2-F(96)/35*x*y*z*v^2-F(36)/109*y^2*z*v^2-F(16)/71*x*z^2*v^2+F(61)/62*y*z^2*v^2+F(110)/41*z^3*v^2-5872*x^2*u*v^2+F(6)/29*x*y*u*v^2-2809*y^2*u*v^2-F(103)/83*x*z*u*v^2+3535*y*z*u*v^2+9107*z^2*u*v^2-F(119)/71*x*u^2*v^2+F(83)/104*y*u^2*v^2+F(80)/27*z*u^2*v^2-15383*u^3*v^2-11968*x^2*v^3-10617*x*y*v^3-13420*y^2*v^3-F(73)/97*x*z*v^3+F(79)/121*y*z*v^3-F(106)/43*z^2*v^3-F(22)/23*x*u*v^3-F(28)/47*y*u*v^3-F(13)/89*z*u*v^3-F(89)/44*u^2*v^3-F(107)/59*x*v^4+F(123)/88*y*v^4+4536*z*v^4-F(111)/29*u*v^4+F(47)/107*v^5,-x^5+F(99)/80*x^4*y+F(76)/117*x^3*y^2-9027*x^2*y^3+10114*x*y^4-4276*y^5+F(43)/23*x^4*z-F(123)/16*x^3*y*z+F(97)/69*x^2*y^2*z-F(107)/104*x*y^3*z+4749*y^4*z+3573*x^3*z^2-13541*x^2*y*z^2+474*x*y^2*z^2+F(27)/115*y^3*z^2+F(48)/7*x^2*z^3+15412*x*y*z^3+F(125)/4*y^2*z^3+F(77)/26*x*z^4+F(67)/23*y*z^4+F(89)/71*x^4*u+F(49)/86*x^3*y*u+F(99)/65*x^2*y^2*u-10979*x*y^3*u+F(100)/109*y^4*u-F(87)/38*x^3*z*u-F(27)/104*x^2*y*z*u-F(5)/89*x*y^2*z*u-F(49)/40*y^3*z*u+F(79)/66*x^2*z^2*u-F(118)/39*x*y*z^2*u-5466*y^2*z^2*u-6086*x*z^3*u-7305*y*z^3*u+85*x^3*u^2-F(44)/81*x^2*y*u^2+F(37)/23*x*y^2*u^2+7086*y^3*u^2+F(62)/111*x^2*z*u^2-8463*x*y*z*u^2-F(57)/121*y^2*z*u^2-F(107)/99*x*z^2*u^2-F(27)/52*y*z^2*u^2+F(25)/92*x^2*u^3+F(121)/113*x*y*u^3-F(55)/94*y^2*u^3+F(51)/28*x*z*u^3-F(97)/33*y*z*u^3-F(34)/67*x*u^4+F(95)/63*y*u^4+8355*x^4*v+F(16)/93*x^3*y*v+F(85)/79*x^2*y^2*v-3848*x*y^3*v-12508*y^4*v+F(107)/81*x^3*z*v-9201*x^2*y*z*v-F(21)/125*x*y^2*z*v+F(89)/61*y^3*z*v+F(55)/17*x^2*z^2*v-1082*x*y*z^2*v-2410*y^2*z^2*v+F(74)/109*x*z^3*v-F(58)/5*y*z^3*v+F(119)/40*z^4*v-7838*x^3*u*v-F(41)/59*x^2*y*u*v-F(104)/125*x*y^2*u*v+F(37)/29*y^3*u*v+F(2)/29*x^2*z*u*v-F(116)/75*x*y*z*u*v-F(29)/103*y^2*z*u*v+12198*x*z^2*u*v+12840*y*z^2*u*v-F(104)/5*z^3*u*v+F(53)/38*x^2*u^2*v-5988*x*y*u^2*v+F(49)/69*y^2*u^2*v-F(101)/119*x*z*u^2*v-F(74)/33*y*z*u^2*v+F(5)/41*z^2*u^2*v+11414*x*u^3*v+15411*y*u^3*v+F(103)/80*z*u^3*v-F(113)/68*u^4*v+F(112)/75*x^3*v^2-5832*x^2*y*v^2+1327*x*y^2*v^2-F(111)/43*y^3*v^2-F(111)/122*x^2*z*v^2-5305*x*y*z*v^2+2576*y^2*z*v^2+F(57)/16*x*z^2*v^2-F(30)/13*y*z^2*v^2-3089*z^3*v^2-10173*x^2*u*v^2-12356*x*y*u*v^2+F(75)/52*y^2*u*v^2-F(22)/81*x*z*u*v^2+F(24)/13*y*z*u*v^2+F(107)/43*z^2*u*v^2-F(121)/46*x*u^2*v^2-F(43)/99*y*u^2*v^2+F(8)/89*z*u^2*v^2-F(37)/122*u^3*v^2+F(43)/26*x^2*v^3-F(88)/79*x*y*v^3-F(105)/124*y^2*v^3+2070*x*z*v^3-13963*y*z*v^3+F(108)/13*z^2*v^3-F(85)/118*x*u*v^3-F(7)/37*y*u*v^3-F(33)/59*z*u*v^3-F(103)/38*u^2*v^3+F(113)/93*x*v^4+F(80)/117*y*v^4+F(110)/51*z*v^4+F(45)/58*u*v^4+F(111)/13*v^5,-x^4*y+F(33)/91*x^3*y^2+F(89)/33*x^2*y^3+6237*x*y^4-F(51)/29*y^5-3540*x^4*z+F(39)/5*x^3*y*z+F(41)/46*x^2*y^2*z+11736*x*y^3*z+104*y^4*z-F(95)/83*x^3*z^2-F(99)/52*x^2*y*z^2+F(125)/4*x*y^2*z^2-235*y^3*z^2+11975*x^2*z^3-315*x*y*z^3+7263*y^2*z^3+15134*x*z^4-F(47)/40*y*z^4+F(101)/4*z^5+F(89)/71*x^3*y*u+F(100)/121*x^2*y^2*u-9325*x*y^3*u+2637*y^4*u+1734*x^3*z*u+F(113)/105*x^2*y*z*u+F(29)/64*x*y^2*z*u+14776*y^3*z*u-F(114)/5*x^2*z^2*u-F(35)/76*x*y*z^2*u-5145*y^2*z^2*u-F(95)/24*x*z^3*u+F(104)/27*y*z^3*u-F(69)/64*z^4*u+85*x^2*y*u^2+2255*x*y^2*u^2-F(16)/91*y^3*u^2-F(59)/69*x^2*z*u^2+10601*x*y*z*u^2-F(101)/99*y^2*z*u^2+14082*x*z^2*u^2+13591*y*z^2*u^2+F(35)/57*z^3*u^2+F(25)/92*x*y*u^3+F(71)/13*y^2*u^3-F(11)/120*x*z*u^3+6592*y*z*u^3-F(101)/90*z^2*u^3-F(34)/67*y*u^4+10618*z*u^4-F(116)/25*x^4*v-F(41)/80*x^3*y*v+F(68)/49*x^2*y^2*v-F(124)/125*x*y^3*v-F(83)/73*y^4*v+3261*x^3*z*v+9957*x^2*y*z*v-13017*x*y^2*z*v-F(65)/81*y^3*z*v-F(116)/77*x^2*z^2*v+F(79)/56*x*y*z^2*v+F(1)/37*y^2*z^2*v+F(106)/37*x*z^3*v+733*y*z^3*v+1530*z^4*v+F(115)/106*x^3*u*v+F(76)/31*x^2*y*u*v-F(58)/17*x*y^2*u*v+11932*y^3*u*v+10379*x^2*z*u*v-1709*x*y*z*u*v+F(62)/25*y^2*z*u*v+6476*x*z^2*u*v-F(46)/15*y*z^2*u*v-F(49)/111*z^3*u*v+F(103)/19*x^2*u^2*v-10919*x*y*u^2*v+7326*y^2*u^2*v-F(82)/47*x*z*u^2*v-F(13)/85*y*z*u^2*v-F(23)/40*z^2*u^2*v-3791*x*u^3*v-F(50)/59*y*u^3*v-F(31)/101*z*u^3*v-F(100)/33*u^4*v+F(63)/47*x^3*v^2-F(17)/87*x^2*y*v^2+4132*x*y^2*v^2-6961*y^3*v^2+F(68)/13*x^2*z*v^2-8230*x*y*z*v^2-436*y^2*z*v^2+F(74)/11*x*z^2*v^2-185*y*z^2*v^2-F(35)/52*z^3*v^2-F(79)/99*x^2*u*v^2-11024*x*y*u*v^2-F(3)/25*y^2*u*v^2-F(10)/59*x*z*u*v^2+F(51)/49*y*z*u*v^2+9771*z^2*u*v^2+F(108)/37*x*u^2*v^2+F(67)/5*y*u^2*v^2+12837*z*u^2*v^2+F(89)/90*u^3*v^2-F(75)/52*x^2*v^3+1131*x*y*v^3+14202*y^2*v^3-6904*x*z*v^3+4934*y*z*v^3-14954*z^2*v^3+F(50)/109*x*u*v^3+14105*y*u*v^3-F(122)/65*z*u*v^3-F(47)/82*u^2*v^3+F(48)/71*x*v^4-2491*y*v^4+8247*z*v^4-F(56)/69*u*v^4+F(57)/122*v^5,-x^4*z+F(33)/91*x^3*y*z+F(89)/33*x^2*y^2*z+6237*x*y^3*z-F(51)/29*y^4*z+F(43)/23*x^3*z^2-379*x^2*y*z^2+F(105)/44*x*y^2*z^2-5805*y^3*z^2+3573*x^2*z^3+F(8)/125*x*y*z^3+F(108)/37*y^2*z^3+F(48)/7*x*z^4+3847*y*z^4+F(77)/26*z^5+F(15)/52*x^4*u+F(17)/96*x^3*y*u+1733*x^2*y^2*u+F(93)/2*x*y^3*u+1634*y^4*u+8440*x^3*z*u+F(122)/3*x^2*y*z*u+F(90)/17*x*y^2*z*u+8618*y^3*z*u+3144*x^2*z^2*u+6630*x*y*z^2*u+F(116)/119*y^2*z^2*u+F(50)/11*x*z^3*u-F(107)/111*y*z^3*u+F(29)/28*z^4*u+F(9)/47*x^3*u^2+F(17)/14*x^2*y*u^2-2847*x*y^2*u^2-F(12)/17*y^3*u^2+15385*x^2*z*u^2-F(22)/9*x*y*z*u^2+F(50)/51*y^2*z*u^2+F(54)/89*x*z^2*u^2-9298*y*z^2*u^2-15111*z^3*u^2+F(107)/96*x^2*u^3-F(4)/35*x*y*u^3+12764*y^2*u^3-F(101)/113*x*z*u^3-F(22)/101*y*z*u^3-F(83)/36*z^2*u^3+9743*x*u^4-4592*y*u^4-F(108)/49*z*u^4+F(69)/95*u^5+4709*x^4*v+F(68)/113*x^3*y*v+F(82)/13*x^2*y^2*v+3579*x*y^3*v+F(30)/11*y^4*v-F(76)/111*x^3*z*v+4792*x^2*y*z*v+12598*x*y^2*z*v-5220*y^3*z*v-F(82)/15*x^2*z^2*v-F(8)/89*x*y*z^2*v+10724*y^2*z^2*v+F(93)/56*x*z^3*v+F(105)/97*y*z^3*v+F(65)/67*z^4*v-15884*x^3*u*v-10613*x^2*y*u*v+F(59)/43*x*y^2*u*v-F(116)/103*y^3*u*v-15774*x^2*z*u*v-11557*x*y*z*u*v+F(53)/12*y^2*z*u*v-F(35)/121*x*z^2*u*v+F(79)/65*y*z^2*u*v-F(75)/49*z^3*u*v+11383*x^2*u^2*v+13315*x*y*u^2*v-10989*y^2*u^2*v-9173*x*z*u^2*v+F(65)/18*y*z*u^2*v-F(101)/53*z^2*u^2*v-F(79)/122*x*u^3*v+F(91)/30*y*u^3*v-F(62)/7*z*u^3*v+11068*u^4*v+13770*x^3*v^2+F(89)/8*x^2*y*v^2-14441*x*y^2*v^2-7060*y^3*v^2+6554*x^2*z*v^2-9419*x*y*z*v^2-F(122)/63*y^2*z*v^2+F(125)/74*x*z^2*v^2+13043*y*z^2*v^2-15872*z^3*v^2+11654*x^2*u*v^2-F(67)/18*x*y*u*v^2+15711*y^2*u*v^2+11037*x*z*u*v^2+9266*y*z*u*v^2-F(56)/27*z^2*u*v^2-F(83)/108*x*u^2*v^2+3908*y*u^2*v^2-F(2)/29*z*u^2*v^2+F(77)/74*u^3*v^2+F(68)/109*x^2*v^3+11305*x*y*v^3-4536*y^2*v^3-F(123)/8*x*z*v^3-F(52)/75*y*z*v^3+F(53)/92*z^2*v^3+F(31)/56*x*u*v^3+11856*y*u*v^3+F(102)/47*z*u*v^3+F(21)/115*u^2*v^3-4810*x*v^4-F(34)/45*y*v^4-F(29)/25*z*v^4+F(4)/83*u*v^4-12014*v^5,12846*x^5+F(23)/63*x^4*y-F(88)/43*x^3*y^2+13657*x^2*y^3+F(75)/46*x*y^4-F(46)/17*y^5-3360*x^4*z-11796*x^3*y*z-F(109)/55*x^2*y^2*z+F(1)/61*x*y^3*z+13149*y^4*z-F(25)/77*x^3*z^2-F(92)/79*x^2*y*z^2+F(73)/11*x*y^2*z^2+4138*y^3*z^2-13318*x^2*z^3-12399*x*y*z^3+F(65)/89*y^2*z^3+10744*x*z^4-2474*y*z^4-4329*x^4*u-11963*x^3*y*u+2022*x^2*y^2*u-F(21)/104*x*y^3*u+13357*y^4*u-12376*x^3*z*u+F(41)/72*x^2*y*z*u-9237*x*y^2*z*u+8164*y^3*z*u+F(57)/88*x^2*z^2*u-F(19)/36*x*y*z^2*u-F(125)/67*y^2*z^2*u-F(26)/109*x*z^3*u-F(49)/81*y*z^3*u+15645*z^4*u-F(77)/32*x^3*u^2-15721*x^2*y*u^2-F(107)/74*x*y^2*u^2+13982*y^3*u^2-F(27)/86*x^2*z*u^2+F(55)/21*x*y*z*u^2-F(105)/34*y^2*z*u^2-13924*x*z^2*u^2+3420*y*z^2*u^2+F(1)/2*z^3*u^2+F(112)/111*x^2*u^3-1972*x*y*u^3+F(80)/39*y^2*u^3-14094*x*z*u^3+11002*y*z*u^3-F(103)/45*z^2*u^3-F(83)/78*x*u^4+15029*y*u^4-15617*z*u^4+3865*u^5+F(3)/38*x^4*v+222*x^3*y*v-F(29)/33*x^2*y^2*v-F(94)/49*x*y^3*v+F(25)/31*y^4*v+F(57)/59*x^3*z*v-3912*x^2*y*z*v-7697*x*y^2*z*v+F(85)/21*y^3*z*v-4486*x^2*z^2*v+5474*x*y*z^2*v+F(49)/41*y^2*z^2*v-10948*x*z^3*v-12507*y*z^3*v+F(109)/58*z^4*v+F(10)/43*x^3*u*v-F(29)/77*x^2*y*u*v-F(88)/5*x*y^2*u*v+4026*y^3*u*v-F(81)/7*x^2*z*u*v-F(25)/18*x*y*z*u*v+7304*y^2*z*u*v-F(107)/111*x*z^2*u*v-F(40)/29*y*z^2*u*v+9829*z^3*u*v-F(78)/23*x^2*u^2*v-F(103)/5*x*y*u^2*v+13640*y^2*u^2*v-1324*x*z*u^2*v+F(85)/104*y*z*u^2*v-F(77)/79*z^2*u^2*v+F(89)/44*x*u^3*v+F(105)/73*y*u^3*v+3799*z*u^3*v+8900*u^4*v-F(50)/3*x^3*v^2+F(71)/50*x^2*y*v^2-6063*x*y^2*v^2-F(4)/79*y^3*v^2+12607*x^2*z*v^2+F(46)/81*x*y*z*v^2-6363*y^2*z*v^2+F(24)/49*x*z^2*v^2+F(115)/91*y*z^2*v^2+F(53)/91*z^3*v^2+F(79)/97*x^2*u*v^2+F(25)/19*x*y*u*v^2-13345*y^2*u*v^2-F(5)/77*x*z*u*v^2-F(81)/71*y*z*u*v^2+F(33)/61*z^2*u*v^2-F(6)/121*x*u^2*v^2-2891*y*u^2*v^2-F(11)/100*z*u^2*v^2-F(95)/108*u^3*v^2-F(58)/55*x^2*v^3+F(84)/47*x*y*v^3-F(8)/63*y^2*v^3+8054*x*z*v^3+F(69)/86*y*z*v^3-F(57)/50*z^2*v^3-F(105)/121*x*u*v^3+F(29)/3*y*u*v^3-F(61)/125*z*u*v^3+F(17)/59*u^2*v^3-9646*x*v^4-F(44)/103*y*v^4-F(119)/79*z*v^4-10058*u*v^4+F(22)/69*v^5,12846*x^4*y-F(105)/47*x^3*y^2+F(121)/75*x^2*y^3-F(70)/11*x*y^4+11003*y^5+15629*x^4*z+3706*x^3*y*z-8040*x^2*y^2*z+4028*x*y^3*z+F(5)/109*y^4*z+F(77)/87*x^3*z^2-F(52)/51*x^2*y*z^2+4250*x*y^2*z^2+2145*y^3*z^2+7160*x^2*z^3-F(119)/59*x*y*z^3+F(48)/17*y^2*z^3+9014*x*z^4+F(110)/71*y*z^4-3539*z^5+F(11)/91*x^4*u-F(79)/35*x^3*y*u+11648*x^2*y^2*u-F(22)/111*x*y^3*u-12768*y^4*u-F(115)/74*x^3*z*u+F(20)/53*x^2*y*z*u-6039*x*y^2*z*u+2922*y^3*z*u+F(11)/52*x^2*z^2*u+F(14)/101*x*y*z^2*u+F(124)/99*y^2*z^2*u+4623*x*z^3*u-F(41)/111*y*z^3*u+1118*z^4*u+F(55)/12*x^3*u^2+F(91)/20*x^2*y*u^2+3448*x*y^2*u^2-1971*y^3*u^2+1236*x^2*z*u^2+F(80)/71*x*y*z*u^2+F(67)/85*y^2*z*u^2+8163*x*z^2*u^2-15644*y*z^2*u^2+F(122)/51*z^3*u^2+13856*x^2*u^3+F(72)/41*x*y*u^3-F(123)/14*y^2*u^3-F(110)/123*x*z*u^3-F(102)/7*y*z*u^3-F(44)/13*z^2*u^3-2272*x*u^4-829*y*u^4+F(56)/125*z*u^4+F(41)/4*u^5+F(78)/95*x^4*v+F(14)/43*x^3*y*v-F(40)/117*x^2*y^2*v+3980*x*y^3*v-13136*y^4*v-F(41)/109*x^3*z*v+10579*x^2*y*z*v-F(107)/17*x*y^2*z*v-F(31)/45*y^3*z*v-F(8)/59*x^2*z^2*v-F(23)/111*x*y*z^2*v+6865*y^2*z^2*v-F(36)/113*x*z^3*v-5383*y*z^3*v-14477*z^4*v+F(29)/91*x^3*u*v-7084*x^2*y*u*v+F(121)/8*x*y^2*u*v+F(53)/99*y^3*u*v+13983*x^2*z*u*v+1843*x*y*z*u*v+F(47)/87*y^2*z*u*v+7647*x*z^2*u*v+F(108)/13*y*z^2*u*v+F(27)/109*z^3*u*v+F(84)/85*x^2*u^2*v+F(37)/104*x*y*u^2*v+F(30)/103*y^2*u^2*v+F(119)/12*x*z*u^2*v-4814*y*z*u^2*v+15707*z^2*u^2*v-F(21)/73*x*u^3*v+836*y*u^3*v-11769*z*u^3*v+9858*u^4*v+F(113)/75*x^3*v^2-2893*x^2*y*v^2-9494*x*y^2*v^2-4315*y^3*v^2-F(49)/85*x^2*z*v^2+4164*x*y*z*v^2+530*y^2*z*v^2+5761*x*z^2*v^2-3715*y*z^2*v^2-15912*z^3*v^2+3803*x^2*u*v^2-F(110)/47*x*y*u*v^2+8911*y^2*u*v^2+14239*x*z*u*v^2-6962*y*z*u*v^2+6837*z^2*u*v^2-3812*x*u^2*v^2-F(13)/14*y*u^2*v^2-F(52)/125*z*u^2*v^2+8621*u^3*v^2+F(30)/91*x^2*v^3-F(51)/73*x*y*v^3-F(120)/13*y^2*v^3+14783*x*z*v^3-F(103)/78*y*z*v^3+3885*z^2*v^3-F(61)/106*x*u*v^3-F(88)/101*y*u*v^3+F(108)/125*z*u*v^3+13766*u^2*v^3-F(11)/35*x*v^4-F(55)/29*y*v^4+3117*z*v^4+F(63)/32*u*v^4-F(115)/52*v^5,12846*x^4*z-F(105)/47*x^3*y*z+F(121)/75*x^2*y^2*z-F(70)/11*x*y^3*z+11003*y^4*z-3360*x^3*z^2+F(109)/87*x^2*y*z^2+F(34)/23*x*y^2*z^2+F(115)/59*y^3*z^2-F(25)/77*x^2*z^3+14317*x*y*z^3+F(31)/74*y^2*z^3-13318*x*z^4+F(4)/9*y*z^4+10744*z^5+F(4)/101*x^4*u-8448*x^3*y*u+F(98)/107*x^2*y^2*u+F(123)/62*x*y^3*u-3569*y^4*u-F(73)/94*x^3*z*u+F(6)/109*x^2*y*z*u+F(6)/83*x*y^2*z*u-13017*y^3*z*u-F(31)/26*x^2*z^2*u-F(52)/11*x*y*z^2*u+F(20)/73*y^2*z^2*u-12051*x*z^3*u-244*y*z^3*u+F(100)/117*z^4*u-5436*x^3*u^2-9336*x^2*y*u^2-F(101)/116*x*y^2*u^2-F(69)/52*y^3*u^2-F(41)/38*x^2*z*u^2+F(97)/104*x*y*z*u^2-15660*y^2*z*u^2+F(115)/106*x*z^2*u^2+F(68)/45*y*z^2*u^2+F(65)/17*z^3*u^2-F(116)/15*x^2*u^3+482*x*y*u^3-F(55)/23*y^2*u^3+3090*x*z*u^3+F(111)/101*y*z*u^3-4991*z^2*u^3+F(91)/75*x*u^4+F(104)/101*y*u^4-5691*z*u^4+F(49)/6*u^5-F(126)/31*x^4*v-F(16)/57*x^3*y*v-F(69)/113*x^2*y^2*v+F(27)/98*x*y^3*v+F(5)/57*y^4*v+F(114)/85*x^3*z*v-F(87)/46*x^2*y*z*v-8243*x*y^2*z*v+F(67)/28*y^3*z*v+F(113)/53*x^2*z^2*v+F(63)/86*x*y*z^2*v-1135*y^2*z^2*v+F(119)/26*x*z^3*v+F(113)/110*y*z^3*v+F(124)/81*z^4*v+5635*x^3*u*v-14147*x^2*y*u*v-F(23)/33*x*y^2*u*v-F(74)/113*y^3*u*v+5410*x^2*z*u*v-4039*x*y*z*u*v-15876*y^2*z*u*v-14433*x*z^2*u*v-861*y*z^2*u*v-F(50)/3*z^3*u*v-F(94)/97*x^2*u^2*v+2802*x*y*u^2*v-F(93)/26*y^2*u^2*v+F(88)/3*x*z*u^2*v-3304*y*z*u^2*v+F(52)/41*z^2*u^2*v+F(125)/61*x*u^3*v+F(7)/88*y*u^3*v-F(111)/43*z*u^3*v+13372*u^4*v-11024*x^3*v^2+12475*x^2*y*v^2-5429*x*y^2*v^2-3601*y^3*v^2-2411*x^2*z*v^2-F(116)/77*x*y*z*v^2-F(63)/44*y^2*z*v^2-8178*x*z^2*v^2+7083*y*z^2*v^2+F(123)/4*z^3*v^2-716*x^2*u*v^2-F(110)/87*x*y*u*v^2+13640*y^2*u*v^2-F(17)/45*x*z*u*v^2+F(94)/9*y*z*u*v^2+3883*z^2*u*v^2-F(35)/97*x*u^2*v^2+F(11)/30*y*u^2*v^2+F(47)/37*z*u^2*v^2-F(9)/49*u^3*v^2-F(44)/57*x^2*v^3-F(35)/93*x*y*v^3+11796*y^2*v^3+F(58)/67*x*z*v^3-1578*y*z*v^3-15610*z^2*v^3-F(56)/9*x*u*v^3+7469*y*u*v^3+5623*z*u*v^3-F(9)/97*u^2*v^3+F(74)/99*x*v^4+5635*y*v^4-6363*z*v^4-11174*u*v^4+F(37)/18*v^5,10610*x^5-F(87)/85*x^4*y-174*x^3*y^2-F(81)/20*x^2*y^3+F(107)/77*x*y^4-F(73)/39*y^5-F(63)/113*x^4*z+4973*x^3*y*z-F(50)/43*x^2*y^2*z+441*x*y^3*z+2310*y^4*z+F(47)/102*x^3*z^2-F(71)/45*x^2*y*z^2-F(71)/8*x*y^2*z^2-F(32)/101*y^3*z^2+15290*x^2*z^3-2774*x*y*z^3-11723*y^2*z^3-F(101)/63*x*z^4-14034*y*z^4-9729*x^4*u+F(115)/54*x^3*y*u-3754*x^2*y^2*u+F(20)/103*x*y^3*u-11547*y^4*u+F(1)/37*x^3*z*u-F(6)/73*x^2*y*z*u+F(90)/121*x*y^2*z*u-6476*y^3*z*u+F(21)/61*x^2*z^2*u+F(7)/76*x*y*z^2*u-F(103)/41*y^2*z^2*u+F(24)/31*x*z^3*u+F(78)/17*y*z^3*u+2297*z^4*u-3322*x^3*u^2+F(107)/61*x^2*y*u^2+F(25)/24*x*y^2*u^2-6196*y^3*u^2+8396*x^2*z*u^2-F(111)/13*x*y*z*u^2+5043*y^2*z*u^2+13742*x*z^2*u^2+F(72)/29*y*z^2*u^2+F(57)/74*z^3*u^2-F(95)/89*x^2*u^3-F(54)/37*x*y*u^3+F(59)/5*y^2*u^3+8036*x*z*u^3-F(114)/91*y*z*u^3+F(101)/23*z^2*u^3+F(63)/43*x*u^4-F(102)/67*y*u^4+11174*z*u^4-F(110)/21*u^5+F(91)/106*x^4*v-F(93)/80*x^3*y*v+F(25)/59*x^2*y^2*v-F(126)/79*x*y^3*v-F(51)/37*y^4*v-F(25)/109*x^3*z*v-8463*x^2*y*z*v+4746*x*y^2*z*v+F(40)/121*y^3*z*v-15428*x^2*z^2*v-15167*x*y*z^2*v-F(77)/104*y^2*z^2*v-F(40)/9*x*z^3*v+F(38)/73*y*z^3*v+F(49)/123*z^4*v+F(126)/71*x^3*u*v+F(14)/97*x^2*y*u*v-2852*x*y^2*u*v+9763*y^3*u*v+F(71)/39*x^2*z*u*v-10514*x*y*z*u*v+F(26)/115*y^2*z*u*v-9120*x*z^2*u*v-F(67)/6*y*z^2*u*v+2620*z^3*u*v+F(118)/125*x^2*u^2*v+F(92)/7*x*y*u^2*v+F(66)/85*y^2*u^2*v+9005*x*z*u^2*v+8502*y*z*u^2*v+F(58)/61*z^2*u^2*v-F(124)/105*x*u^3*v+13869*y*u^3*v-F(31)/73*z*u^3*v-F(63)/32*u^4*v-F(57)/17*x^3*v^2-12762*x^2*y*v^2+F(15)/61*x*y^2*v^2-F(119)/54*y^3*v^2+10544*x^2*z*v^2-8542*x*y*z*v^2-4065*y^2*z*v^2-9487*x*z^2*v^2+F(112)/99*y*z^2*v^2+388*z^3*v^2+F(123)/43*x^2*u*v^2-F(78)/55*x*y*u*v^2-F(35)/13*y^2*u*v^2-F(86)/13*x*z*u*v^2-7910*y*z*u*v^2+3786*z^2*u*v^2-15419*x*u^2*v^2-12944*y*u^2*v^2-F(47)/92*z*u^2*v^2-F(86)/35*u^3*v^2+F(89)/60*x^2*v^3+F(17)/71*x*y*v^3-5588*y^2*v^3+F(125)/6*x*z*v^3-3885*y*z*v^3-F(122)/95*z^2*v^3-F(51)/40*x*u*v^3-F(95)/12*y*u*v^3+2471*z*u*v^3-F(107)/4*u^2*v^3+F(121)/70*x*v^4+1073*y*v^4+F(105)/43*z*v^4+F(3)/23*u*v^4+F(41)/92*v^5,10610*x^4*y-7179*x^3*y^2+5063*x^2*y^3+F(3)/47*x*y^4+10197*y^5+1966*x^4*z+12673*x^3*y*z-8976*x^2*y^2*z-F(115)/68*x*y^3*z+F(76)/105*y^4*z-F(64)/27*x^3*z^2+F(83)/63*x^2*y*z^2+12769*x*y^2*z^2-10202*y^3*z^2+3513*x^2*z^3-F(94)/79*x*y*z^3-F(107)/109*y^2*z^3-2172*x*z^4+F(56)/19*y*z^4+F(31)/11*z^5+8312*x^4*u-F(103)/98*x^3*y*u+6048*x^2*y^2*u+F(12)/31*x*y^3*u+F(72)/95*y^4*u+F(19)/90*x^3*z*u-F(97)/69*x^2*y*z*u-F(71)/95*x*y^2*z*u+F(9)/109*y^3*z*u-F(99)/35*x^2*z^2*u+10531*x*y*z^2*u-F(88)/29*y^2*z^2*u+1647*x*z^3*u-14093*y*z^3*u+F(40)/123*z^4*u+2119*x^3*u^2+7368*x^2*y*u^2+11102*x*y^2*u^2-F(114)/17*y^3*u^2+13182*x^2*z*u^2+F(67)/63*x*y*z*u^2+F(100)/29*y^2*z*u^2-F(17)/64*x*z^2*u^2+12195*y*z^2*u^2-13291*z^3*u^2+1971*x^2*u^3+12146*x*y*u^3+F(80)/19*y^2*u^3+8767*x*z*u^3+F(107)/125*y*z*u^3-F(79)/58*z^2*u^3-10441*x*u^4-F(52)/3*y*u^4-F(125)/19*z*u^4+F(112)/27*u^5+7278*x^4*v+F(59)/27*x^3*y*v+F(23)/86*x^2*y^2*v+F(19)/104*x*y^3*v-F(80)/49*y^4*v+11323*x^3*z*v-F(89)/113*x^2*y*z*v-9239*x*y^2*z*v+F(47)/73*y^3*z*v-503*x^2*z^2*v-F(86)/79*x*y*z^2*v+7637*y^2*z^2*v+F(97)/38*x*z^3*v+F(52)/51*y*z^3*v+F(109)/91*z^4*v-15718*x^3*u*v+F(98)/83*x^2*y*u*v-11141*x*y^2*u*v+F(68)/117*y^3*u*v+12771*x^2*z*u*v-3956*x*y*z*u*v+F(103)/8*y^2*z*u*v-4778*x*z^2*u*v+13318*y*z^2*u*v+12614*z^3*u*v+15385*x^2*u^2*v-F(4)/75*x*y*u^2*v+F(102)/79*y^2*u^2*v-F(33)/106*x*z*u^2*v-F(64)/83*y*z*u^2*v+F(86)/103*z^2*u^2*v-F(32)/25*x*u^3*v+9264*y*u^3*v+F(49)/43*z*u^3*v+F(85)/93*u^4*v+5146*x^3*v^2+11169*x^2*y*v^2-4734*x*y^2*v^2-F(101)/5*y^3*v^2+F(79)/71*x^2*z*v^2+F(68)/59*x*y*z*v^2-F(1)/91*y^2*z*v^2-F(93)/34*x*z^2*v^2-F(107)/101*y*z^2*v^2-F(101)/97*z^3*v^2-F(69)/22*x^2*u*v^2-F(91)/31*x*y*u*v^2-F(75)/38*y^2*u*v^2-F(8)/75*x*z*u*v^2-F(67)/121*y*z*u*v^2-F(103)/4*z^2*u*v^2-F(6)/101*x*u^2*v^2+12022*y*u^2*v^2+10719*z*u^2*v^2-F(9)/122*u^3*v^2-F(13)/95*x^2*v^3-3539*x*y*v^3+F(53)/7*y^2*v^3+8127*x*z*v^3-F(69)/8*y*z*v^3-F(13)/18*z^2*v^3+161*x*u*v^3-12490*y*u*v^3-2070*z*u*v^3-F(37)/89*u^2*v^3-F(51)/44*x*v^4-14493*y*v^4-F(84)/55*z*v^4+12450*u*v^4+9167*v^5,10610*x^4*z-7179*x^3*y*z+5063*x^2*y^2*z+F(3)/47*x*y^3*z+10197*y^4*z-F(63)/113*x^3*z^2+14955*x^2*y*z^2+F(121)/47*x*y^2*z^2-F(119)/4*y^3*z^2+F(47)/102*x^2*z^3+F(25)/19*x*y*z^3-860*y^2*z^3+15290*x*z^4+6428*y*z^4-F(101)/63*z^5+F(63)/121*x^4*u+F(83)/55*x^3*y*u-F(34)/115*x^2*y^2*u-4661*x*y^3*u+F(109)/3*y^4*u+F(118)/115*x^3*z*u+F(75)/53*x^2*y*z*u-F(7)/4*x*y^2*z*u+F(74)/109*y^3*z*u-F(12)/13*x^2*z^2*u+F(44)/123*x*y*z^2*u+F(17)/61*y^2*z^2*u-F(1)/115*x*z^3*u-F(9)/53*y*z^3*u+F(62)/125*z^4*u+606*x^3*u^2-7808*x^2*y*u^2+7411*x*y^2*u^2+F(76)/117*y^3*u^2+5416*x^2*z*u^2+F(38)/15*x*y*z*u^2-5903*y^2*z*u^2+F(65)/121*x*z^2*u^2-119*y*z^2*u^2+F(76)/97*z^3*u^2-F(67)/125*x^2*u^3+F(63)/64*x*y*u^3+F(63)/19*y^2*u^3-F(61)/55*x*z*u^3-F(121)/78*y*z*u^3+F(95)/2*z^2*u^3-F(9)/47*x*u^4-F(62)/103*y*u^4-4395*z*u^4-F(77)/48*u^5-F(121)/12*x^4*v-8353*x^3*y*v-F(94)/31*x^2*y^2*v-F(15)/83*x*y^3*v+F(23)/20*y^4*v+3535*x^3*z*v+F(35)/29*x^2*y*z*v+15636*x*y^2*z*v+F(16)/81*y^3*z*v-F(32)/101*x^2*z^2*v-5275*x*y*z^2*v+F(72)/47*y^2*z^2*v-F(93)/65*x*z^3*v+F(111)/98*y*z^3*v-14101*z^4*v+10076*x^3*u*v-14198*x^2*y*u*v+F(121)/26*x*y^2*u*v-F(63)/65*y^3*u*v-233*x^2*z*u*v-F(37)/105*x*y*z*u*v-F(81)/74*y^2*z*u*v-F(84)/79*x*z^2*u*v+12839*y*z^2*u*v+F(19)/47*z^3*u*v-13420*x^2*u^2*v+F(67)/93*x*y*u^2*v+F(113)/32*y^2*u^2*v+F(42)/107*x*z*u^2*v-6448*y*z*u^2*v-1491*z^2*u^2*v-4709*x*u^3*v+F(124)/53*y*u^3*v+F(83)/63*z*u^3*v+F(91)/10*u^4*v-F(53)/92*x^3*v^2+3421*x^2*y*v^2-F(115)/41*x*y^2*v^2+8558*y^3*v^2-11206*x^2*z*v^2-F(83)/97*x*y*z*v^2-F(103)/67*y^2*z*v^2+F(61)/116*x*z^2*v^2-10724*y*z^2*v^2-9956*z^3*v^2-8792*x^2*u*v^2-F(75)/4*x*y*u*v^2+11079*y^2*u*v^2-F(53)/4*x*z*u*v^2-3652*y*z*u*v^2-F(35)/118*z^2*u*v^2+11440*x*u^2*v^2+F(107)/46*y*u^2*v^2+F(25)/19*z*u^2*v^2-F(24)/5*u^3*v^2-F(101)/57*x^2*v^3-F(39)/8*x*y*v^3+F(123)/77*y^2*v^3+4184*x*z*v^3+F(78)/109*y*z*v^3+F(83)/114*z^2*v^3-2242*x*u*v^3-F(59)/31*y*u*v^3+F(69)/34*z*u*v^3+F(121)/7*u^2*v^3-14707*x*v^4-869*y*v^4-12707*z*v^4+F(83)/75*u*v^4-3577*v^5,-9115*x^5-F(80)/7*x^4*y+F(64)/91*x^3*y^2-2582*x^2*y^3-3462*x*y^4-F(125)/54*x^4*z+F(117)/25*x^3*y*z+F(82)/13*x^2*y^2*z+11267*x*y^3*z+F(84)/115*y^4*z+F(75)/119*x^3*z^2+F(33)/5*x^2*y*z^2-F(103)/33*x*y^2*z^2-F(16)/111*y^3*z^2+F(53)/70*x^2*z^3-F(86)/49*x*y*z^3+F(51)/67*y^2*z^3+F(113)/96*x*z^4+F(43)/78*y*z^4-F(78)/23*z^5+9081*x^4*u-1095*x^3*y*u-F(62)/11*x^2*y^2*u-5755*x*y^3*u+F(103)/107*y^4*u-F(64)/49*x^3*z*u-7473*x^2*y*z*u-F(41)/113*x*y^2*z*u-F(50)/101*y^3*z*u+F(23)/17*x^2*z^2*u-13547*x*y*z^2*u+F(53)/83*y^2*z^2*u+F(50)/107*x*z^3*u-F(29)/82*y*z^3*u+14279*z^4*u-F(7)/64*x^3*u^2-15167*x^2*y*u^2+F(124)/3*x*y^2*u^2-F(102)/7*y^3*u^2-11865*x^2*z*u^2-F(107)/74*x*y*z*u^2-F(67)/30*y^2*z*u^2+11180*x*z^2*u^2-9919*y*z^2*u^2-F(67)/33*z^3*u^2-13283*x^2*u^3-3302*x*y*u^3-F(91)/97*y^2*u^3-11168*x*z*u^3+4181*y*z*u^3+F(35)/86*z^2*u^3-F(97)/109*x*u^4+F(4)/5*y*u^4+F(73)/95*z*u^4+F(48)/47*u^5+8038*x^4*v+F(88)/83*x^3*y*v-3221*x^2*y^2*v-F(103)/63*x*y^3*v+8207*y^4*v-8037*x^3*z*v+F(68)/103*x^2*y*z*v+F(13)/42*x*y^2*z*v+F(13)/23*y^3*z*v+F(107)/43*x^2*z^2*v+3970*x*y*z^2*v+12154*y^2*z^2*v+F(124)/27*x*z^3*v-F(59)/121*y*z^3*v+9676*z^4*v-F(41)/74*x^3*u*v+4676*x^2*y*u*v-12890*x*y^2*u*v-F(47)/100*y^3*u*v-F(29)/42*x^2*z*u*v-F(114)/67*x*y*z*u*v-F(101)/18*y^2*z*u*v+F(101)/72*x*z^2*u*v+15581*y*z^2*u*v-15911*z^3*u*v-13432*x^2*u^2*v+4656*x*y*u^2*v+7158*y^2*u^2*v+8055*x*z*u^2*v+F(13)/121*y*z*u^2*v-F(77)/124*z^2*u^2*v-F(1)/96*x*u^3*v+11617*y*u^3*v+12261*z*u^3*v+F(11)/112*u^4*v-F(113)/108*x^3*v^2-4133*x^2*y*v^2+F(11)/120*x*y^2*v^2+1849*y^3*v^2-F(114)/37*x^2*z*v^2-F(81)/23*x*y*z*v^2+1160*y^2*z*v^2+F(64)/61*x*z^2*v^2-F(116)/65*y*z^2*v^2-F(27)/65*z^3*v^2+F(49)/90*x^2*u*v^2+F(83)/66*x*y*u*v^2+9826*y^2*u*v^2+F(97)/116*x*z*u*v^2+F(35)/37*y*z*u*v^2-F(5)/68*z^2*u*v^2-8411*x*u^2*v^2+3422*y*u^2*v^2-F(12)/59*z*u^2*v^2-F(3)/13*u^3*v^2-F(84)/113*x^2*v^3+1925*x*y*v^3+8038*y^2*v^3+15322*x*z*v^3+F(61)/23*y*z*v^3-F(77)/100*z^2*v^3-F(13)/80*x*u*v^3-12846*y*u*v^3+F(94)/33*z*u*v^3-15876*u^2*v^3-3668*x*v^4-F(19)/93*y*v^4-F(37)/52*z*v^4-F(86)/99*u*v^4+1742*v^5,9115*x^4*y+F(80)/7*x^3*y^2-F(64)/91*x^2*y^3+2582*x*y^4+3462*y^5+15140*x^4*z-11737*x^3*y*z-F(94)/79*x^2*y^2*z+F(92)/29*x*y^3*z-1991*y^4*z+F(77)/115*x^3*z^2-12089*x^2*y*z^2+14199*x*y^2*z^2+222*y^3*z^2-F(1)/121*x^2*z^3+F(113)/119*x*y*z^3-12945*y^2*z^3+F(118)/7*x*z^4-749*y*z^4-F(41)/63*z^5-F(55)/17*x^4*u-7628*x^3*y*u+9559*x^2*y^2*u-F(94)/113*x*y^3*u+F(8)/113*y^4*u-3471*x^3*z*u+F(13)/56*x^2*y*z*u+F(61)/102*x*y^2*z*u+F(95)/27*y^3*z*u-F(76)/31*x^2*z^2*u-3275*x*y*z^2*u-F(22)/75*y^2*z^2*u-F(98)/39*x*z^3*u+F(61)/75*y*z^3*u+F(77)/95*z^4*u-1977*x^3*u^2-F(1)/112*x^2*y*u^2+2152*x*y^2*u^2+F(83)/14*y^3*u^2+F(95)/71*x^2*z*u^2-F(95)/53*x*y*z*u^2-F(123)/98*y^2*z*u^2+F(31)/19*x*z^2*u^2+15286*y*z^2*u^2+4873*z^3*u^2-11233*x^2*u^3+8216*x*y*u^3+F(31)/89*y^2*u^3+F(95)/119*x*z*u^3+F(11)/19*y*z*u^3+F(119)/100*z^2*u^3-6823*x*u^4+8313*y*u^4+4405*z*u^4+13935*u^5+F(20)/97*x^4*v-1693*x^3*y*v+F(51)/104*x^2*y^2*v-F(15)/89*x*y^3*v-F(102)/7*y^4*v+13421*x^3*z*v+F(33)/124*x^2*y*z*v-2193*x*y^2*z*v+6113*y^3*z*v+F(41)/9*x^2*z^2*v+3389*x*y*z^2*v-5667*y^2*z^2*v+1483*x*z^3*v+F(115)/52*y*z^3*v+F(121)/122*z^4*v-F(21)/41*x^3*u*v-1807*x^2*y*u*v+94*x*y^2*u*v+4597*y^3*u*v+F(75)/52*x^2*z*u*v+F(113)/11*x*y*z*u*v+F(115)/61*y^2*z*u*v-F(59)/103*x*z^2*u*v+13844*y*z^2*u*v+F(68)/5*z^3*u*v-F(100)/59*x^2*u^2*v+203*x*y*u^2*v-F(116)/89*y^2*u^2*v-F(124)/117*x*z*u^2*v-739*y*z*u^2*v-F(87)/83*z^2*u^2*v+F(59)/90*x*u^3*v+F(35)/18*y*u^3*v+6263*z*u^3*v+6576*u^4*v-F(2)/41*x^3*v^2+11650*x^2*y*v^2+816*x*y^2*v^2-F(80)/73*y^3*v^2-F(69)/107*x^2*z*v^2-F(2)/111*x*y*z*v^2-8744*y^2*z*v^2-15779*x*z^2*v^2-15929*y*z^2*v^2-15711*z^3*v^2+F(26)/11*x^2*u*v^2+15922*x*y*u*v^2+7393*y^2*u*v^2-8032*x*z*u*v^2+2273*y*z*u*v^2-F(61)/100*z^2*u*v^2-F(9)/103*x*u^2*v^2+14131*y*u^2*v^2-F(97)/45*z*u^2*v^2-F(13)/21*u^3*v^2-6948*x^2*v^3-5030*x*y*v^3+15825*y^2*v^3-F(89)/70*x*z*v^3-F(72)/41*y*z*v^3+F(47)/83*z^2*v^3-F(21)/23*x*u*v^3-F(95)/84*y*u*v^3-F(4)/57*z*u*v^3+F(69)/89*u^2*v^3-F(115)/98*x*v^4+14913*y*v^4+F(116)/17*z*v^4+F(111)/56*u*v^4+8033*v^5,-F(38)/31*x^5-F(17)/96*x^4*y-4017*x^3*y^2+6439*x^2*y^3+12014*x*y^4-F(14)/125*x^4*z-12458*x^3*y*z-530*x^2*y^2*z+12840*x*y^3*z-F(64)/15*y^4*z-F(77)/97*x^3*z^2+F(98)/45*x^2*y*z^2-F(109)/119*x*y^2*z^2+15915*y^3*z^2-4814*x^2*z^3-F(20)/51*x*y*z^3+F(27)/77*y^2*z^3+F(34)/63*x*z^4-9495*y*z^4-F(121)/75*z^5+9872*x^4*u+F(97)/35*x^3*y*u-F(115)/41*x^2*y^2*u-6949*x*y^3*u-6983*y^4*u+F(23)/8*x^3*z*u+F(84)/125*x^2*y*z*u+F(100)/99*x*y^2*z*u-7129*y^3*z*u+6658*x^2*z^2*u-F(53)/118*x*y*z^2*u-F(16)/121*y^2*z^2*u+F(67)/84*x*z^3*u-5868*y*z^3*u+7855*z^4*u-28*x^3*u^2+F(115)/29*x^2*y*u^2+F(75)/92*x*y^2*u^2-F(9)/32*y^3*u^2+1677*x^2*z*u^2+F(117)/20*x*y*z*u^2-F(115)/118*y^2*z*u^2+9616*x*z^2*u^2-4074*y*z^2*u^2+F(11)/79*z^3*u^2-F(61)/44*x^2*u^3+4980*x*y*u^3-F(75)/53*y^2*u^3+F(71)/9*x*z*u^3-13857*y*z*u^3+F(126)/61*z^2*u^3-1807*x*u^4+F(111)/17*y*u^4-12101*z*u^4+6897*u^5+F(73)/108*x^4*v-F(89)/51*x^3*y*v+1382*x^2*y^2*v-14406*x*y^3*v+F(12)/113*y^4*v-12944*x^3*z*v+F(5)/2*x^2*y*z*v-10477*x*y^2*z*v+100*y^3*z*v+F(77)/118*x^2*z^2*v-5153*x*y*z^2*v-F(72)/49*y^2*z^2*v-607*x*z^3*v-F(103)/48*y*z^3*v+8931*z^4*v+F(115)/16*x^3*u*v+F(16)/59*x^2*y*u*v+F(14)/11*x*y^2*u*v-F(41)/49*y^3*u*v+F(112)/89*x^2*z*u*v-F(121)/101*x*y*z*u*v-F(89)/34*y^2*z*u*v-F(67)/32*x*z^2*u*v-F(109)/95*y*z^2*u*v+F(11)/74*z^3*u*v+F(47)/113*x^2*u^2*v+F(31)/47*x*y*u^2*v-F(7)/75*y^2*u^2*v-F(104)/57*x*z*u^2*v+F(73)/60*y*z*u^2*v-15896*z^2*u^2*v-F(40)/91*x*u^3*v+F(101)/26*y*u^3*v+11586*z*u^3*v+F(122)/67*u^4*v-5085*x^3*v^2-F(22)/69*x^2*y*v^2-F(17)/2*x*y^2*v^2-5212*y^3*v^2+F(31)/95*x^2*z*v^2-6009*x*y*z*v^2+5881*y^2*z*v^2-F(52)/67*x*z^2*v^2+F(27)/110*y*z^2*v^2+F(43)/60*z^3*v^2-F(11)/15*x^2*u*v^2-F(41)/110*x*y*u*v^2+F(77)/72*y^2*u*v^2+F(115)/24*x*z*u*v^2-F(75)/43*y*z*u*v^2-F(66)/43*z^2*u*v^2-11280*x*u^2*v^2+F(116)/91*y*u^2*v^2-F(30)/37*z*u^2*v^2+1786*u^3*v^2+F(48)/67*x^2*v^3-F(113)/61*x*y*v^3-10393*y^2*v^3-F(51)/92*x*z*v^3+2323*y*z*v^3-13682*z^2*v^3-12082*x*u*v^3+F(121)/114*y*u*v^3+F(10)/89*z*u*v^3+F(28)/95*u^2*v^3+F(64)/49*x*v^4+F(37)/13*y*v^4+F(49)/43*z*v^4-15415*u*v^4+F(50)/39*v^5,F(38)/31*x^4*y+F(17)/96*x^3*y^2+4017*x^2*y^3-6439*x*y^4-12014*y^5-F(29)/35*x^4*z+7411*x^3*y*z-15920*x^2*y^2*z-6548*x*y^3*z+F(55)/48*y^4*z-10731*x^3*z^2-F(44)/69*x^2*y*z^2+2428*x*y^2*z^2+7547*y^3*z^2-F(86)/31*x^2*z^3-12679*x*y*z^3-F(3)/34*y^2*z^3+270*x*z^4+10247*y*z^4+F(87)/47*z^5-F(50)/11*x^4*u-3013*x^3*y*u+F(109)/114*x^2*y^2*u-12475*x*y^3*u+F(49)/12*y^4*u-6822*x^3*z*u-10092*x^2*y*z*u+F(93)/25*x*y^2*z*u+F(29)/47*y^3*z*u-F(23)/20*x^2*z^2*u-F(125)/106*x*y*z^2*u+2114*y^2*z^2*u+2590*x*z^3*u+8818*y*z^3*u+F(51)/13*z^4*u-13919*x^3*u^2-739*x^2*y*u^2+F(37)/3*x*y^2*u^2-F(3)/118*y^3*u^2-F(109)/10*x^2*z*u^2-F(71)/48*x*y*z*u^2-F(17)/41*y^2*z*u^2+3750*x*z^2*u^2+F(7)/22*y*z^2*u^2-F(105)/23*z^3*u^2+F(86)/119*x^2*u^3+F(67)/24*x*y*u^3+F(107)/81*y^2*u^3+7370*x*z*u^3-F(47)/64*y*z*u^3+7026*z^2*u^3+F(59)/33*x*u^4+F(20)/57*y*u^4-11327*z*u^4+F(78)/17*u^5-F(116)/37*x^4*v+F(71)/14*x^3*y*v+F(4)/17*x^2*y^2*v-F(3)/26*x*y^3*v-F(18)/61*y^4*v-F(62)/77*x^3*z*v+F(93)/65*x^2*y*z*v+F(124)/57*x*y^2*z*v+F(37)/104*y^3*z*v-15619*x^2*z^2*v-F(103)/110*x*y*z^2*v-11065*y^2*z^2*v+F(87)/46*x*z^3*v+10962*y*z^3*v-F(81)/11*z^4*v-F(59)/114*x^3*u*v+F(72)/19*x^2*y*u*v-F(76)/77*x*y^2*u*v+13566*y^3*u*v-4884*x^2*z*u*v+8931*x*y*z*u*v-F(39)/40*y^2*z*u*v+4713*x*z^2*u*v+14375*y*z^2*u*v+F(68)/75*z^3*u*v+F(115)/84*x^2*u^2*v+F(113)/28*x*y*u^2*v-F(80)/101*y^2*u^2*v-2014*x*z*u^2*v-9070*y*z*u^2*v-353*z^2*u^2*v-732*x*u^3*v+F(31)/14*y*u^3*v-14028*z*u^3*v-F(43)/19*u^4*v+F(56)/61*x^3*v^2-189*x^2*y*v^2+F(94)/117*x*y^2*v^2-F(31)/107*y^3*v^2+F(123)/65*x^2*z*v^2+4405*x*y*z*v^2-F(97)/68*y^2*z*v^2+11615*x*z^2*v^2-5508*y*z^2*v^2+F(61)/88*z^3*v^2+F(71)/108*x^2*u*v^2+12142*x*y*u*v^2+F(106)/87*y^2*u*v^2-14610*x*z*u*v^2+7492*y*z*u*v^2-15674*z^2*u*v^2+14839*x*u^2*v^2-4761*y*u^2*v^2+F(71)/96*z*u^2*v^2-F(111)/20*u^3*v^2+7356*x^2*v^3+F(35)/24*x*y*v^3-F(78)/67*y^2*v^3+F(52)/5*x*z*v^3-F(63)/40*y*z*v^3-F(57)/101*z^2*v^3+5081*x*u*v^3-1384*y*u*v^3+F(101)/120*z*u*v^3+5356*u^2*v^3-15308*x*v^4-F(75)/88*y*v^4-5673*z*v^4-10583*u*v^4-F(83)/102*v^5,-14469*x^5-2273*x^4*y+F(83)/72*x^3*y^2+1125*x^2*y^3-F(46)/47*x*y^4-14204*y^5-F(101)/30*x^4*z+F(17)/101*x^3*y*z-F(91)/94*x^2*y^2*z-10500*x*y^3*z-9764*y^4*z-F(109)/15*x^3*z^2+12020*x^2*y*z^2+15784*x*y^2*z^2-F(57)/37*y^3*z^2-F(60)/53*x^2*z^3-F(65)/122*x*y*z^3-6045*y^2*z^3-7686*x*z^4-11686*y*z^4-10992*z^5+9860*x^4*u+F(19)/37*x^3*y*u-899*x^2*y^2*u+14175*x*y^3*u+F(111)/91*y^4*u-F(98)/67*x^3*z*u-F(20)/121*x^2*y*z*u-3443*x*y^2*z*u+5692*y^3*z*u+F(95)/64*x^2*z^2*u-1923*x*y*z^2*u-1986*y^2*z^2*u-3639*x*z^3*u-648*y*z^3*u+13690*z^4*u-F(95)/36*x^3*u^2+F(41)/103*x^2*y*u^2+F(46)/71*x*y^2*u^2-12314*y^3*u^2+13918*x^2*z*u^2-F(35)/99*x*y*z*u^2+F(47)/111*y^2*z*u^2+5039*x*z^2*u^2-F(124)/103*y*z^2*u^2-13804*z^3*u^2-F(107)/102*x^2*u^3-F(91)/20*x*y*u^3-11526*y^2*u^3+F(26)/45*x*z*u^3-F(2)/37*y*z*u^3+F(115)/77*z^2*u^3+F(85)/101*x*u^4-14736*y*u^4+F(33)/85*z*u^4+11440*u^5-F(68)/31*x^4*v-3222*x^3*y*v+F(105)/38*x^2*y^2*v-12838*x*y^3*v-F(35)/73*y^4*v+7851*x^3*z*v-F(121)/29*x^2*y*z*v+F(45)/64*x*y^2*z*v+11888*y^3*z*v-8754*x^2*z^2*v-F(76)/9*x*y*z^2*v-F(105)/74*y^2*z^2*v-2651*x*z^3*v-F(89)/122*y*z^3*v+6082*z^4*v+14464*x^3*u*v-F(7)/95*x^2*y*u*v-F(88)/73*x*y^2*u*v-F(5)/49*y^3*u*v-F(41)/65*x^2*z*u*v-F(21)/59*x*y*z*u*v+1816*y^2*z*u*v+F(117)/49*x*z^2*u*v+F(76)/105*y*z^2*u*v-F(18)/17*z^3*u*v-3969*x^2*u^2*v-F(65)/47*x*y*u^2*v+F(7)/59*y^2*u^2*v-F(111)/121*x*z*u^2*v+F(23)/50*y*z*u^2*v+F(57)/65*z^2*u^2*v-F(43)/110*x*u^3*v+F(43)/54*y*u^3*v-F(123)/4*z*u^3*v+F(65)/57*u^4*v+F(101)/104*x^3*v^2-8691*x^2*y*v^2-F(99)/118*x*y^2*v^2-F(118)/83*y^3*v^2+F(43)/40*x^2*z*v^2-15711*x*y*z*v^2-F(13)/66*y^2*z*v^2+F(21)/11*x*z^2*v^2+11722*y*z^2*v^2-F(44)/93*z^3*v^2-F(38)/77*x^2*u*v^2+8394*x*y*u*v^2-8658*y^2*u*v^2+F(88)/59*x*z*u*v^2+F(77)/8*y*z*u*v^2-F(54)/23*z^2*u*v^2-F(7)/23*x*u^2*v^2-F(7)/66*y*u^2*v^2-5184*z*u^2*v^2+F(107)/81*u^3*v^2+F(17)/32*x^2*v^3-4181*x*y*v^3+F(99)/124*y^2*v^3-F(46)/17*x*z*v^3-F(73)/8*y*z*v^3+F(11)/36*z^2*v^3+F(77)/109*x*u*v^3+6429*y*u*v^3+2984*z*u*v^3-F(32)/37*u^2*v^3-1190*x*v^4-11980*y*v^4-F(44)/123*z*v^4-F(31)/99*u*v^4-F(112)/75*v^5,14198*x^5+F(98)/55*x^4*y-3830*x^3*y^2+5490*x^2*y^3-2616*x*y^4+4076*y^5-F(53)/41*x^4*z+F(70)/27*x^3*y*z+F(55)/6*x^2*y^2*z-F(107)/79*x*y^3*z+F(49)/25*y^4*z+14808*x^3*z^2-F(112)/3*x^2*y*z^2-F(52)/3*x*y^2*z^2+F(13)/16*y^3*z^2+F(6)/49*x^2*z^3+F(15)/64*x*y*z^3+F(121)/87*y^2*z^3-5042*x*z^4+7423*y*z^4+F(68)/97*z^5+6893*x^4*u-F(101)/114*x^3*y*u-13578*x^2*y^2*u+10229*x*y^3*u+11415*y^4*u-F(17)/94*x^3*z*u-F(66)/83*x^2*y*z*u-15730*x*y^2*z*u+F(92)/83*y^3*z*u+4060*x^2*z^2*u+14890*x*y*z^2*u+6270*y^2*z^2*u+F(55)/74*x*z^3*u+F(67)/89*y*z^3*u+F(52)/9*z^4*u+8879*x^3*u^2+222*x^2*y*u^2-8060*x*y^2*u^2+F(4)/55*y^3*u^2-F(58)/61*x^2*z*u^2+F(45)/64*x*y*z*u^2-8097*y^2*z*u^2-F(45)/7*x*z^2*u^2+F(25)/38*y*z^2*u^2-9382*z^3*u^2+15493*x^2*u^3-F(119)/110*x*y*u^3+F(79)/16*y^2*u^3-F(111)/89*x*z*u^3+F(12)/41*y*z*u^3+F(57)/83*z^2*u^3-F(92)/9*x*u^4+F(59)/37*y*u^4-2728*z*u^4-F(33)/112*u^5-F(37)/14*x^4*v+10338*x^3*y*v-5534*x^2*y^2*v-13547*x*y^3*v-8604*y^4*v-F(36)/83*x^3*z*v+2224*x^2*y*z*v+F(113)/73*x*y^2*z*v-2512*y^3*z*v-F(37)/78*x^2*z^2*v+3589*x*y*z^2*v+F(64)/63*y^2*z^2*v-F(19)/29*x*z^3*v+F(103)/13*y*z^3*v+F(55)/3*z^4*v+F(37)/38*x^3*u*v+F(7)/78*x^2*y*u*v+12632*x*y^2*u*v+9382*y^3*u*v+F(110)/67*x^2*z*u*v-10545*x*y*z*u*v-F(36)/85*y^2*z*u*v-7370*x*z^2*u*v-7776*y*z^2*u*v-F(10)/73*z^3*u*v+11558*x^2*u^2*v-F(81)/19*x*y*u^2*v+13956*y^2*u^2*v+F(53)/48*x*z*u^2*v+F(88)/21*y*z*u^2*v+F(76)/59*z^2*u^2*v+F(77)/40*x*u^3*v+F(43)/112*y*u^3*v+1381*z*u^3*v-14595*u^4*v-F(83)/115*x^3*v^2-F(8)/111*x^2*y*v^2-F(5)/76*x*y^2*v^2+F(93)/109*y^3*v^2-F(96)/11*x^2*z*v^2+126*x*y*z*v^2-F(55)/92*y^2*z*v^2-12287*x*z^2*v^2+F(76)/53*y*z^2*v^2-F(117)/23*z^3*v^2+F(23)/35*x^2*u*v^2+F(29)/13*x*y*u*v^2+9455*y^2*u*v^2-F(34)/87*x*z*u*v^2+F(113)/47*y*z*u*v^2+F(93)/58*z^2*u*v^2+F(3)/106*x*u^2*v^2-F(89)/52*y*u^2*v^2-F(82)/23*z*u^2*v^2-7004*u^3*v^2+11717*x^2*v^3+F(105)/19*x*y*v^3-F(75)/116*y^2*v^3+F(2)/117*x*z*v^3-F(9)/101*y*z*v^3-F(67)/122*z^2*v^3+F(12)/73*x*u*v^3+5457*y*u*v^3-4281*z*u*v^3-F(7)/86*u^2*v^3+F(79)/52*x*v^4-1692*y*v^4+F(87)/109*z*v^4-F(63)/74*u*v^4+F(38)/81*v^5]))
end


@doc raw"""
    rational_d11_pi11_ss_inf()

Return a smooth rational surface in $\mathbb P^4$ with degree `11`, sectional genus `11`, and infinitely many 6-secants.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d11_pi11_ss_inf()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^6-12319*x^5*y-F(59)/18*x^4*y^2+699*x^3*y^3+9457*x^2*y^4+F(95)/22*x*y^5+F(120)/47*x^5*z-9779*x^4*y*z-F(2)/85*x^3*y^2*z+F(102)/73*x^2*y^3*z-10816*x*y^4*z+F(125)/122*x^4*z^2+F(49)/68*x^3*y*z^2-F(8)/55*x^2*y^2*z^2+F(77)/54*x*y^3*z^2+15002*x^3*z^3-F(41)/125*x^2*y*z^3+F(11)/103*x*y^2*z^3+8672*x^2*z^4+3607*x*y*z^4-7798*x^5*u+F(43)/107*x^4*y*u-15134*x^3*y^2*u-7902*x^2*y^3*u-12672*x*y^4*u-10289*x^4*z*u-F(79)/44*x^3*y*z*u-F(83)/74*x^2*y^2*z*u+F(117)/76*x*y^3*z*u+F(28)/117*x^3*z^2*u-F(77)/3*x^2*y*z^2*u-F(16)/23*x*y^2*z^2*u+F(83)/64*y^3*z^2*u-F(22)/95*x^2*z^3*u+F(95)/66*x*y*z^3*u-F(35)/109*y^2*z^3*u+F(69)/58*x*z^4*u+6658*y*z^4*u-F(103)/80*z^5*u-F(76)/87*x^4*u^2+9515*x^3*y*u^2+F(86)/87*x^2*y^2*u^2+F(31)/26*x*y^3*u^2+F(33)/113*x^3*z*u^2-F(119)/64*x^2*y*z*u^2-8122*x*y^2*z*u^2-F(109)/120*x^2*z^2*u^2-8114*x*y*z^2*u^2+F(101)/73*y^2*z^2*u^2+8041*x*z^3*u^2+11801*y*z^3*u^2+4777*z^4*u^2-11228*x^3*u^3+F(106)/89*x^2*y*u^3-5070*x*y^2*u^3-14347*x^2*z*u^3-F(46)/45*x*y*z*u^3-F(115)/58*x*z^2*u^3+F(8)/41*y*z^2*u^3-10366*z^3*u^3-F(73)/110*x^2*u^4-9231*x*y*u^4+F(79)/116*x*z*u^4-F(105)/53*z^2*u^4+7411*x^5*v-2330*x^4*y*v+F(95)/43*x^3*y^2*v-F(107)/90*x^2*y^3*v-6439*x*y^4*v+6538*x^4*z*v-F(124)/125*x^3*y*z*v+F(64)/27*x^2*y^2*z*v-F(97)/70*x*y^3*z*v+F(105)/104*x^3*z^2*v+F(39)/121*x^2*y*z^2*v-F(67)/111*x*y^2*z^2*v+F(50)/49*y^3*z^2*v-11277*x^2*z^3*v-F(91)/120*x*y*z^3*v-9164*y^2*z^3*v+F(71)/36*x*z^4*v+F(15)/92*y*z^4*v-F(49)/16*z^5*v-F(57)/32*x^4*u*v-F(1)/3*x^3*y*u*v-F(53)/118*x^2*y^2*u*v-F(47)/11*x*y^3*u*v-13900*x^3*z*u*v+F(13)/42*x^2*y*z*u*v-685*x*y^2*z*u*v-F(57)/101*x^2*z^2*u*v+7231*x*y*z^2*u*v+5129*y^2*z^2*u*v-F(5)/91*x*z^3*u*v-F(9)/95*y*z^3*u*v+5737*z^4*u*v-4482*x^3*u^2*v+11068*x^2*y*u^2*v-10153*x*y^2*u^2*v+838*x^2*z*u^2*v-F(93)/83*x*y*z*u^2*v-F(43)/9*x*z^2*u^2*v-F(21)/74*y*z^2*u^2*v+F(51)/35*z^3*u^2*v-F(3)/43*x^2*u^3*v-6205*x*y*u^3*v+F(4)/37*x*z*u^3*v+15112*z^2*u^3*v+F(84)/65*x^4*v^2+F(86)/99*x^3*y*v^2+1675*x^2*y^2*v^2+F(32)/73*x*y^3*v^2+14629*x^3*z*v^2-F(86)/125*x^2*y*z*v^2+F(109)/15*x*y^2*z*v^2+F(31)/20*x^2*z^2*v^2-13778*x*y*z^2*v^2-1176*y^2*z^2*v^2-4433*x*z^3*v^2+F(73)/33*y*z^3*v^2-F(38)/79*z^4*v^2-2274*x^3*u*v^2-F(99)/122*x^2*y*u*v^2+F(34)/63*x*y^2*u*v^2-F(58)/105*x^2*z*u*v^2+F(53)/67*x*y*z*u*v^2-F(116)/23*x*z^2*u*v^2+F(115)/56*y*z^2*u*v^2+4518*z^3*u*v^2-F(25)/54*x^2*u^2*v^2-F(2)/39*x*y*u^2*v^2+F(13)/55*x*z*u^2*v^2+F(123)/62*z^2*u^2*v^2+F(117)/50*x^3*v^3+3772*x^2*y*v^3+10275*x*y^2*v^3-F(47)/9*x^2*z*v^3-F(5)/117*x*y*z*v^3-12722*x*z^2*v^3-F(77)/36*y*z^2*v^3-F(92)/47*z^3*v^3+4432*x^2*u*v^3-F(79)/57*x*y*u*v^3+F(56)/39*x*z*u*v^3+15902*z^2*u*v^3+F(66)/59*x^2*v^4+F(102)/65*x*y*v^4-F(86)/55*x*z*v^4-8894*z^2*v^4,-x^5*y+12319*x^4*y^2+F(59)/18*x^3*y^3-699*x^2*y^4-9457*x*y^5-F(95)/22*y^6-F(120)/47*x^4*y*z+9779*x^3*y^2*z+F(2)/85*x^2*y^3*z-F(102)/73*x*y^4*z+10816*y^5*z-F(125)/122*x^3*y*z^2-F(49)/68*x^2*y^2*z^2+F(8)/55*x*y^3*z^2-F(77)/54*y^4*z^2-15002*x^2*y*z^3+F(41)/125*x*y^2*z^3-F(11)/103*y^3*z^3-8672*x*y*z^4-3607*y^2*z^4+7798*x^4*y*u-F(43)/107*x^3*y^2*u+15134*x^2*y^3*u+7902*x*y^4*u+12672*y^5*u+10289*x^3*y*z*u+F(79)/44*x^2*y^2*z*u+F(83)/74*x*y^3*z*u-F(117)/76*y^4*z*u-F(46)/101*x^3*z^2*u+F(125)/36*x^2*y*z^2*u-F(89)/120*x*y^2*z^2*u+8263*y^3*z^2*u-4254*x^2*z^3*u-F(77)/58*x*y*z^3*u-F(47)/116*y^2*z^3*u+F(43)/23*x*z^4*u-F(60)/53*y*z^4*u+11972*z^5*u+F(76)/87*x^3*y*u^2-9515*x^2*y^2*u^2-F(86)/87*x*y^3*u^2-F(31)/26*y^4*u^2-F(33)/113*x^2*y*z*u^2+F(119)/64*x*y^2*z*u^2+8122*y^3*z*u^2-F(31)/55*x^2*z^2*u^2-F(29)/98*x*y*z^2*u^2-F(37)/28*y^2*z^2*u^2-F(89)/107*x*z^3*u^2+F(59)/99*y*z^3*u^2-F(54)/83*z^4*u^2+11228*x^2*y*u^3-F(106)/89*x*y^2*u^3+5070*y^3*u^3+14347*x*y*z*u^3+F(46)/45*y^2*z*u^3-F(31)/7*x*z^2*u^3+F(37)/3*y*z^2*u^3-4647*z^3*u^3+F(73)/110*x*y*u^4+9231*y^2*u^4-F(79)/116*y*z*u^4-F(82)/9*z^2*u^4-7411*x^4*y*v+2330*x^3*y^2*v-F(95)/43*x^2*y^3*v+F(107)/90*x*y^4*v+6439*y^5*v-6538*x^3*y*z*v+F(124)/125*x^2*y^2*z*v-F(64)/27*x*y^3*z*v+F(97)/70*y^4*z*v-F(29)/40*x^3*z^2*v+F(23)/14*x^2*y*z^2*v-6582*x*y^2*z^2*v-F(81)/49*y^3*z^2*v+F(81)/31*x^2*z^3*v+11360*x*y*z^3*v+F(107)/108*y^2*z^3*v-11524*x*z^4*v-F(117)/5*y*z^4*v-F(39)/46*z^5*v+F(57)/32*x^3*y*u*v+F(1)/3*x^2*y^2*u*v+F(53)/118*x*y^3*u*v+F(47)/11*y^4*u*v+13900*x^2*y*z*u*v-F(13)/42*x*y^2*z*u*v+685*y^3*z*u*v-2090*x^2*z^2*u*v-F(13)/61*x*y*z^2*u*v-F(10)/31*y^2*z^2*u*v+F(60)/73*x*z^3*u*v-F(46)/9*y*z^3*u*v-F(31)/88*z^4*u*v+4482*x^2*y*u^2*v-11068*x*y^2*u^2*v+10153*y^3*u^2*v-838*x*y*z*u^2*v+F(93)/83*y^2*z*u^2*v+8071*x*z^2*u^2*v+5916*y*z^2*u^2*v-F(93)/116*z^3*u^2*v+F(3)/43*x*y*u^3*v+6205*y^2*u^3*v-F(4)/37*y*z*u^3*v-F(36)/37*z^2*u^3*v-F(84)/65*x^3*y*v^2-F(86)/99*x^2*y^2*v^2-1675*x*y^3*v^2-F(32)/73*y^4*v^2-14629*x^2*y*z*v^2+F(86)/125*x*y^2*z*v^2-F(109)/15*y^3*z*v^2+9611*x^2*z^2*v^2+15177*x*y*z^2*v^2+F(103)/53*y^2*z^2*v^2-F(3)/11*x*z^3*v^2+F(93)/49*y*z^3*v^2+F(97)/106*z^4*v^2+2274*x^2*y*u*v^2+F(99)/122*x*y^2*u*v^2-F(34)/63*y^3*u*v^2+F(58)/105*x*y*z*u*v^2-F(53)/67*y^2*z*u*v^2-965*x*z^2*u*v^2-F(41)/60*y*z^2*u*v^2+10111*z^3*u*v^2+F(25)/54*x*y*u^2*v^2+F(2)/39*y^2*u^2*v^2-F(13)/55*y*z*u^2*v^2+F(124)/13*z^2*u^2*v^2-F(117)/50*x^2*y*v^3-3772*x*y^2*v^3-10275*y^3*v^3+F(47)/9*x*y*z*v^3+F(5)/117*y^2*z*v^3+12855*x*z^2*v^3-12111*y*z^2*v^3+12490*z^3*v^3-4432*x*y*u*v^3+F(79)/57*y^2*u*v^3-F(56)/39*y*z*u*v^3+F(44)/23*z^2*u*v^3-F(66)/59*x*y*v^4-F(102)/65*y^2*v^4+F(86)/55*y*z*v^4-10781*z^2*v^4,14500*x^4*u-11302*x^3*y*u+F(3)/31*x^2*y^2*u+F(109)/81*x*y^3*u+F(1)/65*x^3*z*u-7814*x^2*y*z*u+F(58)/45*x*y^2*z*u-F(122)/29*x^3*u^2+3275*x^2*y*u^2+F(85)/24*x*y^2*u^2-F(95)/123*x^2*z*u^2+8048*x*y*z*u^2-3663*x*z^2*u^2-15309*x^2*u^3+7809*x*y*u^3+F(109)/85*x*z*u^3-5684*x*u^4-F(73)/98*x^4*v-F(65)/31*x^3*y*v-F(49)/95*x^2*y^2*v-F(103)/7*x*y^3*v-F(92)/75*x^3*z*v-F(98)/107*x^2*y*z*v+3357*x*y^2*z*v-11749*x^3*u*v+12836*x^2*y*u*v-F(16)/49*x*y^2*u*v+F(124)/77*y^3*u*v-11106*x^2*z*u*v-F(35)/41*x*y*z*u*v-F(81)/32*y^2*z*u*v+F(87)/106*x*z^2*u*v+3697*x^2*u^2*v-F(89)/72*x*y*u^2*v-F(109)/124*y^2*u^2*v-7581*x*z*u^2*v-F(109)/83*y*z*u^2*v-F(19)/10*z^2*u^2*v+F(29)/56*x*u^3*v+11654*y*u^3*v+F(71)/3*z*u^3*v-F(73)/86*u^4*v-F(100)/21*x^3*v^2+7090*x^2*y*v^2+F(97)/103*x*y^2*v^2-F(51)/67*y^3*v^2+8356*x^2*z*v^2+F(83)/4*x*y*z*v^2-F(104)/51*y^2*z*v^2+F(20)/113*x*z^2*v^2+F(34)/9*x^2*u*v^2+15609*x*y*u*v^2-10735*y^2*u*v^2+6187*x*z*u*v^2-F(103)/51*y*z*u*v^2+F(5)/53*z^2*u*v^2+1401*x*u^2*v^2-F(91)/5*y*u^2*v^2-F(59)/21*z*u^2*v^2-F(34)/27*u^3*v^2-5967*x^2*v^3+6828*x*y*v^3+7397*y^2*v^3-F(62)/87*x*z*v^3-F(2)/87*y*z*v^3+F(63)/115*z^2*v^3+9437*x*u*v^3-6802*y*u*v^3-F(67)/5*z*u*v^3+F(21)/118*u^2*v^3-9294*x*v^4+F(31)/37*y*v^4+1108*z*v^4-F(83)/15*u*v^4-11002*v^5,-14500*x^3*y*u+11302*x^2*y^2*u-F(3)/31*x*y^3*u-F(109)/81*y^4*u-F(1)/65*x^2*y*z*u+7814*x*y^2*z*u-F(58)/45*y^3*z*u+F(122)/29*x^2*y*u^2-3275*x*y^2*u^2-F(85)/24*y^3*u^2+F(95)/123*x*y*z*u^2-8048*y^2*z*u^2+3663*y*z^2*u^2+15309*x*y*u^3-7809*y^2*u^3-F(109)/85*y*z*u^3+5684*y*u^4+F(73)/98*x^3*y*v+F(65)/31*x^2*y^2*v+F(49)/95*x*y^3*v+F(103)/7*y^4*v+F(92)/75*x^2*y*z*v+F(98)/107*x*y^2*z*v-3357*y^3*z*v-3035*x^3*u*v+15543*x^2*y*u*v-14455*x*y^2*u*v-14524*y^3*u*v+F(39)/116*x^2*z*u*v+F(116)/51*x*y*z*u*v+F(110)/17*y^2*z*u*v-F(87)/106*y*z^2*u*v+F(75)/28*x^2*u^2*v-F(63)/20*x*y*u^2*v+14863*y^2*u^2*v+8046*x*z*u^2*v-F(123)/8*y*z*u^2*v+F(36)/103*z^2*u^2*v+F(17)/24*x*u^3*v-F(52)/67*y*u^3*v+F(76)/49*z*u^3*v+F(29)/15*u^4*v-F(91)/82*x^3*v^2+F(4)/89*x^2*y*v^2-F(39)/10*x*y^2*v^2-1252*y^3*v^2-F(97)/53*x^2*z*v^2-F(36)/89*x*y*z*v^2-F(19)/89*y^2*z*v^2-F(20)/113*y*z^2*v^2-F(26)/99*x^2*u*v^2-11300*x*y*u*v^2+10305*y^2*u*v^2-919*x*z*u*v^2-10086*y*z*u*v^2+12705*z^2*u*v^2-4906*x*u^2*v^2-F(59)/27*y*u^2*v^2-F(41)/8*z*u^2*v^2-F(92)/51*u^3*v^2-F(59)/27*x^2*v^3+14845*x*y*v^3+F(102)/113*y^2*v^3+F(69)/56*x*z*v^3-7892*y*z*v^3+F(4)/17*z^2*v^3-2678*x*u*v^3+F(101)/85*y*u*v^3-F(1)/59*z*u*v^3-F(64)/27*u^2*v^3+F(3)/71*x*v^4+F(30)/59*y*v^4+F(61)/3*z*v^4+F(67)/77*u*v^4-F(75)/22*v^5,-14500*x^3*z*u+11302*x^2*y*z*u-F(3)/31*x*y^2*z*u-F(109)/81*y^3*z*u-F(1)/65*x^2*z^2*u+7814*x*y*z^2*u-F(58)/45*y^2*z^2*u+F(122)/29*x^2*z*u^2-3275*x*y*z*u^2-F(85)/24*y^2*z*u^2+F(95)/123*x*z^2*u^2-8048*y*z^2*u^2+3663*z^3*u^2+15309*x*z*u^3-7809*y*z*u^3-F(109)/85*z^2*u^3+5684*z*u^4+F(73)/98*x^3*z*v+F(65)/31*x^2*y*z*v+F(49)/95*x*y^2*z*v+F(103)/7*y^3*z*v+F(92)/75*x^2*z^2*v+F(98)/107*x*y*z^2*v-3357*y^2*z^2*v-F(41)/67*x^3*u*v+F(40)/109*x^2*y*u*v-3423*x*y^2*u*v+F(79)/33*y^3*u*v-F(103)/58*x^2*z*u*v+F(87)/8*x*y*z*u*v+F(109)/70*y^2*z*u*v+1923*x*z^2*u*v-7087*y*z^2*u*v-F(87)/106*z^3*u*v+F(74)/39*x^2*u^2*v+F(6)/37*x*y*u^2*v+12372*y^2*u^2*v+F(61)/50*x*z*u^2*v-1565*y*z*u^2*v+4676*z^2*u^2*v-F(69)/110*x*u^3*v+1446*y*u^3*v+F(90)/121*z*u^3*v+2646*u^4*v+10492*x^3*v^2+F(61)/66*x^2*y*v^2+15192*x*y^2*v^2-F(100)/119*y^3*v^2+14819*x^2*z*v^2-F(100)/117*x*y*z*v^2+12976*y^2*z*v^2-F(31)/61*x*z^2*v^2-F(53)/67*y*z^2*v^2-F(20)/113*z^3*v^2-F(82)/11*x^2*u*v^2+F(54)/67*x*y*u*v^2+8779*y^2*u*v^2-13983*x*z*u*v^2-15146*y*z*u*v^2-F(23)/67*z^2*u*v^2+1238*x*u^2*v^2-F(66)/17*y*u^2*v^2-F(36)/19*z*u^2*v^2-F(69)/88*u^3*v^2-F(81)/100*x^2*v^3+12432*x*y*v^3-77*y^2*v^3-F(31)/39*x*z*v^3-12576*y*z*v^3-14197*z^2*v^3+12755*x*u*v^3-F(83)/82*y*u*v^3-5900*z*u*v^3+11969*u^2*v^3+12673*x*v^4+11174*y*v^4+860*z*v^4+F(17)/74*u*v^4-11133*v^5,-14500*x^3*u^2+11302*x^2*y*u^2-F(3)/31*x*y^2*u^2-F(109)/81*y^3*u^2-F(1)/65*x^2*z*u^2+7814*x*y*z*u^2-F(58)/45*y^2*z*u^2+F(122)/29*x^2*u^3-3275*x*y*u^3-F(85)/24*y^2*u^3+F(95)/123*x*z*u^3-8048*y*z*u^3+3663*z^2*u^3+15309*x*u^4-7809*y*u^4-F(109)/85*z*u^4+5684*u^5+1790*x^3*u*v+F(115)/18*x^2*y*u*v+F(23)/117*x*y^2*u*v-F(63)/10*y^3*u*v+15186*x^2*z*u*v-F(8)/39*x*y*z*u*v+9857*y^2*z*u*v+4022*x^2*u^2*v-F(94)/77*x*y*u^2*v-F(40)/23*y^2*u^2*v-41*x*z*u^2*v-2928*y*z*u^2*v+F(108)/29*z^2*u^2*v-4467*x*u^3*v+3847*y*u^3*v-7772*z*u^3*v+F(91)/85*u^4*v+9102*x^3*v^2+5295*x^2*y*v^2+F(58)/67*x*y^2*v^2-F(67)/93*y^3*v^2-F(44)/7*x^2*z*v^2-F(62)/39*x*y*z*v^2-26*y^2*z*v^2-F(83)/35*x^2*u*v^2+F(124)/49*x*y*u*v^2+2632*y^2*u*v^2-F(29)/115*x*z*u*v^2+F(54)/43*y*z*u*v^2+F(110)/9*z^2*u*v^2+F(99)/104*x*u^2*v^2+F(91)/121*y*u^2*v^2+2212*z*u^2*v^2+5013*u^3*v^2-7940*x^2*v^3+F(11)/32*x*y*v^3-F(23)/31*y^2*v^3+3261*x*z*v^3-8861*y*z*v^3-F(123)/16*z^2*v^3-4026*x*u*v^3+7715*y*u*v^3-F(26)/29*z*u*v^3-F(9)/79*u^2*v^3+F(85)/91*x*v^4-F(50)/123*y*v^4+F(41)/3*z*v^4+F(114)/115*u*v^4-3102*v^5,-F(40)/33*x^4*u-11978*x^3*y*u+10010*x^2*y^2*u+436*x*y^3*u-F(36)/125*x^3*z*u+3395*x^2*y*z*u+F(15)/46*x*y^2*z*u-7727*x^3*u^2+F(54)/101*x^2*y*u^2-8234*x*y^2*u^2+F(124)/77*y^3*u^2+F(91)/66*x^2*z*u^2-9171*x*y*z*u^2-F(81)/32*y^2*z*u^2-1109*x*z^2*u^2+F(72)/83*x^2*u^3+F(33)/23*x*y*u^3-F(109)/124*y^2*u^3+F(67)/25*x*z*u^3-F(109)/83*y*z*u^3-F(19)/10*z^2*u^3+F(65)/79*x*u^4+11654*y*u^4+F(71)/3*z*u^4-F(73)/86*u^5+9102*x^4*v+5295*x^3*y*v+F(58)/67*x^2*y^2*v-F(67)/93*x*y^3*v-F(44)/7*x^3*z*v-F(62)/39*x^2*y*z*v-26*x*y^2*z*v-F(107)/15*x^3*u*v+F(85)/43*x^2*y*u*v+F(90)/53*x*y^2*u*v-F(51)/67*y^3*u*v+F(29)/57*x^2*z*u*v+208*x*y*z*u*v-F(104)/51*y^2*z*u*v-F(121)/8*x*z^2*u*v-4541*x^2*u^2*v+804*x*y*u^2*v-10735*y^2*u^2*v+F(109)/80*x*z*u^2*v-F(103)/51*y*z*u^2*v+F(5)/53*z^2*u^2*v+F(79)/5*x*u^3*v-F(91)/5*y*u^3*v-F(59)/21*z*u^3*v-F(34)/27*u^4*v-7940*x^3*v^2+F(11)/32*x^2*y*v^2-F(23)/31*x*y^2*v^2+3261*x^2*z*v^2-8861*x*y*z*v^2-F(123)/16*x*z^2*v^2+F(67)/16*x^2*u*v^2+F(18)/11*x*y*u*v^2+7397*y^2*u*v^2-8459*x*z*u*v^2-F(2)/87*y*z*u*v^2+F(63)/115*z^2*u*v^2-F(94)/121*x*u^2*v^2-6802*y*u^2*v^2-F(67)/5*z*u^2*v^2+F(21)/118*u^3*v^2+F(85)/91*x^2*v^3-F(50)/123*x*y*v^3+F(41)/3*x*z*v^3-F(76)/77*x*u*v^3+F(31)/37*y*u*v^3+1108*z*u*v^3-F(83)/15*u^2*v^3-3102*x*v^4-11002*u*v^4,F(40)/33*x^3*y*u+11978*x^2*y^2*u-10010*x*y^3*u-436*y^4*u+F(36)/125*x^2*y*z*u-3395*x*y^2*z*u-F(15)/46*y^3*z*u-3035*x^3*u^2+F(106)/25*x^2*y*u^2+7566*x*y^2*u^2+F(80)/77*y^3*u^2+F(39)/116*x^2*z*u^2-584*x*y*z*u^2+F(6)/59*y^2*z*u^2+1109*y*z^2*u^2+F(75)/28*x^2*u^3+F(37)/19*x*y*u^3+11016*y^2*u^3+8046*x*z*u^3-F(19)/83*y*z*u^3+F(36)/103*z^2*u^3+F(17)/24*x*u^4-2378*y*u^4+F(76)/49*z*u^4+F(29)/15*u^5-9102*x^3*y*v-5295*x^2*y^2*v-F(58)/67*x*y^3*v+F(67)/93*y^4*v+F(44)/7*x^2*y*z*v+F(62)/39*x*y^2*z*v+26*y^3*z*v-F(91)/82*x^3*u*v-15115*x^2*y*u*v-F(97)/76*x*y^2*u*v-3884*y^3*u*v-F(97)/53*x^2*z*u*v-F(119)/22*x*y*z*u*v+15179*y^2*z*u*v+F(121)/8*y*z^2*u*v-F(26)/99*x^2*u^2*v-F(47)/30*x*y*u^2*v-F(113)/93*y^2*u^2*v-919*x*z*u^2*v+F(81)/13*y*z*u^2*v+12705*z^2*u^2*v-4906*x*u^3*v+F(123)/64*y*u^3*v-F(41)/8*z*u^3*v-F(92)/51*u^4*v+7940*x^2*y*v^2-F(11)/32*x*y^2*v^2+F(23)/31*y^3*v^2-3261*x*y*z*v^2+8861*y^2*z*v^2+F(123)/16*y*z^2*v^2-F(59)/27*x^2*u*v^2-13120*x*y*u*v^2-F(44)/49*y^2*u*v^2+F(69)/56*x*z*u*v^2+934*y*z*u*v^2+F(4)/17*z^2*u*v^2-2678*x*u^2*v^2+F(41)/101*y*u^2*v^2-F(1)/59*z*u^2*v^2-F(64)/27*u^3*v^2-F(85)/91*x*y*v^3+F(50)/123*y^2*v^3-F(41)/3*y*z*v^3+F(3)/71*x*u*v^3+15375*y*u*v^3+F(61)/3*z*u*v^3+F(67)/77*u^2*v^3+3102*y*v^4-F(75)/22*u*v^4,F(40)/33*x^3*z*u+11978*x^2*y*z*u-10010*x*y^2*z*u-436*y^3*z*u+F(36)/125*x^2*z^2*u-3395*x*y*z^2*u-F(15)/46*y^2*z^2*u-F(41)/67*x^3*u^2+F(40)/109*x^2*y*u^2-3423*x*y^2*u^2+F(79)/33*y^3*u^2+F(15)/88*x^2*z*u^2+F(94)/55*x*y*z*u^2+F(13)/16*y^2*z*u^2-F(41)/114*x*z^2*u^2-F(17)/100*y*z^2*u^2+1109*z^3*u^2+F(74)/39*x^2*u^3+F(6)/37*x*y*u^3+12372*y^2*u^3-8968*x*z*u^3+F(121)/65*y*z*u^3+12448*z^2*u^3-F(69)/110*x*u^4+1446*y*u^4+5275*z*u^4+2646*u^5-9102*x^3*z*v-5295*x^2*y*z*v-F(58)/67*x*y^2*z*v+F(67)/93*y^3*z*v+F(44)/7*x^2*z^2*v+F(62)/39*x*y*z^2*v+26*y^2*z^2*v+10492*x^3*u*v+F(61)/66*x^2*y*u*v+15192*x*y^2*u*v-F(100)/119*y^3*u*v+F(103)/98*x^2*z*u*v-F(25)/42*x*y*z*u*v+10344*y^2*z*u*v+F(99)/23*x*z^2*u*v+F(59)/20*y*z^2*u*v+F(121)/8*z^3*u*v-F(82)/11*x^2*u^2*v+F(54)/67*x*y*u^2*v+8779*y^2*u^2*v-F(23)/87*x*z*u^2*v-F(63)/94*y*z*u^2*v+F(77)/85*z^2*u^2*v+1238*x*u^3*v-F(66)/17*y*u^3*v+8455*z*u^3*v-F(69)/88*u^4*v+7940*x^2*z*v^2-F(11)/32*x*y*z*v^2+F(23)/31*y^2*z*v^2-3261*x*z^2*v^2+8861*y*z^2*v^2+F(123)/16*z^3*v^2-F(81)/100*x^2*u*v^2+12432*x*y*u*v^2-77*y^2*u*v^2-14021*x*z*u*v^2+11700*y*z*u*v^2-5371*z^2*u*v^2+12755*x*u^2*v^2-F(83)/82*y*u^2*v^2+3009*z*u^2*v^2+11969*u^3*v^2-F(85)/91*x*z*v^3+F(50)/123*y*z*v^3-F(41)/3*z^2*v^3+12673*x*u*v^3+11174*y*u*v^3-2201*z*u*v^3+F(17)/74*u^2*v^3+3102*z*v^4-11133*u*v^4,-F(41)/67*x^4*u+F(40)/109*x^3*y*u-3423*x^2*y^2*u+F(79)/33*x*y^3*u-F(115)/119*x^3*z*u-F(27)/47*x^2*y*z*u+1764*x*y^2*z*u+F(124)/77*y^3*z*u-F(43)/108*x^2*z^2*u-F(5)/12*x*y*z^2*u-F(81)/32*y^2*z^2*u+F(74)/39*x^3*u^2+F(6)/37*x^2*y*u^2+12372*x*y^2*u^2-F(37)/23*x^2*z*u^2+1544*x*y*z*u^2-F(109)/124*y^2*z*u^2+F(36)/11*x*z^2*u^2-F(109)/83*y*z^2*u^2-F(19)/10*z^3*u^2-F(69)/110*x^2*u^3+1446*x*y*u^3-F(72)/29*x*z*u^3+11654*y*z*u^3+F(71)/3*z^2*u^3+2646*x*u^4-F(73)/86*z*u^4+10492*x^4*v+F(61)/66*x^3*y*v+15192*x^2*y^2*v-F(100)/119*x*y^3*v-F(77)/120*x^3*z*v+7636*x^2*y*z*v-F(97)/6*x*y^2*z*v-F(51)/67*y^3*z*v-F(45)/16*x^2*z^2*v-F(33)/106*x*y*z^2*v-F(104)/51*y^2*z^2*v-F(82)/11*x^3*u*v+F(54)/67*x^2*y*u*v+8779*x*y^2*u*v+239*x^2*z*u*v-F(44)/69*x*y*z*u*v-10735*y^2*z*u*v-F(11)/47*x*z^2*u*v-F(103)/51*y*z^2*u*v+F(5)/53*z^3*u*v+1238*x^2*u^2*v-F(66)/17*x*y*u^2*v-F(4)/71*x*z*u^2*v-F(91)/5*y*z*u^2*v-F(59)/21*z^2*u^2*v-F(69)/88*x*u^3*v-F(34)/27*z*u^3*v-F(81)/100*x^3*v^2+12432*x^2*y*v^2-77*x*y^2*v^2-F(83)/4*x^2*z*v^2-5748*x*y*z*v^2+7397*y^2*z*v^2-13830*x*z^2*v^2-F(2)/87*y*z^2*v^2+F(63)/115*z^3*v^2+12755*x^2*u*v^2-F(83)/82*x*y*u*v^2+3537*x*z*u*v^2-6802*y*z*u*v^2-F(67)/5*z^2*u*v^2+11969*x*u^2*v^2+F(21)/118*z*u^2*v^2+12673*x^2*v^3+11174*x*y*v^3-F(1)/110*x*z*v^3+F(31)/37*y*z*v^3+1108*z^2*v^3+F(17)/74*x*u*v^3-F(83)/15*z*u*v^3-11133*x*v^4-11002*z*v^4,F(41)/67*x^3*y*u-F(40)/109*x^2*y^2*u+3423*x*y^3*u-F(79)/33*y^4*u-3035*x^3*z*u+F(107)/12*x^2*y*z*u-10467*x*y^2*z*u-F(4)/97*y^3*z*u+F(39)/116*x^2*z^2*u-F(5)/113*x*y*z^2*u+1448*y^2*z^2*u-F(74)/39*x^2*y*u^2-F(6)/37*x*y^2*u^2-12372*y^3*u^2+F(75)/28*x^2*z*u^2+2235*x*y*z*u^2+F(7)/37*y^2*z*u^2+8046*x*z^2*u^2+F(68)/23*y*z^2*u^2+F(36)/103*z^3*u^2+F(69)/110*x*y*u^3-1446*y^2*u^3+F(17)/24*x*z*u^3-7653*y*z*u^3+F(76)/49*z^2*u^3-2646*y*u^4+F(29)/15*z*u^4-10492*x^3*y*v-F(61)/66*x^2*y^2*v-15192*x*y^3*v+F(100)/119*y^4*v-F(91)/82*x^3*z*v+F(33)/101*x^2*y*z*v-3749*x*y^2*z*v-F(88)/9*y^3*z*v-F(97)/53*x^2*z^2*v+F(15)/31*x*y*z^2*v+F(58)/119*y^2*z^2*v+F(82)/11*x^2*y*u*v-F(54)/67*x*y^2*u*v-8779*y^3*u*v-F(26)/99*x^2*z*u*v+2683*x*y*z*u*v-6540*y^2*z*u*v-919*x*z^2*u*v+F(26)/81*y*z^2*u*v+12705*z^3*u*v-1238*x*y*u^2*v+F(66)/17*y^2*u^2*v-4906*x*z*u^2*v+F(49)/46*y*z*u^2*v-F(41)/8*z^2*u^2*v+F(69)/88*y*u^3*v-F(92)/51*z*u^3*v+F(81)/100*x^2*y*v^2-12432*x*y^2*v^2+77*y^3*v^2-F(59)/27*x^2*z*v^2-F(11)/71*x*y*z*v^2-F(93)/11*y^2*z*v^2+F(69)/56*x*z^2*v^2+6305*y*z^2*v^2+F(4)/17*z^3*v^2-12755*x*y*u*v^2+F(83)/82*y^2*u*v^2-2678*x*z*u*v^2-F(122)/79*y*z*u*v^2-F(1)/59*z^2*u*v^2-11969*y*u^2*v^2-F(64)/27*z*u^2*v^2-12673*x*y*v^3-11174*y^2*v^3+F(3)/71*x*z*v^3-14415*y*z*v^3+F(61)/3*z^2*v^3-F(17)/74*y*u*v^3+F(67)/77*z*u*v^3+11133*y*v^4-F(75)/22*z*v^4,3035*x^4*u+F(91)/59*x^3*y*u-F(63)/79*x^2*y^2*u-5062*x*y^3*u-F(124)/77*y^4*u-F(39)/116*x^3*z*u+F(29)/30*x^2*y*z*u-F(13)/35*x*y^2*z*u+F(81)/32*y^3*z*u-F(75)/28*x^3*u^2-F(86)/81*x^2*y*u^2+F(42)/89*x*y^2*u^2+F(109)/124*y^3*u^2-8046*x^2*z*u^2-F(119)/80*x*y*z*u^2+F(109)/83*y^2*z*u^2-F(36)/103*x*z^2*u^2+F(19)/10*y*z^2*u^2-F(17)/24*x^2*u^3-F(123)/118*x*y*u^3-11654*y^2*u^3-F(76)/49*x*z*u^3-F(71)/3*y*z*u^3-F(29)/15*x*u^4+F(73)/86*y*u^4+F(91)/82*x^4*v+193*x^3*y*v-F(26)/107*x^2*y^2*v-F(39)/67*x*y^3*v+F(51)/67*y^4*v+F(97)/53*x^3*z*v-5480*x^2*y*z*v+F(85)/79*x*y^2*z*v+F(104)/51*y^3*z*v+F(26)/99*x^3*u*v-2922*x^2*y*u*v+6077*x*y^2*u*v+10735*y^3*u*v+919*x^2*z*u*v-F(96)/41*x*y*z*u*v+F(103)/51*y^2*z*u*v-12705*x*z^2*u*v-F(5)/53*y*z^2*u*v+4906*x^2*u^2*v+8080*x*y*u^2*v+F(91)/5*y^2*u^2*v+F(41)/8*x*z*u^2*v+F(59)/21*y*z*u^2*v+F(92)/51*x*u^3*v+F(34)/27*y*u^3*v+F(59)/27*x^3*v^2-8878*x^2*y*v^2+F(84)/47*x*y^2*v^2-7397*y^3*v^2-F(69)/56*x^2*z*v^2-F(39)/17*x*y*z*v^2+F(2)/87*y^2*z*v^2-F(4)/17*x*z^2*v^2-F(63)/115*y*z^2*v^2+2678*x^2*u*v^2+F(81)/49*x*y*u*v^2+6802*y^2*u*v^2+F(1)/59*x*z*u*v^2+F(67)/5*y*z*u*v^2+F(64)/27*x*u^2*v^2-F(21)/118*y*u^2*v^2-F(3)/71*x^2*v^3-F(12)/7*x*y*v^3-F(31)/37*y^2*v^3-F(61)/3*x*z*v^3-1108*y*z*v^3-F(67)/77*x*u*v^3+F(83)/15*y*u*v^3+F(75)/22*x*v^4+11002*y*v^4,3378*x^4*u-F(119)/88*x^3*y*u+13981*x^2*y^2*u-F(55)/89*x*y^3*u+11358*x^3*z*u+F(67)/18*x^2*y*z*u+F(27)/19*x*y^2*z*u-F(13)/69*x^2*z^2*u+7851*x*y*z^2*u-447*x^3*u^2+F(122)/19*x^2*y*u^2-F(103)/46*x*y^2*u^2+F(2)/59*x^2*z*u^2+2970*x*y*z*u^2-13481*x*z^2*u^2+F(25)/126*x^2*u^3+F(5)/122*x*y*u^3+2625*x*z*u^3-15341*x*u^4-6643*x^4*v+F(47)/69*x^3*y*v-F(11)/30*x^2*y^2*v-F(37)/45*x*y^3*v-3183*x^3*z*v+14990*x^2*y*z*v-F(46)/15*x*y^2*z*v-F(95)/41*x^2*z^2*v-5361*x*y*z^2*v+F(81)/34*x^3*u*v-F(100)/67*x^2*y*u*v+9422*x*y^2*u*v+F(83)/64*y^3*u*v-5653*x^2*z*u*v-7683*x*y*z*u*v-F(35)/109*y^2*z*u*v+14754*x*z^2*u*v+6658*y*z^2*u*v-F(103)/80*z^3*u*v-14787*x^2*u^2*v+F(48)/91*x*y*u^2*v+F(101)/73*y^2*u^2*v-747*x*z*u^2*v+11801*y*z*u^2*v+4777*z^2*u^2*v-F(41)/71*x*u^3*v+F(8)/41*y*u^3*v-10366*z*u^3*v-F(105)/53*u^4*v-F(27)/35*x^3*v^2-F(96)/43*x^2*y*v^2+F(83)/41*x*y^2*v^2+F(50)/49*y^3*v^2+F(52)/7*x^2*z*v^2+F(26)/89*x*y*z*v^2-9164*y^2*z*v^2-3536*x*z^2*v^2+F(15)/92*y*z^2*v^2-F(49)/16*z^3*v^2-9639*x^2*u*v^2-F(107)/45*x*y*u*v^2+5129*y^2*u*v^2+F(107)/123*x*z*u*v^2-F(9)/95*y*z*u*v^2+5737*z^2*u*v^2-6726*x*u^2*v^2-F(21)/74*y*u^2*v^2+F(51)/35*z*u^2*v^2+15112*u^3*v^2-4237*x^2*v^3-14482*x*y*v^3-1176*y^2*v^3-4590*x*z*v^3+F(73)/33*y*z*v^3-F(38)/79*z^2*v^3-F(66)/113*x*u*v^3+F(115)/56*y*u*v^3+4518*z*u*v^3+F(123)/62*u^2*v^3-6360*x*v^4-F(77)/36*y*v^4-F(92)/47*z*v^4+15902*u*v^4-8894*v^5,3378*x^3*y*u-F(119)/88*x^2*y^2*u+13981*x*y^3*u-F(55)/89*y^4*u+11358*x^2*y*z*u+F(67)/18*x*y^2*z*u+F(27)/19*y^3*z*u-F(13)/69*x*y*z^2*u+7851*y^2*z^2*u-447*x^2*y*u^2+F(122)/19*x*y^2*u^2-F(103)/46*y^3*u^2+F(2)/59*x*y*z*u^2+2970*y^2*z*u^2-13481*y*z^2*u^2+F(25)/126*x*y*u^3+F(5)/122*y^2*u^3+2625*y*z*u^3-15341*y*u^4-6643*x^3*y*v+F(47)/69*x^2*y^2*v-F(11)/30*x*y^3*v-F(37)/45*y^4*v-3183*x^2*y*z*v+14990*x*y^2*z*v-F(46)/15*y^3*z*v-F(95)/41*x*y*z^2*v-5361*y^2*z^2*v+F(46)/101*x^3*u*v+10023*x^2*y*u*v-10213*x*y^2*u*v+12287*y^3*u*v+4254*x^2*z*u*v+F(101)/117*x*y*z*u*v+648*y^2*z*u*v-F(43)/23*x*z^2*u*v+F(45)/112*y*z^2*u*v-11972*z^3*u*v+F(31)/55*x^2*u^2*v+5720*x*y*u^2*v-585*y^2*u^2*v+F(89)/107*x*z*u^2*v-F(17)/56*y*z*u^2*v+F(54)/83*z^2*u^2*v+F(31)/7*x*u^3*v-14085*y*u^3*v+4647*z*u^3*v+F(82)/9*u^4*v+F(29)/40*x^3*v^2+F(96)/31*x^2*y*v^2+F(29)/35*x*y^2*v^2+F(69)/95*y^3*v^2-F(81)/31*x^2*z*v^2-13786*x*y*z*v^2-13889*y^2*z*v^2+11524*x*z^2*v^2-F(67)/65*y*z^2*v^2+F(39)/46*z^3*v^2+2090*x^2*u*v^2-F(4)/65*x*y*u*v^2+F(28)/123*y^2*u*v^2-F(60)/73*x*z*u*v^2+5164*y*z*u*v^2+F(31)/88*z^2*u*v^2-8071*x*u^2*v^2+F(97)/81*y*u^2*v^2+F(93)/116*z*u^2*v^2+F(36)/37*u^3*v^2-9611*x^2*v^3+14175*x*y*v^3-F(2)/59*y^2*v^3+F(3)/11*x*z*v^3+F(121)/28*y*z*v^3-F(97)/106*z^2*v^3+965*x*u*v^3+F(124)/65*y*u*v^3-10111*z*u*v^3-F(124)/13*u^2*v^3-12855*x*v^4-F(48)/71*y*v^4-12490*z*v^4-F(44)/23*u*v^4+10781*v^5,-3378*x^3*u^2+F(119)/88*x^2*y*u^2-13981*x*y^2*u^2+F(55)/89*y^3*u^2-11358*x^2*z*u^2-F(67)/18*x*y*z*u^2-F(27)/19*y^2*z*u^2+F(13)/69*x*z^2*u^2-7851*y*z^2*u^2+447*x^2*u^3-F(122)/19*x*y*u^3+F(103)/46*y^2*u^3-F(2)/59*x*z*u^3-2970*y*z*u^3+13481*z^2*u^3-F(25)/126*x*u^4-F(5)/122*y*u^4-2625*z*u^4+15341*u^5+7019*x^3*u*v+F(66)/119*x^2*y*u*v-F(70)/39*x*y^2*u*v-13818*y^3*u*v-3260*x^2*z*u*v-636*x*y*z*u*v+4252*y^2*z*u*v-F(27)/83*x*z^2*u*v-F(29)/96*y*z^2*u*v-F(122)/35*x^2*u^2*v-F(61)/17*x*y*u^2*v+F(54)/91*y^2*u^2*v-184*x*z*u^2*v-1576*y*z*u^2*v+F(66)/71*z^2*u^2*v+F(118)/3*x*u^3*v+3674*y*u^3*v-F(68)/43*z*u^3*v-F(81)/46*u^4*v-F(43)/7*x^3*v^2-F(105)/71*x^2*y*v^2-14558*x*y^2*v^2-F(17)/15*y^3*v^2+9746*x^2*z*v^2-F(48)/121*x*y*z*v^2-14749*y^2*z*v^2-F(12)/5*x*z^2*v^2-F(40)/33*y*z^2*v^2-5623*x^2*u*v^2+F(93)/10*x*y*u*v^2+29*y^2*u*v^2-934*x*z*u*v^2-F(97)/79*y*z*u*v^2+7300*z^2*u*v^2+F(85)/47*x*u^2*v^2-3212*y*u^2*v^2-12595*z*u^2*v^2+F(10)/119*u^3*v^2-F(43)/29*x^2*v^3+F(9)/40*x*y*v^3+3376*y^2*v^3+F(59)/75*x*z*v^3-3865*y*z*v^3-F(49)/103*z^2*v^3-F(38)/51*x*u*v^3-F(10)/61*y*u*v^3+F(116)/123*z*u*v^3+7033*u^2*v^3+F(86)/49*x*v^4+F(27)/59*y*v^4-F(11)/98*z*v^4-242*u*v^4+F(79)/87*v^5,-F(31)/85*x^4*u-F(110)/97*x^3*y*u+F(71)/81*x^2*y^2*u-F(39)/80*x*y^3*u-6443*x^3*z*u-F(73)/78*x^2*y*z*u-4282*x*y^2*z*u-F(106)/33*x^2*z^2*u+8968*x*y*z^2*u+8951*x^3*u^2+4545*x^2*y*u^2+5204*x*y^2*u^2+F(83)/64*y^3*u^2-5837*x^2*z*u^2+F(59)/38*x*y*z*u^2-F(35)/109*y^2*z*u^2+9348*x*z^2*u^2+6658*y*z^2*u^2-F(103)/80*z^3*u^2-F(2)/47*x^2*u^3-10739*x*y*u^3+F(101)/73*y^2*u^3+12643*x*z*u^3+11801*y*z*u^3+4777*z^2*u^3+F(35)/73*x*u^4+F(8)/41*y*u^4-10366*z*u^4-F(105)/53*u^5-F(43)/7*x^4*v-F(105)/71*x^3*y*v-14558*x^2*y^2*v-F(17)/15*x*y^3*v+9746*x^3*z*v-F(48)/121*x^2*y*z*v-14749*x*y^2*z*v-F(12)/5*x^2*z^2*v-F(40)/33*x*y*z^2*v-12936*x^3*u*v+14961*x^2*y*u*v-F(45)/74*x*y^2*u*v+F(50)/49*y^3*u*v-14637*x^2*z*u*v-F(2)/7*x*y*z*u*v-9164*y^2*z*u*v+F(6)/17*x*z^2*u*v+F(15)/92*y*z^2*u*v-F(49)/16*z^3*u*v+3976*x^2*u^2*v+F(112)/109*x*y*u^2*v+5129*y^2*u^2*v-F(59)/55*x*z*u^2*v-F(9)/95*y*z*u^2*v+5737*z^2*u^2*v-F(87)/17*x*u^3*v-F(21)/74*y*u^3*v+F(51)/35*z*u^3*v+15112*u^4*v-F(43)/29*x^3*v^2+F(9)/40*x^2*y*v^2+3376*x*y^2*v^2+F(59)/75*x^2*z*v^2-3865*x*y*z*v^2-F(49)/103*x*z^2*v^2-F(87)/110*x^2*u*v^2+2300*x*y*u*v^2-1176*y^2*u*v^2+F(48)/23*x*z*u*v^2+F(73)/33*y*z*u*v^2-F(38)/79*z^2*u*v^2-F(73)/108*x*u^2*v^2+F(115)/56*y*u^2*v^2+4518*z*u^2*v^2+F(123)/62*u^3*v^2+F(86)/49*x^2*v^3+F(27)/59*x*y*v^3-F(11)/98*x*z*v^3-F(43)/63*x*u*v^3-F(77)/36*y*u*v^3-F(92)/47*z*u*v^3+15902*u^2*v^3+F(79)/87*x*v^4-8894*u*v^4,-F(31)/85*x^3*y*u-F(110)/97*x^2*y^2*u+F(71)/81*x*y^3*u-F(39)/80*y^4*u-6443*x^2*y*z*u-F(73)/78*x*y^2*z*u-4282*y^3*z*u-F(106)/33*x*y*z^2*u+8968*y^2*z^2*u+F(46)/101*x^3*u^2-6433*x^2*y*u^2-F(116)/119*x*y^2*u^2-F(89)/111*y^3*u^2+4254*x^2*z*u^2-F(23)/27*x*y*z*u^2-F(50)/69*y^2*z*u^2-F(43)/23*x*z^2*u^2-F(78)/23*y*z^2*u^2-11972*z^3*u^2+F(31)/55*x^2*u^3-15568*x*y*u^3+3089*y^2*u^3+F(89)/107*x*z*u^3-F(121)/36*y*z*u^3+F(54)/83*z^2*u^3+F(31)/7*x*u^4+F(96)/113*y*u^4+4647*z*u^4+F(82)/9*u^5-F(43)/7*x^3*y*v-F(105)/71*x^2*y^2*v-14558*x*y^3*v-F(17)/15*y^4*v+9746*x^2*y*z*v-F(48)/121*x*y^2*z*v-14749*y^3*z*v-F(12)/5*x*y*z^2*v-F(40)/33*y^2*z^2*v+F(29)/40*x^3*u*v+F(61)/38*x^2*y*u*v-F(62)/39*x*y^2*u*v-F(59)/32*y^3*u*v-F(81)/31*x^2*z*u*v-14720*x*y*z*u*v-F(12)/113*y^2*z*u*v+11524*x*z^2*u*v+13205*y*z^2*u*v+F(39)/46*z^3*u*v+2090*x^2*u^2*v-F(68)/39*x*y*u^2*v+4851*y^2*u^2*v-F(60)/73*x*z*u^2*v+F(124)/99*y*z*u^2*v+F(31)/88*z^2*u^2*v-8071*x*u^3*v+F(95)/56*y*u^3*v+F(93)/116*z*u^3*v+F(36)/37*u^4*v-F(43)/29*x^2*y*v^2+F(9)/40*x*y^2*v^2+3376*y^3*v^2+F(59)/75*x*y*z*v^2-3865*y^2*z*v^2-F(49)/103*y*z^2*v^2-9611*x^2*u*v^2-F(83)/97*x*y*u*v^2-F(17)/41*y^2*u*v^2+F(3)/11*x*z*u*v^2+11700*y*z*u*v^2-F(97)/106*z^2*u*v^2+965*x*u^2*v^2-7238*y*u^2*v^2-10111*z*u^2*v^2-F(124)/13*u^3*v^2+F(86)/49*x*y*v^3+F(27)/59*y^2*v^3-F(11)/98*y*z*v^3-12855*x*u*v^3-F(40)/93*y*u*v^3-12490*z*u*v^3-F(44)/23*u^2*v^3+F(79)/87*y*v^4+10781*u*v^4,F(46)/101*x^4*u-15384*x^3*y*u-F(54)/109*x^2*y^2*u+F(9)/67*x*y^3*u-F(83)/64*y^4*u+4254*x^3*z*u-F(58)/41*x^2*y*z*u+F(1)/96*x*y^2*z*u+F(35)/109*y^3*z*u-F(43)/23*x^2*z^2*u-F(36)/83*x*y*z^2*u-6658*y^2*z^2*u-11972*x*z^3*u+F(103)/80*y*z^3*u+F(31)/55*x^3*u^2-F(2)/39*x^2*y*u^2+13828*x*y^2*u^2-F(101)/73*y^3*u^2+F(89)/107*x^2*z*u^2-F(9)/26*x*y*z*u^2-11801*y^2*z*u^2+F(54)/83*x*z^2*u^2-4777*y*z^2*u^2+F(31)/7*x^2*u^3+F(70)/11*x*y*u^3-F(8)/41*y^2*u^3+4647*x*z*u^3+10366*y*z*u^3+F(82)/9*x*u^4+F(105)/53*y*u^4+F(29)/40*x^4*v+10412*x^3*y*v+12927*x^2*y^2*v+8672*x*y^3*v-F(50)/49*y^4*v-F(81)/31*x^3*z*v-83*x^2*y*z*v-8857*x*y^2*z*v+9164*y^3*z*v+11524*x^2*z^2*v+F(63)/61*x*y*z^2*v-F(15)/92*y^2*z^2*v+F(39)/46*x*z^3*v+F(49)/16*y*z^3*v+2090*x^3*u*v-10540*x^2*y*u*v+3089*x*y^2*u*v-5129*y^3*u*v-F(60)/73*x^2*z*u*v-3940*x*y*z*u*v+F(9)/95*y^2*z*u*v+F(31)/88*x*z^2*u*v-5737*y*z^2*u*v-8071*x^2*u^2*v+8307*x*y*u^2*v+F(21)/74*y^2*u^2*v+F(93)/116*x*z*u^2*v-F(51)/35*y*z*u^2*v+F(36)/37*x*u^3*v-15112*y*u^3*v-9611*x^3*v^2-13579*x^2*y*v^2+F(123)/61*x*y^2*v^2+1176*y^3*v^2+F(3)/11*x^2*z*v^2-F(109)/90*x*y*z*v^2-F(73)/33*y^2*z*v^2-F(97)/106*x*z^2*v^2+F(38)/79*y*z^2*v^2+965*x^2*u*v^2+168*x*y*u*v^2-F(115)/56*y^2*u*v^2-10111*x*z*u*v^2-4518*y*z*u*v^2-F(124)/13*x*u^2*v^2-F(123)/62*y*u^2*v^2-12855*x^2*v^3-7158*x*y*v^3+F(77)/36*y^2*v^3-12490*x*z*v^3+F(92)/47*y*z*v^3-F(44)/23*x*u*v^3-15902*y*u*v^3+10781*x*v^4+8894*y*v^4,F(57)/34*x^4*u+F(103)/68*x^3*y*u-F(34)/63*x^2*y^2*u-796*x*y^3*u+F(95)/22*y^4*u+F(20)/13*x^3*z*u+14680*x^2*y*z*u-F(59)/80*x*y^2*z*u+F(63)/118*y^3*z*u-F(61)/85*x^2*z^2*u-F(1)/60*x*y*z^2*u-F(120)/61*y^2*z^2*u+F(87)/7*x*z^3*u+9417*y*z^3*u+2922*x^3*u^2+2122*x^2*y*u^2+F(65)/113*x*y^2*u^2-12672*y^3*u^2-F(109)/61*x^2*z*u^2+F(21)/107*x*y*z*u^2+2266*y^2*z*u^2-F(11)/18*x*z^2*u^2-F(61)/105*y*z^2*u^2+F(81)/83*z^3*u^2+F(107)/11*x^2*u^3-13396*x*y*u^3+F(31)/26*y^2*u^3-F(109)/98*x*z*u^3+14582*y*z*u^3+F(119)/2*z^2*u^3-15685*x*u^4-5070*y*u^4-F(29)/47*z*u^4-9231*u^5-70*x^4*v+F(83)/82*x^3*y*v+F(32)/99*x^2*y^2*v+F(21)/83*x*y^3*v-978*x^3*z*v-F(31)/42*x^2*y*z*v+F(83)/102*x*y^2*z*v+2619*y^3*z*v+F(3)/43*x^2*z^2*v-3749*x*y*z^2*v+430*y^2*z^2*v-F(92)/49*x*z^3*v+F(10)/59*y*z^3*v-2763*x^3*u*v-9714*x^2*y*u*v+F(83)/106*x*y^2*u*v-6439*y^3*u*v+F(113)/9*x^2*z*u*v-461*x*y*z*u*v+8857*y^2*z*u*v+F(116)/31*x*z^2*u*v-F(37)/92*y*z^2*u*v-3144*z^3*u*v+5527*x^2*u^2*v-3462*x*y*u^2*v-F(47)/11*y^2*u^2*v-10355*x*z*u^2*v+10197*y*z*u^2*v+3952*z^2*u^2*v+F(72)/113*x*u^3*v-10153*y*u^3*v+F(5)/82*z*u^3*v-6205*u^4*v-F(44)/25*x^3*v^2-5150*x^2*y*v^2-2804*x*y^2*v^2-F(106)/45*x^2*z*v^2-F(99)/104*x*y*z*v^2+F(73)/112*y^2*z*v^2-6102*x*z^2*v^2-F(100)/49*y*z^2*v^2-F(47)/101*z^3*v^2+13206*x^2*u*v^2+F(98)/17*x*y*u*v^2+F(32)/73*y^2*u*v^2+15915*x*z*u*v^2+F(89)/70*y*z*u*v^2-F(122)/117*z^2*u*v^2-F(48)/73*x*u^2*v^2+F(34)/63*y*u^2*v^2+2925*z*u^2*v^2-F(2)/39*u^3*v^2-F(93)/19*x^2*v^3-F(121)/35*x*y*v^3+F(102)/25*x*z*v^3-F(84)/109*y*z*v^3-F(101)/98*z^2*v^3-F(5)/114*x*u*v^3+10275*y*u*v^3+F(104)/9*z*u*v^3-F(79)/57*u^2*v^3-F(59)/8*x*v^4-F(34)/115*z*v^4+F(102)/65*u*v^4,x^4*u-F(95)/107*x^3*y*u-F(4)/99*x^2*y^2*u+11871*x*y^3*u-F(41)/78*y^4*u+14710*x^3*z*u-F(25)/118*x^2*y*z*u+14083*x*y^2*z*u-F(101)/98*y^3*z*u+14360*x^2*z^2*u-13319*x*y*z^2*u+4547*y^2*z^2*u+F(101)/23*x*z^3*u+13879*y*z^3*u-7798*x^3*u^2-12489*x^2*y*u^2+14735*x*y^2*u^2-9035*y^3*u^2+F(100)/13*x^2*z*u^2+F(91)/50*x*y*z*u^2+F(73)/35*y^2*z*u^2-11561*x*z^2*u^2+7280*y*z^2*u^2+10972*z^3*u^2-F(76)/87*x^2*u^3+F(89)/97*x*y*u^3+10823*y^2*u^3-F(9)/7*x*z*u^3-14360*y*z*u^3-F(31)/50*z^2*u^3-11228*x*u^4+9935*y*u^4+10741*z*u^4-F(73)/110*u^5+70*x^3*y*v-F(83)/82*x^2*y^2*v-F(32)/99*x*y^3*v-F(21)/83*y^4*v-836*x^3*z*v-F(98)/95*x^2*y*z*v+F(87)/85*x*y^2*z*v-F(33)/41*y^3*z*v-F(74)/85*x^2*z^2*v+F(99)/100*x*y*z^2*v-10940*y^2*z^2*v+F(50)/83*x*z^3*v+7234*y*z^3*v+7411*x^3*u*v+F(51)/74*x^2*y*u*v-F(89)/115*x*y^2*u*v-230*y^3*u*v-F(32)/35*x^2*z*u*v-2719*x*y*z*u*v-10121*y^2*z*u*v+F(48)/77*x*z^2*u*v+9070*y*z^2*u*v+93*z^3*u*v-F(57)/32*x^2*u^2*v+15800*x*y*u^2*v+F(101)/76*y^2*u^2*v-10158*x*z*u^2*v-F(123)/79*y*z*u^2*v+F(109)/19*z^2*u^2*v-4482*x*u^3*v+12766*y*u^3*v+5075*z*u^3*v-F(3)/43*u^4*v+F(44)/25*x^2*y*v^2+5150*x*y^2*v^2+2804*y^3*v^2+6248*x^2*z*v^2+F(108)/89*x*y*z*v^2+F(29)/122*y^2*z*v^2+F(103)/93*x*z^2*v^2+F(103)/121*y*z^2*v^2+6305*z^3*v^2+F(84)/65*x^2*u*v^2-10620*x*y*u*v^2+F(79)/97*y^2*u*v^2-15549*x*z*u*v^2-8747*y*z*u*v^2+F(22)/113*z^2*u*v^2-2274*x*u^2*v^2-F(80)/123*y*u^2*v^2+F(109)/2*z*u^2*v^2-F(25)/54*u^3*v^2+F(93)/19*x*y*v^3+F(121)/35*y^2*v^3-F(17)/93*x*z*v^3+F(121)/58*y*z*v^3-F(107)/48*z^2*v^3+F(117)/50*x*u*v^3-6611*y*u*v^3-11065*z*u*v^3+4432*u^2*v^3+F(59)/8*y*v^4-F(17)/114*z*v^4+F(66)/59*u*v^4,-F(26)/5*x^4*u-F(111)/113*x^3*y*u+11508*x^2*y^2*u-F(52)/95*x*y^3*u-F(76)/7*y^4*u+F(113)/117*x^3*z*u-7653*x^2*y*z*u+F(11)/78*x*y^2*z*u-F(46)/27*y^3*z*u-F(79)/118*x^2*z^2*u-1483*x*y*z^2*u-8862*y^2*z^2*u-F(95)/41*x*z^3*u-5361*y*z^3*u-F(64)/125*x^3*u^2+F(58)/89*x^2*y*u^2+F(25)/69*x*y^2*u^2-5211*y^3*u^2+F(59)/37*x^2*z*u^2-F(83)/54*x*y*z*u^2-F(109)/62*y^2*z*u^2+F(88)/23*x*z^2*u^2+F(115)/123*y*z^2*u^2+F(65)/113*z^3*u^2-F(1)/25*x^2*u^3-F(71)/15*x*y*u^3-F(22)/31*y^2*u^3-F(109)/5*x*z*u^3-13430*y*z*u^3-4602*z^2*u^3+6903*x*u^4+3055*y*u^4-8412*z*u^4+F(79)/116*u^5+836*x^4*v+F(92)/51*x^3*y*v-4669*x^2*y^2*v+F(91)/90*x*y^3*v-2619*y^4*v-F(21)/89*x^3*z*v-F(35)/46*x^2*y*z*v-F(67)/35*x*y^2*z*v-F(58)/25*y^3*z*v-7434*x^2*z^2*v-3390*x*y*z^2*v+F(122)/31*y^2*z^2*v+F(12)/5*x*z^3*v+F(40)/33*y*z^3*v+9281*x^3*u*v-F(5)/13*x^2*y*u*v-2449*x*y^2*u*v+3481*y^3*u*v+F(116)/83*x^2*z*u*v+F(66)/107*x*y*z*u*v+F(51)/110*y^2*z*u*v+3267*x*z^2*u*v+F(33)/107*y*z^2*u*v+6013*z^3*u*v-3742*x^2*u^2*v-F(119)/76*x*y*u^2*v-10882*y^2*u^2*v-F(3)/74*x*z*u^2*v+1517*y*z*u^2*v+F(55)/98*z^2*u^2*v-4237*x*u^3*v-15066*y*u^3*v+8364*z*u^3*v+F(4)/37*u^4*v-6248*x^3*v^2-F(41)/83*x^2*y*v^2-F(107)/62*x*y^2*v^2-F(73)/112*y^3*v^2+8695*x^2*z*v^2+F(53)/83*x*y*z*v^2+F(14)/125*y^2*z*v^2+14595*x*z^2*v^2+13051*y*z^2*v^2+F(49)/103*z^3*v^2-F(116)/53*x^2*u*v^2+4604*x*y*u*v^2+F(103)/68*y^2*u*v^2-F(47)/125*x*z*u*v^2-F(21)/59*y*z*u*v^2-F(104)/11*z^2*u*v^2-F(95)/46*x*u^2*v^2+F(73)/39*y*u^2*v^2-F(71)/31*z*u^2*v^2+F(13)/55*u^3*v^2+F(17)/93*x^2*v^3+F(105)/76*x*y*v^3+F(84)/109*y^2*v^3-F(7)/100*x*z*v^3+F(79)/22*y*z*v^3+F(11)/98*z^2*v^3+F(94)/81*x*u*v^3+F(20)/69*y*u*v^3+F(2)/115*z*u*v^3+F(56)/39*u^2*v^3+F(17)/114*x*v^4+F(34)/115*y*v^4-F(79)/87*z*v^4-F(86)/55*u*v^4,11801*x^4*u-178*x^3*y*u+F(19)/17*x^2*y^2*u-F(23)/67*x*y^3*u-5309*x^3*z*u+F(20)/91*x^2*y*z*u-F(69)/26*x*y^2*z*u-4769*x^3*u^2+F(27)/22*x^2*y*u^2+F(13)/102*x*y^2*u^2+466*y^3*u^2-F(59)/61*x^2*z*u^2+F(91)/80*x*y*z*u^2-9322*y^2*z*u^2+F(101)/125*x*z^2*u^2+F(46)/15*y*z^2*u^2+7174*x^2*u^3+F(124)/77*x*y*u^3-2069*y^2*u^3+F(101)/28*x*z*u^3+F(55)/111*y*z*u^3+F(100)/9*z^2*u^3-893*x*u^4-F(33)/41*y*u^4-F(79)/12*z*u^4+F(59)/90*u^5-11606*x^4*v+6723*x^3*y*v-5985*x^2*y^2*v+F(15)/17*x*y^3*v+9554*x^3*z*v+6076*x^2*y*z*v+F(61)/91*x*y^2*z*v+F(7)/60*x^3*u*v+F(46)/77*x^2*y*u*v+100*x*y^2*u*v+F(117)/80*y^3*u*v+2011*x^2*z*u*v+F(113)/37*x*y*z*u*v+15886*y^2*z*u*v+95*x*z^2*u*v-F(2)/107*y*z^2*u*v-F(67)/19*x^2*u^2*v+F(67)/19*x*y*u^2*v-11865*y^2*u^2*v-F(9)/118*x*z*u^2*v+15795*y*z*u^2*v-3968*z^2*u^2*v+F(61)/10*x*u^3*v-F(115)/67*y*u^3*v-12020*z*u^3*v-13312*u^4*v-F(47)/82*x^3*v^2-F(91)/93*x^2*y*v^2+7737*x*y^2*v^2-2619*y^3*v^2-12026*x^2*z*v^2+12356*x*y*z*v^2-430*y^2*z*v^2+12610*x*z^2*v^2-F(10)/59*y*z^2*v^2-F(61)/14*x^2*u*v^2+9447*x*y*u*v^2-F(13)/44*y^2*u*v^2-F(76)/81*x*z*u*v^2+15211*y*z*u*v^2-F(50)/61*z^2*u*v^2-1341*x*u^2*v^2-13318*y*u^2*v^2-14083*z*u^2*v^2-11557*u^3*v^2-F(59)/9*x^2*v^3-F(15)/43*x*y*v^3-F(73)/112*y^2*v^3+F(88)/115*x*z*v^3+F(100)/49*y*z*v^3+F(47)/101*z^2*v^3-F(81)/29*x*u*v^3-4438*y*u*v^3-F(113)/57*z*u*v^3+F(19)/120*u^2*v^3-F(117)/43*x*v^4+F(84)/109*y*v^4+F(101)/98*z*v^4-2849*u*v^4+F(34)/115*v^5,-11801*x^3*y*u+178*x^2*y^2*u-F(19)/17*x*y^3*u+F(23)/67*y^4*u+5309*x^2*y*z*u-F(20)/91*x*y^2*z*u+F(69)/26*y^3*z*u-F(50)/13*x^3*u^2-F(91)/82*x^2*y*u^2-F(88)/25*x*y^2*u^2-F(3)/62*y^3*u^2-F(61)/124*x^2*z*u^2-F(80)/67*x*y*z*u^2-5290*y^2*z*u^2-F(53)/69*x*z^2*u^2+4341*y*z^2*u^2+10169*x^2*u^3-F(91)/111*x*y*u^3+12130*y^2*u^3-14528*x*z*u^3+6058*y*z*u^3+F(1)/80*z^2*u^3+F(119)/101*x*u^4-F(105)/47*y*u^4-F(21)/37*z*u^4-7065*u^5+11606*x^3*y*v-6723*x^2*y^2*v+5985*x*y^3*v-F(15)/17*y^4*v-9554*x^2*y*z*v-6076*x*y^2*z*v-F(61)/91*y^3*z*v+9575*x^3*u*v+F(99)/7*x^2*y*u*v+F(25)/28*x*y^2*u*v-6304*y^3*u*v-26*x^2*z*u*v-9500*x*y*z*u*v-F(115)/12*y^2*z*u*v-F(81)/38*x*z^2*u*v+14388*y*z^2*u*v+15877*x^2*u^2*v+F(93)/10*x*y*u^2*v-95*y^2*u^2*v+F(58)/77*x*z*u^2*v-F(7)/93*y*z*u^2*v-F(105)/76*z^2*u^2*v-4053*x*u^3*v-6948*y*u^3*v+F(25)/66*z*u^3*v+F(79)/27*u^4*v+836*x^3*v^2+F(102)/59*x^2*y*v^2-5356*x*y^2*v^2-11646*y^3*v^2+F(74)/85*x^2*z*v^2+F(64)/51*x*y*z*v^2+F(31)/96*y^2*z*v^2-F(50)/83*x*z^2*v^2-F(49)/64*y*z^2*v^2-6145*x^2*u*v^2-F(66)/95*x*y*u*v^2-F(100)/89*y^2*u*v^2-F(57)/14*x*z*u*v^2-F(35)/66*y*z*u*v^2+F(64)/111*z^2*u*v^2+F(99)/41*x*u^2*v^2-F(73)/35*y*u^2*v^2-F(4)/59*z*u^2*v^2+1767*u^3*v^2-6248*x^2*v^3+F(17)/24*x*y*v^3+F(72)/61*y^2*v^3-F(103)/93*x*z*v^3+F(12)/7*y*z*v^3-6305*z^2*v^3-14128*x*u*v^3+F(62)/69*y*u*v^3+F(105)/44*z*u*v^3+F(51)/62*u^2*v^3+F(17)/93*x*v^4-84*y*v^4+F(107)/48*z*v^4+F(61)/112*u*v^4+F(17)/114*v^5,-11801*x^3*z*u+178*x^2*y*z*u-F(19)/17*x*y^2*z*u+F(23)/67*y^3*z*u+5309*x^2*z^2*u-F(20)/91*x*y*z^2*u+F(69)/26*y^2*z^2*u-F(13)/98*x^3*u^2+F(118)/107*x^2*y*u^2-F(97)/84*x*y^2*u^2+10426*y^3*u^2-F(51)/97*x^2*z*u^2-7945*x*y*z*u^2-F(80)/113*y^2*z*u^2-12278*x*z^2*u^2-6177*y*z^2*u^2-10441*z^3*u^2+F(18)/125*x^2*u^3-F(117)/17*x*y*u^3-10095*y^2*u^3+F(103)/104*x*z*u^3-F(107)/90*y*z*u^3+F(67)/34*z^2*u^3+9956*x*u^4-F(9)/113*y*u^4+F(13)/73*z*u^4+F(49)/95*u^5+11606*x^3*z*v-6723*x^2*y*z*v+5985*x*y^2*z*v-F(15)/17*y^3*z*v-9554*x^2*z^2*v-6076*x*y*z^2*v-F(61)/91*y^2*z^2*v+F(19)/43*x^3*u*v+F(60)/19*x^2*y*u*v-F(61)/78*x*y^2*u*v+F(118)/55*y^3*u*v+F(91)/23*x^2*z*u*v-F(16)/47*x*y*z*u*v+F(83)/43*y^2*z*u*v-11620*x*z^2*u*v-13818*y*z^2*u*v-F(120)/119*z^3*u*v+F(88)/35*x^2*u^2*v-F(17)/55*x*y*u^2*v-5255*y^2*u^2*v-2015*x*z*u^2*v-F(26)/31*y*z*u^2*v-F(95)/84*z^2*u^2*v-F(124)/125*x*u^3*v-1461*y*u^3*v+F(36)/121*z*u^3*v-4502*u^4*v+70*x^3*v^2-F(83)/82*x^2*y*v^2-F(32)/99*x*y^2*v^2-F(21)/83*y^3*v^2+F(119)/97*x^2*z*v^2+F(31)/61*x*y*z*v^2+F(26)/55*y^2*z*v^2+F(41)/49*x*z^2*v^2-8607*y*z^2*v^2-13261*z^3*v^2-F(1)/111*x^2*u*v^2+F(22)/35*x*y*u*v^2+F(36)/125*y^2*u*v^2-1923*x*z*u*v^2-F(24)/59*y*z*u*v^2-4891*z^2*u*v^2-F(100)/51*x*u^2*v^2-F(7)/72*y*u^2*v^2+F(107)/101*z*u^2*v^2+F(11)/48*u^3*v^2+F(44)/25*x^2*v^3+5150*x*y*v^3+2804*y^2*v^3+F(100)/91*x*z*v^3-F(119)/101*y*z*v^3+F(1)/74*z^2*v^3+4910*x*u*v^3+F(73)/97*y*u*v^3+F(43)/23*z*u*v^3-3345*u^2*v^3+F(93)/19*x*v^4+F(121)/35*y*v^4-12768*z*v^4-F(59)/117*u*v^4+F(59)/8*v^5,-F(13)/98*x^4*u+F(118)/107*x^3*y*u-F(97)/84*x^2*y^2*u+10426*x*y^3*u-10706*x^3*z*u+F(13)/108*x^2*y*z*u+10355*x*y^2*z*u+466*y^3*z*u-2839*x^2*z^2*u+F(8)/45*x*y*z^2*u-9322*y^2*z^2*u-459*x*z^3*u+F(46)/15*y*z^3*u+F(18)/125*x^3*u^2-F(117)/17*x^2*y*u^2-10095*x*y^2*u^2-F(4)/43*x^2*z*u^2-2497*x*y*z*u^2-2069*y^2*z*u^2+6928*x*z^2*u^2+F(55)/111*y*z^2*u^2+F(100)/9*z^3*u^2+9956*x^2*u^3-F(9)/113*x*y*u^3+10063*x*z*u^3-F(33)/41*y*z*u^3-F(79)/12*z^2*u^3+F(49)/95*x*u^4+F(59)/90*z*u^4+24*x^4*v-961*x^3*y*v-F(42)/17*x^2*y^2*v+6186*x*y^3*v+F(95)/22*y^4*v+F(43)/28*x^3*z*v+15246*x^2*y*z*v-F(71)/98*x*y^2*z*v-F(94)/31*y^3*z*v-F(21)/32*x^2*z^2*v+F(52)/87*x*y*z^2*v-F(53)/24*y^2*z^2*v-15217*x*z^3*v+9118*y*z^3*v-13528*x^3*u*v+3285*x^2*y*u*v-4122*x*y^2*u*v-12672*y^3*u*v-F(49)/4*x^2*z*u*v-F(107)/37*x*y*z*u*v-F(17)/10*y^2*z*u*v+F(1)/3*x*z^2*u*v-F(85)/58*y*z^2*u*v-12832*z^3*u*v+F(117)/64*x^2*u^2*v-F(113)/28*x*y*u^2*v+F(31)/26*y^2*u^2*v+F(100)/101*x*z*u^2*v-699*y*z*u^2*v+F(11)/111*z^2*u^2*v+11804*x*u^3*v-5070*y*u^3*v-8548*z*u^3*v-9231*u^4*v+5595*x^3*v^2+F(7)/59*x^2*y*v^2+3741*x*y^2*v^2-6439*y^3*v^2+5956*x^2*z*v^2+F(105)/121*x*y*z*v^2-9320*y^2*z*v^2+F(81)/110*x*z^2*v^2-3219*y*z^2*v^2+F(112)/61*z^3*v^2-F(85)/58*x^2*u*v^2+14755*x*y*u*v^2-F(47)/11*y^2*u*v^2-9161*x*z*u*v^2+F(3)/41*y*z*u*v^2-F(31)/60*z^2*u*v^2+F(91)/71*x*u^2*v^2-10153*y*u^2*v^2-F(26)/29*z*u^2*v^2-6205*u^3*v^2+F(51)/83*x^2*v^3+F(92)/29*x*y*v^3+F(32)/73*y^2*v^3-F(64)/113*x*z*v^3-13120*y*z*v^3-7328*z^2*v^3+5419*x*u*v^3+F(34)/63*y*u*v^3-4806*z*u*v^3-F(2)/39*u^2*v^3-F(47)/112*x*v^4+10275*y*v^4+F(31)/5*z*v^4-F(79)/57*u*v^4+F(102)/65*v^5,F(13)/98*x^3*y*u-F(118)/107*x^2*y^2*u+F(97)/84*x*y^3*u-10426*y^4*u-F(50)/13*x^3*z*u+F(115)/113*x^2*y*z*u-F(26)/63*x*y^2*z*u+7229*y^3*z*u-F(61)/124*x^2*z^2*u+F(23)/38*x*y*z^2*u-F(59)/36*y^2*z^2*u-F(53)/69*x*z^3*u+14782*y*z^3*u-F(18)/125*x^2*y*u^2+F(117)/17*x*y^2*u^2+10095*y^3*u^2+10169*x^2*z*u^2+2944*x*y*z*u^2+9643*y^2*z*u^2-14528*x*z^2*u^2-4294*y*z^2*u^2+F(1)/80*z^3*u^2-9956*x*y*u^3+F(9)/113*y^2*u^3+F(119)/101*x*z*u^3-F(93)/112*y*z*u^3-F(21)/37*z^2*u^3-F(49)/95*y*u^4-7065*z*u^4+x^4*v-12343*x^3*y*v+F(85)/117*x^2*y^2*v+F(7)/110*x*y^3*v-F(71)/88*y^4*v-7706*x^3*z*v+15113*x^2*y*z*v+2197*x*y^2*z*v-12236*y^3*z*v+14334*x^2*z^2*v-F(43)/20*x*y*z^2*v-306*y^2*z^2*v-F(56)/121*x*z^3*v-F(64)/91*y*z^3*v-7798*x^3*u*v+F(22)/105*x^2*y*u*v+F(2)/33*x*y^2*u*v+F(83)/110*y^3*u*v+10963*x^2*z*u*v-F(104)/113*x*y*z*u*v+F(15)/4*y^2*z*u*v-F(97)/50*x*z^2*u*v+F(31)/117*y*z^2*u*v-F(26)/15*z^3*u*v-F(76)/87*x^2*u^2*v+3015*x*y*u^2*v-F(68)/125*y^2*u^2*v+5086*x*z*u^2*v-686*y*z*u^2*v+15084*z^2*u^2*v-11228*x*u^3*v+14437*y*u^3*v+F(53)/104*z*u^3*v-F(73)/110*u^4*v+7411*x^3*v^2-F(68)/109*x^2*y*v^2-F(79)/113*x*y^2*v^2+F(28)/51*y^3*v^2-F(29)/18*x^2*z*v^2+F(115)/51*x*y*z*v^2-F(23)/55*y^2*z*v^2-8936*x*z^2*v^2+F(119)/29*y*z^2*v^2+9028*z^3*v^2-F(57)/32*x^2*u*v^2-F(108)/67*x*y*u*v^2+15880*y^2*u*v^2+14813*x*z*u*v^2-4251*y*z*u*v^2+F(71)/83*z^2*u*v^2-4482*x*u^2*v^2-F(89)/103*y*u^2*v^2+6842*z*u^2*v^2-F(3)/43*u^3*v^2+F(84)/65*x^2*v^3-F(40)/103*x*y*v^3+F(6)/97*y^2*v^3+F(116)/83*x*z*v^3+13043*y*z*v^3+11359*z^2*v^3-2274*x*u*v^3+F(56)/55*y*u*v^3-F(17)/37*z*u*v^3-F(25)/54*u^2*v^3+F(117)/50*x*v^4+10342*y*v^4-496*z*v^4+4432*u*v^4+F(66)/59*v^5,F(50)/13*x^4*u-15907*x^3*y*u-F(63)/74*x^2*y^2*u+14407*x*y^3*u-466*y^4*u-F(7)/44*x^3*z*u-3723*x^2*y*z*u+15701*x*y^2*z*u+F(71)/106*y^3*z*u+F(87)/46*x^2*z^2*u+15887*x*y*z^2*u-F(121)/102*y^2*z^2*u+F(13)/69*x*z^3*u-7851*y*z^3*u-10169*x^3*u^2+32*x^2*y*u^2+F(87)/94*x*y^2*u^2+2069*y^3*u^2+F(23)/47*x^2*z*u^2-F(77)/30*x*y*z*u^2-F(104)/23*y^2*z*u^2-F(11)/91*x*z^2*u^2+F(4)/31*y*z^2*u^2+13481*z^3*u^2-F(119)/101*x^2*u^3-F(95)/63*x*y*u^3+F(33)/41*y^2*u^3+9422*x*z*u^3+6868*y*z*u^3-2625*z^2*u^3+7065*x*u^4-F(59)/90*y*u^4+15341*z*u^4-3182*x^4*v-F(7)/66*x^3*y*v-9163*x^2*y^2*v+142*x*y^3*v-9781*y^4*v-F(68)/105*x^3*z*v+F(41)/31*x^2*y*z*v+F(97)/54*x*y^2*z*v+F(97)/55*y^3*z*v-3230*x^2*z^2*v+F(73)/79*x*y*z^2*v-4311*y^2*z^2*v-F(106)/33*x*z^3*v+8968*y*z^3*v+10739*x^3*u*v+F(111)/98*x^2*y*u*v+F(36)/91*x*y^2*u*v-F(16)/125*y^3*u*v-14681*x^2*z*u*v+15128*x*y*z*u*v+F(53)/86*y^2*z*u*v+5879*x*z^2*u*v-9051*y*z^2*u*v-F(113)/15*z^3*u*v+F(23)/30*x^2*u^2*v-F(111)/73*x*y*u^2*v-7423*y^2*u^2*v+F(4)/85*x*z*u^2*v-F(96)/113*y*z*u^2*v-F(67)/91*z^2*u^2*v+F(74)/79*x*u^3*v-F(21)/43*y*u^3*v+F(15)/122*z*u^3*v+F(79)/116*u^4*v+F(99)/56*x^3*v^2-F(103)/37*x^2*y*v^2+F(29)/10*x*y^2*v^2-10333*y^3*v^2+F(68)/103*x^2*z*v^2-1425*x*y*z*v^2+F(80)/103*y^2*z*v^2-F(43)/63*x*z^2*v^2+15173*y*z^2*v^2+13313*z^3*v^2-F(122)/39*x^2*u*v^2+F(61)/79*x*y*u*v^2+2436*y^2*u*v^2+6764*x*z*u*v^2+12388*y*z*u*v^2-12268*z^2*u*v^2-F(91)/16*x*u^2*v^2-3509*y*u^2*v^2+F(57)/64*z*u^2*v^2+F(4)/37*u^3*v^2+12315*x^2*v^3+F(103)/69*x*y*v^3+15260*y^2*v^3+F(53)/11*x*z*v^3+8032*y*z*v^3+157*z^2*v^3-F(10)/19*x*u*v^3+F(43)/61*y*u*v^3+F(101)/85*z*u*v^3+F(13)/55*u^2*v^3+F(43)/87*x*v^4-3178*y*v^4-6362*z*v^4+F(56)/39*u*v^4-F(86)/55*v^5]))
end


@doc raw"""
    rational_d7_pi4()

Return a smooth rational surface in $\mathbb P^4$ with degree `7` and sectional genus `4`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d7_pi4()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(94)/121*x^3*y-F(100)/7*x^2*y^2+9354*x*y^3-F(65)/123*x^3*z-7187*x^2*y*z+F(3)/88*x*y^2*z+14966*x^2*z^2+F(114)/73*x*y*z^2-F(31)/13*x*z^3+F(101)/31*x^3*u-F(120)/83*x^2*y*u-1578*x*y^2*u-F(47)/41*x^2*z*u+F(69)/41*x*y*z*u+F(33)/50*x*z^2*u+F(117)/100*x^2*u^2+F(68)/107*x*y*u^2-9474*x*z*u^2+F(113)/62*x*u^3-F(67)/28*x^3*v-11111*x^2*y*v-7540*x*y^2*v+F(61)/60*y^3*v-F(44)/35*x^2*z*v+F(39)/7*x*y*z*v-F(61)/100*y^2*z*v-7557*x*z^2*v+F(115)/56*y*z^2*v+6837*z^3*v-12594*x^2*u*v+F(75)/91*x*y*u*v-F(95)/28*y^2*u*v+2874*x*z*u*v-F(75)/11*y*z*u*v-11535*z^2*u*v+F(30)/49*x*u^2*v-F(118)/11*y*u^2*v+F(102)/103*z*u^2*v-F(89)/110*u^3*v+F(53)/77*x^2*v^2-F(67)/75*x*y*v^2+F(2)/75*y^2*v^2-2886*x*z*v^2-F(25)/48*y*z*v^2-F(65)/89*z^2*v^2+F(84)/125*x*u*v^2-4598*y*u*v^2+F(55)/24*z*u*v^2-F(1)/14*u^2*v^2-F(56)/13*x*v^3+5138*y*v^3-F(64)/63*z*v^3-F(50)/71*u*v^3-F(7)/101*v^4,F(94)/121*x^2*y^2+F(100)/7*x*y^3-9354*y^4+F(65)/123*x^2*y*z+7187*x*y^2*z-F(3)/88*y^3*z-14966*x*y*z^2-F(114)/73*y^2*z^2+F(31)/13*y*z^3-F(101)/31*x^2*y*u+F(120)/83*x*y^2*u+1578*y^3*u+F(47)/41*x*y*z*u-F(69)/41*y^2*z*u-F(33)/50*y*z^2*u-F(117)/100*x*y*u^2-F(68)/107*y^2*u^2+9474*y*z*u^2-F(113)/62*y*u^3+F(39)/28*x^2*y*v-F(105)/53*x*y^2*v-6197*y^3*v-F(125)/92*x^2*z*v-F(34)/41*x*y*z*v-7082*y^2*z*v+468*x*z^2*v-F(54)/55*y*z^2*v-F(26)/67*z^3*v+F(53)/11*x^2*u*v-2300*x*y*u*v-5532*y^2*u*v-F(67)/5*x*z*u*v+3699*y*z*u*v+F(15)/23*z^2*u*v-F(65)/42*x*u^2*v-F(43)/104*y*u^2*v+F(59)/91*z*u^2*v-28*u^3*v+4633*x^2*v^2-11810*x*y*v^2+F(9)/20*y^2*v^2+F(109)/113*x*z*v^2-4427*y*z*v^2+F(9)/32*z^2*v^2-F(5)/48*x*u*v^2-15485*y*u*v^2-3487*z*u*v^2+249*u^2*v^2+4641*x*v^3+11675*y*v^3-306*z*v^3+3793*u*v^3+F(105)/94*v^4,F(94)/121*x^2*y*z+F(100)/7*x*y^2*z-9354*y^3*z+F(65)/123*x^2*z^2+7187*x*y*z^2-F(3)/88*y^2*z^2-14966*x*z^3-F(114)/73*y*z^3+F(31)/13*z^4-F(101)/31*x^2*z*u+F(120)/83*x*y*z*u+1578*y^2*z*u+F(47)/41*x*z^2*u-F(69)/41*y*z^2*u-F(33)/50*z^3*u-F(117)/100*x*z*u^2-F(68)/107*y*z*u^2+9474*z^2*u^2-F(113)/62*z*u^3+F(85)/43*x^2*y*v-1260*x*y^2*v-F(100)/89*y^3*v+96*x^2*z*v+F(71)/79*x*y*z*v+5609*y^2*z*v-F(103)/12*x*z^2*v-F(45)/113*y*z^2*v-3303*z^3*v-F(75)/11*x^2*u*v+F(43)/61*x*y*u*v+9485*y^2*u*v-4640*x*z*u*v-F(114)/17*y*z*u*v-F(50)/103*z^2*u*v+10602*x*u^2*v+10333*y*u^2*v-F(61)/119*z*u^2*v-F(106)/83*u^3*v-F(61)/55*x^2*v^2-F(19)/15*x*y*v^2+F(56)/125*y^2*v^2+F(97)/113*x*z*v^2+F(96)/23*y*z*v^2-F(7)/6*z^2*v^2-14469*x*u*v^2+F(9)/71*y*u*v^2+15127*z*u*v^2-F(19)/72*u^2*v^2+3471*x*v^3-F(61)/11*y*v^3+F(29)/86*z*v^3+F(45)/61*u*v^3-F(38)/113*v^4,F(94)/121*x^2*y*u+F(100)/7*x*y^2*u-9354*y^3*u+F(65)/123*x^2*z*u+7187*x*y*z*u-F(3)/88*y^2*z*u-14966*x*z^2*u-F(114)/73*y*z^2*u+F(31)/13*z^3*u-F(101)/31*x^2*u^2+F(120)/83*x*y*u^2+1578*y^2*u^2+F(47)/41*x*z*u^2-F(69)/41*y*z*u^2-F(33)/50*z^2*u^2-F(117)/100*x*u^3-F(68)/107*y*u^3+9474*z*u^3-F(113)/62*u^4+10584*x^2*y*v+F(119)/118*x*y^2*v+F(94)/27*y^3*v-F(9)/121*x^2*z*v+2236*x*y*z*v-2273*y^2*z*v-10463*x*z^2*v+3215*y*z^2*v+6550*z^3*v+F(79)/18*x^2*u*v-F(65)/2*x*y*u*v-F(45)/47*y^2*u*v+F(83)/61*x*z*u*v+F(95)/46*y*z*u*v-8900*z^2*u*v+F(82)/59*x*u^2*v-15264*y*u^2*v-F(113)/125*z*u^2*v-F(25)/44*u^3*v-3233*x^2*v^2+F(83)/100*x*y*v^2+F(73)/108*y^2*v^2+F(43)/77*x*z*v^2+11889*y*z*v^2-F(60)/53*z^2*v^2+9581*x*u*v^2-1374*y*u*v^2+11457*z*u*v^2-F(94)/5*u^2*v^2+10722*x*v^3-F(91)/8*y*v^3-F(109)/107*z*v^3+F(121)/87*u*v^3+6905*v^4,10584*x^3*y+F(119)/118*x^2*y^2+F(94)/27*x*y^3-F(9)/121*x^3*z+2236*x^2*y*z-2273*x*y^2*z-10463*x^2*z^2+3215*x*y*z^2+6550*x*z^3-F(101)/13*x^3*u+4852*x^2*y*u+F(55)/37*x*y^2*u+F(61)/60*y^3*u-15898*x^2*z*u-9232*x*y*z*u-F(61)/100*y^2*z*u+15534*x*z^2*u+F(115)/56*y*z^2*u+6837*z^3*u+3674*x^2*u^2+205*x*y*u^2-F(95)/28*y^2*u^2+13878*x*z*u^2-F(75)/11*y*z*u^2-11535*z^2*u^2-F(59)/8*x*u^3-F(118)/11*y*u^3+F(102)/103*z*u^3-F(89)/110*u^4-3233*x^3*v+F(83)/100*x^2*y*v+F(73)/108*x*y^2*v+F(43)/77*x^2*z*v+11889*x*y*z*v-F(60)/53*x*z^2*v-805*x^2*u*v-F(112)/37*x*y*u*v+F(2)/75*y^2*u*v+F(111)/56*x*z*u*v-F(25)/48*y*z*u*v-F(65)/89*z^2*u*v+6636*x*u^2*v-4598*y*u^2*v+F(55)/24*z*u^2*v-F(1)/14*u^3*v+10722*x^2*v^2-F(91)/8*x*y*v^2-F(109)/107*x*z*v^2-13382*x*u*v^2+5138*y*u*v^2-F(64)/63*z*u*v^2-F(50)/71*u^2*v^2+6905*x*v^3-F(7)/101*u*v^3,-10584*x^2*y^2-F(119)/118*x*y^3-F(94)/27*y^4+F(9)/121*x^2*y*z-2236*x*y^2*z+2273*y^3*z+10463*x*y*z^2-3215*y^2*z^2-6550*y*z^3+F(88)/13*x^2*y*u+F(61)/91*x*y^2*u+F(79)/28*y^3*u-F(125)/92*x^2*z*u+6176*x*y*z*u+F(67)/123*y^2*z*u+468*x*z^2*u-6224*y*z^2*u-F(26)/67*z^3*u+F(53)/11*x^2*u^2+13423*x*y*u^2-F(101)/23*y^2*u^2-F(67)/5*x*z*u^2-7305*y*z*u^2+F(15)/23*z^2*u^2-F(65)/42*x*u^3-F(55)/6*y*u^3+F(59)/91*z*u^3-28*u^4+3233*x^2*y*v-F(83)/100*x*y^2*v-F(73)/108*y^3*v-F(43)/77*x*y*z*v-11889*y^2*z*v+F(60)/53*y*z^2*v+4633*x^2*u*v+10600*x*y*u*v-F(82)/43*y^2*u*v+F(109)/113*x*z*u*v-15884*y*z*u*v+F(9)/32*z^2*u*v-F(5)/48*x*u^2*v-9068*y*u^2*v-3487*z*u^2*v+249*u^3*v-10722*x*y*v^2+F(91)/8*y^2*v^2+F(109)/107*y*z*v^2+4641*x*u*v^2-11860*y*u*v^2-306*z*u*v^2+3793*u^2*v^2-6905*y*v^3+F(105)/94*u*v^3,-10584*x^2*y*z-F(119)/118*x*y^2*z-F(94)/27*y^3*z+F(9)/121*x^2*z^2-2236*x*y*z^2+2273*y^2*z^2+10463*x*z^3-3215*y*z^3-6550*z^4+F(85)/43*x^2*y*u-1260*x*y^2*u-F(100)/89*y^3*u-F(110)/57*x^2*z*u-F(2)/103*x*y*z*u+F(55)/21*y^2*z*u-F(1)/88*x*z^2*u+16*y*z^2*u-F(57)/40*z^3*u-F(75)/11*x^2*u^2+F(43)/61*x*y*u^2+9485*y^2*u^2+11083*x*z*u^2+7730*y*z*u^2-14421*z^2*u^2+10602*x*u^3+10333*y*u^3+F(19)/122*z*u^3-F(106)/83*u^4+3233*x^2*z*v-F(83)/100*x*y*z*v-F(73)/108*y^2*z*v-F(43)/77*x*z^2*v-11889*y*z^2*v+F(60)/53*z^3*v-F(61)/55*x^2*u*v-F(19)/15*x*y*u*v+F(56)/125*y^2*u*v+1461*x*z*u*v+F(117)/100*y*z*u*v+15201*z^2*u*v-14469*x*u^2*v+F(9)/71*y*u^2*v-F(47)/49*z*u^2*v-F(19)/72*u^3*v-10722*x*z*v^2+F(91)/8*y*z*v^2+F(109)/107*z^2*v^2+3471*x*u*v^2-F(61)/11*y*u*v^2-12747*z*u*v^2+F(45)/61*u^2*v^2-6905*z*v^3-F(38)/113*u*v^3,F(85)/43*x^3*y-1260*x^2*y^2-F(100)/89*x*y^3-F(77)/48*x^3*z-F(91)/51*x^2*y*z-F(59)/116*x*y^2*z+F(61)/60*y^3*z+F(51)/49*x^2*z^2+F(103)/59*x*y*z^2-F(61)/100*y^2*z^2-F(73)/109*x*z^3+F(115)/56*y*z^3+6837*z^4-F(75)/11*x^3*u+F(43)/61*x^2*y*u+9485*x*y^2*u-F(105)/13*x^2*z*u+7935*x*y*z*u-F(95)/28*y^2*z*u-F(46)/59*x*z^2*u-F(75)/11*y*z^2*u-11535*z^3*u+10602*x^2*u^2+10333*x*y*u^2+F(119)/38*x*z*u^2-F(118)/11*y*z*u^2+F(102)/103*z^2*u^2-F(106)/83*x*u^3-F(89)/110*z*u^3-F(61)/55*x^3*v-F(19)/15*x^2*y*v+F(56)/125*x*y^2*v+656*x^2*z*v-F(125)/58*x*y*z*v+F(2)/75*y^2*z*v-F(123)/109*x*z^2*v-F(25)/48*y*z^2*v-F(65)/89*z^3*v-14469*x^2*u*v+F(9)/71*x*y*u*v-F(107)/42*x*z*u*v-4598*y*z*u*v+F(55)/24*z^2*u*v-F(19)/72*x*u^2*v-F(1)/14*z*u^2*v+3471*x^2*v^2-F(61)/11*x*y*v^2+5862*x*z*v^2+5138*y*z*v^2-F(64)/63*z^2*v^2+F(45)/61*x*u*v^2-F(50)/71*z*u*v^2-F(38)/113*x*v^3-F(7)/101*z*v^3,-F(85)/43*x^2*y^2+1260*x*y^3+F(100)/89*y^4+F(29)/48*x^2*y*z-2211*x*y^2*z+F(17)/84*y^3*z-F(125)/92*x^2*z^2+723*x*y*z^2+15850*y^2*z^2+468*x*z^3+F(81)/46*y*z^3-F(26)/67*z^4+F(75)/11*x^2*y*u-F(43)/61*x*y^2*u-9485*y^3*u+F(53)/11*x^2*z*u-F(33)/41*x*y*z*u+F(41)/16*y^2*z*u-F(67)/5*x*z^2*u+F(62)/9*y*z^2*u+F(15)/23*z^3*u-10602*x*y*u^2-10333*y^2*u^2-F(65)/42*x*z*u^2+F(49)/51*y*z*u^2+F(59)/91*z^2*u^2+F(106)/83*y*u^3-28*z*u^3+F(61)/55*x^2*y*v+F(19)/15*x*y^2*v-F(56)/125*y^3*v+4633*x^2*z*v-F(9)/7*x*y*z*v-F(31)/27*y^2*z*v+F(109)/113*x*z^2*v+F(63)/106*y*z^2*v+F(9)/32*z^3*v+14469*x*y*u*v-F(9)/71*y^2*u*v-F(5)/48*x*z*u*v+F(9)/116*y*z*u*v-3487*z^2*u*v+F(19)/72*y*u^2*v+249*z*u^2*v-3471*x*y*v^2+F(61)/11*y^2*v^2+4641*x*z*v^2-F(59)/36*y*z*v^2-306*z^2*v^2-F(45)/61*y*u*v^2+3793*z*u*v^2+F(38)/113*y*v^3+F(105)/94*z*v^3,x^3*y+14131*x^2*y^2+13737*x*y^3-F(61)/60*y^4+F(125)/92*x^3*z-F(77)/102*x^2*y*z-6634*x*y^2*z+F(61)/100*y^3*z-468*x^2*z^2-9310*x*y*z^2-F(115)/56*y^2*z^2+F(26)/67*x*z^3-6837*y*z^3-F(53)/11*x^3*u+F(95)/58*x^2*y*u-9937*x*y^2*u+F(95)/28*y^3*u+F(67)/5*x^2*z*u+F(36)/73*x*y*z*u+F(75)/11*y^2*z*u-F(15)/23*x*z^2*u+11535*y*z^2*u+F(65)/42*x^2*u^2+F(40)/83*x*y*u^2+F(118)/11*y^2*u^2-F(59)/91*x*z*u^2-F(102)/103*y*z*u^2+28*x*u^3+F(89)/110*y*u^3-4633*x^3*v-F(90)/49*x^2*y*v+3946*x*y^2*v-F(2)/75*y^3*v-F(109)/113*x^2*z*v+F(27)/35*x*y*z*v+F(25)/48*y^2*z*v-F(9)/32*x*z^2*v+F(65)/89*y*z^2*v+F(5)/48*x^2*u*v+2432*x*y*u*v+4598*y^2*u*v+3487*x*z*u*v-F(55)/24*y*z*u*v-249*x*u^2*v+F(1)/14*y*u^2*v-4641*x^2*v^2-6749*x*y*v^2-5138*y^2*v^2+306*x*z*v^2+F(64)/63*y*z*v^2-3793*x*u*v^2+F(50)/71*y*u*v^2-F(105)/94*x*v^3+F(7)/101*y*v^3,12525*x^3+F(122)/17*x^2*y+F(99)/71*x*y^2+F(50)/117*y^3+F(81)/94*x^2*z+14938*x*y*z-F(18)/53*y^2*z+F(39)/89*x*z^2-707*y*z^2+F(124)/33*z^3+10463*x^2*u+F(100)/59*x*y*u-F(2)/93*y^2*u+F(40)/113*x*z*u-F(30)/97*y*z*u-F(117)/119*z^2*u-F(55)/36*x*u^2+9225*y*u^2-F(104)/119*z*u^2+F(3)/100*u^3+F(17)/13*x^2*v-F(32)/63*x*y*v+F(65)/66*y^2*v-F(53)/85*x*z*v+F(22)/111*y*z*v-7002*z^2*v+20*x*u*v+12618*y*u*v+374*z*u*v+10958*u^2*v-4119*x*v^2+F(41)/104*y*v^2-F(74)/11*z*v^2+F(49)/12*u*v^2-F(4)/61*v^3]))
end


@doc raw"""
    rational_d8_pi5()

Return a smooth rational surface in $\mathbb P^4$ with degree `8` and sectional genus `5`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d8_pi5()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(21)/41*x^4*y-F(31)/69*x^3*y^2+691*x^2*y^3-F(102)/125*x*y^4-F(102)/5*x^4*z+F(48)/37*x^3*y*z-F(73)/15*x^2*y^2*z-F(17)/111*x*y^3*z-4656*x^3*z^2-F(70)/99*x^2*y*z^2-11722*x*y^2*z^2-F(86)/73*x^2*z^3+9989*x*y*z^3+F(107)/81*x*z^4-F(73)/23*x^4*u-4474*x^3*y*u-F(13)/35*x^2*y^2*u+F(119)/4*x*y^3*u+F(31)/95*x^3*z*u-2706*x^2*y*z*u+F(58)/41*x*y^2*z*u+4657*x^2*z^2*u+F(13)/60*x*y*z^2*u-F(117)/74*x*z^3*u-F(45)/109*x^3*u^2-14147*x^2*y*u^2-15420*x*y^2*u^2+F(37)/30*x^2*z*u^2-F(22)/3*x*y*z*u^2-F(9)/59*x*z^2*u^2+F(5)/96*x^2*u^3+F(57)/118*x*y*u^3-4939*x*z*u^3-F(109)/81*x*u^4-2761*x^4*v+2554*x^3*y*v+F(31)/42*x^2*y^2*v+F(41)/80*x*y^3*v-F(101)/47*y^4*v+10954*x^3*z*v+F(55)/51*x^2*y*z*v-F(83)/116*x*y^2*z*v+F(63)/55*y^3*z*v+F(100)/47*x^2*z^2*v+F(1)/117*x*y*z^2*v+11442*y^2*z^2*v-F(53)/114*x*z^3*v-12579*y*z^3*v-F(63)/82*z^4*v+F(83)/108*x^3*u*v+10714*x^2*y*u*v-F(86)/115*x*y^2*u*v-27*y^3*u*v-F(47)/24*x^2*z*u*v+F(77)/82*x*y*z*u*v+F(117)/110*y^2*z*u*v+F(9)/26*x*z^2*u*v+F(109)/88*y*z^2*u*v+F(112)/75*z^3*u*v-F(107)/43*x^2*u^2*v-F(101)/92*x*y*u^2*v-F(19)/101*y^2*u^2*v-2101*x*z*u^2*v-11736*y*z*u^2*v+F(81)/5*z^2*u^2*v+F(103)/74*x*u^3*v-F(94)/25*y*u^3*v-505*z*u^3*v+14815*u^4*v-3037*x^3*v^2+F(29)/73*x^2*y*v^2+F(39)/17*x*y^2*v^2-4074*y^3*v^2+8639*x^2*z*v^2-F(110)/3*x*y*z*v^2-11039*y^2*z*v^2-10532*x*z^2*v^2+5456*y*z^2*v^2-F(65)/48*z^3*v^2+11206*x^2*u*v^2+F(73)/24*x*y*u*v^2+8668*y^2*u*v^2-5137*x*z*u*v^2-12692*y*z*u*v^2+9162*z^2*u*v^2+F(19)/24*x*u^2*v^2-2028*y*u^2*v^2-13675*z*u^2*v^2+2329*u^3*v^2-F(97)/41*x^2*v^3+F(13)/108*x*y*v^3+4639*y^2*v^3-F(65)/107*x*z*v^3+10422*y*z*v^3-F(74)/25*z^2*v^3-F(107)/105*x*u*v^3+11188*y*u*v^3-F(18)/91*z*u*v^3+6424*u^2*v^3-F(43)/52*x*v^4+7017*y*v^4-F(49)/26*z*v^4+15637*u*v^4-F(103)/51*v^5,F(21)/41*x^3*y^2+F(31)/69*x^2*y^3-691*x*y^4+F(102)/125*y^5+F(102)/5*x^3*y*z-F(48)/37*x^2*y^2*z+F(73)/15*x*y^3*z+F(17)/111*y^4*z+4656*x^2*y*z^2+F(70)/99*x*y^2*z^2+11722*y^3*z^2+F(86)/73*x*y*z^3-9989*y^2*z^3-F(107)/81*y*z^4+F(73)/23*x^3*y*u+4474*x^2*y^2*u+F(13)/35*x*y^3*u-F(119)/4*y^4*u-F(31)/95*x^2*y*z*u+2706*x*y^2*z*u-F(58)/41*y^3*z*u-4657*x*y*z^2*u-F(13)/60*y^2*z^2*u+F(117)/74*y*z^3*u+F(45)/109*x^2*y*u^2+14147*x*y^2*u^2+15420*y^3*u^2-F(37)/30*x*y*z*u^2+F(22)/3*y^2*z*u^2+F(9)/59*y*z^2*u^2-F(5)/96*x*y*u^3-F(57)/118*y^2*u^3+4939*y*z*u^3+F(109)/81*y*u^4+453*x^3*y*v+8248*x^2*y^2*v+F(61)/26*x*y^3*v+4192*y^4*v+F(71)/77*x^3*z*v+F(117)/101*x^2*y*z*v-15002*x*y^2*z*v-F(44)/65*y^3*z*v-F(95)/28*x^2*z^2*v+3318*x*y*z^2*v+11860*y^2*z^2*v-F(57)/34*x*z^3*v+13199*y*z^3*v+10794*z^4*v-5212*x^3*u*v-F(20)/89*x^2*y*u*v+F(39)/43*x*y^2*u*v-F(67)/54*y^3*u*v+7129*x^2*z*u*v-F(17)/78*x*y*z*u*v+F(24)/61*y^2*z*u*v+2273*x*z^2*u*v+962*y*z^2*u*v-F(89)/19*z^3*u*v+1224*x^2*u^2*v-F(107)/113*x*y*u^2*v-F(116)/47*y^2*u^2*v-F(65)/92*x*z*u^2*v-F(38)/43*y*z*u^2*v-F(5)/96*z^2*u^2*v-F(19)/126*x*u^3*v-F(117)/76*y*u^3*v+14500*z*u^3*v-F(29)/48*u^4*v-3487*x^3*v^2+F(122)/47*x^2*y*v^2-F(67)/77*x*y^2*v^2-F(101)/65*y^3*v^2+F(125)/57*x^2*z*v^2-F(58)/79*x*y*z*v^2+6635*y^2*z*v^2-F(41)/57*x*z^2*v^2-F(10)/97*y*z^2*v^2+F(32)/53*z^3*v^2-F(56)/15*x^2*u*v^2-F(111)/56*x*y*u*v^2+F(113)/8*y^2*u*v^2-F(57)/23*x*z*u*v^2+12209*y*z*u*v^2-F(70)/109*z^2*u*v^2-8263*x*u^2*v^2+7670*y*u^2*v^2-12754*z*u^2*v^2-9169*u^3*v^2+10774*x^2*v^3+1179*x*y*v^3-F(120)/17*y^2*v^3-F(21)/23*x*z*v^3+F(99)/80*y*z*v^3-14600*z^2*v^3-3664*x*u*v^3+1870*y*u*v^3+5551*z*u*v^3-F(88)/5*u^2*v^3+988*x*v^4-F(90)/73*y*v^4+F(13)/59*z*v^4+F(82)/77*u*v^4-5608*v^5,F(21)/41*x^3*y*z+F(31)/69*x^2*y^2*z-691*x*y^3*z+F(102)/125*y^4*z+F(102)/5*x^3*z^2-F(48)/37*x^2*y*z^2+F(73)/15*x*y^2*z^2+F(17)/111*y^3*z^2+4656*x^2*z^3+F(70)/99*x*y*z^3+11722*y^2*z^3+F(86)/73*x*z^4-9989*y*z^4-F(107)/81*z^5+F(73)/23*x^3*z*u+4474*x^2*y*z*u+F(13)/35*x*y^2*z*u-F(119)/4*y^3*z*u-F(31)/95*x^2*z^2*u+2706*x*y*z^2*u-F(58)/41*y^2*z^2*u-4657*x*z^3*u-F(13)/60*y*z^3*u+F(117)/74*z^4*u+F(45)/109*x^2*z*u^2+14147*x*y*z*u^2+15420*y^2*z*u^2-F(37)/30*x*z^2*u^2+F(22)/3*y*z^2*u^2+F(9)/59*z^3*u^2-F(5)/96*x*z*u^3-F(57)/118*y*z*u^3+4939*z^2*u^3+F(109)/81*z*u^4-F(11)/83*x^3*y*v+F(31)/88*x^2*y^2*v-F(77)/82*x*y^3*v-F(26)/23*y^4*v-7966*x^3*z*v+F(110)/107*x^2*y*z*v+F(95)/123*x*y^2*z*v-9319*y^3*z*v-F(57)/55*x^2*z^2*v-F(10)/21*x*y*z^2*v+F(123)/59*y^2*z^2*v-F(122)/9*x*z^3*v+F(8)/87*y*z^3*v-8678*z^4*v-F(71)/118*x^3*u*v+F(56)/87*x^2*y*u*v-F(9)/113*x*y^2*u*v+6123*y^3*u*v+F(99)/62*x^2*z*u*v+F(39)/115*x*y*z*u*v+F(27)/55*y^2*z*u*v+F(65)/79*x*z^2*u*v+F(27)/67*y*z^2*u*v-10172*z^3*u*v+9323*x^2*u^2*v-8618*x*y*u^2*v+F(88)/79*y^2*u^2*v-776*x*z*u^2*v+9163*y*z*u^2*v-12151*z^2*u^2*v-10012*x*u^3*v+F(31)/126*y*u^3*v+9437*z*u^3*v-7083*u^4*v+F(43)/101*x^3*v^2-F(97)/87*x^2*y*v^2+F(39)/7*x*y^2*v^2+1590*y^3*v^2+7443*x^2*z*v^2-F(8)/3*x*y*z*v^2-F(64)/3*y^2*z*v^2+F(41)/114*x*z^2*v^2-F(29)/85*y*z^2*v^2-F(3)/104*z^3*v^2-F(67)/94*x^2*u*v^2+F(87)/107*x*y*u*v^2+8541*y^2*u*v^2-3487*x*z*u*v^2+612*y*z*u*v^2+F(22)/89*z^2*u*v^2-10833*x*u^2*v^2+9813*y*u^2*v^2-F(34)/53*z*u^2*v^2+11976*u^3*v^2+F(7)/16*x^2*v^3-8300*x*y*v^3-12027*y^2*v^3-2633*x*z*v^3-315*y*z*v^3+1511*z^2*v^3+F(57)/94*x*u*v^3+F(59)/47*y*u*v^3-3214*z*u*v^3+F(77)/51*u^2*v^3+4121*x*v^4-15308*y*v^4+8629*z*v^4-F(101)/28*u*v^4+F(32)/97*v^5,F(21)/41*x^3*y*u+F(31)/69*x^2*y^2*u-691*x*y^3*u+F(102)/125*y^4*u+F(102)/5*x^3*z*u-F(48)/37*x^2*y*z*u+F(73)/15*x*y^2*z*u+F(17)/111*y^3*z*u+4656*x^2*z^2*u+F(70)/99*x*y*z^2*u+11722*y^2*z^2*u+F(86)/73*x*z^3*u-9989*y*z^3*u-F(107)/81*z^4*u+F(73)/23*x^3*u^2+4474*x^2*y*u^2+F(13)/35*x*y^2*u^2-F(119)/4*y^3*u^2-F(31)/95*x^2*z*u^2+2706*x*y*z*u^2-F(58)/41*y^2*z*u^2-4657*x*z^2*u^2-F(13)/60*y*z^2*u^2+F(117)/74*z^3*u^2+F(45)/109*x^2*u^3+14147*x*y*u^3+15420*y^2*u^3-F(37)/30*x*z*u^3+F(22)/3*y*z*u^3+F(9)/59*z^2*u^3-F(5)/96*x*u^4-F(57)/118*y*u^4+4939*z*u^4+F(109)/81*u^5+F(49)/71*x^3*y*v-F(53)/121*x^2*y^2*v-10243*x*y^3*v+F(67)/97*y^4*v-F(106)/115*x^3*z*v-8913*x^2*y*z*v+F(56)/117*x*y^2*z*v+F(71)/112*y^3*z*v+F(2)/119*x^2*z^2*v-11383*x*y*z^2*v-F(112)/47*y^2*z^2*v-F(88)/67*x*z^3*v-F(116)/63*y*z^3*v+13357*z^4*v+172*x^3*u*v-F(31)/116*x^2*y*u*v-F(53)/91*x*y^2*u*v+F(75)/61*y^3*u*v+F(94)/61*x^2*z*u*v-5231*x*y*z*u*v-206*y^2*z*u*v-796*x*z^2*u*v-F(16)/125*y*z^2*u*v+440*z^3*u*v+F(63)/50*x^2*u^2*v+8171*x*y*u^2*v+11939*y^2*u^2*v-F(121)/116*x*z*u^2*v-F(22)/3*y*z*u^2*v-F(39)/37*z^2*u^2*v-F(43)/52*x*u^3*v+F(125)/61*y*u^3*v-F(74)/83*z*u^3*v-F(86)/97*u^4*v-F(107)/68*x^3*v^2-5166*x^2*y*v^2+908*x*y^2*v^2+8516*y^3*v^2+3668*x^2*z*v^2-F(13)/29*x*y*z*v^2+11547*y^2*z*v^2-1822*x*z^2*v^2+F(110)/27*y*z^2*v^2+8554*z^3*v^2-F(94)/95*x^2*u*v^2+F(79)/70*x*y*u*v^2+F(99)/100*y^2*u*v^2-9105*x*z*u*v^2-F(89)/44*y*z*u*v^2-6362*z^2*u*v^2+F(115)/8*x*u^2*v^2+F(71)/64*y*u^2*v^2+F(107)/60*z*u^2*v^2-F(29)/19*u^3*v^2-505*x^2*v^3-3230*x*y*v^3-F(93)/73*y^2*v^3-F(116)/11*x*z*v^3+F(110)/101*y*z*v^3-F(25)/77*z^2*v^3-F(118)/25*x*u*v^3-F(21)/2*y*u*v^3-6976*z*u*v^3-9958*u^2*v^3+2541*x*v^4-5747*y*v^4+F(28)/45*z*v^4-12528*u*v^4+10989*v^5,F(49)/71*x^4*y-F(53)/121*x^3*y^2-10243*x^2*y^3+F(67)/97*x*y^4-F(106)/115*x^4*z-8913*x^3*y*z+F(56)/117*x^2*y^2*z+F(71)/112*x*y^3*z+F(2)/119*x^3*z^2-11383*x^2*y*z^2-F(112)/47*x*y^2*z^2-F(88)/67*x^2*z^3-F(116)/63*x*y*z^3+13357*x*z^4-2589*x^4*u+F(81)/55*x^3*y*u-F(41)/113*x^2*y^2*u-F(91)/87*x*y^3*u-F(101)/47*y^4*u+F(95)/71*x^3*z*u+F(109)/38*x^2*y*z*u+F(73)/33*x*y^2*z*u+F(63)/55*y^3*z*u-F(98)/67*x^2*z^2*u-F(57)/56*x*y*z^2*u+11442*y^2*z^2*u-11066*x*z^3*u-12579*y*z^3*u-F(63)/82*z^4*u+630*x^3*u^2-F(104)/83*x^2*y*u^2-F(118)/5*x*y^2*u^2-27*y^3*u^2-F(121)/25*x^2*z*u^2+F(47)/74*x*y*z*u^2+F(117)/110*y^2*z*u^2-F(109)/107*x*z^2*u^2+F(109)/88*y*z^2*u^2+F(112)/75*z^3*u^2+7136*x^2*u^3-15396*x*y*u^3-F(19)/101*y^2*u^3-F(97)/91*x*z*u^3-11736*y*z*u^3+F(81)/5*z^2*u^3-4746*x*u^4-F(94)/25*y*u^4-505*z*u^4+14815*u^5-F(107)/68*x^4*v-5166*x^3*y*v+908*x^2*y^2*v+8516*x*y^3*v+3668*x^3*z*v-F(13)/29*x^2*y*z*v+11547*x*y^2*z*v-1822*x^2*z^2*v+F(110)/27*x*y*z^2*v+8554*x*z^3*v-1691*x^3*u*v-F(115)/19*x^2*y*u*v-F(49)/8*x*y^2*u*v-4074*y^3*u*v-466*x^2*z*u*v-F(73)/52*x*y*z*u*v-11039*y^2*z*u*v+F(11)/89*x*z^2*u*v+5456*y*z^2*u*v-F(65)/48*z^3*u*v-8774*x^2*u^2*v-F(64)/105*x*y*u^2*v+8668*y^2*u^2*v+7128*x*z*u^2*v-12692*y*z*u^2*v+9162*z^2*u^2*v-10033*x*u^3*v-2028*y*u^3*v-13675*z*u^3*v+2329*u^4*v-505*x^3*v^2-3230*x^2*y*v^2-F(93)/73*x*y^2*v^2-F(116)/11*x^2*z*v^2+F(110)/101*x*y*z*v^2-F(25)/77*x*z^2*v^2+13039*x^2*u*v^2+F(99)/19*x*y*u*v^2+4639*y^2*u*v^2-F(117)/46*x*z*u*v^2+10422*y*z*u*v^2-F(74)/25*z^2*u*v^2-F(51)/73*x*u^2*v^2+11188*y*u^2*v^2-F(18)/91*z*u^2*v^2+6424*u^3*v^2+2541*x^2*v^3-5747*x*y*v^3+F(28)/45*x*z*v^3-F(78)/25*x*u*v^3+7017*y*u*v^3-F(49)/26*z*u*v^3+15637*u^2*v^3+10989*x*v^4-F(103)/51*u*v^4,-F(49)/71*x^3*y^2+F(53)/121*x^2*y^3+10243*x*y^4-F(67)/97*y^5+F(106)/115*x^3*y*z+8913*x^2*y^2*z-F(56)/117*x*y^3*z-F(71)/112*y^4*z-F(2)/119*x^2*y*z^2+11383*x*y^2*z^2+F(112)/47*y^3*z^2+F(88)/67*x*y*z^3+F(116)/63*y^2*z^3-13357*y*z^4+F(43)/114*x^3*y*u-F(57)/35*x^2*y^2*u+F(41)/14*x*y^3*u+F(14)/107*y^4*u+F(71)/77*x^3*z*u+F(17)/37*x^2*y*z*u-9771*x*y^2*z*u-F(15)/103*y^3*z*u-F(95)/28*x^2*z^2*u+F(61)/70*x*y*z^2*u-F(79)/43*y^2*z^2*u-F(57)/34*x*z^3*u+12759*y*z^3*u+10794*z^4*u-5212*x^3*u^2-F(59)/93*x^2*y*u^2-11146*x*y^2*u^2-1869*y^3*u^2+7129*x^2*z*u^2+8833*x*y*z*u^2-F(113)/8*y^2*z*u^2+2273*x*z^2*u^2+F(93)/80*y*z^2*u^2-F(89)/19*z^3*u^2+1224*x^2*u^3-6909*x*y*u^3+F(107)/85*y^2*u^3-F(65)/92*x*z*u^3+F(126)/65*y*z*u^3-F(5)/96*z^2*u^3-F(19)/126*x*u^4-3980*y*u^4+14500*z*u^4-F(29)/48*u^5+F(107)/68*x^3*y*v+5166*x^2*y^2*v-908*x*y^3*v-8516*y^4*v-3668*x^2*y*z*v+F(13)/29*x*y^2*z*v-11547*y^3*z*v+1822*x*y*z^2*v-F(110)/27*y^2*z^2*v-8554*y*z^3*v-3487*x^3*u*v-8150*x^2*y*u*v+5773*x*y^2*u*v+F(119)/108*y^3*u*v+F(125)/57*x^2*z*u*v-12763*x*y*z*u*v-F(53)/15*y^2*z*u*v-F(41)/57*x*z^2*u*v+F(111)/46*y*z^2*u*v+F(32)/53*z^3*u*v-F(56)/15*x^2*u^2*v+11409*x*y*u^2*v-3486*y^2*u^2*v-F(57)/23*x*z*u^2*v-56*y*z*u^2*v-F(70)/109*z^2*u^2*v-8263*x*u^3*v+11039*y*u^3*v-12754*z*u^3*v-9169*u^4*v+505*x^2*y*v^2+3230*x*y^2*v^2+F(93)/73*y^3*v^2+F(116)/11*x*y*z*v^2-F(110)/101*y^2*z*v^2+F(25)/77*y*z^2*v^2+10774*x^2*u*v^2-F(59)/94*x*y*u*v^2+F(117)/34*y^2*u*v^2-F(21)/23*x*z*u*v^2+F(64)/45*y*z*u*v^2-14600*z^2*u*v^2-3664*x*u^2*v^2-F(72)/119*y*u^2*v^2+5551*z*u^2*v^2-F(88)/5*u^3*v^2-2541*x*y*v^3+5747*y^2*v^3-F(28)/45*y*z*v^3+988*x*u*v^3+12965*y*u*v^3+F(13)/59*z*u*v^3+F(82)/77*u^2*v^3-10989*y*v^4-5608*u*v^4,-F(49)/71*x^3*y*z+F(53)/121*x^2*y^2*z+10243*x*y^3*z-F(67)/97*y^4*z+F(106)/115*x^3*z^2+8913*x^2*y*z^2-F(56)/117*x*y^2*z^2-F(71)/112*y^3*z^2-F(2)/119*x^2*z^3+11383*x*y*z^3+F(112)/47*y^2*z^3+F(88)/67*x*z^4+F(116)/63*y*z^4-13357*z^5-F(11)/83*x^3*y*u+F(31)/88*x^2*y^2*u-F(77)/82*x*y^3*u-F(26)/23*y^4*u+F(7)/114*x^3*z*u+F(111)/26*x^2*y*z*u-F(124)/111*x*y^2*z*u+F(103)/14*y^3*z*u+2448*x^2*z^2*u+F(60)/37*x*y*z^2*u-F(6)/13*y^2*z^2*u-F(45)/59*x*z^3*u+13938*y*z^3*u-9118*z^4*u-F(71)/118*x^3*u^2+F(56)/87*x^2*y*u^2-F(9)/113*x*y^2*u^2+6123*y^3*u^2-F(14)/81*x^2*z*u^2+453*x*y*z*u^2+F(68)/95*y^2*z*u^2+8237*x*z^2*u^2-F(115)/88*y*z^2*u^2+3663*z^3*u^2+9323*x^2*u^3-8618*x*y*u^3+F(88)/79*y^2*u^3+8453*x*z*u^3-F(13)/32*y*z*u^3-F(57)/71*z^2*u^3-10012*x*u^4+F(31)/126*y*u^4+2512*z*u^4-7083*u^5+F(107)/68*x^3*z*v+5166*x^2*y*z*v-908*x*y^2*z*v-8516*y^3*z*v-3668*x^2*z^2*v+F(13)/29*x*y*z^2*v-11547*y^2*z^2*v+1822*x*z^3*v-F(110)/27*y*z^3*v-8554*z^4*v+F(43)/101*x^3*u*v-F(97)/87*x^2*y*u*v+F(39)/7*x*y^2*u*v+1590*y^3*u*v+F(73)/21*x^2*z*u*v+F(103)/13*x*y*z*u*v+F(119)/9*y^2*z*u*v+F(17)/112*x*z^2*u*v+13183*y*z^2*u*v+F(80)/81*z^3*u*v-F(67)/94*x^2*u^2*v+F(87)/107*x*y*u^2*v+8541*y^2*u^2*v-15498*x*z*u^2*v-6887*y*z*u^2*v-F(2)/91*z^2*u^2*v-10833*x*u^3*v+9813*y*u^3*v+10008*z*u^3*v+11976*u^4*v+505*x^2*z*v^2+3230*x*y*z*v^2+F(93)/73*y^2*z*v^2+F(116)/11*x*z^2*v^2-F(110)/101*y*z^2*v^2+F(25)/77*z^3*v^2+F(7)/16*x^2*u*v^2-8300*x*y*u*v^2-12027*y^2*u*v^2-69*x*z*u*v^2+15691*y*z*u*v^2-F(20)/49*z^2*u*v^2+F(57)/94*x*u^2*v^2+F(59)/47*y*u^2*v^2+6744*z*u^2*v^2+F(77)/51*u^3*v^2-2541*x*z*v^3+5747*y*z*v^3-F(28)/45*z^2*v^3+4121*x*u*v^3-15308*y*u*v^3+F(103)/62*z*u*v^3-F(101)/28*u^2*v^3-10989*z*v^4+F(32)/97*u*v^4,-F(11)/83*x^4*y+F(31)/88*x^3*y^2-F(77)/82*x^2*y^3-F(26)/23*x*y^4-10727*x^4*z-12992*x^3*y*z-F(66)/67*x^2*y^2*z+10276*x*y^3*z-F(101)/47*y^4*z-F(101)/66*x^3*z^2-F(33)/94*x^2*y*z^2+8130*x*y^2*z^2+F(63)/55*y^3*z^2+15795*x^2*z^3-9485*x*y*z^3+11442*y^2*z^3+F(67)/84*x*z^4-12579*y*z^4-F(63)/82*z^5-F(71)/118*x^4*u+F(56)/87*x^3*y*u-F(9)/113*x^2*y^2*u+6123*x*y^3*u+15243*x^3*z*u-12653*x^2*y*z*u+14794*x*y^2*z*u-27*y^3*z*u+15910*x^2*z^2*u-F(1)/60*x*y*z^2*u+F(117)/110*y^2*z^2*u-F(85)/19*x*z^3*u+F(109)/88*y*z^3*u+F(112)/75*z^4*u+9323*x^3*u^2-8618*x^2*y*u^2+F(88)/79*x*y^2*u^2+15589*x^2*z*u^2-F(59)/80*x*y*z*u^2-F(19)/101*y^2*z*u^2-14252*x*z^2*u^2-11736*y*z^2*u^2+F(81)/5*z^3*u^2-10012*x^2*u^3+F(31)/126*x*y*u^3-F(89)/43*x*z*u^3-F(94)/25*y*z*u^3-505*z^2*u^3-7083*x*u^4+14815*z*u^4+F(43)/101*x^4*v-F(97)/87*x^3*y*v+F(39)/7*x^2*y^2*v+1590*x*y^3*v+4406*x^3*z*v+8032*x^2*y*z*v+F(17)/65*x*y^2*z*v-4074*y^3*z*v+6675*x^2*z^2*v+F(17)/47*x*y*z^2*v-11039*y^2*z^2*v+F(111)/32*x*z^3*v+5456*y*z^3*v-F(65)/48*z^4*v-F(67)/94*x^3*u*v+F(87)/107*x^2*y*u*v+8541*x*y^2*u*v-F(86)/29*x^2*z*u*v+F(83)/115*x*y*z*u*v+8668*y^2*z*u*v-6934*x*z^2*u*v-12692*y*z^2*u*v+9162*z^3*u*v-10833*x^2*u^2*v+9813*x*y*u^2*v-25*x*z*u^2*v-2028*y*z*u^2*v-13675*z^2*u^2*v+11976*x*u^3*v+2329*z*u^3*v+F(7)/16*x^3*v^2-8300*x^2*y*v^2-12027*x*y^2*v^2+F(25)/37*x^2*z*v^2-F(56)/3*x*y*z*v^2+4639*y^2*z*v^2-F(13)/115*x*z^2*v^2+10422*y*z^2*v^2-F(74)/25*z^3*v^2+F(57)/94*x^2*u*v^2+F(59)/47*x*y*u*v^2+8058*x*z*u*v^2+11188*y*z*u*v^2-F(18)/91*z^2*u*v^2+F(77)/51*x*u^2*v^2+6424*z*u^2*v^2+4121*x^2*v^3-15308*x*y*v^3-600*x*z*v^3+7017*y*z*v^3-F(49)/26*z^2*v^3-F(101)/28*x*u*v^3+15637*z*u*v^3+F(32)/97*x*v^4-F(103)/51*z*v^4,F(11)/83*x^3*y^2-F(31)/88*x^2*y^3+F(77)/82*x*y^4+F(26)/23*y^5+F(6)/19*x^3*y*z-F(116)/121*x^2*y^2*z-10392*x*y^3*z+13511*y^4*z+F(71)/77*x^3*z^2-12823*x^2*y*z^2+F(70)/79*x*y^2*z^2+F(96)/77*y^3*z^2-F(95)/28*x^2*z^3-223*x*y*z^3-F(79)/11*y^2*z^3-F(57)/34*x*z^4-10114*y*z^4+10794*z^5+F(71)/118*x^3*y*u-F(56)/87*x^2*y^2*u+F(9)/113*x*y^3*u-6123*y^4*u-5212*x^3*z*u-F(107)/11*x^2*y*z*u-11599*x*y^2*z*u-F(56)/51*y^3*z*u+7129*x^2*z^2*u+596*x*y*z^2*u+8712*y^2*z^2*u+2273*x*z^3*u+11134*y*z^3*u-F(89)/19*z^4*u-9323*x^2*y*u^2+8618*x*y^2*u^2-F(88)/79*y^3*u^2+1224*x^2*z*u^2-15362*x*y*z*u^2-F(109)/97*y^2*z*u^2-F(65)/92*x*z^2*u^2+15870*y*z^2*u^2-F(5)/96*z^3*u^2+10012*x*y*u^3-F(31)/126*y^2*u^3-F(19)/126*x*z*u^3-F(74)/69*y*z*u^3+14500*z^2*u^3+7083*y*u^4-F(29)/48*z*u^4-F(43)/101*x^3*y*v+F(97)/87*x^2*y^2*v-F(39)/7*x*y^3*v-1590*y^4*v-3487*x^3*z*v-14247*x^2*y*z*v-F(81)/32*x*y^2*z*v-10972*y^3*z*v+F(125)/57*x^2*z^2*v+F(68)/45*x*y*z^2*v-F(26)/11*y^2*z^2*v-F(41)/57*x*z^3*v-F(49)/62*y*z^3*v+F(32)/53*z^4*v+F(67)/94*x^2*y*u*v-F(87)/107*x*y^2*u*v-8541*y^3*u*v-F(56)/15*x^2*z*u*v-5084*x*y*z*u*v-F(108)/47*y^2*z*u*v-F(57)/23*x*z^2*u*v+14006*y*z^2*u*v-F(70)/109*z^3*u*v+10833*x*y*u^2*v-9813*y^2*u^2*v-8263*x*z*u^2*v-F(30)/31*y*z*u^2*v-12754*z^2*u^2*v-11976*y*u^3*v-9169*z*u^3*v-F(7)/16*x^2*y*v^2+8300*x*y^2*v^2+12027*y^3*v^2+10774*x^2*z*v^2+3812*x*y*z*v^2-F(68)/67*y^2*z*v^2-F(21)/23*x*z^2*v^2+2889*y*z^2*v^2-14600*z^3*v^2-F(57)/94*x*y*u*v^2-F(59)/47*y^2*u*v^2-3664*x*z*u*v^2+5084*y*z*u*v^2+5551*z^2*u*v^2-F(77)/51*y*u^2*v^2-F(88)/5*z*u^2*v^2-4121*x*y*v^3+15308*y^2*v^3+988*x*z*v^3+F(67)/82*y*z*v^3+F(13)/59*z^2*v^3+F(101)/28*y*u*v^3+F(82)/77*z*u*v^3-F(32)/97*y*v^4-5608*z*v^4,-F(61)/97*x^4*y+F(12)/77*x^3*y^2-589*x^2*y^3+F(46)/39*x*y^4+F(101)/47*y^5-F(71)/77*x^4*z+3615*x^3*y*z+10610*x^2*y^2*z-F(113)/123*x*y^3*z-F(63)/55*y^4*z+F(95)/28*x^3*z^2-15572*x^2*y*z^2-F(7)/72*x*y^2*z^2-11442*y^3*z^2+F(57)/34*x^2*z^3-1693*x*y*z^3+12579*y^2*z^3-10794*x*z^4+F(63)/82*y*z^4+5212*x^4*u-F(73)/122*x^3*y*u+F(47)/62*x^2*y^2*u-F(73)/98*x*y^3*u+27*y^4*u-7129*x^3*z*u+15485*x^2*y*z*u+F(93)/79*x*y^2*z*u-F(117)/110*y^3*z*u-2273*x^2*z^2*u-F(69)/64*x*y*z^2*u-F(109)/88*y^2*z^2*u+F(89)/19*x*z^3*u-F(112)/75*y*z^3*u-1224*x^3*u^2-227*x^2*y*u^2+10502*x*y^2*u^2+F(19)/101*y^3*u^2+F(65)/92*x^2*z*u^2-1618*x*y*z*u^2+11736*y^2*z*u^2+F(5)/96*x*z^2*u^2-F(81)/5*y*z^2*u^2+F(19)/126*x^2*u^3+F(13)/11*x*y*u^3+F(94)/25*y^2*u^3-14500*x*z*u^3+505*y*z*u^3+F(29)/48*x*u^4-14815*y*u^4+3487*x^4*v-F(31)/13*x^3*y*v-F(47)/33*x^2*y^2*v-2809*x*y^3*v+4074*y^4*v-F(125)/57*x^3*z*v+13229*x^2*y*z*v+14729*x*y^2*z*v+11039*y^3*z*v+F(41)/57*x^2*z^2*v+F(27)/14*x*y*z^2*v-5456*y^2*z^2*v-F(32)/53*x*z^3*v+F(65)/48*y*z^3*v+F(56)/15*x^3*u*v-F(38)/85*x^2*y*u*v-F(103)/6*x*y^2*u*v-8668*y^3*u*v+F(57)/23*x^2*z*u*v-F(29)/95*x*y*z*u*v+12692*y^2*z*u*v+F(70)/109*x*z^2*u*v-9162*y*z^2*u*v+8263*x^2*u^2*v-1006*x*y*u^2*v+2028*y^2*u^2*v+12754*x*z*u^2*v+13675*y*z*u^2*v+9169*x*u^3*v-2329*y*u^3*v-10774*x^3*v^2+F(10)/61*x^2*y*v^2+F(119)/95*x*y^2*v^2-4639*y^3*v^2+F(21)/23*x^2*z*v^2-F(43)/3*x*y*z*v^2-10422*y^2*z*v^2+14600*x*z^2*v^2+F(74)/25*y*z^2*v^2+3664*x^2*u*v^2-13142*x*y*u*v^2-11188*y^2*u*v^2-5551*x*z*u*v^2+F(18)/91*y*z*u*v^2+F(88)/5*x*u^2*v^2-6424*y*u^2*v^2-988*x^2*v^3+8792*x*y*v^3-7017*y^2*v^3-F(13)/59*x*z*v^3+F(49)/26*y*z*v^3-F(82)/77*x*u*v^3-15637*y*u*v^3+5608*x*v^4+F(103)/51*y*v^4,-8198*x^4*y-13439*x^3*y^2-F(111)/77*x^2*y^3+F(14)/109*x*y^4+9224*x^4*z+14009*x^3*y*z-F(50)/101*x^2*y^2*z+5803*x*y^3*z-F(27)/40*x^3*z^2+F(91)/53*x^2*y*z^2-F(88)/53*x*y^2*z^2-10595*x^2*z^3-F(43)/10*x*y*z^3-12198*x*z^4+F(45)/31*x^4*u-F(113)/28*x^3*y*u-14069*x^2*y^2*u+13761*x*y^3*u-F(93)/37*x^3*z*u-6844*x^2*y*z*u+F(109)/11*x*y^2*z*u-2794*x^2*z^2*u+F(48)/11*x*y*z^2*u+7009*x*z^3*u-F(105)/58*x^3*u^2+14285*x^2*y*u^2+F(57)/70*x*y^2*u^2-14781*x^2*z*u^2+F(117)/85*x*y*z*u^2+F(43)/19*x*z^2*u^2+F(121)/48*x^2*u^3-14747*x*y*u^3-21*x*z*u^3-F(95)/8*x*u^4+F(33)/85*x^4*v-F(119)/106*x^3*y*v+F(8)/27*x^2*y^2*v-5093*x*y^3*v+13639*y^4*v+F(102)/73*x^3*z*v-F(31)/48*x^2*y*z*v-6262*x*y^2*z*v+F(68)/39*y^3*z*v+1461*x^2*z^2*v-F(61)/59*x*y*z^2*v-F(1)/63*y^2*z^2*v-12965*x*z^3*v-F(46)/105*y*z^3*v+1479*z^4*v+F(41)/38*x^3*u*v+F(24)/95*x^2*y*u*v+F(73)/98*x*y^2*u*v+F(37)/120*y^3*u*v-1916*x^2*z*u*v+15103*x*y*z*u*v-13003*y^2*z*u*v-F(41)/26*x*z^2*u*v+F(54)/13*y*z^2*u*v-7940*z^3*u*v+1630*x^2*u^2*v+13777*x*y*u^2*v-11049*y^2*u^2*v-F(104)/23*x*z*u^2*v-F(45)/38*y*z*u^2*v-10093*z^2*u^2*v+F(60)/67*x*u^3*v+6513*y*u^3*v-F(79)/113*z*u^3*v+95*u^4*v+F(122)/105*x^3*v^2+F(29)/56*x^2*y*v^2+F(67)/61*x*y^2*v^2-11133*y^3*v^2-F(107)/18*x^2*z*v^2+F(7)/125*x*y*z*v^2-F(31)/99*y^2*z*v^2-F(68)/15*x*z^2*v^2+12069*y*z^2*v^2-7743*z^3*v^2+F(68)/7*x^2*u*v^2-F(113)/121*x*y*u*v^2-F(50)/93*y^2*u*v^2+F(43)/113*x*z*u*v^2+F(34)/3*y*z*u*v^2+F(34)/39*z^2*u*v^2-14670*x*u^2*v^2-F(51)/58*y*u^2*v^2-F(68)/45*z*u^2*v^2+14528*u^3*v^2-F(89)/83*x^2*v^3-14692*x*y*v^3-9236*y^2*v^3-F(33)/41*x*z*v^3-F(113)/7*y*z*v^3-F(39)/67*z^2*v^3-8159*x*u*v^3-F(122)/53*y*u*v^3+F(89)/125*z*u*v^3-F(27)/58*u^2*v^3-F(88)/17*x*v^4-F(73)/89*y*v^4-1611*z*v^4+F(111)/53*u*v^4-F(91)/29*v^5,8198*x^3*y^2+13439*x^2*y^3+F(111)/77*x*y^4-F(14)/109*y^5-9224*x^3*y*z-14009*x^2*y^2*z+F(50)/101*x*y^3*z-5803*y^4*z+F(27)/40*x^2*y*z^2-F(91)/53*x*y^2*z^2+F(88)/53*y^3*z^2+10595*x*y*z^3+F(43)/10*y^2*z^3+12198*y*z^4-F(45)/31*x^3*y*u+F(113)/28*x^2*y^2*u+14069*x*y^3*u-13761*y^4*u+F(93)/37*x^2*y*z*u+6844*x*y^2*z*u-F(109)/11*y^3*z*u+2794*x*y*z^2*u-F(48)/11*y^2*z^2*u-7009*y*z^3*u+F(105)/58*x^2*y*u^2-14285*x*y^2*u^2-F(57)/70*y^3*u^2+14781*x*y*z*u^2-F(117)/85*y^2*z*u^2-F(43)/19*y*z^2*u^2-F(121)/48*x*y*u^3+14747*y^2*u^3+21*y*z*u^3+F(95)/8*y*u^4-F(118)/85*x^3*y*v-F(11)/113*x^2*y^2*v+F(118)/11*x*y^3*v-F(90)/67*y^4*v+6362*x^3*z*v-11958*x^2*y*z*v-F(44)/105*x*y^2*z*v+F(72)/43*y^3*z*v+F(125)/28*x^2*z^2*v+F(48)/73*x*y*z^2*v+2736*y^2*z^2*v+F(75)/34*x*z^3*v+F(14)/97*y*z^3*v+F(78)/101*z^4*v+F(13)/25*x^3*u*v-F(47)/93*x^2*y*u*v-3776*x*y^2*u*v-F(41)/112*y^3*u*v-F(55)/39*x^2*z*u*v+13292*x*y*z*u*v+F(97)/13*y^2*z*u*v-F(9)/2*x*z^2*u*v+10077*y*z^2*u*v-F(16)/81*z^3*u*v-2363*x^2*u^2*v+F(15)/98*x*y*u^2*v+F(61)/66*y^2*u^2*v-F(115)/73*x*z*u^2*v-F(121)/85*y*z*u^2*v-F(83)/54*z^2*u^2*v+F(3)/14*x*u^3*v+F(55)/82*y*u^3*v+F(59)/99*z*u^3*v+F(55)/42*u^4*v+13244*x^3*v^2+7671*x^2*y*v^2+10274*x*y^2*v^2-12422*y^3*v^2-10402*x^2*z*v^2-F(62)/11*x*y*z*v^2-F(103)/57*y^2*z*v^2+F(97)/65*x*z^2*v^2+4151*y*z^2*v^2+F(115)/64*z^3*v^2+F(9)/116*x^2*u*v^2+F(69)/80*x*y*u*v^2-F(87)/49*y^2*u*v^2-F(15)/88*x*z*u*v^2-10332*y*z*u*v^2+F(17)/120*z^2*u*v^2+F(105)/106*x*u^2*v^2+F(67)/84*y*u^2*v^2-F(83)/27*z*u^2*v^2-F(49)/32*u^3*v^2-9506*x^2*v^3+10618*x*y*v^3-8658*y^2*v^3+F(109)/21*x*z*v^3-F(106)/9*y*z*v^3+2219*z^2*v^3+F(78)/7*x*u*v^3-F(21)/100*y*u*v^3+F(82)/77*z*u*v^3+F(71)/123*u^2*v^3+F(63)/86*x*v^4+F(40)/11*y*v^4-F(76)/123*z*v^4+F(76)/89*u*v^4-F(101)/12*v^5,8198*x^3*y*z+13439*x^2*y^2*z+F(111)/77*x*y^3*z-F(14)/109*y^4*z-9224*x^3*z^2-14009*x^2*y*z^2+F(50)/101*x*y^2*z^2-5803*y^3*z^2+F(27)/40*x^2*z^3-F(91)/53*x*y*z^3+F(88)/53*y^2*z^3+10595*x*z^4+F(43)/10*y*z^4+12198*z^5-F(45)/31*x^3*z*u+F(113)/28*x^2*y*z*u+14069*x*y^2*z*u-13761*y^3*z*u+F(93)/37*x^2*z^2*u+6844*x*y*z^2*u-F(109)/11*y^2*z^2*u+2794*x*z^3*u-F(48)/11*y*z^3*u-7009*z^4*u+F(105)/58*x^2*z*u^2-14285*x*y*z*u^2-F(57)/70*y^2*z*u^2+14781*x*z^2*u^2-F(117)/85*y*z^2*u^2-F(43)/19*z^3*u^2-F(121)/48*x*z*u^3+14747*y*z*u^3+21*z^2*u^3+F(95)/8*z*u^4-1962*x^3*y*v+F(76)/69*x^2*y^2*v+F(96)/17*x*y^3*v+F(112)/89*y^4*v-9166*x^3*z*v+12218*x^2*y*z*v-F(19)/32*x*y^2*z*v+F(62)/113*y^3*z*v+F(71)/7*x^2*z^2*v+8911*x*y*z^2*v-6356*y^2*z^2*v-12595*x*z^3*v+F(29)/10*y*z^3*v-F(43)/94*z^4*v+F(17)/117*x^3*u*v+F(103)/75*x^2*y*u*v+3540*x*y^2*u*v-F(6)/71*y^3*u*v+F(110)/83*x^2*z*u*v-F(87)/44*x*y*z*u*v-F(65)/48*y^2*z*u*v-15129*x*z^2*u*v-F(83)/63*y*z^2*u*v-F(59)/30*z^3*u*v+F(122)/121*x^2*u^2*v-F(38)/47*x*y*u^2*v-675*y^2*u^2*v+F(73)/54*x*z*u^2*v+777*y*z*u^2*v-F(42)/79*z^2*u^2*v-8383*x*u^3*v+F(49)/51*y*u^3*v-13743*z*u^3*v-10950*u^4*v-6057*x^3*v^2-F(109)/76*x^2*y*v^2-12344*x*y^2*v^2+F(71)/13*y^3*v^2+F(41)/30*x^2*z*v^2-11594*x*y*z*v^2+7192*y^2*z*v^2+F(126)/43*x*z^2*v^2-5930*y*z^2*v^2-F(61)/31*z^3*v^2-14562*x^2*u*v^2-14367*x*y*u*v^2+F(47)/53*y^2*u*v^2+F(105)/34*x*z*u*v^2-10738*y*z*u*v^2+7084*z^2*u*v^2+F(44)/115*x*u^2*v^2+F(76)/29*y*u^2*v^2-F(25)/114*z*u^2*v^2-123*u^3*v^2+F(2)/7*x^2*v^3+F(31)/9*x*y*v^3+F(52)/73*y^2*v^3+15620*x*z*v^3-F(37)/98*y*z*v^3-11043*z^2*v^3-F(121)/70*x*u*v^3+F(85)/44*y*u*v^3+3125*z*u*v^3-F(89)/117*u^2*v^3+1629*x*v^4-F(27)/55*y*v^4+14263*z*v^4-F(13)/19*u*v^4-7472*v^5,8198*x^3*y*u+13439*x^2*y^2*u+F(111)/77*x*y^3*u-F(14)/109*y^4*u-9224*x^3*z*u-14009*x^2*y*z*u+F(50)/101*x*y^2*z*u-5803*y^3*z*u+F(27)/40*x^2*z^2*u-F(91)/53*x*y*z^2*u+F(88)/53*y^2*z^2*u+10595*x*z^3*u+F(43)/10*y*z^3*u+12198*z^4*u-F(45)/31*x^3*u^2+F(113)/28*x^2*y*u^2+14069*x*y^2*u^2-13761*y^3*u^2+F(93)/37*x^2*z*u^2+6844*x*y*z*u^2-F(109)/11*y^2*z*u^2+2794*x*z^2*u^2-F(48)/11*y*z^2*u^2-7009*z^3*u^2+F(105)/58*x^2*u^3-14285*x*y*u^3-F(57)/70*y^2*u^3+14781*x*z*u^3-F(117)/85*y*z*u^3-F(43)/19*z^2*u^3-F(121)/48*x*u^4+14747*y*u^4+21*z*u^4+F(95)/8*u^5+F(48)/11*x^3*y*v-1646*x^2*y^2*v-F(42)/95*x*y^3*v+F(82)/51*y^4*v+F(107)/88*x^3*z*v-F(7)/46*x^2*y*z*v-F(92)/83*x*y^2*z*v-F(104)/77*y^3*z*v-F(65)/46*x^2*z^2*v+10608*x*y*z^2*v+F(87)/74*y^2*z^2*v-10228*x*z^3*v+9162*y*z^3*v+3983*z^4*v-F(25)/47*x^3*u*v-6011*x^2*y*u*v-F(116)/113*x*y^2*u*v+F(63)/68*y^3*u*v+F(101)/18*x^2*z*u*v-11525*x*y*z*u*v+F(19)/126*y^2*z*u*v+F(39)/8*x*z^2*u*v+12528*y*z^2*u*v+757*z^3*u*v-F(122)/55*x^2*u^2*v+F(102)/83*x*y*u^2*v-14529*y^2*u^2*v-11437*x*z*u^2*v+14821*y*z*u^2*v-F(59)/60*z^2*u^2*v+5194*x*u^3*v+F(91)/57*y*u^3*v+9247*z*u^3*v+F(24)/89*u^4*v+F(112)/81*x^3*v^2-11670*x^2*y*v^2+12596*x*y^2*v^2+14040*y^3*v^2-F(43)/64*x^2*z*v^2-F(115)/47*x*y*z*v^2+608*y^2*z*v^2+6185*x*z^2*v^2+11068*y*z^2*v^2+6425*z^3*v^2+F(59)/57*x^2*u*v^2+F(69)/94*x*y*u*v^2-9920*y^2*u*v^2+F(11)/15*x*z*u*v^2+F(71)/101*y*z*u*v^2-F(52)/9*z^2*u*v^2-9174*x*u^2*v^2+6764*y*u^2*v^2-F(65)/121*z*u^2*v^2+14804*u^3*v^2+15773*x^2*v^3-12024*x*y*v^3-F(106)/81*y^2*v^3+2037*x*z*v^3-F(114)/97*y*z*v^3+F(5)/88*z^2*v^3-12115*x*u*v^3-169*y*u*v^3+7517*z*u*v^3-F(50)/101*u^2*v^3-F(91)/124*x*v^4+F(117)/47*y*v^4+52*z*v^4-8500*u*v^4+14405*v^5,F(48)/11*x^4*y-1646*x^3*y^2-F(42)/95*x^2*y^3+F(82)/51*x*y^4+F(107)/88*x^4*z-F(7)/46*x^3*y*z-F(92)/83*x^2*y^2*z-F(104)/77*x*y^3*z-F(65)/46*x^3*z^2+10608*x^2*y*z^2+F(87)/74*x*y^2*z^2-10228*x^2*z^3+9162*x*y*z^3+3983*x*z^4+4276*x^4*u-F(32)/87*x^3*y*u-5558*x^2*y^2*u-11208*x*y^3*u+13639*y^4*u+6264*x^3*z*u+15800*x^2*y*z*u+13796*x*y^2*z*u+F(68)/39*y^3*z*u+F(95)/101*x^2*z^2*u+2767*x*y*z^2*u-F(1)/63*y^2*z^2*u-F(69)/76*x*z^3*u-F(46)/105*y*z^3*u+1479*z^4*u+15627*x^3*u^2+F(55)/7*x^2*y*u^2+2773*x*y^2*u^2+F(37)/120*y^3*u^2-F(27)/115*x^2*z*u^2-F(95)/31*x*y*z*u^2-13003*y^2*z*u^2+F(100)/9*x*z^2*u^2+F(54)/13*y*z^2*u^2-7940*z^3*u^2-F(56)/75*x^2*u^3+F(1)/60*x*y*u^3-11049*y^2*u^3+F(6)/107*x*z*u^3-F(45)/38*y*z*u^3-10093*z^2*u^3+F(61)/103*x*u^4+6513*y*u^4-F(79)/113*z*u^4+95*u^5+F(112)/81*x^4*v-11670*x^3*y*v+12596*x^2*y^2*v+14040*x*y^3*v-F(43)/64*x^3*z*v-F(115)/47*x^2*y*z*v+608*x*y^2*z*v+6185*x^2*z^2*v+11068*x*y*z^2*v+6425*x*z^3*v+F(20)/31*x^3*u*v-F(107)/99*x^2*y*u*v-F(102)/113*x*y^2*u*v-11133*y^3*u*v+F(48)/59*x^2*z*u*v+12412*x*y*z*u*v-F(31)/99*y^2*z*u*v+F(51)/64*x*z^2*u*v+12069*y*z^2*u*v-7743*z^3*u*v-24*x^2*u^2*v+12844*x*y*u^2*v-F(50)/93*y^2*u^2*v+F(45)/116*x*z*u^2*v+F(34)/3*y*z*u^2*v+F(34)/39*z^2*u^2*v+134*x*u^3*v-F(51)/58*y*u^3*v-F(68)/45*z*u^3*v+14528*u^4*v+15773*x^3*v^2-12024*x^2*y*v^2-F(106)/81*x*y^2*v^2+2037*x^2*z*v^2-F(114)/97*x*y*z*v^2+F(5)/88*x*z^2*v^2+F(95)/119*x^2*u*v^2-14861*x*y*u*v^2-9236*y^2*u*v^2+9857*x*z*u*v^2-F(113)/7*y*z*u*v^2-F(39)/67*z^2*u*v^2+F(52)/55*x*u^2*v^2-F(122)/53*y*u^2*v^2+F(89)/125*z*u^2*v^2-F(27)/58*u^3*v^2-F(91)/124*x^2*v^3+F(117)/47*x*y*v^3+52*x*z*v^3-F(24)/77*x*u*v^3-F(73)/89*y*u*v^3-1611*z*u*v^3+F(111)/53*u^2*v^3+14405*x*v^4-F(91)/29*u*v^4,-F(48)/11*x^3*y^2+1646*x^2*y^3+F(42)/95*x*y^4-F(82)/51*y^5-F(107)/88*x^3*y*z+F(7)/46*x^2*y^2*z+F(92)/83*x*y^3*z+F(104)/77*y^4*z+F(65)/46*x^2*y*z^2-10608*x*y^2*z^2-F(87)/74*y^3*z^2+10228*x*y*z^3-9162*y^2*z^3-3983*y*z^4-4277*x^3*y*u-12391*x^2*y^2*u+10847*x*y^3*u+9456*y^4*u+6362*x^3*z*u+11141*x^2*y*z*u-F(115)/71*x*y^2*z*u-F(92)/21*y^3*z*u+F(125)/28*x^2*z^2*u+F(67)/114*x*y*z^2*u+F(57)/49*y^2*z^2*u+F(75)/34*x*z^3*u+F(55)/13*y*z^3*u+F(78)/101*z^4*u+F(13)/25*x^3*u^2+F(75)/73*x^2*y*u^2-F(77)/4*x*y^2*u^2+14243*y^3*u^2-F(55)/39*x^2*z*u^2-7262*x*y*z*u^2+F(124)/99*y^2*z*u^2-F(9)/2*x*z^2*u^2-F(105)/2*y*z^2*u^2-F(16)/81*z^3*u^2-2363*x^2*u^3-F(86)/53*x*y*u^3+2678*y^2*u^3-F(115)/73*x*z*u^3-F(72)/119*y*z*u^3-F(83)/54*z^2*u^3+F(3)/14*x*u^4+F(114)/11*y*u^4+F(59)/99*z*u^4+F(55)/42*u^5-F(112)/81*x^3*y*v+11670*x^2*y^2*v-12596*x*y^3*v-14040*y^4*v+F(43)/64*x^2*y*z*v+F(115)/47*x*y^2*z*v-608*y^3*z*v-6185*x*y*z^2*v-11068*y^2*z^2*v-6425*y*z^3*v+13244*x^3*u*v+3180*x^2*y*u*v-F(61)/42*x*y^2*u*v-2502*y^3*u*v-10402*x^2*z*u*v-F(39)/67*x*y*z*u*v+5371*y^2*z*u*v+F(97)/65*x*z^2*u*v-13616*y*z^2*u*v+F(115)/64*z^3*u*v+F(9)/116*x^2*u^2*v+1577*x*y*u^2*v-237*y^2*u^2*v-F(15)/88*x*z*u^2*v+11084*y*z*u^2*v+F(17)/120*z^2*u^2*v+F(105)/106*x*u^3*v+F(69)/32*y*u^3*v-F(83)/27*z*u^3*v-F(49)/32*u^4*v-15773*x^2*y*v^2+12024*x*y^2*v^2+F(106)/81*y^3*v^2-2037*x*y*z*v^2+F(114)/97*y^2*z*v^2-F(5)/88*y*z^2*v^2-9506*x^2*u*v^2+F(97)/38*x*y*u*v^2-F(78)/49*y^2*u*v^2+F(109)/21*x*z*u*v^2+10244*y*z*u*v^2+2219*z^2*u*v^2+F(78)/7*x*u^2*v^2-11621*y*u^2*v^2+F(82)/77*z*u^2*v^2+F(71)/123*u^3*v^2+F(91)/124*x*y*v^3-F(117)/47*y^2*v^3-52*y*z*v^3+F(63)/86*x*u*v^3+F(75)/118*y*u*v^3-F(76)/123*z*u*v^3+F(76)/89*u^2*v^3-14405*y*v^4-F(101)/12*u*v^4,-F(48)/11*x^3*y*z+1646*x^2*y^2*z+F(42)/95*x*y^3*z-F(82)/51*y^4*z-F(107)/88*x^3*z^2+F(7)/46*x^2*y*z^2+F(92)/83*x*y^2*z^2+F(104)/77*y^3*z^2+F(65)/46*x^2*z^3-10608*x*y*z^3-F(87)/74*y^2*z^3+10228*x*z^4-9162*y*z^4-3983*z^5-1962*x^3*y*u+F(76)/69*x^2*y^2*u+F(96)/17*x*y^3*u+F(112)/89*y^4*u-F(109)/97*x^3*z*u-13762*x^2*y*z*u+F(34)/99*x*y^2*z*u+F(78)/47*y^3*z*u-F(109)/88*x^2*z^2*u-F(97)/36*x*y*z^2*u+F(64)/109*y^2*z^2*u-F(118)/93*x*z^3*u+F(113)/24*y*z^3*u+F(71)/101*z^4*u+F(17)/117*x^3*u^2+F(103)/75*x^2*y*u^2+3540*x*y^2*u^2-F(6)/71*y^3*u^2-13809*x^2*z*u^2+F(88)/19*x*y*z*u^2-F(8)/5*y^2*z*u^2-F(19)/26*x*z^2*u^2-8221*y*z^2*u^2-F(59)/60*z^3*u^2+F(122)/121*x^2*u^3-F(38)/47*x*y*u^3-675*y^2*u^3+9618*x*z*u^3-F(37)/47*y*z*u^3+2496*z^2*u^3-8383*x*u^4+F(49)/51*y*u^4+5667*z*u^4-10950*u^5-F(112)/81*x^3*z*v+11670*x^2*y*z*v-12596*x*y^2*z*v-14040*y^3*z*v+F(43)/64*x^2*z^2*v+F(115)/47*x*y*z^2*v-608*y^2*z^2*v-6185*x*z^3*v-11068*y*z^3*v-6425*z^4*v-6057*x^3*u*v-F(109)/76*x^2*y*u*v-12344*x*y^2*u*v+F(71)/13*y^3*u*v-5556*x^2*z*u*v-F(97)/62*x*y*z*u*v+F(23)/43*y^2*z*u*v-97*x*z^2*u*v+F(77)/19*y*z^2*u*v+15254*z^3*u*v-14562*x^2*u^2*v-14367*x*y*u^2*v+F(47)/53*y^2*u^2*v+10118*x*z*u^2*v+14489*y*z*u^2*v-F(59)/55*z^2*u^2*v+F(44)/115*x*u^3*v+F(76)/29*y*u^3*v+F(36)/25*z*u^3*v-123*u^4*v-15773*x^2*z*v^2+12024*x*y*z*v^2+F(106)/81*y^2*z*v^2-2037*x*z^2*v^2+F(114)/97*y*z^2*v^2-F(5)/88*z^3*v^2+F(2)/7*x^2*u*v^2+F(31)/9*x*y*u*v^2+F(52)/73*y^2*u*v^2-4256*x*z*u*v^2+F(95)/78*y*z*u*v^2+13431*z^2*u*v^2-F(121)/70*x*u^2*v^2+F(85)/44*y*u^2*v^2+13578*z*u^2*v^2-F(89)/117*u^3*v^2+F(91)/124*x*z*v^3-F(117)/47*y*z*v^3-52*z^2*v^3+1629*x*u*v^3-F(27)/55*y*u*v^3+F(9)/52*z*u*v^3-F(13)/19*u^2*v^3-14405*z*v^4-7472*u*v^4,-1962*x^4*y+F(76)/69*x^3*y^2+F(96)/17*x^2*y^3+F(112)/89*x*y^4+F(109)/70*x^4*z-157*x^3*y*z+7368*x^2*y^2*z-11887*x*y^3*z+13639*y^4*z+F(88)/89*x^3*z^2+4245*x^2*y*z^2-12618*x*y^2*z^2+F(68)/39*y^3*z^2-11134*x^2*z^3-6559*x*y*z^3-F(1)/63*y^2*z^3+F(44)/9*x*z^4-F(46)/105*y*z^4+1479*z^5+F(17)/117*x^4*u+F(103)/75*x^3*y*u+3540*x^2*y^2*u-F(6)/71*x*y^3*u+F(29)/88*x^3*z*u-F(7)/115*x^2*y*z*u-10025*x*y^2*z*u+F(37)/120*y^3*z*u-F(75)/122*x^2*z^2*u-10288*x*y*z^2*u-13003*y^2*z^2*u-12964*x*z^3*u+F(54)/13*y*z^3*u-7940*z^4*u+F(122)/121*x^3*u^2-F(38)/47*x^2*y*u^2-675*x*y^2*u^2-15549*x^2*z*u^2+14554*x*y*z*u^2-11049*y^2*z*u^2+F(1)/66*x*z^2*u^2-F(45)/38*y*z^2*u^2-10093*z^3*u^2-8383*x^2*u^3+F(49)/51*x*y*u^3+5357*x*z*u^3+6513*y*z*u^3-F(79)/113*z^2*u^3-10950*x*u^4+95*z*u^4-6057*x^4*v-F(109)/76*x^3*y*v-12344*x^2*y^2*v+F(71)/13*x*y^3*v+15084*x^3*z*v-4167*x^2*y*z*v+F(6)/109*x*y^2*z*v-11133*y^3*z*v+10206*x^2*z^2*v-12840*x*y*z^2*v-F(31)/99*y^2*z^2*v-F(32)/79*x*z^3*v+12069*y*z^3*v-7743*z^4*v-14562*x^3*u*v-14367*x^2*y*u*v+F(47)/53*x*y^2*u*v+10094*x^2*z*u*v+F(91)/103*x*y*z*u*v-F(50)/93*y^2*z*u*v+F(33)/47*x*z^2*u*v+F(34)/3*y*z^2*u*v+F(34)/39*z^3*u*v+F(44)/115*x^2*u^2*v+F(76)/29*x*y*u^2*v+F(117)/67*x*z*u^2*v-F(51)/58*y*z*u^2*v-F(68)/45*z^2*u^2*v-123*x*u^3*v+14528*z*u^3*v+F(2)/7*x^3*v^2+F(31)/9*x^2*y*v^2+F(52)/73*x*y^2*v^2-10976*x^2*z*v^2+1956*x*y*z*v^2-9236*y^2*z*v^2-8703*x*z^2*v^2-F(113)/7*y*z^2*v^2-F(39)/67*z^3*v^2-F(121)/70*x^2*u*v^2+F(85)/44*x*y*u*v^2-5034*x*z*u*v^2-F(122)/53*y*z*u*v^2+F(89)/125*z^2*u*v^2-F(89)/117*x*u^2*v^2-F(27)/58*z*u^2*v^2+1629*x^2*v^3-F(27)/55*x*y*v^3+12376*x*z*v^3-F(73)/89*y*z*v^3-1611*z^2*v^3-F(13)/19*x*u*v^3+F(111)/53*z*u*v^3-7472*x*v^4-F(91)/29*z*v^4,1962*x^3*y^2-F(76)/69*x^2*y^3-F(96)/17*x*y^4-F(112)/89*y^5-14170*x^3*y*z-F(3)/70*x^2*y^2*z-2079*x*y^3*z-F(77)/101*y^4*z+6362*x^3*z^2-7398*x^2*y*z^2+F(83)/19*x*y^2*z^2-4058*y^3*z^2+F(125)/28*x^2*z^3+3831*x*y*z^3-466*y^2*z^3+F(75)/34*x*z^4+F(112)/87*y*z^4+F(78)/101*z^5-F(17)/117*x^3*y*u-F(103)/75*x^2*y^2*u-3540*x*y^3*u+F(6)/71*y^4*u+F(13)/25*x^3*z*u+8113*x^2*y*z*u+7132*x*y^2*z*u+F(83)/84*y^3*z*u-F(55)/39*x^2*z^2*u-3570*x*y*z^2*u+F(8)/81*y^2*z^2*u-F(9)/2*x*z^3*u-F(67)/22*y*z^3*u-F(16)/81*z^4*u-F(122)/121*x^2*y*u^2+F(38)/47*x*y^2*u^2+675*y^3*u^2-2363*x^2*z*u^2+F(5)/16*x*y*z*u^2-F(59)/110*y^2*z*u^2-F(115)/73*x*z^2*u^2+F(31)/24*y*z^2*u^2-F(83)/54*z^3*u^2+8383*x*y*u^3-F(49)/51*y^2*u^3+F(3)/14*x*z*u^3+11793*y*z*u^3+F(59)/99*z^2*u^3+10950*y*u^4+F(55)/42*z*u^4+6057*x^3*y*v+F(109)/76*x^2*y^2*v+12344*x*y^3*v-F(71)/13*y^4*v+13244*x^3*z*v+F(123)/11*x^2*y*z*v+F(58)/79*x*y^2*z*v+12377*y^3*z*v-10402*x^2*z^2*v-12318*x*y*z^2*v-F(101)/117*y^2*z^2*v+F(97)/65*x*z^3*v-F(3)/41*y*z^3*v+F(115)/64*z^4*v+14562*x^2*y*u*v+14367*x*y^2*u*v-F(47)/53*y^3*u*v+F(9)/116*x^2*z*u*v-8541*x*y*z*u*v+F(1)/63*y^2*z*u*v-F(15)/88*x*z^2*u*v+F(119)/90*y*z^2*u*v+F(17)/120*z^3*u*v-F(44)/115*x*y*u^2*v-F(76)/29*y^2*u^2*v+F(105)/106*x*z*u^2*v+F(8)/67*y*z*u^2*v-F(83)/27*z^2*u^2*v+123*y*u^3*v-F(49)/32*z*u^3*v-F(2)/7*x^2*y*v^2-F(31)/9*x*y^2*v^2-F(52)/73*y^3*v^2-9506*x^2*z*v^2-F(109)/32*x*y*z*v^2+F(21)/67*y^2*z*v^2+F(109)/21*x*z^2*v^2+F(121)/10*y*z^2*v^2+2219*z^3*v^2+F(121)/70*x*y*u*v^2-F(85)/44*y^2*u*v^2+F(78)/7*x*z*u*v^2+6792*y*z*u*v^2+F(82)/77*z^2*u*v^2+F(89)/117*y*u^2*v^2+F(71)/123*z*u^2*v^2-1629*x*y*v^3+F(27)/55*y^2*v^3+F(63)/86*x*z*v^3-F(125)/113*y*z*v^3-F(76)/123*z^2*v^3+F(13)/19*y*u*v^3+F(76)/89*z*u*v^3+7472*y*v^4-F(101)/12*z*v^4,x^4*y+F(67)/79*x^3*y^2-5289*x^2*y^3-F(68)/73*x*y^4-13639*y^5-6362*x^4*z+14586*x^3*y*z+F(84)/61*x^2*y^2*z-F(15)/94*x*y^3*z-F(68)/39*y^4*z-F(125)/28*x^3*z^2+F(65)/92*x^2*y*z^2+F(106)/41*x*y^2*z^2+F(1)/63*y^3*z^2-F(75)/34*x^2*z^3-7483*x*y*z^3+F(46)/105*y^2*z^3-F(78)/101*x*z^4-1479*y*z^4-F(13)/25*x^4*u-F(80)/29*x^3*y*u-F(83)/93*x^2*y^2*u+F(23)/47*x*y^3*u-F(37)/120*y^4*u+F(55)/39*x^3*z*u-F(64)/45*x^2*y*z*u+F(43)/64*x*y^2*z*u+13003*y^3*z*u+F(9)/2*x^2*z^2*u-8845*x*y*z^2*u-F(54)/13*y^2*z^2*u+F(16)/81*x*z^3*u+7940*y*z^3*u+2363*x^3*u^2+F(100)/49*x^2*y*u^2-F(121)/10*x*y^2*u^2+11049*y^3*u^2+F(115)/73*x^2*z*u^2-F(115)/88*x*y*z*u^2+F(45)/38*y^2*z*u^2+F(83)/54*x*z^2*u^2+10093*y*z^2*u^2-F(3)/14*x^2*u^3-F(18)/97*x*y*u^3-6513*y^2*u^3-F(59)/99*x*z*u^3+F(79)/113*y*z*u^3-F(55)/42*x*u^4-95*y*u^4-13244*x^4*v+8171*x^3*y*v+14290*x^2*y^2*v-15899*x*y^3*v+11133*y^4*v+10402*x^3*z*v-F(65)/106*x^2*y*z*v-F(92)/9*x*y^2*z*v+F(31)/99*y^3*z*v-F(97)/65*x^2*z^2*v+119*x*y*z^2*v-12069*y^2*z^2*v-F(115)/64*x*z^3*v+7743*y*z^3*v-F(9)/116*x^3*u*v-F(4)/103*x^2*y*u*v-12607*x*y^2*u*v+F(50)/93*y^3*u*v+F(15)/88*x^2*z*u*v-F(71)/121*x*y*z*u*v-F(34)/3*y^2*z*u*v-F(17)/120*x*z^2*u*v-F(34)/39*y*z^2*u*v-F(105)/106*x^2*u^2*v-F(125)/67*x*y*u^2*v+F(51)/58*y^2*u^2*v+F(83)/27*x*z*u^2*v+F(68)/45*y*z*u^2*v+F(49)/32*x*u^3*v-14528*y*u^3*v+9506*x^3*v^2-F(35)/2*x^2*y*v^2-8641*x*y^2*v^2+9236*y^3*v^2-F(109)/21*x^2*z*v^2-F(52)/113*x*y*z*v^2+F(113)/7*y^2*z*v^2-2219*x*z^2*v^2+F(39)/67*y*z^2*v^2-F(78)/7*x^2*u*v^2-F(23)/91*x*y*u*v^2+F(122)/53*y^2*u*v^2-F(82)/77*x*z*u*v^2-F(89)/125*y*z*u*v^2-F(71)/123*x*u^2*v^2+F(27)/58*y*u^2*v^2-F(63)/86*x^2*v^3-12658*x*y*v^3+F(73)/89*y^2*v^3+F(76)/123*x*z*v^3+1611*y*z*v^3-F(76)/89*x*u*v^3-F(111)/53*y*u*v^3+F(101)/12*x*v^4+F(91)/29*y*v^4,5755*x^3*y-1464*x^2*y^2-2727*x*y^3+836*y^4+8661*x^3*z+F(71)/33*x^2*y*z+12060*x*y^2*z-F(103)/5*y^3*z-2649*x^2*z^2-3326*x*y*z^2+7262*y^2*z^2-2083*x*z^3+F(50)/23*y*z^3-9304*z^4-F(73)/2*x^3*u+14808*x^2*y*u-670*x*y^2*u-F(12)/107*y^3*u-F(37)/67*x^2*z*u+F(66)/7*x*y*z*u+11976*y^2*z*u-9464*x*z^2*u+1766*y*z^2*u-F(42)/73*z^3*u-15871*x^2*u^2+F(79)/84*x*y*u^2+F(113)/75*y^2*u^2+F(36)/115*x*z*u^2-F(87)/28*y*z*u^2-F(125)/22*z^2*u^2-9755*x*u^3-1399*y*u^3+F(122)/103*z*u^3+3090*u^4+12178*x^3*v-8428*x^2*y*v-F(85)/116*x*y^2*v+8762*y^3*v+11908*x^2*z*v-2648*x*y*z*v-F(112)/59*y^2*z*v+14775*x*z^2*v-F(71)/35*y*z^2*v+F(98)/83*z^3*v-F(99)/25*x^2*u*v+F(77)/71*x*y*u*v-F(83)/18*y^2*u*v-F(53)/8*x*z*u*v+8743*y*z*u*v-15736*z^2*u*v+6056*x*u^2*v+8620*y*u^2*v-F(109)/2*z*u^2*v-2839*u^3*v+9764*x^2*v^2-6059*x*y*v^2+F(95)/97*y^2*v^2-F(64)/115*x*z*v^2+F(11)/97*y*z*v^2+F(20)/17*z^2*v^2+F(103)/8*x*u*v^2+F(69)/41*y*u*v^2-14672*z*u*v^2-12146*u^2*v^2+F(25)/51*x*v^3+F(67)/119*y*v^3+F(6)/119*z*v^3-12024*u*v^3+F(115)/99*v^4,F(92)/17*x^3*y+F(9)/55*x^2*y^2+13407*x*y^3+F(70)/61*y^4-F(32)/109*x^3*z+13951*x^2*y*z+4769*x*y^2*z+14357*y^3*z-F(115)/63*x^2*z^2+F(113)/42*x*y*z^2-F(103)/124*y^2*z^2-F(54)/89*x*z^3-1834*y*z^3+1426*z^4-124*x^3*u+9921*x^2*y*u+123*x*y^2*u-F(17)/108*y^3*u+F(33)/64*x^2*z*u-14710*x*y*z*u+13069*y^2*z*u+F(76)/5*x*z^2*u-F(31)/28*y*z^2*u-F(67)/31*z^3*u-5075*x^2*u^2+F(92)/101*x*y*u^2-F(62)/87*y^2*u^2-F(51)/115*x*z*u^2-7362*y*z*u^2+5457*z^2*u^2+241*x*u^3+F(103)/28*y*u^3+F(96)/95*z*u^3-14813*u^4+F(75)/44*x^3*v+F(14)/23*x^2*y*v+F(115)/82*x*y^2*v+F(20)/123*y^3*v+7514*x^2*z*v-7471*x*y*z*v+F(6)/91*y^2*z*v+3291*x*z^2*v+F(108)/31*y*z^2*v-7228*z^3*v+3436*x^2*u*v-F(105)/82*x*y*u*v+F(79)/39*y^2*u*v-9166*x*z*u*v-F(81)/38*y*z*u*v-9921*z^2*u*v-1357*x*u^2*v-6749*y*u^2*v+F(56)/75*z*u^2*v+6771*u^3*v-F(52)/31*x^2*v^2-F(94)/123*x*y*v^2+F(37)/11*y^2*v^2-F(83)/36*x*z*v^2+F(96)/13*y*z*v^2+F(95)/33*z^2*v^2-F(44)/25*x*u*v^2+F(67)/90*y*u*v^2+F(37)/67*z*u*v^2+F(1)/43*u^2*v^2+F(62)/87*x*v^3-12017*y*v^3+14359*z*v^3-F(70)/29*u*v^3-F(10)/107*v^4,F(61)/68*x^3*y+F(70)/3*x^2*y^2-F(59)/45*x*y^3-F(26)/51*y^4+F(49)/61*x^3*z+7777*x^2*y*z-3975*x*y^2*z-2276*y^3*z-5657*x^2*z^2+F(93)/97*x*y*z^2+3508*y^2*z^2-3698*x*z^3+F(75)/106*y*z^3+F(99)/53*z^4+F(113)/22*x^3*u+9862*x^2*y*u+F(47)/19*x*y^2*u-F(119)/90*y^3*u-F(6)/83*x^2*z*u+F(93)/109*x*y*z*u+F(65)/84*y^2*z*u-F(33)/64*x*z^2*u+F(68)/75*y*z^2*u+F(101)/11*z^3*u-5113*x^2*u^2+F(91)/9*x*y*u^2+F(77)/107*y^2*u^2+F(27)/43*x*z*u^2-F(99)/25*y*z*u^2-11244*z^2*u^2+F(51)/59*x*u^3+F(56)/93*y*u^3+8321*z*u^3-F(67)/119*u^4+14387*x^3*v+F(60)/121*x^2*y*v+F(51)/82*x*y^2*v-F(65)/7*y^3*v+2620*x^2*z*v-600*x*y*z*v-15282*y^2*z*v+F(36)/41*x*z^2*v+F(85)/66*y*z^2*v+F(58)/21*z^3*v+F(30)/29*x^2*u*v+F(55)/47*x*y*u*v+F(59)/39*y^2*u*v+F(1)/2*x*z*u*v-6144*y*z*u*v-F(58)/33*z^2*u*v+F(66)/113*x*u^2*v+F(53)/24*y*u^2*v-F(113)/74*z*u^2*v+F(93)/5*u^3*v+F(11)/42*x^2*v^2+F(16)/41*x*y*v^2+13188*y^2*v^2+F(46)/33*x*z*v^2-F(101)/112*y*z*v^2-F(107)/93*z^2*v^2-F(114)/49*x*u*v^2+F(19)/126*y*u*v^2+2199*z*u*v^2+6612*u^2*v^2+F(107)/49*x*v^3-F(27)/74*y*v^3-13971*z*v^3+7142*u*v^3-1985*v^4,103*x^3*y+116*x^2*y^2+F(93)/35*x*y^3+F(44)/41*y^4-F(35)/31*x^3*z-12294*x^2*y*z+2873*x*y^2*z-F(67)/93*y^3*z-3230*x^2*z^2-F(75)/8*x*y*z^2+1418*y^2*z^2-F(35)/111*x*z^3+F(20)/69*y*z^3-F(1)/18*z^4+247*x^3*u+9027*x^2*y*u-F(107)/51*x*y^2*u-F(106)/67*y^3*u+F(29)/84*x^2*z*u+F(57)/116*x*y*z*u-F(98)/115*y^2*z*u+851*x*z^2*u+F(17)/31*y*z^2*u+7346*z^3*u-F(10)/111*x^2*u^2-F(44)/19*x*y*u^2-361*y^2*u^2+5736*x*z*u^2+F(29)/24*y*z*u^2-F(37)/69*z^2*u^2-F(62)/109*x*u^3+F(77)/81*y*u^3+F(119)/22*z*u^3-F(113)/62*u^4-F(53)/34*x^3*v-F(1)/39*x^2*y*v-F(23)/11*x*y^2*v+F(55)/12*y^3*v+11980*x^2*z*v+F(102)/7*x*y*z*v-F(69)/88*y^2*z*v+F(35)/89*x*z^2*v-15040*y*z^2*v-F(38)/23*z^3*v-F(56)/3*x^2*u*v+1081*x*y*u*v-15310*y^2*u*v-7429*x*z*u*v-F(23)/24*y*z*u*v-123*z^2*u*v+F(57)/83*x*u^2*v+5488*y*u^2*v-F(49)/118*z*u^2*v-F(9)/80*u^3*v-F(26)/17*x^2*v^2-F(92)/21*x*y*v^2+8237*y^2*v^2+F(124)/87*x*z*v^2-147*y*z*v^2+6226*z^2*v^2-F(74)/71*x*u*v^2-12096*y*u*v^2-F(64)/53*z*u*v^2+F(18)/23*u^2*v^2-13786*x*v^3-F(100)/81*y*v^3+F(101)/4*z*v^3-F(35)/101*u*v^3+5637*v^4,-5755*x^4+3597*x^3*y+F(87)/121*x^2*y^2+F(40)/19*x*y^3+F(11)/115*y^4+15532*x^3*z+F(118)/79*x^2*y*z-F(68)/83*x*y^2*z-4180*y^3*z-182*x^2*z^2+245*x*y*z^2+F(19)/20*y^2*z^2-F(116)/17*x*z^3+F(61)/99*y*z^3+F(1)/9*z^4+F(6)/49*x^3*u-F(74)/125*x^2*y*u-15825*x*y^2*u-4826*y^3*u-F(69)/35*x^2*z*u-F(58)/85*x*y*z*u+F(104)/59*y^2*z*u+F(109)/17*x*z^2*u-F(16)/81*y*z^2*u-F(10)/37*z^3*u-7136*x^2*u^2+F(64)/27*x*y*u^2-F(46)/57*y^2*u^2-F(52)/37*x*z*u^2+F(62)/23*y*z*u^2-F(115)/62*z^2*u^2-3464*x*u^3-F(57)/50*y*u^3+1251*z*u^3+7473*u^4+8178*x^3*v+9474*x^2*y*v+10451*x*y^2*v-F(105)/74*y^3*v-F(103)/86*x^2*z*v-F(89)/68*x*y*z*v-F(83)/22*y^2*z*v-F(21)/4*x*z^2*v+14039*y*z^2*v-2715*z^3*v-3833*x^2*u*v+F(13)/76*x*y*u*v-F(19)/100*y^2*u*v+13217*x*z*u*v+3214*y*z*u*v+8640*z^2*u*v-F(76)/101*x*u^2*v+F(52)/125*y*u^2*v-F(82)/27*z*u^2*v-7687*u^3*v-F(94)/103*x^2*v^2+F(75)/47*x*y*v^2+F(47)/42*y^2*v^2-12767*x*z*v^2+9516*y*z*v^2+F(63)/59*z^2*v^2+F(59)/65*x*u*v^2+F(75)/16*y*u*v^2+F(22)/91*z*u*v^2+12101*u^2*v^2-F(45)/41*x*v^3+F(27)/125*y*v^3+F(23)/75*z*v^3+F(43)/75*u*v^3+12729*v^4]))
end


@doc raw"""
    rational_d8_pi6()

Return a smooth rational surface in $\mathbb P^4$ with degree `8` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d8_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(19)/125*x^2*y^2-F(87)/119*x*y^3-F(97)/21*y^4+F(36)/53*x^2*y*z+2069*x*y^2*z-F(59)/50*y^3*z-F(65)/33*x^2*z^2-14322*x*y*z^2+F(79)/60*y^2*z^2-9035*x*z^3-14890*y*z^3+F(87)/47*z^4-F(23)/48*x^2*y*u+F(45)/44*x*y^2*u+1972*y^3*u+F(79)/118*x^2*z*u-5173*x*y*z*u+F(115)/121*y^2*z*u+1239*x*z^2*u-F(115)/17*y*z^2*u-15900*z^3*u-F(78)/95*x^2*u^2+F(67)/101*x*y*u^2-12757*y^2*u^2+12752*x*z*u^2+F(68)/21*y*z*u^2+F(103)/90*z^2*u^2-12917*x*u^3+F(97)/92*y*u^3-F(24)/49*z*u^3-F(13)/79*u^4-F(51)/61*x^2*y*v-3103*x*y^2*v+F(77)/117*y^3*v+F(73)/115*x^2*z*v-F(79)/33*x*y*z*v+F(123)/110*y^2*z*v+11969*x*z^2*v-F(31)/95*y*z^2*v-F(123)/95*z^3*v-F(105)/124*x^2*u*v+12624*x*y*u*v+F(2)/63*y^2*u*v+6579*x*z*u*v+F(13)/62*y*z*u*v+4388*z^2*u*v-12747*x*u^2*v-F(26)/105*y*u^2*v-F(78)/61*z*u^2*v-F(125)/53*u^3*v-F(5)/71*x*y*v^2+F(62)/77*y^2*v^2+F(21)/44*x*z*v^2-9806*y*z*v^2+F(3)/91*z^2*v^2+361*x*u*v^2+568*y*u*v^2+2926*z*u*v^2+F(53)/38*u^2*v^2-14523*y*v^3+2082*z*v^3+F(113)/115*u*v^3,F(108)/73*x^2*y^2+4028*x*y^3+F(38)/43*y^4-1944*x^2*y*z+F(39)/80*x*y^2*z+F(8)/109*y^3*z+F(52)/27*x^2*z^2+F(103)/45*x*y*z^2+5834*y^2*z^2+F(63)/101*x*z^3+F(107)/80*y*z^3+1178*z^4-F(1)/6*x^2*y*u+F(78)/25*x*y^2*u-F(21)/43*y^3*u+F(50)/71*x^2*z*u-14693*x*y*z*u+15074*y^2*z*u+F(9)/103*x*z^2*u-7396*y*z^2*u-14493*z^3*u+F(93)/25*x^2*u^2+F(61)/4*x*y*u^2-11306*y^2*u^2-F(79)/81*x*z*u^2+F(59)/82*y*z*u^2-F(5)/106*z^2*u^2+F(89)/71*x*u^3-F(34)/11*y*u^3+F(15)/103*z*u^3-F(115)/52*u^4-F(54)/65*x^2*y*v+F(67)/16*x*y^2*v-F(7)/68*y^3*v-F(10)/13*x^2*z*v+F(32)/85*x*y*z*v+F(1)/91*y^2*z*v+F(107)/118*x*z^2*v+7594*y*z^2*v-F(98)/103*z^3*v+9919*x^2*u*v-965*x*y*u*v+F(53)/34*y^2*u*v+F(119)/11*x*z*u*v-3400*y*z*u*v-8329*z^2*u*v+F(75)/98*x*u^2*v-24*y*u^2*v+F(55)/87*z*u^2*v-F(82)/71*u^3*v-F(73)/115*x^2*v^2+F(85)/19*x*y*v^2-213*y^2*v^2-7704*x*z*v^2-15347*y*z*v^2+14960*z^2*v^2+15065*x*u*v^2-F(125)/17*y*u*v^2+F(32)/83*z*u*v^2-F(14)/73*u^2*v^2-F(21)/44*x*v^3+F(79)/2*y*v^3-F(61)/32*z*v^3+F(46)/119*u*v^3-2082*v^4,F(9)/20*x^2*y^2+F(113)/71*x*y^3-F(88)/65*y^4+9983*x^2*y*z-6722*x*y^2*z+F(87)/68*y^3*z+1893*x^2*z^2+F(65)/32*x*y*z^2+F(51)/55*y^2*z^2-F(102)/53*x*z^3+F(58)/5*y*z^3-7187*z^4-F(96)/7*x^2*y*u-F(14)/87*x*y^2*u-3532*y^3*u+F(95)/54*x^2*z*u+F(19)/65*x*y*z*u-6728*y^2*z*u+F(31)/121*x*z^2*u+F(73)/106*y*z^2*u-F(91)/5*z^3*u-12928*x^2*u^2+707*x*y*u^2-F(55)/48*y^2*u^2-F(96)/25*x*z*u^2+15869*y*z*u^2-F(20)/107*z^2*u^2-10030*x*u^3-13786*y*u^3-F(122)/9*z*u^3+F(19)/59*u^4-F(7)/52*x^2*y*v+F(101)/74*x*y^2*v+F(83)/6*y^3*v-F(91)/55*x^2*z*v-5266*x*y*z*v+F(85)/61*y^2*z*v+F(126)/95*x*z^2*v+F(56)/51*y*z^2*v+13073*z^3*v-F(50)/21*x^2*u*v-13553*x*y*u*v-F(116)/53*y^2*u*v+F(68)/71*x*z*u*v-F(111)/98*y*z*u*v-11037*z^2*u*v+F(68)/121*x*u^2*v-F(124)/53*y*u^2*v+F(54)/55*z*u^2*v+5862*u^3*v+12318*x^2*v^2-F(119)/29*x*y*v^2+F(101)/17*y^2*v^2-F(51)/40*x*z*v^2-F(82)/33*y*z*v^2-F(30)/41*z^2*v^2-F(29)/52*x*u*v^2+7817*y*u*v^2+8121*z*u*v^2-F(28)/99*u^2*v^2+1125*x*v^3-F(73)/55*y*v^3-14141*z*v^3+8742*u*v^3-1203*v^4,x^2*y^2+11357*x*y^3+295*y^4+144*x^2*y*z-F(31)/54*x*y^2*z+F(89)/119*y^3*z+F(1)/46*x^2*z^2+F(29)/26*x*y*z^2+1384*y^2*z^2+1461*x*z^3+F(113)/91*y*z^3+9494*z^4-F(7)/32*x^2*y*u+12850*x*y^2*u-3626*y^3*u-F(33)/106*x^2*z*u-F(7)/60*x*y*z*u-5935*y^2*z*u-8597*x*z^2*u+5527*y*z^2*u+1708*z^3*u+6182*x^2*u^2-15780*x*y*u^2+4669*y^2*u^2-F(38)/69*x*z*u^2+8412*y*z*u^2+9265*z^2*u^2-5679*x*u^3-F(67)/18*y*u^3-F(34)/67*z*u^3-7178*u^4+F(113)/56*x^2*y*v-3669*x*y^2*v+F(17)/113*y^3*v-F(87)/35*x^2*z*v-4871*x*y*z*v-F(111)/11*y^2*z*v-1131*x*z^2*v-F(72)/13*y*z^2*v+838*z^3*v-F(115)/4*x^2*u*v+3395*x*y*u*v-F(43)/68*y^2*u*v-F(82)/13*x*z*u*v+7042*y*z*u*v-F(88)/119*z^2*u*v+F(100)/19*x*u^2*v+F(24)/11*y*u^2*v+F(89)/3*z*u^2*v+7395*u^3*v-F(119)/109*x^2*v^2+F(1)/104*x*y*v^2+F(18)/25*y^2*v^2+700*x*z*v^2-F(59)/9*y*z*v^2-F(92)/87*z^2*v^2+2486*x*u*v^2-F(67)/103*y*u*v^2+1469*z*u*v^2-F(101)/91*u^2*v^2-F(79)/33*x*v^3+10838*y*v^3+F(81)/4*z*v^3-11843*u*v^3+7204*v^4,F(19)/125*x^3-15698*x^2*y-F(22)/117*x*y^2-F(95)/107*y^3+2027*x^2*z-7750*x*y*z+F(85)/104*y^2*z-15326*x*z^2+F(31)/101*y*z^2+F(67)/81*z^3-7879*x^2*u-F(112)/115*x*y*u+F(124)/81*y^2*u+F(99)/61*x*z*u-7458*y*z*u+F(40)/33*z^2*u-1502*x*u^2+6591*y*u^2-F(7)/73*z*u^2-F(42)/95*u^3+F(93)/83*x^2*v-F(15)/112*x*y*v-F(84)/95*y^2*v+F(35)/36*x*z*v+F(5)/24*y*z*v-12768*z^2*v+13232*x*u*v-F(76)/103*y*u*v-F(79)/52*z*u*v-7217*u^2*v+F(75)/92*x*v^2-F(49)/64*y*v^2+F(17)/14*z*v^2-6109*u*v^2+1695*v^3]))
end


@doc raw"""
    rational_d9_pi6()

Return a smooth rational surface in $\mathbb P^4$ with degree `9` and sectional genus `6`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d9_pi6()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([x^4*u^2+F(25)/29*x^3*y*u^2-F(67)/6*x^2*y^2*u^2+F(22)/97*x*y^3*u^2+F(124)/85*y^4*u^2+3036*x^3*z*u^2-10716*x^2*y*z*u^2-F(116)/63*x*y^2*z*u^2-4217*y^3*z*u^2-F(96)/47*x^2*z^2*u^2-F(94)/95*x*y*z^2*u^2-F(79)/13*y^2*z^2*u^2-F(29)/105*x*z^3*u^2-F(42)/25*y*z^3*u^2-13934*z^4*u^2-F(1)/41*x^3*u^3+F(48)/121*x^2*y*u^3-F(62)/49*x*y^2*u^3+F(52)/123*y^3*u^3+3755*x^2*z*u^3+8834*x*y*z*u^3-14186*y^2*z*u^3-3601*x*z^2*u^3+3314*y*z^2*u^3-3056*z^3*u^3+365*x^2*u^4+F(57)/32*x*y*u^4+12038*y^2*u^4+F(27)/95*x*z*u^4-F(9)/121*y*z*u^4+158*z^2*u^4+11617*x*u^5+F(93)/35*y*u^5-F(67)/117*z*u^5-7786*u^6-F(32)/95*x^4*u*v+F(101)/97*x^3*y*u*v-4603*x^2*y^2*u*v+F(103)/82*x*y^3*u*v-F(41)/74*y^4*u*v-F(73)/44*x^3*z*u*v+12129*x^2*y*z*u*v-F(47)/101*x*y^2*z*u*v+F(123)/29*y^3*z*u*v+188*x^2*z^2*u*v+F(114)/59*x*y*z^2*u*v+F(124)/83*y^2*z^2*u*v+F(37)/115*x*z^3*u*v+13499*y*z^3*u*v-11057*z^4*u*v+F(82)/65*x^3*u^2*v-F(19)/3*x^2*y*u^2*v+F(100)/101*x*y^2*u^2*v+F(61)/51*y^3*u^2*v-F(7)/43*x^2*z*u^2*v-35*x*y*z*u^2*v+F(8)/27*y^2*z*u^2*v+F(62)/119*x*z^2*u^2*v-F(21)/50*y*z^2*u^2*v+4475*z^3*u^2*v-F(48)/89*x^2*u^3*v-F(8)/113*x*y*u^3*v-F(111)/80*y^2*u^3*v-4833*x*z*u^3*v+2267*y*z*u^3*v-F(111)/20*z^2*u^3*v+3660*x*u^4*v-F(97)/117*y*u^4*v-14920*z*u^4*v+F(109)/33*u^5*v-1135*x^4*v^2+F(51)/76*x^3*y*v^2-F(59)/22*x^2*y^2*v^2+F(38)/11*x*y^3*v^2-F(30)/59*y^4*v^2-F(103)/31*x^3*z*v^2+F(19)/67*x^2*y*z*v^2+15730*x*y^2*z*v^2-9919*y^3*z*v^2-F(124)/89*x^2*z^2*v^2-F(29)/103*x*y*z^2*v^2-F(21)/32*y^2*z^2*v^2-F(69)/64*x*z^3*v^2+F(89)/37*y*z^3*v^2-F(79)/36*z^4*v^2-2121*x^3*u*v^2-F(55)/61*x^2*y*u*v^2-F(44)/97*x*y^2*u*v^2-F(45)/64*y^3*u*v^2+F(84)/113*x^2*z*u*v^2+F(26)/41*x*y*z*u*v^2-F(42)/53*y^2*z*u*v^2-F(52)/37*x*z^2*u*v^2-2708*y*z^2*u*v^2+F(41)/2*z^3*u*v^2+3777*x^2*u^2*v^2+F(89)/120*x*y*u^2*v^2+F(4)/21*y^2*u^2*v^2+F(38)/17*x*z*u^2*v^2+F(73)/47*y*z*u^2*v^2-7516*z^2*u^2*v^2-F(77)/31*x*u^3*v^2+F(110)/123*y*u^3*v^2-F(109)/24*z*u^3*v^2+F(30)/43*u^4*v^2+F(6)/17*x^3*v^3+10536*x^2*y*v^3+F(43)/10*x*y^2*v^3+F(13)/16*y^3*v^3-10706*x^2*z*v^3-12428*x*y*z*v^3+13232*y^2*z*v^3-15708*x*z^2*v^3-F(124)/89*y*z^2*v^3+14847*z^3*v^3-11156*x^2*u*v^3-10451*x*y*u*v^3+11899*y^2*u*v^3-F(11)/18*x*z*u*v^3-F(39)/106*y*z*u*v^3-F(84)/61*z^2*u*v^3+1906*x*u^2*v^3+F(36)/97*y*u^2*v^3-8251*z*u^2*v^3+4656*u^3*v^3+13232*x^2*v^4+436*x*y*v^4-F(49)/80*y^2*v^4-F(59)/112*x*z*v^4+F(69)/40*y*z*v^4-4334*z^2*v^4-F(11)/38*x*u*v^4-F(44)/31*y*u*v^4-F(28)/51*z*u*v^4+3184*u^2*v^4-9724*x*v^5+F(115)/106*y*v^5+F(119)/60*z*v^5-43*u*v^5+F(93)/19*v^6,-x^4*z-F(25)/29*x^3*y*z+F(67)/6*x^2*y^2*z-F(22)/97*x*y^3*z-F(124)/85*y^4*z-3036*x^3*z^2+10716*x^2*y*z^2+F(116)/63*x*y^2*z^2+4217*y^3*z^2+F(96)/47*x^2*z^3+F(94)/95*x*y*z^3+F(79)/13*y^2*z^3+F(29)/105*x*z^4+F(42)/25*y*z^4+13934*z^5-5496*x^4*u+10706*x^3*y*u+F(74)/49*x^2*y^2*u-F(39)/4*x*y^3*u+F(23)/13*y^4*u+1742*x^3*z*u-4790*x^2*y*z*u+7727*x*y^2*z*u+F(124)/27*y^3*z*u-F(64)/87*x^2*z^2*u-F(19)/60*x*y*z^2*u-F(97)/86*y^2*z^2*u+6947*x*z^3*u+3360*y*z^3*u-F(19)/15*z^4*u-F(86)/11*x^3*u^2-F(103)/30*x^2*y*u^2+7607*x*y^2*u^2-12467*y^3*u^2-3650*x^2*z*u^2+F(69)/86*x*y*z*u^2-F(114)/53*y^2*z*u^2-15711*x*z^2*u^2-25*y*z^2*u^2-F(92)/93*z^3*u^2+F(125)/109*x^2*u^3-F(63)/125*x*y*u^3-F(56)/37*y^2*u^3-15873*x*z*u^3+F(20)/61*y*z*u^3+F(45)/59*z^2*u^3+7004*x*u^4+F(104)/5*y*u^4+10113*z*u^4+F(100)/37*u^5+F(36)/59*x^4*v+F(105)/83*x^3*y*v-F(119)/15*x^2*y^2*v+15651*x*y^3*v-8096*y^4*v+4891*x^3*z*v+11138*x^2*y*z*v-F(8)/27*x*y^2*z*v+F(38)/83*y^3*z*v-F(53)/48*x^2*z^2*v-F(24)/31*x*y*z^2*v-F(124)/111*y^2*z^2*v-F(7)/75*x*z^3*v-F(53)/114*y*z^3*v-F(82)/57*z^4*v-2436*x^3*u*v-5126*x^2*y*u*v-7429*x*y^2*u*v+F(123)/40*y^3*u*v-5420*x^2*z*u*v-F(124)/29*x*y*z*u*v-8820*y^2*z*u*v-F(73)/33*x*z^2*u*v+8198*y*z^2*u*v-14102*z^3*u*v+3315*x^2*u^2*v+F(28)/45*x*y*u^2*v+F(79)/105*y^2*u^2*v-3158*x*z*u^2*v+F(103)/36*y*z*u^2*v+2411*z^2*u^2*v+3070*x*u^3*v+F(10)/53*y*u^3*v+F(119)/20*z*u^3*v+2802*u^4*v+F(69)/38*x^3*v^2+F(74)/61*x^2*y*v^2+F(40)/37*x*y^2*v^2+F(91)/125*y^3*v^2+F(103)/71*x^2*z*v^2-4723*x*y*z*v^2-F(57)/77*y^2*z*v^2-F(123)/74*x*z^2*v^2+F(23)/56*y*z^2*v^2-1008*z^3*v^2-F(40)/53*x^2*u*v^2-9501*x*y*u*v^2-F(43)/97*y^2*u*v^2-5084*x*z*u*v^2+6643*y*z*u*v^2-F(108)/13*z^2*u*v^2+F(69)/8*x*u^2*v^2+12179*y*u^2*v^2+4677*z*u^2*v^2+66*u^3*v^2-F(22)/105*x^2*v^3+13143*x*y*v^3-F(93)/97*y^2*v^3+F(25)/27*x*z*v^3+F(113)/74*y*z*v^3-10110*z^2*v^3-F(1)/61*x*u*v^3-5832*y*u*v^3-8313*z*u*v^3-F(3)/83*u^2*v^3+F(43)/71*x*v^4+F(23)/91*y*v^4-F(8)/39*z*v^4-6870*u*v^4-F(85)/118*v^5,-x^5-F(25)/29*x^4*y+F(67)/6*x^3*y^2-F(22)/97*x^2*y^3-F(124)/85*x*y^4-F(102)/89*x^4*z+F(78)/59*x^3*y*z-F(67)/2*x^2*y^2*z-F(25)/33*x*y^3*z+947*y^4*z-2335*x^3*z^2-F(35)/57*x^2*y*z^2-98*x*y^2*z^2+8845*y^3*z^2+F(115)/88*x^2*z^3+F(108)/67*x*y*z^3-1742*y^2*z^3-4661*x*z^4+6030*y*z^4+6604*z^5+F(26)/15*x^4*u-F(67)/86*x^3*y*u+1154*x^2*y^2*u+F(56)/85*x*y^3*u-1842*y^4*u+7228*x^3*z*u+F(9)/40*x^2*y*z*u-F(19)/61*x*y^2*z*u+F(120)/79*y^3*z*u-F(87)/79*x^2*z^2*u-F(100)/89*x*y*z^2*u+F(22)/65*y^2*z^2*u+F(123)/61*x*z^3*u-F(38)/47*y*z^3*u+3662*z^4*u-F(87)/124*x^3*u^2-F(13)/92*x^2*y*u^2+F(19)/118*x*y^2*u^2-F(68)/37*y^3*u^2-F(51)/58*x^2*z*u^2+3406*x*y*z*u^2-F(12)/109*y^2*z*u^2+F(69)/122*x*z^2*u^2-F(44)/101*y*z^2*u^2-2013*z^3*u^2-F(59)/57*x^2*u^3+F(74)/13*x*y*u^3-F(67)/82*y^2*u^3-5231*x*z*u^3-3359*y*z*u^3-8440*z^2*u^3+F(15)/52*x*u^4-F(109)/30*y*u^4+15*z*u^4+3652*u^5+7093*x^4*v-F(11)/50*x^3*y*v+F(42)/83*x^2*y^2*v-F(80)/71*x*y^3*v-F(101)/10*y^4*v-F(70)/117*x^3*z*v-F(115)/64*x^2*y*z*v+F(55)/126*x*y^2*z*v-F(41)/78*y^3*z*v-F(99)/104*x^2*z^2*v+F(50)/123*x*y*z^2*v+11541*y^2*z^2*v+F(25)/101*x*z^3*v-8819*y*z^3*v-F(39)/109*z^4*v-F(96)/65*x^3*u*v-F(69)/34*x^2*y*u*v-13047*x*y^2*u*v+5974*y^3*u*v-F(17)/60*x^2*z*u*v-6779*x*y*z*u*v-12068*y^2*z*u*v+F(50)/103*x*z^2*u*v-F(86)/27*y*z^2*u*v-2979*z^3*u*v-F(95)/44*x^2*u^2*v-F(99)/67*x*y*u^2*v-F(91)/30*y^2*u^2*v+1870*x*z*u^2*v+F(98)/109*y*z*u^2*v+13244*z^2*u^2*v+F(39)/125*x*u^3*v+402*y*u^3*v+F(103)/85*z*u^3*v+F(67)/113*u^4*v-3303*x^3*v^2-14747*x^2*y*v^2-F(110)/111*x*y^2*v^2+F(77)/38*y^3*v^2-1118*x^2*z*v^2+F(41)/113*x*y*z*v^2+F(76)/87*y^2*z*v^2-9302*x*z^2*v^2-F(24)/43*y*z^2*v^2+F(7)/34*z^3*v^2-F(32)/33*x^2*u*v^2-F(52)/35*x*y*u*v^2-F(37)/70*y^2*u*v^2+F(37)/80*x*z*u*v^2-F(99)/52*y*z*u*v^2+F(101)/102*z^2*u*v^2+F(95)/23*x*u^2*v^2-53*y*u^2*v^2-F(41)/89*z*u^2*v^2-2894*u^3*v^2-F(33)/17*x^2*v^3-15791*x*y*v^3+1110*y^2*v^3-F(33)/40*x*z*v^3-F(49)/24*y*z*v^3-1785*z^2*v^3+1659*x*u*v^3+F(37)/95*y*u*v^3+F(58)/101*z*u*v^3+F(53)/65*u^2*v^3-F(92)/27*x*v^4+F(56)/11*y*v^4+14720*z*v^4-10831*u*v^4-F(25)/79*v^5,-14477*x^5+F(74)/67*x^4*y+F(99)/38*x^3*y^2+10832*x^2*y^3-F(44)/23*x*y^4+F(124)/85*y^5-F(109)/79*x^4*z-F(31)/123*x^3*y*z+F(26)/89*x^2*y^2*z-F(28)/57*x*y^3*z+F(48)/67*y^4*z+4592*x^3*z^2-F(12)/61*x^2*y*z^2+13130*x*y^2*z^2+F(87)/13*y^3*z^2+10074*x^2*z^3-3021*x*y*z^3-F(18)/125*y^2*z^3+F(22)/39*x*z^4-F(82)/3*y*z^4-F(35)/109*z^5-10750*x^4*u+F(98)/55*x^3*y*u+F(86)/75*x^2*y^2*u-13512*x*y^3*u-F(125)/38*y^4*u+F(40)/109*x^3*z*u+F(7)/88*x^2*y*z*u-F(125)/28*x*y^2*z*u-F(87)/71*y^3*z*u-F(23)/81*x^2*z^2*u+F(9)/35*x*y*z^2*u-F(99)/31*y^2*z^2*u+F(55)/119*x*z^3*u+F(102)/109*y*z^3*u-F(38)/11*z^4*u-15044*x^3*u^2+F(37)/48*x^2*y*u^2+F(95)/37*x*y^2*u^2-F(64)/125*y^3*u^2+555*x^2*z*u^2-F(97)/99*x*y*z*u^2+F(4)/43*y^2*z*u^2+F(91)/118*x*z^2*u^2+F(45)/38*y*z^2*u^2+F(3)/32*z^3*u^2+12286*x^2*u^3+4401*x*y*u^3-2984*y^2*u^3-F(116)/59*x*z*u^3+846*y*z*u^3-F(48)/17*z^2*u^3-F(13)/118*x*u^4+F(81)/50*y*u^4-F(65)/114*z*u^4+4354*u^5+F(42)/13*x^4*v+F(121)/52*x^3*y*v-F(50)/101*x^2*y^2*v-F(45)/73*x*y^3*v+4589*y^4*v+F(123)/79*x^3*z*v+7411*x^2*y*z*v+F(49)/15*x*y^2*z*v-F(21)/41*y^3*z*v-F(89)/45*x^2*z^2*v+F(11)/41*x*y*z^2*v+F(95)/13*y^2*z^2*v-F(83)/78*x*z^3*v-F(34)/123*y*z^3*v+2443*z^4*v+F(55)/9*x^3*u*v+F(84)/97*x^2*y*u*v+6781*x*y^2*u*v-F(44)/89*y^3*u*v+F(57)/89*x^2*z*u*v+11945*x*y*z*u*v-F(51)/37*y^2*z*u*v-11808*x*z^2*u*v-10895*y*z^2*u*v-F(62)/83*z^3*u*v-11966*x^2*u^2*v-5804*x*y*u^2*v+8287*y^2*u^2*v+F(67)/92*x*z*u^2*v-14672*y*z*u^2*v+F(8)/63*z^2*u^2*v+5673*x*u^3*v-5521*y*u^3*v-F(70)/57*z*u^3*v-2803*u^4*v-F(79)/98*x^3*v^2-3275*x^2*y*v^2+F(77)/101*x*y^2*v^2+F(50)/73*y^3*v^2+F(12)/43*x^2*z*v^2+F(120)/89*x*y*z*v^2-F(1)/49*y^2*z*v^2+F(114)/115*x*z^2*v^2+F(22)/119*y*z^2*v^2-F(121)/115*z^3*v^2-F(79)/33*x^2*u*v^2+3224*x*y*u*v^2-1708*y^2*u*v^2+F(74)/57*x*z*u*v^2+1212*y*z*u*v^2+14366*z^2*u*v^2-F(103)/27*x*u^2*v^2-F(53)/49*y*u^2*v^2+F(46)/27*z*u^2*v^2-5247*u^3*v^2-F(21)/107*x^2*v^3+F(112)/85*x*y*v^3-1111*y^2*v^3+F(83)/9*x*z*v^3-2972*y*z*v^3+F(115)/59*z^2*v^3+F(61)/59*x*u*v^3+3463*y*u*v^3+F(60)/101*z*u*v^3+F(39)/55*u^2*v^3+7892*x*v^4+F(116)/115*y*v^4-F(21)/50*z*v^4-5365*u*v^4-F(113)/32*v^5,-F(97)/36*x^5+5574*x^4*y-3882*x^3*y^2-1265*x^2*y^3+14383*x*y^4-F(75)/97*y^5+F(107)/69*x^4*z-F(42)/61*x^3*y*z+7452*x^2*y^2*z-216*x*y^3*z-F(55)/13*y^4*z-F(89)/100*x^3*z^2+F(55)/58*x^2*y*z^2-12178*x*y^2*z^2+F(57)/100*y^3*z^2-F(51)/125*x^2*z^3+168*x*y*z^3-F(106)/95*y^2*z^3+F(9)/109*x*z^4-2599*y*z^4-F(100)/121*z^5+1787*x^4*u+11165*x^3*y*u-F(87)/88*x^2*y^2*u+F(97)/93*x*y^3*u+F(54)/79*y^4*u-F(48)/83*x^3*z*u+F(37)/115*x^2*y*z*u-F(109)/95*x*y^2*z*u+F(40)/123*y^3*z*u+3618*x^2*z^2*u-1608*x*y*z^2*u+F(88)/3*y^2*z^2*u-F(17)/97*x*z^3*u-F(97)/20*y*z^3*u+F(7)/48*z^4*u+F(38)/3*x^3*u^2+1607*x^2*y*u^2+F(11)/47*x*y^2*u^2-8052*y^3*u^2+8049*x^2*z*u^2-F(109)/95*x*y*z*u^2+F(82)/23*y^2*z*u^2+F(97)/125*x*z^2*u^2-F(91)/73*y*z^2*u^2+1868*z^3*u^2+14100*x^2*u^3+F(117)/16*x*y*u^3-F(39)/71*y^2*u^3+F(23)/81*x*z*u^3+214*y*z*u^3-3219*z^2*u^3+13684*x*u^4-9387*y*u^4+F(91)/89*z*u^4+14199*u^5+2650*x^4*v-F(122)/117*x^3*y*v+F(31)/9*x^2*y^2*v-F(38)/55*x*y^3*v-9553*y^4*v-F(51)/118*x^3*z*v-F(45)/38*x^2*y*z*v+F(86)/17*x*y^2*z*v+F(57)/65*y^3*z*v-F(83)/25*x^2*z^2*v-F(65)/43*x*y*z^2*v-F(88)/47*y^2*z^2*v+F(125)/123*x*z^3*v+F(57)/29*y*z^3*v-F(70)/9*z^4*v-11798*x^3*u*v-11080*x^2*y*u*v+F(58)/115*x*y^2*u*v-F(26)/31*y^3*u*v+7902*x^2*z*u*v+F(24)/115*x*y*z*u*v+F(115)/17*y^2*z*u*v-F(31)/12*x*z^2*u*v-F(67)/6*y*z^2*u*v-F(125)/64*z^3*u*v-F(86)/25*x^2*u^2*v+8230*x*y*u^2*v+1751*y^2*u^2*v-10230*x*z*u^2*v-F(118)/123*y*z*u^2*v+1213*z^2*u^2*v-15043*x*u^3*v-F(52)/59*y*u^3*v-5084*z*u^3*v+13432*u^4*v-F(46)/59*x^3*v^2-F(62)/57*x^2*y*v^2+F(33)/67*x*y^2*v^2+F(119)/88*y^3*v^2-F(103)/31*x^2*z*v^2+F(88)/93*x*y*z*v^2-F(90)/53*y^2*z*v^2+F(79)/62*x*z^2*v^2-F(81)/121*y*z^2*v^2-2396*z^3*v^2+F(67)/71*x^2*u*v^2-F(123)/86*x*y*u*v^2+9449*y^2*u*v^2+8321*x*z*u*v^2+F(89)/52*y*z*u*v^2+9061*z^2*u*v^2+139*x*u^2*v^2-F(126)/13*y*u^2*v^2+F(16)/13*z*u^2*v^2+F(109)/25*u^3*v^2+11204*x^2*v^3-F(10)/99*x*y*v^3+5383*y^2*v^3-2010*x*z*v^3-F(61)/77*y*z*v^3-4898*z^2*v^3+335*x*u*v^3-9495*y*u*v^3-6650*z*u*v^3-F(115)/27*u^2*v^3-F(53)/25*x*v^4+F(27)/106*y*v^4+10049*z*v^4-F(71)/56*u*v^4+15308*v^5,F(89)/124*x^5-F(92)/37*x^4*y-497*x^3*y^2+F(71)/42*x^2*y^3-10604*x*y^4+12922*y^5+6010*x^4*z-2211*x^3*y*z+13688*x^2*y^2*z-14237*x*y^3*z+F(25)/63*y^4*z-F(35)/19*x^3*z^2-F(29)/93*x^2*y*z^2-F(1)/18*x*y^2*z^2+F(27)/76*y^3*z^2+6162*x^2*z^3-8992*x*y*z^3+F(59)/12*y^2*z^3-5496*x*z^4-13858*y*z^4+F(80)/3*z^5-5124*x^4*u+11040*x^3*y*u+7500*x^2*y^2*u-11118*x*y^3*u+F(12)/115*y^4*u-F(62)/123*x^3*z*u+F(49)/74*x^2*y*z*u+11358*x*y^2*z*u+9050*y^3*z*u-9282*x^2*z^2*u+11682*x*y*z^2*u+F(67)/80*y^2*z^2*u-8057*x*z^3*u+F(5)/16*y*z^3*u-F(40)/11*z^4*u-7261*x^3*u^2-11614*x^2*y*u^2+F(55)/17*x*y^2*u^2+F(33)/91*y^3*u^2-14681*x^2*z*u^2-10412*x*y*z*u^2+8545*y^2*z*u^2+F(101)/8*x*z^2*u^2+F(29)/63*y*z^2*u^2+2019*z^3*u^2+F(97)/104*x^2*u^3-F(9)/20*x*y*u^3-14518*y^2*u^3-8876*x*z*u^3+F(15)/94*y*z*u^3-F(58)/73*z^2*u^3+F(35)/4*x*u^4+1592*y*u^4-10743*z*u^4+14954*u^5-F(17)/93*x^4*v+F(25)/81*x^3*y*v-F(77)/73*x^2*y^2*v-F(52)/113*x*y^3*v+F(71)/64*y^4*v-13286*x^3*z*v+5420*x^2*y*z*v-F(37)/97*x*y^2*z*v+F(47)/110*y^3*z*v+F(77)/83*x^2*z^2*v+F(63)/79*x*y*z^2*v+4698*y^2*z^2*v+F(61)/11*x*z^3*v+2767*y*z^3*v+5029*z^4*v-4058*x^3*u*v+F(5)/31*x^2*y*u*v+3340*x*y^2*u*v+F(66)/25*y^3*u*v-F(53)/67*x^2*z*u*v+F(101)/87*x*y*z*u*v+F(86)/63*y^2*z*u*v+4087*x*z^2*u*v-11367*y*z^2*u*v+9989*z^3*u*v-F(125)/14*x^2*u^2*v-12770*x*y*u^2*v+F(3)/46*y^2*u^2*v-F(46)/87*x*z*u^2*v+15212*y*z*u^2*v+F(77)/94*z^2*u^2*v-7444*x*u^3*v+4961*y*u^3*v+F(109)/64*z*u^3*v-F(113)/96*u^4*v+8052*x^3*v^2+F(67)/98*x^2*y*v^2-F(31)/38*x*y^2*v^2+14246*y^3*v^2+8711*x^2*z*v^2+F(118)/31*x*y*z*v^2+12504*y^2*z*v^2-15895*x*z^2*v^2+F(80)/43*y*z^2*v^2-F(39)/17*z^3*v^2+F(11)/43*x^2*u*v^2+F(89)/74*x*y*u*v^2+10058*y^2*u*v^2+12455*x*z*u*v^2+F(59)/55*y*z*u*v^2-F(93)/125*z^2*u*v^2-F(73)/36*x*u^2*v^2+F(31)/117*y*u^2*v^2-F(21)/83*z*u^2*v^2+F(125)/41*u^3*v^2+F(44)/49*x^2*v^3-3756*x*y*v^3+F(50)/43*y^2*v^3-15903*x*z*v^3+F(65)/93*y*z*v^3+5181*z^2*v^3-7872*x*u*v^3+14918*y*u*v^3+14654*z*u*v^3-F(76)/55*u^2*v^3-F(111)/16*x*v^4+F(125)/99*y*v^4-F(13)/18*z*v^4+F(9)/73*u*v^4+F(22)/69*v^5,6039*x^5+F(85)/94*x^4*y-12499*x^3*y^2-F(41)/22*x^2*y^3-9559*x*y^4-F(123)/23*y^5-F(27)/35*x^4*z-F(55)/21*x^3*y*z+F(32)/55*x^2*y^2*z+F(68)/47*x*y^3*z+F(6)/89*y^4*z+F(85)/36*x^3*z^2+F(113)/22*x^2*y*z^2+F(28)/69*x*y^2*z^2+F(43)/114*y^3*z^2-13740*x^2*z^3+F(6)/79*x*y*z^3+F(61)/101*y^2*z^3-F(39)/20*x*z^4+7556*y*z^4+F(1)/71*z^5-F(11)/123*x^4*u+F(9)/20*x^3*y*u-F(29)/106*x^2*y^2*u-F(68)/27*x*y^3*u+F(11)/124*y^4*u+F(81)/73*x^3*z*u+13600*x^2*y*z*u+F(118)/19*x*y^2*z*u-F(13)/8*y^3*z*u+F(91)/9*x^2*z^2*u+F(68)/3*x*y*z^2*u-F(51)/80*y^2*z^2*u-12861*x*z^3*u-F(33)/112*y*z^3*u-F(82)/3*z^4*u+5961*x^3*u^2+12912*x^2*y*u^2+15924*x*y^2*u^2-F(125)/27*y^3*u^2+1984*x^2*z*u^2-15429*x*y*z*u^2-13917*y^2*z*u^2-F(17)/84*x*z^2*u^2-F(4)/61*y*z^2*u^2-F(117)/49*z^3*u^2-13603*x^2*u^3+F(52)/119*x*y*u^3-189*y^2*u^3-F(16)/3*x*z*u^3+F(117)/77*y*z*u^3-F(123)/97*z^2*u^3+F(7)/69*x*u^4-F(79)/93*y*u^4+4142*z*u^4-F(115)/18*u^5+723*x^4*v-F(1)/74*x^3*y*v-14175*x^2*y^2*v+F(59)/7*x*y^3*v-F(100)/33*y^4*v+F(16)/41*x^3*z*v+11871*x^2*y*z*v-F(121)/24*x*y^2*z*v-5465*y^3*z*v+F(39)/107*x^2*z^2*v+4543*x*y*z^2*v+596*y^2*z^2*v+F(11)/118*x*z^3*v+7963*y*z^3*v+F(95)/101*z^4*v+993*x^3*u*v-3711*x^2*y*u*v-3070*x*y^2*u*v-F(69)/70*y^3*u*v-201*x^2*z*u*v+F(80)/117*x*y*z*u*v-F(3)/77*y^2*z*u*v-F(82)/9*x*z^2*u*v-13396*y*z^2*u*v-1443*z^3*u*v-F(97)/18*x^2*u^2*v+F(70)/99*x*y*u^2*v+F(97)/7*y^2*u^2*v+93*x*z*u^2*v-9885*y*z*u^2*v+F(116)/15*z^2*u^2*v-F(17)/107*x*u^3*v-14163*y*u^3*v+F(49)/111*z*u^3*v+F(61)/43*u^4*v-312*x^3*v^2+F(27)/98*x^2*y*v^2+7557*x*y^2*v^2-10613*y^3*v^2-F(77)/82*x^2*z*v^2-F(84)/113*x*y*z*v^2-3631*y^2*z*v^2+F(103)/63*x*z^2*v^2+F(34)/21*y*z^2*v^2+15361*z^3*v^2+7177*x^2*u*v^2+F(43)/56*x*y*u*v^2-F(119)/114*y^2*u*v^2-F(65)/114*x*z*u*v^2-14944*y*z*u*v^2+9102*z^2*u*v^2+F(115)/8*x*u^2*v^2+3277*y*u^2*v^2-418*z*u^2*v^2+F(25)/96*u^3*v^2-F(102)/97*x^2*v^3-1807*x*y*v^3+13041*y^2*v^3-F(4)/67*x*z*v^3-F(17)/62*y*z*v^3-F(60)/17*z^2*v^3+F(97)/122*x*u*v^3+F(59)/126*y*u*v^3-F(102)/67*z*u*v^3+F(28)/5*u^2*v^3-F(104)/95*x*v^4+F(41)/83*y*v^4-F(107)/36*z*v^4-2249*u*v^4-F(29)/81*v^5,1135*x^4*z-F(51)/76*x^3*y*z+F(59)/22*x^2*y^2*z-F(38)/11*x*y^3*z+F(30)/59*y^4*z+F(103)/31*x^3*z^2-F(19)/67*x^2*y*z^2-15730*x*y^2*z^2+9919*y^3*z^2+F(124)/89*x^2*z^3+F(29)/103*x*y*z^3+F(21)/32*y^2*z^3+F(69)/64*x*z^4-F(89)/37*y*z^4+F(79)/36*z^5+4111*x^4*u-9367*x^3*y*u+F(85)/73*x^2*y^2*u-F(69)/14*x*y^3*u-11347*y^4*u-9515*x^3*z*u-F(16)/75*x^2*y*z*u+F(18)/109*x*y^2*z*u+6793*y^3*z*u-8473*x^2*z^2*u+F(106)/87*x*y*z^2*u+5037*y^2*z^2*u-F(28)/53*x*z^3*u+F(15)/88*y*z^3*u-F(11)/100*z^4*u-F(45)/91*x^3*u^2-15489*x^2*y*u^2+F(11)/20*x*y^2*u^2-F(55)/102*y^3*u^2-F(110)/27*x^2*z*u^2-F(56)/13*x*y*z*u^2+F(113)/76*y^2*z*u^2-6297*x*z^2*u^2-F(57)/71*y*z^2*u^2+5873*z^3*u^2+4116*x^2*u^3-14141*x*y*u^3-4098*y^2*u^3-F(25)/99*x*z*u^3-F(29)/119*y*z*u^3+4027*z^2*u^3-F(79)/87*x*u^4+12526*y*u^4+2924*z*u^4+F(47)/105*u^5+2376*x^4*v-12197*x^3*y*v+6432*x^2*y^2*v+4728*x*y^3*v+3756*y^4*v+F(9)/34*x^3*z*v+F(109)/7*x^2*y*z*v-F(115)/44*x*y^2*z*v-F(73)/68*y^3*z*v-F(7)/60*x^2*z^2*v+F(39)/112*x*y*z^2*v-F(39)/118*y^2*z^2*v+9076*x*z^3*v+F(45)/68*y*z^3*v+F(37)/108*z^4*v-11409*x^3*u*v-11067*x^2*y*u*v+F(119)/43*x*y^2*u*v+12864*y^3*u*v-8121*x^2*z*u*v+F(19)/124*x*y*z*u*v-5787*y^2*z*u*v-5465*x*z^2*u*v-5903*y*z^2*u*v+F(39)/44*z^3*u*v-5679*x^2*u^2*v-F(116)/51*x*y*u^2*v-F(105)/79*y^2*u^2*v+10737*x*z*u^2*v+F(53)/108*y*z*u^2*v+1989*z^2*u^2*v-F(97)/94*x*u^3*v-F(15)/91*y*u^3*v+F(103)/43*z*u^3*v-F(52)/15*u^4*v-F(47)/12*x^3*v^2-F(71)/84*x^2*y*v^2+F(33)/17*x*y^2*v^2-F(26)/123*y^3*v^2-F(109)/93*x^2*z*v^2-F(47)/62*x*y*z*v^2-6033*y^2*z*v^2+6839*x*z^2*v^2+13552*y*z^2*v^2-15134*z^3*v^2-15254*x^2*u*v^2+F(19)/43*x*y*u*v^2-F(125)/86*y^2*u*v^2+F(121)/119*x*z*u*v^2-F(13)/98*y*z*u*v^2-F(38)/121*z^2*u*v^2-6605*x*u^2*v^2+14559*y*u^2*v^2+F(105)/17*z*u^2*v^2+F(53)/87*u^3*v^2+9562*x^2*v^3-F(118)/79*x*y*v^3+2445*y^2*v^3-F(17)/47*x*z*v^3-14198*y*z*v^3-F(30)/23*z^2*v^3+4540*x*u*v^3+F(55)/51*y*u*v^3+F(49)/114*z*u*v^3+F(48)/91*u^2*v^3+F(68)/21*x*v^4-F(18)/29*y*v^4+10118*z*v^4-F(14)/125*u*v^4+3243*v^5,-1135*x^4*y+F(51)/76*x^3*y^2-F(59)/22*x^2*y^3+F(38)/11*x*y^4-F(30)/59*y^5+12332*x^4*z-13901*x^3*y*z-F(117)/61*x^2*y^2*z+F(122)/117*x*y^3*z-765*y^4*z-F(72)/65*x^3*z^2-F(75)/59*x^2*y*z^2-9354*x*y^2*z^2+11401*y^3*z^2+F(52)/79*x^2*z^3+F(24)/113*x*y*z^3+F(22)/57*y^2*z^3+13511*x*z^4+5636*y*z^4+11840*z^5+10012*x^4*u-57*x^3*y*u+F(6)/85*x^2*y^2*u+15870*x*y^3*u+F(25)/73*y^4*u-10609*x^3*z*u-1074*x^2*y*z*u-F(97)/50*x*y^2*z*u+1799*y^3*z*u+1225*x^2*z^2*u+F(116)/105*x*y*z^2*u-F(13)/87*y^2*z^2*u+F(15)/28*x*z^3*u+11889*y*z^3*u+F(118)/11*z^4*u+F(49)/2*x^3*u^2+F(80)/87*x^2*y*u^2+6161*x*y^2*u^2+F(95)/6*y^3*u^2-6913*x^2*z*u^2+F(73)/79*x*y*z*u^2+F(28)/69*y^2*z*u^2-F(77)/118*x*z^2*u^2+805*y*z^2*u^2-F(65)/33*z^3*u^2+F(45)/118*x^2*u^3-F(58)/35*x*y*u^3-F(94)/105*y^2*u^3-F(83)/98*x*z*u^3-F(69)/34*y*z*u^3+12478*z^2*u^3+F(51)/58*x*u^4+F(56)/81*y*u^4-F(50)/17*z*u^4+F(83)/36*u^5+13829*x^4*v+13869*x^3*y*v-F(17)/46*x^2*y^2*v+F(110)/113*x*y^3*v-F(15)/76*y^4*v-F(47)/120*x^3*z*v-F(48)/13*x^2*y*z*v-F(78)/107*x*y^2*z*v-52*y^3*z*v+F(32)/39*x^2*z^2*v-173*x*y*z^2*v+F(15)/73*y^2*z^2*v+F(53)/54*x*z^3*v+F(125)/68*y*z^3*v+F(121)/39*z^4*v+13796*x^3*u*v+2101*x^2*y*u*v-12963*x*y^2*u*v-4405*y^3*u*v+F(10)/81*x^2*z*u*v+F(13)/15*x*y*z*u*v-12706*y^2*z*u*v-F(2)/15*x*z^2*u*v-F(53)/109*y*z^2*u*v-F(95)/126*z^3*u*v-F(33)/50*x^2*u^2*v+5275*x*y*u^2*v+F(7)/4*y^2*u^2*v-F(46)/63*x*z*u^2*v+F(91)/58*y*z*u^2*v+12917*z^2*u^2*v-F(113)/87*x*u^3*v+6297*y*u^3*v-6964*z*u^3*v+F(74)/107*u^4*v+6311*x^3*v^2-F(119)/9*x^2*y*v^2-F(32)/77*x*y^2*v^2+F(6)/43*y^3*v^2+12888*x^2*z*v^2-9857*x*y*z*v^2+F(93)/86*y^2*z*v^2+F(101)/104*x*z^2*v^2-12140*y*z^2*v^2+F(22)/21*z^3*v^2-10927*x^2*u*v^2-F(37)/97*x*y*u*v^2-6837*y^2*u*v^2-F(14)/61*x*z*u*v^2-F(19)/22*y*z*u*v^2+F(6)/61*z^2*u*v^2+7306*x*u^2*v^2+F(107)/118*y*u^2*v^2+F(22)/35*z*u^2*v^2+F(112)/47*u^3*v^2-F(3)/29*x^2*v^3+1988*x*y*v^3+14990*y^2*v^3+F(59)/27*x*z*v^3-F(25)/86*y*z*v^3-11842*z^2*v^3-F(16)/53*x*u*v^3+15618*y*u*v^3-F(73)/107*z*u*v^3+F(87)/55*u^2*v^3+F(118)/109*x*v^4-6691*y*v^4+F(35)/9*z*v^4-F(57)/118*u*v^4-10975*v^5,1135*x^5-F(51)/76*x^4*y+F(59)/22*x^3*y^2-F(38)/11*x^2*y^3+F(30)/59*x*y^4+F(93)/116*x^4*z-11359*x^3*y*z+6102*x^2*y^2*z+7823*x*y^3*z-F(112)/121*y^4*z+F(64)/3*x^3*z^2+F(101)/94*x^2*y*z^2+F(89)/125*x*y^2*z^2-F(40)/43*y^3*z^2+F(42)/71*x^2*z^3+14586*x*y*z^3+14289*y^2*z^3-6102*x*z^4-F(120)/37*y*z^4+F(37)/53*z^5-F(59)/119*x^4*u-8517*x^3*y*u+5213*x^2*y^2*u-12917*x*y^3*u+3589*y^4*u+F(85)/37*x^3*z*u-F(8)/115*x^2*y*z*u-13793*x*y^2*z*u+F(2)/31*y^3*z*u+F(93)/110*x^2*z^2*u+F(53)/82*x*y*z^2*u-13351*y^2*z^2*u+7029*x*z^3*u+F(117)/83*y*z^3*u+F(85)/101*z^4*u+10160*x^3*u^2+4600*x^2*y*u^2+F(63)/101*x*y^2*u^2+F(92)/37*y^3*u^2+F(111)/73*x^2*z*u^2-8673*x*y*z*u^2-884*y^2*z*u^2+12388*x*z^2*u^2+10607*y*z^2*u^2-F(125)/107*z^3*u^2-F(11)/62*x^2*u^3-F(71)/88*x*y*u^3-F(44)/97*y^2*u^3+9045*x*z*u^3-2363*y*z*u^3+13069*z^2*u^3+15869*x*u^4-F(38)/111*y*u^4-F(32)/71*z*u^4-F(63)/58*u^5+5247*x^4*v-F(37)/69*x^3*y*v-1236*x^2*y^2*v+F(15)/122*x*y^3*v+2450*y^4*v-F(14)/89*x^3*z*v+10919*x^2*y*z*v+3823*x*y^2*z*v+F(55)/113*y^3*z*v+F(23)/68*x^2*z^2*v-F(55)/112*x*y*z^2*v+F(47)/29*y^2*z^2*v-F(99)/56*x*z^3*v+3538*y*z^3*v+F(64)/21*z^4*v+F(2)/87*x^3*u*v+F(119)/94*x^2*y*u*v+6448*x*y^2*u*v+F(113)/4*y^3*u*v-F(29)/60*x^2*z*u*v-10372*x*y*z*u*v+F(82)/13*y^2*z*u*v+F(91)/16*x*z^2*u*v-F(81)/29*y*z^2*u*v+2476*z^3*u*v+F(43)/12*x^2*u^2*v+F(17)/27*x*y*u^2*v-F(40)/23*y^2*u^2*v+F(124)/49*x*z*u^2*v-51*y*z*u^2*v-4420*z^2*u^2*v-6333*x*u^3*v-F(49)/40*y*u^3*v+F(109)/54*z*u^3*v+F(94)/9*u^4*v+122*x^3*v^2+F(125)/106*x^2*y*v^2-F(118)/95*x*y^2*v^2-F(93)/110*y^3*v^2-F(67)/83*x^2*z*v^2-F(13)/3*x*y*z*v^2+13080*y^2*z*v^2-F(83)/17*x*z^2*v^2+F(101)/89*y*z^2*v^2-2951*z^3*v^2-10153*x^2*u*v^2+F(90)/71*x*y*u*v^2+F(124)/87*y^2*u*v^2-690*x*z*u*v^2-4935*y*z*u*v^2-11941*z^2*u*v^2-F(113)/33*x*u^2*v^2+F(91)/53*y*u^2*v^2-10620*z*u^2*v^2-2251*u^3*v^2-9727*x^2*v^3+13529*x*y*v^3-F(9)/28*y^2*v^3+8307*x*z*v^3-F(75)/46*y*z*v^3+F(31)/20*z^2*v^3+F(17)/40*x*u*v^3-1533*y*u*v^3-F(13)/72*z*u*v^3+10441*u^2*v^3-F(57)/86*x*v^4+708*y*v^4+F(59)/96*z*v^4-F(22)/7*u*v^4+11774*v^5,F(88)/23*x^4*z-F(58)/53*x^3*y*z+6276*x^2*y^2*z-F(36)/107*x*y^3*z+26*y^4*z-F(106)/73*x^3*z^2-631*x^2*y*z^2-7430*x*y^2*z^2-F(52)/35*y^3*z^2+6326*x^2*z^3-F(42)/121*x*y*z^3+F(33)/124*y^2*z^3-9431*x*z^4+10427*y*z^4+F(78)/37*z^5+F(41)/64*x^4*u-11896*x^3*y*u+F(61)/7*x^2*y^2*u+1636*x*y^3*u-2056*y^4*u+12847*x^3*z*u-F(109)/51*x^2*y*z*u+F(87)/38*x*y^2*z*u-F(1)/75*y^3*z*u-F(119)/103*x^2*z^2*u+F(13)/56*x*y*z^2*u+F(47)/9*y^2*z^2*u-4276*x*z^3*u+4068*y*z^3*u+F(97)/26*z^4*u-F(59)/75*x^3*u^2-F(101)/123*x^2*y*u^2-F(90)/101*x*y^2*u^2+F(123)/103*y^3*u^2-1925*x^2*z*u^2-F(91)/61*x*y*z*u^2-F(24)/49*y^2*z*u^2-F(99)/43*x*z^2*u^2+F(114)/83*y*z^2*u^2+F(17)/9*z^3*u^2-F(28)/29*x^2*u^3-4601*x*y*u^3+F(103)/56*y^2*u^3-F(97)/59*x*z*u^3+F(111)/11*y*z*u^3-F(109)/126*z^2*u^3-F(105)/47*x*u^4+F(23)/44*y*u^4-93*z*u^4-4309*u^5+13205*x^4*v-F(85)/31*x^3*y*v-F(84)/47*x^2*y^2*v+F(21)/58*x*y^3*v-1006*y^4*v-F(97)/119*x^3*z*v+F(103)/98*x^2*y*z*v-F(99)/17*x*y^2*z*v-F(105)/4*y^3*z*v-F(90)/31*x^2*z^2*v+F(83)/33*x*y*z^2*v+F(123)/17*y^2*z^2*v+F(67)/103*x*z^3*v-F(77)/26*y*z^3*v+F(111)/47*z^4*v-F(92)/35*x^3*u*v-3406*x^2*y*u*v-F(83)/11*x*y^2*u*v-F(119)/121*y^3*u*v+F(16)/113*x^2*z*u*v+5796*x*y*z*u*v-F(3)/104*y^2*z*u*v-F(83)/21*x*z^2*u*v+F(31)/45*y*z^2*u*v-F(126)/17*z^3*u*v-F(59)/52*x^2*u^2*v+F(49)/17*x*y*u^2*v-F(5)/11*y^2*u^2*v-3538*x*z*u^2*v-4074*y*z*u^2*v-F(37)/122*z^2*u^2*v-F(28)/53*x*u^3*v-F(81)/104*y*u^3*v+F(118)/87*z*u^3*v-2146*u^4*v-F(114)/37*x^3*v^2-F(115)/17*x^2*y*v^2-F(125)/81*x*y^2*v^2-F(97)/54*y^3*v^2+F(20)/87*x^2*z*v^2-F(59)/50*x*y*z*v^2+174*y^2*z*v^2-1400*x*z^2*v^2+F(97)/14*y*z^2*v^2-F(40)/107*z^3*v^2+F(97)/50*x^2*u*v^2+F(103)/6*x*y*u*v^2+12760*y^2*u*v^2-F(103)/80*x*z*u*v^2+13268*y*z*u*v^2+F(13)/51*z^2*u*v^2-F(65)/2*x*u^2*v^2+7937*y*u^2*v^2-F(70)/41*z*u^2*v^2+F(58)/85*u^3*v^2+F(13)/49*x^2*v^3+5492*x*y*v^3+12337*y^2*v^3-F(52)/31*x*z*v^3+F(122)/97*y*z*v^3+F(67)/84*z^2*v^3+F(16)/71*x*u*v^3-F(25)/87*y*u*v^3+F(107)/104*z*u*v^3-F(46)/111*u^2*v^3-F(65)/94*x*v^4+F(66)/95*y*v^4+636*z*v^4-F(115)/72*u*v^4+F(73)/78*v^5,F(94)/55*x^4*z-F(115)/39*x^3*y*z+9120*x^2*y^2*z-F(31)/23*x*y^3*z+F(3)/82*y^4*z+F(85)/116*x^3*z^2-F(67)/12*x^2*y*z^2+9984*x*y^2*z^2+F(110)/63*y^3*z^2-F(59)/79*x^2*z^3+2714*x*y*z^3-F(7)/29*y^2*z^3+F(21)/106*x*z^4+F(5)/109*y*z^4+10896*z^5+2966*x^4*u-F(82)/99*x^3*y*u+F(42)/95*x^2*y^2*u-13903*x*y^3*u-F(81)/104*y^4*u-F(83)/3*x^3*z*u-F(49)/23*x^2*y*z*u-5797*x*y^2*z*u-F(90)/71*y^3*z*u-F(32)/25*x^2*z^2*u-953*x*y*z^2*u+F(21)/107*y^2*z^2*u-13209*x*z^3*u-F(52)/43*y*z^3*u+F(14)/41*z^4*u+F(7)/93*x^3*u^2-F(122)/117*x^2*y*u^2-F(45)/82*x*y^2*u^2-10857*y^3*u^2+3981*x^2*z*u^2-2520*x*y*z*u^2+F(53)/120*y^2*z*u^2-13039*x*z^2*u^2-F(103)/45*y*z^2*u^2-F(45)/73*z^3*u^2+F(47)/86*x^2*u^3+3982*x*y*u^3+6710*y^2*u^3-8816*x*z*u^3-9162*y*z*u^3-7339*z^2*u^3-F(17)/115*x*u^4-F(19)/2*y*u^4+F(47)/42*z*u^4+F(78)/61*u^5+F(2)/115*x^4*v-13787*x^3*y*v-F(82)/37*x^2*y^2*v-F(52)/53*x*y^3*v-F(31)/20*y^4*v+7410*x^3*z*v-F(123)/80*x^2*y*z*v+F(5)/51*x*y^2*z*v+F(79)/52*y^3*z*v-F(57)/50*x^2*z^2*v+F(53)/63*x*y*z^2*v-11205*y^2*z^2*v+15134*x*z^3*v+F(99)/70*y*z^3*v-F(121)/54*z^4*v-14387*x^3*u*v-1445*x^2*y*u*v-208*x*y^2*u*v+F(81)/98*y^3*u*v-F(35)/53*x^2*z*u*v-F(47)/2*x*y*z*u*v+2531*y^2*z*u*v-F(87)/89*x*z^2*u*v+13503*y*z^2*u*v+8276*z^3*u*v-730*x^2*u^2*v-F(66)/119*x*y*u^2*v-13297*y^2*u^2*v-F(26)/89*x*z*u^2*v+F(98)/15*y*z*u^2*v+F(10)/19*z^2*u^2*v-F(109)/56*x*u^3*v-F(61)/37*y*u^3*v+13818*z*u^3*v+8151*u^4*v-11066*x^3*v^2+10393*x^2*y*v^2+F(81)/86*x*y^2*v^2+F(13)/74*y^3*v^2-F(107)/24*x^2*z*v^2-393*x*y*z*v^2-7808*y^2*z*v^2-F(120)/59*x*z^2*v^2+F(113)/120*y*z^2*v^2-1444*z^3*v^2+F(75)/38*x^2*u*v^2-10707*x*y*u*v^2+F(56)/121*y^2*u*v^2-F(71)/77*x*z*u*v^2-F(85)/121*y*z*u*v^2-F(113)/122*z^2*u*v^2-5362*x*u^2*v^2-F(13)/70*y*u^2*v^2+13879*z*u^2*v^2+F(23)/25*u^3*v^2-11827*x^2*v^3+14526*x*y*v^3-F(86)/57*y^2*v^3+F(63)/107*x*z*v^3-F(103)/82*y*z*v^3+F(71)/42*z^2*v^3+F(56)/125*x*u*v^3-14819*y*u*v^3-F(89)/104*z*u*v^3-F(109)/113*u^2*v^3-F(109)/12*x*v^4+2921*y*v^4+F(16)/111*z*v^4-2590*u*v^4+F(64)/39*v^5,8739*x^4*z-F(74)/27*x^3*y*z+F(105)/97*x^2*y^2*z-F(111)/112*x*y^3*z+14876*y^4*z+F(28)/23*x^3*z^2+F(72)/43*x^2*y*z^2-F(119)/66*x*y^2*z^2+13902*y^3*z^2-F(20)/39*x^2*z^3-F(43)/119*x*y*z^3-10830*y^2*z^3+F(55)/47*x*z^4-F(52)/49*y*z^4-F(25)/84*z^5+F(21)/31*x^4*u-7416*x^3*y*u-F(47)/9*x^2*y^2*u+F(116)/103*x*y^3*u+F(35)/26*y^4*u+F(92)/123*x^3*z*u+13838*x^2*y*z*u+F(79)/28*x*y^2*z*u-F(1)/12*y^3*z*u+F(58)/27*x^2*z^2*u+5184*x*y*z^2*u-8879*y^2*z^2*u+8492*x*z^3*u+5908*y*z^3*u+10795*z^4*u+F(9)/67*x^3*u^2+F(30)/31*x^2*y*u^2-F(115)/78*x*y^2*u^2-F(55)/102*y^3*u^2+545*x^2*z*u^2-F(20)/51*x*y*z*u^2+F(42)/85*y^2*z*u^2-5636*x*z^2*u^2-F(118)/31*y*z^2*u^2+F(47)/19*z^3*u^2-F(126)/73*x^2*u^3+F(71)/92*x*y*u^3+F(96)/19*y^2*u^3+6547*x*z*u^3-F(107)/57*y*z*u^3+F(27)/62*z^2*u^3-5358*x*u^4+F(101)/2*y*u^4+15897*z*u^4+F(13)/30*u^5+F(57)/124*x^4*v-F(123)/11*x^3*y*v+13640*x^2*y^2*v-758*x*y^3*v+70*y^4*v+12474*x^3*z*v-F(125)/99*x^2*y*z*v+F(59)/97*x*y^2*z*v+7535*y^3*z*v-3217*x^2*z^2*v+F(86)/99*x*y*z^2*v+F(34)/95*y^2*z^2*v+3144*x*z^3*v+F(1)/109*y*z^3*v+F(72)/53*z^4*v+2754*x^3*u*v+14694*x^2*y*u*v-F(13)/2*x*y^2*u*v-F(59)/5*y^3*u*v-F(57)/74*x^2*z*u*v-1571*x*y*z*u*v+F(6)/89*y^2*z*u*v-F(41)/71*x*z^2*u*v-F(77)/113*y*z^2*u*v-7445*z^3*u*v+F(105)/101*x^2*u^2*v+8605*x*y*u^2*v-817*y^2*u^2*v+8661*x*z*u^2*v+1151*y*z*u^2*v+7706*z^2*u^2*v-F(103)/33*x*u^3*v-F(59)/117*y*u^3*v+F(77)/82*z*u^3*v+15145*u^4*v-F(107)/29*x^3*v^2-F(8)/57*x^2*y*v^2-12210*x*y^2*v^2+F(83)/111*y^3*v^2-F(110)/71*x^2*z*v^2+F(20)/123*x*y*z*v^2+8174*y^2*z*v^2-14084*x*z^2*v^2+4786*y*z^2*v^2-5181*z^3*v^2-F(82)/65*x^2*u*v^2-F(4)/121*x*y*u*v^2-15814*y^2*u*v^2-F(43)/12*x*z*u*v^2+F(28)/111*y*z*u*v^2+13652*z^2*u*v^2+12551*x*u^2*v^2-13481*y*u^2*v^2-F(83)/22*z*u^2*v^2-F(25)/99*u^3*v^2+F(53)/12*x^2*v^3-F(59)/86*x*y*v^3+9770*y^2*v^3+F(79)/6*x*z*v^3+F(8)/29*y*z*v^3+7456*z^2*v^3-1172*x*u*v^3+F(111)/56*y*u*v^3+5588*z*u*v^3-3377*u^2*v^3-F(31)/26*x*v^4+F(123)/83*y*v^4-F(92)/11*z*v^4-F(29)/67*u*v^4-F(81)/5*v^5,-F(64)/21*x^4*z-4890*x^3*y*z-F(44)/107*x^2*y^2*z-F(77)/46*x*y^3*z-F(60)/41*y^4*z-2529*x^3*z^2-11412*x^2*y*z^2+F(29)/36*x*y^2*z^2+F(25)/39*y^3*z^2+3257*x^2*z^3+F(39)/80*x*y*z^3-13194*y^2*z^3-F(63)/92*x*z^4+F(31)/106*y*z^4+F(33)/13*z^5+15920*x^4*u+F(7)/110*x^3*y*u+4184*x^2*y^2*u+8704*x*y^3*u-1660*y^4*u+7174*x^3*z*u-F(9)/122*x^2*y*z*u-F(85)/67*x*y^2*z*u+F(94)/89*y^3*z*u-F(107)/79*x^2*z^2*u+F(5)/13*x*y*z^2*u-F(37)/63*y^2*z^2*u+11645*x*z^3*u+4534*y*z^3*u+F(78)/31*z^4*u+559*x^3*u^2+12499*x^2*y*u^2+1588*x*y^2*u^2-F(3)/88*y^3*u^2-3358*x^2*z*u^2+F(73)/122*x*y*z*u^2+F(49)/16*y^2*z*u^2-F(1)/57*x*z^2*u^2-10405*y*z^2*u^2+F(74)/81*z^3*u^2-F(123)/22*x^2*u^3-F(76)/89*x*y*u^3+F(90)/89*y^2*u^3+10294*x*z*u^3-F(82)/99*y*z*u^3+F(53)/85*z^2*u^3-F(32)/73*x*u^4+F(15)/58*y*u^4-4283*z*u^4+F(108)/43*u^5+F(85)/81*x^4*v-9171*x^3*y*v+11251*x^2*y^2*v+F(73)/83*x*y^3*v-10086*y^4*v+F(97)/13*x^3*z*v-F(95)/32*x^2*y*z*v-F(89)/17*x*y^2*z*v-F(53)/66*y^3*z*v-8046*x^2*z^2*v+4193*x*y*z^2*v+F(117)/125*y^2*z^2*v-F(63)/85*x*z^3*v-F(28)/15*y*z^3*v-F(23)/18*z^4*v+F(1)/86*x^3*u*v-7018*x^2*y*u*v-7035*x*y^2*u*v-F(83)/110*y^3*u*v+9205*x^2*z*u*v-6171*x*y*z*u*v+F(36)/83*y^2*z*u*v-F(81)/121*x*z^2*u*v-F(106)/29*y*z^2*u*v+F(38)/123*z^3*u*v+14858*x^2*u^2*v-F(80)/69*x*y*u^2*v-2442*y^2*u^2*v-F(45)/59*x*z*u^2*v+8238*y*z*u^2*v-8097*z^2*u^2*v+F(19)/56*x*u^3*v+F(40)/121*y*u^3*v+13407*z*u^3*v-F(33)/107*u^4*v-F(79)/92*x^3*v^2+5682*x^2*y*v^2-F(31)/112*x*y^2*v^2-F(35)/93*y^3*v^2-F(46)/101*x^2*z*v^2+9956*x*y*z*v^2-F(68)/121*y^2*z*v^2-F(96)/89*x*z^2*v^2-F(115)/18*y*z^2*v^2-13001*z^3*v^2-F(97)/101*x^2*u*v^2+F(45)/49*x*y*u*v^2-11722*y^2*u*v^2+F(23)/110*x*z*u*v^2-3503*y*z*u*v^2+F(23)/85*z^2*u*v^2+F(18)/29*x*u^2*v^2+8408*y*u^2*v^2+F(11)/71*z*u^2*v^2+F(124)/45*u^3*v^2-F(76)/69*x^2*v^3-F(57)/37*x*y*v^3-11259*y^2*v^3+9170*x*z*v^3+F(25)/4*y*z*v^3+F(71)/79*z^2*v^3+9796*x*u*v^3-F(89)/37*y*u*v^3-F(25)/9*z*u*v^3+10508*u^2*v^3+9191*x*v^4-F(11)/112*y*v^4-6682*z*v^4+F(47)/56*u*v^4-14897*v^5,-8212*x^4*z-2095*x^3*y*z-6277*x^2*y^2*z+8563*x*y^3*z+5861*y^4*z+F(89)/2*x^3*z^2-3054*x^2*y*z^2-F(52)/49*x*y^2*z^2-F(76)/65*y^3*z^2+F(61)/63*x^2*z^3-1151*x*y*z^3-13423*y^2*z^3-F(2)/73*x*z^4-F(101)/116*y*z^4+F(7)/121*z^5+6827*x^4*u+F(69)/19*x^3*y*u-14611*x^2*y^2*u+F(47)/25*x*y^3*u+91*y^4*u+F(114)/121*x^3*z*u-F(115)/22*x^2*y*z*u+F(72)/31*x*y^2*z*u+F(101)/32*y^3*z*u-F(105)/62*x^2*z^2*u+6634*x*y*z^2*u+F(51)/88*y^2*z^2*u+F(21)/59*x*z^3*u-F(5)/108*y*z^3*u+F(73)/50*z^4*u+F(104)/5*x^3*u^2-F(45)/97*x^2*y*u^2+F(101)/62*x*y^2*u^2-9572*y^3*u^2+F(31)/13*x^2*z*u^2+F(30)/29*x*y*z*u^2+F(118)/73*y^2*z*u^2+F(55)/64*x*z^2*u^2-9668*y*z^2*u^2+8263*z^3*u^2+F(48)/79*x^2*u^3+F(29)/117*x*y*u^3-15518*y^2*u^3+F(58)/65*x*z*u^3-15900*y*z*u^3-788*z^2*u^3+F(11)/7*x*u^4-8502*y*u^4+10591*z*u^4-F(92)/109*u^5-14779*x^4*v-F(118)/67*x^3*y*v-F(101)/120*x^2*y^2*v-F(97)/96*x*y^3*v-9910*y^4*v-F(29)/103*x^3*z*v-F(44)/65*x^2*y*z*v-F(2)/119*x*y^2*z*v-F(56)/69*y^3*z*v+F(107)/70*x^2*z^2*v+F(61)/88*x*y*z^2*v+10740*y^2*z^2*v-F(71)/119*x*z^3*v+6820*y*z^3*v-F(39)/64*z^4*v-F(73)/104*x^3*u*v-F(83)/3*x^2*y*u*v-F(101)/84*x*y^2*u*v+8745*y^3*u*v-15871*x^2*z*u*v+12237*x*y*z*u*v-F(121)/90*y^2*z*u*v-F(16)/5*x*z^2*u*v+11686*y*z^2*u*v+F(2)/111*z^3*u*v+F(42)/109*x^2*u^2*v-F(120)/107*x*y*u^2*v-F(103)/53*y^2*u^2*v+F(113)/119*x*z*u^2*v+F(17)/15*y*z*u^2*v-F(9)/20*z^2*u^2*v+12595*x*u^3*v+F(23)/101*y*u^3*v+11946*z*u^3*v+14724*u^4*v-F(79)/15*x^3*v^2-F(48)/17*x^2*y*v^2+F(57)/88*x*y^2*v^2+F(69)/124*y^3*v^2+15746*x^2*z*v^2+F(53)/75*x*y*z*v^2-7249*y^2*z*v^2+15643*x*z^2*v^2-F(7)/117*y*z^2*v^2+F(125)/13*z^3*v^2+F(75)/53*x^2*u*v^2-F(23)/122*x*y*u*v^2-237*y^2*u*v^2+F(115)/8*x*z*u*v^2+11557*y*z*u*v^2+12389*z^2*u*v^2-F(5)/16*x*u^2*v^2+9252*y*u^2*v^2-F(77)/39*z*u^2*v^2+F(37)/68*u^3*v^2-15068*x^2*v^3-F(23)/41*x*y*v^3-F(25)/11*y^2*v^3+F(89)/39*x*z*v^3+F(32)/109*y*z*v^3+F(55)/52*z^2*v^3+F(87)/64*x*u*v^3-F(67)/108*y*u*v^3-1109*z*u*v^3+F(121)/114*u^2*v^3-F(27)/43*x*v^4+F(1)/117*y*v^4-F(119)/108*z*v^4+15731*u*v^4-F(6)/59*v^5,-F(67)/116*x^4*z+2201*x^3*y*z+F(72)/55*x^2*y^2*z+6114*x*y^3*z+F(81)/82*y^4*z-7566*x^3*z^2-8092*x^2*y*z^2+3215*x*y^2*z^2+2680*y^3*z^2-F(84)/23*x^2*z^3-3533*x*y*z^3-F(17)/36*y^2*z^3-F(116)/121*x*z^4+F(23)/125*y*z^4+F(70)/87*z^5+71*x^4*u+F(53)/48*x^3*y*u-6238*x^2*y^2*u+F(13)/51*x*y^3*u+9730*y^4*u+6461*x^3*z*u-F(23)/106*x^2*y*z*u-F(66)/61*x*y^2*z*u+F(39)/80*y^3*z*u+F(95)/21*x^2*z^2*u-15429*x*y*z^2*u-11586*y^2*z^2*u+F(53)/105*x*z^3*u+F(52)/19*y*z^3*u+13880*z^4*u-F(122)/33*x^3*u^2+F(79)/101*x^2*y*u^2-F(49)/40*x*y^2*u^2+F(26)/69*y^3*u^2+F(77)/53*x^2*z*u^2+F(26)/29*x*y*z*u^2+F(66)/113*y^2*z*u^2+13348*x*z^2*u^2+197*y*z^2*u^2+12551*z^3*u^2-14693*x^2*u^3-8354*x*y*u^3+F(121)/90*y^2*u^3+9465*x*z*u^3-F(98)/85*y*z*u^3-9516*z^2*u^3+F(119)/50*x*u^4+213*y*u^4-F(118)/65*z*u^4-F(124)/3*u^5-10932*x^4*v+F(59)/37*x^3*y*v+9074*x^2*y^2*v+F(92)/39*x*y^3*v-F(60)/119*y^4*v-F(29)/47*x^3*z*v-F(8)/79*x^2*y*z*v+F(71)/77*x*y^2*z*v-2949*y^3*z*v+1576*x^2*z^2*v-F(85)/2*x*y*z^2*v+F(37)/33*y^2*z^2*v-2300*x*z^3*v-F(99)/98*y*z^3*v+F(71)/14*z^4*v-F(49)/34*x^3*u*v+12526*x^2*y*u*v-1977*x*y^2*u*v-F(90)/23*y^3*u*v+F(126)/67*x^2*z*u*v-F(77)/96*x*y*z*u*v+F(67)/7*y^2*z*u*v-F(118)/45*x*z^2*u*v-F(122)/19*y*z^2*u*v+F(88)/61*z^3*u*v+F(121)/3*x^2*u^2*v+11721*x*y*u^2*v+F(31)/114*y^2*u^2*v+F(4)/5*x*z*u^2*v+7017*y*z*u^2*v-F(31)/103*z^2*u^2*v-F(119)/94*x*u^3*v-F(123)/31*y*u^3*v-F(89)/108*z*u^3*v-3314*u^4*v-15600*x^3*v^2-2037*x^2*y*v^2-6728*x*y^2*v^2+8830*y^3*v^2+F(25)/18*x^2*z*v^2+F(4)/3*x*y*z*v^2+F(112)/53*y^2*z*v^2+F(76)/39*x*z^2*v^2+F(62)/113*y*z^2*v^2-F(13)/109*z^3*v^2+F(20)/19*x^2*u*v^2+14703*x*y*u*v^2-F(58)/105*y^2*u*v^2-7686*x*z*u*v^2+F(76)/99*y*z*u*v^2+10077*z^2*u*v^2-F(34)/115*x*u^2*v^2-F(23)/22*y*u^2*v^2-F(39)/28*z*u^2*v^2-1237*u^3*v^2+F(57)/100*x^2*v^3-F(40)/13*x*y*v^3+12843*y^2*v^3-11489*x*z*v^3+F(91)/120*y*z*v^3-F(83)/121*z^2*v^3-F(57)/44*x*u*v^3+10294*y*u*v^3+F(33)/94*z*u*v^3+F(5)/36*u^2*v^3+F(119)/2*x*v^4-1151*y*v^4+F(84)/29*z*v^4-4203*u*v^4+F(30)/53*v^5]))
end


@doc raw"""
    rational_d9_pi7()

Return a smooth rational surface in $\mathbb P^4$ with degree `9` and sectional genus `7`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function rational_d9_pi7()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([-F(73)/105*x^4*y+F(43)/51*x^3*y^2+8216*x^2*y^3-F(29)/72*x*y^4-9111*x^4*z+2619*x^3*y*z+F(103)/27*x^2*y^2*z+F(15)/17*x*y^3*z+F(92)/57*x^3*z^2-F(89)/59*x^2*y*z^2-F(52)/105*x*y^2*z^2+F(119)/87*x^2*z^3+F(35)/6*x*y*z^3-15024*x*z^4-F(93)/118*x^4*u+F(11)/101*x^3*y*u+9707*x^2*y^2*u-1571*x*y^3*u-5860*x^3*z*u+11682*x^2*y*z*u+F(32)/91*x*y^2*z*u-F(79)/29*x^2*z^2*u-F(29)/83*x*y*z^2*u-F(113)/114*x*z^3*u+F(97)/109*x^3*u^2-F(66)/7*x^2*y*u^2-F(11)/113*x*y^2*u^2+F(85)/22*x^2*z*u^2-F(99)/8*x*y*z*u^2+14469*x*z^2*u^2+13042*x^2*u^3+105*x*y*u^3+15822*x*z*u^3-F(35)/58*x*u^4-6893*x^4*v-1294*x^3*y*v+F(81)/26*x^2*y^2*v-F(17)/71*x*y^3*v+15164*y^4*v+F(11)/9*x^3*z*v+2589*x^2*y*z*v+7666*x*y^2*z*v+11565*y^3*z*v+2552*x^2*z^2*v+12644*x*y*z^2*v-F(59)/31*y^2*z^2*v+F(41)/77*x*z^3*v+F(65)/48*y*z^3*v+F(29)/23*z^4*v-9279*x^3*u*v+F(60)/41*x^2*y*u*v-F(5)/113*x*y^2*u*v+15034*y^3*u*v-F(28)/103*x^2*z*u*v+F(15)/104*x*y*z*u*v+F(125)/23*y^2*z*u*v+10277*x*z^2*u*v+F(125)/4*y*z^2*u*v+F(82)/109*z^3*u*v+F(3)/79*x^2*u^2*v-F(77)/86*x*y*u^2*v+F(30)/113*y^2*u^2*v-F(75)/88*x*z*u^2*v-F(30)/121*y*z*u^2*v+F(107)/66*z^2*u^2*v+F(25)/68*x*u^3*v-F(108)/107*y*u^3*v-4216*z*u^3*v-F(27)/79*u^4*v+F(35)/41*x^3*v^2-F(122)/71*x^2*y*v^2-14120*x*y^2*v^2+9486*y^3*v^2-F(111)/112*x^2*z*v^2+F(13)/89*x*y*z*v^2+7212*y^2*z*v^2-F(103)/57*x*z^2*v^2+10709*y*z^2*v^2-7127*z^3*v^2-12902*x^2*u*v^2-649*x*y*u*v^2-F(84)/83*y^2*u*v^2-9505*x*z*u*v^2-3359*y*z*u*v^2-F(115)/4*z^2*u*v^2-110*x*u^2*v^2+F(29)/16*y*u^2*v^2+F(115)/99*z*u^2*v^2+11119*u^3*v^2+F(43)/61*x^2*v^3+2334*x*y*v^3-F(119)/109*y^2*v^3+F(22)/91*x*z*v^3+F(19)/22*y*z*v^3+4439*z^2*v^3+F(14)/97*x*u*v^3+F(23)/21*y*u*v^3-5062*z*u*v^3+15398*u^2*v^3-F(3)/53*x*v^4-3533*y*v^4-10713*z*v^4+12855*u*v^4-F(112)/31*v^5,F(73)/105*x^3*y^2-F(43)/51*x^2*y^3-8216*x*y^4+F(29)/72*y^5+9111*x^3*y*z-2619*x^2*y^2*z-F(103)/27*x*y^3*z-F(15)/17*y^4*z-F(92)/57*x^2*y*z^2+F(89)/59*x*y^2*z^2+F(52)/105*y^3*z^2-F(119)/87*x*y*z^3-F(35)/6*y^2*z^3+15024*y*z^4+F(93)/118*x^3*y*u-F(11)/101*x^2*y^2*u-9707*x*y^3*u+1571*y^4*u+5860*x^2*y*z*u-11682*x*y^2*z*u-F(32)/91*y^3*z*u+F(79)/29*x*y*z^2*u+F(29)/83*y^2*z^2*u+F(113)/114*y*z^3*u-F(97)/109*x^2*y*u^2+F(66)/7*x*y^2*u^2+F(11)/113*y^3*u^2-F(85)/22*x*y*z*u^2+F(99)/8*y^2*z*u^2-14469*y*z^2*u^2-13042*x*y*u^3-105*y^2*u^3-15822*y*z*u^3+F(35)/58*y*u^4-6118*x^3*y*v-14978*x^2*y^2*v+F(113)/65*x*y^3*v-F(98)/67*y^4*v+11940*x^3*z*v+11175*x^2*y*z*v-F(71)/91*x*y^2*z*v+F(65)/96*y^3*z*v-5081*x^2*z^2*v-F(97)/14*x*y*z^2*v+F(23)/87*y^2*z^2*v+F(19)/77*x*z^3*v-F(45)/103*y*z^3*v+F(101)/16*z^4*v-F(43)/18*x^3*u*v+13388*x^2*y*u*v-F(2)/115*x*y^2*u*v+F(23)/91*y^3*u*v+F(124)/105*x^2*z*u*v+F(125)/121*x*y*z*u*v+10170*y^2*z*u*v-11646*x*z^2*u*v+12018*y*z^2*u*v-F(29)/27*z^3*u*v+F(3)/40*x^2*u^2*v+2378*x*y*u^2*v+F(57)/107*y^2*u^2*v-F(29)/84*x*z*u^2*v+15602*y*z*u^2*v+14233*z^2*u^2*v+F(119)/93*x*u^3*v-F(31)/92*y*u^3*v-1397*z*u^3*v+F(94)/91*u^4*v-F(111)/97*x^3*v^2+12090*x^2*y*v^2-F(3)/13*x*y^2*v^2-15533*y^3*v^2-F(11)/114*x^2*z*v^2+F(22)/23*x*y*z*v^2-F(18)/59*y^2*z*v^2+F(111)/40*x*z^2*v^2-15144*y*z^2*v^2+5441*z^3*v^2+F(117)/55*x^2*u*v^2-F(81)/56*x*y*u*v^2+F(67)/6*y^2*u*v^2+F(90)/77*x*z*u*v^2+F(94)/45*y*z*u*v^2+3419*z^2*u*v^2-5932*x*u^2*v^2-F(77)/43*y*u^2*v^2-F(89)/117*z*u^2*v^2-F(91)/115*u^3*v^2-9108*x^2*v^3+F(44)/51*x*y*v^3+F(44)/75*y^2*v^3-F(119)/87*x*z*v^3+F(15)/49*y*z*v^3-F(96)/47*z^2*v^3-1119*x*u*v^3+F(103)/38*y*u*v^3-F(81)/103*z*u*v^3-F(1)/3*u^2*v^3-12976*x*v^4-F(13)/50*y*v^4+F(39)/22*z*v^4-12562*u*v^4+F(59)/112*v^5,F(73)/105*x^3*y*z-F(43)/51*x^2*y^2*z-8216*x*y^3*z+F(29)/72*y^4*z+9111*x^3*z^2-2619*x^2*y*z^2-F(103)/27*x*y^2*z^2-F(15)/17*y^3*z^2-F(92)/57*x^2*z^3+F(89)/59*x*y*z^3+F(52)/105*y^2*z^3-F(119)/87*x*z^4-F(35)/6*y*z^4+15024*z^5+F(93)/118*x^3*z*u-F(11)/101*x^2*y*z*u-9707*x*y^2*z*u+1571*y^3*z*u+5860*x^2*z^2*u-11682*x*y*z^2*u-F(32)/91*y^2*z^2*u+F(79)/29*x*z^3*u+F(29)/83*y*z^3*u+F(113)/114*z^4*u-F(97)/109*x^2*z*u^2+F(66)/7*x*y*z*u^2+F(11)/113*y^2*z*u^2-F(85)/22*x*z^2*u^2+F(99)/8*y*z^2*u^2-14469*z^3*u^2-13042*x*z*u^3-105*y*z*u^3-15822*z^2*u^3+F(35)/58*z*u^4-F(95)/4*x^3*y*v+11795*x^2*y^2*v-148*x*y^3*v+9566*y^4*v-13051*x^3*z*v-F(51)/26*x^2*y*z*v-F(67)/63*x*y^2*z*v+F(5)/63*y^3*z*v+F(119)/23*x^2*z^2*v+7587*x*y*z^2*v+F(19)/9*y^2*z^2*v-F(23)/90*x*z^3*v-F(103)/6*y*z^3*v-3290*z^4*v-F(89)/51*x^3*u*v+11915*x^2*y*u*v+482*x*y^2*u*v-F(41)/26*y^3*u*v-F(51)/88*x^2*z*u*v+F(3)/94*x*y*z*u*v-2124*y^2*z*u*v+F(32)/7*x*z^2*u*v-F(27)/47*y*z^2*u*v-F(17)/106*z^3*u*v-F(41)/121*x^2*u^2*v-6625*x*y*u^2*v+F(80)/29*y^2*u^2*v+15243*x*z*u^2*v+F(103)/40*y*z*u^2*v-10778*z^2*u^2*v-F(60)/19*x*u^3*v-1154*y*u^3*v+10243*z*u^3*v+F(93)/80*u^4*v+F(98)/23*x^3*v^2-F(101)/74*x^2*y*v^2+F(63)/124*x*y^2*v^2+F(61)/44*y^3*v^2-9382*x^2*z*v^2+F(109)/121*x*y*z*v^2-F(101)/103*y^2*z*v^2+4329*x*z^2*v^2+F(1)/94*y*z^2*v^2+F(15)/91*z^3*v^2-F(52)/45*x^2*u*v^2+15459*x*y*u*v^2+F(27)/79*y^2*u*v^2-15674*x*z*u*v^2+F(113)/82*y*z*u*v^2-F(72)/61*z^2*u*v^2+F(5)/54*x*u^2*v^2+F(97)/106*y*u^2*v^2-6582*z*u^2*v^2+F(119)/116*u^3*v^2+2194*x^2*v^3+F(89)/87*x*y*v^3+13232*y^2*v^3+11043*x*z*v^3+F(100)/107*y*z*v^3+F(94)/21*z^2*v^3+987*x*u*v^3+9949*y*u*v^3+6921*z*u*v^3+F(109)/24*u^2*v^3-F(49)/124*x*v^4+F(1)/64*y*v^4+6548*z*v^4+F(37)/22*u*v^4+F(15)/19*v^5,F(73)/105*x^3*y*u-F(43)/51*x^2*y^2*u-8216*x*y^3*u+F(29)/72*y^4*u+9111*x^3*z*u-2619*x^2*y*z*u-F(103)/27*x*y^2*z*u-F(15)/17*y^3*z*u-F(92)/57*x^2*z^2*u+F(89)/59*x*y*z^2*u+F(52)/105*y^2*z^2*u-F(119)/87*x*z^3*u-F(35)/6*y*z^3*u+15024*z^4*u+F(93)/118*x^3*u^2-F(11)/101*x^2*y*u^2-9707*x*y^2*u^2+1571*y^3*u^2+5860*x^2*z*u^2-11682*x*y*z*u^2-F(32)/91*y^2*z*u^2+F(79)/29*x*z^2*u^2+F(29)/83*y*z^2*u^2+F(113)/114*z^3*u^2-F(97)/109*x^2*u^3+F(66)/7*x*y*u^3+F(11)/113*y^2*u^3-F(85)/22*x*z*u^3+F(99)/8*y*z*u^3-14469*z^2*u^3-13042*x*u^4-105*y*u^4-15822*z*u^4+F(35)/58*u^5+9163*x^3*y*v-F(116)/119*x^2*y^2*v-F(97)/16*x*y^3*v+F(77)/92*y^4*v-14558*x^3*z*v-F(79)/50*x^2*y*z*v-F(100)/13*x*y^2*z*v+F(57)/17*y^3*z*v-F(108)/83*x^2*z^2*v-F(8)/41*x*y*z^2*v+F(4)/111*y^2*z^2*v+F(2)/67*x*z^3*v-F(15)/92*y*z^3*v-F(34)/35*z^4*v+7864*x^3*u*v+F(77)/103*x^2*y*u*v-6843*x*y^2*u*v-10894*y^3*u*v-F(60)/67*x^2*z*u*v+7416*x*y*z*u*v+F(41)/46*y^2*z*u*v-F(29)/61*x*z^2*u*v-F(1)/124*y*z^2*u*v+F(103)/81*z^3*u*v+F(23)/126*x^2*u^2*v+7192*x*y*u^2*v+12753*y^2*u^2*v-7399*x*z*u^2*v-F(67)/59*y*z*u^2*v-F(59)/93*z^2*u^2*v-12826*x*u^3*v-F(26)/97*y*u^3*v+7009*z*u^3*v+124*u^4*v-F(14)/43*x^3*v^2-F(99)/13*x^2*y*v^2+825*x*y^2*v^2-F(37)/6*y^3*v^2+F(64)/117*x^2*z*v^2+F(4)/51*x*y*z*v^2-F(31)/64*y^2*z*v^2+8199*x*z^2*v^2+F(67)/29*y*z^2*v^2-F(115)/21*z^3*v^2+F(32)/71*x^2*u*v^2-F(121)/40*x*y*u*v^2-F(82)/33*y^2*u*v^2+246*x*z*u*v^2+F(107)/6*y*z*u*v^2-2490*z^2*u*v^2+F(59)/117*x*u^2*v^2+14238*y*u^2*v^2-F(54)/7*z*u^2*v^2+F(89)/14*u^3*v^2+F(52)/25*x^2*v^3-F(25)/77*x*y*v^3+F(20)/39*y^2*v^3+F(29)/100*x*z*v^3-F(69)/85*y*z*v^3+F(28)/101*z^2*v^3-F(124)/91*x*u*v^3+9507*y*u*v^3-F(57)/125*z*u*v^3+F(61)/20*u^2*v^3+F(91)/53*x*v^4-13770*y*v^4+6276*z*v^4+F(57)/100*u*v^4-575*v^5,9163*x^4*y-F(116)/119*x^3*y^2-F(97)/16*x^2*y^3+F(77)/92*x*y^4-14558*x^4*z-F(79)/50*x^3*y*z-F(100)/13*x^2*y^2*z+F(57)/17*x*y^3*z-F(108)/83*x^3*z^2-F(8)/41*x^2*y*z^2+F(4)/111*x*y^2*z^2+F(2)/67*x^2*z^3-F(15)/92*x*y*z^3-F(34)/35*x*z^4+F(52)/33*x^4*u-F(103)/118*x^3*y*u-12992*x^2*y^2*u-3685*x*y^3*u+15164*y^4*u-1326*x^3*z*u+F(125)/16*x^2*y*z*u+F(43)/85*x*y^2*z*u+11565*y^3*z*u-6364*x^2*z^2*u+F(74)/31*x*y*z^2*u-F(59)/31*y^2*z^2*u-F(26)/105*x*z^3*u+F(65)/48*y*z^3*u+F(29)/23*z^4*u+13318*x^3*u^2+8754*x^2*y*u^2+F(113)/116*x*y^2*u^2+15034*y^3*u^2+F(74)/7*x^2*z*u^2+F(97)/102*x*y*z*u^2+F(125)/23*y^2*z*u^2-10019*x*z^2*u^2+F(125)/4*y*z^2*u^2+F(82)/109*z^3*u^2+11471*x^2*u^3-7207*x*y*u^3+F(30)/113*y^2*u^3+101*x*z*u^3-F(30)/121*y*z*u^3+F(107)/66*z^2*u^3-1287*x*u^4-F(108)/107*y*u^4-4216*z*u^4-F(27)/79*u^5-F(14)/43*x^4*v-F(99)/13*x^3*y*v+825*x^2*y^2*v-F(37)/6*x*y^3*v+F(64)/117*x^3*z*v+F(4)/51*x^2*y*z*v-F(31)/64*x*y^2*z*v+8199*x^2*z^2*v+F(67)/29*x*y*z^2*v-F(115)/21*x*z^3*v+15255*x^3*u*v-F(118)/51*x^2*y*u*v+5266*x*y^2*u*v+9486*y^3*u*v-F(23)/99*x^2*z*u*v-7830*x*y*z*u*v+7212*y^2*z*u*v-9788*x*z^2*u*v+10709*y*z^2*u*v-7127*z^3*u*v+F(119)/79*x^2*u^2*v-F(11)/113*x*y*u^2*v-F(84)/83*y^2*u^2*v+F(101)/12*x*z*u^2*v-3359*y*z*u^2*v-F(115)/4*z^2*u^2*v-F(79)/111*x*u^3*v+F(29)/16*y*u^3*v+F(115)/99*z*u^3*v+11119*u^4*v+F(52)/25*x^3*v^2-F(25)/77*x^2*y*v^2+F(20)/39*x*y^2*v^2+F(29)/100*x^2*z*v^2-F(69)/85*x*y*z*v^2+F(28)/101*x*z^2*v^2+2927*x^2*u*v^2+11841*x*y*u*v^2-F(119)/109*y^2*u*v^2+14433*x*z*u*v^2+F(19)/22*y*z*u*v^2+4439*z^2*u*v^2-F(19)/56*x*u^2*v^2+F(23)/21*y*u^2*v^2-5062*z*u^2*v^2+15398*u^3*v^2+F(91)/53*x^2*v^3-13770*x*y*v^3+6276*x*z*v^3+F(118)/77*x*u*v^3-3533*y*u*v^3-10713*z*u*v^3+12855*u^2*v^3-575*x*v^4-F(112)/31*u*v^4,-9163*x^3*y^2+F(116)/119*x^2*y^3+F(97)/16*x*y^4-F(77)/92*y^5+14558*x^3*y*z+F(79)/50*x^2*y^2*z+F(100)/13*x*y^3*z-F(57)/17*y^4*z+F(108)/83*x^2*y*z^2+F(8)/41*x*y^2*z^2-F(4)/111*y^3*z^2-F(2)/67*x*y*z^3+F(15)/92*y^2*z^3+F(34)/35*y*z^4-13982*x^3*y*u+11111*x^2*y^2*u-6936*x*y^3*u-9639*y^4*u+11940*x^3*z*u-F(123)/56*x^2*y*z*u-F(105)/116*x*y^2*z*u-8592*y^3*z*u-5081*x^2*z^2*u+6624*x*y*z^2*u-4522*y^2*z^2*u+F(19)/77*x*z^3*u-11647*y*z^3*u+F(101)/16*z^4*u-F(43)/18*x^3*u^2+F(35)/66*x^2*y*u^2-1072*x*y^2*u^2+F(61)/32*y^3*u^2+F(124)/105*x^2*z*u^2-5555*x*y*z*u^2-F(77)/13*y^2*z*u^2-11646*x*z^2*u^2-F(14)/99*y*z^2*u^2-F(29)/27*z^3*u^2+F(3)/40*x^2*u^3+F(36)/101*x*y*u^3+3080*y^2*u^3-F(29)/84*x*z*u^3-F(107)/67*y*z*u^3+14233*z^2*u^3+F(119)/93*x*u^4-F(80)/37*y*u^4-1397*z*u^4+F(94)/91*u^5+F(14)/43*x^3*y*v+F(99)/13*x^2*y^2*v-825*x*y^3*v+F(37)/6*y^4*v-F(64)/117*x^2*y*z*v-F(4)/51*x*y^2*z*v+F(31)/64*y^3*z*v-8199*x*y*z^2*v-F(67)/29*y^2*z^2*v+F(115)/21*y*z^3*v-F(111)/97*x^3*u*v+F(101)/52*x^2*y*u*v-F(70)/41*x*y^2*u*v-2928*y^3*u*v-F(11)/114*x^2*z*u*v+F(7)/17*x*y*z*u*v+F(104)/69*y^2*z*u*v+F(111)/40*x*z^2*u*v-12654*y*z^2*u*v+5441*z^3*u*v+F(117)/55*x^2*u^2*v-F(41)/6*x*y*u^2*v-8895*y^2*u^2*v+F(90)/77*x*z*u^2*v+9861*y*z*u^2*v+3419*z^2*u^2*v-5932*x*u^3*v-13878*y*u^3*v-F(89)/117*z*u^3*v-F(91)/115*u^4*v-F(52)/25*x^2*y*v^2+F(25)/77*x*y^2*v^2-F(20)/39*y^3*v^2-F(29)/100*x*y*z*v^2+F(69)/85*y^2*z*v^2-F(28)/101*y*z^2*v^2-9108*x^2*u*v^2-7594*x*y*u*v^2-F(63)/4*y^2*u*v^2-F(119)/87*x*z*u*v^2-F(51)/77*y*z*u*v^2-F(96)/47*z^2*u*v^2-1119*x*u^2*v^2-3452*y*u^2*v^2-F(81)/103*z*u^2*v^2-F(1)/3*u^3*v^2-F(91)/53*x*y*v^3+13770*y^2*v^3-6276*y*z*v^3-12976*x*u*v^3-F(83)/100*y*u*v^3+F(39)/22*z*u*v^3-12562*u^2*v^3+575*y*v^4+F(59)/112*u*v^4,-9163*x^3*y*z+F(116)/119*x^2*y^2*z+F(97)/16*x*y^3*z-F(77)/92*y^4*z+14558*x^3*z^2+F(79)/50*x^2*y*z^2+F(100)/13*x*y^2*z^2-F(57)/17*y^3*z^2+F(108)/83*x^2*z^3+F(8)/41*x*y*z^3-F(4)/111*y^2*z^3-F(2)/67*x*z^4+F(15)/92*y*z^4+F(34)/35*z^5-F(95)/4*x^3*y*u+11795*x^2*y^2*u-148*x*y^3*u+9566*y^4*u+F(57)/26*x^3*z*u-F(124)/59*x^2*y*z*u+1764*x*y^2*z*u-F(31)/77*y^3*z*u-F(30)/19*x^2*z^2*u+171*x*y*z^2*u-13367*y^2*z^2*u+2873*x*z^3*u-F(67)/44*y*z^3*u+10532*z^4*u-F(89)/51*x^3*u^2+11915*x^2*y*u^2+482*x*y^2*u^2-F(41)/26*y^3*u^2-F(81)/76*x^2*z*u^2-F(35)/31*x*y*z*u^2+F(109)/43*y^2*z*u^2-F(3)/50*x*z^2*u^2+15898*y*z^2*u^2-F(94)/55*z^3*u^2-F(41)/121*x^2*u^3-6625*x*y*u^3+F(80)/29*y^2*u^3-3922*x*z*u^3+F(114)/101*y*z*u^3+14204*z^2*u^3-F(60)/19*x*u^4-1154*y*u^4-F(59)/98*z*u^4+F(93)/80*u^5+F(14)/43*x^3*z*v+F(99)/13*x^2*y*z*v-825*x*y^2*z*v+F(37)/6*y^3*z*v-F(64)/117*x^2*z^2*v-F(4)/51*x*y*z^2*v+F(31)/64*y^2*z^2*v-8199*x*z^3*v-F(67)/29*y*z^3*v+F(115)/21*z^4*v+F(98)/23*x^3*u*v-F(101)/74*x^2*y*u*v+F(63)/124*x*y^2*u*v+F(61)/44*y^3*u*v+F(33)/40*x^2*z*u*v-F(1)/25*x*y*z*u*v-2615*y^2*z*u*v-F(45)/47*x*z^2*u*v+F(15)/101*y*z^2*u*v-F(117)/8*z^3*u*v-F(52)/45*x^2*u^2*v+15459*x*y*u^2*v+F(27)/79*y^2*u^2*v-636*x*z*u^2*v+F(116)/95*y*z*u^2*v-10782*z^2*u^2*v+F(5)/54*x*u^3*v+F(97)/106*y*u^3*v+F(99)/86*z*u^3*v+F(119)/116*u^4*v-F(52)/25*x^2*z*v^2+F(25)/77*x*y*z*v^2-F(20)/39*y^2*z*v^2-F(29)/100*x*z^2*v^2+F(69)/85*y*z^2*v^2-F(28)/101*z^3*v^2+2194*x^2*u*v^2+F(89)/87*x*y*u*v^2+13232*y^2*u*v^2-12861*x*z*u*v^2-F(107)/47*y*z*u*v^2-13608*z^2*u*v^2+987*x*u^2*v^2+9949*y*u^2*v^2+F(32)/77*z*u^2*v^2+F(109)/24*u^3*v^2-F(91)/53*x*z*v^3+13770*y*z*v^3-6276*z^2*v^3-F(49)/124*x*u*v^3+F(1)/64*y*u*v^3+15185*z*u*v^3+F(37)/22*u^2*v^3+575*z*v^4+F(15)/19*u*v^4,-F(95)/4*x^4*y+11795*x^3*y^2-148*x^2*y^3+9566*x*y^4-F(120)/77*x^4*z+7317*x^3*y*z-11228*x^2*y^2*z+F(117)/118*x*y^3*z+15164*y^4*z-F(65)/14*x^3*z^2-F(65)/22*x^2*y*z^2+559*x*y^2*z^2+11565*y^3*z^2-F(59)/55*x^2*z^3-F(68)/57*x*y*z^3-F(59)/31*y^2*z^3-2874*x*z^4+F(65)/48*y*z^4+F(29)/23*z^5-F(89)/51*x^4*u+11915*x^3*y*u+482*x^2*y^2*u-F(41)/26*x*y^3*u+12896*x^3*z*u-F(17)/83*x^2*y*z*u+F(75)/89*x*y^2*z*u+15034*y^3*z*u+F(16)/45*x^2*z^2*u+1158*x*y*z^2*u+F(125)/23*y^2*z^2*u+F(107)/65*x*z^3*u+F(125)/4*y*z^3*u+F(82)/109*z^4*u-F(41)/121*x^3*u^2-6625*x^2*y*u^2+F(80)/29*x*y^2*u^2+F(50)/89*x^2*z*u^2+8948*x*y*z*u^2+F(30)/113*y^2*z*u^2+F(10)/123*x*z^2*u^2-F(30)/121*y*z^2*u^2+F(107)/66*z^3*u^2-F(60)/19*x^2*u^3-1154*x*y*u^3+8832*x*z*u^3-F(108)/107*y*z*u^3-4216*z^2*u^3+F(93)/80*x*u^4-F(27)/79*z*u^4+F(98)/23*x^4*v-F(101)/74*x^3*y*v+F(63)/124*x^2*y^2*v+F(61)/44*x*y^3*v-3139*x^3*z*v-F(4)/55*x^2*y*z*v+2651*x*y^2*z*v+9486*y^3*z*v+F(87)/2*x^2*z^2*v-F(10)/107*x*y*z^2*v+7212*y^2*z^2*v+2194*x*z^3*v+10709*y*z^3*v-7127*z^4*v-F(52)/45*x^3*u*v+15459*x^2*y*u*v+F(27)/79*x*y^2*u*v+3415*x^2*z*u*v-F(117)/55*x*y*z*u*v-F(84)/83*y^2*z*u*v-F(82)/25*x*z^2*u*v-3359*y*z^2*u*v-F(115)/4*z^3*u*v+F(5)/54*x^2*u^2*v+F(97)/106*x*y*u^2*v-6692*x*z*u^2*v+F(29)/16*y*z*u^2*v+F(115)/99*z^2*u^2*v+F(119)/116*x*u^3*v+11119*z*u^3*v+2194*x^3*v^2+F(89)/87*x^2*y*v^2+13232*x*y^2*v^2-9934*x^2*z*v^2-14707*x*y*z*v^2-F(119)/109*y^2*z*v^2+825*x*z^2*v^2+F(19)/22*y*z^2*v^2+4439*z^3*v^2+987*x^2*u*v^2+9949*x*y*u*v^2-F(97)/90*x*z*u*v^2+F(23)/21*y*z*u*v^2-5062*z^2*u*v^2+F(109)/24*x*u^2*v^2+15398*z*u^2*v^2-F(49)/124*x^2*v^3+F(1)/64*x*y*v^3+15602*x*z*v^3-3533*y*z*v^3-10713*z^2*v^3+F(37)/22*x*u*v^3+12855*z*u*v^3+F(15)/19*x*v^4-F(112)/31*z*v^4,F(95)/4*x^3*y^2-11795*x^2*y^3+148*x*y^4-9566*y^5+F(97)/60*x^3*y*z+F(32)/99*x^2*y^2*z-F(79)/114*x*y^3*z-10885*y^4*z+11940*x^3*z^2+8388*x^2*y*z^2+F(12)/101*x*y^2*z^2+F(15)/67*y^3*z^2-5081*x^2*z^3+3751*x*y*z^3+569*y^2*z^3+F(19)/77*x*z^4+F(107)/75*y*z^4+F(101)/16*z^5+F(89)/51*x^3*y*u-11915*x^2*y^2*u-482*x*y^3*u+F(41)/26*y^4*u-F(43)/18*x^3*z*u-8787*x^2*y*z*u+3057*x*y^2*z*u+3882*y^3*z*u+F(124)/105*x^2*z^2*u-F(59)/6*x*y*z^2*u+F(7)/121*y^2*z^2*u-11646*x*z^3*u+F(31)/61*y*z^3*u-F(29)/27*z^4*u+F(41)/121*x^2*y*u^2+6625*x*y^2*u^2-F(80)/29*y^3*u^2+F(3)/40*x^2*z*u^2+F(87)/92*x*y*z*u^2-F(48)/115*y^2*z*u^2-F(29)/84*x*z^2*u^2+F(83)/57*y*z^2*u^2+14233*z^3*u^2+F(60)/19*x*y*u^3+1154*y^2*u^3+F(119)/93*x*z*u^3-F(51)/44*y*z*u^3-1397*z^2*u^3-F(93)/80*y*u^4+F(94)/91*z*u^4-F(98)/23*x^3*y*v+F(101)/74*x^2*y^2*v-F(63)/124*x*y^3*v-F(61)/44*y^4*v-F(111)/97*x^3*z*v-F(103)/73*x^2*y*z*v+5429*x*y^2*z*v+F(65)/102*y^3*z*v-F(11)/114*x^2*z^2*v+10972*x*y*z^2*v+F(26)/9*y^2*z^2*v+F(111)/40*x*z^3*v+F(65)/87*y*z^3*v+5441*z^4*v+F(52)/45*x^2*y*u*v-15459*x*y^2*u*v-F(27)/79*y^3*u*v+F(117)/55*x^2*z*u*v+5961*x*y*z*u*v-5192*y^2*z*u*v+F(90)/77*x*z^2*u*v+F(113)/31*y*z^2*u*v+3419*z^3*u*v-F(5)/54*x*y*u^2*v-F(97)/106*y^2*u^2*v-5932*x*z*u^2*v+13276*y*z*u^2*v-F(89)/117*z^2*u^2*v-F(119)/116*y*u^3*v-F(91)/115*z*u^3*v-2194*x^2*y*v^2-F(89)/87*x*y^2*v^2-13232*y^3*v^2-9108*x^2*z*v^2+5267*x*y*z*v^2+F(2)/63*y^2*z*v^2-F(119)/87*x*z^2*v^2+11530*y*z^2*v^2-F(96)/47*z^3*v^2-987*x*y*u*v^2-9949*y^2*u*v^2-1119*x*z*u*v^2+4026*y*z*u*v^2-F(81)/103*z^2*u*v^2-F(109)/24*y*u^2*v^2-F(1)/3*z*u^2*v^2+F(49)/124*x*y*v^3-F(1)/64*y^2*v^3-12976*x*z*v^3-11027*y*z*v^3+F(39)/22*z^2*v^3-F(37)/22*y*u*v^3-12562*z*u*v^3-F(15)/19*y*v^4+F(59)/112*z*v^4,13011*x^4*y+F(83)/116*x^3*y^2-F(50)/61*x^2*y^3-F(67)/12*x*y^4-15164*y^5-11940*x^4*z-F(100)/21*x^3*y*z-15244*x^2*y^2*z-F(13)/6*x*y^3*z-11565*y^4*z+5081*x^3*z^2+F(11)/123*x^2*y*z^2-7864*x*y^2*z^2+F(59)/31*y^3*z^2-F(19)/77*x^2*z^3-F(16)/83*x*y*z^3-F(65)/48*y^2*z^3-F(101)/16*x*z^4-F(29)/23*y*z^4+F(43)/18*x^4*u-F(7)/109*x^3*y*u-F(104)/25*x^2*y^2*u+F(8)/81*x*y^3*u-15034*y^4*u-F(124)/105*x^3*z*u+F(90)/47*x^2*y*z*u-2480*x*y^2*z*u-F(125)/23*y^3*z*u+11646*x^2*z^2*u+F(58)/33*x*y*z^2*u-F(125)/4*y^2*z^2*u+F(29)/27*x*z^3*u-F(82)/109*y*z^3*u-F(3)/40*x^3*u^2-F(95)/6*x^2*y*u^2-F(27)/31*x*y^2*u^2-F(30)/113*y^3*u^2+F(29)/84*x^2*z*u^2-F(73)/92*x*y*z*u^2+F(30)/121*y^2*z*u^2-14233*x*z^2*u^2-F(107)/66*y*z^2*u^2-F(119)/93*x^2*u^3+10800*x*y*u^3+F(108)/107*y^2*u^3+1397*x*z*u^3+4216*y*z*u^3-F(94)/91*x*u^4+F(27)/79*y*u^4+F(111)/97*x^4*v-F(96)/89*x^3*y*v-F(5)/33*x^2*y^2*v+F(115)/41*x*y^3*v-9486*y^4*v+F(11)/114*x^3*z*v+4980*x^2*y*z*v-5617*x*y^2*z*v-7212*y^3*z*v-F(111)/40*x^2*z^2*v+F(37)/67*x*y*z^2*v-10709*y^2*z^2*v-5441*x*z^3*v+7127*y*z^3*v-F(117)/55*x^3*u*v+F(39)/58*x^2*y*u*v-4694*x*y^2*u*v+F(84)/83*y^3*u*v-F(90)/77*x^2*z*u*v+8792*x*y*z*u*v+3359*y^2*z*u*v-3419*x*z^2*u*v+F(115)/4*y*z^2*u*v+5932*x^2*u^2*v+F(81)/34*x*y*u^2*v-F(29)/16*y^2*u^2*v+F(89)/117*x*z*u^2*v-F(115)/99*y*z*u^2*v+F(91)/115*x*u^3*v-11119*y*u^3*v+9108*x^3*v^2+F(79)/48*x^2*y*v^2+12168*x*y^2*v^2+F(119)/109*y^3*v^2+F(119)/87*x^2*z*v^2-12355*x*y*z*v^2-F(19)/22*y^2*z*v^2+F(96)/47*x*z^2*v^2-4439*y*z^2*v^2+1119*x^2*u*v^2+596*x*y*u*v^2-F(23)/21*y^2*u*v^2+F(81)/103*x*z*u*v^2+5062*y*z*u*v^2+F(1)/3*x*u^2*v^2-15398*y*u^2*v^2+12976*x^2*v^3-F(34)/7*x*y*v^3+3533*y^2*v^3-F(39)/22*x*z*v^3+10713*y*z*v^3+12562*x*u*v^3-12855*y*u*v^3-F(59)/112*x*v^4+F(112)/31*y*v^4,F(19)/123*x^4*y-11120*x^3*y^2+F(20)/101*x^2*y^3-8923*x*y^4-F(72)/7*x^4*z-5929*x^3*y*z-F(20)/53*x^2*y^2*z+F(87)/103*x*y^3*z+F(91)/86*x^3*z^2+F(91)/103*x^2*y*z^2-F(39)/4*x*y^2*z^2-F(37)/114*x^2*z^3-F(17)/105*x*y*z^3-F(113)/110*x*z^4+226*x^4*u+F(94)/77*x^3*y*u+F(45)/83*x^2*y^2*u-5695*x*y^3*u+F(47)/67*x^3*z*u+13902*x^2*y*z*u+F(99)/112*x*y^2*z*u-423*x^2*z^2*u-F(64)/107*x*y*z^2*u+F(73)/108*x*z^3*u+F(4)/97*x^3*u^2-F(23)/5*x^2*y*u^2+F(67)/26*x*y^2*u^2-F(11)/61*x^2*z*u^2-8400*x*y*z*u^2-F(4)/85*x*z^2*u^2+F(38)/47*x^2*u^3-F(63)/8*x*y*u^3-F(1)/33*x*z*u^3-F(59)/14*x*u^4+F(17)/96*x^4*v+F(98)/125*x^3*y*v-F(68)/103*x^2*y^2*v-5257*x*y^3*v-11081*y^4*v+F(83)/121*x^3*z*v-F(94)/11*x^2*y*z*v-4540*x*y^2*z*v+F(19)/73*y^3*z*v-5025*x^2*z^2*v-F(95)/27*x*y*z^2*v+F(31)/23*y^2*z^2*v+F(3)/80*x*z^3*v-F(43)/49*y*z^3*v-F(11)/111*z^4*v+5762*x^3*u*v+7318*x^2*y*u*v-F(33)/28*x*y^2*u*v+F(30)/37*y^3*u*v+11187*x^2*z*u*v+9302*x*y*z*u*v-11154*y^2*z*u*v-F(15)/64*x*z^2*u*v+F(32)/13*y*z^2*u*v+F(81)/55*z^3*u*v-F(53)/62*x^2*u^2*v+F(14)/113*x*y*u^2*v-13291*y^2*u^2*v-2994*x*z*u^2*v+2991*y*z*u^2*v+F(27)/52*z^2*u^2*v+F(29)/96*x*u^3*v-F(105)/101*y*u^3*v-7395*z*u^3*v+379*u^4*v+F(37)/69*x^3*v^2+4028*x^2*y*v^2-171*x*y^2*v^2+9790*y^3*v^2-F(99)/35*x^2*z*v^2+2260*x*y*z*v^2-4311*y^2*z*v^2-14576*x*z^2*v^2-F(111)/121*y*z^2*v^2+F(100)/31*z^3*v^2-7168*x^2*u*v^2+9922*x*y*u*v^2+13144*y^2*u*v^2-F(103)/7*x*z*u*v^2+4932*y*z*u*v^2+F(119)/83*z^2*u*v^2-5682*x*u^2*v^2-F(104)/109*y*u^2*v^2+7249*z*u^2*v^2+3104*u^3*v^2+F(101)/77*x^2*v^3-10605*x*y*v^3+14163*y^2*v^3+F(65)/71*x*z*v^3-F(19)/126*y*z*v^3+F(95)/103*z^2*v^3-F(77)/30*x*u*v^3-6185*y*u*v^3+662*z*u*v^3-2925*u^2*v^3+7094*x*v^4+9205*y*v^4-10805*z*v^4+F(91)/37*u*v^4-F(52)/31*v^5,-F(19)/123*x^3*y^2+11120*x^2*y^3-F(20)/101*x*y^4+8923*y^5+F(72)/7*x^3*y*z+5929*x^2*y^2*z+F(20)/53*x*y^3*z-F(87)/103*y^4*z-F(91)/86*x^2*y*z^2-F(91)/103*x*y^2*z^2+F(39)/4*y^3*z^2+F(37)/114*x*y*z^3+F(17)/105*y^2*z^3+F(113)/110*y*z^4-226*x^3*y*u-F(94)/77*x^2*y^2*u-F(45)/83*x*y^3*u+5695*y^4*u-F(47)/67*x^2*y*z*u-13902*x*y^2*z*u-F(99)/112*y^3*z*u+423*x*y*z^2*u+F(64)/107*y^2*z^2*u-F(73)/108*y*z^3*u-F(4)/97*x^2*y*u^2+F(23)/5*x*y^2*u^2-F(67)/26*y^3*u^2+F(11)/61*x*y*z*u^2+8400*y^2*z*u^2+F(4)/85*y*z^2*u^2-F(38)/47*x*y*u^3+F(63)/8*y^2*u^3+F(1)/33*y*z*u^3+F(59)/14*y*u^4-F(113)/96*x^3*y*v+F(62)/95*x^2*y^2*v-F(29)/23*x*y^3*v+F(31)/2*y^4*v+F(65)/126*x^3*z*v+F(1)/83*x^2*y*z*v-5478*x*y^2*z*v+F(24)/67*y^3*z*v-10187*x^2*z^2*v-2323*x*y*z^2*v+15002*y^2*z^2*v-15332*x*z^3*v-11940*y*z^3*v+F(73)/18*z^4*v+F(96)/43*x^3*u*v+F(95)/8*x^2*y*u*v-F(36)/73*x*y^2*u*v+1636*y^3*u*v+11236*x^2*z*u*v-7896*x*y*z*u*v-8448*y^2*z*u*v-5986*x*z^2*u*v-F(27)/19*y*z^2*u*v-9266*z^3*u*v+10541*x^2*u^2*v-172*x*y*u^2*v+F(78)/73*y^2*u^2*v+4960*x*z*u^2*v-F(7)/66*y*z*u^2*v+10276*z^2*u^2*v+6041*x*u^3*v+F(50)/3*y*u^3*v+F(88)/79*z*u^3*v-F(49)/125*u^4*v-F(83)/97*x^3*v^2-F(3)/4*x^2*y*v^2+6159*x*y^2*v^2-F(97)/124*y^3*v^2+12480*x^2*z*v^2+F(42)/23*x*y*z*v^2-11749*y^2*z*v^2-15931*x*z^2*v^2-F(13)/81*y*z^2*v^2+F(53)/55*z^3*v^2+F(59)/109*x^2*u*v^2-8618*x*y*u*v^2-F(36)/77*y^2*u*v^2+6083*x*z*u*v^2-F(45)/46*y*z*u*v^2+F(108)/103*z^2*u*v^2+F(30)/119*x*u^2*v^2+F(64)/19*y*u^2*v^2-15601*z*u^2*v^2+1963*u^3*v^2-7518*x^2*v^3-F(73)/110*x*y*v^3+F(83)/119*y^2*v^3-8520*x*z*v^3-9613*y*z*v^3+2432*z^2*v^3-5547*x*u*v^3-1179*y*u*v^3+F(43)/56*z*u*v^3-648*u^2*v^3+F(113)/71*x*v^4-F(95)/106*y*v^4+165*z*v^4-F(27)/91*u*v^4-15905*v^5,-F(19)/123*x^3*y*z+11120*x^2*y^2*z-F(20)/101*x*y^3*z+8923*y^4*z+F(72)/7*x^3*z^2+5929*x^2*y*z^2+F(20)/53*x*y^2*z^2-F(87)/103*y^3*z^2-F(91)/86*x^2*z^3-F(91)/103*x*y*z^3+F(39)/4*y^2*z^3+F(37)/114*x*z^4+F(17)/105*y*z^4+F(113)/110*z^5-226*x^3*z*u-F(94)/77*x^2*y*z*u-F(45)/83*x*y^2*z*u+5695*y^3*z*u-F(47)/67*x^2*z^2*u-13902*x*y*z^2*u-F(99)/112*y^2*z^2*u+423*x*z^3*u+F(64)/107*y*z^3*u-F(73)/108*z^4*u-F(4)/97*x^2*z*u^2+F(23)/5*x*y*z*u^2-F(67)/26*y^2*z*u^2+F(11)/61*x*z^2*u^2+8400*y*z^2*u^2+F(4)/85*z^3*u^2-F(38)/47*x*z*u^3+F(63)/8*y*z*u^3+F(1)/33*z^2*u^3+F(59)/14*z*u^4-F(19)/80*x^3*y*v-F(27)/113*x^2*y^2*v-5370*x*y^3*v+13408*y^4*v-2315*x^3*z*v+3054*x^2*y*z*v+F(119)/101*x*y^2*z*v+F(122)/47*y^3*z*v+8769*x^2*z^2*v+F(12)/67*x*y*z^2*v-4373*y^2*z^2*v-F(121)/72*x*z^3*v-3215*y*z^3*v+11818*z^4*v+10515*x^3*u*v+14620*x^2*y*u*v+11185*x*y^2*u*v-F(42)/61*y^3*u*v+F(41)/89*x^2*z*u*v-13410*x*y*z*u*v-F(75)/26*y^2*z*u*v+F(101)/62*x*z^2*u*v-F(26)/71*y*z^2*u*v-F(37)/48*z^3*u*v+11380*x^2*u^2*v-F(79)/43*x*y*u^2*v-F(17)/76*y^2*u^2*v-F(50)/19*x*z*u^2*v-F(33)/4*y*z*u^2*v+13770*z^2*u^2*v-F(55)/3*x*u^3*v+4105*y*u^3*v+F(79)/74*z*u^3*v-F(60)/119*u^4*v-F(18)/89*x^3*v^2-F(125)/126*x^2*y*v^2+12576*x*y^2*v^2+5204*y^3*v^2-14896*x^2*z*v^2-F(8)/101*x*y*z*v^2-F(77)/104*y^2*z*v^2-F(89)/48*x*z^2*v^2+F(6)/61*y*z^2*v^2-F(123)/49*z^3*v^2-861*x^2*u*v^2+F(49)/17*x*y*u*v^2+F(27)/80*y^2*u*v^2-F(103)/64*x*z*u*v^2+F(121)/19*y*z*u*v^2+F(10)/81*z^2*u*v^2-F(81)/52*x*u^2*v^2-F(1)/38*y*u^2*v^2+F(69)/65*z*u^2*v^2+F(44)/59*u^3*v^2-F(74)/15*x^2*v^3+F(102)/55*x*y*v^3+3926*y^2*v^3+2241*x*z*v^3-F(9)/101*y*z*v^3+F(57)/25*z^2*v^3-F(27)/28*x*u*v^3-F(29)/125*y*u*v^3-12906*z*u*v^3-5213*u^2*v^3-F(65)/86*x*v^4+9280*y*v^4+F(77)/53*z*v^4+13811*u*v^4-F(103)/78*v^5,-F(19)/123*x^3*y*u+11120*x^2*y^2*u-F(20)/101*x*y^3*u+8923*y^4*u+F(72)/7*x^3*z*u+5929*x^2*y*z*u+F(20)/53*x*y^2*z*u-F(87)/103*y^3*z*u-F(91)/86*x^2*z^2*u-F(91)/103*x*y*z^2*u+F(39)/4*y^2*z^2*u+F(37)/114*x*z^3*u+F(17)/105*y*z^3*u+F(113)/110*z^4*u-226*x^3*u^2-F(94)/77*x^2*y*u^2-F(45)/83*x*y^2*u^2+5695*y^3*u^2-F(47)/67*x^2*z*u^2-13902*x*y*z*u^2-F(99)/112*y^2*z*u^2+423*x*z^2*u^2+F(64)/107*y*z^2*u^2-F(73)/108*z^3*u^2-F(4)/97*x^2*u^3+F(23)/5*x*y*u^3-F(67)/26*y^2*u^3+F(11)/61*x*z*u^3+8400*y*z*u^3+F(4)/85*z^2*u^3-F(38)/47*x*u^4+F(63)/8*y*u^4+F(1)/33*z*u^4+F(59)/14*u^5-F(46)/27*x^3*y*v-F(97)/27*x^2*y^2*v-F(60)/49*x*y^3*v+F(21)/5*y^4*v+F(121)/122*x^3*z*v+25*x^2*y*z*v-3574*x*y^2*z*v+F(102)/91*y^3*z*v+14389*x^2*z^2*v-F(117)/107*x*y*z^2*v-F(111)/104*y^2*z^2*v+8543*x*z^3*v+F(2)/79*y*z^3*v+15460*z^4*v-F(101)/85*x^3*u*v-14093*x^2*y*u*v-F(11)/79*x*y^2*u*v-F(13)/40*y^3*u*v+F(43)/84*x^2*z*u*v-F(101)/9*x*y*z*u*v-F(37)/3*y^2*z*u*v+14670*x*z^2*u*v-F(19)/102*y*z^2*u*v-F(120)/7*z^3*u*v-F(73)/83*x^2*u^2*v-7629*x*y*u^2*v+F(5)/11*y^2*u^2*v-F(100)/93*x*z*u^2*v+F(117)/49*y*z*u^2*v-12237*z^2*u^2*v+6014*x*u^3*v+12763*y*u^3*v+F(7)/99*z*u^3*v+1369*u^4*v-14010*x^3*v^2+F(81)/77*x^2*y*v^2+F(123)/83*x*y^2*v^2-F(118)/117*y^3*v^2+F(53)/90*x^2*z*v^2+F(40)/101*x*y*z*v^2+13529*y^2*z*v^2-14713*x*z^2*v^2+9164*y*z^2*v^2+F(33)/19*z^3*v^2+F(1)/78*x^2*u*v^2+F(53)/101*x*y*u*v^2+F(109)/122*y^2*u*v^2-1815*x*z*u*v^2-F(102)/67*y*z*u*v^2-F(83)/88*z^2*u*v^2+2681*x*u^2*v^2-F(112)/41*y*u^2*v^2-F(95)/91*z*u^2*v^2+F(106)/55*u^3*v^2+13186*x^2*v^3+F(47)/25*x*y*v^3+F(13)/2*y^2*v^3-F(13)/61*x*z*v^3-9398*y*z*v^3-F(110)/63*z^2*v^3-F(31)/70*x*u*v^3+612*y*u*v^3+F(118)/83*z*u*v^3+F(86)/11*u^2*v^3+F(5)/22*x*v^4-14985*y*v^4+8188*z*v^4+1008*u*v^4-F(5)/49*v^5,-F(46)/27*x^4*y-F(97)/27*x^3*y^2-F(60)/49*x^2*y^3+F(21)/5*x*y^4+F(121)/122*x^4*z+25*x^3*y*z-3574*x^2*y^2*z+F(102)/91*x*y^3*z+14389*x^3*z^2-F(117)/107*x^2*y*z^2-F(111)/104*x*y^2*z^2+8543*x^2*z^3+F(2)/79*x*y*z^3+15460*x*z^4+15920*x^4*u-14860*x^3*y*u+12474*x^2*y^2*u-F(93)/56*x*y^3*u-11081*y^4*u+61*x^3*z*u-11976*x^2*y*z*u-F(63)/82*x*y^2*z*u+F(19)/73*y^3*z*u+9645*x^2*z^2*u-F(80)/31*x*y*z^2*u+F(31)/23*y^2*z^2*u+15350*x*z^3*u-F(43)/49*y*z^3*u-F(11)/111*z^4*u-F(91)/99*x^3*u^2-F(42)/103*x^2*y*u^2-4467*x*y^2*u^2+F(30)/37*y^3*u^2+8778*x^2*z*u^2+6040*x*y*z*u^2-11154*y^2*z*u^2+F(39)/31*x*z^2*u^2+F(32)/13*y*z^2*u^2+F(81)/55*z^3*u^2-F(20)/3*x^2*u^3-F(73)/9*x*y*u^3-13291*y^2*u^3+12840*x*z*u^3+2991*y*z*u^3+F(27)/52*z^2*u^3-12960*x*u^4-F(105)/101*y*u^4-7395*z*u^4+379*u^5-14010*x^4*v+F(81)/77*x^3*y*v+F(123)/83*x^2*y^2*v-F(118)/117*x*y^3*v+F(53)/90*x^3*z*v+F(40)/101*x^2*y*z*v+13529*x*y^2*z*v-14713*x^2*z^2*v+9164*x*y*z^2*v+F(33)/19*x*z^3*v+F(73)/68*x^3*u*v-F(92)/65*x^2*y*u*v-F(43)/28*x*y^2*u*v+9790*y^3*u*v+F(27)/92*x^2*z*u*v+F(67)/18*x*y*z*u*v-4311*y^2*z*u*v-F(99)/13*x*z^2*u*v-F(111)/121*y*z^2*u*v+F(100)/31*z^3*u*v-4487*x^2*u^2*v-F(9)/7*x*y*u^2*v+13144*y^2*u^2*v-5289*x*z*u^2*v+4932*y*z*u^2*v+F(119)/83*z^2*u^2*v-9170*x*u^3*v-F(104)/109*y*u^3*v+7249*z*u^3*v+3104*u^4*v+13186*x^3*v^2+F(47)/25*x^2*y*v^2+F(13)/2*x*y^2*v^2-F(13)/61*x^2*z*v^2-9398*x*y*z*v^2-F(110)/63*x*z^2*v^2-7436*x^2*u*v^2+F(67)/16*x*y*u*v^2+14163*y^2*u*v^2-F(31)/75*x*z*u*v^2-F(19)/126*y*z*u*v^2+F(95)/103*z^2*u*v^2-F(23)/51*x*u^2*v^2-6185*y*u^2*v^2+662*z*u^2*v^2-2925*u^3*v^2+F(5)/22*x^2*v^3-14985*x*y*v^3+8188*x*z*v^3+8102*x*u*v^3+9205*y*u*v^3-10805*z*u*v^3+F(91)/37*u^2*v^3-F(5)/49*x*v^4-F(52)/31*u*v^4,F(46)/27*x^3*y^2+F(97)/27*x^2*y^3+F(60)/49*x*y^4-F(21)/5*y^5-F(121)/122*x^3*y*z-25*x^2*y^2*z+3574*x*y^3*z-F(102)/91*y^4*z-14389*x^2*y*z^2+F(117)/107*x*y^2*z^2+F(111)/104*y^3*z^2-8543*x*y*z^3-F(2)/79*y^2*z^3-15460*y*z^4-15921*x^3*y*u+1634*x^2*y^2*u+F(50)/81*x*y^3*u-F(56)/47*y^4*u+F(65)/126*x^3*z*u-519*x^2*y*z*u+F(23)/13*x*y^2*z*u-13675*y^3*z*u-10187*x^2*z^2*u+F(71)/32*x*y*z^2*u-F(111)/50*y^2*z^2*u-15332*x*z^3*u+15498*y*z^3*u+F(73)/18*z^4*u+F(96)/43*x^3*u^2+F(95)/49*x^2*y*u^2+14202*x*y^2*u^2-4181*y^3*u^2+11236*x^2*z*u^2-F(99)/35*x*y*z*u^2+F(64)/37*y^2*z*u^2-5986*x*z^2*u^2-F(63)/116*y*z^2*u^2-9266*z^3*u^2+10541*x^2*u^3-6186*x*y*u^3-F(125)/93*y^2*u^3+4960*x*z*u^3-7594*y*z*u^3+10276*z^2*u^3+6041*x*u^4-12016*y*u^4+F(88)/79*z*u^4-F(49)/125*u^5+14010*x^3*y*v-F(81)/77*x^2*y^2*v-F(123)/83*x*y^3*v+F(118)/117*y^4*v-F(53)/90*x^2*y*z*v-F(40)/101*x*y^2*z*v-13529*y^3*z*v+14713*x*y*z^2*v-9164*y^2*z^2*v-F(33)/19*y*z^3*v-F(83)/97*x^3*u*v+5126*x^2*y*u*v+F(61)/23*x*y^2*u*v+5653*y^3*u*v+12480*x^2*z*u*v-965*x*y*z*u*v-F(39)/88*y^2*z*u*v-15931*x*z^2*u*v+1989*y*z^2*u*v+F(53)/55*z^3*u*v+F(59)/109*x^2*u^2*v-11299*x*y*u^2*v+1198*y^2*u^2*v+6083*x*z*u^2*v-F(6)/25*y*z*u^2*v+F(108)/103*z^2*u^2*v+F(30)/119*x*u^3*v-13346*y*u^3*v-15601*z*u^3*v+1963*u^4*v-13186*x^2*y*v^2-F(47)/25*x*y^2*v^2-F(13)/2*y^3*v^2+F(13)/61*x*y*z*v^2+9398*y^2*z*v^2+F(110)/63*y*z^2*v^2-7518*x^2*u*v^2-F(17)/77*x*y*u*v^2+5303*y^2*u*v^2-8520*x*z*u*v^2+F(43)/20*y*z*u*v^2+2432*z^2*u*v^2-5547*x*u^2*v^2+7538*y*u^2*v^2+F(43)/56*z*u^2*v^2-648*u^3*v^2-F(5)/22*x*y*v^3+14985*y^2*v^3-8188*y*z*v^3+F(113)/71*x*u*v^3+14383*y*u*v^3+165*z*u*v^3-F(27)/91*u^2*v^3+F(5)/49*y*v^4-15905*u*v^4,F(46)/27*x^3*y*z+F(97)/27*x^2*y^2*z+F(60)/49*x*y^3*z-F(21)/5*y^4*z-F(121)/122*x^3*z^2-25*x^2*y*z^2+3574*x*y^2*z^2-F(102)/91*y^3*z^2-14389*x^2*z^3+F(117)/107*x*y*z^3+F(111)/104*y^2*z^3-8543*x*z^4-F(2)/79*y*z^4-15460*z^5-F(19)/80*x^3*y*u-F(27)/113*x^2*y^2*u-5370*x*y^3*u+13408*y^4*u-F(13)/7*x^3*z*u-F(22)/125*x^2*y*z*u+F(9)/89*x*y^2*z*u-F(8)/73*y^3*z*u-12178*x^2*z^2*u-F(24)/77*x*y*z^2*u+F(115)/66*y^2*z^2*u+F(57)/91*x*z^3*u+F(75)/107*y*z^3*u-F(125)/22*z^4*u+10515*x^3*u^2+14620*x^2*y*u^2+11185*x*y^2*u^2-F(42)/61*y^3*u^2+F(77)/93*x^2*z*u^2+F(42)/83*x*y*z*u^2-11972*y^2*z*u^2-9457*x*z^2*u^2-12058*y*z^2*u^2+3572*z^3*u^2+11380*x^2*u^3-F(79)/43*x*y*u^3-F(17)/76*y^2*u^3-F(23)/7*x*z*u^3+11222*y*z*u^3-F(2)/31*z^2*u^3-F(55)/3*x*u^4+4105*y*u^4-71*z*u^4-F(60)/119*u^5+14010*x^3*z*v-F(81)/77*x^2*y*z*v-F(123)/83*x*y^2*z*v+F(118)/117*y^3*z*v-F(53)/90*x^2*z^2*v-F(40)/101*x*y*z^2*v-13529*y^2*z^2*v+14713*x*z^3*v-9164*y*z^3*v-F(33)/19*z^4*v-F(18)/89*x^3*u*v-F(125)/126*x^2*y*u*v+12576*x*y^2*u*v+5204*y^3*u*v+F(52)/9*x^2*z*u*v-F(61)/101*x*y*z*u*v-F(26)/19*y^2*z*u*v+F(119)/121*x*z^2*u*v+F(71)/10*y*z^2*u*v-F(108)/121*z^3*u*v-861*x^2*u^2*v+F(49)/17*x*y*u^2*v+F(27)/80*y^2*u^2*v-F(17)/11*x*z*u^2*v-F(105)/2*y*z*u^2*v+F(69)/100*z^2*u^2*v-F(81)/52*x*u^3*v-F(1)/38*y*u^3*v-F(17)/123*z*u^3*v+F(44)/59*u^4*v-13186*x^2*z*v^2-F(47)/25*x*y*z*v^2-F(13)/2*y^2*z*v^2+F(13)/61*x*z^2*v^2+9398*y*z^2*v^2+F(110)/63*z^3*v^2-F(74)/15*x^2*u*v^2+F(102)/55*x*y*u*v^2+3926*y^2*u*v^2-11926*x*z*u*v^2-F(13)/29*y*z*u*v^2-13736*z^2*u*v^2-F(27)/28*x*u^2*v^2-F(29)/125*y*u^2*v^2+F(25)/84*z*u^2*v^2-5213*u^3*v^2-F(5)/22*x*z*v^3+14985*y*z*v^3-8188*z^2*v^3-F(65)/86*x*u*v^3+9280*y*u*v^3-9457*z*u*v^3+13811*u^2*v^3+F(5)/49*z*v^4-F(103)/78*u*v^4,-F(19)/80*x^4*y-F(27)/113*x^3*y^2-5370*x^2*y^3+13408*x*y^4-F(113)/31*x^4*z+F(27)/14*x^3*y*z+F(81)/97*x^2*y^2*z+F(72)/61*x*y^3*z-11081*y^4*z+F(53)/66*x^3*z^2-F(37)/103*x^2*y*z^2-8913*x*y^2*z^2+F(19)/73*y^3*z^2+F(87)/76*x^2*z^3+8630*x*y*z^3+F(31)/23*y^2*z^3+F(39)/58*x*z^4-F(43)/49*y*z^4-F(11)/111*z^5+10515*x^4*u+14620*x^3*y*u+11185*x^2*y^2*u-F(42)/61*x*y^3*u-F(19)/94*x^3*z*u+F(32)/21*x^2*y*z*u+F(59)/72*x*y^2*z*u+F(30)/37*y^3*z*u+F(78)/47*x^2*z^2*u+F(11)/101*x*y*z^2*u-11154*y^2*z^2*u+F(47)/119*x*z^3*u+F(32)/13*y*z^3*u+F(81)/55*z^4*u+11380*x^3*u^2-F(79)/43*x^2*y*u^2-F(17)/76*x*y^2*u^2-12197*x^2*z*u^2+F(97)/110*x*y*z*u^2-13291*y^2*z*u^2+F(8)/95*x*z^2*u^2+2991*y*z^2*u^2+F(27)/52*z^3*u^2-F(55)/3*x^2*u^3+4105*x*y*u^3+F(64)/27*x*z*u^3-F(105)/101*y*z*u^3-7395*z^2*u^3-F(60)/119*x*u^4+379*z*u^4-F(18)/89*x^4*v-F(125)/126*x^3*y*v+12576*x^2*y^2*v+5204*x*y^3*v-11650*x^3*z*v-F(73)/82*x^2*y*z*v+F(116)/105*x*y^2*z*v+9790*y^3*z*v+F(98)/97*x^2*z^2*v+F(25)/68*x*y*z^2*v-4311*y^2*z^2*v-11967*x*z^3*v-F(111)/121*y*z^3*v+F(100)/31*z^4*v-861*x^3*u*v+F(49)/17*x^2*y*u*v+F(27)/80*x*y^2*u*v-F(119)/24*x^2*z*u*v-6909*x*y*z*u*v+13144*y^2*z*u*v+F(39)/94*x*z^2*u*v+4932*y*z^2*u*v+F(119)/83*z^3*u*v-F(81)/52*x^2*u^2*v-F(1)/38*x*y*u^2*v+14498*x*z*u^2*v-F(104)/109*y*z*u^2*v+7249*z^2*u^2*v+F(44)/59*x*u^3*v+3104*z*u^3*v-F(74)/15*x^3*v^2+F(102)/55*x^2*y*v^2+3926*x*y^2*v^2+F(37)/38*x^2*z*v^2-14406*x*y*z*v^2+14163*y^2*z*v^2+3752*x*z^2*v^2-F(19)/126*y*z^2*v^2+F(95)/103*z^3*v^2-F(27)/28*x^2*u*v^2-F(29)/125*x*y*u*v^2+F(60)/47*x*z*u*v^2-6185*y*z*u*v^2+662*z^2*u*v^2-5213*x*u^2*v^2-2925*z*u^2*v^2-F(65)/86*x^2*v^3+9280*x*y*v^3+F(65)/118*x*z*v^3+9205*y*z*v^3-10805*z^2*v^3+13811*x*u*v^3+F(91)/37*z*u*v^3-F(103)/78*x*v^4-F(52)/31*z*v^4,F(19)/80*x^3*y^2+F(27)/113*x^2*y^3+5370*x*y^4-13408*y^5+F(82)/31*x^3*y*z-F(73)/33*x^2*y^2*z-11777*x*y^3*z-9176*y^4*z+F(65)/126*x^3*z^2+11659*x^2*y*z^2-9298*x*y^2*z^2+12013*y^3*z^2-10187*x^2*z^3+F(82)/17*x*y*z^3-F(7)/72*y^2*z^3-15332*x*z^4+8233*y*z^4+F(73)/18*z^5-10515*x^3*y*u-14620*x^2*y^2*u-11185*x*y^3*u+F(42)/61*y^4*u+F(96)/43*x^3*z*u-10817*x^2*y*z*u-F(91)/8*x*y^2*z*u+7791*y^3*z*u+11236*x^2*z^2*u+3970*x*y*z^2*u+6872*y^2*z^2*u-5986*x*z^3*u-13225*y*z^3*u-9266*z^4*u-11380*x^2*y*u^2+F(79)/43*x*y^2*u^2+F(17)/76*y^3*u^2+10541*x^2*z*u^2+F(50)/119*x*y*z*u^2+F(2)/59*y^2*z*u^2+4960*x*z^2*u^2-F(56)/81*y*z^2*u^2+10276*z^3*u^2+F(55)/3*x*y*u^3-4105*y^2*u^3+6041*x*z*u^3-11945*y*z*u^3+F(88)/79*z^2*u^3+F(60)/119*y*u^4-F(49)/125*z*u^4+F(18)/89*x^3*y*v+F(125)/126*x^2*y^2*v-12576*x*y^3*v-5204*y^4*v-F(83)/97*x^3*z*v+F(39)/109*x^2*y*z*v-1126*x*y^2*z*v+F(104)/123*y^3*z*v+12480*x^2*z^2*v-F(5)/101*x*y*z^2*v+F(53)/99*y^2*z^2*v-15931*x*z^3*v+4105*y*z^3*v+F(53)/55*z^4*v+861*x^2*y*u*v-F(49)/17*x*y^2*u*v-F(27)/80*y^3*u*v+F(59)/109*x^2*z*u*v-F(109)/43*x*y*z*u*v-14745*y^2*z*u*v+6083*x*z^2*u*v-F(93)/100*y*z^2*u*v+F(108)/103*z^3*u*v+F(81)/52*x*y*u^2*v+F(1)/38*y^2*u^2*v+F(30)/119*x*z*u^2*v+F(46)/121*y*z*u^2*v-15601*z^2*u^2*v-F(44)/59*y*u^3*v+1963*z*u^3*v+F(74)/15*x^2*y*v^2-F(102)/55*x*y^2*v^2-3926*y^3*v^2-7518*x^2*z*v^2-10094*x*y*z*v^2+9716*y^2*z*v^2-8520*x*z^2*v^2+F(57)/68*y*z^2*v^2+2432*z^3*v^2+F(27)/28*x*y*u*v^2+F(29)/125*y^2*u*v^2-5547*x*z*u*v^2-F(91)/30*y*z*u*v^2+F(43)/56*z^2*u*v^2+5213*y*u^2*v^2-648*z*u^2*v^2+F(65)/86*x*y*v^3-9280*y^2*v^3+F(113)/71*x*z*v^3-8151*y*z*v^3+165*z^2*v^3-13811*y*u*v^3-F(27)/91*z*u*v^3+F(103)/78*y*v^4-15905*z*v^4,x^4*y-F(109)/104*x^3*y^2-F(53)/113*x^2*y^3-F(86)/119*x*y^4+11081*y^5-F(65)/126*x^4*z+F(69)/70*x^3*y*z-F(19)/97*x^2*y^2*z-3100*x*y^3*z-F(19)/73*y^4*z+10187*x^3*z^2-F(95)/74*x^2*y*z^2+5144*x*y^2*z^2-F(31)/23*y^3*z^2+15332*x^2*z^3+F(13)/28*x*y*z^3+F(43)/49*y^2*z^3-F(73)/18*x*z^4+F(11)/111*y*z^4-F(96)/43*x^4*u+F(41)/18*x^3*y*u-F(13)/76*x^2*y^2*u+F(66)/37*x*y^3*u-F(30)/37*y^4*u-11236*x^3*z*u-3291*x^2*y*z*u-F(68)/75*x*y^2*z*u+11154*y^3*z*u+5986*x^2*z^2*u+F(117)/23*x*y*z^2*u-F(32)/13*y^2*z^2*u+9266*x*z^3*u-F(81)/55*y*z^3*u-10541*x^3*u^2+F(80)/93*x^2*y*u^2+3908*x*y^2*u^2+13291*y^3*u^2-4960*x^2*z*u^2-F(96)/61*x*y*z*u^2-2991*y^2*z*u^2-10276*x*z^2*u^2-F(27)/52*y*z^2*u^2-6041*x^2*u^3+F(65)/114*x*y*u^3+F(105)/101*y^2*u^3-F(88)/79*x*z*u^3+7395*y*z*u^3+F(49)/125*x*u^4-379*y*u^4+F(83)/97*x^4*v-F(108)/37*x^3*y*v-10187*x^2*y^2*v+F(62)/113*x*y^3*v-9790*y^4*v-12480*x^3*z*v-F(30)/89*x^2*y*z*v+F(17)/118*x*y^2*z*v+4311*y^3*z*v+15931*x^2*z^2*v-F(23)/118*x*y*z^2*v+F(111)/121*y^2*z^2*v-F(53)/55*x*z^3*v-F(100)/31*y*z^3*v-F(59)/109*x^3*u*v+15786*x^2*y*u*v-F(94)/65*x*y^2*u*v-13144*y^3*u*v-6083*x^2*z*u*v-15185*x*y*z*u*v-4932*y^2*z*u*v-F(108)/103*x*z^2*u*v-F(119)/83*y*z^2*u*v-F(30)/119*x^2*u^2*v+F(103)/27*x*y*u^2*v+F(104)/109*y^2*u^2*v+15601*x*z*u^2*v-7249*y*z*u^2*v-1963*x*u^3*v-3104*y*u^3*v+7518*x^3*v^2-F(107)/101*x^2*y*v^2+4690*x*y^2*v^2-14163*y^3*v^2+8520*x^2*z*v^2+3304*x*y*z*v^2+F(19)/126*y^2*z*v^2-2432*x*z^2*v^2-F(95)/103*y*z^2*v^2+5547*x^2*u*v^2+F(53)/56*x*y*u*v^2+6185*y^2*u*v^2-F(43)/56*x*z*u*v^2-662*y*z*u*v^2+648*x*u^2*v^2+2925*y*u^2*v^2-F(113)/71*x^2*v^3+9506*x*y*v^3-9205*y^2*v^3-165*x*z*v^3+10805*y*z*v^3+F(27)/91*x*u*v^3-F(91)/37*y*u*v^3+15905*x*v^4+F(52)/31*y*v^4,-F(23)/66*x^3*y-F(84)/115*x^2*y^2+F(6)/19*x*y^3+F(37)/33*y^4+14611*x^3*z-F(19)/121*x^2*y*z-13293*x*y^2*z+F(83)/68*y^3*z+F(73)/71*x^2*z^2+2124*x*y*z^2+F(21)/92*y^2*z^2-7443*x*z^3+11253*y*z^3+F(31)/42*z^4+F(23)/34*x^3*u+F(20)/7*x^2*y*u-F(119)/123*x*y^2*u+10344*y^3*u-243*x^2*z*u+2165*x*y*z*u-F(73)/101*y^2*z*u+F(99)/25*x*z^2*u+F(91)/71*y*z^2*u+1108*z^3*u+15730*x^2*u^2-4782*x*y*u^2-F(66)/95*y^2*u^2+1051*x*z*u^2-F(77)/114*y*z*u^2-F(23)/9*z^2*u^2+F(16)/25*x*u^3+4906*y*u^3+4376*z*u^3-F(122)/71*u^4+2521*x^3*v-15886*x^2*y*v-F(61)/106*x*y^2*v+F(37)/80*y^3*v+1610*x^2*z*v-F(45)/71*x*y*z*v+F(98)/9*y^2*z*v+F(19)/15*x*z^2*v-F(109)/9*y*z^2*v+F(106)/101*z^3*v-F(96)/43*x^2*u*v-13344*x*y*u*v-874*y^2*u*v-4213*x*z*u*v-F(49)/124*y*z*u*v+15800*z^2*u*v-F(93)/16*x*u^2*v+F(65)/126*y*u^2*v-F(55)/27*z*u^2*v+F(85)/2*u^3*v-F(41)/26*x^2*v^2+F(120)/91*x*y*v^2+13161*y^2*v^2+F(81)/28*x*z*v^2-F(89)/74*y*z*v^2+5436*z^2*v^2-F(56)/61*x*u*v^2-4394*y*u*v^2+F(97)/117*z*u*v^2+F(51)/10*u^2*v^2+F(91)/47*x*v^3+F(109)/94*y*v^3+F(71)/72*z*v^3-F(123)/113*u*v^3-F(109)/7*v^4,F(110)/89*x^3*y+8678*x^2*y^2-F(114)/17*x*y^3-F(50)/117*y^4+F(61)/110*x^3*z+2651*x^2*y*z+F(82)/103*x*y^2*z-F(95)/31*y^3*z-6435*x^2*z^2+10588*x*y*z^2-F(107)/103*y^2*z^2+F(124)/37*x*z^3-7229*y*z^3-F(68)/63*z^4+F(43)/116*x^3*u+F(19)/37*x^2*y*u-F(95)/94*x*y^2*u-6426*y^3*u-F(11)/119*x^2*z*u-F(126)/23*x*y*z*u+7556*y^2*z*u+F(11)/101*x*z^2*u+10591*y*z^2*u+3215*z^3*u-F(71)/50*x^2*u^2-F(16)/67*x*y*u^2+9937*y^2*u^2+11530*x*z*u^2-10092*y*z*u^2+F(19)/98*z^2*u^2+5015*x*u^3+153*y*u^3+F(74)/79*z*u^3+F(39)/17*u^4-F(65)/43*x^3*v-15929*x^2*y*v-F(62)/33*x*y^2*v-F(111)/106*y^3*v-F(58)/33*x^2*z*v-F(81)/22*x*y*z*v-F(76)/97*y^2*z*v+10790*x*z^2*v-14790*y*z^2*v-747*z^3*v-606*x^2*u*v-10482*x*y*u*v+F(43)/106*y^2*u*v+39*x*z*u*v+F(49)/95*y*z*u*v-F(73)/25*z^2*u*v+3652*x*u^2*v-F(52)/11*y*u^2*v+F(53)/7*z*u^2*v-F(111)/94*u^3*v+F(61)/91*x^2*v^2+F(119)/86*x*y*v^2+1790*y^2*v^2-F(49)/87*x*z*v^2-F(60)/43*y*z*v^2-F(30)/119*z^2*v^2+F(105)/79*x*u*v^2+14959*y*u*v^2+F(51)/52*z*u*v^2-5194*u^2*v^2-4777*x*v^3+5404*y*v^3+15449*z*v^3+F(23)/25*u*v^3+F(47)/92*v^4,F(23)/66*x^4-F(31)/91*x^3*y+1657*x^2*y^2+F(99)/34*x*y^3-F(83)/30*y^4-12390*x^3*z-F(9)/28*x^2*y*z-F(83)/17*x*y^2*z-12576*y^3*z+F(29)/39*x^2*z^2-F(79)/43*x*y*z^2+F(32)/57*y^2*z^2-F(19)/96*x*z^3-F(100)/59*y*z^3+F(75)/107*z^4-F(37)/46*x^3*u-F(99)/64*x^2*y*u-13786*x*y^2*u-F(115)/68*y^3*u-5594*x^2*z*u+F(103)/107*x*y*z*u-9212*y^2*z*u-F(71)/76*x*z^2*u-13810*y*z^2*u-4122*z^3*u+F(42)/25*x^2*u^2+F(97)/91*x*y*u^2+F(103)/16*y^2*u^2+4304*x*z*u^2+F(79)/10*y*z*u^2+F(75)/28*z^2*u^2+11154*x*u^3+F(123)/37*y*u^3-F(119)/55*z*u^3-F(30)/119*u^4-13297*x^3*v+F(70)/71*x^2*y*v+F(95)/62*x*y^2*v-15785*y^3*v-F(93)/97*x^2*z*v+7610*x*y*z*v+F(69)/74*y^2*z*v-F(23)/18*x*z^2*v+F(11)/107*y*z^2*v-F(82)/119*z^3*v+F(108)/109*x^2*u*v-F(95)/27*x*y*u*v+12075*y^2*u*v-7844*x*z*u*v+2513*y*z*u*v-8942*z^2*u*v-F(49)/116*x*u^2*v+12176*y*u^2*v+6249*z*u^2*v+F(87)/89*u^3*v+F(11)/8*x^2*v^2-3723*x*y*v^2-8541*y^2*v^2-F(51)/2*x*z*v^2+2378*y*z*v^2+F(71)/126*z^2*v^2-F(94)/95*x*u*v^2+F(28)/113*y*u*v^2+F(99)/7*z*u*v^2-4132*u^2*v^2+F(38)/105*x*v^3+F(74)/17*y*v^3+10389*z*v^3-F(96)/73*u*v^3+F(116)/69*v^4]))
end


@doc raw"""
    veronese()

Return a smooth rational surface in $\mathbb P^4$ with degree `4` and sectional genus `0`.

The returned surface is defined over a prime field of characteristic 31991.
"""
function veronese()
   F = GF(31991)
   R, (x,y,z,u,v) = graded_polynomial_ring(F, ["x", "y", "z", "u", "v"])
   return ProjectiveScheme(R, ideal([F(45)/16*x^3-8565*x^2*y-7937*x*y^2+14060*x^2*z+F(53)/113*x*y*z-F(125)/17*x*z^2-15712*x^2*u-5990*x*y*u-F(71)/88*x*z*u-6141*x*u^2+F(49)/104*x^2*v-194*x*y*v+F(63)/103*y^2*v+F(74)/103*x*z*v+11618*y*z*v+F(41)/111*z^2*v+14387*x*u*v-F(64)/9*y*u*v+13097*z*u*v+F(84)/101*u^2*v+12211*x*v^2+F(85)/63*y*v^2-F(119)/100*z*v^2-6247*u*v^2-F(74)/51*v^3,-F(45)/16*x^2*y+8565*x*y^2+7937*y^3-14060*x*y*z-F(53)/113*y^2*z+F(125)/17*y*z^2+15712*x*y*u+5990*y^2*u+F(71)/88*y*z*u+6141*y*u^2-x^2*v+3340*x*y*v+F(111)/40*y^2*v+F(22)/37*x*z*v-F(110)/29*y*z*v-F(71)/25*z^2*v-F(71)/87*x*u*v+F(112)/55*y*u*v+F(103)/80*z*u*v-F(100)/7*u^2*v-5013*x*v^2+15010*y*v^2+F(52)/51*z*v^2+F(126)/5*u*v^2-6580*v^3,-F(45)/16*x^2*z+8565*x*y*z+7937*y^2*z-14060*x*z^2-F(53)/113*y*z^2+F(125)/17*z^3+15712*x*z*u+5990*y*z*u+F(71)/88*z^2*u+6141*z*u^2+F(41)/79*x^2*v+F(121)/48*x*y*v+9621*y^2*v+F(105)/83*x*z*v+F(27)/26*y*z*v-649*z^2*v-12278*x*u*v+14655*y*u*v+10594*z*u*v+10462*u^2*v+F(76)/13*x*v^2+F(97)/50*y*v^2-7761*z*v^2+F(52)/31*u*v^2-6636*v^3,-F(45)/16*x^2*u+8565*x*y*u+7937*y^2*u-14060*x*z*u-F(53)/113*y*z*u+F(125)/17*z^2*u+15712*x*u^2+5990*y*u^2+F(71)/88*z*u^2+6141*u^3-F(40)/31*x^2*v+13297*x*y*v+F(126)/113*y^2*v-F(51)/100*x*z*v-F(97)/38*y*z*v+10372*z^2*v-F(4)/31*x*u*v+F(32)/9*y*u*v-F(1)/7*z*u*v+15473*u^2*v+F(101)/36*x*v^2+F(97)/109*y*v^2-14100*z*v^2+F(109)/32*u*v^2-F(5)/111*v^3,-F(40)/31*x^3+13297*x^2*y+F(126)/113*x*y^2-F(51)/100*x^2*z-F(97)/38*x*y*z+10372*x*z^2+F(26)/105*x^2*u-3745*x*y*u+F(63)/103*y^2*u-F(98)/61*x*z*u+11618*y*z*u+F(41)/111*z^2*u+F(26)/15*x*u^2-F(64)/9*y*u^2+13097*z*u^2+F(84)/101*u^3+F(101)/36*x^2*v+F(97)/109*x*y*v-14100*x*z*v-14778*x*u*v+F(85)/63*y*u*v-F(119)/100*z*u*v-6247*u^2*v-F(5)/111*x*v^2-F(74)/51*u*v^2,F(40)/31*x^2*y-13297*x*y^2-F(126)/113*y^3+F(51)/100*x*y*z+F(97)/38*y^2*z-10372*y*z^2-x^2*u+F(103)/30*x*y*u+2754*y^2*u+F(22)/37*x*z*u+F(126)/95*y*z*u-F(71)/25*z^2*u-F(71)/87*x*u^2-F(109)/7*y*u^2+F(103)/80*z*u^2-F(100)/7*u^3-F(101)/36*x*y*v-F(97)/109*y^2*v+14100*y*z*v-5013*x*u*v+10008*y*u*v+F(52)/51*z*u*v+F(126)/5*u^2*v+F(5)/111*y*v^2-6580*u*v^2,F(40)/31*x^2*z-13297*x*y*z-F(126)/113*y^2*z+F(51)/100*x*z^2+F(97)/38*y*z^2-10372*z^3+F(41)/79*x^2*u+F(121)/48*x*y*u+9621*y^2*u+5671*x*z*u-F(42)/71*y*z*u-5219*z^2*u-12278*x*u^2+14655*y*u^2+F(58)/59*z*u^2+10462*u^3-F(101)/36*x*z*v-F(97)/109*y*z*v+14100*z^2*v+F(76)/13*x*u*v+F(97)/50*y*u*v-12763*z*u*v+F(52)/31*u^2*v+F(5)/111*z*v^2-6636*u*v^2,F(41)/79*x^3+F(121)/48*x^2*y+9621*x*y^2-F(22)/109*x^2*z+F(25)/19*x*y*z+F(63)/103*y^2*z+F(23)/45*x*z^2+11618*y*z^2+F(41)/111*z^3-12278*x^2*u+14655*x*y*u+F(126)/73*x*z*u-F(64)/9*y*z*u+13097*z^2*u+10462*x*u^2+F(84)/101*z*u^2+F(76)/13*x^2*v+F(97)/50*x*y*v-F(106)/115*x*z*v+F(85)/63*y*z*v-F(119)/100*z^2*v+F(52)/31*x*u*v-6247*z*u*v-6636*x*v^2-F(74)/51*z*v^2,-F(41)/79*x^2*y-F(121)/48*x*y^2-9621*y^3-x^2*z-F(22)/89*x*y*z-F(32)/17*y^2*z+F(22)/37*x*z^2-F(86)/111*y*z^2-F(71)/25*z^3+12278*x*y*u-14655*y^2*u-F(71)/87*x*z*u+F(74)/47*y*z*u+F(103)/80*z^2*u-10462*y*u^2-F(100)/7*z*u^2-F(76)/13*x*y*v-F(97)/50*y^2*v-5013*x*z*v-9220*y*z*v+F(52)/51*z^2*v-F(52)/31*y*u*v+F(126)/5*z*u*v+6636*y*v^2-6580*z*v^2,x^3+11117*x^2*y+991*x*y^2-F(63)/103*y^3-F(22)/37*x^2*z-F(35)/13*x*y*z-11618*y^2*z+F(71)/25*x*z^2-F(41)/111*y*z^2+F(71)/87*x^2*u+F(68)/115*x*y*u+F(64)/9*y^2*u-F(103)/80*x*z*u-13097*y*z*u+F(100)/7*x*u^2-F(84)/101*y*u^2+5013*x^2*v-F(67)/114*x*y*v-F(85)/63*y^2*v-F(52)/51*x*z*v+F(119)/100*y*z*v-F(126)/5*x*u*v+6247*y*u*v+6580*x*v^2+F(74)/51*y*v^2]))
end
